# $Id: test.py 35 2010-01-13 23:46:28Z tlev $
# -*- coding: iso-8859-1 -*-
"""
This is the main calculation routine for the emission routines
in the ARTEMIS project. This software is part a PHD written by
Tomas Levin 2007 - 2010

"""
from math import sqrt
  
########_________Globl hashes _________###############
H_HDV      = {}
H_LDV      = {}
H_PC       = {}
H_PCGrade  = {}
H_SHIP     = {}
H_TRAIN    = {}
########__________________________________###############


########_________Global variables_________###############
#
#Energy densities:
EDensity_Diesel = 43.41 #MJ/kg http://www3.statoil.com/MAR/SVG01997.nsf/searchview/8F5B3347CDA38076C1256C2400336556?OpenDocument
EDensity_Gasoline = 47.3 #MJ/kg
EDensity_Ethanol = 30.0 #MJ/kg
EDensity_Biodiesel = 42.2 #MJ/kg

WightDensity_Diesel = 0.85 # kg/liter
WeightDensity_Petrol = 0.72 # kg/liter

#Gravety Constant
g =  9.80665 # m/s2  The international system of units (SI) - United States Department of Commerce, NIST Special Publication 330, 2001, p. 29
#The air density
rho = 1.2 # kg/m^3 

#Need to add volume densities for petrol and diesel
#
#DieselLiterToKilogram =
#DieselKilogramToLiter =
#PetrolLiterToKilogram =
#PetrolKilogramToLiter=

###########____________________________##################




###########____Conversion functions___##################
"""
Aktuelle konverteringer:
Konvertering mellom kW, hp og Joule
Konvertering mellom mega, kilo, giga o.l.
Konvertering mellom hastigheter km/h, m/s, knop

"""


def Convert_gpkm_to_gps(Emission, avgSpeed):
  E = float(Emission) * float(avgSpeed) / (1000.0 * 3.6)
  return E, "g/s"

def Convert_gps_to_gpkm(Emission, avgSpeed):
  E = float(Emission) * (1000.0 * 3.6) / float(avgSpeed)
  return E, "g/km"

def Convert_J_to_kW(Joule):
  E = float(Joule) / 3600000.0
  return E, "kW"

def Convert_kW_to_Joule(Joule):
  E = 3600000 * float(Joule)
  return E, "J"

def Convert_gpkm_to_gpLoadkgkm(Emission, Load, Distance):
  E = Emission / float(Load)
  return E, "g/Loadkgkm"
def Convert_gpLoadkgkm_to_gpkm(Emission, Load, Distance):
  E = Emission * float(Load)
  return E, "g/km"

def Convert_liters_to_kilo(Density,Liters):
  kilos = float(Liters) / float(Density)
  return kilos

def Convert_kilos_to_liters(Density,Kilos):
  liters = float(Kilos)*float(Density)
  return liters


def linreg(X, Y):
  """
  Summary
      Linear regression of y = ax + b
  Usage
      real, real, real = linreg(list, list)
  Returns coefficients to the regression line "y=ax+b" from x[] and y[], and R^2 Value
  Function by Greg Pinero found on http://www.answermysearches.com/how-to-do-a-simple-linear-regression-in-python/124/ 20100620 TLEV
  """
  if len(X) != len(Y):  raise ValueError, 'unequal length'
  N = len(X)
  Sx = Sy = Sxx = Syy = Sxy = 0.0
  for x, y in map(None, X, Y):
    Sx = Sx + x
    Sy = Sy + y
    Sxx = Sxx + x*x
    Syy = Syy + y*y
    Sxy = Sxy + x*y
  det = Sxx * N - Sx * Sx
  a, b = (Sxy * N - Sy * Sx)/det, (Sxx * Sy - Sx * Sxy)/det
  meanerror = residual = 0.0
  for x, y in map(None, X, Y):
    meanerror = meanerror + (y - Sy/N)**2
    residual = residual + (y - a * x - b)**2
  RR = 1 - residual/meanerror
  ss = residual / (N-2)
  Var_a, Var_b = ss * N / det, ss * Sxx / det
  #print "y=ax+b"
  #print "N= %d" % N
  #print "a= %g \\pm t_{%d;\\alpha/2} %g" % (a, N-2, sqrt(Var_a))
  #print "b= %g \\pm t_{%d;\\alpha/2} %g" % (b, N-2, sqrt(Var_b))
  #print "R^2= %g" % RR
  #print "s^2= %g" % ss
  return a, b, RR


###########____Filling hashes___##################

###            PRIVATE CAR
def load_PC():
  H_PC['1_CO_all'] = [' 1','CO','DIESEL','all','EURO0','Power','0','3.9660134489516','-0.5153218294927','0','0','0','0']
  H_PC['2_CO_all'] = [' 2','CO','DIESEL','all','EURO1','Power','0','1.1908594708','-0.2768161568','0','0','0','0']
  H_PC['3_CO_all'] = [' 3','CO','DIESEL','all','EURO2','Power','0','7.4030275469472','-0.9804513272359','0','0','0','0']
  H_PC['4_CO_all'] = [' 4','CO','DIESEL','all','EURO3','Polyn.','3','0.4340726183844','-0.010373193366','0.0000873659792','-0.0000002497877','0','0']
  H_PC['5_CO_all'] = [' 5','CO','DIESEL','all','EURO4','Power','0','2.77591','-1.29524','0','0','0','0']
  H_PC['1_HC_all'] = [' 1','HC','DIESEL','all','EURO0','Power','0','1.255795488','-0.70774519','0','0','0','0']
  H_PC['2_HC_all'] = [' 2','HC','DIESEL','all','EURO1','Polyn.','4','0.13189114263','-0.00524324841','0.00009552141','-0.00000075054','0.00000000214','0']
  H_PC['3_HC_all'] = [' 3','HC','DIESEL','all','EURO2','Power','0','0.777435','-0.790252','0','0','0','0']
  H_PC['4_HC_all'] = [' 4','HC','DIESEL','all','EURO3','Power','0','0.49531','-0.84573','0','0','0','0']
  H_PC['5_HC_all'] = [' 5','HC','DIESEL','all','EURO4','Power','0','0.077','-0.4635','0','0','0','0']
  H_PC['1_NOx_all'] = [' 1','NOx','DIESEL','all','EURO0','Polyn.','5','2.0912838359','-0.0900144801','0.00227212','-0.0000296598','0.0000001929','-0.0000000005']
  H_PC['2_NOx_all'] = [' 2','NOx','DIESEL','all','EURO1','Polyn.','5','2.02149544103969','-0.09983830447789','0.00277252771995','-0.00003890978825','0.00000026935423','-0.000000000714']
  H_PC['3_NOx_all'] = [' 3','NOx','DIESEL','all','EURO2','Polyn.','4','2.12341032437','-0.08882496171','0.00186200713','-0.00001694811','0.00000005778','0']
  H_PC['4_NOx_all'] = [' 4','NOx','DIESEL','all','EURO3','Polyn.','4','2.25413','-0.0934458','0.001932741','-0.00001637522','0.0000000503116','0']
  H_PC['5_NOx_all'] = [' 5','NOx','DIESEL','all','EURO4','Polyn.','3','0.78737','-0.0191047','0.000256467','-0.00000070976','0','0']
  H_PC['1_PM_all'] = [' 1','PM','DIESEL','all','EURO0','Polyn.','4','0.4049846824','-0.018768082','0.0003917685','-0.0000035787','0.0000000123','0']
  H_PC['2_PM_all'] = [' 2','PM','DIESEL','all','EURO1','Polyn.','3','0.084283068','0.000141427','-0.000030544','0.000000279','0','0']
  H_PC['3_PM_all'] = [' 3','PM','DIESEL','all','EURO2','Polyn.','2','0.079399931','-0.000868462','0.000007496','0','0','0']
  H_PC['4_PM_all'] = [' 4','PM','DIESEL','all','EURO3','Polyn.','2','0.059947844','-0.001241491','0.000010291','0','0','0']
  H_PC['5_PM_all'] = [' 5','PM','DIESEL','all','EURO4','Polyn.','4','0.0502967812','-0.0011471559','0.0000281512','-0.000000404','0.0000000024','0']
  H_PC['1_CO2_all'] = [' 1','CO2','DIESEL','all','EURO0','Polyn.','5','541.4588864945','-29.145465837','0.8401011441','-0.0117170292','0.000078952','-0.0000002032']
  H_PC['2_CO2_all'] = [' 2','CO2','DIESEL','all','EURO1','Polyn.','4','389.8831339439','-13.3756933369','0.2561021185','-0.0020814923','0.0000063387','0']
  H_PC['3_CO2_all'] = [' 3','CO2','DIESEL','all','EURO2','Polyn.','4','444.2898846977','-16.238433356','0.3124155177','-0.0025620634','0.0000077935','0']
  H_PC['4_CO2_all'] = [' 4','CO2','DIESEL','all','EURO3','Polyn.','4','429.7','-15.161','0.29301','-0.0024806','0.000007802','0']
  H_PC['5_CO2_all'] = [' 5','CO2','DIESEL','all','EURO4','Polyn.','4','313.2','-9.594','0.17565','-0.0014072','0.000004398','0']
  H_PC['6_CO_all'] = [' 6','CO','PETROL','all','EURO0','Polyn.','4','58.40623534','-2.85958801','0.05969181','-0.00053623','0.00000176','0']
  H_PC['7_CO_all'] = [' 7','CO','PETROL','all','EURO1','Polyn.','4','8.744956179','-0.41392186','0.008366938','-0.000073635','0.000000242','0']
  H_PC['8_CO_all'] = [' 8','CO','PETROL','all','EURO2','Polyn.','3','1.88068315064606','-0.00626259341106','-0.000596506471','0.00000579342731','0','0']
  H_PC['9_CO_all'] = [' 9','CO','PETROL','all','EURO3','Polyn.','2','1.29','-0.04327','0.0004914','0','0','0']
  H_PC['10_CO_all'] = [' 10','CO','PETROL','all','EURO4','Polyn.','2','0.635','-0.02808','0.0003343','0','0','0']
  H_PC['6_HC_all'] = [' 6','HC','PETROL','all','EURO0','Polyn.','4','5.57575694376','-0.19068277209','0.00298332867','-0.0000211512','0.00000005666','0']
  H_PC['7_HC_all'] = [' 7','HC','PETROL','all','EURO1','Polyn.','4','0.86029871333417','-0.033333827566281','0.000562090522745','-0.000004338581476','0.000000012834451','0']
  H_PC['8_HC_all'] = [' 8','HC','PETROL','all','EURO2','Polyn.','3','0.263356346978','-0.008089197883','0.000093836743','-0.000000349094','0','0']
  H_PC['9_HC_all'] = [' 9','HC','PETROL','all','EURO3','Polyn.','3','0.04445351','-0.00140222','0.00001968','-0.00000006','0','0']
  H_PC['10_HC_all'] = [' 10','HC','PETROL','all','EURO4','Polyn.','2','0.0053120134','-0.0000402455','0.0000006971','0','0','0']
  H_PC['6_NOx_all'] = [' 6','NOx','PETROL','all','EURO0','Polyn.','3','1.533211844','-0.014748215','0.000082264','0.00000098','0','0']
  H_PC['7_NOx_all'] = [' 7','NOx','PETROL','all','EURO1','Polyn.','4','0.757005404','-0.030289932','0.000651466','-0.00000575','0.000000019','0']
  H_PC['8_NOx_all'] = [' 8','NOx','PETROL','all','EURO2','Polyn.','3','0.550400155','-0.010560415','0.000120798','-0.000000443','0','0']
  H_PC['9_NOx_all'] = [' 9','NOx','PETROL','all','EURO3','Polyn.','2','0.126998185','-0.001634074','0.000009709','0','0','0']
  H_PC['10_NOx_all'] = [' 10','NOx','PETROL','all','EURO4','Polyn.','3','0.043985972','0.001246418','-0.000039284','0.00000024','0','0']
  H_PC['6_PM_all'] = [' 6','PM','PETROL','all','EURO0','Polyn.','2','0.01217','-0.0003603','0.000003265','0','0','0']
  H_PC['7_PM_all'] = [' 7','PM','PETROL','all','EURO1','Polyn.','2','0.01217','-0.0003603','0.000003265','0','0','0']
  H_PC['8_PM_all'] = [' 8','PM','PETROL','all','EURO2','Polyn.','3','0.00733','-0.0002552','0.000003021','-0.00000000589','0','0']
  H_PC['9_PM_all'] = [' 9','PM','PETROL','all','EURO3','Power','0','0.00823','-0.22603','0','0','0','0']
  H_PC['10_PM_all'] = [' 10','PM','PETROL','all','EURO4','Power','0','0.00223','-0.047','0','0','0','0']
  H_PC['6_CO2_all'] = [' 6','CO2','PETROL','all','EURO0','Polyn.','5','675.302363493','-37.049152806','1.030976222','-0.013928833','0.0000905','-0.000000224']
  H_PC['7_CO2_all'] = [' 7','CO2','PETROL','all','EURO1','Polyn.','5','571.300069292104','-28.881936744792','0.781769520312','-0.010321281609','0.000065748289','-0.000000160077']
  H_PC['8_CO2_all'] = [' 8','CO2','PETROL','all','EURO2','Polyn.','5','666.406084785661','-36.547953642386','1.00575117487','-0.013086547614','0.000080019841','-0.000000183537']
  H_PC['9_CO2_all'] = [' 9','CO2','PETROL','all','EURO3','Polyn.','4','521.9','-19.823','0.39285','-0.0033902','0.000010803','0']
  H_PC['10_CO2_all'] = [' 10','CO2','PETROL','all','EURO4','Polyn.','4','523.7','-16.544','0.26354','-0.0017715','0.000004429','0']
  H_PC['6_CO2_medium'] = [' 6','CO2','PETROL','medium','EURO0','Polyn.','5','651.69114325514','-38.57384251872','1.17818381911','-0.01741411232','0.0001224744','-0.0000003265']
  H_PC['7_CO2_medium'] = [' 7','CO2','PETROL','medium','EURO1','Polyn.','5','670.8576388742','-35.6644665061','0.9913821672','-0.0133270275','0.000085789','-0.0000002105']
  H_PC['8_CO2_medium'] = [' 8','CO2','PETROL','medium','EURO2','Polyn.','5','709.2365028915','-37.3989820727','0.9864565839','-0.0124204612','0.0000741371','-0.0000001667']
  H_PC['9_CO2_medium'] = [' 9','CO2','PETROL','medium','EURO3','Polyn.','4','561.3','-21.319','0.41301','-0.0034474','0.000010601','0']
  H_PC['10_CO2_medium'] = [' 10','CO2','PETROL','medium','EURO4','Polyn.','4','497.8452452029','-17.2870588404','0.3078851138','-0.0023070641','0.0000063482','0']
  H_PC['1_CO2_large'] = [' 1','CO2','DIESEL','large','EURO0','Polyn.','5','549.874085193','-26.9124599949','0.771996839','-0.0108063415','0.0000731593','-0.0000001892']
  H_PC['2_CO2_large'] = [' 2','CO2','DIESEL','large','EURO1','Polyn.','4','539.8839966579','-18.8839812654','0.3345990339','-0.0023441855','0.0000058459','0']
  H_PC['3_CO2_large'] = [' 3','CO2','DIESEL','large','EURO2','Polyn.','4','542.0598346884','-20.0392912272','0.3796437971','-0.0029934363','0.0000086332','0']
  H_PC['4_CO2_large'] = [' 4','CO2','DIESEL','large','EURO3','Polyn.','5','534.9136648918','-24.0371118351','0.5923516257','-0.0068194522','0.0000368585','-0.0000000752']
  H_PC['6_CO2_large'] = [' 6','CO2','PETROL','large','EURO0','Polyn.','5','1052.6844243583','-63.75991496147','1.88276216018','-0.02695595291','0.00018485778','-0.00000048291']
  H_PC['7_CO2_large'] = [' 7','CO2','PETROL','large','EURO1','Polyn.','4','644.82088279665','-27.08036584118','0.53963191532','-0.00448754673','0.00001374711','0']
  H_PC['8_CO2_large'] = [' 8','CO2','PETROL','large','EURO2','Polyn.','4','931.8008784997','-45.8431323502','0.9521844852','-0.0080183927','0.0000238214','0']
  H_PC['9_CO2_large'] = [' 9','CO2','PETROL','large','EURO3','Polyn.','4','649.8','-22.675','0.40493','-0.0031821','0.000009358','0']
  H_PC['10_CO2_large'] = [' 10','CO2','PETROL','large','EURO4','Polyn.','3','672','-17.235','0.2025','-0.000755','0','0']
  H_PC['1_CO2_medium'] = [' 1','CO2','DIESEL','medium','EURO0','Polyn.','4','455.9673561472','-19.42343932','0.4084759275','-0.003634518','0.0000118131','0']
  H_PC['2_CO2_medium'] = [' 2','CO2','DIESEL','medium','EURO1','Polyn.','4','393.4883311705','-14.0436306459','0.2833628591','-0.0024767244','0.0000080826','0']
  H_PC['3_CO2_medium'] = [' 3','CO2','DIESEL','medium','EURO2','Polyn.','4','433.5243050572','-16.4217013256','0.3290150294','-0.0028190314','0.0000089012','0']
  H_PC['4_CO2_medium'] = [' 4','CO2','DIESEL','medium','EURO3','Polyn.','4','412','-14.491','0.28188','-0.0023886','0.000007502','0']
  H_PC['5_CO2_medium'] = [' 5','CO2','DIESEL','medium','EURO4','Polyn.','4','313.2','-9.594','0.17565','-0.0014072','0.000004398','0']
  H_PC['6_CO2_small'] = [' 6','CO2','PETROL','small','EURO0','Polyn.','5','497.24776295437','-27.6805083754','0.81127517317','-0.01162480229','0.00008031306','-0.00000021177']
  H_PC['7_CO2_small'] = [' 7','CO2','PETROL','small','EURO1','Polyn.','4','476.9','-20.851','0.45156','-0.004151','0.000013777','0']
  H_PC['8_CO2_small'] = [' 8','CO2','PETROL','small','EURO2','Polyn.','4','505.9','-23.044','0.51889','-0.0048986','0.000016456','0']
  H_PC['9_CO2_small'] = [' 9','CO2','PETROL','small','EURO3','Polyn.','4','434','-15.08','0.29196','-0.0025074','0.000008044','0']
  H_PC['10_CO2_small'] = [' 10','CO2','PETROL','small','EURO4','Polyn.','4','395','-11.36','0.17318','-0.001086','0.000002485','0']


###   PRIVATE CAR GRADIENT
def load_PCGradient():
  H_PCGrade['EURO0_urban_0'] = [' EURO0','urban','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO0_urban_2'] = [' EURO0','urban','2','1.09851680895112','1.04771897290747','1.35855460374559','1.12870230792464','1.03724069247558','0.980730746885288','1.34527823551781','1.08330832708177','1.2414226242282']
  H_PCGrade['EURO0_urban_-2'] = [' EURO0','urban','-2','0.909632868433332','0.968295797444469','0.688585393612373','0.868774604772319','0.97122454601318','1.00629397714802','0.692356445683864','0.921417854463616','0.772577340234259']
  H_PCGrade['EURO0_urban_4'] = [' EURO0','urban','4','1.20346177737547','1.12816696353773','1.8826022010182','1.2906984318427','1.06718157353845','0.994932541475696','1.78301432374192','1.26301575393848','1.53423419339164']
  H_PCGrade['EURO0_urban_-4'] = [' EURO0','urban','-4','0.8420589431503','1.01293219139643','0.489995833714396','0.767166301605526','0.953149831542814','1.03531082564069','0.453416149068323','0.876144036009002','0.577802720481796']
  H_PCGrade['EURO0_urban_6'] = [' EURO0','urban','6','1.27670840365441','1.17591033680632','2.45922629027833','1.46039632303608','1.65478990528256','1.04486476018333','2.38207630878438','1.54996249062266','1.90161199282407']
  H_PCGrade['EURO0_urban_-6'] = [' EURO0','urban','-6','0.775153530995534','1.01541289477751','0.331514394008678','0.679577683285091','0.813849511580107','0.918468788328707','0.265014577259475','0.698799699924981','0.386479725915885']
  H_PCGrade['EURO0_road_0'] = [' EURO0','road','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO0_road_2'] = [' EURO0','road','2','1.28117528607861','1.11142321093541','1.3353747645951','1.18304604277069','1.10872664373016','0.975103230507651','1.42442536621062','1.15630697828609','1.30818152458501']
  H_PCGrade['EURO0_road_-2'] = [' EURO0','road','-2','0.793461206257698','0.950565532028947','0.758885122410546','0.868428630250835','1.15284457032067','1.29196016516881','0.734773347968691','1.16487098734737','0.818058113894398']
  H_PCGrade['EURO0_road_4'] = [' EURO0','road','4','1.65608111906697','1.26348968105066','1.69887005649718','1.38132325789535','1.12948354844228','0.958222006315278','1.89705594104042','1.38372285761644','1.65565052691959']
  H_PCGrade['EURO0_road_-4'] = [' EURO0','road','-4','0.675486934167421','0.962294291074779','0.493627118644068','0.698658818144894','1.1049630291395','1.29311391790138','0.450965018882225','1.04282004530637','0.576907538710089']
  H_PCGrade['EURO0_road_6'] = [' EURO0','road','6','1.95602234454138','1.42437952291611','2.00119020715631','1.5759805360494','1.09658043300576','1.11227835802769','2.44822088935931','1.81866401458644','2.06483877797715']
  H_PCGrade['EURO0_road_-6'] = [' EURO0','road','-6','0.595018986727179','1.04573572768695','0.301981167608286','0.581771876077065','0.835845577752856','1.04311391790138','0.214002900936924','0.686391513343279','0.320866405255968']
  H_PCGrade['EURO0_road_8'] = [' EURO0','road','8','1.93451700117038','1.65579201286518','2.47462146892655','1.82713822496087','1.3160295766884','1.27987612339082','2.88832699570086','2.10166307530803','2.43659148626323']
  H_PCGrade['EURO0_road_-8'] = [' EURO0','road','-8','0.575924932153754','1.19086571964621','0.147111111111111','0.528686123428508','0.839362086335038','1.10912071897012','0.0851856207612999','0.609702193491353','0.174268392332147']
  H_PCGrade['EURO0_road_10'] = [' EURO0','road','10','1.75466184204474','1.98088448137229','2.56916760828625','2.07002485311978','1.89651960691137','1.82122905027933','3.32519241574869','2.60881816674954','2.90494989407583']
  H_PCGrade['EURO0_road_-10'] = [' EURO0','road','-10','0.527692001042609','1.13188957384079','0.0969566854990584','0.481273495743259','0.726472656769754','0.993988341025018','0.0464149907875652','0.471296756726891','0.109130690916342']
  H_PCGrade['EURO0_motorway_0'] = [' EURO0','motorway','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO0_motorway_2'] = [' EURO0','motorway','2','1.79136098581035','1.32296557596399','1.14835248081334','1.24832348682383','0.873574615765989','0.885099586621571','1.48515537089933','1.17992435612655','1.3539068285661']
  H_PCGrade['EURO0_motorway_-2'] = [' EURO0','motorway','-2','0.553668851669642','0.754445931151928','0.747921989039178','0.775131133780917','1.2057157022452','1.12711386696731','0.584300923693289','0.789878129315003','0.663434810516383']
  H_PCGrade['EURO0_motorway_4'] = [' EURO0','motorway','4','2.19216366777658','1.57075781535093','1.2779284150693','1.43238612441463','0.974219137332672','0.824408117249154','1.94596332430661','1.27790118268596','1.66681009563985']
  H_PCGrade['EURO0_motorway_-4'] = [' EURO0','motorway','-4','0.359592738461352','0.620435470137453','0.394432863658449','0.564206149969203','1.13053332388979','1.13030815482901','0.218261181387642','0.587981028996818','0.339320731483702']
  H_PCGrade['EURO0_motorway_6'] = [' EURO0','motorway','6','2.88679246711518','1.85639216640311','1.54553338662236','1.70251400898224','0.945959345562717','0.809751972942503','2.62012340641268','1.57813531848472','2.05689920097404']
  H_PCGrade['EURO0_motorway_-6'] = [' EURO0','motorway','-6','0.281557242383909','0.587130519401533','0.193590688003176','0.416402190589955','0.974573270061619','1.07412626832018','0.0467782023222169','0.473914870624962','0.133880962616209']
  H_PCGrade['EURO1_urban_0'] = [' EURO1','urban','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO1_urban_2'] = [' EURO1','urban','2','1.09851680895112','1.04771897290747','1.35855460374559','1.12870230792464','1.0249360894018','0.960553514010106','1.36839542244948','1.12357857240718','1.24272261748436']
  H_PCGrade['EURO1_urban_-2'] = [' EURO1','urban','-2','0.909632868433332','0.968295797444469','0.688585393612373','0.868774604772319','0.982864655613176','1.02916858061553','0.67391039688337','0.877654473215509','0.771423538324529']
  H_PCGrade['EURO1_urban_4'] = [' EURO1','urban','4','1.20346177737547','1.12816696353773','1.8826022010182','1.2906984318427','1.05400628149879','0.964630225080386','1.8291331872413','1.34614330730237','1.53654683480792']
  H_PCGrade['EURO1_urban_-4'] = [' EURO1','urban','-4','0.8420589431503','1.01293219139643','0.489995833714396','0.767166301605526','0.980483529325834','1.08549609554433','0.42464085707329','0.789286203589533','0.575671531992414']
  H_PCGrade['EURO1_urban_6'] = [' EURO1','urban','6','1.27670840365441','1.17591033680632','2.45922629027833','1.46039632303608','1.63289752392082','1.00763665594855','2.45137874360847','1.66981778325798','1.90467281359794']
  H_PCGrade['EURO1_urban_-6'] = [' EURO1','urban','-6','0.775153530995534','1.01541289477751','0.331514394008678','0.679577683285091','0.845781900518589','0.983578318787322','0.240777940102264','0.580969995889848','0.384006817407969']
  H_PCGrade['EURO1_road_0'] = [' EURO1','road','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO1_road_2'] = [' EURO1','road','2','1.28117528607861','1.11142321093541','1.3353747645951','1.18304604277069','1.08888579387187','0.930072379173477','1.44251999938582','1.20477268439917','1.30931530463142']
  H_PCGrade['EURO1_road_-2'] = [' EURO1','road','-2','0.793461206257698','0.950565532028947','0.758885122410546','0.868428630250835','1.21590529247911','1.39586738267569','0.711225758901838','1.0890398038106','0.816197954393759']
  H_PCGrade['EURO1_road_4'] = [' EURO1','road','4','1.65608111906697','1.26348968105066','1.69887005649718','1.38132325789535','1.09838440111421','0.882091991594677','1.93157983631981','1.4740614978306','1.65773480148458']
  H_PCGrade['EURO1_road_-4'] = [' EURO1','road','-4','0.675486934167421','0.962294291074779','0.493627118644068','0.698658818144894','1.18317548746518','1.43882792435209','0.422881447018902','0.917562724014337','0.574388351616998']
  H_PCGrade['EURO1_road_6'] = [' EURO1','road','6','1.95602234454138','1.42437952291611','2.00119020715631','1.5759805360494','1.08423398328691','1.05101564324072','2.48810785081456','1.92265610262215','2.06691315517089']
  H_PCGrade['EURO1_road_-6'] = [' EURO1','road','-6','0.595018986727179','1.04573572768695','0.301981167608286','0.581771876077065','0.897214484679666','1.19028718188186','0.197997758226235','0.528768156951519','0.318742630264583']
  H_PCGrade['EURO1_road_8'] = [' EURO1','road','8','1.93451700117038','1.65579201286518','2.47462146892655','1.82713822496087','1.30679665738162','1.21013308428671','2.93873508683035','2.22590077343897','2.43925147947343']
  H_PCGrade['EURO1_road_-8'] = [' EURO1','road','-8','0.575924932153754','1.19086571964621','0.147111111111111','0.528686123428508','0.921838440111421','1.31519962643007','0.0743931088488645','0.389737785323524','0.172162955045663']
  H_PCGrade['EURO1_road_10'] = [' EURO1','road','10','1.75466184204474','1.98088448137229','2.56916760828625','2.07002485311978','1.90643454038997','1.77854307728228','3.37494433952124','2.71080928126768','2.90742251198449']
  H_PCGrade['EURO1_road_-10'] = [' EURO1','road','-10','0.527692001042609','1.13188957384079','0.0969566854990584','0.481273495743259','0.804986072423398','1.20943264067243','0.0394306508821226','0.244859460479155','0.107848408751188']
  H_PCGrade['EURO1_motorway_0'] = [' EURO1','motorway','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO1_motorway_2'] = [' EURO1','motorway','2','1.79136098581035','1.32296557596399','1.14835248081334','1.24832348682383','0.835700332611794','0.813763680207754','1.49662821395776','1.22839879154079','1.35475636144169']
  H_PCGrade['EURO1_motorway_-2'] = [' EURO1','motorway','-2','0.553668851669642','0.754445931151928','0.747921989039178','0.775131133780917','1.23752705770551','1.19087367835281','0.57328971533517','0.742396777442095','0.662509042064972']
  H_PCGrade['EURO1_motorway_4'] = [' EURO1','motorway','4','2.19216366777658','1.57075781535093','1.2779284150693','1.43238612441463','0.89139960931313','0.671489519569653','1.97202134986226','1.37462235649547','1.66882839723896']
  H_PCGrade['EURO1_motorway_-4'] = [' EURO1','motorway','-4','0.359592738461352','0.620435470137453','0.394432863658449','0.564206149969203','1.1951850483079','1.27007976256724','0.200140610651974','0.482980866062437','0.337690672813284']
  H_PCGrade['EURO1_motorway_6'] = [' EURO1','motorway','6','2.88679246711518','1.85639216640311','1.54553338662236','1.70251400898224','0.850799852172536','0.615655722500464','2.65897612488522','1.71550855991944','2.06006900724139']
  H_PCGrade['EURO1_motorway_-6'] = [' EURO1','motorway','-6','0.281557242383909','0.587130519401533','0.193590688003176','0.416402190589955','1.06108442004118','1.27694305323688','0.0372618227731864','0.318126888217523','0.131968245543483']
  H_PCGrade['EURO2_urban_0'] = [' EURO2','urban','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO2_urban_2'] = [' EURO2','urban','2','1.3544756802989','1.31164069660862','1.1251794430089','1.18976017749748','0.986845323838774','0.944539116963594','1.35233000443554','1.23524832632726','1.24474959786409']
  H_PCGrade['EURO2_urban_-2'] = [' EURO2','urban','-2','0.716086340003151','0.727772685609532','0.818547229399943','0.806796266613091','1.01926996983122','1.04732765298218','0.686729660344897','0.763506149774249','0.769622921010339']
  H_PCGrade['EURO2_urban_4'] = [' EURO2','urban','4','2.1183011096356','1.91750687442713','1.23083548664944','1.43779962526369','1.01310031799527','0.940511231603408','1.79705372384037','1.57745601743733','1.54015053554048']
  H_PCGrade['EURO2_urban_-4'] = [' EURO2','urban','-4','0.52355443516622','0.563703024747938','0.623456790123457','0.636870733443687','1.06577338080613','1.12548412083656','0.444441457546472','0.575432041102289','0.572332546367339']
  H_PCGrade['EURO2_urban_6'] = [' EURO2','urban','6','3.42334961399086','2.79835013748854','1.41386735572782','1.72200191298955','1.56800206561031','0.978698683191325','2.40312369789917','2.00311380974622','1.90943615461421']
  H_PCGrade['EURO2_urban_-6'] = [' EURO2','urban','-6','0.375880618514934','0.438130155820348','0.511771461383865','0.51128882211381','0.946892071861495','1.03555383423703','0.257073347760051','0.337692666978048','0.380195475598246']
  H_PCGrade['EURO2_road_0'] = [' EURO2','road','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO2_road_2'] = [' EURO2','road','2','1.48795227395987','1.44540540540541','1.2362480127186','1.22933643660308','1.01060296371998','0.889767754964658','1.42993613038212','1.31270417422868','1.31108256600212']
  H_PCGrade['EURO2_road_-2'] = [' EURO2','road','-2','0.709353451615403','0.707567567567568','0.814626391096979','0.821931314100495','1.46352835973429','1.49141703130259','0.727439158682964','0.921778584392015','0.81330570848173']
  H_PCGrade['EURO2_road_4'] = [' EURO2','road','4','2.27604207019447','2.0972972972973','1.46931637519873','1.46862662112667','0.978921819110884','0.81386738471895','1.90759552912675','1.67531760435572','1.66097639261435']
  H_PCGrade['EURO2_road_-4'] = [' EURO2','road','-4','0.430744944603703','0.463243243243243','0.535612082670906','0.600940407219854','1.49291006642821','1.57270279367216','0.442118158793084','0.649001814882033','0.570486517574977']
  H_PCGrade['EURO2_road_6'] = [' EURO2','road','6','2.92008212597815','2.6627027027027','1.65373608903021','1.70229091582796','1.03008431272356','0.995287781891619','2.46038431890761','2.15444646098004','2.07013952962523']
  H_PCGrade['EURO2_road_-6'] = [' EURO2','road','-6','0.257951111799799','0.298918918918919','0.396502384737679','0.448350886617935','1.14103219213081','1.3256479299899','0.208773813456668','0.282577132486388','0.315513862281007']
  H_PCGrade['EURO2_road_8'] = [' EURO2','road','8','3.39571550321531','2.98810810810811','2.53656597774245','2.10974604368552','1.26475472662238','1.14692022887917','2.90370003303601','2.50254083484573','2.44338893205867']
  H_PCGrade['EURO2_road_-8'] = [' EURO2','road','-8','0.138093670101495','0.189189189189189','0.310810810810811','0.33721599688549','1.25114971895759','1.50471221810838','0.0817090628785376','0.127404718693285','0.169060965600765']
  H_PCGrade['EURO2_road_10'] = [' EURO2','road','10','2.46121290772449','2.53243243243243','2.69872813990461','2.47119816087585','1.92884517118038','1.74015483002356','3.34038101530668','2.93811252268603','2.91126444293178']
  H_PCGrade['EURO2_road_-10'] = [' EURO2','road','-10','0.0916750600449369','0.151891891891892','0.256279809220986','0.272014570142665','1.11944302503832','1.40760686637496','0.0439929523180266','0.0667876588021779','0.105964830988711']
  H_PCGrade['EURO2_motorway_0'] = [' EURO2','motorway','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO2_motorway_2'] = [' EURO2','motorway','2','2.04842389604047','1.78645833333333','1.73450508788159','1.27105179124701','0.665524963772889','0.744651673066374','1.48867897598256','1.33078358208955','1.35608562840852']
  H_PCGrade['EURO2_motorway_-2'] = [' EURO2','motorway','-2','0.493159151780604','0.536892361111111','0.540086339808819','0.731838816406302','1.38124094322224','1.25287986834888','0.58090801749082','0.642537313432836','0.661076947018165']
  H_PCGrade['EURO2_motorway_4'] = [' EURO2','motorway','4','3.02209063646977','2.59288194444444','1.7008942337342','1.47402179891241','0.519167435120537','0.523861766319254','1.95397630694572','1.57910447761194','1.67197611349978']
  H_PCGrade['EURO2_motorway_-4'] = [' EURO2','motorway','-4','0.219723619049072','0.243923611111111','0.360777058279371','0.46072386605877','1.48781451719141','1.40565002742732','0.212575743813985','0.276119402985075','0.335174886739766']
  H_PCGrade['EURO2_motorway_6'] = [' EURO2','motorway','6','4.46539896681445','3.88194444444444','1.97163120567376','1.66544702912929','0.450665261493874','0.431980252331322','2.63206996327966','2.00578358208955','2.06501491485593']
  H_PCGrade['EURO2_motorway_-6'] = [' EURO2','motorway','-6','0.120372984563262','0.129774305555556','0.226641998149861','0.265291171299249','1.45303649058095','1.47394404827208','0.0435064682297681','0.0811567164179104','0.129023120649469']
  H_PCGrade['EURO3_urban_0'] = [' EURO3','urban','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO3_urban_2'] = [' EURO3','urban','2','1.3544756802989','1.31164069660862','1.1251794430089','1.18976017749748','1.65037152913571','1.07518796992481','1.40449891969388','1.20661444734552','1.19744026357708']
  H_PCGrade['EURO3_urban_-2'] = [' EURO3','urban','-2','0.716086340003151','0.727772685609532','0.818547229399943','0.806796266613091','0.655455612045366','0.904761904761905','0.673101387234214','0.855178416013925','0.827382897787817']
  H_PCGrade['EURO3_urban_4'] = [' EURO3','urban','4','2.1183011096356','1.91750687442713','1.23083548664944','1.43779962526369','2.22643723113023','1.19172932330827','1.95406174908628','1.42576153176675','1.43974142526815']
  H_PCGrade['EURO3_urban_-4'] = [' EURO3','urban','-4','0.52355443516622','0.563703024747938','0.623456790123457','0.636870733443687','0.504497457958545','0.794486215538847','0.410940371140682','0.772323759791123','0.685877018422934']
  H_PCGrade['EURO3_urban_6'] = [' EURO3','urban','6','3.42334961399086','2.79835013748854','1.41386735572782','1.72200191298955','2.6421587798201','1.31453634085213','2.68586313430123','1.56362053959965','1.71488545214156']
  H_PCGrade['EURO3_urban_-6'] = [' EURO3','urban','-6','0.375880618514934','0.438130155820348','0.511771461383865','0.51128882211381','0.423934298005475','0.591478696741855','0.253457989217131','0.634986945169713','0.544022070905637']
  H_PCGrade['EURO3_road_0'] = [' EURO3','road','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO3_road_2'] = [' EURO3','road','2','1.48795227395987','1.44540540540541','1.2362480127186','1.22933643660308','1.62237561115905','1.11442193087008','1.48105383465698','1.5031342791158','1.27100953155535']
  H_PCGrade['EURO3_road_-2'] = [' EURO3','road','-2','0.709353451615403','0.707567567567568','0.814626391096979','0.821931314100495','0.559677883232672','0.879618593563766','0.641751897547109','0.680963378422963','0.765884999224888']
  H_PCGrade['EURO3_road_4'] = [' EURO3','road','4','2.27604207019447','2.0972972972973','1.46931637519873','1.46862662112667','1.872878918608','1.23480333730632','2.03005304340181','1.89211481359287','1.57338432567566']
  H_PCGrade['EURO3_road_-4'] = [' EURO3','road','-4','0.430744944603703','0.463243243243243','0.535612082670906','0.600940407219854','0.332470520563704','0.721096543504172','0.387392667702136','0.488947542065325','0.58740890410862']
  H_PCGrade['EURO3_road_6'] = [' EURO3','road','6','2.92008212597815','2.6627027027027','1.65373608903021','1.70229091582796','1.55162496404947','1.51489868891538','2.46842305776163','2.10227647641043','1.91492886248904']
  H_PCGrade['EURO3_road_-6'] = [' EURO3','road','-6','0.257951111799799','0.298918918918919','0.396502384737679','0.448350886617935','0.210238711532931','0.569725864123957','0.207191630278698','0.391949851534147','0.470515521343274']
  H_PCGrade['EURO3_road_8'] = [' EURO3','road','8','3.39571550321531','2.98810810810811','2.53656597774245','2.10974604368552','1.8622375611159','1.78426698450536','2.85544940362805','2.5859452325965','2.244168215712']
  H_PCGrade['EURO3_road_-8'] = [' EURO3','road','-8','0.138093670101495','0.189189189189189','0.310810810810811','0.33721599688549','0.13920046016681','0.470798569725864','0.0736453418515371','0.368525239194985','0.382663277327858']
  H_PCGrade['EURO3_road_10'] = [' EURO3','road','10','2.46121290772449','2.53243243243243','2.69872813990461','2.47119816087585','2.50071901064136','2.38974970202622','3.27357339500055','3.55031342791158','2.69067809884691']
  H_PCGrade['EURO3_road_-10'] = [' EURO3','road','-10','0.0916750600449369','0.151891891891892','0.256279809220986','0.272014570142665','0.104400345125108','0.382598331346842','0.0399876915863201','0.437809303860112','0.313248395052945']
  H_PCGrade['EURO3_motorway_0'] = [' EURO3','motorway','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO3_motorway_2'] = [' EURO3','motorway','2','2.04842389604047','1.78645833333333','1.73450508788159','1.27105179124701','3.22306238185255','1.12372881355932','1.71452400631152','1.28669997620747','1.31467428056203']
  H_PCGrade['EURO3_motorway_-2'] = [' EURO3','motorway','-2','0.493159151780604','0.536892361111111','0.540086339808819','0.731838816406302','0.103969754253308','0.922033898305085','0.527041851378766','0.761598857958601','0.706457983298511']
  H_PCGrade['EURO3_motorway_4'] = [' EURO3','motorway','4','3.02209063646977','2.59288194444444','1.7008942337342','1.47402179891241','5.86200378071834','1.23220338983051','2.42840183334586','1.54794194622888','1.60159433823871']
  H_PCGrade['EURO3_motorway_-4'] = [' EURO3','motorway','-4','0.219723619049072','0.243923611111111','0.360777058279371','0.46072386605877','0.0396975425330813','0.735593220338983','0.192005409873018','0.553890078515346','0.433643079291977']
  H_PCGrade['EURO3_motorway_6'] = [' EURO3','motorway','6','4.46539896681445','3.88194444444444','1.97163120567376','1.66544702912929','6.93950850661626','1.45762711864407','3.33532196258171','1.90530573399952','1.91923769282302']
  H_PCGrade['EURO3_motorway_-6'] = [' EURO3','motorway','-6','0.120372984563262','0.129774305555556','0.226641998149861','0.265291171299249','0.0132325141776938','0.38135593220339','0.045277631677812','0.428979300499643','0.276817376586219']
  H_PCGrade['EURO4_urban_0'] = [' EURO4','urban','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO4_urban_2'] = [' EURO4','urban','2','1.2956254069733','1.09970238095238','1.12712727272727','1.18017951421446','1.65037152913571','1.07518796992481','1.40449891969388','1.20661444734552','1.19744026357708']
  H_PCGrade['EURO4_urban_-2'] = [' EURO4','urban','-2','0.780855975847984','0.930059523809524','0.842036363636363','0.830797542670614','0.655455612045366','0.904761904761905','0.673101387234214','0.855178416013925','0.827382897787817']
  H_PCGrade['EURO4_urban_4'] = [' EURO4','urban','4','1.81968862842598','1.20684523809524','1.11970909090909','1.39952518606995','2.22643723113023','1.19172932330827','1.95406174908628','1.42576153176675','1.43974142526815']
  H_PCGrade['EURO4_urban_-4'] = [' EURO4','urban','-4','0.614455691706624','0.895833333333333','0.661527272727273','0.66169546390068','0.504497457958545','0.794486215538847','0.410940371140682','0.772323759791123','0.685877018422934']
  H_PCGrade['EURO4_urban_6'] = [' EURO4','urban','6','2.69283135026342','1.39583333333333','1.20334545454545','1.65901792418695','2.6421587798201','1.31453634085213','2.68586313430123','1.56362053959965','1.71488545214156']
  H_PCGrade['EURO4_urban_-6'] = [' EURO4','urban','-6','0.410821050139111','0.892857142857143','0.535272727272727','0.537354908239212','0.423934298005475','0.591478696741855','0.253457989217131','0.634986945169713','0.544022070905637']
  H_PCGrade['EURO4_road_0'] = [' EURO4','road','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO4_road_2'] = [' EURO4','road','2','1.27820522039261','1.21428571428571','1.0608029705268','1.22919324792858','1.62237561115905','1.11442193087008','1.48105383465698','1.5031342791158','1.27100953155535']
  H_PCGrade['EURO4_road_-2'] = [' EURO4','road','-2','0.836772848205939','0.826007326007326','0.851937804595034','0.804298792715768','0.559677883232672','0.879618593563766','0.641751897547109','0.680963378422963','0.765884999224888']
  H_PCGrade['EURO4_road_4'] = [' EURO4','road','4','1.55349823829726','1.48351648351648','1.11557205848225','1.47397941929055','1.872878918608','1.23480333730632','2.03005304340181','1.89211481359287','1.57338432567566']
  H_PCGrade['EURO4_road_-4'] = [' EURO4','road','-4','0.665024807650823','0.686813186813187','0.69923416105825','0.624062988681798','0.332470520563704','0.721096543504172','0.387392667702136','0.488947542065325','0.58740890410862']
  H_PCGrade['EURO4_road_6'] = [' EURO4','road','6','1.65934421514345','1.6978021978022','1.28057553956835','1.70874302831437','1.55162496404947','1.51489868891538','2.46842305776163','2.10227647641043','1.91492886248904']
  H_PCGrade['EURO4_road_-6'] = [' EURO4','road','-6','0.502480765082333','0.593406593406593','0.578556509631005','0.4858513736544','0.210238711532931','0.569725864123957','0.207191630278698','0.391949851534147','0.470515521343274']
  H_PCGrade['EURO4_road_8'] = [' EURO4','road','8','2.69198964550226','2.06043956043956','1.36249709909492','2.034804974475','1.8622375611159','1.78426698450536','2.85544940362805','2.5859452325965','2.244168215712']
  H_PCGrade['EURO4_road_-8'] = [' EURO4','road','-8','0.29395268569785','0.648351648351648','0.481782316082618','0.383851472875584','0.13920046016681','0.470798569725864','0.0736453418515371','0.368525239194985','0.382663277327858']
  H_PCGrade['EURO4_road_10'] = [' EURO4','road','10','2.93532034227368','2.19047619047619','2.09979113483407','2.39892278635443','2.50071901064136','2.38974970202622','3.27357339500055','3.55031342791158','2.69067809884691']
  H_PCGrade['EURO4_road_-10'] = [' EURO4','road','-10','0.177752211116704','0.677655677655678','0.412624738918543','0.312126261052331','0.104400345125108','0.382598331346842','0.0399876915863201','0.437809303860112','0.313248395052945']
  H_PCGrade['EURO4_motorway_0'] = [' EURO4','motorway','0','1','1','1','1','1','1','1','1','1']
  H_PCGrade['EURO4_motorway_2'] = [' EURO4','motorway','2','1.23478784231116','1.41521739130435','0.922134387351779','1.28099324789754','3.22306238185255','1.12372881355932','1.71452400631152','1.28669997620747','1.31467428056203']
  H_PCGrade['EURO4_motorway_-2'] = [' EURO4','motorway','-2','0.722118567559434','0.73695652173913','0.988537549407115','0.739624508920799','0.103969754253308','0.922033898305085','0.527041851378766','0.761598857958601','0.706457983298511']
  H_PCGrade['EURO4_motorway_4'] = [' EURO4','motorway','4','1.36373758651821','1.90217391304348','1.0300395256917','1.53049176046689','5.86200378071834','1.23220338983051','2.42840183334586','1.54794194622888','1.60159433823871']
  H_PCGrade['EURO4_motorway_-4'] = [' EURO4','motorway','-4','0.434757749021968','0.543478260869565','0.898418972332016','0.482802955180271','0.0396975425330813','0.735593220338983','0.192005409873018','0.553890078515346','0.433643079291977']
  H_PCGrade['EURO4_motorway_6'] = [' EURO4','motorway','6','1.78495335540175','2.76304347826087','1.00632411067194','1.92266798882724','6.93950850661626','1.45762711864407','3.33532196258171','1.90530573399952','1.91923769282302']
  H_PCGrade['EURO4_motorway_-6'] = [' EURO4','motorway','-6','0.32542882937105','0.376086956521739','0.583794466403162','0.299322140386202','0.0132325141776938','0.38135593220339','0.045277631677812','0.428979300499643','0.276817376586219']

##   LIGHT DUTY VEHICLES
def load_LDV():
  H_LDV['1_CO'] = [' 1','DIESEL','N1-I','EURO0','CO','0','0','0','0','0','0','0.000583','-0.0699','2.53','0','0','100']
  H_LDV['1_CO2'] = [' 1','DIESEL','N1-I','EURO0','CO2','0','0','0','0','0','0','0.146','-15.6','590','0','0','100']
  H_LDV['1_FC'] = [' 1','DIESEL','N1-I','EURO0','FC','0','0','0','0','0','0','0.0475','-5.05','191','0','0','100']
  H_LDV['1_HC'] = [' 1','DIESEL','N1-I','EURO0','HC','0','0','0','0','0','0','0.0000992','-0.0115','0.485','0','0','100']
  H_LDV['1_NOx'] = [' 1','DIESEL','N1-I','EURO0','NOx','0','0','0','0','0','0','0.000441','-0.0446','1.69','0','0','100']
  H_LDV['1_PMm'] = [' 1','DIESEL','N1-I','EURO0','PMm','0','0','0','0','0','0','0.000058','-0.0086','0.45','0','0','100']
  H_LDV['4_CO'] = [' 4','DIESEL','N1-I','EURO1','CO','0','0','0','-0.00000383','0.00033','0.0128','0.000184','-0.025','1.26','0','0','25']
  H_LDV['4_CO2'] = [' 4','DIESEL','N1-I','EURO1','CO2','0','0','0','-0.0012','0.0654','6.0995','0.0249','-2.3223','176.92','0','0','25']
  H_LDV['4_FC'] = [' 4','DIESEL','N1-I','EURO1','FC','-0.0000586','0.00884','-0.491','0.0019','-0.2989','18.565','0','0.4963','-32.605','0','7','25']
  H_LDV['4_HC'] = [' 4','DIESEL','N1-I','EURO1','HC','0','0.0000242','-0.00243','0','-0.000844','0.0841','0','0.0041','-0.3375','0.04','7','25']
  H_LDV['4_NOx'] = [' 4','DIESEL','N1-I','EURO1','NOx','0','0.000048','-0.0068','0','-0.00173','0.246','0','0.0102','-0.955','0','7','25']
  H_LDV['4_PMm'] = [' 4','DIESEL','N1-I','EURO1','PMm','0','0','0','0','0','0','-0.0000171','0.002418670438472','0.0231','0','0','100']
  H_LDV['7_CO'] = [' 7','DIESEL','N1-I','EURO2','CO','0','0','0','0','0','0','0.0000866','-0.0156','0.912','0','0','100']
  H_LDV['7_CO2'] = [' 7','DIESEL','N1-I','EURO2','CO2','0','0','0','0','0','0','0.0245','-3.4055','273.56','0','0','100']
  H_LDV['7_FC'] = [' 7','DIESEL','N1-I','EURO2','FC','0','0','0','0','0','0','0.00835','-1.2','84.3','0','0','100']
  H_LDV['7_HC'] = [' 7','DIESEL','N1-I','EURO2','HC','0','0','0','0','0','0','0.0000347','-0.00617','0.293','0','0','100']
  H_LDV['7_NOx'] = [' 7','DIESEL','N1-I','EURO2','NOx','0','0','0','0','0','0','0.000223','-0.0289','1.47','0','0','100']
  H_LDV['7_PMm'] = [' 7','DIESEL','N1-I','EURO2','PMm','0','0','0','0','0','0','0.000015','-0.00219','0.113','0','0','100']
  H_LDV['2_CO'] = [' 2','DIESEL','N1-II','EURO0','CO','-0.000000211','0.0000376','-0.00203','0.0000133','-0.0027','0.161','0.000146','-0.0114','0.398','0','0','62']
  H_LDV['2_CO2'] = [' 2','DIESEL','N1-II','EURO0','CO2','0.00000621','-0.00235','0.0758','0.000249','0.0725','-4.04','0.0211','-3.7377','357.35','0','0','62']
  H_LDV['2_FC'] = [' 2','DIESEL','N1-II','EURO0','FC','0.00000245','-0.000632','0.0272','0','0.0225','-1.77','0.00798','-1.31','125','0','0','62']
  H_LDV['2_HC'] = [' 2','DIESEL','N1-II','EURO0','HC','-0.0000000529','0.0000109','-0.000856','0.00000348','-0.000724','0.0576','-0.0000439','0.00554','-0.138','0.05','6','62']
  H_LDV['2_NOx'] = [' 2','DIESEL','N1-II','EURO0','NOx','0','-0.0000323','0.00242','0','0.00141','-0.0683','0','-0.0117','1.61','0.5','0','62']
  H_LDV['2_PMm'] = [' 2','DIESEL','N1-II','EURO0','PMm','-0.0000000133','0.00000948','-0.00102','0.00000472','-0.00119','0.0835','0.00000282','0.00536','-0.341','0','6','50']
  H_LDV['5_CO'] = [' 5','DIESEL','N1-II','EURO1','CO','0.00000004998','0.0000183','-0.002301','0.000005086','-0.001822','0.1435','0.000091','-0.0148','0.499','0','5','32']
  H_LDV['5_CO2'] = [' 5','DIESEL','N1-II','EURO1','CO2','0','0','0','0.00017','-0.0137','1.38','0.0281','-4.13','306','0','0','32']
  H_LDV['5_FC'] = [' 5','DIESEL','N1-II','EURO1','FC','-0.00000299','0.000784','-0.00845','0.0000833','-0.0216','0.359','0.0117','-1.6','108','0','0','32']
  H_LDV['5_HC'] = [' 5','DIESEL','N1-II','EURO1','HC','0','0','0','0','0','0','0.0000251','-0.00556','0.342','0','0','100']
  H_LDV['5_NOx'] = [' 5','DIESEL','N1-II','EURO1','NOx','-0.000000647','0.000101','-0.00249','0.0000285','-0.00414','0.106','0.0000183','0.00266','0.64','0','0','32']
  H_LDV['5_PMm'] = [' 5','DIESEL','N1-II','EURO1','PMm','0','0.00000485','-0.000103','0','-0.000185','0.00648','0','0.00117','0.0288','0','0','32']
  H_LDV['8_CO'] = [' 8','DIESEL','N1-II','EURO2','CO','0','0','0','0','0','0','0.0000165','-0.00519','0.412','0','0','100']
  H_LDV['8_CO2'] = [' 8','DIESEL','N1-II','EURO2','CO2','0','0','0','0','0','0','0.0343','-5.1159','367.86','0','0','100']
  H_LDV['8_FC'] = [' 8','DIESEL','N1-II','EURO2','FC','0','0','0','0','0','0','0.0105','-1.513','107.74','0','0','100']
  H_LDV['8_HC'] = [' 8','DIESEL','N1-II','EURO2','HC','0','0','0','0','0','0','-0.000000549','-0.000438','0.0729','0','0','100']
  H_LDV['8_NOx'] = [' 8','DIESEL','N1-II','EURO2','NOx','0','0','0','0','0','0','0.000185','-0.0216','1.42','0','0','100']
  H_LDV['8_PMm'] = [' 8','DIESEL','N1-II','EURO2','PMm','0','0','0','0','0','0','0.00000503','-0.0007','0.049','0','0','100']
  H_LDV['3_CO'] = [' 3','DIESEL','N1-III','EURO0','CO','0.000000137','-0.0000127','0.000398','-0.0000122','0.00112','-0.0373','0.000412','-0.0486','2.25','0','0','100']
  H_LDV['3_CO2'] = [' 3','DIESEL','N1-III','EURO0','CO2','-0.00000406','0.000771','0.0286','0.000384','-0.0611','-4.07','0.0444','-6.1129','509.84','0','0','100']
  H_LDV['3_FC'] = [' 3','DIESEL','N1-III','EURO0','FC','-0.00000274','0.000379','0.00749','0.000252','-0.0325','-1.11','0.0115','-1.6528','161.8','0','0','100']
  H_LDV['3_HC'] = [' 3','DIESEL','N1-III','EURO0','HC','0.00000000667','-0.000000678','0.0000595','-0.000000616','0.0000976','-0.00861','0.0000377','-0.00682','0.436','0','0','100']
  H_LDV['3_NOx'] = [' 3','DIESEL','N1-III','EURO0','NOx','0.000000266','-0.0000483','0.00266','-0.0000262','0.00479','-0.262','0.0009','-0.1511','8.3427','0','0','100']
  H_LDV['3_PMm'] = [' 3','DIESEL','N1-III','EURO0','PMm','0','0.00000134','0.0000289','0','-0.000136','-0.00388','0.0000149','0.000651','0.352','0','0','100']
  H_LDV['6_CO'] = [' 6','DIESEL','N1-III','EURO1','CO','0.000000179','-0.0000187','0.00043','-0.000021','0.00266','-0.0694','0.00288','-0.195','4.05','0','7','50']
  H_LDV['6_CO2'] = [' 6','DIESEL','N1-III','EURO1','CO2','0.0000256','-0.00189','0.0654','0.00074','0.108','-7.2126','0.304','-19.8','662.1','0','7','50']
  H_LDV['6_FC'] = [' 6','DIESEL','N1-III','EURO1','FC','0.0000237','-0.0013','0.0279','-0.00273','0.176','-3.8935','0.193','-11.1','262.46','0','7','50']
  H_LDV['6_HC'] = [' 6','DIESEL','N1-III','EURO1','HC','0.0000001219','-0.00002872','0.002059','-0.000007833','0.001743','-0.1157','0.0001357','-0.02788','1.679','0','7','50']
  H_LDV['6_NOx'] = [' 6','DIESEL','N1-III','EURO1','NOx','0','0','0','0','0','0','0.000375','-0.0451','2.24','0','0','100']
  H_LDV['6_PMm'] = [' 6','DIESEL','N1-III','EURO1','PMm','0','0','0','-0.000002176','0.0003695','-0.0176','0.0000898','-0.0133','0.642','0.03','0','50']
  H_LDV['9_CO'] = [' 9','DIESEL','N1-III','EURO2','CO','0.000000925','-0.00014','0.00475','-0.0000336','0.00514','-0.19','0.000249','-0.04','1.88','0.01','0','30']
  H_LDV['9_CO2'] = [' 9','DIESEL','N1-III','EURO2','CO2','0.000208','-0.0251','0.331','-0.00408','0.583','-9.1','0.0549','-7.89','436','0','0','30']
  H_LDV['9_FC'] = [' 9','DIESEL','N1-III','EURO2','FC','-0.0000174','0.00224','-0.111','0.000685','-0.0821','4.54','0.00866','-1.19','87.6','0','0','30']
  H_LDV['9_HC'] = [' 9','DIESEL','N1-III','EURO2','HC','0.000000185','-0.0000267','0.00108','-0.00000687','0.00102','-0.0435','0.0000581','-0.00899','0.427','0.01','0','30']
  H_LDV['9_NOx'] = [' 9','DIESEL','N1-III','EURO2','NOx','0','0','0','0','0','0','0.000403','-0.0482','2.25','0','0','100']
  H_LDV['9_PMm'] = [' 9','DIESEL','N1-III','EURO2','PMm','-0.0000000852','0.0000126','-0.000342','0.00000249','-0.000336','0.00883','0.00000898','-0.00125','0.114','0','0','30']
  H_LDV['10_CO'] = [' 10','PETROL','N1-I','EURO0','CO','0','0','0','0','0','0','0.00775','-0.889','32.4','0','0','100']
  H_LDV['10_CO2'] = [' 10','PETROL','N1-I','EURO0','CO2','0','0','0','0','0','0','0.0219','-3.388','307.69','0','0','100']
  H_LDV['10_FC'] = [' 10','PETROL','N1-I','EURO0','FC','0','0','0','0','0','0','0.0092','-1.233','103.07','0','0','100']
  H_LDV['10_HC'] = [' 10','PETROL','N1-I','EURO0','HC','0','0','0','0','0','0','0.000195','-0.0451','3.22','0','0','100']
  H_LDV['10_NOx'] = [' 10','PETROL','N1-I','EURO0','NOx','0','0','0','0','0','0','-0.0000224','0.0271','1.04','0','0','100']
  H_LDV['12_CO'] = [' 12','PETROL','N1-I','EURO1','CO','0','0','0','0','0','0','0.00309','-0.273','8.07','0','0','100']
  H_LDV['12_CO2'] = [' 12','PETROL','N1-I','EURO1','CO2','0','0','0','0','0','0','0.0372','-5.373','398.66','0','0','100']
  H_LDV['12_FC'] = [' 12','PETROL','N1-I','EURO1','FC','0','0','0','0','0','0','0.0092','-1.233','103.07','0','0','100']
  H_LDV['12_HC'] = [' 12','PETROL','N1-I','EURO1','HC','0','0','0','0','0','0','0.0000513','-0.00793','0.422','0','0','100']
  H_LDV['12_NOx'] = [' 12','PETROL','N1-I','EURO1','NOx','0','0','0','0','0','0','-0.00000815','-0.00103','0.729','0','0','100']
  H_LDV['11_CO'] = [' 11','PETROL','N1-II','EURO0','CO','0','0','0','0.0000197','-0.00442','0.35','0.0087','-1.2106','42.747','0','0','100']
  H_LDV['11_CO2'] = [' 11','PETROL','N1-II','EURO0','CO2','0','0','0','0','0','0','0.055','-8.0246','486.46','0','0','100']
  H_LDV['11_FC'] = [' 11','PETROL','N1-II','EURO0','FC','0','0','0','0','0','0','0.023','-3.3138','182.39','0','0','100']
  H_LDV['11_HC'] = [' 11','PETROL','N1-II','EURO0','HC','0','0','0','0','0','0','0.000891','-0.142','5.8','0','0','100']
  H_LDV['11_NOx'] = [' 11','PETROL','N1-II','EURO0','NOx','0','0','0','-0.0000088','0.00143','0.00447','0.000338','-0.0394','1.83','0','0','100']
  H_LDV['13_CO'] = [' 13','PETROL','N1-II','EURO1','CO','-0.00000756','0.000692','-0.0129','0.000486','-0.0464','0.974','-0.00508','0.497','-9.28','0.01','12','43']
  H_LDV['13_CO2'] = [' 13','PETROL','N1-II','EURO1','CO2','-0.0000196','0.00318','-0.111','0.00157','-0.244','8.82','0.0421','-5.7','371','0','0','100']
  H_LDV['13_FC'] = [' 13','PETROL','N1-II','EURO1','FC','0','0','0','0','0','0','0.0587','-8.578','501.9','0','0','100']
  H_LDV['13_HC'] = [' 13','PETROL','N1-II','EURO1','HC','0','0','0','0','0','0','0.0000581','-0.00835','0.396','0','0','100']
  H_LDV['13_NOx'] = [' 13','PETROL','N1-II','EURO1','NOx','0','0','0','0','0','0','0.0000943','-0.00658','0.484','0','0','100']
  H_LDV['15_CO'] = [' 15','PETROL','N1-II','EURO2','CO','0.000000837','-0.000143','0.00656','-0.0000119','0.00552','-0.255','0.000686','-0.112','4.63','0','4','56']
  H_LDV['15_CO2'] = [' 15','PETROL','N1-II','EURO2','CO2','0','0','0','0','0','0','0.0848','-12.8','635','0','0','100']
  H_LDV['15_FC'] = [' 15','PETROL','N1-II','EURO2','FC','0','0','0','0','0','0','0.0248','-3.719','191.22','0','0','100']
  H_LDV['15_HC'] = [' 15','PETROL','N1-II','EURO2','HC','0.000000169','-0.0000332','0.00183','-0.00000585','0.00124','-0.066','0.000071','-0.0133','0.641','0','4','56']
  H_LDV['15_NOx'] = [' 15','PETROL','N1-II','EURO2','NOx','0.000000429','-0.0000358','0.00145','-0.0000156','0.00146','-0.0516','0.000152','-0.0161','0.684','0','0','56']
  H_LDV['14_CO'] = [' 14','PETROL','N1-III','EURO1','CO','0','0','0','-0.000141','0.0127','0.0524','0.00631','-0.642','15.3','1','7','87']
  H_LDV['14_CO2'] = [' 14','PETROL','N1-III','EURO1','CO2','0','0','0','0','0','0','0.0803','-11.572','632.69','0','0','100']
  H_LDV['14_FC'] = [' 14','PETROL','N1-III','EURO1','FC','0','0','0','0','0','0','0.0272','-3.8434','208.16','0','0','100']
  H_LDV['14_HC'] = [' 14','PETROL','N1-III','EURO1','HC','0','0','0','0','0','0','0.000114','-0.0141','0.692','0','0','100']
  H_LDV['14_NOx'] = [' 14','PETROL','N1-III','EURO1','NOx','0','0','0','0','0','0','0.0000952','-0.00896','0.565','0','0','100']
  H_LDV['16_CO'] = [' 16','PETROL','N1-III','EURO2','CO','0','0','0','0','0','0','0.00074','-0.122','6.16','0','0','100']
  H_LDV['16_CO2'] = [' 16','PETROL','N1-III','EURO2','CO2','0','0','0','0','0','0','0.0876','-13.253','644.56','0','0','100']
  H_LDV['16_FC'] = [' 16','PETROL','N1-III','EURO2','FC','0','0','0','0','0','0','0.0275','-4.1999','207.56','0','0','100']
  H_LDV['16_HC'] = [' 16','PETROL','N1-III','EURO2','HC','0','0','0','0','0','0','0.0000101','-0.00556','0.484','0.01','0','100']
  H_LDV['16_NOx'] = [' 16','PETROL','N1-III','EURO2','NOx','0','0','0','0','0','0','0.000012','-0.00543','0.535','0','0','100']

### LOAD SHIP
def load_Ship():
  # VehicleID,WeightGroup,DWTSpeed, GTSpeed,  DWTk,  DWTn, GTk , GTn, Description of type 
  H_SHIP['1_1'] = [' 1','1','11','12','52.842','0.4044','38.764','0.5052','Bulk SHIP']
  H_SHIP['2_1'] = [' 2','1','13','12','18.997','0.5646','19.414','0.5602','Dry cargo']
  H_SHIP['3_1'] = [' 3','1','0','0','0','0','0','0','Container']
  H_SHIP['4_1'] = [' 4','1','14','13','74.333','0.4527','75.99','0.4279','Ro-Ro']
  H_SHIP['5_1'] = [' 5','1','0','16','0','0','44.127','0.5472','Reefer']
  H_SHIP['6_1'] = [' 6','1','20','16','290.14','0.4739','38.518','0.5738','Cargo ferry']
  H_SHIP['7_1'] = [' 7','1','0','0','0','0','0','0','Passenger ferry']
  H_SHIP['8_1'] = [' 8','1','30','31','152.9','0.7392','47.013','0.7052','High-Speed ferry']
  H_SHIP['9_1'] = [' 9','1','13','10','19.463','0.5656','30.878','0.4703','Tanker']
  H_SHIP['10_1'] = [' 10','1','17','0','70.335','0.4815','0','0','Gas tanker']
  H_SHIP['11_1'] = [' 11','1','15','19','134.28','0.4553','20.572','0.6528','Cruise vessel']
  H_SHIP['12_1'] = [' 12','1','14','15','71.328','0.4563','118.43','0.3946','Vehicle carrier']
  H_SHIP['1_2'] = [' 1','2','12','13','44.177','0.4478','53.396','0.4852','Bulk H_SHIP']
  H_SHIP['2_2'] = [' 2','2','13','13','18.997','0.5646','20.299','0.5773','Dry cargo']
  H_SHIP['3_2'] = [' 3','2','14','14','3.8556','0.7612','44.842','0','Container']
  H_SHIP['4_2'] = [' 4','2','16','15','72.454','0.4949','115.36','0.41','Ro-Ro']
  H_SHIP['5_2'] = [' 5','2','15','16','34.009','0.5437','44.127','0.5472','Reefer']
  H_SHIP['6_2'] = [' 6','2','19','18','290.14','0.4739','38.518','0.5738','Cargo ferry']
  H_SHIP['7_2'] = [' 7','2','18','26','299.51','0.3402','87.564','0.5742','Passenger ferry']
  H_SHIP['8_2'] = [' 8','2','0','0','0','0','0','0','High-Speed ferry']
  H_SHIP['9_2'] = [' 9','2','13','13','19.463','0.5656','51.742','0.4763','Tanker']
  H_SHIP['10_2'] = [' 10','2','13','13','41.119','0.4929','60.473','0.4472','Gas tanker']
  H_SHIP['11_2'] = [' 11','2','18','15','111.4','0.5854','10.974','0.6852','Cruise vessel']
  H_SHIP['12_2'] = [' 12','2','16','15','251.55','0.3546','118.43','0.3946','Vehicle carrier']
  H_SHIP['1_3'] = [' 1','3','13','14','37.042','0.4879','53.396','0.4852','Bulk H_SHIP']
  H_SHIP['2_3'] = [' 2','3','14','15','29.68','0.5305','27.881','0.5725','Dry cargo']
  H_SHIP['3_3'] = [' 3','3','16','17','19.536','0.6089','77.312','0.4946','Container']
  H_SHIP['4_3'] = [' 4','3','18','16','101.46','0.497','130.25','0.4159','Ro-Ro']
  H_SHIP['5_3'] = [' 5','3','19','19','112.03','0.4697','217.46','0.3974','Reefer']
  H_SHIP['6_3'] = [' 6','3','22','20','290.14','0.4739','84.472','0.5393','Cargo ferry']
  H_SHIP['7_3'] = [' 7','3','19','17','215.61','0.5162','140.32','0.3801','Passenger ferry']
  H_SHIP['8_3'] = [' 8','3','0','0','0','0','0','0','High-Speed ferry']
  H_SHIP['9_3'] = [' 9','3','13','14','43.715','0.5095','19.535','0.5787','Tanker']
  H_SHIP['10_3'] = [' 10','3','15','15','52.035','0.4911','74.578','0.4636','Gas tanker']
  H_SHIP['11_3'] = [' 11','3','21','17','123.02','0.6091','17.658','0.6516','Cruise vessel']
  H_SHIP['12_3'] = [' 12','3','19','16','756.85','0.2826','210.96','0.3492','Vehicle carrier']
  H_SHIP['1_4'] = [' 1','4','14','15','27.429','0.5329','60.935','0.4871','Bulk H_SHIP']
  H_SHIP['2_4'] = [' 2','4','16','17','44.297','0.5151','55.548','0.5275','Dry cargo']
  H_SHIP['3_4'] = [' 3','4','19','19','160.65','0.433','132.25','0.4644','Container']
  H_SHIP['4_4'] = [' 4','4','18','18','101.46','0.497','189.44','0.4142','Ro-Ro']
  H_SHIP['5_4'] = [' 5','4','21','21','172.15','0.4488','239.9','0.4087','Reefer']
  H_SHIP['6_4'] = [' 6','4','19','20','290.14','0.4739','84.472','0.5393','Cargo ferry']
  H_SHIP['7_4'] = [' 7','4','23','21','215.61','0.5162','69.114','0.554','Passenger ferry']
  H_SHIP['8_4'] = [' 8','4','0','0','0','0','0','0','High-Speed ferry']
  H_SHIP['9_4'] = [' 9','4','14','15','43.715','0.5095','47.845','0.4911','Tanker']
  H_SHIP['10_4'] = [' 10','4','16','16','70.171','0.4712','95.104','0.4501','Gas tanker']
  H_SHIP['11_4'] = [' 11','4','23','19','91.224','0.696','20.572','0.6528','Cruise vessel']
  H_SHIP['12_4'] = [' 12','4','19','18','756.85','0.2826','629.84','0.2654','Vehicle carrier']
  H_SHIP['1_5'] = [' 1','5','15','15','69.407','0.451','60.935','0.4871','Bulk H_SHIP']
  H_SHIP['2_5'] = [' 2','5','15','15','44.083','0.5012','27.881','0.5725','Dry cargo']
  H_SHIP['3_5'] = [' 3','5','22','22','384.24','0.3892','155.96','0.4813','Container']
  H_SHIP['4_5'] = [' 4','5','20','21','308.55','0.4106','664.6','0.3268','Ro-Ro']
  H_SHIP['5_5'] = [' 5','5','21','21','172.15','0.4488','239.9','0.4087','Reefer']
  H_SHIP['6_5'] = [' 6','5','18','23','166.08','0.454','84.472','0.5393','Cargo ferry']
  H_SHIP['7_5'] = [' 7','5','0','23','0','0','69.114','0.554','Passenger ferry']
  H_SHIP['8_5'] = [' 8','5','0','0','0','0','0','0','High-Speed ferry']
  H_SHIP['9_5'] = [' 9','5','15','15','52.8','0.5082','47.845','0.4911','Tanker']
  H_SHIP['10_5'] = [' 10','5','17','17','70.335','0.4815','135.15','0.4269','Gas tanker']
  H_SHIP['11_5'] = [' 11','5','0','21','0','0','30.969','0.6306','Cruise vessel']
  H_SHIP['12_5'] = [' 12','5','19','19','756.85','0.2826','1187.5','0.2156','Vehicle carrier']
  H_SHIP['1_6'] = [' 1','6','15','15','69.407','0.451','60.935','0.4871','Bulk H_SHIP']
  H_SHIP['2_6'] = [' 2','6','15','15','44.083','0.5012','27.881','0.5725','Dry cargo']
  H_SHIP['3_6'] = [' 3','6','24','24','164.05','0.4972','53.756','0.605','Container']
  H_SHIP['4_6'] = [' 4','6','0','22','0','0','1179.4','0.2794','Ro-Ro']
  H_SHIP['5_6'] = [' 5','6','0','0','0','0','0','0','Reefer']
  H_SHIP['6_6'] = [' 6','6','0','22','0','0','84.472','0.5393','Cargo ferry']
  H_SHIP['7_6'] = [' 7','6','0','0','0','0','0','0','Passenger ferry']
  H_SHIP['8_6'] = [' 8','6','0','0','0','0','0','0','High-Speed ferry']
  H_SHIP['9_6'] = [' 9','6','15','15','52.8','0.5082','39.921','0.5025','Tanker']
  H_SHIP['10_6'] = [' 10','6','19','20','96.644','0.4902','604.05','0.3252','Gas tanker']
  H_SHIP['11_6'] = [' 11','6','0','22','0','0','41.075','0.6257','Cruise vessel']
  H_SHIP['12_6'] = [' 12','6','15','20','141.91','0.4007','2058.7','0.1787','Vehicle carrier']
  H_SHIP['1_7'] = [' 1','7','15','14','69.407','0.451','53.396','0.4852','Bulk H_SHIP']
  H_SHIP['2_7'] = [' 2','7','0','0','0','0','0','0','Dry cargo']
  H_SHIP['3_7'] = [' 3','7','25','25','72.141','0.5869','34.97','0.6562','Container']
  H_SHIP['4_7'] = [' 4','7','0','0','0','0','0','0','Ro-Ro']
  H_SHIP['5_7'] = [' 5','7','0','0','0','0','0','0','Reefer']
  H_SHIP['6_7'] = [' 6','7','0','0','0','0','0','0','Cargo ferry']
  H_SHIP['7_7'] = [' 7','7','0','0','0','0','0','0','Passenger ferry']
  H_SHIP['8_7'] = [' 8','7','0','0','0','0','0','0','High-Speed ferry']
  H_SHIP['9_7'] = [' 9','7','15','15','52.8','0.5082','39.921','0.5025','Tanker']
  H_SHIP['10_7'] = [' 10','7','0','20','0','0','604.05','0.3252','Gas tanker']
  H_SHIP['11_7'] = [' 11','7','0','22','0','0','41.075','0.6257','Cruise vessel']
  H_SHIP['12_7'] = [' 12','7','0','0','0','0','0','0','Vehicle carrier']
  H_SHIP['1_8'] = [' 1','8','14','0','69.921','0.4351','0','0','Bulk H_SHIP']
  H_SHIP['2_8'] = [' 2','8','0','0','0','0','0','0','Dry cargo']
  H_SHIP['3_8'] = [' 3','8','0','0','0','0','0','0','Container']
  H_SHIP['4_8'] = [' 4','8','0','0','0','0','0','0','Ro-Ro']
  H_SHIP['5_8'] = [' 5','8','0','0','0','0','0','0','Reefer']
  H_SHIP['6_8'] = [' 6','8','0','0','0','0','0','0','Cargo ferry']
  H_SHIP['7_8'] = [' 7','8','0','0','0','0','0','0','Passenger ferry']
  H_SHIP['8_8'] = [' 8','8','0','0','0','0','0','0','High-Speed ferry']
  H_SHIP['9_8'] = [' 9','8','15','17','52.8','0.5082','105.71','0.4468','Tanker']
  H_SHIP['10_8'] = [' 10','8','0','0','0','0','0','0','Gas tanker']
  H_SHIP['11_8'] = [' 11','8','0','0','0','0','0','0','Cruise vessel']
  H_SHIP['12_8'] = [' 12','8','0','0','0','0','0','0','Vehicle carrier']  
### LASTE Tog
def load_Train():
  H_TRAIN['1'] = [' 1','ELECTRIC','EL16','0.8','0.153','10','90000','369000','129000','20','0.0006','0.0005','0.0006','0.004','66','0.65','0.86','Forsk p  gjennskape tog 5707 basert p data fra CN']
  H_TRAIN['2'] = [' 2','ELECTRIC','EL16','0.8','0.153','10','90000','369000','129000','20','0.0006','0.0005','0.0006','0.004','66','0.65','0.86','Forsk p  gjennskape tog 5707 basert p data fra CN']
  H_TRAIN['3'] = [' 3','ELECTRIC','CD66','0.8','0.153','10','126000','333000','413000','20','0.0006','0.0005','0.0006','0.004','66','0.65','0.3','Forsk p  gjennskape fult tog p nordlandsbanen som kjort i togkjor']

## Load truck data
def load_HDV():
  H_HDV['1_THC_0_0'] = [' 1','EURO0','Coach Std <=18t 80ties','0','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.109624397922924','0.021575297318021','0.000064272234634','0','0','6','103']
  H_HDV['1_NOx_0_0'] = [' 1','EURO0','Coach Std <=18t 80ties','0','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.012603138423552','0.00300229519348','-0.000019776586026','0','0','6','103']
  H_HDV['1_CO_0_0'] = [' 1','EURO0','Coach Std <=18t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7998941511586780','0.045090957412867','14.7117001870759','0.194689235106513','1.12562478941571','6','103']
  H_HDV['1_PM_0_0'] = [' 1','EURO0','Coach Std <=18t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.48594548365939','0.045820225363184','2.28855529610017','0.20751606091069','0.257476454543761','6','103']
  H_HDV['1_FC_0_0'] = [' 1','EURO0','Coach Std <=18t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','746.990763534542','0.056258579056276','2046.34934033896','0.352334843559251','180.498096503829','6','103']
  H_HDV['1_CO_0_50'] = [' 1','EURO0','Coach Std <=18t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.43448089347279','0.047014607653507','17.7156732639217','0.223856493432231','1.2374142610252','6','103']
  H_HDV['1_THC_0_50'] = [' 1','EURO0','Coach Std <=18t 80ties','0','50','THC','y=((a+(b*x))**((-1)/c))','0.164335084865499','0.023674285313866','0.837803220941278','0','0','6','103']
  H_HDV['1_NOx_0_50'] = [' 1','EURO0','Coach Std <=18t 80ties','0','50','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.015355574350559','0.00252178534954','-0.000015972523239','0','0','6','103']
  H_HDV['1_PM_0_50'] = [' 1','EURO0','Coach Std <=18t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.50797491755461','0.045514991169709','2.67500447628517','0.230306720507304','0.277394459399096','6','103']
  H_HDV['1_FC_0_50'] = [' 1','EURO0','Coach Std <=18t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','738.280641648805','0.053025785424166','2369.61308055718','0.381587794681817','191.094101496311','6','103']
  H_HDV['1_CO_0_100'] = [' 1','EURO0','Coach Std <=18t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.78133071376838','0.046750393003795','23.0528738576775','0.26007171524348','1.29974389285463','6','103']
  H_HDV['1_THC_0_100'] = [' 1','EURO0','Coach Std <=18t 80ties','0','100','THC','y=(1/(a+(b*(x**c))))','0.124241744643557','0.015576724065516','1.14453085564044','0','0','6','103']
  H_HDV['1_NOx_0_100'] = [' 1','EURO0','Coach Std <=18t 80ties','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.1826856325659','0.050271665837181','114.307856106351','0.459937651899659','8.88406487430784','6','103']
  H_HDV['1_PM_0_100'] = [' 1','EURO0','Coach Std <=18t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53598648975483','0.044106038793345','3.29056840470945','0.259813447696003','0.290469836620161','6','103']
  H_HDV['1_FC_0_100'] = [' 1','EURO0','Coach Std <=18t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','736.529737165267','0.04997960379266','2981.72055189702','0.424495339138321','201.170627083545','6','103']
  H_HDV['1_CO_-2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.046289878503961','0.008721952639318','0.000074791130558','0','0','6','103']
  H_HDV['1_THC_-2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.02230506445071','0.041885931280008','6.58637772260609','0.280629074846021','0.240912055494676','6','103']
  H_HDV['1_NOx_-2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','0','NOx','y=((a*(b**x))*(x**c))','111.029433731852','0.993614431253331','-0.742129752266854','0','0','6','103']
  H_HDV['1_PM_-2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.50991054276685','0.043596560169824','2.29215240517069','0.245212526422804','0.138165122587424','6','103']
  H_HDV['1_FC_-2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','623.067718243917','0.046116519942571','3245.32367911399','0.397969674054721','57.9923235110629','6','103']
  H_HDV['1_CO_-2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.98516436264812','0.050063314341722','19.5288883772401','0.337912978433895','0.587810104469402','6','103']
  H_HDV['1_THC_-2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.1052467405258','0.043422145644206','7.82444438930565','0.319140319309452','0.194571710623578','6','103']
  H_HDV['1_NOx_-2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4255293048319','0.041828780409598','133.417111197458','0.432772099216714','1.44172936872188','6','103']
  H_HDV['1_PM_-2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.57837608800409','0.044633799826056','2.97765218805776','0.30498755283894','0.126480910107066','6','103']
  H_HDV['1_FC_-2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','633.113963678314','0.043637741548363','4831.22761077587','0.470879133657246','39.9295524086408','6','103']
  H_HDV['1_CO_-2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.15043731164215','0.049797463643333','34.1823039923289','0.423736619101708','0.506729248452149','6','103']
  H_HDV['1_THC_-2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.08079687339606','0.043736818234155','8.72710703171383','0.337847654079695','0.15751141332659','6','103']
  H_HDV['1_NOx_-2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4404419856316','0.038361127143901','216.866210019975','0.517813661492083','0.930877235824165','6','103']
  H_HDV['1_PM_-2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.61223442775741','0.044388926547068','4.44367616727843','0.370013104653732','0.117354828814798','6','103']
  H_HDV['1_FC_-2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','627.996572495924','0.04092653870664','7170.23138330936','0.537572246488503','27.1001716600483','6','103']
  H_HDV['1_CO_-4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.17226072867038','0.058059885688095','43.4170994177387','0.52162773744445','0.264739382849964','6','103']
  H_HDV['1_THC_-4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.26247985436838','0.050776806481488','15.9979190529703','0.462623239339077','0.1325887607499','6','103']
  H_HDV['1_NOx_-4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.2992433747496','0.056345075540023','271.242294371366','0.567321693752804','0.459002757533296','6','103']
  H_HDV['1_PM_-4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54619617584186','0.049278643110987','3.31402639459689','0.338797506525415','0.089029617891253','6','103']
  H_HDV['1_FC_-4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','621.921600659002','0.056587192107751','10428.8852342477','0.610651988576538','12.4929046660467','6','103']
  H_HDV['1_CO_-4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.63219632143374','0.057681794780613','67.5603215484879','0.577187285473609','0.223539399909741','6','103']
  H_HDV['1_THC_-4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.1143753682497','0.051070042462146','21.140815237058','0.504345889016519','0.117705402845721','6','103']
  H_HDV['1_NOx_-4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.8575256170017','0.053171135050439','435.328881241898','0.63748506317791','0.317299237443571','6','103']
  H_HDV['1_PM_-4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51601521687444','0.049506316195437','4.58888278909881','0.390429329558035','0.087328341399322','6','103']
  H_HDV['1_FC_-4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','554.337841692656','0.053695791695456','15337.5367706921','0.666534316114415','7.86257598242079','6','103']
  H_HDV['1_CO_-4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.18871323689302','0.056090578666228','110.572158560593','0.642745045511129','0.197571490707836','6','103']
  H_HDV['1_THC_-4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.98237901258042','0.051173112661357','25.4806126701075','0.530938508124455','0.113388581522527','6','103']
  H_HDV['1_NOx_-4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.1507691538467','0.050012596059623','642.219897910723','0.697860389967899','0.224020074327293','6','103']
  H_HDV['1_PM_-4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51111337139313','0.049414892783997','7.14943585933446','0.463417079693246','0.086634516997456','6','103']
  H_HDV['1_FC_-4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','506.729939736771','0.05102104531587','21839.1783350827','0.721680296655827','5.11055630411948','6','103']
  H_HDV['1_CO_-6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.225347218309','0.064723383373422','75.6251265147244','0.602010566505966','0.188947160364677','6','103']
  H_HDV['1_THC_-6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.00859192810073','0.055760686223778','23.9852205597489','0.526926861364306','0.121135642226111','6','103']
  H_HDV['1_NOx_-6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.0262373209097','0.062950321650946','407.52174714098','0.624867406457249','0.204955421167063','6','103']
  H_HDV['1_PM_-6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.46517378077297','0.053934052414644','4.45183657058526','0.386953182220087','0.091481956882361','6','103']
  H_HDV['1_FC_-6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','496.857181800927','0.062558443447689','14773.6184349031','0.657312874368009','4.23123437363588','6','103']
  H_HDV['1_CO_-6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.30017625441237','0.062521908449027','73.9915995734614','0.574836994374607','0.173013656777447','6','103']
  H_HDV['1_THC_-6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.76197939472458','0.054775381184002','25.2444833911538','0.524626757519056','0.117537541791694','6','103']
  H_HDV['1_NOx_-6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.3590518507152','0.05962808475606','517.309461666857','0.659776887715115','0.159679868665933','6','103']
  H_HDV['1_PM_-6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.39547733068373','0.053034924034643','5.00429221666476','0.398372019397797','0.090149203695538','6','103']
  H_HDV['1_FC_-6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','417.535149158551','0.059116762779109','17898.2583629831','0.683319682770538','2.66511141882642','6','103']
  H_HDV['1_CO_-6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.61574535312348','0.059706305241486','90.0620467326523','0.588298581797132','0.1547233693924','6','103']
  H_HDV['1_THC_-6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.58497953578587','0.05384850245298','25.4581045599847','0.519161341087305','0.114961922483029','6','103']
  H_HDV['1_NOx_-6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.807252065387','0.057132187829764','576.617046889641','0.676602338720296','0.130714490502384','6','103']
  H_HDV['1_PM_-6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.33318021584151','0.051607898785665','6.39640109639331','0.429000030543326','0.087580129734932','6','103']
  H_HDV['1_FC_-6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','375.054623488637','0.057168999061146','19916.4472604461','0.701247472323758','1.98749410225801','6','103']
  H_HDV['1_CO_2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.1377325299848','-0.016058484816454','-1.85109957813685','0.095622481536266','0','6','103']
  H_HDV['1_THC_2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.37303812591241','0.036771487202506','4.95649300183146','0.19750143398255','0.197732853569549','6','103']
  H_HDV['1_NOx_2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','2','0','NOx','y=((a*(x**b))+(c*(x**d)))','0.009442465034573','1.46545578251212','114.568972245332','-0.598257595268781','0','6','103']
  H_HDV['1_PM_2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.4635550013479','-0.002185630359337','-0.256892878449058','0.088803525828924','0','6','103']
  H_HDV['1_FC_2_0'] = [' 1','EURO0','Coach Std <=18t 80ties','2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000475995407401','0.000078936159017','-0.000000533163666','0','0','6','103']
  H_HDV['1_CO_2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.9624494546473','-0.018771265578649','-2.31136371887856','0.106030301824523','0','6','103']
  H_HDV['1_THC_2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.43371092024845','0.04337656489856','4.55526482635877','0.187334986966969','0.291870208265384','6','103']
  H_HDV['1_NOx_2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','2','50','NOx','y=((a*(b**x))*(x**c))','118.718318867614','1.0096321854342','-0.62012454485335','0','0','6','103']
  H_HDV['1_PM_2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.73543748751466','-0.002489838686881','-0.293202991446188','0.09512203139221','0','6','103']
  H_HDV['1_FC_2_50'] = [' 1','EURO0','Coach Std <=18t 80ties','2','50','FC','y=((a*(b**x))*(x**c))','3707.49377441452','1.00833697489021','-0.685189295687702','0','0','6','103']
  H_HDV['1_CO_2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.7355798187739','-0.019443830409653','-2.4815749540485','0.105983273000014','0','6','101']
  H_HDV['1_THC_2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.12257978570841','0.046686861802908','4.46337392783044','0.163257053335539','0.368039131721999','6','101']
  H_HDV['1_NOx_2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','2','100','NOx','y=((a*(b**x))*(x**c))','106.995844997667','1.00796928995956','-0.532535806140237','0','0','6','101']
  H_HDV['1_PM_2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.462778511529917','3.49331342298141','2.91207109823732','1.38175606751723','0.009650793929687','6','101']
  H_HDV['1_FC_2_100'] = [' 1','EURO0','Coach Std <=18t 80ties','2','100','FC','y=((a*(b**x))*(x**c))','3534.59029560966','1.00839902523141','-0.63898153965343','0','0','6','101']
  H_HDV['1_CO_4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.925172840096','-0.01584883891894','-2.56385505517381','0.107734389278424','0','6','101']
  H_HDV['1_THC_4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.76603446035042','0.046953436942557','4.21908190524854','0.185501394306476','0.314000887339526','6','101']
  H_HDV['1_NOx_4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','4','0','NOx','y=((a*(x**b))+(c*(x**d)))','0.316672789903815','0.813001500659772','123.048061001717','-0.584380788265948','0','6','101']
  H_HDV['1_PM_4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.511748664607514','2.67978818603619','4.49664910079508','1.89327417786758','0.002907345531091','6','101']
  H_HDV['1_FC_4_0'] = [' 1','EURO0','Coach Std <=18t 80ties','4','0','FC','y=((a*(b**x))*(x**c))','3812.19762925996','1.00968723055692','-0.674021435886959','0','0','6','101']
  H_HDV['1_CO_4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.72197522641855','0.018095471071783','26.3754487736744','0.127914295148572','0.73992241526511','6','87']
  H_HDV['1_THC_4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.231104086044432','5.72348237605418','4.3058811316595','1.82474606299909','-0.008001413758339','6','87']
  H_HDV['1_NOx_4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','40.9359627285801','0.106186545247543','1150815.84480359','2.1981846087536','25.0865635445002','6','87']
  H_HDV['1_PM_4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.8184516917799','0.030011875620934','3.3361630702228','0.128819963197417','0.519101365540559','6','87']
  H_HDV['1_FC_4_50'] = [' 1','EURO0','Coach Std <=18t 80ties','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1199.65459659305','0.093045800143321','1177841.58991939','1.6073976395554','533.758503022585','6','87']
  H_HDV['1_CO_4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.98829497523008','0.046318725304081','27.7592951127751','0.164148277498137','2.08222815772822','6','77']
  H_HDV['1_THC_4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.360937914866535','5.06891431086893','5.37186088255187','2.23720111165265','-0.014723831685638','6','77']
  H_HDV['1_NOx_4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.4791356841712','0.079236364247019','88.5930770503033','0.42550341662364','29.4478363057716','6','77']
  H_HDV['1_PM_4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.130508178918026','0.035345208683249','-0.000262346248978','0','0','6','77']
  H_HDV['1_FC_4_100'] = [' 1','EURO0','Coach Std <=18t 80ties','4','100','FC','y=((a*(b**x))*(x**c))','3442.67660569762','1.00955071224329','-0.551102484347978','0','0','6','77']
  H_HDV['1_CO_6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.8784209229312','-0.026361470755715','-3.35043902525946','0.119180182598804','0','6','80']
  H_HDV['1_THC_6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','6','0','THC','y=(a-(b*exp(((-1)*c)*(x**d))))','0.492745915780175','-8.07644223395529','0.159132433231385','0.807762949904771','0','6','80']
  H_HDV['1_NOx_6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','44.2885386769081','0.116946499259385','1769832.41439495','2.3010856308916','26.5190601235904','6','80']
  H_HDV['1_PM_6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.504061975519739','3.20859020369613','4.31652811926924','1.95426943481356','-0.014925996457806','6','80']
  H_HDV['1_FC_6_0'] = [' 1','EURO0','Coach Std <=18t 80ties','6','0','FC','y=((a*(b**x))*(x**c))','3716.05888084206','1.01075111874113','-0.617073349662347','0','0','6','80']
  H_HDV['1_CO_6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','37.1216800923532','-0.028981363741423','-3.96902115535245','0.120780013082257','0','6','70']
  H_HDV['1_THC_6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.333137735375974','5.18179159293201','5.357803819579','2.23692055228011','-0.016657436880884','6','70']
  H_HDV['1_NOx_6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.7510138842271','0.077513852266986','74.9881462434006','0.342719854141096','32.9609633321438','6','70']
  H_HDV['1_PM_6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.77870932377482','3.83412466716719','3.24631543217439','1.49045235144315','0.021171370365526','6','70']
  H_HDV['1_FC_6_50'] = [' 1','EURO0','Coach Std <=18t 80ties','6','50','FC','y=((a*(b**x))*(x**c))','3452.73366730809','1.0095520123827','-0.527345029438511','0','0','6','70']
  H_HDV['1_CO_6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.9625378344147','-0.061109273387654','-4.70353310430076','0.125434114445541','0','6','53']
  H_HDV['1_THC_6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.712146237330369','4.33289765255625','6.83165429446016','2.89745781526572','-0.026101795650604','6','53']
  H_HDV['1_NOx_6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','6','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.76021681964242','2.47886790240246','-0.030686401201758','0','0','6','53']
  H_HDV['1_PM_6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.81114920202294','-0.006853625618096','-0.547185245965694','0.120373790884554','0','6','53']
  H_HDV['1_FC_6_100'] = [' 1','EURO0','Coach Std <=18t 80ties','6','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','869.850346435291','4456.04053706783','0.699434230035147','1.24402347352258','0.029975820789278','6','53']
  H_HDV['2_CO_0_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.83068930469641','0.043086215623439','11.0287835818455','0.177594547960913','0.945454722442678','6','103']
  H_HDV['2_THC_0_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','0','THC','y=(1/(a+(b*(x**c))))','0.145456867478455','0.021551676760392','1.05081927299372','0','0','6','103']
  H_HDV['2_NOx_0_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.1067040758696','0.062875732381663','76.6034908592374','0.362557800652745','5.95707633966798','6','103']
  H_HDV['2_PM_0_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.12226130557093','0.040221850028503','1.35041336307067','0.171203682300278','0.175077416561005','6','103']
  H_HDV['2_FC_0_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','648.802377021805','0.054408387041838','2655.65011862029','0.401063682828445','165.238154236465','6','103']
  H_HDV['2_CO_0_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.04164185183975','0.043950398132679','12.8806568134191','0.202538170719922','1.01974341308004','6','103']
  H_HDV['2_THC_0_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.17331289615912','0.037438970102412','3.41566105651941','0.151043954326755','0.332351787247489','6','103']
  H_HDV['2_NOx_0_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4906993117649','0.058334003322157','85.7766356671849','0.382956540968124','6.31670636044002','6','103']
  H_HDV['2_PM_0_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.11854340829562','0.040238503862888','1.43430862113027','0.177601630123017','0.186272041887835','6','103']
  H_HDV['2_FC_0_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','646.643177204604','0.051416652457774','3159.81891806033','0.432772373623944','175.161626205305','6','103']
  H_HDV['2_CO_0_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.37945154731997','0.044274779259384','15.4076382948346','0.237452049022753','1.10957415414203','6','103']
  H_HDV['2_THC_0_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.12706202099084','0.037685442763148','3.41639360601228','0.147923926491645','0.331888439085909','6','103']
  H_HDV['2_NOx_0_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.1391197998396','0.053902227721591','97.5686993626791','0.407387228969457','6.61140496807641','6','103']
  H_HDV['2_PM_0_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.20530010672886','0.042163482691778','1.60256195153249','0.215668672175243','0.202978689812081','6','103']
  H_HDV['2_FC_0_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','651.176529323422','0.048611702736734','3902.73347677455','0.471629693095523','183.678758652479','6','103']
  H_HDV['2_CO_-2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.9421525767368','-0.011753653285813','-0.88668800402937','0.084929720010929','0','6','103']
  H_HDV['2_THC_-2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','0','THC','y=((a*(b**x))*(x**c))','9.29035185572039','0.986265368344271','-0.547600438226917','0','0','6','103']
  H_HDV['2_NOx_-2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','0','NOx','y=((a*(b**x))*(x**c))','107.929154894828','0.993127115935884','-0.789878993952555','0','0','6','103']
  H_HDV['2_PM_-2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.82488746975506','-7.06621157224695','-1.25978418294352','0','0','6','103']
  H_HDV['2_FC_-2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','558.956527992005','0.044080879155145','3292.31869055974','0.411311904723247','45.3829226343872','6','103']
  H_HDV['2_CO_-2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.5504401764131','-0.010976899062682','-0.837158916573072','0.082121154386537','0','6','103']
  H_HDV['2_THC_-2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','50','THC','y=((a*(b**x))*(x**c))','8.88523023522846','0.983598907453578','-0.525284589763451','0','0','6','103']
  H_HDV['2_NOx_-2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.7375221059625','0.042027998794622','96.9668226877246','0.375568994157457','1.07930719843355','6','103']
  H_HDV['2_PM_-2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.11188992280761','-7.99532705689567','-1.34785450921644','0','0','6','103']
  H_HDV['2_FC_-2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','573.581012080255','0.042608700438452','4423.36051438317','0.466783862817873','34.6016736351557','6','103']
  H_HDV['2_CO_-2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.064958561105497','0.009885988225161','0.000149843622936','0','0','6','103']
  H_HDV['2_THC_-2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','100','THC','y=((a*(b**x))*(x**c))','8.56409805304431','0.98141515078365','-0.507234608876499','0','0','6','103']
  H_HDV['2_NOx_-2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.9482360966809','0.040255198204072','125.6806337396','0.424665519703562','0.786707841617232','6','103']
  H_HDV['2_PM_-2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.35103271049626','-8.64685753121853','-1.42086608715075','0','0','6','103']
  H_HDV['2_FC_-2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','577.854513486885','0.040701902096882','5937.63866942242','0.517750508389462','25.8391011253412','6','103']
  H_HDV['2_CO_-4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.47672073307441','0.058429982849371','18.001026105283','0.42077467563357','0.18796327649517','6','103']
  H_HDV['2_THC_-4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.89926046007764','0.051307118932358','8.80169651925566','0.412467404354303','0.089816073262861','6','103']
  H_HDV['2_NOx_-4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.351224981573','0.05634083310396','149.332342461414','0.471588869898829','0.3471781991386','6','103']
  H_HDV['2_PM_-4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.56250455384672','-12.0023652844557','-1.7969073817574','0','0','6','103']
  H_HDV['2_FC_-4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','564.31491117727','0.05573427833309','7846.82341717362','0.575674803024424','10.7007623257333','6','103']
  H_HDV['2_CO_-4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.38494410536217','0.06006102943572','37.1799164720592','0.53916686270785','0.174048626522621','6','103']
  H_HDV['2_THC_-4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.80373380783492','0.052448099190297','11.8061918805603','0.45447162913749','0.081161823947974','6','103']
  H_HDV['2_NOx_-4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.0562944171889','0.05536956080465','212.98060940261','0.529146663871579','0.286291230930026','6','103']
  H_HDV['2_PM_-4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.57274784230796','-11.7614089980058','-1.82490162455726','0','0','6','103']
  H_HDV['2_FC_-4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','524.961251809178','0.053999792635423','10757.0847503292','0.62348869608001','7.74429894673472','6','103']
  H_HDV['2_CO_-4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.11853152897595','0.059186551878837','58.6445143348578','0.602317890942058','0.158890717891794','6','103']
  H_HDV['2_THC_-4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.68980898379582','0.052860474782342','14.3063801357259','0.477493906978447','0.078394448526505','6','103']
  H_HDV['2_NOx_-4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9150302304356','0.053725308695869','293.871589200491','0.580523728947706','0.242883375440844','6','103']
  H_HDV['2_PM_-4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','5.43544488574269','-11.0713469586026','-1.80400455151431','0','0','6','103']
  H_HDV['2_FC_-4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','494.010609768663','0.052158864450432','14581.1836725847','0.672173186739604','5.8203941042912','6','103']
  H_HDV['2_CO_-6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.00671473019958','0.067627549507888','34.8308615820233','0.529133941079011','0.130075418187774','6','103']
  H_HDV['2_THC_-6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.74124562671395','0.059067858234237','13.657429902317','0.48215048634589','0.081361783974587','6','103']
  H_HDV['2_NOx_-6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9319214231869','0.065309299961257','234.882333784326','0.549100878044706','0.204651610274346','6','103']
  H_HDV['2_PM_-6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.83884838238493','-12.0303892603583','-1.94400106355369','0','0','6','103']
  H_HDV['2_FC_-6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','473.061990825413','0.063376980595149','10921.2880723937','0.624874808503793','4.41576957929655','6','103']
  H_HDV['2_CO_-6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.24824540859732','0.065519352272414','37.9831503564341','0.504947955523514','0.119043546081262','6','103']
  H_HDV['2_THC_-6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.49906637688427','0.058088807663221','13.905017556697','0.463111658940092','0.078401001543295','6','103']
  H_HDV['2_NOx_-6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.738983482844','0.062431672854899','280.244077867187','0.571052570807681','0.17154781970072','6','103']
  H_HDV['2_PM_-6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.47036637276306','-10.4773193097594','-1.86808161465123','0','0','6','103']
  H_HDV['2_FC_-6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','411.833165785234','0.060540630424475','12553.2680538595','0.640710627494669','3.24710039259074','6','103']
  H_HDV['2_CO_-6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.69400991651911','0.06311253689506','36.0533481790263','0.482784855078051','0.107556933244571','6','103']
  H_HDV['2_THC_-6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.32406490962523','0.057097390061482','14.6317383787467','0.456632778272923','0.075988616199208','6','103']
  H_HDV['2_NOx_-6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4634533087521','0.060426831932301','327.026049319883','0.595375100752539','0.150458166586201','6','103']
  H_HDV['2_PM_-6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.023773128443661','1.89124982499224','4.95784516052208','2.04377962894941','-0.011548086073444','6','103']
  H_HDV['2_FC_-6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','376.493348754125','0.058747579817038','14437.3366802402','0.663212936382372','2.62936028022153','6','103']
  H_HDV['2_CO_2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.32854650194185','11.4105395461581','4.79140876216091','1.86190034235777','0.010997068213868','6','103']
  H_HDV['2_THC_2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.66376345314958','0.038084960094553','3.52351867377574','0.175228579123608','0.348777713266273','6','103']
  H_HDV['2_NOx_2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','32.9344415861242','0.092878061437423','121.253073776343','0.538344118595843','10.6457485696172','6','103']
  H_HDV['2_PM_2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','0','PM','y=(c+(a*exp(((-1)*b)*x)))','1.79954173293395','0.062062026051784','0.299386691295649','0','0','6','103']
  H_HDV['2_FC_2_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','781.414391897722','0.068414044027191','3520.6445149955','0.48495106616467','288.935308692387','6','103']
  H_HDV['2_CO_2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.0237398020981','-0.004292625012654','-1.48705569528309','0.093389450966327','0','6','103']
  H_HDV['2_THC_2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.83096053151233','0.043824759512822','3.29573500983734','0.180730954590094','0.427964836552612','6','103']
  H_HDV['2_NOx_2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','27.3843607249226','0.081210086870449','84.3588617210113','0.42041393733762','12.060650366896','6','103']
  H_HDV['2_PM_2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','50','PM','y=(c+(a*exp(b*x)))','1.86396938977573','-0.067424564091158','0.352813714359365','0','0','6','103']
  H_HDV['2_FC_2_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.051467976708121','1.71351023151995','2758.48140067022','-0.554008780069081','0','6','103']
  H_HDV['2_CO_2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.6526603556343','-0.007415979069525','-1.75785838642793','0.103058362904286','0','6','102']
  H_HDV['2_THC_2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.93718287338429','0.049360661586852','3.01741828053191','0.177874088108978','0.51036882443732','6','102']
  H_HDV['2_NOx_2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','100','NOx','y=((a*(x**b))+(c*(x**d)))','1.0120515322113','0.457992554635976','118.839289010265','-0.69031647104972','0','6','102']
  H_HDV['2_PM_2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','100','PM','y=(c+(a*exp(b*x)))','1.90329741664477','-0.072214561616006','0.413049912007696','0','0','6','102']
  H_HDV['2_FC_2_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','849.641200570038','0.070482005449009','10361.9078597741','0.705375386779758','378.956620288048','6','102']
  H_HDV['2_CO_4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.96924467741867','11.1621862494742','6.75549594431194','2.71417394813183','-0.008462712793882','6','102']
  H_HDV['2_THC_4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.15386506995368','0.04974069891166','2.94612671214012','0.185845299716899','0.523946295201568','6','102']
  H_HDV['2_NOx_4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.0810031390943','0.086817923651939','51.3471293856542','0.307674244719251','15.0082731905545','6','102']
  H_HDV['2_PM_4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.24320465330079','0.002013523163822','-0.121728870283719','0.062874986001103','0','6','102']
  H_HDV['2_FC_4_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','3225.38020938398','1.00896913149957','-0.628492450139699','0','0','6','102']
  H_HDV['2_CO_4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.3019670044272','-0.012417031128666','-2.42523674566972','0.120760876659444','0','6','94']
  H_HDV['2_THC_4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.562627785161632','4.49121413960027','3.90985809016221','1.60996561751508','0.005514479880145','6','94']
  H_HDV['2_NOx_4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','16.9027522635413','209.0751270063','-0.193716068131079','0.912616884986997','0.032564142740676','6','94']
  H_HDV['2_PM_4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.41967839438549','0.001362384456767','-0.146165930821969','0.074262429590114','0','6','94']
  H_HDV['2_FC_4_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','50','FC','y=((a*(b**x))*(x**c))','3064.749083159','1.00826332215269','-0.563483922232528','0','0','6','94']
  H_HDV['2_CO_4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.494226118628','-0.018841372266692','-2.746029747488','0.127573463635948','0','6','80']
  H_HDV['2_THC_4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.596141253401998','4.00706993253211','5.18710576492431','2.12471517410169','-0.012850276357221','6','80']
  H_HDV['2_NOx_4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','17.4778231881379','133.372010554522','0.658514279393107','1.1516424300408','-0.000945319172467','6','80']
  H_HDV['2_PM_4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.5844750901867','0.001229493616732','-0.165736108914119','0.081893150522629','0','6','80']
  H_HDV['2_FC_4_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','820.551593994565','0.077095559198353','4228.19684435825','0.534752809326485','570.820444700006','6','80']
  H_HDV['2_CO_6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.69662573592737','13.7108556374376','4.83394856841047','1.91380977798164','0.036846336147432','6','85']
  H_HDV['2_THC_6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.96907189822854','-0.005419894350896','-0.448186949812436','0.091176100645094','0','6','85']
  H_HDV['2_NOx_6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','18.382305867449','62.864902233364','2.26299618075009','1.46781447477231','0.00892925607958','6','85']
  H_HDV['2_PM_6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.49225645985856','0.002130443230872','-0.155701293714345','0.076709311118763','0','6','85']
  H_HDV['2_FC_6_0'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','858.213309867146','0.083062214155578','4671.52588055004','0.555544254513147','542.895139725978','6','85']
  H_HDV['2_CO_6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.4567473969298','-0.032198834459702','-3.9993609273934','0.157004192564363','0','6','75']
  H_HDV['2_THC_6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.97172770671853','0.074187767703861','1.68738191078028','0.185836043944729','0.850213168773691','6','75']
  H_HDV['2_NOx_6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','20.7550174022319','106.584935788362','1.190010008125','1.28682977220291','-0.002291298354568','6','75']
  H_HDV['2_PM_6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.605408064242','0.004480626443284','-0.157743890716761','0.075024370828643','0','6','75']
  H_HDV['2_FC_6_50'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','906.361690485684','0.088005800370255','9564.04731984895','0.694488920252707','655.626948448614','6','75']
  H_HDV['2_CO_6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.6538905669155','0.000794012815888','-5.08151670509791','0.167793877732209','0','6','62']
  H_HDV['2_THC_6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','100','THC','y=exp((a+(b/x))+(c*log(x)))','3.24078598786585','-2.09073246293986','-0.840365173053132','0','0','6','62']
  H_HDV['2_NOx_6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.6990311809034','0.078824893155727','78.1231093612118','0.356501335855984','26.5613652687559','6','62']
  H_HDV['2_PM_6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','100','PM','y=(c+(a*exp(b*x)))','2.0311006373698','-0.093064854566366','0.810552288595441','0','0','6','62']
  H_HDV['2_FC_6_100'] = [' 2','EURO1','Coach Std <=18t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','506.647210835426','0.042648359933666','2626.55187907761','0.335585641104781','696.834747891695','6','62']
  H_HDV['3_CO_0_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.91751972358675','0.054594830971968','14.5272181995603','0.234033092020708','0.94425465805943','6','103']
  H_HDV['3_THC_0_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','0','THC','y=(1/(a+(b*x)))','0.173126380865605','0.040273721278685','0','0','0','6','103']
  H_HDV['3_NOx_0_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.3058773084969','0.059393651116242','73.005232373851','0.336642090362501','6.58169147651815','6','103']
  H_HDV['3_PM_0_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.444898991161853','0.049470083034721','0.588266793949002','0.230938594876073','0.106393505104902','6','103']
  H_HDV['3_FC_0_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','594.01314166657','0.0516297553465','2921.98873261403','0.423126762294571','166.08622382279','6','103']
  H_HDV['3_CO_0_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.92534609673967','0.052615109579929','16.031866052335','0.244223968035646','1.0008850031912','6','103']
  H_HDV['3_THC_0_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','50','THC','y=(1/(a+(b*x)))','0.170452633296351','0.040964444787221','0','0','0','6','103']
  H_HDV['3_NOx_0_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.2137681687706','0.056571921705511','81.2148544422182','0.358042508344665','6.83481038799362','6','103']
  H_HDV['3_PM_0_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.424929311536733','0.044498862704997','0.792130437431232','0.259271964140274','0.109356689160901','6','103']
  H_HDV['3_FC_0_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','596.162545047804','0.048974075282995','3566.31377631362','0.459079093021153','173.847511007266','6','103']
  H_HDV['3_CO_0_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.15408393108906','0.051572434895003','17.8022001171456','0.260820637859355','1.07186880135295','6','103']
  H_HDV['3_THC_0_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','100','THC','y=(1/(a+(b*x)))','0.168581913019541','0.041598595964348','0','0','0','6','103']
  H_HDV['3_NOx_0_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.0608243875992','0.053578465460904','85.6404149537839','0.371083557783568','7.03748446197086','6','103']
  H_HDV['3_PM_0_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.435708357058309','0.042411189804646','0.914439920119505','0.282701717646611','0.114917716854438','6','103']
  H_HDV['3_FC_0_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','599.204462591089','0.046256142239609','4201.63582010537','0.490142467256894','180.23837489345','6','103']
  H_HDV['3_CO_-2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.039117817935126','0.016329103163593','0.000112095677908','0','0','6','103']
  H_HDV['3_THC_-2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','0','THC','y=((a*(b**x))*(x**c))','6.83455248433634','0.987853766610778','-0.590446834349239','0','0','6','103']
  H_HDV['3_NOx_-2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','0','NOx','y=((a*(b**x))*(x**c))','113.613850304051','0.991615238869097','-0.751743793798676','0','0','6','103']
  H_HDV['3_PM_-2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','0','PM','y=((a+(b*x))**((-1)/c))','1.04422990848593','0.106051585356019','0.756800994927138','0','0','6','103']
  H_HDV['3_FC_-2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','516.64453895192','0.041169763256438','3384.66976981081','0.419430704983532','39.0169537710272','6','103']
  H_HDV['3_CO_-2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.045944420019475','0.015460077171985','0.000138672824421','0','0','6','103']
  H_HDV['3_THC_-2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66030565498191','0.038430984414124','4.64188621703136','0.296989176802612','0.100474291769142','6','103']
  H_HDV['3_NOx_-2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.9097971686628','0.040961563013364','100.981403615529','0.363410977831619','1.14671186738233','6','103']
  H_HDV['3_PM_-2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','50','PM','y=((a+(b*x))**((-1)/c))','1.10635509402841','0.097505519330832','0.737015067016115','0','0','6','103']
  H_HDV['3_FC_-2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','533.231998964969','0.040368196777044','4374.76055536419','0.469007097460052','31.3789500496965','6','103']
  H_HDV['3_CO_-2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','100','CO','y=((a*(b**x))*(x**c))','28.3283860692471','0.988285247468405','-0.740747389094618','0','0','6','103']
  H_HDV['3_THC_-2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.67064718677946','0.039017279942388','5.08730208224187','0.315895438633035','0.089280930643805','6','103']
  H_HDV['3_NOx_-2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.2981663006883','0.040202041143008','125.883630077256','0.407148048985491','0.898621829860088','6','103']
  H_HDV['3_PM_-2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.10058641785575','0.1506188835896','0.000912181350936','0','0','6','103']
  H_HDV['3_FC_-2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','542.276849128027','0.039149681803106','5732.65969601858','0.51798158296812','24.7453745240545','6','103']
  H_HDV['3_CO_-4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.98823530857323','0.060936538163231','36.8772883507163','0.511081942729434','0.157229134325473','6','103']
  H_HDV['3_THC_-4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.78366860269731','0.047268045998543','8.75058926700864','0.435145654018556','0.071186774060969','6','103']
  H_HDV['3_NOx_-4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.7024728582101','0.055715961674094','170.662252370349','0.480315773836451','0.387025169662062','6','103']
  H_HDV['3_PM_-4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.964014272279806','0.197721429914117','0.002152771128041','0','0','6','103']
  H_HDV['3_FC_-4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','522.54585032359','0.053923479524721','7721.55759644609','0.581866284584314','9.94327526478183','6','103']
  H_HDV['3_CO_-4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.684860240398','0.059924700468461','47.6958284627383','0.546520935650811','0.143983874677085','6','103']
  H_HDV['3_THC_-4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.7502296844568','0.048279510475445','10.7425502960909','0.468797778568732','0.068421704755194','6','103']
  H_HDV['3_NOx_-4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.5215818940959','0.055099662298428','224.194956960126','0.524051737941618','0.338384490603761','6','103']
  H_HDV['3_PM_-4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','1.60268368520289','0.988264286248009','-0.675466254986824','0','0','6','103']
  H_HDV['3_FC_-4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','493.931164239674','0.052540834246554','9733.30698596966','0.616779686067805','7.8064942774242','6','103']
  H_HDV['3_CO_-4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.63563566775241','0.059296297139742','62.6010719099377','0.594399820490818','0.137897470770435','6','103']
  H_HDV['3_THC_-4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.7069205348888','0.048807450418317','12.1351731881172','0.487017350047998','0.0680318751466','6','103']
  H_HDV['3_NOx_-4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.5503609267224','0.054146466274729','289.032936997064','0.565289365983136','0.30213399040945','6','103']
  H_HDV['3_PM_-4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.373160251279524','0.047211481131846','1.15057304563442','0.351965788260277','0.027690177906884','6','103']
  H_HDV['3_FC_-4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','472.421791036044','0.051181729650169','12455.2649128735','0.657427796511662','6.28580044609292','6','103']
  H_HDV['3_CO_-6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.34122895076341','0.067850611454308','52.0615677450883','0.580051763397722','0.108442319873165','6','103']
  H_HDV['3_THC_-6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70865270430141','0.054340978064109','13.381244964006','0.512703498825056','0.072541767377586','6','103']
  H_HDV['3_NOx_-6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.7458871918931','0.064492478505355','246.277548163127','0.541377895943903','0.238754909179134','6','103']
  H_HDV['3_PM_-6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01891224143811','7.81699249984382','-1.45090060319803','0.736807760372007','0.021619425731946','6','103']
  H_HDV['3_FC_-6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','433.960271400315','0.061649602225953','9618.35928715172','0.613301476660769','4.6669959574518','6','103']
  H_HDV['3_CO_-6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.94098983061815','0.065828238561772','65.6130387297769','0.595977736138827','0.100994426974868','6','103']
  H_HDV['3_THC_-6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.61602846075774','0.053909022265497','13.7565393396311','0.509576399990874','0.071823175647792','6','103']
  H_HDV['3_NOx_-6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.8810047367714','0.062429867255962','296.023220197245','0.56590844685648','0.209607531054228','6','103']
  H_HDV['3_PM_-6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.325347888762277','0.050709547164775','1.27773104829292','0.33423894244531','0.027515200954221','6','103']
  H_HDV['3_FC_-6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','392.205803306555','0.059590807830042','11343.1503044471','0.636426285326263','3.8114587114094','6','103']
  H_HDV['3_CO_-6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.43554405382483','0.062104252020719','61.719057578153','0.575672073282008','0.089098654601992','6','103']
  H_HDV['3_THC_-6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53957718820099','0.053285667769809','13.3160704764109','0.49814498130854','0.070912364435069','6','103']
  H_HDV['3_NOx_-6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.4877053835803','0.060496001140321','329.09967792347','0.581257307010854','0.186797310720976','6','103']
  H_HDV['3_PM_-6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.302844203173902','0.047214650153547','1.2338731380375','0.328065116181421','0.02579779411568','6','103']
  H_HDV['3_FC_-6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','359.7776063089','0.057609422827532','12560.3748199982','0.651605775020308','3.12979834734977','6','103']
  H_HDV['3_CO_2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.9258854149047','-0.005178852069823','-1.85979834900046','0.109443654524711','0','6','103']
  H_HDV['3_THC_2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.69693143022765','0.035626966104236','3.37758178094589','0.202900108866234','0.206834298139864','6','103']
  H_HDV['3_NOx_2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','0','NOx','y=((a*(b**x))*(x**c))','137.057579137195','1.0096399244972','-0.74171538876614','0','0','6','103']
  H_HDV['3_PM_2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.01580289795319','-0.000612987557641','-0.068864021160906','0.087240836118623','0','6','103']
  H_HDV['3_FC_2_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','703.980573409567','0.066549717951916','3949.65401484723','0.509471424170033','301.406719080449','6','103']
  H_HDV['3_CO_2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.8347304629482','-0.009026760274973','-2.16087345859289','0.116739544383096','0','6','103']
  H_HDV['3_THC_2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72582323869254','0.038810062142216','3.29639430961364','0.202501130379208','0.241490480765229','6','103']
  H_HDV['3_NOx_2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','50','NOx','y=((a*(x**b))+(c*(x**d)))','0.318869834461104','0.663057931260556','129.574731849441','-0.676795404619624','0','6','103']
  H_HDV['3_PM_2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.09632354742285','-0.000889342342849','-0.077233624630044','0.092407314200782','0','6','103']
  H_HDV['3_FC_2_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','721.009545006765','0.066419315818675','5338.85636342843','0.571972907760487','338.617878908463','6','103']
  H_HDV['3_CO_2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.8210087626977','-0.013049621927283','-2.47075873327133','0.12278201019674','0','6','103']
  H_HDV['3_THC_2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.74367074455768','0.041814353643283','3.21026141714564','0.200748726485338','0.275717948350173','6','103']
  H_HDV['3_NOx_2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','100','NOx','y=((a*(b**x))*(x**c))','121.520752994203','1.00736085614149','-0.635841862924229','0','0','6','103']
  H_HDV['3_PM_2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.16626792939862','-0.001063159936819','-0.0829349597061','0.094356801710268','0','6','103']
  H_HDV['3_FC_2_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','742.596864536041','0.066626978883361','8250.48682954579','0.659167834622222','375.980112960375','6','103']
  H_HDV['3_CO_4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.2993239081108','-0.0141532283073','-2.93857633210379','0.130980491262053','0','6','102']
  H_HDV['3_THC_4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.90339792485429','0.045699084673387','3.06906004985701','0.203179102196958','0.317548694563114','6','102']
  H_HDV['3_NOx_4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','14.1518205543356','63.4342293963111','2.49013573418395','1.4806692109234','-0.002685123755697','6','102']
  H_HDV['3_PM_4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.2483337568985','-0.001119806284879','-0.091738141930197','0.097600557919344','0','6','102']
  H_HDV['3_FC_4_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','2870.44704184498','1.00847310156125','-0.579405761287359','0','0','6','102']
  H_HDV['3_CO_4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.3405564498935','-0.019700710505287','-3.10635178563476','0.12887241999725','0','6','99']
  H_HDV['3_THC_4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72380835453274','0.048151898595383','2.97999756101447','0.178863883787312','0.381267270807577','6','99']
  H_HDV['3_NOx_4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.7134270956808','0.037944244547284','52.2457255014416','0.181053366939146','16.4206495309483','6','99']
  H_HDV['3_PM_4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.34214900643028','-0.001257145657386','-0.095101301872083','0.094264262698451','0','6','99']
  H_HDV['3_FC_4_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','50','FC','y=((a*(b**x))*(x**c))','2777.35855660121','1.007941981501','-0.529023807115012','0','0','6','99']
  H_HDV['3_CO_4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.32082860127877','0.018731324858132','26.8394043449863','0.154641424130002','0.393804772484568','6','90']
  H_HDV['3_THC_4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.304044914111088','3.60183841261456','3.62783599401748','1.66750052817702','-0.00705155587367','6','90']
  H_HDV['3_NOx_4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.4722631919183','0.042290503539623','52.0504238571092','0.19575235827151','18.3881410074452','6','90']
  H_HDV['3_PM_4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.47466089858475','-0.001965374173575','-0.107985787336125','0.098919167008192','0','6','90']
  H_HDV['3_FC_4_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','769.984682476993','0.07821733025834','6024.54339061183','0.614464266017863','573.95932334203','6','90']
  H_HDV['3_CO_6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.9229327905917','-0.030717489947775','-3.78388459083371','0.139552471664568','0','6','89']
  H_HDV['3_THC_6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.74820933015277','0.052221061795879','2.79149015199888','0.166598647002364','0.43529010618742','6','89']
  H_HDV['3_NOx_6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.402726216938','0.035445683489194','55.2622048436178','0.179878313665068','18.4936991778158','6','89']
  H_HDV['3_PM_6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.51617831331526','-0.001999435879784','-0.11954114309921','0.105996383590321','0','6','89']
  H_HDV['3_FC_6_0'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','890.31648973616','0.09076015439601','12139.8328130115','0.782552736281674','582.886004908733','6','89']
  H_HDV['3_CO_6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.4701733382823','-0.038654830660378','-4.17105071302591','0.138574039866253','0','6','79']
  H_HDV['3_THC_6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.61417122541893','0.074040480663907','1.82309858553609','0.198760926700331','0.536505608221444','6','79']
  H_HDV['3_NOx_6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','89.3184912275028','-0.12441168619526','-9.53512598384995','0.15825449609349','0','6','79']
  H_HDV['3_PM_6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.460264485034139','0.079810036493972','-0.000609425233458','0','0','6','79']
  H_HDV['3_FC_6_50'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','50','FC','y=((a*(b**x))*(x**c))','2850.46044244322','1.00881765543152','-0.476992840757313','0','0','6','79']
  H_HDV['3_CO_6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','38.1888361033252','-0.0454780058048','-4.4515112038115','0.133143284961732','0','6','70']
  H_HDV['3_THC_6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70897774039305','0.058666363778287','2.82078296399742','0.165239506792318','0.53777976120651','6','70']
  H_HDV['3_NOx_6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','85.1372983315743','-0.086468148372982','-7.25362340512728','0.131556760864717','0','6','70']
  H_HDV['3_PM_6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.80602109436701','-0.001917637925987','-0.133276748963618','0.097083114311891','0','6','70']
  H_HDV['3_FC_6_100'] = [' 3','EURO2','Coach Std <=18t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','662.267001337437','0.073936998610705','4532.43584187133','0.520986631144748','759.203019607072','6','70']
  H_HDV['4_CO_0_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.37080649284831','0.045890693529978','14.3161963196983','0.214687786352687','1.04233259443181','6','103']
  H_HDV['4_THC_0_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.223462479057506','16.9179704142865','-0.633088204425594','0.641430506612717','0.025932930340183','6','103']
  H_HDV['4_NOx_0_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.7882250999464','0.054342455694624','89.0430437854198','0.245150262613114','5.14950068351378','6','103']
  H_HDV['4_PM_0_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.081274297604571','0.984217528303947','2.78349425731847','1.20176859539431','0.009398168027605','6','103']
  H_HDV['4_FC_0_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','708.073604534447','0.055348506139231','3362.34132338213','0.431499156581408','185.324322977357','6','103']
  H_HDV['4_CO_0_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.46823166382744','0.045790967608809','15.0010348169023','0.221904651804259','1.08632604031267','6','103']
  H_HDV['4_THC_0_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.227231246172132','15.6623993601925','-0.530825258433305','0.64893877880533','0.027034244630971','6','103']
  H_HDV['4_NOx_0_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.8812199241423','0.052996714418024','90.0551365078442','0.247649925809256','5.30542698745506','6','103']
  H_HDV['4_PM_0_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.082467369875621','1.06820321325441','2.35097203495455','1.08187915615308','0.011843368441971','6','103']
  H_HDV['4_FC_0_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','700.026927912708','0.052834996519373','3813.80268106653','0.452232380842497','191.107562411866','6','103']
  H_HDV['4_CO_0_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.75579976516698','0.046706269538188','15.2486868469509','0.227209628180747','1.14646769636032','6','103']
  H_HDV['4_THC_0_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.230567108174008','31.561588841446','-1.41139128491162','0.588262581526971','0.029437342214943','6','103']
  H_HDV['4_NOx_0_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.8925755391018','0.051371445715558','90.1575712516754','0.248702275716549','5.43939134034842','6','103']
  H_HDV['4_PM_0_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.085296261867328','1.07230333619232','2.31867050173148','1.0664884374594','0.012891322629327','6','103']
  H_HDV['4_FC_0_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','698.381977354617','0.050636257785012','4317.8081532902','0.474915617898317','196.409725869769','6','103']
  H_HDV['4_CO_-2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.2541861086434','-0.015016821798424','-1.01209604757971','0.089413575110433','0','6','103']
  H_HDV['4_THC_-2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','0','THC','y=((a*(b**x))*(x**c))','7.53036745317654','0.990650003396511','-0.65479810082583','0','0','6','103']
  H_HDV['4_NOx_-2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','-0.001215971035971','0.004660104963316','0.000025896312189','0','0','6','103']
  H_HDV['4_PM_-2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.42460410442142','-5.23267329267514','-1.07312429669289','0','0','6','103']
  H_HDV['4_FC_-2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','585.283436874063','0.041680374370743','3295.90873673373','0.387363286321553','46.1629277538559','6','103']
  H_HDV['4_CO_-2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.7438765290651','-0.013975592965059','-0.926533476220694','0.084662960659234','0','6','103']
  H_HDV['4_THC_-2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','50','THC','y=((a*(b**x))*(x**c))','7.27483619187937','0.988900072204993','-0.640393821830666','0','0','6','103']
  H_HDV['4_NOx_-2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','50','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.000109720766728','0.004477201423372','0.000034450485429','0','0','6','103']
  H_HDV['4_PM_-2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.64787394829021','-6.17810735999522','-1.13496226464107','0','0','6','103']
  H_HDV['4_FC_-2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','594.517488933809','0.040897920117241','3601.2688188643','0.409915147019972','39.2095624910784','6','103']
  H_HDV['4_CO_-2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.5935022835587','-0.01356501907334','-0.898990470310924','0.082637815036313','0','6','103']
  H_HDV['4_THC_-2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','100','THC','y=((a*(b**x))*(x**c))','6.97858163306922','0.98713525973335','-0.623374389380999','0','0','6','103']
  H_HDV['4_NOx_-2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.096480035232128','0.499723152730667','229.913850609335','-1.04694129511082','0','6','103']
  H_HDV['4_PM_-2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.78060861490175','-6.68494048123421','-1.17565443910328','0','0','6','103']
  H_HDV['4_FC_-2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','599.239347365985','0.039998448393326','4127.83590674239','0.436782607936162','33.2885282269038','6','103']
  H_HDV['4_CO_-4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.101607942673278','0.003033768098891','0.00038162688036','0','0','6','103']
  H_HDV['4_THC_-4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.01990904866111','0.055940725222389','5.12593837767114','0.401204867128629','0.083961165326279','6','103']
  H_HDV['4_NOx_-4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.2792338390673','0.070222734931947','204.891525681746','0.448715901199405','0.578187724193462','6','103']
  H_HDV['4_PM_-4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.69378890956331','-9.14637891398221','-1.4518591689083','0','0','6','103']
  H_HDV['4_FC_-4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','622.204608924851','0.05579431366838','5542.76294700101','0.510127251933059','16.5564571039389','6','103']
  H_HDV['4_CO_-4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.8442950309584','-0.00662820765108','-0.727122721171416','0.071971935187183','0','6','103']
  H_HDV['4_THC_-4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.96478484876956','0.057202974690095','5.57316554760701','0.41702733298192','0.081905073698923','6','103']
  H_HDV['4_NOx_-4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.8303805561126','0.069623507832618','246.727370031734','0.479156078854183','0.559002059223471','6','103']
  H_HDV['4_PM_-4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.64128606178907','-9.13557815029284','-1.4473300807797','0','0','6','103']
  H_HDV['4_FC_-4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','596.228434249855','0.055225231977578','6502.01746141614','0.537719236235488','15.2650260874923','6','103']
  H_HDV['4_CO_-4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.8481089295776','-0.006628941701241','-0.741019133288288','0.073224550074847','0','6','103']
  H_HDV['4_THC_-4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.91882039220937','0.058275425852186','6.24234023068111','0.436968957689656','0.080996683819061','6','103']
  H_HDV['4_NOx_-4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.2202956364889','0.068110494159787','292.961620913303','0.50557498378691','0.530603342234492','6','103']
  H_HDV['4_PM_-4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.52828948885274','-8.72394532273484','-1.42644610141273','0','0','6','103']
  H_HDV['4_FC_-4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','574.636411188412','0.054507197071719','7905.63594855343','0.568897084460163','14.3102497361774','6','103']
  H_HDV['4_CO_-6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.4065517298287','-0.003797783448307','-0.761019837698991','0.076580085498928','0','6','103']
  H_HDV['4_THC_-6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92865034222786','0.065096314411845','7.71577689076613','0.489619645724378','0.077731352291695','6','103']
  H_HDV['4_NOx_-6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','0','NOx','y=((a*(x**b))+(c*(x**d)))','-0.04869657496589','0.410922181840067','381.72314322353','-1.47732591972143','0','6','103']
  H_HDV['4_PM_-6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.863396558117599','0.064356377163391','3082746979.56345','44661384.4857428','0.061856181473064','6','103']
  H_HDV['4_FC_-6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','538.445239026672','0.064449006335316','7093.26998600512','0.552709032657289','12.1003924841303','6','103']
  H_HDV['4_CO_-6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.1787900304025','-0.004101383062878','-0.760375614153508','0.078473859173146','0','6','103']
  H_HDV['4_THC_-6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.78560118849466','0.064397985402935','8.91831788263274','0.498389439155291','0.075962921872542','6','103']
  H_HDV['4_NOx_-6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','5.47243801017807','1.92943451248088','-1.41307084612599','0','0','6','103']
  H_HDV['4_PM_-6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.023454241672246','0.693448581450421','5.61464940577732','2.10500163735603','-0.009955209195759','6','103']
  H_HDV['4_FC_-6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','492.424728058341','0.062632653010281','8298.23243725591','0.573384517475237','11.33896670271','6','103']
  H_HDV['4_CO_-6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.25429861831623','0.066837303917636','24.4816049684687','0.579285896630689','0.209422246970036','6','103']
  H_HDV['4_THC_-6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66371054135772','0.063616829720018','10.0145671468902','0.504744536759053','0.074538495042811','6','103']
  H_HDV['4_NOx_-6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.2688305591534','0.072821399276138','455.390834191429','0.57829750728569','0.323648044919964','6','103']
  H_HDV['4_PM_-6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.159195112192497','0.007805617415443','0.785573611486799','0.077978200318766','-0.02606745716976','6','103']
  H_HDV['4_FC_-6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','456.279960903668','0.060926177619417','9562.8971706188','0.59400112051412','10.6722745815454','6','103']
  H_HDV['4_CO_2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.7402488672166','-0.011882013118708','-1.54934889853565','0.100433794361335','0','6','103']
  H_HDV['4_THC_2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.61845451678079','0.039995675408677','3.8667551651222','0.231632692527358','0.226166819965399','6','103']
  H_HDV['4_NOx_2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','8.84438460498245','93.5952875300646','2.23625875445245','1.39220906780266','0.023124574904368','6','103']
  H_HDV['4_PM_2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.307115804110951','0.023501568798676','0.6611985283586','0.101801897283225','0.097131021184337','6','103']
  H_HDV['4_FC_2_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','794.673333210332','0.068435639912135','4492.61256219123','0.500245741946607','329.635589045531','6','103']
  H_HDV['4_CO_2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.5970268795743','-0.012963452057306','-1.66405794337842','0.103272249781703','0','6','103']
  H_HDV['4_THC_2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5857831059338','0.040620763378349','3.75941061370499','0.22354266346995','0.23504219139179','6','103']
  H_HDV['4_NOx_2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','9.16357457333587','78.3529804282988','2.93082344067743','1.64024817995495','0.007300685323203','6','103']
  H_HDV['4_PM_2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.312049585459945','0.023927800980551','0.675923778239892','0.104664172878157','0.102393400647301','6','103']
  H_HDV['4_FC_2_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','778.524130835072','0.066074271766205','5076.74198767392','0.518767261388084','357.221032794094','6','103']
  H_HDV['4_CO_2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.3079035633477','-0.014339971940452','-1.74992461413414','0.105139329185295','0','6','103']
  H_HDV['4_THC_2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56477773114814','0.041870497445519','3.70375946478676','0.218969653982875','0.249348756652048','6','103']
  H_HDV['4_NOx_2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','9.12558333514139','75.0297094204587','3.21403156467247','1.76513819084543','-0.005812209772333','6','103']
  H_HDV['4_PM_2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.295121772344372','0.024626806422914','0.681600747009487','0.101015517912246','0.113808476468829','6','103']
  H_HDV['4_FC_2_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','778.890571667006','0.064673953474038','5958.1882145398','0.551025113377864','385.079951703445','6','103']
  H_HDV['4_CO_4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.0111607874328','-0.018006322059341','-2.05263640451242','0.113501924933276','0','6','103']
  H_HDV['4_THC_4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.80197436449387','0.048110816226588','3.65232261008948','0.238072060423832','0.295362860661911','6','103']
  H_HDV['4_NOx_4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.04807316523396','0.031504010388185','78.997609818033','0.151066371039192','11.7763945751785','6','103']
  H_HDV['4_PM_4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.149083588291172','0.800042087829921','4.11005411237572','1.55823445255515','0.007517375409376','6','103']
  H_HDV['4_FC_4_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','3189.22454063905','1.0080377668958','-0.579097598244545','0','0','6','103']
  H_HDV['4_CO_4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.0888459750764','-0.020200829502689','-2.14636453005539','0.113090833972472','0','6','101']
  H_HDV['4_THC_4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.86141234394489','0.052744356653632','3.49342249666905','0.23867915815419','0.334219917114987','6','101']
  H_HDV['4_NOx_4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.831984538569','0.038761627012551','78.028045638087','0.155780492865241','13.0815832714005','6','101']
  H_HDV['4_PM_4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.09346262060198','0.00035455026315','-0.059114072702826','0.064227825564043','0','6','101']
  H_HDV['4_FC_4_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','3121.00320032355','1.00772972015133','-0.544136560921262','0','0','6','101']
  H_HDV['4_CO_4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.8771673320057','-0.021927732329986','-2.18153515040066','0.111319900995064','0','6','97']
  H_HDV['4_THC_4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.71465879496813','0.052686768358948','3.29195677100771','0.213072555508583','0.360670109717381','6','97']
  H_HDV['4_NOx_4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8329972111167','0.03860360293342','77.9986788459615','0.155417759983842','14.062363078927','6','97']
  H_HDV['4_PM_4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.13066468423118','0.00037398131922','-0.063380512058686','0.067419465433624','0','6','97']
  H_HDV['4_FC_4_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','4','100','FC','y=((a*(b**x))*(x**c))','3028.90342336818','1.00714303011899','-0.505597863008722','0','0','6','97']
  H_HDV['4_CO_6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.5977849499664','-0.022992611656052','-2.33011180249523','0.114762093268392','0','6','91']
  H_HDV['4_THC_6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.371045027069974','14.2078983287592','-0.293795365000097','0.667606609156891','0.035832102876628','6','91']
  H_HDV['4_NOx_6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.30382906646086','0.024943624001912','81.4102224351591','0.143882634074091','14.2460851523835','6','91']
  H_HDV['4_PM_6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.1938778586964','0.000335060558658','-0.070323947707452','0.07122500145737','0','6','91']
  H_HDV['4_FC_6_0'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','832.019497425282','0.078434644066474','4237.35188851156','0.512441471366795','623.99928748123','6','91']
  H_HDV['4_CO_6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.6723884348921','-0.029578648249316','-2.36890429785053','0.112404429863662','0','6','82']
  H_HDV['4_THC_6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.28190435979079','-0.003973122856242','-0.396064134595086','0.110158377955646','0','6','82']
  H_HDV['4_NOx_6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','13.9502222005396','65.2038478333331','4.51562416821817','2.1604246458524','-0.026824851460436','6','82']
  H_HDV['4_PM_6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.228209603976267','0.724172294364495','6.57133602753411','2.55801704044061','-0.014190023425528','6','82']
  H_HDV['4_FC_6_50'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','752.989535668195','0.074312924742442','3687.29215490945','0.461420069375181','697.198327442531','6','82']
  H_HDV['4_CO_6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.8003692508774','-0.030573332711028','-2.40240630029975','0.108741429382856','0','6','76']
  H_HDV['4_THC_6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.09533670395796','0.070175468400637','2.66074979379837','0.221179204549088','0.496149887904843','6','76']
  H_HDV['4_NOx_6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','18.2540791845881','59.9965333105923','4.88315944215588','2.23816994456874','-0.011460173731105','6','76']
  H_HDV['4_PM_6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.2920178440358','0.002529116063073','-0.077898068639587','0.069566430404863','0','6','76']
  H_HDV['4_FC_6_100'] = [' 4','EURO3','Coach Std <=18t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','933.6327385794','0.087438654255372','16643.0787613295','0.784331971060082','781.85414633716','6','76']
  H_HDV['5_CO_0_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.555901298336896','0.048799362837864','1.34971681284139','0.201486438885508','0.083152548877112','6','103']
  H_HDV['5_THC_0_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010702533691137','1.11066793393796','-0.693725546484866','0.697027183693401','0.023649994624382','6','103']
  H_HDV['5_NOx_0_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3405705412772','0.055077651010815','45.8942800737223','0.271774276533129','3.17494503850689','6','103']
  H_HDV['5_PM_0_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.14313473931191','0.051368709129152','0.458074026555082','0.225837001136062','0.019924792941483','6','103']
  H_HDV['5_FC_0_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','686.839825778743','0.055687044138147','3621.07998999325','0.438098107512592','175.264508060917','6','103']
  H_HDV['5_CO_0_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.068173061605537','2.16890242564152','1.69231714063082','1.15487817176261','0.0126517312697','6','103']
  H_HDV['5_THC_0_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010905614798994','1.02147340072376','-0.581420694990767','0.706338179056865','0.024721988677982','6','103']
  H_HDV['5_NOx_0_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3535876017876','0.053103648204429','47.1957136966233','0.277569463120213','3.28872039233486','6','103']
  H_HDV['5_PM_0_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.14512021281482','0.052008221890407','0.460849925080576','0.227087949802011','0.020223013717455','6','103']
  H_HDV['5_FC_0_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','684.674248298268','0.053666299048214','4077.74009360636','0.458265247215033','180.597864459991','6','103']
  H_HDV['5_CO_0_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.073810704283937','2.26400142075276','1.54734209807907','1.10017568236086','0.015996267799912','6','103']
  H_HDV['5_THC_0_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011195213835253','2.03465293467652','-1.42976985595848','0.649305917201575','0.027516231425167','6','103']
  H_HDV['5_NOx_0_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3725873173088','0.051009428128067','47.801289514428','0.281459689935614','3.38652950420639','6','103']
  H_HDV['5_PM_0_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.147576741360261','0.052489676084191','0.461249462505772','0.228732713185136','0.020419023898756','6','103']
  H_HDV['5_FC_0_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','682.878187215885','0.051609447118986','4446.98001352227','0.47454326948578','185.331374977265','6','103']
  H_HDV['5_CO_-2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','0','CO','y=((a*(b**x))*(x**c))','2.83720558533179','0.990103557143569','-0.712248692954589','0','0','6','103']
  H_HDV['5_THC_-2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','0','THC','y=((a*(b**x))*(x**c))','0.46121349683994','0.991300538879732','-0.69480761259013','0','0','6','103']
  H_HDV['5_NOx_-2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','0','NOx','y=((a*(x**b))+(c*(x**d)))','-0.000539712507974','1.45103911037866','112.392073971429','-1.01667295236787','0','6','103']
  H_HDV['5_PM_-2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.13862616748786','-2.53269462199333','-1.22672468005332','0','0','6','103']
  H_HDV['5_FC_-2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','566.167050703193','0.042043553655523','3541.05242248555','0.397071546975337','41.7609160811227','6','103']
  H_HDV['5_CO_-2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','50','CO','y=((a*(b**x))*(x**c))','2.682306359004','0.987724635522737','-0.682480149296098','0','0','6','103']
  H_HDV['5_THC_-2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','50','THC','y=((a*(b**x))*(x**c))','0.448867391755208','0.989711058742234','-0.68357362710615','0','0','6','103']
  H_HDV['5_NOx_-2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3900672882889','0.045695504361948','65.5396742214988','0.33433954219432','0.671819187559841','6','103']
  H_HDV['5_PM_-2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','1.32030073076729','-3.09522218004117','-1.28263776527564','0','0','6','103']
  H_HDV['5_FC_-2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','575.863063117209','0.04138260332851','3912.56722699139','0.419205042389317','35.0161296945597','6','103']
  H_HDV['5_CO_-2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','100','CO','y=((a*(b**x))*(x**c))','2.47781728101106','0.985039358091132','-0.642676011579955','0','0','6','103']
  H_HDV['5_THC_-2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','100','THC','y=((a*(b**x))*(x**c))','0.429796197967057','0.98788959158047','-0.664888895265717','0','0','6','103']
  H_HDV['5_NOx_-2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.007417384824485','0.988439778649687','102.215848575329','-0.978567535290395','0','6','103']
  H_HDV['5_PM_-2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.922454836299959','0.553879704357238','0.006600063409219','0','0','6','103']
  H_HDV['5_FC_-2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','582.67971871351','0.040668917916845','4313.20510867714','0.441544282083476','29.2073892868582','6','103']
  H_HDV['5_CO_-4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.713841928196516','0.05960219895893','3.14073569587517','0.562683583958226','0.015556506825511','6','103']
  H_HDV['5_THC_-4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.112730700402531','0.054982335701413','0.243381817010662','0.352230002459515','0.00399113376948','6','103']
  H_HDV['5_NOx_-4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.6791703247207','0.062619733492868','109.772126737708','0.455014349951669','0.193160398194736','6','103']
  H_HDV['5_PM_-4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.02416563427377','0.34222744816495','0.019547939662233','0','0','6','103']
  H_HDV['5_FC_-4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','615.544353467796','0.057168352664877','6266.04099367542','0.532369283391589','10.7448066101414','6','103']
  H_HDV['5_CO_-4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.70445313688457','0.06106523766194','4.04631111519357','0.615190846710822','0.014257765601344','6','103']
  H_HDV['5_THC_-4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.110647346744606','0.056391408873496','0.260573836840973','0.367470770636913','0.003841388540551','6','103']
  H_HDV['5_NOx_-4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1937888617939','0.062031135756668','130.071974752101','0.483694320690749','0.168859768143558','6','103']
  H_HDV['5_PM_-4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.07935841893188','0.33851922782937','0.021127291554223','0','0','6','103']
  H_HDV['5_FC_-4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','593.592593064575','0.056913669415201','7290.72914500068','0.558222916930719','9.37787298428784','6','103']
  H_HDV['5_CO_-4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.696129928020258','0.062324909662852','6.04412095415275','0.689333205227196','0.013639465152565','6','103']
  H_HDV['5_THC_-4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.109315715951157','0.057844945573998','0.298131643666599','0.393308393696755','0.003816014327059','6','103']
  H_HDV['5_NOx_-4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.70211763681','0.061117033108978','152.729065341998','0.509781493938505','0.150464539391912','6','103']
  H_HDV['5_PM_-4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.0658652466582','0.346930489643026','0.022134292194065','0','0','6','103']
  H_HDV['5_FC_-4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','572.464275005295','0.056393839686223','8601.2076783378','0.584831234030796','8.42231010142404','6','103']
  H_HDV['5_CO_-6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.68024731404643','0.069614470657879','10.0672400382759','0.80819390409221','0.010880984609985','6','103']
  H_HDV['5_THC_-6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.110963371294492','0.064598939414353','0.382846384967521','0.45244848099221','0.003612986056266','6','103']
  H_HDV['5_NOx_-6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.9647552474695','0.071186569393771','164.63093586197','0.527421111662489','0.071311053127568','6','103']
  H_HDV['5_PM_-6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00343283459307','0.845915738347989','0.608037683654321','0.892305904063193','0.037963469097326','6','103']
  H_HDV['5_FC_-6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','539.220001798763','0.067229254563263','8247.18295021921','0.581577260347646','5.32802650961428','6','103']
  H_HDV['5_CO_-6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.633619087436541','0.069200554066504','15.0231511369454','0.849732864623063','0.010371994891103','6','103']
  H_HDV['5_THC_-6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.103850372200554','0.06419055737165','0.472568270354128','0.475112449322382','0.003535582684102','6','103']
  H_HDV['5_NOx_-6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0846358523303','0.069522617627495','190.152032119563','0.549394581379311','0.058937410778012','6','103']
  H_HDV['5_PM_-6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003343573836395','1.6938557037254','-0.381250192014285','0.813209791283832','0.038900600195711','6','103']
  H_HDV['5_FC_-6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','492.739989070525','0.065624297955527','9611.19310697903','0.601524489629051','4.63380131566035','6','103']
  H_HDV['5_CO_-6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.583535360432204','0.06784042063351','14.2088083922199','0.820631542921295','0.00966449411079','6','103']
  H_HDV['5_THC_-6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.097743413031312','0.06363807159911','0.519757866560015','0.482674755922718','0.003471753329609','6','103']
  H_HDV['5_NOx_-6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.41531858269147','0.068130567277534','209.961860127504','0.565882310100983','0.049861528865202','6','103']
  H_HDV['5_PM_-6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003095650889842','2.482759924918','-0.803435046678697','0.831132025045721','0.036509852813279','6','103']
  H_HDV['5_FC_-6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','456.15464863973','0.064175087730324','10787.3043901571','0.618155057098032','4.10538346443972','6','103']
  H_HDV['5_CO_2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108169893593335','1.20819854725952','5.20785145863256','2.25813751966827','-0.011683470555353','6','103']
  H_HDV['5_THC_2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.0841372969354','0.037835994550973','0.214321694343948','0.209123079570337','0.009205743206895','6','103']
  H_HDV['5_NOx_2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.7578068522399','0.081371594112771','43.4086862985438','0.301711632297015','5.99767931413016','6','103']
  H_HDV['5_PM_2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.101067123795027','0.037600715314357','0.434064198792026','0.173140461321255','0.021609240770242','6','103']
  H_HDV['5_FC_2_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','759.891800840842','0.067177543334923','4350.48990317965','0.481808535796279','309.592269958656','6','103']
  H_HDV['5_CO_2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.119959518537836','1.2158603943512','5.37038480515135','2.31962319920609','-0.012125478129096','6','103']
  H_HDV['5_THC_2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082428056507631','0.038429562526368','0.211416021582286','0.203337845331792','0.009433910929322','6','103']
  H_HDV['5_NOx_2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.37246347208291','164.744780304427','-0.238001176023357','0.893073854944258','0.041609362574861','6','103']
  H_HDV['5_PM_2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.091666177136798','0.033850605510775','0.434078379326292','0.163546004143784','0.021064842788664','6','103']
  H_HDV['5_FC_2_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','755.482769001135','0.065786179221311','4755.0022687016','0.498141264129902','335.081293064355','6','103']
  H_HDV['5_CO_2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.02577676897403','-0.001035888433719','-0.210022458475864','0.116546559323542','0','6','103']
  H_HDV['5_THC_2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080841262118466','0.039281211777513','0.20950179821581','0.198811125321724','0.009845310186489','6','103']
  H_HDV['5_NOx_2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.68169093538679','93.4817253815382','0.718389210874841','1.10052063150532','0.025410981137907','6','103']
  H_HDV['5_PM_2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082351582433408','0.028621686190099','0.439877321407924','0.155707287961673','0.019243659886566','6','103']
  H_HDV['5_FC_2_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','754.147534792626','0.064384752579029','5239.32031277533','0.517475181272303','360.09291719286','6','103']
  H_HDV['5_CO_4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.44341172984923','-0.001488723882981','-0.293638849877899','0.135720318737876','0','6','103']
  H_HDV['5_THC_4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095586429402907','0.045874214958453','0.202386487942956','0.215736312534742','0.011327193747121','6','103']
  H_HDV['5_NOx_4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','48.1751026116218','-0.02204110207085','-5.33540010931138','0.139528303804523','0','6','103']
  H_HDV['5_PM_4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.076093683274712','0.013431009314622','0.453173528398802','0.13812067287412','0.003642265954772','6','103']
  H_HDV['5_FC_4_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','3271.14723905285','1.00847078110046','-0.605938601791348','0','0','6','103']
  H_HDV['5_CO_4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.57498794682613','-0.002006900771461','-0.312920878890962','0.138373101451535','0','6','101']
  H_HDV['5_THC_4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095289369959953','0.048621150529222','0.196390730237895','0.209708483108648','0.012530579763745','6','101']
  H_HDV['5_NOx_4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.10320924141728','0.015154735022389','39.2486188277043','0.154936203614525','7.41769946181689','6','101']
  H_HDV['5_PM_4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082477818630969','0.020132384705232','0.465319183179808','0.147490619548863','0.013057985906271','6','101']
  H_HDV['5_FC_4_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','50','FC','y=((a*(b**x))*(x**c))','3188.4376950164','1.008125284734','-0.570278911107478','0','0','6','101']
  H_HDV['5_CO_4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.67839498212075','-0.002214714761985','-0.322600701681934','0.137466943861735','0','6','99']
  H_HDV['5_THC_4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094428155769069','0.051406682741796','0.19058288396534','0.203295173332187','0.013951432605502','6','99']
  H_HDV['5_NOx_4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.15775105310302','0.017802439462037','39.7512036877568','0.161509843246124','8.03184405580597','6','99']
  H_HDV['5_PM_4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080939604557324','0.023525894191763','0.468176412596893','0.146548303281389','0.019147493411098','6','99']
  H_HDV['5_FC_4_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','4','100','FC','y=((a*(b**x))*(x**c))','3131.43305729878','1.00787286310153','-0.539660067707683','0','0','6','99']
  H_HDV['5_CO_6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.91550975692293','-0.002988513560554','-0.376665524675266','0.148515141548688','0','6','93']
  H_HDV['5_THC_6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.103161319563564','0.055441854098535','0.177672560218283','0.207018137500879','0.014999511870575','6','93']
  H_HDV['5_NOx_6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.76740452363211','0.01733701296283','42.4434174087518','0.17144942140371','8.46153208550009','6','93']
  H_HDV['5_PM_6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.088516297801897','0.018020532510112','0.484348063076797','0.146654114764205','0.009891820045293','6','93']
  H_HDV['5_FC_6_0'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','892.927853400355','0.083773629185605','4874.22089036512','0.55218862314041','592.709979418989','6','93']
  H_HDV['5_CO_6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.97444595477052','-0.004069453549871','-0.364114958371335','0.142451154434868','0','6','84']
  H_HDV['5_THC_6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014353173520312','0.247457004249405','2.47129617223098','1.3082426052783','0.014786619885663','6','84']
  H_HDV['5_NOx_6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.2881084455714','0.033370909422903','43.7275038518373','0.198383127633913','10.462405338754','6','84']
  H_HDV['5_PM_6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.11016488572218','0.012952832006107','0.49632765019126','0.144830711096465','-0.011889560572766','6','84']
  H_HDV['5_FC_6_50'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','810.612881331836','0.079741594031057','3753.95705529165','0.478071877684437','660.006699197944','6','84']
  H_HDV['5_CO_6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.04411576172022','-0.004519668621954','-0.352922569022937','0.135347401255438','0','6','77']
  H_HDV['5_THC_6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.10055222138605','0.064304104132773','0.162321536717126','0.185855552659581','0.019303529762074','6','77']
  H_HDV['5_NOx_6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.90307577656422','0.020079751252152','43.906479989265','0.184738863506432','9.57327422884465','6','77']
  H_HDV['5_PM_6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108768005258706','0.031435530858374','0.483975681820847','0.154123606533779','0.023928028231276','6','77']
  H_HDV['5_FC_6_100'] = [' 5','EURO4','Coach Std <=18t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','959.505634581775','0.091436757837487','11121.3085275278','0.717103111328215','738.571776597451','6','77']
  H_HDV['6_CO_0_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.5697687580462','0.048426059723944','1.39356728883867','0.201102974511113','0.084774140029873','6','103']
  H_HDV['6_THC_0_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011073739231845','1.21939688328453','-0.77257606773924','0.690994018177013','0.023690151219202','6','103']
  H_HDV['6_NOx_0_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.88863326568083','0.055203524231413','32.2165068160941','0.255670057489258','1.8831407944291','6','103']
  H_HDV['6_PM_0_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.147447758300341','0.051188123264388','0.473172383083325','0.225785069084378','0.02039963577403','6','103']
  H_HDV['6_FC_0_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','714.506607650021','0.055414376917201','3670.18415266806','0.431328799704804','179.660780982674','6','103']
  H_HDV['6_CO_0_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.06861026367615','2.27015017262344','1.65614651735167','1.14944965032956','0.012349161854387','6','103']
  H_HDV['6_THC_0_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01126647116598','1.05305351883026','-0.582695620949416','0.705085483980276','0.024599946859957','6','103']
  H_HDV['6_NOx_0_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.88717768372985','0.053446172345991','32.7918039161749','0.259123636356854','1.94837422617714','6','103']
  H_HDV['6_PM_0_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.148559240660242','0.051605786029699','0.473309505416681','0.225594295185903','0.020650208794583','6','103']
  H_HDV['6_FC_0_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','712.579790714479','0.053541000700669','4086.27546399386','0.449767909287385','184.823258175661','6','103']
  H_HDV['6_CO_0_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.073955134104428','2.34332037932604','1.55027866358774','1.10699825252954','0.015220068418843','6','103']
  H_HDV['6_THC_0_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011573746569527','2.27165648916929','-1.51773003818269','0.646163340350282','0.027398572620823','6','103']
  H_HDV['6_NOx_0_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.91523776401893','0.051656185804025','33.18284886646','0.262342973907322','2.00466052745926','6','103']
  H_HDV['6_PM_0_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.151511078520349','0.052251746576285','0.476518472518167','0.22825031181612','0.0208898932199','6','103']
  H_HDV['6_FC_0_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','710.447103226996','0.051600007270039','4430.15555484553','0.46462266064379','189.420517824433','6','103']
  H_HDV['6_CO_-2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','0','CO','y=((a*(b**x))*(x**c))','2.9216321852925','0.99010246551191','-0.710667343518898','0','0','6','103']
  H_HDV['6_THC_-2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','0','THC','y=((a*(b**x))*(x**c))','0.474960644555935','0.991264577288525','-0.692446851309571','0','0','6','103']
  H_HDV['6_NOx_-2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.219012935456113','145.649853768644','-0.352976142744957','1.11004768588492','-0.00090000450728','6','103']
  H_HDV['6_PM_-2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.16646202092534','-2.52381100740677','-1.22504380035769','0','0','6','103']
  H_HDV['6_FC_-2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','589.992656785103','0.041931139058352','3660.51441322434','0.393990320293246','42.8979937243363','6','103']
  H_HDV['6_CO_-2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','50','CO','y=((a*(b**x))*(x**c))','2.78401106457977','0.987907206908581','-0.684784185577314','0','0','6','103']
  H_HDV['6_THC_-2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','50','THC','y=((a*(b**x))*(x**c))','0.464511580488451','0.989795154097853','-0.683370325919238','0','0','6','103']
  H_HDV['6_NOx_-2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.005470524373907','0.863106346852157','85.1372494708986','-1.06827775957258','0','6','103']
  H_HDV['6_PM_-2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','1.33458195990784','-3.03508395085863','-1.27728058432268','0','0','6','103']
  H_HDV['6_FC_-2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','599.902599258071','0.041365135573169','3978.36815379731','0.41335968382254','36.219830081178','6','103']
  H_HDV['6_CO_-2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','100','CO','y=((a*(b**x))*(x**c))','2.58054628101045','0.985309085466575','-0.646470537329112','0','0','6','103']
  H_HDV['6_THC_-2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','100','THC','y=((a*(b**x))*(x**c))','0.44578121739186','0.988029043403624','-0.665334471622962','0','0','6','103']
  H_HDV['6_NOx_-2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.00656333777467','0.87204922045692','81.9099349286494','-1.05427942354439','0','6','103']
  H_HDV['6_PM_-2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.875877753535127','0.538291717934504','0.006256784745604','0','0','6','103']
  H_HDV['6_FC_-2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','607.511290312042','0.040770264473214','4324.82409000916','0.433541491576425','30.4913155312169','6','103']
  H_HDV['6_CO_-4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.736059422190316','0.059248422191967','3.13050551491395','0.553730238221782','0.015945532942392','6','103']
  H_HDV['6_THC_-4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.116518916628763','0.054706171576773','0.250620866456572','0.350638416799445','0.004097552020265','6','103']
  H_HDV['6_NOx_-4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.57852128089307','0.064852473138715','69.7337013562048','0.428498711960501','0.12292770436913','6','103']
  H_HDV['6_PM_-4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.95802408770358','0.331121694095505','0.018793987378465','0','0','6','103']
  H_HDV['6_FC_-4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','644.791313004065','0.057155100350843','6394.90214820335','0.527862049804764','11.1528865180545','6','103']
  H_HDV['6_CO_-4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.72869298919354','0.060794820080839','4.03133717097936','0.6060750994171','0.014682356633648','6','103']
  H_HDV['6_THC_-4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.114728140285105','0.056174576874789','0.267797318144181','0.366028598381336','0.003957925800032','6','103']
  H_HDV['6_NOx_-4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.32183169503696','0.06457094650225','81.4200922011855','0.455641864686185','0.109793662014491','6','103']
  H_HDV['6_PM_-4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.0109487679218','0.326786003843815','0.020308186545511','0','0','6','103']
  H_HDV['6_FC_-4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','625.108730761894','0.057105560595417','7437.80686273019','0.554342515336456','9.85365630935523','6','103']
  H_HDV['6_CO_-4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.722538808428039','0.062184051657935','5.56593319417753','0.669598651402394','0.01412609998746','6','103']
  H_HDV['6_THC_-4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.113337792321803','0.0575808388957','0.291636661348357','0.38404040149699','0.003926430347093','6','103']
  H_HDV['6_NOx_-4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.01562615068372','0.063790797922795','92.9016414198554','0.477582706500661','0.099224798603154','6','103']
  H_HDV['6_PM_-4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.02278070658626','0.329925886890199','0.021390681852962','0','0','6','103']
  H_HDV['6_FC_-4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','604.002067763126','0.056675117265706','8484.84305995315','0.57576843455554','8.90449390445278','6','103']
  H_HDV['6_CO_-6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.716565394572068','0.070079634300194','14.7478553725172','0.877432021302752','0.011440625592105','6','103']
  H_HDV['6_THC_-6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115746744584038','0.064678234138801','0.385177526986803','0.45073448728491','0.003735427558192','6','103']
  H_HDV['6_NOx_-6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.57425158797681','0.07324362722121','104.893567070967','0.503681673756073','0.045802331085492','6','103']
  H_HDV['6_PM_-6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003532499400781','0.791589441982411','0.77718614335669','0.91460531572715','0.037487476481768','6','103']
  H_HDV['6_FC_-6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','570.797678238513','0.067493638473024','8433.59311880738','0.579287407671371','5.62817169503259','6','103']
  H_HDV['6_CO_-6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.664597833417353','0.069407821075079','17.4856145746118','0.875960767221739','0.010791529843562','6','103']
  H_HDV['6_THC_-6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108771255765491','0.06435758499759','0.485444349042697','0.477753627076407','0.003660864208225','6','103']
  H_HDV['6_NOx_-6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.01771028095299','0.07174156452101','122.049486808283','0.527668879725829','0.038648419790538','6','103']
  H_HDV['6_PM_-6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003486379727878','1.65365938050269','-0.321892524811943','0.807960682760637','0.039378912635258','6','103']
  H_HDV['6_FC_-6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','523.185266977097','0.065968818084665','9853.46948718521','0.599580838834293','4.93510359300552','6','103']
  H_HDV['6_CO_-6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.617310972578241','0.068392360023047','15.0574889077229','0.831374008287235','0.010200076628374','6','103']
  H_HDV['6_THC_-6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.102682954410586','0.063899425435826','0.523138192676832','0.481075538606147','0.003603234001407','6','103']
  H_HDV['6_NOx_-6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.57853703728365','0.070433265428242','134.023087925209','0.54290632924032','0.03311061621397','6','103']
  H_HDV['6_PM_-6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003287510260618','2.35809170324801','-0.721605185484787','0.820756960163539','0.037474919200204','6','103']
  H_HDV['6_FC_-6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','486.138346688328','0.064666520674315','10850.6668801246','0.612448172767473','4.42733039716904','6','103']
  H_HDV['6_CO_2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108341282736105','1.2537019935937','5.12697636244352','2.22838905460791','-0.011372267532288','6','103']
  H_HDV['6_THC_2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087121747664492','0.037776615284231','0.221296345567764','0.209874476516261','0.009507100281085','6','103']
  H_HDV['6_NOx_2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.53882240369628','50.4744978828107','1.07808397526009','1.04536147247011','0.046459646705967','6','103']
  H_HDV['6_PM_2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.104762603032937','0.037688224708429','0.448201785441974','0.173896219546575','0.022137291978114','6','103']
  H_HDV['6_FC_2_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','785.373937977746','0.066357935316727','4341.59498647808','0.470771252385118','316.545905589204','6','103']
  H_HDV['6_CO_2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.120098627406163','1.2550196520324','5.31570951380627','2.29851099561099','-0.011935588599893','6','103']
  H_HDV['6_THC_2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.085364712970179','0.038220878797189','0.218530282653271','0.204195607847522','0.009659120360623','6','103']
  H_HDV['6_NOx_2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.74676041546762','36.6642649102172','1.94046077895392','1.32431959879265','0.028310116478671','6','103']
  H_HDV['6_PM_2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.098627782521922','0.035458978054732','0.447411809132483','0.166845964316486','0.022109290911332','6','103']
  H_HDV['6_FC_2_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','779.160981559922','0.065066882492265','4679.49359811138','0.484004924553843','341.558417750055','6','103']
  H_HDV['6_CO_2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.05889280543606','-0.000989979252805','-0.210977088064507','0.114938581389034','0','6','103']
  H_HDV['6_THC_2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083513910741396','0.038756257434351','0.216749325916608','0.199331695393516','0.009934884988033','6','103']
  H_HDV['6_NOx_2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.91569054070619','40.0332931002625','1.83832121659182','1.36079208445482','0.018672795088762','6','103']
  H_HDV['6_PM_2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087516191992401','0.030225047145934','0.452053642220562','0.157808129560923','0.02055370535282','6','103']
  H_HDV['6_FC_2_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','776.214080206813','0.063631444413998','5063.08344651155','0.499363375746122','365.815979192987','6','103']
  H_HDV['6_CO_4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.47325987548905','-0.001362543738278','-0.291812455962522','0.132919719616784','0','6','103']
  H_HDV['6_THC_4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095545203247392','0.043933641517665','0.208972354158247','0.210053282634446','0.011114023506423','6','103']
  H_HDV['6_NOx_4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.2494885977345','-0.017161946549616','-4.31195093986106','0.148087779924956','0','6','103']
  H_HDV['6_PM_4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.081982175565914','0.010938857437184','0.465100697809905','0.135549519853112','-0.00372110982786','6','103']
  H_HDV['6_FC_4_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','783.310976870609','0.070328737227787','3549.26971441853','0.438873991825712','455.914794413964','6','103']
  H_HDV['6_CO_4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.62038524558101','-0.001926768555242','-0.316535319032726','0.137270533820024','0','6','102']
  H_HDV['6_THC_4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.097364720804513','0.047394290580565','0.204336926444046','0.209195691437721','0.012453556908127','6','102']
  H_HDV['6_NOx_4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.0393385894944','-0.023712195408563','-4.35690481170039','0.149261648977313','0','6','102']
  H_HDV['6_PM_4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083376542104786','0.018340961066979','0.47752912231108','0.145155457876463','0.011073079448868','6','102']
  H_HDV['6_FC_4_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','50','FC','y=((a*(b**x))*(x**c))','3335.92733978138','1.00806097298155','-0.575792521691509','0','0','6','102']
  H_HDV['6_CO_4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.72927652113267','-0.002222205541933','-0.328629674034791','0.137276405951063','0','6','101']
  H_HDV['6_THC_4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.099763356511277','0.051591337366398','0.197367222962303','0.208078638048146','0.014218974318014','6','101']
  H_HDV['6_NOx_4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.5366180634451','-0.029042271938949','-4.29907272785233','0.147785758031408','0','6','101']
  H_HDV['6_PM_4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086923283115391','0.025454960878273','0.481316920392486','0.149655069970041','0.020717474993954','6','101']
  H_HDV['6_FC_4_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','4','100','FC','y=((a*(b**x))*(x**c))','3294.5865681129','1.0079677957842','-0.549499588526558','0','0','6','101']
  H_HDV['6_CO_6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.00484801472502','-0.00294713621309','-0.392313493542261','0.149805381661938','0','6','95']
  H_HDV['6_THC_6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.110609264092373','0.056131070113834','0.184663742216051','0.216054435963769','0.01531697725178','6','95']
  H_HDV['6_NOx_6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','39.2604106941933','-0.034703599783516','-4.91388827274743','0.158227951084852','0','6','95']
  H_HDV['6_PM_6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.090336746807083','0.018269023783241','0.499960655157581','0.147558059399775','0.011124400512446','6','95']
  H_HDV['6_FC_6_0'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','3333.53939742191','1.00832257737272','-0.537489433181584','0','0','6','95']
  H_HDV['6_CO_6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.11983407771711','-0.00398056405713','-0.398327859295798','0.148118328929926','0','6','86']
  H_HDV['6_THC_6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015611506902565','0.303780810320209','1.78543923819208','1.07508879604075','0.026063407476945','6','86']
  H_HDV['6_NOx_6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.90972749751653','0.014550832464941','32.3605523291856','0.1784842600944','3.76100298227192','6','86']
  H_HDV['6_PM_6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.105804885585336','0.014472798490786','0.510123795161386','0.145642608064475','-0.003913473593053','6','86']
  H_HDV['6_FC_6_50'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','877.622857833515','0.080793966605368','4313.66506582129','0.504388191488059','672.065300296985','6','86']
  H_HDV['6_CO_6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.13467669241532','-0.004717758580806','-0.371916170301363','0.138585112826548','0','6','77']
  H_HDV['6_THC_6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016416823128791','0.235002771399281','2.82260442547939','1.42297959090311','0.013270281732242','6','77']
  H_HDV['6_NOx_6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.73013960983793','0.017566473117896','31.9931326755746','0.179449981860022','4.26295155408744','6','77']
  H_HDV['6_PM_6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.110397139417457','0.013935822459793','0.516273878716193','0.143016674633248','-0.007134315231014','6','77']
  H_HDV['6_FC_6_100'] = [' 6','EURO5','Coach Std <=18t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','861.875849878824','0.081912109734286','4625.43054913336','0.511130513230043','743.360161318508','6','77']
  H_HDV['7_CO_0_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.79682563490132','26.4986285742148','2.40804837333661','1.35018323435863','-0.001003525479253','6','103']
  H_HDV['7_THC_0_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','0','THC','y=(1/(a+(b*x)))','0.060583149768845','0.022633063031291','0','0','0','6','103']
  H_HDV['7_NOx_0_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.010677550946913','0.002422204308149','-0.000015307937988','0','0','6','103']
  H_HDV['7_PM_0_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.59599836013509','0.047194650141235','3.42581484544012','0.215061982107944','0.32329944693592','6','103']
  H_HDV['7_FC_0_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','924.533703257219','0.058603074614283','3832.62634292606','0.422741837750421','219.25460987095','6','103']
  H_HDV['7_CO_0_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.32546133979232','0.049398322753561','23.3487534487779','0.229077642577511','1.59577896799085','6','103']
  H_HDV['7_THC_0_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.360926425153091','15.0322801024862','1.01765039364994','0.962550426206119','0.015759618517432','6','103']
  H_HDV['7_NOx_0_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','29.3431419502912','0.050675306370405','97.4302346912876','0.376677436878402','9.895033913305','6','103']
  H_HDV['7_PM_0_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.65504968021944','0.046167032322055','3.99630322704817','0.244752631633199','0.346214729297498','6','103']
  H_HDV['7_FC_0_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','898.996872664284','0.054012137378847','4451.05847483387','0.449694547159573','234.426372457067','6','103']
  H_HDV['7_CO_0_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.58925242975391','0.047984218098045','28.5943752356308','0.254457948730829','1.6809745694009','6','103']
  H_HDV['7_THC_0_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.372744777047769','16.1808509952484','0.870543698165298','0.941948133241654','0.018178752028717','6','103']
  H_HDV['7_NOx_0_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','29.5799538378931','0.04485599611668','170.728127300969','0.484687748647044','10.5591057244224','6','103']
  H_HDV['7_PM_0_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68031211366106','0.043691053179066','4.83662067599438','0.271422665353901','0.363731221099495','6','103']
  H_HDV['7_FC_0_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','879.882393905317','0.049304736221809','5800.84772612492','0.494953224775751','248.055680251325','6','103']
  H_HDV['7_CO_-2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.028285912251197','0.009146648311644','0.000050243226445','0','0','6','103']
  H_HDV['7_THC_-2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','0','THC','y=(a-(b*exp(((-1)*c)*(x**d))))','0.035455836102435','-96.2055520602914','1.99553692456244','0.235224025799962','0','6','103']
  H_HDV['7_NOx_-2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','0','NOx','y=((a*(b**x))*(x**c))','130.608708773626','0.991590364065336','-0.723694699298617','0','0','6','103']
  H_HDV['7_PM_-2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.136386200787553','16.1600921596224','-0.580171544204609','0.677615212646385','0.021899536945186','6','103']
  H_HDV['7_FC_-2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','741.223132665923','0.045952444063265','5475.66115290344','0.443051215257749','60.4500251419664','6','103']
  H_HDV['7_CO_-2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.029985391631079','0.008889061753275','0.000075491747072','0','0','6','103']
  H_HDV['7_THC_-2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.45316526500642','0.046134200653541','11.7272008746368','0.322178962533457','0.249460077895652','6','103']
  H_HDV['7_NOx_-2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.1327432322073','0.039221367115812','201.435312864113','0.458199492677569','1.26706132708422','6','103']
  H_HDV['7_PM_-2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72734995155725','0.046325073386016','4.78301396087331','0.31314450367024','0.163848781841646','6','103']
  H_HDV['7_FC_-2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','736.047219540388','0.042536992616138','7748.42551116938','0.503364106082776','39.675447091932','6','103']
  H_HDV['7_CO_-2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4170482943046','0.052048721965698','52.2249277325877','0.439418085987067','0.640060092095601','6','103']
  H_HDV['7_THC_-2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.42975295238871','0.046786162767792','12.519136039856','0.337785735157285','0.212537522120727','6','103']
  H_HDV['7_NOx_-2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.4913209965221','0.036402011275783','341.921657155277','0.554959820815257','0.768085898801081','6','103']
  H_HDV['7_PM_-2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.80018210530276','0.046335221240027','6.89678252469726','0.380616435004795','0.157361674646152','6','103']
  H_HDV['7_FC_-2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','734.8629120691','0.040095212815967','11606.3676836083','0.574685146055928','27.1638429525508','6','103']
  H_HDV['7_CO_-4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5302244621397','0.06073905696914','39.2541348655204','0.451479421363748','0.326688105915632','6','103']
  H_HDV['7_THC_-4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.68441515897906','0.053670092238144','18.3597965436747','0.423531865510137','0.175628305420323','6','103']
  H_HDV['7_NOx_-4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.5412552144219','0.056288250580454','335.580652385197','0.560557572805705','0.48975668279681','6','103']
  H_HDV['7_PM_-4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.73423461666557','0.052314449578163','4.01916413173326','0.310546132830896','0.12265732528149','6','103']
  H_HDV['7_FC_-4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','710.323876744775','0.056541933243135','13111.2948373497','0.604082193857356','13.0120710159986','6','103']
  H_HDV['7_CO_-4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.87561531503436','0.059900370063431','71.3509128266176','0.529962015631886','0.29009498620461','6','103']
  H_HDV['7_THC_-4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.5067105433503','0.05424948141103','23.1647667001738','0.462939111856261','0.164585700488156','6','103']
  H_HDV['7_NOx_-4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.6972150439221','0.052943926174599','545.535072217901','0.638656294675873','0.3548840951989','6','103']
  H_HDV['7_PM_-4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.71971026313754','0.052737071486314','6.46479552271209','0.385618259766818','0.122609657977897','6','103']
  H_HDV['7_FC_-4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','631.40988477377','0.053550737217359','19130.529698242','0.663189289105403','8.58095029435282','6','103']
  H_HDV['7_CO_-4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.10848310714029','0.056677956172779','111.61151562635','0.579557593548088','0.248283751079748','6','103']
  H_HDV['7_THC_-4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.31945490432167','0.054092650938684','25.4178237509928','0.476271290855276','0.160026272937578','6','103']
  H_HDV['7_NOx_-4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.615020891659','0.049292958385318','751.111163682436','0.690925234184344','0.248898831585583','6','103']
  H_HDV['7_PM_-4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68574429082002','0.051578425370387','10.0262478281463','0.451140131896089','0.119401213076536','6','103']
  H_HDV['7_FC_-4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','573.200008988055','0.050560517181846','24806.720888121','0.705379534289755','5.74104440289505','6','103']
  H_HDV['7_CO_-6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.51105325897467','0.06748454751402','59.0827508001723','0.523120857689609','0.249983155071125','6','103']
  H_HDV['7_THC_-6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.33435518387099','0.058261646665156','23.689172834583','0.469172055061798','0.1690023850649','6','103']
  H_HDV['7_NOx_-6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.6784946325857','0.064288422182604','471.629599550762','0.615816621948804','0.252598343837817','6','103']
  H_HDV['7_PM_-6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.62288703681392','0.05687388112051','4.94540126332667','0.345520971336711','0.128668206520551','6','103']
  H_HDV['7_FC_-6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','565.558919398548','0.063321150884721','16695.1611729556','0.641564253158636','5.1414042868632','6','103']
  H_HDV['7_CO_-6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.30680280622092','0.064235519487433','80.8725263213299','0.549898853619833','0.225818845799642','6','103']
  H_HDV['7_THC_-6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.00282073290161','0.056578135916466','25.407733314397','0.472860840328728','0.163114707497638','6','103']
  H_HDV['7_NOx_-6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.2203901124349','0.060100494328407','579.050962257781','0.647007302997083','0.194191393699659','6','103']
  H_HDV['7_PM_-6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54885151305703','0.055835866378441','6.94462952484306','0.397241483747315','0.126886573968697','6','103']
  H_HDV['7_FC_-6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','469.795782513559','0.059550003491744','19722.3301534807','0.666765450175063','3.4466211124358','6','103']
  H_HDV['7_CO_-6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.58452365047696','0.061553276704682','97.7371250850585','0.564589727631943','0.207878242236781','6','103']
  H_HDV['7_THC_-6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.85204769928567','0.056219950856609','27.261577762474','0.483389035174227','0.161967191716443','6','103']
  H_HDV['7_NOx_-6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.603007661006','0.057881903292785','676.083506868554','0.673053769725946','0.168308000197907','6','103']
  H_HDV['7_PM_-6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.49547460665098','0.054623389735812','8.89553732786264','0.431304349917495','0.124690656908836','6','103']
  H_HDV['7_FC_-6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','422.306411284718','0.057521818416245','22845.5910562965','0.691602374801592','2.67296694032837','6','103']
  H_HDV['7_CO_2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.7542873575547','-0.020192926295627','-2.66192187766162','0.109350337944447','0','6','103']
  H_HDV['7_THC_2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.62817067408151','0.041430061848122','8.43610561711328','0.221658378760636','0.35225099021025','6','103']
  H_HDV['7_NOx_2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','0','NOx','y=((a*(b**x))*(x**c))','159.493417172398','1.01066811377311','-0.701096140408326','0','0','6','103']
  H_HDV['7_PM_2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.41058103057712','-0.002972357357944','-0.381521004989659','0.103294655038807','0','6','103']
  H_HDV['7_FC_2_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','0','FC','y=((a*(b**x))*(x**c))','4931.73072999921','1.00916809656185','-0.757301786155149','0','0','6','103']
  H_HDV['7_CO_2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.4478077728512','-0.023908949343302','-3.08867750519094','0.116244020864492','0','6','102']
  H_HDV['7_THC_2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.69227819667215','0.048469112789253','7.63291346897564','0.20780239960345','0.441360421427049','6','102']
  H_HDV['7_NOx_2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','50','NOx','y=((a*(b**x))*(x**c))','136.35957113324','1.00855318372181','-0.580629472424013','0','0','6','102']
  H_HDV['7_PM_2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.766976852473236','0.022446449385341','3.7631549493065','0.129021413644941','0.449073431554394','6','102']
  H_HDV['7_FC_2_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','50','FC','y=((a*(b**x))*(x**c))','4547.71535088935','1.00897420049844','-0.689568588149491','0','0','6','102']
  H_HDV['7_CO_2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.82312207741151','0.024482798446','27.3656187235231','0.131193954367415','1.68525810145918','6','99']
  H_HDV['7_THC_2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.03636676892702','0.047292309370774','7.42191898013506','0.176391901535311','0.492093776701205','6','99']
  H_HDV['7_NOx_2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','100','NOx','y=((a*(b**x))*(x**c))','121.442512085279','1.00660897406398','-0.482292119358589','0','0','6','99']
  H_HDV['7_PM_2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66275915636454','0.049155629643161','3.71245754455254','0.192585195194833','0.631119287259726','6','99']
  H_HDV['7_FC_2_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','2','100','FC','y=((a*(b**x))*(x**c))','4206.74112520645','1.00848733230258','-0.623005667748492','0','0','6','99']
  H_HDV['7_CO_4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.95488249212758','21.2360592070229','4.80514740424459','2.11972812168824','-0.01079081337505','6','100']
  H_HDV['7_THC_4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.18432446576264','0.052669165196795','7.24267862993046','0.211792290638887','0.457307949746327','6','100']
  H_HDV['7_NOx_4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','0','NOx','y=((a*(x**b))+(c*(x**d)))','1.09227693894104','0.601167355466463','154.945704550209','-0.615214045892893','0','6','100']
  H_HDV['7_PM_4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.18513562633785','0.048149950775147','3.56903550175719','0.150543520948323','0.656310686764404','6','100']
  H_HDV['7_FC_4_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','0','FC','y=((a*(b**x))*(x**c))','4741.57516959433','1.01057205858851','-0.686428013041586','0','0','6','100']
  H_HDV['7_CO_4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.97834752877989','0.02086926686847','30.1326547419936','0.13040879560609','1.50442717934022','6','84']
  H_HDV['7_THC_4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.43485719074571','0.021092064428529','8.37993088006457','0.126966499207091','0.305928641029531','6','84']
  H_HDV['7_NOx_4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','32.5541794704274','0.082447310838705','124.039971571151','0.446787785491247','31.8860650079185','6','84']
  H_HDV['7_PM_4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.17222477134229','0.03992404499469','3.95561782138809','0.15098243933488','0.725339288613748','6','84']
  H_HDV['7_FC_4_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1132.81710242855','0.084851688722889','5789.26133832184','0.530965651044144','678.673488613421','6','84']
  H_HDV['7_CO_4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.15446228986869','26.1528085733271','4.605142490655','2.04335467262319','-0.011421851717689','6','72']
  H_HDV['7_THC_4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.31820629722556','0.02583455607124','8.53511575008095','0.133599611169029','0.492832004177257','6','72']
  H_HDV['7_NOx_4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.80502764279754','2.82653169528193','-0.049808211459475','0','0','6','72']
  H_HDV['7_PM_4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.07859125101273','0.063801782860461','3.14357562190546','0.16833357082244','0.946352667159714','6','72']
  H_HDV['7_FC_4_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1094.06357734948','0.086982163050524','8504.60385033897','0.597922892023738','828.739261966644','6','72']
  H_HDV['7_CO_6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.9846033926477','-0.03436177176345','-3.8228336790761','0.122132785301722','0','6','78']
  H_HDV['7_THC_6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.60179130768315','-0.006967254911797','-0.918275444386385','0.107549173216713','0','6','78']
  H_HDV['7_NOx_6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','35.6767113776114','0.095843104347651','92.6537888550704','0.407381670163936','33.4184226635791','6','78']
  H_HDV['7_PM_6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.48383760668607','-0.004765233072768','-0.515218239628088','0.1173767647642','0','6','78']
  H_HDV['7_FC_6_0'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1432.13234982953','0.101175564007885','31451.1075632336','0.906279811855343','708.73623023279','6','78']
  H_HDV['7_CO_6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.56544963363037','23.4438840491975','5.80501613584008','2.45471027949723','-0.017132403581694','6','65']
  H_HDV['7_THC_6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.79053576558559','0.032640050765253','8.55939981825913','0.147214291771344','0.519368695277059','6','65']
  H_HDV['7_NOx_6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.79648833344349','2.95712955109998','-0.023036142473392','0','0','6','65']
  H_HDV['7_PM_6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.81325962322306','-0.002528610390371','-0.492525526374943','0.106381938940729','0','6','65']
  H_HDV['7_FC_6_50'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','780.646106566767','0.073063269658713','3854.0549032127','0.384499648456055','899.739028233804','6','65']
  H_HDV['7_CO_6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','4.43102872205082','24.5533890892635','5.7827999802922','2.32162354425974','0.018746822388045','6','54']
  H_HDV['7_THC_6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.3890003324375','-0.008980522032911','-1.19907956821917','0.132927145187761','0','6','54']
  H_HDV['7_NOx_6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.0119385147977','0.030570548112326','145.374877554373','0.371405915558302','47.991100774402','6','54']
  H_HDV['7_PM_6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.42562352653748','7.44547994730519','0.292625221449667','0.041570953167878','0.140559171044126','6','54']
  H_HDV['7_FC_6_100'] = [' 7','EURO0','Coach 3-Axes >18t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1112.34337958628','0.111656241423876','8164.60184068946','0.593914908002763','1154.86700503362','6','54']
  H_HDV['8_CO_0_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.76635106052366','0.042155329177389','14.5379173993676','0.171341672513958','1.14005464557278','6','103']
  H_HDV['8_THC_0_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.382426800935108','16.8439224785188','0.339457571720196','0.816775240969009','0.018670846966056','6','103']
  H_HDV['8_NOx_0_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.8132656551056','0.058060190275515','84.3150740651779','0.348283497966206','6.81515239092042','6','103']
  H_HDV['8_PM_0_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.1955004155955','0.043115603247918','2.10556044827915','0.184101383649118','0.225249000633911','6','103']
  H_HDV['8_FC_0_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','757.555095656569','0.056285324297407','4210.13026410613','0.453666966593171','192.372182414288','6','103']
  H_HDV['8_CO_0_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.34746615654797','0.043969594994269','16.7604389918787','0.203866839382104','1.27496774900926','6','103']
  H_HDV['8_THC_0_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.379844263660441','12.412848559031','0.870241881603911','0.894905607815503','0.017631293125271','6','103']
  H_HDV['8_NOx_0_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.3329945675314','0.052582694406875','94.2012909848843','0.373081327274208','7.32592398490945','6','103']
  H_HDV['8_PM_0_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.25719461866139','0.044283370608317','2.27223949949041','0.206581691922764','0.245473671489254','6','103']
  H_HDV['8_FC_0_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','747.184269680819','0.051577373132415','5103.04637980811','0.487253488140636','207.151331546296','6','103']
  H_HDV['8_CO_0_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.81904944917887','0.043053130507414','22.3376915289061','0.253045709868162','1.39399465386509','6','103']
  H_HDV['8_THC_0_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.400427724342114','17.7687685040873','0.20548627896028','0.786937440393153','0.021843789077298','6','103']
  H_HDV['8_NOx_0_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.4209319318176','0.047677434295178','123.634830795994','0.426229550648669','7.6977805800255','6','103']
  H_HDV['8_PM_0_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.28708835139635','0.042983263151856','2.86764332199601','0.243078240796197','0.261276454522067','6','103']
  H_HDV['8_FC_0_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','744.77758540806','0.047003715718758','6813.24031123269','0.536763742433635','219.095772309057','6','103']
  H_HDV['8_CO_-2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.3719219563972','-0.013625479431019','-1.16096852767719','0.091760593774915','0','6','103']
  H_HDV['8_THC_-2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','0','THC','y=((a*(b**x))*(x**c))','13.9355644782825','0.990571112815597','-0.689391226586515','0','0','6','103']
  H_HDV['8_NOx_-2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','0','NOx','y=((a*(b**x))*(x**c))','124.59049754225','0.992146510408883','-0.782038139640808','0','0','6','103']
  H_HDV['8_PM_-2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.69140079398005','-0.003203419239414','-0.19600842903079','0.085702898101294','0','6','103']
  H_HDV['8_FC_-2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','629.442369969282','0.044687056517058','4962.89438277275','0.446279601662854','49.6295080879071','6','103']
  H_HDV['8_CO_-2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.043685058223574','0.010546607617314','0.000099966541067','0','0','6','103']
  H_HDV['8_THC_-2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','50','THC','y=((a*(b**x))*(x**c))','13.0265364701846','0.987269574214571','-0.658143274027305','0','0','6','103']
  H_HDV['8_NOx_-2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.4095332908717','0.040243394981698','129.163804449092','0.395604422236532','0.994683924120836','6','103']
  H_HDV['8_PM_-2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.93037655864031','-6.42893151995393','-1.29635599745036','0','0','6','103']
  H_HDV['8_FC_-2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','637.637551096153','0.041902364510494','6568.53403645284','0.497990173992236','35.1281383417627','6','103']
  H_HDV['8_CO_-2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','100','CO','y=((a*(b**x))*(x**c))','33.5451448427963','0.986455028612949','-0.676221799963401','0','0','6','103']
  H_HDV['8_THC_-2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','100','THC','y=((a*(b**x))*(x**c))','12.324068276741','0.984711800441509','-0.633020663978934','0','0','6','103']
  H_HDV['8_NOx_-2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.833853840013','0.038565694324401','182.474993553548','0.46210577010982','0.686534664065351','6','103']
  H_HDV['8_PM_-2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.23480555939981','-7.34764646626341','-1.38488456552181','0','0','6','103']
  H_HDV['8_FC_-2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','647.588149328211','0.039769994499613','9396.05777634247','0.561679961516151','25.3950436424339','6','103']
  H_HDV['8_CO_-4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.127873554011253','81.7386485360213','-1.11592253111458','0.513848838948405','0.038354649597027','6','103']
  H_HDV['8_THC_-4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.16617784387224','0.053226348427109','9.28486128837385','0.360984919286912','0.108444119101336','6','103']
  H_HDV['8_NOx_-4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.0788217526945','0.057852055602299','180.433081686512','0.473825356362728','0.412763883741584','6','103']
  H_HDV['8_PM_-4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.67940608100321','-11.5359544032283','-1.82532447078603','0','0','6','103']
  H_HDV['8_FC_-4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','623.94407373549','0.056280012225677','9689.50625046387','0.578032482919','11.5906064793709','6','103']
  H_HDV['8_CO_-4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.11787168898766','0.060753768777131','31.7795328516934','0.458047152516448','0.197448046626661','6','103']
  H_HDV['8_THC_-4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.04537316809554','0.05460406482067','10.7656329454769','0.387837426691993','0.100872436742349','6','103']
  H_HDV['8_NOx_-4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.3049334378085','0.056466702816437','263.250359777122','0.538981990551536','0.349407498979517','6','103']
  H_HDV['8_PM_-4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.66095929411639','-11.1671582057136','-1.84612408116367','0','0','6','103']
  H_HDV['8_FC_-4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','574.857046380058','0.054186983876603','13160.98761256','0.62757796600205','8.45541942647739','6','103']
  H_HDV['8_CO_-4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.58359104328406','0.058250247880139','56.3361486284995','0.52905173093938','0.172693907141234','6','103']
  H_HDV['8_THC_-4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.89161354778317','0.054640292668083','13.0137444189187','0.412557269432989','0.096993996041603','6','103']
  H_HDV['8_NOx_-4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.6542261909883','0.053736474495306','369.058477791973','0.593981780856788','0.287570889572589','6','103']
  H_HDV['8_PM_-4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','5.38786979670863','-10.0412796481618','-1.7889062190518','0','0','6','103']
  H_HDV['8_FC_-4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','532.391700866238','0.051473755195209','17426.3021115065','0.672363712183385','6.07071883606063','6','103']
  H_HDV['8_CO_-6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.77487569272701','0.068771580930419','27.7190923782096','0.450959978483265','0.148124724588204','6','103']
  H_HDV['8_THC_-6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.95184402766031','0.059944709586153','11.8360082998747','0.407367247883682','0.095822683686646','6','103']
  H_HDV['8_NOx_-6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.3480465577761','0.06805175955623','266.076649099005','0.545929079398789','0.251659686203962','6','103']
  H_HDV['8_PM_-6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','6.01503833323042','-11.8319694787364','-1.98585700027243','0','0','6','103']
  H_HDV['8_FC_-6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','521.045895946482','0.064261799709743','11971.8195743102','0.613079870981937','4.97944226241975','6','103']
  H_HDV['8_CO_-6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.83438483081102','0.065752940741708','34.7771908224756','0.461904371142102','0.130698722139058','6','103']
  H_HDV['8_THC_-6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.62995211042033','0.058266382131254','13.0675412933183','0.40648545161593','0.090923235005467','6','103']
  H_HDV['8_NOx_-6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.4473041832964','0.064674919816836','335.290140725559','0.581279021783171','0.211764936921582','6','103']
  H_HDV['8_PM_-6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.57180362557317','-10.1328979528141','-1.89111212837446','0','0','6','103']
  H_HDV['8_FC_-6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','447.090611101936','0.061134476073737','14112.036662271','0.637516037310946','3.69857284431593','6','103']
  H_HDV['8_CO_-6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.40318717524202','0.064076452431841','49.9236482220022','0.505126441097694','0.121221243233609','6','103']
  H_HDV['8_THC_-6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.44575438555059','0.057339430697512','14.1097982253439','0.410382398121207','0.088503487544981','6','103']
  H_HDV['8_NOx_-6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.8213524962255','0.062302497473856','394.232517052843','0.606565785183574','0.187280433310208','6','103']
  H_HDV['8_PM_-6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','5.20022251993253','-8.75650801343793','-1.8063060565764','0','0','6','103']
  H_HDV['8_FC_-6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','404.097963115607','0.058926379730772','16303.9590416104','0.660516232538829','2.93685189364163','6','103']
  H_HDV['8_CO_2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.119023789758','13.3143043669593','5.72856587496926','2.39099131091283','-0.022222501745218','6','103']
  H_HDV['8_THC_2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.86836219001612','0.042693964270633','5.88877016221643','0.205151505073776','0.482857037365144','6','103']
  H_HDV['8_NOx_2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','28.3315197603427','0.073794854227829','89.5664434985454','0.374828471367608','12.4865717647158','6','103']
  H_HDV['8_PM_2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.336851895370958','2.153786147216','3.52982619263978','1.42643203578719','0.019466409019336','6','103']
  H_HDV['8_FC_2_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','879.7079264615','0.070584713732715','5473.0787051774','0.516409426749083','339.874389037254','6','103']
  H_HDV['8_CO_2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.7747787166329','-0.00538051612054','-1.97270106599516','0.103059449437025','0','6','103']
  H_HDV['8_THC_2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.09276748608127','0.049849940536758','5.45717058578625','0.208107912064726','0.580451339085132','6','103']
  H_HDV['8_NOx_2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.7371444402776','0.06040057951302','88.5541966462202','0.345481150291323','14.4511080049821','6','103']
  H_HDV['8_PM_2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','50','PM','y=(c+(a*exp(b*x)))','2.34331639968773','-0.080399853121271','0.452853861726511','0','0','6','103']
  H_HDV['8_FC_2_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','904.233489375875','0.069872434668788','8437.91594707444','0.605256865720461','406.87663774516','6','103']
  H_HDV['8_CO_2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.2128379269834','-0.007487275989256','-2.1492068518758','0.10728840259221','0','6','101']
  H_HDV['8_THC_2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.8563034070233','0.051344930450332','5.17447021410106','0.18704321554378','0.661755673263146','6','101']
  H_HDV['8_NOx_2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.0804653057608','0.053185315085779','95.9155192378697','0.358785503911888','16.255850780322','6','101']
  H_HDV['8_PM_2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','100','PM','y=(c+(a*exp(b*x)))','2.37520867956559','-0.084437496995135','0.541706852130552','0','0','6','101']
  H_HDV['8_FC_2_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','903.500099709','0.068058193074497','12306.0122336061','0.675693107752459','473.913837643299','6','101']
  H_HDV['8_CO_4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.50055740448611','11.722188800365','8.07465030529093','3.24355210762047','-0.016260969746492','6','101']
  H_HDV['8_THC_4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.29719745261073','0.052796608735835','5.24481595352973','0.207038543703115','0.64629349100737','6','101']
  H_HDV['8_NOx_4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','0','NOx','y=((a*(x**b))+(c*(x**d)))','3.95589592373221','0.264469121297423','156.063704301602','-0.774966114618009','0','6','101']
  H_HDV['8_PM_4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.215591448390148','0.050793470051524','-0.000366023895823','0','0','6','101']
  H_HDV['8_FC_4_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','3684.46751002674','1.00908307197463','-0.622809642870453','0','0','6','101']
  H_HDV['8_CO_4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.0718816375838','-0.011329386805662','-2.93726621844847','0.126743641859084','0','6','89']
  H_HDV['8_THC_4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.8744891074525','0.040553011268928','5.65972457826867','0.144841173896463','0.740006580977808','6','89']
  H_HDV['8_NOx_4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.3340839831937','0.054961781135947','72.8669588957284','0.289416228304193','21.0607386041457','6','89']
  H_HDV['8_PM_4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','50','PM','y=(c+(a*exp(b*x)))','2.46092951597983','-0.097480364898803','0.670751423342732','0','0','6','89']
  H_HDV['8_FC_4_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','851.364273943655','0.073971543532658','5028.1464179659','0.49997369303231','608.332756367422','6','89']
  H_HDV['8_CO_4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','29.4948549611249','-0.035155453480007','-3.33004955278431','0.138039231976462','0','6','76']
  H_HDV['8_THC_4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.80658874197543','0.07658409305681','3.65833900628193','0.189959363405512','1.03488505502393','6','76']
  H_HDV['8_NOx_4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.7251595046051','2.96961684573611','-0.132107579797875','0','0','6','76']
  H_HDV['8_PM_4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.15353630049934','0.002139541386676','-0.22029348849809','0.091348186360976','0','6','76']
  H_HDV['8_FC_4_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','851.359738704459','0.074947904891024','7019.34025079704','0.560411729052473','731.415638641888','6','76']
  H_HDV['8_CO_6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.08414591393605','12.563864331927','7.96543335202073','3.31714801719679','-0.028160162178146','6','84']
  H_HDV['8_THC_6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.55502053467995','-0.005265405146211','-0.656795977802925','0.10344649760739','0','6','84']
  H_HDV['8_NOx_6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.8018045868109','0.049706392710554','65.6819048252146','0.2430589091373','22.2788237551368','6','84']
  H_HDV['8_PM_6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.07934289877151','0.002199842500893','-0.222459654216204','0.089374334646617','0','6','84']
  H_HDV['8_FC_6_0'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','953.227914016274','0.084825059529189','5439.12919275535','0.540683255459382','647.608851008685','6','84']
  H_HDV['8_CO_6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','32.5495114156679','-0.042317461863856','-4.15137930611107','0.15676888758724','0','6','69']
  H_HDV['8_THC_6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.7887760446846','0.059433497004507','4.84434939268476','0.171080038297539','1.00602229545195','6','69']
  H_HDV['8_NOx_6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','25.4142027263234','176.406394017715','0.411163299386145','1.08502148246245','-0.000701020349409','6','69']
  H_HDV['8_PM_6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.21975683797421','0.002586507374177','-0.22839169656964','0.094434675018428','0','6','69']
  H_HDV['8_FC_6_50'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','811.625123734859','0.075953902459257','5262.50384673404','0.497034492459038','811.256333848668','6','69']
  H_HDV['8_CO_6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.5353562036429','-0.056339652198632','-3.22112433292192','0.130482575541671','0','6','56']
  H_HDV['8_THC_6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','100','THC','y=(c+(a*exp(b*x)))','6.63497795187747','-0.119758384141157','1.46747548475492','0','0','6','56']
  H_HDV['8_NOx_6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.9605931674219','2.43785079632335','-0.12264726998834','0','0','6','56']
  H_HDV['8_PM_6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.65595787531306','0.003363686162047','-0.303383008711736','0.118343151087316','0','6','56']
  H_HDV['8_FC_6_100'] = [' 8','EURO1','Coach 3-Axes >18t Euro-1','6','100','FC','y=exp((a+(b/x))+(c*log(x)))','6.93796640697451','3.40282445789035','-0.021676896830283','0','0','6','56']
  H_HDV['9_CO_0_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.51359471094387','0.050572913462558','15.5791356364176','0.202278187215247','1.05502141534297','6','103']
  H_HDV['9_THC_0_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.252464860104161','23.4763595671895','-0.708752976375202','0.704345590759679','0.022661135032443','6','103']
  H_HDV['9_NOx_0_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.1547990047078','0.055444289668404','77.0632982550243','0.312153515097516','7.34242268740026','6','103']
  H_HDV['9_PM_0_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.481968484865231','0.053151553700671','0.690586407479056','0.216839006126986','0.126006723352908','6','103']
  H_HDV['9_FC_0_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','700.727762280702','0.055043102175406','4542.39984380375','0.482859735187412','191.095818065859','6','103']
  H_HDV['9_CO_0_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.02906692677036','0.04988135754195','17.9016419032205','0.228574478426881','1.16971386225957','6','103']
  H_HDV['9_THC_0_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.256115732255709','29.8903643800092','-1.02195526149913','0.679830905164359','0.024364324104108','6','103']
  H_HDV['9_NOx_0_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.189280924331','0.051239739185229','84.5876819278028','0.33483147505211','7.74179517054987','6','103']
  H_HDV['9_PM_0_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.474657090797158','0.046836298987593','0.942101171487828','0.254012589536848','0.134621952072181','6','103']
  H_HDV['9_FC_0_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','692.916786682652','0.050111127759995','5741.37170169546','0.522801003361283','204.667201875905','6','103']
  H_HDV['9_CO_0_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.42762511771298','0.047195522528727','20.9163817701205','0.25370787241421','1.24653276888813','6','103']
  H_HDV['9_THC_0_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.254575689771733','23.0144981889875','-0.696327836602663','0.704978672360536','0.024059354257814','6','103']
  H_HDV['9_NOx_0_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.7906487019133','0.048019762189461','104.391040843596','0.379469226800449','8.07342622508818','6','103']
  H_HDV['9_PM_0_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.480608897928279','0.040690917913691','1.30410495574077','0.295597329843338','0.139391960397405','6','103']
  H_HDV['9_FC_0_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','696.72027835363','0.045729177397518','8015.96890734211','0.581264207145241','215.829604294275','6','103']
  H_HDV['9_CO_-2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','0','CO','y=exp((a+(b/x))+(c*log(x)))','4.73517459234837','-3.08027556429844','-1.20533129443273','0','0','6','103']
  H_HDV['9_THC_-2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','0','THC','y=((a*(b**x))*(x**c))','9.63992159986071','0.991447458499373','-0.709931051286597','0','0','6','103']
  H_HDV['9_NOx_-2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','0','NOx','y=((a*(x**b))+(c*(x**d)))','-0.07805892038473','0.667104132777768','144.480051302174','-0.836095608910495','0','6','103']
  H_HDV['9_PM_-2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.871136117716306','-0.00085821031388','-0.065413480045744','0.08684576826722','0','6','103']
  H_HDV['9_FC_-2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','583.35559638027','0.042637983542327','4657.4686849815','0.449567992879797','44.4545002556628','6','103']
  H_HDV['9_CO_-2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.03578549963447','0.014939268484735','0.000108354538857','0','0','6','103']
  H_HDV['9_THC_-2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','50','THC','y=((a*(b**x))*(x**c))','9.23128046818745','0.989410622783369','-0.69268451054972','0','0','6','103']
  H_HDV['9_NOx_-2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.6069377315358','0.040116038044171','129.350141932683','0.384918094594923','1.07424191500677','6','103']
  H_HDV['9_PM_-2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','50','PM','y=(1/(a+(b*(x**c))))','1.01925055482303','0.100583784855579','1.1487313146145','0','0','6','103']
  H_HDV['9_FC_-2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','594.599660280993','0.040154160461861','6076.31881163934','0.499635241833403','31.9493287414552','6','103']
  H_HDV['9_CO_-2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.55535531176343','0.051421743274916','36.5712994183492','0.421764384238365','0.439702447471526','6','103']
  H_HDV['9_THC_-2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.83882244471923','0.042369224883743','7.15364292907636','0.336013251040744','0.111737336283582','6','103']
  H_HDV['9_NOx_-2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.2226510466547','0.039166312123642','179.311380440653','0.450104880964302','0.781442101793434','6','103']
  H_HDV['9_PM_-2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','100','PM','y=((a*(b**x))*(x**c))','1.7949060209771','0.990499570721061','-0.594844087116293','0','0','6','103']
  H_HDV['9_FC_-2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','609.671293821963','0.038390775541463','8874.24871525131','0.568758544124812','23.3907045653872','6','103']
  H_HDV['9_CO_-4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.5701546897066','0.062029607755591','30.4609588544461','0.449986568339427','0.183023002226776','6','103']
  H_HDV['9_THC_-4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.99323037697097','0.050322660161756','10.3285392416663','0.423742784828224','0.090517331348182','6','103']
  H_HDV['9_NOx_-4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.6799204751329','0.058076527250574','197.003737638589','0.483076812089317','0.474129153415973','6','103']
  H_HDV['9_PM_-4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','0','PM','y=((a+(b*x))**((-1)/c))','0.864495965605421','0.127860876376158','0.740490821944485','0','0','6','103']
  H_HDV['9_FC_-4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','581.97167132712','0.055443943514155','8934.19496524371','0.581713662104529','11.6293341288194','6','103']
  H_HDV['9_CO_-4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.00387178217619','0.0590390091727','49.175411866827','0.512068755625435','0.158267661573454','6','103']
  H_HDV['9_THC_-4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.93416034572262','0.051345302044505','12.4153027467034','0.456964509007929','0.08695064385637','6','103']
  H_HDV['9_NOx_-4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.8148093134205','0.056881486539578','290.528139483336','0.548976594402114','0.408101211704165','6','103']
  H_HDV['9_PM_-4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','2.06852001769357','0.991271661399264','-0.74595193995553','0','0','6','103']
  H_HDV['9_FC_-4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','539.448244771802','0.053301044979359','12322.5758731457','0.633672263424328','8.79003511205452','6','103']
  H_HDV['9_CO_-4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.76773259123039','0.056168186002013','73.9513463347283','0.574988662560865','0.140871881378578','6','103']
  H_HDV['9_THC_-4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.83988635224442','0.05104468215767','14.0031494802438','0.473318048049187','0.08433746505536','6','103']
  H_HDV['9_NOx_-4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.0457615829135','0.054521461651254','408.667098582509','0.603994632784221','0.344962014643846','6','103']
  H_HDV['9_PM_-4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.395437339002752','0.043960600340829','2.09427942783313','0.395356536337257','0.030305338254277','6','103']
  H_HDV['9_FC_-4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','503.703176429249','0.050741462852329','16658.8891759984','0.683224255250579','6.56895806022361','6','103']
  H_HDV['9_CO_-6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.84910976524832','0.068019084334298','52.4720703126126','0.544094163198659','0.12542887180158','6','103']
  H_HDV['9_THC_-6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.84569185479652','0.055140680413975','13.5112197235873','0.475420315713525','0.085964954003447','6','103']
  H_HDV['9_NOx_-6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.1935209495331','0.067655750489129','287.134783542367','0.548564030568947','0.29982425539273','6','103']
  H_HDV['9_PM_-6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.349465400003034','0.267418332848382','0.000965849486163','0','0','6','103']
  H_HDV['9_FC_-6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','478.095655468523','0.063126351397911','10698.293505006','0.609673494801143','5.62142307353458','6','103']
  H_HDV['9_CO_-6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.29318684837356','0.06518960200323','59.2565308000771','0.560555733495226','0.114989315581267','6','103']
  H_HDV['9_THC_-6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.69654370127082','0.053819794960186','13.6908355146329','0.470531615948831','0.083398282330021','6','103']
  H_HDV['9_NOx_-6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.1077027656771','0.064431155212577','356.048414699809','0.578796645002651','0.258045878499128','6','103']
  H_HDV['9_PM_-6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','50','PM','y=((a*(b**x))*(x**c))','2.30290783603949','0.993883224612342','-0.840649479375913','0','0','6','103']
  H_HDV['9_FC_-6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','412.928965842925','0.059895857241665','13057.6719335945','0.639198253396616','4.36294769411174','6','103']
  H_HDV['9_CO_-6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.7820319185072','0.061012942414578','70.2010975529828','0.57280571984703','0.099088822100323','6','103']
  H_HDV['9_THC_-6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.60150348144417','0.052971976459724','13.3947872445441','0.46234657392384','0.082150199770021','6','103']
  H_HDV['9_NOx_-6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.3613473309387','0.062236683029868','400.227881851539','0.596783674363423','0.23339201576406','6','103']
  H_HDV['9_PM_-6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.345608729275188','0.04651852652482','1.5638394514841','0.340190907126902','0.029417848264553','6','103']
  H_HDV['9_FC_-6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','377.974903117189','0.058070957069771','14885.695473603','0.66202254325655','3.83014298523156','6','103']
  H_HDV['9_CO_2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.64654436668287','11.377658769521','6.69973941701788','2.77576736966566','-0.015418012825074','6','103']
  H_HDV['9_THC_2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.89415549291657','0.04108466606343','4.98955491159496','0.233411626339901','0.281710180395483','6','103']
  H_HDV['9_NOx_2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','0','NOx','y=((a*(b**x))*(x**c))','148.256029615769','1.00856710294601','-0.709553766838547','0','0','6','103']
  H_HDV['9_PM_2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.20687941022372','-0.000728164269823','-0.093888959432016','0.099295802159245','0','6','103']
  H_HDV['9_FC_2_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','806.260781846747','0.069716847021949','6308.95650329182','0.55929171914704','344.986812115275','6','103']
  H_HDV['9_CO_2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.3601479096554','-0.014064206483393','-2.51557173497886','0.123129697074126','0','6','103']
  H_HDV['9_THC_2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.01562533276475','0.047375137610741','4.79817076625305','0.238649878047615','0.338039800459299','6','103']
  H_HDV['9_NOx_2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','50','NOx','y=((a*(x**b))+(c*(x**d)))','1.08846760806331','0.425433201418111','142.837630441443','-0.665194969426113','0','6','103']
  H_HDV['9_PM_2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.29457713869077','-0.001072878976988','-0.099651762381797','0.101374162053451','0','6','103']
  H_HDV['9_FC_2_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','824.08399633765','0.068485607340462','10254.5982839189','0.656238819741142','408.837371560452','6','103']
  H_HDV['9_CO_2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.2487821890794','-0.020244956694779','-2.70443481569998','0.124021884166869','0','6','102']
  H_HDV['9_THC_2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.14205976006913','0.053924716367427','4.48279846494341','0.241203774525947','0.398704643903024','6','102']
  H_HDV['9_NOx_2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','100','NOx','y=((a*(x**b))+(c*(x**d)))','3.45946262691537','0.200963871389599','137.256314924447','-0.652188171964146','0','6','102']
  H_HDV['9_PM_2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.37432459035974','-0.001208283444394','-0.098521288168442','0.09576962846961','0','6','102']
  H_HDV['9_FC_2_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','854.274336677856','0.068599282046831','24215.1013260593','0.821632341201701','475.053941111471','6','102']
  H_HDV['9_CO_4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.7730105340299','-0.018916623661638','-3.13809684018842','0.134328088049298','0','6','102']
  H_HDV['9_THC_4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.30603849618685','0.054131976633267','4.61397602806957','0.251131866563284','0.39690381963138','6','102']
  H_HDV['9_NOx_4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','15.036948660664','155.759147890276','0.681785589042226','1.08108390092715','-0.000447308376615','6','102']
  H_HDV['9_PM_4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.42305580288589','-0.001312302740324','-0.112351501926472','0.105074303200422','0','6','102']
  H_HDV['9_FC_4_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','3244.81724214283','1.00873773851407','-0.580212547828359','0','0','6','102']
  H_HDV['9_CO_4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.1577685449897','-0.027371681940149','-3.01953519148381','0.124550480066029','0','6','93']
  H_HDV['9_THC_4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.479532325066666','9.05400358396787','0.896871923727772','0.888753560791344','0.033224031999816','6','93']
  H_HDV['9_NOx_4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.3549582147514','0.035587182484575','60.5892401175152','0.200145748705575','19.460148581121','6','93']
  H_HDV['9_PM_4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.51569711980329','-0.001614718353189','-0.107137865887765','0.095865188726249','0','6','93']
  H_HDV['9_FC_4_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','884.142901188613','0.080899649148008','11022.1418338809','0.694165611866376','621.60823004821','6','93']
  H_HDV['9_CO_4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.53783081068609','0.015531023576922','25.9406212500861','0.136518080699513','-0.165809134478661','6','78']
  H_HDV['9_THC_4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.35557700715132','-0.003676421011098','-0.52316336831239','0.115352062789872','0','6','78']
  H_HDV['9_NOx_4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.0142333665033','0.035669512363851','62.5120482953344','0.213306521118282','22.0380094023622','6','78']
  H_HDV['9_PM_4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.72242453029546','-0.00228674133374','-0.125767288011271','0.10005173903774','0','6','78']
  H_HDV['9_FC_4_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','773.955810449629','0.073967564994428','8401.98531776305','0.613049087377568','733.654258911316','6','78']
  H_HDV['9_CO_6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.1408892962259','-0.035999989200972','-3.46201233151535','0.134310948792585','0','6','87']
  H_HDV['9_THC_6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.73733959627289','0.070088314969462','3.66513366695787','0.253116063461933','0.543214696543192','6','87']
  H_HDV['9_NOx_6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.8887538693033','0.031219339425984','61.7841871354741','0.186985494367019','20.6006357078661','6','87']
  H_HDV['9_PM_6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.6020299798644','-0.002034720892973','-0.120207225205567','0.1026246236441','0','6','87']
  H_HDV['9_FC_6_0'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1000.63520478928','0.093883961316888','20270.7555308494','0.836977101699521','672.831897988416','6','87']
  H_HDV['9_CO_6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','32.2811444635137','-0.041315499946379','-3.28037366106052','0.11853914358141','0','6','73']
  H_HDV['9_THC_6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.53418973951459','0.076010126268838','3.2056724284699','0.211436455350438','0.665028659589634','6','73']
  H_HDV['9_NOx_6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','96.5359015228795','-0.138395561488501','-8.72058023812554','0.141235451991284','0','6','73']
  H_HDV['9_PM_6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.434253450241952','0.066232901654615','-0.000502131091195','0','0','6','73']
  H_HDV['9_FC_6_50'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','896.164171691012','0.091285818905201','14877.779995226','0.745954190526867','839.946054699013','6','73']
  H_HDV['9_CO_6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','37.4495908314416','-0.049197392346918','-3.7719380416426','0.115992674858481','0','6','59']
  H_HDV['9_THC_6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54981979885941','0.055370733522267','4.2288643296145','0.187166496075754','0.707804958030097','6','59']
  H_HDV['9_NOx_6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','106.023583183925','-0.177514488928982','-9.946038377131','0.151882987245987','0','6','59']
  H_HDV['9_PM_6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.0478859667224','-0.002111293218287','-0.143961103039938','0.094820333348982','0','6','59']
  H_HDV['9_FC_6_100'] = [' 9','EURO2','Coach 3-Axes >18t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','550.949353234657','0.056173764976866','5295.96300046531','0.471778748679033','970.061161919733','6','59']
  H_HDV['10_CO_0_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.21143776776035','0.041775731942036','18.5226694653309','0.193219906651624','1.06303991209969','6','103']
  H_HDV['10_THC_0_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.113684159025884','5.04600778187447','3.14584469848099','1.56591365222434','-0.006602804204457','6','103']
  H_HDV['10_NOx_0_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','0','NOx','y=((a*(b**x))*(x**c))','259.522904363933','1.00859207199277','-1.02588447188565','0','0','6','103']
  H_HDV['10_PM_0_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.058338938476599','1.40463930124551','3.426599834143','1.57217786770705','-0.006117799740123','6','103']
  H_HDV['10_FC_0_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000308135577325','0.00010364699849','-0.000000549442683','0','0','6','103']
  H_HDV['10_CO_0_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.39465989259282','0.041483597168966','20.8366611607355','0.210746884277712','1.15076221758063','6','103']
  H_HDV['10_THC_0_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.131941035672716','4.75057103023634','3.35337152432431','1.62722288652003','-0.006441260606665','6','103']
  H_HDV['10_NOx_0_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','50','NOx','y=((a*(b**x))*(x**c))','237.225739354611','1.00761090681921','-0.971372368292345','0','0','6','103']
  H_HDV['10_PM_0_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.057318131418219','1.54564762193278','2.97764091021318','1.45092288178072','-0.004462491557442','6','103']
  H_HDV['10_FC_0_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','765.974948580329','0.049194359542523','1835.94997371618','0.277731500214313','199.048195130164','6','103']
  H_HDV['10_CO_0_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.62575807803182','0.040919949831307','22.7402244703899','0.224767391542003','1.22799751460009','6','103']
  H_HDV['10_THC_0_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.134533321992562','4.60841779570274','3.44527294284065','1.65727094617941','-0.006778119628341','6','103']
  H_HDV['10_NOx_0_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','100','NOx','y=((a*(x**b))+(c*(x**d)))','3.62979333940883','-0.026508859606739','231.84042078158','-0.976790619957547','0','6','103']
  H_HDV['10_PM_0_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.522973115991152','0.038183480357988','1.27469791057869','0.167395369544283','0.105075950230361','6','103']
  H_HDV['10_FC_0_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','778.015097897244','0.045888074300494','2054.29562635815','0.30443276146582','209.26985194069','6','103']
  H_HDV['10_CO_-2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.0415267289862','-0.014823184233494','-1.50838320902382','0.0938120164622','0','6','103']
  H_HDV['10_THC_-2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.73058221910246','-0.003966584246452','-0.404351763130144','0.0952340209855','0','6','103']
  H_HDV['10_NOx_-2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','-0.001175945059123','0.004339979258851','0.000028893473811','0','0','6','103']
  H_HDV['10_PM_-2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.058052362372978','1.0640545783383','5.88964988385037','2.24713168583462','-0.011989493290595','6','103']
  H_HDV['10_FC_-2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','0','FC','y=((a*(b**x))*(x**c))','3932.34022017419','0.989016395591791','-0.746634844309735','0','0','6','103']
  H_HDV['10_CO_-2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.6934750345309','-0.014290779517524','-1.44756786200678','0.091204770531182','0','6','103']
  H_HDV['10_THC_-2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','50','THC','y=exp((a+(b/x))+(c*log(x)))','4.75371227811435','-6.60869777735187','-1.47204281584172','0','0','6','103']
  H_HDV['10_NOx_-2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.4533377522424','0.049897577280584','173.809567836122','0.36936170120936','1.17008940563403','6','103']
  H_HDV['10_PM_-2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.6639490709755','-7.32407353830014','-1.39021731437323','0','0','6','103']
  H_HDV['10_FC_-2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-177.837879720673','-0.070525372416108','3760.61882026215','-0.687046784611898','0','6','103']
  H_HDV['10_CO_-2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.5422758264354','-0.01435953149725','-1.4107264070359','0.08936711459869','0','6','103']
  H_HDV['10_THC_-2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','4.97966365943896','-7.361350137666','-1.54890919515678','0','0','6','103']
  H_HDV['10_NOx_-2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.6128800469096','0.049099023100323','201.15684711044','0.400505700002718','1.02328407242775','6','103']
  H_HDV['10_PM_-2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.82237207363636','-7.88601697841817','-1.43894865897754','0','0','6','103']
  H_HDV['10_FC_-2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-2','100','FC','y=((a*(b**x))*(x**c))','3081.4370581864','0.98381302338099','-0.621592867721914','0','0','6','103']
  H_HDV['10_CO_-4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','8.04521368278904','-12.6320301864868','-2.03527063290457','0','0','6','103']
  H_HDV['10_THC_-4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','0','THC','y=exp((a+(b/x))+(c*log(x)))','5.700482890202','-9.27638605758365','-1.7935746844435','0','0','6','103']
  H_HDV['10_NOx_-4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','27.8640511251388','0.070353901038491','243.835202660775','0.45609702352421','0.610199106000688','6','103']
  H_HDV['10_PM_-4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','4.86406257107389','-10.7535112949368','-1.74038396103252','0','0','6','103']
  H_HDV['10_FC_-4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','776.939168609194','0.060845181334028','3743.23560965338','0.425564524002329','19.2173620082189','6','103']
  H_HDV['10_CO_-4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','50','CO','y=exp((a+(b/x))+(c*log(x)))','7.88842204522525','-12.0475357512794','-2.01500295828698','0','0','6','103']
  H_HDV['10_THC_-4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006582874580132','2.98120930495935','5.30389871641903','2.15925799985154','-0.009637962610805','6','103']
  H_HDV['10_NOx_-4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.0058941934085','0.068226222481109','290.78225795288','0.483680548368962','0.562208262318485','6','103']
  H_HDV['10_PM_-4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.6165857763934','-9.97237063016942','-1.69150683541143','0','0','6','103']
  H_HDV['10_FC_-4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','729.106448725576','0.05964555109801','5416.7420400524','0.488714194441788','17.7022676676328','6','103']
  H_HDV['10_CO_-4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','100','CO','y=exp((a+(b/x))+(c*log(x)))','7.55401629644932','-10.8404984253635','-1.93952546332746','0','0','6','103']
  H_HDV['10_THC_-4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.98660083741836','-0.002017151127012','-0.355072007525605','0.094730345276915','0','6','103']
  H_HDV['10_NOx_-4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.5794214617257','0.065502840084515','368.011523192378','0.519199019321458','0.51263706800056','6','103']
  H_HDV['10_PM_-4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.044703277555803','0.96363027948027','7.25093468587387','2.7676279240338','-0.018863910819368','6','103']
  H_HDV['10_FC_-4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','674.641525427602','0.057151852844747','7756.96227875624','0.540833805937609','16.0294761631095','6','103']
  H_HDV['10_CO_-6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.64338834536186','0.003350810562744','15.261420529837','0.08957284305939','-0.997643362582918','6','103']
  H_HDV['10_THC_-6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.02082949664492','-0.001433798591673','-0.368214073094254','0.096118380991875','0','6','103']
  H_HDV['10_NOx_-6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','0','NOx','y=((a*(x**b))+(c*(x**d)))','-0.124570054128344','0.221026632174765','448.076970297611','-1.501432037182','0','6','103']
  H_HDV['10_PM_-6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','0','PM','y=(c+(a*exp(b*x)))','1.4440251009206','-0.079876787501355','0.073833849091431','0','0','6','103']
  H_HDV['10_FC_-6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','690.947691765427','0.069500246642064','5632.1967631022','0.497535942261101','14.2006501603562','6','103']
  H_HDV['10_CO_-6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.091153997366956','-0.000600588267336','0.000694339088639','0','0','6','103']
  H_HDV['10_THC_-6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.072515433832772','5.72627961555186','1.38462817395019','0.927277864408532','0.039535886432009','6','103']
  H_HDV['10_NOx_-6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','5.50729194469071','2.38277330855035','-1.42127290666025','0','0','6','103']
  H_HDV['10_PM_-6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04463189949831','0.934096016241898','7.24047043061471','2.79922791352919','-0.019656898047508','6','103']
  H_HDV['10_FC_-6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','594.711056055954','0.065828245438652','8006.21733521877','0.545459655561752','12.7468204121793','6','103']
  H_HDV['10_CO_-6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.07524047298348','0.00363530046759','0.000603919209082','0','0','6','103']
  H_HDV['10_THC_-6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.07363800483055','0.069742285726591','4.93160079228982','0.339041961752947','0.084598662710691','6','103']
  H_HDV['10_NOx_-6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.9546740762741','0.071931164883718','524.441076352774','0.577461300995453','0.318466506884517','6','103']
  H_HDV['10_PM_-6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.036531615610026','0.932776183981718','6.09503956129413','2.39777614581603','-0.014038860547174','6','103']
  H_HDV['10_FC_-6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','525.61885409971','0.062479484492347','10824.0197785076','0.592113100231013','11.5235362375289','6','103']
  H_HDV['10_CO_2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.5925603761168','-0.013152001964174','-1.75200481164473','0.102855587493861','0','6','103']
  H_HDV['10_THC_2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.170293760141306','3.99434772357523','4.3552165678','1.86241994724981','-0.007973329816757','6','103']
  H_HDV['10_NOx_2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','0','NOx','y=((a*(x**b))+(c*(x**d)))','4.04828032049987','0.135642226950546','304.017238123233','-1.08950671304345','0','6','103']
  H_HDV['10_PM_2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.112899960702361','0.994784444278157','5.05562001860237','1.98588659454381','-0.00861723399652','6','103']
  H_HDV['10_FC_2_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1028.19361697154','0.072978291376429','2502.97978903438','0.39485390696585','351.120849059202','6','103']
  H_HDV['10_CO_2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.4396160692527','-0.015466576435969','-1.86741231786644','0.106446264436722','0','6','103']
  H_HDV['10_THC_2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.20053583979579','3.79204044937058','4.71745001599062','1.99529225874962','-0.009948669733288','6','103']
  H_HDV['10_NOx_2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','8.94386712523294','380.917082217816','0.039470079674259','1.21816664663854','-0.002170882140141','6','103']
  H_HDV['10_PM_2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.123831348096243','0.997925365388424','4.99941423534456','1.97415571309339','-0.008682498287794','6','103']
  H_HDV['10_FC_2_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','50','FC','y=((a*(b**x))*(x**c))','4114.0159706973','1.0080887050925','-0.673633830593687','0','0','6','103']
  H_HDV['10_CO_2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.3332418371943','-0.019055616909784','-1.98228872851601','0.109835468656325','0','6','103']
  H_HDV['10_THC_2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.226965149971017','3.59742017237401','5.17309171984344','2.17410888979801','-0.014458132278979','6','103']
  H_HDV['10_NOx_2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','100','NOx','y=((a*(x**b))+(c*(x**d)))','8.48230658561812','0.024683892149087','265.538129322641','-1.03059666230445','0','6','103']
  H_HDV['10_PM_2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.141571710319149','0.957597362752338','5.51892345301067','2.1695094278955','-0.013578300232871','6','103']
  H_HDV['10_FC_2_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1044.84213379104','0.069931089098443','5652.30701109465','0.563253917761855','467.776333295938','6','103']
  H_HDV['10_CO_4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.7565364123301','-0.020288348459399','-1.97728585260195','0.113038069129748','0','6','103']
  H_HDV['10_THC_4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.216721047633355','3.64548792212908','5.2034969684006','2.16515822299091','-0.014957656867033','6','103']
  H_HDV['10_NOx_4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','11.7018500517164','379.994821642059','0.10676976787157','1.24788035927408','-0.001846726212515','6','103']
  H_HDV['10_PM_4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.130992932505706','0.925996352314871','6.0903462118327','2.34583174133293','-0.020374559884243','6','103']
  H_HDV['10_FC_4_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','4050.89787168323','1.00894308469373','-0.632565320950641','0','0','6','103']
  H_HDV['10_CO_4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.1279901493105','-0.025696737291061','-1.91540478973645','0.109455508165367','0','6','97']
  H_HDV['10_THC_4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.324390479364655','3.37851943376928','5.70999137160617','2.36247959786928','-0.014935990286413','6','97']
  H_HDV['10_NOx_4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.7772144965793','0.044243433418052','121.56719381372','0.273504742889917','14.9613563992881','6','97']
  H_HDV['10_PM_4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.142846778859923','0.915640234836501','6.81171816330734','2.65805100057008','-0.032733249684325','6','97']
  H_HDV['10_FC_4_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','3770.16445000699','1.00799074383474','-0.55612869547258','0','0','6','97']
  H_HDV['10_CO_4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.28664852257175','0.019170673301413','17.8806398759216','0.145836863020859','0.335397503854482','6','84']
  H_HDV['10_THC_4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.379638769666377','3.34377355935594','5.60552435606405','2.34518649222584','-0.013531785835048','6','84']
  H_HDV['10_NOx_4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.9760561749978','0.050560683694896','130.864092778964','0.301082833017573','17.4868411687841','6','84']
  H_HDV['10_PM_4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.230372966076626','0.867711296986491','6.47245251998676','2.51294729698393','-0.014283817881792','6','84']
  H_HDV['10_FC_4_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','909.973445622396','0.069520774874819','3032.31382422805','0.415321819826115','713.563884999234','6','84']
  H_HDV['10_CO_6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.4229197462099','-0.030429185028444','-2.03550040734254','0.116022971920788','0','6','89']
  H_HDV['10_THC_6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.355991678092448','3.3460981179576','5.76978787622653','2.37917038946272','-0.014074615381546','6','89']
  H_HDV['10_NOx_6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.1526662054861','0.043806867692477','118.49836562725','0.264859550346874','16.4647126083468','6','89']
  H_HDV['10_PM_6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','0','PM','y=(c+(a*exp(b*x)))','1.22948797246189','-0.079344451731908','0.247151879290569','0','0','6','89']
  H_HDV['10_FC_6_0'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1103.94563623283','0.084794554156385','2843.88145292257','0.464350543439413','679.481480744266','6','89']
  H_HDV['10_CO_6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.44022173924202','-1.73068638988334','-0.869796007921097','0','0','6','76']
  H_HDV['10_THC_6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.427284362829411','3.27681661282182','5.8045065646745','2.43510916192401','-0.016310483430498','6','76']
  H_HDV['10_NOx_6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.5118947911631','0.051620060767624','127.752383332948','0.290398366450135','20.1335538574323','6','76']
  H_HDV['10_PM_6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.314031795601025','3.92384519767122','-1.60166986913084','-0.524811124892093','0.140770245930475','6','76']
  H_HDV['10_FC_6_50'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1055.91080559836','0.083046511352652','3810.12334715271','0.502839298819437','827.696080702594','6','76']
  H_HDV['10_CO_6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','100','CO','y=(1/(a+(b*x)))','0.030952356469637','0.006577941481673','0','0','0','6','64']
  H_HDV['10_THC_6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.532061144443894','3.1166741206478','6.0620317826755','2.56522351806924','-0.018420035393579','6','64']
  H_HDV['10_NOx_6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','34.4349420205372','0.068545826775991','147.01477611786','0.346606399787388','24.3574478432678','6','64']
  H_HDV['10_PM_6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.70155472413083','0.00033850292564','-0.133432180958822','0.099572513872597','0','6','64']
  H_HDV['10_FC_6_100'] = [' 10','EURO3','Coach 3-Axes >18t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','882.765054064229','0.075076774805053','3355.25678087701','0.42800211925689','975.367317987564','6','64']
  H_HDV['11_CO_0_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.511552330628063','0.044144620830656','1.55020177150702','0.186205756663076','0.083544911302378','6','103']
  H_HDV['11_THC_0_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006111137137069','0.294122979981751','3.00109251937845','1.54395545425629','-0.004675846560649','6','103']
  H_HDV['11_NOx_0_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','0','NOx','y=((a*(b**x))*(x**c))','110.666660879508','1.00694738248137','-0.916032280479575','0','0','6','103']
  H_HDV['11_PM_0_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017500704589832','2.18219331722515','-0.19495368217716','0.979353823063585','0.018147639628091','6','103']
  H_HDV['11_FC_0_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000310083894136','0.000108869080451','-0.000000572968929','0','0','6','103']
  H_HDV['11_CO_0_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.500037407062865','0.043136178532465','1.6081891627591','0.190566373641885','0.086654414174713','6','103']
  H_HDV['11_THC_0_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005729079753433','0.275707249280308','3.26437508004125','1.6285316785891','-0.006959848564003','6','103']
  H_HDV['11_NOx_0_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','50','NOx','y=((a*(b**x))*(x**c))','99.1392128806658','1.00561836163678','-0.848544206544709','0','0','6','103']
  H_HDV['11_PM_0_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017531580938155','1.911099738964','-0.016019536833958','1.00050395602661','0.01692629341135','6','103']
  H_HDV['11_FC_0_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','749.123944061427','0.050191050433848','1990.58991134439','0.296548411699595','188.529779237274','6','103']
  H_HDV['11_CO_0_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.506901027266552','0.042877259083662','1.65205453753694','0.196523919759001','0.089541077556427','6','103']
  H_HDV['11_THC_0_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006125765205084','0.26649126631523','3.38397934058497','1.66760641317846','-0.007079989591175','6','103']
  H_HDV['11_NOx_0_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','100','NOx','y=((a*(b**x))*(x**c))','89.2559383120755','1.00413799558654','-0.782715531757983','0','0','6','103']
  H_HDV['11_PM_0_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017570266649851','1.92156058798769','-0.031140595918742','0.997091798956756','0.01668758969483','6','103']
  H_HDV['11_FC_0_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','753.610386004387','0.046964866798251','2127.2669913756','0.313171660487408','197.198916268126','6','103']
  H_HDV['11_CO_-2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.4796087632735','-0.001219018678989','-0.129643511775733','0.096936576277948','0','6','103']
  H_HDV['11_THC_-2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.267901716714203','-0.000218979934644','-0.023521019040057','0.097370276498807','0','6','103']
  H_HDV['11_NOx_-2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.6569520368277','0.047236431451297','80.9683323196275','0.356657677742777','0.719064057338742','6','103']
  H_HDV['11_PM_-2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.54107954827893','-3.36460758493228','-1.32057025199737','0','0','6','103']
  H_HDV['11_FC_-2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','-210.577068449059','-0.151898387049933','4071.06083453503','-0.735422155698023','0','6','103']
  H_HDV['11_CO_-2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.42891183514772','-0.001147635646615','-0.122969824795115','0.094482164900169','0','6','103']
  H_HDV['11_THC_-2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','50','THC','y=exp((a+(b/x))+(c*log(x)))','1.8854352778388','-6.51818804825469','-1.4866214841361','0','0','6','103']
  H_HDV['11_NOx_-2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.7441189932246','0.0453525100673','95.2532911711125','0.389297557347246','0.564786258682097','6','103']
  H_HDV['11_PM_-2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.002186221972551','0.573012696194463','2.11274725892441','1.33441397410871','-0.00053612858143','6','103']
  H_HDV['11_FC_-2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-124.774514173715','-0.022985543773911','3706.23496221851','-0.71660444196078','0','6','103']
  H_HDV['11_CO_-2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.04384522254672','-7.92205161641499','-1.62161542793247','0','0','6','103']
  H_HDV['11_THC_-2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','2.1208909320109','-7.27204932061583','-1.56541699369343','0','0','6','103']
  H_HDV['11_NOx_-2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8817038557938','0.04381013317621','115.252592602842','0.426857402271681','0.456142062895602','6','103']
  H_HDV['11_PM_-2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.16762096181438','0.439101833445778','0.009373670033447','0','0','6','103']
  H_HDV['11_FC_-2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','725.321472896353','0.044636706172935','3659.71124058087','0.403562571433429','27.7731443130437','6','103']
  H_HDV['11_CO_-4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.13001254931944','-0.022515287739168','0.00764549584031','0','0','6','103']
  H_HDV['11_THC_-4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','0','THC','y=exp((a+(b/x))+(c*log(x)))','3.01974518861762','-9.76562765866107','-1.85724021671997','0','0','6','103']
  H_HDV['11_NOx_-4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.6548574709871','0.063144061695289','149.328250078279','0.489725014495602','0.189300455015714','6','103']
  H_HDV['11_PM_-4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.93681384172646','0.267752258414575','0.020197554643442','0','0','6','103']
  H_HDV['11_FC_-4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','771.440291886351','0.063024069414532','4519.0760420545','0.465784505925315','12.6994359755401','6','103']
  H_HDV['11_CO_-4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.19240797981518','-0.027691999285312','0.008567336955878','0','0','6','103']
  H_HDV['11_THC_-4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000459100469137','0.172574269847681','5.20803902777558','2.13397429727646','-0.009208530459948','6','103']
  H_HDV['11_NOx_-4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.701051359417','0.061456013594695','184.684862111614','0.525587558459278','0.158865620228898','6','103']
  H_HDV['11_PM_-4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.89589075065574','0.288810980221332','0.021591036845613','0','0','6','103']
  H_HDV['11_FC_-4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','726.867530843986','0.062094536504531','6112.68834460747','0.5181777672072','11.1477591720084','6','103']
  H_HDV['11_CO_-4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.24155743118656','-0.000539225551101','-0.110834440596442','0.09335201926347','0','6','103']
  H_HDV['11_THC_-4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.2299315933288','-0.000116923411322','-0.02072042342216','0.095474974962943','0','6','103']
  H_HDV['11_NOx_-4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7216075409272','0.058944952295022','228.614774393773','0.557985571688437','0.131007713562771','6','103']
  H_HDV['11_PM_-4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003705331429095','2.0902333315449','-0.475077452575527','0.813256005940971','0.034516246988221','6','103']
  H_HDV['11_FC_-4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','672.222469931461','0.059834548639915','8447.93552807082','0.565672938470103','9.59128023080526','6','103']
  H_HDV['11_CO_-6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.22171694485234','-0.000298060758981','-0.111600610634946','0.093945751224215','0','6','103']
  H_HDV['11_THC_-6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','4.96608579029766','0.175005577032591','0.041245634422763','0','0','6','103']
  H_HDV['11_NOx_-6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.814378190618','0.071494831391457','203.743532626885','0.544390860512087','0.068279245900133','6','103']
  H_HDV['11_PM_-6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003893636568737','2.44088843361867','-0.712589833570943','0.754994608142273','0.043357987718981','6','103']
  H_HDV['11_FC_-6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','689.341194982809','0.072917123025309','7061.60419132632','0.545323613153333','6.44959934475975','6','103']
  H_HDV['11_CO_-6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010754474880928','3.58857676644149','-0.187684070881477','0.533778062140337','0.058604808738798','6','103']
  H_HDV['11_THC_-6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003352406455528','0.436693869866167','0.758481970453353','0.794588117048292','0.043303459290596','6','103']
  H_HDV['11_NOx_-6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2528781502784','0.068112475304861','245.32515020527','0.570684940466098','0.046492262126689','6','103']
  H_HDV['11_PM_-6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.192721017418313','0.071544473268125','0.690048944317766','0.349625188778396','0.004963441698506','6','103']
  H_HDV['11_FC_-6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','588.884419998391','0.069185508875723','9461.65465859733','0.580546173044433','5.07416275110073','6','103']
  H_HDV['11_CO_-6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.747820277314287','0.076683566005954','2.38493638345427','0.464156967831282','0.012329950436168','6','103']
  H_HDV['11_THC_-6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.120279262724143','0.069238369538585','0.286533095994353','0.337914745897231','0.003935927850693','6','103']
  H_HDV['11_NOx_-6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.18850763947252','0.065413398699153','298.793360438234','0.603492995586551','0.031941922865763','6','103']
  H_HDV['11_PM_-6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.180193627898516','0.070616874114191','0.964676717078517','0.404417497264037','0.004845513908325','6','103']
  H_HDV['11_FC_-6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','520.475113122016','0.066194753734996','12658.4708620897','0.625776500449475','4.16116395108108','6','103']
  H_HDV['11_CO_2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.79666754927369','-0.000861751437738','-0.174516454301285','0.1098889447895','0','6','103']
  H_HDV['11_THC_2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050949159640759','0.023383850253949','0.250643309491829','0.119822286948278','0.006622106536451','6','103']
  H_HDV['11_NOx_2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.6323216495858','0.069966076977708','68.4907190952582','0.345668165889503','6.56326023794319','6','103']
  H_HDV['11_PM_2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014983520861479','0.919824947256407','1.23815229560423','1.22497620909997','0.000355931501574','6','103']
  H_HDV['11_FC_2_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.633179854988121','1.20123410685604','3949.65654099169','-0.665398952764191','0','6','103']
  H_HDV['11_CO_2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.88878059681319','-0.001199556186148','-0.19440541536702','0.118243283123362','0','6','103']
  H_HDV['11_THC_2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007577892900804','0.21947752913281','4.46181506448815','1.92821222081737','-0.008711773268249','6','103']
  H_HDV['11_NOx_2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','50','NOx','y=((a*(x**b))+(c*(x**d)))','5.84835201991543','0.011948657909135','141.448326714713','-1.08659383961225','0','6','103']
  H_HDV['11_PM_2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014096068507869','0.989737892214142','1.111227792749','1.20375646115516','-0.00090767728089','6','103']
  H_HDV['11_FC_2_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','50','FC','y=((a*(b**x))*(x**c))','4057.96036991574','1.00823708305283','-0.686481909321057','0','0','6','103']
  H_HDV['11_CO_2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.00041920711258','-0.001697018583122','-0.219824128327985','0.128151520511702','0','6','103']
  H_HDV['11_THC_2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008930663341698','0.210403106757381','4.77179704090424','2.04808288254273','-0.010592005621854','6','103']
  H_HDV['11_NOx_2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','2.92682995764623','4.49179804641239','-0.205784089322152','0','0','6','103']
  H_HDV['11_PM_2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014023126161724','1.11085598693516','0.917022926898755','1.17204122844426','-0.000904103593477','6','103']
  H_HDV['11_FC_2_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','979.747473962788','0.068900946679937','4807.87175657488','0.519300535393637','436.326103686269','6','103']
  H_HDV['11_CO_4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.15609916330147','-0.001761356080796','-0.25977650692917','0.141262728050246','0','6','103']
  H_HDV['11_THC_4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006885432506274','0.211560856602699','4.82735797756073','2.04678961246483','-0.012556839914856','6','103']
  H_HDV['11_NOx_4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','8.40609665669806','164.708690579034','0.233973656138003','1.25346455809922','-0.001624614681323','6','103']
  H_HDV['11_PM_4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01498569314059','1.26552813779743','0.682523072293333','1.12503667286321','-0.000685943475477','6','103']
  H_HDV['11_FC_4_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','3983.83797386585','1.0091214507274','-0.641221976927052','0','0','6','103']
  H_HDV['11_CO_4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.24287946512417','-0.002823180875093','-0.275569169869606','0.147650846988895','0','6','98']
  H_HDV['11_THC_4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01114707888263','0.197652455598465','5.2839653708779','2.22621741967113','-0.013187389537447','6','98']
  H_HDV['11_NOx_4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.03031866395573','4.03381303060457','-0.170502675440426','0','0','6','98']
  H_HDV['11_PM_4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01265900682227','1.47185456756872','0.432799703894773','1.06922935039505','-0.000457004408129','6','98']
  H_HDV['11_FC_4_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','50','FC','y=((a*(b**x))*(x**c))','3741.78296219445','1.00837376620334','-0.573058062224886','0','0','6','98']
  H_HDV['11_CO_4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.687036094775607','0.011211880865133','2.01268281299876','0.179298394592862','-0.139404436367889','6','86']
  H_HDV['11_THC_4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013702401551418','0.193667555938685','5.34037261876372','2.26618068238406','-0.01249708670373','6','86']
  H_HDV['11_NOx_4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1969227927181','0.036775166510886','74.1022536515225','0.320716616548149','10.881729516591','6','86']
  H_HDV['11_PM_4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.121741006247158','0.027000832218209','0.515206063374476','0.186192310227627','0.016724019682324','6','86']
  H_HDV['11_FC_4_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','927.829294996526','0.073713156046724','3375.18940646409','0.445343862519805','673.652240325778','6','86']
  H_HDV['11_CO_6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.51812369779659','-0.003980693541171','-0.358207796422511','0.173269370287573','0','6','89']
  H_HDV['11_THC_6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012655552766727','0.191880312937865','5.49937824419876','2.2971389243431','-0.013162245367684','6','89']
  H_HDV['11_NOx_6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.52018145025661','0.026241330620666','62.7111257746509','0.266847866467504','10.1502019778591','6','89']
  H_HDV['11_PM_6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.11530608746298','0.022280231598208','0.502761664303675','0.176485927190752','0.012372435517186','6','89']
  H_HDV['11_FC_6_0'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1111.53015753338','0.087679112696649','3992.00906243015','0.530313174545834','656.34071809487','6','89']
  H_HDV['11_CO_6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.62664953490231','-0.005412912543925','-0.374750638595592','0.177346585098486','0','6','77']
  H_HDV['11_THC_6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015248358233033','0.189207125510696','5.52347831418911','2.34895189233267','-0.015018999531743','6','77']
  H_HDV['11_NOx_6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.4493559873121','0.028916566788129','69.2692466557778','0.292079116776185','11.891697893911','6','77']
  H_HDV['11_PM_6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','50','PM','y=((a*(b**x))*(x**c))','1.38670683179615','1.00062751146088','-0.856475819181951','0','0','6','77']
  H_HDV['11_FC_6_50'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1061.20856869593','0.086645753699525','4220.42436061701','0.526796326136079','789.433919112447','6','77']
  H_HDV['11_CO_6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.798760131563909','0.024160871640196','2.20710141476733','0.218356721978976','0.005127946668327','6','64']
  H_HDV['11_THC_6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016979325290354','0.19300705292667','5.20562693682799','2.26416547567463','-0.014403725189833','6','64']
  H_HDV['11_NOx_6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.1921523875255','0.047253039928113','89.1312331947899','0.367367288059914','14.952978476338','6','64']
  H_HDV['11_PM_6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.158983563399542','0.034897882860349','0.551410383246122','0.210383394268146','0.020836564054206','6','64']
  H_HDV['11_FC_6_100'] = [' 11','EURO4','Coach 3-Axes >18t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','703.999639606203','0.063926451386687','2648.27698148106','0.343392773708845','902.556041555476','6','64']
  H_HDV['12_CO_0_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.526465210982567','0.044056520566062','1.5957933438825','0.186077710213804','0.085470279204181','6','103']
  H_HDV['12_THC_0_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006371624801739','0.302389382976996','3.00431438720791','1.54463606600585','-0.004581180760454','6','103']
  H_HDV['12_NOx_0_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','0','NOx','y=((a*(b**x))*(x**c))','81.7670671261194','1.00793366827605','-0.979439300824105','0','0','6','103']
  H_HDV['12_PM_0_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01787668868193','2.24959204371434','-0.196895949541633','0.979040111938938','0.018081279344796','6','103']
  H_HDV['12_FC_0_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000297781535794','0.000103858404087','-0.000000537101756','0','0','6','103']
  H_HDV['12_CO_0_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.51536109537034','0.043086955803389','1.65112666937994','0.190268554311317','0.088165602170678','6','103']
  H_HDV['12_THC_0_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005532184169492','0.285520124117534','3.2492491343388','1.62575747537601','-0.007459259683595','6','103']
  H_HDV['12_NOx_0_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','50','NOx','y=((a*(b**x))*(x**c))','74.1764488365101','1.00682436750216','-0.920264352979395','0','0','6','103']
  H_HDV['12_PM_0_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017846920380882','1.9738209969486','-0.01918782904326','1.00088767339413','0.01678870496164','6','103']
  H_HDV['12_FC_0_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','778.333207993353','0.049854419354383','2050.90596548087','0.291827260049215','193.554686861105','6','103']
  H_HDV['12_CO_0_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.521664997737137','0.042954843868712','1.69383829099465','0.1958064259211','0.091123182968759','6','103']
  H_HDV['12_THC_0_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006144924702778','0.276060533631456','3.36448827711374','1.66150583395441','-0.00718438031668','6','103']
  H_HDV['12_NOx_0_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','100','NOx','y=(1/(a+(b*(x**c))))','-0.060292837719364','0.049302726099506','0.525209346740781','0','0','6','103']
  H_HDV['12_PM_0_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018020002487752','2.07618670940274','-0.09603783158756','0.990365330379571','0.01698107824902','6','103']
  H_HDV['12_FC_0_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','784.220882373421','0.047006964462747','2171.49625579129','0.306816848129002','201.775492681221','6','103']
  H_HDV['12_CO_-2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.51972765221595','-0.001253677090367','-0.132825347500491','0.096645371344445','0','6','103']
  H_HDV['12_THC_-2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.275246859780691','-0.000225422667095','-0.024116028991214','0.097131255773891','0','6','103']
  H_HDV['12_NOx_-2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.12702475872123','0.048439390630957','63.9786580508332','0.373415562285026','0.431659551570453','6','103']
  H_HDV['12_PM_-2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.59926671302956','-3.45665820590027','-1.32903299913627','0','0','6','103']
  H_HDV['12_FC_-2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','-222.400795684794','-0.148508597835316','4242.01623196746','-0.732508884605309','0','6','103']
  H_HDV['12_CO_-2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.83263488131343','-7.14260140144419','-1.54439507824411','0','0','6','103']
  H_HDV['12_THC_-2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','50','THC','y=exp((a+(b/x))+(c*log(x)))','1.92188967235156','-6.54183944289839','-1.48778038131945','0','0','6','103']
  H_HDV['12_NOx_-2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.11994412650566','0.046542794250537','72.6536730235735','0.398232621435128','0.345729071750595','6','103']
  H_HDV['12_PM_-2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.00245487758685','0.588758799626643','2.12089747302679','1.33610934631593','-0.00071601327857','6','103']
  H_HDV['12_FC_-2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-138.777978336652','-0.033400689644732','3895.89459539173','-0.716038656900388','0','6','103']
  H_HDV['12_CO_-2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.07113301151422','-7.90562867915267','-1.62015412039272','0','0','6','103']
  H_HDV['12_THC_-2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','2.14392611243103','-7.25009065443358','-1.56186473827446','0','0','6','103']
  H_HDV['12_NOx_-2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.17450234768669','0.045108794702254','84.3299552852149','0.42736969320001','0.285752410934694','6','103']
  H_HDV['12_PM_-2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.128241633658','0.426212334790339','0.009044589974428','0','0','6','103']
  H_HDV['12_FC_-2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','756.946597186881','0.044780156247957','3669.89988606182','0.394969490942626','28.9099357634652','6','103']
  H_HDV['12_CO_-4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.10714990918127','-0.023477177555491','0.007497276894064','0','0','6','103']
  H_HDV['12_THC_-4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','0','THC','y=exp((a+(b/x))+(c*log(x)))','3.06361161950132','-9.81458412315609','-1.86188209367328','0','0','6','103']
  H_HDV['12_NOx_-4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.73073702181161','0.063856317744671','109.307168126794','0.489221655951236','0.112324126133413','6','103']
  H_HDV['12_PM_-4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.898768938444','0.257143054814139','0.019768854154966','0','0','6','103']
  H_HDV['12_FC_-4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','807.2136304342','0.063099281517335','4719.898934778','0.466113380720385','13.0817002481522','6','103']
  H_HDV['12_CO_-4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.15476054556735','-0.027218232621258','0.008304191204949','0','0','6','103']
  H_HDV['12_THC_-4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000537210831213','0.178637040210142','5.20284681197479','2.13268204152741','-0.009273560133148','6','103']
  H_HDV['12_NOx_-4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.16636181218181','0.062311815639494','129.294091517043','0.517086665200063','0.095407905855184','6','103']
  H_HDV['12_PM_-4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.8418470567451','0.27816544400549','0.020975857176297','0','0','6','103']
  H_HDV['12_FC_-4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','763.094628337261','0.06221910788477','6204.10930977034','0.512381661910724','11.5941440159584','6','103']
  H_HDV['12_CO_-4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.11163519149514','-0.013611645264767','0.008468620939211','0','0','6','103']
  H_HDV['12_THC_-4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.237690979955617','-0.000120350843842','-0.021348765493652','0.095139195857934','0','6','103']
  H_HDV['12_NOx_-4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.61353637895595','0.060202719076766','152.459780980557','0.542874916417216','0.08081116461183','6','103']
  H_HDV['12_PM_-4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003934283129564','1.92371651920391','-0.346399776329641','0.810878150511308','0.035224716664519','6','103']
  H_HDV['12_FC_-4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','711.772928148157','0.060286971823284','8243.00412775845','0.554997115714553','10.175395583339','6','103']
  H_HDV['12_CO_-6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.25700451252835','-0.000306553396295','-0.114930486377583','0.094029955850505','0','6','103']
  H_HDV['12_THC_-6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','4.81548429966605','0.172055975798072','0.040113275328995','0','0','6','103']
  H_HDV['12_NOx_-6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.57067741950222','0.07160725555261','144.172759250751','0.537841914785329','0.038099594448159','6','103']
  H_HDV['12_PM_-6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004000066436576','2.549180622478','-0.728582276934607','0.755265293449335','0.043331534336809','6','103']
  H_HDV['12_FC_-6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','720.567982052209','0.072977762790732','7429.07475376732','0.546201454128944','6.70607765280807','6','103']
  H_HDV['12_CO_-6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011440268918064','4.52487317577352','-0.506953154959134','0.480619846678235','0.060868971365226','6','103']
  H_HDV['12_THC_-6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003433253346494','0.40116139943894','0.999276783353974','0.838077066883266','0.042283549404996','6','103']
  H_HDV['12_NOx_-6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.70041115380615','0.068762660540561','169.32523166981','0.561679667381731','0.026666371515973','6','103']
  H_HDV['12_PM_-6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.199694077316847','0.071666228419608','0.698478387734367','0.347141374804478','0.005115910949475','6','103']
  H_HDV['12_FC_-6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','622.398758296909','0.069501641057532','9723.16349808696','0.578007034054192','5.37612320472661','6','103']
  H_HDV['12_CO_-6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.788524694133362','0.07725578952807','1.96394542352563','0.433981893424737','0.012867334310165','6','103']
  H_HDV['12_THC_-6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.124663408092459','0.069227045422018','0.26154144240647','0.317223251364427','0.004053044244669','6','103']
  H_HDV['12_NOx_-6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.06725199848312','0.066261953898873','195.299100821435','0.585825734574633','0.017789551111146','6','103']
  H_HDV['12_PM_-6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.188005321514186','0.070904959245811','0.92558851147789','0.394105177931303','0.005014088659981','6','103']
  H_HDV['12_FC_-6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','556.846508894134','0.066867763536599','12588.6183559114','0.6186113390521','4.53842551100423','6','103']
  H_HDV['12_CO_2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.84966142637342','-0.000902108123993','-0.179691328978944','0.109926953502154','0','6','103']
  H_HDV['12_THC_2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.052364098212219','0.023342774908796','0.258085648121609','0.119783085835691','0.006810877618405','6','103']
  H_HDV['12_NOx_2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.26091024678861','0.066580495796571','52.896852321162','0.344984586453642','3.9541470667097','6','103']
  H_HDV['12_PM_2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015043460889593','0.944860501323078','1.24338156550572','1.22616312115187','0.000053967411128','6','103']
  H_HDV['12_FC_2_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','993.239938590005','0.070159264641103','2937.13250330428','0.399102665868309','347.750547125177','6','103']
  H_HDV['12_CO_2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.92289195675345','-0.001138669923591','-0.194979042694638','0.116193884191332','0','6','103']
  H_HDV['12_THC_2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007619509160565','0.22736871446831','4.41548749860109','1.91136373413142','-0.008473641658364','6','103']
  H_HDV['12_NOx_2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.65893931523711','113.216038835389','0.237123556526302','1.26315779137283','-0.003684703813587','6','103']
  H_HDV['12_PM_2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014628816025972','0.983145219110856','1.17536869132954','1.21407918717756','-0.000954244923318','6','103']
  H_HDV['12_FC_2_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','977.969337886155','0.068092681104944','3481.103639748','0.430904573903237','395.366792554636','6','103']
  H_HDV['12_CO_2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.02774007806043','-0.001599656877077','-0.218453806345737','0.125226905626798','0','6','103']
  H_HDV['12_THC_2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008772705337538','0.219737801757699','4.65766851127339','2.00542513058103','-0.009804320648733','6','103']
  H_HDV['12_NOx_2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','2.44796564963906','4.98150805692529','-0.220436488178805','0','0','6','103']
  H_HDV['12_PM_2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014018446856778','1.15813959485209','0.891328584974803','1.16722823243154','-0.000871456804541','6','103']
  H_HDV['12_FC_2_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','996.690300622123','0.067367609170523','4263.27269685154','0.479433058337085','443.266498057959','6','103']
  H_HDV['12_CO_4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.21896303042675','-0.001816037419002','-0.26718967272445','0.141180198374022','0','6','103']
  H_HDV['12_THC_4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00704556178417','0.217679837399286','4.83452035694145','2.0496707615391','-0.01270686228516','6','103']
  H_HDV['12_NOx_4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','39.5282966489199','-0.028357142480202','-5.84307447373802','0.18234194558423','0','6','103']
  H_HDV['12_PM_4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015580472034485','1.2702083999757','0.723594799003984','1.13061905465134','-0.000718298851788','6','103']
  H_HDV['12_FC_4_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','4163.09077873358','1.00904635882687','-0.640166194012829','0','0','6','103']
  H_HDV['12_CO_4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.29295605949914','-0.002836535069732','-0.280001186849783','0.146613073806854','0','6','99']
  H_HDV['12_THC_4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01128070476717','0.203774704735983','5.28234967417345','2.22464101250711','-0.013329704623044','6','99']
  H_HDV['12_NOx_4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.51977173559049','0.021020137836614','46.8395065293582','0.272823536847992','4.54373437987884','6','99']
  H_HDV['12_PM_4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013099552136555','1.53153122898978','0.414412933123925','1.06756665944896','-0.000388646221249','6','99']
  H_HDV['12_FC_4_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','50','FC','y=((a*(b**x))*(x**c))','3934.13496018472','1.00840727668427','-0.578559210901469','0','0','6','99']
  H_HDV['12_CO_4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.715220486202927','0.009780079351634','2.04462357426703','0.174314724330452','-0.177552069002227','6','88']
  H_HDV['12_THC_4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013839649810829','0.198685690648888','5.38598060029356','2.27658209207651','-0.012629177486788','6','88']
  H_HDV['12_NOx_4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.93761707911232','0.026216875037064','53.1301401362049','0.307174692286336','5.24446696445341','6','88']
  H_HDV['12_PM_4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.127947066046758','0.030059719830999','0.528978787849639','0.190668690363428','0.020997396357664','6','88']
  H_HDV['12_FC_4_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1004.37918681807','0.075261436851629','3778.76736712392','0.465936699191614','687.463532762431','6','88']
  H_HDV['12_CO_6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.59134955671639','-0.004096039266788','-0.368328880823833','0.173136743262257','0','6','89']
  H_HDV['12_THC_6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013025322783724','0.19753589110616','5.49751470499239','2.29650205006873','-0.013152553582221','6','89']
  H_HDV['12_NOx_6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.65592940195192','0.015480377645766','47.6518824117033','0.272406511726628','4.04415831319684','6','89']
  H_HDV['12_PM_6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.118883515830502','0.022604314957459','0.518072266006695','0.177057062337474','0.013291455719798','6','89']
  H_HDV['12_FC_6_0'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1147.88735781366','0.086765551760418','3914.38094814127','0.513387171229704','686.11019271988','6','89']
  H_HDV['12_CO_6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.64959208488833','-0.005376636450115','-0.369326874174883','0.173250150759355','0','6','78']
  H_HDV['12_THC_6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015516138338257','0.19436660436652','5.55770337015483','2.35989378478607','-0.015472896587327','6','78']
  H_HDV['12_NOx_6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.87044781273071','0.019722760023663','51.9274873679252','0.294066967017156','5.02877769869271','6','78']
  H_HDV['12_PM_6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.14736334481572','0.034659101591066','0.560120247981662','0.208716697052263','0.0257622310894','6','78']
  H_HDV['12_FC_6_50'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1126.78578260147','0.087210905871176','4633.31960554395','0.540283983329185','813.795637209564','6','78']
  H_HDV['12_CO_6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.825001337521582','0.023043287564084','2.33161937771959','0.223223830705568','-0.009050067722635','6','68']
  H_HDV['12_THC_6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.316743404339265','-0.000282717530968','-0.033322320718834','0.119684294175703','0','6','68']
  H_HDV['12_NOx_6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5387749146693','0.040460494613766','64.3117719126366','0.359877302352609','7.66531933329075','6','68']
  H_HDV['12_PM_6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.155814622978551','0.025181750161413','0.567684600406705','0.197257794502373','0.005270533953062','6','68']
  H_HDV['12_FC_6_100'] = [' 12','EURO5','Coach 3-Axes >18t Euro-5','6','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','830.772382149384','7020.6598575128','0.014638420943223','1.05233057133667','-0.000481705806982','6','68']
  H_HDV['13_CO_0_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.520096125136','0.062825463538905','32.8490121250327','0.370355226864105','2.48464959861408','6','75']
  H_HDV['13_THC_0_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.00398600820304','0.059694987760899','56.0776745689812','0.448028383878481','1.59621426556521','6','75']
  H_HDV['13_NOx_0_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.7965207694787','0.081154325971028','34493.3151448841','1.5594851715975','6.49962387913489','6','75']
  H_HDV['13_PM_0_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.60578158232982','0.061839311840989','6.37479068434696','0.308969519850173','0.416931999569699','6','75']
  H_HDV['13_FC_0_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','725.871180537035','0.081246948385014','155164.461381229','1.18138065356464','174.218769089294','6','75']
  H_HDV['13_CO_0_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.3555053868527','0.063188660058194','29.6107801517052','0.359007591718','2.53310265848634','6','75']
  H_HDV['13_THC_0_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.95337780283218','0.060291659422188','54.15319879171','0.438063102412409','1.52275925270464','6','75']
  H_HDV['13_NOx_0_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.0876283529072','0.074845599508524','688321.46646523','2.06707746441885','6.99500201445407','6','75']
  H_HDV['13_PM_0_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.71980434789255','0.061534045664108','6.24184470196358','0.305662906484128','0.428892310643271','6','75']
  H_HDV['13_FC_0_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','728.021489653163','0.078440222758271','911005.637718214','1.49102439960919','185.310579137752','6','75']
  H_HDV['13_CO_0_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.7524266711595','0.062355421287214','23.047781882813','0.31529413197654','2.64694587058257','6','75']
  H_HDV['13_THC_0_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.84892924547383','0.060628945349864','50.3899004643155','0.42316581170831','1.46348902364294','6','75']
  H_HDV['13_NOx_0_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','100','NOx','y=(c+(a*exp(b*x)))','22.0103361336699','-0.073059791874333','7.54790252342619','0','0','6','75']
  H_HDV['13_PM_0_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.71931358579103','0.0594515380973','5.52581616531497','0.278071786617428','0.443279830988283','6','75']
  H_HDV['13_FC_0_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','715.751413255814','0.074640790543167','3209941.04157588','1.70977050422997','196.786256995598','6','75']
  H_HDV['13_CO_-2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.02580940769384','66.0193111154196','-0.605272693500769','0.398862245998475','0.057576299112214','6','75']
  H_HDV['13_THC_-2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.07701551664468','0.06800791666529','64.643738556878','0.497147163363317','1.54049872376928','6','75']
  H_HDV['13_NOx_-2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','0','NOx','y=(c+(a*exp(b*x)))','22.4469852354084','-0.075195244385556','3.2934596618354','0','0','6','75']
  H_HDV['13_PM_-2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.15467233546409','0.07701839106881','67.9200711984532','0.829437021670249','0.35685229013118','6','75']
  H_HDV['13_FC_-2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','99.4207169026514','4854.77418824721','-1.18369712186311','0.330417766530045','0.063690827226809','6','75']
  H_HDV['13_CO_-2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','50','CO','y=(c+(a*exp(b*x)))','16.333980162449','-0.075256311953357','1.79118063670561','0','0','6','75']
  H_HDV['13_THC_-2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.13377688801177','0.067506732602779','64.4369774936086','0.500701486159641','1.33029196062802','6','75']
  H_HDV['13_NOx_-2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000085697445597','0.015246682671469','-0.91373701551537','21.7179869277081','0','6','75']
  H_HDV['13_PM_-2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.27590261898725','0.074995170762467','611.553832924284','1.22127733510145','0.312572004395482','6','75']
  H_HDV['13_FC_-2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','50','FC','y=(c+(a*exp(b*x)))','765.848578013563','-0.074878305286982','86.4151450114066','0','0','6','75']
  H_HDV['13_CO_-2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','100','CO','y=(c+(a*exp(b*x)))','16.2950917533275','-0.068801749786624','1.45837680807504','0','0','6','75']
  H_HDV['13_THC_-2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.0371637046462','0.065386604010154','64.0910496475454','0.499181979102269','1.12871749949911','6','75']
  H_HDV['13_NOx_-2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00006590091287','0.012549151279717','-0.816076415526031','20.9417118946851','0','6','75']
  H_HDV['13_PM_-2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.255510674950746','12.4461903766224','-0.411326967450968','0.385455949705604','0.055780660507123','6','75']
  H_HDV['13_FC_-2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-2','100','FC','y=(c+(a*exp(b*x)))','730.907137433735','-0.065861710350432','70.1061896667788','0','0','6','75']
  H_HDV['13_CO_-4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','0','CO','y=(c+(a*exp(b*x)))','14.9927147782431','-0.071098896742945','0.741799093515332','0','0','6','75']
  H_HDV['13_THC_-4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6043048147611','0.06931529070381','197.012498720893','0.74006382759564','0.764667092890534','6','75']
  H_HDV['13_NOx_-4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','0','NOx','y=(c+(a*exp(b*x)))','20.6914736099397','-0.067868806885569','0.792649138630367','0','0','6','75']
  H_HDV['13_PM_-4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.141761437102602','12.3122292270057','-0.433768626471625','0.42350552144206','0.054109213936755','6','75']
  H_HDV['13_FC_-4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','0','FC','y=(c+(a*exp(b*x)))','753.897417742115','-0.076222716486257','29.6120587327973','0','0','6','75']
  H_HDV['13_CO_-4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','50','CO','y=(c+(a*exp(b*x)))','14.6175624774374','-0.067540558768141','0.457265412233042','0','0','6','75']
  H_HDV['13_THC_-4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8142340997748','0.070697231446243','304.114355151876','0.824957552023823','0.600488580226208','6','75']
  H_HDV['13_NOx_-4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000081161877296','0.014228239450331','-0.844620080251431','18.0671176964886','0','6','75']
  H_HDV['13_PM_-4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','50','PM','y=(c+(a*exp(b*x)))','3.36477583492234','-0.076992638579963','0.131903728370028','0','0','6','75']
  H_HDV['13_FC_-4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','50','FC','y=(c+(a*exp(b*x)))','698.30641010949','-0.07031052505444','18.0517375764623','0','0','6','75']
  H_HDV['13_CO_-4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000064883809765','0.011171557842981','-0.647735713364268','13.4199333431706','0','6','75']
  H_HDV['13_THC_-4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.9233528009778','0.072747962225515','416.298852308647','0.883396571381773','0.55102841984446','6','75']
  H_HDV['13_NOx_-4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','100','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.093790776771542','-0.004397121288977','0.000377986141845','0','0','6','75']
  H_HDV['13_PM_-4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','100','PM','y=(c+(a*exp(b*x)))','3.29979825683846','-0.074790892313901','0.112561861070245','0','0','6','75']
  H_HDV['13_FC_-4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003059222995028','0.521933504291336','-29.9527944755638','610.449155416909','0','6','75']
  H_HDV['13_CO_-6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','0','CO','y=(c+(a*exp(b*x)))','14.1663951039205','-0.07332993084936','0.268158423770603','0','0','6','70']
  H_HDV['13_THC_-6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.4440439276959','0.07582864724766','899.080315287907','1.03091901108253','0.47364248575188','6','70']
  H_HDV['13_NOx_-6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','0','NOx','y=(c+(a*exp(b*x)))','18.5749136025517','-0.071059163627339','0.199280344856682','0','0','6','70']
  H_HDV['13_PM_-6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.94620627635575','0.075144277913063','9040.06347565858','1.69457469381283','0.084722404954435','6','70']
  H_HDV['13_FC_-6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','0','FC','y=(c+(a*exp(b*x)))','686.494332690074','-0.078318191125485','8.39109657515035','0','0','6','70']
  H_HDV['13_CO_-6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','50','CO','y=(c+(a*exp(((-1)*b)*x)))','13.7305688920306','0.072670753744668','0.19671163300328','0','0','6','70']
  H_HDV['13_THC_-6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.5140369101692','0.078952625061116','1704.24088253889','1.14756173065095','0.450829015630783','6','70']
  H_HDV['13_NOx_-6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','68.312863082792','3.27637451630792','-0.071381930943613','0','0','6','70']
  H_HDV['13_PM_-6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','50','PM','y=(c+(a*exp(b*x)))','3.19838898072578','-0.081446655839056','0.088375487532215','0','0','6','70']
  H_HDV['13_FC_-6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','50','FC','y=(c+(a*exp(b*x)))','628.941478627399','-0.074571090938673','4.29249142944361','0','0','6','70']
  H_HDV['13_CO_-6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','100','CO','y=(c+(a*exp(b*x)))','13.3460773247012','-0.070758831674963','0.146123601690739','0','0','6','70']
  H_HDV['13_THC_-6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.506732265606','0.080847217640818','2347.32102735325','1.20638712041822','0.443194915886222','6','70']
  H_HDV['13_NOx_-6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','29.7496152235544','1.10482860416799','-0.078436530524849','0','0','6','70']
  H_HDV['13_PM_-6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','100','PM','y=(c+(a*exp(b*x)))','3.11228204030896','-0.079750334202187','0.080639761829621','0','0','6','70']
  H_HDV['13_FC_-6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','3215.82754969379','4.70692708879476','-0.074507876887033','0','0','6','70']
  H_HDV['13_CO_2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7573422972604','0.051950236452205','24.6800213660695','0.221734295384262','2.57315247694525','6','75']
  H_HDV['13_THC_2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.34029604639377','0.044109836557383','34.737021637567','0.309271455483638','1.2203206897104','6','75']
  H_HDV['13_NOx_2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','0','NOx','y=((a*(b**x))*(x**c))','76.1501896219253','1.00979947100866','-0.643005745222485','0','0','6','75']
  H_HDV['13_PM_2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.90204461654577','0.044572503937296','6.06234399504325','0.20371250328474','0.429554167120205','6','75']
  H_HDV['13_FC_2_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','751.954152570981','0.082695457088364','83251.0297659023','1.07095274474934','240.44080326945','6','75']
  H_HDV['13_CO_2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.8646769268783','0.063440152876647','21.1845906693019','0.217613322045203','3.20948331028354','6','75']
  H_HDV['13_THC_2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.88664069310587','0.042391594589345','32.7541617492594','0.283546922707958','1.11284552835499','6','75']
  H_HDV['13_NOx_2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','50','NOx','y=((a*(b**x))*(x**c))','69.2428745582516','1.00781603587056','-0.554810716975268','0','0','6','75']
  H_HDV['13_PM_2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.07885026536878','0.04939739562894','6.05030335222421','0.196473660719143','0.533938727938624','6','75']
  H_HDV['13_FC_2_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.026998256287234','1.89529936970042','2336.18128153181','-0.57762993577532','0','6','75']
  H_HDV['13_CO_2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4360637379002','0.08000344311434','11.7036227302933','0.220380273720051','4.03504924362331','6','75']
  H_HDV['13_THC_2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.46364000196598','0.040008703998995','32.1951517180875','0.268076069980406','1.00516412286726','6','75']
  H_HDV['13_NOx_2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','100','NOx','y=(c+(a*exp(b*x)))','24.1618313942901','-0.082255202701247','12.8433237095172','0','0','6','75']
  H_HDV['13_PM_2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.47278293435544','0.058681463394988','5.47507000703522','0.18736532361188','0.67097868848933','6','75']
  H_HDV['13_FC_2_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.0006913099422','0.000085394536641','-0.000000751654195','0','0','6','75']
  H_HDV['13_CO_4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.36549251755824','26.3744145766128','4.09453550116221','1.93707763991324','-0.01656998659093','6','70']
  H_HDV['13_THC_4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.44004851021161','0.041727653796988','30.6587644247024','0.27280591783629','0.99608579083036','6','70']
  H_HDV['13_NOx_4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.3510302347001','0.088393263362215','1003.30458542446','0.945083150978448','12.9170441691382','6','70']
  H_HDV['13_PM_4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.61380091246059','0.042035620665291','7.0664898988257','0.182237617777491','0.575975628682211','6','70']
  H_HDV['13_FC_4_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','773.830135184052','0.086077868719611','54888.6558237872','1.00703652755984','312.775729508846','6','70']
  H_HDV['13_CO_4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','50','CO','y=(c+(a*exp(b*x)))','33.9003013423082','-0.111949241963158','5.14655316948011','0','0','6','69']
  H_HDV['13_THC_4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.47844663034744','0.036233002009221','29.7176023818308','0.242601780440692','0.814792718487037','6','69']
  H_HDV['13_NOx_4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.2623155595746','0.083358455695533','763.60104595449','0.88883040386277','15.5883667231524','6','69']
  H_HDV['13_PM_4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.714232948443668','5.23777902524419','5.30604074318552','2.46071029248161','-0.025368711707889','6','69']
  H_HDV['13_FC_4_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','813.304189009524','0.092104002636378','243395.875831033','1.28171791084701','388.755981868566','6','69']
  H_HDV['13_CO_4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.1719724864102','0.053302479276083','-3.44069455491969','0.096351679177828','0','6','68']
  H_HDV['13_THC_4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.04141658837479','0.030646644560987','30.4895443175332','0.232332002680194','0.507763920316272','6','68']
  H_HDV['13_NOx_4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.1142278863069','0.08067024079179','1712.77181230414','1.03193299956504','18.2698750664082','6','68']
  H_HDV['13_PM_4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.876202893349822','4.96010798633722','6.7618974963452','3.07702299909438','-0.050433350077511','6','68']
  H_HDV['13_FC_4_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','4','100','FC','y=((a*(b**x))*(x**c))','2556.4675971119','1.01023595812251','-0.559928435439441','0','0','6','68']
  H_HDV['13_CO_6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.75796150914041','26.7521014571737','5.53225040727551','2.54999525579461','-0.050529856696237','6','69']
  H_HDV['13_THC_6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.99556758354925','0.037214778137132','28.4407602928113','0.240642881303499','0.790155288961982','6','69']
  H_HDV['13_NOx_6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.4238793871112','0.081459771774591','153.612870013767','0.575699512313968','16.0382947857683','6','69']
  H_HDV['13_PM_6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.738898621336681','5.10744574602137','5.91600690155688','2.74561379165522','-0.035290024494157','6','69']
  H_HDV['13_FC_6_0'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','834.851174723711','0.096760278483296','92181.9495708105','1.11758336699608','398.662701307931','6','69']
  H_HDV['13_CO_6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','44.2506552834665','0.054559818514923','-4.14763791862718','0.105553454910792','0','6','62']
  H_HDV['13_THC_6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.23937177738043','0.0272194109181','29.032778386982','0.220382635755097','0.211142747359655','6','62']
  H_HDV['13_NOx_6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.9568135060888','0.076493473910875','143.308789531493','0.547506107976725','19.8577776724543','6','62']
  H_HDV['13_PM_6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.06979307497545','4.85292534545017','7.82107239394773','3.55337504743981','-0.057820169790834','6','62']
  H_HDV['13_FC_6_50'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','852.732832502643','0.104616968385358','398449.20542924','1.38358043036767','511.83447417579','6','62']
  H_HDV['13_CO_6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-28.1241486166098','0.014346484162477','54.5146667987872','0.095840756721094','25.2369108426529','6','54']
  H_HDV['13_THC_6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.37029046066537','0.01854398172985','31.236425985922','0.227015873557535','-1.33544897191196','6','54']
  H_HDV['13_NOx_6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.2332804617197','0.080697686494408','266.033569424571','0.673838075426313','23.9301238415726','6','54']
  H_HDV['13_PM_6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.51339243097094','4.62339675371962','11.4283769858253','5.38543654725479','-0.136682382260964','6','54']
  H_HDV['13_FC_6_100'] = [' 13','EURO0','Ubus Midi <=15t 80ties','6','100','FC','y=((a*(b**x))*(x**c))','2630.81339148076','1.01319081341929','-0.522691438873907','0','0','6','54']
  H_HDV['14_CO_0_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.42580801199841','0.056340790353796','6.66584116505467','0.232333590238774','0.72463088471653','6','75']
  H_HDV['14_THC_0_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.62064626067106','0.060950596959772','16.6283084966515','0.572938454983275','0.325633760828965','6','75']
  H_HDV['14_NOx_0_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.5565371209696','0.080998741037697','986.349862644029','0.991863538815242','4.55464880218061','6','75']
  H_HDV['14_PM_0_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.854120740218933','0.065118970399105','4.74879282171049','0.621910517429365','0.163126406389896','6','75']
  H_HDV['14_FC_0_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000797094265041','0.000199683823513','-0.000001575175322','0','0','6','75']
  H_HDV['14_CO_0_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.38558944957644','0.05358298528866','5.6583498434077','0.191888126892919','0.77137005429584','6','75']
  H_HDV['14_THC_0_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.63236520506951','0.061947848820842','15.7069730147478','0.564903632404684','0.322110802278382','6','75']
  H_HDV['14_NOx_0_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.5462114483592','0.076206809831916','17248.3897659034','1.4907698115943','4.98743324459909','6','75']
  H_HDV['14_PM_0_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.924002326209009','0.066596048104717','32.6699194838594','0.99484108846699','0.174841837195839','6','75']
  H_HDV['14_FC_0_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000937848133366','0.000168204291133','-0.000001269264007','','','6','75']
  H_HDV['14_CO_0_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.066237380595814','0.016442080503198','-0.000024926012116','0','0','6','75']
  H_HDV['14_THC_0_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.63056791146101','0.062486685331435','14.1503254292565','0.550587970833216','0.320481379596945','6','75']
  H_HDV['14_NOx_0_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.0425270328902','-0.01920778679463','-1.60842660910223','0.08636981971203','0','6','75']
  H_HDV['14_PM_0_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.801181032439145','0.058445067372101','0.390028772305483','0.179112330768817','0.181697115675203','6','75']
  H_HDV['14_FC_0_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','0','100','FC','y=(c+(a*exp(b*x)))','569.512655162035','-0.075209557302477','164.831829038741','0','0','6','75']
  H_HDV['14_CO_-2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','0','CO','y=(c+(a*exp(b*x)))','6.02408046455698','-0.076953193841614','0.686665723209443','0','0','6','75']
  H_HDV['14_THC_-2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.60568721657869','0.066492781074136','21.0645195909062','0.627745907257041','0.29114271308028','6','75']
  H_HDV['14_NOx_-2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','0','NOx','y=(c+(a*exp(b*x)))','19.1156752214565','-0.079013522020223','2.30987841433016','0','0','6','75']
  H_HDV['14_PM_-2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.862685052139015','0.064050723326687','3.85339538586414','0.563599445927548','0.139852552998935','6','75']
  H_HDV['14_FC_-2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','620.763028577833','0.610607927895854','-39.5695122133181','0.068353780918989','0','6','75']
  H_HDV['14_CO_-2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.27575106273644','0.009394917724011','-0.360049389263216','0.058064033783382','0','6','75']
  H_HDV['14_THC_-2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.63842803348236','0.065778980781334','21.6954572920107','0.640167546911579','0.25322187159674','6','75']
  H_HDV['14_NOx_-2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','50','NOx','y=(c+(a*exp(b*x)))','17.7856540528819','-0.06714668549389','1.89318475731792','0','0','6','75']
  H_HDV['14_PM_-2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.135195909308639','3.05356312137262','-0.278428765406461','0.316249880813158','0.05920930362199','6','75']
  H_HDV['14_FC_-2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','574.027602974568','1.20409719909388','-30.5627742549285','0.052905636802449','0','6','75']
  H_HDV['14_CO_-2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000023553315865','0.004263313528509','-0.259023831704699','5.93820290471204','0','6','75']
  H_HDV['14_THC_-2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.63777506408436','0.063484284523201','20.7602412211372','0.635218425600172','0.217021146981351','6','75']
  H_HDV['14_NOx_-2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000068795912986','0.012172939992713','-0.736771277464182','17.4129869367909','0','6','75']
  H_HDV['14_PM_-2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','100','PM','y=(c+(a*exp(b*x)))','1.0968155706345','-0.069023925952165','0.127848201898079','0','0','6','75']
  H_HDV['14_FC_-2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001890834523864','0.349885182366344','-21.9184422854523','532.052643601029','0','6','75']
  H_HDV['14_CO_-4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','0','CO','y=(c+(a*exp(b*x)))','5.44952389784851','-0.068687899671823','0.260918438364457','0','0','6','75']
  H_HDV['14_THC_-4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.79384109480471','0.0650230808588','44.4958913151896','0.793134982542037','0.140280654470448','6','75']
  H_HDV['14_NOx_-4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','0','NOx','y=(c+(a*exp(b*x)))','17.7514654218788','-0.073973487512058','0.677821620275454','0','0','6','75']
  H_HDV['14_PM_-4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.843448541393043','0.058761903127985','0.989442531832918','0.265985667337372','0.078217350232355','6','75']
  H_HDV['14_FC_-4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','0','FC','y=(c+(a*exp(b*x)))','529.888006005268','-0.071602643591213','20.2206887823591','0','0','6','75']
  H_HDV['14_CO_-4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000023899263466','0.004130870991626','-0.240988032745797','5.06650544559651','0','6','75']
  H_HDV['14_THC_-4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.84069425250705','0.065592115192315','47.0199728149268','0.814783363339218','0.104266279876654','6','75']
  H_HDV['14_NOx_-4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000079331437793','0.013381067963426','-0.758429288220075','15.3348503498923','0','6','75']
  H_HDV['14_PM_-4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.76760477266204','-7.26633259659142','-1.50301013715329','0','0','6','75']
  H_HDV['14_FC_-4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002267565554503','0.388929526586047','-22.4686222066095','464.198132910592','0','6','75']
  H_HDV['14_CO_-4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.290256665591102','-0.01061901861714','0.001239081315281','0','0','6','75']
  H_HDV['14_THC_-4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.884291359978','0.067826962164219','50.0432617056922','0.834589786999483','0.095026013990561','6','75']
  H_HDV['14_NOx_-4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000067061690054','0.011771458469554','-0.695985321805682','14.6369592921722','0','6','75']
  H_HDV['14_PM_-4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.26041127981063','-8.53409515818568','-1.64106141385501','0','0','6','75']
  H_HDV['14_FC_-4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00188077023631','0.338126073854346','-20.517188472792','443.208571718306','0','6','75']
  H_HDV['14_CO_-6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','0','CO','y=(c+(a*exp(b*x)))','5.29611552722468','-0.071773547871414','0.104365292125938','0','0','6','70']
  H_HDV['14_THC_-6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92403770700596','0.070206508309991','61.9303533805194','0.878675665767909','0.080417914309709','6','70']
  H_HDV['14_NOx_-6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','0','NOx','y=(c+(a*exp(b*x)))','16.476617362167','-0.078198733952833','0.239184802517586','0','0','6','70']
  H_HDV['14_PM_-6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','4.10700829914391','-7.53515131680377','-1.6420348438781','0','0','6','70']
  H_HDV['14_FC_-6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','0','FC','y=(c+(a*exp(b*x)))','485.791628341951','-0.074421702907637','5.44404401854632','0','0','6','70']
  H_HDV['14_CO_-6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','50','CO','y=(c+(a*exp(b*x)))','5.19454599035759','-0.070252546389016','0.074303540216506','0','0','6','70']
  H_HDV['14_THC_-6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.95221944221766','0.073524479269476','56.8225366872419','0.87221206017509','0.078242754481384','6','70']
  H_HDV['14_NOx_-6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','50','NOx','y=(c+(a*exp(b*x)))','15.2257160602855','-0.074011702262509','0.138788717656302','0','0','6','70']
  H_HDV['14_PM_-6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.47403183204874','-8.31678477742971','-1.75764271084648','0','0','6','70']
  H_HDV['14_FC_-6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','50','FC','y=(c+(a*exp(b*x)))','447.696567062982','-0.069873369060483','1.96722460155951','0','0','6','70']
  H_HDV['14_CO_-6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000024118686586','0.00408776019596','-0.233038359917269','4.66069718868171','0','6','70']
  H_HDV['14_THC_-6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.96198017624574','0.075463490826463','49.1171129963863','0.850388266587305','0.077835445938183','6','70']
  H_HDV['14_NOx_-6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','100','NOx','y=(c+(a*exp(b*x)))','14.4424894506305','-0.070376825626898','0.068945287932031','0','0','6','70']
  H_HDV['14_PM_-6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.57245291068324','-8.42725698255002','-1.78813290817343','0','0','6','70']
  H_HDV['14_FC_-6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','940.112066024061','1.47804814275497','-0.078192939272187','0','0','6','70']
  H_HDV['14_CO_2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.536733499444226','8.5620724506115','3.77435825704688','1.68983400765174','-0.004798067408192','6','75']
  H_HDV['14_THC_2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.45671296461745','0.053673949518611','7.8311852636308','0.403520957161561','0.32820082249647','6','75']
  H_HDV['14_NOx_2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.2673906951285','0.090647226397866','1126.24556486279','1.03583376917752','7.12436321291894','6','75']
  H_HDV['14_PM_2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','0','PM','y=(c+(a*exp(b*x)))','0.988385959812833','-0.072354809125472','0.204290478740931','0','0','6','75']
  H_HDV['14_FC_2_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.001557046592009','2.49350272937211','1595.59896163331','-0.554445067469962','0','6','75']
  H_HDV['14_CO_2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','50','CO','y=(c+(a*exp(b*x)))','9.74778673425246','-0.086319050059794','1.23113225043411','0','0','6','75']
  H_HDV['14_THC_2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.45000787800325','0.055727801256923','6.78406462228046','0.375394069079725','0.346467383953893','6','75']
  H_HDV['14_NOx_2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.932964207532','0.084352591490838','6504.70449785596','1.34287690005972','8.37130877007302','6','75']
  H_HDV['14_PM_2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.22598165130948','0.001336350218154','-0.065443161391186','0.061864031634371','0','6','75']
  H_HDV['14_FC_2_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000995811221309','0.000109749219507','-0.000000967471781','0','0','6','75']
  H_HDV['14_CO_2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','100','CO','y=(c+(a*exp(b*x)))','10.1926106313064','-0.082272149477128','1.46232627422402','0','0','6','75']
  H_HDV['14_THC_2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.42073856913574','0.056395675484509','6.19362449493462','0.355768564169335','0.367157224521722','6','75']
  H_HDV['14_NOx_2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.6291272791281','0.078548892231035','44832.8940197118','1.67501121199544','9.5618257445652','6','75']
  H_HDV['14_PM_2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004503537006','0.000791607635935','-0.046121060560194','1.19330656478089','0','6','75']
  H_HDV['14_FC_2_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','851.464363375723','0.890436529172306','-41.0933227235033','0.066581184356307','0','6','75']
  H_HDV['14_CO_4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.909680006768887','7.85365796258251','6.25878337963965','2.735906290823','-0.038424859136833','6','70']
  H_HDV['14_THC_4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.48177364789238','0.053786277880254','6.00725817698667','0.351005036389417','0.348801141443237','6','70']
  H_HDV['14_NOx_4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.8291151088713','0.088566280711636','56.3791481922745','0.462865296459502','9.77033354281114','6','70']
  H_HDV['14_PM_4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','0','PM','y=(c+(a*exp(b*x)))','1.06348021278372','-0.081381578412122','0.267477795522448','0','0','6','70']
  H_HDV['14_FC_4_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','1853.4447714459','1.0096824207234','-0.603342292862942','0','0','6','70']
  H_HDV['14_CO_4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.3820130669809','7.88398350154285','7.24197252130127','3.05985020286862','-0.043190626320307','6','70']
  H_HDV['14_THC_4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.41659405446709','0.057520302564551','5.33156366977424','0.322188609282843','0.4109184220561','6','70']
  H_HDV['14_NOx_4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.8651574125942','0.07629974337758','40.852573517299','0.374643331527091','11.7367247500356','6','70']
  H_HDV['14_PM_4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.36175909861314','0.00220901789171','-0.074664864033741','0.066283504997835','0','6','70']
  H_HDV['14_FC_4_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','582.11717075952','0.08804753272711','1968249.59083408','1.72020107711632','343.107615082568','6','70']
  H_HDV['14_CO_4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.60351667915812','9.15601363080571','5.76456311347408','2.35408195607459','-0.016796082666645','6','68']
  H_HDV['14_THC_4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.45884369213313','0.065772342011485','5.27903999231686','0.328773101463014','0.494573824565571','6','68']
  H_HDV['14_NOx_4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.5836326824393','0.069966614543597','35.6572407210441','0.345376804973734','13.6491443889842','6','68']
  H_HDV['14_PM_4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006293617279','0.001005258942745','-0.052476799361694','1.3437607688662','0','6','68']
  H_HDV['14_FC_4_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','4','100','FC','y=(c+(a*exp(b*x)))','639.487863195619','-0.091893575820936','411.370170846427','0','0','6','68']
  H_HDV['14_CO_6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.69228765249441','7.52659956207718','8.10536966645984','3.51370605289451','-0.048245016375522','6','70']
  H_HDV['14_THC_6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51429370096422','0.05832226892532','5.34595409900397','0.331644225050091','0.411154558627984','6','70']
  H_HDV['14_NOx_6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.152464519146','0.069164626330274','34.5074465169605','0.275976994558479','12.1472087826668','6','70']
  H_HDV['14_PM_6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.42971875306399','0.001890345608241','-0.090619687538613','0.077664150907088','0','6','70']
  H_HDV['14_FC_6_0'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','603.496063309134','0.096318771380762','289418.3266596','1.39863328896965','353.455365633977','6','70']
  H_HDV['14_CO_6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.7067424517347','-0.023532537991199','-1.85247978216395','0.128815286729556','0','6','63']
  H_HDV['14_THC_6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.58604273479131','0.069916685920191','5.77598104684343','0.358183568037277','0.529684900377898','6','63']
  H_HDV['14_NOx_6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.68635065462153','0.056230143708898','34.8696906821341','0.249225306054877','14.8026953575527','6','63']
  H_HDV['14_PM_6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.48111871217869','0.004558276682259','-0.078458242434967','0.064762605603664','0','6','63']
  H_HDV['14_FC_6_50'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','594.604242500142','0.094657143231102','567877.78666226','1.5148245435727','448.702237746257','6','63']
  H_HDV['14_CO_6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','100','CO','y=(1/(a+(b*x)))','0.053646169437707','0.005776636956922','0','0','0','6','51']
  H_HDV['14_THC_6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.089824650754933','0.063962067461041','-0.000760926402552','0','0','6','51']
  H_HDV['14_NOx_6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','51.1330518253633','-0.053440965694853','-4.79600403562606','0.156004633260634','0','6','51']
  H_HDV['14_PM_6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.568951207150663','0.045269351499992','-0.000476633289526','0','0','6','51']
  H_HDV['14_FC_6_100'] = [' 14','EURO1','Ubus Midi <=15t Euro-1','6','100','FC','y=((a*(b**x))*(x**c))','1810.741103413','1.0079645994069','-0.406592515363824','0','0','6','51']
  H_HDV['15_CO_0_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.224305770560185','8.04767200593104','3.57296671716715','1.67029366774919','-0.009210259496751','6','75']
  H_HDV['15_THC_0_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.11489157040656','0.062454054740432','10.2462828033822','0.546961178768906','0.210557450583653','6','75']
  H_HDV['15_NOx_0_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.0247254383001','0.076981553720024','349.52414198285','0.775007372859797','4.85363737690481','6','75']
  H_HDV['15_PM_0_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','0','PM','y=(c+(a*exp(b*x)))','0.444412826707218','-0.08498728214672','0.083839254084466','0','0','6','75']
  H_HDV['15_FC_0_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','486.6405839669','0.080972385563407','1196062.93385361','1.62984603287574','134.051225727017','6','75']
  H_HDV['15_CO_0_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.294444133592442','7.67811287557201','3.89313856643345','1.69372598945914','-0.006696289722874','6','75']
  H_HDV['15_THC_0_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.11188995011114','0.062936805467237','9.64731484273651','0.536033834272477','0.20706234042683','6','75']
  H_HDV['15_NOx_0_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.5136710457335','0.074043473013214','2779.01263543827','1.14875407049243','5.25154028977105','6','75']
  H_HDV['15_PM_0_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','50','PM','y=(c+(a*exp(b*x)))','0.415452560317313','-0.072671588182818','0.089366962279193','0','0','6','75']
  H_HDV['15_FC_0_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','474.496872983834','0.07433390660379','10230354.3068603','1.99415048670657','146.567795850206','6','75']
  H_HDV['15_CO_0_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.5387889396868','-4.11551756127658','-1.13228321541918','0','0','6','75']
  H_HDV['15_THC_0_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.10521906098557','0.063232513683404','8.94739997463689','0.525523976310588','0.205217090717638','6','75']
  H_HDV['15_NOx_0_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.7396087888578','-0.025518869348361','-1.88149377622355','0.090738262654858','0','6','75']
  H_HDV['15_PM_0_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','100','PM','y=(c+(a*exp(b*x)))','0.424161317260505','-0.0654681212069','0.095258296088091','0','0','6','75']
  H_HDV['15_FC_0_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','0','100','FC','y=(c+(a*exp(b*x)))','503.410039151056','-0.072506562888369','160.59089819915','0','0','6','75']
  H_HDV['15_CO_-2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','0','CO','y=(c+(a*exp(((-1)*b)*x)))','5.82007479220806','0.07660215730069','0.474512365542712','0','0','6','75']
  H_HDV['15_THC_-2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.11263286896115','0.068529552839346','16.0054375897449','0.646166355905653','0.193978283356149','6','75']
  H_HDV['15_NOx_-2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.5238953630399','0.07193518896687','51922.4940899756','1.65845042117174','2.4422551722386','6','75']
  H_HDV['15_PM_-2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','0','PM','y=(c+(a*exp(b*x)))','0.374990609619869','-0.067653415385554','0.044436693048398','0','0','6','75']
  H_HDV['15_FC_-2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','559.192774909764','0.608904295606699','-34.0860658711214','0.065321660624641','0','6','75']
  H_HDV['15_CO_-2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000024840111519','0.004339649897627','-0.253156431876431','5.47684451283508','0','6','75']
  H_HDV['15_THC_-2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.10911667718287','0.066750710311201','17.4805521256551','0.66367141610557','0.17017493966165','6','75']
  H_HDV['15_NOx_-2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','50','NOx','y=(c+(a*exp(b*x)))','19.5560506441971','-0.068751873804034','2.12597034889719','0','0','6','75']
  H_HDV['15_PM_-2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','50','PM','y=(c+(a*exp(b*x)))','0.355672332858386','-0.055392299390346','0.035520659302274','0','0','6','75']
  H_HDV['15_FC_-2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001973614752665','0.352535639010901','-21.1130419274563','495.061722668966','0','6','75']
  H_HDV['15_CO_-2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','100','CO','y=(c+(a*exp(b*x)))','5.64769707162419','-0.058445499360785','0.292095866367384','0','0','6','75']
  H_HDV['15_THC_-2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.09012484989921','0.063889723312967','17.6777472280898','0.663897403523853','0.148223851821392','6','75']
  H_HDV['15_NOx_-2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000080797889704','0.013960091143845','-0.824720658938434','19.0692799190616','0','6','75']
  H_HDV['15_PM_-2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','100','PM','y=(c+(a*exp(b*x)))','0.357915178161877','-0.045994937889033','0.02522961351923','0','0','6','75']
  H_HDV['15_FC_-2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001547045694563','0.29569150868555','-19.1528916328318','483.932259320516','0','6','75']
  H_HDV['15_CO_-4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','0','CO','y=(c+(a*exp(b*x)))','4.72767972155253','-0.067683135770376','0.169385209422594','0','0','6','75']
  H_HDV['15_THC_-4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.19008239379957','0.065755665645732','61.0151755221712','0.90619621340626','0.107565498703125','6','75']
  H_HDV['15_NOx_-4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','0','NOx','y=(c+(a*exp(b*x)))','19.4683660849748','-0.074959039614798','0.813113729889659','0','0','6','75']
  H_HDV['15_PM_-4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','0','PM','y=(c+(a*exp(b*x)))','0.347442013581288','-0.064023561508748','0.023197001041937','0','0','6','75']
  H_HDV['15_FC_-4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','0','FC','y=(c+(a*exp(b*x)))','476.28548914075','-0.0699035378142','19.6059784154665','0','0','6','75']
  H_HDV['15_CO_-4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.365424673206305','-0.016402886989945','0.001581372027245','0','0','6','75']
  H_HDV['15_THC_-4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.18120434871875','0.064008760227889','79.6201821120498','0.952372313588521','0.082477268674291','6','75']
  H_HDV['15_NOx_-4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','50','NOx','y=(c+(a*exp(b*x)))','18.0819743689171','-0.068588979589348','0.492906230260348','0','0','6','75']
  H_HDV['15_PM_-4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','50','PM','y=(c+(a*exp(b*x)))','0.325917285520335','-0.055001546488479','0.016767867140795','0','0','6','75']
  H_HDV['15_FC_-4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','50','FC','y=(c+(a*exp(b*x)))','446.430140749146','-0.063056422361139','10.7764476122928','0','0','6','75']
  H_HDV['15_CO_-4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','100','CO','y=(c+(a*exp(b*x)))','4.42273373185506','-0.053871406377299','0.002420729821279','0','0','6','75']
  H_HDV['15_THC_-4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.18450834782056','0.064827999407932','111.336070670927','1.0130127726765','0.074598513711725','6','75']
  H_HDV['15_NOx_-4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000075587204944','0.013095395091422','-0.764644343891867','15.9316623683303','0','6','75']
  H_HDV['15_PM_-4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','100','PM','y=(c+(a*exp(b*x)))','0.323523777848023','-0.048643470705967','0.011489189925024','0','0','6','75']
  H_HDV['15_FC_-4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-4','100','FC','y=(c+(a*exp(b*x)))','422.623262478602','-0.057488300828335','5.12972625675794','0','0','6','75']
  H_HDV['15_CO_-6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','0','CO','y=(c+(a*exp(b*x)))','4.08713588754907','-0.065651845677379','0.044367503439656','0','0','6','70']
  H_HDV['15_THC_-6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.20747575800797','0.066722074823248','171.445338599315','1.09278877934459','0.066775321458565','6','70']
  H_HDV['15_NOx_-6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','0','NOx','y=(c+(a*exp(b*x)))','18.1245904180963','-0.079132773617591','0.299656618408623','0','0','6','70']
  H_HDV['15_PM_-6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','0','PM','y=(c+(a*exp(b*x)))','0.320822637143952','-0.06477863170871','0.01894403068744','0','0','6','70']
  H_HDV['15_FC_-6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','0','FC','y=(c+(a*exp(b*x)))','436.687814245992','-0.072921306821833','5.52468400198082','0','0','6','70']
  H_HDV['15_CO_-6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','50','CO','y=(c+(a*exp(b*x)))','3.8399333902314','-0.060415739655331','0.00177001419666','0','0','6','70']
  H_HDV['15_THC_-6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.19563392555295','0.068401523885368','203.488191315466','1.12438847110558','0.065017327966055','6','70']
  H_HDV['15_NOx_-6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','50','NOx','y=(c+(a*exp(b*x)))','16.620895730337','-0.074914680264461','0.185941129288472','0','0','6','70']
  H_HDV['15_PM_-6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001203917602','0.000214620976661','-0.013048024191631','0.297436488506216','0','6','70']
  H_HDV['15_FC_-6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','50','FC','y=(c+(a*exp(b*x)))','398.692441091959','-0.067664282880059','1.86170229628833','0','0','6','70']
  H_HDV['15_CO_-6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.12200742653626','3.52512027709471','1.46970475340583','-0.039849096252478','0.114935758550129','6','70']
  H_HDV['15_THC_-6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.19319477906676','0.069838533481429','218.387261305885','1.14254245935236','0.065233434232704','6','70']
  H_HDV['15_NOx_-6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','100','NOx','y=(c+(a*exp(b*x)))','15.7137661071196','-0.071450050217927','0.112696369923263','0','0','6','70']
  H_HDV['15_PM_-6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','100','PM','y=(1/(a+(b*(x**c))))','3.87526872047002','0.013732279713103','1.94018156398206','0','0','6','70']
  H_HDV['15_FC_-6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','751.843623001217','1.25119382833903','-0.077375225055922','0','0','6','70']
  H_HDV['15_CO_2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.0963857096802','-0.013020008870572','-1.22294048427771','0.116489345966393','0','6','75']
  H_HDV['15_THC_2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.975151456811552','0.053728134254335','5.08376897156332','0.380658775550763','0.205044357044838','6','75']
  H_HDV['15_NOx_2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.5540605153667','0.081884698410134','180.109659401227','0.654709487320102','7.39208803225695','6','75']
  H_HDV['15_PM_2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','0','PM','y=(c+(a*exp(b*x)))','0.47525480196779','-0.078916512252011','0.114790027527116','0','0','6','75']
  H_HDV['15_FC_2_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.002178253944216','2.40608524983969','1389.68659802552','-0.524026424853049','0','6','75']
  H_HDV['15_CO_2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.7888880113902','-0.014063014350967','-1.15939160929768','0.105256987740939','0','6','75']
  H_HDV['15_THC_2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.959858666412963','0.05533399997614','4.61808824233594','0.357394438026983','0.21491455387111','6','75']
  H_HDV['15_NOx_2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.1577121176648','0.075265455983469','211.360188279313','0.683996029592329','8.47326927201844','6','75']
  H_HDV['15_PM_2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','50','PM','y=(c+(a*exp(b*x)))','0.478070002876924','-0.065157935787817','0.123035417321531','0','0','6','75']
  H_HDV['15_FC_2_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001149529277001','0.000106984250547','-0.00000094763206','0','0','6','75']
  H_HDV['15_CO_2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.2723253391824','-0.016344293657578','-1.25166997396752','0.102119721609152','0','6','75']
  H_HDV['15_THC_2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.939573699388638','0.05615466977896','4.24617462062165','0.338835421068003','0.225821846050874','6','75']
  H_HDV['15_NOx_2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.4341404263234','0.071677188743233','346.85794939315','0.780236950018689','9.53067349042325','6','75']
  H_HDV['15_PM_2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','100','PM','y=(c+(a*exp(b*x)))','0.533001617469373','-0.060152587265095','0.132558237744101','0','0','6','75']
  H_HDV['15_FC_2_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','759.531859556492','1.29150329506811','-31.7894021139903','0.057745314138585','0','6','75']
  H_HDV['15_CO_4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','0','CO','y=((a*(x**b))+(c*(x**d)))','-12043535.3427925','-8.77520098325543','51.9591051975872','-0.945351974165883','0','6','70']
  H_HDV['15_THC_4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.99820119113875','0.054560504105066','4.20054982267004','0.341088635479397','0.217417471319359','6','70']
  H_HDV['15_NOx_4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.9071860024654','0.067814075036369','46.7345956475139','0.340626977074988','9.68451658134559','6','70']
  H_HDV['15_PM_4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.777860665210587','-0.000994844142114','-0.063207757805363','0.108697663521168','0','6','70']
  H_HDV['15_FC_4_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1587.37421533895','1.00915125339105','-0.560688626319585','0','0','6','70']
  H_HDV['15_CO_4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.0789012365622','-0.025040291546884','-1.95910025295066','0.126622535207908','0','6','70']
  H_HDV['15_THC_4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.955813215641546','0.057559864704643','3.77872018231988','0.311886748731194','0.249593261589406','6','70']
  H_HDV['15_NOx_4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.0862666241003','0.060052151166543','43.7483988662257','0.312520563904095','11.4280877194706','6','70']
  H_HDV['15_PM_4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.04770177846323','0.124711539309421','-0.00065182776614','0','0','6','70']
  H_HDV['15_FC_4_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','50','FC','y=((a*(x**b))+(c*(x**d)))','0.015291853011056','2.03555142027807','1320.50347212328','-0.379959789178084','0','6','70']
  H_HDV['15_CO_4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.27627727067081','-4.07276011355026','-1.20918650361669','0','0','6','68']
  H_HDV['15_THC_4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.872347705734575','0.058253285145541','3.42994567305473','0.285480758005391','0.288164739222487','6','68']
  H_HDV['15_NOx_4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.1374763344228','0.05548527698359','43.0173596911335','0.300920855152671','13.0442730875904','6','68']
  H_HDV['15_PM_4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','100','PM','y=(c+(a*exp(((-1)*b)*x)))','0.716588887187902','0.068013463953531','0.174175960966977','0','0','6','68']
  H_HDV['15_FC_4_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','4','100','FC','y=(c+(a*exp(b*x)))','532.453990168055','-0.086155786793856','406.376211882522','0','0','6','68']
  H_HDV['15_CO_6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','0','CO','y=((a*(x**b))+(c*(x**d)))','-3171541.13494216','-7.86141169543929','67.2336643439105','-0.979540041814294','0','6','70']
  H_HDV['15_THC_6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.06007143849036','0.06110088396919','3.76806148064625','0.326385763090835','0.25657697439928','6','70']
  H_HDV['15_NOx_6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4927819282789','0.050789734980445','42.3366799701289','0.266904737065475','11.6696349043518','6','70']
  H_HDV['15_PM_6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.00639602035925','-0.001881362278216','-0.100155802446693','0.13477632142513','0','6','70']
  H_HDV['15_FC_6_0'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','1572.04666158812','1.0090242166777','-0.496521728062437','0','0','6','70']
  H_HDV['15_CO_6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.6645035035715','-0.036264762150042','-2.78042342739158','0.136063462602033','0','6','63']
  H_HDV['15_THC_6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.938933338241168','0.063378492203425','3.36629143720163','0.289933173806591','0.31550417888088','6','63']
  H_HDV['15_NOx_6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.80648736358341','0.039354213712534','44.0420460181864','0.24794099346765','13.6726189867949','6','63']
  H_HDV['15_PM_6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.07729676937325','-0.001614763046522','-0.084895452190229','0.105382913677889','0','6','63']
  H_HDV['15_FC_6_50'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','495.231584280567','0.090891344670404','1822620.19061461','1.71746360413071','444.642594228422','6','63']
  H_HDV['15_CO_6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','6.03349518202239','-5.16115187531907','-1.39078661469528','0','0','6','56']
  H_HDV['15_THC_6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.586309833351642','0.030859605381619','3.29106707588688','0.234940408392989','0.269199982260506','6','56']
  H_HDV['15_NOx_6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.9015937495651','0.059638597157537','44.9720357293857','0.281477261075962','17.0700063273366','6','56']
  H_HDV['15_PM_6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.24824645144104','-0.002576685062123','-0.096638933977267','0.105432291015413','0','6','56']
  H_HDV['15_FC_6_100'] = [' 15','EURO2','Ubus Midi <=15t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','436.768994164429','0.075229380114272','611227.177457812','1.50658832389609','529.119040590104','6','56']
  H_HDV['16_CO_0_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.87800906487039','0.04964453037273','5.41797415729717','0.186813458435924','0.624641005061161','6','75']
  H_HDV['16_THC_0_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.961819044343184','0.061114344805583','11.4709697207497','0.603768237782425','0.189452148949669','6','75']
  H_HDV['16_NOx_0_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.8447369654133','97.1488709541444','0.925994011086849','1.21092076356312','-0.001221387437549','6','75']
  H_HDV['16_PM_0_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.081848941226415','0.869311940746541','-0.017412834105173','0.30581225677204','0.060691500737676','6','75']
  H_HDV['16_FC_0_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','522.676107861158','0.081014265826113','1093461.74369624','1.60380602211961','141.851567167099','6','75']
  H_HDV['16_CO_0_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.16444529739403','-3.51975018621111','-1.03846951294544','0','0','6','75']
  H_HDV['16_THC_0_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.950389106717416','0.061675173384866','9.84942741066971','0.57244391267455','0.18481109331432','6','75']
  H_HDV['16_NOx_0_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.98332929688237','92.1371379289988','0.919753210284612','1.16006019048807','-0.000486919790101','6','75']
  H_HDV['16_PM_0_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','50','PM','y=(c+(a*exp(b*x)))','0.363233767936217','-0.066561940293408','0.082488114363717','0','0','6','75']
  H_HDV['16_FC_0_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','506.669415680286','0.074632173954079','13593124.6526038','2.03140084807286','154.224301018509','6','75']
  H_HDV['16_CO_0_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.39840836757709','-4.15423772994811','-1.07383716991325','0','0','6','75']
  H_HDV['16_THC_0_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.943119341873647','0.062107589897886','8.95986228972052','0.556800124275616','0.181427366127943','6','75']
  H_HDV['16_NOx_0_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','100','NOx','y=((a*(b**x))*(x**c))','111.62586314553','1.00451563372867','-0.861041601048762','0','0','6','75']
  H_HDV['16_PM_0_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','100','PM','y=(c+(a*exp(b*x)))','0.364465468907337','-0.064064896977423','0.08355913701229','0','0','6','75']
  H_HDV['16_FC_0_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','0','100','FC','y=(c+(a*exp(b*x)))','533.91549977332','-0.072805904304251','168.026959776382','0','0','6','75']
  H_HDV['16_CO_-2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','0','CO','y=(c+(a*exp(b*x)))','5.48012329123718','-0.070035188041564','0.697578138037299','0','0','6','75']
  H_HDV['16_THC_-2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.869180186119802','0.064923452861046','9.86028858269311','0.574393043442394','0.188548685884092','6','75']
  H_HDV['16_NOx_-2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.403138917184','0.08042814987466','482.784780193118','0.793606709229938','1.96978916132886','6','75']
  H_HDV['16_PM_-2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.312826534064279','0.058385312301954','22.563104057341','1.04440955046482','0.068565653953794','6','75']
  H_HDV['16_FC_-2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','0','FC','y=(c+(a*exp(b*x)))','550.954882131657','-0.075774797039419','75.5227215444513','0','0','6','75']
  H_HDV['16_CO_-2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.91715171747579','0.011130593297639','-0.307550171759449','0.052395448406982','0','6','75']
  H_HDV['16_THC_-2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.866017245798742','0.064451802742911','9.28282218813903','0.567338452006967','0.166810350637854','6','75']
  H_HDV['16_NOx_-2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.784215780864','0.076806169872248','7387.68100852858','1.27551256856401','1.71526219707081','6','75']
  H_HDV['16_PM_-2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','50','PM','y=(c+(a*exp(b*x)))','0.338275343966215','-0.061935212915801','0.068667954363911','0','0','6','75']
  H_HDV['16_FC_-2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002185174348676','0.384630651729538','-22.7668988107185','531.48135441229','0','6','75']
  H_HDV['16_CO_-2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000019463958462','0.003677714859747','-0.232961988755367','5.70487155554815','0','6','75']
  H_HDV['16_THC_-2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.857980057438635','0.062166207364514','8.73491440243786','0.560448179878086','0.143147953248915','6','75']
  H_HDV['16_NOx_-2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.28549458757085','107.417587434155','-0.572356969068594','0.438722629235469','0.050422822213196','6','75']
  H_HDV['16_PM_-2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.379933477505505','0.00094730161107','-0.016123658219845','0.043584659013236','0','6','75']
  H_HDV['16_FC_-2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001717079648929','0.321663352155156','-20.5195892472209','515.717957138373','0','6','75']
  H_HDV['16_CO_-4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','0','CO','y=(c+(a*exp(b*x)))','4.80013627662797','-0.062378079206127','0.389885107512857','0','0','6','75']
  H_HDV['16_THC_-4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.938461536493602','0.059664280611812','15.1555408681278','0.675365330403395','0.093758084218972','6','75']
  H_HDV['16_NOx_-4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.42311944974054','147.293441268149','-0.854545999960334','0.526537769517017','0.05068904017272','6','75']
  H_HDV['16_PM_-4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.291985397652548','0.050368812989243','1.95866491109866','0.597358023399282','0.048944373137287','6','75']
  H_HDV['16_FC_-4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','0','FC','y=(c+(a*exp(b*x)))','517.772864614044','-0.070657497900243','25.9120101006858','0','0','6','75']
  H_HDV['16_CO_-4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000016691267309','0.003121287846252','-0.198387057394146','4.67042528682372','0','6','75']
  H_HDV['16_THC_-4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.974849642020935','0.062262588390806','23.953353832363','0.772362767083315','0.071148913170132','6','75']
  H_HDV['16_NOx_-4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','50','NOx','y=(c+(a*exp(b*x)))','23.8648454652845','-0.084582920855814','0.55747760116862','0','0','6','75']
  H_HDV['16_PM_-4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037344530367709','0.981219807876982','-0.181394701554477','0.332387019125514','0.042968933586749','6','75']
  H_HDV['16_FC_-4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002160448380684','0.371296973953994','-21.598835819804','456.716041902317','0','6','75']
  H_HDV['16_CO_-4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','100','CO','y=(c+(a*exp(b*x)))','4.78051749900494','-0.052196099288996','0.112285466454176','0','0','6','75']
  H_HDV['16_THC_-4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.00152516674061','0.066020169644658','45.0276265099484','0.895809624048027','0.06695248672162','6','75']
  H_HDV['16_NOx_-4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','100','NOx','y=(c+(a*exp(((-1)*b)*x)))','21.7589391690778','0.078932508553952','0.455332143620138','0','0','6','75']
  H_HDV['16_PM_-4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','100','PM','y=(c+(a*exp(b*x)))','0.347378713575253','-0.056671634420733','0.039102208623508','0','0','6','75']
  H_HDV['16_FC_-4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-4','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.003319159277632','-0.000102775015484','0.000012673330333','0','0','6','75']
  H_HDV['16_CO_-6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.53879087079809','0.008316922163361','-0.245425277853669','0.050623724748099','0','6','70']
  H_HDV['16_THC_-6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.05297206640754','0.068211546960861','67.8217861593799','0.976980902314881','0.058693181943114','6','70']
  H_HDV['16_NOx_-6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.146944887682834','173.541120404505','-1.26466770417725','0.470386048879748','0.057480608870867','6','70']
  H_HDV['16_PM_-6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.3045185423172','0.051651463105744','0.660683411475185','0.418615192388093','0.034256114655283','6','70']
  H_HDV['16_FC_-6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','0','FC','y=(c+(a*exp(b*x)))','479.02954372212','-0.073556844638893','10.7137984173513','0','0','6','70']
  H_HDV['16_CO_-6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000015557770432','0.002898098863189','-0.18233287893657','4.11215936482793','0','6','70']
  H_HDV['16_THC_-6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.05591384854356','0.072502493043218','209.745671392554','1.1804751425587','0.055671251066093','6','70']
  H_HDV['16_NOx_-6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','50','NOx','y=(c+(a*exp(b*x)))','20.212008683824','-0.087203478156639','0.253077510606397','0','0','6','70']
  H_HDV['16_PM_-6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.344422609940616','0.061207123901488','11.0820435744203','1.03456043101189','0.037804910356152','6','70']
  H_HDV['16_FC_-6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','50','FC','y=(c+(a*exp(b*x)))','433.498324689187','-0.068239209463474','6.63968057333661','0','0','6','70']
  H_HDV['16_CO_-6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.369139633999428','-0.013853040850327','0.00142762761312','0','0','6','70']
  H_HDV['16_THC_-6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040619417471551','7.3940548539044','-0.87343631663239','0.551963773416209','0.043393323016197','6','70']
  H_HDV['16_NOx_-6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','100','NOx','y=(c+(a*exp(b*x)))','18.2796445266487','-0.081700576170466','0.170124939459705','0','0','6','70']
  H_HDV['16_PM_-6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.349741836328071','0.062748127150256','1941.32193694437','1.88991345514691','0.03805003306412','6','70']
  H_HDV['16_FC_-6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','-6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00170189735965','0.301278335325869','-17.9099536983077','373.677465552704','0','6','70']
  H_HDV['16_CO_2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.95306223441143','-0.012776121772935','-0.84551958593334','0.100345296735149','0','6','75']
  H_HDV['16_THC_2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.835588206427988','0.051652602581462','4.41999650558913','0.395708700147322','0.176542858707075','6','75']
  H_HDV['16_NOx_2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2369607576235','0.041248122022161','49.2073115281575','0.206269190422038','4.57408738264313','6','75']
  H_HDV['16_PM_2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.65495037163657','0.205555486892352','-0.001048239027038','0','0','6','75']
  H_HDV['16_FC_2_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.002225967483828','2.42128543309417','1479.23149755997','-0.52738759485751','0','6','75']
  H_HDV['16_CO_2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.78237144774886','-4.71838320800688','-1.15834448573689','0','0','6','75']
  H_HDV['16_THC_2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.799530509316465','0.052226023528103','3.81922059588426','0.357500883725797','0.182645395447924','6','75']
  H_HDV['16_NOx_2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.412928696932','0.042271682068058','49.4725742844338','0.208977168038891','5.43076272474416','6','75']
  H_HDV['16_PM_2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.286614082404639','0.050622519108622','0.152466917073988','0.153902882942298','0.089582645251311','6','75']
  H_HDV['16_FC_2_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001090207876972','0.000102798897854','-0.000000912691707','0','0','6','75']
  H_HDV['16_CO_2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.05611580945636','-5.25027416786526','-1.19709566055787','0','0','6','75']
  H_HDV['16_THC_2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.758739654344468','0.051746001403996','3.47039971754316','0.329766840286974','0.188171342909517','6','75']
  H_HDV['16_NOx_2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.068647397488','0.046139805672208','48.001702313496','0.211633813546351','6.44453304759433','6','75']
  H_HDV['16_PM_2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','100','PM','y=(c+(a*exp(b*x)))','0.411617617082795','-0.06424645929418','0.102900140617818','0','0','6','75']
  H_HDV['16_FC_2_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','801.463521121737','1.08697345432152','-34.9411724270257','0.061153066737808','0','6','75']
  H_HDV['16_CO_4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.8553562817832','-0.017685407957465','-1.27054605610738','0.116480304116706','0','6','70']
  H_HDV['16_THC_4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.853129216269367','0.052364815803645','3.46160749781822','0.347315121236906','0.186233966405409','6','70']
  H_HDV['16_NOx_4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.26444128102428','0.034613841408474','55.8022620310531','0.20167828720806','6.11604315061008','6','70']
  H_HDV['16_PM_4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.403721012129','0.195999291556961','-0.001141180087659','0','0','6','70']
  H_HDV['16_FC_4_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','541.753290970896','0.089407312937437','1056092.49530394','1.6153505840426','286.393549205797','6','70']
  H_HDV['16_CO_4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.89589757835714','-4.11653837747365','-1.15654115671793','0','0','6','70']
  H_HDV['16_THC_4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.774592742287092','0.051478555919254','3.01359875963971','0.3000781881801','0.201545126766146','6','70']
  H_HDV['16_NOx_4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7519869976258','0.041117197518372','57.8176722173816','0.206739464819587','7.90871092806212','6','70']
  H_HDV['16_PM_4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.32574897460654','0.181328838804777','-0.001109298584158','0','0','6','70']
  H_HDV['16_FC_4_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','1558.29508558171','1.00715042079855','-0.467500689992354','0','0','6','70']
  H_HDV['16_CO_4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.38628620057811','-5.16166121367938','-1.25222622271086','0','0','6','68']
  H_HDV['16_THC_4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.649554075697157','0.045863207526722','2.72268446374284','0.257796719598175','0.21263966504422','6','68']
  H_HDV['16_NOx_4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.3380222298172','0.052245624700923','57.0692571156349','0.211448556380437','9.83496423618571','6','68']
  H_HDV['16_PM_4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','100','PM','y=(c+(a*exp(b*x)))','0.510954654487523','-0.07896716080855','0.139950058857417','0','0','6','68']
  H_HDV['16_FC_4_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','504.478800629918','0.078121217066016','13184068.7250331','2.03458898957952','412.126947127115','6','68']
  H_HDV['16_CO_6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.0426607399384','-0.023582020030657','-1.76526302957835','0.129061176736419','0','6','70']
  H_HDV['16_THC_6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.851217487381821','0.054155014566074','2.94447029495168','0.310309464458397','0.207724374569364','6','70']
  H_HDV['16_NOx_6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.60188132580088','0.038240619433554','60.6922073180961','0.203207533086454','8.15534092318371','6','70']
  H_HDV['16_PM_6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.230967882028166','0.039481820278609','0.515586874510663','0.190052863193951','0.102336337460459','6','70']
  H_HDV['16_FC_6_0'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','530.572493669384','0.090468686791984','241492.398728962','1.35849570562538','361.296767811679','6','70']
  H_HDV['16_CO_6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.31009679092722','-4.5112479788198','-1.24224702156323','0','0','6','65']
  H_HDV['16_THC_6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.65245720818324','0.045704237859361','2.61900557849635','0.248229542424215','0.226097194802686','6','65']
  H_HDV['16_NOx_6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7268840236505','0.049533487192897','62.2518004032821','0.205216909654113','10.6976464326105','6','65']
  H_HDV['16_PM_6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.123743375763516','0.466021481829219','4.98434615211925','2.16055951094349','-0.020243899808896','6','65']
  H_HDV['16_FC_6_50'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','50','FC','y=((a*(b**x))*(x**c))','1610.27373405757','1.00715255418033','-0.411860268070843','0','0','6','65']
  H_HDV['16_CO_6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.76614724480596','-5.1915711792038','-1.32665729631339','0','0','6','58']
  H_HDV['16_THC_6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.533477603559383','0.027687091793564','2.76685120223278','0.229459112033078','0.184671556192095','6','58']
  H_HDV['16_NOx_6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','12.5166891434707','42.1144834061388','4.75547452024457','2.34508018279615','-0.022044797212982','6','58']
  H_HDV['16_PM_6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.87398164897952','-0.001024607935517','-0.077645396632421','0.120910149266777','0','6','58']
  H_HDV['16_FC_6_100'] = [' 16','EURO3','Ubus Midi <=15t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','477.493369088282','0.07922876233819','376404.83385353','1.42224298291068','541.879050328883','6','58']
  H_HDV['17_CO_0_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.452188110544723','0.061739679284051','0.719558906995726','0.268431827060633','0.056248474968449','6','75']
  H_HDV['17_THC_0_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.054739195106647','0.062089296005433','0.449853853668332','0.530364956206179','0.00929076930283','6','75']
  H_HDV['17_NOx_0_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.88318123458991','57.2124128024503','0.274721786061897','0.967678491389236','0.021877500901996','6','75']
  H_HDV['17_PM_0_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108478224990334','0.066836209009252','0.325616284730596','0.314649097545989','0.013894499842543','6','75']
  H_HDV['17_FC_0_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','509.585465205125','0.080963988278006','290717.052002486','1.3762389299547','133.655451076637','6','75']
  H_HDV['17_CO_0_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.480236966961724','0.059837363936266','0.601168602771788','0.231298588840528','0.057418202067969','6','75']
  H_HDV['17_THC_0_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053916588983412','0.06246659404548','0.404735896001811','0.504846584944577','0.008943283387553','6','75']
  H_HDV['17_NOx_0_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.98299423073155','31.4522336627376','1.18103322893077','1.05766434425937','0.015195190756017','6','75']
  H_HDV['17_PM_0_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.109052002045894','0.065380066201441','0.321726762276094','0.308515812985508','0.013845883949712','6','75']
  H_HDV['17_FC_0_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000985626617358','0.000174685186296','-0.000001328106024','','','6','75']
  H_HDV['17_CO_0_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.548476092500524','0.060550921475816','0.507701852931042','0.221469651447463','0.0595426934537','6','75']
  H_HDV['17_THC_0_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053331788151845','0.06283364973706','0.372129639506562','0.489175722671406','0.008697105376062','6','75']
  H_HDV['17_NOx_0_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.2060258800342','-0.025102303923796','-1.71408549571001','0.112908274237702','0','6','75']
  H_HDV['17_PM_0_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.11196210910889','0.06477493107052','0.315641590105721','0.307069654479345','0.013836531030492','6','75']
  H_HDV['17_FC_0_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','490.240463912951','0.069564710419415','23213460.7617927','2.12317221846894','156.02156043552','6','75']
  H_HDV['17_CO_-2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.05141788917671','1.77518909504819','-0.463895150825292','0.373037140159851','0.057580192463922','6','75']
  H_HDV['17_THC_-2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050332246094409','0.067032223660376','0.427073799394325','0.523045393762263','0.009914459232902','6','75']
  H_HDV['17_NOx_-2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0491782814655','0.078787351613467','4410.57647696092','1.32544243491391','1.12475904409517','6','75']
  H_HDV['17_PM_-2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.119688060601106','0.074222952908709','0.435040181705448','0.408246344597673','0.011843140619746','6','75']
  H_HDV['17_FC_-2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','0','FC','y=(c+(a*exp(b*x)))','546.877300338094','-0.077782797413876','71.3477649185226','0','0','6','75']
  H_HDV['17_CO_-2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.548189341341745','0.000501297874462','-0.034339344365587','0.065336061904573','0','6','75']
  H_HDV['17_THC_-2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050334260038179','0.06750360268989','0.414573739267071','0.521510083555639','0.00893010450703','6','75']
  H_HDV['17_NOx_-2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6577548071154','0.070655960870128','115486.406125873','1.87633313912371','0.946767359380137','6','75']
  H_HDV['17_PM_-2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.126489141184117','0.075480548964827','0.523675754621127','0.458313716294818','0.01101741306485','6','75']
  H_HDV['17_FC_-2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','469.861734144725','0.061313985292543','109104799.547055','2.37616894862648','56.1814080856551','6','75']
  H_HDV['17_CO_-2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.526830439496053','0.001219644387899','-0.028639272366655','0.051876429571299','0','6','75']
  H_HDV['17_THC_-2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.049697506610029','0.065684593697254','0.390052256198387','0.512426651380994','0.007778984925576','6','75']
  H_HDV['17_NOx_-2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','100','NOx','y=(c+(a*exp(b*x)))','12.3310299152348','-0.068712870586293','0.858150016665792','0','0','6','75']
  H_HDV['17_PM_-2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.132925114689428','0.075869735702682','0.673775025027691','0.522251765316106','0.010047113349616','6','75']
  H_HDV['17_FC_-2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001884901630972','0.34201003298715','-21.0944673082717','508.902162702227','0','6','75']
  H_HDV['17_CO_-4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.417744359640292','0.00044551925886','-0.024296327038954','0.058339222842097','0','6','75']
  H_HDV['17_THC_-4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053912592053771','0.06213539127394','0.654378989160359','0.622495718245648','0.005282921809831','6','75']
  H_HDV['17_NOx_-4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.283875482253487','46.2215771120945','-0.526852104642876','0.394463225530048','0.059232034628585','6','75']
  H_HDV['17_PM_-4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.121260383656226','0.072717905750508','0.477202866123538','0.448740961221656','0.006650909233102','6','75']
  H_HDV['17_FC_-4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','0','FC','y=(c+(a*exp(b*x)))','513.25557122466','-0.072850124753452','23.0874726397816','0','0','6','75']
  H_HDV['17_CO_-4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001494039963','0.000272357145889','-0.016748994625507','0.372381227481378','0','6','75']
  H_HDV['17_THC_-4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055764008013571','0.063327598472482','0.91904874982175','0.695965450673018','0.00374159537079','6','75']
  H_HDV['17_NOx_-4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','50','NOx','y=(c+(a*exp(b*x)))','11.565983806022','-0.076426908817877','0.238674448163745','0','0','6','75']
  H_HDV['17_PM_-4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.130098817727626','0.074902451386082','0.685483477728693','0.541639032197134','0.004983864305847','6','75']
  H_HDV['17_FC_-4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002204558482451','0.376654789461495','-21.6693983417951','447.054157002492','0','6','75']
  H_HDV['17_CO_-4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.019131136021741','0.319369726601971','2.66649623053923','0.426258048259445','0.095283237932857','6','75']
  H_HDV['17_THC_-4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057699194313902','0.066998470512176','1.58576235482156','0.805256733138155','0.00341975749563','6','75']
  H_HDV['17_NOx_-4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000054188900565','0.009031854691574','-0.502999632403235','9.80478176798298','0','6','75']
  H_HDV['17_PM_-4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.139794590288243','0.078715110224188','1.46491522466842','0.704480499121214','0.004665352438772','6','75']
  H_HDV['17_FC_-4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001810851131008','0.324669756265558','-19.6354013359884','423.472342810566','0','6','75']
  H_HDV['17_CO_-6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001495666082','0.000261833077106','-0.015529277766961','0.329623982183791','0','6','70']
  H_HDV['17_THC_-6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.060368718778027','0.068316662070821','2.37846637072086','0.885446909670847','0.002900949852189','6','70']
  H_HDV['17_NOx_-6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','0','NOx','y=(c+(a*exp(b*x)))','11.4138358886081','-0.086141693412743','0.125255773259458','0','0','6','70']
  H_HDV['17_PM_-6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.126528902742005','0.076216043414453','0.500302732669696','0.482403428233505','0.003619491755745','6','70']
  H_HDV['17_FC_-6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','0','FC','y=(c+(a*exp(b*x)))','475.879388071401','-0.075682371566137','6.68022425241884','0','0','6','70']
  H_HDV['17_CO_-6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001196033088','0.000223740733716','-0.013981531660882','0.306127534354759','0','6','70']
  H_HDV['17_THC_-6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061395095727608','0.073178121426179','7.48741126020241','1.09615277261815','0.002758205482453','6','70']
  H_HDV['17_NOx_-6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','50','NOx','y=(c+(a*exp(b*x)))','10.3306720382783','-0.080715673595662','0.062566860278781','0','0','6','70']
  H_HDV['17_PM_-6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.136128739995592','0.081830870071176','1.05811926062224','0.645746651635024','0.003561136911042','6','70']
  H_HDV['17_FC_-6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','50','FC','y=(c+(a*exp(b*x)))','431.732043182665','-0.0708832733602','2.87395620912024','0','0','6','70']
  H_HDV['17_CO_-6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011225995550612','0.272839112816865','2.22248265767049','0.273012652632011','0.10034211020029','6','70']
  H_HDV['17_THC_-6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061096167409914','0.075599232300043','12.2180930023206','1.18717189667535','0.002705524119583','6','70']
  H_HDV['17_NOx_-6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','86.9719790807895','8.32063374699456','-0.077914949662349','0','0','6','70']
  H_HDV['17_PM_-6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.139933296996983','0.084335241734862','2.34756052367378','0.799849004126423','0.003535726848719','6','70']
  H_HDV['17_FC_-6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1111.66838285271','2.00786136907642','-0.075655750238488','0','0','6','70']
  H_HDV['17_CO_2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.17812734604597','-0.001341201945191','-0.128926936040942','0.125234719111495','0','6','75']
  H_HDV['17_THC_2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.045983069744022','0.049928376605291','0.201459963842997','0.344168482062306','0.007581037307283','6','75']
  H_HDV['17_NOx_2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.26295255832777','72.437328865633','-0.021762466168593','0.906348002881853','0.032404594745351','6','75']
  H_HDV['17_PM_2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.074150900392501','0.046558686516145','0.276234466909673','0.21594648811075','0.012939228877476','6','75']
  H_HDV['17_FC_2_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.005954009360724','2.1811700487361','1522.41928052382','-0.554035397934575','0','6','75']
  H_HDV['17_CO_2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.30220148369222','-0.001538051082778','-0.136969926094524','0.120789080043756','0','6','75']
  H_HDV['17_THC_2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.043822207064988','0.050037586977312','0.185928858086479','0.316142280704207','0.007559841693886','6','75']
  H_HDV['17_NOx_2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','50','NOx','y=((a*(b**x))*(x**c))','49.7883096387838','1.00881492345819','-0.736919236420567','0','0','6','75']
  H_HDV['17_PM_2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.069642310823533','0.041192736758223','0.284669396122133','0.203188989790287','0.012388952991186','6','75']
  H_HDV['17_FC_2_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001062349707341','0.000112651659687','-0.00000099874523','0','0','6','75']
  H_HDV['17_CO_2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.44461877437388','-0.001804349708237','-0.148364940507587','0.118347100768935','0','6','75']
  H_HDV['17_THC_2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.041771556633411','0.049753265300235','0.175555007540943','0.294956189186825','0.007605504137964','6','75']
  H_HDV['17_NOx_2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','100','NOx','y=((a*(b**x))*(x**c))','46.5024704101714','1.00722043421971','-0.663171079882837','0','0','6','75']
  H_HDV['17_PM_2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067325435627078','0.03622492042793','0.293682709675316','0.194017316980967','0.011175427756006','6','75']
  H_HDV['17_FC_2_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001105441601857','0.000089560551866','-0.000000780609027','0','0','6','75']
  H_HDV['17_CO_4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.58525983420414','-0.001627910080837','-0.196046285549964','0.14044137897062','0','6','70']
  H_HDV['17_THC_4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.04653646263284','0.049626322534615','0.16953164164674','0.304827676751734','0.007229823552735','6','70']
  H_HDV['17_NOx_4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.23130313105209','0.039586815260909','21.9776866371353','0.188851374474272','4.41073730717013','6','70']
  H_HDV['17_PM_4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061100260911409','0.032901599715141','0.300717595572662','0.190946182445705','0.011158979345562','6','70']
  H_HDV['17_FC_4_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','547.622493559064','0.090591099245764','282219.551573333','1.39122865517429','269.84236470726','6','70']
  H_HDV['17_CO_4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.86154255762837','-0.002345862208142','-0.228943250479782','0.140995074466664','0','6','70']
  H_HDV['17_THC_4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.043427109668124','0.050179795687223','0.159242503396509','0.275234103288948','0.007684254685895','6','70']
  H_HDV['17_NOx_4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.91893800835369','0.036568694763124','22.4626903374111','0.191426542078163','5.24751353101573','6','70']
  H_HDV['17_PM_4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.064153935674194','0.025511886405401','0.327299652482292','0.186768502197997','0.006158895452332','6','70']
  H_HDV['17_FC_4_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','534.435125695721','0.086702790613612','647816.887700829','1.53573167807755','331.326346051153','6','70']
  H_HDV['17_CO_4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.12558936837811','-0.003169255405707','-0.25461567985514','0.138104364578659','0','6','69']
  H_HDV['17_THC_4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.038368246785041','0.048449293736959','0.14765853819541','0.244067141323365','0.008315253198898','6','69']
  H_HDV['17_NOx_4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.63126567735783','0.044246999492996','23.234190057568','0.21118584371284','6.2457139185942','6','69']
  H_HDV['17_PM_4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072282564626587','0.020871982096998','0.353766641541134','0.18616305063934','-0.000805305733663','6','69']
  H_HDV['17_FC_4_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','4','100','FC','y=((a*(b**x))*(x**c))','1578.60411989217','1.00671964007507','-0.436258154034624','0','0','6','69']
  H_HDV['17_CO_6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.02243955947518','-0.002437769187877','-0.275182939903652','0.155189706219256','0','6','70']
  H_HDV['17_THC_6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047537912678363','0.053083333443748','0.15468072009155','0.282079149081804','0.007914222314745','6','70']
  H_HDV['17_NOx_6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.01541846586276','0.023470109511859','25.1307803998701','0.187328177554613','5.01412856730051','6','70']
  H_HDV['17_PM_6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065439681523923','0.018963737209868','0.335996096188056','0.183100718333464','-0.00038742097896','6','70']
  H_HDV['17_FC_6_0'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','547.910845691122','0.093929614887263','90931.0969444692','1.20066713730361','342.573732346777','6','70']
  H_HDV['17_CO_6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.4404881527943','-0.00401500066504','-0.324123158481173','0.153156824242088','0','6','66']
  H_HDV['17_THC_6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.039250220396547','0.049554321608366','0.142457143115179','0.237505938944688','0.008759740777296','6','66']
  H_HDV['17_NOx_6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.74676187078006','0.038695964989247','27.0048734050264','0.209891470333816','6.72314808894093','6','66']
  H_HDV['17_PM_6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.41893938008935','-0.000632434157264','-0.061600724849153','0.168503096369905','0','6','66']
  H_HDV['17_FC_6_50'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','532.766143295726','0.090823128689824','80254.3646289079','1.17600162570433','430.477249091136','6','66']
  H_HDV['17_CO_6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.11167080585247','-0.00596028152606','-0.432086097952568','0.16029091173271','0','6','59']
  H_HDV['17_THC_6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.031237747983933','0.040174181771985','0.147681441124852','0.219854502724658','0.008818515956475','6','59']
  H_HDV['17_NOx_6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.35587003448595','0.04729496405949','27.5008900647509','0.220873250707722','8.13416520647455','6','59']
  H_HDV['17_PM_6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.472475277446117','-0.000801955601783','-0.072965744200675','0.177066807632277','0','6','59']
  H_HDV['17_FC_6_100'] = [' 17','EURO4','Ubus Midi <=15t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','493.709101750696','0.0833869394467','52845.7873925306','1.08801215122912','516.079536858872','6','59']
  H_HDV['18_CO_0_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.458571782638048','0.061715842003384','0.740005694680463','0.269945432813017','0.057321147007227','6','75']
  H_HDV['18_THC_0_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.056241695636116','0.062206509311794','0.473040584314713','0.53591501543981','0.009562280690218','6','75']
  H_HDV['18_NOx_0_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.801119533540058','22.247113631594','1.65285155801929','1.32462697652424','-0.000866579409846','6','75']
  H_HDV['18_PM_0_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.111349568950988','0.067036768730369','0.336384850656432','0.317105965387124','0.014197553987216','6','75']
  H_HDV['18_FC_0_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','527.690582367391','0.080815669989096','262110.442970632','1.3484937038182','136.556478229772','6','75']
  H_HDV['18_CO_0_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.491328934705972','0.060403478505179','0.63037831709593','0.239285590257392','0.058825962122733','6','75']
  H_HDV['18_THC_0_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055398076390216','0.062656166459364','0.421503689490218','0.50897708542773','0.00921560212656','6','75']
  H_HDV['18_NOx_0_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','50','NOx','y=(a-(b*exp(((-1)*c)*(x**d))))','1.08530181586272','-126.048861277103','1.66831607085792','0.303068613657714','0','6','75']
  H_HDV['18_PM_0_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.111690865947885','0.065647264250981','0.330100991594239','0.309717916285387','0.014171420274674','6','75']
  H_HDV['18_FC_0_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','519.500709006639','0.075586507862591','1596867.46153298','1.65991843943019','148.314328662912','6','75']
  H_HDV['18_CO_0_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.5482909603492','0.060268420136812','0.520004621370087','0.217433944791665','0.060448332321333','6','75']
  H_HDV['18_THC_0_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.054736760042389','0.062925510673735','0.392670457934492','0.493551842621807','0.008947258784047','6','75']
  H_HDV['18_NOx_0_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.4107676576644','-0.018719643434671','-1.45524116848821','0.122915012174581','0','6','75']
  H_HDV['18_PM_0_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.114930632629905','0.065093105101332','0.330279628463697','0.311131551179913','0.01415885805427','6','75']
  H_HDV['18_FC_0_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','509.058783371456','0.069799074385329','12830150.8495907','2.01559425900684','159.112461827659','6','75']
  H_HDV['18_CO_-2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.052544053596362','1.97754297783378','-0.562716591017146','0.380918703740151','0.056506394828296','6','75']
  H_HDV['18_THC_-2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.051563198459774','0.066912876428374','0.444247366577258','0.524784889913947','0.010183491081517','6','75']
  H_HDV['18_NOx_-2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.58266750053016','0.082065815331168','472.713399833604','0.996738705440573','0.673232635061211','6','75']
  H_HDV['18_PM_-2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.121753092181852','0.073999674062899','0.444739351563179','0.405781225882469','0.012111178408321','6','75']
  H_HDV['18_FC_-2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','0','FC','y=(c+(a*exp(b*x)))','570.412201705075','-0.078398278923043','73.7172243436681','0','0','6','75']
  H_HDV['18_CO_-2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','50','CO','y=(c+(a*exp(b*x)))','0.523842710360181','-0.07382827667283','0.050190885926804','0','0','6','75']
  H_HDV['18_THC_-2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.05150840329931','0.067322999653603','0.425834524371508','0.521923637833529','0.009176519045955','6','75']
  H_HDV['18_NOx_-2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.51379614050787','38.749982502932','-0.530338943478253','0.469650701547514','0.051969941336079','6','75']
  H_HDV['18_PM_-2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.12849831876528','0.075219325262104','0.521949903736993','0.451755311365488','0.011277577143079','6','75']
  H_HDV['18_FC_-2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','50','FC','y=(c+(a*exp(b*x)))','536.515166925002','-0.067621154358534','61.8938744682512','0','0','6','75']
  H_HDV['18_CO_-2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.538953684291185','0.001109536641888','-0.029813491081485','0.053562385644285','0','6','75']
  H_HDV['18_THC_-2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050784498678172','0.065448976674532','0.397624241339966','0.511179492343852','0.007998565854212','6','75']
  H_HDV['18_NOx_-2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.20740777701313','0.068986381322301','131543.188035448','1.95189813368664','0.490510684914429','6','75']
  H_HDV['18_PM_-2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.134835790665476','0.07563139092036','0.652286549951666','0.509831369596592','0.01029628422735','6','75']
  H_HDV['18_FC_-2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002009250825125','0.361563312900074','-22.1142030635763','528.719221083569','0','6','75']
  H_HDV['18_CO_-4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','0','CO','y=(c+(a*exp(b*x)))','0.41055358547805','-0.066061454345303','0.026338912427357','0','0','6','75']
  H_HDV['18_THC_-4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055037077858604','0.061849754279187','0.655383417698042','0.617697693204099','0.005463204629681','6','75']
  H_HDV['18_NOx_-4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.01287052547994','0.080346754383178','13797.6193480422','1.57758088948605','0.206796063902059','6','75']
  H_HDV['18_PM_-4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.122864260749682','0.072268786176872','0.475151965433118','0.440749487236908','0.006855390300257','6','75']
  H_HDV['18_FC_-4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','0','FC','y=(c+(a*exp(b*x)))','535.070330262123','-0.073419108736678','24.3842444926754','0','0','6','75']
  H_HDV['18_CO_-4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001522771589','0.000277192892282','-0.017033469447627','0.379028656408326','0','6','75']
  H_HDV['18_THC_-4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.056966807640514','0.062972591773184','0.900098147340983','0.687815615703212','0.003838367234776','6','75']
  H_HDV['18_NOx_-4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.47504005761191','0.07441572766055','176163.458973035','1.9971315178563','0.118499775584593','6','75']
  H_HDV['18_PM_-4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.131964787179853','0.074447813702375','0.654516786984008','0.526961871798671','0.005099259705022','6','75']
  H_HDV['18_FC_-4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002322860923017','0.395322688766588','-22.647059976494','465.173369008726','0','6','75']
  H_HDV['18_CO_-4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.019462148987249','0.332327046158484','2.51264367941152','0.391949947964619','0.094731786706533','6','75']
  H_HDV['18_THC_-4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.058977397513282','0.066718053344307','1.54074135529042','0.79565251167788','0.00350510557776','6','75']
  H_HDV['18_NOx_-4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','100','NOx','y=(c+(a*exp(b*x)))','7.88250438530899','-0.07650275363557','0.114860217263389','0','0','6','75']
  H_HDV['18_PM_-4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.142005095044265','0.078346504582521','1.32752536416638','0.680595058694465','0.004764207026849','6','75']
  H_HDV['18_FC_-4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001923272882386','0.342513401246858','-20.5745934996622','440.826203167034','0','6','75']
  H_HDV['18_CO_-6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.353230653408758','0.000953007765888','-0.019425679715607','0.048989660004747','0','6','70']
  H_HDV['18_THC_-6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061705775339537','0.067931857089572','2.29701790069508','0.874029246381825','0.002967402203685','6','70']
  H_HDV['18_NOx_-6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.98637311441584','0.07998904779219','23314.8835681371','1.6413929442603','0.040868002715528','6','70']
  H_HDV['18_PM_-6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.128575587701673','0.075762523812789','0.492935349679953','0.472999946440158','0.003695837477917','6','70']
  H_HDV['18_FC_-6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','0','FC','y=(c+(a*exp(b*x)))','497.409308527846','-0.076232485766932','7.22144814440022','0','0','6','70']
  H_HDV['18_CO_-6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001241147979','0.000230735465391','-0.014349946717297','0.313236844701847','0','6','70']
  H_HDV['18_THC_-6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.062875545903707','0.072975530632672','7.34916473838947','1.08839378557123','0.002825881989669','6','70']
  H_HDV['18_NOx_-6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','50','NOx','y=(c+(a*exp(((-1)*b)*x)))','7.54893591065267','0.085946106385367','0.05231695028812','0','0','6','70']
  H_HDV['18_PM_-6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.138572666548399','0.081522638220028','0.981262678834963','0.62650818829126','0.003638607899948','6','70']
  H_HDV['18_FC_-6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','50','FC','y=(c+(a*exp(b*x)))','450.671745500472','-0.071482051885572','3.27655821946763','0','0','6','70']
  H_HDV['18_CO_-6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000987904756','0.000199024363408','-0.013123744421047','0.297628170372155','0','6','70']
  H_HDV['18_THC_-6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.062567991465763','0.075467471809332','12.0947130704124','1.18116951473883','0.002772032605043','6','70']
  H_HDV['18_NOx_-6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','100','NOx','y=(c+(a*exp(((-1)*b)*x)))','7.01407447331168','0.081176255430427','0.024871944394586','0','0','6','70']
  H_HDV['18_PM_-6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.142691859382394','0.084166415224128','2.14238784454903','0.778625890555048','0.003618149512212','6','70']
  H_HDV['18_FC_-6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1307.51096622561','2.35974132154813','-0.07486713483187','0','0','6','70']
  H_HDV['18_CO_2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.19723322093169','-0.001368916273551','-0.131883769332511','0.126073664640991','0','6','75']
  H_HDV['18_THC_2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047250246902351','0.049982167949001','0.208538199941205','0.347301878444251','0.007791895532624','6','75']
  H_HDV['18_NOx_2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.57463787315663','19.1431437128518','2.24167137352031','1.4951708475902','-0.003673815946485','6','75']
  H_HDV['18_PM_2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.076160273666995','0.046859848247238','0.282509919789272','0.217110075296904','0.013226526623273','6','75']
  H_HDV['18_FC_2_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.005578138769397','2.20089848413034','1586.93478017714','-0.559372211487492','0','6','75']
  H_HDV['18_CO_2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.31684708870051','-0.001553015479883','-0.138856716007461','0.121061641716881','0','6','75']
  H_HDV['18_THC_2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.045118109588037','0.050110430486453','0.191820028549571','0.318947213610442','0.007746886980537','6','75']
  H_HDV['18_NOx_2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.84422414483939','0.02690366559869','14.5095754963024','0.164903976400816','1.83464886547111','6','75']
  H_HDV['18_PM_2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.071718210840296','0.041764919986051','0.29005114111012','0.204202848854393','0.012753371151201','6','75']
  H_HDV['18_FC_2_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001018005925362','0.000111283584297','-0.000000984215445','0','0','6','75']
  H_HDV['18_CO_2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.45473241291347','-0.001801333024117','-0.149183615336122','0.118144340612525','0','6','75']
  H_HDV['18_THC_2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.042782547192823','0.049503818324138','0.181115221975186','0.296620509599379','0.007739095822695','6','75']
  H_HDV['18_NOx_2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.16157675949719','0.024410423311005','14.1667560270385','0.159189725207135','2.00223611657871','6','75']
  H_HDV['18_PM_2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.069043312226201','0.03675809279858','0.298768566839365','0.19462678021822','0.011605093388188','6','75']
  H_HDV['18_FC_2_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001061759336127','0.000088823104252','-0.000000772961306','0','0','6','75']
  H_HDV['18_CO_4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.60308224164098','-0.001663436263127','-0.198695396870005','0.140752470028211','0','6','70']
  H_HDV['18_THC_4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047903555248857','0.049705347319588','0.174704251894507','0.307912750883254','0.007420528417031','6','70']
  H_HDV['18_NOx_4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.56722787692211','0.022398033648295','17.3098696457128','0.17827627386101','2.12865570265837','6','70']
  H_HDV['18_PM_4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.062975128102769','0.033631864155495','0.305774130064966','0.191855713831993','0.011605503438775','6','70']
  H_HDV['18_FC_4_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','564.341820343328','0.090018783343776','179363.799987966','1.30753097205271','274.339560080599','6','70']
  H_HDV['18_CO_4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.88109692646578','-0.002347279994765','-0.232669619387111','0.141783752308123','0','6','70']
  H_HDV['18_THC_4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.044555891022422','0.050003768134792','0.162738283860431','0.275856060377123','0.007795573148929','6','70']
  H_HDV['18_NOx_4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.32659439375559','0.022826149829022','18.3893932342265','0.187471316158229','2.47073735397211','6','70']
  H_HDV['18_PM_4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065468616923742','0.026437581754814','0.334618637607699','0.188679280887211','0.007129318160895','6','70']
  H_HDV['18_FC_4_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','50','FC','y=((a*(x**b))+(c*(x**d)))','0.118347031180706','1.58739885227308','1533.33049639612','-0.432109694025916','0','6','70']
  H_HDV['18_CO_4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.12539488832187','-0.003116406430293','-0.253456740954452','0.137510669701447','0','6','69']
  H_HDV['18_THC_4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.040012234579206','0.048947684255195','0.153979545875265','0.249386340072128','0.008466742678273','6','69']
  H_HDV['18_NOx_4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.1846391384871','0.02619117302106','19.7355183548193','0.19889184477487','2.84636728574359','6','69']
  H_HDV['18_PM_4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072549053026662','0.021807869340906','0.357266958009746','0.185968213717493','0.000806616943945','6','69']
  H_HDV['18_FC_4_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','545.950728735917','0.082954555884821','1868589.02884949','1.70869288371052','398.394148875927','6','69']
  H_HDV['18_CO_6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.04076814523026','-0.002412038630664','-0.278275144519414','0.155381045745245','0','6','70']
  H_HDV['18_THC_6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.049024433806718','0.052951581808845','0.159231300106061','0.285071060821543','0.00802155409416','6','70']
  H_HDV['18_NOx_6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.4261862829458','-0.025503275156809','-3.45977513918097','0.17646726225536','0','6','70']
  H_HDV['18_PM_6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065868469876403','0.019440334984493','0.34104310102429','0.183084367073308','0.00054010643446','6','70']
  H_HDV['18_FC_6_0'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','565.896207426625','0.093353123170824','78251.4025891918','1.16710979577693','348.11220270674','6','70']
  H_HDV['18_CO_6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.43638470483596','-0.003834329095017','-0.321799859060357','0.152107036120357','0','6','67']
  H_HDV['18_THC_6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.04141661855356','0.050479292157285','0.147677904738812','0.243191149339768','0.008915102355806','6','67']
  H_HDV['18_NOx_6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.48216481973986','0.017104216305705','21.9017458473935','0.197448907210834','2.37866523768958','6','67']
  H_HDV['18_PM_6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.424503944747647','-0.000627609980215','-0.062258805270316','0.167973881074483','0','6','67']
  H_HDV['18_FC_6_50'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','553.754738010906','0.090868378785679','85826.7153124879','1.17822867853299','437.276064678087','6','67']
  H_HDV['18_CO_6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.08028673965328','-0.005893218836753','-0.425055038642662','0.159720554821626','0','6','60']
  H_HDV['18_THC_6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.034043703703281','0.045522295334414','0.149354350820242','0.224565349187643','0.009785398617646','6','60']
  H_HDV['18_NOx_6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.22149508092665','0.024245313967962','23.9075680606371','0.210031986006861','3.25521560475934','6','60']
  H_HDV['18_PM_6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.473091980328108','-0.000770754479358','-0.071966375482674','0.174288886003179','0','6','60']
  H_HDV['18_FC_6_100'] = [' 18','EURO5','Ubus Midi <=15t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','532.693313122543','0.087429783442419','96319.2798720681','1.19317590131972','526.880065101029','6','60']
  H_HDV['19_CO_0_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.9696140603568','0.054086236914787','28.5699879360863','0.255026025711435','1.93702396643529','6','75']
  H_HDV['19_THC_0_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.29021403874467','0.051078312158178','28.0419227211758','0.358849337929387','0.77004571668776','6','75']
  H_HDV['19_NOx_0_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','31.8391760649327','0.074864482905655','69354.1582971385','1.60472070523934','9.90923333322483','6','75']
  H_HDV['19_PM_0_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.16586768559693','0.055975628255675','4.96526840326794','0.254466684093624','0.313815140843414','6','75']
  H_HDV['19_FC_0_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','790.3517247556','0.069842734667553','35043.6034592496','0.870545331646494','200.37104763889','6','75']
  H_HDV['19_CO_0_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4887688587286','0.050525599858495','23.5557007665851','0.211694370574096','2.06009658797102','6','75']
  H_HDV['19_THC_0_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.27683496706808','0.052255089806226','28.1631776942575','0.358045190114825','0.729005985398433','6','75']
  H_HDV['19_NOx_0_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','50','NOx','y=(c+(a*exp(((-1)*b)*x)))','33.6358775966976','0.07191921090948','10.9405646505257','0','0','6','75']
  H_HDV['19_PM_0_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.12626141356873','0.05317584074574','4.3197507742442','0.223814793279102','0.330392709350419','6','75']
  H_HDV['19_FC_0_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','812.927971989443','0.067824287309969','156466.268157966','1.14658015563429','216.443587975171','6','75']
  H_HDV['19_CO_0_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7820754667943','0.046410182415916','19.8823482249879','0.167318753269535','2.21798918935943','6','75']
  H_HDV['19_THC_0_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.23213421096141','0.053578211080149','27.8928487749129','0.355669575158568','0.709911094532759','6','75']
  H_HDV['19_NOx_0_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','100','NOx','y=(c+(a*exp(b*x)))','31.7582570605093','-0.06245736095355','11.6170316118723','0','0','6','75']
  H_HDV['19_PM_0_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.08172961546857','0.050463115267027','3.75910035699878','0.191519593684573','0.355128737919831','6','75']
  H_HDV['19_FC_0_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','821.304952644782','0.064838166694094','1513488.17991348','1.54191016249673','232.754441497356','6','75']
  H_HDV['19_CO_-2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.0591744468876','0.071938773011387','450.299734398602','0.883169353042262','1.57675725997271','6','75']
  H_HDV['19_THC_-2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.2454144863313','0.057034732055277','24.7102078597243','0.364441056442997','0.715541764978359','6','75']
  H_HDV['19_NOx_-2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','0','NOx','y=(c+(a*exp(b*x)))','32.2431486857808','-0.065082667441818','3.82486246198164','0','0','6','75']
  H_HDV['19_PM_-2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.88467426479481','0.072817607326989','95.2791473929165','0.913062956742212','0.24148207756867','6','75']
  H_HDV['19_FC_-2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','89.6763775174357','3467.69793813103','-0.555774545308535','0.370028610037487','0.053546011468802','6','75']
  H_HDV['19_CO_-2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','50','CO','y=(c+(a*exp(b*x)))','19.5572638486632','-0.073364093151373','1.34763777887261','0','0','6','75']
  H_HDV['19_THC_-2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.29052766059676','0.056176004698297','24.6613213437897','0.370300367225737','0.568063061935097','6','75']
  H_HDV['19_NOx_-2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000102360325553','0.019212690421068','-1.24102839275115','31.4973194826751','0','6','75']
  H_HDV['19_PM_-2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.178238903598282','18.0058725400048','-1.03582336648671','0.351493409183308','0.052249139362482','6','75']
  H_HDV['19_FC_-2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','50','FC','y=(c+(a*exp(b*x)))','876.484130800597','-0.064418337775756','74.386595202673','0','0','6','75']
  H_HDV['19_CO_-2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','100','CO','y=(c+(a*exp(b*x)))','19.2614741647726','-0.066142015586137','1.0486478934731','0','0','6','75']
  H_HDV['19_THC_-2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.22994877705599','0.054504578418471','24.170609527528','0.36698921235246','0.445277176954163','6','75']
  H_HDV['19_NOx_-2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.22046655479096','21.4582644827239','10.7211878469439','3.64458335638224','-0.029313865741088','6','75']
  H_HDV['19_PM_-2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','100','PM','y=(c+(a*exp(b*x)))','3.25756403565832','-0.070029380833113','0.18356454689232','0','0','6','75']
  H_HDV['19_FC_-2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003376760536954','0.594406244300095','-36.0988081326627','845.655384357161','0','6','75']
  H_HDV['19_CO_-4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.6889202370693','0.070547377647007','113932.003975833','1.85977360241644','0.493155764514217','6','75']
  H_HDV['19_THC_-4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.67633811556716','0.065061311807141','43.9389485154682','0.530563349550998','0.334302788913353','6','75']
  H_HDV['19_NOx_-4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','0','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000128888646465','0.022232264406898','-1.29883403122147','27.2781437830038','0','6','75']
  H_HDV['19_PM_-4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.089894763833473','12.2547636444244','-0.705806333873192','0.344190063792791','0.056556228969912','6','75']
  H_HDV['19_FC_-4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','0','FC','y=(c+(a*exp(b*x)))','840.059424873891','-0.071098994943411','21.8446173656791','0','0','6','75']
  H_HDV['19_CO_-4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','50','CO','y=(c+(a*exp(b*x)))','17.6676919626065','-0.072246584682177','0.407217641468399','0','0','6','75']
  H_HDV['19_THC_-4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.84371678109409','0.068409849919961','65.5794548616569','0.610132940322923','0.30410871420899','6','75']
  H_HDV['19_NOx_-4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','50','NOx','y=(c+(a*exp(b*x)))','26.6488880139059','-0.055683522106141','0.119446157836902','0','0','6','75']
  H_HDV['19_PM_-4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','50','PM','y=(c+(a*exp(b*x)))','2.91986380723925','-0.07403675941187','0.097520025068409','0','0','6','75']
  H_HDV['19_FC_-4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003600791859865','0.612338664165142','-35.1175594284026','716.234973722944','0','6','75']
  H_HDV['19_CO_-4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000084307671865','0.014156421646292','-0.797759155358976','15.8719641463798','0','6','75']
  H_HDV['19_THC_-4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.87511580164477','0.070474705783997','79.9891078100761','0.646596755487987','0.292594842465899','6','75']
  H_HDV['19_NOx_-4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','100','NOx','y=(c+(a*exp(b*x)))','25.4020785576411','-0.050326346739213','-0.264461169782194','0','0','6','75']
  H_HDV['19_PM_-4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','100','PM','y=(c+(a*exp(b*x)))','2.84573152966017','-0.070637925052748','0.083151699971531','0','0','6','75']
  H_HDV['19_FC_-4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00296027107719','0.527549618206664','-31.7673021320757','677.814484069028','0','6','75']
  H_HDV['19_CO_-6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.698986625435','0.07398149024439','299201.610300065','2.01792741213334','0.211074108731044','6','70']
  H_HDV['19_THC_-6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.23136673378287','0.072888832878427','131.722621830459','0.750913927848818','0.240468334638641','6','70']
  H_HDV['19_NOx_-6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','0','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','78.7128158154606','2.35384487028776','-0.074663178437088','0','0','6','70']
  H_HDV['19_PM_-6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','0','PM','y=(c+(a*exp(b*x)))','2.81035452585533','-0.07963611361211','0.079689469641366','0','0','6','70']
  H_HDV['19_FC_-6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','0','FC','y=(c+(a*exp(b*x)))','738.919650767131','-0.074067746051787','4.96245297281994','0','0','6','70']
  H_HDV['19_CO_-6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.2656566042884','0.072208031092375','1117590.40704136','2.23630893708034','0.15111738781094','6','70']
  H_HDV['19_THC_-6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.23168543753399','0.075312296569552','186.308314594305','0.816847259031514','0.239022448709978','6','70']
  H_HDV['19_NOx_-6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','33.2006688014734','0.680584725890589','-0.085385282565884','0','0','6','70']
  H_HDV['19_PM_-6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','50','PM','y=(c+(a*exp(b*x)))','2.70373208412789','-0.077112665672288','0.070830618760011','0','0','6','70']
  H_HDV['19_FC_-6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','2350.57735877173','2.72772019334367','-0.076554271584006','0','0','6','70']
  H_HDV['19_CO_-6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','100','CO','y=(c+(a*exp(b*x)))','16.0194809098577','-0.074209151001532','0.166432735168057','0','0','6','70']
  H_HDV['19_THC_-6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.18908257143478','0.076653673150734','194.285455524656','0.826964205913669','0.238520108595789','6','70']
  H_HDV['19_NOx_-6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','24.8613815158678','0.383561013158624','-0.092936207494568','0','0','6','70']
  H_HDV['19_PM_-6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','100','PM','y=(c+(a*exp(b*x)))','2.61692994528499','-0.074274706211977','0.063250334463305','0','0','6','70']
  H_HDV['19_FC_-6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1181.22452458193','1.10288766509094','-0.082687198684396','0','0','6','70']
  H_HDV['19_CO_2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.5791763282436','0.058488365285822','24.9444647709347','0.193312205469966','2.74284711401245','6','75']
  H_HDV['19_THC_2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.30592682395992','0.033887892568113','23.0442187325741','0.271833848392581','0.374988055528467','6','75']
  H_HDV['19_NOx_2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','34.3162370377441','0.083451466902189','39293.8290579765','1.51384041424179','16.2068663613227','6','75']
  H_HDV['19_PM_2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.94366670300944','0.052798599072168','5.10190944016487','0.195524473484666','0.424637199877522','6','75']
  H_HDV['19_FC_2_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','865.018613522681','0.074973453992948','26332.7487077751','0.819837639288948','303.535279508098','6','75']
  H_HDV['19_CO_2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','50','CO','y=(c+(a*exp(b*x)))','30.3841083038958','-0.092439869962296','3.90896366240253','0','0','6','75']
  H_HDV['19_THC_2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.08753023669269','0.034708723885144','23.2031598890607','0.263862073362615','0.387501784617574','6','75']
  H_HDV['19_NOx_2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','50','NOx','y=(c+(a*exp(b*x)))','37.0735470367461','-0.082620881012625','19.2929419696189','0','0','6','75']
  H_HDV['19_PM_2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.71779429884054','0.067441909764744','4.11916767898589','0.194002466826092','0.569521173278671','6','75']
  H_HDV['19_FC_2_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','912.21637758035','0.07624107331311','127562.288699429','1.11309414435328','362.746779507268','6','75']
  H_HDV['19_CO_2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','100','CO','y=(c+(a*exp(((-1)*b)*x)))','31.0602702974999','0.089005174686895','4.69991063760181','0','0','6','74']
  H_HDV['19_THC_2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.87786472020207','0.036457879727866','23.2025896790687','0.25831825260013','0.453476913025311','6','74']
  H_HDV['19_NOx_2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','100','NOx','y=(c+(a*exp(b*x)))','35.105175167142','-0.073131271094674','21.9685015272589','0','0','6','74']
  H_HDV['19_PM_2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.681472876987194','4.84165065834544','3.56324958185369','1.52180764365851','0.028320824049712','6','74']
  H_HDV['19_FC_2_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000576150054278','0.000060931177771','-0.000000531234276','0','0','6','74']
  H_HDV['19_CO_4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.30696769130456','24.1151381641669','5.78289907493487','2.56715911208746','-0.029322015347045','6','70']
  H_HDV['19_THC_4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.41055907143362','0.037055353002565','21.8500725547651','0.254504539309571','0.410897931293486','6','70']
  H_HDV['19_NOx_4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','32.7902403937668','0.086243615484111','1380.25912442754','0.924643097800731','22.6213755752972','6','70']
  H_HDV['19_PM_4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.508954130452105','4.55271090336108','5.17972185704701','2.38742255978617','-0.024747839053722','6','70']
  H_HDV['19_FC_4_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','921.829227353149','0.085497387456559','19922.6870439655','0.786832765581607','426.17868708931','6','70']
  H_HDV['19_CO_4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.5357047578841','0.023121512778163','-3.42595941571424','0.096470505754094','0','6','68']
  H_HDV['19_THC_4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.79632033025852','0.033219166869705','21.8267050956837','0.236610296175671','0.367422322081985','6','68']
  H_HDV['19_NOx_4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','31.168325930908','0.077415695224656','1484.30189718267','0.926476938270703','27.550406755193','6','68']
  H_HDV['19_PM_4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.691351296272246','4.28307833358501','6.67369856702528','2.98068470061967','-0.046338862402964','6','68']
  H_HDV['19_FC_4_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','972.392925891056','0.09087976924667','94749.3308839643','1.08143434760806','538.158500486399','6','68']
  H_HDV['19_CO_4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.015019884148286','0.003612197935506','-0.000028249729321','0','0','6','62']
  H_HDV['19_THC_4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.37062340267285','0.035486882644192','21.8016970041512','0.229094621721191','0.513139871656299','6','62']
  H_HDV['19_NOx_4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','34.1958202856444','0.081035495684954','32606.0297955802','1.49308647589459','32.5938708539329','6','62']
  H_HDV['19_PM_4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.049594455868824','0.027803523397119','-0.000250242698261','0','0','6','62']
  H_HDV['19_FC_4_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','997.836386513232','0.095209772356395','776889.12762992','1.46061105780716','646.287856460476','6','62']
  H_HDV['19_CO_6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.8384961679255','0.035310795828142','-3.93857798294833','0.103135091354186','0','6','65']
  H_HDV['19_THC_6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.0150239272452','0.033768770862848','20.588278408479','0.228454175292367','0.350898641559886','6','65']
  H_HDV['19_NOx_6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.4771317350255','0.072737198663457','159.750003342252','0.481271920605681','28.6009802006565','6','65']
  H_HDV['19_PM_6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.71447650170321','4.27979375434232','7.33721521184681','3.3288861795697','-0.059504591315194','6','65']
  H_HDV['19_FC_6_0'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','974.103619039562','0.096365929772139','22871.3063124542','0.828635229955907','557.862109518516','6','65']
  H_HDV['19_CO_6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','50','CO','y=(c+(a*exp(b*x)))','41.0767508069314','-0.11602113246507','8.67797600060779','0','0','6','56']
  H_HDV['19_THC_6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.93761915406426','0.046688385734565','21.6644724792291','0.240192541382849','0.752398630880067','6','56']
  H_HDV['19_NOx_6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','28.8904556222573','0.080827532803105','243.997528472484','0.590351963660185','36.2599187326946','6','56']
  H_HDV['19_PM_6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.22085817239499','4.07037957987079','9.20222574288807','4.13238484409067','-0.074991951857361','6','56']
  H_HDV['19_FC_6_50'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1078.15002363428','0.114338200580828','306935.989475262','1.31685493331289','728.644242562763','6','56']
  H_HDV['19_CO_6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','100','CO','y=(c+(a*exp(b*x)))','36.5236415507354','-0.052974342262678','6.43389899773928','0','0','6','41']
  H_HDV['19_THC_6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.32310558979','-0.034784061886222','-5.63253477550916','0.226348323393009','0','6','41']
  H_HDV['19_NOx_6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','100','NOx','y=(c+(a*exp(b*x)))','63.6492020245181','-0.170005213724342','44.3858197794194','0','0','6','41']
  H_HDV['19_PM_6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','100','PM','y=(c+(a*exp(b*x)))','6.49588454291822','-0.068737850377477','1.14563388438508','0','0','6','41']
  H_HDV['19_FC_6_100'] = [' 19','EURO0','Ubus Std >15-18t 80ties','6','100','FC','y=exp((a+(b/x))+(c*log(x)))','7.07100397061613','2.59896390407882','-0.111847282253301','0','0','6','41']
  H_HDV['20_CO_0_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.23961879222833','0.049113283116061','8.02312899878128','0.200842533707388','0.847954484445879','6','75']
  H_HDV['20_THC_0_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.95997454274884','0.051303996827038','12.7211998629043','0.44152881492687','0.346385715105546','6','75']
  H_HDV['20_NOx_0_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.3601737570596','0.07922410678857','4200.63094498283','1.20385227169628','5.96902911320271','6','75']
  H_HDV['20_PM_0_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.991590062751806','0.052273473914434','1.61125772014489','0.29240540860439','0.18021624098058','6','75']
  H_HDV['20_FC_0_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','650.673913749237','0.073039769602628','240583.451562564','1.27880803911296','172.05016997784','6','75']
  H_HDV['20_CO_0_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','50','CO','y=(1/(a+(b*x)))','0.059027676921308','0.01237945651684','0','0','0','6','75']
  H_HDV['20_THC_0_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.97556227639746','0.052456399614832','12.4509232851955','0.441753379929044','0.344339282281007','6','75']
  H_HDV['20_NOx_0_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.9073040842316','-0.021648519752604','-2.13385494968916','0.088303287735171','0','6','75']
  H_HDV['20_PM_0_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.411713000205326','0.070725550846242','-0.000145182007348','0','0','6','75']
  H_HDV['20_FC_0_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','650.877463193704','0.067531668757562','2948894.4640992','1.71276232066011','189.016313313149','6','75']
  H_HDV['20_CO_0_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','100','CO','y=(1/(a+(b*x)))','0.060811993961557','0.01067797202551','0','0','0','6','75']
  H_HDV['20_THC_0_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.96757057170245','0.0533487960764','12.0713630602996','0.440121645653744','0.349509534053057','6','75']
  H_HDV['20_NOx_0_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.6236848268271','-0.022052888513412','-1.73243015182561','0.077424969635701','0','6','75']
  H_HDV['20_PM_0_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.423618998734953','0.063376084600072','-0.000129405300883','0','0','6','75']
  H_HDV['20_FC_0_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','0','100','FC','y=(c+(a*exp(b*x)))','708.136746483425','-0.067012389172507','208.869438922924','0','0','6','75']
  H_HDV['20_CO_-2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','0','CO','y=(c+(a*exp(b*x)))','7.52583725576026','-0.070038999667292','0.720357980003664','0','0','6','75']
  H_HDV['20_THC_-2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.93296684057292','0.055823347358419','12.2262315775216','0.462769595382993','0.282145412612958','6','75']
  H_HDV['20_NOx_-2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.2421638030327','0.065465111071484','1197316.10825147','2.15970751638384','2.26374412905576','6','75']
  H_HDV['20_PM_-2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.08835504696774','0.055670318624747','1.90751564936843','0.355736609704407','0.144168485156329','6','75']
  H_HDV['20_FC_-2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','0','FC','y=(c+(a*exp(b*x)))','701.360398401143','-0.069218275209249','73.0388574037388','0','0','6','75']
  H_HDV['20_CO_-2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000030137041485','0.005368431166966','-0.324258470697964','7.42377870692292','0','6','75']
  H_HDV['20_THC_-2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.9859441499845','0.054960717250945','12.4590667660547','0.479988894286398','0.225795687869247','6','75']
  H_HDV['20_NOx_-2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000092148263777','0.016038650778717','-0.955620909035171','22.0925258488195','0','6','75']
  H_HDV['20_PM_-2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.35902738427971','0.062545291022454','18.0831851839524','0.929447879660154','0.138020271132925','6','75']
  H_HDV['20_FC_-2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002656315490694','0.473123041946065','-28.800308732261','678.349121161201','0','6','75']
  H_HDV['20_CO_-2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000025956387913','0.004905721896362','-0.315784121125745','7.62451289434515','0','6','75']
  H_HDV['20_THC_-2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.02096423731685','0.053924562329931','12.0460017239124','0.484165031725331','0.181397122675264','6','75']
  H_HDV['20_NOx_-2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.30914833170932','15.3021266057653','8.71285575031266','2.97197806432168','-0.014387252491414','6','75']
  H_HDV['20_PM_-2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','100','PM','y=(c+(a*exp(b*x)))','1.48083868586404','-0.061429927143087','0.118957980875879','0','0','6','75']
  H_HDV['20_FC_-2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-2','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','42.7514699918402','473.274451275089','8.87962783166677','3.02193327674655','-0.014366597415804','6','75']
  H_HDV['20_CO_-4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','0','CO','y=(c+(a*exp(((-1)*b)*x)))','6.85780698797464','0.06523012593533','0.202846082854547','0','0','6','75']
  H_HDV['20_THC_-4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.30502330665196','0.061300707007001','22.0457987371213','0.627877130509793','0.117816439673109','6','75']
  H_HDV['20_NOx_-4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','0','NOx','y=(c+(a*exp(b*x)))','21.3713672084141','-0.069681085922916','0.532838154986386','0','0','6','75']
  H_HDV['20_PM_-4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.478554122076737','0.055609490339724','0.001710055506021','0','0','6','75']
  H_HDV['20_FC_-4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','0','FC','y=(c+(a*exp(b*x)))','644.847058917733','-0.06758174304059','15.3349620184243','0','0','6','75']
  H_HDV['20_CO_-4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','50','CO','y=(c+(a*exp(b*x)))','6.82451142647882','-0.061168968706042','0.115907993959779','0','0','6','75']
  H_HDV['20_THC_-4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.3979832092231','0.065127280390183','33.444809450406','0.719291628587819','0.106619797360664','6','75']
  H_HDV['20_NOx_-4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','50','NOx','y=(c+(a*exp(b*x)))','19.745646757859','-0.063294245970283','0.283583895077846','0','0','6','75']
  H_HDV['20_PM_-4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.647608702682542','0.042716361569964','1.2642135689532','0.113441001283132','0.038065460301179','6','75']
  H_HDV['20_FC_-4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','50','FC','y=(c+(a*exp(b*x)))','599.404867092219','-0.061109359346555','6.96667020513044','0','0','6','75']
  H_HDV['20_CO_-4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','100','CO','y=(c+(a*exp(b*x)))','6.86932997617111','-0.057313181038741','0.04707532206436','0','0','6','75']
  H_HDV['20_THC_-4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.45245745531738','0.067869692644687','46.418516768792','0.786508811168991','0.104716980706505','6','75']
  H_HDV['20_NOx_-4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','100','NOx','y=(c+(a*exp(b*x)))','18.8414843358838','-0.058342654078289','0.097081800694047','0','0','6','75']
  H_HDV['20_PM_-4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.22351583796185','0.020099418394416','1.69177017273945','0.08672836647111','0.010723750411124','6','75']
  H_HDV['20_FC_-4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-4','100','FC','y=(c+(a*exp(b*x)))','570.057823628926','-0.055713035884173','0.139652757942193','0','0','6','75']
  H_HDV['20_CO_-6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','0','CO','y=(c+(a*exp(b*x)))','6.48524984547146','-0.069566543492083','0.085617273321875','0','0','6','70']
  H_HDV['20_THC_-6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.47223239804765','0.070373353054455','50.5931112953801','0.807940978605943','0.088502959846434','6','70']
  H_HDV['20_NOx_-6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','0','NOx','y=(c+(a*exp(b*x)))','19.5631250433237','-0.075282918291443','0.183265062987836','0','0','6','70']
  H_HDV['20_PM_-6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','4.86195459606984','-8.97147032886929','-1.79003072025296','0','0','6','70']
  H_HDV['20_FC_-6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','0','FC','y=(c+(a*exp(b*x)))','575.166506221755','-0.071510882660007','3.0713853086421','0','0','6','70']
  H_HDV['20_CO_-6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','50','CO','y=(c+(a*exp(b*x)))','6.46169061461379','-0.067674107660959','0.056945372566765','0','0','6','70']
  H_HDV['20_THC_-6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.48055393725493','0.073003782923253','59.9422883493408','0.844594657063452','0.088637663068468','6','70']
  H_HDV['20_NOx_-6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','116.456128624803','5.699252235485','-0.072778698967171','0','0','6','70']
  H_HDV['20_PM_-6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.96436316588007','-9.00962004130233','-1.82635500591653','0','0','6','70']
  H_HDV['20_FC_-6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1185.860719734','1.49280749881302','-0.078768209957935','0','0','6','70']
  H_HDV['20_CO_-6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000025467668366','0.004525090174205','-0.270740480449345','5.66198570831786','0','6','70']
  H_HDV['20_THC_-6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.4831286626307','0.074563305504744','56.7576781842367','0.841000762598116','0.088968885400058','6','70']
  H_HDV['20_NOx_-6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','39.5666810553795','1.5328550509016','-0.078752538287025','0','0','6','70']
  H_HDV['20_PM_-6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','5.09436083349305','-9.49018267832199','-1.85736123136228','0','0','6','70']
  H_HDV['20_FC_-6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','715.859647205606','0.684981097453267','-0.085920670477254','0','0','6','70']
  H_HDV['20_CO_2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','0','CO','y=(c+(a*exp(b*x)))','11.5482039151535','-0.084828879114595','1.4437379756707','0','0','6','75']
  H_HDV['20_THC_2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70466928735138','0.042427539885535','8.31761010378392','0.320432608136349','0.341153227111414','6','75']
  H_HDV['20_NOx_2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.57765301656','0.089991016804804','8735.74839355574','1.35370844047709','9.88854159367276','6','75']
  H_HDV['20_PM_2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','0','PM','y=(c+(a*exp(b*x)))','1.4060966996276','-0.073020558978428','0.278833442119713','0','0','6','75']
  H_HDV['20_FC_2_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.003796660491769','2.31705081536484','2034.26684661896','-0.532128964286943','0','6','75']
  H_HDV['20_CO_2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','50','CO','y=(c+(a*exp(b*x)))','12.2010281152168','-0.081145620640721','1.81550337479109','0','0','6','75']
  H_HDV['20_THC_2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.64933470157296','0.044577490487859','8.00688249121842','0.309292420596972','0.389839433896556','6','75']
  H_HDV['20_NOx_2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.5471977068062','0.081125658976925','59945.9384247431','1.67533444187859','11.6669782100297','6','75']
  H_HDV['20_PM_2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','50','PM','y=(c+(a*exp(b*x)))','1.42269224686699','-0.070734247826541','0.33514776145149','0','0','6','75']
  H_HDV['20_FC_2_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.0007858071093','0.000074971853967','-0.000000641176515','0','0','6','75']
  H_HDV['20_CO_2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','100','CO','y=(c+(a*exp(b*x)))','12.6634490433557','-0.075110398581043','2.10110711728799','0','0','6','75']
  H_HDV['20_THC_2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.6531840138904','0.048991992575101','7.92770543642181','0.311213419371519','0.461719724278156','6','75']
  H_HDV['20_NOx_2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.9356929238692','0.072639778740882','757263.815209223','2.10146559432311','13.2695515594137','6','75']
  H_HDV['20_PM_2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006181178438','0.001091297241448','-0.063827301588169','1.674267323105','0','6','75']
  H_HDV['20_FC_2_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1120.69652744902','1.14304527990877','-49.7537022622543','0.062735966083929','0','6','75']
  H_HDV['20_CO_4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','0','CO','y=(c+(a*exp(((-1)*b)*x)))','13.9554611616858','0.099191313505191','2.13667079708494','0','0','6','70']
  H_HDV['20_THC_4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70584975270247','0.046977707672713','7.21954181491178','0.292273397245198','0.441292225696907','6','70']
  H_HDV['20_NOx_4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.7511982978865','0.08814691906936','120.25301935366','0.567033574529341','13.8077027361543','6','70']
  H_HDV['20_PM_4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','0','PM','y=(c+(a*exp(b*x)))','1.45892304052324','-0.082667413620106','0.389464985216814','0','0','6','70']
  H_HDV['20_FC_4_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','738.652906163658','0.087460825872578','298575.250138912','1.34414289674599','390.358385256831','6','70']
  H_HDV['20_CO_4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','50','CO','y=(c+(a*exp(b*x)))','14.2135564003839','-0.086471633740311','2.58355799708295','0','0','6','68']
  H_HDV['20_THC_4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.47129624955939','0.045534077780696','6.88343972270694','0.269612971946086','0.528402432757902','6','68']
  H_HDV['20_NOx_4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4637212372176','0.081021175277454','108.181643969945','0.547826833483371','16.8234836813836','6','68']
  H_HDV['20_PM_4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','50','PM','y=(c+(a*exp(b*x)))','1.41471529764789','-0.081735544399084','0.514818734319179','0','0','6','68']
  H_HDV['20_FC_4_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','728.655067331914','0.083459068546308','1136792.40850168','1.57483333201954','487.801784550692','6','68']
  H_HDV['20_CO_4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000121501024656','0.017558092453744','-0.835764710605143','16.7705422254799','0','6','60']
  H_HDV['20_THC_4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.28246501180436','0.043152583118784','6.96565065346529','0.263850892095379','0.616618498332418','6','60']
  H_HDV['20_NOx_4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','100','NOx','y=((a*(b**x))*(x**c))','74.2218421715473','1.00611229644095','-0.4112454088551','0','0','6','60']
  H_HDV['20_PM_4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','100','PM','y=(c+(a*exp(b*x)))','1.39503860421415','-0.08034278354366','0.635840482711869','0','0','6','60']
  H_HDV['20_FC_4_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','707.154961178198','0.077468755140001','3586334.93268577','1.76884448069684','579.164021331081','6','60']
  H_HDV['20_CO_6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','0','CO','y=(c+(a*exp(b*x)))','15.5133284807005','-0.101606564945963','2.71537668255033','0','0','6','66']
  H_HDV['20_THC_6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.49048549804805','0.044928685289807','6.43095473891702','0.25816446605512','0.5368877525463','6','66']
  H_HDV['20_NOx_6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.5784028967545','0.072500785510927','48.0493151563331','0.310198695996931','17.4755108380645','6','66']
  H_HDV['20_PM_6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.93818116775985','0.003393596781016','-0.116880545093735','0.074979321134706','0','6','66']
  H_HDV['20_FC_6_0'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','739.877483904911','0.092618426239602','83932.1522130998','1.12610013813409','508.406817759065','6','66']
  H_HDV['20_CO_6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.5831429961376','-2.77347709993496','-0.871572011570902','0','0','6','57']
  H_HDV['20_THC_6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','50','THC','y=((a*(x**b))+(c*(x**d)))','0.003422114105209','1.21764612122393','14.0937035678853','-0.853493348485432','0','6','57']
  H_HDV['20_NOx_6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7115150980919','0.046209561251199','41.0323493674714','0.238316847207291','20.9437312672197','6','57']
  H_HDV['20_PM_6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000011404494002','0.001687617719788','-0.079043745756702','1.93038329756341','0','6','57']
  H_HDV['20_FC_6_50'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','790.867961894719','0.10264643555618','2356168.46732477','1.72771390642531','661.652809734658','6','57']
  H_HDV['20_CO_6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','100','CO','y=(c+(a*exp(b*x)))','16.1401504688749','-0.046647258757933','2.85759176084658','0','0','6','45']
  H_HDV['20_THC_6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.85117597584862','-0.008900082945091','-1.75947314993508','0.230723355295853','0','6','45']
  H_HDV['20_NOx_6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','100','NOx','y=(c+(a*exp(b*x)))','40.013080306922','-0.147384873947833','27.3934497931465','0','0','6','45']
  H_HDV['20_PM_6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000417136385946','-0.043993570458043','1.94295209163268','0','0','6','45']
  H_HDV['20_FC_6_100'] = [' 20','EURO1','Ubus Std >15-18t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','730.46569684075','0.086466940651378','165545.486411095','1.2390820860893','761.493641574643','6','45']
  H_HDV['21_CO_0_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.362028137441595','9.65215215264524','3.50024897761349','1.60054808442727','-0.006097601650586','6','75']
  H_HDV['21_THC_0_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.40423158668547','0.056057283000695','11.8591909777209','0.518573715177783','0.238593062377099','6','75']
  H_HDV['21_NOx_0_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.5299292916504','0.076431591048293','1237.41970484428','0.967295461321979','6.31898874596835','6','75']
  H_HDV['21_PM_0_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','0','PM','y=(c+(a*exp(((-1)*b)*x)))','0.506496830598195','0.07168550904565','0.105391643251034','0','0','6','75']
  H_HDV['21_FC_0_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','592.88451106353','0.072884004421961','2373425.40491252','1.69954469749065','169.672884765393','6','75']
  H_HDV['21_CO_0_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.8955256871342','-0.015940585925798','-1.00479006566861','0.100335336466763','0','6','75']
  H_HDV['21_THC_0_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.39402527860679','0.056647380413716','10.9828352155318','0.507803202245263','0.235792239598881','6','75']
  H_HDV['21_NOx_0_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.53723270740573','128.918291648142','-0.749160869317697','0.452362447844676','0.047071766858144','6','75']
  H_HDV['21_PM_0_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','50','PM','y=(c+(a*exp(b*x)))','0.517385292936967','-0.063330834726553','0.114057293662852','0','0','6','75']
  H_HDV['21_FC_0_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','50','FC','y=(c+(a*exp(b*x)))','640.218297661017','-0.071264045677655','189.916479584143','','','6','75']
  H_HDV['21_CO_0_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.77011722509669','-4.30980599089161','-1.10501313580241','0','0','6','75']
  H_HDV['21_THC_0_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.38095890989115','0.056934317117701','10.9444374570999','0.507326369335108','0.234973616817429','6','75']
  H_HDV['21_NOx_0_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.4013486528551','-0.033838202699773','-2.13918441025222','0.084670357890762','0','6','75']
  H_HDV['21_PM_0_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','100','PM','y=(c+(a*exp(b*x)))','0.545675316409801','-0.057222238600868','0.122109232739117','0','0','6','75']
  H_HDV['21_FC_0_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','0','100','FC','y=(c+(a*exp(b*x)))','615.353402494557','-0.062888742876777','204.121628013233','0','0','6','75']
  H_HDV['21_CO_-2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','0','CO','y=(a-(b*exp(((-1)*c)*(x**d))))','0.582075958596811','-5.71157074057287','0.029456229230731','1.24291279381466','0','6','75']
  H_HDV['21_THC_-2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3754863203086','0.06020194211333','17.1248427888782','0.604785100431304','0.201210388806157','6','75']
  H_HDV['21_NOx_-2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.88131505704','0.0657890585259','523742.994454888','2.00366978398977','2.55080472187797','6','75']
  H_HDV['21_PM_-2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.059625672062124','0.649817512632528','-0.432380111777556','-0.51180399792915','0.119261419384344','6','75']
  H_HDV['21_FC_-2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','0','FC','y=(c+(a*exp(b*x)))','616.930619551301','-0.065643689026435','69.335525771826','0','0','6','75']
  H_HDV['21_CO_-2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.67990210954976','0.058398601531147','2.14149610811762','0.059410222549672','0.359083261602321','6','75']
  H_HDV['21_THC_-2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.35908204815362','0.05769700656059','17.4219738428047','0.609054518879638','0.166050908027112','6','75']
  H_HDV['21_NOx_-2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000107021204693','0.018248964290422','-1.0638782696342','24.1437065560288','0','6','75']
  H_HDV['21_PM_-2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.447719355696829','0.045161743054788','365.68528470644','151.882166196859','0.027696350180713','6','75']
  H_HDV['21_FC_-2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00205271986407','0.383852262619671','-24.5077730117618','608.9524364712','0','6','75']
  H_HDV['21_CO_-2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.38438472881282','0.05310935346267','1.78771185069036','0.052364860415278','0.248635798596322','6','75']
  H_HDV['21_THC_-2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.35697718422272','0.05611920189214','17.9653998411529','0.61813674703548','0.140280010972081','6','75']
  H_HDV['21_NOx_-2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000085885815279','0.015420338248021','-0.961753133878927','23.3650952986581','0','6','75']
  H_HDV['21_PM_-2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.371164562525979','0.040722897799009','0.111924902842871','0.039962184884884','0.017991567493601','6','75']
  H_HDV['21_FC_-2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','426.743461331943','0.046965151452658','160.884903551091','0.045891208106265','33.0758373555578','6','75']
  H_HDV['21_CO_-4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','0','CO','y=(c+(a*exp(b*x)))','5.51796258864427','-0.061168303998432','0.107438511476548','0','0','6','75']
  H_HDV['21_THC_-4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53492360535413','0.062083936942115','96.2127958180052','0.94760155210271','0.101867526916971','6','75']
  H_HDV['21_NOx_-4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','0','NOx','y=(c+(a*exp(b*x)))','23.3326073884571','-0.070193302250297','0.653576069395067','0','0','6','75']
  H_HDV['21_PM_-4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','0','PM','y=(c+(a*exp(b*x)))','0.404430365065159','-0.055160070094669','0.020915642379549','0','0','6','75']
  H_HDV['21_FC_-4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','0','FC','y=(c+(a*exp(b*x)))','570.350470069615','-0.064310359671452','14.1180788414279','0','0','6','75']
  H_HDV['21_CO_-4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','50','CO','y=(c+(a*exp(b*x)))','5.44397405566373','-0.054995784246522','0.01706978513243','0','0','6','75']
  H_HDV['21_THC_-4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54107436181404','0.06343845475253','164.261048596564','1.04339174979238','0.089174601846178','6','75']
  H_HDV['21_NOx_-4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000099106175705','0.016966485555549','-0.980026029089624','20.1938413888515','0','6','75']
  H_HDV['21_PM_-4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','50','PM','y=(c+(a*exp(b*x)))','0.407272603674313','-0.0497583332331','0.015617447547691','0','0','6','75']
  H_HDV['21_FC_-4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','50','FC','y=(c+(a*exp(b*x)))','533.47315247855','-0.057806555173216','5.45687455496014','0','0','6','75']
  H_HDV['21_CO_-4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','100','CO','y=(c+(a*exp(b*x)))','5.56172892050924','-0.05014918971273','-0.066249296976959','0','0','6','75']
  H_HDV['21_THC_-4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54162814986275','0.065166024574335','228.984512762148','1.10413371938821','0.08738120454805','6','75']
  H_HDV['21_NOx_-4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','100','NOx','y=(c+(a*exp(b*x)))','20.5478388810317','-0.059907717354602','0.203910061965699','0','0','6','75']
  H_HDV['21_PM_-4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','100','PM','y=(c+(a*exp(b*x)))','0.425980785242905','-0.045014696010127','0.00844900477822','0','0','6','75']
  H_HDV['21_FC_-4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-4','100','FC','y=(c+(a*exp(b*x)))','509.8540001529','-0.052494005617704','-1.53603861353366','0','0','6','75']
  H_HDV['21_CO_-6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.13189799169553','4.69557201757846','1.86816109112692','0.324540979113829','0.09042923133872','6','70']
  H_HDV['21_THC_-6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56685706459125','0.06728707350259','411.125741756506','1.21093670104368','0.07860835699146','6','70']
  H_HDV['21_NOx_-6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','21.4630400015045','0.076060469883825','0.248131707991361','0','0','6','70']
  H_HDV['21_PM_-6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','0','PM','y=(c+(a*exp(b*x)))','0.398945737091813','-0.062253669804612','0.022254450772003','0','0','6','70']
  H_HDV['21_FC_-6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','0','FC','y=(c+(a*exp(b*x)))','512.276555156272','-0.068732842537919','2.70687010779817','0','0','6','70']
  H_HDV['21_CO_-6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.352470488955652','3.61494658495105','10.1367144435467','3.71514847902175','-0.054748442220174','6','70']
  H_HDV['21_THC_-6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.55313950544878','0.069204137990104','427.678552481571','1.22131485501039','0.079176365050654','6','70']
  H_HDV['21_NOx_-6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','50','NOx','y=(c+(a*exp(b*x)))','19.8126129870334','-0.07152654193558','0.125941985286292','0','0','6','70']
  H_HDV['21_PM_-6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','50','PM','y=(1/(a+(b*(x**c))))','3.03988856018655','0.011475009955732','1.92884147044994','0','0','6','70']
  H_HDV['21_FC_-6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','913.394508373543','1.18178972542619','-0.078442769707536','0','0','6','70']
  H_HDV['21_CO_-6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.151374205541947','6.01079837731768','-0.53027842389561','-0.715617256788768','0.134319880180953','6','70']
  H_HDV['21_THC_-6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54538039159115','0.070660016070262','383.05398084461','1.20413927562375','0.080260959707229','6','70']
  H_HDV['21_NOx_-6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','61.4112727766515','2.49649071977511','-0.075250830503906','0','0','6','70']
  H_HDV['21_PM_-6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','100','PM','y=(c+(a*exp(b*x)))','0.377127797635645','-0.04880366450741','0.010707530992263','0','0','6','70']
  H_HDV['21_FC_-6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','578.354825561936','0.55157171067943','-0.086361730984237','0','0','6','70']
  H_HDV['21_CO_2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.7064356586611','-0.013813211840491','-1.36397251969718','0.107569875243385','0','6','75']
  H_HDV['21_THC_2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.21430857885572','0.047511211333345','6.07628187371389','0.353442504353299','0.234770194173859','6','75']
  H_HDV['21_NOx_2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.2233144108612','0.08142404499317','317.792421335352','0.725848450337816','10.1026337174384','6','75']
  H_HDV['21_PM_2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','0','PM','y=(c+(a*exp(b*x)))','0.567932107173175','-0.067611831825623','0.151514565903889','0','0','6','75']
  H_HDV['21_FC_2_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00089192319369','0.00009464062902','-0.000000820583572','0','0','6','75']
  H_HDV['21_CO_2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.997471950155866','10.5054491178371','5.65597205455949','2.28262719734976','-0.014464146026699','6','75']
  H_HDV['21_THC_2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.17033960701609','0.048387559829792','5.71811900501316','0.33502874170182','0.252482071083477','6','75']
  H_HDV['21_NOx_2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.1201821846068','0.075355338115617','616.61918354751','0.846150405380975','11.703167628445','6','75']
  H_HDV['21_PM_2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002824831653','0.000479890698648','-0.028042770653066','0.75368976849585','0','6','75']
  H_HDV['21_FC_2_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','951.366920976045','1.23650216790371','-41.5469082613131','0.06050401505308','0','6','75']
  H_HDV['21_CO_2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.827984034102419','12.509439921142','5.25705000492433','2.09735468476897','-0.012200324675965','6','75']
  H_HDV['21_THC_2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.14592609503433','0.051315502315066','5.32301185195461','0.321443706813428','0.287371563391856','6','75']
  H_HDV['21_NOx_2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.4692253060671','0.070582420812691','1896.70119560209','1.05023683416874','13.171014207203','6','75']
  H_HDV['21_PM_2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002628908417','0.000473990951771','-0.02955029837587','0.84149417559661','0','6','75']
  H_HDV['21_FC_2_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002633898359159','0.470708749557766','-27.8620262215534','943.879762234706','0','6','75']
  H_HDV['21_CO_4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','0','CO','y=((a*(x**b))+(c*(x**d)))','-2603306.18390209','-7.7983632183408','62.9417619863669','-0.90753167769808','0','6','70']
  H_HDV['21_THC_4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.20501508332535','0.048909665640811','5.11024106042009','0.311912885942472','0.26897263872701','6','70']
  H_HDV['21_NOx_4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.3047001680032','0.065289443446695','55.5761126726406','0.324616439125418','13.5378567929636','6','70']
  H_HDV['21_PM_4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.910799285757109','0.107734138119892','-0.000648978931949','0','0','6','70']
  H_HDV['21_FC_4_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1894.47503970056','1.00750303822746','-0.495895344838096','0','0','6','70']
  H_HDV['21_CO_4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.826773766916','-0.030306263146021','-2.22977490237384','0.114087518049788','0','6','69']
  H_HDV['21_THC_4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.10482227898159','0.051911149670263','4.72581696569092','0.288272477590525','0.335481241375725','6','69']
  H_HDV['21_NOx_4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.5852074077743','0.057135035255474','53.7300357262804','0.301540150053801','16.0168446630516','6','69']
  H_HDV['21_PM_4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','50','PM','y=(c+(a*exp(((-1)*b)*x)))','0.8327727104674','0.065365040932924','0.212523488775438','0','0','6','69']
  H_HDV['21_FC_4_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','50','FC','y=(c+(a*exp(b*x)))','673.575817504516','-0.085098783505058','487.185509814644','0','0','6','69']
  H_HDV['21_CO_4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.885260470166','-5.03597965203928','-1.28723242935433','0','0','6','67']
  H_HDV['21_THC_4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.950721602723073','0.050323250641795','4.42012264483221','0.265325734076704','0.39415527493647','6','67']
  H_HDV['21_NOx_4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.3126466386345','0.06152538666678','55.6314781046105','0.328548616676513','18.7509522178874','6','67']
  H_HDV['21_PM_4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','100','PM','y=(c+(a*exp(b*x)))','0.980717342658681','-0.064690944985154','0.242279488597042','0','0','6','67']
  H_HDV['21_FC_4_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','4','100','FC','y=(c+(a*exp(b*x)))','643.679147848248','-0.077908065144701','582.122646213759','0','0','6','67']
  H_HDV['21_CO_6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.5211395032134','-0.037057783321959','-2.96617111214421','0.135775532468456','0','6','67']
  H_HDV['21_THC_6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.24872857514779','0.0577151358719','4.76967966602765','0.303340947440765','0.356706984629267','6','67']
  H_HDV['21_NOx_6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.7095818328021','0.04635662386215','54.7464254976396','0.256773385082489','16.4291405298117','6','67']
  H_HDV['21_PM_6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.20122285973191','-0.002060170605454','-0.09324690257843','0.107332977184914','0','6','67']
  H_HDV['21_FC_6_0'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','1930.68168369925','1.00782675510693','-0.439200995046399','0','0','6','67']
  H_HDV['21_CO_6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.45217958019948','19.3221190234653','5.43215950935893','2.38114837410443','-0.022099650413856','6','52']
  H_HDV['21_THC_6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.881608199923349','0.031404405062539','4.47142250895131','0.247437246676514','0.29002064240645','6','52']
  H_HDV['21_NOx_6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4340958113792','0.035247503271145','57.0847223687333','0.248101435625795','18.9001429216051','6','52']
  H_HDV['21_PM_6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','50','PM','y=(c+(a*exp(b*x)))','1.09300092440154','-0.080994033257445','0.292244152914698','0','0','6','52']
  H_HDV['21_FC_6_50'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','556.933289285102','0.074599858608336','508728.58315042','1.42780465938767','634.702426840446','6','52']
  H_HDV['21_CO_6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','100','CO','y=(c+(a*exp(b*x)))','41.0222325891525','-0.106281724208194','2.86530295015662','0','0','6','51']
  H_HDV['21_THC_6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.83240117311159','-0.002176943795989','-0.786431843218482','0.190581645426606','0','6','51']
  H_HDV['21_NOx_6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','82.2269105591596','-0.087713003321137','-9.86186100466258','0.185961285236961','0','6','51']
  H_HDV['21_PM_6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000430354165572','-0.044817700578947','1.49901665522097','0','0','6','51']
  H_HDV['21_FC_6_100'] = [' 21','EURO2','Ubus Std >15-18t Euro-2','6','100','FC','y=((a*(b**x))*(x**c))','2022.45147628092','1.0061354319232','-0.314868849936082','0','0','6','51']
  H_HDV['22_CO_0_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','0','CO','y=exp((a+(b/x))+(c*log(x)))','4.23741053282948','-3.21755476744238','-1.00555742381694','0','0','6','75']
  H_HDV['22_THC_0_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.19134246869021','0.0528216580977','11.3745465181145','0.537100031617386','0.206524738693362','6','75']
  H_HDV['22_NOx_0_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.54687430605752','90.3940734202043','1.38568259113761','1.24862709111776','0.007924457263016','6','75']
  H_HDV['22_PM_0_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.09069157895052','1.15533898832677','-0.009824342153703','0.330919452284246','0.047709549307254','6','75']
  H_HDV['22_FC_0_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','632.67322331661','0.07296520989079','2412224.89645656','1.69350421274969','177.686322899652','6','75']
  H_HDV['22_CO_0_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.49459237978435','-3.87613016307628','-1.04287581210089','0','0','6','75']
  H_HDV['22_THC_0_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.1801818895166','0.053959654622248','10.3079031432216','0.52183677102291','0.203552960707172','6','75']
  H_HDV['22_NOx_0_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.97204458653341','93.4011475168263','1.1663708654914','1.13974993702192','0.01152364219672','6','75']
  H_HDV['22_PM_0_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.453225667665789','0.055584252946669','1250.73523278467','1.74277938177595','0.09414006783905','6','75']
  H_HDV['22_FC_0_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','617.446606071137','0.065915151501999','52222212.2276168','2.21550769693125','193.61077011167','6','75']
  H_HDV['22_CO_0_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.75379891993199','-4.57897071576634','-1.08056631653379','0','0','6','75']
  H_HDV['22_THC_0_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.16117081698038','0.054278804770209','9.78206905003229','0.511100412026058','0.200372096732337','6','75']
  H_HDV['22_NOx_0_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','53.2218919453823','-0.063616523059622','-5.43429873959981','0.123221751261239','0','6','75']
  H_HDV['22_PM_0_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.45830775863948','0.053486309870117','761.287070170832','1.64142591222625','0.096567639543385','6','75']
  H_HDV['22_FC_0_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','0','100','FC','y=(c+(a*exp(b*x)))','649.800794657713','-0.063298110575323','211.355678934085','0','0','6','75']
  H_HDV['22_CO_-2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','0','CO','y=(c+(a*exp(((-1)*b)*x)))','6.74985271754882','0.063856345831773','0.794675242006228','0','0','6','75']
  H_HDV['22_THC_-2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.06181319439236','0.054816882107824','8.86114830847574','0.500367329654092','0.191103071823324','6','75']
  H_HDV['22_NOx_-2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','29.9454875661631','0.080013396260259','3843.35812068661','1.12005761531334','2.21893504379919','6','75']
  H_HDV['22_PM_-2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.418017010365363','0.052638996584006','24.8813975949517','1.08727130327107','0.07629317134435','6','75']
  H_HDV['22_FC_-2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','0','FC','y=(c+(a*exp(b*x)))','661.742570383173','-0.066235901673546','76.781584114886','0','0','6','75']
  H_HDV['22_CO_-2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000024325623019','0.004544874258273','-0.287356687680919','7.05881300041854','0','6','75']
  H_HDV['22_THC_-2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.06074633955548','0.053575859365992','8.11158001112606','0.493954146786595','0.154553945341752','6','75']
  H_HDV['22_NOx_-2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.55204310342571','149.386825793598','-0.655900201448147','0.464092407729896','0.049498137762083','6','75']
  H_HDV['22_PM_-2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.501544406774784','0.000645699488513','-0.021479479299967','0.045971316091905','0','6','75']
  H_HDV['22_FC_-2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00227367807878','0.417282881379692','-26.2469566343349','648.311362216467','0','6','75']
  H_HDV['22_CO_-2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.18363920586074','0.052130236387082','2.04087908056891','0.049876387772436','0.483479146295823','6','75']
  H_HDV['22_THC_-2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.06604122121052','0.051854913442905','7.5909877538341','0.490384207752856','0.120249385751556','6','75']
  H_HDV['22_NOx_-2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','27.9781046669068','0.065859239267408','463238.46211642','1.93073871669317','1.4361475523548','6','75']
  H_HDV['22_PM_-2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001168262544','0.000236594948945','-0.01655608373837','0.477838263947633','0','6','75']
  H_HDV['22_FC_-2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','410.034055106165','0.047181934391811','207.676935844596','0.04682845676844','38.3610769773946','6','75']
  H_HDV['22_CO_-4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','0','CO','y=(c+(a*exp(b*x)))','6.03427079957251','-0.055878902433812','0.25815151781825','0','0','6','75']
  H_HDV['22_THC_-4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.26418419874237','0.058604725548704','18.5342889284453','0.692646756220397','0.082481474676193','6','75']
  H_HDV['22_NOx_-4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.458697878364338','265.570992118958','-1.54916293912212','0.367929609266576','0.059502005458402','6','75']
  H_HDV['22_PM_-4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.402655928668562','0.046770244155027','0.19412075262767','0.243938393746119','0.041844504606886','6','75']
  H_HDV['22_FC_-4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','0','FC','y=(c+(a*exp(b*x)))','615.794364647335','-0.064878506215713','20.118313035408','0','0','6','75']
  H_HDV['22_CO_-4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','50','CO','y=(c+(a*exp(b*x)))','6.09336250849219','-0.05277902114888','0.131916611436807','0','0','6','75']
  H_HDV['22_THC_-4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.30965121697393','0.063897222838507','47.6183047050817','0.876220251567509','0.077226560496218','6','75']
  H_HDV['22_NOx_-4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','50','NOx','y=(c+(a*exp(b*x)))','27.5254496904889','-0.080278228449091','0.566094157055095','0','0','6','75']
  H_HDV['22_PM_-4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','50','PM','y=(c+(a*exp(b*x)))','0.464890015284163','-0.055568865986425','0.045848794969462','0','0','6','75']
  H_HDV['22_FC_-4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','50','FC','y=(c+(a*exp(b*x)))','572.015478025288','-0.058517522644965','11.00421993193','0','0','6','75']
  H_HDV['22_CO_-4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','100','CO','y=(c+(a*exp(b*x)))','6.24153040271789','-0.050194903975527','0.050349021321884','0','0','6','75']
  H_HDV['22_THC_-4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.32025144656085','0.066972839308041','119.775229397579','1.04140173492778','0.074699765188588','6','75']
  H_HDV['22_NOx_-4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','100','NOx','y=(c+(a*exp(b*x)))','25.1520703648756','-0.073798845635195','0.407920819318586','0','0','6','75']
  H_HDV['22_PM_-4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','100','PM','y=(c+(a*exp(b*x)))','0.472442890481521','-0.0553994220742','0.043744475017584','0','0','6','75']
  H_HDV['22_FC_-4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-4','100','FC','y=(c+(a*exp(b*x)))','544.810997874058','-0.053337954114099','3.97732901010639','0','0','6','75']
  H_HDV['22_CO_-6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','0','CO','y=(c+(a*exp(b*x)))','5.73032194413088','-0.059716704394834','0.110396705936349','0','0','6','70']
  H_HDV['22_THC_-6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.043555734844825','8.12296419503924','-0.682741985509479','0.534307869591493','0.040382175333881','6','70']
  H_HDV['22_NOx_-6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','0','NOx','y=(c+(a*exp(b*x)))','26.4271713098009','-0.089556880496427','0.351161135283225','0','0','6','70']
  H_HDV['22_PM_-6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','0','PM','y=(1/(a+(b*(x**c))))','1.78864398135109','0.070127596667239','1.32639937838159','0','0','6','70']
  H_HDV['22_FC_-6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','0','FC','y=(c+(a*exp(b*x)))','561.463125768383','-0.06999496408599','9.12875772632211','0','0','6','70']
  H_HDV['22_CO_-6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','5.48510687836329','-0.056656462842756','0.059318069225798','0','0','6','70']
  H_HDV['22_THC_-6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.044070631090411','8.42465418844715','-0.76826652495001','0.53613689716827','0.041986909484693','6','70']
  H_HDV['22_NOx_-6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','50','NOx','y=(c+(a*exp(b*x)))','23.040201940881','-0.082226988415801','0.215352364637374','0','0','6','70']
  H_HDV['22_PM_-6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','50','PM','y=(1/(a+(b*(x**c))))','1.85055771800999','0.07023833341919','1.3311130809045','0','0','6','70']
  H_HDV['22_FC_-6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002225512893989','0.388508391938925','-22.8226497217627','471.201974077615','0','6','70']
  H_HDV['22_CO_-6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','5.15331617528866','-0.0506523512053','-0.023891334803796','0','0','6','70']
  H_HDV['22_THC_-6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045572059914247','8.42311417352279','-0.787061677999654','0.53906975332662','0.043449621883761','6','70']
  H_HDV['22_NOx_-6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','100','NOx','y=(c+(a*exp(b*x)))','20.9948269439608','-0.076515112711357','0.115131117623063','0','0','6','70']
  H_HDV['22_PM_-6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','100','PM','y=(c+(a*exp(b*x)))','0.464556920549242','-0.061094679362217','0.044457184598422','0','0','6','70']
  H_HDV['22_FC_-6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','-6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001669360328751','0.318242898147074','-20.1463944510838','441.802411574225','0','6','70']
  H_HDV['22_CO_2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.6503772722157','-0.015397633580256','-1.01336735289835','0.095044275624395','0','6','75']
  H_HDV['22_THC_2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.03442131021549','0.044746524781074','5.32310522495737','0.362630823552106','0.196709962629171','6','75']
  H_HDV['22_NOx_2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.27276935081484','70.9576028423031','2.3280511221278','1.52906564215941','0.000051843409204','6','75']
  H_HDV['22_PM_2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.33411760313937','0.139343632391112','-0.000494155607073','0','0','6','75']
  H_HDV['22_FC_2_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000142338823278','3.06981128826347','1748.16726703515','-0.476710559389204','0','6','75']
  H_HDV['22_CO_2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.9887507138955','-0.013253116038981','-1.03002687754834','0.086029078281934','0','6','75']
  H_HDV['22_THC_2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.966987153023761','0.044635115225534','4.82981733538882','0.331677693681128','0.207028010428413','6','75']
  H_HDV['22_NOx_2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.25922800089106','62.8030003231222','2.67763009000188','1.58238587973801','-0.004131774695457','6','75']
  H_HDV['22_PM_2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.424018173481036','0.049923752784653','0.211622112551754','0.191042869590884','0.114572801647259','6','75']
  H_HDV['22_FC_2_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000892259504951','0.000067951042443','-0.00000057377873','0','0','6','75']
  H_HDV['22_CO_2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.47332776046651','-5.80225191152458','-1.21794642433035','0','0','6','75']
  H_HDV['22_THC_2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.910344586032494','0.044873930591554','4.58050950797656','0.313067926111536','0.224295742777946','6','75']
  H_HDV['22_NOx_2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','9.21788222405727','70.582283223206','2.08452467697406','1.30272993506606','0.015377596428189','6','75']
  H_HDV['22_PM_2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','100','PM','y=(c+(a*exp(b*x)))','0.552956225997583','-0.061040460049519','0.136770260413553','0','0','6','75']
  H_HDV['22_FC_2_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00285939198855','0.50261865236795','-29.4292931836076','981.236288613753','0','6','75']
  H_HDV['22_CO_4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.1798873383479','-0.020111200854016','-1.47179749436987','0.107343877547805','0','6','70']
  H_HDV['22_THC_4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.0314192135611','0.046131237209639','4.26606089342699','0.314522453851192','0.221980259151606','6','70']
  H_HDV['22_NOx_4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.86162619129','0.0481111578959','67.5217833900183','0.202070645224495','9.72057256075151','6','70']
  H_HDV['22_PM_4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.09400748459404','0.139480088454559','-0.000720660239893','0','0','6','70']
  H_HDV['22_FC_4_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','1977.59938887266','1.00720688742415','-0.494659009285478','0','0','6','70']
  H_HDV['22_CO_4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.32826148477218','-4.61943979492628','-1.1819525086614','0','0','6','69']
  H_HDV['22_THC_4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.875972054808404','0.042872945911869','3.87149762490198','0.272377020051101','0.247431812705828','6','69']
  H_HDV['22_NOx_4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.6466556316586','0.059558754048065','66.2019050259733','0.210705842810164','12.2580381709874','6','69']
  H_HDV['22_PM_4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.942365342516955','0.133605591961064','-0.000877444810066','0','0','6','69']
  H_HDV['22_FC_4_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','624.671830045944','0.075762229867029','5131522.37918806','1.82830004023025','491.895837680573','6','69']
  H_HDV['22_CO_4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.91542501917014','-5.87446872204471','-1.30272993530863','0','0','6','68']
  H_HDV['22_THC_4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.775094798086004','0.041535314816521','3.89993423776572','0.261607594471461','0.28706913316845','6','68']
  H_HDV['22_NOx_4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','13.2646778006964','55.3607863868855','3.89966593270766','1.99176726618191','-0.013840196562809','6','68']
  H_HDV['22_PM_4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','100','PM','y=(c+(a*exp(b*x)))','0.677941735586948','-0.080703826389684','0.210230695613724','0','0','6','68']
  H_HDV['22_FC_4_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','606.623034769115','0.07127170141586','12837983.0190229','1.98212286051456','587.079491428517','6','68']
  H_HDV['22_CO_6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.2336659235273','-0.029785678105328','-2.02099163670363','0.117735059030234','0','6','69']
  H_HDV['22_THC_6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.985035177253288','0.047552264779202','3.76426838913768','0.281185605272879','0.265341020311655','6','69']
  H_HDV['22_NOx_6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','11.9345708625513','52.9831601490423','4.21437824241568','2.16522905088866','-0.017455074790033','6','69']
  H_HDV['22_PM_6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.844535485305825','0.14497666523702','-0.001085289829333','0','0','6','69']
  H_HDV['22_FC_6_0'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','631.74594545112','0.084373972845839','209236.04497147','1.27953583833602','513.909734982488','6','69']
  H_HDV['22_CO_6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.79203761598356','-5.0596310085524','-1.27534674064767','0','0','6','58']
  H_HDV['22_THC_6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.730330752735567','0.028513326772786','3.61845216278164','0.233389330652429','0.224583622500354','6','58']
  H_HDV['22_NOx_6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','15.2367991865126','50.1012027820731','4.96464988557076','2.42949634872559','-0.024317859434911','6','58']
  H_HDV['22_PM_6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.06073196990292','-0.001359274292828','-0.090344739562596','0.117133839912789','0','6','58']
  H_HDV['22_FC_6_50'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','587.975413345183','0.076558795929843','166821.460820855','1.23227114886731','648.819276179379','6','58']
  H_HDV['22_CO_6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','100','CO','y=(c+(a*exp(b*x)))','29.1550624735533','-0.076017444835474','1.84542072506594','0','0','6','52']
  H_HDV['22_THC_6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.1074758961889','-0.002884581217889','-0.658132024010098','0.188046433118468','0','6','52']
  H_HDV['22_NOx_6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','107.209069522955','-0.161869993065063','-14.3074744862273','0.176496347069259','0','6','52']
  H_HDV['22_PM_6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','100','PM','y=(c+(a*exp(b*x)))','0.752310485442242','-0.063157750769222','0.271267749724455','0','0','6','52']
  H_HDV['22_FC_6_100'] = [' 22','EURO3','Ubus Std >15-18t Euro-3','6','100','FC','y=((a*(b**x))*(x**c))','2172.22785551665','1.0069352529174','-0.338911229849263','0','0','6','52']
  H_HDV['23_CO_0_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.539358657891781','0.055399841714264','0.960814646100464','0.271174878041271','0.066487320248753','6','75']
  H_HDV['23_THC_0_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068281716002359','0.054110291638184','0.525317590845503','0.495059625678048','0.009976487810039','6','75']
  H_HDV['23_NOx_0_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.66670128489664','72.6607351495088','0.106876038570422','0.869380547058384','0.030595448740913','6','75']
  H_HDV['23_PM_0_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.132607928709249','0.061406378235892','0.422292740797942','0.311096487827601','0.016305541661483','6','75']
  H_HDV['23_FC_0_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','619.874531606693','0.073859590400614','577120.167927959','1.44945479107595','167.43963322109','6','75']
  H_HDV['23_CO_0_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.6036094751893','0.055585043780794','0.757440920028454','0.236884670740502','0.07034141230147','6','75']
  H_HDV['23_THC_0_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067340970289578','0.055134497167','0.475556106350036','0.47633293826866','0.009710698503014','6','75']
  H_HDV['23_NOx_0_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.2160304631022','-0.027118762387465','-2.22459286826097','0.114349812592948','0','6','75']
  H_HDV['23_PM_0_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.13512990542155','0.060743755894854','0.405570960885754','0.304358596146081','0.016397518218286','6','75']
  H_HDV['23_FC_0_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','611.197341831563','0.067602987498704','8033392.96527131','1.90035407203748','182.662733446995','6','75']
  H_HDV['23_CO_0_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.68402180543096','-5.07780166067874','-1.20949090678332','0','0','6','75']
  H_HDV['23_THC_0_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.06622308289666','0.055666306617366','0.455846044556623','0.465214091078307','0.009515168623645','6','75']
  H_HDV['23_NOx_0_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.4569954879471','-0.029920928667866','-1.93181884553205','0.105814105189473','0','6','75']
  H_HDV['23_PM_0_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.13823414649497','0.059948262045125','0.390748115929636','0.298567301059694','0.016242046968769','6','75']
  H_HDV['23_FC_0_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','0','100','FC','y=(c+(a*exp(((-1)*b)*x)))','654.247447064164','0.066258933455573','200.203791906812','0','0','6','75']
  H_HDV['23_CO_-2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053046657602145','2.06898157125965','-0.412533599217494','0.376006926303642','0.050161516188978','6','75']
  H_HDV['23_THC_-2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.060427107296112','0.056197824042215','0.401975693246984','0.454832622914967','0.01009630588983','6','75']
  H_HDV['23_NOx_-2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.13208743095857','65.2229507908099','-0.538735331025366','0.432456616825668','0.054054496978289','6','75']
  H_HDV['23_PM_-2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.141161103361295','0.067169526024262','0.467566589034143','0.369577245033826','0.012740419665457','6','75']
  H_HDV['23_FC_-2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','0','FC','y=(c+(a*exp(b*x)))','657.26135473178','-0.068729645691577','72.8623518239481','0','0','6','75']
  H_HDV['23_CO_-2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','50','CO','y=(c+(a*exp(b*x)))','0.611137387769102','-0.063983384634072','0.046406355104648','0','0','6','75']
  H_HDV['23_THC_-2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.060364408038397','0.055606182301894','0.373226429122377','0.449200973511923','0.008331367815139','6','75']
  H_HDV['23_NOx_-2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','50','NOx','y=(c+(a*exp(b*x)))','15.5763738837831','-0.070686535312604','1.05632350055377','0','0','6','75']
  H_HDV['23_PM_-2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.151487720587439','0.06857466258294','0.509105273607406','0.409833234666626','0.011263717364552','6','75']
  H_HDV['23_FC_-2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002433482957387','0.435778627563691','-26.6695604569103','636.011232517913','0','6','75']
  H_HDV['23_CO_-2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002260756228','0.000416906310395','-0.026119255651834','0.609623904979464','0','6','75']
  H_HDV['23_THC_-2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.06040040657177','0.053780344496257','0.34659304805761','0.441655559810437','0.006510595074935','6','75']
  H_HDV['23_NOx_-2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.4807933393669','0.010715698314617','-0.873970484723995','0.05738752744106','0','6','75']
  H_HDV['23_PM_-2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.159461053404058','0.068384025909704','0.608130534950485','0.461167670544267','0.009602138433471','6','75']
  H_HDV['23_FC_-2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','388.757744160942','0.049650958648366','219.912272992481','0.049535265163896','37.4211582897642','6','75']
  H_HDV['23_CO_-4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.020776702929708','0.633558769850945','1.08243501219437','0.317982257902028','0.066910785054039','6','75']
  H_HDV['23_THC_-4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.071625356794226','0.058830531105723','0.699761726535963','0.61119553833357','0.004251807308755','6','75']
  H_HDV['23_NOx_-4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','14.9218938888212','0.079292871898002','0.331055563196426','0','0','6','75']
  H_HDV['23_PM_-4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.150451372010355','0.068915089113664','0.404400711089555','0.388533135854947','0.005691429115331','6','75']
  H_HDV['23_FC_-4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','0','FC','y=(c+(a*exp(b*x)))','610.353516968444','-0.066909221015365','16.1820944762867','0','0','6','75']
  H_HDV['23_CO_-4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.020719215496849','0.436728778860399','2.18686192171941','0.372259250851252','0.086401606496916','6','75']
  H_HDV['23_THC_-4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.074941446251682','0.064156396907239','1.47364098604986','0.762939295738594','0.003837551220083','6','75']
  H_HDV['23_NOx_-4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','50','NOx','y=(c+(a*exp(b*x)))','13.6318900379915','-0.071983679016632','0.194315279280617','0','0','6','75']
  H_HDV['23_PM_-4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.168026134091311','0.074975709691159','0.629914579323668','0.512574452439945','0.005309769818416','6','75']
  H_HDV['23_FC_-4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002345304799655','0.415107424965261','-24.863973753103','531.421030294995','0','6','75']
  H_HDV['23_CO_-4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.0219154233894','0.365520143386133','3.13755637550747','0.462146330486476','0.102566051368095','6','75']
  H_HDV['23_THC_-4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.076100904719419','0.067539666398144','3.2638043075296','0.90588726614231','0.00371321447375','6','75']
  H_HDV['23_NOx_-4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000059166784494','0.010145013629947','-0.583371660128021','11.7367756974163','0','6','75']
  H_HDV['23_PM_-4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.181500436062684','0.079379434977091','2.47530428035638','0.780343643884683','0.005312645107278','6','75']
  H_HDV['23_FC_-4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-4','100','FC','y=(c+(a*exp(b*x)))','536.533669774658','-0.055668553300979','0.914973324843345','0','0','6','75']
  H_HDV['23_CO_-6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001727709825','0.000306926674501','-0.01849880348588','0.396047331843559','0','6','70']
  H_HDV['23_THC_-6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080684172566368','0.070667184720872','8.5591080475726','1.09258511363433','0.003233601443964','6','70']
  H_HDV['23_NOx_-6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','0','NOx','y=(c+(a*exp(b*x)))','13.3724844989962','-0.083079202159758','0.098052268218223','0','0','6','70']
  H_HDV['23_PM_-6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.161854788598059','0.077316379720641','0.425832269186775','0.435648237431264','0.00406968567252','6','70']
  H_HDV['23_FC_-6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','0','FC','y=(c+(a*exp(b*x)))','553.672102442669','-0.071719134245904','3.69683504605445','0','0','6','70']
  H_HDV['23_CO_-6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','50','CO','y=(c+(a*exp(b*x)))','0.397261184647737','-0.057671250542544','-0.000150708369772','0','0','6','70']
  H_HDV['23_THC_-6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.07876167898669','0.073268222713461','20.1131348409001','1.23815722622881','0.003148968595055','6','70']
  H_HDV['23_NOx_-6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','50','NOx','y=(c+(a*exp(b*x)))','12.1425164486873','-0.07705069519915','0.025911009730267','0','0','6','70']
  H_HDV['23_PM_-6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.173134911220472','0.082507078086759','1.4205146955694','0.676197751980522','0.004227793382712','6','70']
  H_HDV['23_FC_-6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1295.72239169845','1.81945230562333','-0.076817530519897','0','0','6','70']
  H_HDV['23_CO_-6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012042011790438','0.287642752825521','3.90613782790501','0.845852373480836','0.083165966432701','6','70']
  H_HDV['23_THC_-6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.0020303079646','0.487452598528849','-0.743452467072707','0.557292032524767','0.042886323736816','6','70']
  H_HDV['23_NOx_-6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','29.0885794064298','1.81210475955513','-0.083768821672664','0','0','6','70']
  H_HDV['23_PM_-6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.178487682767927','0.084988123329926','5.59111163888604','0.93035458395516','0.004293343837108','6','70']
  H_HDV['23_FC_-6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','704.094023475751','0.741988998592501','-0.084626748012297','0','0','6','70']
  H_HDV['23_CO_2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','0','CO','y=((a*(x**b))+(c*(x**d)))','-806817.816665963','-8.64719216696124','5.0038295595762','-0.944924231703785','0','6','75']
  H_HDV['23_THC_2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057629945429208','0.044097817743338','0.263623281130378','0.328948422611301','0.008082191529618','6','75']
  H_HDV['23_NOx_2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9297654769999','0.087971661170142','44.1436940531618','0.458748548295301','4.88398786994674','6','75']
  H_HDV['23_PM_2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.091754450625857','0.043093871363225','0.360758501086599','0.213958636304058','0.015997199819406','6','75']
  H_HDV['23_FC_2_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.001047284024366','2.60215692203049','1801.95001877545','-0.505398590994103','0','6','75']
  H_HDV['23_CO_2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.6467529936952','-0.001861044654847','-0.165747694627459','0.116447530748766','0','6','75']
  H_HDV['23_THC_2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.054090342606546','0.044064206994691','0.24812685442194','0.303764185593228','0.008198157235533','6','75']
  H_HDV['23_NOx_2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.245521951302','0.085078008112006','113.845619365428','0.661599281296486','5.81601474792776','6','75']
  H_HDV['23_PM_2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086149854278857','0.037385022969292','0.365551272172693','0.198017348881565','0.01481161006023','6','75']
  H_HDV['23_FC_2_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000879818341992','0.000075317724893','-0.000000642029773','0','0','6','75']
  H_HDV['23_CO_2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.8269924905631','-0.00238526140056','-0.177028466473097','0.112640661112742','0','6','75']
  H_HDV['23_THC_2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.051908858332942','0.045266331952264','0.237954851875982','0.288899732240475','0.008855708385897','6','75']
  H_HDV['23_NOx_2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.7375492368546','0.07939521025973','308.781260562034','0.84581117167211','6.59882799318026','6','75']
  H_HDV['23_PM_2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087443429311736','0.033672680545065','0.380844578588276','0.193193399515643','0.012681737509683','6','75']
  H_HDV['23_FC_2_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','620.925727684392','0.066037649655047','153361044.753083','2.40894513557758','365.389435959886','6','75']
  H_HDV['23_CO_4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.01479509154861','-0.001948314183631','-0.239932558331346','0.136141273653807','0','6','70']
  H_HDV['23_THC_4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057921014282448','0.045397196972946','0.222387856592475','0.289578049344108','0.008235432572744','6','70']
  H_HDV['23_NOx_4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.63904904778085','40.001674923661','1.62355019715586','1.22846304093236','0.026502076946015','6','70']
  H_HDV['23_PM_4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079678799148864','0.030572919173263','0.393831823637556','0.191344623754257','0.013096523222013','6','70']
  H_HDV['23_FC_4_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','655.684673296143','0.083942751038153','360840.580356817','1.38324030617466','375.542153713193','6','70']
  H_HDV['23_CO_4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.35831835470772','-0.003166593896235','-0.270442504253181','0.132485774820993','0','6','70']
  H_HDV['23_THC_4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.051891290298365','0.046033879056154','0.210632372612344','0.261554770990264','0.009662316955123','6','70']
  H_HDV['23_NOx_4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.68239009929081','0.053360849709714','26.2703165650809','0.207811178690894','7.89444149445357','6','70']
  H_HDV['23_PM_4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08791312624267','0.023276404655967','0.426891560359251','0.186401342917793','0.003669723501348','6','70']
  H_HDV['23_FC_4_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','674.604867194751','0.082773988539759','173712.678176676','1.30095513033795','468.045084087535','6','70']
  H_HDV['23_CO_4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.86385126997592','-0.005150692493698','-0.335619015141954','0.136927091162056','0','6','68']
  H_HDV['23_THC_4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.046708944547327','0.046161326120366','0.205228897160075','0.246129619457579','0.011283356471216','6','68']
  H_HDV['23_NOx_4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.70762669704142','0.049608447279253','26.6337991764235','0.20626950102069','9.01120495795501','6','68']
  H_HDV['23_PM_4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.100397812713865','0.025505377327811','0.464180342746042','0.193591013523052','0.00380013622396','6','68']
  H_HDV['23_FC_4_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','4','100','FC','y=((a*(b**x))*(x**c))','1927.27857510292','1.00538045311468','-0.378941825027544','0','0','6','68']
  H_HDV['23_CO_6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.63791114546244','-0.003758974522173','-0.348379238448629','0.152639841917812','0','6','69']
  H_HDV['23_THC_6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057361760614223','0.050048734538289','0.201785793515236','0.265976929174608','0.010231514470736','6','69']
  H_HDV['23_NOx_6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.52897249879233','0.036646109904932','31.1045032362929','0.190621758158488','7.97764053206907','6','69']
  H_HDV['23_PM_6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.495295255303167','-0.000710690294966','-0.071467942958296','0.165718790142461','0','6','69']
  H_HDV['23_FC_6_0'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','649.562668205999','0.088193628426932','69629.5870448283','1.0969088809294','487.089577445346','6','69']
  H_HDV['23_CO_6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.48612558457155','-0.006897786462213','-0.478565788594931','0.160125348950833','0','6','60']
  H_HDV['23_THC_6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.041571352702316','0.036149193381867','0.192736742598766','0.220330154755927','0.009518561669177','6','60']
  H_HDV['23_NOx_6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.95339420528167','0.057886334082087','32.0569812328995','0.224165675497264','10.2348471707313','6','60']
  H_HDV['23_PM_6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.57541693447475','-0.001007148047165','-0.088882347721964','0.178061871845086','0','6','60']
  H_HDV['23_FC_6_50'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','590.088791500959','0.078358240593441','34749.754375574','0.955601149540293','614.339647212111','6','60']
  H_HDV['23_CO_6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','100','CO','y=(c+(a*exp(b*x)))','3.85098966301807','-0.098761717288169','0.242299924374725','0','0','6','53']
  H_HDV['23_THC_6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.224366986845379','-0.000179974436207','-0.03527997897517','0.178743793115465','0','6','53']
  H_HDV['23_NOx_6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5762688855232','0.03041389866884','35.3819955836674','0.217089554282748','9.92287091864861','6','53']
  H_HDV['23_PM_6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','100','PM','y=((a*(b**x))*(x**c))','0.776379921478901','0.977146220752961','-0.503261145938095','0','0','6','53']
  H_HDV['23_FC_6_100'] = [' 23','EURO4','Ubus Std >15-18t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','661.499539426401','0.095013002708951','133088.271767769','1.25648882206526','765.165461354976','6','53']
  H_HDV['24_CO_0_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.543737117014893','0.055218651648846','0.978611412158154','0.271734887372115','0.067179655984062','6','75']
  H_HDV['24_THC_0_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.06971679610804','0.054165814800777','0.54254413243973','0.498222306479142','0.010206033365089','6','75']
  H_HDV['24_NOx_0_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.38414713994535','23.7665698213013','1.85221991737125','1.3091802373883','0.009487185619236','6','75']
  H_HDV['24_PM_0_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.134955563870104','0.061427548208267','0.429937789974737','0.311470579143288','0.016557290295439','6','75']
  H_HDV['24_FC_0_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','638.562802853811','0.073721094483786','470765.055746985','1.40844192083141','170.533173381896','6','75']
  H_HDV['24_CO_0_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.609774046438594','0.055485110072642','0.793899693123864','0.241723735799355','0.070888344206618','6','75']
  H_HDV['24_THC_0_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068841796276254','0.055207122329913','0.493662396402987','0.480011105539151','0.009922603194517','6','75']
  H_HDV['24_NOx_0_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.9967407056745','-0.019906438467619','-1.86169389069881','0.124240440445553','0','6','75']
  H_HDV['24_PM_0_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.137705271459985','0.060788270777467','0.418522292973827','0.30668259207856','0.016625458583108','6','75']
  H_HDV['24_FC_0_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','631.204353578088','0.067753714852095','5907007.37466811','1.84179861686763','185.99666042039','6','75']
  H_HDV['24_CO_0_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.64265000145812','-4.89889087460115','-1.19662376047026','0','0','6','75']
  H_HDV['24_THC_0_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067756693688947','0.055822110409108','0.467480077101','0.467548682999355','0.009733059323706','6','75']
  H_HDV['24_NOx_0_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.477229869475','-0.02196714449229','-1.65789412623257','0.116797720827355','0','6','75']
  H_HDV['24_PM_0_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.140461931227448','0.060055961070682','0.402510336954858','0.300479366403453','0.016554128968508','6','75']
  H_HDV['24_FC_0_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','0','100','FC','y=(c+(a*exp(b*x)))','678.275608952003','-0.066708461927988','203.934230796986','0','0','6','75']
  H_HDV['24_CO_-2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.542401379868759','0.063436165685999','37.964752142976','1.0460332249475','0.056495853393823','6','75']
  H_HDV['24_THC_-2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061620254793016','0.056056420449193','0.412943887395055','0.456144445876398','0.010296144013791','6','75']
  H_HDV['24_NOx_-2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5691724699133','0.079070409248177','3906.59717752978','1.31863100200266','0.742126906015547','6','75']
  H_HDV['24_PM_-2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.143117280744788','0.066976929663063','0.474450659596693','0.367691654160672','0.012961105237528','6','75']
  H_HDV['24_FC_-2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','0','FC','y=(c+(a*exp(b*x)))','681.3638520189','-0.069211064761404','75.0306272208115','0','0','6','75']
  H_HDV['24_CO_-2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','50','CO','y=(c+(a*exp(b*x)))','0.621441462928182','-0.064517133688304','0.047960569435114','0','0','6','75']
  H_HDV['24_THC_-2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061482592921764','0.055434736577552','0.378685809397998','0.448159779675876','0.008508368715297','6','75']
  H_HDV['24_NOx_-2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1983988575088','0.07062919344084','101657.310838129','1.86739411859213','0.603245394315325','6','75']
  H_HDV['24_PM_-2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.153144242385977','0.068314108231561','0.506216763307295','0.403305172764618','0.011470169728408','6','75']
  H_HDV['24_FC_-2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002568811911511','0.457016787813737','-27.7822287148658','657.784279259109','0','6','75']
  H_HDV['24_CO_-2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002283881543','0.000420858131469','-0.026356895396785','0.615627697836092','0','6','75']
  H_HDV['24_THC_-2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061418305355633','0.053575711621949','0.352346373124047','0.440704431413392','0.006646636804678','6','75']
  H_HDV['24_NOx_-2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','100','NOx','y=(c+(a*exp(b*x)))','10.8277222959148','-0.069144447138472','0.552006955030913','0','0','6','75']
  H_HDV['24_PM_-2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.161458772323043','0.068164175235361','0.606186206462892','0.455993991568028','0.009769013789799','6','75']
  H_HDV['24_FC_-2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','395.861135242488','0.050165219561523','233.005771467245','0.050112757733386','39.1177013782472','6','75']
  H_HDV['24_CO_-4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.503223497359541','0.000604019199861','-0.027447714583157','0.052730330924582','0','6','75']
  H_HDV['24_THC_-4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072904821970216','0.058608492020232','0.696730218967772','0.606721533932192','0.004342304587348','6','75']
  H_HDV['24_NOx_-4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.49485103378693','0.07686144803083','90236.1504565511','1.84655155235893','0.167831276418316','6','75']
  H_HDV['24_PM_-4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.152320271699156','0.068600792445588','0.404419387235855','0.383256752684149','0.005796654786239','6','75']
  H_HDV['24_FC_-4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','0','FC','y=(c+(a*exp(((-1)*b)*x)))','633.03124329565','0.067366675964118','17.0512004575898','0','0','6','75']
  H_HDV['24_CO_-4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001754212787','0.000325001226308','-0.020443131557538','0.461766146396506','0','6','75']
  H_HDV['24_THC_-4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.076293654106661','0.063919792880189','1.44676195288712','0.756144392355309','0.003905012082545','6','75']
  H_HDV['24_NOx_-4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','50','NOx','y=(c+(a*exp(b*x)))','9.89618078150236','-0.078052641518547','0.145763495408992','0','0','6','75']
  H_HDV['24_PM_-4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.169913768109044','0.074600805282172','0.608037510764308','0.499856921468833','0.005382585389002','6','75']
  H_HDV['24_FC_-4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002466467574703','0.434401748224187','-25.8870932448225','550.50638661817','0','6','75']
  H_HDV['24_CO_-4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','100','CO','y=(c+(a*exp(b*x)))','0.482684734853656','-0.053766883056933','0.000539394920775','0','0','6','75']
  H_HDV['24_THC_-4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.077491054136189','0.06735526150977','3.16475710319848','0.89703122091976','0.003779587327424','6','75']
  H_HDV['24_NOx_-4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000047349669526','0.007849116034448','-0.433631590038973','8.31164283214697','0','6','75']
  H_HDV['24_PM_-4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.183840206695838','0.079099233716448','2.15168193928157','0.750720088656421','0.005390940399837','6','75']
  H_HDV['24_FC_-4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-4','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','23.5034382686149','397.953828859099','4.59836672426214','1.04572225544447','0.085538039916068','6','75']
  H_HDV['24_CO_-6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001788980589','0.000316144456212','-0.018967391958657','0.404683302729479','0','6','70']
  H_HDV['24_THC_-6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08219251492818','0.070448586737697','8.35382364817555','1.0844061029302','0.003288059539471','6','70']
  H_HDV['24_NOx_-6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.65123216604299','0.082111289364319','2851.7541263868','1.41608572713537','0.096085338447133','6','70']
  H_HDV['24_PM_-6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.163984775968813','0.076999710650765','0.419867917520973','0.427619210263714','0.004129752752002','6','70']
  H_HDV['24_FC_-6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','0','FC','y=(c+(a*exp(b*x)))','575.399158828665','-0.072235358062789','4.13065820374196','0','0','6','70']
  H_HDV['24_CO_-6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001405184367','0.000266488438417','-0.016923408425074','0.375468765160185','0','6','70']
  H_HDV['24_THC_-6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079911747757119','0.072874318498698','14.3204679543039','1.17852537086224','0.003185651835426','6','70']
  H_HDV['24_NOx_-6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','50','NOx','y=(c+(a*exp(b*x)))','8.80519785983783','-0.082463178284443','0.035205478522347','0','0','6','70']
  H_HDV['24_PM_-6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.175501914358079','0.082229273958081','1.27663684227146','0.652301744299548','0.004288613257304','6','70']
  H_HDV['24_FC_-6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1461.66140094213','2.04401669545871','-0.07618688732513','0','0','6','70']
  H_HDV['24_CO_-6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012687235465946','0.320056522263321','2.68002880674515','0.401886509965845','0.096598326620319','6','70']
  H_HDV['24_THC_-6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002054992163196','0.503753865355682','-0.751154079983162','0.560917681464737','0.042571807119101','6','70']
  H_HDV['24_NOx_-6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','59.5415358026577','6.48291960363993','-0.080992256434622','0','0','6','70']
  H_HDV['24_PM_-6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.181046079111548','0.084792970917844','4.59193428265869','0.891084223778984','0.004364482803059','6','70']
  H_HDV['24_FC_-6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','764.93956268542','0.810133413250847','-0.083764470785395','0','0','6','70']
  H_HDV['24_CO_2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','0','CO','y=((a*(x**b))+(c*(x**d)))','-428522.00262637','-8.28700387712747','5.08305825860027','-0.947328097354925','0','6','75']
  H_HDV['24_THC_2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.058809368600313','0.043979671076748','0.269057481975278','0.330033342972696','0.008212553031495','6','75']
  H_HDV['24_NOx_2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.65266560389025','21.207838644256','2.35334250004484','1.42041527552425','0.016897639561641','6','75']
  H_HDV['24_PM_2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094007602405041','0.04342037647838','0.366486041800075','0.214963686662396','0.016272496488966','6','75']
  H_HDV['24_FC_2_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.001196703194359','2.57568345935133','1872.14318764595','-0.510999663389779','0','6','75']
  H_HDV['24_CO_2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.65390525419086','-0.001869465097422','-0.165635980632895','0.11587361338988','0','6','75']
  H_HDV['24_THC_2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055058981145306','0.043922263755879','0.252683628282954','0.303953294886693','0.008329577031681','6','75']
  H_HDV['24_NOx_2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.8237321558138','-0.01805930192555','-2.26392493483722','0.129578093590202','0','6','75']
  H_HDV['24_PM_2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.088946251580151','0.038213037071169','0.370372420212464','0.199435811516823','0.015315041234315','6','75']
  H_HDV['24_FC_2_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000849277399177','0.000074552230423','-0.000000634397307','0','0','6','75']
  H_HDV['24_CO_2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.84193625060262','-0.002373625915886','-0.178994429092047','0.112903239163982','0','6','75']
  H_HDV['24_THC_2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.052902237207737','0.044957634759808','0.244844721679694','0.290698670731455','0.00892620587484','6','75']
  H_HDV['24_NOx_2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.3582099416211','-0.022356927567785','-2.17969141415065','0.125484307813218','0','6','75']
  H_HDV['24_PM_2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.088662968587187','0.033836289848628','0.38634247511175','0.193353710541995','0.013042922032867','6','75']
  H_HDV['24_FC_2_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1050.50225683243','1.02569805711316','-44.421568740956','0.060970595595248','0','6','75']
  H_HDV['24_CO_4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.03868931872125','-0.001944334102423','-0.244583229341551','0.137040336276984','0','6','70']
  H_HDV['24_THC_4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.059486815733779','0.045312666762537','0.228147069814851','0.292396465318031','0.008298386267892','6','70']
  H_HDV['24_NOx_4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.80632793604951','0.026955125946004','21.252601994928','0.171356290000791','3.44286294799347','6','70']
  H_HDV['24_PM_4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080881662129397','0.030729034866009','0.400517109319053','0.191915255519942','0.013425500282803','6','70']
  H_HDV['24_FC_4_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','677.089358800472','0.083769961301348','334402.149061634','1.36426207874787','381.45237739081','6','70']
  H_HDV['24_CO_4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.3608399036638','-0.003051336903181','-0.269997313223513','0.132007991518145','0','6','70']
  H_HDV['24_THC_4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053584646294713','0.046219158365489','0.216143407871894','0.264369025301021','0.009718622842682','6','70']
  H_HDV['24_NOx_4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.30659240605822','0.027591798016547','22.4342680187884','0.18211218704816','3.78449901690834','6','70']
  H_HDV['24_PM_4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087820912087588','0.023939717459549','0.430445569794793','0.185797429904435','0.005153852996857','6','70']
  H_HDV['24_FC_4_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','663.85009311703','0.079996209970588','933281.185010592','1.54156562144914','473.962552446085','6','70']
  H_HDV['24_CO_4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.49076011042544','-4.59217861885664','-1.30687248013564','0','0','6','68']
  H_HDV['24_THC_4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048944362824275','0.047550761419143','0.208261692500251','0.248634827695641','0.011584230520981','6','68']
  H_HDV['24_NOx_4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.66254546956596','0.028110435425099','24.30535905067','0.193244321577057','4.07405927934133','6','68']
  H_HDV['24_PM_4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.101672472508674','0.026682266621606','0.470835198527898','0.195048285700404','0.005431119197757','6','68']
  H_HDV['24_FC_4_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','4','100','FC','y=((a*(b**x))*(x**c))','1996.36569825104','1.00551506458217','-0.385686405425133','0','0','6','68']
  H_HDV['24_CO_6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.63813091836737','-0.00363344181549','-0.346458109596456','0.151587523945159','0','6','70']
  H_HDV['24_THC_6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.05929594118408','0.050200610981625','0.207500471446857','0.26975256981517','0.010303527262904','6','70']
  H_HDV['24_NOx_6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.1604628973264','-0.032964113968045','-4.20050943836409','0.170782197961044','0','6','70']
  H_HDV['24_PM_6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.100090333662255','0.01418694400649','0.441115306392733','0.179812212430413','-0.016429063284143','6','70']
  H_HDV['24_FC_6_0'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','670.044634142542','0.087855646754154','67391.3292538845','1.0852066104755','494.425979219645','6','70']
  H_HDV['24_CO_6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.43226399428074','-0.006728722528302','-0.463953962115213','0.158013935306671','0','6','62']
  H_HDV['24_THC_6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.046323090056506','0.04279310959116','0.196206803303581','0.230609799771171','0.011103292500892','6','62']
  H_HDV['24_NOx_6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.66011703086722','0.02900266064944','28.1419782205777','0.203945863231121','4.59023937848198','6','62']
  H_HDV['24_PM_6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.57281960510123','-0.000969701554472','-0.086740670173164','0.174428482057182','0','6','62']
  H_HDV['24_FC_6_50'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','622.107162034226','0.080102718705251','44488.6630292594','0.996152306600094','624.946514651911','6','62']
  H_HDV['24_CO_6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','100','CO','y=(c+(a*exp(b*x)))','3.43857508249657','-0.084129916813916','0.24071010361467','0','0','6','54']
  H_HDV['24_THC_6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017424991332244','0.157916501129693','3.28841522412238','1.81573525311342','0.008143481303625','6','54']
  H_HDV['24_NOx_6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.5731169373625','0.024244824092893','30.5458072950481','0.222655569457112','3.96261479347871','6','54']
  H_HDV['24_PM_6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.190815620164521','0.028777592121517','0.797567415324392','0.300614409855261','-0.010168620585591','6','54']
  H_HDV['24_FC_6_100'] = [' 24','EURO5','Ubus Std >15-18t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','695.119185602712','0.097050139602136','207214.216502125','1.33568080771911','780.522803959115','6','54']
  H_HDV['25_CO_0_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.7918855528296','0.050569342134994','25.6742704034397','0.21180070750928','2.25377483643513','6','75']
  H_HDV['25_THC_0_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.77627186139254','0.051784805265453','30.9134762517967','0.357332202553898','0.794792897877211','6','75']
  H_HDV['25_NOx_0_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','0','NOx','y=(c+(a*exp(b*x)))','37.2327628224471','-0.073135458270468','12.1104828155978','0','0','6','75']
  H_HDV['25_PM_0_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.32938922307275','0.052979890500167','4.75627547201624','0.223495693321757','0.362121248530057','6','75']
  H_HDV['25_FC_0_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','891.455917057372','0.068074233840673','145960.658290441','1.11468438544042','238.659161369502','6','75']
  H_HDV['25_CO_0_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.6538561017715','0.04478642815765','20.7058695588132','0.151276809303501','2.56783002911681','6','75']
  H_HDV['25_THC_0_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.63246739694877','0.052965179946296','30.25212986688','0.349576658439387','0.756423382484179','6','75']
  H_HDV['25_NOx_0_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','50','NOx','y=(c+(a*exp(b*x)))','34.4112003426204','-0.059810021025861','13.4127282970899','0','0','6','75']
  H_HDV['25_PM_0_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.26080931872057','0.049118070275773','3.98625669665621','0.1799320591407','0.409720689595196','6','75']
  H_HDV['25_FC_0_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','897.360317133994','0.063341980256644','2210881.13486091','1.59061241865661','268.415515106533','6','75']
  H_HDV['25_CO_0_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.1425262933542','-0.043037477118865','-2.47093412288675','0.090937638308568','0','6','75']
  H_HDV['25_THC_0_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.5198135231675','0.053786716828123','29.4082544276443','0.343731884600726','0.725665774309068','6','75']
  H_HDV['25_NOx_0_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000126959963704','0.021970660510615','-1.36369726678043','46.3828788632646','0','6','75']
  H_HDV['25_PM_0_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.79119097107725','0.038255659992945','3.96683674246944','0.135692646872519','0.419165051479928','6','75']
  H_HDV['25_FC_0_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','0','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1321.67282448196','-1.28005313067204','-70.3253446004617','0.073886115726748','0','6','75']
  H_HDV['25_CO_-2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','0','CO','y=(c+(a*exp(b*x)))','21.4131858329207','-0.073586714717301','1.54870785308616','0','0','6','75']
  H_HDV['25_THC_-2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.72910464599162','0.05573653450598','27.1250887609257','0.368188972503552','0.647110479533076','6','75']
  H_HDV['25_NOx_-2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','0','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000116799908407','0.021659908042183','-1.38143440684628','34.847500665476','0','6','75']
  H_HDV['25_PM_-2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.201676551419843','22.9746660866453','-1.20384329940319','0.355889688464055','0.051172996636544','6','75']
  H_HDV['25_FC_-2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','0','FC','y=(c+(a*exp(b*x)))','967.589814125281','-0.06551391921589','87.7860367175585','0','0','6','75']
  H_HDV['25_CO_-2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','50','CO','y=(c+(a*exp(((-1)*b)*x)))','21.1752430627587','0.063685565081029','1.11145094310505','0','0','6','75']
  H_HDV['25_THC_-2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.6351871968257','0.053539097219774','26.6935668257792','0.364837366578084','0.46580041120768','6','75']
  H_HDV['25_NOx_-2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','4.20962099957764','38.593841854793','-0.067287836536262','-0.579732748949324','0.117923685575609','6','75']
  H_HDV['25_PM_-2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','50','PM','y=(c+(a*exp(b*x)))','3.57340513385074','-0.068000605162434','0.198646289890083','0','0','6','75']
  H_HDV['25_FC_-2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003457281743402','0.619483907972856','-38.4797937507091','926.630003596266','0','6','75']
  H_HDV['25_CO_-2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000079431578821','0.014437079123398','-0.896614033950087','20.6575819303799','0','6','75']
  H_HDV['25_THC_-2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.65093974087653','0.053979721037663','27.8145829767437','0.372664405543768','0.383596418569513','6','75']
  H_HDV['25_NOx_-2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.4666223820672','0.034066165642418','7.18446344574142','0.032282417699065','-0.286937340708942','6','75']
  H_HDV['25_PM_-2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000015081834906','0.00262826426192','-0.1556954192046','3.42010556178951','0','6','75']
  H_HDV['25_FC_-2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','571.896806735447','0.046174528015418','324.230486824583','0.046285433314824','35.1220809197942','6','75']
  H_HDV['25_CO_-4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','0','CO','y=(c+(a*exp(b*x)))','19.358313210247','-0.071930298254543','0.466684577314089','0','0','6','75']
  H_HDV['25_THC_-4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.40722463205551','0.067281306977651','65.7818295151498','0.591112207076831','0.33404186957523','6','75']
  H_HDV['25_NOx_-4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','0','NOx','y=(c+(a*exp(b*x)))','29.473853577609','-0.055935571626737','0.199998573555728','0','0','6','75']
  H_HDV['25_PM_-4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','0','PM','y=(c+(a*exp(b*x)))','3.20104864407126','-0.073750091516154','0.109140727673546','0','0','6','75']
  H_HDV['25_FC_-4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00403885110834','0.683646476011973','-39.041766224681','794.77821671646','0','6','75']
  H_HDV['25_CO_-4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000088305257157','0.015030797220771','-0.86038407461568','17.4116457413099','0','6','75']
  H_HDV['25_THC_-4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.44312825055108','0.070165046817448','84.3962264025618','0.63756497239069','0.317537467109151','6','75']
  H_HDV['25_NOx_-4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','50','NOx','y=(c+(a*exp(b*x)))','27.6088420002159','-0.048095027068559','-0.426254411538494','0','0','6','75']
  H_HDV['25_PM_-4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','50','PM','y=(c+(a*exp(b*x)))','3.10191807955716','-0.068861656254426','0.086707753121326','0','0','6','75']
  H_HDV['25_FC_-4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','50','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','31.8930222896777','611.944841753492','3.10666838301504','0.519449773393281','0.100675687255964','6','75']
  H_HDV['25_CO_-4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','100','CO','y=(c+(a*exp(b*x)))','18.616776279124','-0.061312121457393','0.117921606548108','0','0','6','75']
  H_HDV['25_THC_-4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.39115744521851','0.071601918591133','88.0882737381157','0.645314257660194','0.307479718380053','6','75']
  H_HDV['25_NOx_-4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.8487489564263','0.045214611995893','7.47512033515292','2.62780792255752','-0.387565587736228','6','75']
  H_HDV['25_PM_-4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000012988466823','0.002283001006452','-0.134794964679328','2.8338529755169','0','6','75']
  H_HDV['25_FC_-4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-4','100','FC','y=(c+(a*exp(b*x)))','746.335325105394','-0.052512084343358','-6.15758623728569','0','0','6','75']
  H_HDV['25_CO_-6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.7988884417631','0.072134940448156','875412.687132083','2.17665970893683','0.17517106348822','6','70']
  H_HDV['25_THC_-6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.87335539988736','0.074478761311281','189.458270340698','0.80192250739525','0.260730586167152','6','70']
  H_HDV['25_NOx_-6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','0','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','38.4534983864601','0.748376097427463','-0.083680166739788','0','0','6','70']
  H_HDV['25_PM_-6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','0','PM','y=(c+(a*exp(((-1)*b)*x)))','2.98221455809967','0.077224906617436','0.079120371160113','0','0','6','70']
  H_HDV['25_FC_-6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','0','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','3142.17604954185','3.45507825149914','-0.074942228658007','0','0','6','70']
  H_HDV['25_CO_-6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','50','CO','y=(c+(a*exp(b*x)))','17.4138695808295','-0.072402799201779','0.15914918687362','0','0','6','70']
  H_HDV['25_THC_-6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.80971325795824','0.076379272135874','193.25301633316','0.810153846590598','0.260461759291675','6','70']
  H_HDV['25_NOx_-6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','25.4513806745959','0.321773210645388','-0.094920075862916','0','0','6','70']
  H_HDV['25_PM_-6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','50','PM','y=(c+(a*exp(b*x)))','2.84628791738207','-0.072742952816539','0.066059829793522','0','0','6','70']
  H_HDV['25_FC_-6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1130.62464416121','0.891366921093522','-0.084078980807635','0','0','6','70']
  H_HDV['25_CO_-6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','100','CO','y=(a/(1+(b*exp(((-1)*c)*x))))','81.6397014055856','4.05716017609653','-0.07173197671635','0','0','6','70']
  H_HDV['25_THC_-6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.70917898020702','0.077212745869248','160.274647739563','0.776847543125144','0.258710079423176','6','70']
  H_HDV['25_NOx_-6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','21.6844029072133','0.196417164457692','-0.103046387975573','0','0','6','70']
  H_HDV['25_PM_-6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','100','PM','y=(c+(a*exp(b*x)))','2.78992661049869','-0.069308805395061','0.054451968401583','0','0','6','70']
  H_HDV['25_FC_-6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','826.704584892146','0.502886313308898','-0.090773491703459','0','0','6','70']
  H_HDV['25_CO_2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','0','CO','y=(c+(a*exp(b*x)))','33.268259414095','-0.092208619647607','4.24487094193582','0','0','6','75']
  H_HDV['25_THC_2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.47528010455676','0.034306918044693','25.3098702987219','0.26241409034242','0.41267210011758','6','75']
  H_HDV['25_NOx_2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','0','NOx','y=(c+(a*exp(b*x)))','41.0568508826796','-0.083858833703399','21.0198019855199','0','0','6','75']
  H_HDV['25_PM_2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.569745743484105','5.75040356998886','3.20693452577555','1.51604145873665','0.016887549964748','6','75']
  H_HDV['25_FC_2_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','995.919993037013','0.076329372468023','100707.459268923','1.0550106753536','394.376962900305','6','75']
  H_HDV['25_CO_2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','50','CO','y=(c+(a*exp(b*x)))','34.7289206551478','-0.087809313523179','5.61908378595397','0','0','6','74']
  H_HDV['25_THC_2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.12811891473276','0.036160469985901','25.420932283257','0.25438318461968','0.495838490872164','6','74']
  H_HDV['25_NOx_2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','50','NOx','y=(c+(a*exp(b*x)))','38.1496469894813','-0.0707061321706','25.5832922018141','0','0','6','74']
  H_HDV['25_PM_2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','50','PM','y=(c+(a*exp(b*x)))','6.32639788724496','-0.097790117666536','0.883574149872311','0','0','6','74']
  H_HDV['25_FC_2_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','50','FC','y=(c+(a*exp(b*x)))','1178.1356212107','-0.084846727519882','503.905055357196','0','0','6','74']
  H_HDV['25_CO_2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','100','CO','y=(c+(a*exp(b*x)))','37.3371958895893','-0.084999428551896','6.94166368507378','0','0','6','69']
  H_HDV['25_THC_2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.82113573199157','0.033122061441232','25.9505017322447','0.250705402975583','0.420930167833118','6','69']
  H_HDV['25_NOx_2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000168943118288','0.028199624307549','-1.62902325792509','63.6096875924221','0','6','69']
  H_HDV['25_PM_2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','100','PM','y=(c+(a*exp(b*x)))','6.73008377520865','-0.096818928559187','1.08528208479468','0','0','6','69']
  H_HDV['25_FC_2_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','2','100','FC','y=(c+(a*exp(b*x)))','1100.94491410659','-0.076740206605426','594.662371170109','0','0','6','69']
  H_HDV['25_CO_4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','0','CO','y=(c+(a*exp(b*x)))','39.3442603828323','-0.105849786725477','6.5629746314838','0','0','6','68']
  H_HDV['25_THC_4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.27024417010189','0.035362999016312','23.951796390306','0.239297827696082','0.460468511239404','6','68']
  H_HDV['25_NOx_4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','34.266149504603','0.078046107133553','1238.81184599457','0.878572135287261','29.7945393260765','6','68']
  H_HDV['25_PM_4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.761539666197006','4.70274331385802','6.59743763375374','2.94312924451265','-0.043470949649725','6','68']
  H_HDV['25_FC_4_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1075.51899946953','0.091259335091309','92495.8789859882','1.06306235233818','580.719601101231','6','68']
  H_HDV['25_CO_4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','50.0656638093887','0.046232929718761','-3.84927420160145','0.090161007761451','0','6','60']
  H_HDV['25_THC_4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.74113549088227','0.040059993653454','25.3552442173521','0.241819777246916','0.725006008827032','6','60']
  H_HDV['25_NOx_4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','35.5563014985432','0.077363243012685','15075.4631589683','1.33326506058779','38.3242037067548','6','60']
  H_HDV['25_PM_4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.4007877153632','0.006181346056201','-0.902553033498962','0.110514279903631','0','6','60']
  H_HDV['25_FC_4_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','50','FC','y=((a*(b**x))*(x**c))','3380.7665923058','1.01020893608752','-0.503427583880918','0','0','6','60']
  H_HDV['25_CO_4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','78.9258154732304','0.008480584518258','77.3842672817676','0.381807954964966','-44.1927339509814','6','48']
  H_HDV['25_THC_4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','29.7329111424334','-0.030808583135505','-6.13604952871587','0.22518080644613','0','6','48']
  H_HDV['25_NOx_4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','103.753545149533','-0.040614104538485','-7.84516047324459','0.141808424270431','0','6','48']
  H_HDV['25_PM_4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.23104822917324','0.307080589493678','-0.228718406008887','0.009486390100229','0','6','48']
  H_HDV['25_FC_4_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','4','100','FC','y=((a*(b**x))*(x**c))','3372.77021890132','1.00851993779','-0.445889334034652','0','0','6','48']
  H_HDV['25_CO_6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','0','CO','y=(c+(a*exp(b*x)))','44.9390939167494','-0.11680005570456','9.44898603492171','0','0','6','57']
  H_HDV['25_THC_6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.27272327710927','0.046301005850548','23.3716647733767','0.236225150635097','0.824585209917489','6','57']
  H_HDV['25_NOx_6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','31.7881654542383','0.081749487191421','247.250912134807','0.575440320527212','39.1122929897206','6','57']
  H_HDV['25_PM_6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','0','PM','y=(c+(a*exp(b*x)))','8.99662591128817','-0.134372911183965','1.48172937410291','0','0','6','57']
  H_HDV['25_FC_6_0'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1226.57131382901','0.117384829769107','534662.086951547','1.40949787164271','787.680307077471','6','57']
  H_HDV['25_CO_6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','50','CO','y=(c+(a*exp(b*x)))','40.952012799057','-0.060236192548635','10.7367463620089','0','0','6','43']
  H_HDV['25_THC_6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','29.7290093888605','-0.026341890331083','-6.14305840382452','0.225721390723803','0','6','43']
  H_HDV['25_NOx_6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','50','NOx','y=(c+(a*exp(b*x)))','72.6379670567771','-0.181500578540685','52.7069283300463','0','0','6','43']
  H_HDV['25_PM_6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','50','PM','y=(((a*(x**2))+(b*x))+c)','0.002402716477099','-0.229511450266333','7.53592246701235','0','0','6','43']
  H_HDV['25_FC_6_50'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1301.92739662685','0.149968383878951','1096524.05150235','1.53786655321469','1075.21380369185','6','43']
  H_HDV['25_CO_6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','68.4138824539587','9.48977957539786','-2.58885352692598','0.005089923749693','0','6','35']
  H_HDV['25_THC_6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','38.5738423589169','-0.041665254098347','-10.4617088963575','0.293220098350614','0','6','35']
  H_HDV['25_NOx_6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','100','NOx','y=(c+(a*exp(b*x)))','125.102471267064','-0.287654303992112','65.3944947351138','0','0','6','35']
  H_HDV['25_PM_6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','100','PM','y=(((a*(x**2))+(b*x))+c)','0.005187007281226','-0.409939496406327','10.0697599709827','0','0','6','35']
  H_HDV['25_FC_6_100'] = [' 25','EURO0','Ubus Artic >18t 80ties','6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1256.57074128899','-0.902639273941614','0.164452782309795','0','0','6','35']
  H_HDV['26_CO_0_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','0','CO','y=(1/(a+(b*x)))','0.053465503847447','0.011337741767654','0','0','0','6','75']
  H_HDV['26_THC_0_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.15106158654511','0.05196432600174','13.5855972991127','0.439688917882806','0.375652856919407','6','75']
  H_HDV['26_NOx_0_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.3034994374781','-0.024180252205065','-2.40430291309398','0.089934439413817','0','6','75']
  H_HDV['26_PM_0_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.377978399154206','0.064951439417634','-0.000138910592845','0','0','6','75']
  H_HDV['26_FC_0_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','708.39558306694','0.068119589952021','2169619.62775389','1.64569367663543','207.1905804426','6','75']
  H_HDV['26_CO_0_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','50','CO','y=(1/(a+(b*x)))','0.056567647888989','0.009036061269427','0','0','0','6','75']
  H_HDV['26_THC_0_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.115130514761','0.052692544484041','12.5130749945799','0.427367548138493','0.38570309242945','6','75']
  H_HDV['26_NOx_0_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','50','NOx','y=(c+(a*exp(b*x)))','24.546939458132','-0.065927191333881','8.21560740649267','0','0','6','75']
  H_HDV['26_PM_0_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.394091524277129','0.05540930620465','-0.000134412244726','0','0','6','75']
  H_HDV['26_FC_0_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','50','FC','y=(c+(a*exp(b*x)))','762.518580437747','-0.064565538821417','240.163445463422','','','6','75']
  H_HDV['26_CO_0_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.77325895838058','-4.41200826238351','-0.987756110522535','0','0','6','75']
  H_HDV['26_THC_0_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.11665668560311','0.053647154138573','11.8412119245346','0.425115789340309','0.398836337177389','6','75']
  H_HDV['26_NOx_0_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000110311331212','0.017984390236219','-1.03686580923122','31.1739405453351','0','6','75']
  H_HDV['26_PM_0_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.399480063728566','0.047607610214435','-0.00011427846444','0','0','6','75']
  H_HDV['26_FC_0_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','0','100','FC','y=(c+(a*exp(b*x)))','743.658856973186','-0.05534210052961','260.866090487906','0','0','6','75']
  H_HDV['26_CO_-2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000033073617919','0.005874365294536','-0.353848603746859','8.11358842250102','0','6','75']
  H_HDV['26_THC_-2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.15024327465663','0.054646483578897','13.7583912242687','0.478457275778294','0.257216851207068','6','75']
  H_HDV['26_NOx_-2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','0','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000103430817268','0.017885557068597','-1.05742107159474','24.36066270366','0','6','75']
  H_HDV['26_PM_-2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.45155225137881','0.061746666076829','13.7091935289148','0.842108642760525','0.154393563277981','6','75']
  H_HDV['26_FC_-2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002957662754561','0.523400088821521','-31.6225279466261','742.523095262841','0','6','75']
  H_HDV['26_CO_-2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.77469723540669','0.051967302708093','2.73529539006281','0.050605971457859','0.385853676050248','6','75']
  H_HDV['26_THC_-2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.18474972841168','0.052915468726089','12.8786348333826','0.477086458789191','0.191361820735291','6','75']
  H_HDV['26_NOx_-2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0550793920298','0.047723337757368','8.894205278304','0.047617354079117','1.22263145043356','6','75']
  H_HDV['26_PM_-2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','50','PM','y=(c+(a*exp(b*x)))','1.61507230313667','-0.059186206529251','0.12598101297356','0','0','6','75']
  H_HDV['26_FC_-2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','453.075980660821','0.04758910313752','260.067326738077','0.047770363992821','38.1810553121313','6','75']
  H_HDV['26_CO_-2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','100','CO','y=(c+(a*exp(((-1)*b)*x)))','8.94616875453099','0.045459873321384','0.208506339953679','0','0','6','75']
  H_HDV['26_THC_-2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.25847523051577','0.053520573358488','14.698139261513','0.510470416145448','0.160809659708919','6','75']
  H_HDV['26_NOx_-2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.5317897797606','0.039393318664226','6.37457016840866','0.04005897819901','0.400340652569901','6','75']
  H_HDV['26_PM_-2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006552037852','0.001168051354583','-0.071913587296829','1.70434692123712','0','6','75']
  H_HDV['26_FC_-2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','507.704828920793','0.040218534391671','209.134248153078','0.038957177571833','13.6394397626312','6','75']
  H_HDV['26_CO_-4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','0','CO','y=(c+(a*exp(b*x)))','7.44222779222787','-0.060820129965039','0.134620980061499','0','0','6','75']
  H_HDV['26_THC_-4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.59587093169328','0.064137417960261','33.0047866374402','0.697527669096859','0.117590730738128','6','75']
  H_HDV['26_NOx_-4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','0','NOx','y=(c+(a*exp(b*x)))','21.7421372413542','-0.063378010455306','0.347957058719763','0','0','6','75']
  H_HDV['26_PM_-4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.740552456024065','0.042981928788932','1.3269264008274','0.115011897713613','0.042164147262401','6','75']
  H_HDV['26_FC_-4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','0','FC','y=(c+(a*exp(b*x)))','657.363553620246','-0.061375743129049','8.97566238467355','0','0','6','75']
  H_HDV['26_CO_-4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','50','CO','y=(c+(a*exp(b*x)))','7.59146808116687','-0.055345737423431','0.025142778810258','0','0','6','75']
  H_HDV['26_THC_-4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.68129570065935','0.068056146995722','55.1675404852485','0.803151871844467','0.115295865572016','6','75']
  H_HDV['26_NOx_-4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','50','NOx','y=(c+(a*exp(b*x)))','20.3669080866222','-0.056004488078811','0.042019308729383','0','0','6','75']
  H_HDV['26_PM_-4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.215716778848055','0.0120855979866','1.90416391727703','0.082617584280631','-0.021397025446795','6','75']
  H_HDV['26_FC_-4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','50','FC','y=(c+(a*exp(b*x)))','614.091501521656','-0.053502390666032','-2.00162350473124','0','0','6','75']
  H_HDV['26_CO_-4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','100','CO','y=(c+(a*exp(b*x)))','7.65285311716829','-0.049460541354043','-0.099993319736217','0','0','6','75']
  H_HDV['26_THC_-4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.71788061066778','0.070077390013847','72.7948050487079','0.85894515504584','0.113814718943564','6','75']
  H_HDV['26_NOx_-4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','100','NOx','y=(c+(a*exp(((-1)*b)*x)))','19.7074563232508','0.050005861364693','-0.25717435491398','0','0','6','75']
  H_HDV['26_PM_-4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.596284312682242','0.064400335563344','1.41752496139735','0.075704948538295','0.068565181307445','6','75']
  H_HDV['26_FC_-4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-4','100','FC','y=(c+(a*exp(b*x)))','593.644115608732','-0.047320386947061','-12.7596437854238','0','0','6','75']
  H_HDV['26_CO_-6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','0','CO','y=(c+(a*exp(b*x)))','7.1077199719162','-0.067793880168155','0.06844438655618','0','0','6','70']
  H_HDV['26_THC_-6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.70553057833064','0.072459405046167','67.3267642901917','0.848472562154211','0.097565595612297','6','70']
  H_HDV['26_NOx_-6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','0','NOx','y=(c+(a*exp(b*x)))','19.9288820804787','-0.070321326280045','0.082500970558687','0','0','6','70']
  H_HDV['26_PM_-6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.00511037071546','-8.87963408366139','-1.81111570025471','0','0','6','70']
  H_HDV['26_FC_-6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','0','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1453.08691375612','1.75324139504579','-0.077114702760244','0','0','6','70']
  H_HDV['26_CO_-6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.163299931005962','7.22390941873317','1.24380751407501','0.120270661013863','0.094325446423865','6','70']
  H_HDV['26_THC_-6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.66220683793384','0.073633126061485','37.3244560236558','0.753776577436397','0.095784707349138','6','70']
  H_HDV['26_NOx_-6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','35.8551948049111','1.16197955945874','-0.080306245706599','0','0','6','70']
  H_HDV['26_PM_-6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.19391595264838','-9.52140268870942','-1.85528315020135','0','0','6','70']
  H_HDV['26_FC_-6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','696.666271403705','0.554413202423809','-0.087732485874084','0','0','6','70']
  H_HDV['26_CO_-6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.181879465128767','7.28022835671804','0.697395436140615','-0.216563838598949','0.112116572919587','6','70']
  H_HDV['26_THC_-6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.63953332815579','0.074300197808498','31.0937174244113','0.722987872740687','0.094738873354191','6','70']
  H_HDV['26_NOx_-6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','24.7678074558004','0.611090199876052','-0.086568504191889','0','0','6','70']
  H_HDV['26_PM_-6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','5.4608403972704','-10.4663630915578','-1.91608082403507','0','0','6','70']
  H_HDV['26_FC_-6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','539.001382890272','0.30599325482336','-0.09591025496164','0','0','6','70']
  H_HDV['26_CO_2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','0','CO','y=(c+(a*exp(b*x)))','13.2921068181357','-0.081179221018957','1.96809538409268','0','0','6','75']
  H_HDV['26_THC_2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.79654940578312','0.044064666994004','8.73197661200943','0.30778560007665','0.421809241563141','6','75']
  H_HDV['26_NOx_2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','25.913427658668','0.081999442912268','38727.3390582679','1.58759176376438','12.7021421220627','6','75']
  H_HDV['26_PM_2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','0','PM','y=(c+(a*exp(b*x)))','1.54277899002938','-0.070583440602548','0.363603478288313','0','0','6','75']
  H_HDV['26_FC_2_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000719964028531','0.000069887458642','-0.000000598628266','0','0','6','75']
  H_HDV['26_CO_2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.4784198459813','0.007610987226653','-0.980847898946757','0.067911323486977','0','6','74']
  H_HDV['26_THC_2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.80971350635348','0.051082280927766','8.70412531591491','0.31270148571723','0.542232716565211','6','74']
  H_HDV['26_NOx_2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','50','NOx','y=(c+(a*exp(b*x)))','27.2656179751613','-0.075720985505891','15.5814518116588','0','0','6','74']
  H_HDV['26_PM_2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006590566286','0.001180126826832','-0.069677457304922','1.8752569619266','0','6','74']
  H_HDV['26_FC_2_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1240.57440120325','1.44885707217164','-51.9439884272229','0.060028678144864','0','6','74']
  H_HDV['26_CO_2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.8220825596028','0.002909858399098','-1.09121300717946','0.067435255909749','0','6','70']
  H_HDV['26_THC_2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.59708850092242','0.044845305630757','8.52154472010461','0.298900226974415','0.577546108741508','6','70']
  H_HDV['26_NOx_2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000138494177372','0.022316449510225','-1.22802114462115','42.1939722950084','0','6','70']
  H_HDV['26_PM_2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00000775990047','0.00130254452664','-0.073819875850193','2.04267767051547','0','6','70']
  H_HDV['26_FC_2_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003779847139945','0.635147849325054','-36.3515904827737','1262.97340383066','0','6','70']
  H_HDV['26_CO_4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','0','CO','y=(c+(a*exp(b*x)))','15.5301055605159','-0.086210195205776','2.78291926705122','0','0','6','68']
  H_HDV['26_THC_4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.64948584852414','0.047239659122704','7.48983118712114','0.271644308026344','0.584205755863117','6','68']
  H_HDV['26_NOx_4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.6908961298284','0.079429471290858','88.6293481462117','0.484174379473488','18.1530451584119','6','68']
  H_HDV['26_PM_4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000009413762018','0.001479756037194','-0.076645478682638','1.89214097140207','0','6','68']
  H_HDV['26_FC_4_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','798.812390681913','0.084372005111506','1058820.97905794','1.55133532033789','523.896545823536','6','68']
  H_HDV['26_CO_4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.14235693989407','-4.28865647195405','-0.979726605598111','0','0','6','61']
  H_HDV['26_THC_4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','50','THC','y=((a*(b**x))*(x**c))','18.6374641337558','1.02173266047463','-1.03114269745237','0','0','6','61']
  H_HDV['26_NOx_4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','50','NOx','y=((a*(b**x))*(x**c))','80.0091872312162','1.00511569496673','-0.378789046074404','0','0','6','61']
  H_HDV['26_PM_4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000010837159726','0.001674628233565','-0.08254743616192','2.13525998247098','0','6','61']
  H_HDV['26_FC_4_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','50','FC','y=(c+(a*exp(b*x)))','893.423091828132','-0.09491563020291','699.345624668233','0','0','6','61']
  H_HDV['26_CO_4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','100','CO','y=(a/(1+(b*exp(((-1)*c)*x))))','-36922709.8853934','-1647920.36693397','-0.030563567234099','0','0','6','50']
  H_HDV['26_THC_4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.00066257415334','-0.007647545750906','-1.67511358812811','0.216167725210126','0','6','50']
  H_HDV['26_NOx_4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','65.9609272911897','-0.024277519256433','-4.55082058599594','0.126214775689704','0','6','50']
  H_HDV['26_PM_4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000510324893422','-0.051967902764103','2.21925847468693','0','0','6','50']
  H_HDV['26_FC_4_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.014914186447453','1.6251881280054','-61.6377350546427','1667.53857450464','0','6','50']
  H_HDV['26_CO_6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','0','CO','y=exp((a+(b/x))+(c*log(x)))','4.64268321666671','-2.67403539837478','-0.865320617374371','0','0','6','58']
  H_HDV['26_THC_6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','0','THC','y=((a*(b**x))*(x**c))','18.6946288522605','1.02131022299827','-1.02281635196846','0','0','6','58']
  H_HDV['26_NOx_6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4774193846269','0.045161507568871','45.8512464179742','0.23661743198068','22.4971500329499','6','58']
  H_HDV['26_PM_6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000012153647467','0.001825474800316','-0.086199745024418','2.10128511880853','0','6','58']
  H_HDV['26_FC_6_0'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2572.69564710155','1.00921671417231','-0.444531095986765','0','0','6','58']
  H_HDV['26_CO_6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','50','CO','y=((a*(b**x))*(x**c))','21.38743124241','0.965474177410699','0.01794644673129','0','0','6','44']
  H_HDV['26_THC_6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.694535013759','0.004314657159178','-1.90415708085236','0.222996465484354','0','6','44']
  H_HDV['26_NOx_6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','83.7055614147239','-0.061525087184282','-9.03751425507788','0.180319407066119','0','6','44']
  H_HDV['26_PM_6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.00001789064332','-0.000848710330114','-0.018587004315766','2.07699853755771','0','6','44']
  H_HDV['26_FC_6_50'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','900.735839547274','0.116262892680544','2037586.08563794','1.6890505076324','945.307992864523','6','44']
  H_HDV['26_CO_6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','100','CO','y=((a+(b*x))**((-1)/c))','0.839771574433212','0.001952087195527','0.049934825777961','0','0','6','36']
  H_HDV['26_THC_6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','100','THC','y=(c+(a*exp(b*x)))','10.7550152172239','-0.28042914321902','1.36213717114987','0','0','6','36']
  H_HDV['26_NOx_6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','106.615155308684','-0.140648384819619','-15.8188694440433','0.253245834276023','0','6','36']
  H_HDV['26_PM_6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','100','PM','y=(((a*(x**2))+(b*x))+c)','0.001116119175385','-0.084735317793555','2.87049426534003','0','0','6','36']
  H_HDV['26_FC_6_100'] = [' 26','EURO1','Ubus Artic >18t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','977.851114582462','0.124925536787666','10912.0463117754','0.831957957613125','1140.24092693628','6','36']
  H_HDV['27_CO_0_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.9638970133302','-0.017298809511713','-1.08910269845645','0.099805397899093','0','6','75']
  H_HDV['27_THC_0_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51956926962235','0.05610728073749','12.1954634896784','0.508148526741603','0.256729756135469','6','75']
  H_HDV['27_NOx_0_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.20845344445887','165.381066651458','-0.925167100913456','0.45556638967691','0.046572860417429','6','75']
  H_HDV['27_PM_0_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','0','PM','y=(c+(a*exp(b*x)))','0.561064007854469','-0.063314040271489','0.125227893870423','0','0','6','75']
  H_HDV['27_FC_0_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','632.580409577172','0.066172950360267','26151233.2061243','2.09312201317561','204.163095216505','6','75']
  H_HDV['27_CO_0_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.08167756515509','-4.97131328498535','-1.14386350816316','0','0','6','75']
  H_HDV['27_THC_0_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.48650999280894','0.056334611264687','11.2186971720316','0.493082468860923','0.257893155270414','6','75']
  H_HDV['27_NOx_0_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','37.6584792352195','-0.038939368590436','-2.23939118603142','0.082513721601479','0','6','75']
  H_HDV['27_PM_0_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','50','PM','y=(c+(a*exp(b*x)))','0.617661314460034','-0.054927107179329','0.138688539817644','0','0','6','75']
  H_HDV['27_FC_0_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','50','FC','y=(c+(a*exp(b*x)))','660.83814360688','-0.059886702817712','234.615732192584','','','6','75']
  H_HDV['27_CO_0_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.56539346181335','-6.20135785998048','-1.22872901161956','0','0','6','75']
  H_HDV['27_THC_0_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.46539377374332','0.057082751106699','9.73208234361448','0.474949646207559','0.266826104370408','6','75']
  H_HDV['27_NOx_0_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.2992257443607','-0.036729034189552','-1.74478131040377','0.069118350629906','0','6','75']
  H_HDV['27_PM_0_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.96546109756201','-4.80080498299581','-0.881114749823143','0','0','6','75']
  H_HDV['27_FC_0_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','0','100','FC','y=(c+(a*exp(b*x)))','646.047602637001','-0.049789987364202','253.530981681086','0','0','6','75']
  H_HDV['27_CO_-2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000027988567134','0.005150112038707','-0.319085540985248','7.343127518801','0','6','75']
  H_HDV['27_THC_-2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.48070349758531','0.057686289948926','19.3945238778579','0.611022416725676','0.187603708115039','6','75']
  H_HDV['27_NOx_-2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','0','NOx','y=(c+(a*exp(b*x)))','26.1456056174154','-0.061941425180649','2.42303079818707','0','0','6','75']
  H_HDV['27_PM_-2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','0','PM','y=(c+(a*exp(((-1)*b)*x)))','0.490044581609528','0.046012148598086','0.033187024221974','0','0','6','75']
  H_HDV['27_FC_-2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002282001431502','0.42395463436106','-26.8712610863746','666.260522479988','0','6','75']
  H_HDV['27_CO_-2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.58240671907776','0.051457057676514','0.538886974167366','0.051046896474522','0.259160604958199','6','75']
  H_HDV['27_THC_-2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.47756658573265','0.055592867564185','20.0442620014296','0.621307497932409','0.149963822389729','6','75']
  H_HDV['27_NOx_-2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000085511061087','0.015746085532084','-1.0123859053142','25.4637215204988','0','6','75']
  H_HDV['27_PM_-2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','50','PM','y=(c+(a*exp(b*x)))','0.545844029739616','-0.039025802726275','0.017354298436641','0','0','6','75']
  H_HDV['27_FC_-2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','50','FC','y=(c+(a*exp(b*x)))','641.629874672784','-0.043964600136015','32.2882300791558','0','0','6','75']
  H_HDV['27_CO_-2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','100','CO','y=(c+(a*exp(((-1)*b)*x)))','8.66247275445523','0.045144156652402','0.0791243238974','0','0','6','75']
  H_HDV['27_THC_-2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.48900967433038','0.054783849237768','22.7216287063268','0.649599531505464','0.126903360493851','6','75']
  H_HDV['27_NOx_-2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.8967853143024','0.042737714164721','2.83991023928299','0.043540539404609','0.7958310510179','6','75']
  H_HDV['27_PM_-2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','100','PM','y=(c+(a*exp(b*x)))','0.605554672296404','-0.034104605102022','-0.000869555144385','0','0','6','75']
  H_HDV['27_FC_-2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-2','100','FC','y=(c+(a*exp(b*x)))','655.846708998359','-0.035926948217413','2.85597288856649','0','0','6','75']
  H_HDV['27_CO_-4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','0','CO','y=(c+(a*exp(b*x)))','5.93392518919398','-0.054578804958435','0.020443216202322','0','0','6','75']
  H_HDV['27_THC_-4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.673353677555','0.062706083092489','157.698098551023','1.01971104412673','0.098071431959339','6','75']
  H_HDV['27_NOx_-4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','0','NOx','y=(c+(a*exp(b*x)))','23.7728880732347','-0.06439959808333','0.461739228332296','0','0','6','75']
  H_HDV['27_PM_-4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','0','PM','y=(c+(a*exp(b*x)))','0.441819580301213','-0.0493678685253','0.016878565459107','0','0','6','75']
  H_HDV['27_FC_-4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','0','FC','y=(c+(a*exp(b*x)))','584.102140404894','-0.057964719501326','7.20368846155536','0','0','6','75']
  H_HDV['27_CO_-4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','50','CO','y=(c+(a*exp(b*x)))','6.1460600513163','-0.047469108455118','-0.119907453500584','0','0','6','75']
  H_HDV['27_THC_-4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.67726214165038','0.06517532195262','254.626116581867','1.10594895156431','0.095385299369945','6','75']
  H_HDV['27_NOx_-4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','50','NOx','y=(c+(a*exp(b*x)))','22.1619693155202','-0.057685893404075','0.162331867580124','0','0','6','75']
  H_HDV['27_PM_-4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','50','PM','y=(c+(a*exp(b*x)))','0.469695124883448','-0.042074763265328','0.00412310435866','0','0','6','75']
  H_HDV['27_FC_-4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','50','FC','y=(c+(a*exp(b*x)))','549.768937639711','-0.050059797140499','-4.45386281535024','0','0','6','75']
  H_HDV['27_CO_-4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','100','CO','y=(c+(a*exp(b*x)))','6.43704164521387','-0.042396111498167','-0.264439083766416','0','0','6','75']
  H_HDV['27_THC_-4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68075553523853','0.066918882676326','351.50040722848','1.16764352109461','0.095317291054525','6','75']
  H_HDV['27_NOx_-4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','100','NOx','y=(c+(a*exp(b*x)))','21.2414527977145','-0.052117200411117','-0.111669730250906','0','0','6','75']
  H_HDV['27_PM_-4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','100','PM','y=(c+(a*exp(b*x)))','0.501053465952699','-0.036709149922675','-0.011570206564292','0','0','6','75']
  H_HDV['27_FC_-4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-4','100','FC','y=(c+(a*exp(b*x)))','534.349480853002','-0.043761439319282','-16.394151338555','0','0','6','75']
  H_HDV['27_CO_-6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.383491796187906','3.96644799701275','10.0994361276162','3.69987559107473','-0.054266032699095','6','70']
  H_HDV['27_THC_-6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.69076909878289','0.068744460413202','461.74410537169','1.21837845990589','0.086620953419809','6','70']
  H_HDV['27_NOx_-6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','0','NOx','y=(c+(a*exp(b*x)))','21.7518244768761','-0.071372327136603','0.148460060751112','0','0','6','70']
  H_HDV['27_PM_-6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','0','PM','y=(1/(a+(b*(x**c))))','2.76429833116278','0.010856364484986','1.92112511978573','0','0','6','70']
  H_HDV['27_FC_-6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','0','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1081.13894908746','1.33663647358639','-0.07688503104279','0','0','6','70']
  H_HDV['27_CO_-6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0002553130412','0.028393473179997','-8.7114624196103','0.009022475485151','3.40604905308662','6','70']
  H_HDV['27_THC_-6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68344213516107','0.070799640720692','405.043151799937','1.19989711748237','0.088125478058378','6','70']
  H_HDV['27_NOx_-6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','50.6028962952969','1.72699963874734','-0.076842899542714','0','0','6','70']
  H_HDV['27_PM_-6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','50','PM','y=((a+(b*x))**((-1)/c))','1.05178409676108','0.002908707561115','0.059705321199245','0','0','6','70']
  H_HDV['27_FC_-6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','567.642226238818','0.444122803743728','-0.088453389681266','0','0','6','70']
  H_HDV['27_CO_-6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.55442409934073','0.22623132367704','-0.165897658867694','0.009379483617608','0','6','70']
  H_HDV['27_THC_-6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.65259417830149','0.070887687263871','222.002463453698','1.09779135579831','0.087076146514453','6','70']
  H_HDV['27_NOx_-6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','32.0741070848366','0.862959216161421','-0.082543470298397','0','0','6','70']
  H_HDV['27_PM_-6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','100','PM','y=((a+(b*x))**((-1)/c))','1.00082833968805','0.000042266419362','0.000995382170133','0','0','6','70']
  H_HDV['27_FC_-6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','449.540345263492','0.240859389351136','-0.097413164962685','0','0','6','70']
  H_HDV['27_CO_2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.07388837689676','11.6531660703964','5.56044982997591','2.25227903595998','-0.014085442399313','6','75']
  H_HDV['27_THC_2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.28052733154015','0.048114178761905','6.18778195121663','0.333436708372096','0.273749567339325','6','75']
  H_HDV['27_NOx_2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.1415375923867','0.075310299570968','469.993344189729','0.780339900363491','12.7325401213553','6','75']
  H_HDV['27_PM_2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003206858658','0.000540377019854','-0.031271364905491','0.829306707101855','0','6','75']
  H_HDV['27_FC_2_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000859482472493','0.000065091585065','-0.000000553213242','0','0','6','75']
  H_HDV['27_CO_2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.64435294054918','-5.28186424504454','-1.2177566054119','0','0','6','75']
  H_HDV['27_THC_2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.26668855829103','0.053465853674006','5.69050239705054','0.320219778858905','0.336721449116692','6','75']
  H_HDV['27_NOx_2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.9998715685727','0.068968068714274','2265.77802496418','1.06771173275024','15.216551870358','6','75']
  H_HDV['27_PM_2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00000304847723','0.000552565819575','-0.034629720680884','0.985183959787064','0','6','75']
  H_HDV['27_FC_2_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002727734998745','0.494336411932406','-29.725543458399','1057.247827698','0','6','75']
  H_HDV['27_CO_2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','6.36681170846249','-6.81459800923147','-1.3781503069973','0','0','6','72']
  H_HDV['27_THC_2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.13596123480298','0.05149738892287','5.08466119307871','0.293386980827079','0.382881517098712','6','72']
  H_HDV['27_NOx_2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','28.2618052602413','0.064782616571573','112480.777387583','1.75390962890993','17.5737858663957','6','72']
  H_HDV['27_PM_2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004249273684','0.000728938626182','-0.043774877903144','1.18817926217977','0','6','72']
  H_HDV['27_FC_2_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','2','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','552.648415878768','486.429675972123','3.9730891906772','0.81888601123826','0.091702706327633','6','72']
  H_HDV['27_CO_4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.9414025858947','-0.033457466947863','-2.43915994622719','0.114358669231073','0','6','69']
  H_HDV['27_THC_4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.21235017220204','0.05169240475874','5.18259676758706','0.288619538368804','0.362207081033643','6','69']
  H_HDV['27_NOx_4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.7858940411601','0.056565085068171','58.0987898797984','0.295597229329896','17.3078265926364','6','69']
  H_HDV['27_PM_4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','0','PM','y=(c+(a*exp(b*x)))','0.908845399579353','-0.06591621725696','0.231943096138926','0','0','6','69']
  H_HDV['27_FC_4_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','0','FC','y=(c+(a*exp(b*x)))','737.857353892966','-0.085853594350616','523.380841530872','0','0','6','69']
  H_HDV['27_CO_4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','6.38639153262786','-5.93819517533276','-1.37785779884051','0','0','6','62']
  H_HDV['27_THC_4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.866785027093438','0.030219972066427','4.91079269404026','0.244410484322436','0.333733446141274','6','62']
  H_HDV['27_NOx_4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.7855142772374','0.060493255676308','62.6545623957142','0.336665035713162','21.6677854451627','6','62']
  H_HDV['27_PM_4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','50','PM','y=(c+(a*exp(b*x)))','1.17299734784332','-0.065979495838487','0.28218589189814','0','0','6','62']
  H_HDV['27_FC_4_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','50','FC','y=(c+(a*exp(b*x)))','685.661650951311','-0.071305996686362','669.957486412917','0','0','6','62']
  H_HDV['27_CO_4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000360097199682','-0.023705977617051','-0.251006891012467','28.8118005626531','0','6','54']
  H_HDV['27_THC_4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.0774223190513','-0.002357576027709','-0.806981279106879','0.186852907096899','0','6','54']
  H_HDV['27_NOx_4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','77.8950226220227','-0.094424497438478','-7.26685078099855','0.155322358088258','0','6','54']
  H_HDV['27_PM_4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000447671299605','-0.048815754167439','1.68578734305219','0','0','6','54']
  H_HDV['27_FC_4_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','4','100','FC','y=(c+(a*exp(b*x)))','738.403230680038','-0.083696366538635','842.124291121533','0','0','6','54']
  H_HDV['27_CO_6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.17388414016175','22.5322385818243','4.86616447732588','2.14580027966257','-0.01730994515547','6','54']
  H_HDV['27_THC_6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.990530605832838','0.020167015924527','4.68309859724452','0.228300220668044','0.149987382344613','6','54']
  H_HDV['27_NOx_6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6710218219604','0.039069689832093','62.8704050092865','0.251779192656514','20.8714228778619','6','54']
  H_HDV['27_PM_6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.56002393756546','-0.002460836632244','-0.111055841741663','0.096634366975056','0','6','54']
  H_HDV['27_FC_6_0'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','604.483811126513','0.074176967494311','377653.174809328','1.36136781366222','678.475615738552','6','54']
  H_HDV['27_CO_6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','50','CO','y=(c+(a*exp(b*x)))','48.0023597848947','-0.090829067130208','2.93900303583792','0','0','6','48']
  H_HDV['27_THC_6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.59492781654391','-0.003631768828068','-0.98708784929018','0.207153100793745','0','6','48']
  H_HDV['27_NOx_6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','94.6414638936449','-0.079651106163891','-11.7793667618322','0.192687470766728','0','6','48']
  H_HDV['27_PM_6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000514463241644','-0.055262739369069','1.82574439815626','0','0','6','48']
  H_HDV['27_FC_6_50'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','50','FC','y=((a*(b**x))*(x**c))','2220.65434384778','1.00491066912554','-0.288000949713901','0','0','6','48']
  H_HDV['27_CO_6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','100','CO','y=(c+(a*exp(b*x)))','63.638114104016','-0.084749193988343','1.96923737165549','0','0','6','39']
  H_HDV['27_THC_6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.909817309910441','1.6617987360964','9.65654765994212','5.10266423193327','-0.114749620093907','6','39']
  H_HDV['27_NOx_6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','106.329996586241','-0.190949611022879','-14.3219074073427','0.223866214041035','0','6','39']
  H_HDV['27_PM_6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000935009212992','-0.085072509912695','2.40238653282432','0','0','6','39']
  H_HDV['27_FC_6_100'] = [' 27','EURO2','Ubus Artic >18t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','702.561421292366','0.109711416623062','259727.466323445','1.42187382619223','1180.78515017303','6','39']
  H_HDV['28_CO_0_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','0','CO','y=exp((a+(b/x))+(c*log(x)))','4.59301479267527','-3.90989767101744','-1.04496974852484','0','0','6','75']
  H_HDV['28_THC_0_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.29063366082103','0.053354041362531','11.5829205362207','0.524577457080564','0.221684148765779','6','75']
  H_HDV['28_NOx_0_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','4.29092945014977','93.774084686419','1.3942858679503','1.20199094048831','0.008479042619821','6','75']
  H_HDV['28_PM_0_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.100922870223122','1.27741388124283','-0.043393593276228','0.305216955187868','0.047531444619978','6','75']
  H_HDV['28_FC_0_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','0','FC','y=(c+(a*exp(b*x)))','741.066475636723','-0.07214171386815','216.469736420671','0','0','6','75']
  H_HDV['28_CO_0_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.04205829725713','-5.16210048137134','-1.1168053951598','0','0','6','75']
  H_HDV['28_THC_0_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.2586776230628','0.054057169591029','10.2737586924692','0.501389343556654','0.219165196055592','6','75']
  H_HDV['28_NOx_0_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','58.7034536706685','-0.07270085859274','-5.89098749251743','0.122267967278873','0','6','75']
  H_HDV['28_PM_0_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','50','PM','y=(c+(a*exp(b*x)))','0.536012048646535','-0.056185528883417','0.112192729746928','0','0','6','75']
  H_HDV['28_FC_0_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','50','FC','y=(c+(a*exp(b*x)))','697.732944228101','-0.060592372724453','242.116449256914','','','6','75']
  H_HDV['28_CO_0_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.49444486286736','-6.32504681835108','-1.20008300853647','0','0','6','75']
  H_HDV['28_THC_0_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.2266877854582','0.054434299141107','8.7398854454391','0.47687369147136','0.22027625510293','6','75']
  H_HDV['28_NOx_0_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','56.2267824185816','-0.078433764013664','-4.98365463488778','0.111534454753468','0','6','75']
  H_HDV['28_PM_0_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','100','PM','y=(c+(a*exp(((-1)*b)*x)))','0.558095666433584','0.055219998702531','0.121561782037013','0','0','6','75']
  H_HDV['28_FC_0_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','0','100','FC','y=(c+(a*exp(b*x)))','676.132307830432','-0.050747925526096','260.926662955193','0','0','6','75']
  H_HDV['28_CO_-2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000026848247757','0.004997989682647','-0.314861982041131','7.73321576919255','0','6','75']
  H_HDV['28_THC_-2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.1603688707859','0.053657487570485','9.20714495195015','0.499104482305872','0.176509012497468','6','75']
  H_HDV['28_NOx_-2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','32.0163918550665','0.073432113614161','37009.7097115557','1.49375748374857','2.09551316568203','6','75']
  H_HDV['28_PM_-2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.548195426500947','0.000688718495032','-0.023432073859381','0.046115162204017','0','6','75']
  H_HDV['28_FC_-2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002538764431291','0.462500632428286','-28.8469582552931','709.860715983242','0','6','75']
  H_HDV['28_CO_-2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.52325451847069','5.90213424156551','9.38634570562966','3.23401959166176','-0.022659344993212','6','75']
  H_HDV['28_THC_-2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.16339881345826','0.051366962452946','8.56866399256194','0.495915363686733','0.126890791705919','6','75']
  H_HDV['28_NOx_-2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','30.5143739054338','0.063723718888056','986892.039970227','2.04156491820285','1.57111908472945','6','75']
  H_HDV['28_PM_-2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001170351311','0.000245789670043','-0.0177343346979','0.523619142567009','0','6','75']
  H_HDV['28_FC_-2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','50','FC','y=(c+(a*exp(b*x)))','672.56839458748','-0.044517464123698','38.2781621139507','0','0','6','75']
  H_HDV['28_CO_-2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.150855826017381','-0.002592714795529','0.000355206003305','0','0','6','75']
  H_HDV['28_THC_-2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.17339721275271','0.051215806096963','8.967519538397','0.511921974370083','0.102678235462177','6','75']
  H_HDV['28_NOx_-2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000151212632818','0.02545800884878','-1.46295730983071','31.3777396897755','0','6','75']
  H_HDV['28_PM_-2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.076914968945068','0.821797009397067','-0.192289682264275','-0.234706101622172','0.082531260549639','6','75']
  H_HDV['28_FC_-2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-2','100','FC','y=(c+(a*exp(b*x)))','683.085014693379','-0.03665899446429','9.0697992885334','0','0','6','75']
  H_HDV['28_CO_-4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','0','CO','y=(c+(a*exp(b*x)))','6.66014282075613','-0.052355451967157','0.151098461984089','0','0','6','75']
  H_HDV['28_THC_-4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.41205222822767','0.062420704200176','40.6388299427114','0.827905188335201','0.084625606464994','6','75']
  H_HDV['28_NOx_-4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','0','NOx','y=(c+(a*exp(b*x)))','30.5212886392036','-0.080494513830952','0.65655372270637','0','0','6','75']
  H_HDV['28_PM_-4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','0','PM','y=(c+(a*exp(b*x)))','0.506088674580992','-0.054772099675567','0.049982591635035','0','0','6','75']
  H_HDV['28_FC_-4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','0','FC','y=(c+(a*exp(b*x)))','626.047924203106','-0.058655138176195','13.232428278628','0','0','6','75']
  H_HDV['28_CO_-4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','50','CO','y=(c+(a*exp(b*x)))','6.86219745012199','-0.047972387738948','0.008519948677902','0','0','6','75']
  H_HDV['28_THC_-4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.43217741971337','0.066835957084808','143.632657539831','1.05435293863573','0.080950525768652','6','75']
  H_HDV['28_NOx_-4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000134775197246','0.022564279931864','-1.25811075817834','24.4738327598853','0','6','75']
  H_HDV['28_PM_-4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001814884716','0.00033120913813','-0.020956583519667','0.526551224760854','0','6','75']
  H_HDV['28_FC_-4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','50','FC','y=(c+(a*exp(b*x)))','585.513846022972','-0.050972711328661','1.65597786749963','0','0','6','75']
  H_HDV['28_CO_-4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','100','CO','y=(c+(a*exp(b*x)))','7.04119594136366','-0.043403319422514','-0.138480010577085','0','0','6','75']
  H_HDV['28_THC_-4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.42578160114918','0.069019470627302','336.107442131003','1.20603813604198','0.078904045426643','6','75']
  H_HDV['28_NOx_-4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000104311806851','0.018543374601038','-1.10200171268825','22.873835125099','0','6','75']
  H_HDV['28_PM_-4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001576130523','0.000302536841975','-0.020080575724706','0.524561020692906','0','6','75']
  H_HDV['28_FC_-4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-4','100','FC','y=(c+(a*exp(b*x)))','565.447848545625','-0.044681371870912','-10.2105153468222','0','0','6','75']
  H_HDV['28_CO_-6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','0','CO','y=(c+(a*exp(b*x)))','5.97944790522817','-0.056057038996891','0.062059600176714','0','0','6','70']
  H_HDV['28_THC_-6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.47216573003111','0.071876528333977','609.050646065923','1.31619716644271','0.070564469945045','6','70']
  H_HDV['28_NOx_-6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','0','NOx','y=(c+(a*exp(b*x)))','25.5673891502609','-0.082449024396706','0.25245134398778','0','0','6','70']
  H_HDV['28_PM_-6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.444211374114912','0.05457307565155','0.200615998198676','0.223993976027179','0.044679420206883','6','70']
  H_HDV['28_FC_-6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','0','FC','y=(c+(a*exp(b*x)))','560.574584451731','-0.064492193090309','5.3989637887425','0','0','6','70']
  H_HDV['28_CO_-6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','5.61120987857317','-0.048294724508129','-0.067599627002491','0','0','6','70']
  H_HDV['28_THC_-6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.049350733268816','9.24055587058189','-0.78404213930687','0.546347923894467','0.042928158916627','6','70']
  H_HDV['28_NOx_-6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','101.797590725528','3.8012530253564','-0.078198045770804','0','0','6','70']
  H_HDV['28_PM_-6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','50','PM','y=(c+(a*exp(b*x)))','0.503791107223066','-0.059739277123581','0.047587179717138','0','0','6','70']
  H_HDV['28_FC_-6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','50','FC','y=(c+(a*exp(b*x)))','513.927691479763','-0.057383362134906','-1.38253091943647','0','0','6','70']
  H_HDV['28_CO_-6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','5.57113167847748','-0.043249392795865','-0.186958479838291','0','0','6','70']
  H_HDV['28_THC_-6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.42960601961078','0.075889019342361','838.590571710246','1.37823562269012','0.06924851599609','6','70']
  H_HDV['28_NOx_-6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','38.8713903206381','1.09408387849768','-0.085385542670585','0','0','6','70']
  H_HDV['28_PM_-6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','100','PM','y=(c+(a*exp(b*x)))','0.504769198043783','-0.057496103259381','0.044930891205613','0','0','6','70']
  H_HDV['28_FC_-6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','-6','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-28.574223922844','368.683912081885','11.8210487745651','4.26786783557558','-0.062815089855009','6','70']
  H_HDV['28_CO_2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.1991600251086','-0.014149563996647','-1.10953275584437','0.08521052000358','0','6','75']
  H_HDV['28_THC_2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.06411803159687','0.044380568385937','5.35654992017875','0.334203219657885','0.225043941151901','6','75']
  H_HDV['28_NOx_2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','8.65974579721241','85.2857836299668','1.9680603286583','1.35509400463658','0.011903426269575','6','75']
  H_HDV['28_PM_2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.454104571264342','0.048891685865761','0.228520855111533','0.181174568143683','0.123922401486636','6','75']
  H_HDV['28_FC_2_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000817611381233','0.000063485986953','-0.000000537542504','0','0','6','75']
  H_HDV['28_CO_2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','50','CO','y=exp((a+(b/x))+(c*log(x)))','5.8217163346155','-6.43484315504251','-1.26998311612094','0','0','6','75']
  H_HDV['28_THC_2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.972178661548538','0.044758782809999','4.92958134241318','0.30613445963704','0.255620521572784','6','75']
  H_HDV['28_NOx_2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.8502404539891','0.061757897070774','55.9773567434963','0.219243859256196','11.3738467381638','6','75']
  H_HDV['28_PM_2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','50','PM','y=(c+(a*exp(b*x)))','0.61865502950686','-0.061311516072208','0.159024588336351','0','0','6','75']
  H_HDV['28_FC_2_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00300227971876','0.531634183486859','-31.4685258233445','1095.36607348788','0','6','75']
  H_HDV['28_CO_2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','6.45201871319514','-7.67793080458279','-1.4062776359544','0','0','6','73']
  H_HDV['28_THC_2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.869520366643648','0.040823413333067','4.38638749328413','0.276490676514634','0.27214817137782','6','73']
  H_HDV['28_NOx_2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','33.1598842729538','0.069939971216114','50.6420230001388','0.252524769319659','13.6933280055964','6','73']
  H_HDV['28_PM_2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','100','PM','y=(c+(a*exp(((-1)*b)*x)))','0.673540037686972','0.06225750147114','0.188112410607025','0','0','6','73']
  H_HDV['28_FC_2_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','2','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','555.201755775608','578.037965635299','2.70548687981541','0.479831144250353','0.090275137317735','6','73']
  H_HDV['28_CO_4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','0','CO','y=exp((a+(b/x))+(c*log(x)))','5.40290144034007','-4.58957444956398','-1.1787420015099','0','0','6','70']
  H_HDV['28_THC_4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.992522742702787','0.044311213169257','4.37772784381297','0.281521639148626','0.272966189831618','6','70']
  H_HDV['28_NOx_4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.6824792964839','0.056157981552583','73.6788970827761','0.20513691927615','13.1692024163205','6','70']
  H_HDV['28_PM_4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.862994509740421','0.123060093313782','-0.000803329760985','0','0','6','70']
  H_HDV['28_FC_4_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','2071.28275820778','1.00579770453471','-0.415864128991904','0','0','6','70']
  H_HDV['28_CO_4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','6.23130542881407','-6.22659493988218','-1.34182862155334','0','0','6','66']
  H_HDV['28_THC_4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.786795340392088','0.0319199581311','4.22762692163138','0.248396282437313','0.269105460079712','6','66']
  H_HDV['28_NOx_4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','15.742493777629','57.5052372379481','4.33697053563442','2.15369025417704','-0.019244214751146','6','66']
  H_HDV['28_PM_4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.06723743483162','-0.001409849241159','-0.075143247962874','0.09906442765907','0','6','66']
  H_HDV['28_FC_4_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','648.796689952394','0.066407450871513','8298719.36819933','1.89927544975769','675.788100936585','6','66']
  H_HDV['28_CO_4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000264050748158','-0.017616351986989','-0.235230707803285','24.6400756088813','0','6','55']
  H_HDV['28_THC_4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.466176631679152','2.6250628744114','4.13070963060002','2.30630079834531','-0.022135427652863','6','55']
  H_HDV['28_NOx_4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','109.490062173149','-0.166398639322979','-13.4436095264861','0.16500752633261','0','6','55']
  H_HDV['28_PM_4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','100','PM','y=(1/(a+(b*x)))','0.890517951796823','0.045640869875269','0','0','0','6','55']
  H_HDV['28_FC_4_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.009482748968958','1.1696582844779','-48.8234588786901','1573.59333987024','0','6','55']
  H_HDV['28_CO_6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','0','CO','y=exp((a+(b/x))+(c*log(x)))','5.90840028580786','-5.17417392431019','-1.2813304200046','0','0','6','60']
  H_HDV['28_THC_6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.807970291219185','0.029159249097966','3.95894410651986','0.234126230044056','0.246432151251121','6','60']
  H_HDV['28_NOx_6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','16.4563229462328','54.3651049961442','5.06162057316982','2.47304704193336','-0.025633065537472','6','60']
  H_HDV['28_PM_6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.15158434478587','-0.001479131257734','-0.097310328504292','0.115919981322584','0','6','60']
  H_HDV['28_FC_6_0'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','645.710917279831','0.077348718355174','181223.773617886','1.23041046728452','693.681866300777','6','60']
  H_HDV['28_CO_6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','50','CO','y=(c+(a*exp(b*x)))','35.9152276683514','-0.069552657113145','1.61448883848568','0','0','6','49']
  H_HDV['28_THC_6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.67500807219968','-0.003766851957044','-0.790021685245607','0.198636288552609','0','6','49']
  H_HDV['28_NOx_6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','126.19177986525','-0.190242101713507','-18.0922865420478','0.189868660853837','0','6','49']
  H_HDV['28_PM_6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','50','PM','y=(1/(a+(b*x)))','0.800645142175615','0.047798902281379','0','0','0','6','49']
  H_HDV['28_FC_6_50'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','732.121376499504','0.091098338342788','1903784.55826897','1.73721976734154','928.96400528475','6','49']
  H_HDV['28_CO_6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.8016937164542','2.1020154819102','-1.12115432218668','0.003539324659914','0','6','40']
  H_HDV['28_THC_6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.745139844472684','1.78558628048191','8.65420261724832','4.69298079305638','-0.09960747484964','6','40']
  H_HDV['28_NOx_6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','117.435822499985','-0.198071343213536','-14.0051089396623','0.171044875244406','0','6','40']
  H_HDV['28_PM_6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','100','PM','y=(c+(a*exp(b*x)))','0.995309023962058','-0.03535645083075','0.219528524611487','0','0','6','40']
  H_HDV['28_FC_6_100'] = [' 28','EURO3','Ubus Artic >18t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','693.710844224424','0.105930688719905','427162.69770389','1.39916495618305','1194.30120012414','6','40']
  H_HDV['29_CO_0_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.66065621939425','0.055222199203805','0.823682471522097','0.232698843368384','0.077282413503185','6','75']
  H_HDV['29_THC_0_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.073933158137322','0.054574860767035','0.53913076939528','0.480270279376723','0.01058021931485','6','75']
  H_HDV['29_NOx_0_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.0040599625365','-0.029752036572675','-2.52904766633032','0.116282645475711','0','6','75']
  H_HDV['29_PM_0_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.147825267834218','0.060370864956608','0.449618008598579','0.304334278634667','0.018036608903596','6','75']
  H_HDV['29_FC_0_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','664.76770815189','0.0680309406287','4886664.93849978','1.79921234906843','200.039416878693','6','75']
  H_HDV['29_CO_0_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.05668726406805','-5.95799695925275','-1.26668169395013','0','0','6','75']
  H_HDV['29_THC_0_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072107462121493','0.055497034480223','0.489205258522153','0.458444453821704','0.010340506951126','6','75']
  H_HDV['29_NOx_0_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.9173889355461','-0.034766460710908','-2.07113334701318','0.104239119598855','0','6','75']
  H_HDV['29_PM_0_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.153144761130836','0.05915938909981','0.415536501504101','0.292030445824454','0.017814122565488','6','75']
  H_HDV['29_FC_0_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','50','FC','y=(c+(a*exp(b*x)))','703.746507663447','-0.063642635266947','229.332728820752','','','6','75']
  H_HDV['29_CO_0_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.31525568159153','-0.000822331009876','-0.082571569972673','0.069846042904252','0','6','75']
  H_HDV['29_THC_0_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.069707326351531','0.055694482252289','0.420099306783426','0.430236296563091','0.010242582843757','6','75']
  H_HDV['29_NOx_0_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.9131633710243','-0.037090225180045','-1.65288774709408','0.090222723837173','0','6','75']
  H_HDV['29_PM_0_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.166138491437334','0.059663599794279','0.410745999730702','0.301384073325838','0.01757490930641','6','75']
  H_HDV['29_FC_0_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','0','100','FC','y=(c+(a*exp(b*x)))','675.768256688834','-0.053426956555982','247.427394058888','0','0','6','75']
  H_HDV['29_CO_-2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','0','CO','y=(c+(a*exp(b*x)))','0.675242809186666','-0.064389111617119','0.05364307267345','0','0','6','75']
  H_HDV['29_THC_-2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.066112149215623','0.055533998516012','0.421916599066123','0.453007383733281','0.009506475065895','6','75']
  H_HDV['29_NOx_-2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.630605002986','0.065585037704023','621902.58080744','2.09572730958821','1.1274449668006','6','75']
  H_HDV['29_PM_-2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.165699584397825','0.068374924098668','0.585925093455937','0.414732329253749','0.0127855867915','6','75']
  H_HDV['29_FC_-2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002728164062721','0.484698129750224','-29.3981713719491','698.015357285613','0','6','75']
  H_HDV['29_CO_-2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002381227452','0.000449614061909','-0.028844223286028','0.686410046390936','0','6','75']
  H_HDV['29_THC_-2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065973042182324','0.053243065749848','0.390970488322257','0.445387121694351','0.006879325244111','6','75']
  H_HDV['29_NOx_-2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','50','NOx','y=(c+(a*exp(b*x)))','16.2091356519326','-0.059335454842076','0.902265103921786','0','0','6','75']
  H_HDV['29_PM_-2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.179630589000895','0.068703550488827','0.836395275032902','0.506557354373377','0.010519803499463','6','75']
  H_HDV['29_FC_-2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','440.963278314277','0.047167725956309','220.904238120092','0.046849168442601','37.9875536415697','6','75']
  H_HDV['29_CO_-2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','100','CO','y=(c+(a*exp(b*x)))','0.736259432226184','-0.049201209349596','0.009996625980998','0','0','6','75']
  H_HDV['29_THC_-2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.066319878439493','0.052759604694973','0.39592670657746','0.451409435587596','0.005442031019502','6','75']
  H_HDV['29_NOx_-2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','100','NOx','y=(c+(a*exp(b*x)))','15.7788075668521','-0.050659195097227','0.586685200863081','0','0','6','75']
  H_HDV['29_PM_-2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.19674736071167','0.070791302684516','2.15928557794838','0.704585100978199','0.009459649271338','6','75']
  H_HDV['29_FC_-2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-2','100','FC','y=(c+(a*exp(b*x)))','664.959758482958','-0.039004451305207','11.9451781468256','0','0','6','75']
  H_HDV['29_CO_-4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.023273272309433','0.478558059588258','2.20846814769786','0.384336245783869','0.084663284998091','6','75']
  H_HDV['29_THC_-4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.081122841537502','0.0627562561549','1.3809888445568','0.730499996005035','0.004231217140899','6','75']
  H_HDV['29_NOx_-4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','0','NOx','y=(c+(a*exp(b*x)))','15.1144337612166','-0.072231989828769','0.236674455641892','0','0','6','75']
  H_HDV['29_PM_-4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.182001265124455','0.07382676184184','0.693948839051162','0.506708054704729','0.005889690571429','6','75']
  H_HDV['29_FC_-4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002605142192229','0.458597550553014','-27.3519454251219','584.057496454176','0','6','75']
  H_HDV['29_CO_-4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','50','CO','y=(c+(a*exp(b*x)))','0.523144610890331','-0.051304148019304','-0.003238737731428','0','0','6','75']
  H_HDV['29_THC_-4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082993393256104','0.067539931368394','3.94995605108857','0.920947123145934','0.004048487924929','6','75']
  H_HDV['29_NOx_-4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000060860235033','0.01061468558054','-0.622206431687617','12.7883776190607','0','6','75']
  H_HDV['29_PM_-4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.201953265178106','0.079623411798059','5.4674106183477','0.9052095423137','0.005855045642416','6','75']
  H_HDV['29_FC_-4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.003234717054474','-0.000191875249216','0.000013297032631','0','0','6','75']
  H_HDV['29_CO_-4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','100','CO','y=(c+(a*exp(b*x)))','0.529884264079734','-0.047158318670355','-0.012362532424045','0','0','6','75']
  H_HDV['29_THC_-4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083294697861526','0.070130440836916','7.83213961453355','1.04537692339083','0.003971480278776','6','75']
  H_HDV['29_NOx_-4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','100','NOx','y=(c+(a*exp(b*x)))','13.1309130611537','-0.056830600089596','-0.080549534456954','0','0','6','75']
  H_HDV['29_PM_-4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.206164723496679','0.08032513485854','41.1766820820399','1.26300621413162','0.005609717377919','6','75']
  H_HDV['29_FC_-4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-4','100','FC','y=(c+(a*exp(b*x)))','553.654357971308','-0.047291915531371','-11.1997972646217','0','0','6','75']
  H_HDV['29_CO_-6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001469430861','0.000281884078693','-0.018089404087122','0.405319311010546','0','6','70']
  H_HDV['29_THC_-6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086190504870972','0.072574175303856','17.349496554415','1.19418782104355','0.003467840869568','6','70']
  H_HDV['29_NOx_-6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','0','NOx','y=(c+(a*exp(b*x)))','13.4609114018937','-0.077222515392298','0.037911721286537','0','0','6','70']
  H_HDV['29_PM_-6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.188474343869836','0.081655620036426','1.39849942030001','0.652271242391602','0.004637735724355','6','70']
  H_HDV['29_FC_-6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','0','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1673.50756728389','2.26671663122929','-0.074970404516231','0','0','6','70']
  H_HDV['29_CO_-6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000876742217','0.000209673711228','-0.015496605852237','0.379381316787046','0','6','70']
  H_HDV['29_THC_-6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00219854591839','0.54127764827877','-0.75079433515618','0.564650916023148','0.042303180755356','6','70']
  H_HDV['29_NOx_-6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','25.0392586266749','1.29111181922497','-0.08557587676785','0','0','6','70']
  H_HDV['29_PM_-6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.194633503254084','0.084303879221106','5.98393238365591','0.928735330082366','0.00464098430512','6','70']
  H_HDV['29_FC_-6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','683.272579116713','0.601214830655485','-0.086429055073364','0','0','6','70']
  H_HDV['29_CO_-6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000431505575','0.000158672117845','-0.013905982312153','0.369959486498317','0','6','70']
  H_HDV['29_THC_-6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08428112375721','0.076630266796133','41.0157360730317','1.35064892007094','0.003417697545896','6','70']
  H_HDV['29_NOx_-6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','15.683115464399','0.596077947892592','-0.093043727431707','0','0','6','70']
  H_HDV['29_PM_-6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.199880264879855','0.085829252245288','37.4860958824308','1.25731453290539','0.004640974263855','6','70']
  H_HDV['29_FC_-6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','515.821008247061','0.324265177603524','-0.094671982246341','0','0','6','70']
  H_HDV['29_CO_2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.78952238198063','-0.002016323931152','-0.176916378477093','0.114418194243211','0','6','75']
  H_HDV['29_THC_2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.05972961537967','0.043876007527833','0.274014093454937','0.305492019003431','0.00893879565059','6','75']
  H_HDV['29_NOx_2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.5873277208253','0.084530353905355','75.9955961598548','0.56552791966613','6.32861997902203','6','75']
  H_HDV['29_PM_2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095738836941458','0.037708086073929','0.399854579850825','0.198390105338924','0.016360272598585','6','75']
  H_HDV['29_FC_2_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000803574551406','0.000070412669861','-0.000000601403887','0','0','6','75']
  H_HDV['29_CO_2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.18587670306569','-4.59855027061766','-1.23376510385853','0','0','6','75']
  H_HDV['29_THC_2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055430136614743','0.045088683207576','0.256472861999259','0.281220948546292','0.010023715513625','6','75']
  H_HDV['29_NOx_2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.5528215734164','0.076788770394066','198.686966141785','0.749103552477696','7.64104131874025','6','75']
  H_HDV['29_PM_2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.098692828788928','0.031733876602954','0.429752239524931','0.192144369436186','0.012066409258259','6','75']
  H_HDV['29_FC_2_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1129.95574013386','1.33532671891589','-44.0228356082799','0.057062260055299','0','6','75']
  H_HDV['29_CO_2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.08602833246609','-6.76332327932833','-1.43908300042941','0','0','6','73']
  H_HDV['29_THC_2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.049536601860029','0.042288018557465','0.244052533746649','0.260419061580649','0.010615784479867','6','73']
  H_HDV['29_NOx_2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4226116716097','0.073803057565358','4689.45908439821','1.32735111261778','8.86696092212448','6','73']
  H_HDV['29_PM_2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115910283461724','0.033727405069747','0.456558202214479','0.19919041159308','0.011281659579259','6','73']
  H_HDV['29_FC_2_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002941907105196','0.517940457910607','-30.943655178855','1157.8229535663','0','6','73']
  H_HDV['29_CO_4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.55226719211738','-0.003390495136997','-0.288267158447803','0.130664219751328','0','6','70']
  H_HDV['29_THC_4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057948251586393','0.046372194261378','0.234074132849363','0.265088527270994','0.010514208115223','6','70']
  H_HDV['29_NOx_4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.5225879357752','0.049579826410465','29.3563320526576','0.19792199027035','8.50845603717638','6','70']
  H_HDV['29_PM_4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095330429142884','0.023312884403919','0.46482562599975','0.184910214127465','0.004633409303223','6','70']
  H_HDV['29_FC_4_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','702.89990230555','0.080536636868455','832571.102914363','1.51378248462033','500.361949468889','6','70']
  H_HDV['29_CO_4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.99134064666133','-5.4950216474862','-1.39165811103818','0','0','6','66']
  H_HDV['29_THC_4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.045607237400843','0.038582153822833','0.223176612050645','0.231729714651969','0.011163568812267','6','66']
  H_HDV['29_NOx_4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.7382002224142','0.061775233895726','28.8682998371741','0.234205543790912','10.8645427205125','6','66']
  H_HDV['29_PM_4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.126000614527633','0.021945913659992','0.532000672667192','0.194336433219439','-0.007820786913097','6','66']
  H_HDV['29_FC_4_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','672.154416166272','0.071305287577564','1751249.51567671','1.64065663710061','643.091698443847','6','66']
  H_HDV['29_CO_4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000035093765211','-0.002533817199367','-0.013439789495079','2.75341501830071','0','6','56']
  H_HDV['29_THC_4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.247138469773572','-0.000178233161531','-0.039586272214442','0.181454241955986','0','6','56']
  H_HDV['29_NOx_4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.571407241118','0.033503833803776','32.3859462673543','0.20197351282694','10.7586271034513','6','56']
  H_HDV['29_PM_4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','100','PM','y=((a*(b**x))*(x**c))','0.886518347136345','0.979685520231298','-0.532503441703002','0','0','6','56']
  H_HDV['29_FC_4_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','730.967305252257','0.084832595716504','9288050.97117264','2.02160406507045','812.723701761711','6','56']
  H_HDV['29_CO_6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.75440728628203','-0.007547175810528','-0.507981681723948','0.158576614901392','0','6','62']
  H_HDV['29_THC_6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050517786019084','0.043568949853718','0.211599208497635','0.231495387562621','0.012090509563479','6','62']
  H_HDV['29_NOx_6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.41062194976044','0.051161866200838','36.6763612554071','0.214684015321012','10.9335792415673','6','62']
  H_HDV['29_PM_6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.621399459886786','-0.001056226040527','-0.094276959420218','0.174805196010269','0','6','62']
  H_HDV['29_FC_6_0'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','660.928967346678','0.080691235551527','44696.3547236118','0.989662777696095','657.310233803469','6','62']
  H_HDV['29_CO_6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','50','CO','y=(c+(a*exp(b*x)))','4.45504885674869','-0.078148349725959','0.21313872746523','0','0','6','50']
  H_HDV['29_THC_6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.257825408871962','-0.000219338047322','-0.04316235460917','0.1901181987049','0','6','50']
  H_HDV['29_NOx_6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.8417013126477','0.029514126846684','40.1372907729293','0.216911284411089','11.6100910471389','6','50']
  H_HDV['29_PM_6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','50','PM','y=((a*(b**x))*(x**c))','0.770102350962418','0.970126361007755','-0.394258833233085','0','0','6','50']
  H_HDV['29_FC_6_50'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','760.262818690488','0.095530569059779','458316.415855052','1.4901987156508','885.887972930287','6','50']
  H_HDV['29_CO_6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','100','CO','y=(c+(a*exp(b*x)))','6.21580820441567','-0.023525436037738','-2.21769803747225','0','0','6','41']
  H_HDV['29_THC_6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.385862933333598','-0.000220268416361','-0.093693053656933','0.269004696788214','0','6','41']
  H_HDV['29_NOx_6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','55.6474948238047','-0.141073406220741','-4.98984164212525','0.148257890946037','0','6','41']
  H_HDV['29_PM_6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','100','PM','y=((a*(b**x))*(x**c))','0.511406425025729','0.952115112241611','-0.090385978878676','0','0','6','41']
  H_HDV['29_FC_6_100'] = [' 29','EURO4','Ubus Artic >18t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','802.418559361137','0.120758881215436','317584354.79821','2.56132983785898','1150.72968899574','6','41']
  H_HDV['30_CO_0_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.666155979375507','0.055224030058962','0.837821992880751','0.234359420852725','0.078000511590744','6','75']
  H_HDV['30_THC_0_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.075266883190087','0.05454823296156','0.554349995173675','0.482138456372341','0.010790303019372','6','75']
  H_HDV['30_NOx_0_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.9852629368058','-0.021548583824298','-2.09395316498192','0.125547377871002','0','6','75']
  H_HDV['30_PM_0_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.150459600769071','0.060509704467453','0.458613406038381','0.305520420617962','0.01829500687858','6','75']
  H_HDV['30_FC_0_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','684.840733593531','0.068103516766812','3844376.07876194','1.75267934631782','203.608894715233','6','75']
  H_HDV['30_CO_0_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.02614513449055','-5.82589999064299','-1.25677044368321','0','0','6','75']
  H_HDV['30_THC_0_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.07355837046093','0.055519136571641','0.499673224800345','0.460021573009572','0.010533584719481','6','75']
  H_HDV['30_NOx_0_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.3083065538609','-0.025341034964644','-1.79025092141328','0.115422343736355','0','6','75']
  H_HDV['30_PM_0_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.154783933144799','0.059100386476429','0.422852371020241','0.291551004715723','0.01809215184605','6','75']
  H_HDV['30_FC_0_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','50','FC','y=(c+(a*exp(b*x)))','727.767042191229','-0.064046520397867','233.47035419351','','','6','75']
  H_HDV['30_CO_0_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.33180679178542','-0.00097780566758','-0.084957370083322','0.071490497053204','0','6','75']
  H_HDV['30_THC_0_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.071306189613662','0.055776740670713','0.435920131297789','0.434693573128015','0.010423850304238','6','75']
  H_HDV['30_NOx_0_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.5936259059121','-0.027647122941382','-1.48888920999967','0.102992561952802','0','6','75']
  H_HDV['30_PM_0_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.16724413766209','0.059401947093297','0.420231699102522','0.300782233731952','0.017797831159068','6','75']
  H_HDV['30_FC_0_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','0','100','FC','y=(c+(a*exp(b*x)))','698.522218520529','-0.053906267563829','251.982331183245','0','0','6','75']
  H_HDV['30_CO_-2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','0','CO','y=(c+(a*exp(b*x)))','0.682481806892908','-0.064585203994983','0.054903310595033','0','0','6','75']
  H_HDV['30_THC_-2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067171884783608','0.055358637526514','0.421765111038673','0.449950780247397','0.009682710740254','6','75']
  H_HDV['30_NOx_-2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2561703548048','0.071430987492798','76198.8995855346','1.80409612144539','0.705314765142474','6','75']
  H_HDV['30_PM_-2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.167147734147927','0.068072864354077','0.572936017501586','0.406327937632559','0.012974246825409','6','75']
  H_HDV['30_FC_-2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002862896164658','0.505960387612056','-30.5205839825658','720.280854026169','0','6','75']
  H_HDV['30_CO_-2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002419267285','0.000455828043844','-0.029187983519292','0.693885517056328','0','6','75']
  H_HDV['30_THC_-2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.066940521859439','0.053045788536044','0.395743441262649','0.443831368948219','0.007009327988351','6','75']
  H_HDV['30_NOx_-2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','50','NOx','y=(c+(a*exp(b*x)))','11.7815720261856','-0.066636933130469','0.607745472592826','0','0','6','75']
  H_HDV['30_PM_-2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.181845180902655','0.068546696172856','0.829789743619659','0.501013048993507','0.010689376343485','6','75']
  H_HDV['30_FC_-2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','50','FC','y=(c+(a*exp(b*x)))','682.958579440104','-0.047608378823309','39.8404335397999','0','0','6','75']
  H_HDV['30_CO_-2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','100','CO','y=(c+(a*exp(b*x)))','0.743020616525198','-0.049339871441081','0.010896127967764','0','0','6','75']
  H_HDV['30_THC_-2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067437892287585','0.052624298227437','0.398497819780564','0.449692103507458','0.005534149730946','6','75']
  H_HDV['30_NOx_-2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000049356756944','0.008481072598736','-0.500965788620514','11.0984380848975','0','6','75']
  H_HDV['30_PM_-2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.19523492724646','0.069569405194652','1.31236353518509','0.614083078663713','0.009417892236615','6','75']
  H_HDV['30_FC_-2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-2','100','FC','y=(c+(a*exp(b*x)))','684.612992279356','-0.039551965371302','13.7866439858342','0','0','6','75']
  H_HDV['30_CO_-4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','0','CO','y=(c+(a*exp(b*x)))','0.533281111120443','-0.05596959579934','0.007450803213314','0','0','6','75']
  H_HDV['30_THC_-4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082403691608368','0.06251063632683','1.36983456269557','0.725478712017614','0.0042940909828','6','75']
  H_HDV['30_NOx_-4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','10.9566031053009','0.07827147443924','0.173574864144917','0','0','6','75']
  H_HDV['30_PM_-4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.183755207775064','0.073473556860337','0.670415845366118','0.494913857635408','0.005960980939931','6','75']
  H_HDV['30_FC_-4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','0','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002734040464982','0.479052450261584','-28.4332140698248','604.20140022652','0','6','75']
  H_HDV['30_CO_-4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','50','CO','y=(c+(a*exp(b*x)))','0.530765429669737','-0.051654381471626','-0.002614825446271','0','0','6','75']
  H_HDV['30_THC_-4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.084402837011005','0.067394330181342','3.87977003120239','0.915131016530332','0.004115074332201','6','75']
  H_HDV['30_NOx_-4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000049334949633','0.008275596601078','-0.463823274059991','9.04690190909686','0','6','75']
  H_HDV['30_PM_-4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.204590944642984','0.079481379292247','4.70839409662349','0.875495436939972','0.00594812656139','6','75']
  H_HDV['30_FC_-4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','50','FC','y=(c+(a*exp(b*x)))','597.05917489851','-0.053950474689078','-0.558750365544163','0','0','6','75']
  H_HDV['30_CO_-4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','100','CO','y=(c+(a*exp(b*x)))','0.536371165027536','-0.047510178115227','-0.01168916653011','0','0','6','75']
  H_HDV['30_THC_-4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.084748327650177','0.070033083560117','7.92184877898992','1.04424414624101','0.004040717421179','6','75']
  H_HDV['30_NOx_-4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000039750325865','0.007031296764567','-0.417365460029329','8.6309208662735','0','6','75']
  H_HDV['30_PM_-4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.209106872665902','0.080329336022632','32.5241844798977','1.21903473368878','0.005715735027301','6','75']
  H_HDV['30_FC_-4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-4','100','FC','y=(c+(a*exp(b*x)))','571.431437894311','-0.047784088739281','-10.7184753291775','0','0','6','75']
  H_HDV['30_CO_-6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001526655307','0.00029044051382','-0.018520393150428','0.413127208837699','0','6','70']
  H_HDV['30_THC_-6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087691100743652','0.072434235068936','17.0296138402839','1.18780023369','0.003522465377587','6','70']
  H_HDV['30_NOx_-6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','0','NOx','y=(c+(a*exp(b*x)))','9.74628941223689','-0.082607850039057','0.044299616683085','0','0','6','70']
  H_HDV['30_PM_-6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.190770722122807','0.081383514902675','1.28948608543716','0.633395552225749','0.004696711514621','6','70']
  H_HDV['30_FC_-6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','0','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','1924.5087714026','2.59908081565198','-0.074335315638594','0','0','6','70']
  H_HDV['30_CO_-6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000930970319','0.00021761040787','-0.015884884827068','0.386158425038902','0','6','70']
  H_HDV['30_THC_-6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002235658567616','0.550496956659107','-0.751245057676864','0.563674324599056','0.042321643835934','6','70']
  H_HDV['30_NOx_-6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','33.2756332568624','3.00308328540141','-0.082888324889776','0','0','6','70']
  H_HDV['30_PM_-6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.20000281152537','0.084943455947628','8.96077816175103','0.994971672900953','0.004792841494553','6','70']
  H_HDV['30_FC_-6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','729.684073256332','0.643151126636161','-0.085707605020397','0','0','6','70']
  H_HDV['30_CO_-6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000481659213','0.000165998958151','-0.01426244518251','0.376127711511528','0','6','70']
  H_HDV['30_THC_-6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.085382224637471','0.076326818698876','31.6634225401976','1.30525162318945','0.003457916014889','6','70']
  H_HDV['30_NOx_-6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','100','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','14.5696251557006','0.9840339216876','-0.090194310191505','0','0','6','70']
  H_HDV['30_PM_-6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.204261967261226','0.086185748241781','53.2462854444911','1.31242818218438','0.00476135596143','6','70']
  H_HDV['30_FC_-6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','-6','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','541.515912927778','0.340704748598782','-0.094015668543589','0','0','6','70']
  H_HDV['30_CO_2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.80649998693724','-0.00205807234355','-0.179582308172504','0.11511327654731','0','6','75']
  H_HDV['30_THC_2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061014993254747','0.043940852622377','0.279585664750621','0.30703256265997','0.009090227150133','6','75']
  H_HDV['30_NOx_2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.2356650537496','-0.01986162433256','-2.56093274879201','0.13148070183806','0','6','75']
  H_HDV['30_PM_2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.09880849521016','0.038546081520352','0.407695575566825','0.201103817626577','0.016903585267107','6','75']
  H_HDV['30_FC_2_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000776816629248','0.000069652346502','-0.000000593834222','0','0','6','75']
  H_HDV['30_CO_2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.13661389837107','-4.4404755375878','-1.21864840045208','0','0','6','75']
  H_HDV['30_THC_2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.056998601807597','0.045508630024165','0.261963987941605','0.283350122931963','0.010192466606368','6','75']
  H_HDV['30_NOx_2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.29011401120155','26.3978950770841','2.14538392083419','1.3256892908366','-0.00263493834723','6','75']
  H_HDV['30_PM_2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.10063591928407','0.032588732125408','0.433728652601527','0.192853802399133','0.012977667608195','6','75']
  H_HDV['30_FC_2_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1162.03933261305','1.29336534033122','-46.072453760142','0.057980898385885','0','6','75']
  H_HDV['30_CO_2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.99279748316269','-6.4500440090712','-1.41271722539885','0','0','6','73']
  H_HDV['30_THC_2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.051188723391894','0.043034294382513','0.251107190971332','0.26385143630269','0.010798204754578','6','73']
  H_HDV['30_NOx_2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','4.10337195867952','30.2895687643694','1.66031203618866','1.10198856666528','0.0110133570847','6','73']
  H_HDV['30_PM_2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.116495403291601','0.033823008759684','0.463200668629628','0.199085913195764','0.011761974023437','6','73']
  H_HDV['30_FC_2_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003056891271892','0.538000918386102','-32.0862997942744','1187.24141390878','0','6','73']
  H_HDV['30_CO_4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.57005646453829','-0.003389527330273','-0.292128580465233','0.131519933301255','0','6','70']
  H_HDV['30_THC_4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.059898970111568','0.046858251062906','0.239641006329819','0.268432913601928','0.01067876301946','6','70']
  H_HDV['30_NOx_4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.52890864054309','0.024369155233698','24.812923984696','0.1787726669028','3.9548086872966','6','70']
  H_HDV['30_PM_4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095599990357491','0.023800749833535','0.469321400627422','0.184786678529932','0.005841513904572','6','70']
  H_HDV['30_FC_4_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','724.722036539902','0.080517425081083','737365.391738065','1.48747346003154','508.066818655076','6','70']
  H_HDV['30_CO_4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.93964357931094','-5.35147063951208','-1.37796512911473','0','0','6','67']
  H_HDV['30_THC_4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047427853932849','0.040569633257938','0.223405969821146','0.231854585474865','0.011807899515718','6','67']
  H_HDV['30_NOx_4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.29265322055015','0.038311093758416','27.7172341980833','0.210153870093093','5.213880410076','6','67']
  H_HDV['30_PM_4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.125068339808567','0.02406189288057','0.541415228972043','0.197490381825427','-0.002573246650436','6','67']
  H_HDV['30_FC_4_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','700.826147485471','0.072648609471139','2613430.0630156','1.7046760837938','654.891738469877','6','67']
  H_HDV['30_CO_4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','100','CO','y=(((a*(x**2))+(b*x))+c)','0.000911263028407','-0.103272485076317','3.08736209242134','0','0','6','57']
  H_HDV['30_THC_4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017258967212597','0.170089312663013','3.63646496454789','2.05953040266368','-0.015564951603863','6','57']
  H_HDV['30_NOx_4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.73805717232614','0.024392178770306','29.6280187229074','0.209222583387433','4.1143996199056','6','57']
  H_HDV['30_PM_4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','100','PM','y=(1/(a+(b*x)))','0.612622576247893','0.44111850049163','0','0','0','6','57']
  H_HDV['30_FC_4_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','4','100','FC','y=(c+(a*exp(((-1)*b)*x)))','839.613049512654','0.092224394548604','832.43647325958','0','0','6','57']
  H_HDV['30_CO_6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.72247809128742','-0.007240482754758','-0.498994842294819','0.157026118343974','0','6','62']
  H_HDV['30_THC_6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.052721320462518','0.04478155668564','0.216123105581373','0.235072865012627','0.01245571240245','6','62']
  H_HDV['30_NOx_6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.96326924207988','0.024931943898635','31.0912180252194','0.199589295466562','4.70329145002311','6','62']
  H_HDV['30_PM_6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.622867247620278','-0.001027447706617','-0.093542943084571','0.172917652071045','0','6','62']
  H_HDV['30_FC_6_0'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','686.185954666275','0.081469259614721','48521.6478749981','0.999389835075733','668.581735622767','6','62']
  H_HDV['30_CO_6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','50','CO','y=(c+(a*exp(b*x)))','4.32652938341238','-0.071329444025687','0.164555703448753','0','0','6','50']
  H_HDV['30_THC_6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.260095686391057','-0.000219632120234','-0.043083254497992','0.18809724683737','0','6','50']
  H_HDV['30_NOx_6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1330510353189','0.025576986430386','33.221636884534','0.222475218537185','4.71337693954674','6','50']
  H_HDV['30_PM_6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','50','PM','y=((a*(b**x))*(x**c))','0.811999229523131','0.972237555724716','-0.425307021347727','0','0','6','50']
  H_HDV['30_FC_6_50'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','783.030458566919','0.095761394232217','437767.208065685','1.47411286360891','901.043066500501','6','50']
  H_HDV['30_CO_6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.9347664605387','0.007451394519102','5.45322859767832','0.34680076737459','-7.83664298532754','6','41']
  H_HDV['30_THC_6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.380660492928865','-0.000218450428506','-0.089989966151743','0.262581876926668','0','6','41']
  H_HDV['30_NOx_6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','100','NOx','y=((a*(b**x))*(x**c))','63.4873711762181','1.00261873331444','-0.552265228314556','0','0','6','41']
  H_HDV['30_PM_6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','100','PM','y=((a*(b**x))*(x**c))','0.536764007368034','0.954911724348503','-0.12310043613918','0','0','6','41']
  H_HDV['30_FC_6_100'] = [' 30','EURO5','Ubus Artic >18t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','809.185177609505','0.119303440898753','139615649.883479','2.4074624836228','1171.06025612675','6','41']
  H_HDV['31_CO_0_0'] = [' 31','EURO0','RT <=7.5t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.07895468427765','0.050500640996967','5.90538023404435','0.352844596197085','1.22669602763019','6','86']
  H_HDV['31_THC_0_0'] = [' 31','EURO0','RT <=7.5t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.03568503722406','0.028490897261888','4.55404334234803','0.147231978764931','0.344084310594379','6','86']
  H_HDV['31_NOx_0_0'] = [' 31','EURO0','RT <=7.5t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.313274351884649','-0.023073738788448','6.70587952083912','0.066876321724638','2.70588503187426','6','86']
  H_HDV['31_PM_0_0'] = [' 31','EURO0','RT <=7.5t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.917026797142707','0.053617967414078','1.24359556306112','0.244431707661112','0.229571395739387','6','86']
  H_HDV['31_FC_0_0'] = [' 31','EURO0','RT <=7.5t 80ties','0','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000240843211634','2.73652615758085','676.066722558394','-0.499696736888167','0','6','86']
  H_HDV['31_CO_0_50'] = [' 31','EURO0','RT <=7.5t 80ties','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.33100726733857','21.8449743141392','-0.504971268965296','0.312821696485325','0.047556882497556','6','86']
  H_HDV['31_THC_0_50'] = [' 31','EURO0','RT <=7.5t 80ties','0','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.111912289529115','0.015857327328183','0.000025584463042','0','0','6','86']
  H_HDV['31_NOx_0_50'] = [' 31','EURO0','RT <=7.5t 80ties','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.46863277232646','0.044296467882353','-0.32701557278449','0.044431139375323','0','6','86']
  H_HDV['31_PM_0_50'] = [' 31','EURO0','RT <=7.5t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.984448830087052','0.057388564737311','1.35752845755518','0.26749736498926','0.246679578562812','6','86']
  H_HDV['31_FC_0_50'] = [' 31','EURO0','RT <=7.5t 80ties','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002308557272388','0.000227623094934','-0.000001915727824','','','6','86']
  H_HDV['31_CO_0_100'] = [' 31','EURO0','RT <=7.5t 80ties','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.47468743965493','24.6675920197789','-0.639338086257226','0.296539492749853','0.051585214408591','6','86']
  H_HDV['31_THC_0_100'] = [' 31','EURO0','RT <=7.5t 80ties','0','100','THC','y=exp((a+(b/x))+(c*log(x)))','3.97518282719069','-3.75586162591426','-0.996279243652408','0','0','6','86']
  H_HDV['31_NOx_0_100'] = [' 31','EURO0','RT <=7.5t 80ties','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.52922886129344','0.047995266035535','-0.282436225244069','0.038075129211927','0','6','86']
  H_HDV['31_PM_0_100'] = [' 31','EURO0','RT <=7.5t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.00835291020517','0.058606344236948','1.6868638033832','0.296299211863054','0.264511762521369','6','86']
  H_HDV['31_FC_0_100'] = [' 31','EURO0','RT <=7.5t 80ties','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002349550915244','0.000205542755142','-0.000001711373277','0','0','6','86']
  H_HDV['31_CO_-2_0'] = [' 31','EURO0','RT <=7.5t 80ties','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.1116352965944','15.9947165824607','-0.343578576215108','0.303172983291961','0.051289250796142','6','86']
  H_HDV['31_THC_-2_0'] = [' 31','EURO0','RT <=7.5t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.02931919441916','0.038484382129317','4.20870762240747','0.181302635033247','0.591718179503845','6','86']
  H_HDV['31_NOx_-2_0'] = [' 31','EURO0','RT <=7.5t 80ties','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.08496306553817','0.037228296003152','-0.406275627404946','0.054177858368605','0','6','86']
  H_HDV['31_PM_-2_0'] = [' 31','EURO0','RT <=7.5t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.944406029783219','0.057323867932875','1.09455870214333','0.287842794086165','0.185386635746073','6','86']
  H_HDV['31_FC_-2_0'] = [' 31','EURO0','RT <=7.5t 80ties','-2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001946478426173','0.000432308111335','-0.000003726782551','0','0','6','86']
  H_HDV['31_CO_-2_50'] = [' 31','EURO0','RT <=7.5t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.36052008370854','0.010632020552388','-0.308223004468962','0.052336559617175','0','6','86']
  H_HDV['31_THC_-2_50'] = [' 31','EURO0','RT <=7.5t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.49162396603004','0.048318581298242','3.88056565042878','0.209737908451293','0.646727620516102','6','86']
  H_HDV['31_NOx_-2_50'] = [' 31','EURO0','RT <=7.5t 80ties','-2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.8634587831804','0.029710540373591','-0.375871565715306','0.052631252987151','0','6','86']
  H_HDV['31_PM_-2_50'] = [' 31','EURO0','RT <=7.5t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.10182017861644','0.065451188455519','2.33801026654692','0.484055467058561','0.18418147743173','6','86']
  H_HDV['31_FC_-2_50'] = [' 31','EURO0','RT <=7.5t 80ties','-2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001901843717428','0.000447853183778','-0.000003724581267','0','0','6','86']
  H_HDV['31_CO_-2_100'] = [' 31','EURO0','RT <=7.5t 80ties','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.38254677299139','0.017888717058317','-0.299991877139096','0.047036784951171','0','6','86']
  H_HDV['31_THC_-2_100'] = [' 31','EURO0','RT <=7.5t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.0004368344401','0.055654029699015','3.7509551901382','0.255152224823161','0.642790390939765','6','86']
  H_HDV['31_NOx_-2_100'] = [' 31','EURO0','RT <=7.5t 80ties','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000019751343373','0.003966614127491','-0.252121033391969','7.24072928243964','0','6','86']
  H_HDV['31_PM_-2_100'] = [' 31','EURO0','RT <=7.5t 80ties','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.47847560499765','0.000644464599674','-0.090147733807716','0.067470461079346','0','6','86']
  H_HDV['31_FC_-2_100'] = [' 31','EURO0','RT <=7.5t 80ties','-2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001950752153217','0.000444586603889','-0.000003456549786','0','0','6','86']
  H_HDV['31_CO_-4_0'] = [' 31','EURO0','RT <=7.5t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.71725356288081','0.023186359304822','-0.302705167363551','0.04843290450547','0','6','86']
  H_HDV['31_THC_-4_0'] = [' 31','EURO0','RT <=7.5t 80ties','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.79769230448969','0.005816128818641','-0.365088266812046','0.066157345273895','0','6','86']
  H_HDV['31_NOx_-4_0'] = [' 31','EURO0','RT <=7.5t 80ties','-4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.82947721186427','0.026019088922799','-0.387215307440076','0.054000971674176','0','6','86']
  H_HDV['31_PM_-4_0'] = [' 31','EURO0','RT <=7.5t 80ties','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.35390468695558','0.001713085513748','-0.086499816251933','0.065755426454609','0','6','86']
  H_HDV['31_FC_-4_0'] = [' 31','EURO0','RT <=7.5t 80ties','-4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','265.795149837025','0.812301774959744','-16.3694609608698','0.059268121755979','0','6','86']
  H_HDV['31_CO_-4_50'] = [' 31','EURO0','RT <=7.5t 80ties','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.59077049839084','0.014969688458308','-0.300498282054989','0.050340325800817','0','6','86']
  H_HDV['31_THC_-4_50'] = [' 31','EURO0','RT <=7.5t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.00336596571195','0.069851896368857','86.5797846227219','0.946126498241955','0.428943001863082','6','86']
  H_HDV['31_NOx_-4_50'] = [' 31','EURO0','RT <=7.5t 80ties','-4','50','NOx','y=(c+(a*exp(b*x)))','6.04842893765098','-0.069731528923731','0.651877770983345','0','0','6','86']
  H_HDV['31_PM_-4_50'] = [' 31','EURO0','RT <=7.5t 80ties','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.32930110253818','0.000907458692846','-0.085917950187822','0.066819977197701','0','6','86']
  H_HDV['31_FC_-4_50'] = [' 31','EURO0','RT <=7.5t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','183.860642968263','0.064769145303979','44282.5348741813','1.11242133396225','20.1114518867927','6','86']
  H_HDV['31_CO_-4_100'] = [' 31','EURO0','RT <=7.5t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.8299398136324','0.060334989369879','210.117910683711','0.979208860707034','0.313991342854034','6','86']
  H_HDV['31_THC_-4_100'] = [' 31','EURO0','RT <=7.5t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.03536659390415','0.056847903531651','7.83793703581127','0.379835779815041','0.276746554711963','6','86']
  H_HDV['31_NOx_-4_100'] = [' 31','EURO0','RT <=7.5t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.14784976496531','0.042934330947804','441.303615373303','0.937021585949045','0.285980941520664','6','86']
  H_HDV['31_PM_-4_100'] = [' 31','EURO0','RT <=7.5t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.996592167633231','0.060147512150921','4.43464081452974','0.531642274069905','0.063959076812892','6','86']
  H_HDV['31_FC_-4_100'] = [' 31','EURO0','RT <=7.5t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','157.413624279564','0.049232283430293','16252.4181894163','0.912712657535522','10.3130723709442','6','86']
  H_HDV['31_CO_-6_0'] = [' 31','EURO0','RT <=7.5t 80ties','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.27663451735263','0.010839708657805','-0.315015173471909','0.055432584369661','0','6','72']
  H_HDV['31_THC_-6_0'] = [' 31','EURO0','RT <=7.5t 80ties','-6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.211446424666302','17.4950964387706','-0.382436632395534','0.372981080905479','0.053248929174925','6','72']
  H_HDV['31_NOx_-6_0'] = [' 31','EURO0','RT <=7.5t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.49195819638058','0.060989690007982','522.831613353356','0.999608582432212','0.176167400204726','6','72']
  H_HDV['31_PM_-6_0'] = [' 31','EURO0','RT <=7.5t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.07195728340057','0.068031734532156','3.97810075617101','0.587639937010927','0.046671239767545','6','72']
  H_HDV['31_FC_-6_0'] = [' 31','EURO0','RT <=7.5t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','166.630563915554','0.062635558091847','16619.7081546104','0.933471042658552','5.15246705327736','6','72']
  H_HDV['31_CO_-6_50'] = [' 31','EURO0','RT <=7.5t 80ties','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.087048162491314','30.3767037502675','-1.10576092168416','0.343715259466419','0.050236149235595','6','72']
  H_HDV['31_THC_-6_50'] = [' 31','EURO0','RT <=7.5t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.95046451598998','0.059571255434267','8.78178612746618','0.397759014253481','0.163429669292641','6','72']
  H_HDV['31_NOx_-6_50'] = [' 31','EURO0','RT <=7.5t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.98969749641976','0.056005691925297','407.021301438451','0.940502084960109','0.096538657701089','6','72']
  H_HDV['31_PM_-6_50'] = [' 31','EURO0','RT <=7.5t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.944899826425058','0.063137237601652','3.0824389262263','0.476695489074389','0.034562577868243','6','72']
  H_HDV['31_FC_-6_50'] = [' 31','EURO0','RT <=7.5t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','147.624905999181','0.057820853544085','14321.6132294208','0.893662978691612','2.28337980670341','6','72']
  H_HDV['31_CO_-6_100'] = [' 31','EURO0','RT <=7.5t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.12107966441607','0.060738234759598','53.2266588084358','0.700087622205516','0.080357304392568','6','72']
  H_HDV['31_THC_-6_100'] = [' 31','EURO0','RT <=7.5t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.66736390940028','0.057582326833266','10.2529476296473','0.400530530791461','0.143579302563836','6','72']
  H_HDV['31_NOx_-6_100'] = [' 31','EURO0','RT <=7.5t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.73362185550846','0.054017307159479','373.371177826091','0.919340059666112','0.072423745605924','6','72']
  H_HDV['31_PM_-6_100'] = [' 31','EURO0','RT <=7.5t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.877785965103002','0.060689570300228','4.12340435048634','0.494340263030168','0.03005691901936','6','72']
  H_HDV['31_FC_-6_100'] = [' 31','EURO0','RT <=7.5t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','134.839365453027','0.05514808096324','10846.3627167232','0.841305121068972','1.25360016392785','6','72']
  H_HDV['31_CO_2_0'] = [' 31','EURO0','RT <=7.5t 80ties','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.40866948125776','0.024386455231075','-0.361922535572747','0.044544232147161','0','6','86']
  H_HDV['31_THC_2_0'] = [' 31','EURO0','RT <=7.5t 80ties','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.18781846023697','0.034204183789569','4.05583013773564','0.176557675592438','0.364900602720009','6','86']
  H_HDV['31_NOx_2_0'] = [' 31','EURO0','RT <=7.5t 80ties','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.9106818563524','0.0407219899541','-0.506798932984754','0.061520453485858','0','6','86']
  H_HDV['31_PM_2_0'] = [' 31','EURO0','RT <=7.5t 80ties','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.86822836665521','0.001179900804807','-0.113021054464941','0.069942994141122','0','6','86']
  H_HDV['31_FC_2_0'] = [' 31','EURO0','RT <=7.5t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','411.740924813593','1.13219018245626','-18.4908804784982','0.056185723133645','0','6','86']
  H_HDV['31_CO_2_50'] = [' 31','EURO0','RT <=7.5t 80ties','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.45556666523482','0.027381676084609','-0.478808537897919','0.054446224619194','0','6','84']
  H_HDV['31_THC_2_50'] = [' 31','EURO0','RT <=7.5t 80ties','2','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.112078069665881','0.012529992330973','0.000071294462105','0','0','6','84']
  H_HDV['31_NOx_2_50'] = [' 31','EURO0','RT <=7.5t 80ties','2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000028176654111','0.00539940672022','-0.310419231677006','11.7408532374649','0','6','84']
  H_HDV['31_PM_2_50'] = [' 31','EURO0','RT <=7.5t 80ties','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.03295026270886','0.002202663284569','-0.130137060702914','0.073095991377541','0','6','84']
  H_HDV['31_FC_2_50'] = [' 31','EURO0','RT <=7.5t 80ties','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.00544124750066','2.16316875601593','760.549820210652','-0.440700326603099','0','6','84']
  H_HDV['31_CO_2_100'] = [' 31','EURO0','RT <=7.5t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.82120321731273','0.093100695653137','-0.443532318674071','0.037518198708052','0','6','82']
  H_HDV['31_THC_2_100'] = [' 31','EURO0','RT <=7.5t 80ties','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.23604317024273','0.026476063986265','5.31927546184383','0.132836758635008','0.189895455175569','6','82']
  H_HDV['31_NOx_2_100'] = [' 31','EURO0','RT <=7.5t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000027206953255','0.005202173184765','-0.300998820649186','12.4466793756679','0','6','82']
  H_HDV['31_PM_2_100'] = [' 31','EURO0','RT <=7.5t 80ties','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.09404908785991','0.006957877059123','-0.121891058487601','0.060483996677934','0','6','82']
  H_HDV['31_FC_2_100'] = [' 31','EURO0','RT <=7.5t 80ties','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','451.739077888412','1.7310163148007','-18.8218916603148','0.053325268910562','0','6','82']
  H_HDV['31_CO_4_0'] = [' 31','EURO0','RT <=7.5t 80ties','4','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000044163186981','0.007563346330031','-0.415736211148897','9.72151525584235','0','6','71']
  H_HDV['31_THC_4_0'] = [' 31','EURO0','RT <=7.5t 80ties','4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.82089453699983','0.032300406580678','4.74853146480569','0.14168765686732','0.333629866601119','6','71']
  H_HDV['31_NOx_4_0'] = [' 31','EURO0','RT <=7.5t 80ties','4','0','NOx','y=((a*(b**x))*(x**c))','23.5170400924234','1.01046044880906','-0.43786575501521','0','0','6','71']
  H_HDV['31_PM_4_0'] = [' 31','EURO0','RT <=7.5t 80ties','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.24417934713754','0.001990474065171','-0.158371311061235','0.08167759612427','0','6','71']
  H_HDV['31_FC_4_0'] = [' 31','EURO0','RT <=7.5t 80ties','4','0','FC','y=((a*(b**x))*(x**c))','1016.24391425381','1.0121367426797','-0.593542363578936','0','0','6','71']
  H_HDV['31_CO_4_50'] = [' 31','EURO0','RT <=7.5t 80ties','4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000070327688452','0.010813112043473','-0.528159019109061','11.0461860401665','0','6','70']
  H_HDV['31_THC_4_50'] = [' 31','EURO0','RT <=7.5t 80ties','4','50','THC','y=exp((a+(b/x))+(c*log(x)))','4.81400092599797','-5.78662850084575','-1.195012032','0','0','6','70']
  H_HDV['31_NOx_4_50'] = [' 31','EURO0','RT <=7.5t 80ties','4','50','NOx','y=((a*(b**x))*(x**c))','23.0530833218032','1.00818639730698','-0.362032836989568','0','0','6','70']
  H_HDV['31_PM_4_50'] = [' 31','EURO0','RT <=7.5t 80ties','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00001445410169','0.002193656854385','-0.10735334457807','2.18455330461627','0','6','70']
  H_HDV['31_FC_4_50'] = [' 31','EURO0','RT <=7.5t 80ties','4','50','FC','y=((a*(b**x))*(x**c))','999.434580243707','1.01152916161301','-0.540995821318575','0','0','6','70']
  H_HDV['31_CO_4_100'] = [' 31','EURO0','RT <=7.5t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.3755550990862','0.027050785113686','-0.927277685514702','0.081699499321222','0','6','67']
  H_HDV['31_THC_4_100'] = [' 31','EURO0','RT <=7.5t 80ties','4','100','THC','y=exp((a+(b/x))+(c*log(x)))','4.94473226432027','-5.94142727294979','-1.24532315182931','0','0','6','67']
  H_HDV['31_NOx_4_100'] = [' 31','EURO0','RT <=7.5t 80ties','4','100','NOx','y=((a*(b**x))*(x**c))','22.6469802051747','1.00578457670138','-0.294424892384957','0','0','6','67']
  H_HDV['31_PM_4_100'] = [' 31','EURO0','RT <=7.5t 80ties','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.83199732388528','0.002355944332012','-0.233303471259904','0.098138469675596','0','6','67']
  H_HDV['31_FC_4_100'] = [' 31','EURO0','RT <=7.5t 80ties','4','100','FC','y=((a*(x**b))+(c*(x**d)))','0.15016685425736','1.50190655906423','861.178886834853','-0.388894624897397','0','6','67']
  H_HDV['31_CO_6_0'] = [' 31','EURO0','RT <=7.5t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.8783696213024','0.197480556675073','-0.481493901421464','0.028313230575606','0','6','70']
  H_HDV['31_THC_6_0'] = [' 31','EURO0','RT <=7.5t 80ties','6','0','THC','y=exp((a+(b/x))+(c*log(x)))','5.23574998269058','-7.12570054367871','-1.30327043762458','0','0','6','70']
  H_HDV['31_NOx_6_0'] = [' 31','EURO0','RT <=7.5t 80ties','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.0128600831852','0.07685545054595','-0.422641158196335','0.04252222302464','0','6','70']
  H_HDV['31_PM_6_0'] = [' 31','EURO0','RT <=7.5t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.4189879503813','0.008517001427553','-0.156144784583423','0.067364998679764','0','6','70']
  H_HDV['31_FC_6_0'] = [' 31','EURO0','RT <=7.5t 80ties','6','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001717121554664','0.000131638064211','-0.000001418424383','0','0','6','70']
  H_HDV['31_CO_6_50'] = [' 31','EURO0','RT <=7.5t 80ties','6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.20135451887161','7.19518342251334','6.12191228314651','2.3880420848578','0.02008745541508','6','59']
  H_HDV['31_THC_6_50'] = [' 31','EURO0','RT <=7.5t 80ties','6','50','THC','y=exp((a+(b/x))+(c*log(x)))','5.09070034577666','-6.33454609043682','-1.28041726032365','0','0','6','59']
  H_HDV['31_NOx_6_50'] = [' 31','EURO0','RT <=7.5t 80ties','6','50','NOx','y=((a*(b**x))*(x**c))','23.19189164781','1.00459655573786','-0.267900460150815','0','0','6','59']
  H_HDV['31_PM_6_50'] = [' 31','EURO0','RT <=7.5t 80ties','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.592016151961055','13.7354630724716','-2.22347005682371','-0.499256729072015','0.162435177545208','6','59']
  H_HDV['31_FC_6_50'] = [' 31','EURO0','RT <=7.5t 80ties','6','50','FC','y=((a*(b**x))*(x**c))','922.008538218484','1.00724689979394','-0.422443894722081','0','0','6','59']
  H_HDV['31_CO_6_100'] = [' 31','EURO0','RT <=7.5t 80ties','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.5804414949748','0.02103764806111','-1.94438181265471','0.133699851022253','0','6','50']
  H_HDV['31_THC_6_100'] = [' 31','EURO0','RT <=7.5t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.16755088364504','30.707537773503','-1.56849834928534','-0.336918351849221','0.127587743124318','6','50']
  H_HDV['31_NOx_6_100'] = [' 31','EURO0','RT <=7.5t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.34209833789101','0.100409010531401','221252.255638816','2.19182752767621','12.2072811686379','6','50']
  H_HDV['31_PM_6_100'] = [' 31','EURO0','RT <=7.5t 80ties','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000037726377692','0.004393777405102','-0.169451506902759','2.9193578759898','0','6','50']
  H_HDV['31_FC_6_100'] = [' 31','EURO0','RT <=7.5t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','401.718990802525','0.158287605297963','15243.4640314599','1.12425910673492','324.394406406831','6','50']
  H_HDV['32_CO_0_0'] = [' 32','EURO1','RT <=7.5t Euro-1','0','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.63943355383207','0.001909524293073','-0.133715863841706','0.057704735285132','0','6','86']
  H_HDV['32_THC_0_0'] = [' 32','EURO1','RT <=7.5t Euro-1','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.649717432049243','0.039360008365876','0.610987578105214','0.187736353296627','0.125435721868896','6','86']
  H_HDV['32_NOx_0_0'] = [' 32','EURO1','RT <=7.5t Euro-1','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.16596624483157','0.027156606798175','-0.453780793118778','0.069107404427405','0','6','86']
  H_HDV['32_PM_0_0'] = [' 32','EURO1','RT <=7.5t Euro-1','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.089140006639071','3.64462471650909','-1.4533229796098','0.267996045571494','0.052418434747244','6','86']
  H_HDV['32_FC_0_0'] = [' 32','EURO1','RT <=7.5t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.398467611935688','-0.050858413677788','195.750161210826','0.081511269700731','78.0794065962741','6','86']
  H_HDV['32_CO_0_50'] = [' 32','EURO1','RT <=7.5t Euro-1','0','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.76260822702778','0.004218897608729','-0.138811740772604','0.055438902139166','0','6','86']
  H_HDV['32_THC_0_50'] = [' 32','EURO1','RT <=7.5t Euro-1','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.655681828989109','0.041408469952394','0.579903628294934','0.179881862657029','0.130363868426115','6','86']
  H_HDV['32_NOx_0_50'] = [' 32','EURO1','RT <=7.5t Euro-1','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.13180354133249','0.024220074948414','-0.407155211816616','0.065873609320494','0','6','86']
  H_HDV['32_PM_0_50'] = [' 32','EURO1','RT <=7.5t Euro-1','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.096416727383386','3.43779854251746','-1.41939095053924','0.249711482093828','0.054873079373245','6','86']
  H_HDV['32_FC_0_50'] = [' 32','EURO1','RT <=7.5t Euro-1','0','50','FC','y=((a*(x**b))+(c*(x**d)))','0.0003003643133','2.68877245308691','465.2532001309','-0.439861821573344','','6','86']
  H_HDV['32_CO_0_100'] = [' 32','EURO1','RT <=7.5t Euro-1','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.199370202995123','0.041133686921451','-0.000287675881576','0','0','6','86']
  H_HDV['32_THC_0_100'] = [' 32','EURO1','RT <=7.5t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.661618754402958','0.044238759870734','0.521874734375024','0.167024422320977','0.139207388882808','6','86']
  H_HDV['32_NOx_0_100'] = [' 32','EURO1','RT <=7.5t Euro-1','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.07368489497934','0.021305290213518','-0.355125995017086','0.061353820962406','0','6','86']
  H_HDV['32_PM_0_100'] = [' 32','EURO1','RT <=7.5t Euro-1','0','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.05080459805945','0.196492832237495','-0.001197601392479','0','0','6','86']
  H_HDV['32_FC_0_100'] = [' 32','EURO1','RT <=7.5t Euro-1','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.003361162367043','0.000232904566795','-0.000002068484658','0','0','6','86']
  H_HDV['32_CO_-2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.30556967084471','0.003148528607109','-0.110320909087182','0.051643228409391','0','6','86']
  H_HDV['32_THC_-2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.614703177890644','0.0418673619711','0.590856696487379','0.198803115537446','0.12160842978945','6','86']
  H_HDV['32_NOx_-2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.14784657956415','0.021554227171524','-0.43948382435108','0.066912240478387','0','6','86']
  H_HDV['32_PM_-2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','0','PM','y=(c+(a*exp(b*x)))','0.545838601137441','-0.060119591931304','0.069132056649891','0','0','6','86']
  H_HDV['32_FC_-2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002464274881985','0.000603651865089','-0.00000546487046','0','0','6','86']
  H_HDV['32_CO_-2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.32433225119164','0.008777539016184','-0.106073418250113','0.042977226800989','0','6','86']
  H_HDV['32_THC_-2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.659519885836613','0.047681720595086','0.547022949413572','0.212744070195298','0.1234677964591','6','86']
  H_HDV['32_NOx_-2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.0063971816567','0.01451767165234','-0.423183145854303','0.068072855933854','0','6','86']
  H_HDV['32_PM_-2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.637684523524265','0.069887390874174','-18.6676588344741','0.943175706538789','0.073168659134277','6','86']
  H_HDV['32_FC_-2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00246131627706','0.000602494474179','-0.000005201181492','0','0','6','86']
  H_HDV['32_CO_-2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.39147714312764','0.014897152390211','-0.104426121344218','0.036330083799587','0','6','86']
  H_HDV['32_THC_-2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.702415767742323','0.05144320138611','0.506428668973742','0.226829046901251','0.119014787703224','6','86']
  H_HDV['32_NOx_-2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000021040268743','0.003974998387082','-0.240589467624806','6.1372667665133','0','6','86']
  H_HDV['32_PM_-2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.625888561273718','0.000977183192461','-0.032605960374207','0.05322303319793','0','6','86']
  H_HDV['32_FC_-2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002599965873979','0.000574796241028','-0.000004585356781','0','0','6','86']
  H_HDV['32_CO_-4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.19751061367321','0.008223249330677','-0.118539160876695','0.049240664722228','0','6','86']
  H_HDV['32_THC_-4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.02052662884552','0.000958633548319','-0.06073857488662','0.06284507035203','0','6','86']
  H_HDV['32_NOx_-4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.28571121735882','0.011105635651243','-0.423235252682598','0.06852121308135','0','6','86']
  H_HDV['32_PM_-4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.674348188679949','0.000506205004234','-0.042445098218407','0.06591851864013','0','6','86']
  H_HDV['32_FC_-4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000000088914121','4.32452801248522','707.936942944008','-0.876198154340502','0','6','86']
  H_HDV['32_CO_-4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.16580665392152','0.007247874851794','-0.115853100808756','0.048159296533959','0','6','86']
  H_HDV['32_THC_-4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.905588155021663','0.066876620727109','5.44018281114013','0.829063839380176','0.076096551921468','6','86']
  H_HDV['32_NOx_-4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.45119327610159','0.065082350721837','182.094005929445','0.875920207976037','0.422040883375391','6','86']
  H_HDV['32_PM_-4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.685767869732335','0.000554080675525','-0.044462016290761','0.066253448110344','0','6','86']
  H_HDV['32_FC_-4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','141.277766556723','0.063342328663596','27496.5687005573','1.08399847947934','14.7623851122828','6','86']
  H_HDV['32_CO_-4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.2930340419909','0.001811540674098','-0.138394033307419','0.061029954019408','0','6','86']
  H_HDV['32_THC_-4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.733505753665747','0.054773631262368','0.728486593951728','0.280768885814122','0.047761964481654','6','86']
  H_HDV['32_NOx_-4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.57526227695911','0.048172094352217','62.2964013978831','0.632531467588617','0.21669574652433','6','86']
  H_HDV['32_PM_-4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.010321926121735','0.538952359909884','6.22446956529026','2.42760267727843','-0.025783058616279','6','86']
  H_HDV['32_FC_-4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','123.517944988382','0.049229406680888','12031.8562854374','0.9141221136025','8.11040096173875','6','86']
  H_HDV['32_CO_-6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.13087262250022','0.002880412457272','-0.135876281380294','0.061347230266842','0','6','72']
  H_HDV['32_THC_-6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.981285536114353','0.0009793089721','-0.060772122806579','0.061222561324177','0','6','72']
  H_HDV['32_NOx_-6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.31339253716923','0.067731647897723','111.877617460449','0.79194446742079','0.139502365882624','6','72']
  H_HDV['32_PM_-6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','4.40913089798578','-11.1507164768282','-1.86719218713155','0','0','6','72']
  H_HDV['32_FC_-6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','130.5206122778','0.063021903251431','11634.7725012479','0.929015858715165','3.90557644450026','6','72']
  H_HDV['32_CO_-6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.87443985796023','0.067526949326767','3.64434928796377','0.509065752796748','0.050126312690916','6','72']
  H_HDV['32_THC_-6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.79704938987662','0.0628292802175','0.883244945833083','0.357131455806432','0.030986611274192','6','72']
  H_HDV['32_NOx_-6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.7517143235269','0.062740181650253','77.7557388915266','0.701518130186704','0.088233467884036','6','72']
  H_HDV['32_PM_-6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.46440722253046','-11.0720883937486','-1.90682982888053','0','0','6','72']
  H_HDV['32_FC_-6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','115.899651666795','0.058257485469272','8859.02525932356','0.86785634775927','1.98649003184655','6','72']
  H_HDV['32_CO_-6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.71133864138808','0.063720787969508','7.89813148724856','0.553975319751334','0.040608564219883','6','72']
  H_HDV['32_THC_-6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.743172719957012','0.061422501577743','1.06330314467936','0.350943681734332','0.02822291555885','6','72']
  H_HDV['32_NOx_-6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.47406593010219','0.06060973294404','76.38255913779','0.6866537553947','0.072743280575685','6','72']
  H_HDV['32_PM_-6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.23066930484743','-10.1927907112709','-1.85845090920212','0','0','6','72']
  H_HDV['32_FC_-6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','107.805927029668','0.056014595181597','8562.94441414237','0.855412625573553','1.33989333987325','6','72']
  H_HDV['32_CO_2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.11025024437094','0.011092550115436','-0.157281302465194','0.051141119154857','0','6','86']
  H_HDV['32_THC_2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.824485203523988','0.044869681565641','0.365395927066377','0.196381420303572','0.152231051708611','6','86']
  H_HDV['32_NOx_2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.2450592603391','0.02803947812315','-0.651397552891607','0.086958801899282','0','6','86']
  H_HDV['32_PM_2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.538108668170892','0.001662710361817','-0.023434015586201','0.045878398318646','0','6','86']
  H_HDV['32_FC_2_0'] = [' 32','EURO1','RT <=7.5t Euro-1','2','0','FC','y=((a*(b**x))*(x**c))','700.786037239985','1.01485316950891','-0.636983185583489','0','0','6','86']
  H_HDV['32_CO_2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.50929771427324','0.007279842423752','-0.198683348043727','0.062265786786252','0','6','85']
  H_HDV['32_THC_2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','2','50','THC','y=(c+(a*exp(b*x)))','1.01156325237718','-0.060129330053196','0.198117893044024','0','0','6','85']
  H_HDV['32_NOx_2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.4055205895427','0.025684201603878','-0.562619683846025','0.080530374413247','0','6','85']
  H_HDV['32_PM_2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.556803052400425','0.001819477737771','-0.025333191191253','0.049829442208887','0','6','85']
  H_HDV['32_FC_2_50'] = [' 32','EURO1','RT <=7.5t Euro-1','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.051045317485336','1.66064487175644','550.804918683531','-0.421094066223424','0','6','85']
  H_HDV['32_CO_2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.22513666343458','-0.001453057134948','-0.313361068509557','0.093396679362788','0','6','83']
  H_HDV['32_THC_2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.22386003479106','0.001558086522431','-0.060959638211381','0.056071734281122','0','6','83']
  H_HDV['32_NOx_2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.902497779597','0.019450530666065','-0.550079506708266','0.083134654968355','0','6','83']
  H_HDV['32_PM_2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.575035571928469','0.002557408911733','-0.025433337589525','0.047373016475482','0','6','83']
  H_HDV['32_FC_2_100'] = [' 32','EURO1','RT <=7.5t Euro-1','2','100','FC','y=((a*(x**b))+(c*(x**d)))','0.027604553182354','1.80980914673937','539.704684045696','-0.371271152015214','0','6','83']
  H_HDV['32_CO_4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.763254526905934','10.414468412019','-1.85036919313911','-0.706555213206338','0.158434375077776','6','71']
  H_HDV['32_THC_4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','4','0','THC','y=(c+(a*exp(b*x)))','1.06240282993581','-0.063690766671482','0.218958980332888','0','0','6','71']
  H_HDV['32_NOx_4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.1211341762226','0.020441715777113','-0.809044935721869','0.103299095753667','0','6','71']
  H_HDV['32_PM_4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','4','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001789104923','0.000323795201183','-0.018839435594322','0.516535033556628','0','6','71']
  H_HDV['32_FC_4_0'] = [' 32','EURO1','RT <=7.5t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','683.431153113155','1.01286322662277','-0.537990348445202','0','0','6','71']
  H_HDV['32_CO_4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.14470783630868','-0.003935813688866','-0.49608084945856','0.122752444323625','0','6','71']
  H_HDV['32_THC_4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','4','50','THC','y=(c+(a*exp(b*x)))','1.09978540700108','-0.069831317383131','0.241673927411507','0','0','6','71']
  H_HDV['32_NOx_4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00004884596693','0.007387310779237','-0.348026658796461','11.2310133020546','0','6','71']
  H_HDV['32_PM_4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002481286464','0.000402387547116','-0.021075743288072','0.552457853344255','0','6','71']
  H_HDV['32_FC_4_50'] = [' 32','EURO1','RT <=7.5t Euro-1','4','50','FC','y=((a*(b**x))*(x**c))','655.862760472272','1.01059418391391','-0.452825757686833','0','0','6','71']
  H_HDV['32_CO_4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.62214992669232','-0.00740341443136','-0.848657391916246','0.166478412661114','0','6','67']
  H_HDV['32_THC_4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.205981705807899','0.887635114252503','5.6803746696518','2.16714105519115','-0.008761817581541','6','67']
  H_HDV['32_NOx_4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-6.14714444575181','0.04113379281667','11.702444869553','0.072048332542999','7.66254575576708','6','67']
  H_HDV['32_PM_4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00000253851838','0.000407639622753','-0.021073664162864','0.578509851579755','0','6','67']
  H_HDV['32_FC_4_100'] = [' 32','EURO1','RT <=7.5t Euro-1','4','100','FC','y=((a*(x**b))+(c*(x**d)))','0.799211197321951','1.09454133347623','618.458363881921','-0.345371688744314','0','6','67']
  H_HDV['32_CO_6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.922098686395509','16.7814584094369','-2.30952179131401','-0.74551767099485','0.169416642398244','6','70']
  H_HDV['32_THC_6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.27339351284346','0.004284896938986','-0.057863558906721','0.045539981744533','0','6','70']
  H_HDV['32_NOx_6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-12.8341939806288','0.046140471252242','18.6316397478481','0.067801550005817','7.11311605683772','6','70']
  H_HDV['32_PM_6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001281427837','0.000281778153312','-0.017703440146059','0.528163180227859','0','6','70']
  H_HDV['32_FC_6_0'] = [' 32','EURO1','RT <=7.5t Euro-1','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','374.262781054033','2.33777323943369','-12.6724885709175','0.04438659006408','0','6','70']
  H_HDV['32_CO_6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','6','50','CO','y=(c+(a*exp(b*x)))','6.05607749929071','-0.179784553967256','1.46777360650932','0','0','6','61']
  H_HDV['32_THC_6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','6','50','THC','y=exp((a+(b/x))+(c*log(x)))','2.03691975665392','-3.23781480161947','-0.835024624451692','0','0','6','61']
  H_HDV['32_NOx_6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.3590757216393','0.022116057368484','-0.689762626081202','0.092348823685711','0','6','61']
  H_HDV['32_PM_6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.43144728542782','0.120129101281318','-0.001279560507685','0','0','6','61']
  H_HDV['32_FC_6_50'] = [' 32','EURO1','RT <=7.5t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','169.306565368534','0.092332931369086','2777.86261894101','0.727201237561384','230.94270145194','6','61']
  H_HDV['32_CO_6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','6','100','CO','y=(c+(a*exp(b*x)))','4.48671959428592','-0.071739281766514','0.942224373417509','0','0','6','50']
  H_HDV['32_THC_6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.430231181326617','0.579252116360646','9.54933931261749','2.93110050105477','0.113781365815868','6','50']
  H_HDV['32_NOx_6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.99142594045411','0.04348595573756','14.0935216696494','0.356997946334161','8.10893119693962','6','50']
  H_HDV['32_PM_6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000005876648681','0.000697030385209','-0.026937064141636','0.649904270242297','0','6','50']
  H_HDV['32_FC_6_100'] = [' 32','EURO1','RT <=7.5t Euro-1','6','100','FC','y=exp((a+(b/x))+(c*log(x)))','5.39120358010296','3.10867008787363','0.052082383217802','0','0','6','50']
  H_HDV['33_CO_0_0'] = [' 33','EURO2','RT <=7.5t Euro-2','0','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.282513590797492','0.059624111864476','-0.000415434746298','0','0','6','86']
  H_HDV['33_THC_0_0'] = [' 33','EURO2','RT <=7.5t Euro-2','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.414154994198976','0.038038136754259','0.440148858196882','0.185537270959206','0.07651352868667','6','86']
  H_HDV['33_NOx_0_0'] = [' 33','EURO2','RT <=7.5t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.4403759973842','-0.016101350081485','6.94772826734183','0.08293532393469','1.85299005581034','6','86']
  H_HDV['33_PM_0_0'] = [' 33','EURO2','RT <=7.5t Euro-2','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','5.84007971020185','0.471607668947544','-0.003982382437053','0','0','6','86']
  H_HDV['33_FC_0_0'] = [' 33','EURO2','RT <=7.5t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.707092530502654','-0.04480212658809','169.397705425092','0.079606348123849','72.6177304115249','6','86']
  H_HDV['33_CO_0_50'] = [' 33','EURO2','RT <=7.5t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.251138399156686','0.05609380556131','-0.000401995677832','0','0','6','86']
  H_HDV['33_THC_0_50'] = [' 33','EURO2','RT <=7.5t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.408108802852382','0.039071896358025','0.415617455652576','0.170177395412051','0.078487704238827','6','86']
  H_HDV['33_NOx_0_50'] = [' 33','EURO2','RT <=7.5t Euro-2','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.95291684317713','0.019499319332321','-0.460742352779971','0.068871083189924','0','6','86']
  H_HDV['33_PM_0_50'] = [' 33','EURO2','RT <=7.5t Euro-2','0','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','5.35203029620273','0.430120737842256','-0.003546226561708','0','0','6','86']
  H_HDV['33_FC_0_50'] = [' 33','EURO2','RT <=7.5t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.691265263701946','-0.044303661670572','160.749812580608','0.07108987260224','79.2923051995812','6','86']
  H_HDV['33_CO_0_100'] = [' 33','EURO2','RT <=7.5t Euro-2','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.241128504662141','0.049803611992631','-0.000350511714735','0','0','6','86']
  H_HDV['33_THC_0_100'] = [' 33','EURO2','RT <=7.5t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.38863533411734','0.039125942888725','0.405570347500517','0.151206557450282','0.079869347094868','6','86']
  H_HDV['33_NOx_0_100'] = [' 33','EURO2','RT <=7.5t Euro-2','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.90643138423607','0.017025014241691','-0.408801134226587','0.064352151797022','0','6','86']
  H_HDV['33_PM_0_100'] = [' 33','EURO2','RT <=7.5t Euro-2','0','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.15157292658032','0.002324970941029','-0.00383384156424','0.016608141986355','0','6','86']
  H_HDV['33_FC_0_100'] = [' 33','EURO2','RT <=7.5t Euro-2','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00389902098584','0.000229223354233','-0.000002060145852','0','0','6','86']
  H_HDV['33_CO_-2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.7946676406182','0.003996517074149','-0.093745369005295','0.054181134138682','0','6','86']
  H_HDV['33_THC_-2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.406554894728842','0.04177807404793','0.594111945315415','0.272358635574375','0.077857105458804','6','86']
  H_HDV['33_NOx_-2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.89267232928832','0.019440483545924','-0.494777126054828','0.0697072081012','0','6','86']
  H_HDV['33_PM_-2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.13189824835086','0.001264799417423','-0.00496040334554','0.029258391260453','0','6','86']
  H_HDV['33_FC_-2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.001377178825754','2.29223727187074','500.387831550387','-0.656379555954794','0','6','86']
  H_HDV['33_CO_-2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.79607697210544','0.00829818620253','-0.085618059883959','0.043427721672702','0','6','86']
  H_HDV['33_THC_-2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.429420507959109','0.046205387386494','0.621758016799062','0.30038547295768','0.078015466412731','6','86']
  H_HDV['33_NOx_-2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.65976820740148','0.013176924249111','-0.464972249878297','0.069339377813649','0','6','86']
  H_HDV['33_PM_-2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.130147548060389','0.002843026388541','-0.004279134007155','0.016608627176747','0','6','86']
  H_HDV['33_FC_-2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00296635586595','0.0006355404334','-0.000005546024485','0','0','6','86']
  H_HDV['33_CO_-2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.84077043157115','0.014430673362705','-0.08031948167415','0.034088965310957','0','6','86']
  H_HDV['33_THC_-2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.445645400021527','0.048517514451174','0.662868694859052','0.322659726086138','0.074595452126173','6','86']
  H_HDV['33_NOx_-2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2153.28911379993','1.33065116334616','5.56324239944488','0.0784698911011','1.53194464384599','6','86']
  H_HDV['33_PM_-2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','100','PM','y=(((a*(x**2))+(b*x))+c)','0.00003136975085','-0.003594268286915','0.131804000535173','0','0','6','86']
  H_HDV['33_FC_-2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.003138406263752','0.000598594981321','-0.000004793163291','0','0','6','86']
  H_HDV['33_CO_-4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.58953578712012','0.006646860367636','-0.085275348079889','0.047803406669723','0','6','86']
  H_HDV['33_THC_-4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.645105260834776','0.000442213875644','-0.036784066950243','0.061474834162401','0','6','86']
  H_HDV['33_NOx_-4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.90595295067589','0.009843067093254','-0.4694285253579','0.070277753152311','0','6','86']
  H_HDV['33_PM_-4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.134007421247286','0.000531336143237','-0.006393446180344','0.043579726084856','0','6','86']
  H_HDV['33_FC_-4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','177.76742475659','0.482576972127902','-10.721661235297','0.058810954857116','0','6','86']
  H_HDV['33_CO_-4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.55103787938671','0.00620419402055','-0.0808485825485','0.045553773986501','0','6','86']
  H_HDV['33_THC_-4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.522836634459696','0.059364144639752','3.55304074241982','0.704923017946302','0.051513621417935','6','86']
  H_HDV['33_NOx_-4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.83450502533673','0.063588311128182','171.506346770938','0.856607945574904','0.447377191453161','6','86']
  H_HDV['33_PM_-4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.133612181857176','0.000380775895816','-0.006356965205304','0.045150959773272','0','6','86']
  H_HDV['33_FC_-4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','123.67149967552','0.060936974055452','21692.5092363971','1.06939861428189','13.5358321762864','6','86']
  H_HDV['33_CO_-4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.43347360523476','0.064726245514324','130.291098756792','1.12958585121935','0.104373352246602','6','86']
  H_HDV['33_THC_-4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.462692920291296','0.051375569298561','1.09430632999805','0.419366399473742','0.035306521288358','6','86']
  H_HDV['33_NOx_-4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.89994139549646','0.047545358868996','62.1476411959255','0.6224678306394','0.226333900357867','6','86']
  H_HDV['33_PM_-4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','100','PM','y=(c+(a*exp(b*x)))','0.127932538378827','-0.060071504526769','0.014393556988521','0','0','6','86']
  H_HDV['33_FC_-4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','108.125974410633','0.046814347947953','9389.77016392416','0.897175179841687','7.34993209328252','6','86']
  H_HDV['33_CO_-6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.52864975497062','0.001800012074966','-0.098414500145492','0.062645076267063','0','6','72']
  H_HDV['33_THC_-6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.609358493514788','0.00068153830084','-0.034695143512917','0.056519564700918','0','6','72']
  H_HDV['33_NOx_-6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.72760100868555','0.06763093117762','106.959651744925','0.774383662363029','0.157853579424391','6','72']
  H_HDV['33_PM_-6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.090897184789337','0.052707725579297','0.087106374258724','0.201614287649797','0.009599901390794','6','72']
  H_HDV['33_FC_-6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','115.440824996174','0.061952819915245','10073.4460330421','0.929136699990043','3.81718903829886','6','72']
  H_HDV['33_CO_-6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','50','CO','y=(c+(a*exp(b*x)))','1.48672792829419','-0.072121513066658','0.045653337917678','0','0','6','72']
  H_HDV['33_THC_-6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.022871816240308','2.97491214014081','-0.932429482186911','0.347895107543166','0.045649262967877','6','72']
  H_HDV['33_NOx_-6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.10271657396056','0.062758627525032','80.8347673834191','0.698860254866645','0.102216577731131','6','72']
  H_HDV['33_PM_-6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.092404340936837','0.052465556229186','0.08776989664292','0.216554141306519','0.009515904644504','6','72']
  H_HDV['33_FC_-6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','102.991460130885','0.057196876230173','8413.58690303794','0.885434821390069','2.06272482874636','6','72']
  H_HDV['33_CO_-6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.21451024886446','0.062340763213471','16.5703550982637','0.733179276554227','0.033682263571149','6','72']
  H_HDV['33_THC_-6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.478310641405995','0.058157573159917','1.37082331790773','0.482257030822553','0.024997437933442','6','72']
  H_HDV['33_NOx_-6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.78007017788074','0.060582410352628','81.2216350250848','0.687961720195417','0.084810595624479','6','72']
  H_HDV['33_PM_-6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.097608130560372','0.052446987959344','0.134758322269397','0.293393540930213','0.009501291335301','6','72']
  H_HDV['33_FC_-6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','95.8814963206774','0.054930822622784','8534.6533342957','0.882301534893324','1.47560017619225','6','72']
  H_HDV['33_CO_2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','2','0','CO','y=(c+(a*exp(b*x)))','2.44050127207503','-0.09357265447157','0.542140175005088','0','0','6','86']
  H_HDV['33_THC_2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.506314781850025','0.040958572256728','0.353181572607354','0.187161817248396','0.085111456973597','6','86']
  H_HDV['33_NOx_2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-7.61773098881578','0.027208155147823','12.7875191038057','0.059905202451821','5.38553007481328','6','86']
  H_HDV['33_PM_2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.169101487882397','0.00011520774567','-0.006509503300051','0.058408544494539','0','6','86']
  H_HDV['33_FC_2_0'] = [' 33','EURO2','RT <=7.5t Euro-2','2','0','FC','y=((a*(b**x))*(x**c))','590.378767841653','1.01460127596246','-0.599559863871763','0','0','6','86']
  H_HDV['33_CO_2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.48390735071603','-0.002227591001488','-0.282341847155662','0.101058350018357','0','6','86']
  H_HDV['33_THC_2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.558090840452294','0.051819193385795','0.252989898603721','0.181343293574831','0.111845249757329','6','86']
  H_HDV['33_NOx_2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-9.6626713187337','0.036897235095011','15.3891146984191','0.058805886444736','5.3452950757544','6','86']
  H_HDV['33_PM_2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','4.99718711933424','0.266157305634656','-0.001967477797055','0','0','6','86']
  H_HDV['33_FC_2_50'] = [' 33','EURO2','RT <=7.5t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','288.122515611384','1.16300098677652','-11.4805554483106','0.056052941005511','0','6','86']
  H_HDV['33_CO_2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.22633475380908','-0.005153043700225','-0.379453476166062','0.113471880842442','0','6','84']
  H_HDV['33_THC_2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.582417724236837','0.058866590555808','0.203792481040091','0.163294743394111','0.128008844720289','6','84']
  H_HDV['33_NOx_2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.7906989457471','0.011576344531193','-0.588537622871202','0.081904381255611','0','6','84']
  H_HDV['33_PM_2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','2','100','PM','y=(c+(a*exp(b*x)))','0.143029891340223','-0.059241652130433','0.075678139897059','0','0','6','84']
  H_HDV['33_FC_2_100'] = [' 33','EURO2','RT <=7.5t Euro-2','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000682356893687','0.132771003265075','-7.64894053164142','288.605513060156','0','6','84']
  H_HDV['33_CO_4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.5602467108127','0.00674733386896','-0.240854935655874','0.074261650632201','0','6','71']
  H_HDV['33_THC_4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','4','0','THC','y=exp((a+(b/x))+(c*log(x)))','1.46307731326078','-2.62362630310564','-0.823757719100946','0','0','6','71']
  H_HDV['33_NOx_4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-4.96449271688158','0.043493998162849','12.0829023818391','0.080159319468157','5.59668114220851','6','71']
  H_HDV['33_PM_4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','4.44204405871802','0.283910379239076','-0.002542739478037','0','0','6','71']
  H_HDV['33_FC_4_0'] = [' 33','EURO2','RT <=7.5t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','568.603473979815','1.01197715323941','-0.488097363195624','0','0','6','71']
  H_HDV['33_CO_4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.03624551086746','-0.00837323254204','-0.50732370681525','0.127298229946715','0','6','71']
  H_HDV['33_THC_4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','4','50','THC','y=(c+(a*exp(b*x)))','0.769245526875548','-0.073597702968367','0.151274190162089','0','0','6','71']
  H_HDV['33_NOx_4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.8952690642668','0.005299315714807','-0.810312735233052','0.101174695351879','0','6','71']
  H_HDV['33_PM_4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','4','50','PM','y=(c+(a*exp(b*x)))','0.158192727835685','-0.059963039176749','0.083049964298206','0','0','6','71']
  H_HDV['33_FC_4_50'] = [' 33','EURO2','RT <=7.5t Euro-2','4','50','FC','y=((a*(b**x))*(x**c))','563.425106671577','1.01069328609867','-0.419753256759701','0','0','6','71']
  H_HDV['33_CO_4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.03139473764378','-0.011901919401656','-0.639667884112383','0.134709040756623','0','6','69']
  H_HDV['33_THC_4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.150858383601854','0.583375933999964','5.64667194183454','2.15893625978757','0.002230752010354','6','69']
  H_HDV['33_NOx_4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.6697757366112','0.003935102440079','-0.747034821833871','0.095127772740296','0','6','69']
  H_HDV['33_PM_4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.292358000110738','-0.00045183374171','-0.013707473029912','0.076978971910828','0','6','69']
  H_HDV['33_FC_4_100'] = [' 33','EURO2','RT <=7.5t Euro-2','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001202969375329','0.188142001686177','-9.14627582828113','354.193906879327','0','6','69']
  H_HDV['33_CO_6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.21363347172731','-0.009655798581001','-0.857219581851665','0.171127689296119','0','6','71']
  H_HDV['33_THC_6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.868601601791722','0.003033259907921','-0.04215010804691','0.047811760971969','0','6','71']
  H_HDV['33_NOx_6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','6','0','NOx','y=(c+(a*exp(b*x)))','8.84726048307177','-0.122466026663966','6.40330358814017','0','0','6','71']
  H_HDV['33_PM_6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.281463091241749','-0.000406844614224','-0.018698271420344','0.111603569686815','0','6','71']
  H_HDV['33_FC_6_0'] = [' 33','EURO2','RT <=7.5t Euro-2','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','334.118238105149','2.80238442495298','-9.90798387523319','0.037437967036937','0','6','71']
  H_HDV['33_CO_6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.58579397839735','-0.013570739128312','-1.05777243390504','0.173421797195889','0','6','66']
  H_HDV['33_THC_6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.926687063106617','0.001476220265871','-0.051830183957557','0.061753103168377','0','6','66']
  H_HDV['33_NOx_6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.7250017899845','0.007690301532366','-0.798613407217624','0.09446473048047','0','6','66']
  H_HDV['33_PM_6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00000161890154','0.000222375936533','-0.010689503887878','0.292099450795671','0','6','66']
  H_HDV['33_FC_6_50'] = [' 33','EURO2','RT <=7.5t Euro-2','6','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000372499471135','2.85461260146947','487.305338171408','-0.223837618984346','0','6','66']
  H_HDV['33_CO_6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','6','100','CO','y=(c+(a*exp(b*x)))','5.85776770754467','-0.087384020615707','0.785747236937199','0','0','6','51']
  H_HDV['33_THC_6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.980836271678323','0.002117393972986','-0.061465739733157','0.071384734093028','0','6','51']
  H_HDV['33_NOx_6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000123262886988','0.013601795208937','-0.533683069073375','16.1891212847924','0','6','51']
  H_HDV['33_PM_6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002423196724','0.000312030553372','-0.014252486235727','0.355324180192441','0','6','51']
  H_HDV['33_FC_6_100'] = [' 33','EURO2','RT <=7.5t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','107.343417045202','0.095225541317132','2332.64280175932','0.676720900228814','275.614012878782','6','51']
  H_HDV['34_CO_0_0'] = [' 34','EURO3','RT <=7.5t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.75313289531446','0.048798255746579','1.40627006497211','0.227051970279711','0.345132338327913','6','86']
  H_HDV['34_THC_0_0'] = [' 34','EURO3','RT <=7.5t Euro-3','0','0','THC','y=(1/(a+(b*(x**c))))','1.24830386182253','0.114645148489997','1.03882838317161','0','0','6','86']
  H_HDV['34_NOx_0_0'] = [' 34','EURO3','RT <=7.5t Euro-3','0','0','NOx','y=((a*(b**x))*(x**c))','40.4811591190016','1.01901918999505','-0.982784127898758','0','0','6','86']
  H_HDV['34_PM_0_0'] = [' 34','EURO3','RT <=7.5t Euro-3','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.211016290102144','-0.000145935977224','-0.009373531106762','0.057458242152747','0','6','86']
  H_HDV['34_FC_0_0'] = [' 34','EURO3','RT <=7.5t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.01459485977671','-0.041221525964322','187.457323075978','0.07727455380177','75.342406225817','6','86']
  H_HDV['34_CO_0_50'] = [' 34','EURO3','RT <=7.5t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.84804863651226','0.050961954059183','1.93685793022589','0.269738391999291','0.384175685753069','6','86']
  H_HDV['34_THC_0_50'] = [' 34','EURO3','RT <=7.5t Euro-3','0','50','THC','y=exp((a+(b/x))+(c*log(x)))','1.61796979092607','-3.95602439373772','-0.928626415189699','0','0','6','86']
  H_HDV['34_NOx_0_50'] = [' 34','EURO3','RT <=7.5t Euro-3','0','50','NOx','y=((a*(x**b))+(c*(x**d)))','0.007370392422726','1.21572452124539','31.1925613553879','-0.770235702682926','0','6','86']
  H_HDV['34_PM_0_50'] = [' 34','EURO3','RT <=7.5t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.139633163159989','0.043683050104418','0.042585757431934','0.138843359773111','0.036788054983651','6','86']
  H_HDV['34_FC_0_50'] = [' 34','EURO3','RT <=7.5t Euro-3','0','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000522014929286','2.55885160394032','437.509114853118','-0.421565701925295','','6','86']
  H_HDV['34_CO_0_100'] = [' 34','EURO3','RT <=7.5t Euro-3','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92025735107113','0.051080361353606','2.54838503979733','0.302799631760966','0.42194706007717','6','86']
  H_HDV['34_THC_0_100'] = [' 34','EURO3','RT <=7.5t Euro-3','0','100','THC','y=exp((a+(b/x))+(c*log(x)))','1.53065070991171','-3.62847266933565','-0.912511860326711','0','0','6','86']
  H_HDV['34_NOx_0_100'] = [' 34','EURO3','RT <=7.5t Euro-3','0','100','NOx','y=((a*(b**x))*(x**c))','33.83691582753','1.0146509925742','-0.842551160136272','0','0','6','86']
  H_HDV['34_PM_0_100'] = [' 34','EURO3','RT <=7.5t Euro-3','0','100','PM','y=(c+(a*exp(b*x)))','0.161608235499623','-0.049523665236211','0.039780646861082','0','0','6','86']
  H_HDV['34_FC_0_100'] = [' 34','EURO3','RT <=7.5t Euro-3','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.003613832850143','0.000218886117548','-0.00000193522789','0','0','6','86']
  H_HDV['34_CO_-2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.26910184834838','0.008350633514086','-0.087274242795409','0.03522646914631','0','6','86']
  H_HDV['34_THC_-2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.391373457739153','0.038707644591521','0.277317798724075','0.191591153560935','0.064708375965319','6','86']
  H_HDV['34_NOx_-2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','0','NOx','y=((a*(b**x))*(x**c))','47.4509098266387','1.02155917769554','-1.17929802211143','0','0','6','86']
  H_HDV['34_PM_-2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.195994982582655','0.000882928883075','-0.006106972735289','0.026653731930317','0','6','86']
  H_HDV['34_FC_-2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000253623534049','2.65519107736683','523.595680018146','-0.627105754146401','0','6','86']
  H_HDV['34_CO_-2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.27700013606115','0.015496532995561','-0.084985351159325','0.029506548444486','0','6','86']
  H_HDV['34_THC_-2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.399835477431185','0.043735855648363','0.240679396580316','0.193082636665327','0.067970162170018','6','86']
  H_HDV['34_NOx_-2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','50','NOx','y=((a*(b**x))*(x**c))','43.2627975489516','1.01860578274181','-1.13006901420943','0','0','6','86']
  H_HDV['34_PM_-2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.187693409891832','0.001315627898079','-0.005629128760286','0.022527292768274','0','6','86']
  H_HDV['34_FC_-2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000052753601744','2.94578041288511','496.88644795146','-0.605950845087051','0','6','86']
  H_HDV['34_CO_-2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.3170309535313','0.02468680418125','-0.084191845764226','0.024551010550893','0','6','86']
  H_HDV['34_THC_-2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.414029856122716','0.048118162449521','0.219862738244475','0.204846363959174','0.067799090175764','6','86']
  H_HDV['34_NOx_-2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','342.929929523444','0.94842673234371','6.9905177418078','0.089936239289894','1.17306741289539','6','86']
  H_HDV['34_PM_-2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.18485948944249','0.001750557746782','-0.005485441409374','0.020014286741812','0','6','86']
  H_HDV['34_FC_-2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','210.940359222396','0.481273789566707','-9.8085954200528','0.051114694711268','0','6','86']
  H_HDV['34_CO_-4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.13612423644985','0.012413161294089','-0.091175443424327','0.035376866761178','0','6','86']
  H_HDV['34_THC_-4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.569408080970502','0.000913044194257','-0.029846154166977','0.053970189257379','0','6','86']
  H_HDV['34_NOx_-4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','338.09150076239','0.966609439700547','6.84292640915152','0.097492857829968','0.60548620147439','6','86']
  H_HDV['34_PM_-4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.205155740697105','0.000270995018079','-0.008293281442102','0.042383767605049','0','6','86']
  H_HDV['34_FC_-4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','198.885192376982','0.388441526229197','-11.9094009861125','0.060644518241497','0','6','86']
  H_HDV['34_CO_-4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.0814896871976','0.014926955403797','-0.087975059945737','0.032060267395058','0','6','86']
  H_HDV['34_THC_-4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.553053372644986','0.000367396911069','-0.030445989689609','0.057789227356902','0','6','86']
  H_HDV['34_NOx_-4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.06111941999922','0.072660934039518','148.754932363739','0.764164932575616','0.358152945668518','6','86']
  H_HDV['34_PM_-4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.19962014887247','0.000335825943289','-0.00830067686882','0.041874841263932','0','6','86']
  H_HDV['34_FC_-4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','144.661881030859','0.060035448240277','24632.1377705985','1.11427192189782','15.1042570987772','6','86']
  H_HDV['34_CO_-4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.12452783759552','0.005829803418102','-0.098954738956734','0.042498844060973','0','6','86']
  H_HDV['34_THC_-4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.423491336117467','0.052679806295327','0.451567297738945','0.350531348835137','0.030848727267122','6','86']
  H_HDV['34_NOx_-4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.84650341741594','0.055918179517592','99.8700751138033','0.651646110277297','0.225183469352924','6','86']
  H_HDV['34_PM_-4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.8274420377261','-8.62529167093538','-1.28444894134893','0','0','6','86']
  H_HDV['34_FC_-4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','123.792904473985','0.046331801815436','6772.63368956306','0.849099788063095','8.3775282750246','6','86']
  H_HDV['34_CO_-6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.96412961821109','0.011807651602407','-0.089865771729376','0.035280473109375','0','6','72']
  H_HDV['34_THC_-6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.534166952079754','0.00045315405605','-0.030756787468823','0.058070609922328','0','6','72']
  H_HDV['34_NOx_-6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.20740887069612','0.072212417656484','101.818829889703','0.684370519297947','0.162539735547939','6','72']
  H_HDV['34_PM_-6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.056207085013026','0.052993381287839','0.138188877053007','0.053128484782813','0.016685605980152','6','72']
  H_HDV['34_FC_-6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','139.718427833346','0.06307275736912','11512.2120801273','0.975304065566887','6.07420799496004','6','72']
  H_HDV['34_CO_-6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.91808373840592','0.005422977682334','-0.095657769665841','0.04416135844454','0','6','72']
  H_HDV['34_THC_-6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.42907860444409','0.059178692437889','0.860992897033081','0.491147192936227','0.023439406554998','6','72']
  H_HDV['34_NOx_-6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.35045307116668','0.064405676343662','119.452362438261','0.694158909126008','0.12346023562565','6','72']
  H_HDV['34_PM_-6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065700943468337','0.053070231918772','0.120953775587773','0.053722009082309','0.015862900845639','6','72']
  H_HDV['34_FC_-6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','122.690431575113','0.058572003886558','7843.34132666763','0.889015072529712','4.33959216808077','6','72']
  H_HDV['34_CO_-6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.84786562855871','0.004197623919981','-0.093014748000365','0.045703197203253','0','6','72']
  H_HDV['34_THC_-6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.387520660719722','0.056781373102681','1.00108079245168','0.466004569528211','0.020962870682458','6','72']
  H_HDV['34_NOx_-6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.92612123675803','0.059866505005723','142.964722836943','0.718248131711705','0.101398768832165','6','72']
  H_HDV['34_PM_-6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.145267876150847','0.05170652457279','0.035122853719708','0.059561076367515','0.015523105373631','6','72']
  H_HDV['34_FC_-6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','112.693203869135','0.055985078331338','7691.67617274496','0.874814489725604','3.70009499285284','6','72']
  H_HDV['34_CO_2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','2','0','CO','y=(c+(a*exp(b*x)))','2.4720864350644','-0.064642040711251','0.460190603051719','0','0','6','86']
  H_HDV['34_THC_2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.4667323435016','0.038756686410539','0.286211324109405','0.187972902048029','0.067729831503213','6','86']
  H_HDV['34_NOx_2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-16.4022952987179','0.049835504904496','26.6344246086759','0.070878622229437','3.59870214551476','6','86']
  H_HDV['34_PM_2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.186551727578608','0.000352627561788','-0.005282261723346','0.030444957520275','0','6','86']
  H_HDV['34_FC_2_0'] = [' 34','EURO3','RT <=7.5t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','294.515222287776','0.925095202230811','-13.5812292827178','0.063389143704161','0','6','86']
  H_HDV['34_CO_2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','2','50','CO','y=(c+(a*exp(b*x)))','2.80365841315423','-0.067558858920068','0.501533857991949','0','0','6','86']
  H_HDV['34_THC_2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.460186185625919','0.044289462469557','0.259220017862916','0.163674872053327','0.084815553356537','6','86']
  H_HDV['34_NOx_2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.6964388162324','0.013062367066874','-1.39667634306841','0.108644759044472','0','6','86']
  H_HDV['34_PM_2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.191133155681377','0.00083281221357','-0.005426861679612','0.027094234792631','0','6','86']
  H_HDV['34_FC_2_50'] = [' 34','EURO3','RT <=7.5t Euro-3','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.02486697419354','1.79837326871144','481.102284348882','-0.3694587795187','0','6','86']
  H_HDV['34_CO_2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.177994154780034','0.033105590170658','-0.000130669620235','0','0','6','86']
  H_HDV['34_THC_2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.424806358480379','0.047641814924327','0.281581124448324','0.140137218255987','0.098174198852697','6','86']
  H_HDV['34_NOx_2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.0584919943395','0.013285740921409','-1.31151291562854','0.102255929141686','0','6','86']
  H_HDV['34_PM_2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000349396655','0.00007706515696','-0.005663552793755','0.19928065871343','0','6','86']
  H_HDV['34_FC_2_100'] = [' 34','EURO3','RT <=7.5t Euro-3','2','100','FC','y=((a*(x**b))+(c*(x**d)))','0.039359873697664','1.7010963731877','480.677748803477','-0.331804481669418','0','6','86']
  H_HDV['34_CO_4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.41719202000177','0.002884318196041','-0.20037852255558','0.066321027850091','0','6','71']
  H_HDV['34_THC_4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','4','0','THC','y=exp((a+(b/x))+(c*log(x)))','1.78569749849168','-4.27727351811825','-0.935234745029241','0','0','6','71']
  H_HDV['34_NOx_4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.7807257713844','0.019591932184899','-1.4565933993078','0.105490505127476','0','6','71']
  H_HDV['34_PM_4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.191397322486344','0.000889155175335','-0.005282817758704','0.026797271797085','0','6','71']
  H_HDV['34_FC_4_0'] = [' 34','EURO3','RT <=7.5t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','577.107626136937','1.0105581183772','-0.461945082384677','0','0','6','71']
  H_HDV['34_CO_4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.42649959217364','-0.008504030651081','-0.357743410048025','0.103975203556777','0','6','71']
  H_HDV['34_THC_4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.075230395049194','0.559721239096064','5.44372262038211','2.09481007962109','-0.017031505527826','6','71']
  H_HDV['34_NOx_4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-7.83932644089251','0.071530698274003','21.7097843875921','0.099635416535014','4.6149107906785','6','71']
  H_HDV['34_PM_4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000354303627','0.000081968136669','-0.005907298180441','0.206152224191416','0','6','71']
  H_HDV['34_FC_4_50'] = [' 34','EURO3','RT <=7.5t Euro-3','4','50','FC','y=((a*(x**b))+(c*(x**d)))','0.520172718445675','1.19076271414442','547.928714270841','-0.351014343409157','0','6','71']
  H_HDV['34_CO_4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.29749790287427','-0.011542747386423','-0.47418072945363','0.115605549665169','0','6','70']
  H_HDV['34_THC_4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108534787443323','0.523854045795417','5.66926482404201','2.17823079694203','-0.008789524157765','6','70']
  H_HDV['34_NOx_4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','4','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','5.29840743587102','16.5207910284534','0.914781794642515','0.1933975725471','0.140811367998042','6','70']
  H_HDV['34_PM_4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000707458734','0.000127567772747','-0.00762992667022','0.230259388769482','0','6','70']
  H_HDV['34_FC_4_100'] = [' 34','EURO3','RT <=7.5t Euro-3','4','100','FC','y=((a*(b**x))*(x**c))','567.111701322356','1.00767732764743','-0.335014189215922','0','0','6','70']
  H_HDV['34_CO_6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.69979667830092','-0.008352454608154','-0.430167782122588','0.116676380722545','0','6','71']
  H_HDV['34_THC_6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.74742654992307','0.001721591974537','-0.035974279238361','0.049958918060496','0','6','71']
  H_HDV['34_NOx_6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.9456103689094','0.009322288690928','-1.93046652716545','0.123890890412712','0','6','71']
  H_HDV['34_PM_6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000311563176','0.000082156698318','-0.006135061881409','0.212924319248552','0','6','71']
  H_HDV['34_FC_6_0'] = [' 34','EURO3','RT <=7.5t Euro-3','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.55351815895859','-0.038103271476666','197.523730316475','0.073704705779991','167.789718962662','6','71']
  H_HDV['34_CO_6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.9966092875518','-0.012069694915312','-0.642058677643753','0.13623054915283','0','6','67']
  H_HDV['34_THC_6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','6','50','THC','y=(c+(a*exp(b*x)))','0.678745131035667','-0.070247643077763','0.130090898314003','0','0','6','67']
  H_HDV['34_NOx_6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','6','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','5.86873090451148','15.0008776621694','0.6555581066558','-0.274515554517526','0.202768752179853','6','67']
  H_HDV['34_PM_6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000562665687','0.000116883539088','-0.007539708197553','0.23650792732974','0','6','67']
  H_HDV['34_FC_6_50'] = [' 34','EURO3','RT <=7.5t Euro-3','6','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000101912209683','3.13674522468606','505.042732259336','-0.217236733745046','0','6','67']
  H_HDV['34_CO_6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','6','100','CO','y=(c+(a*exp(b*x)))','5.2315586259286','-0.087084458792356','0.780435306795949','0','0','6','56']
  H_HDV['34_THC_6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.905503319781658','-0.002334061291629','-0.07065161795896','0.106884546876334','0','6','56']
  H_HDV['34_NOx_6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000182461630381','0.022941601256861','-0.937295080740033','19.373525838673','0','6','56']
  H_HDV['34_PM_6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.063559016047166','0.179057060034267','4.79753686111135','1.75971497593319','-0.00573388018778','6','56']
  H_HDV['34_FC_6_100'] = [' 34','EURO3','RT <=7.5t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','92.1390025724737','0.040073942192723','1012.95367373733','0.485592676687791','264.587879594173','6','56']
  H_HDV['35_CO_0_0'] = [' 35','EURO4','RT <=7.5t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.028623352307686','0.528053206465334','-0.010777563446059','0.483476937947164','0.036421154052574','6','86']
  H_HDV['35_THC_0_0'] = [' 35','EURO4','RT <=7.5t Euro-4','0','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','21.0762451166817','2.30197711040746','0.008866997248096','0','0','6','86']
  H_HDV['35_NOx_0_0'] = [' 35','EURO4','RT <=7.5t Euro-4','0','0','NOx','y=((a*(b**x))*(x**c))','17.3496888639386','1.01968518306813','-0.898947934496231','0','0','6','86']
  H_HDV['35_PM_0_0'] = [' 35','EURO4','RT <=7.5t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025474481022844','0.037701021529164','0.07225133053656','0.167316415128695','0.005727576343323','6','86']
  H_HDV['35_FC_0_0'] = [' 35','EURO4','RT <=7.5t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.654281872637357','-0.044412202760045','190.756285128343','0.081763445322372','73.0598302448656','6','86']
  H_HDV['35_CO_0_50'] = [' 35','EURO4','RT <=7.5t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031240862670378','0.573353408054832','-0.039268882352246','0.498883262568034','0.038397084023931','6','86']
  H_HDV['35_THC_0_50'] = [' 35','EURO4','RT <=7.5t Euro-4','0','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','20.1411188422034','2.49039216016707','0.007626348849119','0','0','6','86']
  H_HDV['35_NOx_0_50'] = [' 35','EURO4','RT <=7.5t Euro-4','0','50','NOx','y=((a*(b**x))*(x**c))','15.2832157750178','1.01661409126031','-0.803918129100159','0','0','6','86']
  H_HDV['35_PM_0_50'] = [' 35','EURO4','RT <=7.5t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.026101946262237','0.039068293399305','0.072345086683086','0.168339868083922','0.005937834233893','6','86']
  H_HDV['35_FC_0_50'] = [' 35','EURO4','RT <=7.5t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.444182617447664','-0.047521056899194','179.623239920551','0.075265353328602','80.7747877793519','6','86']
  H_HDV['35_CO_0_100'] = [' 35','EURO4','RT <=7.5t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031265809930247','0.377069703225284','1.08699534594821','0.784749795728359','0.0256882461935','6','86']
  H_HDV['35_THC_0_100'] = [' 35','EURO4','RT <=7.5t Euro-4','0','100','THC','y=exp((a+(b/x))+(c*log(x)))','-0.965837929497559','-4.52277346560021','-1.04119289978959','0','0','6','86']
  H_HDV['35_NOx_0_100'] = [' 35','EURO4','RT <=7.5t Euro-4','0','100','NOx','y=((a*(b**x))*(x**c))','13.6492164883494','1.01369774112229','-0.715548302121433','0','0','6','86']
  H_HDV['35_PM_0_100'] = [' 35','EURO4','RT <=7.5t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025183834548829','0.037056547639275','0.073413847702404','0.164490368472291','0.005854302958188','6','86']
  H_HDV['35_FC_0_100'] = [' 35','EURO4','RT <=7.5t Euro-4','0','100','FC','y=((a*(x**b))+(c*(x**d)))','0.00020349695663','2.73071381496067','423.641131458929','-0.399280107138907','0','6','86']
  H_HDV['35_CO_-2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','0','CO','y=(c+(a*exp(b*x)))','0.151896612719762','-0.04977987725858','0.022588851574094','0','0','6','86']
  H_HDV['35_THC_-2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.02300086173902','0.038470752198737','0.017211391754867','0.205168073487267','0.003068878619864','6','86']
  H_HDV['35_NOx_-2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.97040231425068','0.007485947743115','-0.395355565989519','0.086777383681951','0','6','86']
  H_HDV['35_PM_-2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000653673545618','0.061984568708365','4.57404640145222','1.90581081741467','-0.017878054303178','6','86']
  H_HDV['35_FC_-2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002541417414366','0.000621280787322','-0.000005474446488','0','0','6','86']
  H_HDV['35_CO_-2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.174347353502275','0.00007672919129','-0.007749302074855','0.049430027863691','0','6','86']
  H_HDV['35_THC_-2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003329263291445','0.097968272471268','-0.349945547486286','0.362448772339102','0.036186206534076','6','86']
  H_HDV['35_NOx_-2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.84172417626512','0.005223619159569','-0.372516310807504','0.085330028608066','0','6','86']
  H_HDV['35_PM_-2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000081370435551','0.061877316268135','4.65050984414738','1.93614023478125','-0.019753523276021','6','86']
  H_HDV['35_FC_-2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000057598135719','2.91059075252442','504.150692967651','-0.626588598822876','0','6','86']
  H_HDV['35_CO_-2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.177817487609376','0.000084207147254','-0.008333485961796','0.051757284789466','0','6','86']
  H_HDV['35_THC_-2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003486889641951','0.102229981471656','-0.466299855939488','0.326429818128475','0.042338481028257','6','86']
  H_HDV['35_NOx_-2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','100','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.072254560506981','0.042087065626557','-0.000328890328437','0','0','6','86']
  H_HDV['35_PM_-2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000801586273104','0.062918109468917','4.59773682341648','1.92513615028159','-0.020559228923742','6','86']
  H_HDV['35_FC_-2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','110372.527346307','1.36948885157098','151.049727471511','0.069023100001093','43.6416023757336','6','86']
  H_HDV['35_CO_-4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.15201031872387','0.000494811242015','-0.00682013209729','0.041871799492067','0','6','86']
  H_HDV['35_THC_-4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.032544508608981','0.000050385823446','-0.001695527093716','0.053275435850292','0','6','86']
  H_HDV['35_NOx_-4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','175.83591580186','0.97669874818387','3.50705928189087','0.094038128850509','0.337859442372298','6','86']
  H_HDV['35_PM_-4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004229028615722','0.051891433163919','4.87496906219363','1.84719881808495','0.021210980597036','6','86']
  H_HDV['35_FC_-4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','194.70794444431','0.369811028391044','-11.9819678891225','0.062012967031886','0','6','86']
  H_HDV['35_CO_-4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.146828330207709','0.000375027588611','-0.00681774536359','0.043190589627716','0','6','86']
  H_HDV['35_THC_-4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.031778700864721','0.000027707274245','-0.001739876144799','0.056378969564979','0','6','86']
  H_HDV['35_NOx_-4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.7209158490919','0.069762983757061','70.4823772996583','0.756576421686062','0.198072660541853','6','86']
  H_HDV['35_PM_-4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002790757696703','0.05654551863178','4.17038075574537','1.63736350024549','0.021069884267043','6','86']
  H_HDV['35_FC_-4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','139.229140996983','0.060375786651957','23018.6286378124','1.08865543160415','12.9951485735668','6','86']
  H_HDV['35_CO_-4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','100','CO','y=(c+(a*exp(b*x)))','0.141231888510787','-0.054727827371503','0.006972528937112','0','0','6','86']
  H_HDV['35_THC_-4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.024990690008851','0.053929263905344','0.023279477928748','0.336866071067065','0.00161230574025','6','86']
  H_HDV['35_NOx_-4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.18992696869889','0.054341318679778','42.3508373753412','0.62570715252615','0.111077319291861','6','86']
  H_HDV['35_PM_-4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','0.966222700459993','-7.26281398346239','-1.59790497067684','0','0','6','86']
  H_HDV['35_FC_-4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','118.797893258249','0.046649369575646','6952.03914353022','0.845504716116381','6.31369833626324','6','86']
  H_HDV['35_CO_-6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.130762782362342','0.000978445451911','-0.005742380401716','0.031445613718612','0','6','72']
  H_HDV['35_THC_-6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.030478816608271','0.000045041218471','-0.001694322846239','0.05365274193096','0','6','72']
  H_HDV['35_NOx_-6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.51979512759568','0.071281576569908','51.0505769708477','0.697534653554182','0.054827985311123','6','72']
  H_HDV['35_PM_-6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.77805453953298','-9.54207107633744','-1.85442964502892','0','0','6','72']
  H_HDV['35_FC_-6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','133.350336710532','0.062786956115227','10482.0262524135','0.948569514728551','3.61701875227965','6','72']
  H_HDV['35_CO_-6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.125082174224159','0.000459683274691','-0.00601411941249','0.040394347106113','0','6','72']
  H_HDV['35_THC_-6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025628824790647','0.059695402956072','0.043520510491394','0.487751493122868','0.001072714293045','6','72']
  H_HDV['35_NOx_-6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.21409910805957','0.066760676596337','49.598564617919','0.678579677283047','0.030690507123988','6','72']
  H_HDV['35_PM_-6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.039339973740776','0.064122835835842','0.042892926725475','0.174737238094274','0.001396211852156','6','72']
  H_HDV['35_FC_-6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','116.733362171784','0.058578048351166','8039.04536244335','0.88451455649025','1.87791058395895','6','72']
  H_HDV['35_CO_-6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.119575178056292','0.000330837941954','-0.005909103462168','0.043297681889138','0','6','72']
  H_HDV['35_THC_-6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.023272778940705','0.057570066315084','0.054035587444431','0.466943809346381','0.000941928781492','6','72']
  H_HDV['35_NOx_-6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.04334747312605','0.064419735182467','53.4419074765309','0.684052121290856','0.022407213539241','6','72']
  H_HDV['35_PM_-6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.038069210289628','0.063137188005379','0.051834299226485','0.206024234444275','0.001257425353619','6','72']
  H_HDV['35_FC_-6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','106.65288865655','0.056127295623614','7900.9947141034','0.872621250230217','1.24058798549523','6','72']
  H_HDV['35_CO_2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','2','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000000907085335','0.000161466311841','-0.00945386247466','0.231324573321901','0','6','86']
  H_HDV['35_THC_2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025669323072185','0.036359994674155','0.019375779493942','0.174867796212505','0.0022861238603','6','86']
  H_HDV['35_NOx_2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.15974645609234','0.012675821273486','-0.61133212748735','0.10569948113588','0','6','86']
  H_HDV['35_PM_2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.018420143728196','0.014275119034684','0.071281412001518','0.116015744456859','0.001879959499247','6','86']
  H_HDV['35_FC_2_0'] = [' 35','EURO4','RT <=7.5t Euro-4','2','0','FC','y=((a*(b**x))*(x**c))','635.282651066475','1.01418504237095','-0.614647578653731','0','0','6','86']
  H_HDV['35_CO_2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.343965539494094','-0.000377548115653','-0.031445221472765','0.115296294487608','0','6','86']
  H_HDV['35_THC_2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','2','50','THC','y=exp((a+(b/x))+(c*log(x)))','-0.557366592431118','-5.76715803538864','-1.12226642212695','0','0','6','86']
  H_HDV['35_NOx_2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.59067924790889','0.009267910965185','-0.637679433328008','0.110528991385679','0','6','86']
  H_HDV['35_PM_2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.092658996305285','-0.000110562899405','-0.008213053249927','0.106562430803824','0','6','86']
  H_HDV['35_FC_2_50'] = [' 35','EURO4','RT <=7.5t Euro-4','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.039835499545132','1.6904571898645','497.588868008918','-0.400431384186231','0','6','86']
  H_HDV['35_CO_2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.412900877777548','-0.000625225154979','-0.043105462631989','0.131680015397865','0','6','86']
  H_HDV['35_THC_2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.023165729234982','0.044712327335861','0.019825091654548','0.138854840586596','0.00375409680392','6','86']
  H_HDV['35_NOx_2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.79207563985356','0.006152154132192','-0.598803690441009','0.107576973764461','0','6','86']
  H_HDV['35_PM_2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.097956714305805','-0.000122816489852','-0.009033353861993','0.111008348229289','0','6','86']
  H_HDV['35_FC_2_100'] = [' 35','EURO4','RT <=7.5t Euro-4','2','100','FC','y=((a*(x**b))+(c*(x**d)))','0.063970461390696','1.5957389591142','498.09656341022','-0.364086816464766','0','6','86']
  H_HDV['35_CO_4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.325385966577626','0.000799256131751','-0.02080418414935','0.068088967460398','0','6','71']
  H_HDV['35_THC_4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','4','0','THC','y=exp((a+(b/x))+(c*log(x)))','-0.284920240873683','-6.55038256285079','-1.19039387194418','0','0','6','71']
  H_HDV['35_NOx_4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.90075182698812','0.016241426473875','-0.602298104081665','0.098811428969965','0','6','71']
  H_HDV['35_PM_4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006674912393579','0.064798707535484','4.83581964042912','2.03528230938421','-0.01441150065858','6','71']
  H_HDV['35_FC_4_0'] = [' 35','EURO4','RT <=7.5t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','596.753573450623','1.01120970832528','-0.493907213500312','0','0','6','71']
  H_HDV['35_CO_4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.483509743972405','-0.00077214774102','-0.052318637465816','0.134724994555292','0','6','71']
  H_HDV['35_THC_4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003096670347647','0.033841657410653','4.72640682753179','1.82132297488527','-0.003598224697795','6','71']
  H_HDV['35_NOx_4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.43185343213448','0.001306544428542','-0.882547884449804','0.135128614533858','0','6','71']
  H_HDV['35_PM_4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.019115660464241','0.01856492768908','0.080870261116506','0.118587705233539','0.004631717992165','6','71']
  H_HDV['35_FC_4_50'] = [' 35','EURO4','RT <=7.5t Euro-4','4','50','FC','y=((a*(b**x))*(x**c))','598.7738099453','1.01059063444585','-0.437654513165336','0','0','6','71']
  H_HDV['35_CO_4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.582800254364419','-0.001229632304189','-0.06907876033072','0.1488181956434','0','6','70']
  H_HDV['35_THC_4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003059065121915','0.032924649549109','5.40843301951713','2.12596774751015','-0.014779186727733','6','70']
  H_HDV['35_NOx_4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.0441735429119','-0.002783248827342','-0.896714307806215','0.138214272171432','0','6','70']
  H_HDV['35_PM_4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.026032497070505','0.021993078539812','0.084831397344944','0.131893787592523','0.003647912317949','6','70']
  H_HDV['35_FC_4_100'] = [' 35','EURO4','RT <=7.5t Euro-4','4','100','FC','y=((a*(b**x))*(x**c))','582.925709055905','1.00846227416281','-0.36542786684357','0','0','6','70']
  H_HDV['35_CO_6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.61409578793376','-0.001044184531254','-0.091687954773029','0.184061449170999','0','6','71']
  H_HDV['35_THC_6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.042456287372267','0.000067062714201','-0.002264366711984','0.054775839108877','0','6','71']
  H_HDV['35_NOx_6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','6','0','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00004826152725','0.006816380624169','-0.304632735712729','7.40694396907865','0','6','71']
  H_HDV['35_PM_6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.108314023310428','-0.000154827365318','-0.010636337937079','0.119509194099572','0','6','71']
  H_HDV['35_FC_6_0'] = [' 35','EURO4','RT <=7.5t Euro-4','6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.51011079757166','-0.038835288376073','207.669763138285','0.07496772600173','156.043389024852','6','71']
  H_HDV['35_CO_6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.802809080718718','-0.001463876623712','-0.133655985006088','0.202933556795544','0','6','68']
  H_HDV['35_THC_6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','6','50','THC','y=(c+(a*exp(b*x)))','0.040685077379072','-0.072866282511518','0.005168290931331','0','0','6','68']
  H_HDV['35_NOx_6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.6919339212198','-0.002135899728293','-1.2764317095128','0.163967409471071','0','6','68']
  H_HDV['35_PM_6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.129534842439528','-0.00024083509397','-0.015269681044352','0.144758073520548','0','6','68']
  H_HDV['35_FC_6_50'] = [' 35','EURO4','RT <=7.5t Euro-4','6','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000396563500135','2.83429187224322','519.543394484508','-0.244355375230639','0','6','68']
  H_HDV['35_CO_6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.740500578400125','-0.000527981555307','-0.083222000487818','0.131567818308085','0','6','58']
  H_HDV['35_THC_6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','6','100','THC','y=exp((a+(b/x))+(c*log(x)))','-0.962010486628807','-3.82785879233125','-1.03012643496347','0','0','6','58']
  H_HDV['35_NOx_6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000061579067123','0.008157302313026','-0.351118703226458','9.31928738113785','0','6','58']
  H_HDV['35_PM_6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.125195340523444','-0.000249318451143','-0.012312889529209','0.11965411707693','0','6','58']
  H_HDV['35_FC_6_100'] = [' 35','EURO4','RT <=7.5t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','96.8879826909832','0.061903413573662','929.152051197869','0.47566833128068','262.381744468394','6','58']
  H_HDV['36_CO_0_0'] = [' 36','EURO5','RT <=7.5t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.028342496981756','0.534024282213433','-0.000726538718738','0.489966633943005','0.035485785312939','6','86']
  H_HDV['36_THC_0_0'] = [' 36','EURO5','RT <=7.5t Euro-5','0','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','20.8928743602391','2.26648138684051','0.008821324281633','0','0','6','86']
  H_HDV['36_NOx_0_0'] = [' 36','EURO5','RT <=7.5t Euro-5','0','0','NOx','y=((a*(b**x))*(x**c))','13.8772825283467','1.02106618257553','-0.996825967822448','0','0','6','86']
  H_HDV['36_PM_0_0'] = [' 36','EURO5','RT <=7.5t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025714295216862','0.037528256182305','0.073060876597034','0.1669814169925','0.005740630828961','6','86']
  H_HDV['36_FC_0_0'] = [' 36','EURO5','RT <=7.5t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.598290449402457','-0.045297665283896','196.34070806008','0.08187119888562','74.4052389994931','6','86']
  H_HDV['36_CO_0_50'] = [' 36','EURO5','RT <=7.5t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031000113056944','0.566056517180494','0.003295129145601','0.507125593826194','0.037523843082865','6','86']
  H_HDV['36_THC_0_50'] = [' 36','EURO5','RT <=7.5t Euro-5','0','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','19.9693179111002','2.45266399009596','0.007595687766661','0','0','6','86']
  H_HDV['36_NOx_0_50'] = [' 36','EURO5','RT <=7.5t Euro-5','0','50','NOx','y=((a*(b**x))*(x**c))','12.2272866475602','1.01807615667254','-0.903548251678021','0','0','6','86']
  H_HDV['36_PM_0_50'] = [' 36','EURO5','RT <=7.5t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.026368787185743','0.038943219125929','0.073053308390647','0.16798935853418','0.005960429479955','6','86']
  H_HDV['36_FC_0_50'] = [' 36','EURO5','RT <=7.5t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.40527196926134','-0.048431667805289','184.930487449033','0.075369679863232','82.1323200474053','6','86']
  H_HDV['36_CO_0_100'] = [' 36','EURO5','RT <=7.5t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.030784585343842','0.306006456511071','1.68849365736963','0.920148128124972','0.021812586431545','6','86']
  H_HDV['36_THC_0_100'] = [' 36','EURO5','RT <=7.5t Euro-5','0','100','THC','y=exp((a+(b/x))+(c*log(x)))','-0.942889003712714','-4.56761079873303','-1.04364290509989','0','0','6','86']
  H_HDV['36_NOx_0_100'] = [' 36','EURO5','RT <=7.5t Euro-5','0','100','NOx','y=((a*(x**b))+(c*(x**d)))','0.004325146796838','1.11930693800889','9.7011290930036','-0.708212543687891','0','6','86']
  H_HDV['36_PM_0_100'] = [' 36','EURO5','RT <=7.5t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025468521626216','0.037091094257915','0.074067293787117','0.164224591186064','0.005889110420764','6','86']
  H_HDV['36_FC_0_100'] = [' 36','EURO5','RT <=7.5t Euro-5','0','100','FC','y=((a*(x**b))+(c*(x**d)))','0.00020642909271','2.7272587648907','436.63656362847','-0.403140673260641','0','6','86']
  H_HDV['36_CO_-2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','0','CO','y=(c+(a*exp(b*x)))','0.153690374382204','-0.049676745428957','0.022719719231221','0','0','6','86']
  H_HDV['36_THC_-2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.023318499536551','0.038662075112121','0.017318506504783','0.205358634100784','0.003135320771342','6','86']
  H_HDV['36_NOx_-2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.59442003886594','0.003676866780714','-0.32220223942721','0.097092522692867','0','6','86']
  H_HDV['36_PM_-2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000589976786394','0.062848407655019','4.56402292239106','1.90168863094561','-0.017856699444152','6','86']
  H_HDV['36_FC_-2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000629242898323','2.43717227396393','556.155114510082','-0.658809245806161','0','6','86']
  H_HDV['36_CO_-2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.175633955381053','0.000085262399178','-0.007752143955086','0.048903748699449','0','6','86']
  H_HDV['36_THC_-2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003396270014568','0.098524796466954','-0.34943426264004','0.357443586044524','0.036668416623082','6','86']
  H_HDV['36_NOx_-2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','50','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.010357597583603','0.077802095730374','-0.000649481393878','0','0','6','86']
  H_HDV['36_PM_-2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000111196429356','0.062614231725738','4.64885037396139','1.93443840796304','-0.019673750287615','6','86']
  H_HDV['36_FC_-2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002602008949697','0.000595768410985','-0.00000487633714','0','0','6','86']
  H_HDV['36_CO_-2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.179493487816017','0.000090906266787','-0.008375707230659','0.051382905688314','0','6','86']
  H_HDV['36_THC_-2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003539577945513','0.102034211860402','-0.451479004346754','0.324162436379503','0.042634634787601','6','86']
  H_HDV['36_NOx_-2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.46957556606654','0.091139656681612','88.9689719405345','0.894348159434531','0.410573554020216','6','86']
  H_HDV['36_PM_-2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000813145976747','0.0636198534852','4.60103617335762','1.92546296185393','-0.020488803604535','6','86']
  H_HDV['36_FC_-2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-2','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002846728409459','0.000556900541171','-0.000004055075248','0','0','6','86']
  H_HDV['36_CO_-4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.153775250641556','0.000507909945495','-0.006877012471793','0.041588035357302','0','6','86']
  H_HDV['36_THC_-4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.032921349169491','0.000051670242347','-0.001709507153343','0.053000514259794','0','6','86']
  H_HDV['36_NOx_-4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.45840415054603','0.096374345993113','78.6133831183924','0.893833897057983','0.194563729333673','6','86']
  H_HDV['36_PM_-4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004224166514873','0.052169509422449','4.97434236778308','1.88605788685136','0.019524795795014','6','86']
  H_HDV['36_FC_-4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','200.160094040151','0.358954887406831','-12.3705268964549','0.062498589605789','0','6','86']
  H_HDV['36_CO_-4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.148437974356296','0.000372051538134','-0.006885141589652','0.043205411551343','0','6','86']
  H_HDV['36_THC_-4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.032155846898499','0.00002708016726','-0.001758546770243','0.056371080384522','0','6','86']
  H_HDV['36_NOx_-4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.82967501165726','0.072144083417818','29.8983931572733','0.658728456608132','0.114407522722331','6','86']
  H_HDV['36_PM_-4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002750408618529','0.05706605513395','4.21665648330659','1.65830494078474','0.019674824430958','6','86']
  H_HDV['36_FC_-4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','50','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','11.331750338183','1859.21403916401','-1.50454904592011','0.509747717638679','0.036023793992053','6','86']
  H_HDV['36_CO_-4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','100','CO','y=(c+(a*exp(b*x)))','0.142484123403255','-0.054427241591728','0.006910707876015','0','0','6','86']
  H_HDV['36_THC_-4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025203584641621','0.053595266833028','0.023259548573376','0.331744286144793','0.001607528688206','6','86']
  H_HDV['36_NOx_-4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.47133111256467','0.057563441739314','22.6150616101166','0.572198644362453','0.066803442397635','6','86']
  H_HDV['36_PM_-4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.0056335055736','-7.35831908823264','-1.60507399161329','0','0','6','86']
  H_HDV['36_FC_-4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','121.755993835149','0.046529120981813','6758.60193108816','0.835717006422661','6.27794162175323','6','86']
  H_HDV['36_CO_-6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.132740945248894','0.000964749085843','-0.005849208190267','0.031814471107203','0','6','72']
  H_HDV['36_THC_-6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.030903615142919','0.000044702624966','-0.001719325914269','0.053757696241339','0','6','72']
  H_HDV['36_NOx_-6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70099183572373','0.073957354516114','25.6389185147342','0.631046755482763','0.032659021248306','6','72']
  H_HDV['36_PM_-6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.81004459170282','-9.60413551621217','-1.85965521690286','0','0','6','72']
  H_HDV['36_FC_-6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','136.853104762572','0.062680115256445','10227.9113650665','0.939091480390152','3.62496579281849','6','72']
  H_HDV['36_CO_-6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.126730692310273','0.000462575370059','-0.006099181398326','0.04047743647364','0','6','72']
  H_HDV['36_THC_-6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.026027002017481','0.059787385098046','0.044128552962184','0.490929991971895','0.001085625483624','6','72']
  H_HDV['36_NOx_-6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.4982130023683','0.06962282676511','26.4283938728826','0.62500455291181','0.019097394872837','6','72']
  H_HDV['36_PM_-6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.039528530201053','0.063979060563433','0.043241960629348','0.172202146878438','0.001411182780569','6','72']
  H_HDV['36_FC_-6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','119.933403554724','0.058643387362587','8024.1247739511','0.879853365661716','1.9061759963877','6','72']
  H_HDV['36_CO_-6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.1213304306369','0.000327670727394','-0.006015824963535','0.043586549521417','0','6','72']
  H_HDV['36_THC_-6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.023581082078888','0.057592150696235','0.053633501787347','0.46335590475222','0.000950896610483','6','72']
  H_HDV['36_NOx_-6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.37868150460945','0.067179873422536','28.6528931363267','0.631273513498277','0.014257202926866','6','72']
  H_HDV['36_PM_-6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.0384823860131','0.063112290574213','0.052194081397362','0.20466682108252','0.001269661847691','6','72']
  H_HDV['36_FC_-6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','109.350160565721','0.056186240326676','7759.67106494049','0.864548963877302','1.26419423939113','6','72']
  H_HDV['36_CO_2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','2','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00000090386881','0.000161373880261','-0.009484427148531','0.232863213251441','0','6','86']
  H_HDV['36_THC_2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025930490986097','0.036074520524232','0.019706107789384','0.175191182892817','0.002274963037185','6','86']
  H_HDV['36_NOx_2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.18210305365591','0.006554068124721','-0.507334735333938','0.116820989449584','0','6','86']
  H_HDV['36_PM_2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.018465072157063','0.015698972601557','0.071750114757226','0.117260291163309','0.002583936105028','6','86']
  H_HDV['36_FC_2_0'] = [' 36','EURO5','RT <=7.5t Euro-5','2','0','FC','y=((a*(b**x))*(x**c))','653.710560616897','1.01410927216011','-0.617447689641716','0','0','6','86']
  H_HDV['36_CO_2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.347610448618458','-0.000373167284391','-0.032017351580193','0.116084491542013','0','6','86']
  H_HDV['36_THC_2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025563070516313','0.040786559519626','0.018724504717749','0.158783275603136','0.002994871706657','6','86']
  H_HDV['36_NOx_2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-10.5998855741711','0.063269186804626','14.2625093883888','0.076054538660668','1.43054512455514','6','86']
  H_HDV['36_PM_2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.093620465472647','-0.000112376719076','-0.008307743063035','0.106712969873494','0','6','86']
  H_HDV['36_FC_2_50'] = [' 36','EURO5','RT <=7.5t Euro-5','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.034780548877446','1.72000966967132','508.875574765882','-0.400519319498718','0','6','86']
  H_HDV['36_CO_2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.416430752195622','-0.000618497995973','-0.043743340672334','0.132372932303147','0','6','86']
  H_HDV['36_THC_2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.02405393732683','0.045048622679694','0.01953093709478','0.142617809741563','0.00376476426645','6','86']
  H_HDV['36_NOx_2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','2','100','NOx','y=(c+(a*exp(b*x)))','4.54743729105634','-0.134694694203638','1.44428611312176','0','0','6','86']
  H_HDV['36_PM_2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.098774749616831','-0.000122954886265','-0.009089431172008','0.11071621502528','0','6','86']
  H_HDV['36_FC_2_100'] = [' 36','EURO5','RT <=7.5t Euro-5','2','100','FC','y=((a*(x**b))+(c*(x**d)))','0.117974446873533','1.46711626491269','523.558087035601','-0.380197218965685','0','6','86']
  H_HDV['36_CO_4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.327392117535446','0.000773842670779','-0.020963906368396','0.068415479406029','0','6','71']
  H_HDV['36_THC_4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.044938233650368','-0.000098698980247','-0.002950728363366','0.083921331870132','0','6','71']
  H_HDV['36_NOx_4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.58767840237343','0.009315657349961','-0.476453065615','0.104920701382891','0','6','71']
  H_HDV['36_PM_4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006554020929113','0.066831348245977','4.64112834725525','1.96234873765633','-0.012569053113558','6','71']
  H_HDV['36_FC_4_0'] = [' 36','EURO5','RT <=7.5t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','613.078590080004','1.01114578349952','-0.496595933724655','0','0','6','71']
  H_HDV['36_CO_4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.484981063962178','-0.000708645551363','-0.052203933741744','0.133332213185878','0','6','71']
  H_HDV['36_THC_4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002885417921016','0.034407782617767','4.7478510088678','1.83205791100097','-0.005617278650559','6','71']
  H_HDV['36_NOx_4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.81052938478661','-0.001391973379433','-0.725770606006963','0.143376236466854','0','6','71']
  H_HDV['36_PM_4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.019070260333006','0.020670535692949','0.081260840395141','0.119879448082044','0.005562383755052','6','71']
  H_HDV['36_FC_4_50'] = [' 36','EURO5','RT <=7.5t Euro-5','4','50','FC','y=((a*(b**x))*(x**c))','613.609137379674','1.01050201794788','-0.439377340376879','0','0','6','71']
  H_HDV['36_CO_4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.586625163737139','-0.001203986497781','-0.069797953563045','0.148814830400299','0','6','70']
  H_HDV['36_THC_4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00304016047616','0.033281458920992','5.42309231610036','2.12751939312972','-0.015005016766885','6','70']
  H_HDV['36_NOx_4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.24908121619831','-0.005413725869174','-0.745403236769895','0.146314506551826','0','6','70']
  H_HDV['36_PM_4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.025235869191389','0.020690697439447','0.085892464445325','0.128664193595542','0.003439518024229','6','70']
  H_HDV['36_FC_4_100'] = [' 36','EURO5','RT <=7.5t Euro-5','4','100','FC','y=((a*(b**x))*(x**c))','598.193774688892','1.00849823465555','-0.368748598121606','0','0','6','70']
  H_HDV['36_CO_6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.616254913794893','-0.001039606266162','-0.091912578739674','0.183883355818424','0','6','71']
  H_HDV['36_THC_6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.042887213593199','0.00006678683223','-0.002280963825247','0.05466666884368','0','6','71']
  H_HDV['36_NOx_6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.89823139931203','-0.003591404981725','-1.05091413675938','0.177464777946559','0','6','71']
  H_HDV['36_PM_6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.109711839257414','-0.000157614112983','-0.010863916225653','0.120608346812012','0','6','71']
  H_HDV['36_FC_6_0'] = [' 36','EURO5','RT <=7.5t Euro-5','6','0','FC','y=((a*(x**b))+(c*(x**d)))','0.001165898311516','2.59101700183205','521.326574775885','-0.304656038463813','0','6','71']
  H_HDV['36_CO_6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.798807431888381','-0.001461058837533','-0.131555391953996','0.200784867472946','0','6','68']
  H_HDV['36_THC_6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','6','50','THC','y=(c+(a*exp(b*x)))','0.041090333862998','-0.072646233377531','0.005216118526793','0','0','6','68']
  H_HDV['36_NOx_6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.97310694667312','-0.006941466722006','-1.21876628480079','0.187053976658974','0','6','68']
  H_HDV['36_PM_6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.128819662442346','-0.00023389700936','-0.014839376169764','0.141247430170216','0','6','68']
  H_HDV['36_FC_6_50'] = [' 36','EURO5','RT <=7.5t Euro-5','6','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000360921920072','2.86147148164711','532.561242781831','-0.246971700776242','0','6','68']
  H_HDV['36_CO_6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.765977738037549','-0.000799587249677','-0.091074090962318','0.140647548192609','0','6','59']
  H_HDV['36_THC_6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','6','100','THC','y=exp((a+(b/x))+(c*log(x)))','-0.960182881256246','-3.79939649232371','-1.02764776659029','0','0','6','59']
  H_HDV['36_NOx_6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.45122742768471','0.000417751177998','-0.550987220740629','0.10841039793764','0','6','59']
  H_HDV['36_PM_6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.127425970764291','-0.000259365333743','-0.012781253912185','0.122355704321762','0','6','59']
  H_HDV['36_FC_6_100'] = [' 36','EURO5','RT <=7.5t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','108.176444191297','0.06758295387267','1058.5165217051','0.506048919701153','267.326678840293','6','59']
  H_HDV['37_CO_0_0'] = [' 37','EURO0','RT >7.5-12t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.88640121234395','0.054216220668142','9.12834699276328','0.208330131251673','1.312757208525','6','86']
  H_HDV['37_THC_0_0'] = [' 37','EURO0','RT >7.5-12t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.96380041252083','0.027552627329153','5.27116604362621','0.127206466610518','0.29753305917882','6','86']
  H_HDV['37_NOx_0_0'] = [' 37','EURO0','RT >7.5-12t 80ties','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.4176848577849','0.04627266139533','-0.978361042364423','0.061538038017655','0','6','86']
  H_HDV['37_PM_0_0'] = [' 37','EURO0','RT >7.5-12t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.981635885278864','0.051778552865753','1.54092813359065','0.187325738175817','0.211039631333272','6','86']
  H_HDV['37_FC_0_0'] = [' 37','EURO0','RT >7.5-12t 80ties','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001138117808261','0.000187115490944','-0.000001462280062','0','0','6','86']
  H_HDV['37_CO_0_50'] = [' 37','EURO0','RT >7.5-12t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.4769831875209','0.05865685328539','10.5657635823999','0.24552776962419','1.49596932422556','6','86']
  H_HDV['37_THC_0_50'] = [' 37','EURO0','RT >7.5-12t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.0502173880109','0.033019170542139','5.18611901874107','0.129912017540677','0.353513816639592','6','86']
  H_HDV['37_NOx_0_50'] = [' 37','EURO0','RT >7.5-12t 80ties','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.3862757401837','0.039903561730976','-0.820239987619949','0.05493277765175','0','6','86']
  H_HDV['37_PM_0_50'] = [' 37','EURO0','RT >7.5-12t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.13577478932831','0.056905019252404','1.77421696608681','0.234265941809932','0.235763753840974','6','86']
  H_HDV['37_FC_0_50'] = [' 37','EURO0','RT >7.5-12t 80ties','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001221663809674','0.000161539721521','-0.000001222194656','','','6','86']
  H_HDV['37_CO_0_100'] = [' 37','EURO0','RT >7.5-12t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.34500567918787','0.057234770864949','13.2930448346755','0.272385627328846','1.67795680658786','6','86']
  H_HDV['37_THC_0_100'] = [' 37','EURO0','RT >7.5-12t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.161804534855016','6.20985188264266','3.55224988800622','1.58148094344001','-0.006379918046003','6','86']
  H_HDV['37_NOx_0_100'] = [' 37','EURO0','RT >7.5-12t 80ties','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.2614851902601','0.037385227196864','-0.662139139204159','0.046660419432855','0','6','86']
  H_HDV['37_PM_0_100'] = [' 37','EURO0','RT >7.5-12t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.15806408063306','0.056503103236043','2.18141376253128','0.266462250955625','0.260920754516785','6','86']
  H_HDV['37_FC_0_100'] = [' 37','EURO0','RT >7.5-12t 80ties','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001302798733378','0.000138701065559','-0.000001015726615','0','0','6','86']
  H_HDV['37_CO_-2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.4544091751221','0.005576253890968','-0.767281941462409','0.072647805315205','0','6','86']
  H_HDV['37_THC_-2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.511053903079031','4.68279419956924','4.05762835929575','1.57341124667458','0.019384369955618','6','86']
  H_HDV['37_NOx_-2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.4025744827618','0.028647453754515','-1.01773395552488','0.066057325844712','0','6','86']
  H_HDV['37_PM_-2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.67155757854085','0.078316936950128','380.937336853488','1.53715503566747','0.183998379417172','6','86']
  H_HDV['37_FC_-2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.001328573497137','2.3309359149323','1457.16267378743','-0.753840700667232','0','6','86']
  H_HDV['37_CO_-2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.303504166127','0.012651815517427','-0.727602900333367','0.066636637441439','0','6','86']
  H_HDV['37_THC_-2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.83876609067932','0.067222314576958','1.92281773818522','0.120387784806004','0.495009692754291','6','86']
  H_HDV['37_NOx_-2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.6373245658624','0.009481764236986','-0.900010940608803','0.063582863699047','0','6','86']
  H_HDV['37_PM_-2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.8539534157624','0.000907837533924','-0.125201223957468','0.072020769976082','0','6','86']
  H_HDV['37_FC_-2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','501.365042074268','0.421120631945428','-29.7720866908608','0.06544336956165','0','6','86']
  H_HDV['37_CO_-2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.2247145222437','0.014078252216325','-0.701711451020034','0.063593625178326','0','6','86']
  H_HDV['37_THC_-2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.45581090219282','0.069144647352997','1.36557077939019','0.146848748615207','0.423911410057039','6','86']
  H_HDV['37_NOx_-2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.1687494312777','-0.011419584874388','-0.81423851356204','0.062796499315462','0','6','86']
  H_HDV['37_PM_-2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.84028810516653','0.001024909140744','-0.120431661898028','0.068933367979054','0','6','86']
  H_HDV['37_FC_-2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','352.665019913677','0.056931979049053','97333.3108414761','1.18281652544641','56.5099769410102','6','86']
  H_HDV['37_CO_-4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.4687147221058','0.00748971332001','-0.737821148086203','0.071503318151483','0','6','86']
  H_HDV['37_THC_-4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.74699534878065','0.001200254155219','-0.413096557522553','0.075320657094867','0','6','86']
  H_HDV['37_NOx_-4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3041887222976','0.062784666283359','1421.6138647702','1.07789329242459','0.896842775079819','6','86']
  H_HDV['37_PM_-4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.55406643869883','0.07631824506022','43.4918566228475','1.06922842964746','0.095645426403095','6','86']
  H_HDV['37_FC_-4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','339.862616381291','0.066905888384334','51062.6841263022','1.07687542212129','24.2420870160823','6','86']
  H_HDV['37_CO_-4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.37457951104835','0.068534331186969','31.1157972755892','0.575171793362849','0.327009988608925','6','86']
  H_HDV['37_THC_-4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.29966135706535','0.066447109688101','4.02217562065732','0.318600242123984','0.214242753177588','6','86']
  H_HDV['37_NOx_-4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.0824361037499','0.046337268406892','551.229849077673','0.863465039048335','0.324302124880135','6','86']
  H_HDV['37_PM_-4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.2829082806262','0.065447466327973','5.33707272613624','0.55096221510993','0.065097619816706','6','86']
  H_HDV['37_FC_-4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','50','FC','y=((a*(x**b))+(c*(x**d)))','-3.56560088982394','0.441639019792908','1280.85129591973','-0.81326531004263','0','6','86']
  H_HDV['37_CO_-4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.36150821304626','0.062178045805097','37.2949782157549','0.533989427296895','0.239431663133218','6','86']
  H_HDV['37_THC_-4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.99757713659681','0.064225036640601','6.78399170855812','0.371480088958122','0.182357827988641','6','86']
  H_HDV['37_NOx_-4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.33870868862628','0.04030810084831','697.418648167365','0.891478005289821','0.105577636468147','6','86']
  H_HDV['37_PM_-4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.19045625146674','0.061605827656324','7.34221510281418','0.568271778780968','0.056261475304208','6','86']
  H_HDV['37_FC_-4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','246.373960313523','0.045884992978951','19208.543553428','0.857888844836397','4.74089366076916','6','86']
  H_HDV['37_CO_-6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.55358319942123','0.075804122408133','36.8910435875747','0.646317915649302','0.190581288014632','6','72']
  H_HDV['37_THC_-6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.60698032295778','0.07152414077282','7.35506211125526','0.471854170653241','0.147189308035283','6','72']
  H_HDV['37_NOx_-6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.24253398243761','0.061899349942233','845.81189900502','0.962174829375435','0.150831759722659','6','72']
  H_HDV['37_PM_-6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.30034179375293','0.070462649464941','6.22327027159421','0.60567432060933','0.044434969566663','6','72']
  H_HDV['37_FC_-6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','269.662429137822','0.063080855967353','23121.9582571808','0.914871733954676','3.81726468867703','6','72']
  H_HDV['37_CO_-6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.11284944013087','0.070625078051494','47.232967036714','0.600704904293208','0.146456313495579','6','72']
  H_HDV['37_THC_-6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.05639001354592','0.068588018843972','11.5023830328784','0.4779573937542','0.128599349019217','6','72']
  H_HDV['37_NOx_-6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.73500416088038','0.057243523412788','742.723766731744','0.920115682104655','0.086275536993034','6','72']
  H_HDV['37_PM_-6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.14842739772289','0.066747487744689','8.91889301166989','0.622555780656334','0.03911252594907','6','72']
  H_HDV['37_FC_-6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','225.995431752997','0.058782008446148','19630.3483096178','0.870126742054694','1.99519738830042','6','72']
  H_HDV['37_CO_-6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.38241830821629','0.067669615282979','63.291769739219','0.615462541858609','0.130669894508642','6','72']
  H_HDV['37_THC_-6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.77415744732765','0.067368888046485','14.398527720733','0.495562911352946','0.122730316408808','6','72']
  H_HDV['37_NOx_-6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.01273781536776','0.055052742524706','725.163852959837','0.907268207266896','0.070337286397877','6','72']
  H_HDV['37_PM_-6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.06895364059968','0.064621455666671','10.9689283976207','0.633257771149665','0.037082461929065','6','72']
  H_HDV['37_FC_-6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','204.518466212342','0.056898384135402','18894.549318484','0.85707751246754','1.53820549184858','6','72']
  H_HDV['37_CO_2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.4769969944922','0.006751081131573','-0.974876483604731','0.075458158341602','0','6','86']
  H_HDV['37_THC_2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','4.79021163731167','-6.08024940009828','-1.25225563635243','0','0','6','86']
  H_HDV['37_NOx_2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-18.8984081664691','0.025049965492665','30.5270049052563','0.053206068446226','13.411544700705','6','86']
  H_HDV['37_PM_2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.39440434214253','0.000327866602924','-0.170607224423021','0.081252160939164','0','6','86']
  H_HDV['37_FC_2_0'] = [' 37','EURO0','RT >7.5-12t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','718.795939060289','1.20246929425481','-37.8198994932597','0.064586138324368','0','6','86']
  H_HDV['37_CO_2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.630057795059','0.015377845642514','-1.08412831278419','0.077340862017064','0','6','86']
  H_HDV['37_THC_2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.125866184616486','5.63731919229292','4.88057198822456','1.92933914967853','-0.011723129764001','6','86']
  H_HDV['37_NOx_2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.9414052368331','0.047303626498219','-1.15699491899293','0.068638555331024','0','6','86']
  H_HDV['37_PM_2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.55102789053703','0.002232563507579','-0.179207469686719','0.078435689670218','0','6','86']
  H_HDV['37_FC_2_50'] = [' 37','EURO0','RT >7.5-12t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','762.9524480682','1.52391568379483','-38.4421197407641','0.063773512066338','0','6','86']
  H_HDV['37_CO_2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.7110330842197','0.019274632746499','-1.20477528560148','0.08216338151316','0','6','85']
  H_HDV['37_THC_2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.227094913004671','5.49906521751329','5.14512199330146','2.03999706101397','-0.010580848431803','6','85']
  H_HDV['37_NOx_2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.0542189945575','0.043940365382971','-0.982560673098951','0.061961646507143','0','6','85']
  H_HDV['37_PM_2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.74046694207679','0.002872015343636','-0.199496699522477','0.082549968731604','0','6','85']
  H_HDV['37_FC_2_100'] = [' 37','EURO0','RT >7.5-12t 80ties','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','808.45090328458','1.30352003764433','-40.4711392306269','0.068183526165378','0','6','85']
  H_HDV['37_CO_4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.7164429192224','0.018764682095782','-1.24560837394498','0.082830426730313','0','6','71']
  H_HDV['37_THC_4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','4','0','THC','y=exp((a+(b/x))+(c*log(x)))','5.00971153096772','-6.35786191589408','-1.31887237353022','0','0','6','71']
  H_HDV['37_NOx_4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.2024880387577','0.033469031900828','-1.72893190490051','0.091057215628504','0','6','71']
  H_HDV['37_PM_4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.76503355893231','0.002113550767769','-0.211552294585815','0.086144890654195','0','6','71']
  H_HDV['37_FC_4_0'] = [' 37','EURO0','RT >7.5-12t 80ties','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','886.16602836459','0.501012571067685','-57.1412862621008','0.087869229626845','0','6','71']
  H_HDV['37_CO_4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.4067891797788','0.015515672381183','-1.70712532958423','0.103027107142587','0','6','70']
  H_HDV['37_THC_4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.203872482755191','5.82311862994638','5.10614833694557','2.03291865196918','-0.010458457648494','6','70']
  H_HDV['37_NOx_4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.7739729889927','0.015528866616587','-1.58944947008188','0.091777631971891','0','6','70']
  H_HDV['37_PM_4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000018373059555','0.00275051848208','-0.132594872015003','2.61259844035782','0','6','70']
  H_HDV['37_FC_4_50'] = [' 37','EURO0','RT >7.5-12t 80ties','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.004252322880655','0.646453492861237','-31.8161575559982','842.285558879774','0','6','70']
  H_HDV['37_CO_4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.2186384183404','0.030048362949854','-1.70727171197528','0.101373064494255','0','6','66']
  H_HDV['37_THC_4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.418510810616337','5.09324735120805','6.78033387569344','2.67410845631201','-0.019819875528753','6','66']
  H_HDV['37_NOx_4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','50931.4526896524','1.83192036078373','15.2624636515426','0.092580173175233','20.5587038964419','6','66']
  H_HDV['37_PM_4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-2.98288999782235','0.040638806292724','5.26235611827829','0.06974399654753','0.97615314863049','6','66']
  H_HDV['37_FC_4_100'] = [' 37','EURO0','RT >7.5-12t 80ties','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.004415276360958','0.662603959878776','-31.766194021588','897.205049693854','0','6','66']
  H_HDV['37_CO_6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.541323832114','0.091343874896642','-0.996046123817066','0.056968624513981','0','6','70']
  H_HDV['37_THC_6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.413461876144','-0.01207114072831','-0.727650152838035','0.101228232725875','0','6','70']
  H_HDV['37_NOx_6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','6','0','NOx','y=((a*(b**x))*(x**c))','50.9888036298978','1.0082409862001','-0.38395978115703','0','0','6','70']
  H_HDV['37_PM_6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.84645176782783','0.009531188741481','-0.188852909718751','0.068066641007898','0','6','70']
  H_HDV['37_FC_6_0'] = [' 37','EURO0','RT >7.5-12t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','940.009486609924','1.4666455602825','-53.9867159534738','0.079527320902643','0','6','70']
  H_HDV['37_CO_6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','6','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.024728790405752','0.009345980316829','-0.000111672946587','0','0','6','62']
  H_HDV['37_THC_6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.43385169875832','5.05748837295141','7.32980922424398','2.87691391398768','-0.024666987271305','6','62']
  H_HDV['37_NOx_6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','6','50','NOx','y=((a*(b**x))*(x**c))','51.371646160442','1.00588398519056','-0.291103448340263','0','0','6','62']
  H_HDV['37_PM_6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.10340826357117','0.062383314597403','-0.000750084448895','0','0','6','62']
  H_HDV['37_FC_6_50'] = [' 37','EURO0','RT >7.5-12t 80ties','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','998.611260172899','2.66645280042717','-48.5766748477555','0.071350002214391','0','6','62']
  H_HDV['37_CO_6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','5.9252848513481','8.09159686928603','11.4112326776621','5.10112694622892','-0.101449046228751','6','50']
  H_HDV['37_THC_6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.344849542673176','5.18456684947056','7.19365811134131','2.84570206541457','-0.026375278230098','6','50']
  H_HDV['37_NOx_6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0119620201141','0.058164959178714','30.7252182469676','0.445743156113753','25.6055688816147','6','50']
  H_HDV['37_PM_6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.844722783585233','1.65254694258284','9.05320156504635','4.03231620333709','-0.072081683741653','6','50']
  H_HDV['37_FC_6_100'] = [' 37','EURO0','RT >7.5-12t 80ties','6','100','FC','y=((a*(b**x))*(x**c))','1585.84661899209','1.00636742369008','-0.370396316909928','0','0','6','50']
  H_HDV['38_CO_0_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.26462206934743','0.054773011340076','3.13759740411741','0.227611879353399','0.590833546750436','6','86']
  H_HDV['38_THC_0_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.126734796465523','1.99280337707756','3.69960632949904','1.52367885541102','-0.001976439983371','6','86']
  H_HDV['38_NOx_0_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.3563656497822','0.021411966353575','-0.856787672304083','0.075653460217562','0','6','86']
  H_HDV['38_PM_0_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.75654224758191','0.061374430123423','0.525613861795428','0.162124542835909','0.12885515680047','6','86']
  H_HDV['38_FC_0_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','0','FC','y=((a*(x**b))+(c*(x**d)))','0.0000950808712','2.99460572585698','963.464545843859','-0.551104472538849','0','6','86']
  H_HDV['38_CO_0_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.70989407281618','48.0006493271742','-1.68434776494678','0.373361891947389','0.047592031007196','6','86']
  H_HDV['38_THC_0_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.165814916100868','1.91236052980266','3.80261333003669','1.53828121617178','0.00347408242664','6','86']
  H_HDV['38_NOx_0_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.0951272784585','0.014377734819077','-0.727546548653183','0.070712848048217','0','6','86']
  H_HDV['38_PM_0_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.805555628994851','0.063274688755179','0.566806992026376','0.210393321904537','0.142793504813704','6','86']
  H_HDV['38_FC_0_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001673245117544','0.000183408598247','-0.000001472024993','','','6','86']
  H_HDV['38_CO_0_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.33715882556459','0.055576932164149','6.35647502727286','0.312723111976843','0.828206573674179','6','86']
  H_HDV['38_THC_0_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.1364907960704','0.051206845375032','1.0914790284814','0.128303618521466','0.233983848565104','6','86']
  H_HDV['38_NOx_0_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','100','NOx','y=(c+(a*exp(b*x)))','9.14506781688611','-0.072295711654608','4.94411316443782','0','0','6','86']
  H_HDV['38_PM_0_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.16125660400085','9.12236777551129','-1.61820444999485','0.292865089521025','0.05652208290141','6','86']
  H_HDV['38_FC_0_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001774137495465','0.000148843612091','-0.00000113309311','0','0','6','86']
  H_HDV['38_CO_-2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.46257515419529','0.00742342825864','-0.250524678784201','0.05774416761346','0','6','86']
  H_HDV['38_THC_-2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.204396773395217','1.50978963875715','4.43418885854553','1.61087967055927','0.021145568151608','6','86']
  H_HDV['38_NOx_-2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.7670387147783','0.010503630351897','-0.868939659910467','0.077077784776377','0','6','86']
  H_HDV['38_PM_-2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.113709145179288','12.7229078001369','-2.75652458760007','-0.279011940378077','0.093397965383812','6','86']
  H_HDV['38_FC_-2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000087219288495','2.9026067109477','1150.96185454104','-0.748326302342122','0','6','86']
  H_HDV['38_CO_-2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.48758764292431','0.015216350970738','-0.236666460838082','0.048975104995262','0','6','86']
  H_HDV['38_THC_-2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','50','THC','y=(c+(a*exp(b*x)))','1.84743875136139','-0.070683410769449','0.195279621894228','0','0','6','86']
  H_HDV['38_NOx_-2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','50','NOx','y=(c+(a*exp(b*x)))','10.3984545897008','-0.075066119662952','1.78022121848432','0','0','6','86']
  H_HDV['38_PM_-2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.115688921302241','12.3289000374031','-2.9195150943532','-0.442612618557509','0.11058234888117','6','86']
  H_HDV['38_FC_-2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','403.145694161521','0.353728003664944','-23.5533219770992','0.064306971996698','0','6','86']
  H_HDV['38_CO_-2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.52283570293795','0.020091120920535','-0.224294704784406','0.043239008831137','0','6','86']
  H_HDV['38_THC_-2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.02713278834142','-0.000520710238051','-0.132397688739861','0.07237781846069','0','6','86']
  H_HDV['38_NOx_-2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.92996193031','-0.013669530469341','-0.750314055160763','0.07827994071954','0','6','86']
  H_HDV['38_PM_-2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.23206191435483','0.001300298227515','-0.076859522915377','0.063822783207987','0','6','86']
  H_HDV['38_FC_-2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','284.067404527674','0.055197671814385','72747.1247309535','1.17040245999024','47.0811899666761','6','86']
  H_HDV['38_CO_-4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.15704022451393','0.005224836039395','-0.262148408250722','0.062392915027361','0','6','86']
  H_HDV['38_THC_-4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','0','THC','y=(c+(a*exp(b*x)))','1.83100287825382','-0.072506852057742','0.120053931858898','0','0','6','86']
  H_HDV['38_NOx_-4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.80343631865412','0.06915514300131','162.911892250363','0.80539966762969','0.577931436609294','6','86']
  H_HDV['38_PM_-4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.080341631423126','13.4448398080089','-3.05838416993006','-0.520424830270946','0.119906885976059','6','86']
  H_HDV['38_FC_-4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','278.65720619432','0.066899381009075','29122.2310819029','1.02449274514752','19.0347251141652','6','86']
  H_HDV['38_CO_-4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.23730044554048','-0.00166093229966','-0.300436684894775','0.075643853478476','0','6','86']
  H_HDV['38_THC_-4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.94638947096043','-0.001443755193378','-0.143507170129507','0.080675910937328','0','6','86']
  H_HDV['38_NOx_-4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.5913053347715','0.051971000585282','79.7746022498548','0.591112916041933','0.25889136542634','6','86']
  H_HDV['38_PM_-4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018135928152642','0.866259562792856','7.12472683266632','2.71797214036854','-0.018450319248866','6','86']
  H_HDV['38_FC_-4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','50','FC','y=((a*(x**b))+(c*(x**d)))','-4.93025068079145','0.339283982101751','1040.53322044152','-0.797481857606886','0','6','86']
  H_HDV['38_CO_-4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.22584979140968','0.016094412877334','0.000931700789584','0','0','6','86']
  H_HDV['38_THC_-4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.9249888068002','-0.001847771041725','-0.148504103057404','0.085233196997737','0','6','86']
  H_HDV['38_NOx_-4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.95241425513262','0.046287542520651','111.264476147326','0.631851045178968','0.157141784782662','6','86']
  H_HDV['38_PM_-4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.49742284077887','-9.44773878986379','-1.7522993602683','0','0','6','86']
  H_HDV['38_FC_-4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','206.117464796744','0.045730558946926','12025.7726040109','0.814627741988284','4.14417401915408','6','86']
  H_HDV['38_CO_-6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','0','CO','y=(c+(a*exp(b*x)))','3.87857605291773','-0.076577115708516','0.093650059884603','0','0','6','72']
  H_HDV['38_THC_-6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.6077858449626','0.069526747422671','1.76246943111097','0.466107852552059','0.055257627752459','6','72']
  H_HDV['38_NOx_-6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.47885582953291','0.070071394253512','142.950328063401','0.734838596196505','0.166020615431276','6','72']
  H_HDV['38_PM_-6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-26.4457870531104','0.920836433226451','1.42468932512738','0.088051440673215','0.042810314812325','6','72']
  H_HDV['38_FC_-6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','227.951910860041','0.064511024906003','13635.2584513348','0.866322976876525','3.5595186957837','6','72']
  H_HDV['38_CO_-6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.0702796422898','0.068243011828289','19.9820319607435','0.657508137753272','0.069325632118664','6','72']
  H_HDV['38_THC_-6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.38150575245945','0.066183695076725','2.68351475839854','0.426800048941742','0.048507117421751','6','72']
  H_HDV['38_NOx_-6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.14473783708323','0.064909149931602','148.44538921838','0.707762120612388','0.126770135425842','6','72']
  H_HDV['38_PM_-6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.003773051347595','0.889753155984123','6.20933513487292','2.49412896240357','-0.019683988401104','6','72']
  H_HDV['38_FC_-6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','193.314022804633','0.060128159769148','12390.0742100625','0.831159800469982','2.18490686271679','6','72']
  H_HDV['38_CO_-6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.77043502226006','0.064575688191179','28.5394975930852','0.659572723758389','0.061490038799356','6','72']
  H_HDV['38_THC_-6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.26443179848481','0.064681211346946','3.34507161541053','0.432420011895267','0.046440719946804','6','72']
  H_HDV['38_NOx_-6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.51119893194821','0.062324367603048','165.005920099616','0.712846111955637','0.11604707677215','6','72']
  H_HDV['38_PM_-6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.027102932217324','1.12041522329615','3.16119254765017','1.31487215269826','0.031324815468051','6','72']
  H_HDV['38_FC_-6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','176.213443543583','0.057936614098925','12863.8641813185','0.830642535165503','1.79337016648364','6','72']
  H_HDV['38_CO_2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.91420489422692','0.009310714775242','-0.349644634742439','0.06336111062644','0','6','86']
  H_HDV['38_THC_2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','0','THC','y=(c+(a*exp(b*x)))','2.10501613641889','-0.064038356596595','0.2698063057929','0','0','6','86']
  H_HDV['38_NOx_2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-27.1959817949692','0.044925508903747','36.9281803727552','0.05915108500639','7.09472468719369','6','86']
  H_HDV['38_PM_2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.1516527310318','0.001847601865481','-0.064845682641905','0.060279970198618','0','6','86']
  H_HDV['38_FC_2_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','554.479693365736','1.25423010865467','-28.77204852552','0.065647064725217','0','6','86']
  H_HDV['38_CO_2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.80725841268935','0.004356835090375','-0.465807900340566','0.078889643250882','0','6','86']
  H_HDV['38_THC_2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.44445249303602','0.001525625101656','-0.142703908631386','0.064400185867446','0','6','86']
  H_HDV['38_NOx_2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.333431550227','0.010677240761928','-1.10447687686796','0.09693217458322','0','6','86']
  H_HDV['38_PM_2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.15772265787609','0.003010612505031','-0.062489340010763','0.057103373639843','0','6','86']
  H_HDV['38_FC_2_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','586.733967983143','1.3885014254512','-27.2952482075381','0.062552520622272','0','6','86']
  H_HDV['38_CO_2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.7163021731544','-0.004445403512667','-0.608714131722134','0.099116075678151','0','6','85']
  H_HDV['38_THC_2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.361784300166087','10.4813528428546','-1.59285547043454','-0.150668812118317','0.09318175994959','6','85']
  H_HDV['38_NOx_2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.2659196277874','0.011797281892447','-0.826223990960773','0.081085710054817','0','6','85']
  H_HDV['38_PM_2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.20178602615377','0.002230612731219','-0.067610708093822','0.064647023494332','0','6','85']
  H_HDV['38_FC_2_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','2','100','FC','y=((a*(x**b))+(c*(x**d)))','0.03317072573754','1.82177457523093','991.447764974138','-0.385790988293661','0','6','85']
  H_HDV['38_CO_4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.27857810755174','5.78880459297709','-0.073609707824606','-0.947695458761713','0.221672951193342','6','71']
  H_HDV['38_THC_4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.220456538934846','1.77415230251602','6.17666067365964','2.3364580812445','-0.014583369424892','6','71']
  H_HDV['38_NOx_4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.2694234370882','0.018007318386668','-1.34447547675802','0.105352168667654','0','6','71']
  H_HDV['38_PM_4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.246958006126512','6.24487907438795','-3.29335382571818','-1.11337694207173','0.174644819580853','6','71']
  H_HDV['38_FC_4_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','1278.98868589484','1.01137863167525','-0.570946571511019','0','0','6','71']
  H_HDV['38_CO_4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.91412483182222','0.008309929306547','-0.579008063292209','0.08596201641983','0','6','70']
  H_HDV['38_THC_4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.298858892934504','1.73129782274961','6.35066883751076','2.44766769092844','-0.015369255226456','6','70']
  H_HDV['38_NOx_4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','50','NOx','y=((a*(b**x))*(x**c))','30.6206508566632','1.0078740780934','-0.377074672334997','0','0','6','70']
  H_HDV['38_PM_4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006405571803','0.000972796101136','-0.048204653409289','1.10481709665358','0','6','70']
  H_HDV['38_FC_4_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002873956949109','0.437847054267966','-21.5162054234803','651.442679139016','0','6','70']
  H_HDV['38_CO_4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.37805722438421','-0.018805534327879','-0.862843150986019','0.127247311013011','0','6','66']
  H_HDV['38_THC_4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.533734076847793','7.86906647857219','-2.61940530519645','-1.2253486332144','0.203226922867373','6','66']
  H_HDV['38_NOx_4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.83946520721138','0.068272219533679','29.5208460976629','0.465857957119063','12.0982454244834','6','66']
  H_HDV['38_PM_4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006413468843','0.000976375019157','-0.04709993916588','1.13920331187649','0','6','66']
  H_HDV['38_FC_4_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002854526144945','0.432281786908021','-20.9990896053842','708.065818540314','0','6','66']
  H_HDV['38_CO_6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.57688246791026','0.014987128814599','-0.51400547085076','0.075040866311131','0','6','71']
  H_HDV['38_THC_6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.324700477837398','1.6913748017265','6.79274970201424','2.60968107765183','-0.017779128875685','6','71']
  H_HDV['38_NOx_6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','0','NOx','y=((a*(b**x))*(x**c))','32.6304726797586','1.00931163571345','-0.413410192339012','0','0','6','71']
  H_HDV['38_PM_6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.13654407819878','0.003998088401125','-0.060735815699054','0.059508720470397','0','6','71']
  H_HDV['38_FC_6_0'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','0','FC','y=((a*(x**b))+(c*(x**d)))','0.055471226126637','1.79631473763117','1090.26409546736','-0.386626217789094','0','6','71']
  H_HDV['38_CO_6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','50','CO','y=(c+(a*exp(b*x)))','6.41002173953717','-0.080630857482687','1.7420350936729','0','0','6','63']
  H_HDV['38_THC_6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.548413370960429','9.92808698099894','-2.84227161095488','-1.18409871872639','0.195820983130037','6','63']
  H_HDV['38_NOx_6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.36146066211731','0.082974745512515','16.9752431350068','0.37276138718779','13.1284927438695','6','63']
  H_HDV['38_PM_6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.10456444820699','0.0184792172687','-0.042146964317279','0.028892583476842','0','6','63']
  H_HDV['38_FC_6_50'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','50','FC','y=((a*(b**x))*(x**c))','1269.85827825251','1.00994122140489','-0.419179504772074','0','0','6','63']
  H_HDV['38_CO_6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','100','CO','y=(c+(a*exp(b*x)))','6.2879043430048','-0.085629992028121','2.56768591145314','0','0','6','51']
  H_HDV['38_THC_6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.87762824648298','-0.002765087358933','-0.201574633789822','0.088605980101334','0','6','51']
  H_HDV['38_NOx_6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.97977963574144','0.100980153127632','911.202968595506','1.21577177545334','16.3137517018513','6','51']
  H_HDV['38_PM_6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000008786209536','0.001094526664241','-0.04631468470064','1.19158878219319','0','6','51']
  H_HDV['38_FC_6_100'] = [' 38','EURO1','RT >7.5-12t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','256.220137482858','0.058041344830585','1512.33968026084','0.491263194380125','462.756778785221','6','51']
  H_HDV['39_CO_0_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.47276398992005','0.065894501646594','2.7698641132728','0.281185842190639','0.588184777230859','6','86']
  H_HDV['39_THC_0_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.585419022794309','0.038256224139971','0.930444132067567','0.124803723676243','0.124913424212397','6','86']
  H_HDV['39_NOx_0_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.894557945855','0.015868965667708','-0.985708233915499','0.079658941791475','0','6','86']
  H_HDV['39_PM_0_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.284704658135858','0.000652118905598','-0.013491767739728','0.054137383744812','0','6','86']
  H_HDV['39_FC_0_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001814934273412','0.000235761876026','-0.000002032336846','0','0','6','86']
  H_HDV['39_CO_0_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.134471625348833','0.03393523127663','-0.000219740766483','0','0','6','86']
  H_HDV['39_THC_0_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108682277270042','1.35814697335046','3.23091435408864','1.35597315335383','0.00903377392083','6','86']
  H_HDV['39_NOx_0_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.6632819198165','0.009059683476815','-0.853921545218428','0.074795332792945','0','6','86']
  H_HDV['39_PM_0_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.287663551016496','0.001385136019689','-0.010700677347108','0.037181703838854','0','6','86']
  H_HDV['39_FC_0_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001963212344637','0.000184891398873','-0.000001507498314','','','6','86']
  H_HDV['39_CO_0_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.136363715230067','0.028945751193497','-0.000180894911418','0','0','6','86']
  H_HDV['39_THC_0_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.116943876264894','1.30269984788839','3.42995535104506','1.41373598083812','0.009782354073854','6','86']
  H_HDV['39_NOx_0_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','100','NOx','y=(c+(a*exp(b*x)))','10.487795168557','-0.071639661836027','5.05095669213637','0','0','6','86']
  H_HDV['39_PM_0_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.298367272563474','0.003995555941229','-0.008780981710223','0.019704413232333','0','6','86']
  H_HDV['39_FC_0_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.002051904483505','0.000147118193141','-0.000001125994287','0','0','6','86']
  H_HDV['39_CO_-2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.17777846181157','0.007002575376177','-0.177426492354085','0.056049856274713','0','6','86']
  H_HDV['39_THC_-2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.755268570802323','0.056388106910156','0.514390495873435','0.115906890873237','0.13703195965179','6','86']
  H_HDV['39_NOx_-2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.1190147614225','0.008499238823009','-0.97886982604642','0.079255637778821','0','6','86']
  H_HDV['39_PM_-2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.247110275122384','0.001394991092335','-0.01087278759664','0.038007925175654','0','6','86']
  H_HDV['39_FC_-2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000080625473763','2.90704688059522','999.959953117982','-0.729138230115732','0','6','86']
  H_HDV['39_CO_-2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.23231226807418','0.014160654475432','-0.164155790878697','0.045221395262138','0','6','86']
  H_HDV['39_THC_-2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.121696706035111','1.52280078653727','1.60587005621174','0.687072388820855','0.047950404974452','6','86']
  H_HDV['39_NOx_-2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','50','NOx','y=(c+(a*exp(b*x)))','11.3379033848017','-0.07363414343125','1.89082219382432','0','0','6','86']
  H_HDV['39_PM_-2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.249979756133752','0.002787504093467','-0.009589144567716','0.025828686126382','0','6','86']
  H_HDV['39_FC_-2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','357.883947152267','0.326680421414527','-20.1236139801563','0.062316554285133','0','6','86']
  H_HDV['39_CO_-2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.34964487569521','0.019522504564534','-0.158228777425794','0.0388935507058','0','6','86']
  H_HDV['39_THC_-2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.3121977723129','-0.00058763015171','-0.084874816246276','0.072772192849342','0','6','86']
  H_HDV['39_NOx_-2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.973341397871','-0.015256569798665','-0.813917893673098','0.077847258473332','0','6','86']
  H_HDV['39_PM_-2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.264438570909695','0.004386888549001','-0.009316405973544','0.019336933828037','0','6','86']
  H_HDV['39_FC_-2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-2','100','FC','y=(c+(a*exp(b*x)))','303.600416835465','-0.060825883201413','46.5414471011459','0','0','6','86']
  H_HDV['39_CO_-4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.88173376998955','0.004250754200224','-0.177525674095571','0.060369067041786','0','6','86']
  H_HDV['39_THC_-4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.08165765731711','0.066383379016389','0.965485594807677','0.507827899398403','0.083190019403872','6','86']
  H_HDV['39_NOx_-4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.59778566304863','0.068494595083789','113.394504369927','0.73073132171985','0.630348724208802','6','86']
  H_HDV['39_PM_-4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','0','PM','y=(c+(a*exp(b*x)))','0.230594780143434','-0.066050467259324','0.021784572395512','0','0','6','86']
  H_HDV['39_FC_-4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','249.624020132058','0.065931208019829','23744.9928644657','1.01606789328172','17.8344592919393','6','86']
  H_HDV['39_CO_-4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.42975004081159','0.063265642309581','25.0595394948804','0.745440228876859','0.120426407588199','6','86']
  H_HDV['39_THC_-4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.23372020267014','-0.000986356900709','-0.086319588502686','0.077673508371627','0','6','86']
  H_HDV['39_NOx_-4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.19918410337041','0.051742303390055','70.2039404246627','0.557941315953188','0.280194071041396','6','86']
  H_HDV['39_PM_-4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','50','PM','y=(1/(a+(b*(x**c))))','3.50574012569552','0.208060087025092','1.24717604495676','0','0','6','86']
  H_HDV['39_FC_-4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','200.935508221228','0.050388616955127','7710.33604729735','0.779181465678284','7.99455833287435','6','86']
  H_HDV['39_CO_-4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.23157161793108','0.055438716827899','29.5051277797066','0.699039334478986','0.088765099052761','6','86']
  H_HDV['39_THC_-4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.682709753051309','0.081047016622797','0.001957578528663','0','0','6','86']
  H_HDV['39_NOx_-4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.56732925511281','0.046866387618562','110.312914181974','0.619723352880094','0.178533590924161','6','86']
  H_HDV['39_PM_-4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','100','PM','y=((a+(b*x))**((-1)/c))','2.11812375738576','0.134146557790828','0.645431378211806','0','0','6','86']
  H_HDV['39_FC_-4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','186.018243778485','0.04451754945954','10848.6055513763','0.820748878922452','3.98186746302595','6','86']
  H_HDV['39_CO_-6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.50614914881859','0.072670899260432','24.5022697184836','0.842130779431005','0.06870234355107','6','72']
  H_HDV['39_THC_-6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.988974411538831','0.064348747426467','2.07758003374579','0.532693561612399','0.044621770787658','6','72']
  H_HDV['39_NOx_-6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.3651835143983','0.07071761158346','153.084819240024','0.743488235679807','0.188328680993137','6','72']
  H_HDV['39_PM_-6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010766853683855','0.210160445455842','4.29640790379367','1.69619537408902','0.004752607748908','6','72']
  H_HDV['39_FC_-6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','206.996381741306','0.064231887205804','12255.9321723069','0.87648279312603','3.67876453697959','6','72']
  H_HDV['39_CO_-6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045728892371282','10.969574885273','-0.643672507629894','0.427626280581836','0.049681592750143','6','72']
  H_HDV['39_THC_-6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.874168966145061','0.061806285467606','1.76477597645377','0.429410675608281','0.041398198819203','6','72']
  H_HDV['39_NOx_-6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.82202007790193','0.065582416733612','152.595162884245','0.704518467918576','0.146049763066043','6','72']
  H_HDV['39_PM_-6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.170142686020812','0.056686979873824','0.112655648985416','0.198701431649799','0.016091980828471','6','72']
  H_HDV['39_FC_-6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','175.605078876868','0.059853035061153','11167.0487952112','0.840340642091627','2.46289827075076','6','72']
  H_HDV['39_CO_-6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.84877644575152','0.060345252200627','35.1008057242397','0.722051603043651','0.047279675100643','6','72']
  H_HDV['39_THC_-6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.819353081368393','0.06085844302361','1.86388703787304','0.414031043531252','0.040541461205114','6','72']
  H_HDV['39_NOx_-6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.06577591690447','0.062767649327068','175.516762277','0.713416066037341','0.132454093121395','6','72']
  H_HDV['39_PM_-6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.167524116707425','0.052769987518757','0.390912977253596','0.381879514840021','0.015154783240804','6','72']
  H_HDV['39_FC_-6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','159.913911756288','0.057487164619532','11808.3010690141','0.84223200372877','2.08063030709898','6','72']
  H_HDV['39_CO_2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.870657111099117','2.81089332106485','4.6320353238505','1.49492657842206','0.065193660955998','6','86']
  H_HDV['39_THC_2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.056050416616051','1.2681929453918','4.87933221609062','1.86008645007031','-0.010981740058818','6','86']
  H_HDV['39_NOx_2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-8.57735320306271','0.035653347798449','19.7598026128653','0.068150890652802','7.14049037160897','6','86']
  H_HDV['39_PM_2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-0.633701070713654','0.199486451913257','0.675702378790641','0.114585344177289','0.102539142160276','6','86']
  H_HDV['39_FC_2_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','499.466661064406','1.10259338522189','-25.5548549894642','0.067290906307742','0','6','86']
  H_HDV['39_CO_2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.68049917442671','-0.002379552779305','-0.429350481925039','0.093554172532937','0','6','86']
  H_HDV['39_THC_2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.104238387249787','1.14327777928511','6.27305632517976','2.38744491859333','-0.020451988826575','6','86']
  H_HDV['39_NOx_2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.7532083692902','0.006334057533457','-1.13677023785638','0.090566633888419','0','6','86']
  H_HDV['39_PM_2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.23657950347732','0.196314565814197','-0.001515805511862','0','0','6','86']
  H_HDV['39_FC_2_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.026725252081528','1.85321659453692','867.884108697448','-0.407402203455326','0','6','86']
  H_HDV['39_CO_2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','100','CO','y=(1/(a+(b*(x**c))))','-0.013238937296741','0.085254394707229','0.596395063658698','0','0','6','86']
  H_HDV['39_THC_2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','100','THC','y=(c+(a*exp(b*x)))','1.48418613954499','-0.07626373195593','0.214569777365235','0','0','6','86']
  H_HDV['39_NOx_2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','100','NOx','y=(c+(a*exp(b*x)))','11.6378286662657','-0.083300756104711','8.5290760280179','0','0','6','86']
  H_HDV['39_PM_2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.1212109909593','0.154942789029722','-0.00106040081825','0','0','6','86']
  H_HDV['39_FC_2_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','554.802659891151','1.33482090285931','-21.4321709963927','0.058691826916526','0','6','86']
  H_HDV['39_CO_4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.08977914997739','0.029726477831272','-0.000269570562434','0','0','6','71']
  H_HDV['39_THC_4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.112447449720353','1.26317745415111','5.42347637719181','2.08367851009314','-0.011098908819166','6','71']
  H_HDV['39_NOx_4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.2358131444618','0.005332354560268','-1.48787965291886','0.105588945073321','0','6','71']
  H_HDV['39_PM_4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.2812520468212','0.186362508348512','-0.001575449939253','0','0','6','71']
  H_HDV['39_FC_4_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1082.9382205811','1.01097567571937','-0.529693482572482','0','0','6','71']
  H_HDV['39_CO_4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.79360172285288','-0.015735503474596','-0.763511011105602','0.128769994447759','0','6','71']
  H_HDV['39_THC_4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.194715229220239','1.10461801319458','7.08806730401272','2.73762537126669','-0.0202594320419','6','71']
  H_HDV['39_NOx_4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','50','NOx','y=((a*(b**x))*(x**c))','35.1539659869336','1.00709649918743','-0.404008194112037','0','0','6','71']
  H_HDV['39_PM_4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.488178291761103','-0.00085604684967','-0.026900777803733','0.087578389454589','0','6','71']
  H_HDV['39_FC_4_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002380980136705','0.367176740488178','-18.0905769342727','589.954260521564','0','6','71']
  H_HDV['39_CO_4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.17689546247932','-0.019638884925821','-0.674917502737219','0.110273804112129','0','6','67']
  H_HDV['39_THC_4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.238698049689999','1.07233418173617','7.17633150472819','2.80843668791647','-0.020706343619336','6','67']
  H_HDV['39_NOx_4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','100','NOx','y=((a*(b**x))*(x**c))','35.0717488014915','1.00488598151812','-0.335377495982785','0','0','6','67']
  H_HDV['39_PM_4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.557126015731404','-0.001504948101095','-0.032117574474978','0.099041732585598','0','6','67']
  H_HDV['39_FC_4_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002414162544201','0.359992317557939','-17.396687590457','646.361621319756','0','6','67']
  H_HDV['39_CO_6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.04032149477453','-0.016057768690518','-0.900680640914717','0.147251509829015','0','6','71']
  H_HDV['39_THC_6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-6.32066969466599','0.611468144530199','1.88662182475988','0.087743520980866','0.246093786927342','6','71']
  H_HDV['39_NOx_6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','10.2775393868804','29.109852467419','0.404995167718624','0.563105441043186','0.074329665035222','6','71']
  H_HDV['39_PM_6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.508867189751145','-0.001114228476108','-0.037483246742812','0.122360763753183','0','6','71']
  H_HDV['39_FC_6_0'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','0','FC','y=((a*(x**b))+(c*(x**d)))','0.00541463405656','2.32578999976652','908.251846161454','-0.333178608978308','0','6','71']
  H_HDV['39_CO_6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.46757025732715','0.022201101658698','5.9560244275329','0.133514801033492','0.469207417218855','6','64']
  H_HDV['39_THC_6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.330179536385793','6.78625834616696','-2.5968659455448','-0.993589961870924','0.180302530130108','6','64']
  H_HDV['39_NOx_6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000131190290796','0.016557139681853','-0.734054871368098','24.4335555030843','0','6','64']
  H_HDV['39_PM_6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.511300940463927','0.006109357832278','-0.017977958288833','0.029609558277179','0','6','64']
  H_HDV['39_FC_6_50'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','678.984368649668','4.61471827105696','-19.5880911921759','0.041492444061953','0','6','64']
  H_HDV['39_CO_6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.95596447630896','-0.027582112805445','-0.84460968054109','0.11242130505923','0','6','56']
  H_HDV['39_THC_6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.366403751930936','9.99202765924656','-2.82303375977008','-0.816936599280154','0.166338367468597','6','56']
  H_HDV['39_NOx_6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','100','NOx','y=((a*(b**x))*(x**c))','39.6019467417494','1.00442186803243','-0.296712590728594','0','0','6','56']
  H_HDV['39_PM_6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.45083294985479','0.101716363049068','-0.000685624655777','0','0','6','56']
  H_HDV['39_FC_6_100'] = [' 39','EURO2','RT >7.5-12t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','271.913668093659','0.095580251440634','12479.175631918','0.965923880597201','487.183615899153','6','56']
  H_HDV['40_CO_0_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.34220692115333','0.054884592906919','2.87352560050026','0.191945644675078','0.562725281030979','6','86']
  H_HDV['40_THC_0_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.051232754474076','1.20898403138294','4.06501341537555','1.63748269065622','-0.006510005596449','6','86']
  H_HDV['40_NOx_0_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.9673811667177','0.09590914243257','32.0018115769612','0.415189820696775','3.29934620558281','6','86']
  H_HDV['40_PM_0_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.500347072248415','-0.000206950402875','-0.031150406123783','0.073152607065663','0','6','86']
  H_HDV['40_FC_0_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001565325179478','0.000225410814778','-0.000001913796655','0','0','6','86']
  H_HDV['40_CO_0_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.5897758563258','0.056387708398454','3.83629023761502','0.242346548988918','0.635694408016525','6','86']
  H_HDV['40_THC_0_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.087634985920937','1.15560536973565','4.03414662016902','1.61176582460381','0.001851084081331','6','86']
  H_HDV['40_NOx_0_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.44567822500211','0.077735764371147','29.6909677652412','0.353966115781091','3.56650223937148','6','86']
  H_HDV['40_PM_0_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.306197094044543','0.05638593418848','0.142078394898821','0.126016798508572','0.060988257471745','6','86']
  H_HDV['40_FC_0_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001745361395777','0.000178441615182','-0.000001435185435','','','6','86']
  H_HDV['40_CO_0_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.693605911731975','51.3334958985451','-1.61960916716211','0.39568196211505','0.043445720297651','6','86']
  H_HDV['40_THC_0_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.083020520941748','1.09987131324335','4.4478768673189','1.76521381240145','-0.003134023221725','6','86']
  H_HDV['40_NOx_0_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','100','NOx','y=((a*(x**b))+(c*(x**d)))','0.28429820049779','0.454711627671977','50.4329700771307','-0.770488641419258','0','6','86']
  H_HDV['40_PM_0_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.328711298827266','0.056829843289255','0.142668394413516','0.174689234918381','0.063687525949711','6','86']
  H_HDV['40_FC_0_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001854937227217','0.000143140371541','-0.000001077494224','0','0','6','86']
  H_HDV['40_CO_-2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-24.3431172340292','0.551587237069074','5.91139715003771','0.079270303824917','0.570873754956312','6','86']
  H_HDV['40_THC_-2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045738054358156','0.906511028361572','6.70536460651467','2.50994501045262','-0.026275726084004','6','86']
  H_HDV['40_NOx_-2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2002028478545','0.094927398847186','239.187157924564','0.836756921880988','1.77302085477178','6','86']
  H_HDV['40_PM_-2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.052412992432741','10.5113351230358','-3.71040298922707','-0.35408487478933','0.084489989996352','6','86']
  H_HDV['40_FC_-2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','418.047033481085','0.484060308456307','-25.4995133749832','0.067115538940544','0','6','86']
  H_HDV['40_CO_-2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.84914316112331','0.017496689323945','-0.24151007614222','0.045533338578398','0','6','86']
  H_HDV['40_THC_-2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','50','THC','y=(c+(a*exp(b*x)))','1.11056538103635','-0.068671921978167','0.119741767221509','0','0','6','86']
  H_HDV['40_NOx_-2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1270235928649','0.078936731353266','179.455049030631','0.736612122193024','1.50848495011785','6','86']
  H_HDV['40_PM_-2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.05387320119818','11.5717140931162','-4.01500837851757','-0.487057991675749','0.097228375125943','6','86']
  H_HDV['40_FC_-2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','50','FC','y=(c+(a*exp(b*x)))','353.57885747757','-0.071392920167817','57.5541726369549','0','0','6','86']
  H_HDV['40_CO_-2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.92196573298912','0.022787467802119','-0.23813163143388','0.041874474900284','0','6','86']
  H_HDV['40_THC_-2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','100','THC','y=(c+(a*exp(b*x)))','1.09202527163194','-0.067973280452895','0.10222044621288','0','0','6','86']
  H_HDV['40_NOx_-2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.38328906885398','0.063337196273419','137.46441149993','0.647124704011206','1.24304171261497','6','86']
  H_HDV['40_PM_-2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.052867078039613','8.88097294739667','-3.71235380960748','-0.469223166278011','0.097595395490512','6','86']
  H_HDV['40_FC_-2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','292.594298929766','0.053843148671113','65209.5131103934','1.19884998708297','47.095628833169','6','86']
  H_HDV['40_CO_-4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.62953428147765','0.01052209973866','-0.259799148281507','0.053118734758976','0','6','86']
  H_HDV['40_THC_-4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.084674817112665','4.39763481309404','-1.44828526643488','-0.18752589765141','0.090849053212892','6','86']
  H_HDV['40_NOx_-4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0839616463171','0.077610476790899','132.725822608165','0.674968935908375','0.514088808513565','6','86']
  H_HDV['40_PM_-4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045358114241045','12.2280615674526','-4.05961211245032','-0.528248590311094','0.103872392373069','6','86']
  H_HDV['40_FC_-4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','301.731159109885','0.066840670183075','14062.7965526617','0.964665548512926','21.3918941593164','6','86']
  H_HDV['40_CO_-4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-19.227549262863','0.684075282793543','4.94847307726872','0.071239372590436','0.223520793456606','6','86']
  H_HDV['40_THC_-4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014557694638868','0.819061955644104','5.92409688163469','2.2159786308357','-0.009700354042431','6','86']
  H_HDV['40_NOx_-4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.93186004697824','0.059522002338423','106.194703549961','0.570583625371948','0.30841402868168','6','86']
  H_HDV['40_PM_-4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.90512218495177','-8.692169983708','-1.45337309457513','0','0','6','86']
  H_HDV['40_FC_-4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','233.908761724919','0.051241530707553','3789.61071073084','0.657368819195895','10.6415826038472','6','86']
  H_HDV['40_CO_-4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.69667506417762','-0.003140220818872','-0.316344222120351','0.073038375938715','0','6','86']
  H_HDV['40_THC_-4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.08985969367761','-0.000838627467543','-0.0790107673565','0.079769279987294','0','6','86']
  H_HDV['40_NOx_-4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.16250979938212','0.05471561885265','175.077180677367','0.648013429874155','0.260687521138755','6','86']
  H_HDV['40_PM_-4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.72864199953751','-8.18943841894043','-1.41733776517412','0','0','6','86']
  H_HDV['40_FC_-4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','212.709295934752','0.04562348837551','5606.25175635437','0.705985058317205','6.88242284329065','6','86']
  H_HDV['40_CO_-6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.163044009394369','21.3166963034354','-1.80776754117615','-0.280362483955196','0.097189656984909','6','72']
  H_HDV['40_THC_-6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','0','THC','y=(c+(a*exp(b*x)))','1.03988201736071','-0.073190882974829','0.044997296472411','0','0','6','72']
  H_HDV['40_NOx_-6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.40502219613366','0.075844354668207','141.888625812448','0.647355663743738','0.229581262417569','6','72']
  H_HDV['40_PM_-6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.29209690993327','-9.68136800822839','-1.57797626179175','0','0','6','72']
  H_HDV['40_FC_-6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','258.501394128871','0.067454545006832','8841.61781093689','0.842757442159755','7.95097193189478','6','72']
  H_HDV['40_CO_-6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','4.08227567840807','-0.071001281192521','0.116321538165616','0','0','6','72']
  H_HDV['40_THC_-6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.860315304912136','0.069656659278655','0.653066306097176','0.395013776427832','0.040021746791532','6','72']
  H_HDV['40_NOx_-6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.62940787230124','0.067922674588502','205.398116321762','0.68817042129006','0.181309578190032','6','72']
  H_HDV['40_PM_-6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.7654532499658','-8.16385322567952','-1.45763198028416','0','0','6','72']
  H_HDV['40_FC_-6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','211.317911654245','0.062000647587344','7575.72646430901','0.7782252605748','6.50069404865864','6','72']
  H_HDV['40_CO_-6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.4472475252835','0.065141990883096','16.1499507386127','0.707013650285303','0.100214055288911','6','72']
  H_HDV['40_THC_-6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.763539320345188','0.06772071821433','2.31760495624115','0.540406358792366','0.038256989135309','6','72']
  H_HDV['40_NOx_-6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.74442702702305','0.063274979720566','261.622467579879','0.723910466622167','0.162229389607236','6','72']
  H_HDV['40_PM_-6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006857529853733','0.320781900199415','5.64338348467282','2.12839496345923','-0.013118189344254','6','72']
  H_HDV['40_FC_-6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','187.064481443438','0.058568040729969','8615.93004527899','0.785160444141636','5.90394820489466','6','72']
  H_HDV['40_CO_2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.528075590517532','3.71202131551371','7.49956412996515','2.85554430396631','-0.029083275548885','6','86']
  H_HDV['40_THC_2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.057108554491416','1.12622040160629','5.37500846989789','2.0066219124504','-0.01155266727835','6','86']
  H_HDV['40_NOx_2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.6838641870123','0.003586329079271','-2.68085719665215','0.127625576306829','0','6','86']
  H_HDV['40_PM_2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.469791193343418','-0.000225070413466','-0.025226837515056','0.065405741287946','0','6','86']
  H_HDV['40_FC_2_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','545.724494496841','1.02281714799369','-28.7963332357925','0.069500075599996','0','6','86']
  H_HDV['40_CO_2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','50','CO','y=(c+(a*exp(b*x)))','5.48846324665699','-0.074036107871262','0.861279088862176','0','0','6','86']
  H_HDV['40_THC_2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.097175812865703','1.04095124666114','6.26447702277651','2.34575283032053','-0.015116799463789','6','86']
  H_HDV['40_NOx_2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.8803961260693','0.113258935854574','27.3623240671702','0.477784007300167','5.71422107387936','6','86']
  H_HDV['40_PM_2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.462592520334096','0.000284130695967','-0.022525220198651','0.056175623901822','0','6','86']
  H_HDV['40_FC_2_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.02787660959083','1.83543553989106','936.483930457768','-0.412857845207423','0','6','86']
  H_HDV['40_CO_2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','100','CO','y=(c+(a*exp(b*x)))','5.8185003692936','-0.068869932854274','0.897901718331777','0','0','6','86']
  H_HDV['40_THC_2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.129250775795287','0.988261087937942','6.84962739397945','2.58943416538508','-0.017811934658409','6','86']
  H_HDV['40_NOx_2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4387216710096','0.092282988077412','32.861281538241','0.408699761776757','6.50175098021567','6','86']
  H_HDV['40_PM_2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.095267841029344','3.12723241587088','-2.37529731146489','-0.280456444946334','0.089935254596811','6','86']
  H_HDV['40_FC_2_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','592.356863039972','0.989211056825135','-23.8900342360998','0.062888630142501','0','6','86']
  H_HDV['40_CO_4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','0','CO','y=(c+(a*exp(b*x)))','5.38549497903456','-0.078799301436949','0.914708412422299','0','0','6','72']
  H_HDV['40_THC_4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.090460123693036','1.11280353855497','5.87022934866084','2.20054289024058','-0.01264704293939','6','72']
  H_HDV['40_NOx_4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.7871083363764','0.007417070588796','-2.92138159568204','0.128361372800284','0','6','72']
  H_HDV['40_PM_4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.087783768039081','5.57592332048996','-3.13413279377746','-0.318835969293891','0.083863856477578','6','72']
  H_HDV['40_FC_4_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','1136.73984623949','1.01010636831656','-0.51982496748384','0','0','6','72']
  H_HDV['40_CO_4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.47180796838215','-0.014470944155001','-0.581033414634565','0.099942174123012','0','6','71']
  H_HDV['40_THC_4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.64362836172053','-0.000665458320339','-0.116042814794584','0.082204955487334','0','6','71']
  H_HDV['40_NOx_4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.38136202608359','22.3430336436833','3.34961608510546','1.84670927749834','0.001940684857273','6','71']
  H_HDV['40_PM_4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.107978629251456','5.20273294973972','-3.30244133553951','-0.531906646887675','0.109180742998072','6','71']
  H_HDV['40_FC_4_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','1069.61668977706','1.00817578720912','-0.417017689108143','0','0','6','71']
  H_HDV['40_CO_4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.17784638007782','-0.01710572010788','-0.615807246725261','0.09707402303974','0','6','68']
  H_HDV['40_THC_4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.171176275396299','1.00115013983649','6.75733169052518','2.60286494873957','-0.017931874508147','6','68']
  H_HDV['40_NOx_4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','100','NOx','y=((a*(b**x))*(x**c))','59.3965812536193','1.01203564500035','-0.628171695317001','0','0','6','68']
  H_HDV['40_PM_4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002604270922','0.000388890560521','-0.019768468098525','0.480335891013703','0','6','68']
  H_HDV['40_FC_4_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002508739846916','0.372987403682325','-18.2218606865664','676.378511276344','0','6','68']
  H_HDV['40_CO_6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.1425813948333','-0.010804516006006','-0.551399490223252','0.097227832574744','0','6','71']
  H_HDV['40_THC_6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.205579695197736','9.30425136074863','-2.05441977389734','-0.205489323138904','0.094237263539811','6','71']
  H_HDV['40_NOx_6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','0','NOx','y=((a*(b**x))*(x**c))','61.7941504610455','1.01362621812745','-0.708317052389202','0','0','6','71']
  H_HDV['40_PM_6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.03223234399837','0.07061036932061','0.349473939178901','0.069764442501365','0.107839102202434','6','71']
  H_HDV['40_FC_6_0'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','0','FC','y=((a*(x**b))+(c*(x**d)))','0.02187855210299','1.98987136490907','971.836782502617','-0.34050676420195','0','6','71']
  H_HDV['40_CO_6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.11122072641327','-3.9217770032193','-0.998079527378909','0','0','6','65']
  H_HDV['40_THC_6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.251461272423984','11.187877932556','-2.67479142200148','-0.502438894749147','0.124983129364301','6','65']
  H_HDV['40_NOx_6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','50','NOx','y=((a*(b**x))*(x**c))','64.9315661432194','1.01354305690042','-0.656535393294945','0','0','6','65']
  H_HDV['40_PM_6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.093654438490921','0.088007931886533','0.329523280988562','0.088549351548391','0.141288994521481','6','65']
  H_HDV['40_FC_6_50'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','50','FC','y=((a*(b**x))*(x**c))','1098.38006288344','1.00739030558532','-0.347105686320671','0','0','6','65']
  H_HDV['40_CO_6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.64319874680731','-5.24393877693637','-1.1061531257051','0','0','6','57']
  H_HDV['40_THC_6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.292515701370105','14.9657553824146','-3.44129225690581','-0.834189444908453','0.159351310642812','6','57']
  H_HDV['40_NOx_6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.99846082540333','0.025727873782536','30.4675244798958','0.199642727573223','10.0983864211453','6','57']
  H_HDV['40_PM_6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','100','PM','y=(c+(a*exp(((-1)*b)*x)))','0.437681971616911','0.101553000232562','0.177189679781607','0','0','6','57']
  H_HDV['40_FC_6_100'] = [' 40','EURO3','RT >7.5-12t Euro-3','6','100','FC','y=((a*(b**x))*(x**c))','1098.92358547342','1.0055598356879','-0.271405565066661','0','0','6','57']
  H_HDV['41_CO_0_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.455695761563192','-0.000314839315025','-0.032403442192482','0.083583464234057','0','6','86']
  H_HDV['41_THC_0_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002935405494239','0.072137986221683','3.76396721234414','1.55253979626692','-0.000877141121435','6','86']
  H_HDV['41_NOx_0_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','0','NOx','y=((a*(b**x))*(x**c))','26.303694909518','1.01497457129563','-0.844093304455486','0','0','6','86']
  H_HDV['41_PM_0_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048445325876592','0.047499630286612','0.128573254826672','0.165751898301539','0.009947113423016','6','86']
  H_HDV['41_FC_0_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001573581065871','0.000241529321156','-0.000002034569476','0','0','6','86']
  H_HDV['41_CO_0_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.039322261252786','0.380143996154247','3.78075318713967','1.57958267600658','0.004228776897928','6','86']
  H_HDV['41_THC_0_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004256574329265','0.071251719999069','3.59781144296764','1.48427104171061','0.007418473386664','6','86']
  H_HDV['41_NOx_0_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','50','NOx','y=((a*(b**x))*(x**c))','21.9275862327577','1.01106534141798','-0.714140310897456','0','0','6','86']
  H_HDV['41_PM_0_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004223312153058','0.152038776291652','3.19027982667236','1.60789537338058','-0.010072250333453','6','86']
  H_HDV['41_FC_0_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001773129528533','0.000192227173366','-0.000001535595703','','','6','86']
  H_HDV['41_CO_0_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040652260936048','0.493329034403401','2.50163403339241','1.20889445587558','0.010898846407651','6','86']
  H_HDV['41_THC_0_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.029228574949809','0.042643756823298','0.053868705295016','0.11856547536068','0.005352949023918','6','86']
  H_HDV['41_NOx_0_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.58789083155732','0.066783155328454','40.5430062538847','0.565761750645812','2.44041132604131','6','86']
  H_HDV['41_PM_0_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005093945294733','0.160789349273939','2.91760148379768','1.52091409009047','-0.006293121611531','6','86']
  H_HDV['41_FC_0_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001896074342942','0.000154785667723','-0.00000115762165','0','0','6','86']
  H_HDV['41_CO_-2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.039231179187425','1.99191916825822','-1.75621727667759','-0.094753243695458','0.077975850630292','6','86']
  H_HDV['41_THC_-2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','0','THC','y=(c+(a*exp(b*x)))','0.065879976991331','-0.06739613233372','0.006864541098576','0','0','6','86']
  H_HDV['41_NOx_-2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.3478257722826','0.093192276372906','183.00493565595','0.90353689372952','1.02698777217083','6','86']
  H_HDV['41_PM_-2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007773144768214','0.107798612941682','4.21427292798482','1.71363224691028','0.015666083263407','6','86']
  H_HDV['41_FC_-2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','410.47173145045','0.421486452369532','-26.0408919603121','0.069819799241786','0','6','86']
  H_HDV['41_CO_-2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.03726349670492','1.52253769197755','-1.57636477999776','-0.190654757461719','0.089078028730963','6','86']
  H_HDV['41_THC_-2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006600414700345','0.346321471590474','-1.69810460847986','-0.129285688433421','0.084939201202325','6','86']
  H_HDV['41_NOx_-2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.35616180184649','0.073053260318655','151.508853788497','0.817199776221237','0.87504759126576','6','86']
  H_HDV['41_PM_-2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.0074175091633','0.110206974851682','3.84952964501957','1.55516325952723','0.024355637118528','6','86']
  H_HDV['41_FC_-2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','50','FC','y=(c+(a*exp(b*x)))','347.567716001459','-0.072787555401623','53.2634893371129','0','0','6','86']
  H_HDV['41_CO_-2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033027784337877','1.06801511940401','-1.00182582093465','-0.10130224888216','0.08502483409502','6','86']
  H_HDV['41_THC_-2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005761032497714','0.347682284470077','-1.72491335446004','-0.134232698098403','0.085292145954763','6','86']
  H_HDV['41_NOx_-2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.60068612747411','0.055912943867675','104.520107208754','0.712349385887132','0.714716392848742','6','86']
  H_HDV['41_PM_-2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006467974523732','0.114750441782945','3.52589059156989','1.44650124058025','0.026358233363137','6','86']
  H_HDV['41_FC_-2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-2','100','FC','y=(c+(a*exp(b*x)))','329.62236275421','-0.061329634604186','44.9186456521956','0','0','6','86']
  H_HDV['41_CO_-4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.023115407329081','1.76995909546985','-2.18499201826315','-0.447065959402796','0.107117881275063','6','86']
  H_HDV['41_THC_-4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004606160732976','0.266057999336708','-1.52590386345089','-0.213694271212108','0.093678711247466','6','86']
  H_HDV['41_NOx_-4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.89394301477587','0.071457187638735','69.8012166228846','0.669752811509238','0.276145430627715','6','86']
  H_HDV['41_PM_-4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004931086693267','0.116659818136282','3.25180390858747','1.29004957417561','0.040409173682182','6','86']
  H_HDV['41_FC_-4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','298.477450316618','0.069144866161691','20568.1932090743','1.03085852183775','18.4554242995308','6','86']
  H_HDV['41_CO_-4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001956692875005','0.231008404774107','6.83532180217677','2.48674031361774','-0.011310065516805','6','86']
  H_HDV['41_THC_-4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000567573166114','0.047313696347535','6.1571225611069','2.30674245136433','-0.011251112573824','6','86']
  H_HDV['41_NOx_-4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.71671780593652','0.054934847749525','57.9902532951044','0.585765893217833','0.124146081690034','6','86']
  H_HDV['41_PM_-4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.138016962708984','-0.000082022523636','-0.012809189342225','0.099074377471636','0','6','86']
  H_HDV['41_FC_-4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','234.611050450264','0.054163465498761','5524.78893946547','0.731986837467765','8.13079849724581','6','86']
  H_HDV['41_CO_-4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.315584672115136','-0.000224500337302','-0.022059682756918','0.075224707413549','0','6','86']
  H_HDV['41_THC_-4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.064046863445017','-0.000047277889732','-0.004740250121726','0.080621396086703','0','6','86']
  H_HDV['41_NOx_-4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.36655481622434','0.049056276472757','77.0816353507904','0.62274381693969','0.070417288205587','6','86']
  H_HDV['41_PM_-4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','5.58088483702083','0.90276463344208','0.046915612486671','0','0','6','86']
  H_HDV['41_FC_-4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','208.416269262302','0.047974132548041','6870.08605864031','0.745458196139974','4.41828244838131','6','86']
  H_HDV['41_CO_-6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.300602407284396','0.000322900962869','-0.019299180315997','0.061899195507191','0','6','72']
  H_HDV['41_THC_-6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','0','THC','y=(c+(a*exp(b*x)))','0.061344670313512','-0.073301678486602','0.002109541629969','0','0','6','72']
  H_HDV['41_NOx_-6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.11280187779631','0.071290881185387','82.2897471810285','0.677064122855931','0.055068303700593','6','72']
  H_HDV['41_PM_-6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089460454202674','0.076073128507881','0.074910360402381','0.225455719917245','0.002727759084423','6','72']
  H_HDV['41_FC_-6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','252.318542415845','0.069095165536551','10724.7673175663','0.873847085241235','4.05075345808697','6','72']
  H_HDV['41_CO_-6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','50','CO','y=(c+(a*exp(b*x)))','0.274681378560305','-0.072096377565694','0.005736269770664','0','0','6','72']
  H_HDV['41_THC_-6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.052021273674531','0.070801419952219','0.050577588433527','0.465323449519374','0.001867052571799','6','72']
  H_HDV['41_NOx_-6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.45264786140596','0.066794209551192','95.6757902414346','0.683813274129493','0.035897060495148','6','72']
  H_HDV['41_PM_-6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080586703101576','0.074406724745094','0.13602302405279','0.306695173158007','0.002453347374251','6','72']
  H_HDV['41_FC_-6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','205.768656394003','0.064092897732927','9572.76381591278','0.818553305604226','2.67036532431822','6','72']
  H_HDV['41_CO_-6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.225267120643034','0.066721658481606','3.3850078111278','0.852861211165928','0.004690534574728','6','72']
  H_HDV['41_THC_-6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.045631420122044','0.068304156790718','0.145972407400166','0.555820554797316','0.001757401893654','6','72']
  H_HDV['41_NOx_-6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.10675799737031','0.064142572217186','105.675064564675','0.694511290027326','0.02979773827905','6','72']
  H_HDV['41_PM_-6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.074283437254677','0.073053919137253','0.197988252030212','0.360584680077075','0.00235659553534','6','72']
  H_HDV['41_FC_-6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','180.053851317994','0.060747985228309','10057.9867641416','0.813205986947461','2.13594418857826','6','72']
  H_HDV['41_CO_2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.061408724038642','0.295993126276141','7.10098060111032','2.72656195839903','-0.014867385676948','6','86']
  H_HDV['41_THC_2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','0.502253084817598','-6.4446869329926','-1.27300882201489','0','0','6','86']
  H_HDV['41_NOx_2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.8550960181061','0.00879725195659','-1.06942856457222','0.116261993948672','0','6','86']
  H_HDV['41_PM_2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.162331756753762','-0.000132811351969','-0.014654706983087','0.104269240957869','0','6','86']
  H_HDV['41_FC_2_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','530.531307834278','0.908846527687236','-29.0191547618227','0.071777322126323','0','6','86']
  H_HDV['41_CO_2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.608335719782802','-0.000625562331193','-0.055878091995379','0.113329969931049','0','6','86']
  H_HDV['41_THC_2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002823989795433','0.062601775863826','5.88445355540269','2.23598287016085','-0.014554084673245','6','86']
  H_HDV['41_NOx_2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.7302102020611','0.002234642077534','-1.14824875605796','0.126046327014916','0','6','86']
  H_HDV['41_PM_2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.167752723711562','-0.000160381580589','-0.015489790073915','0.107805982931732','0','6','86']
  H_HDV['41_FC_2_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.053599034131774','1.68424179897243','936.378602290431','-0.434441449607326','0','6','86']
  H_HDV['41_CO_2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.655024602366248','-0.001073750017081','-0.060784389898944','0.118083664507719','0','6','86']
  H_HDV['41_THC_2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004647907015892','0.058791874728745','6.58032287958715','2.51889428393342','-0.017431502049639','6','86']
  H_HDV['41_NOx_2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.25578236074522','0.082201414745466','28.8706134744165','0.513354267270819','4.20876886406094','6','86']
  H_HDV['41_PM_2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.035589354512744','0.009444752852899','0.142809825580633','0.114322653712341','-0.003887639459011','6','86']
  H_HDV['41_FC_2_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','579.077283027951','0.881756998593838','-25.0775888528228','0.066811602142316','0','6','86']
  H_HDV['41_CO_4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.5659038412781','0.000850681926684','-0.040339410928398','0.079190708154128','0','6','72']
  H_HDV['41_THC_4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','0','THC','y=exp((a+(b/x))+(c*log(x)))','0.603197785470829','-6.53430636884587','-1.30543036283102','0','0','6','72']
  H_HDV['41_NOx_4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','0','NOx','y=((a*(x**b))+(c*(x**d)))','0.120245190180777','0.750276340742944','28.1999953576839','-0.696457640031347','0','6','72']
  H_HDV['41_PM_4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014257490275747','0.119449778517294','4.1579221939997','1.77152587863919','0.007160063009147','6','72']
  H_HDV['41_FC_4_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','1129.03947736353','1.01054747152918','-0.539024244617508','0','0','6','72']
  H_HDV['41_CO_4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.697261723122831','-0.00126190010867','-0.063237791972366','0.116737162202815','0','6','71']
  H_HDV['41_THC_4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00529143171859','0.058915918178092','6.97925171382187','2.67319464801518','-0.019723133817171','6','71']
  H_HDV['41_NOx_4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.29821942720589','0.067998073851063','14.3091375482575','0.292743260454288','4.9634783369097','6','71']
  H_HDV['41_PM_4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.036284043489528','0.014828267043798','0.142677351176186','0.118692521767142','0.002203881145635','6','71']
  H_HDV['41_FC_4_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','667.381141933369','0.569357417349249','-34.997206084952','0.088288615275662','0','6','71']
  H_HDV['41_CO_4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.257000664364699','0.02482487584719','0.555020967844902','0.152315159080757','0.02985804238732','6','69']
  H_HDV['41_THC_4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006386024666705','0.057345683789747','7.04774899383552','2.72844290923609','-0.020056961613913','6','69']
  H_HDV['41_NOx_4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.48631281973559','0.069880411290961','19.2036618123328','0.382875920591431','5.87633300311758','6','69']
  H_HDV['41_PM_4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.196309522414657','-0.000355168016402','-0.020253181905232','0.125567936196159','0','6','69']
  H_HDV['41_FC_4_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.0025323113454','0.378361715810324','-18.4276523024963','655.675708998104','0','6','69']
  H_HDV['41_CO_6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.801321194878978','-0.001500699249016','-0.094621712296673','0.151044990333','0','6','71']
  H_HDV['41_THC_6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005417144868446','0.057956775697356','7.51314352094323','2.85209566274421','-0.023584692596754','6','71']
  H_HDV['41_NOx_6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.3400963109473','0.072061385416294','12.3783024151005','0.232462950121947','5.00512558958634','6','71']
  H_HDV['41_PM_6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.190155115944299','-0.00029119686987','-0.019903318792546','0.126736349596451','0','6','71']
  H_HDV['41_FC_6_0'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','0','FC','y=((a*(x**b))+(c*(x**d)))','0.015084312016089','2.07536043672446','959.578428781662','-0.354063223047136','0','6','71']
  H_HDV['41_CO_6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.919554859561975','-0.002292744906581','-0.106787718956747','0.150794154005776','0','6','66']
  H_HDV['41_THC_6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007143266430943','0.054833033487222','7.94209261139786','3.05429608111006','-0.026192511213151','6','66']
  H_HDV['41_NOx_6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','50','NOx','y=((a*(b**x))*(x**c))','22.7576963145267','1.00802220635162','-0.428061877127925','0','0','6','66']
  H_HDV['41_PM_6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.2141213107474','-0.000439633737957','-0.024829835900409','0.142665021167573','0','6','66']
  H_HDV['41_FC_6_50'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','50','FC','y=((a*(b**x))*(x**c))','1074.41861439167','1.00728682803986','-0.355433003154089','0','0','6','66']
  H_HDV['41_CO_6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.66450435278441','-6.40103569723975','-1.23111551103709','0','0','6','58']
  H_HDV['41_THC_6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011552901119717','0.675430458073116','-2.90895971351891','-0.649379341558345','0.145364470720505','6','58']
  H_HDV['41_NOx_6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.05906687601863','0.031509414342147','16.539704819023','0.296952633727705','6.70306134100475','6','58']
  H_HDV['41_PM_6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012575872031272','0.139161476408506','4.45119139136832','1.86855084490721','-0.003854403170564','6','58']
  H_HDV['41_FC_6_100'] = [' 41','EURO4','RT >7.5-12t Euro-4','6','100','FC','y=((a*(b**x))*(x**c))','1099.65994813197','1.0063080995111','-0.294219206939264','0','0','6','58']
  H_HDV['42_CO_0_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040067020706795','0.363383794022657','3.84682534945166','1.53536813122235','0.011174212876931','6','86']
  H_HDV['42_THC_0_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003536575141412','0.074047259191827','3.64635570244111','1.50571899181147','0.00224851684291','6','86']
  H_HDV['42_NOx_0_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','0','NOx','y=((a*(b**x))*(x**c))','20.340576440102','1.01628549562649','-0.930642978992005','0','0','6','86']
  H_HDV['42_PM_0_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050786763581455','0.048310072376211','0.130792999849757','0.16793382651672','0.010142063886854','6','86']
  H_HDV['42_FC_0_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00150480361885','0.000237215267778','-0.000001983149037','0','0','6','86']
  H_HDV['42_CO_0_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04117566804345','0.389806022006634','3.6598149679362','1.52602897386456','0.007518989340722','6','86']
  H_HDV['42_THC_0_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004573121147434','0.073981306002659','3.44237729069182','1.4261749853646','0.01009531967114','6','86']
  H_HDV['42_NOx_0_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.44099364880209','0.084756452150753','20.4435041613841','0.499357865499268','1.3261325645088','6','86']
  H_HDV['42_PM_0_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00401435242291','0.154978031828111','3.20408366784649','1.61171754131462','-0.010574830703191','6','86']
  H_HDV['42_FC_0_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001701891752959','0.000189427638168','-0.000001502808428','','','6','86']
  H_HDV['42_CO_0_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.036013190272493','0.442185979436662','3.22778985134631','1.43830696102153','0.001421889047987','6','86']
  H_HDV['42_THC_0_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.030315171517787','0.042841496103012','0.054670033146918','0.119333577052567','0.00547700137954','6','86']
  H_HDV['42_NOx_0_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.74996267963323','0.063936901025369','16.5352093812571','0.426345523950771','1.41523876427158','6','86']
  H_HDV['42_PM_0_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005264316533238','0.162065761196869','2.96527014853013','1.53374426754297','-0.006330330857797','6','86']
  H_HDV['42_FC_0_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001822985949003','0.00015302007899','-0.000001137263379','0','0','6','86']
  H_HDV['42_CO_-2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','0','CO','y=(c+(a*exp(b*x)))','0.349186855720737','-0.066549439909931','0.038560373184584','0','0','6','86']
  H_HDV['42_THC_-2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','0','THC','y=(c+(a*exp(b*x)))','0.067327103631515','-0.067336069315076','0.007028299903377','0','0','6','86']
  H_HDV['42_NOx_-2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.26327973333518','0.094865072436275','73.6377730354495','0.779795869087472','0.602920595971569','6','86']
  H_HDV['42_PM_-2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007826498565172','0.109613654259312','4.27066001751119','1.73694663113446','0.014374063059887','6','86']
  H_HDV['42_FC_-2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','425.526408765719','0.406561464110256','-27.1632639100499','0.070345792694647','0','6','86']
  H_HDV['42_CO_-2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037927319561402','1.70324849190834','-1.70667191607832','-0.204011793419084','0.089267426757211','6','86']
  H_HDV['42_THC_-2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006747287899723','0.370066811728854','-1.75721199522235','-0.134586380434218','0.084959792477638','6','86']
  H_HDV['42_NOx_-2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.46933661264809','0.074289995406428','53.0462795707752','0.670440184537521','0.512298092821286','6','86']
  H_HDV['42_PM_-2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00749334789568','0.111666209064968','3.92549736610974','1.58442917506532','0.023027803567991','6','86']
  H_HDV['42_FC_-2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','50','FC','y=(c+(a*exp(b*x)))','360.710800672815','-0.072694276928018','54.1238942564019','0','0','6','86']
  H_HDV['42_CO_-2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033559165825518','1.0543756906487','-0.946096855566489','-0.093043552981383','0.084636868303686','6','86']
  H_HDV['42_THC_-2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005883347082816','0.350216028824167','-1.70005024519572','-0.12940938835355','0.084858648854964','6','86']
  H_HDV['42_NOx_-2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.93014725237694','0.057266025043148','45.389087877266','0.606528888869712','0.418928008857025','6','86']
  H_HDV['42_PM_-2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.020418327853544','0.036715955186036','0.120149724396992','0.103628725001598','0.006275923191491','6','86']
  H_HDV['42_FC_-2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','293.114034664127','0.054675143084031','52322.1452692645','1.15520040079829','43.425087611537','6','86']
  H_HDV['42_CO_-4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.023532101398304','1.91721726207526','-2.24403291507161','-0.440696425598851','0.106117089385572','6','86']
  H_HDV['42_THC_-4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004685170576925','0.272410450908965','-1.50948553960115','-0.201868750611327','0.092575753797609','6','86']
  H_HDV['42_NOx_-4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.12991725385612','0.072306064143205','34.1670650934127','0.583471347811943','0.161942118146938','6','86']
  H_HDV['42_PM_-4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004983634446689','0.118220175588226','3.33087885852564','1.32266828986191','0.038730177120347','6','86']
  H_HDV['42_FC_-4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','308.770147183725','0.069000943188921','18402.4864313914','1.0050850178922','18.8196389027578','6','86']
  H_HDV['42_CO_-4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001295582289796','0.236960415863723','6.86439821472883','2.5029801677771','-0.012707944778321','6','86']
  H_HDV['42_THC_-4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000618791133854','0.048444020550105','6.12602499860069','2.29195392240075','-0.010533940373016','6','86']
  H_HDV['42_NOx_-4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.41347646117953','0.056984064845875','35.495108780112','0.550606818340352','0.075891897561381','6','86']
  H_HDV['42_PM_-4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.141056862354955','-0.000084008947534','-0.013076380910454','0.098961789061911','0','6','86']
  H_HDV['42_FC_-4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','243.674752959964','0.054396872040256','5490.51352494582','0.725244133438636','8.33130684247763','6','86']
  H_HDV['42_CO_-4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.32256361539175','-0.000228814871717','-0.022534949292278','0.075162257377635','0','6','86']
  H_HDV['42_THC_-4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.065509028341774','-0.000048485098659','-0.00484253536626','0.080527183767476','0','6','86']
  H_HDV['42_NOx_-4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.18809110875111','0.051389897069644','45.8346731944865','0.585372785336695','0.04619070605382','6','86']
  H_HDV['42_PM_-4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.13981523029075','-0.000096275052765','-0.013468179092434','0.103177795923685','0','6','86']
  H_HDV['42_FC_-4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','216.149516134682','0.048298787796411','6768.66670914607','0.737262202341351','4.59956064231924','6','86']
  H_HDV['42_CO_-6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.306905655411145','0.000336853663871','-0.019631821616628','0.06158706955314','0','6','72']
  H_HDV['42_THC_-6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','0','THC','y=(c+(a*exp(((-1)*b)*x)))','0.062800343548101','0.073165195710894','0.002147626915585','0','0','6','72']
  H_HDV['42_NOx_-6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.69250741278665','0.07299229208922','47.2719217775169','0.628890257630539','0.034313997325548','6','72']
  H_HDV['42_PM_-6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.091179749167154','0.075930936540602','0.074125050949892','0.219173456565951','0.002780880880414','6','72']
  H_HDV['42_FC_-6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','262.718188106793','0.069331530187806','11032.9225417271','0.874192015580046','4.19893565128661','6','72']
  H_HDV['42_CO_-6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','50','CO','y=(c+(a*exp(b*x)))','0.281578894359285','-0.072147584022127','0.005862001545977','0','0','6','72']
  H_HDV['42_THC_-6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053470936482744','0.070929934745697','0.043149712241312','0.442982179809859','0.001910694967898','6','72']
  H_HDV['42_NOx_-6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.26433032292487','0.068435566009996','54.545823108307','0.637559740538419','0.022714694065577','6','72']
  H_HDV['42_PM_-6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082385454006026','0.074386641846652','0.13400745886382','0.300537590028378','0.002504146538248','6','72']
  H_HDV['42_FC_-6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','214.052604213379','0.064381773996322','9485.58011542049','0.811668188064828','2.79164002148827','6','72']
  H_HDV['42_CO_-6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','100','CO','y=(c+(a*exp(b*x)))','0.257307790424424','-0.071817616268308','0.005639735565853','0','0','6','72']
  H_HDV['42_THC_-6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.046973130610206','0.068525831829344','0.141604348870777','0.551901305270781','0.00180175036347','6','72']
  H_HDV['42_NOx_-6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.05616003955655','0.066062668659952','60.8783395567383','0.653063378365592','0.019333293050102','6','72']
  H_HDV['42_PM_-6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.076024850078987','0.073079028782943','0.197133198214715','0.356267390987726','0.002404669444559','6','72']
  H_HDV['42_FC_-6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','187.312242064996','0.061082737269935','10126.5875608352','0.809361831445611','2.24705884785866','6','72']
  H_HDV['42_CO_2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.060231342716165','0.304360679273106','6.92193820842878','2.65695880682882','-0.01432891715398','6','86']
  H_HDV['42_THC_2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','0.531015054369017','-6.48046897836015','-1.27411454060456','0','0','6','86']
  H_HDV['42_NOx_2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.63943151045966','0.003604567943282','-0.925534941236619','0.132308191915147','0','6','86']
  H_HDV['42_PM_2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.030248242883985','0.007275772148251','0.141061605692762','0.107500573660032','-0.004258936705284','6','86']
  H_HDV['42_FC_2_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','547.46416574081','0.898905510068492','-30.1641067777262','0.072102493568855','0','6','86']
  H_HDV['42_CO_2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.611287475975374','-0.000609911046144','-0.055454263251366','0.111815426411193','0','6','86']
  H_HDV['42_THC_2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002685370359638','0.064333675561798','5.82145499830417','2.21147964370421','-0.014417591082766','6','86']
  H_HDV['42_NOx_2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.60615687464148','-0.00125478298829','-1.10447190818356','0.150143897028972','0','6','86']
  H_HDV['42_PM_2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.170496441624648','-0.00016073531016','-0.015689057027755','0.107342963312401','0','6','86']
  H_HDV['42_FC_2_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.050659089919245','1.69808691029318','969.252178523026','-0.438334344016681','0','6','86']
  H_HDV['42_CO_2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.666231005594494','-0.001044865766796','-0.062205358397474','0.118329228476895','0','6','86']
  H_HDV['42_THC_2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004554505618775','0.060406526924114','6.5164320527023','2.49270729089231','-0.017264866914637','6','86']
  H_HDV['42_NOx_2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','1.13635779102378','4.61535727430849','-0.08013446870943','0','0','6','86']
  H_HDV['42_PM_2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.038207095112961','0.008061867019444','0.145894989040151','0.113131187389229','-0.006911059714234','6','86']
  H_HDV['42_FC_2_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','596.809745295825','0.880418126366848','-26.1322708305019','0.067055332580859','0','6','86']
  H_HDV['42_CO_4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.571299256270151','0.000969778038003','-0.040105435696696','0.077191640043588','0','6','72']
  H_HDV['42_THC_4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','0','THC','y=exp((a+(b/x))+(c*log(x)))','0.663779879925656','-6.67359466990519','-1.31507449045757','0','0','6','72']
  H_HDV['42_NOx_4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','0','NOx','y=((a*(x**b))+(c*(x**d)))','0.06266667658594','0.790506949527503','21.6828643893397','-0.776219578346066','0','6','72']
  H_HDV['42_PM_4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.167395905333549','-0.000086939980171','-0.014369109317566','0.098868051697446','0','6','72']
  H_HDV['42_FC_4_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','1174.829782419','1.01055572162552','-0.545383215430074','0','0','6','72']
  H_HDV['42_CO_4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.671321918252853','-0.000748066442982','-0.054347468509168','0.101169138461094','0','6','71']
  H_HDV['42_THC_4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005038277862649','0.061119955063903','6.75463974529763','2.58612023854305','-0.018350607930798','6','71']
  H_HDV['42_NOx_4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','50','NOx','y=((a*(x**b))+(c*(x**d)))','1.71780437591663','0.085632359242509','25.8092464269975','-0.996065684775469','0','6','71']
  H_HDV['42_PM_4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.041680013600841','0.035046509736223','0.132272581087654','0.129658743512359','0.012763985807266','6','71']
  H_HDV['42_FC_4_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','50','FC','y=((a*(x**b))+(c*(x**d)))','1.23528790842048','1.07530091896956','1068.82077937938','-0.405583094909738','0','6','71']
  H_HDV['42_CO_4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.258648802150941','0.02500251015054','0.547851180204072','0.150203780337818','0.032392351235078','6','70']
  H_HDV['42_THC_4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006549630828769','0.058458854839966','7.08724240508711','2.73982223086006','-0.019784422980534','6','70']
  H_HDV['42_NOx_4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.51141447573732','0.039559377617056','13.0694127124565','0.307195250890461','2.99238263709891','6','70']
  H_HDV['42_PM_4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.198224343018793','-0.000349141188365','-0.020211965248359','0.123960220496671','0','6','70']
  H_HDV['42_FC_4_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002609823278777','0.391758380705777','-19.142174467853','673.128747687918','0','6','70']
  H_HDV['42_CO_6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.809607226863108','-0.001471023591154','-0.095140959178498','0.150054578942285','0','6','71']
  H_HDV['42_THC_6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005343984140889','0.059539580698513','7.45026403680619','2.82679995297231','-0.023422977431675','6','71']
  H_HDV['42_NOx_6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.326568781997','-0.009225185563532','-2.37792355812255','0.217260259029728','0','6','71']
  H_HDV['42_PM_6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.193808801240347','-0.00029290543319','-0.020338422927542','0.126959617399738','0','6','71']
  H_HDV['42_FC_6_0'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','0','FC','y=((a*(x**b))+(c*(x**d)))','0.016402039997011','2.05995530111297','995.691130858394','-0.36014865552494','0','6','71']
  H_HDV['42_CO_6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.940142977329928','-0.002428922213308','-0.111912123118417','0.155635678019362','0','6','67']
  H_HDV['42_THC_6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007129928156914','0.056458504133197','7.81197472032363','3.00421422848566','-0.025194168492521','6','67']
  H_HDV['42_NOx_6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.09620571180986','0.025515104993249','11.7145827208716','0.254479309923767','2.99260449986668','6','67']
  H_HDV['42_PM_6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.218512308539702','-0.000453167094719','-0.025520106279829','0.143953108753962','0','6','67']
  H_HDV['42_FC_6_50'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','780.915852103952','0.379324121055634','-39.2553187608085','0.096200602899083','0','6','67']
  H_HDV['42_CO_6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.66648898182171','-6.42634621741094','-1.22597568254776','0','0','6','59']
  H_HDV['42_THC_6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011764776899379','0.734315859795972','-3.00901815439236','-0.673343746374617','0.147059472996049','6','59']
  H_HDV['42_NOx_6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.50910637314407','0.031331703302013','12.0869497463217','0.290918265897243','3.34409091791152','6','59']
  H_HDV['42_PM_6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010619302018549','0.136358887775042','4.99826054360138','2.08207416692057','-0.015315560079528','6','59']
  H_HDV['42_FC_6_100'] = [' 42','EURO5','RT >7.5-12t Euro-5','6','100','FC','y=((a*(b**x))*(x**c))','1146.88040949946','1.00670790739562','-0.305731978133945','0','0','6','59']
  H_HDV['43_CO_0_0'] = [' 43','EURO0','RT >12-14t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.17059770540902','0.055198616477192','10.425997414107','0.199024738291489','1.46873738371049','6','86']
  H_HDV['43_THC_0_0'] = [' 43','EURO0','RT >12-14t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.99183196333313','0.030774742936779','6.32561199866224','0.13023830194583','0.424579655947924','6','86']
  H_HDV['43_NOx_0_0'] = [' 43','EURO0','RT >12-14t 80ties','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.0657972278383','0.035649010753927','-1.21653619760612','0.065159272081906','0','6','86']
  H_HDV['43_PM_0_0'] = [' 43','EURO0','RT >12-14t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.01540251549142','0.052878064442067','1.7766786732758','0.189481699506009','0.234694161372988','6','86']
  H_HDV['43_FC_0_0'] = [' 43','EURO0','RT >12-14t 80ties','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000896339794193','0.000180310983395','-0.000001386836492','0','0','6','86']
  H_HDV['43_CO_0_50'] = [' 43','EURO0','RT >12-14t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.53707525840325','0.056913341983869','11.8122149716005','0.227682503769612','1.64079704915359','6','86']
  H_HDV['43_THC_0_50'] = [' 43','EURO0','RT >12-14t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.243760787720553','7.10826259551006','3.51236829384046','1.60927601441032','-0.00670056870942','6','86']
  H_HDV['43_NOx_0_50'] = [' 43','EURO0','RT >12-14t 80ties','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.9635210324861','0.024770116829053','-1.01040586360789','0.058606680384842','0','6','86']
  H_HDV['43_PM_0_50'] = [' 43','EURO0','RT >12-14t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.12883195812743','0.055287587892741','2.07272117719613','0.231507360306843','0.257464183060985','6','86']
  H_HDV['43_FC_0_50'] = [' 43','EURO0','RT >12-14t 80ties','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001014512596771','0.000150552030374','-0.000001108428155','','','6','86']
  H_HDV['43_CO_0_100'] = [' 43','EURO0','RT >12-14t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.25326046796549','0.053563906144893','15.052960281868','0.254659762912255','1.83179952558672','6','86']
  H_HDV['43_THC_0_100'] = [' 43','EURO0','RT >12-14t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.231110427650115','6.65897712159753','3.9195049812497','1.74942306042141','-0.010190530724236','6','86']
  H_HDV['43_NOx_0_100'] = [' 43','EURO0','RT >12-14t 80ties','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.7792679904378','0.017955644140826','-0.803569736694065','0.049800235079196','0','6','86']
  H_HDV['43_PM_0_100'] = [' 43','EURO0','RT >12-14t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.10244819892281','0.052463478568167','2.5019823039918','0.253314825880842','0.283032616822959','6','86']
  H_HDV['43_FC_0_100'] = [' 43','EURO0','RT >12-14t 80ties','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001120183184423','0.000124898577135','-0.000000871690662','0','0','6','86']
  H_HDV['43_CO_-2_0'] = [' 43','EURO0','RT >12-14t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.6758628563093','0.010969017945095','-0.871329006562666','0.072325641804679','0','6','86']
  H_HDV['43_THC_-2_0'] = [' 43','EURO0','RT >12-14t 80ties','-2','0','THC','y=(c+(a*exp(b*x)))','6.3615141874361','-0.083723491329964','0.607184992588928','0','0','6','86']
  H_HDV['43_NOx_-2_0'] = [' 43','EURO0','RT >12-14t 80ties','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3773.89099869451','1.24725102892583','15.5383842717273','0.077946859637821','3.32596987757856','6','86']
  H_HDV['43_PM_-2_0'] = [' 43','EURO0','RT >12-14t 80ties','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.06563420766894','0.000790541385395','-0.15184921526125','0.079026204911838','0','6','86']
  H_HDV['43_FC_-2_0'] = [' 43','EURO0','RT >12-14t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','87370.8561581753','1.16508202461867','467.310436974249','0.080933586258157','78.5325879873932','6','86']
  H_HDV['43_CO_-2_50'] = [' 43','EURO0','RT >12-14t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.2714629430924','0.014405660480033','-0.804232098478806','0.066961887940558','0','6','86']
  H_HDV['43_THC_-2_50'] = [' 43','EURO0','RT >12-14t 80ties','-2','50','THC','y=(c+(a*exp(b*x)))','6.25818158109624','-0.082237289289761','0.503479825914302','0','0','6','86']
  H_HDV['43_NOx_-2_50'] = [' 43','EURO0','RT >12-14t 80ties','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000056549469569','0.01065084948787','-0.672195966168067','17.2777724402344','0','6','86']
  H_HDV['43_PM_-2_50'] = [' 43','EURO0','RT >12-14t 80ties','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.00121244344825','0.001113796226622','-0.140206499940221','0.073815444546873','0','6','86']
  H_HDV['43_FC_-2_50'] = [' 43','EURO0','RT >12-14t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','417.58333471689','0.064145976292052','70002.3310792317','1.10169341192282','61.5864781314973','6','86']
  H_HDV['43_CO_-2_100'] = [' 43','EURO0','RT >12-14t 80ties','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.423837666626','0.009570402668104','-0.820396848348768','0.068257020709215','0','6','86']
  H_HDV['43_THC_-2_100'] = [' 43','EURO0','RT >12-14t 80ties','-2','100','THC','y=(c+(a*exp(b*x)))','6.17693329109953','-0.079910685942423','0.415320607348104','0','0','6','86']
  H_HDV['43_NOx_-2_100'] = [' 43','EURO0','RT >12-14t 80ties','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000050989409052','0.009535046811903','-0.621849913278304','16.960030040616','0','6','86']
  H_HDV['43_PM_-2_100'] = [' 43','EURO0','RT >12-14t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.71536577922709','0.074350332514481','16.6267222329594','0.797480789050661','0.147734505845706','6','86']
  H_HDV['43_FC_-2_100'] = [' 43','EURO0','RT >12-14t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','382.304457248636','0.050871055165946','47914.6626491807','1.00991863349471','46.0082644118742','6','86']
  H_HDV['43_CO_-4_0'] = [' 43','EURO0','RT >12-14t 80ties','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.3065411126058','0.076724329668478','28.4664656987261','0.63741550521449','0.454708073538036','6','86']
  H_HDV['43_THC_-4_0'] = [' 43','EURO0','RT >12-14t 80ties','-4','0','THC','y=(c+(a*exp(b*x)))','6.10613413769865','-0.082608874207475','0.289830384156885','0','0','6','86']
  H_HDV['43_NOx_-4_0'] = [' 43','EURO0','RT >12-14t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.4432518998426','0.054805743265658','483.870287374377','0.823754326009054','0.555243957454489','6','86']
  H_HDV['43_PM_-4_0'] = [' 43','EURO0','RT >12-14t 80ties','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.61480754028037','0.077122057540213','6.56831539216621','0.65028333460632','0.082605179756323','6','86']
  H_HDV['43_FC_-4_0'] = [' 43','EURO0','RT >12-14t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','340.580345130069','0.059612867508834','18256.4283977226','0.839411280785768','15.695799029383','6','86']
  H_HDV['43_CO_-4_50'] = [' 43','EURO0','RT >12-14t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.24523980724152','0.065668098827347','31.3845755855452','0.497719411796419','0.285071824167878','6','86']
  H_HDV['43_THC_-4_50'] = [' 43','EURO0','RT >12-14t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.82480579560455','0.07320382812105','4.44403277014268','0.340328929867708','0.216977396764242','6','86']
  H_HDV['43_NOx_-4_50'] = [' 43','EURO0','RT >12-14t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.74871932656949','0.044793177725158','546.3916155423','0.819549689862683','0.213924944726264','6','86']
  H_HDV['43_PM_-4_50'] = [' 43','EURO0','RT >12-14t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3946272304279','0.070258816506602','7.38186428510059','0.579704729862393','0.066284785943669','6','86']
  H_HDV['43_FC_-4_50'] = [' 43','EURO0','RT >12-14t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','288.735275750123','0.050133420992943','17129.2490249863','0.807002835746504','7.05074956681867','6','86']
  H_HDV['43_CO_-4_100'] = [' 43','EURO0','RT >12-14t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.39848135150601','0.060680504690365','52.810777257165','0.554158676450201','0.228686352352889','6','86']
  H_HDV['43_THC_-4_100'] = [' 43','EURO0','RT >12-14t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.56243676845062','0.072424684925213','9.0921833412697','0.440336769454121','0.198493563486626','6','86']
  H_HDV['43_NOx_-4_100'] = [' 43','EURO0','RT >12-14t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.08437156506811','0.039642031097007','910.571345819579','0.90316041461146','0.03848772863578','6','86']
  H_HDV['43_PM_-4_100'] = [' 43','EURO0','RT >12-14t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.28325493342626','0.066418646512493','11.2843677316994','0.62514706480286','0.060944149877407','6','86']
  H_HDV['43_FC_-4_100'] = [' 43','EURO0','RT >12-14t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','265.88410819024','0.045711214098758','22791.1868254122','0.851906131802356','3.51346320781751','6','86']
  H_HDV['43_CO_-6_0'] = [' 43','EURO0','RT >12-14t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.67054014977714','0.075011574036912','55.7945334934113','0.660596853486632','0.166107473928122','6','72']
  H_HDV['43_THC_-6_0'] = [' 43','EURO0','RT >12-14t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.05817516260947','0.078071112367957','8.18916773170031','0.500132664338026','0.145755516516884','6','72']
  H_HDV['43_NOx_-6_0'] = [' 43','EURO0','RT >12-14t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.8631107121429','0.061812390609935','750.892741240855','0.904229541414155','0.116951734942018','6','72']
  H_HDV['43_PM_-6_0'] = [' 43','EURO0','RT >12-14t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.41239353636457','0.075925709254797','9.61940829726701','0.667018908938723','0.045020327396068','6','72']
  H_HDV['43_FC_-6_0'] = [' 43','EURO0','RT >12-14t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','287.453291193196','0.063129021333616','20555.8538901356','0.857728293896534','2.83197093055272','6','72']
  H_HDV['43_CO_-6_50'] = [' 43','EURO0','RT >12-14t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.21462628792605','0.070105798520687','88.7124521724096','0.671513561227746','0.137416239887437','6','72']
  H_HDV['43_THC_-6_50'] = [' 43','EURO0','RT >12-14t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.45253832215736','0.075614606374419','20.3851982499549','0.585884637179977','0.13392749688055','6','72']
  H_HDV['43_NOx_-6_50'] = [' 43','EURO0','RT >12-14t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.25433634650457','0.057234977046537','879.604653234018','0.916739001092842','0.073981029210347','6','72']
  H_HDV['43_PM_-6_50'] = [' 43','EURO0','RT >12-14t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.23764505051904','0.07223672706733','13.7412388875671','0.679713141425417','0.041707146160968','6','72']
  H_HDV['43_FC_-6_50'] = [' 43','EURO0','RT >12-14t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','243.302211800765','0.059263326776662','22862.2963205864','0.866519764336477','1.70404908330058','6','72']
  H_HDV['43_CO_-6_100'] = [' 43','EURO0','RT >12-14t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.42041940824005','0.066797338356613','109.458834531384','0.682767725626187','0.123016664294698','6','72']
  H_HDV['43_THC_-6_100'] = [' 43','EURO0','RT >12-14t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.11877174997695','0.074432764919268','25.42408229151','0.607155312305077','0.129403699840876','6','72']
  H_HDV['43_NOx_-6_100'] = [' 43','EURO0','RT >12-14t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.49077709274015','0.054804273705031','985.872422689808','0.933514468197762','0.061770566327681','6','72']
  H_HDV['43_PM_-6_100'] = [' 43','EURO0','RT >12-14t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.14291781097758','0.069906635799346','15.3345211852997','0.678163818517685','0.040175642965033','6','72']
  H_HDV['43_FC_-6_100'] = [' 43','EURO0','RT >12-14t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','221.208969071263','0.057260220492292','24585.144075845','0.876917695396734','1.38863847721657','6','72']
  H_HDV['43_CO_2_0'] = [' 43','EURO0','RT >12-14t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.1229489103244','9.87438631263199','5.7828545461228','2.07254582894421','0.028059263652486','6','86']
  H_HDV['43_THC_2_0'] = [' 43','EURO0','RT >12-14t 80ties','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.13710202215298','6.71015934085749','4.42896290121791','1.79318430274892','-0.008999976114597','6','86']
  H_HDV['43_NOx_2_0'] = [' 43','EURO0','RT >12-14t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-18.6753456970703','0.028734064118807','35.1442735919924','0.057879248042136','13.8512186512714','6','86']
  H_HDV['43_PM_2_0'] = [' 43','EURO0','RT >12-14t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.338253038692094','1.72156369966227','4.69542509293897','1.77052827185858','0.02796975770366','6','86']
  H_HDV['43_FC_2_0'] = [' 43','EURO0','RT >12-14t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','870.730704152088','1.1183281017023','-52.3500813440167','0.073538625854537','0','6','86']
  H_HDV['43_CO_2_50'] = [' 43','EURO0','RT >12-14t 80ties','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.4377819947032','0.012410367464293','-1.27284040523324','0.082869699865179','0','6','86']
  H_HDV['43_THC_2_50'] = [' 43','EURO0','RT >12-14t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.282696590400188','6.1948741986348','5.31562812026555','2.12068412783573','-0.012021508510053','6','86']
  H_HDV['43_NOx_2_50'] = [' 43','EURO0','RT >12-14t 80ties','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','32.6272475862673','0.037617415526594','-1.49105476694734','0.072792131402447','0','6','86']
  H_HDV['43_PM_2_50'] = [' 43','EURO0','RT >12-14t 80ties','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.78242048715462','0.00143136171259','-0.206745013234257','0.085238551385351','0','6','86']
  H_HDV['43_FC_2_50'] = [' 43','EURO0','RT >12-14t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','922.611677950769','1.26047601887804','-52.9250361963236','0.073638354093084','0','6','86']
  H_HDV['43_CO_2_100'] = [' 43','EURO0','RT >12-14t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.4304089496435','0.014816547943848','-1.37698992376168','0.087458791962422','0','6','84']
  H_HDV['43_THC_2_100'] = [' 43','EURO0','RT >12-14t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.325706847975966','6.12914791573268','5.53408381324802','2.21694472657627','-0.012591393049186','6','84']
  H_HDV['43_NOx_2_100'] = [' 43','EURO0','RT >12-14t 80ties','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.7181679824281','0.032614208313597','-1.22575879801055','0.064545833791934','0','6','84']
  H_HDV['43_PM_2_100'] = [' 43','EURO0','RT >12-14t 80ties','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.93109665448955','0.001914186253688','-0.220567336758875','0.088342760686859','0','6','84']
  H_HDV['43_FC_2_100'] = [' 43','EURO0','RT >12-14t 80ties','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','967.23122616537','0.957867108330761','-53.6852497511397','0.0771436263828','0','6','84']
  H_HDV['43_CO_4_0'] = [' 43','EURO0','RT >12-14t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.00508230474584','13.8708754516891','1.12138896986059','-0.19718170488424','0.168802736058454','6','71']
  H_HDV['43_THC_4_0'] = [' 43','EURO0','RT >12-14t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.220581602216081','6.62954627739813','5.14652696084012','2.05538709912278','-0.011150413996997','6','71']
  H_HDV['43_NOx_4_0'] = [' 43','EURO0','RT >12-14t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','37.383339387269','0.027242346038235','-2.13206893026705','0.09293595239274','0','6','71']
  H_HDV['43_PM_4_0'] = [' 43','EURO0','RT >12-14t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.468823033797798','1.68844353119066','4.8316723346623','1.63041060885248','0.063645633818973','6','71']
  H_HDV['43_FC_4_0'] = [' 43','EURO0','RT >12-14t 80ties','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1060.60473983337','0.575990459272255','-74.9185609034956','0.094384935186465','0','6','71']
  H_HDV['43_CO_4_50'] = [' 43','EURO0','RT >12-14t 80ties','4','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000121435575834','0.017904827813896','-0.845023594286173','16.8468697150971','0','6','70']
  H_HDV['43_THC_4_50'] = [' 43','EURO0','RT >12-14t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.373361021818258','6.20018804430776','6.09439050106515','2.43497367810713','-0.016560442979759','6','70']
  H_HDV['43_NOx_4_50'] = [' 43','EURO0','RT >12-14t 80ties','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','39.7017294450383','0.014331375620853','-1.76602027734961','0.086177533339332','0','6','70']
  H_HDV['43_PM_4_50'] = [' 43','EURO0','RT >12-14t 80ties','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000018902530608','0.00280749495453','-0.13411935164308','2.69901506085965','0','6','70']
  H_HDV['43_FC_4_50'] = [' 43','EURO0','RT >12-14t 80ties','4','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1143.05253661454','0.559190275818059','-78.7524803526388','0.100581078457977','0','6','70']
  H_HDV['43_CO_4_100'] = [' 43','EURO0','RT >12-14t 80ties','4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000125592066403','0.018124528655511','-0.834026461603512','17.5569674089022','0','6','65']
  H_HDV['43_THC_4_100'] = [' 43','EURO0','RT >12-14t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.475398936831842','5.86891189566618','6.87381101554268','2.74633837156256','-0.021768311625831','6','65']
  H_HDV['43_NOx_4_100'] = [' 43','EURO0','RT >12-14t 80ties','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.7242577182757','0.076507527648527','4612.6503672394','1.40471747465406','23.7468358585134','6','65']
  H_HDV['43_PM_4_100'] = [' 43','EURO0','RT >12-14t 80ties','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-4.12689984017193','0.056806189837493','6.56445411724489','0.075415191094966','0.927739012893654','6','65']
  H_HDV['43_FC_4_100'] = [' 43','EURO0','RT >12-14t 80ties','4','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1194.26976922011','0.457007122907183','-75.269633872955','0.101812146225028','0','6','65']
  H_HDV['43_CO_6_0'] = [' 43','EURO0','RT >12-14t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.2994634355616','0.068351141902369','-1.21224967295001','0.06902613939842','0','6','70']
  H_HDV['43_THC_6_0'] = [' 43','EURO0','RT >12-14t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.339326855483507','6.17021010514862','6.50901323433122','2.56821202258728','-0.020086828055637','6','70']
  H_HDV['43_NOx_6_0'] = [' 43','EURO0','RT >12-14t 80ties','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.6699883907106','0.027898959158589','-1.93418388684544','0.087594831830679','0','6','70']
  H_HDV['43_PM_6_0'] = [' 43','EURO0','RT >12-14t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.00936874065754','0.008096102086564','-0.208792634601168','0.075262035977316','0','6','70']
  H_HDV['43_FC_6_0'] = [' 43','EURO0','RT >12-14t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1056.49680524161','3.10804035132136','-54.4756479928231','0.065863936527692','0','6','70']
  H_HDV['43_CO_6_50'] = [' 43','EURO0','RT >12-14t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.492329347744','0.049686692120085','-1.51903613250716','0.089422877836851','0','6','59']
  H_HDV['43_THC_6_50'] = [' 43','EURO0','RT >12-14t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.46480448225359','5.93054915589765','7.132243514888','2.84339176289738','-0.02525176816078','6','59']
  H_HDV['43_NOx_6_50'] = [' 43','EURO0','RT >12-14t 80ties','6','50','NOx','y=((a*(b**x))*(x**c))','60.9665343010443','1.00482962056121','-0.281683376311528','0','0','6','59']
  H_HDV['43_PM_6_50'] = [' 43','EURO0','RT >12-14t 80ties','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.3820343446839','0.007294480113752','-0.253357080000386','0.089329109900838','0','6','59']
  H_HDV['43_FC_6_50'] = [' 43','EURO0','RT >12-14t 80ties','6','50','FC','y=((a*(b**x))*(x**c))','2044.22831033029','1.0102826158714','-0.488669791050639','0','0','6','59']
  H_HDV['43_CO_6_100'] = [' 43','EURO0','RT >12-14t 80ties','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.14006248919146','8.58372960629619','8.72778672310631','4.11436369070041','-0.077139375116327','6','49']
  H_HDV['43_THC_6_100'] = [' 43','EURO0','RT >12-14t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.888185029258244','38.605524670948','-1.90047946152311','-0.463527421216781','0.142716461085663','6','49']
  H_HDV['43_NOx_6_100'] = [' 43','EURO0','RT >12-14t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6360727159412','0.059248685571622','33.8477864063237','0.397596291474157','30.6228706245797','6','49']
  H_HDV['43_PM_6_100'] = [' 43','EURO0','RT >12-14t 80ties','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.13104395563376','1.64354200495174','5.60919659917138','2.18642619793015','0.073042564797685','6','49']
  H_HDV['43_FC_6_100'] = [' 43','EURO0','RT >12-14t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','418.068223061054','0.087198184391524','577.62973478854','0.2495934955497','622.732732299209','6','49']
  H_HDV['44_CO_0_0'] = [' 44','EURO1','RT >12-14t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.54283879399967','0.055800037847099','3.96131855666543','0.228859494520018','0.671778145377557','6','86']
  H_HDV['44_THC_0_0'] = [' 44','EURO1','RT >12-14t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.12122094035979','2.30820926154725','3.88887462766706','1.65951910063253','-0.008895610750205','6','86']
  H_HDV['44_NOx_0_0'] = [' 44','EURO1','RT >12-14t Euro-1','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.4689261382203','0.014873498860268','-0.985696833031754','0.076472879872802','0','6','86']
  H_HDV['44_PM_0_0'] = [' 44','EURO1','RT >12-14t Euro-1','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009483615174529','1.17161506556917','4.59513955276058','1.93301425049717','-0.023447138826233','6','86']
  H_HDV['44_FC_0_0'] = [' 44','EURO1','RT >12-14t Euro-1','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001241628988906','0.00021656082788','-0.000001767327758','0','0','6','86']
  H_HDV['44_CO_0_50'] = [' 44','EURO1','RT >12-14t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.61193159728498','0.056898683456157','5.46377659396398','0.273781481461152','0.790317454194181','6','86']
  H_HDV['44_THC_0_50'] = [' 44','EURO1','RT >12-14t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.178064012692493','2.23782149885438','3.89160727992757','1.6498335797659','-0.002558934936941','6','86']
  H_HDV['44_NOx_0_50'] = [' 44','EURO1','RT >12-14t Euro-1','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.1498049828405','0.00591776809543','-0.825360976649647','0.071153100930499','0','6','86']
  H_HDV['44_PM_0_50'] = [' 44','EURO1','RT >12-14t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.750557266305765','0.05955759654486','0.784750538199954','0.190122668531955','0.156206861399955','6','86']
  H_HDV['44_FC_0_50'] = [' 44','EURO1','RT >12-14t Euro-1','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001420118659783','0.000168239652398','-0.000001290479798','','','6','86']
  H_HDV['44_CO_0_100'] = [' 44','EURO1','RT >12-14t Euro-1','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.086719761712154','0.021851647725215','-0.000127225446637','0','0','6','86']
  H_HDV['44_THC_0_100'] = [' 44','EURO1','RT >12-14t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.00093789028741','0.044967046128589','1.73696907527249','0.134543877321655','0.251860886123801','6','86']
  H_HDV['44_NOx_0_100'] = [' 44','EURO1','RT >12-14t Euro-1','0','100','NOx','y=(c+(a*exp(b*x)))','10.4218582222717','-0.061239799740899','5.24095510420952','0','0','6','86']
  H_HDV['44_PM_0_100'] = [' 44','EURO1','RT >12-14t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.742743287814354','0.057107687580697','0.885684887637316','0.214532823088233','0.175625862324848','6','86']
  H_HDV['44_FC_0_100'] = [' 44','EURO1','RT >12-14t Euro-1','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001545651232836','0.000131519853305','-0.000000929640834','0','0','6','86']
  H_HDV['44_CO_-2_0'] = [' 44','EURO1','RT >12-14t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.15645759169516','0.010035424670584','-0.305697892650855','0.059447433679727','0','6','86']
  H_HDV['44_THC_-2_0'] = [' 44','EURO1','RT >12-14t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.232270441979082','1.74269370854856','4.03877816181035','1.40599451059245','0.040514580760906','6','86']
  H_HDV['44_NOx_-2_0'] = [' 44','EURO1','RT >12-14t Euro-1','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2183.82323096162','1.28131985697485','11.7125265817816','0.08042142121743','1.93813628045433','6','86']
  H_HDV['44_PM_-2_0'] = [' 44','EURO1','RT >12-14t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.129191731592493','26.9262142500788','-4.01824341263941','-0.705543757769158','0.130875339052665','6','86']
  H_HDV['44_FC_-2_0'] = [' 44','EURO1','RT >12-14t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','62254.3542775354','1.14857473584309','373.655622566368','0.079890568892351','61.567179300202','6','86']
  H_HDV['44_CO_-2_50'] = [' 44','EURO1','RT >12-14t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.03685037967782','0.016147352150306','-0.274576464039586','0.050535927594744','0','6','86']
  H_HDV['44_THC_-2_50'] = [' 44','EURO1','RT >12-14t Euro-1','-2','50','THC','y=(c+(a*exp(b*x)))','2.1378254461143','-0.074234221800075','0.195959825249733','0','0','6','86']
  H_HDV['44_NOx_-2_50'] = [' 44','EURO1','RT >12-14t Euro-1','-2','50','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.044739661466588','0.010073996638186','-0.000029798972301','0','0','6','86']
  H_HDV['44_PM_-2_50'] = [' 44','EURO1','RT >12-14t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.123800288619133','21.766480179154','-3.78730172271062','-0.700558874796745','0.132779309518965','6','86']
  H_HDV['44_FC_-2_50'] = [' 44','EURO1','RT >12-14t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','334.646350757039','0.062449354591645','56131.4862157047','1.10063238779828','50.2463420130281','6','86']
  H_HDV['44_CO_-2_100'] = [' 44','EURO1','RT >12-14t Euro-1','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.25956440183126','0.01444073501404','-0.286936769780041','0.051170467274156','0','6','86']
  H_HDV['44_THC_-2_100'] = [' 44','EURO1','RT >12-14t Euro-1','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.32416697945511','-0.001023355855228','-0.163248704163189','0.077750029662362','0','6','86']
  H_HDV['44_NOx_-2_100'] = [' 44','EURO1','RT >12-14t Euro-1','-2','100','NOx','y=((a*(b**x))*(x**c))','24.9519829103577','0.990192419650136','-0.503573265161104','0','0','6','86']
  H_HDV['44_PM_-2_100'] = [' 44','EURO1','RT >12-14t Euro-1','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.113455973556877','13.1783414062572','-2.84277797943399','-0.424699674002838','0.111631357201183','6','86']
  H_HDV['44_FC_-2_100'] = [' 44','EURO1','RT >12-14t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','308.607023773318','0.048938814083274','33331.0496208382','0.986512691447699','38.9350970763467','6','86']
  H_HDV['44_CO_-4_0'] = [' 44','EURO1','RT >12-14t Euro-1','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.38653295754653','0.072942088142113','21.7809233547157','0.77004020930435','0.212098166472407','6','86']
  H_HDV['44_THC_-4_0'] = [' 44','EURO1','RT >12-14t Euro-1','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.2333590739939','-0.000946126049991','-0.166148322358438','0.080268554772088','0','6','86']
  H_HDV['44_NOx_-4_0'] = [' 44','EURO1','RT >12-14t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.49241201110848','0.060066708889734','76.1550163188582','0.587082683774467','0.393127561529236','6','86']
  H_HDV['44_PM_-4_0'] = [' 44','EURO1','RT >12-14t Euro-1','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.069384439940553','13.6188882135704','-2.79638045965541','-0.380824520774313','0.108846815477908','6','86']
  H_HDV['44_FC_-4_0'] = [' 44','EURO1','RT >12-14t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','282.778234359072','0.060486416266725','11854.0851568691','0.807849739710175','13.0769123184702','6','86']
  H_HDV['44_CO_-4_50'] = [' 44','EURO1','RT >12-14t Euro-1','-4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.182010644526609','0.017895184402461','0.000834887618259','0','0','6','86']
  H_HDV['44_THC_-4_50'] = [' 44','EURO1','RT >12-14t Euro-1','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.20352346710247','-0.001903910801128','-0.174302399309098','0.08690827639512','0','6','86']
  H_HDV['44_NOx_-4_50'] = [' 44','EURO1','RT >12-14t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.03198295500711','0.050177118292448','103.195161293774','0.6033868926328','0.212474905714625','6','86']
  H_HDV['44_PM_-4_50'] = [' 44','EURO1','RT >12-14t Euro-1','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017479833485708','0.923238739547161','6.78830099129405','2.60567684594145','-0.016238068958574','6','86']
  H_HDV['44_FC_-4_50'] = [' 44','EURO1','RT >12-14t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','241.059658792222','0.0503639334812','10778.364826616','0.767301306417428','6.22487133172307','6','86']
  H_HDV['44_CO_-4_100'] = [' 44','EURO1','RT >12-14t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.31105978282372','0.056006595952748','25.4955287443225','0.589955677505341','0.103163334049513','6','86']
  H_HDV['44_THC_-4_100'] = [' 44','EURO1','RT >12-14t Euro-1','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.14795743220194','-0.002059594983788','-0.17359106369246','0.089202419479095','0','6','86']
  H_HDV['44_NOx_-4_100'] = [' 44','EURO1','RT >12-14t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.45820459477296','0.045217152117897','175.603650083582','0.68567051018295','0.131440739697856','6','86']
  H_HDV['44_PM_-4_100'] = [' 44','EURO1','RT >12-14t Euro-1','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.35009028545792','-8.9464619485935','-1.69980479709825','0','0','6','86']
  H_HDV['44_FC_-4_100'] = [' 44','EURO1','RT >12-14t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','224.063755710799','0.045395289214699','14540.5347600219','0.812157072483289','3.12830809768282','6','86']
  H_HDV['44_CO_-6_0'] = [' 44','EURO1','RT >12-14t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.73390436020758','0.071716011930707','24.0886156869913','0.718365667824689','0.080172608257463','6','72']
  H_HDV['44_THC_-6_0'] = [' 44','EURO1','RT >12-14t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70854419465924','0.069903440765739','1.90213250861195','0.419576639118597','0.057029800489853','6','72']
  H_HDV['44_NOx_-6_0'] = [' 44','EURO1','RT >12-14t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.91842512449026','0.069283080174134','181.139250773544','0.746837566971806','0.147610243706613','6','72']
  H_HDV['44_PM_-6_0'] = [' 44','EURO1','RT >12-14t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.37505736571351','-11.9897085408154','-2.01035695193498','0','0','6','72']
  H_HDV['44_FC_-6_0'] = [' 44','EURO1','RT >12-14t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','245.842623321303','0.064802000270141','13246.7272980607','0.825696910283163','2.94272536688626','6','72']
  H_HDV['44_CO_-6_50'] = [' 44','EURO1','RT >12-14t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.20645957179155','0.067204768349926','32.8849898028484','0.677531982861044','0.068779887820022','6','72']
  H_HDV['44_THC_-6_50'] = [' 44','EURO1','RT >12-14t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.50752819671231','0.067393912402014','4.79896360031101','0.509894796089445','0.052937428980222','6','72']
  H_HDV['44_NOx_-6_50'] = [' 44','EURO1','RT >12-14t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.36769426700095','0.063326404891862','200.922576312517','0.73352494517857','0.116656709310964','6','72']
  H_HDV['44_PM_-6_50'] = [' 44','EURO1','RT >12-14t Euro-1','-6','50','PM','y=(c+(a*exp(((-1)*b)*x)))','1.21690843650969','0.081819927215286','0.039610011138329','0','0','6','72']
  H_HDV['44_FC_-6_50'] = [' 44','EURO1','RT >12-14t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','209.410358349531','0.060482779903207','14927.0126902715','0.833200713168493','2.00511405387858','6','72']
  H_HDV['44_CO_-6_100'] = [' 44','EURO1','RT >12-14t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.90314171281199','0.063301753931652','46.5148378744521','0.707228139819708','0.060726796752106','6','72']
  H_HDV['44_THC_-6_100'] = [' 44','EURO1','RT >12-14t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.39905383289976','0.066232678192017','6.39933037922102','0.542895817613544','0.051557795613802','6','72']
  H_HDV['44_NOx_-6_100'] = [' 44','EURO1','RT >12-14t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.74818659994423','0.060822565410232','278.577993824955','0.780020352535549','0.109371744155568','6','72']
  H_HDV['44_PM_-6_100'] = [' 44','EURO1','RT >12-14t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.948273471793526','0.072825092859484','0.570361100470038','0.279422561256122','0.035751682419737','6','72']
  H_HDV['44_FC_-6_100'] = [' 44','EURO1','RT >12-14t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','188.675459019051','0.057547372200731','14624.4818823798','0.827358394016958','1.60016004106735','6','72']
  H_HDV['44_CO_2_0'] = [' 44','EURO1','RT >12-14t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.92582732223791','0.008372473161642','-0.438516490943246','0.068637386628052','0','6','86']
  H_HDV['44_THC_2_0'] = [' 44','EURO1','RT >12-14t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.11185166582464','2.08473360335561','6.1180650648401','2.37312306785385','-0.025839468722211','6','86']
  H_HDV['44_NOx_2_0'] = [' 44','EURO1','RT >12-14t Euro-1','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-14.5828295447514','0.043362271672441','26.7808027041889','0.066415716140422','7.61871831324107','6','86']
  H_HDV['44_PM_2_0'] = [' 44','EURO1','RT >12-14t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.210879721788375','16.2927498802721','-3.65186439278243','-0.793882976908421','0.147215272544323','6','86']
  H_HDV['44_FC_2_0'] = [' 44','EURO1','RT >12-14t Euro-1','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','662.66361950623','1.11846052464506','-38.4140788368607','0.073629942570585','0','6','86']
  H_HDV['44_CO_2_50'] = [' 44','EURO1','RT >12-14t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.56387471914483','0.0063450492381','-0.515193457192771','0.077824245375921','0','6','86']
  H_HDV['44_THC_2_50'] = [' 44','EURO1','RT >12-14t Euro-1','2','50','THC','y=(c+(a*exp(b*x)))','2.64948420151329','-0.078454571945441','0.36009756867347','0','0','6','86']
  H_HDV['44_NOx_2_50'] = [' 44','EURO1','RT >12-14t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.2411082337562','0.005799015071327','-1.19985291879703','0.091975405220294','0','6','86']
  H_HDV['44_PM_2_50'] = [' 44','EURO1','RT >12-14t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.31262344495474','0.002454081576128','-0.076344180630743','0.064096071712046','0','6','86']
  H_HDV['44_FC_2_50'] = [' 44','EURO1','RT >12-14t Euro-1','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','696.748972166559','1.1596325737743','-35.6209339682142','0.070055095301368','0','6','86']
  H_HDV['44_CO_2_100'] = [' 44','EURO1','RT >12-14t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.40804273768348','-0.002654675607967','-0.635398768513175','0.094031704129044','0','6','85']
  H_HDV['44_THC_2_100'] = [' 44','EURO1','RT >12-14t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.326441139825581','1.78578828525669','8.00274717704501','3.10795648262086','-0.031168349167335','6','85']
  H_HDV['44_NOx_2_100'] = [' 44','EURO1','RT >12-14t Euro-1','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.1506080622386','0.007894831364309','-0.868250840099029','0.074497049281803','0','6','85']
  H_HDV['44_PM_2_100'] = [' 44','EURO1','RT >12-14t Euro-1','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.36016019636753','0.001629831696532','-0.08117059080729','0.071454425515973','0','6','85']
  H_HDV['44_FC_2_100'] = [' 44','EURO1','RT >12-14t Euro-1','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','731.348247204729','0.896242450840861','-33.8055809917925','0.070200736943121','0','6','85']
  H_HDV['44_CO_4_0'] = [' 44','EURO1','RT >12-14t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.41325637703331','15.3135740167498','-2.17168442150559','-1.29204259641982','0.219586189162138','6','71']
  H_HDV['44_THC_4_0'] = [' 44','EURO1','RT >12-14t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.286064116521556','1.97724103661516','6.9106036794426','2.67232714107146','-0.020065729665459','6','71']
  H_HDV['44_NOx_4_0'] = [' 44','EURO1','RT >12-14t Euro-1','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.490923135167','0.012803856641524','-1.46709547132246','0.10112206025068','0','6','71']
  H_HDV['44_PM_4_0'] = [' 44','EURO1','RT >12-14t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.275702850781995','10.1086920563917','-3.69471095158831','-1.13488636829158','0.180209794161596','6','71']
  H_HDV['44_FC_4_0'] = [' 44','EURO1','RT >12-14t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','1510.65709969686','1.01157008136125','-0.585304305838973','0','0','6','71']
  H_HDV['44_CO_4_50'] = [' 44','EURO1','RT >12-14t Euro-1','4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.064662756297007','0.019254021877149','-0.000187867484083','0','0','6','70']
  H_HDV['44_THC_4_50'] = [' 44','EURO1','RT >12-14t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.372595411709423','1.92626195550059','7.08932311419737','2.77820065425078','-0.020889479082149','6','70']
  H_HDV['44_NOx_4_50'] = [' 44','EURO1','RT >12-14t Euro-1','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.4615907301431','0.00649522404529','-1.13473461160751','0.08932493934133','0','6','70']
  H_HDV['44_PM_4_50'] = [' 44','EURO1','RT >12-14t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000007081425803','0.001079336433642','-0.053531351560346','1.2354728349358','0','6','70']
  H_HDV['44_FC_4_50'] = [' 44','EURO1','RT >12-14t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','70560.6841607742','1.28762569761837','487.21146320186','0.107620530747083','356.736323368111','6','70']
  H_HDV['44_CO_4_100'] = [' 44','EURO1','RT >12-14t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.3589026048831','-0.02043959359618','-1.01262249383258','0.13648160833857','0','6','66']
  H_HDV['44_THC_4_100'] = [' 44','EURO1','RT >12-14t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.565871876654107','19.3676848974361','-2.65636300206998','-0.608219569451862','0.144540774989558','6','66']
  H_HDV['44_NOx_4_100'] = [' 44','EURO1','RT >12-14t Euro-1','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.84934402978099','0.064220311733226','39.6718696572654','0.517870239198417','14.2122305420161','6','66']
  H_HDV['44_PM_4_100'] = [' 44','EURO1','RT >12-14t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006346752628','0.000975028324108','-0.048507324365132','1.25540417710469','0','6','66']
  H_HDV['44_FC_4_100'] = [' 44','EURO1','RT >12-14t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','418.583956271996','0.094661843077938','33980.0849255542','1.10874375095588','437.37918876775','6','66']
  H_HDV['44_CO_6_0'] = [' 44','EURO1','RT >12-14t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.77047445698097','29.1260348258449','-2.96720179996092','-1.30706113641451','0.224103043427425','6','71']
  H_HDV['44_THC_6_0'] = [' 44','EURO1','RT >12-14t Euro-1','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-20.0404456522186','0.692719357571919','3.43354590497882','0.092547556265181','0.441687133731929','6','71']
  H_HDV['44_NOx_6_0'] = [' 44','EURO1','RT >12-14t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.7614632416778','0.006998373281385','-1.43021657195566','0.102775467786514','0','6','71']
  H_HDV['44_PM_6_0'] = [' 44','EURO1','RT >12-14t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.26667123287419','0.004825934874631','-0.067694811734443','0.059033136356429','0','6','71']
  H_HDV['44_FC_6_0'] = [' 44','EURO1','RT >12-14t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','1515.95531886099','1.01141747923414','-0.521890712458086','0','0','6','71']
  H_HDV['44_CO_6_50'] = [' 44','EURO1','RT >12-14t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.1425248736527','-0.020884813019541','-1.27757887782512','0.159142088560383','0','6','61']
  H_HDV['44_THC_6_50'] = [' 44','EURO1','RT >12-14t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.582396575824539','16.690243648432','-2.3672005392076','-0.53228083984842','0.138711629110502','6','61']
  H_HDV['44_NOx_6_50'] = [' 44','EURO1','RT >12-14t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2639851227744','0.080152630731069','33.3277828576145','0.520552827559345','15.4819616446427','6','61']
  H_HDV['44_PM_6_50'] = [' 44','EURO1','RT >12-14t Euro-1','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.642645880138297','0.064208119355941','-0.000719822735278','0','0','6','61']
  H_HDV['44_FC_6_50'] = [' 44','EURO1','RT >12-14t Euro-1','6','50','FC','y=((a*(b**x))*(x**c))','1388.26491882755','1.00792198774754','-0.37969021029271','0','0','6','61']
  H_HDV['44_CO_6_100'] = [' 44','EURO1','RT >12-14t Euro-1','6','100','CO','y=(c+(a*exp(b*x)))','6.59116532330696','-0.051513517563005','1.77747020527478','0','0','6','51']
  H_HDV['44_THC_6_100'] = [' 44','EURO1','RT >12-14t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.723993996956213','18.9174606636011','-2.96416072690209','-0.873107217545814','0.177818783226461','6','51']
  H_HDV['44_NOx_6_100'] = [' 44','EURO1','RT >12-14t Euro-1','6','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.25335076901503','1.18528157969143','-0.085967022493527','0','0','6','51']
  H_HDV['44_PM_6_100'] = [' 44','EURO1','RT >12-14t Euro-1','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000010581156942','0.001272795345349','-0.051268920922751','1.3397999682038','0','6','51']
  H_HDV['44_FC_6_100'] = [' 44','EURO1','RT >12-14t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','296.122482134775','0.073076050806388','2481.2323071396','0.546734894334346','571.488768491775','6','51']
  H_HDV['45_CO_0_0'] = [' 45','EURO2','RT >12-14t Euro-2','0','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.121073751175991','0.035311587352946','-0.000228638752496','0','0','6','86']
  H_HDV['45_THC_0_0'] = [' 45','EURO2','RT >12-14t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.052071809903321','1.59464755955423','3.63576661206893','1.59114715038941','-0.010193954030835','6','86']
  H_HDV['45_NOx_0_0'] = [' 45','EURO2','RT >12-14t Euro-2','0','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.3829834702056','0.008121001185401','-1.15116889965624','0.08104208168566','0','6','86']
  H_HDV['45_PM_0_0'] = [' 45','EURO2','RT >12-14t Euro-2','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.17306858614353','0.344394422625347','-0.002772413283725','0','0','6','86']
  H_HDV['45_FC_0_0'] = [' 45','EURO2','RT >12-14t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001488958222682','0.000221811725306','-0.000001832451709','0','0','6','86']
  H_HDV['45_CO_0_50'] = [' 45','EURO2','RT >12-14t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.133525099498037','0.030107337417979','-0.000199057522186','0','0','6','86']
  H_HDV['45_THC_0_50'] = [' 45','EURO2','RT >12-14t Euro-2','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.12496517747328','1.62340617874031','3.17355884949529','1.41626576250196','0.006760561565521','6','86']
  H_HDV['45_NOx_0_50'] = [' 45','EURO2','RT >12-14t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0162763231528','0.071868829319446','1472.97429951381','1.22281170978883','5.03805229909189','6','86']
  H_HDV['45_PM_0_50'] = [' 45','EURO2','RT >12-14t Euro-2','0','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.295842247035719','0.002213469776434','-0.009872294267078','0.029307067077858','0','6','86']
  H_HDV['45_FC_0_50'] = [' 45','EURO2','RT >12-14t Euro-2','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001666691238572','0.000168681125922','-0.000001300797777','','','6','86']
  H_HDV['45_CO_0_100'] = [' 45','EURO2','RT >12-14t Euro-2','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.140168023346085','0.025278002708538','-0.000158769321774','0','0','6','86']
  H_HDV['45_THC_0_100'] = [' 45','EURO2','RT >12-14t Euro-2','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.089966816341858','1.56091728987823','3.6211834621682','1.59274368080025','-0.005515060649703','6','86']
  H_HDV['45_NOx_0_100'] = [' 45','EURO2','RT >12-14t Euro-2','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.9098612656185','0.057931025861774','987.565311672622','1.11405477057297','5.32254939533237','6','86']
  H_HDV['45_PM_0_100'] = [' 45','EURO2','RT >12-14t Euro-2','0','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000056155870019','-0.006713564488438','0.296267642131518','0','0','6','86']
  H_HDV['45_FC_0_100'] = [' 45','EURO2','RT >12-14t Euro-2','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001784084376794','0.000128959757085','-0.000000903175119','0','0','6','86']
  H_HDV['45_CO_-2_0'] = [' 45','EURO2','RT >12-14t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.69323012633045','0.008005900698558','-0.22093362303451','0.05945015276402','0','6','86']
  H_HDV['45_THC_-2_0'] = [' 45','EURO2','RT >12-14t Euro-2','-2','0','THC','y=(c+(a*exp(b*x)))','1.41462544082684','-0.076339190076554','0.15263219536231','0','0','6','86']
  H_HDV['45_NOx_-2_0'] = [' 45','EURO2','RT >12-14t Euro-2','-2','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','14.0549172939888','0.083089151735575','2.10966659917075','0','0','6','86']
  H_HDV['45_PM_-2_0'] = [' 45','EURO2','RT >12-14t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.284226789791305','0.001115792635076','-0.013840242878528','0.044979074246016','0','6','86']
  H_HDV['45_FC_-2_0'] = [' 45','EURO2','RT >12-14t Euro-2','-2','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000595567687589','0.000462498693596','-0.000003299545713','0','0','6','86']
  H_HDV['45_CO_-2_50'] = [' 45','EURO2','RT >12-14t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.63719283648451','0.013623958653848','-0.193231216805076','0.048343275535715','0','6','86']
  H_HDV['45_THC_-2_50'] = [' 45','EURO2','RT >12-14t Euro-2','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.12103866422529','1.85842853003703','1.55705537741136','0.73070943140573','0.047029660785697','6','86']
  H_HDV['45_NOx_-2_50'] = [' 45','EURO2','RT >12-14t Euro-2','-2','50','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.040992720213896','0.009108410131036','-0.000023039453595','0','0','6','86']
  H_HDV['45_PM_-2_50'] = [' 45','EURO2','RT >12-14t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.27981894026058','0.00219253965471','-0.011440142055803','0.03121228063243','0','6','86']
  H_HDV['45_FC_-2_50'] = [' 45','EURO2','RT >12-14t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','294.433576133262','0.059656536711603','39243.2073110948','1.0589174913537','46.8136730308809','6','86']
  H_HDV['45_CO_-2_100'] = [' 45','EURO2','RT >12-14t Euro-2','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.75272470178029','0.014508338590626','-0.188721898360917','0.0450499831722','0','6','86']
  H_HDV['45_THC_-2_100'] = [' 45','EURO2','RT >12-14t Euro-2','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.091096781825985','1.5926595098033','2.33379426457467','1.01974296774303','0.029734382788684','6','86']
  H_HDV['45_NOx_-2_100'] = [' 45','EURO2','RT >12-14t Euro-2','-2','100','NOx','y=((a*(b**x))*(x**c))','27.2632574308812','0.989688640732482','-0.503156434172619','0','0','6','86']
  H_HDV['45_PM_-2_100'] = [' 45','EURO2','RT >12-14t Euro-2','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.302529583677062','0.002474395241669','-0.011777629398546','0.028910499959724','0','6','86']
  H_HDV['45_FC_-2_100'] = [' 45','EURO2','RT >12-14t Euro-2','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','276.288300688632','0.046513504058209','32001.7687031781','1.00253688930784','36.0633366363451','6','86']
  H_HDV['45_CO_-4_0'] = [' 45','EURO2','RT >12-14t Euro-2','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.98480035995004','0.071656028322745','30.2557985709536','0.80143134931976','0.156173210312771','6','86']
  H_HDV['45_THC_-4_0'] = [' 45','EURO2','RT >12-14t Euro-2','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.069255625088924','2.61699910976681','0.666979361425022','0.590627679359974','0.044973845150653','6','86']
  H_HDV['45_NOx_-4_0'] = [' 45','EURO2','RT >12-14t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.02414031463914','0.058931456091875','56.2548918387835','0.513566098975771','0.430952855181291','6','86']
  H_HDV['45_PM_-4_0'] = [' 45','EURO2','RT >12-14t Euro-2','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.302818712651185','-0.000216868076945','-0.020599085473163','0.076946954454659','0','6','86']
  H_HDV['45_FC_-4_0'] = [' 45','EURO2','RT >12-14t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','251.179877425437','0.059339039089115','8982.94009646595','0.781215011923131','12.501742905108','6','86']
  H_HDV['45_CO_-4_50'] = [' 45','EURO2','RT >12-14t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.41675615745454','0.058811130410975','16.6397703564786','0.558627603106559','0.104407338282879','6','86']
  H_HDV['45_THC_-4_50'] = [' 45','EURO2','RT >12-14t Euro-2','-4','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.516490114856474','0.082398451528444','0.001633500830305','0','0','6','86']
  H_HDV['45_NOx_-4_50'] = [' 45','EURO2','RT >12-14t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.59995542609047','0.049925585088538','89.0388104885398','0.563702477535936','0.235961328491924','6','86']
  H_HDV['45_PM_-4_50'] = [' 45','EURO2','RT >12-14t Euro-2','-4','50','PM','y=((a+(b*x))**((-1)/c))','1.95844480148708','0.201085893588695','0.744827044433434','0','0','6','86']
  H_HDV['45_FC_-4_50'] = [' 45','EURO2','RT >12-14t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','215.693256807704','0.049039935225279','8724.45616152012','0.753383176302157','6.00741187355094','6','86']
  H_HDV['45_CO_-4_100'] = [' 45','EURO2','RT >12-14t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.2881087190045','0.052135723321448','33.771844661472','0.646605438611287','0.078612550188764','6','86']
  H_HDV['45_THC_-4_100'] = [' 45','EURO2','RT >12-14t Euro-2','-4','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.527959064530007','0.084637740020423','0.001716124351177','0','0','6','86']
  H_HDV['45_NOx_-4_100'] = [' 45','EURO2','RT >12-14t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.9812895577461','0.045369980251994','149.308362556961','0.644411874487647','0.152610496358835','6','86']
  H_HDV['45_PM_-4_100'] = [' 45','EURO2','RT >12-14t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.208088407337355','0.045900936679706','0.992205689963086','0.486063733892141','0.016583859666395','6','86']
  H_HDV['45_FC_-4_100'] = [' 45','EURO2','RT >12-14t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','201.883716004849','0.044078687747527','12082.9376085554','0.802804727248013','3.03611756333896','6','86']
  H_HDV['45_CO_-6_0'] = [' 45','EURO2','RT >12-14t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.61180907331651','0.071479858406737','24.9615061958414','0.750185809760678','0.068315802166092','6','72']
  H_HDV['45_THC_-6_0'] = [' 45','EURO2','RT >12-14t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.04532780362708','0.064544955879211','1.6609176666171','0.40751254174862','0.048467217002058','6','72']
  H_HDV['45_NOx_-6_0'] = [' 45','EURO2','RT >12-14t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.50756976450666','0.068967503034106','119.924173189166','0.662358654919365','0.171743839440552','6','72']
  H_HDV['45_PM_-6_0'] = [' 45','EURO2','RT >12-14t Euro-2','-6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.28949866869344','-0.000186935988574','-0.020450737990859','0.079143508134995','0','6','72']
  H_HDV['45_FC_-6_0'] = [' 45','EURO2','RT >12-14t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','221.010693902184','0.064487441162411','10668.893522281','0.81297597115913','3.31058665946811','6','72']
  H_HDV['45_CO_-6_50'] = [' 45','EURO2','RT >12-14t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.15756343588636','0.064088859144248','29.8751084462931','0.673011187698857','0.056776142682711','6','72']
  H_HDV['45_THC_-6_50'] = [' 45','EURO2','RT >12-14t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.967098567857196','0.063162526555948','2.2696981112719','0.437193041669851','0.046855031102871','6','72']
  H_HDV['45_NOx_-6_50'] = [' 45','EURO2','RT >12-14t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.05306611145255','0.064427762436099','220.300713373111','0.738712957322054','0.144735041524595','6','72']
  H_HDV['45_PM_-6_50'] = [' 45','EURO2','RT >12-14t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.191989198391183','0.055731533315823','0.261060504415797','0.299677108124309','0.018093399824357','6','72']
  H_HDV['45_FC_-6_50'] = [' 45','EURO2','RT >12-14t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','188.858440536542','0.060167085325732','12449.7608646663','0.826865368716023','2.46227905960204','6','72']
  H_HDV['45_CO_-6_100'] = [' 45','EURO2','RT >12-14t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.01081694079468','0.06001332647143','42.8759243897043','0.723342745215384','0.050593674133413','6','72']
  H_HDV['45_THC_-6_100'] = [' 45','EURO2','RT >12-14t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.920350899786775','0.062478653118576','2.46273422663976','0.44240168910167','0.046297277117127','6','72']
  H_HDV['45_NOx_-6_100'] = [' 45','EURO2','RT >12-14t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.23418578321239','0.061194075324055','271.854042625956','0.764874142730462','0.131878909027595','6','72']
  H_HDV['45_PM_-6_100'] = [' 45','EURO2','RT >12-14t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.195758946934451','0.053396523590587','0.373898053497518','0.35997235818142','0.017497874104821','6','72']
  H_HDV['45_FC_-6_100'] = [' 45','EURO2','RT >12-14t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','172.453944975901','0.057496899927913','14766.0304494498','0.853067303274676','2.11239759053205','6','72']
  H_HDV['45_CO_2_0'] = [' 45','EURO2','RT >12-14t Euro-2','2','0','CO','y=(c+(a*exp(b*x)))','4.35120867897355','-0.092352500047984','0.994728052711869','0','0','6','86']
  H_HDV['45_THC_2_0'] = [' 45','EURO2','RT >12-14t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.110695354080911','1.38903550562738','5.27641854475727','2.0638127031779','-0.011220102598942','6','86']
  H_HDV['45_NOx_2_0'] = [' 45','EURO2','RT >12-14t Euro-2','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.3880351144061','0.011669205158206','-1.42600139416717','0.090086245568996','0','6','86']
  H_HDV['45_PM_2_0'] = [' 45','EURO2','RT >12-14t Euro-2','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.358701751072552','0.000471924812563','-0.017845773405218','0.065777580266367','0','6','86']
  H_HDV['45_FC_2_0'] = [' 45','EURO2','RT >12-14t Euro-2','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.066923957325051','1.65765286612194','1123.17024703721','-0.519736331893015','0','6','86']
  H_HDV['45_CO_2_50'] = [' 45','EURO2','RT >12-14t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.136730396555671','0.020902737188987','-0.000138679230995','0','0','6','86']
  H_HDV['45_THC_2_50'] = [' 45','EURO2','RT >12-14t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.143945706109699','1.27669220034458','6.72896360094679','2.62237445946232','-0.023737674929404','6','86']
  H_HDV['45_NOx_2_50'] = [' 45','EURO2','RT >12-14t Euro-2','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13091.2631101018','1.65809812597745','14.2101775286362','0.086402736051166','8.52391466325484','6','86']
  H_HDV['45_PM_2_50'] = [' 45','EURO2','RT >12-14t Euro-2','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.28278486988614','0.167819429087741','-0.001334732898147','0','0','6','86']
  H_HDV['45_FC_2_50'] = [' 45','EURO2','RT >12-14t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','624.014732945054','1.17830872135382','-29.5198253657189','0.066780950372229','0','6','86']
  H_HDV['45_CO_2_100'] = [' 45','EURO2','RT >12-14t Euro-2','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.73457679179275','-0.971064266639712','-0.622033741918545','0','0','6','86']
  H_HDV['45_THC_2_100'] = [' 45','EURO2','RT >12-14t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.193979282525548','1.19414682247251','7.71041148259323','3.01021108414253','-0.029168259527707','6','86']
  H_HDV['45_NOx_2_100'] = [' 45','EURO2','RT >12-14t Euro-2','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.0220742052217','-0.006988639704228','-1.13951651128681','0.082137635994023','0','6','86']
  H_HDV['45_PM_2_100'] = [' 45','EURO2','RT >12-14t Euro-2','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.401790128224075','0.001671096219623','-0.011174505210054','0.030232983318387','0','6','86']
  H_HDV['45_FC_2_100'] = [' 45','EURO2','RT >12-14t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','652.99464029649','1.14855573582372','-26.5339309672076','0.063081350692159','0','6','86']
  H_HDV['45_CO_4_0'] = [' 45','EURO2','RT >12-14t Euro-2','4','0','CO','y=(c+(a*exp(b*x)))','4.40970378495336','-0.084142397371385','1.22557795485682','0','0','6','71']
  H_HDV['45_THC_4_0'] = [' 45','EURO2','RT >12-14t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.16143927681248','1.3595425867926','6.32320716330523','2.47256747268917','-0.01689435269375','6','71']
  H_HDV['45_NOx_4_0'] = [' 45','EURO2','RT >12-14t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.8807414615848','0.099109129245525','1077.92263633089','1.31657996433966','9.61681349229598','6','71']
  H_HDV['45_PM_4_0'] = [' 45','EURO2','RT >12-14t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.41227045744203','0.146919861194118','-0.001185127242632','0','0','6','71']
  H_HDV['45_FC_4_0'] = [' 45','EURO2','RT >12-14t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1288.30390238025','1.01117048341097','-0.54547428137378','0','0','6','71']
  H_HDV['45_CO_4_50'] = [' 45','EURO2','RT >12-14t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.68795727258137','0.024832497107091','4.35815134996032','0.172298834252809','0.623349977197223','6','71']
  H_HDV['45_THC_4_50'] = [' 45','EURO2','RT >12-14t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.226584123810769','1.27078985451607','7.16485300389994','2.81931378723913','-0.021224304979627','6','71']
  H_HDV['45_NOx_4_50'] = [' 45','EURO2','RT >12-14t Euro-2','4','50','NOx','y=((a*(b**x))*(x**c))','41.3636039052061','1.00594118262567','-0.390679644401363','0','0','6','71']
  H_HDV['45_PM_4_50'] = [' 45','EURO2','RT >12-14t Euro-2','4','50','PM','y=((a+(b*x))**((-1)/c))','3.85281852344185','0.852638472467208','2.18983052473504','0','0','6','71']
  H_HDV['45_FC_4_50'] = [' 45','EURO2','RT >12-14t Euro-2','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002832063325442','0.432343413895188','-21.2250615014104','686.418836245285','0','6','71']
  H_HDV['45_CO_4_100'] = [' 45','EURO2','RT >12-14t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.82247766893976','0.015169465683511','4.53808555067177','0.136856062808671','-0.349062393159925','6','67']
  H_HDV['45_THC_4_100'] = [' 45','EURO2','RT >12-14t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.284605723258082','1.2035252281808','7.62202503728128','3.02342183288329','-0.024945682610665','6','67']
  H_HDV['45_NOx_4_100'] = [' 45','EURO2','RT >12-14t Euro-2','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8068200094902','0.06864918589279','22.8720018391701','0.462802709496','13.8685003482057','6','67']
  H_HDV['45_PM_4_100'] = [' 45','EURO2','RT >12-14t Euro-2','4','100','PM','y=((a*(b**x))*(x**c))','0.712293209802971','0.994503998855918','-0.258772444459355','0','0','6','67']
  H_HDV['45_FC_4_100'] = [' 45','EURO2','RT >12-14t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','336.614196187471','0.092510888009717','22669.6945104159','1.03762137736919','431.578677262663','6','67']
  H_HDV['45_CO_6_0'] = [' 45','EURO2','RT >12-14t Euro-2','6','0','CO','y=(c+(a*exp(b*x)))','4.04592849707401','-0.046496676603169','1.0686401876518','0','0','6','71']
  H_HDV['45_THC_6_0'] = [' 45','EURO2','RT >12-14t Euro-2','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.274997238330733','7.95310990781224','-1.66062917639089','-0.320113741376383','0.118055243725024','6','71']
  H_HDV['45_NOx_6_0'] = [' 45','EURO2','RT >12-14t Euro-2','6','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','11.858644668084','23.614526210547','1.42909594028211','0.849598490012967','0.050247804566373','6','71']
  H_HDV['45_PM_6_0'] = [' 45','EURO2','RT >12-14t Euro-2','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001973667817','0.00026897206204','-0.013119254316534','0.415035370323767','0','6','71']
  H_HDV['45_FC_6_0'] = [' 45','EURO2','RT >12-14t Euro-2','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','723.734220668823','2.08822362125875','-30.425566229494','0.063862827863043','0','6','71']
  H_HDV['45_CO_6_50'] = [' 45','EURO2','RT >12-14t Euro-2','6','50','CO','y=(c+(a*exp(b*x)))','5.57906663426394','-0.052590416819735','1.00175656580916','0','0','6','62']
  H_HDV['45_THC_6_50'] = [' 45','EURO2','RT >12-14t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.353926513026603','8.36242097925959','-1.90969946828832','-0.453344533902167','0.137344997434638','6','62']
  H_HDV['45_NOx_6_50'] = [' 45','EURO2','RT >12-14t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.5400420646376','-0.032224303622433','-1.94218223697283','0.114692644492696','0','6','62']
  H_HDV['45_PM_6_50'] = [' 45','EURO2','RT >12-14t Euro-2','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000001906664322','0.000287169866521','-0.015928805768665','0.520546445369089','0','6','62']
  H_HDV['45_FC_6_50'] = [' 45','EURO2','RT >12-14t Euro-2','6','50','FC','y=((a*(b**x))*(x**c))','1220.09309523664','1.00764483065498','-0.346208164880321','0','0','6','62']
  H_HDV['45_CO_6_100'] = [' 45','EURO2','RT >12-14t Euro-2','6','100','CO','y=((a*(b**x))*(x**c))','13.5443674705913','0.98560780537572','-0.390388250613778','0','0','6','52']
  H_HDV['45_THC_6_100'] = [' 45','EURO2','RT >12-14t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.444808282011142','8.15847670392307','-2.3725757418227','-0.805873826332263','0.177452754948822','6','52']
  H_HDV['45_NOx_6_100'] = [' 45','EURO2','RT >12-14t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000203097085468','0.024148215847143','-1.02017906412006','33.9348200362569','0','6','52']
  H_HDV['45_PM_6_100'] = [' 45','EURO2','RT >12-14t Euro-2','6','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.41120917977464','0.083015558999784','-0.000547095051773','0','0','6','52']
  H_HDV['45_FC_6_100'] = [' 45','EURO2','RT >12-14t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','285.407761256158','0.090944570360031','6512.53507860093','0.778225502133119','577.805400000738','6','52']
  H_HDV['46_CO_0_0'] = [' 46','EURO3','RT >12-14t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.57796048138866','0.057645395475272','4.32114432505578','0.194652462138059','0.650831641412205','6','86']
  H_HDV['46_THC_0_0'] = [' 46','EURO3','RT >12-14t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.079248912803968','1.35340109492603','4.4807172361999','1.86966065542267','-0.010685782033328','6','86']
  H_HDV['46_NOx_0_0'] = [' 46','EURO3','RT >12-14t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.80994091601265','0.070810190757993','33.3336601574568','0.338790793382917','3.49562515157964','6','86']
  H_HDV['46_PM_0_0'] = [' 46','EURO3','RT >12-14t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040475439221827','0.417015825366655','5.93416292190512','2.37780287704316','-0.022956839894926','6','86']
  H_HDV['46_FC_0_0'] = [' 46','EURO3','RT >12-14t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001269321864097','0.000213304371378','-0.000001732905564','0','0','6','86']
  H_HDV['46_CO_0_50'] = [' 46','EURO3','RT >12-14t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.6645785309034','0.056368339317076','5.23028829144801','0.22940672493427','0.731687393919072','6','86']
  H_HDV['46_THC_0_50'] = [' 46','EURO3','RT >12-14t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.083736033445732','1.32104434472513','4.53135180004797','1.89348725872261','-0.010385314558494','6','86']
  H_HDV['46_NOx_0_50'] = [' 46','EURO3','RT >12-14t Euro-3','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.83991867276675','0.058209543779107','32.8119093290992','0.324655578422129','3.75961273247849','6','86']
  H_HDV['46_PM_0_50'] = [' 46','EURO3','RT >12-14t Euro-3','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007530003394181','0.481778214802105','4.57741464608742','1.88064486426566','-0.022416579494905','6','86']
  H_HDV['46_FC_0_50'] = [' 46','EURO3','RT >12-14t Euro-3','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001474861891353','0.000164240816415','-0.000001251106636','','','6','86']
  H_HDV['46_CO_0_100'] = [' 46','EURO3','RT >12-14t Euro-3','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.806336654383942','26.5384799014787','-0.605344559207775','0.487937537843305','0.043358787814982','6','86']
  H_HDV['46_THC_0_100'] = [' 46','EURO3','RT >12-14t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.088387866851479','1.30513113725768','4.43873469359695','1.86336071195264','-0.008936188067247','6','86']
  H_HDV['46_NOx_0_100'] = [' 46','EURO3','RT >12-14t Euro-3','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.78033824922454','0.049854441293363','35.7286733425402','0.340995987925842','3.94789871157259','6','86']
  H_HDV['46_PM_0_100'] = [' 46','EURO3','RT >12-14t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.30926446669771','0.055278656207755','0.270822396140801','0.177452702798492','0.070912708408934','6','86']
  H_HDV['46_FC_0_100'] = [' 46','EURO3','RT >12-14t Euro-3','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001613703346669','0.000126783388277','-0.000000879452647','0','0','6','86']
  H_HDV['46_CO_-2_0'] = [' 46','EURO3','RT >12-14t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.87066159093745','0.011009877773236','-0.341078112311811','0.057817724922137','0','6','86']
  H_HDV['46_THC_-2_0'] = [' 46','EURO3','RT >12-14t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.149066801687733','7.93833005931918','-1.93455821649947','-0.228659155060206','0.097846786183375','6','86']
  H_HDV['46_NOx_-2_0'] = [' 46','EURO3','RT >12-14t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2232698550269','0.083216335063884','160.112729912346','0.70586655464763','1.70590913981374','6','86']
  H_HDV['46_PM_-2_0'] = [' 46','EURO3','RT >12-14t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.06029582419284','17.7834180839479','-4.39431998078086','-0.614321884461462','0.114567096013876','6','86']
  H_HDV['46_FC_-2_0'] = [' 46','EURO3','RT >12-14t Euro-3','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','490.059224788031','0.265478116157577','-32.5019683746981','0.073223450498183','0','6','86']
  H_HDV['46_CO_-2_50'] = [' 46','EURO3','RT >12-14t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.70628929529401','0.017938563787618','-0.31107277229509','0.050727173566129','0','6','86']
  H_HDV['46_THC_-2_50'] = [' 46','EURO3','RT >12-14t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.125673280307961','6.4614659915174','-1.63316041665114','-0.158052445738044','0.09252110061913','6','86']
  H_HDV['46_NOx_-2_50'] = [' 46','EURO3','RT >12-14t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5693220935683','0.064551843030582','112.645566723495','0.591990053891407','1.34651383732238','6','86']
  H_HDV['46_PM_-2_50'] = [' 46','EURO3','RT >12-14t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.060076802260797','18.9783538297009','-4.74799094443968','-0.787534719939258','0.129026668367744','6','86']
  H_HDV['46_FC_-2_50'] = [' 46','EURO3','RT >12-14t Euro-3','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','350.251551671518','0.061103370637614','29998.6264383508','1.05108388239562','50.3081979018431','6','86']
  H_HDV['46_CO_-2_100'] = [' 46','EURO3','RT >12-14t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.77538736341475','0.018331407799859','-0.30793465877692','0.049112376014175','0','6','86']
  H_HDV['46_THC_-2_100'] = [' 46','EURO3','RT >12-14t Euro-3','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053266723755653','0.942024133523332','7.15481949287767','2.7122457398723','-0.022362864391606','6','86']
  H_HDV['46_NOx_-2_100'] = [' 46','EURO3','RT >12-14t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.90192094751218','0.049342055963412','98.9843408303704','0.535560050101035','0.991800882700627','6','86']
  H_HDV['46_PM_-2_100'] = [' 46','EURO3','RT >12-14t Euro-3','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.056149360232286','13.3132437949281','-4.09604382884874','-0.577537824984682','0.112322575926022','6','86']
  H_HDV['46_FC_-2_100'] = [' 46','EURO3','RT >12-14t Euro-3','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','318.960400725875','0.048223758881299','17464.7035859608','0.91606473593627','39.4405451598257','6','86']
  H_HDV['46_CO_-4_0'] = [' 46','EURO3','RT >12-14t Euro-3','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.329259419242312','55.2208929335415','-2.86479213596964','-0.439956864102884','0.108230311771208','6','86']
  H_HDV['46_THC_-4_0'] = [' 46','EURO3','RT >12-14t Euro-3','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.032343348231458','0.939990807822771','6.9336305804324','2.61801022801279','-0.016719129497592','6','86']
  H_HDV['46_NOx_-4_0'] = [' 46','EURO3','RT >12-14t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.71774765372474','0.064549591534155','69.2287432859564','0.483048782502229','0.379301211422868','6','86']
  H_HDV['46_PM_-4_0'] = [' 46','EURO3','RT >12-14t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.06232304548803','-8.4927376722485','-1.46581613448458','0','0','6','86']
  H_HDV['46_FC_-4_0'] = [' 46','EURO3','RT >12-14t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','298.860992978919','0.059781624180355','2987.64816089451','0.610283207644136','15.6995018863913','6','86']
  H_HDV['46_CO_-4_50'] = [' 46','EURO3','RT >12-14t Euro-3','-4','50','CO','y=exp((a+(b/x))+(c*log(x)))','6.14802483236591','-10.3601531640113','-1.7553105697004','0','0','6','86']
  H_HDV['46_THC_-4_50'] = [' 46','EURO3','RT >12-14t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.32992859214768','-0.000858334666911','-0.10204368814734','0.08356226347124','0','6','86']
  H_HDV['46_NOx_-4_50'] = [' 46','EURO3','RT >12-14t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.4147435816743','0.059020722738743','117.690477345537','0.56549254500669','0.317970555281848','6','86']
  H_HDV['46_PM_-4_50'] = [' 46','EURO3','RT >12-14t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.90977310863121','-7.99253278959348','-1.4467858763484','0','0','6','86']
  H_HDV['46_FC_-4_50'] = [' 46','EURO3','RT >12-14t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','251.78244179367','0.05021713177089','3778.23851951409','0.6144549952992','9.35881923578642','6','86']
  H_HDV['46_CO_-4_100'] = [' 46','EURO3','RT >12-14t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.5303059815719','-0.004629568148037','-0.404864024772798','0.080019911077063','0','6','86']
  H_HDV['46_THC_-4_100'] = [' 46','EURO3','RT >12-14t Euro-3','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.25930501210165','-0.000876923866668','-0.09812816406422','0.085123948275433','0','6','86']
  H_HDV['46_NOx_-4_100'] = [' 46','EURO3','RT >12-14t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.60841339492148','0.054335230692984','196.331221142849','0.647559431598667','0.270236256528122','6','86']
  H_HDV['46_PM_-4_100'] = [' 46','EURO3','RT >12-14t Euro-3','-4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017253117334158','0.368318406732976','6.54537669524723','2.4402679792172','-0.014735665130945','6','86']
  H_HDV['46_FC_-4_100'] = [' 46','EURO3','RT >12-14t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','233.16321336012','0.045433347541922','6226.57735468741','0.690337693670734','6.54329787689995','6','86']
  H_HDV['46_CO_-6_0'] = [' 46','EURO3','RT >12-14t Euro-3','-6','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.26470029832357','-13.8257727646869','-2.11893640785093','0','0','6','72']
  H_HDV['46_THC_-6_0'] = [' 46','EURO3','RT >12-14t Euro-3','-6','0','THC','y=(c+(a*exp(b*x)))','1.20095308921118','-0.077527525962973','0.049053583368421','0','0','6','72']
  H_HDV['46_NOx_-6_0'] = [' 46','EURO3','RT >12-14t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.89788385382242','0.077058925018313','138.749319576528','0.615389746459113','0.234462564671182','6','72']
  H_HDV['46_PM_-6_0'] = [' 46','EURO3','RT >12-14t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.38143262493388','-9.49144653227649','-1.58712061345071','0','0','6','72']
  H_HDV['46_FC_-6_0'] = [' 46','EURO3','RT >12-14t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','279.513356788771','0.067556447377356','6584.62113660678','0.756649813870347','8.06408861495953','6','72']
  H_HDV['46_CO_-6_50'] = [' 46','EURO3','RT >12-14t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','4.54279890206212','-0.072358503349407','0.125203381790033','0','0','6','72']
  H_HDV['46_THC_-6_50'] = [' 46','EURO3','RT >12-14t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.934343651479454','0.071604515202432','1.46207358749552','0.462798581607614','0.044529341802207','6','72']
  H_HDV['46_NOx_-6_50'] = [' 46','EURO3','RT >12-14t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.8695859088095','0.069566143333133','217.507019067039','0.672040909726663','0.20049117789066','6','72']
  H_HDV['46_PM_-6_50'] = [' 46','EURO3','RT >12-14t Euro-3','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.74755544222014','-7.75461583385152','-1.43547527704688','0','0','6','72']
  H_HDV['46_FC_-6_50'] = [' 46','EURO3','RT >12-14t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','230.668434658905','0.062085621393373','9054.15136733577','0.782672293154376','6.96843821257809','6','72']
  H_HDV['46_CO_-6_100'] = [' 46','EURO3','RT >12-14t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','4.12564596347578','-0.069501799164756','0.119376065268023','0','0','6','72']
  H_HDV['46_THC_-6_100'] = [' 46','EURO3','RT >12-14t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.809007124186016','0.068805527932279','3.90499102009961','0.57785139073902','0.042803900531955','6','72']
  H_HDV['46_NOx_-6_100'] = [' 46','EURO3','RT >12-14t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.81644085556675','0.064278987509541','283.315091391631','0.711452735097586','0.184433262448703','6','72']
  H_HDV['46_PM_-6_100'] = [' 46','EURO3','RT >12-14t Euro-3','-6','100','PM','y=(c+(a*exp(b*x)))','0.425240786072007','-0.068175857044315','0.032699815874057','0','0','6','72']
  H_HDV['46_FC_-6_100'] = [' 46','EURO3','RT >12-14t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','203.543059484897','0.058060593616808','9244.57882922032','0.778411387196246','6.29219683361022','6','72']
  H_HDV['46_CO_2_0'] = [' 46','EURO3','RT >12-14t Euro-3','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.600405724892165','4.26066790435464','7.48732223798066','2.93082497189637','-0.034245599243665','6','86']
  H_HDV['46_THC_2_0'] = [' 46','EURO3','RT >12-14t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.10579622660088','1.23582622531039','5.94050249840563','2.29275613046509','-0.013615882974308','6','86']
  H_HDV['46_NOx_2_0'] = [' 46','EURO3','RT >12-14t Euro-3','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','5.41143061818062','28.8147392144224','2.0381119821111','1.25800343300708','0.036955239149849','6','86']
  H_HDV['46_PM_2_0'] = [' 46','EURO3','RT >12-14t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.06983052419786','0.338687250579119','7.40215303049216','2.77675564442141','-0.019366886788262','6','86']
  H_HDV['46_FC_2_0'] = [' 46','EURO3','RT >12-14t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','652.148265158798','0.957110606686804','-38.0938339582182','0.076143551200194','0','6','86']
  H_HDV['46_CO_2_50'] = [' 46','EURO3','RT >12-14t Euro-3','2','50','CO','y=(c+(a*exp(b*x)))','6.06134578626129','-0.076129811961415','1.0012256697257','0','0','6','86']
  H_HDV['46_THC_2_50'] = [' 46','EURO3','RT >12-14t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.132443678319449','1.16538825167176','6.67252331088686','2.57288252759583','-0.017554579349569','6','86']
  H_HDV['46_NOx_2_50'] = [' 46','EURO3','RT >12-14t Euro-3','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.04697456472803','30.2177667223494','2.23397645479501','1.4552155864023','0.004811009521892','6','86']
  H_HDV['46_PM_2_50'] = [' 46','EURO3','RT >12-14t Euro-3','2','50','PM','y=(c+(a*exp(b*x)))','0.452456546507211','-0.071001432675448','0.092098984874661','0','0','6','86']
  H_HDV['46_FC_2_50'] = [' 46','EURO3','RT >12-14t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','675.196385780341','0.929926954731107','-33.827480149345','0.071438694098618','0','6','86']
  H_HDV['46_CO_2_100'] = [' 46','EURO3','RT >12-14t Euro-3','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.74954565184456','0.049597399804599','3.3523381332269','0.14233721054248','0.940010024583118','6','86']
  H_HDV['46_THC_2_100'] = [' 46','EURO3','RT >12-14t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.164726832766744','1.08945812645267','7.58009581380756','2.92350384088948','-0.02291765370987','6','86']
  H_HDV['46_NOx_2_100'] = [' 46','EURO3','RT >12-14t Euro-3','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.88144803026252','68.1422809740455','0.432444220488872','1.0193906942291','0.015186123062405','6','86']
  H_HDV['46_PM_2_100'] = [' 46','EURO3','RT >12-14t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.525769096281291','0.000454410457823','-0.027160016008977','0.060818434398244','0','6','86']
  H_HDV['46_FC_2_100'] = [' 46','EURO3','RT >12-14t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','692.734372302068','0.969183468269815','-28.8466726072227','0.064811768988363','0','6','86']
  H_HDV['46_CO_4_0'] = [' 46','EURO3','RT >12-14t Euro-3','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.760873453487977','4.49622721610461','5.70975897398126','2.22287646828768','-0.012969465007327','6','71']
  H_HDV['46_THC_4_0'] = [' 46','EURO3','RT >12-14t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.138516581676972','1.19601075379197','6.85989779755432','2.64334465505059','-0.019473998325578','6','71']
  H_HDV['46_NOx_4_0'] = [' 46','EURO3','RT >12-14t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.20178017105767','0.04243243605096','24.9542349468226','0.172747182151622','7.00651505647919','6','71']
  H_HDV['46_PM_4_0'] = [' 46','EURO3','RT >12-14t Euro-3','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-5.86106427578916','0.769517015694477','0.505111987392148','0.075916230548012','0.099820139657318','6','71']
  H_HDV['46_FC_4_0'] = [' 46','EURO3','RT >12-14t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','1384.98726687177','1.01090110594196','-0.550493407163033','0','0','6','71']
  H_HDV['46_CO_4_50'] = [' 46','EURO3','RT >12-14t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.62803719806558','-0.016187089121188','-0.564973725036694','0.09864782314223','0','6','71']
  H_HDV['46_THC_4_50'] = [' 46','EURO3','RT >12-14t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.183639481728823','1.11937508936835','7.65036605169435','2.96615984077648','-0.024279148506601','6','71']
  H_HDV['46_NOx_4_50'] = [' 46','EURO3','RT >12-14t Euro-3','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','8.30360388465655','27.9300735648262','2.60301081916678','1.52717243947713','0.000124247978007','6','71']
  H_HDV['46_PM_4_50'] = [' 46','EURO3','RT >12-14t Euro-3','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-2.45685115778503','0.6366766571829','0.497140436528548','0.077963457726351','0.118922167009813','6','71']
  H_HDV['46_FC_4_50'] = [' 46','EURO3','RT >12-14t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','1236.73868948938','1.00775460159526','-0.414331466507691','0','0','6','71']
  H_HDV['46_CO_4_100'] = [' 46','EURO3','RT >12-14t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.241281989636','-0.02116781728575','-0.583474243530757','0.095896524689561','0','6','68']
  H_HDV['46_THC_4_100'] = [' 46','EURO3','RT >12-14t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.218444719330614','1.08190044806554','7.69494850806342','3.0103536564421','-0.025369346095406','6','68']
  H_HDV['46_NOx_4_100'] = [' 46','EURO3','RT >12-14t Euro-3','4','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','10.504925505196','33.3880896386162','1.7785275783543','1.1404908406875','0.03122515495426','6','68']
  H_HDV['46_PM_4_100'] = [' 46','EURO3','RT >12-14t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002320596063','0.000369750134529','-0.01990165518714','0.515138427312856','0','6','68']
  H_HDV['46_FC_4_100'] = [' 46','EURO3','RT >12-14t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','369.337220965911','0.085767017289613','72724.6191151996','1.25584926912717','438.49847293317','6','68']
  H_HDV['46_CO_6_0'] = [' 46','EURO3','RT >12-14t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.00924644579757','-0.019370908622245','-0.749297491806584','0.127547643791204','0','6','71']
  H_HDV['46_THC_6_0'] = [' 46','EURO3','RT >12-14t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.230078419944189','11.3791698470585','-2.20637211033947','-0.30069450114839','0.109422602159611','6','71']
  H_HDV['46_NOx_6_0'] = [' 46','EURO3','RT >12-14t Euro-3','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.6543005821168','-0.021322898456555','-3.53531464622334','0.13923854354747','0','6','71']
  H_HDV['46_PM_6_0'] = [' 46','EURO3','RT >12-14t Euro-3','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.124498851702182','5.1372045572366','-3.17197354976193','-0.494056780424236','0.110475914754797','6','71']
  H_HDV['46_FC_6_0'] = [' 46','EURO3','RT >12-14t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','1352.26455038185','1.01016534768564','-0.471227598407238','0','0','6','71']
  H_HDV['46_CO_6_50'] = [' 46','EURO3','RT >12-14t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.23695720988763','-4.6867712444046','-0.994377632793552','0','0','6','64']
  H_HDV['46_THC_6_50'] = [' 46','EURO3','RT >12-14t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.220646968599499','1.11466536119906','7.36684339864845','2.88835868761238','-0.024300138557896','6','64']
  H_HDV['46_NOx_6_50'] = [' 46','EURO3','RT >12-14t Euro-3','6','50','NOx','y=((a*(b**x))*(x**c))','66.1022460401545','1.01108055044103','-0.58524583763016','0','0','6','64']
  H_HDV['46_PM_6_50'] = [' 46','EURO3','RT >12-14t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.156867770026232','4.75282307363365','-3.50765956251041','-0.817356848398025','0.142878198191729','6','64']
  H_HDV['46_FC_6_50'] = [' 46','EURO3','RT >12-14t Euro-3','6','50','FC','y=((a*(b**x))*(x**c))','1245.92894168425','1.00628833484597','-0.329404223646142','0','0','6','64']
  H_HDV['46_CO_6_100'] = [' 46','EURO3','RT >12-14t Euro-3','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.12616737561573','-0.028486054156273','-0.645332823993282','0.096441198301237','0','6','53']
  H_HDV['46_THC_6_100'] = [' 46','EURO3','RT >12-14t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.253621929641541','1.07430180129078','7.5144171999117','2.96378866433184','-0.028552333085584','6','53']
  H_HDV['46_NOx_6_100'] = [' 46','EURO3','RT >12-14t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','43.3311029941396','-0.019203054001332','-3.77593773063967','0.134149271845293','0','6','53']
  H_HDV['46_PM_6_100'] = [' 46','EURO3','RT >12-14t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003805650993','0.000501825108985','-0.023109871350371','0.570216393359476','0','6','53']
  H_HDV['46_FC_6_100'] = [' 46','EURO3','RT >12-14t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','251.056476794747','0.054013654071676','1852.90185787859','0.500259345717672','561.40178747824','6','53']
  H_HDV['47_CO_0_0'] = [' 47','EURO4','RT >12-14t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033323243042697','0.382035205565207','4.73828154646235','1.98868848960964','-0.013132617057688','6','86']
  H_HDV['47_THC_0_0'] = [' 47','EURO4','RT >12-14t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002975237783638','0.078753195363685','4.31421228591216','1.83636090639358','-0.011352780847722','6','86']
  H_HDV['47_NOx_0_0'] = [' 47','EURO4','RT >12-14t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.17223543682272','0.082395134689342','43.2840099739378','0.584053794368338','2.15635949275647','6','86']
  H_HDV['47_PM_0_0'] = [' 47','EURO4','RT >12-14t Euro-4','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007162358311324','0.143076664842621','3.62618812027199','1.74327904274682','-0.008606448073764','6','86']
  H_HDV['47_FC_0_0'] = [' 47','EURO4','RT >12-14t Euro-4','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001303949490254','0.000227614067904','-0.000001837119394','0','0','6','86']
  H_HDV['47_CO_0_50'] = [' 47','EURO4','RT >12-14t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.035959181293332','0.414096502412669','3.95027709752422','1.72844274016971','-0.008126526307729','6','86']
  H_HDV['47_THC_0_50'] = [' 47','EURO4','RT >12-14t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002967847116648','0.076745023887104','4.43356156829053','1.88976661022612','-0.012616774681861','6','86']
  H_HDV['47_NOx_0_50'] = [' 47','EURO4','RT >12-14t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.26904751969034','0.064951494287191','41.2441926557917','0.54563689173399','2.39225386913061','6','86']
  H_HDV['47_PM_0_50'] = [' 47','EURO4','RT >12-14t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.044896313330481','0.042940100638557','0.140516325338579','0.164229315527863','0.010835504189757','6','86']
  H_HDV['47_FC_0_50'] = [' 47','EURO4','RT >12-14t Euro-4','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00152331368798','0.000176016626456','-0.000001330251823','','','6','86']
  H_HDV['47_CO_0_100'] = [' 47','EURO4','RT >12-14t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.034812910230755','0.420579226196558','3.75516336074507','1.63778508774623','-0.007136501656724','6','86']
  H_HDV['47_THC_0_100'] = [' 47','EURO4','RT >12-14t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00299594420077','0.075872450124019','4.42081800211816','1.8954569125593','-0.012950761869292','6','86']
  H_HDV['47_NOx_0_100'] = [' 47','EURO4','RT >12-14t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.10821956361727','0.05330257313364','60.7956797296943','0.610123398429254','2.55195786416333','6','86']
  H_HDV['47_PM_0_100'] = [' 47','EURO4','RT >12-14t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047058540625223','0.04293181435301','0.137238298838722','0.166166883913093','0.010830068309136','6','86']
  H_HDV['47_FC_0_100'] = [' 47','EURO4','RT >12-14t Euro-4','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001673850598001','0.000136626477857','-0.000000940784084','0','0','6','86']
  H_HDV['47_CO_-2_0'] = [' 47','EURO4','RT >12-14t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.043344889789277','3.55822930672033','-2.72256920985574','-0.445947918477549','0.112137341429202','6','86']
  H_HDV['47_THC_-2_0'] = [' 47','EURO4','RT >12-14t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007842031303767','0.365875275986357','-1.65793035940836','-0.204463213294852','0.098846025627007','6','86']
  H_HDV['47_NOx_-2_0'] = [' 47','EURO4','RT >12-14t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.41212585940627','0.079440251392408','182.160312732536','0.842977987836281','0.960137922020852','6','86']
  H_HDV['47_PM_-2_0'] = [' 47','EURO4','RT >12-14t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003776915709829','0.100075542535257','6.39127865596053','2.61715038791923','-0.028448877699405','6','86']
  H_HDV['47_FC_-2_0'] = [' 47','EURO4','RT >12-14t Euro-4','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','469.987903200889','0.246110196805001','-32.0036416014282','0.074937874221566','0','6','86']
  H_HDV['47_CO_-2_50'] = [' 47','EURO4','RT >12-14t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037904713917389','2.66476694829102','-2.33053364756313','-0.367430348854747','0.106064643857823','6','86']
  H_HDV['47_THC_-2_50'] = [' 47','EURO4','RT >12-14t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006738989917324','0.369977400159437','-1.68743128155806','-0.194277550425286','0.097825945124888','6','86']
  H_HDV['47_NOx_-2_50'] = [' 47','EURO4','RT >12-14t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.20826551591187','0.058412504529243','104.594758068183','0.692964537091403','0.755393453540484','6','86']
  H_HDV['47_PM_-2_50'] = [' 47','EURO4','RT >12-14t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006302584669922','0.097185227613026','5.8077318938494','2.3188672381641','-0.005513981683313','6','86']
  H_HDV['47_FC_-2_50'] = [' 47','EURO4','RT >12-14t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','330.675850886029','0.061707868099443','26230.1681529565','1.02209456586453','45.8181200696692','6','86']
  H_HDV['47_CO_-2_100'] = [' 47','EURO4','RT >12-14t Euro-4','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-2.59072132733463','0.731828314962592','0.423947708326682','0.076725913112123','0.030773903644938','6','86']
  H_HDV['47_THC_-2_100'] = [' 47','EURO4','RT >12-14t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000992018406551','0.055083557598034','7.28954507211037','2.8210631982979','-0.033335601830719','6','86']
  H_HDV['47_NOx_-2_100'] = [' 47','EURO4','RT >12-14t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.63721855896398','0.043512838846276','90.9061002447486','0.638809217832456','0.535606723302606','6','86']
  H_HDV['47_PM_-2_100'] = [' 47','EURO4','RT >12-14t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002898198690526','0.103305285791072','5.38011835904649','2.21798715414383','-0.014282746753352','6','86']
  H_HDV['47_FC_-2_100'] = [' 47','EURO4','RT >12-14t Euro-4','-2','100','FC','y=((a*(b**x))*(x**c))','913.587889053279','0.99127736586529','-0.566865441038616','0','0','6','86']
  H_HDV['47_CO_-4_0'] = [' 47','EURO4','RT >12-14t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017579564060518','1.94437963886226','-1.77873330262034','-0.209231467897077','0.09328873626988','6','86']
  H_HDV['47_THC_-4_0'] = [' 47','EURO4','RT >12-14t Euro-4','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001228109905','0.053027477165626','7.24246226021795','2.75981165178308','-0.022774155662747','6','86']
  H_HDV['47_NOx_-4_0'] = [' 47','EURO4','RT >12-14t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.50746757209143','0.060606248902846','54.6736909055477','0.561760269271101','0.184347112453304','6','86']
  H_HDV['47_PM_-4_0'] = [' 47','EURO4','RT >12-14t Euro-4','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001169638748095','0.102335525911851','5.52745906268061','2.27918000804911','-0.012717436648048','6','86']
  H_HDV['47_FC_-4_0'] = [' 47','EURO4','RT >12-14t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','293.151372694492','0.062901196690421','4750.47580075992','0.696711432609584','12.6860612026389','6','86']
  H_HDV['47_CO_-4_50'] = [' 47','EURO4','RT >12-14t Euro-4','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000223700757955','0.266863749199943','6.17734860939221','2.32804396729169','-0.010110356447283','6','86']
  H_HDV['47_THC_-4_50'] = [' 47','EURO4','RT >12-14t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.076015125895736','-0.00004864870617','-0.005962071154821','0.084878405125647','0','6','86']
  H_HDV['47_NOx_-4_50'] = [' 47','EURO4','RT >12-14t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.80586917002731','0.050970823343673','67.1669110893839','0.579918869392983','0.087915360938082','6','86']
  H_HDV['47_PM_-4_50'] = [' 47','EURO4','RT >12-14t Euro-4','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.144071746942886','-0.000096001275102','-0.013693418957142','0.101897260490657','0','6','86']
  H_HDV['47_FC_-4_50'] = [' 47','EURO4','RT >12-14t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','241.997356283494','0.052524710279739','4865.72097215868','0.665332441647278','6.35736207240674','6','86']
  H_HDV['47_CO_-4_100'] = [' 47','EURO4','RT >12-14t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.354866118751091','-0.000269623868512','-0.027094868294942','0.082382251556499','0','6','86']
  H_HDV['47_THC_-4_100'] = [' 47','EURO4','RT >12-14t Euro-4','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.072706256089233','-0.000053178632341','-0.005841470374522','0.087369950287471','0','6','86']
  H_HDV['47_NOx_-4_100'] = [' 47','EURO4','RT >12-14t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.51567086280062','0.046075461919581','95.1862027638704','0.633848319020875','0.044078519368917','6','86']
  H_HDV['47_PM_-4_100'] = [' 47','EURO4','RT >12-14t Euro-4','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.140504014399519','-0.000102757058942','-0.013662283197048','0.104595740063235','0','6','86']
  H_HDV['47_FC_-4_100'] = [' 47','EURO4','RT >12-14t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','218.63418883836','0.04709941105172','7594.91339877801','0.728111790186897','3.56703347809819','6','86']
  H_HDV['47_CO_-6_0'] = [' 47','EURO4','RT >12-14t Euro-4','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.33513168770552','0.000308245165401','-0.022801537752858','0.066013327051768','0','6','72']
  H_HDV['47_THC_-6_0'] = [' 47','EURO4','RT >12-14t Euro-4','-6','0','THC','y=(c+(a*exp(b*x)))','0.068644554695652','-0.077313598320476','0.002277035992694','0','0','6','72']
  H_HDV['47_NOx_-6_0'] = [' 47','EURO4','RT >12-14t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.22925059631962','0.069675816730422','92.6120780482357','0.667258133638247','0.04389559935477','6','72']
  H_HDV['47_PM_-6_0'] = [' 47','EURO4','RT >12-14t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087098965173307','0.074127470846845','0.07204126610426','0.201376465104897','0.002736346132719','6','72']
  H_HDV['47_FC_-6_0'] = [' 47','EURO4','RT >12-14t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','263.235654937672','0.069364918576658','8533.44110079838','0.797160424909147','3.5292829745946','6','72']
  H_HDV['47_CO_-6_50'] = [' 47','EURO4','RT >12-14t Euro-4','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.296249581777614','0.000194171016489','-0.020456689283534','0.068016604474143','0','6','72']
  H_HDV['47_THC_-6_50'] = [' 47','EURO4','RT >12-14t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.054279791996483','0.071704060211021','0.104889439073164','0.507178898343468','0.002032586587683','6','72']
  H_HDV['47_NOx_-6_50'] = [' 47','EURO4','RT >12-14t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.54554499671721','0.065076018978913','123.686991285077','0.703498825676741','0.030011012280573','6','72']
  H_HDV['47_PM_-6_50'] = [' 47','EURO4','RT >12-14t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082012549318078','0.0733553179329','0.151697829915622','0.329977483065245','0.002567400343888','6','72']
  H_HDV['47_FC_-6_50'] = [' 47','EURO4','RT >12-14t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','212.650525858859','0.063776186913179','10977.7026515971','0.81274436138898','2.51009870571701','6','72']
  H_HDV['47_CO_-6_100'] = [' 47','EURO4','RT >12-14t Euro-4','-6','100','CO','y=(c+(a*exp(b*x)))','0.265709935909401','-0.073631934853882','0.005924275165713','0','0','6','72']
  H_HDV['47_THC_-6_100'] = [' 47','EURO4','RT >12-14t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047333672879944','0.068862562176457','0.275518452143526','0.618620525222685','0.001933378028041','6','72']
  H_HDV['47_NOx_-6_100'] = [' 47','EURO4','RT >12-14t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.18349975042122','0.062057931982863','150.357199301973','0.73357026868089','0.024209654701533','6','72']
  H_HDV['47_PM_-6_100'] = [' 47','EURO4','RT >12-14t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.075714622954622','0.071918554723159','0.249284999881113','0.409078095117505','0.002489094826031','6','72']
  H_HDV['47_FC_-6_100'] = [' 47','EURO4','RT >12-14t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','186.202248872983','0.060089763632039','14039.7580476703','0.845926740121451','2.0163100341124','6','72']
  H_HDV['47_CO_2_0'] = [' 47','EURO4','RT >12-14t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.072086482113828','0.300147644441111','7.69685196589643','3.00129364377193','-0.019684021467228','6','86']
  H_HDV['47_THC_2_0'] = [' 47','EURO4','RT >12-14t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003837423546468','0.071692222346495','5.59239107526358','2.18808195553278','-0.012347753311939','6','86']
  H_HDV['47_NOx_2_0'] = [' 47','EURO4','RT >12-14t Euro-4','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.3310433416151','0.003957109714593','-1.13524757127959','0.11337634271776','0','6','86']
  H_HDV['47_PM_2_0'] = [' 47','EURO4','RT >12-14t Euro-4','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.172893871168123','-0.000137675690277','-0.016101046477628','0.108054068085544','0','6','86']
  H_HDV['47_FC_2_0'] = [' 47','EURO4','RT >12-14t Euro-4','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','629.784091118152','0.817939419237385','-37.7758081638454','0.078225402798285','0','6','86']
  H_HDV['47_CO_2_50'] = [' 47','EURO4','RT >12-14t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.618736691275951','-0.000653060019089','-0.057371396192841','0.116505198897398','0','6','86']
  H_HDV['47_THC_2_50'] = [' 47','EURO4','RT >12-14t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004811038992736','0.067926199950006','6.25778843938026','2.44377352928115','-0.016203113240305','6','86']
  H_HDV['47_NOx_2_50'] = [' 47','EURO4','RT >12-14t Euro-4','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.07293909252559','0.090934570795716','36.6330896987672','0.588307808852034','4.23525531097379','6','86']
  H_HDV['47_PM_2_50'] = [' 47','EURO4','RT >12-14t Euro-4','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.175453234426177','-0.000171350527893','-0.016335145373532','0.109729538968232','0','6','86']
  H_HDV['47_FC_2_50'] = [' 47','EURO4','RT >12-14t Euro-4','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.144345985943133','1.48904941847287','1137.05224616003','-0.46190372099682','0','6','86']
  H_HDV['47_CO_2_100'] = [' 47','EURO4','RT >12-14t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.651290807311985','-0.001202423880414','-0.063484851877341','0.128262087160967','0','6','86']
  H_HDV['47_THC_2_100'] = [' 47','EURO4','RT >12-14t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006142743479925','0.063932720398386','7.03037851013847','2.74237212164955','-0.020704364797532','6','86']
  H_HDV['47_NOx_2_100'] = [' 47','EURO4','RT >12-14t Euro-4','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','1.98646045674672','3.21377932630485','-0.110036115340323','0','0','6','86']
  H_HDV['47_PM_2_100'] = [' 47','EURO4','RT >12-14t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.035368154868386','0.017134298099415','0.145160440775764','0.125775953493043','0.005462764840917','6','86']
  H_HDV['47_FC_2_100'] = [' 47','EURO4','RT >12-14t Euro-4','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','674.000904664886','0.784811972375783','-30.1004724689023','0.069421375424175','0','6','86']
  H_HDV['47_CO_4_0'] = [' 47','EURO4','RT >12-14t Euro-4','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.111620971878931','1.59991573003285','-1.9375144939268','-0.688512209107193','0.160928677391938','6','72']
  H_HDV['47_THC_4_0'] = [' 47','EURO4','RT >12-14t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004390182903895','0.070684969410165','6.14012849873408','2.39267600486999','-0.01570001808092','6','72']
  H_HDV['47_NOx_4_0'] = [' 47','EURO4','RT >12-14t Euro-4','4','0','NOx','y=((a*(x**b))+(c*(x**d)))','0.584621256088236','0.414417705166701','32.1248776766345','-0.726640756494622','0','6','72']
  H_HDV['47_PM_4_0'] = [' 47','EURO4','RT >12-14t Euro-4','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.176415969095045','-0.000165298826651','-0.016122127923681','0.108580719957527','0','6','72']
  H_HDV['47_FC_4_0'] = [' 47','EURO4','RT >12-14t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','1355.79390300372','1.01101347512383','-0.561946549378512','0','0','6','72']
  H_HDV['47_CO_4_50'] = [' 47','EURO4','RT >12-14t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.631549733136133','-0.001234235047782','-0.053234747794546','0.114994812557891','0','6','71']
  H_HDV['47_THC_4_50'] = [' 47','EURO4','RT >12-14t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006547974607978','0.0651427815757','7.11967471739516','2.78041464530114','-0.02131153107378','6','71']
  H_HDV['47_NOx_4_50'] = [' 47','EURO4','RT >12-14t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.56583358478744','0.047709798963824','14.5606730453119','0.270522145553663','5.6590316910465','6','71']
  H_HDV['47_PM_4_50'] = [' 47','EURO4','RT >12-14t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.042815990014749','0.02109866059518','0.142664105570981','0.131964794589323','0.007066510315103','6','71']
  H_HDV['47_FC_4_50'] = [' 47','EURO4','RT >12-14t Euro-4','4','50','FC','y=((a*(b**x))*(x**c))','1225.58664204367','1.00837220565917','-0.434270267420803','0','0','6','71']
  H_HDV['47_CO_4_100'] = [' 47','EURO4','RT >12-14t Euro-4','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.287700710093489','0.014981066682429','0.441527416303746','0.136326224388034','-0.02195987292429','6','68']
  H_HDV['47_THC_4_100'] = [' 47','EURO4','RT >12-14t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008116547400928','0.062399809235579','7.45816433831464','2.93572923853303','-0.024097910511924','6','68']
  H_HDV['47_NOx_4_100'] = [' 47','EURO4','RT >12-14t Euro-4','4','100','NOx','y=exp((a+(b/x))+(c*log(x)))','2.35781075761955','2.27285364302418','-0.113869323255541','0','0','6','68']
  H_HDV['47_PM_4_100'] = [' 47','EURO4','RT >12-14t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047356750963197','0.015747500262765','0.147802451531826','0.124064585362565','0.000478988494166','6','68']
  H_HDV['47_FC_4_100'] = [' 47','EURO4','RT >12-14t Euro-4','4','100','FC','y=((a*(b**x))*(x**c))','1162.28085929832','1.0060208968293','-0.337721191809134','0','0','6','68']
  H_HDV['47_CO_6_0'] = [' 47','EURO4','RT >12-14t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.856810712007258','-0.001990553479367','-0.13014453927248','0.205850811995345','0','6','71']
  H_HDV['47_THC_6_0'] = [' 47','EURO4','RT >12-14t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009242571520033','0.46199998192596','-1.64721017123701','-0.170253073330287','0.100889598760622','6','71']
  H_HDV['47_NOx_6_0'] = [' 47','EURO4','RT >12-14t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.0310692624198','-0.016820200889298','-2.78451337690182','0.197019662222852','0','6','71']
  H_HDV['47_PM_6_0'] = [' 47','EURO4','RT >12-14t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.208519236476523','-0.00038436103108','-0.024271909896693','0.14463681434181','0','6','71']
  H_HDV['47_FC_6_0'] = [' 47','EURO4','RT >12-14t Euro-4','6','0','FC','y=((a*(x**b))+(c*(x**d)))','0.050047221924333','1.8285594708362','1143.17435285075','-0.370437561911431','0','6','71']
  H_HDV['47_CO_6_50'] = [' 47','EURO4','RT >12-14t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.766522170218587','-0.002614310111709','-0.078667340696139','0.1479642113908','0','6','65']
  H_HDV['47_THC_6_50'] = [' 47','EURO4','RT >12-14t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008111369503656','0.062437672501938','7.64846691254774','3.00651380155112','-0.026969959393283','6','65']
  H_HDV['47_NOx_6_50'] = [' 47','EURO4','RT >12-14t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.0925312963203','0.068989746969119','17.8486523099288','0.388206425680774','7.37412686922735','6','65']
  H_HDV['47_PM_6_50'] = [' 47','EURO4','RT >12-14t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.203780777459183','-0.000446391168811','-0.020633608023077','0.127129136173668','0','6','65']
  H_HDV['47_FC_6_50'] = [' 47','EURO4','RT >12-14t Euro-4','6','50','FC','y=((a*(b**x))*(x**c))','1234.35948266544','1.00689879034988','-0.348611315710783','0','0','6','65']
  H_HDV['47_CO_6_100'] = [' 47','EURO4','RT >12-14t Euro-4','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','1.71910381324172','-3.97018284407616','-0.971928057392856','0','0','6','55']
  H_HDV['47_THC_6_100'] = [' 47','EURO4','RT >12-14t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009206051653109','0.061686580198635','7.29724247254824','2.90280367289616','-0.026701247165638','6','55']
  H_HDV['47_NOx_6_100'] = [' 47','EURO4','RT >12-14t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.26977374014693','0.074706975782328','24.2140682442642','0.460170273116636','9.01391340582515','6','55']
  H_HDV['47_PM_6_100'] = [' 47','EURO4','RT >12-14t Euro-4','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.204730774769468','-0.000430571286214','-0.018540867525278','0.111572157377907','0','6','55']
  H_HDV['47_FC_6_100'] = [' 47','EURO4','RT >12-14t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','226.789971259114','0.053672895255134','1360.3061888697','0.43154276167872','532.734047736263','6','55']
  H_HDV['48_CO_0_0'] = [' 48','EURO5','RT >12-14t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033116994895147','0.389880516138446','4.74378236084588','1.98872045636548','-0.013151039771182','6','86']
  H_HDV['48_THC_0_0'] = [' 48','EURO5','RT >12-14t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002937583930759','0.08034657320174','4.32761130379282','1.84208724325462','-0.011853863392281','6','86']
  H_HDV['48_NOx_0_0'] = [' 48','EURO5','RT >12-14t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.58959124403848','0.077649565733791','19.3886626865205','0.459439785028575','1.26599704701907','6','86']
  H_HDV['48_PM_0_0'] = [' 48','EURO5','RT >12-14t Euro-5','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007162615635924','0.145510111316867','3.64316347297434','1.74827843998213','-0.008813705457831','6','86']
  H_HDV['48_FC_0_0'] = [' 48','EURO5','RT >12-14t Euro-5','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001247838741527','0.000223748149995','-0.000001792621573','0','0','6','86']
  H_HDV['48_CO_0_50'] = [' 48','EURO5','RT >12-14t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.036137448410388','0.420824812256247','3.97191463324465','1.73457489802776','-0.008152943558744','6','86']
  H_HDV['48_THC_0_50'] = [' 48','EURO5','RT >12-14t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002950648638389','0.078186934960576','4.44760757776771','1.8942502221606','-0.012959905438949','6','86']
  H_HDV['48_NOx_0_50'] = [' 48','EURO5','RT >12-14t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.09561449495645','0.061209055484887','19.926670275366','0.444080438086012','1.39479545542295','6','86']
  H_HDV['48_PM_0_50'] = [' 48','EURO5','RT >12-14t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.045729832040605','0.042994674032195','0.143086923597241','0.163979256963125','0.010974801411818','6','86']
  H_HDV['48_FC_0_50'] = [' 48','EURO5','RT >12-14t Euro-5','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001465006488454','0.000173554737458','-0.000001302676834','','','6','86']
  H_HDV['48_CO_0_100'] = [' 48','EURO5','RT >12-14t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.034460389333935','0.431262700985766','3.71309224483562','1.62527165881068','-0.007113396914071','6','86']
  H_HDV['48_THC_0_100'] = [' 48','EURO5','RT >12-14t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003083010588426','0.077092360680632','4.44608427886312','1.90285107803616','-0.012986107258604','6','86']
  H_HDV['48_NOx_0_100'] = [' 48','EURO5','RT >12-14t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.12403546494301','0.051621192519263','24.7148743094197','0.486645635274023','1.4745822133438','6','86']
  H_HDV['48_PM_0_100'] = [' 48','EURO5','RT >12-14t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047946144404768','0.042913386474843','0.139931067111686','0.166367329786091','0.010962452062191','6','86']
  H_HDV['48_FC_0_100'] = [' 48','EURO5','RT >12-14t Euro-5','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001613386973847','0.000135181883327','-0.000000925366877','0','0','6','86']
  H_HDV['48_CO_-2_0'] = [' 48','EURO5','RT >12-14t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04405602465705','3.62527770754667','-2.71637903821733','-0.443760598083758','0.111929668697985','6','86']
  H_HDV['48_THC_-2_0'] = [' 48','EURO5','RT >12-14t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007994014772002','0.371191195498778','-1.64682843120833','-0.202550253083','0.098704984610548','6','86']
  H_HDV['48_NOx_-2_0'] = [' 48','EURO5','RT >12-14t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.10538060540473','0.079659801520076','70.54964755458','0.719282040169607','0.564498812339041','6','86']
  H_HDV['48_PM_-2_0'] = [' 48','EURO5','RT >12-14t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003814504419635','0.102245939686579','6.37739195160136','2.61133079534886','-0.028249093728729','6','86']
  H_HDV['48_FC_-2_0'] = [' 48','EURO5','RT >12-14t Euro-5','-2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','486.149682106425','0.236394364462377','-33.2589400902974','0.07528752308263','0','6','86']
  H_HDV['48_CO_-2_50'] = [' 48','EURO5','RT >12-14t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.038445352801632','2.80684178285995','-2.3720616840462','-0.371150256183896','0.106114656094262','6','86']
  H_HDV['48_THC_-2_50'] = [' 48','EURO5','RT >12-14t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006860062064711','0.379444304166553','-1.69096010255908','-0.192218269058128','0.097507297666403','6','86']
  H_HDV['48_NOx_-2_50'] = [' 48','EURO5','RT >12-14t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.23811240361049','0.058346383132569','47.2951672145329','0.599957854532291','0.440701584280901','6','86']
  H_HDV['48_PM_-2_50'] = [' 48','EURO5','RT >12-14t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00293256951447','0.101978043888652','6.10201237984213','2.50289926306305','-0.024968970632113','6','86']
  H_HDV['48_FC_-2_50'] = [' 48','EURO5','RT >12-14t Euro-5','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','341.097428658568','0.061627045903679','24670.0755035762','1.00644908388824','46.4387316793378','6','86']
  H_HDV['48_CO_-2_100'] = [' 48','EURO5','RT >12-14t Euro-5','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-2.42312054483695','0.716874087905702','0.433349513550962','0.076731915546096','0.031132667633511','6','86']
  H_HDV['48_THC_-2_100'] = [' 48','EURO5','RT >12-14t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001160395828454','0.056020266933829','7.30660558777512','2.82430152723184','-0.032791883565616','6','86']
  H_HDV['48_NOx_-2_100'] = [' 48','EURO5','RT >12-14t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.87895022367258','0.044024971097906','45.5459477523485','0.569284777609336','0.311987038196261','6','86']
  H_HDV['48_PM_-2_100'] = [' 48','EURO5','RT >12-14t Euro-5','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003127238526729','0.105318638416089','5.35026689422836','2.20273214525117','-0.013079058371644','6','86']
  H_HDV['48_FC_-2_100'] = [' 48','EURO5','RT >12-14t Euro-5','-2','100','FC','y=((a*(b**x))*(x**c))','943.23519549432','0.991094129167184','-0.5678545067989','0','0','6','86']
  H_HDV['48_CO_-4_0'] = [' 48','EURO5','RT >12-14t Euro-5','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017864791023515','2.09806417390233','-1.85968640081081','-0.218875250854654','0.093472285412286','6','86']
  H_HDV['48_THC_-4_0'] = [' 48','EURO5','RT >12-14t Euro-5','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001342514213617','0.054116018715004','7.20551038664338','2.7412964336594','-0.021621283625735','6','86']
  H_HDV['48_NOx_-4_0'] = [' 48','EURO5','RT >12-14t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.84574678998205','0.061257714803523','32.9145822342416','0.524825904521963','0.108780736109728','6','86']
  H_HDV['48_PM_-4_0'] = [' 48','EURO5','RT >12-14t Euro-5','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001150873374918','0.104234914838477','5.54703043378911','2.28526583631239','-0.012934160732452','6','86']
  H_HDV['48_FC_-4_0'] = [' 48','EURO5','RT >12-14t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','293.978581854447','0.061780367783817','3183.37769203304','0.61903154976628','12.6823469095732','6','86']
  H_HDV['48_CO_-4_50'] = [' 48','EURO5','RT >12-14t Euro-5','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000171992311925','0.271315164691657','6.24162016578235','2.34896640059765','-0.010537014372357','6','86']
  H_HDV['48_THC_-4_50'] = [' 48','EURO5','RT >12-14t Euro-5','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000535288837492','0.055888703455885','5.60183400490954','2.17422185781375','-0.009011679616543','6','86']
  H_HDV['48_NOx_-4_50'] = [' 48','EURO5','RT >12-14t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.42359774100218','0.052351113621543','40.2728857149124','0.546886131823995','0.054392473023847','6','86']
  H_HDV['48_PM_-4_50'] = [' 48','EURO5','RT >12-14t Euro-5','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.146844521961569','-0.00009763591567','-0.01393814272375','0.101742569755096','0','6','86']
  H_HDV['48_FC_-4_50'] = [' 48','EURO5','RT >12-14t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','250.534416740967','0.052808524780336','4814.93766259766','0.659365189090573','6.5436656038523','6','86']
  H_HDV['48_CO_-4_100'] = [' 48','EURO5','RT >12-14t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.361995149712526','-0.000271601019693','-0.027606690478073','0.082222507549876','0','6','86']
  H_HDV['48_THC_-4_100'] = [' 48','EURO5','RT >12-14t Euro-5','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.074180792405411','-0.000054123928776','-0.005950210793184','0.087207342141336','0','6','86']
  H_HDV['48_NOx_-4_100'] = [' 48','EURO5','RT >12-14t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.2438970261286','0.047772656257743','54.2106291350224','0.593527846510034','0.030411098351763','6','86']
  H_HDV['48_PM_-4_100'] = [' 48','EURO5','RT >12-14t Euro-5','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.143196239485526','-0.000104055977391','-0.013903378977313','0.104402759773654','0','6','86']
  H_HDV['48_FC_-4_100'] = [' 48','EURO5','RT >12-14t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','226.312142902507','0.047490819105746','7452.32885093578','0.720140370499783','3.77997758489306','6','86']
  H_HDV['48_CO_-6_0'] = [' 48','EURO5','RT >12-14t Euro-5','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.342375526381823','0.000318130559324','-0.023273083256711','0.065913957527652','0','6','72']
  H_HDV['48_THC_-6_0'] = [' 48','EURO5','RT >12-14t Euro-5','-6','0','THC','y=(c+(a*exp(b*x)))','0.070116217774357','-0.077239661032387','0.002314948007705','0','0','6','72']
  H_HDV['48_NOx_-6_0'] = [' 48','EURO5','RT >12-14t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.73443999691414','0.070873024336409','53.2395974675676','0.624580265066283','0.027186135865347','6','72']
  H_HDV['48_PM_-6_0'] = [' 48','EURO5','RT >12-14t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08818098653096','0.073974483709672','0.07184606173843','0.194941052176773','0.002786291843317','6','72']
  H_HDV['48_FC_-6_0'] = [' 48','EURO5','RT >12-14t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','273.574015505958','0.069672485384418','8527.24170282911','0.793223144622498','3.66592396336676','6','72']
  H_HDV['48_CO_-6_50'] = [' 48','EURO5','RT >12-14t Euro-5','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.303089841967274','0.000199188505445','-0.020947391321775','0.068064298544753','0','6','72']
  H_HDV['48_THC_-6_50'] = [' 48','EURO5','RT >12-14t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.05574247834739','0.071923710523504','0.093705223801681','0.49152130253915','0.002077032729644','6','72']
  H_HDV['48_NOx_-6_50'] = [' 48','EURO5','RT >12-14t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.31257069582995','0.066523697669081','70.3078549575448','0.661789212891926','0.018993980389738','6','72']
  H_HDV['48_PM_-6_50'] = [' 48','EURO5','RT >12-14t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083634529364439','0.073376692234299','0.145891662702897','0.320419931204245','0.002615301215361','6','72']
  H_HDV['48_FC_-6_50'] = [' 48','EURO5','RT >12-14t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','221.719257939147','0.064282709317323','11303.3238420656','0.813881797283037','2.66042505268588','6','72']
  H_HDV['48_CO_-6_100'] = [' 48','EURO5','RT >12-14t Euro-5','-6','100','CO','y=(c+(a*exp(b*x)))','0.271683367546729','-0.073696124117515','0.006027021260107','0','0','6','72']
  H_HDV['48_THC_-6_100'] = [' 48','EURO5','RT >12-14t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048566805512732','0.069050559152136','0.270300411658061','0.615396707791564','0.001973375159577','6','72']
  H_HDV['48_NOx_-6_100'] = [' 48','EURO5','RT >12-14t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.08661776905051','0.063648350283001','84.3083352608768','0.690793468967053','0.015570948991662','6','72']
  H_HDV['48_PM_-6_100'] = [' 48','EURO5','RT >12-14t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.077367642314078','0.07199564535143','0.246387586101681','0.404254809783244','0.002535179036326','6','72']
  H_HDV['48_FC_-6_100'] = [' 48','EURO5','RT >12-14t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','192.964271455686','0.060386860770938','14129.6200348352','0.842423432860288','2.11704082700853','6','72']
  H_HDV['48_CO_2_0'] = [' 48','EURO5','RT >12-14t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.071591786836924','0.307902151336416','7.55846275458914','2.94319625381337','-0.018509206952302','6','86']
  H_HDV['48_THC_2_0'] = [' 48','EURO5','RT >12-14t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003886012609144','0.073141588563812','5.57511143921668','2.18131532448409','-0.012266519154334','6','86']
  H_HDV['48_NOx_2_0'] = [' 48','EURO5','RT >12-14t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.42103342412234','0.121391287593746','40.2667540535341','0.776007187175874','2.08744267105661','6','86']
  H_HDV['48_PM_2_0'] = [' 48','EURO5','RT >12-14t Euro-5','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.175524565405223','-0.000138203069869','-0.01628006967112','0.107494402237236','0','6','86']
  H_HDV['48_FC_2_0'] = [' 48','EURO5','RT >12-14t Euro-5','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','648.931101460471','0.806918886779495','-39.2194306630247','0.078609578758442','0','6','86']
  H_HDV['48_CO_2_50'] = [' 48','EURO5','RT >12-14t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.627153901580167','-0.000650540128495','-0.058015619389502','0.115984187042627','0','6','86']
  H_HDV['48_THC_2_50'] = [' 48','EURO5','RT >12-14t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004820258734793','0.069357259536697','6.23199138940189','2.4327665836919','-0.016087456605461','6','86']
  H_HDV['48_NOx_2_50'] = [' 48','EURO5','RT >12-14t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.1251792625983','-0.005822536790139','-1.01989462222084','0.139498442206308','0','6','86']
  H_HDV['48_PM_2_50'] = [' 48','EURO5','RT >12-14t Euro-5','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.178369502580417','-0.000172667460217','-0.016603171148453','0.109616994832146','0','6','86']
  H_HDV['48_FC_2_50'] = [' 48','EURO5','RT >12-14t Euro-5','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.14076340398158','1.49670987658813','1176.13325924153','-0.466149020495817','0','6','86']
  H_HDV['48_CO_2_100'] = [' 48','EURO5','RT >12-14t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.66293833813832','-0.001165744833728','-0.064847085191615','0.128027597697492','0','6','86']
  H_HDV['48_THC_2_100'] = [' 48','EURO5','RT >12-14t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006095461660004','0.065495998854712','6.98279157500596','2.72425086552714','-0.02068164267919','6','86']
  H_HDV['48_NOx_2_100'] = [' 48','EURO5','RT >12-14t Euro-5','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','1.71151164881735','2.96472777838633','-0.179663650490068','0','0','6','86']
  H_HDV['48_PM_2_100'] = [' 48','EURO5','RT >12-14t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.035547294695961','0.019623984713835','0.147112461435665','0.127693602699967','0.007482026880785','6','86']
  H_HDV['48_FC_2_100'] = [' 48','EURO5','RT >12-14t Euro-5','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','693.37850415761','0.805689684462025','-31.2332750675109','0.06934269986617','0','6','86']
  H_HDV['48_CO_4_0'] = [' 48','EURO5','RT >12-14t Euro-5','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.112902316093898','1.60871295577171','-2.01544895379549','-0.759611345999997','0.167419784621697','6','72']
  H_HDV['48_THC_4_0'] = [' 48','EURO5','RT >12-14t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004385326923374','0.07220284586667','6.10913719715173','2.37975276564205','-0.015528277815442','6','72']
  H_HDV['48_NOx_4_0'] = [' 48','EURO5','RT >12-14t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.88456739809513','0.098096444440016','8.45241530979743','0.297708327078148','2.84585543571764','6','72']
  H_HDV['48_PM_4_0'] = [' 48','EURO5','RT >12-14t Euro-5','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.178798957757343','-0.000159268173623','-0.016243476375423','0.107671059436237','0','6','72']
  H_HDV['48_FC_4_0'] = [' 48','EURO5','RT >12-14t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','1406.81932842781','1.01105661474501','-0.568217207069732','0','0','6','72']
  H_HDV['48_CO_4_50'] = [' 48','EURO5','RT >12-14t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.636081804409875','-0.001121687001543','-0.053079659634254','0.112929710277839','0','6','71']
  H_HDV['48_THC_4_50'] = [' 48','EURO5','RT >12-14t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006544416534536','0.066526168564554','7.11061410551412','2.77430720426744','-0.021259603410073','6','71']
  H_HDV['48_NOx_4_50'] = [' 48','EURO5','RT >12-14t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.25315960463597','0.032032939649748','10.9613441922491','0.256104813412319','2.99440174496759','6','71']
  H_HDV['48_PM_4_50'] = [' 48','EURO5','RT >12-14t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.04304047281679','0.025396618701686','0.142484569994996','0.133761570710845','0.010287703554451','6','71']
  H_HDV['48_FC_4_50'] = [' 48','EURO5','RT >12-14t Euro-5','4','50','FC','y=((a*(b**x))*(x**c))','1271.11036227995','1.00848854994514','-0.441353501443963','0','0','6','71']
  H_HDV['48_CO_4_100'] = [' 48','EURO5','RT >12-14t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.314031935141628','0.012055963679898','0.45597143066721','0.135280427523943','-0.054618434755882','6','69']
  H_HDV['48_THC_4_100'] = [' 48','EURO5','RT >12-14t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008195516133891','0.063428855847191','7.55859757899166','2.97187416101711','-0.024835228560424','6','69']
  H_HDV['48_NOx_4_100'] = [' 48','EURO5','RT >12-14t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.10240741120879','0.024883749708983','10.7943537024599','0.25691401651583','3.03947438690862','6','69']
  H_HDV['48_PM_4_100'] = [' 48','EURO5','RT >12-14t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.046808890001282','0.018339544454309','0.148381674445613','0.12566625788451','0.003707832722151','6','69']
  H_HDV['48_FC_4_100'] = [' 48','EURO5','RT >12-14t Euro-5','4','100','FC','y=((a*(b**x))*(x**c))','1205.78367747553','1.00622208980572','-0.345993639787143','0','0','6','69']
  H_HDV['48_CO_6_0'] = [' 48','EURO5','RT >12-14t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.864348326993213','-0.001986678267829','-0.13087942367424','0.205391897537876','0','6','71']
  H_HDV['48_THC_6_0'] = [' 48','EURO5','RT >12-14t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009338929581787','0.454977976722306','-1.57778522931524','-0.149131697275741','0.099247780692628','6','71']
  H_HDV['48_NOx_6_0'] = [' 48','EURO5','RT >12-14t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.6733074786801','-0.017935588110508','-2.96328891031691','0.241921094840481','0','6','71']
  H_HDV['48_PM_6_0'] = [' 48','EURO5','RT >12-14t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.213760720326866','-0.000389932805047','-0.025276954490475','0.146818149619213','0','6','71']
  H_HDV['48_FC_6_0'] = [' 48','EURO5','RT >12-14t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','1405.60128337219','1.01102212407418','-0.503261464328791','0','0','6','71']
  H_HDV['48_CO_6_50'] = [' 48','EURO5','RT >12-14t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.800118855959061','-0.00268555055473','-0.088310495955594','0.15885610886447','0','6','66']
  H_HDV['48_THC_6_50'] = [' 48','EURO5','RT >12-14t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00826506420678','0.063163675504643','7.85180424560927','3.08212502446961','-0.028487302131321','6','66']
  H_HDV['48_NOx_6_50'] = [' 48','EURO5','RT >12-14t Euro-5','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.3144014055905','-0.023255393246536','-1.60482436261286','0.177798428439367','0','6','66']
  H_HDV['48_PM_6_50'] = [' 48','EURO5','RT >12-14t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.207907458576591','-0.00044017777892','-0.021278008966306','0.128247264924127','0','6','66']
  H_HDV['48_FC_6_50'] = [' 48','EURO5','RT >12-14t Euro-5','6','50','FC','y=((a*(b**x))*(x**c))','1283.1934352468','1.00721614635367','-0.35860733082277','0','0','6','66']
  H_HDV['48_CO_6_100'] = [' 48','EURO5','RT >12-14t Euro-5','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','1.630609634653','-3.6911272626053','-0.944015521657857','0','0','6','56']
  H_HDV['48_THC_6_100'] = [' 48','EURO5','RT >12-14t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00957041270983','0.062160421126801','7.51734569227456','2.98718057620241','-0.028158555267111','6','56']
  H_HDV['48_NOx_6_100'] = [' 48','EURO5','RT >12-14t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.40726202623622','0.049505804387436','10.946877819404','0.296513006788833','4.51813685401049','6','56']
  H_HDV['48_PM_6_100'] = [' 48','EURO5','RT >12-14t Euro-5','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.206143817464298','-0.000388279154606','-0.018391728712271','0.109098592444727','0','6','56']
  H_HDV['48_FC_6_100'] = [' 48','EURO5','RT >12-14t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','257.608779890864','0.062593474194948','1638.7355615098','0.479719324348273','547.25807606731','6','56']
  H_HDV['49_CO_0_0'] = [' 49','EURO0','RT >14-20t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.81719022126014','0.053477979659956','14.5534979045098','0.179437774859475','1.99128556955185','6','86']
  H_HDV['49_THC_0_0'] = [' 49','EURO0','RT >14-20t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.22508428279114','0.036496584849078','9.02434331093858','0.138565756013851','0.75356909827634','6','86']
  H_HDV['49_NOx_0_0'] = [' 49','EURO0','RT >14-20t 80ties','0','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.02254222481215','0.002914472005872','-0.000022785202906','0','0','6','86']
  H_HDV['49_PM_0_0'] = [' 49','EURO0','RT >14-20t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.34656271887982','0.050150229369297','2.54907618260806','0.170350768854914','0.310437204521294','6','86']
  H_HDV['49_FC_0_0'] = [' 49','EURO0','RT >14-20t 80ties','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00057618083714','0.000136799665247','-0.000000962826501','0','0','6','86']
  H_HDV['49_CO_0_50'] = [' 49','EURO0','RT >14-20t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.655922223004','0.055706555357486','17.3777240266438','0.219678342545186','2.12158921024015','6','86']
  H_HDV['49_THC_0_50'] = [' 49','EURO0','RT >14-20t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.445045383266793','10.3742978759505','3.47842749326664','1.60208157704417','-0.0049924993492','6','86']
  H_HDV['49_NOx_0_50'] = [' 49','EURO0','RT >14-20t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.0034611298173','0.062624177872447','60.7552042898158','0.567441795445192','9.56283945256179','6','86']
  H_HDV['49_PM_0_50'] = [' 49','EURO0','RT >14-20t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53190961943603','0.052784267998817','2.89209108590079','0.207938060561436','0.32963870677828','6','86']
  H_HDV['49_FC_0_50'] = [' 49','EURO0','RT >14-20t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','692.849914256594','0.065870370954668','13057.7812184812','0.784971641604332','194.461584361101','6','86']
  H_HDV['49_CO_0_100'] = [' 49','EURO0','RT >14-20t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2722236412034','0.052951376117994','18.9720113741649','0.228357800171547','2.28776746705404','6','86']
  H_HDV['49_THC_0_100'] = [' 49','EURO0','RT >14-20t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.362110698749797','9.95709522247399','3.77701489987176','1.70830402800987','-0.009038642471033','6','86']
  H_HDV['49_NOx_0_100'] = [' 49','EURO0','RT >14-20t 80ties','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.7970602887587','0.050216436992107','75.1899209514808','0.597230071804695','10.1730150773228','6','86']
  H_HDV['49_PM_0_100'] = [' 49','EURO0','RT >14-20t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.55318784020516','0.051171373334978','3.2729478245349','0.230838628795682','0.351533245369354','6','86']
  H_HDV['49_FC_0_100'] = [' 49','EURO0','RT >14-20t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','648.277710832733','0.057782358240214','7543.44710319419','0.677563805570155','207.226763993479','6','86']
  H_HDV['49_CO_-2_0'] = [' 49','EURO0','RT >14-20t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.4460630850184','0.011441850731316','-1.27932849611503','0.07345505958314','0','6','86']
  H_HDV['49_THC_-2_0'] = [' 49','EURO0','RT >14-20t 80ties','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.822073950370388','7.58088621222179','3.88205695765916','1.41509696093246','0.039686566360154','6','86']
  H_HDV['49_NOx_-2_0'] = [' 49','EURO0','RT >14-20t 80ties','-2','0','NOx','y=(c+(a*exp(b*x)))','26.1067056083653','-0.07903944205821','3.91423240748967','0','0','6','86']
  H_HDV['49_PM_-2_0'] = [' 49','EURO0','RT >14-20t 80ties','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.02100653479009','0.000530813066371','-0.225310700029646','0.080546325955438','0','6','86']
  H_HDV['49_FC_-2_0'] = [' 49','EURO0','RT >14-20t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','86987.1324093394','1.09696746248837','672.55491392252','0.07642088940785','95.1485818594992','6','86']
  H_HDV['49_CO_-2_50'] = [' 49','EURO0','RT >14-20t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.9308836910931','0.011105733332312','-1.22079758656063','0.070947708041883','0','6','86']
  H_HDV['49_THC_-2_50'] = [' 49','EURO0','RT >14-20t 80ties','-2','50','THC','y=(c+(a*exp(b*x)))','9.15815400319613','-0.079787107058399','0.676501500572024','0','0','6','86']
  H_HDV['49_NOx_-2_50'] = [' 49','EURO0','RT >14-20t 80ties','-2','50','NOx','y=(1/(a+(b*x)))','0.025440426461424','0.004167683117186','0','0','0','6','86']
  H_HDV['49_PM_-2_50'] = [' 49','EURO0','RT >14-20t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.56120753952368','0.07716562892217','7.31793158132387','0.68169097116239','0.20980224651894','6','86']
  H_HDV['49_FC_-2_50'] = [' 49','EURO0','RT >14-20t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','583.010656794553','0.059150513522183','45106.8000625896','0.948174905869166','67.5191406074514','6','86']
  H_HDV['49_CO_-2_100'] = [' 49','EURO0','RT >14-20t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.5673268365607','0.072269971117248','36.9103503977146','0.595568657821967','1.13782375696194','6','86']
  H_HDV['49_THC_-2_100'] = [' 49','EURO0','RT >14-20t 80ties','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.487723627057081','8.45491805600944','3.14651626208249','1.22769483274902','0.035233042423025','6','86']
  H_HDV['49_NOx_-2_100'] = [' 49','EURO0','RT >14-20t 80ties','-2','100','NOx','y=((a*(b**x))*(x**c))','44.4207010461965','0.985567226288054','-0.410420954176305','0','0','6','86']
  H_HDV['49_PM_-2_100'] = [' 49','EURO0','RT >14-20t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.29200548173879','0.069650479265445','4.97435280504747','0.480212171926627','0.17843736753348','6','86']
  H_HDV['49_FC_-2_100'] = [' 49','EURO0','RT >14-20t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','534.105892949015','0.048082343901825','34589.2009660116','0.881628842840188','47.068519105058','6','86']
  H_HDV['49_CO_-4_0'] = [' 49','EURO0','RT >14-20t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.4163394366664','-0.005859075939882','-1.39536264484453','0.084761693020488','0','6','86']
  H_HDV['49_THC_-4_0'] = [' 49','EURO0','RT >14-20t 80ties','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.53439163157729','-0.001715726692146','-0.760790696424641','0.084192800542042','0','6','86']
  H_HDV['49_NOx_-4_0'] = [' 49','EURO0','RT >14-20t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.3969004744282','0.055096663166926','371.743513274143','0.71234884613785','0.619604300297824','6','86']
  H_HDV['49_PM_-4_0'] = [' 49','EURO0','RT >14-20t 80ties','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.17778573855767','0.072990983381022','1.48733717049584','0.291883185998307','0.10775806838351','6','86']
  H_HDV['49_FC_-4_0'] = [' 49','EURO0','RT >14-20t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','482.556666550135','0.058610088809007','15823.1898218028','0.745663733394842','17.8116266957545','6','86']
  H_HDV['49_CO_-4_50'] = [' 49','EURO0','RT >14-20t 80ties','-4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.053730730690318','0.005129531480873','0.000334282355151','0','0','6','86']
  H_HDV['49_THC_-4_50'] = [' 49','EURO0','RT >14-20t 80ties','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.275502686547561','13.0477081227935','1.26331649183114','0.728829815584582','0.049463075295767','6','86']
  H_HDV['49_NOx_-4_50'] = [' 49','EURO0','RT >14-20t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.9897590134682','0.047199969857975','526.472443357705','0.756019494719818','0.276313409951164','6','86']
  H_HDV['49_PM_-4_50'] = [' 49','EURO0','RT >14-20t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.94687746549135','0.069143451413837','4.3108432128886','0.41985857423915','0.089007096705097','6','86']
  H_HDV['49_FC_-4_50'] = [' 49','EURO0','RT >14-20t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','414.716278619184','0.051419695375303','18567.8802883722','0.760429165488454','8.50473523939693','6','86']
  H_HDV['49_CO_-4_100'] = [' 49','EURO0','RT >14-20t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4618276682732','0.061116826975293','47.9816613526937','0.468615184851447','0.301636619477785','6','86']
  H_HDV['49_THC_-4_100'] = [' 49','EURO0','RT >14-20t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.44194383321358','0.07058872667063','8.6554854589821','0.347750444136812','0.272658187995514','6','86']
  H_HDV['49_NOx_-4_100'] = [' 49','EURO0','RT >14-20t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.7526329107077','0.042228899632129','887.110806182569','0.83746537117012','0.092865307237734','6','86']
  H_HDV['49_PM_-4_100'] = [' 49','EURO0','RT >14-20t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.82005656085532','0.066746418300345','9.31887020777367','0.522905847829772','0.08344489962758','6','86']
  H_HDV['49_FC_-4_100'] = [' 49','EURO0','RT >14-20t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','376.423058695553','0.04722749383587','25238.4420990388','0.805709883472445','4.48314428276578','6','86']
  H_HDV['49_CO_-6_0'] = [' 49','EURO0','RT >14-20t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2327855594609','0.077027368460419','19.8473263239994','0.462393388332341','0.240623150749482','6','72']
  H_HDV['49_THC_-6_0'] = [' 49','EURO0','RT >14-20t 80ties','-6','0','THC','y=(c+(a*exp(b*x)))','8.60219866979421','-0.083832041244193','0.222101554611772','0','0','6','72']
  H_HDV['49_NOx_-6_0'] = [' 49','EURO0','RT >14-20t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6134052893809','0.064584280757186','674.518507897664','0.827221983580363','0.176953140207192','6','72']
  H_HDV['49_PM_-6_0'] = [' 49','EURO0','RT >14-20t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.09036228693907','0.076748439563147','3.59776469521583','0.450905614376191','0.064279662894466','6','72']
  H_HDV['49_FC_-6_0'] = [' 49','EURO0','RT >14-20t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','423.10287906165','0.064988265447154','21113.3089659942','0.799865538684414','4.14859532313695','6','72']
  H_HDV['49_CO_-6_50'] = [' 49','EURO0','RT >14-20t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5468258595154','0.071525715424759','71.6104938503651','0.576007661147328','0.198291087893147','6','72']
  H_HDV['49_THC_-6_50'] = [' 49','EURO0','RT >14-20t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.53220656313526','0.075506040742867','16.7725421381719','0.483035504882173','0.18900357737861','6','72']
  H_HDV['49_NOx_-6_50'] = [' 49','EURO0','RT >14-20t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.8031690543137','0.05964543053904','821.664566115933','0.841731840388499','0.124999813344391','6','72']
  H_HDV['49_PM_-6_50'] = [' 49','EURO0','RT >14-20t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.77429905454548','0.072604216448634','10.1262937976551','0.561143422364203','0.05927058098586','6','72']
  H_HDV['49_FC_-6_50'] = [' 49','EURO0','RT >14-20t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','346.998969853482','0.06067053604331','23382.7252278457','0.804194390091075','2.66137199669693','6','72']
  H_HDV['49_CO_-6_100'] = [' 49','EURO0','RT >14-20t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.14234461441148','0.067670499963284','108.211036021104','0.617135712719047','0.176837880240453','6','72']
  H_HDV['49_THC_-6_100'] = [' 49','EURO0','RT >14-20t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.92913980752473','0.073739536509754','26.1441961905609','0.53229648259623','0.181504522928275','6','72']
  H_HDV['49_NOx_-6_100'] = [' 49','EURO0','RT >14-20t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4229940332753','0.056512240723472','970.434356544459','0.865168854825682','0.102251550245759','6','72']
  H_HDV['49_PM_-6_100'] = [' 49','EURO0','RT >14-20t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.61959859580303','0.070128397093871','14.3865767122105','0.601486878771406','0.057140688554714','6','72']
  H_HDV['49_FC_-6_100'] = [' 49','EURO0','RT >14-20t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','309.757529040627','0.058232974928925','26679.2301872426','0.823975780809423','2.13026464939669','6','72']
  H_HDV['49_CO_2_0'] = [' 49','EURO0','RT >14-20t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.3023952885473','16.2772444760196','5.93361581733758','2.37534617620161','-0.025070043672587','6','86']
  H_HDV['49_THC_2_0'] = [' 49','EURO0','RT >14-20t 80ties','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','4.90419291961864','-4.59002842370441','-1.16240286995294','0','0','6','86']
  H_HDV['49_NOx_2_0'] = [' 49','EURO0','RT >14-20t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-21.561748748975','0.035593393280556','49.2719384315034','0.063785250640588','15.4317487324514','6','86']
  H_HDV['49_PM_2_0'] = [' 49','EURO0','RT >14-20t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.297467547792708','2.68299115322087','5.00907728035082','2.07354625134111','-0.012395952885334','6','86']
  H_HDV['49_FC_2_0'] = [' 49','EURO0','RT >14-20t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1261.94143360283','0.472163191150432','-83.3166339435924','0.081473612235612','0','6','86']
  H_HDV['49_CO_2_50'] = [' 49','EURO0','RT >14-20t 80ties','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.9620638420369','0.004311190855768','-1.86101078291065','0.08488895235948','0','6','86']
  H_HDV['49_THC_2_50'] = [' 49','EURO0','RT >14-20t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.28852219984509','9.47398438145853','4.82888988333041','1.94547604329032','-0.010725195084431','6','86']
  H_HDV['49_NOx_2_50'] = [' 49','EURO0','RT >14-20t 80ties','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','46.6384148314168','0.0255111566745','-2.46359176740504','0.080397578887186','0','6','86']
  H_HDV['49_PM_2_50'] = [' 49','EURO0','RT >14-20t 80ties','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.507988096007631','2.55146698116573','4.85532143943349','1.85699737872077','0.023567333086023','6','86']
  H_HDV['49_FC_2_50'] = [' 49','EURO0','RT >14-20t 80ties','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.015471973593962','2.04843412906639','2624.68140997826','-0.563196430864357','0','6','86']
  H_HDV['49_CO_2_100'] = [' 49','EURO0','RT >14-20t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.0344657591993','0.009920412013419','-1.96032966529371','0.087267042387234','0','6','86']
  H_HDV['49_THC_2_100'] = [' 49','EURO0','RT >14-20t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.44586432303044','8.92001943449374','5.52312930212504','2.20426483533607','-0.013021052888989','6','86']
  H_HDV['49_NOx_2_100'] = [' 49','EURO0','RT >14-20t 80ties','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','47.9098838517722','0.016654862275472','-2.08030039544818','0.073561680263002','0','6','86']
  H_HDV['49_PM_2_100'] = [' 49','EURO0','RT >14-20t 80ties','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.16780583391771','0.000942260152746','-0.318848926817336','0.089207845630063','0','6','86']
  H_HDV['49_FC_2_100'] = [' 49','EURO0','RT >14-20t 80ties','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1372.16259699368','0.989846861702101','-79.3934126067753','0.077182007773813','0','6','86']
  H_HDV['49_CO_4_0'] = [' 49','EURO0','RT >14-20t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.92609432346416','14.6308271413172','7.84971980523266','3.08705888108697','-0.025338457407673','6','72']
  H_HDV['49_THC_4_0'] = [' 49','EURO0','RT >14-20t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.232165971288776','10.0275127309107','4.68222220813888','1.88078754430004','-0.009053684695013','6','72']
  H_HDV['49_NOx_4_0'] = [' 49','EURO0','RT >14-20t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','53.3279594338917','0.024694952139779','-3.45703063997374','0.097705410548261','0','6','72']
  H_HDV['49_PM_4_0'] = [' 49','EURO0','RT >14-20t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.458956452091708','2.53603272397503','6.24361043470583','2.53239299516256','-0.016703092888025','6','72']
  H_HDV['49_FC_4_0'] = [' 49','EURO0','RT >14-20t 80ties','4','0','FC','y=(c+(a*exp(b*x)))','1166.14890242822','-0.098270364944393','355.610751750956','0','0','6','72']
  H_HDV['49_CO_4_50'] = [' 49','EURO0','RT >14-20t 80ties','4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','4.88578414738952','60.0925054417767','-1.6629080566632','-0.794000168405565','0.184512626075912','6','71']
  H_HDV['49_THC_4_50'] = [' 49','EURO0','RT >14-20t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.409063752713395','9.43163763043677','5.54388682759081','2.21595102440224','-0.013166933092176','6','71']
  H_HDV['49_NOx_4_50'] = [' 49','EURO0','RT >14-20t 80ties','4','50','NOx','y=(c+(a*exp(b*x)))','31.9211009751042','-0.096980269627272','24.923250514292','0','0','6','71']
  H_HDV['49_PM_4_50'] = [' 49','EURO0','RT >14-20t 80ties','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.739845656847173','2.33861524956402','6.57850389504197','2.50512444888502','0.019642056625777','6','71']
  H_HDV['49_FC_4_50'] = [' 49','EURO0','RT >14-20t 80ties','4','50','FC','y=(c+(a*exp(b*x)))','1186.74082439416','-0.106404470669329','474.506447264549','0','0','6','71']
  H_HDV['49_CO_4_100'] = [' 49','EURO0','RT >14-20t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.5587983018266','0.011709697757564','-2.9349821504726','0.114502269887069','0','6','68']
  H_HDV['49_THC_4_100'] = [' 49','EURO0','RT >14-20t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.551404677056332','9.05817469495394','6.09612869195349','2.44457088948837','-0.016549822851318','6','68']
  H_HDV['49_NOx_4_100'] = [' 49','EURO0','RT >14-20t 80ties','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.763021536677','0.074211738861071','34.4629666440383','0.450859685937248','30.093279554089','6','68']
  H_HDV['49_PM_4_100'] = [' 49','EURO0','RT >14-20t 80ties','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000027929648255','0.004135574970714','-0.197131063753996','4.02408153613768','0','6','68']
  H_HDV['49_FC_4_100'] = [' 49','EURO0','RT >14-20t 80ties','4','100','FC','y=(c+(a*exp(b*x)))','1134.0761951672','-0.107018489274216','594.144452847358','0','0','6','68']
  H_HDV['49_CO_6_0'] = [' 49','EURO0','RT >14-20t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.2770270816584','0.038527603111737','-2.04572377782029','0.08285592367106','0','6','71']
  H_HDV['49_THC_6_0'] = [' 49','EURO0','RT >14-20t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.353396245496211','9.26309974687163','6.05837430799721','2.38355382820993','-0.018216645400735','6','71']
  H_HDV['49_NOx_6_0'] = [' 49','EURO0','RT >14-20t 80ties','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','57.9905782950052','0.020522383294355','-3.38847117477701','0.098064903310223','0','6','71']
  H_HDV['49_PM_6_0'] = [' 49','EURO0','RT >14-20t 80ties','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.738659405897639','8.27626460216151','-1.03295649156746','-0.488240532492104','0.158199337881589','6','71']
  H_HDV['49_FC_6_0'] = [' 49','EURO0','RT >14-20t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1517.46944184526','2.81549554843025','-87.7626004996216','0.072890750143172','0','6','71']
  H_HDV['49_CO_6_50'] = [' 49','EURO0','RT >14-20t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.2244788033179','0.113236462857274','-1.74791743598873','0.068079970043111','0','6','66']
  H_HDV['49_THC_6_50'] = [' 49','EURO0','RT >14-20t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.525153538351403','9.10023980050178','6.37789535391842','2.53873981374967','-0.01896267750765','6','66']
  H_HDV['49_NOx_6_50'] = [' 49','EURO0','RT >14-20t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.9385350570887','0.07019938474659','18.1206752279107','0.184903415920215','31.9739876033197','6','66']
  H_HDV['49_PM_6_50'] = [' 49','EURO0','RT >14-20t 80ties','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.53027868983199','0.014171555737875','-0.303901503733961','0.073130994883544','0','6','66']
  H_HDV['49_FC_6_50'] = [' 49','EURO0','RT >14-20t 80ties','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1709.02255918976','1.35909704369222','-104.205462233619','0.09050203948644','0','6','66']
  H_HDV['49_CO_6_100'] = [' 49','EURO0','RT >14-20t 80ties','6','100','CO','y=(c+(a*exp(b*x)))','25.7684552545638','-0.135274758327084','8.92711882170988','0','0','6','54']
  H_HDV['49_THC_6_100'] = [' 49','EURO0','RT >14-20t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.771065643784756','8.26057993458938','7.98663785388467','3.18761741970706','-0.033159962725927','6','54']
  H_HDV['49_NOx_6_100'] = [' 49','EURO0','RT >14-20t 80ties','6','100','NOx','y=((a*(b**x))*(x**c))','89.869639507138','1.00376762272026','-0.253446778758957','0','0','6','54']
  H_HDV['49_PM_6_100'] = [' 49','EURO0','RT >14-20t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.89388739911241','-0.000144064936539','-0.61366941696387','0.136118680117806','0','6','54']
  H_HDV['49_FC_6_100'] = [' 49','EURO0','RT >14-20t 80ties','6','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1934.09158271828','-0.623638722747254','-134.106969701431','0.120161468424223','0','6','54']
  H_HDV['50_CO_0_0'] = [' 50','EURO1','RT >14-20t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.7302612073357','0.054426753586341','5.07901308048506','0.188913686142935','0.908155492410233','6','86']
  H_HDV['50_THC_0_0'] = [' 50','EURO1','RT >14-20t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.202277173996146','3.41299262577991','3.71880142130913','1.6120238802044','-0.00599291843045','6','86']
  H_HDV['50_NOx_0_0'] = [' 50','EURO1','RT >14-20t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.2042639844008','0.079646893586987','33.2221922849948','0.495705544446516','5.11254363929227','6','86']
  H_HDV['50_PM_0_0'] = [' 50','EURO1','RT >14-20t Euro-1','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.08529099533829','1.6303532124215','4.91961233713793','2.06181747984211','-0.018007114067381','6','86']
  H_HDV['50_FC_0_0'] = [' 50','EURO1','RT >14-20t Euro-1','0','0','FC','y=((a*(x**b))+(c*(x**d)))','0.000120535838693','2.93531284828053','1720.37638223587','-0.630159604479976','0','6','86']
  H_HDV['50_CO_0_50'] = [' 50','EURO1','RT >14-20t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.98529056278109','0.055786222376323','7.55124831473753','0.251697402915709','1.0026257102683','6','86']
  H_HDV['50_THC_0_50'] = [' 50','EURO1','RT >14-20t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.204570947555763','3.41239014991898','3.73785937309701','1.62190168128156','-0.005495557826879','6','86']
  H_HDV['50_NOx_0_50'] = [' 50','EURO1','RT >14-20t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0156843032383','0.06408380130236','57.1349911728901','0.542638162910389','5.68546580615668','6','86']
  H_HDV['50_PM_0_50'] = [' 50','EURO1','RT >14-20t Euro-1','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.085911677149831','1.6982647935879','4.40157690513561','1.89116714378216','-0.016296343691378','6','86']
  H_HDV['50_FC_0_50'] = [' 50','EURO1','RT >14-20t Euro-1','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000961725116635','0.000138071349013','-0.000000964741688','','','6','86']
  H_HDV['50_CO_0_100'] = [' 50','EURO1','RT >14-20t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.93522056785631','0.053645954946913','9.50603028438802','0.280572764655454','1.13621991573986','6','86']
  H_HDV['50_THC_0_100'] = [' 50','EURO1','RT >14-20t Euro-1','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.193581911538785','3.39888404265866','3.775250606581','1.63878356131688','-0.006781341914296','6','86']
  H_HDV['50_NOx_0_100'] = [' 50','EURO1','RT >14-20t Euro-1','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.9059888666329','0.050672095674507','71.1900236774182','0.568048612546804','6.04269044543553','6','86']
  H_HDV['50_PM_0_100'] = [' 50','EURO1','RT >14-20t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.99427658307837','0.05334668902577','1.4594209715606','0.198992913254407','0.212979123165399','6','86']
  H_HDV['50_FC_0_100'] = [' 50','EURO1','RT >14-20t Euro-1','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001072172504245','0.000108846023858','-0.000000691101441','0','0','6','86']
  H_HDV['50_CO_-2_0'] = [' 50','EURO1','RT >14-20t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.31625924453773','0.010130914107366','-0.441669934874643','0.061938741352101','0','6','86']
  H_HDV['50_THC_-2_0'] = [' 50','EURO1','RT >14-20t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.31242362387485','2.55953243520581','4.18264353120545','1.48674767172742','0.034344632051505','6','86']
  H_HDV['50_NOx_-2_0'] = [' 50','EURO1','RT >14-20t Euro-1','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.0214658607783','0.076398264101877','958.271236728962','1.08569081988004','2.28053673086826','6','86']
  H_HDV['50_PM_-2_0'] = [' 50','EURO1','RT >14-20t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.183093879671394','45.6462055673164','-4.14552685283095','-0.701422262196923','0.129937451194745','6','86']
  H_HDV['50_FC_-2_0'] = [' 50','EURO1','RT >14-20t Euro-1','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','69.6204742050147','5132.96090270936','-1.3491794567695','0.448490988153318','0.056252988285574','6','86']
  H_HDV['50_CO_-2_50'] = [' 50','EURO1','RT >14-20t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.2536004877276','0.01327961291498','-0.423156862821097','0.057349488360767','0','6','86']
  H_HDV['50_THC_-2_50'] = [' 50','EURO1','RT >14-20t Euro-1','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.221816831780747','2.59788954948467','4.29348973149232','1.60048096496775','0.019951072223463','6','86']
  H_HDV['50_NOx_-2_50'] = [' 50','EURO1','RT >14-20t Euro-1','-2','50','NOx','y=(1/(a+(b*x)))','0.030952711658226','0.007078314428396','0','0','0','6','86']
  H_HDV['50_PM_-2_50'] = [' 50','EURO1','RT >14-20t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.16540126690353','35.961320636689','-3.82588055023247','-0.648304156927503','0.128078912832222','6','86']
  H_HDV['50_FC_-2_50'] = [' 50','EURO1','RT >14-20t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','450.272317830224','0.058326939644885','29500.9499544749','0.921607305932717','53.4244650919364','6','86']
  H_HDV['50_CO_-2_100'] = [' 50','EURO1','RT >14-20t Euro-1','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.48852381988771','0.009558060065602','-0.445485561878727','0.059619256091002','0','6','86']
  H_HDV['50_THC_-2_100'] = [' 50','EURO1','RT >14-20t Euro-1','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.38604060638297','-0.002095437118497','-0.240985715715481','0.0790237714368','0','6','86']
  H_HDV['50_NOx_-2_100'] = [' 50','EURO1','RT >14-20t Euro-1','-2','100','NOx','y=((a*(b**x))*(x**c))','35.0607065763774','0.986828639643531','-0.494722916698567','0','0','6','86']
  H_HDV['50_PM_-2_100'] = [' 50','EURO1','RT >14-20t Euro-1','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.141960338876576','20.1314582856204','-2.71585997392624','-0.303000073865636','0.101575281686515','6','86']
  H_HDV['50_FC_-2_100'] = [' 50','EURO1','RT >14-20t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','416.071581439313','0.046793412305886','24016.4192083104','0.863650346884968','39.0365694079635','6','86']
  H_HDV['50_CO_-4_0'] = [' 50','EURO1','RT >14-20t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.96207790488254','-0.002068014763375','-0.510337900234102','0.07773239941155','0','6','86']
  H_HDV['50_THC_-4_0'] = [' 50','EURO1','RT >14-20t Euro-1','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.30077003766862','-0.001903905133889','-0.249600904490124','0.082082369453689','0','6','86']
  H_HDV['50_NOx_-4_0'] = [' 50','EURO1','RT >14-20t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.9260851436275','0.059342422214324','52.4434374570461','0.451237718658686','0.455220228782747','6','86']
  H_HDV['50_PM_-4_0'] = [' 50','EURO1','RT >14-20t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.32963106041611','-10.9830176520351','-1.81846247050668','0','0','6','86']
  H_HDV['50_FC_-4_0'] = [' 50','EURO1','RT >14-20t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','387.284852472657','0.060032845024922','9892.10804549882','0.714948790960189','14.7587809915765','6','86']
  H_HDV['50_CO_-4_50'] = [' 50','EURO1','RT >14-20t Euro-1','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.89602343345124','-0.005578769353636','-0.543757857296561','0.085655615703314','0','6','86']
  H_HDV['50_THC_-4_50'] = [' 50','EURO1','RT >14-20t Euro-1','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.21023725764947','-0.002835802135452','-0.255076171143956','0.087227552119312','0','6','86']
  H_HDV['50_NOx_-4_50'] = [' 50','EURO1','RT >14-20t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1415587224611','0.052476841289532','91.0954995771074','0.524483240596911','0.277017799321395','6','86']
  H_HDV['50_PM_-4_50'] = [' 50','EURO1','RT >14-20t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.31620938883639','-10.7634712605469','-1.85108677846644','0','0','6','86']
  H_HDV['50_FC_-4_50'] = [' 50','EURO1','RT >14-20t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','332.3101174493','0.051721305752405','10669.3206826411','0.71005800623629','7.33584569978487','6','86']
  H_HDV['50_CO_-4_100'] = [' 50','EURO1','RT >14-20t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.4931739873185','0.056276195437869','19.8596586855343','0.472388995145507','0.13339866190822','6','86']
  H_HDV['50_THC_-4_100'] = [' 50','EURO1','RT >14-20t Euro-1','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.12587230118505','-0.002971230946656','-0.254237654397004','0.089585927260995','0','6','86']
  H_HDV['50_NOx_-4_100'] = [' 50','EURO1','RT >14-20t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.14110541886869','0.04781923700783','165.793561776474','0.614549348981564','0.194192927175926','6','86']
  H_HDV['50_PM_-4_100'] = [' 50','EURO1','RT >14-20t Euro-1','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.89840238096654','-9.33087226731073','-1.75658251942533','0','0','6','86']
  H_HDV['50_FC_-4_100'] = [' 50','EURO1','RT >14-20t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','304.79993469249','0.047091384476766','14974.4008104929','0.758699966517151','4.05640846567254','6','86']
  H_HDV['50_CO_-6_0'] = [' 50','EURO1','RT >14-20t Euro-1','-6','0','CO','y=(c+(a*exp(b*x)))','6.1593922268453','-0.078161225723006','0.12734162996393','0','0','6','72']
  H_HDV['50_THC_-6_0'] = [' 50','EURO1','RT >14-20t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.48879646659691','0.069958351240588','1.74528032191705','0.332765945362593','0.08046595758769','6','72']
  H_HDV['50_NOx_-6_0'] = [' 50','EURO1','RT >14-20t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.9586084417106','0.072301461167417','153.177367071676','0.667107783850675','0.214191602593103','6','72']
  H_HDV['50_PM_-6_0'] = [' 50','EURO1','RT >14-20t Euro-1','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-0.696564743048345','0.047359031272149','2.56637677040627','0.069140962294714','0.071132431699667','6','72']
  H_HDV['50_FC_-6_0'] = [' 50','EURO1','RT >14-20t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.082375236676','0.066623319292969','12899.5544092221','0.764895262479406','4.1344285609927','6','72']
  H_HDV['50_CO_-6_50'] = [' 50','EURO1','RT >14-20t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.52765199705702','0.06838389302729','21.095261035356','0.551381259137954','0.098486777654607','6','72']
  H_HDV['50_THC_-6_50'] = [' 50','EURO1','RT >14-20t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.17170323847372','0.067491978291203','4.73511807554122','0.433723229302386','0.075099357392972','6','72']
  H_HDV['50_NOx_-6_50'] = [' 50','EURO1','RT >14-20t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.41063787680407','0.066661211438863','226.988783343958','0.695140834766033','0.178609396309893','6','72']
  H_HDV['50_PM_-6_50'] = [' 50','EURO1','RT >14-20t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.29059397590053','-10.4105472668689','-1.91500802893744','0','0','6','72']
  H_HDV['50_FC_-6_50'] = [' 50','EURO1','RT >14-20t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','289.658043925526','0.062239774043327','15183.9078628535','0.777135787831931','3.01700016212463','6','72']
  H_HDV['50_CO_-6_100'] = [' 50','EURO1','RT >14-20t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.98048155904438','0.06402409697985','36.9979573171189','0.605563048642238','0.087694914569812','6','72']
  H_HDV['50_THC_-6_100'] = [' 50','EURO1','RT >14-20t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.98350152012117','0.065843127901572','7.66236872685075','0.492126545527774','0.072618422202004','6','72']
  H_HDV['50_NOx_-6_100'] = [' 50','EURO1','RT >14-20t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.09196666015506','0.062664608298585','288.91301437174','0.7218942175244','0.159929038711956','6','72']
  H_HDV['50_PM_-6_100'] = [' 50','EURO1','RT >14-20t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.29931026984701','0.072136926857066','0.775552044153521','0.217537851972209','0.049956938937873','6','72']
  H_HDV['50_FC_-6_100'] = [' 50','EURO1','RT >14-20t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','258.781493931902','0.059220675631018','17706.0376972786','0.798396781475097','2.47729636642435','6','72']
  H_HDV['50_CO_2_0'] = [' 50','EURO1','RT >14-20t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.60274145931663','0.004101307916556','-0.595749618452804','0.068543322033192','0','6','86']
  H_HDV['50_THC_2_0'] = [' 50','EURO1','RT >14-20t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.224857207413472','3.15773446602902','5.01226572236591','1.95746943944424','-0.00972976983397','6','86']
  H_HDV['50_NOx_2_0'] = [' 50','EURO1','RT >14-20t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.7723973675518','0.019087725605448','-1.95491978937614','0.09142814482364','0','6','86']
  H_HDV['50_PM_2_0'] = [' 50','EURO1','RT >14-20t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.255349493047879','20.9023558878471','-3.01496450030593','-0.444264103765082','0.11550499304623','6','86']
  H_HDV['50_FC_2_0'] = [' 50','EURO1','RT >14-20t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.245117185471925','1.39988946257777','2002.47447041489','-0.621020003102529','0','6','86']
  H_HDV['50_CO_2_50'] = [' 50','EURO1','RT >14-20t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.4032233126235','0.011904487714716','-0.676588550997418','0.071244383977187','0','6','86']
  H_HDV['50_THC_2_50'] = [' 50','EURO1','RT >14-20t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.235233038418461','2.95254338387858','6.43918969598569','2.50514775785798','-0.025737128922667','6','86']
  H_HDV['50_NOx_2_50'] = [' 50','EURO1','RT >14-20t Euro-1','2','50','NOx','y=(c+(a*exp(b*x)))','19.6928880041923','-0.096747134747502','10.4920020526617','0','0','6','86']
  H_HDV['50_PM_2_50'] = [' 50','EURO1','RT >14-20t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.9503415014003','0.002137790336155','-0.12180083654615','0.069129387345011','0','6','86']
  H_HDV['50_FC_2_50'] = [' 50','EURO1','RT >14-20t Euro-1','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.053152154217223','1.75625987959349','1765.88145423141','-0.503970836488957','0','6','86']
  H_HDV['50_CO_2_100'] = [' 50','EURO1','RT >14-20t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.1186171761574','0.008304229973273','-0.742969947191057','0.076990885663276','0','6','86']
  H_HDV['50_THC_2_100'] = [' 50','EURO1','RT >14-20t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.369215163926138','2.72814910288644','7.51566951240364','2.92349922925325','-0.031099733609031','6','86']
  H_HDV['50_NOx_2_100'] = [' 50','EURO1','RT >14-20t Euro-1','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.687083093326','-0.013165121301611','-1.63193165789426','0.092682238655038','0','6','86']
  H_HDV['50_PM_2_100'] = [' 50','EURO1','RT >14-20t Euro-1','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.93553163996621','0.002979227027742','-0.113177449029571','0.065883360309389','0','6','86']
  H_HDV['50_FC_2_100'] = [' 50','EURO1','RT >14-20t Euro-1','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','998.370860916134','0.935508223798173','-49.6571424616083','0.071845885028595','0','6','86']
  H_HDV['50_CO_4_0'] = [' 50','EURO1','RT >14-20t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.61735222052298','58.169503616878','-2.68143849748165','-0.754100304433543','0.153809034985967','6','72']
  H_HDV['50_THC_4_0'] = [' 50','EURO1','RT >14-20t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.321415618455583','3.05214108256311','6.13901900256712','2.37496533020313','-0.015756473470866','6','72']
  H_HDV['50_NOx_4_0'] = [' 50','EURO1','RT >14-20t Euro-1','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.2682667249755','0.01260971854011','-2.32292525872002','0.104577302808253','0','6','72']
  H_HDV['50_PM_4_0'] = [' 50','EURO1','RT >14-20t Euro-1','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-10.1830310975032','0.576248507002767','2.26083368198487','0.097737266555748','0.322839365197315','6','72']
  H_HDV['50_FC_4_0'] = [' 50','EURO1','RT >14-20t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','2245.0221837376','1.01158524845051','-0.64457255683828','0','0','6','72']
  H_HDV['50_CO_4_50'] = [' 50','EURO1','RT >14-20t Euro-1','4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.27114711453094','53.9147599780316','-2.90409175169084','-1.02669936317079','0.191291073910391','6','71']
  H_HDV['50_THC_4_50'] = [' 50','EURO1','RT >14-20t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.466517882570424','2.83284162401515','7.23646531279411','2.8134863417349','-0.021878909082501','6','71']
  H_HDV['50_NOx_4_50'] = [' 50','EURO1','RT >14-20t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.6642368385361','0.096564591499491','98022.0746449159','1.99462931225151','15.1532244993612','6','71']
  H_HDV['50_PM_4_50'] = [' 50','EURO1','RT >14-20t Euro-1','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.441826570818526','17.6027625076215','-3.66650913762074','-0.951783019815789','0.165991929294244','6','71']
  H_HDV['50_FC_4_50'] = [' 50','EURO1','RT >14-20t Euro-1','4','50','FC','y=((a*(b**x))*(x**c))','1993.19949446976','1.00920358299','-0.50944048610843','0','0','6','71']
  H_HDV['50_CO_4_100'] = [' 50','EURO1','RT >14-20t Euro-1','4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.24908575308143','7.34079007517552','5.6790873700762','2.35008841779481','-0.016423690797425','6','69']
  H_HDV['50_THC_4_100'] = [' 50','EURO1','RT >14-20t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.608275987296254','2.66770896764809','7.83739885424292','3.06639029168618','-0.025225101155953','6','69']
  H_HDV['50_NOx_4_100'] = [' 50','EURO1','RT >14-20t Euro-1','4','100','NOx','y=((a*(b**x))*(x**c))','49.3120693606597','1.0044882983996','-0.306893439660717','0','0','6','69']
  H_HDV['50_PM_4_100'] = [' 50','EURO1','RT >14-20t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000010192734768','0.001535393514753','-0.075457797299875','1.79552120663942','0','6','69']
  H_HDV['50_FC_4_100'] = [' 50','EURO1','RT >14-20t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','612.245067731183','0.091413586798239','18664.6787379315','0.960923941323304','534.318293883078','6','69']
  H_HDV['50_CO_6_0'] = [' 50','EURO1','RT >14-20t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.09163125149783','44.4342503820289','-3.24119501922094','-1.46037159682436','0.228256057137101','6','71']
  H_HDV['50_THC_6_0'] = [' 50','EURO1','RT >14-20t Euro-1','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.579549504222095','25.7032406084948','-2.05145114155679','-0.263684192640408','0.10604740045149','6','71']
  H_HDV['50_NOx_6_0'] = [' 50','EURO1','RT >14-20t Euro-1','6','0','NOx','y=(c+(a*exp(b*x)))','21.6396495555238','-0.114022542217925','14.9203952249571','0','0','6','71']
  H_HDV['50_PM_6_0'] = [' 50','EURO1','RT >14-20t Euro-1','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.425574541529065','16.3503274176081','-4.05743988559263','-1.30802935521715','0.197970683447118','6','71']
  H_HDV['50_FC_6_0'] = [' 50','EURO1','RT >14-20t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2248.71246088462','1.01217844821562','-0.589770547387835','0','0','6','71']
  H_HDV['50_CO_6_50'] = [' 50','EURO1','RT >14-20t Euro-1','6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.69439664890297','5.94230352218573','8.36378696346297','3.40095764140231','-0.031904298696708','6','67']
  H_HDV['50_THC_6_50'] = [' 50','EURO1','RT >14-20t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.719334476263718','22.0428762519787','-1.88308462263563','-0.269646988648116','0.113352849058713','6','67']
  H_HDV['50_NOx_6_50'] = [' 50','EURO1','RT >14-20t Euro-1','6','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','19.5645385381438','59.896334550728','-0.020791585598876','0.629486023948175','0.055207339281892','6','67']
  H_HDV['50_PM_6_50'] = [' 50','EURO1','RT >14-20t Euro-1','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.82357171630235','0.008460539541567','-0.08850531976862','0.054376627068028','0','6','67']
  H_HDV['50_FC_6_50'] = [' 50','EURO1','RT >14-20t Euro-1','6','50','FC','y=((a*(b**x))*(x**c))','1989.94970985914','1.00856163994857','-0.4310444251483','0','0','6','67']
  H_HDV['50_CO_6_100'] = [' 50','EURO1','RT >14-20t Euro-1','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.33360002613236','6.46775601509387','6.84284767090049','2.82095215208947','-0.024102582928008','6','55']
  H_HDV['50_THC_6_100'] = [' 50','EURO1','RT >14-20t Euro-1','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.06345193707473','-0.00356464818454','-0.399808326556068','0.098109437868518','0','6','55']
  H_HDV['50_NOx_6_100'] = [' 50','EURO1','RT >14-20t Euro-1','6','100','NOx','y=((a*(b**x))*(x**c))','55.971952611041','1.00446954019152','-0.267564634699188','0','0','6','55']
  H_HDV['50_PM_6_100'] = [' 50','EURO1','RT >14-20t Euro-1','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00001385999984','0.001753217242253','-0.074356082022389','1.85477616221739','0','6','55']
  H_HDV['50_FC_6_100'] = [' 50','EURO1','RT >14-20t Euro-1','6','100','FC','y=exp((a+(b/x))+(c*log(x)))','6.76809032507228','2.15118513727721','-0.063391213591264','0','0','6','55']
  H_HDV['51_CO_0_0'] = [' 51','EURO2','RT >14-20t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.3912435296971','0.060669484853257','4.02589270378647','0.220135265353309','0.795983278160224','6','86']
  H_HDV['51_THC_0_0'] = [' 51','EURO2','RT >14-20t Euro-2','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.01146877974539','0.043509461927831','1.77943337110804','0.153139292786697','0.213951904314124','6','86']
  H_HDV['51_NOx_0_0'] = [' 51','EURO2','RT >14-20t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.0165162646335','0.07298199767084','29.1865498505568','0.386265644975418','5.45992321923075','6','86']
  H_HDV['51_PM_0_0'] = [' 51','EURO2','RT >14-20t Euro-2','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.29841180118672','0.28167305256874','-0.00212838840783','0','0','6','86']
  H_HDV['51_FC_0_0'] = [' 51','EURO2','RT >14-20t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000966094473898','0.000181962634884','-0.000001380670632','0','0','6','86']
  H_HDV['51_CO_0_50'] = [' 51','EURO2','RT >14-20t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.086940659975356','0.023409454248663','-0.000144152588486','0','0','6','86']
  H_HDV['51_THC_0_50'] = [' 51','EURO2','RT >14-20t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.938046534338853','0.041666471532275','1.83847983712541','0.14466438903499','0.205113429300517','6','86']
  H_HDV['51_NOx_0_50'] = [' 51','EURO2','RT >14-20t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6783993756789','0.060192378381405','36.2683500897865','0.417463377278026','5.93888418107942','6','86']
  H_HDV['51_PM_0_50'] = [' 51','EURO2','RT >14-20t Euro-2','0','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.49931608681888','0.221709310948875','-0.001616525443489','0','0','6','86']
  H_HDV['51_FC_0_50'] = [' 51','EURO2','RT >14-20t Euro-2','0','50','FC','y=((a*(x**b))+(c*(x**d)))','0.000001824111607','3.80778224266752','1352.04871383928','-0.526820282752607','','6','86']
  H_HDV['51_CO_0_100'] = [' 51','EURO2','RT >14-20t Euro-2','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.091259377552717','0.02033385047413','-0.000127071238788','0','0','6','86']
  H_HDV['51_THC_0_100'] = [' 51','EURO2','RT >14-20t Euro-2','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108604082043792','2.46249846746038','3.26210422572881','1.496885884793','-0.004739564846013','6','86']
  H_HDV['51_NOx_0_100'] = [' 51','EURO2','RT >14-20t Euro-2','0','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','5.97369364536643','58.4143217743418','-0.098206494325773','0.523491440968504','0.030189411227852','6','86']
  H_HDV['51_PM_0_100'] = [' 51','EURO2','RT >14-20t Euro-2','0','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.440381507400656','0.003885500869012','-0.014094971883651','0.025543891549015','0','6','86']
  H_HDV['51_FC_0_100'] = [' 51','EURO2','RT >14-20t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','412.857480850015','0.051646859981','21568.3710968114','0.909345677779464','172.137340410014','6','86']
  H_HDV['51_CO_-2_0'] = [' 51','EURO2','RT >14-20t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.15115600546329','0.008591618394055','-0.311680077064578','0.061100182570471','0','6','86']
  H_HDV['51_THC_-2_0'] = [' 51','EURO2','RT >14-20t Euro-2','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.79295455850043','0.069548016932253','0.513877389475751','0.212099494258773','0.204439086152522','6','86']
  H_HDV['51_NOx_-2_0'] = [' 51','EURO2','RT >14-20t Euro-2','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.2138023094888','0.073962920194003','402.986222950148','0.908044475411095','2.47631102740453','6','86']
  H_HDV['51_PM_-2_0'] = [' 51','EURO2','RT >14-20t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.418893643678218','0.000747977004771','-0.022605076415248','0.054679626946478','0','6','86']
  H_HDV['51_FC_-2_0'] = [' 51','EURO2','RT >14-20t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','451.646651734244','0.072667315193429','36014.1273256351','1.00544155351813','65.7468906481213','6','86']
  H_HDV['51_CO_-2_50'] = [' 51','EURO2','RT >14-20t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.16198857159499','0.011022697737859','-0.296223472689833','0.055814931128901','0','6','86']
  H_HDV['51_THC_-2_50'] = [' 51','EURO2','RT >14-20t Euro-2','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.148105862712697','2.30652665845903','2.33061781525965','0.997740898882025','0.033205460677003','6','86']
  H_HDV['51_NOx_-2_50'] = [' 51','EURO2','RT >14-20t Euro-2','-2','50','NOx','y=(1/(a+(b*x)))','0.027743751966061','0.006484073139131','0','0','0','6','86']
  H_HDV['51_PM_-2_50'] = [' 51','EURO2','RT >14-20t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.414158073428415','0.001126386145667','-0.020157340222591','0.046636820199776','0','6','86']
  H_HDV['51_FC_-2_50'] = [' 51','EURO2','RT >14-20t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','398.684292446057','0.055711856123266','22062.8446848037','0.888851567024415','49.6711370355381','6','86']
  H_HDV['51_CO_-2_100'] = [' 51','EURO2','RT >14-20t Euro-2','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.36884392762045','0.008678096504696','-0.307535868186527','0.056947380222097','0','6','86']
  H_HDV['51_THC_-2_100'] = [' 51','EURO2','RT >14-20t Euro-2','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.584089439587323','0.032845414446309','1.62925838018103','0.100340427113896','0.096438583730543','6','86']
  H_HDV['51_NOx_-2_100'] = [' 51','EURO2','RT >14-20t Euro-2','-2','100','NOx','y=((a*(b**x))*(x**c))','38.5434436859282','0.986536835834741','-0.495475490599511','0','0','6','86']
  H_HDV['51_PM_-2_100'] = [' 51','EURO2','RT >14-20t Euro-2','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.442071573504236','0.000982216841124','-0.021296818973','0.047409681848261','0','6','86']
  H_HDV['51_FC_-2_100'] = [' 51','EURO2','RT >14-20t Euro-2','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','371.082068076451','0.04416261978508','17973.1360882351','0.834078144553058','35.7039238534593','6','86']
  H_HDV['51_CO_-4_0'] = [' 51','EURO2','RT >14-20t Euro-2','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.87886220564301','-0.001536765039316','-0.361113018484129','0.078827977803098','0','6','86']
  H_HDV['51_THC_-4_0'] = [' 51','EURO2','RT >14-20t Euro-2','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.14604440991769','-0.00146333020194','-0.160023034354131','0.082258854840024','0','6','86']
  H_HDV['51_NOx_-4_0'] = [' 51','EURO2','RT >14-20t Euro-2','-4','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.03314195731806','0.006069912385089','0.000183470414422','0','0','6','86']
  H_HDV['51_PM_-4_0'] = [' 51','EURO2','RT >14-20t Euro-2','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.07504838187991','-6.22043976280278','-1.28313765461596','0','0','6','86']
  H_HDV['51_FC_-4_0'] = [' 51','EURO2','RT >14-20t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','347.756938341555','0.059293524934116','7580.446672551','0.691828512859757','14.4728369643582','6','86']
  H_HDV['51_CO_-4_50'] = [' 51','EURO2','RT >14-20t Euro-2','-4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.173283777145445','0.019728373034962','0.000789288357399','0','0','6','86']
  H_HDV['51_THC_-4_50'] = [' 51','EURO2','RT >14-20t Euro-2','-4','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.370863095925964','0.053986027692477','0.001220265788877','0','0','6','86']
  H_HDV['51_NOx_-4_50'] = [' 51','EURO2','RT >14-20t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.9465328266539','0.051977549191916','76.1882352451816','0.477057054610477','0.310286087944407','6','86']
  H_HDV['51_PM_-4_50'] = [' 51','EURO2','RT >14-20t Euro-2','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','1.85583372936517','-5.42421715306464','-1.22502182048518','0','0','6','86']
  H_HDV['51_FC_-4_50'] = [' 51','EURO2','RT >14-20t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','298.489454085784','0.050532845510476','8445.44275437039','0.689475870606618','7.29513048458471','6','86']
  H_HDV['51_CO_-4_100'] = [' 51','EURO2','RT >14-20t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.97189008684981','0.051924691021275','24.0034015022112','0.521124184483282','0.100038144430323','6','86']
  H_HDV['51_THC_-4_100'] = [' 51','EURO2','RT >14-20t Euro-2','-4','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.373189369535802','0.056695014061084','0.001254307270802','0','0','6','86']
  H_HDV['51_NOx_-4_100'] = [' 51','EURO2','RT >14-20t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.98242340211491','0.048008900427138','141.107555611942','0.57514106890981','0.227811733423469','6','86']
  H_HDV['51_PM_-4_100'] = [' 51','EURO2','RT >14-20t Euro-2','-4','100','PM','y=((a*(b**x))*(x**c))','1.00409124959224','0.988980054120579','-0.615689103407723','0','0','6','86']
  H_HDV['51_FC_-4_100'] = [' 51','EURO2','RT >14-20t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','276.714430997495','0.045970105208661','12173.2919512943','0.744781139491034','4.15226333683227','6','86']
  H_HDV['51_CO_-6_0'] = [' 51','EURO2','RT >14-20t Euro-2','-6','0','CO','y=(c+(a*exp(b*x)))','4.39036218544246','-0.079055673886082','0.108122663680484','0','0','6','72']
  H_HDV['51_THC_-6_0'] = [' 51','EURO2','RT >14-20t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51062091370003','0.064304985486653','1.83415705091292','0.355408686520355','0.068468596274908','6','72']
  H_HDV['51_NOx_-6_0'] = [' 51','EURO2','RT >14-20t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2356187322827','0.072791165657338','116.769737283109','0.610664419840886','0.253749201914365','6','72']
  H_HDV['51_PM_-6_0'] = [' 51','EURO2','RT >14-20t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012682492820114','0.30536571861155','5.96036482716024','2.3096906950968','-0.01575393931703','6','72']
  H_HDV['51_FC_-6_0'] = [' 51','EURO2','RT >14-20t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','316.119474165273','0.066491609792124','9804.2309634371','0.741874374996292','4.68989976890659','6','72']
  H_HDV['51_CO_-6_50'] = [' 51','EURO2','RT >14-20t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.17334817288134','0.067741978945896','14.6487346311255','0.526018913729212','0.087287010605293','6','72']
  H_HDV['51_THC_-6_50'] = [' 51','EURO2','RT >14-20t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3784289736524','0.062625051230218','2.73019394833314','0.393953769465921','0.066057038867283','6','72']
  H_HDV['51_NOx_-6_50'] = [' 51','EURO2','RT >14-20t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4444612333519','0.067497429439159','203.556020046404','0.668116362878365','0.216405133770439','6','72']
  H_HDV['51_PM_-6_50'] = [' 51','EURO2','RT >14-20t Euro-2','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','1.70781928406717','-4.86446852009658','-1.21667796778829','0','0','6','72']
  H_HDV['51_FC_-6_50'] = [' 51','EURO2','RT >14-20t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','263.407136390093','0.061951472938902','12036.9552423078','0.761106684024336','3.61486843997099','6','72']
  H_HDV['51_CO_-6_100'] = [' 51','EURO2','RT >14-20t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.7701222030547','0.061735093524524','29.4606922685858','0.60743421643618','0.075902162506674','6','72']
  H_HDV['51_THC_-6_100'] = [' 51','EURO2','RT >14-20t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.30882849662284','0.061871490780663','3.4724727957967','0.4232710023908','0.065283513341873','6','72']
  H_HDV['51_NOx_-6_100'] = [' 51','EURO2','RT >14-20t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.92994303000469','0.0633950786896','275.387792418027','0.704752741342212','0.194962983360599','6','72']
  H_HDV['51_PM_-6_100'] = [' 51','EURO2','RT >14-20t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.243075414814474','0.05062437159746','0.334383591248972','0.242340765467407','0.024177646743838','6','72']
  H_HDV['51_FC_-6_100'] = [' 51','EURO2','RT >14-20t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','236.55560744807','0.058962623430298','14869.8471682883','0.79252541102569','3.1320775248667','6','72']
  H_HDV['51_CO_2_0'] = [' 51','EURO2','RT >14-20t Euro-2','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.24146841968041','19.760658738675','-1.31527549875652','-0.285507877179015','0.120124079734365','6','86']
  H_HDV['51_THC_2_0'] = [' 51','EURO2','RT >14-20t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.119209557123196','2.19714810355616','4.49087563044247','1.8002914140728','-0.008102435187921','6','86']
  H_HDV['51_NOx_2_0'] = [' 51','EURO2','RT >14-20t Euro-2','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.7405403931341','0.007797744815486','-2.17233810077466','0.092891720097789','0','6','86']
  H_HDV['51_PM_2_0'] = [' 51','EURO2','RT >14-20t Euro-2','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.523305596933844','0.000753820061099','-0.028987826185023','0.068626280876834','0','6','86']
  H_HDV['51_FC_2_0'] = [' 51','EURO2','RT >14-20t Euro-2','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.07359023293341','1.65483243669564','1652.55929341309','-0.567286975717122','0','6','86']
  H_HDV['51_CO_2_50'] = [' 51','EURO2','RT >14-20t Euro-2','2','50','CO','y=(c+(a*exp(b*x)))','6.30401486489512','-0.08625926936167','1.48680861539068','0','0','6','86']
  H_HDV['51_THC_2_50'] = [' 51','EURO2','RT >14-20t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.162879216346884','2.00370488483104','5.69115800769587','2.23184500004326','-0.015701817165042','6','86']
  H_HDV['51_NOx_2_50'] = [' 51','EURO2','RT >14-20t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.2018676931616','-0.009826100267595','-2.09162792639081','0.095170619723381','0','6','86']
  H_HDV['51_PM_2_50'] = [' 51','EURO2','RT >14-20t Euro-2','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.57586484775867','0.140778540262371','-0.00115328646376','0','0','6','86']
  H_HDV['51_FC_2_50'] = [' 51','EURO2','RT >14-20t Euro-2','2','50','FC','y=((a*(b**x))*(x**c))','1808.16221284912','1.01032478571463','-0.595944603339086','0','0','6','86']
  H_HDV['51_CO_2_100'] = [' 51','EURO2','RT >14-20t Euro-2','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.089551479053385','0.014054196180409','-0.000091841471018','0','0','6','86']
  H_HDV['51_THC_2_100'] = [' 51','EURO2','RT >14-20t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.21726715503202','1.88025745270722','6.5728654787331','2.57224080244218','-0.021429674765665','6','86']
  H_HDV['51_NOx_2_100'] = [' 51','EURO2','RT >14-20t Euro-2','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.0963290119831','-0.022030281063195','-1.86586386877301','0.090919808114227','0','6','86']
  H_HDV['51_PM_2_100'] = [' 51','EURO2','RT >14-20t Euro-2','2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.43289174677047','0.115538914180542','-0.000917765188008','0','0','6','86']
  H_HDV['51_FC_2_100'] = [' 51','EURO2','RT >14-20t Euro-2','2','100','FC','y=((a*(b**x))*(x**c))','1673.78475378889','1.0084717967614','-0.502138908395066','0','0','6','86']
  H_HDV['51_CO_4_0'] = [' 51','EURO2','RT >14-20t Euro-2','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.44801819880979','4.30903945623886','6.99406777705322','2.79890424127268','-0.020305153645196','6','72']
  H_HDV['51_THC_4_0'] = [' 51','EURO2','RT >14-20t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.180959014365119','2.09427587113818','5.66091021543877','2.22341324449248','-0.013817114832817','6','72']
  H_HDV['51_NOx_4_0'] = [' 51','EURO2','RT >14-20t Euro-2','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.9479395141368','-0.00613182279211','-2.64015418334278','0.106744031832405','0','6','72']
  H_HDV['51_PM_4_0'] = [' 51','EURO2','RT >14-20t Euro-2','4','0','PM','y=(c+(a*exp(b*x)))','0.399977372469478','-0.083813270085603','0.185962724035342','0','0','6','72']
  H_HDV['51_FC_4_0'] = [' 51','EURO2','RT >14-20t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1908.00446160114','1.01111673185272','-0.602584409696644','0','0','6','72']
  H_HDV['51_CO_4_50'] = [' 51','EURO2','RT >14-20t Euro-2','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.48330899339417','-0.016656269898128','-0.78461033943484','0.11457384623603','0','6','71']
  H_HDV['51_THC_4_50'] = [' 51','EURO2','RT >14-20t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.271322312469558','1.9301417410697','6.79374610852408','2.66739980085451','-0.019776949151844','6','71']
  H_HDV['51_NOx_4_50'] = [' 51','EURO2','RT >14-20t Euro-2','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','14.7003467712983','34.7049045799656','1.42592935870827','0.900552266400452','0.036883365232856','6','71']
  H_HDV['51_PM_4_50'] = [' 51','EURO2','RT >14-20t Euro-2','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.53636201944852','0.086077816199604','-0.000568797312849','0','0','6','71']
  H_HDV['51_FC_4_50'] = [' 51','EURO2','RT >14-20t Euro-2','4','50','FC','y=((a*(b**x))*(x**c))','1704.38429316724','1.00856905088582','-0.465140175744999','0','0','6','71']
  H_HDV['51_CO_4_100'] = [' 51','EURO2','RT >14-20t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.39713899532567','0.026344431577885','6.07020953075924','0.167503704258421','0.863011229350019','6','70']
  H_HDV['51_THC_4_100'] = [' 51','EURO2','RT >14-20t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.340010900602955','1.88510446349914','6.83487837043421','2.71650001276494','-0.020004564568541','6','70']
  H_HDV['51_NOx_4_100'] = [' 51','EURO2','RT >14-20t Euro-2','4','100','NOx','y=((a*(b**x))*(x**c))','58.6339484731953','1.00441662731189','-0.353014536892908','0','0','6','70']
  H_HDV['51_PM_4_100'] = [' 51','EURO2','RT >14-20t Euro-2','4','100','PM','y=(1/(a+(b*(x**c))))','1.05111575338835','0.215560944792886','0.654584590627282','0','0','6','70']
  H_HDV['51_FC_4_100'] = [' 51','EURO2','RT >14-20t Euro-2','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003997943694379','0.600787080287437','-29.5394299785865','1011.43989569163','0','6','70']
  H_HDV['51_CO_6_0'] = [' 51','EURO2','RT >14-20t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.3180837721693','-0.025936531025602','-1.79013028658069','0.198380536673189','0','6','71']
  H_HDV['51_THC_6_0'] = [' 51','EURO2','RT >14-20t Euro-2','6','0','THC','y=(c+(a*exp(b*x)))','2.82240852081899','-0.082202557474987','0.339509537476599','0','0','6','71']
  H_HDV['51_NOx_6_0'] = [' 51','EURO2','RT >14-20t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.6871017429687','-0.021196443660835','-2.82831075581391','0.114462678735125','0','6','71']
  H_HDV['51_PM_6_0'] = [' 51','EURO2','RT >14-20t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.834566040216461','-0.002057084956061','-0.096608970125968','0.192837951241142','0','6','71']
  H_HDV['51_FC_6_0'] = [' 51','EURO2','RT >14-20t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','1922.96563725879','1.01166082852353','-0.547708429483632','0','0','6','71']
  H_HDV['51_CO_6_50'] = [' 51','EURO2','RT >14-20t Euro-2','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.7594678641873','-0.038179797471321','-1.54950040872446','0.175646232019845','0','6','70']
  H_HDV['51_THC_6_50'] = [' 51','EURO2','RT >14-20t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.448329381560592','12.8352848921447','-1.85663501739832','-0.383266278798323','0.128006114159974','6','70']
  H_HDV['51_NOx_6_50'] = [' 51','EURO2','RT >14-20t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.6428245563545','-0.047754803579695','-2.82722987523416','0.117035836524222','0','6','70']
  H_HDV['51_PM_6_50'] = [' 51','EURO2','RT >14-20t Euro-2','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.813453731533008','-0.002139635345177','-0.052618937711469','0.118051451441609','0','6','70']
  H_HDV['51_FC_6_50'] = [' 51','EURO2','RT >14-20t Euro-2','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1093.98618362197','2.56888247324062','-42.095395704429','0.064514319361686','0','6','70']
  H_HDV['51_CO_6_100'] = [' 51','EURO2','RT >14-20t Euro-2','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.4956970364202','-0.028622406077219','-0.757285732544743','0.09137308894733','0','6','60']
  H_HDV['51_THC_6_100'] = [' 51','EURO2','RT >14-20t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.468886298748806','1.71123969291837','8.55655972405536','3.44987992518113','-0.040579138711891','6','60']
  H_HDV['51_NOx_6_100'] = [' 51','EURO2','RT >14-20t Euro-2','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','50.3489520583985','-0.057817473711673','-2.71596157537301','0.111306446390166','0','6','60']
  H_HDV['51_PM_6_100'] = [' 51','EURO2','RT >14-20t Euro-2','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.843745240112224','0.001397941819172','-0.029698594168264','0.047005355228442','0','6','60']
  H_HDV['51_FC_6_100'] = [' 51','EURO2','RT >14-20t Euro-2','6','100','FC','y=((a*(b**x))*(x**c))','1710.95201090126','1.00600770875316','-0.299660642376204','0','0','6','60']
  H_HDV['52_CO_0_0'] = [' 52','EURO3','RT >14-20t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.56864398786746','0.053731425720334','6.38479997321685','0.170659097707132','0.843046444625228','6','86']
  H_HDV['52_THC_0_0'] = [' 52','EURO3','RT >14-20t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.100136735763364','2.14541219802166','3.99898735765331','1.72253787463531','-0.0088288834339','6','86']
  H_HDV['52_NOx_0_0'] = [' 52','EURO3','RT >14-20t Euro-3','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.91191969806057','320.176916943352','-1.17977366970143','0.902310681361423','0.024854311549025','6','86']
  H_HDV['52_PM_0_0'] = [' 52','EURO3','RT >14-20t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.060067978290845','0.651467583577212','5.54522984543755','2.24658628924816','-0.016443637424292','6','86']
  H_HDV['52_FC_0_0'] = [' 52','EURO3','RT >14-20t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000802091086573','0.000173689949157','-0.000001284563732','0','0','6','86']
  H_HDV['52_CO_0_50'] = [' 52','EURO3','RT >14-20t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.14845646262149','0.056455573267777','6.88088842498904','0.211155742913391','0.957653877728337','6','86']
  H_HDV['52_THC_0_50'] = [' 52','EURO3','RT >14-20t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.101638837335944','2.05914742570242','4.20388927706735','1.79779081868679','-0.010183111257285','6','86']
  H_HDV['52_NOx_0_50'] = [' 52','EURO3','RT >14-20t Euro-3','0','50','NOx','y=((a*(b**x))*(x**c))','78.9645941515278','1.00750501530015','-0.784096020910954','0','0','6','86']
  H_HDV['52_PM_0_50'] = [' 52','EURO3','RT >14-20t Euro-3','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04084028599178','0.671119256560045','4.94846641382394','2.03450703421729','-0.01879645759595','6','86']
  H_HDV['52_FC_0_50'] = [' 52','EURO3','RT >14-20t Euro-3','0','50','FC','y=((a*(x**b))+(c*(x**d)))','0.00001022171071','3.42037495433072','1550.25769228117','-0.552440613454744','','6','86']
  H_HDV['52_CO_0_100'] = [' 52','EURO3','RT >14-20t Euro-3','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.06316354743098','78.9490703695933','-1.47315378721495','0.457284963941233','0.044401742483504','6','86']
  H_HDV['52_THC_0_100'] = [' 52','EURO3','RT >14-20t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.107847132087605','2.02722580219844','4.17072613740268','1.78957046479833','-0.009172870334407','6','86']
  H_HDV['52_NOx_0_100'] = [' 52','EURO3','RT >14-20t Euro-3','0','100','NOx','y=((a*(b**x))*(x**c))','69.1418828465997','1.00495159068741','-0.691668414256334','0','0','6','86']
  H_HDV['52_PM_0_100'] = [' 52','EURO3','RT >14-20t Euro-3','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.024534016165735','0.835902406358908','3.58622999321155','1.54412024891334','-0.01794675470419','6','86']
  H_HDV['52_FC_0_100'] = [' 52','EURO3','RT >14-20t Euro-3','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001113395412303','0.000105701527391','-0.000000655813643','0','0','6','86']
  H_HDV['52_CO_-2_0'] = [' 52','EURO3','RT >14-20t Euro-3','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.915104491156481','164.632102381035','-4.03050321532166','-0.749098780768571','0.132003620647443','6','86']
  H_HDV['52_THC_-2_0'] = [' 52','EURO3','RT >14-20t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.209494036615909','9.62568048053555','-1.572841896222','-0.140557436786957','0.091102652943777','6','86']
  H_HDV['52_NOx_-2_0'] = [' 52','EURO3','RT >14-20t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.857176036737','0.077664288525003','112.660773285815','0.562774379081468','2.07106946050302','6','86']
  H_HDV['52_PM_-2_0'] = [' 52','EURO3','RT >14-20t Euro-3','-2','0','PM','y=(c+(a*exp(b*x)))','0.803277397661988','-0.069097574755749','0.079334715084501','0','0','6','86']
  H_HDV['52_FC_-2_0'] = [' 52','EURO3','RT >14-20t Euro-3','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','69.9250484840492','2532.86658674823','-0.602548883204866','0.401195112709677','0.057116092271062','6','86']
  H_HDV['52_CO_-2_50'] = [' 52','EURO3','RT >14-20t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.30735687188761','0.018585447229079','-0.472918934531916','0.054997932539258','0','6','86']
  H_HDV['52_THC_-2_50'] = [' 52','EURO3','RT >14-20t Euro-3','-2','50','THC','y=(c+(a*exp(b*x)))','1.99027194200904','-0.073647091115419','0.162804129921598','0','0','6','86']
  H_HDV['52_NOx_-2_50'] = [' 52','EURO3','RT >14-20t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.9697546673272','0.0594900287842','85.0752483539412','0.464048683246777','1.50570346827146','6','86']
  H_HDV['52_PM_-2_50'] = [' 52','EURO3','RT >14-20t Euro-3','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-0.359464898811253','0.045397621760573','1.10641648819502','0.060733457259742','0.080516423554165','6','86']
  H_HDV['52_FC_-2_50'] = [' 52','EURO3','RT >14-20t Euro-3','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','469.754285373228','0.05727256584948','7337.15467177706','0.728544677241463','54.769278191833','6','86']
  H_HDV['52_CO_-2_100'] = [' 52','EURO3','RT >14-20t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.35234032143041','0.015737109754803','-0.47549432160593','0.055469293665904','0','6','86']
  H_HDV['52_THC_-2_100'] = [' 52','EURO3','RT >14-20t Euro-3','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.060549329012744','1.4341860405959','6.64858436771751','2.50883711806873','-0.017101357550268','6','86']
  H_HDV['52_NOx_-2_100'] = [' 52','EURO3','RT >14-20t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1135621457465','0.046984537545578','92.3735721767925','0.456046427739592','1.03560522790172','6','86']
  H_HDV['52_PM_-2_100'] = [' 52','EURO3','RT >14-20t Euro-3','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-0.173544975669265','0.051389282674848','0.915989452548301','0.065061645009515','0.069073846184147','6','86']
  H_HDV['52_FC_-2_100'] = [' 52','EURO3','RT >14-20t Euro-3','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','421.85389910627','0.045553747049069','6608.81542678661','0.670977342895839','40.0537261137938','6','86']
  H_HDV['52_CO_-4_0'] = [' 52','EURO3','RT >14-20t Euro-3','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.446163471888402','159.919131572526','-3.63101633054777','-0.465680565102457','0.106646796842727','6','86']
  H_HDV['52_THC_-4_0'] = [' 52','EURO3','RT >14-20t Euro-3','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.035244775147299','1.43359390763979','6.76856474153385','2.5522565213287','-0.015934002345738','6','86']
  H_HDV['52_NOx_-4_0'] = [' 52','EURO3','RT >14-20t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.9490605679665','0.065724864510611','71.6293989295148','0.431547045028665','0.460717284779518','6','86']
  H_HDV['52_PM_-4_0'] = [' 52','EURO3','RT >14-20t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.85233589083776','-9.64624712654694','-1.56648572354687','0','0','6','86']
  H_HDV['52_FC_-4_0'] = [' 52','EURO3','RT >14-20t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','406.491253444224','0.058629504426919','1931.8033248708','0.467205912661888','18.7644110602225','6','86']
  H_HDV['52_CO_-4_50'] = [' 52','EURO3','RT >14-20t Euro-3','-4','50','CO','y=exp((a+(b/x))+(c*log(x)))','7.01247993355123','-12.0394250570541','-1.88847594679204','0','0','6','86']
  H_HDV['52_THC_-4_50'] = [' 52','EURO3','RT >14-20t Euro-3','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018147782353553','1.40947824695121','5.81845419903586','2.23736905391131','-0.010543679487591','6','86']
  H_HDV['52_NOx_-4_50'] = [' 52','EURO3','RT >14-20t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.0826559146112','0.062257555217372','122.729532968529','0.518899640159628','0.426700413576731','6','86']
  H_HDV['52_PM_-4_50'] = [' 52','EURO3','RT >14-20t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.65849556463301','-9.08830190785868','-1.54280798121904','0','0','6','86']
  H_HDV['52_FC_-4_50'] = [' 52','EURO3','RT >14-20t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','349.72978458732','0.05168592703862','3027.53336457715','0.520703387961842','12.0704876922271','6','86']
  H_HDV['52_CO_-4_100'] = [' 52','EURO3','RT >14-20t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.95935522703355','-0.00653931836638','-0.596435444253758','0.081662143151356','0','6','86']
  H_HDV['52_THC_-4_100'] = [' 52','EURO3','RT >14-20t Euro-3','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.86856815825363','-0.001268554706137','-0.146213182878142','0.085188398829568','0','6','86']
  H_HDV['52_NOx_-4_100'] = [' 52','EURO3','RT >14-20t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.7701218522884','0.057990228119401','224.99543287822','0.617183683507192','0.374837852725053','6','86']
  H_HDV['52_PM_-4_100'] = [' 52','EURO3','RT >14-20t Euro-3','-4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024845549074192','0.545901841545641','6.71275667032563','2.52032836665526','-0.016414365882308','6','86']
  H_HDV['52_FC_-4_100'] = [' 52','EURO3','RT >14-20t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','320.514568328702','0.047427890010567','5731.38420487093','0.618179881468684','9.26167643896082','6','86']
  H_HDV['52_CO_-6_0'] = [' 52','EURO3','RT >14-20t Euro-3','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.889807629117574','0.075482952178858','6.69273736029335','0.074015907366267','0.183111869602547','6','72']
  H_HDV['52_THC_-6_0'] = [' 52','EURO3','RT >14-20t Euro-3','-6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.050387509650994','1.362864554435','5.29276062823956','1.96373449748296','0.013998443593133','6','72']
  H_HDV['52_NOx_-6_0'] = [' 52','EURO3','RT >14-20t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3136412201208','0.080670940283841','132.849053563771','0.557059014038298','0.335857683790318','6','72']
  H_HDV['52_PM_-6_0'] = [' 52','EURO3','RT >14-20t Euro-3','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-0.171985864384409','0.039985788407216','0.948486219042007','0.064668558373092','0.053820895432731','6','72']
  H_HDV['52_FC_-6_0'] = [' 52','EURO3','RT >14-20t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','407.413613045465','0.069762239600913','4471.38878600777','0.639856922726259','11.3446679696212','6','72']
  H_HDV['52_CO_-6_50'] = [' 52','EURO3','RT >14-20t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','6.75705425975521','-0.074054813487117','0.177940744331662','0','0','6','72']
  H_HDV['52_THC_-6_50'] = [' 52','EURO3','RT >14-20t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3504268044491','0.071131702223976','0.87623001316121','0.28316777373665','0.06284531648516','6','72']
  H_HDV['52_NOx_-6_50'] = [' 52','EURO3','RT >14-20t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.80156653780509','0.073101478086675','235.618514516701','0.628832422589862','0.289815682666398','6','72']
  H_HDV['52_PM_-6_50'] = [' 52','EURO3','RT >14-20t Euro-3','-6','50','PM','y=(c+(a*exp(((-1)*b)*x)))','0.706826948012929','0.071875326222832','0.04739566920851','0','0','6','72']
  H_HDV['52_FC_-6_50'] = [' 52','EURO3','RT >14-20t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','329.768829998197','0.064613675127032','7611.41174817059','0.698599251344044','10.0934637587762','6','72']
  H_HDV['52_CO_-6_100'] = [' 52','EURO3','RT >14-20t Euro-3','-6','100','CO','y=(c+(a*exp(((-1)*b)*x)))','6.01099635192766','0.071524535081272','0.172904132045148','0','0','6','72']
  H_HDV['52_THC_-6_100'] = [' 52','EURO3','RT >14-20t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.17578070025014','0.06882602036941','3.33736712762793','0.472063798408193','0.060734854271479','6','72']
  H_HDV['52_NOx_-6_100'] = [' 52','EURO3','RT >14-20t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.92027950174209','0.066943254328929','327.704612580387','0.67705110961894','0.266215373147423','6','72']
  H_HDV['52_PM_-6_100'] = [' 52','EURO3','RT >14-20t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015832890791232','0.498227709169824','6.20886837401618','2.35513234614301','-0.017073931920136','6','72']
  H_HDV['52_FC_-6_100'] = [' 52','EURO3','RT >14-20t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','284.706238443906','0.060259270447229','10608.618897729','0.74187956678366','9.22141810685838','6','72']
  H_HDV['52_CO_2_0'] = [' 52','EURO3','RT >14-20t Euro-3','2','0','CO','y=(c+(a*exp(b*x)))','8.60178610955384','-0.078529474765086','1.12622046051934','0','0','6','86']
  H_HDV['52_THC_2_0'] = [' 52','EURO3','RT >14-20t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.129248853499961','1.88761444392634','5.46410107158444','2.12152662047558','-0.011324214077914','6','86']
  H_HDV['52_NOx_2_0'] = [' 52','EURO3','RT >14-20t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','39.6530698120204','-0.016149834563243','-4.02775876508073','0.126160846806105','0','6','86']
  H_HDV['52_PM_2_0'] = [' 52','EURO3','RT >14-20t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.115511764757603','13.2416514704408','-3.40814849916752','-0.389956572609253','0.104546047810637','6','86']
  H_HDV['52_FC_2_0'] = [' 52','EURO3','RT >14-20t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','909.003420200577','0.851981991524133','-57.4921357968307','0.080029284059858','0','6','86']
  H_HDV['52_CO_2_50'] = [' 52','EURO3','RT >14-20t Euro-3','2','50','CO','y=(c+(a*exp(b*x)))','8.66589122226809','-0.079067208362045','1.35125065295133','0','0','6','86']
  H_HDV['52_THC_2_50'] = [' 52','EURO3','RT >14-20t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.159516845638168','1.78660348230707','6.12338917636475','2.36718777426181','-0.014553582436239','6','86']
  H_HDV['52_NOx_2_50'] = [' 52','EURO3','RT >14-20t Euro-3','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','41.5197080829103','-0.026291397809524','-4.07234738598678','0.128172552037565','0','6','86']
  H_HDV['52_PM_2_50'] = [' 52','EURO3','RT >14-20t Euro-3','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.095350905880675','0.498728772671681','7.39244951149835','2.78345657560219','-0.024132503752741','6','86']
  H_HDV['52_FC_2_50'] = [' 52','EURO3','RT >14-20t Euro-3','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.105755137849587','1.59405181100779','1679.41915115308','-0.489677179897342','0','6','86']
  H_HDV['52_CO_2_100'] = [' 52','EURO3','RT >14-20t Euro-3','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.99358805571637','0.006071080477682','-0.589026898121858','0.066442528268211','0','6','86']
  H_HDV['52_THC_2_100'] = [' 52','EURO3','RT >14-20t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.1948121462652','1.69762841412803','6.72022474332437','2.59563702995518','-0.017630519524371','6','86']
  H_HDV['52_NOx_2_100'] = [' 52','EURO3','RT >14-20t Euro-3','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.17808476696922','0.015643377705681','31.9706869126933','0.145101788350576','7.24060535454864','6','86']
  H_HDV['52_PM_2_100'] = [' 52','EURO3','RT >14-20t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.781817716132258','0.000423422048381','-0.042856925407712','0.063824978168665','0','6','86']
  H_HDV['52_FC_2_100'] = [' 52','EURO3','RT >14-20t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','961.071329844115','0.765313236631739','-45.7427870525152','0.071088494216631','0','6','86']
  H_HDV['52_CO_4_0'] = [' 52','EURO3','RT >14-20t Euro-3','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.962788330130338','6.27228858106579','5.90370352739583','2.28247856946535','-0.013273418695409','6','72']
  H_HDV['52_THC_4_0'] = [' 52','EURO3','RT >14-20t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.17036523317271','1.79575494459323','6.54323284420933','2.5159703123919','-0.017895165637166','6','72']
  H_HDV['52_NOx_4_0'] = [' 52','EURO3','RT >14-20t Euro-3','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','48.6428680122046','-0.036748817422622','-5.77495326095078','0.153583180676857','0','6','72']
  H_HDV['52_PM_4_0'] = [' 52','EURO3','RT >14-20t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.136471538842607','9.99149460447005','-3.18660945029328','-0.340908491200432','0.095966175873051','6','72']
  H_HDV['52_FC_4_0'] = [' 52','EURO3','RT >14-20t Euro-3','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1047.74125447875','0.47243770101167','-71.2036029057064','0.095567913799586','0','6','72']
  H_HDV['52_CO_4_50'] = [' 52','EURO3','RT >14-20t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.5133565129754','-0.016485832615569','-0.773480464495473','0.094763043012633','0','6','71']
  H_HDV['52_THC_4_50'] = [' 52','EURO3','RT >14-20t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.228221484320355','1.6903253231952','7.32758373841139','2.83224742509149','-0.02240773075039','6','71']
  H_HDV['52_NOx_4_50'] = [' 52','EURO3','RT >14-20t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.69256963960952','0.023649849698516','36.2254781544317','0.167701423619818','9.73240506616851','6','71']
  H_HDV['52_PM_4_50'] = [' 52','EURO3','RT >14-20t Euro-3','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-6.61173096159899','0.714863130122101','0.746508134335071','0.076529097084301','0.152810083120534','6','71']
  H_HDV['52_FC_4_50'] = [' 52','EURO3','RT >14-20t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','1811.71696059325','1.00835185897845','-0.470702301290325','0','0','6','71']
  H_HDV['52_CO_4_100'] = [' 52','EURO3','RT >14-20t Euro-3','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.09829900845262','-3.16154611587537','-0.866922086371537','0','0','6','71']
  H_HDV['52_THC_4_100'] = [' 52','EURO3','RT >14-20t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.26099324932201','1.68642686295744','6.90478415409763','2.69615940917314','-0.019500362665008','6','71']
  H_HDV['52_NOx_4_100'] = [' 52','EURO3','RT >14-20t Euro-3','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9168239514264','0.066848867251925','31.3732779891351','0.236033926843082','13.5195253267726','6','71']
  H_HDV['52_PM_4_100'] = [' 52','EURO3','RT >14-20t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003545455618','0.000556626667688','-0.029782265847436','0.739965284497924','0','6','71']
  H_HDV['52_FC_4_100'] = [' 52','EURO3','RT >14-20t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','521.256977426504','0.080565922096393','8503.17910471285','0.81615896170775','532.979525160149','6','71']
  H_HDV['52_CO_6_0'] = [' 52','EURO3','RT >14-20t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.808811826567','-0.025241562495367','-1.23362583950168','0.138870970567244','0','6','72']
  H_HDV['52_THC_6_0'] = [' 52','EURO3','RT >14-20t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.218805758277297','1.69607402793911','7.79327055927219','2.99726963096926','-0.027320554998423','6','72']
  H_HDV['52_NOx_6_0'] = [' 52','EURO3','RT >14-20t Euro-3','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','55.1201368690115','-0.049322182224718','-6.99929619393369','0.170453320120876','0','6','72']
  H_HDV['52_PM_6_0'] = [' 52','EURO3','RT >14-20t Euro-3','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.835087790471717','-0.001429358795408','-0.059351459935344','0.098448146247915','0','6','72']
  H_HDV['52_FC_6_0'] = [' 52','EURO3','RT >14-20t Euro-3','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1052.51419204739','1.61509607151799','-53.456282355476','0.074716805671203','0','6','72']
  H_HDV['52_CO_6_50'] = [' 52','EURO3','RT >14-20t Euro-3','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.5715953668344','-0.029353060611863','-0.952098129455474','0.112947921406356','0','6','70']
  H_HDV['52_THC_6_50'] = [' 52','EURO3','RT >14-20t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.289502965403879','1.62239291514114','8.00322307606358','3.1165814842343','-0.028591622231427','6','70']
  H_HDV['52_NOx_6_50'] = [' 52','EURO3','RT >14-20t Euro-3','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.7377447505725','0.068594210924495','29.9893869420242','0.21762273523519','14.3504760738403','6','70']
  H_HDV['52_PM_6_50'] = [' 52','EURO3','RT >14-20t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.201123292496435','8.29613797507609','-3.56507378194687','-0.741801988803348','0.135479096288271','6','70']
  H_HDV['52_FC_6_50'] = [' 52','EURO3','RT >14-20t Euro-3','6','50','FC','y=((a*(b**x))*(x**c))','1822.27988166136','1.00768845954797','-0.394456937972564','0','0','6','70']
  H_HDV['52_CO_6_100'] = [' 52','EURO3','RT >14-20t Euro-3','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.52538329751562','-4.21176882483042','-0.962530868905993','0','0','6','61']
  H_HDV['52_THC_6_100'] = [' 52','EURO3','RT >14-20t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.37260996326448','1.5066884613862','8.72618666636701','3.44892259639873','-0.037750733666478','6','61']
  H_HDV['52_NOx_6_100'] = [' 52','EURO3','RT >14-20t Euro-3','6','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','16.0111630113428','40.8239161629434','2.83915806981093','1.5564711512252','-0.005034970061132','6','61']
  H_HDV['52_PM_6_100'] = [' 52','EURO3','RT >14-20t Euro-3','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.821729020045267','0.004693053370114','-0.037207654943214','0.047780563878105','0','6','61']
  H_HDV['52_FC_6_100'] = [' 52','EURO3','RT >14-20t Euro-3','6','100','FC','y=((a*(b**x))*(x**c))','1801.94362596649','1.00590442187921','-0.307830593628656','0','0','6','61']
  H_HDV['53_CO_0_0'] = [' 53','EURO4','RT >14-20t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029410239145694','0.601891573871963','4.38449069615679','1.86871696188319','-0.014309019503608','6','86']
  H_HDV['53_THC_0_0'] = [' 53','EURO4','RT >14-20t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003232820278381','0.125853061033266','3.89854908560882','1.71768716657361','-0.011117960761056','6','86']
  H_HDV['53_NOx_0_0'] = [' 53','EURO4','RT >14-20t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.2093979272099','0.067390675989176','30.9543683124803','0.402825676134768','2.49332586655884','6','86']
  H_HDV['53_PM_0_0'] = [' 53','EURO4','RT >14-20t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.070453944570062','0.04580802201365','0.215026901275594','0.170798415377119','0.014455320308013','6','86']
  H_HDV['53_FC_0_0'] = [' 53','EURO4','RT >14-20t Euro-4','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000816387987253','0.000185884122289','-0.000001372643019','0','0','6','86']
  H_HDV['53_CO_0_50'] = [' 53','EURO4','RT >14-20t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.043321526867663','0.61727497413209','3.94061962972727','1.72411404075898','-0.007988689033276','6','86']
  H_HDV['53_THC_0_50'] = [' 53','EURO4','RT >14-20t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003222202691423','0.120240871530431','4.14338040175059','1.80960380702775','-0.013126153790963','6','86']
  H_HDV['53_NOx_0_50'] = [' 53','EURO4','RT >14-20t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.70093565004028','0.055723531170774','33.8550766212567','0.414463084178338','2.78059431062037','6','86']
  H_HDV['53_PM_0_50'] = [' 53','EURO4','RT >14-20t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067606276625642','0.04367531620653','0.21143092453731','0.167766093650721','0.01458280652329','6','86']
  H_HDV['53_FC_0_50'] = [' 53','EURO4','RT >14-20t Euro-4','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001011933313243','0.000145446534497','-0.000000997818519','','','6','86']
  H_HDV['53_CO_0_100'] = [' 53','EURO4','RT >14-20t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.041718910427323','0.645268994605868','3.58764657605493','1.60293259157182','-0.00724702029458','6','86']
  H_HDV['53_THC_0_100'] = [' 53','EURO4','RT >14-20t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003813623525334','0.116767478575167','4.2357865237187','1.84786598221982','-0.012906606449301','6','86']
  H_HDV['53_NOx_0_100'] = [' 53','EURO4','RT >14-20t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.67716977058514','0.046557026188018','42.0785256873341','0.455678685029194','2.96507136811591','6','86']
  H_HDV['53_PM_0_100'] = [' 53','EURO4','RT >14-20t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.069341318632001','0.043260401592903','0.20846149925601','0.170153681671835','0.014679893400939','6','86']
  H_HDV['53_FC_0_100'] = [' 53','EURO4','RT >14-20t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','424.957700303673','0.05174521619206','4214.68593603389','0.627216582911738','164.669026648658','6','86']
  H_HDV['53_CO_-2_0'] = [' 53','EURO4','RT >14-20t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.060562587457952','5.49415632086144','-2.69439842696604','-0.395479692995853','0.106857656136362','6','86']
  H_HDV['53_THC_-2_0'] = [' 53','EURO4','RT >14-20t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011173547623338','0.631794445337395','-1.78014722609455','-0.170383060008132','0.09408122074505','6','86']
  H_HDV['53_NOx_-2_0'] = [' 53','EURO4','RT >14-20t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.78605771940426','0.074140134548678','122.61348005669','0.697085827247072','1.12985446305887','6','86']
  H_HDV['53_PM_-2_0'] = [' 53','EURO4','RT >14-20t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006004853067672','0.149619672617643','6.15681511686799','2.51761847438318','-0.023263781003296','6','86']
  H_HDV['53_FC_-2_0'] = [' 53','EURO4','RT >14-20t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','535.036995362721','0.074947659466142','38180.0310234209','1.0674826417767','66.1158531438093','6','86']
  H_HDV['53_CO_-2_50'] = [' 53','EURO4','RT >14-20t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.050046564394933','4.27640266026413','-2.33430863134908','-0.310281587296212','0.100093303593489','6','86']
  H_HDV['53_THC_-2_50'] = [' 53','EURO4','RT >14-20t Euro-4','-2','50','THC','y=(c+(a*exp(b*x)))','0.114253178762941','-0.076286208838086','0.008827773409088','0','0','6','86']
  H_HDV['53_NOx_-2_50'] = [' 53','EURO4','RT >14-20t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.9034953565162','0.053573931927734','71.7732770852523','0.548067298233264','0.808146521316847','6','86']
  H_HDV['53_PM_-2_50'] = [' 53','EURO4','RT >14-20t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005081816903397','0.148870300256737','5.87068072659152','2.4004357777894','-0.018192613338445','6','86']
  H_HDV['53_FC_-2_50'] = [' 53','EURO4','RT >14-20t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','448.200905704746','0.058433778965973','8770.44891586898','0.756291211731417','49.8132976140295','6','86']
  H_HDV['53_CO_-2_100'] = [' 53','EURO4','RT >14-20t Euro-4','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-6.42229243989071','0.817832505164117','0.620798045047878','0.074655232431595','0.038285075853359','6','86']
  H_HDV['53_THC_-2_100'] = [' 53','EURO4','RT >14-20t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00214106578935','0.081487974120286','7.0456144103798','2.70056770572247','-0.025760114473535','6','86']
  H_HDV['53_NOx_-2_100'] = [' 53','EURO4','RT >14-20t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.31126856228745','0.041541571474837','78.8522813261582','0.545322117173622','0.527698237054068','6','86']
  H_HDV['53_PM_-2_100'] = [' 53','EURO4','RT >14-20t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003203701265919','0.156387803304755','5.09528973912409','2.10912136352586','-0.010957577409773','6','86']
  H_HDV['53_FC_-2_100'] = [' 53','EURO4','RT >14-20t Euro-4','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','401.549523564321','0.046429269484524','7122.0446033729','0.687388839565312','35.12777026628','6','86']
  H_HDV['53_CO_-4_0'] = [' 53','EURO4','RT >14-20t Euro-4','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','4.3524222474555','-11.818228392616','-1.87854494234321','0','0','6','86']
  H_HDV['53_THC_-4_0'] = [' 53','EURO4','RT >14-20t Euro-4','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001721370988547','0.080784470392794','6.96567482061934','2.63653351887033','-0.018010562989821','6','86']
  H_HDV['53_NOx_-4_0'] = [' 53','EURO4','RT >14-20t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.43432985283946','0.060827185762716','53.2620013988106','0.496190302776394','0.219019716133366','6','86']
  H_HDV['53_PM_-4_0'] = [' 53','EURO4','RT >14-20t Euro-4','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001156762164418','0.151671171871596','5.60460414151633','2.29604637098736','-0.013074473935166','6','86']
  H_HDV['53_FC_-4_0'] = [' 53','EURO4','RT >14-20t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','408.137294801862','0.062581812012396','3075.6034919248','0.559113598924383','14.9150941076732','6','86']
  H_HDV['53_CO_-4_50'] = [' 53','EURO4','RT >14-20t Euro-4','-4','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.63318970514039','-12.7724144203547','-1.99904764545966','0','0','6','86']
  H_HDV['53_THC_-4_50'] = [' 53','EURO4','RT >14-20t Euro-4','-4','50','THC','y=exp((a+(b/x))+(c*log(x)))','2.06286608437635','-9.67236522707314','-1.73809769777489','0','0','6','86']
  H_HDV['53_NOx_-4_50'] = [' 53','EURO4','RT >14-20t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.51265025216108','0.053438449032049','72.2793502933373','0.53551989781172','0.113252113623239','6','86']
  H_HDV['53_PM_-4_50'] = [' 53','EURO4','RT >14-20t Euro-4','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.213398240649866','-0.000140103226344','-0.020322481311474','0.10187254434875','0','6','86']
  H_HDV['53_FC_-4_50'] = [' 53','EURO4','RT >14-20t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','341.54474526349','0.054569068269692','4054.66394717413','0.578800823158214','7.9798166711259','6','86']
  H_HDV['53_CO_-4_100'] = [' 53','EURO4','RT >14-20t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.524080576164701','-0.000372414064518','-0.040344046844902','0.082641124535348','0','6','86']
  H_HDV['53_THC_-4_100'] = [' 53','EURO4','RT >14-20t Euro-4','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.108411404354869','-0.000077946813281','-0.008748795234678','0.087517911104904','0','6','86']
  H_HDV['53_NOx_-4_100'] = [' 53','EURO4','RT >14-20t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.01919945312398','0.048778566210021','108.35259010062','0.597614684362535','0.066415135692194','6','86']
  H_HDV['53_PM_-4_100'] = [' 53','EURO4','RT >14-20t Euro-4','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.208463507176198','-0.000146059533335','-0.020442016342188','0.105129322325912','0','6','86']
  H_HDV['53_FC_-4_100'] = [' 53','EURO4','RT >14-20t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','306.032989869009','0.049700357338543','7021.51526581856','0.658414390206424','5.1085976146454','6','86']
  H_HDV['53_CO_-6_0'] = [' 53','EURO4','RT >14-20t Euro-4','-6','0','CO','y=(1/(a+(b*(x**c))))','2.66910666839464','0.005064895245036','2.31889692421281','0','0','6','72']
  H_HDV['53_THC_-6_0'] = [' 53','EURO4','RT >14-20t Euro-4','-6','0','THC','y=(c+(a*exp(b*x)))','0.105119137661353','-0.077703091468099','0.003244505290941','0','0','6','72']
  H_HDV['53_NOx_-6_0'] = [' 53','EURO4','RT >14-20t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.28882137286395','0.072015105013115','90.8919994744399','0.607081303053569','0.062893392069399','6','72']
  H_HDV['53_PM_-6_0'] = [' 53','EURO4','RT >14-20t Euro-4','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003249082130838','0.176243460915572','3.32318931867179','1.37890149153562','0.034311463321896','6','72']
  H_HDV['53_FC_-6_0'] = [' 53','EURO4','RT >14-20t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','392.190695595054','0.072307715362564','6923.12188010657','0.713847938731386','5.03872225917712','6','72']
  H_HDV['53_CO_-6_50'] = [' 53','EURO4','RT >14-20t Euro-4','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.451433111783157','0.000261134781845','-0.032118585846129','0.070302842838556','0','6','72']
  H_HDV['53_THC_-6_50'] = [' 53','EURO4','RT >14-20t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080864528710611','0.072237746882916','0.052294345946855','0.318845171655909','0.002908895979212','6','72']
  H_HDV['53_NOx_-6_50'] = [' 53','EURO4','RT >14-20t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.16425856132385','0.067721667045907','132.2817074768','0.656516035611577','0.046701696364577','6','72']
  H_HDV['53_PM_-6_50'] = [' 53','EURO4','RT >14-20t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.117435596551996','0.073095420364953','0.164708627373327','0.265904513038799','0.003644444545753','6','72']
  H_HDV['53_FC_-6_50'] = [' 53','EURO4','RT >14-20t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','308.644006981885','0.06683080042329','9998.223884994','0.742706523159883','3.79213637091833','6','72']
  H_HDV['53_CO_-6_100'] = [' 53','EURO4','RT >14-20t Euro-4','-6','100','CO','y=(c+(a*exp(b*x)))','0.397605189447412','-0.075467308170327','0.008535786590261','0','0','6','72']
  H_HDV['53_THC_-6_100'] = [' 53','EURO4','RT >14-20t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.069768653826067','0.069298661779604','0.234115265469412','0.517102069089868','0.002763061506503','6','72']
  H_HDV['53_NOx_-6_100'] = [' 53','EURO4','RT >14-20t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.51376477256077','0.064091471608204','169.728613355779','0.692913419521643','0.037662845532668','6','72']
  H_HDV['53_PM_-6_100'] = [' 53','EURO4','RT >14-20t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.10892086226218','0.071936506550375','0.294989127539366','0.363098917549332','0.003533745208895','6','72']
  H_HDV['53_FC_-6_100'] = [' 53','EURO4','RT >14-20t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','260.98139740856','0.062298540383093','13654.2276078499','0.782415826852227','3.03870908332635','6','72']
  H_HDV['53_CO_2_0'] = [' 53','EURO4','RT >14-20t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.079718951822995','0.474887593426105','6.78150608370566','2.6465609056553','-0.016360948801032','6','86']
  H_HDV['53_THC_2_0'] = [' 53','EURO4','RT >14-20t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005125370059792','0.108416748114595','5.30286368522002','2.08759507065651','-0.010988552612217','6','86']
  H_HDV['53_NOx_2_0'] = [' 53','EURO4','RT >14-20t Euro-4','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.4486655784532','0.00291556522632','-1.58185186914809','0.11010375368875','0','6','86']
  H_HDV['53_PM_2_0'] = [' 53','EURO4','RT >14-20t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.037113002933412','0.010630693948799','0.215772490333535','0.112711756749487','0.003127431391336','6','86']
  H_HDV['53_FC_2_0'] = [' 53','EURO4','RT >14-20t Euro-4','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.126080513787263','1.52617424231932','1817.17524971605','-0.599635368886558','0','6','86']
  H_HDV['53_CO_2_50'] = [' 53','EURO4','RT >14-20t Euro-4','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.104290814688748','0.457695993046478','6.86349957142381','2.69340126250992','-0.015819426302371','6','86']
  H_HDV['53_THC_2_50'] = [' 53','EURO4','RT >14-20t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005998391642393','0.103556331963722','5.86814983045943','2.30097722935148','-0.014122543068198','6','86']
  H_HDV['53_NOx_2_50'] = [' 53','EURO4','RT >14-20t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.5996188502275','-0.006172105231773','-1.65086605615612','0.118440265254041','0','6','86']
  H_HDV['53_PM_2_50'] = [' 53','EURO4','RT >14-20t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.04233788010426','0.009165247583247','0.214388949399918','0.112650892623527','0.000312200200079','6','86']
  H_HDV['53_FC_2_50'] = [' 53','EURO4','RT >14-20t Euro-4','2','50','FC','y=((a*(b**x))*(x**c))','1881.93795992555','1.00985756245479','-0.607608930641687','0','0','6','86']
  H_HDV['53_CO_2_100'] = [' 53','EURO4','RT >14-20t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.914085934321336','-0.001135516289028','-0.08562667366647','0.11869668106204','0','6','86']
  H_HDV['53_THC_2_100'] = [' 53','EURO4','RT >14-20t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007301380953588','0.09861136978697','6.46386687402389','2.52873256255279','-0.017594097241048','6','86']
  H_HDV['53_NOx_2_100'] = [' 53','EURO4','RT >14-20t Euro-4','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.4333505743575','-0.015957819170479','-1.61041658226044','0.120795598031222','0','6','86']
  H_HDV['53_PM_2_100'] = [' 53','EURO4','RT >14-20t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.046152522383638','0.012636634318378','0.214703580127602','0.118967054660987','0.004133015374241','6','86']
  H_HDV['53_FC_2_100'] = [' 53','EURO4','RT >14-20t Euro-4','2','100','FC','y=((a*(b**x))*(x**c))','1720.54215427357','1.00797633468724','-0.51220069590102','0','0','6','86']
  H_HDV['53_CO_4_0'] = [' 53','EURO4','RT >14-20t Euro-4','4','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004700075458','0.000703316172953','-0.03463845993668','0.70541846216739','0','6','72']
  H_HDV['53_THC_4_0'] = [' 53','EURO4','RT >14-20t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005860318044229','0.104889312847588','6.04068483176438','2.3513739777008','-0.015437892834238','6','72']
  H_HDV['53_NOx_4_0'] = [' 53','EURO4','RT >14-20t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.2129252474256','-0.006907297372719','-2.46524056920779','0.145224520215873','0','6','72']
  H_HDV['53_PM_4_0'] = [' 53','EURO4','RT >14-20t Euro-4','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.258288213495849','-0.000270675898154','-0.024602314292144','0.112767407699744','0','6','72']
  H_HDV['53_FC_4_0'] = [' 53','EURO4','RT >14-20t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','2002.75479896527','1.01106714571418','-0.620306578679312','0','0','6','72']
  H_HDV['53_CO_4_50'] = [' 53','EURO4','RT >14-20t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.895472347692','-0.001033218727565','-0.075879672730573','0.110415828703807','0','6','71']
  H_HDV['53_THC_4_50'] = [' 53','EURO4','RT >14-20t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007825477489808','0.099338447442001','6.71081483530136','2.61787145885941','-0.019047992692016','6','71']
  H_HDV['53_NOx_4_50'] = [' 53','EURO4','RT >14-20t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.47674941317433','0.034860273640691','17.835582794891','0.205834709889881','7.05193069590899','6','71']
  H_HDV['53_PM_4_50'] = [' 53','EURO4','RT >14-20t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.054182347884121','0.013646490433255','0.2125812128379','0.122692015539228','0.00413666711308','6','71']
  H_HDV['53_FC_4_50'] = [' 53','EURO4','RT >14-20t Euro-4','4','50','FC','y=((a*(b**x))*(x**c))','1769.81886684134','1.0085989513166','-0.483672751828468','0','0','6','71']
  H_HDV['53_CO_4_100'] = [' 53','EURO4','RT >14-20t Euro-4','4','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.880979097444879','0.133296634710229','-0.000688756225456','0','0','6','71']
  H_HDV['53_THC_4_100'] = [' 53','EURO4','RT >14-20t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009518747573241','0.096716661361574','6.79030595393702','2.6705964973507','-0.019309201380029','6','71']
  H_HDV['53_NOx_4_100'] = [' 53','EURO4','RT >14-20t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.70595400604033','0.051309344330607','22.1850027857396','0.316844097429517','8.61294174419268','6','71']
  H_HDV['53_PM_4_100'] = [' 53','EURO4','RT >14-20t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068078613578097','0.022984402105216','0.208037311399433','0.135427372782294','0.010275119532576','6','71']
  H_HDV['53_FC_4_100'] = [' 53','EURO4','RT >14-20t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','540.596297548611','0.088270148310702','19846.7400757137','0.98255333954578','506.062545270459','6','71']
  H_HDV['53_CO_6_0'] = [' 53','EURO4','RT >14-20t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.11018721297932','-0.001720698201131','-0.144479561016496','0.17015085743017','0','6','72']
  H_HDV['53_THC_6_0'] = [' 53','EURO4','RT >14-20t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007434340404478','0.099390463551613','6.99734064666236','2.71020442599084','-0.021764169593105','6','72']
  H_HDV['53_NOx_6_0'] = [' 53','EURO4','RT >14-20t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.6390083738546','-0.019413058103774','-3.79652521388636','0.188682997923481','0','6','72']
  H_HDV['53_PM_6_0'] = [' 53','EURO4','RT >14-20t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.296695370482733','-0.00047532832479','-0.034551312879071','0.142843085568566','0','6','72']
  H_HDV['53_FC_6_0'] = [' 53','EURO4','RT >14-20t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','1982.52339252753','1.01131376664884','-0.558767822654036','0','0','6','72']
  H_HDV['53_CO_6_50'] = [' 53','EURO4','RT >14-20t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.31177915232498','-0.003706406263724','-0.201738223805442','0.215453618789558','0','6','70']
  H_HDV['53_THC_6_50'] = [' 53','EURO4','RT >14-20t Euro-4','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-1.51371972902684','0.787513109606541','0.162762898066092','0.091682810743457','0.013899769345432','6','70']
  H_HDV['53_NOx_6_50'] = [' 53','EURO4','RT >14-20t Euro-4','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.5848792360217','-0.033174271016994','-3.66112369239812','0.190847451756551','0','6','70']
  H_HDV['53_PM_6_50'] = [' 53','EURO4','RT >14-20t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.333903728545094','-0.000691208208952','-0.043251311469664','0.161790320775661','0','6','70']
  H_HDV['53_FC_6_50'] = [' 53','EURO4','RT >14-20t Euro-4','6','50','FC','y=((a*(b**x))*(x**c))','1811.77880825655','1.00830794900142','-0.417116424044976','0','0','6','70']
  H_HDV['53_CO_6_100'] = [' 53','EURO4','RT >14-20t Euro-4','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.11882440680901','-0.00355572055242','-0.099757470121197','0.125199266670973','0','6','62']
  H_HDV['53_THC_6_100'] = [' 53','EURO4','RT >14-20t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017280878313089','0.633002893094544','-1.74035933988821','-0.283882744210861','0.119403806604265','6','62']
  H_HDV['53_NOx_6_100'] = [' 53','EURO4','RT >14-20t Euro-4','6','100','NOx','y=exp((a+(b/x))+(c*log(x)))','2.95733939292961','1.76356563496711','-0.138926748350882','0','0','6','62']
  H_HDV['53_PM_6_100'] = [' 53','EURO4','RT >14-20t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01619930816152','0.205567724317046','4.43511145764572','1.87288108232506','-0.01078188785097','6','62']
  H_HDV['53_FC_6_100'] = [' 53','EURO4','RT >14-20t Euro-4','6','100','FC','y=((a*(b**x))*(x**c))','1779.00163161846','1.0065864150116','-0.327550694971312','0','0','6','62']
  H_HDV['54_CO_0_0'] = [' 54','EURO5','RT >14-20t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029256258002474','0.61454824451541','4.37288379798154','1.86496057464588','-0.014277487714174','6','86']
  H_HDV['54_THC_0_0'] = [' 54','EURO5','RT >14-20t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003228696346022','0.128529678787696','3.88481978567033','1.71368830378613','-0.011158902169312','6','86']
  H_HDV['54_NOx_0_0'] = [' 54','EURO5','RT >14-20t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.34670213654854','0.064961351057795','20.286173984487','0.370179093880745','1.46306101070287','6','86']
  H_HDV['54_PM_0_0'] = [' 54','EURO5','RT >14-20t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.071591490167062','0.045669373186763','0.219634502628785','0.170892963862572','0.014585391722715','6','86']
  H_HDV['54_FC_0_0'] = [' 54','EURO5','RT >14-20t Euro-5','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000784043028639','0.000182163224858','-0.00000133551344','0','0','6','86']
  H_HDV['54_CO_0_50'] = [' 54','EURO5','RT >14-20t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.041379916259725','0.623070048246216','4.02778696139254','1.75322241416501','-0.009516652021533','6','86']
  H_HDV['54_THC_0_50'] = [' 54','EURO5','RT >14-20t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003082806285787','0.122469830936826','4.14848709730996','1.81067038177333','-0.013497997776959','6','86']
  H_HDV['54_NOx_0_50'] = [' 54','EURO5','RT >14-20t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.05363994319323','0.053964778524993','21.2354317019052','0.374290057405629','1.62694882673688','6','86']
  H_HDV['54_PM_0_50'] = [' 54','EURO5','RT >14-20t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068950293416991','0.043757458646563','0.214824508672341','0.167659148105039','0.014746001447617','6','86']
  H_HDV['54_FC_0_50'] = [' 54','EURO5','RT >14-20t Euro-5','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000973504533607','0.000143103790695','-0.000000975226058','','','6','86']
  H_HDV['54_CO_0_100'] = [' 54','EURO5','RT >14-20t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04227383026234','0.658553145290766','3.56575624281285','1.59580320749315','-0.006848050097655','6','86']
  H_HDV['54_THC_0_100'] = [' 54','EURO5','RT >14-20t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003435715653359','0.11945688518222','4.23196315892817','1.84831595065926','-0.013840171163496','6','86']
  H_HDV['54_NOx_0_100'] = [' 54','EURO5','RT >14-20t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.10012898432322','0.0457243967259','24.9616444693122','0.406713858306204','1.72417543519144','6','86']
  H_HDV['54_PM_0_100'] = [' 54','EURO5','RT >14-20t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.070873923201473','0.043392647331861','0.212527098857082','0.170844232414589','0.014849109335115','6','86']
  H_HDV['54_FC_0_100'] = [' 54','EURO5','RT >14-20t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','437.349935114938','0.051710400981226','4781.66440636133','0.636716914677686','166.844098232028','6','86']
  H_HDV['54_CO_-2_0'] = [' 54','EURO5','RT >14-20t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.06156070480495','5.34278766577472','-2.64292126826511','-0.394556773748098','0.106829230012101','6','86']
  H_HDV['54_THC_-2_0'] = [' 54','EURO5','RT >14-20t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011369152230044','0.649047638014037','-1.79216052489898','-0.171366901731189','0.093960887241351','6','86']
  H_HDV['54_NOx_-2_0'] = [' 54','EURO5','RT >14-20t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.47455638785509','0.073737049764613','53.5166838608778','0.590324582586729','0.664655623854217','6','86']
  H_HDV['54_PM_-2_0'] = [' 54','EURO5','RT >14-20t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00629114015476','0.151891019333416','6.17186515227531','2.51984595163995','-0.022820297124263','6','86']
  H_HDV['54_FC_-2_0'] = [' 54','EURO5','RT >14-20t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','546.893945001173','0.074418662101641','22703.363706332','0.972201591841854','67.2185186150348','6','86']
  H_HDV['54_CO_-2_50'] = [' 54','EURO5','RT >14-20t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.050761686966253','4.33167746890453','-2.32480819793518','-0.307609126565415','0.099794887207637','6','86']
  H_HDV['54_THC_-2_50'] = [' 54','EURO5','RT >14-20t Euro-5','-2','50','THC','y=(c+(a*exp(b*x)))','0.116373463879247','-0.076143754688161','0.008975878500811','0','0','6','86']
  H_HDV['54_NOx_-2_50'] = [' 54','EURO5','RT >14-20t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.2637403867337','0.053565845927264','41.348572852451','0.498490289680998','0.472525028892879','6','86']
  H_HDV['54_PM_-2_50'] = [' 54','EURO5','RT >14-20t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00515313925603','0.151530960749483','5.87181587941652','2.3995517057021','-0.018097399882582','6','86']
  H_HDV['54_FC_-2_50'] = [' 54','EURO5','RT >14-20t Euro-5','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','453.187637779934','0.057675441733626','6014.19805887604','0.682687233289423','50.3179623603579','6','86']
  H_HDV['54_CO_-2_100'] = [' 54','EURO5','RT >14-20t Euro-5','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-6.50212908287524','0.813782693431384','0.633849177437393','0.074756694216624','0.038822531737265','6','86']
  H_HDV['54_THC_-2_100'] = [' 54','EURO5','RT >14-20t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002449906112718','0.082646396318128','7.07309514827422','2.70540998067991','-0.024937898083713','6','86']
  H_HDV['54_NOx_-2_100'] = [' 54','EURO5','RT >14-20t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.91108593158575','0.04220868447283','45.6679290576685','0.501639460208275','0.311353299359583','6','86']
  H_HDV['54_PM_-2_100'] = [' 54','EURO5','RT >14-20t Euro-5','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003245791893398','0.158770584753324','5.12684496371847','2.11939137681137','-0.011151359076642','6','86']
  H_HDV['54_FC_-2_100'] = [' 54','EURO5','RT >14-20t Euro-5','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','413.591445617444','0.046589824787197','6905.00276109491','0.676213622035739','35.7660517913877','6','86']
  H_HDV['54_CO_-4_0'] = [' 54','EURO5','RT >14-20t Euro-5','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','4.38600085641317','-11.860128026575','-1.88211703656844','0','0','6','86']
  H_HDV['54_THC_-4_0'] = [' 54','EURO5','RT >14-20t Euro-5','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00173117869782','0.08248919775876','6.95364838819583','2.6318081578453','-0.017927300643213','6','86']
  H_HDV['54_NOx_-4_0'] = [' 54','EURO5','RT >14-20t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.05392739432401','0.06145367391878','34.7059651082329','0.472093154119902','0.129934468795189','6','86']
  H_HDV['54_PM_-4_0'] = [' 54','EURO5','RT >14-20t Euro-5','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001160319052268','0.154597997125162','5.61019765612039','2.29769281903343','-0.013108714467127','6','86']
  H_HDV['54_FC_-4_0'] = [' 54','EURO5','RT >14-20t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','420.581166354046','0.062563637121733','3026.30899747644','0.550053213382632','15.2377123782751','6','86']
  H_HDV['54_CO_-4_50'] = [' 54','EURO5','RT >14-20t Euro-5','-4','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.67847701954237','-12.8637026428992','-2.00555198656587','0','0','6','86']
  H_HDV['54_THC_-4_50'] = [' 54','EURO5','RT >14-20t Euro-5','-4','50','THC','y=exp((a+(b/x))+(c*log(x)))','2.101772391851','-9.74075337525094','-1.74289299148312','0','0','6','86']
  H_HDV['54_NOx_-4_50'] = [' 54','EURO5','RT >14-20t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.51318035140091','0.05475399618336','46.1482425918294','0.512174267318216','0.070055967119015','6','86']
  H_HDV['54_PM_-4_50'] = [' 54','EURO5','RT >14-20t Euro-5','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.217186522373197','-0.000142438632481','-0.020653795114718','0.101718370792956','0','6','86']
  H_HDV['54_FC_-4_50'] = [' 54','EURO5','RT >14-20t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','353.330297355821','0.054800821646566','4072.0103807584','0.574489251575052','8.21606876851488','6','86']
  H_HDV['54_CO_-4_100'] = [' 54','EURO5','RT >14-20t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.534174641620915','-0.000376493608882','-0.04107771032024','0.082514073697093','0','6','86']
  H_HDV['54_THC_-4_100'] = [' 54','EURO5','RT >14-20t Euro-5','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.110504250356243','-0.000079230450363','-0.008907063307782','0.087394454277377','0','6','86']
  H_HDV['54_NOx_-4_100'] = [' 54','EURO5','RT >14-20t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.20433140249618','0.050357944713445','64.56527251755','0.564784833175246','0.044228094929437','6','86']
  H_HDV['54_PM_-4_100'] = [' 54','EURO5','RT >14-20t Euro-5','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.212275915886613','-0.000148165727515','-0.020794238308928','0.104996034790989','0','6','86']
  H_HDV['54_FC_-4_100'] = [' 54','EURO5','RT >14-20t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','316.280817658313','0.049988492539126','6901.07828909495','0.650836987654636','5.33791020354712','6','86']
  H_HDV['54_CO_-6_0'] = [' 54','EURO5','RT >14-20t Euro-5','-6','0','CO','y=(1/(a+(b*(x**c))))','2.6131155245776','0.004897201778539','2.3227696802239','0','0','6','72']
  H_HDV['54_THC_-6_0'] = [' 54','EURO5','RT >14-20t Euro-5','-6','0','THC','y=(c+(a*exp(b*x)))','0.107382406145501','-0.077712108521089','0.003300949449748','0','0','6','72']
  H_HDV['54_NOx_-6_0'] = [' 54','EURO5','RT >14-20t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.05689047295073','0.073103233941468','55.7217846279544','0.574750798810565','0.03892176234429','6','72']
  H_HDV['54_PM_-6_0'] = [' 54','EURO5','RT >14-20t Euro-5','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00326435237638','0.177406786884453','3.41756582788006','1.41258604944661','0.033065351527523','6','72']
  H_HDV['54_FC_-6_0'] = [' 54','EURO5','RT >14-20t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','406.964223372831','0.07253622321337','6953.02212806961','0.710258292906011','5.21068712264312','6','72']
  H_HDV['54_CO_-6_50'] = [' 54','EURO5','RT >14-20t Euro-5','-6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.461596780989714','0.000265401395269','-0.032890843799315','0.07041512341888','0','6','72']
  H_HDV['54_THC_-6_50'] = [' 54','EURO5','RT >14-20t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082626684147296','0.072307078788576','0.048602666228589','0.304978285734064','0.002959670225545','6','72']
  H_HDV['54_NOx_-6_50'] = [' 54','EURO5','RT >14-20t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.35088525444206','0.068917835510701','78.4088499024587','0.621406628011418','0.029030460846692','6','72']
  H_HDV['54_PM_-6_50'] = [' 54','EURO5','RT >14-20t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.119488181657117','0.073095346312312','0.163390964091275','0.260835634536121','0.003710240009394','6','72']
  H_HDV['54_FC_-6_50'] = [' 54','EURO5','RT >14-20t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','320.532115635148','0.067168659764964','10035.1030158355','0.739579275931858','3.95646945481384','6','72']
  H_HDV['54_CO_-6_100'] = [' 54','EURO5','RT >14-20t Euro-5','-6','100','CO','y=(c+(a*exp(b*x)))','0.4070844483007','-0.075598345932941','0.008685139482563','0','0','6','72']
  H_HDV['54_THC_-6_100'] = [' 54','EURO5','RT >14-20t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.071538902007293','0.069510232989854','0.228398937490796','0.511941551565506','0.002818498641684','6','72']
  H_HDV['54_NOx_-6_100'] = [' 54','EURO5','RT >14-20t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.92373956957125','0.065081072965405','95.1261708479519','0.648844908592214','0.022692345801592','6','72']
  H_HDV['54_PM_-6_100'] = [' 54','EURO5','RT >14-20t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.111195679067254','0.072018534858193','0.293632382652743','0.359083851762752','0.003597733544229','6','72']
  H_HDV['54_FC_-6_100'] = [' 54','EURO5','RT >14-20t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','270.637317641491','0.062606200548707','13700.3694142425','0.778097130122995','3.18034262831881','6','72']
  H_HDV['54_CO_2_0'] = [' 54','EURO5','RT >14-20t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.079134981357585','0.486248138275454','6.68635280687788','2.60869129023495','-0.015753784066961','6','86']
  H_HDV['54_THC_2_0'] = [' 54','EURO5','RT >14-20t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005215419099041','0.110277643139339','5.29876497124832','2.0852883923976','-0.010952889100946','6','86']
  H_HDV['54_NOx_2_0'] = [' 54','EURO5','RT >14-20t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.03913222347106','0.112740167007141','93.1437689700245','0.856094514154343','2.51309636288059','6','86']
  H_HDV['54_PM_2_0'] = [' 54','EURO5','RT >14-20t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.037145104155696','0.011160338812751','0.219245014299255','0.11288916960383','0.003985893029368','6','86']
  H_HDV['54_FC_2_0'] = [' 54','EURO5','RT >14-20t Euro-5','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.134137265461605','1.51455538750741','1885.42023623152','-0.604882907141498','0','6','86']
  H_HDV['54_CO_2_50'] = [' 54','EURO5','RT >14-20t Euro-5','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.104536763266449','0.466059657306721','6.89038927795061','2.70343770273309','-0.016207340354183','6','86']
  H_HDV['54_THC_2_50'] = [' 54','EURO5','RT >14-20t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006014812210407','0.105952589450046','5.80102022955488','2.27602999176857','-0.01362681021526','6','86']
  H_HDV['54_NOx_2_50'] = [' 54','EURO5','RT >14-20t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.753582483346','-0.006383185454998','-1.3622723293652','0.132884694665263','0','6','86']
  H_HDV['54_PM_2_50'] = [' 54','EURO5','RT >14-20t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.042633130098311','0.00946025081344','0.218225760596512','0.112970807436927','0.000857969664392','6','86']
  H_HDV['54_FC_2_50'] = [' 54','EURO5','RT >14-20t Euro-5','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','930.99366213568','0.529823827613041','-54.544051616102','0.080700923413768','0','6','86']
  H_HDV['54_CO_2_100'] = [' 54','EURO5','RT >14-20t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.917789983846463','-0.001050562068259','-0.084149175828788','0.115566759420294','0','6','86']
  H_HDV['54_THC_2_100'] = [' 54','EURO5','RT >14-20t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007243351928778','0.101173073578963','6.34000451819611','2.48087237092019','-0.016574263015851','6','86']
  H_HDV['54_NOx_2_100'] = [' 54','EURO5','RT >14-20t Euro-5','2','100','NOx','y=((a*(x**b))+(c*(x**d)))','6.12480188003252','-0.140244594615312','47.1399966191114','-1.34872009495157','0','6','86']
  H_HDV['54_PM_2_100'] = [' 54','EURO5','RT >14-20t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048501933314156','0.010814876299251','0.218755695232845','0.117390682461438','0.000770133602706','6','86']
  H_HDV['54_FC_2_100'] = [' 54','EURO5','RT >14-20t Euro-5','2','100','FC','y=((a*(b**x))*(x**c))','1776.5663211983','1.00791635216017','-0.515853927833846','0','0','6','86']
  H_HDV['54_CO_4_0'] = [' 54','EURO5','RT >14-20t Euro-5','4','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004766150761','0.000713904475808','-0.035212142600568','0.716456567287774','0','6','72']
  H_HDV['54_THC_4_0'] = [' 54','EURO5','RT >14-20t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005850384111175','0.107154320324757','5.98436531389181','2.33049884664122','-0.01513214192281','6','72']
  H_HDV['54_NOx_4_0'] = [' 54','EURO5','RT >14-20t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.5234663401716','-0.006820392486995','-2.0484317946997','0.161632929202488','0','6','72']
  H_HDV['54_PM_4_0'] = [' 54','EURO5','RT >14-20t Euro-5','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.262566858730845','-0.000276667287392','-0.02504582713606','0.112885427251177','0','6','72']
  H_HDV['54_FC_4_0'] = [' 54','EURO5','RT >14-20t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','2076.21007645898','1.01106936646397','-0.625634494002299','0','0','6','72']
  H_HDV['54_CO_4_50'] = [' 54','EURO5','RT >14-20t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.913702086618283','-0.000984247672861','-0.078496763630473','0.111380823402105','0','6','71']
  H_HDV['54_THC_4_50'] = [' 54','EURO5','RT >14-20t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007754950668086','0.101454105362624','6.66529455761692','2.59775889208568','-0.018712683510467','6','71']
  H_HDV['54_NOx_4_50'] = [' 54','EURO5','RT >14-20t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.40375921974639','0.018972645251384','13.9359258007418','0.208405982284334','3.57513380439398','6','71']
  H_HDV['54_PM_4_50'] = [' 54','EURO5','RT >14-20t Euro-5','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.268762104627921','-0.000336375898614','-0.025350823116104','0.113668598374487','0','6','71']
  H_HDV['54_FC_4_50'] = [' 54','EURO5','RT >14-20t Euro-5','4','50','FC','y=((a*(b**x))*(x**c))','1830.23855572597','1.00862470208513','-0.488922867697857','0','0','6','71']
  H_HDV['54_CO_4_100'] = [' 54','EURO5','RT >14-20t Euro-5','4','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.88023191000767','0.130530125543903','-0.000662261435945','0','0','6','71']
  H_HDV['54_THC_4_100'] = [' 54','EURO5','RT >14-20t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009647208788228','0.098220641784698','6.84215382010504','2.68770338477447','-0.019526878605476','6','71']
  H_HDV['54_NOx_4_100'] = [' 54','EURO5','RT >14-20t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.19330856941289','0.035444331150749','15.8480430695339','0.28079418552555','4.47628579107562','6','71']
  H_HDV['54_PM_4_100'] = [' 54','EURO5','RT >14-20t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068421911600906','0.024760772741917','0.208989083075884','0.135600885419831','0.012507184699604','6','71']
  H_HDV['54_FC_4_100'] = [' 54','EURO5','RT >14-20t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','565.678757457761','0.089188134288736','24633.7087851237','1.01935064784863','513.718914359496','6','71']
  H_HDV['54_CO_6_0'] = [' 54','EURO5','RT >14-20t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.06881556013055','-0.001397554304738','-0.12703011106633','0.154501738160322','0','6','72']
  H_HDV['54_THC_6_0'] = [' 54','EURO5','RT >14-20t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007182094813258','0.102644667408155','6.73634208121947','2.61047620715896','-0.01975089571426','6','72']
  H_HDV['54_NOx_6_0'] = [' 54','EURO5','RT >14-20t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.5694588179763','-0.016262922174472','-3.5287783344146','0.217004650490547','0','6','72']
  H_HDV['54_PM_6_0'] = [' 54','EURO5','RT >14-20t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.29527062434002','-0.000456404570718','-0.033267947583824','0.137907671015332','0','6','72']
  H_HDV['54_FC_6_0'] = [' 54','EURO5','RT >14-20t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','2041.97589747975','1.01118683187383','-0.561140524667487','0','0','6','72']
  H_HDV['54_CO_6_50'] = [' 54','EURO5','RT >14-20t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.26976211349091','-0.00365514096873','-0.182863734575522','0.204018517491787','0','6','71']
  H_HDV['54_THC_6_50'] = [' 54','EURO5','RT >14-20t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009544055536698','0.095041057668139','8.00172911573163','3.1341618403536','-0.032645468726628','6','71']
  H_HDV['54_NOx_6_50'] = [' 54','EURO5','RT >14-20t Euro-5','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.1782523717076','-0.033853222538489','-4.02053422671695','0.237446609612517','0','6','71']
  H_HDV['54_PM_6_50'] = [' 54','EURO5','RT >14-20t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.329812757036541','-0.000679634110636','-0.041042015951438','0.155581742675502','0','6','71']
  H_HDV['54_FC_6_50'] = [' 54','EURO5','RT >14-20t Euro-5','6','50','FC','y=((a*(b**x))*(x**c))','1869.2719138705','1.0083779270968','-0.422274202317821','0','0','6','71']
  H_HDV['54_CO_6_100'] = [' 54','EURO5','RT >14-20t Euro-5','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.12624967161484','-0.003744621050619','-0.100467043516803','0.12655592523438','0','6','62']
  H_HDV['54_THC_6_100'] = [' 54','EURO5','RT >14-20t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017368122679517','0.674694939404606','-1.79517960423538','-0.285841970819126','0.119088404971761','6','62']
  H_HDV['54_NOx_6_100'] = [' 54','EURO5','RT >14-20t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.41878694649235','0.016453336275936','13.004326517353','0.203000942709811','3.99599773885969','6','62']
  H_HDV['54_PM_6_100'] = [' 54','EURO5','RT >14-20t Euro-5','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.288757466072703','-0.000414043863291','-0.025395513105107','0.106482707224137','0','6','62']
  H_HDV['54_FC_6_100'] = [' 54','EURO5','RT >14-20t Euro-5','6','100','FC','y=((a*(b**x))*(x**c))','1827.88760237658','1.00656352363875','-0.330778719691862','0','0','6','62']
  H_HDV['55_CO_0_0'] = [' 55','EURO0','RT >20-26t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.56887641642476','0.058949602275853','9.4899083603199','0.228020956371543','1.42703937414201','6','86']
  H_HDV['55_THC_0_0'] = [' 55','EURO0','RT >20-26t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.6786590286322','0.030817599017374','4.85889793727534','0.122834400119656','0.343641654655617','6','86']
  H_HDV['55_NOx_0_0'] = [' 55','EURO0','RT >20-26t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.119173713394','0.066594097150293','52.5959070861934','0.435260569839835','8.21214504039315','6','86']
  H_HDV['55_PM_0_0'] = [' 55','EURO0','RT >20-26t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.50002488647735','0.056481324025478','1.74562846241545','0.198733171952496','0.307298550794363','6','86']
  H_HDV['55_FC_0_0'] = [' 55','EURO0','RT >20-26t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','740.229916004452','0.065122866435054','4881.34455644538','0.566300653450172','195.824472665819','6','86']
  H_HDV['55_CO_0_50'] = [' 55','EURO0','RT >20-26t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.63683834502667','0.058436672531358','14.1899642794278','0.277935357059671','1.59754617998694','6','86']
  H_HDV['55_THC_0_50'] = [' 55','EURO0','RT >20-26t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56130173891621','0.030240851677469','5.11689372558113','0.122729520885325','0.30866299515759','6','86']
  H_HDV['55_NOx_0_50'] = [' 55','EURO0','RT >20-26t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.5027718516142','0.048924595180432','70.7109869452517','0.469604728194189','9.48970447579185','6','86']
  H_HDV['55_PM_0_50'] = [' 55','EURO0','RT >20-26t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.50776852421109','0.053664385657064','2.19963038468657','0.23916847480486','0.347195590790528','6','86']
  H_HDV['55_FC_0_50'] = [' 55','EURO0','RT >20-26t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','694.902206994305','0.054658595405341','4400.70477971901','0.535794966062572','220.495605873485','6','86']
  H_HDV['55_CO_0_100'] = [' 55','EURO0','RT >20-26t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.9376980346767','0.057148652166947','18.4718399476647','0.31609038188986','1.77301144294452','6','86']
  H_HDV['55_THC_0_100'] = [' 55','EURO0','RT >20-26t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56499268040546','0.031519173212444','5.22799426505423','0.127820197295527','0.308126075775721','6','86']
  H_HDV['55_NOx_0_100'] = [' 55','EURO0','RT >20-26t 80ties','0','100','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.027099565224408','0.001201578287346','-0.000005344062225','0','0','6','86']
  H_HDV['55_PM_0_100'] = [' 55','EURO0','RT >20-26t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54498556312325','0.050278639292838','2.9222080596227','0.289435449706135','0.383349139473231','6','86']
  H_HDV['55_FC_0_100'] = [' 55','EURO0','RT >20-26t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','669.440727875052','0.044575445935387','5412.19017317538','0.567152644388354','238.810628222996','6','86']
  H_HDV['55_CO_-2_0'] = [' 55','EURO0','RT >20-26t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.2160341723648','0.011834129584264','-0.770631217545629','0.064846822997854','0','6','86']
  H_HDV['55_THC_-2_0'] = [' 55','EURO0','RT >20-26t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.39796016793396','0.069437930165384','3.14611246946102','0.380056070772103','0.401960963539563','6','86']
  H_HDV['55_NOx_-2_0'] = [' 55','EURO0','RT >20-26t 80ties','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.4312830893658','0.062165440501174','445.337948444753','0.77913597831716','3.05515238434555','6','86']
  H_HDV['55_PM_-2_0'] = [' 55','EURO0','RT >20-26t 80ties','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.199180524138748','1.96293608319966','3.98775372893934','1.4312306453707','0.031651228260479','6','86']
  H_HDV['55_FC_-2_0'] = [' 55','EURO0','RT >20-26t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','698.906482169576','0.065826713120222','32653.4086407295','0.883427095641758','84.151313428091','6','86']
  H_HDV['55_CO_-2_50'] = [' 55','EURO0','RT >20-26t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.3342003125023','0.008977830484395','-0.777814166537506','0.0647136613457','0','6','86']
  H_HDV['55_THC_-2_50'] = [' 55','EURO0','RT >20-26t 80ties','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.260932920034621','14.7613147373821','-0.14014408430389','0.477050482898328','0.045358197118597','6','86']
  H_HDV['55_NOx_-2_50'] = [' 55','EURO0','RT >20-26t 80ties','-2','50','NOx','y=((a*(b**x))*(x**c))','54.4976793001134','0.986905905651676','-0.485863437820001','0','0','6','86']
  H_HDV['55_PM_-2_50'] = [' 55','EURO0','RT >20-26t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.95985349400508','0.062482564714904','1.22476070958367','0.275982219338776','0.182132591326166','6','86']
  H_HDV['55_FC_-2_50'] = [' 55','EURO0','RT >20-26t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','595.119229871375','0.047605329244398','24184.0531431285','0.790500565816877','50.8056836538696','6','86']
  H_HDV['55_CO_-2_100'] = [' 55','EURO0','RT >20-26t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0009101129272','0.062034531716711','115.419324850359','0.730166763217799','0.728274517270591','6','86']
  H_HDV['55_THC_-2_100'] = [' 55','EURO0','RT >20-26t 80ties','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.18923669502195','22.31739505235','-0.680245727065833','0.478196321406747','0.040070193069197','6','86']
  H_HDV['55_NOx_-2_100'] = [' 55','EURO0','RT >20-26t 80ties','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.055548988462094','0.956420434265464','52.8605415193215','-0.514659926928445','0','6','86']
  H_HDV['55_PM_-2_100'] = [' 55','EURO0','RT >20-26t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.85780918093971','0.056470326788617','3.44728752040675','0.395935541944194','0.167777487919394','6','86']
  H_HDV['55_FC_-2_100'] = [' 55','EURO0','RT >20-26t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','561.429950659975','0.037549214019242','29920.3873287263','0.811158473391288','26.73470686761','6','86']
  H_HDV['55_CO_-4_0'] = [' 55','EURO0','RT >20-26t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.6150703876822','-0.006600335515687','-0.888659952554105','0.081872464799487','0','6','86']
  H_HDV['55_THC_-4_0'] = [' 55','EURO0','RT >20-26t 80ties','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.142826987022537','26.6603957700286','-0.943866803461456','0.468927565906561','0.041643962670957','6','86']
  H_HDV['55_NOx_-4_0'] = [' 55','EURO0','RT >20-26t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.3650538041195','0.052987425457157','281.886970075062','0.638051882127413','0.474165456511353','6','86']
  H_HDV['55_PM_-4_0'] = [' 55','EURO0','RT >20-26t 80ties','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.54151969367835','-0.001693169834974','-0.193652711604552','0.083734890847618','0','6','86']
  H_HDV['55_FC_-4_0'] = [' 55','EURO0','RT >20-26t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','505.612861867246','0.054956213235211','17242.6243575613','0.729012252752873','12.9079067918008','6','86']
  H_HDV['55_CO_-4_50'] = [' 55','EURO0','RT >20-26t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.03775280160575','0.060650450169253','61.1868311083667','0.620961381677225','0.249377473536042','6','86']
  H_HDV['55_THC_-4_50'] = [' 55','EURO0','RT >20-26t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.24873233736803','0.059279848402217','11.7832820229159','0.475518026439649','0.145108146267912','6','86']
  H_HDV['55_NOx_-4_50'] = [' 55','EURO0','RT >20-26t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.9938647341142','0.045417577056164','583.617723557315','0.756626260133192','0.220912472894677','6','86']
  H_HDV['55_PM_-4_50'] = [' 55','EURO0','RT >20-26t 80ties','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.09154709648917','8.38270620532157','-0.327014529895124','0.536673779594689','0.038780561547646','6','86']
  H_HDV['55_FC_-4_50'] = [' 55','EURO0','RT >20-26t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','437.655335809142','0.048318162730849','28514.1280035632','0.810352691119394','5.53088792728401','6','86']
  H_HDV['55_CO_-4_100'] = [' 55','EURO0','RT >20-26t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.1767042372611','0.054443446476286','134.185516840803','0.713703701557841','0.192763923286862','6','86']
  H_HDV['55_THC_-4_100'] = [' 55','EURO0','RT >20-26t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.07604202060721','0.058816689113277','17.1711700799999','0.534012161352578','0.137905736957394','6','86']
  H_HDV['55_NOx_-4_100'] = [' 55','EURO0','RT >20-26t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8989909131764','0.039533610911969','1073.58523003076','0.858623635546475','0.008009757934719','6','86']
  H_HDV['55_PM_-4_100'] = [' 55','EURO0','RT >20-26t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.57454123078944','0.055379240848177','5.16639481073708','0.440614256346208','0.100288097313524','6','86']
  H_HDV['55_FC_-4_100'] = [' 55','EURO0','RT >20-26t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','403.645855172499','0.0437287966731','42947.890075465','0.878867449013373','1.15379164274905','6','86']
  H_HDV['55_CO_-6_0'] = [' 55','EURO0','RT >20-26t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.53804315207527','0.070924231455173','60.0759946110696','0.705138519961778','0.184278374746884','6','72']
  H_HDV['55_THC_-6_0'] = [' 55','EURO0','RT >20-26t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.52083262266833','0.066800737520279','19.4350987698694','0.612484956948242','0.130946679190678','6','72']
  H_HDV['55_NOx_-6_0'] = [' 55','EURO0','RT >20-26t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.074135929045','0.064709828386955','669.069942249252','0.812570472564027','0.219915767755451','6','72']
  H_HDV['55_PM_-6_0'] = [' 55','EURO0','RT >20-26t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.77103210610772','0.066909560065046','1.19623906445639','0.267917491379214','0.101461847140634','6','72']
  H_HDV['55_FC_-6_0'] = [' 55','EURO0','RT >20-26t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','445.774327543526','0.063134176759299','26873.579271616','0.818574205397959','3.13147031211048','6','72']
  H_HDV['55_CO_-6_50'] = [' 55','EURO0','RT >20-26t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.03120151703757','0.064993107602091','110.923962871796','0.732296906784111','0.153529165762486','6','72']
  H_HDV['55_THC_-6_50'] = [' 55','EURO0','RT >20-26t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.11444663013806','0.06449593116208','28.0454540494335','0.645923935219025','0.124302815056196','6','72']
  H_HDV['55_NOx_-6_50'] = [' 55','EURO0','RT >20-26t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0612618848848','0.058718630258388','879.581214910594','0.846075315769339','0.164580531929382','6','72']
  H_HDV['55_PM_-6_50'] = [' 55','EURO0','RT >20-26t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.64793159176701','0.064543579419521','3.74290418635482','0.43579861982557','0.098492136228672','6','72']
  H_HDV['55_FC_-6_50'] = [' 55','EURO0','RT >20-26t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','370.493417179939','0.058748848253034','36009.4490677132','0.864591833949356','1.77068203418491','6','72']
  H_HDV['55_CO_-6_100'] = [' 55','EURO0','RT >20-26t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.28253856339548','0.061249970154466','133.126668206141','0.728939249250096','0.138589776173939','6','72']
  H_HDV['55_THC_-6_100'] = [' 55','EURO0','RT >20-26t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.90604237157546','0.063442482588828','28.7452304238815','0.641860982232217','0.121806635538369','6','72']
  H_HDV['55_NOx_-6_100'] = [' 55','EURO0','RT >20-26t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8469140290166','0.055663374933167','1042.03461903494','0.873680079622698','0.151168818002822','6','72']
  H_HDV['55_PM_-6_100'] = [' 55','EURO0','RT >20-26t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56458213043761','0.062604212219327','5.31283918675032','0.475254152947132','0.096844641161207','6','72']
  H_HDV['55_FC_-6_100'] = [' 55','EURO0','RT >20-26t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','333.899213726409','0.056180394999268','40063.7312995318','0.881350265934446','1.37594348791324','6','72']
  H_HDV['55_CO_2_0'] = [' 55','EURO0','RT >20-26t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.18774889396871','10.6038424379144','6.97527612909906','2.7416211419068','-0.031296422999175','6','86']
  H_HDV['55_THC_2_0'] = [' 55','EURO0','RT >20-26t 80ties','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','4.7184146095723','-5.73750397699474','-1.26731706311296','0','0','6','86']
  H_HDV['55_NOx_2_0'] = [' 55','EURO0','RT >20-26t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-23.4240786863346','0.033940606140067','52.7411045043462','0.062926229538042','16.4044195039354','6','86']
  H_HDV['55_PM_2_0'] = [' 55','EURO0','RT >20-26t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.452062797988997','1.74714146890843','6.40211780347044','2.40935606704216','0.003037272412359','6','86']
  H_HDV['55_FC_2_0'] = [' 55','EURO0','RT >20-26t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1464.12533312412','0.458324952121069','-97.6665036469738','0.083362976604219','0','6','86']
  H_HDV['55_CO_2_50'] = [' 55','EURO0','RT >20-26t 80ties','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.663920796203941','14.0365953718762','4.64633695895533','1.94191770670989','-0.022162233119108','6','86']
  H_HDV['55_THC_2_50'] = [' 55','EURO0','RT >20-26t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.266435709457952','4.92574263390947','5.88567803690185','2.3216596486037','-0.015067816680817','6','86']
  H_HDV['55_NOx_2_50'] = [' 55','EURO0','RT >20-26t 80ties','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-27.1230816357273','0.039005303146452','54.0916231881265','0.060693292937955','21.472394735325','6','86']
  H_HDV['55_PM_2_50'] = [' 55','EURO0','RT >20-26t 80ties','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.399017997796161','2.2024852483416','4.81103445492381','1.99703582535074','-0.017804970577886','6','86']
  H_HDV['55_FC_2_50'] = [' 55','EURO0','RT >20-26t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1550.74550536191','0.793224832428584','-93.6528117332658','0.080637828056058','0','6','86']
  H_HDV['55_CO_2_100'] = [' 55','EURO0','RT >20-26t 80ties','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.032514636979161','0.007893427107374','-0.00003289322666','0','0','6','85']
  H_HDV['55_THC_2_100'] = [' 55','EURO0','RT >20-26t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.405525095338871','4.67605375438739','6.42985125993312','2.55487735537662','-0.016204949742535','6','85']
  H_HDV['55_NOx_2_100'] = [' 55','EURO0','RT >20-26t 80ties','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','50.4973665701114','0.021958076227241','-1.82955019878619','0.06989578859856','0','6','85']
  H_HDV['55_PM_2_100'] = [' 55','EURO0','RT >20-26t 80ties','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.561820380661369','2.74172230288076','2.97942904389149','1.33779835037112','0.007217783564729','6','85']
  H_HDV['55_FC_2_100'] = [' 55','EURO0','RT >20-26t 80ties','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1630.11095171618','0.413244822468845','-91.6670611450537','0.08321598439544','0','6','85']
  H_HDV['55_CO_4_0'] = [' 55','EURO0','RT >20-26t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.90733216405505','11.1013804110086','6.44578290490654','2.55164096609729','-0.01754825243058','6','71']
  H_HDV['55_THC_4_0'] = [' 55','EURO0','RT >20-26t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.139611493199195','5.65155453317298','5.00391859640155','1.99700321911172','-0.009949798558176','6','71']
  H_HDV['55_NOx_4_0'] = [' 55','EURO0','RT >20-26t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','55.4013793140267','0.063481775929449','-3.41405624714972','0.091601444477499','0','6','71']
  H_HDV['55_PM_4_0'] = [' 55','EURO0','RT >20-26t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.546358539180428','1.80089754914174','6.57478683794902','2.61257266101575','-0.018563533215906','6','71']
  H_HDV['55_FC_4_0'] = [' 55','EURO0','RT >20-26t 80ties','4','0','FC','y=(c+(a*exp(b*x)))','1327.42638282845','-0.101581018914478','445.211331532456','0','0','6','71']
  H_HDV['55_CO_4_50'] = [' 55','EURO0','RT >20-26t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.5329999982122','-0.037255428971219','-1.99391783363473','0.117263472999845','0','6','70']
  H_HDV['55_THC_4_50'] = [' 55','EURO0','RT >20-26t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.47219343973977','4.75213582850642','7.14477653995222','2.85462745825113','-0.022235556046534','6','70']
  H_HDV['55_NOx_4_50'] = [' 55','EURO0','RT >20-26t 80ties','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','59.5764139963778','0.027779582244723','-2.83894278523553','0.093986668078862','0','6','70']
  H_HDV['55_PM_4_50'] = [' 55','EURO0','RT >20-26t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.61102099916183','-0.004763551743823','-0.281251720113563','0.108289077278103','0','6','70']
  H_HDV['55_FC_4_50'] = [' 55','EURO0','RT >20-26t 80ties','4','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1917.85024566755','0.418390226162854','-134.539133845089','0.104858542707191','0','6','70']
  H_HDV['55_CO_4_100'] = [' 55','EURO0','RT >20-26t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.1081459037776','-0.033939183476485','-1.857603314941','0.107123351649116','0','6','63']
  H_HDV['55_THC_4_100'] = [' 55','EURO0','RT >20-26t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.600660335078017','4.44338730195376','7.8431075811919','3.15339869451326','-0.028368571519739','6','63']
  H_HDV['55_NOx_4_100'] = [' 55','EURO0','RT >20-26t 80ties','4','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.87461350492467','1.33110885697606','-0.055427990005978','0','0','6','63']
  H_HDV['55_PM_4_100'] = [' 55','EURO0','RT >20-26t 80ties','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.70673033442343','-0.003375174908458','-0.250283646159812','0.097549193644149','0','6','63']
  H_HDV['55_FC_4_100'] = [' 55','EURO0','RT >20-26t 80ties','4','100','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','844.226137444987','4419.80159262085','-0.327219892055946','0.607496396358095','0.062167244035023','6','63']
  H_HDV['55_CO_6_0'] = [' 55','EURO0','RT >20-26t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.0018366037133','-0.042086646281711','-2.56917206594318','0.141908977566672','0','6','71']
  H_HDV['55_THC_6_0'] = [' 55','EURO0','RT >20-26t 80ties','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.62677913447103','0.004710009664989','-0.56471293445219','0.077120330479647','0','6','71']
  H_HDV['55_NOx_6_0'] = [' 55','EURO0','RT >20-26t 80ties','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','60.1069403512305','0.069924121497982','-3.30527379855004','0.093480931612351','0','6','71']
  H_HDV['55_PM_6_0'] = [' 55','EURO0','RT >20-26t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.89629879346184','-0.00579983620418','-0.385494975815255','0.135707266171115','0','6','71']
  H_HDV['55_FC_6_0'] = [' 55','EURO0','RT >20-26t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1746.4356778395','3.63843416378099','-97.1835826015269','0.072592822100276','0','6','71']
  H_HDV['55_CO_6_50'] = [' 55','EURO0','RT >20-26t 80ties','6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.086027424979','14.2178906135824','4.95695086236184','2.03698298444011','-0.015035485987556','6','60']
  H_HDV['55_THC_6_50'] = [' 55','EURO0','RT >20-26t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.584899761025057','4.49603450699591','8.1786210387368','3.28579402865473','-0.03369917152137','6','60']
  H_HDV['55_NOx_6_50'] = [' 55','EURO0','RT >20-26t 80ties','6','50','NOx','y=((a*(b**x))*(x**c))','82.6108158136512','1.00460566481991','-0.237162133136272','0','0','6','60']
  H_HDV['55_PM_6_50'] = [' 55','EURO0','RT >20-26t 80ties','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.233624374797252','0.027380674544791','-0.000251825058419','0','0','6','60']
  H_HDV['55_FC_6_50'] = [' 55','EURO0','RT >20-26t 80ties','6','50','FC','y=((a*(b**x))*(x**c))','3183.43875299389','1.00918198420678','-0.447545592578081','0','0','6','60']
  H_HDV['55_CO_6_100'] = [' 55','EURO0','RT >20-26t 80ties','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.0265395785106','-0.073299082944088','-2.58484517209377','0.133920132431036','0','6','46']
  H_HDV['55_THC_6_100'] = [' 55','EURO0','RT >20-26t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.987795297198059','29.0764756241362','-2.29268934047209','-0.794627610436881','0.178409446706685','6','46']
  H_HDV['55_NOx_6_100'] = [' 55','EURO0','RT >20-26t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.6511366982928','0.019217577724274','151.397232142813','0.522501884932757','46.5503159340615','6','46']
  H_HDV['55_PM_6_100'] = [' 55','EURO0','RT >20-26t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.4683463662595','-0.009768325921328','-0.369519039331487','0.131833045645274','0','6','46']
  H_HDV['55_FC_6_100'] = [' 55','EURO0','RT >20-26t 80ties','6','100','FC','y=exp((a+(b/x))+(c*log(x)))','7.18530174901071','2.17700035427671','-0.047256122169206','0','0','6','46']
  H_HDV['56_CO_0_0'] = [' 56','EURO1','RT >20-26t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.10475472219707','0.054943224029261','8.21633778619375','0.222107019626088','1.07451851792051','6','86']
  H_HDV['56_THC_0_0'] = [' 56','EURO1','RT >20-26t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.182360260322728','5.07229716524162','3.0683164749628','1.4484121911196','-0.004457965587001','6','86']
  H_HDV['56_NOx_0_0'] = [' 56','EURO1','RT >20-26t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.3562874176997','0.067643907990714','60.5322213445908','0.469778752608248','5.8826631754264','6','86']
  H_HDV['56_PM_0_0'] = [' 56','EURO1','RT >20-26t Euro-1','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002137398946117','2.80411895450092','3.71362364295426','1.71057750001636','-0.016354115520719','6','86']
  H_HDV['56_FC_0_0'] = [' 56','EURO1','RT >20-26t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','569.115417267766','0.065399446599477','16111.8162322336','0.817995724827678','165.909351047651','6','86']
  H_HDV['56_CO_0_50'] = [' 56','EURO1','RT >20-26t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.02325724057499','0.053857589353214','13.0719923773089','0.273316459483587','1.26708109918295','6','86']
  H_HDV['56_THC_0_50'] = [' 56','EURO1','RT >20-26t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.186871672321732','4.97886535641238','3.15415611694986','1.47225859769382','-0.004325738697539','6','86']
  H_HDV['56_NOx_0_50'] = [' 56','EURO1','RT >20-26t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6798879225676','0.050099700427571','96.6621930724731','0.544560812728235','6.71861593117207','6','86']
  H_HDV['56_PM_0_50'] = [' 56','EURO1','RT >20-26t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.32375323669085','0.053588773600498','2.53946324145554','0.212808453059734','0.234474892980576','6','86']
  H_HDV['56_FC_0_50'] = [' 56','EURO1','RT >20-26t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','541.947111180293','0.053045715384043','20605.0049129905','0.848950493597528','192.505926451822','6','86']
  H_HDV['56_CO_0_100'] = [' 56','EURO1','RT >20-26t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.23070178972467','0.051728758001272','17.5587419478383','0.317932069963899','1.52043584863354','6','86']
  H_HDV['56_THC_0_100'] = [' 56','EURO1','RT >20-26t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.34852133056729','0.030479780372225','4.00936491704337','0.12637509551278','0.313921018227142','6','86']
  H_HDV['56_NOx_0_100'] = [' 56','EURO1','RT >20-26t Euro-1','0','100','NOx','y=((a+(b*x))**((-1)/c))','0.000691473620584','0.000177645059871','2.06976466938541','0','0','6','86']
  H_HDV['56_PM_0_100'] = [' 56','EURO1','RT >20-26t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.32777412722157','0.050417962863292','2.9149627604072','0.246419621433001','0.270297217346658','6','86']
  H_HDV['56_FC_0_100'] = [' 56','EURO1','RT >20-26t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','538.853915617603','0.042449749365274','37865.679997208','0.952632313946179','210.266413035277','6','86']
  H_HDV['56_CO_-2_0'] = [' 56','EURO1','RT >20-26t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.55329327183273','0.00872050949742','-0.617114348995541','0.066711892473182','0','6','86']
  H_HDV['56_THC_-2_0'] = [' 56','EURO1','RT >20-26t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.312762709960771','7.74567971441597','0.671415315945207','0.560293227493921','0.05107089919498','6','86']
  H_HDV['56_NOx_-2_0'] = [' 56','EURO1','RT >20-26t Euro-1','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.3702092339565','0.061282038500653','132.68210846151','0.594075259739471','2.09148030795815','6','86']
  H_HDV['56_PM_-2_0'] = [' 56','EURO1','RT >20-26t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.077624884622743','1.88265140936611','7.75222482624983','3.03562392734508','-0.034913382661501','6','86']
  H_HDV['56_FC_-2_0'] = [' 56','EURO1','RT >20-26t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','540.975067527055','0.063666592406822','29247.1007908411','0.905858178432333','65.4819547379629','6','86']
  H_HDV['56_CO_-2_50'] = [' 56','EURO1','RT >20-26t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.60198566130132','0.007517306402733','-0.608537172342276','0.064919457897309','0','6','86']
  H_HDV['56_THC_-2_50'] = [' 56','EURO1','RT >20-26t Euro-1','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.19454510786306','9.18707582211749','0.454866948978516','0.618322712184298','0.037834407483516','6','86']
  H_HDV['56_NOx_-2_50'] = [' 56','EURO1','RT >20-26t Euro-1','-2','50','NOx','y=((a*(b**x))*(x**c))','44.9360510641788','0.98755574401333','-0.533860446816991','0','0','6','86']
  H_HDV['56_PM_-2_50'] = [' 56','EURO1','RT >20-26t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.121736762655157','1.92451642153519','5.82523807683331','2.22235232966264','0.002560721676506','6','86']
  H_HDV['56_FC_-2_50'] = [' 56','EURO1','RT >20-26t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','478.710502194481','0.04635905332709','22235.219460587','0.825764756089778','42.7002957995822','6','86']
  H_HDV['56_CO_-2_100'] = [' 56','EURO1','RT >20-26t Euro-1','-2','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.540359255179793','110.748842586022','-1.84542391066646','0.380955819188077','0.044337633284664','6','86']
  H_HDV['56_THC_-2_100'] = [' 56','EURO1','RT >20-26t Euro-1','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.124353230893932','40.2737442496541','-1.5554259399615','0.492010017443436','0.033336563676212','6','86']
  H_HDV['56_NOx_-2_100'] = [' 56','EURO1','RT >20-26t Euro-1','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.007193014066033','1.29160406241381','46.2086491832666','-0.598300739573146','0','6','86']
  H_HDV['56_PM_-2_100'] = [' 56','EURO1','RT >20-26t Euro-1','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.18394580052213','0.05177697427158','1.75472900081924','0.128312490296083','0.118487532081901','6','86']
  H_HDV['56_FC_-2_100'] = [' 56','EURO1','RT >20-26t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','465.859158910507','0.03647360047131','34075.8752231557','0.880835780895027','23.496377775569','6','86']
  H_HDV['56_CO_-4_0'] = [' 56','EURO1','RT >20-26t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.10583735945559','-0.00633791548479','-0.731455523301346','0.086503516602539','0','6','86']
  H_HDV['56_THC_-4_0'] = [' 56','EURO1','RT >20-26t Euro-1','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.201574515011926','0.024310305618049','0.00089955999967','0','0','6','86']
  H_HDV['56_NOx_-4_0'] = [' 56','EURO1','RT >20-26t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.6470626270775','0.055154958467551','121.447282930024','0.533991131356109','0.383144815758847','6','86']
  H_HDV['56_PM_-4_0'] = [' 56','EURO1','RT >20-26t Euro-1','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012555350489632','2.00399852064812','5.87847458788874','2.33572906151755','-0.012702907737702','6','86']
  H_HDV['56_FC_-4_0'] = [' 56','EURO1','RT >20-26t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','415.137606275759','0.05515395533242','13450.7682509145','0.736570007916745','11.1565615829159','6','86']
  H_HDV['56_CO_-4_50'] = [' 56','EURO1','RT >20-26t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.77655571749498','0.058307500145385','42.0170388131141','0.555594752606173','0.164721511315331','6','86']
  H_HDV['56_THC_-4_50'] = [' 56','EURO1','RT >20-26t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.51997623233116','0.057011265557194','8.754352298621','0.430970239341427','0.093117641450264','6','86']
  H_HDV['56_NOx_-4_50'] = [' 56','EURO1','RT >20-26t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7544255986432','0.047988955754962','242.68104788731','0.640592301980751','0.229544258994156','6','86']
  H_HDV['56_PM_-4_50'] = [' 56','EURO1','RT >20-26t Euro-1','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.76688047060901','-0.001887655516672','-0.247311249398293','0.096176952189751','0','6','86']
  H_HDV['56_FC_-4_50'] = [' 56','EURO1','RT >20-26t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','365.616514353386','0.047925813201528','21129.6485081513','0.802478213025886','4.97740180400992','6','86']
  H_HDV['56_CO_-4_100'] = [' 56','EURO1','RT >20-26t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.22861804360064','0.051592662255203','100.430323435476','0.677406817201522','0.113933395483156','6','86']
  H_HDV['56_THC_-4_100'] = [' 56','EURO1','RT >20-26t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.34886137496946','0.055727537485993','12.6323552392217','0.485892791923225','0.085878195578447','6','86']
  H_HDV['56_NOx_-4_100'] = [' 56','EURO1','RT >20-26t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.83069781337149','0.042391831381199','401.635789084664','0.721575458249892','0.106036022739592','6','86']
  H_HDV['56_PM_-4_100'] = [' 56','EURO1','RT >20-26t Euro-1','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.277918761253716','0.050287398461456','0.00167368608329','0','0','6','86']
  H_HDV['56_FC_-4_100'] = [' 56','EURO1','RT >20-26t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','343.165644149903','0.043095815499989','29449.4591565562','0.854046654770887','1.06292218712649','6','86']
  H_HDV['56_CO_-6_0'] = [' 56','EURO1','RT >20-26t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.42385706771641','0.072024369468036','34.4231611337993','0.620982959074101','0.125069531529366','6','72']
  H_HDV['56_THC_-6_0'] = [' 56','EURO1','RT >20-26t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.87011518365705','0.067641457654277','14.6138168535815','0.587766975807851','0.088415077125348','6','72']
  H_HDV['56_NOx_-6_0'] = [' 56','EURO1','RT >20-26t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.2151783896526','0.067815735002097','270.228830067476','0.695512964980835','0.212743856775462','6','72']
  H_HDV['56_PM_-6_0'] = [' 56','EURO1','RT >20-26t Euro-1','-6','0','PM','y=(c+(a*exp(b*x)))','2.50619251580008','-0.084518810430853','0.068515709251499','0','0','6','72']
  H_HDV['56_FC_-6_0'] = [' 56','EURO1','RT >20-26t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','376.082521727274','0.063776577495986','19953.1710313521','0.816415255256443','3.57059747939786','6','72']
  H_HDV['56_CO_-6_50'] = [' 56','EURO1','RT >20-26t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.21603804528427','0.064988079797544','72.7636793974354','0.673523655761105','0.099351668202819','6','72']
  H_HDV['56_THC_-6_50'] = [' 56','EURO1','RT >20-26t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.47734496274374','0.064690457341248','22.7858701622498','0.629187913038208','0.082203233129045','6','72']
  H_HDV['56_NOx_-6_50'] = [' 56','EURO1','RT >20-26t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.78201571322695','0.062039033017557','415.405674933551','0.751028836965888','0.175399642738525','6','72']
  H_HDV['56_PM_-6_50'] = [' 56','EURO1','RT >20-26t Euro-1','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.65351742197159','0.07025279528022','3.66280754718687','0.39795232201378','0.055660665626596','6','72']
  H_HDV['56_FC_-6_50'] = [' 56','EURO1','RT >20-26t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','318.057397757129','0.059239701493802','25425.3545046969','0.847357449538044','2.44857946938444','6','72']
  H_HDV['56_CO_-6_100'] = [' 56','EURO1','RT >20-26t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.69776972058826','0.061132508086455','98.5789210035936','0.702507599272399','0.089251970057348','6','72']
  H_HDV['56_THC_-6_100'] = [' 56','EURO1','RT >20-26t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.28448292514464','0.063272126089945','24.8401858406204','0.637916405646298','0.079908592596861','6','72']
  H_HDV['56_NOx_-6_100'] = [' 56','EURO1','RT >20-26t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.78209070667405','0.0589983546996','492.765241052284','0.778943657562932','0.163627464551065','6','72']
  H_HDV['56_PM_-6_100'] = [' 56','EURO1','RT >20-26t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.49787261176068','0.06749646531418','6.15884839052752','0.479129527498758','0.053538723688181','6','72']
  H_HDV['56_FC_-6_100'] = [' 56','EURO1','RT >20-26t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','288.844783910078','0.05629012455569','29048.0903334891','0.866799223185938','1.97185736268712','6','72']
  H_HDV['56_CO_2_0'] = [' 56','EURO1','RT >20-26t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.4907960566203','0.008565172596317','-0.800556353725719','0.069513194904592','0','6','86']
  H_HDV['56_THC_2_0'] = [' 56','EURO1','RT >20-26t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.235081278118389','4.39158366678474','4.36139218349309','1.77257528815844','-0.006984536238722','6','86']
  H_HDV['56_NOx_2_0'] = [' 56','EURO1','RT >20-26t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.7478684537787','0.013545650387901','-2.37534725281517','0.094708172052168','0','6','86']
  H_HDV['56_PM_2_0'] = [' 56','EURO1','RT >20-26t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.278148856504034','1.74483907448071','6.536108843375','2.52704371119925','-0.00387746322944','6','86']
  H_HDV['56_FC_2_0'] = [' 56','EURO1','RT >20-26t Euro-1','2','0','FC','y=((a*(b**x))*(x**c))','2593.39403394192','1.01106606654896','-0.70009757104147','0','0','6','86']
  H_HDV['56_CO_2_50'] = [' 56','EURO1','RT >20-26t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.7083428817934','0.015815634865386','-0.922618707518269','0.074335588531437','0','6','86']
  H_HDV['56_THC_2_50'] = [' 56','EURO1','RT >20-26t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.363878004459274','3.8110267396191','6.11201746807306','2.42343428036753','-0.021074426967298','6','86']
  H_HDV['56_NOx_2_50'] = [' 56','EURO1','RT >20-26t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.3719844572708','-0.016195122833614','-2.13958515689041','0.098481430117291','0','6','86']
  H_HDV['56_PM_2_50'] = [' 56','EURO1','RT >20-26t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.74486267363692','0.002080612327882','-0.189086814241961','0.077530807858099','0','6','86']
  H_HDV['56_FC_2_50'] = [' 56','EURO1','RT >20-26t Euro-1','2','50','FC','y=((a*(b**x))*(x**c))','2277.33056914171','1.00872657986704','-0.557465772994623','0','0','6','86']
  H_HDV['56_CO_2_100'] = [' 56','EURO1','RT >20-26t Euro-1','2','100','CO','y=(c+(a*exp(b*x)))','12.3188056307292','-0.08559414930294','2.60332600183026','0','0','6','86']
  H_HDV['56_THC_2_100'] = [' 56','EURO1','RT >20-26t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.576921646072652','3.57213925170101','6.43615714634177','2.54461585715911','-0.015292195820124','6','86']
  H_HDV['56_NOx_2_100'] = [' 56','EURO1','RT >20-26t Euro-1','2','100','NOx','y=((a*(b**x))*(x**c))','51.0802574070522','1.0031189039184','-0.313404691592745','0','0','6','86']
  H_HDV['56_PM_2_100'] = [' 56','EURO1','RT >20-26t Euro-1','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.8260535750686','0.000811307047146','-0.200751269532614','0.086561828648917','0','6','86']
  H_HDV['56_FC_2_100'] = [' 56','EURO1','RT >20-26t Euro-1','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','98646.40778266','1.17577258807575','654.863272283957','0.074304241032855','489.097067613273','6','86']
  H_HDV['56_CO_4_0'] = [' 56','EURO1','RT >20-26t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.13296795477359','63.0472775735187','-2.73817938386726','-0.957080353828128','0.178927616246399','6','71']
  H_HDV['56_THC_4_0'] = [' 56','EURO1','RT >20-26t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.370892155934762','4.17230828476553','5.36337257471922','2.1404489483107','-0.011885071051326','6','71']
  H_HDV['56_NOx_4_0'] = [' 56','EURO1','RT >20-26t Euro-1','4','0','NOx','y=((a*(b**x))*(x**c))','66.2630745921822','1.00963035699395','-0.506410144417625','0','0','6','71']
  H_HDV['56_PM_4_0'] = [' 56','EURO1','RT >20-26t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.395329442900974','1.49235512019674','6.68835901502256','2.50371830153725','0.00912777685251','6','71']
  H_HDV['56_FC_4_0'] = [' 56','EURO1','RT >20-26t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','2512.96023860401','1.01045760401892','-0.600280848831566','0','0','6','71']
  H_HDV['56_CO_4_50'] = [' 56','EURO1','RT >20-26t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.1424613838495','-0.015486462780345','-1.63412662585264','0.121190619358456','0','6','71']
  H_HDV['56_THC_4_50'] = [' 56','EURO1','RT >20-26t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.692756117623682','3.51050249698816','7.48532146666575','2.98064098116035','-0.024978620621563','6','71']
  H_HDV['56_NOx_4_50'] = [' 56','EURO1','RT >20-26t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.1474870367893','0.063344410682922','37.2483020883646','0.382415525934894','19.9731731792146','6','71']
  H_HDV['56_PM_4_50'] = [' 56','EURO1','RT >20-26t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000013857805508','0.002117194678055','-0.105293648461329','2.34369994266484','0','6','71']
  H_HDV['56_FC_4_50'] = [' 56','EURO1','RT >20-26t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','732.44289735805','0.094157899291287','20666.6045556937','0.939959032367852','582.503298731439','6','71']
  H_HDV['56_CO_4_100'] = [' 56','EURO1','RT >20-26t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.1299781366608','-0.048732571725971','-2.07627023356956','0.155164453595479','0','6','65']
  H_HDV['56_THC_4_100'] = [' 56','EURO1','RT >20-26t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.859075044834754','3.26324430502049','8.58221145031073','3.47538944666556','-0.043030492006816','6','65']
  H_HDV['56_NOx_4_100'] = [' 56','EURO1','RT >20-26t Euro-1','4','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.68878849128705','1.03815880056061','-0.114254697632416','0','0','6','65']
  H_HDV['56_PM_4_100'] = [' 56','EURO1','RT >20-26t Euro-1','4','100','PM','y=(c+(a*exp(b*x)))','1.93967701281364','-0.102437680626349','0.851966141984226','0','0','6','65']
  H_HDV['56_FC_4_100'] = [' 56','EURO1','RT >20-26t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','577.447778803396','0.073710220002099','11632.7669040466','0.771481761874969','746.954982280335','6','65']
  H_HDV['56_CO_6_0'] = [' 56','EURO1','RT >20-26t Euro-1','6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.23126734937724','0.108964002946188','12.5617577066377','0.108461619459099','2.63585733392944','6','71']
  H_HDV['56_THC_6_0'] = [' 56','EURO1','RT >20-26t Euro-1','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.09385531174858','0.002700423659492','-0.440424325206069','0.079906737081092','0','6','71']
  H_HDV['56_NOx_6_0'] = [' 56','EURO1','RT >20-26t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','46.7047323883209','-0.021459110046837','-3.55332295334061','0.133817068411141','0','6','71']
  H_HDV['56_PM_6_0'] = [' 56','EURO1','RT >20-26t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.55483555180981','0.004432722635456','-0.167661365966258','0.075277111450169','0','6','71']
  H_HDV['56_FC_6_0'] = [' 56','EURO1','RT >20-26t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2534.88854365591','1.01110513101904','-0.543948251948079','0','0','6','71']
  H_HDV['56_CO_6_50'] = [' 56','EURO1','RT >20-26t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.6302247526231','-0.04080940487746','-2.63953075527524','0.175968393640723','0','6','61']
  H_HDV['56_THC_6_50'] = [' 56','EURO1','RT >20-26t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.71666506930459','3.54854615348073','7.86786497087304','3.22010223778091','-0.0477582690683','6','61']
  H_HDV['56_NOx_6_50'] = [' 56','EURO1','RT >20-26t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.378921596276','0.05875395429609','33.472066871817','0.33996774295452','26.3717708890365','6','61']
  H_HDV['56_PM_6_50'] = [' 56','EURO1','RT >20-26t Euro-1','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.321789278436579','0.038022915969136','-0.000416369599627','0','0','6','61']
  H_HDV['56_FC_6_50'] = [' 56','EURO1','RT >20-26t Euro-1','6','50','FC','y=((a*(b**x))*(x**c))','2232.12478715428','1.00676543009778','-0.355786579335089','0','0','6','61']
  H_HDV['56_CO_6_100'] = [' 56','EURO1','RT >20-26t Euro-1','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.73664683464','-0.032158870395664','-1.98806262563058','0.150552896781231','0','6','46']
  H_HDV['56_THC_6_100'] = [' 56','EURO1','RT >20-26t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.878980287627916','3.76899920361386','5.72043409441644','2.28842740289403','-0.012397690222378','6','46']
  H_HDV['56_NOx_6_100'] = [' 56','EURO1','RT >20-26t Euro-1','6','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.81013448047011','1.17457746621445','-0.070177233552353','0','0','6','46']
  H_HDV['56_PM_6_100'] = [' 56','EURO1','RT >20-26t Euro-1','6','100','PM','y=exp((a+(b/x))+(c*log(x)))','0.651397948098548','2.08129749323916','-0.148524286391952','0','0','6','46']
  H_HDV['56_FC_6_100'] = [' 56','EURO1','RT >20-26t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','485.605082544549','0.029806000405597','4305.51494349111','0.506617764365171','871.093439107736','6','46']
  H_HDV['57_CO_0_0'] = [' 57','EURO2','RT >20-26t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.28116390429354','0.061068701788236','5.46935547470328','0.253486109399985','0.95536846045838','6','86']
  H_HDV['57_THC_0_0'] = [' 57','EURO2','RT >20-26t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.104170541938868','3.3438371616432','2.8754562291575','1.38764178130742','-0.003837569597086','6','86']
  H_HDV['57_NOx_0_0'] = [' 57','EURO2','RT >20-26t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.9851308653745','0.063856922912371','41.9028445745031','0.371467068238129','6.2812899981471','6','86']
  H_HDV['57_PM_0_0'] = [' 57','EURO2','RT >20-26t Euro-2','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.551226164277287','0.000516642171294','-0.028854353756764','0.060767454757402','0','6','86']
  H_HDV['57_FC_0_0'] = [' 57','EURO2','RT >20-26t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000916497331915','0.000141642917348','-0.000000980972743','0','0','6','86']
  H_HDV['57_CO_0_50'] = [' 57','EURO2','RT >20-26t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.074600778076438','0.018146273843162','-0.000114552916985','0','0','6','86']
  H_HDV['57_THC_0_50'] = [' 57','EURO2','RT >20-26t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.818490371773208','0.028840619125953','2.58536117483348','0.122596668495594','0.188097207524592','6','86']
  H_HDV['57_NOx_0_50'] = [' 57','EURO2','RT >20-26t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.1995768514064','0.049649510549487','53.9017333205519','0.4260269734257','7.00624085360509','6','86']
  H_HDV['57_PM_0_50'] = [' 57','EURO2','RT >20-26t Euro-2','0','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.528833455074519','0.003535772284321','-0.017985502544777','0.029543627354746','0','6','86']
  H_HDV['57_FC_0_50'] = [' 57','EURO2','RT >20-26t Euro-2','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001072779195402','0.00009652935372','-0.000000579861717','','','6','86']
  H_HDV['57_CO_0_100'] = [' 57','EURO2','RT >20-26t Euro-2','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.079078810588334','0.014756524023586','-0.000091025740544','0','0','6','86']
  H_HDV['57_THC_0_100'] = [' 57','EURO2','RT >20-26t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.750074629791472','0.023158532140787','2.65632298542654','0.118508533114589','0.148342471388905','6','86']
  H_HDV['57_NOx_0_100'] = [' 57','EURO2','RT >20-26t Euro-2','0','100','NOx','y=exp((a+(b/x))+(c*log(x)))','4.18873360238733','-0.967403822194819','-0.48305536262514','0','0','6','86']
  H_HDV['57_PM_0_100'] = [' 57','EURO2','RT >20-26t Euro-2','0','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000101588396997','-0.012485831266778','0.541280378410087','0','0','6','86']
  H_HDV['57_FC_0_100'] = [' 57','EURO2','RT >20-26t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','486.56469497488','0.040125473515517','58850.8442408335','1.05430769721655','203.861205773702','6','86']
  H_HDV['57_CO_-2_0'] = [' 57','EURO2','RT >20-26t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.32839612658293','0.007453048904181','-0.397375092020362','0.064230179132998','0','6','86']
  H_HDV['57_THC_-2_0'] = [' 57','EURO2','RT >20-26t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.195104741758799','8.28321761290849','-0.194512686106922','0.490187518063859','0.045915913870628','6','86']
  H_HDV['57_NOx_-2_0'] = [' 57','EURO2','RT >20-26t Euro-2','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.7069070263255','0.060301903369367','99.5137927798246','0.530451524224961','2.30360606244684','6','86']
  H_HDV['57_PM_-2_0'] = [' 57','EURO2','RT >20-26t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.541025295782061','0.00039366862436','-0.031460721200479','0.061598287889568','0','6','86']
  H_HDV['57_FC_-2_0'] = [' 57','EURO2','RT >20-26t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','482.257501410432','0.062054285804544','24834.6805501182','0.904111334769466','61.8566821821796','6','86']
  H_HDV['57_CO_-2_50'] = [' 57','EURO2','RT >20-26t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.41345104223132','0.00745710676719','-0.381888441603282','0.060512131256359','0','6','86']
  H_HDV['57_THC_-2_50'] = [' 57','EURO2','RT >20-26t Euro-2','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.124370139785687','7.19558370392339','0.159902880280286','0.622759202165119','0.032479113009318','6','86']
  H_HDV['57_NOx_-2_50'] = [' 57','EURO2','RT >20-26t Euro-2','-2','50','NOx','y=((a*(b**x))*(x**c))','49.0592473454225','0.987394431620762','-0.532426282532252','0','0','6','86']
  H_HDV['57_PM_-2_50'] = [' 57','EURO2','RT >20-26t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.546936459712233','0.000626323337012','-0.028465584380841','0.054086478817219','0','6','86']
  H_HDV['57_FC_-2_50'] = [' 57','EURO2','RT >20-26t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','431.143034941872','0.044541116451758','20168.7100590351','0.835435583656542','40.2984803731959','6','86']
  H_HDV['57_CO_-2_100'] = [' 57','EURO2','RT >20-26t Euro-2','-2','100','CO','y=(c+(a*exp(b*x)))','6.24466257665879','-0.065720660323932','0.538586974006759','0','0','6','86']
  H_HDV['57_THC_-2_100'] = [' 57','EURO2','RT >20-26t Euro-2','-2','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.26236544755173','0.04501272954625','0.000572479137054','0','0','6','86']
  H_HDV['57_NOx_-2_100'] = [' 57','EURO2','RT >20-26t Euro-2','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.026005448205978','1.03455531550828','49.3334573638678','-0.584122668625241','0','6','86']
  H_HDV['57_PM_-2_100'] = [' 57','EURO2','RT >20-26t Euro-2','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.470281254279384','0.052404428480326','13.3540975028407','0.901324275592218','0.059071575126618','6','86']
  H_HDV['57_FC_-2_100'] = [' 57','EURO2','RT >20-26t Euro-2','-2','100','FC','y=((a*(b**x))*(x**c))','1006.34882650337','0.981942297592748','-0.365895576119088','0','0','6','86']
  H_HDV['57_CO_-4_0'] = [' 57','EURO2','RT >20-26t Euro-2','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.129722309800198','0.016577642060806','0.000663861311516','0','0','6','86']
  H_HDV['57_THC_-4_0'] = [' 57','EURO2','RT >20-26t Euro-2','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.281408469295346','0.045185491418863','0.000933843553648','0','0','6','86']
  H_HDV['57_NOx_-4_0'] = [' 57','EURO2','RT >20-26t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.82458018846','0.054978897461652','109.167647555858','0.50227203126787','0.437591379500462','6','86']
  H_HDV['57_PM_-4_0'] = [' 57','EURO2','RT >20-26t Euro-2','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.598786344727311','-0.000638646817967','-0.046800196693648','0.089217149489745','0','6','86']
  H_HDV['57_FC_-4_0'] = [' 57','EURO2','RT >20-26t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','372.294254110974','0.054290211666595','10738.5713814046','0.720661637403224','11.251197268092','6','86']
  H_HDV['57_CO_-4_50'] = [' 57','EURO2','RT >20-26t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.79434634288213','0.055207804871673','22.932302116851','0.508677779864968','0.129618932088408','6','86']
  H_HDV['57_THC_-4_50'] = [' 57','EURO2','RT >20-26t Euro-2','-4','50','THC','y=((a*(b**x))*(x**c))','5.79809456303235','0.982449375229306','-0.640045481838515','0','0','6','86']
  H_HDV['57_NOx_-4_50'] = [' 57','EURO2','RT >20-26t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7975217142331','0.048248764276474','227.886655149159','0.618305580543118','0.270147977602336','6','86']
  H_HDV['57_PM_-4_50'] = [' 57','EURO2','RT >20-26t Euro-2','-4','50','PM','y=((a+(b*x))**((-1)/c))','1.25137794547123','0.117780332402367','0.698206225072519','0','0','6','86']
  H_HDV['57_FC_-4_50'] = [' 57','EURO2','RT >20-26t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','329.528463146056','0.046641842724889','17224.8913810941','0.789359307313502','5.0195471693699','6','86']
  H_HDV['57_CO_-4_100'] = [' 57','EURO2','RT >20-26t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.59138369219992','0.047983732391944','53.8131249813004','0.625524841779382','0.087002074098447','6','86']
  H_HDV['57_THC_-4_100'] = [' 57','EURO2','RT >20-26t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.46486249692421','0.051575459229162','5.09272423216099','0.393397502552619','0.070442757829571','6','86']
  H_HDV['57_NOx_-4_100'] = [' 57','EURO2','RT >20-26t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7854156008509','0.043061574117763','390.621168082171','0.70720741204833','0.143907686950392','6','86']
  H_HDV['57_PM_-4_100'] = [' 57','EURO2','RT >20-26t Euro-2','-4','100','PM','y=((a*(b**x))*(x**c))','1.07527538847689','0.98445479267558','-0.508426825592711','0','0','6','86']
  H_HDV['57_FC_-4_100'] = [' 57','EURO2','RT >20-26t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','309.251357703683','0.041338846880737','22499.9774176733','0.832944825128337','0.708497727681832','6','86']
  H_HDV['57_CO_-6_0'] = [' 57','EURO2','RT >20-26t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.47554381393816','0.073336852620533','10.3482461105911','0.508344565291246','0.115087144762931','6','72']
  H_HDV['57_THC_-6_0'] = [' 57','EURO2','RT >20-26t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.78304434622824','0.063225747341507','6.74413199694742','0.513357020406855','0.079007368673559','6','72']
  H_HDV['57_NOx_-6_0'] = [' 57','EURO2','RT >20-26t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.5306265191392','0.06834103658737','247.135525261223','0.671722842774552','0.256664874185455','6','72']
  H_HDV['57_PM_-6_0'] = [' 57','EURO2','RT >20-26t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014550566472904','0.387338140336985','5.91916413704088','2.30335999792881','-0.015587057601581','6','72']
  H_HDV['57_FC_-6_0'] = [' 57','EURO2','RT >20-26t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','341.926104352466','0.063703443211994','16499.7247525347','0.809225193955819','4.40633733793413','6','72']
  H_HDV['57_CO_-6_50'] = [' 57','EURO2','RT >20-26t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.51889480260513','0.063973982527098','32.9017686211055','0.586967171823538','0.094705109986356','6','72']
  H_HDV['57_THC_-6_50'] = [' 57','EURO2','RT >20-26t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.63502774080994','0.061628130670455','8.37252315619452','0.534554543646556','0.076592468118299','6','72']
  H_HDV['57_NOx_-6_50'] = [' 57','EURO2','RT >20-26t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.736098916803','0.062484962680488','398.400842120758','0.734077878550222','0.215788931394589','6','72']
  H_HDV['57_PM_-6_50'] = [' 57','EURO2','RT >20-26t Euro-2','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','1.94679089695518','-4.53632646726686','-1.22716842109044','0','0','6','72']
  H_HDV['57_FC_-6_50'] = [' 57','EURO2','RT >20-26t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','287.639725803425','0.058844935976111','20368.0993533588','0.832904698004872','3.35077091210499','6','72']
  H_HDV['57_CO_-6_100'] = [' 57','EURO2','RT >20-26t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.34890331098692','0.060136772732628','57.0829618396706','0.67169181655725','0.086146461739179','6','72']
  H_HDV['57_THC_-6_100'] = [' 57','EURO2','RT >20-26t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54687918462672','0.060706068331476','8.74016503502524','0.536677059570592','0.075567052713912','6','72']
  H_HDV['57_NOx_-6_100'] = [' 57','EURO2','RT >20-26t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.52686099253321','0.059046845524283','508.052068794399','0.774279172866554','0.197526145395416','6','72']
  H_HDV['57_PM_-6_100'] = [' 57','EURO2','RT >20-26t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.346701602643501','0.054006078070162','0.51551755974515','0.276751113747785','0.030324149325682','6','72']
  H_HDV['57_FC_-6_100'] = [' 57','EURO2','RT >20-26t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','258.555825839632','0.05521067252031','22171.7935540654','0.845795513226263','2.68637014195847','6','72']
  H_HDV['57_CO_2_0'] = [' 57','EURO2','RT >20-26t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.31903776633038','0.006545747655095','-0.546493259081653','0.076638715089133','0','6','86']
  H_HDV['57_THC_2_0'] = [' 57','EURO2','RT >20-26t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.750325026806923','0.020932699722008','2.81156291004966','0.103195297659103','0.144586435592503','6','86']
  H_HDV['57_NOx_2_0'] = [' 57','EURO2','RT >20-26t Euro-2','2','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','27.4554696762916','0.095121588700678','10.7744011922104','0','0','6','86']
  H_HDV['57_PM_2_0'] = [' 57','EURO2','RT >20-26t Euro-2','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.594459809070383','0.00116223750796','-0.028567717135519','0.059154504863847','0','6','86']
  H_HDV['57_FC_2_0'] = [' 57','EURO2','RT >20-26t Euro-2','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','973.29232477973','0.718337568534764','-58.1446457430568','0.079171637124591','0','6','86']
  H_HDV['57_CO_2_50'] = [' 57','EURO2','RT >20-26t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.076989674929529','0.012954726787214','-0.00009294996554','0','0','6','86']
  H_HDV['57_THC_2_50'] = [' 57','EURO2','RT >20-26t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.21765819922556','2.53274357749065','5.48236872630862','2.18688961395256','-0.014215303853092','6','86']
  H_HDV['57_NOx_2_50'] = [' 57','EURO2','RT >20-26t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.117144931616','-0.025137947067675','-2.33186938606543','0.093677762735893','0','6','86']
  H_HDV['57_PM_2_50'] = [' 57','EURO2','RT >20-26t Euro-2','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.24678821795249','0.103422534302413','-0.000831171502107','0','0','6','86']
  H_HDV['57_FC_2_50'] = [' 57','EURO2','RT >20-26t Euro-2','2','50','FC','y=((a*(b**x))*(x**c))','1950.81843526869','1.00821583151387','-0.515822147005796','0','0','6','86']
  H_HDV['57_CO_2_100'] = [' 57','EURO2','RT >20-26t Euro-2','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.081092481835356','0.009425260968549','-0.000045919497227','0','0','6','86']
  H_HDV['57_THC_2_100'] = [' 57','EURO2','RT >20-26t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.328479068593306','2.35074655424764','6.145985958922','2.44039848980943','-0.0144701894121','6','86']
  H_HDV['57_NOx_2_100'] = [' 57','EURO2','RT >20-26t Euro-2','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.8349417132255','-0.034216798252328','-1.79193906361766','0.079822854099849','0','6','86']
  H_HDV['57_PM_2_100'] = [' 57','EURO2','RT >20-26t Euro-2','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.721997761475352','0.004710586999061','-0.019272048781214','0.024787109703482','0','6','86']
  H_HDV['57_FC_2_100'] = [' 57','EURO2','RT >20-26t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1097.79940481387','0.689983744417892','-43.0697163686847','0.066185383273557','0','6','86']
  H_HDV['57_CO_4_0'] = [' 57','EURO2','RT >20-26t Euro-2','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.86621005660067','4.59415136889944','9.11548584114326','3.6765606669898','-0.0500785396132','6','72']
  H_HDV['57_THC_4_0'] = [' 57','EURO2','RT >20-26t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.201545788622965','2.80945999469795','4.92687691860147','1.99481829380656','-0.010025132859417','6','72']
  H_HDV['57_NOx_4_0'] = [' 57','EURO2','RT >20-26t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.8385110229495','0.075388203716667','17.833487318381','0.195425056712663','14.8193806299045','6','72']
  H_HDV['57_PM_4_0'] = [' 57','EURO2','RT >20-26t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.41457261768578','0.096541180607426','-0.000813377524208','0','0','6','72']
  H_HDV['57_FC_4_0'] = [' 57','EURO2','RT >20-26t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','2145.13388154337','1.00997760245872','-0.558122848864737','0','0','6','72']
  H_HDV['57_CO_4_50'] = [' 57','EURO2','RT >20-26t Euro-2','4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.083724604680988','0.008790581991122','-0.000043466775005','0','0','6','71']
  H_HDV['57_THC_4_50'] = [' 57','EURO2','RT >20-26t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.361353507426986','2.52554456444223','5.9376956609583','2.40448047443415','-0.014666924153208','6','71']
  H_HDV['57_NOx_4_50'] = [' 57','EURO2','RT >20-26t Euro-2','4','50','NOx','y=((a*(b**x))*(x**c))','68.4002929311645','1.00432669537081','-0.364105190519836','0','0','6','71']
  H_HDV['57_PM_4_50'] = [' 57','EURO2','RT >20-26t Euro-2','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.22985103771316','0.059169737642973','-0.000330226926397','0','0','6','71']
  H_HDV['57_FC_4_50'] = [' 57','EURO2','RT >20-26t Euro-2','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','616.208072762423','0.092160853145255','14744.3351118586','0.895674944096208','575.839958803039','6','71']
  H_HDV['57_CO_4_100'] = [' 57','EURO2','RT >20-26t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.91827953234941','0.022158728554936','11.0071173666185','0.248722281853995','0.204380943560672','6','67']
  H_HDV['57_THC_4_100'] = [' 57','EURO2','RT >20-26t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.483379128578748','2.33784411103298','6.45743815604876','2.62516410417598','-0.018820597058456','6','67']
  H_HDV['57_NOx_4_100'] = [' 57','EURO2','RT >20-26t Euro-2','4','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','23.9321518933846','83.7629029243762','-0.122572299651608','0.537208143557322','0.034727524910338','6','67']
  H_HDV['57_PM_4_100'] = [' 57','EURO2','RT >20-26t Euro-2','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.570690594016739','0.023175999788391','3.08977069034193','0.519753161589646','0.166832449178857','6','67']
  H_HDV['57_FC_4_100'] = [' 57','EURO2','RT >20-26t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','444.791721354988','0.063467353070215','8020.52346832176','0.701542477454978','736.991528948686','6','67']
  H_HDV['57_CO_6_0'] = [' 57','EURO2','RT >20-26t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.9205475995668','-0.03528719425695','-2.30630199457774','0.215211087408347','0','6','71']
  H_HDV['57_THC_6_0'] = [' 57','EURO2','RT >20-26t Euro-2','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.418699907167361','2.46140005330937','5.51636646246632','2.00537316538357','0.025548493990736','6','71']
  H_HDV['57_NOx_6_0'] = [' 57','EURO2','RT >20-26t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','50.881134487398','-0.042022475097443','-3.65019529827024','0.122318231519613','0','6','71']
  H_HDV['57_PM_6_0'] = [' 57','EURO2','RT >20-26t Euro-2','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004113205463','0.000543558293372','-0.024920173334782','0.711718707785244','0','6','71']
  H_HDV['57_FC_6_0'] = [' 57','EURO2','RT >20-26t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','2187.32720113783','1.01055488847059','-0.502404599396862','0','0','6','71']
  H_HDV['57_CO_6_50'] = [' 57','EURO2','RT >20-26t Euro-2','6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.38357915634342','0.017624299977193','6.76305858820314','0.151753327384249','-0.150689333942396','6','63']
  H_HDV['57_THC_6_50'] = [' 57','EURO2','RT >20-26t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.489434357246713','2.33393309922054','6.88983239931445','2.79969657802108','-0.024116118627065','6','63']
  H_HDV['57_NOx_6_50'] = [' 57','EURO2','RT >20-26t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','56.2306860254879','-0.046593773581469','-2.8856278592207','0.103449107573995','0','6','63']
  H_HDV['57_PM_6_50'] = [' 57','EURO2','RT >20-26t Euro-2','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.05473246951958','0.048900931456903','-0.00025405520735','0','0','6','63']
  H_HDV['57_FC_6_50'] = [' 57','EURO2','RT >20-26t Euro-2','6','50','FC','y=((a*(b**x))*(x**c))','1951.88619151718','1.00586819586657','-0.312533055920858','0','0','6','63']
  H_HDV['57_CO_6_100'] = [' 57','EURO2','RT >20-26t Euro-2','6','100','CO','y=((a*(b**x))*(x**c))','21.4393462118149','0.987789470059953','-0.37958790041843','0','0','6','51']
  H_HDV['57_THC_6_100'] = [' 57','EURO2','RT >20-26t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.717764854674662','1.99961700351568','8.46638423690259','3.4854864580693','-0.0398765221619','6','51']
  H_HDV['57_NOx_6_100'] = [' 57','EURO2','RT >20-26t Euro-2','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','65.8348666321415','-0.109920799883121','-3.26662924170902','0.115051728533919','0','6','51']
  H_HDV['57_PM_6_100'] = [' 57','EURO2','RT >20-26t Euro-2','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.624140933312441','0.044697860562164','3.70835331275506','0.58586122189617','0.374681381708212','6','51']
  H_HDV['57_FC_6_100'] = [' 57','EURO2','RT >20-26t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','370.527528745833','0.063135293551748','6905.96122969787','0.650069808940243','1013.20239222649','6','51']
  H_HDV['58_CO_0_0'] = [' 58','EURO3','RT >20-26t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.06374397835931','0.055305957281251','9.06250332077919','0.193557240657836','1.01459562929878','6','86']
  H_HDV['58_THC_0_0'] = [' 58','EURO3','RT >20-26t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.111153422067922','3.04674326764635','3.45367800791712','1.58035577806097','-0.005309859926051','6','86']
  H_HDV['58_NOx_0_0'] = [' 58','EURO3','RT >20-26t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0806513993218','0.05981969254621','46.3435517200677','0.296244687086556','4.89188793195336','6','86']
  H_HDV['58_PM_0_0'] = [' 58','EURO3','RT >20-26t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.05934907965767','0.933551164165128','4.72775095222166','1.97252552598641','-0.01112720381612','6','86']
  H_HDV['58_FC_0_0'] = [' 58','EURO3','RT >20-26t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000771111682941','0.000136667664084','-0.000000918042656','0','0','6','86']
  H_HDV['58_CO_0_50'] = [' 58','EURO3','RT >20-26t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.67565723679917','0.057405018563615','10.9767981066719','0.247864595709354','1.2235852358684','6','86']
  H_HDV['58_THC_0_50'] = [' 58','EURO3','RT >20-26t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.121488680575649','2.91411252038413','3.56940410258337','1.61952671447323','-0.004840141643649','6','86']
  H_HDV['58_NOx_0_50'] = [' 58','EURO3','RT >20-26t Euro-3','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.8309715569804','0.04967317760913','45.25339943624','0.29961621348839','5.50024687796705','6','86']
  H_HDV['58_PM_0_50'] = [' 58','EURO3','RT >20-26t Euro-3','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.007977023254746','1.08042045993439','3.68173738405044','1.62126069624978','-0.016731661379494','6','86']
  H_HDV['58_FC_0_50'] = [' 58','EURO3','RT >20-26t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','520.710271225613','0.050951953788406','3644.25050198968','0.580686014010721','190.853379208127','6','86']
  H_HDV['58_CO_0_100'] = [' 58','EURO3','RT >20-26t Euro-3','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.39972252679608','59.25391803787','-0.843455613446116','0.484109759216111','0.043976920026393','6','86']
  H_HDV['58_THC_0_100'] = [' 58','EURO3','RT >20-26t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.634326074710769','0.024636426264743','2.67228151815791','0.117782640940105','0.145381388396244','6','86']
  H_HDV['58_NOx_0_100'] = [' 58','EURO3','RT >20-26t Euro-3','0','100','NOx','y=((a*(b**x))*(x**c))','66.1603333998834','1.00230658509238','-0.579459236932674','0','0','6','86']
  H_HDV['58_PM_0_100'] = [' 58','EURO3','RT >20-26t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.529815342258866','0.049173771854666','0.729576605967625','0.188507434375357','0.110072225062911','6','86']
  H_HDV['58_FC_0_100'] = [' 58','EURO3','RT >20-26t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','523.390930530549','0.040803184975018','12217.5149565667','0.787953327299584','207.725266017439','6','86']
  H_HDV['58_CO_-2_0'] = [' 58','EURO3','RT >20-26t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.1755616200401','0.011447494912835','-0.711382394526777','0.065245729172426','0','6','86']
  H_HDV['58_THC_-2_0'] = [' 58','EURO3','RT >20-26t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.052616863161017','2.08680164741969','6.26051106503494','2.46193789130354','-0.025416854510614','6','86']
  H_HDV['58_NOx_-2_0'] = [' 58','EURO3','RT >20-26t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.8164963023433','0.068099090031056','107.075579982552','0.501573485131276','2.02846059236167','6','86']
  H_HDV['58_PM_-2_0'] = [' 58','EURO3','RT >20-26t Euro-3','-2','0','PM','y=(c+(a*exp(b*x)))','1.08541209818028','-0.072164786968956','0.090634390917851','0','0','6','86']
  H_HDV['58_FC_-2_0'] = [' 58','EURO3','RT >20-26t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','584.41948519831','0.063476262007604','7490.91744159764','0.737653742095318','67.9163140318094','6','86']
  H_HDV['58_CO_-2_50'] = [' 58','EURO3','RT >20-26t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.6784798355098','0.013677558966156','-0.646857584903298','0.060634386174774','0','6','86']
  H_HDV['58_THC_-2_50'] = [' 58','EURO3','RT >20-26t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.0591219348219','2.01205110415962','5.60569542411743','2.17706834405148','-0.011149735740075','6','86']
  H_HDV['58_NOx_-2_50'] = [' 58','EURO3','RT >20-26t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0170204913948','0.047986353300876','117.140422547601','0.475367231557721','1.24774457307092','6','86']
  H_HDV['58_PM_-2_50'] = [' 58','EURO3','RT >20-26t Euro-3','-2','50','PM','y=(c+(a*exp(b*x)))','1.00267637977701','-0.070401066552114','0.078845031006351','0','0','6','86']
  H_HDV['58_FC_-2_50'] = [' 58','EURO3','RT >20-26t Euro-3','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','494.07050617742','0.046223651827731','6485.23156871477','0.653168684341631','45.3823749778162','6','86']
  H_HDV['58_CO_-2_100'] = [' 58','EURO3','RT >20-26t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.8246028287211','0.006427337478515','-0.661640521262763','0.063214863032982','0','6','86']
  H_HDV['58_THC_-2_100'] = [' 58','EURO3','RT >20-26t Euro-3','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.66893526122137','-0.00210641826954','-0.202966219030149','0.084199235535427','0','6','86']
  H_HDV['58_NOx_-2_100'] = [' 58','EURO3','RT >20-26t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8548333362965','0.037369187838447','171.561684837601','0.526726489305668','0.737487114972332','6','86']
  H_HDV['58_PM_-2_100'] = [' 58','EURO3','RT >20-26t Euro-3','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037422337633924','0.735853459480403','6.69516257728102','2.5016522000132','-0.0169045571894','6','86']
  H_HDV['58_FC_-2_100'] = [' 58','EURO3','RT >20-26t Euro-3','-2','100','FC','y=((a*(b**x))*(x**c))','1155.93486351938','0.982414920436287','-0.387530387371688','0','0','6','86']
  H_HDV['58_CO_-4_0'] = [' 58','EURO3','RT >20-26t Euro-3','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','6.98002488701583','-10.8081824225426','-1.82447809855067','0','0','6','86']
  H_HDV['58_THC_-4_0'] = [' 58','EURO3','RT >20-26t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.69551069342104','-0.0017623683655','-0.214003042344477','0.086068729876571','0','6','86']
  H_HDV['58_NOx_-4_0'] = [' 58','EURO3','RT >20-26t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4216835810111','0.063450916622134','123.995212221257','0.492861289114443','0.509377794889099','6','86']
  H_HDV['58_PM_-4_0'] = [' 58','EURO3','RT >20-26t Euro-3','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.030815209551813','0.776773122281118','7.23025157761348','2.71432333378609','-0.01846796793953','6','86']
  H_HDV['58_FC_-4_0'] = [' 58','EURO3','RT >20-26t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','451.782979099247','0.056140540283971','3536.34201008405','0.5432012620037','17.212867605324','6','86']
  H_HDV['58_CO_-4_50'] = [' 58','EURO3','RT >20-26t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.96717543021468','-0.007231587357306','-0.756220175149079','0.082042582456688','0','6','86']
  H_HDV['58_THC_-4_50'] = [' 58','EURO3','RT >20-26t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.44776679406758','-0.001698728853719','-0.197921982398694','0.087839392113342','0','6','86']
  H_HDV['58_NOx_-4_50'] = [' 58','EURO3','RT >20-26t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.102843375567','0.057427344401074','257.315442041724','0.611477636159434','0.436962140112658','6','86']
  H_HDV['58_PM_-4_50'] = [' 58','EURO3','RT >20-26t Euro-3','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.026588994357013','0.720382233652391','6.48284709011142','2.45872103560817','-0.015328791098996','6','86']
  H_HDV['58_FC_-4_50'] = [' 58','EURO3','RT >20-26t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','390.95189464384','0.049398279183479','7465.3722418982','0.655774510520894','12.0200131005563','6','86']
  H_HDV['58_CO_-4_100'] = [' 58','EURO3','RT >20-26t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.26932810174533','-0.008410805087151','-0.678500330751571','0.080163796515731','0','6','86']
  H_HDV['58_THC_-4_100'] = [' 58','EURO3','RT >20-26t Euro-3','-4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.061388466590787','15.5669250029716','-1.19633146781251','0.500614701208026','0.041030318986413','6','86']
  H_HDV['58_NOx_-4_100'] = [' 58','EURO3','RT >20-26t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.59272481595983','0.050987950829858','430.839806214048','0.694262598187981','0.355683645012765','6','86']
  H_HDV['58_PM_-4_100'] = [' 58','EURO3','RT >20-26t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.13134925880273','-6.7831488968422','-1.37436675412623','0','0','6','86']
  H_HDV['58_FC_-4_100'] = [' 58','EURO3','RT >20-26t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','360.759477370378','0.04404871809605','13716.2433880373','0.753910711319255','8.0506378833845','6','86']
  H_HDV['58_CO_-6_0'] = [' 58','EURO3','RT >20-26t Euro-3','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-98.9298017139813','0.918959309562847','9.67739523129291','0.078871872095314','0.23513904482262','6','72']
  H_HDV['58_THC_-6_0'] = [' 58','EURO3','RT >20-26t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.93037887194421','0.07370265834404','1.2598765300503','0.331117546732722','0.077952774474337','6','72']
  H_HDV['58_NOx_-6_0'] = [' 58','EURO3','RT >20-26t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.9749080001015','0.076597250093352','217.358710502452','0.601134150987972','0.366792466685154','6','72']
  H_HDV['58_PM_-6_0'] = [' 58','EURO3','RT >20-26t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.9564181651882','-8.91715920182577','-1.59754050212814','0','0','6','72']
  H_HDV['58_FC_-6_0'] = [' 58','EURO3','RT >20-26t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','441.865580127138','0.06758359918348','9463.83815570054','0.746832915845975','12.3842183594432','6','72']
  H_HDV['58_CO_-6_50'] = [' 58','EURO3','RT >20-26t Euro-3','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.92399362113987','0.068250550407399','12.5786841954456','0.542306986015879','0.188104692178078','6','72']
  H_HDV['58_THC_-6_50'] = [' 58','EURO3','RT >20-26t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53489176552218','0.069408681458393','6.56750648711209','0.539725763645858','0.072300572350361','6','72']
  H_HDV['58_NOx_-6_50'] = [' 58','EURO3','RT >20-26t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.28632206522862','0.067560260617474','349.104847164001','0.66277856221991','0.320730589872692','6','72']
  H_HDV['58_PM_-6_50'] = [' 58','EURO3','RT >20-26t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01890853155943','0.656943796678074','5.95876630127184','2.28259916749298','-0.014581614232442','6','72']
  H_HDV['58_FC_-6_50'] = [' 58','EURO3','RT >20-26t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','353.681893912337','0.061256430328218','14021.0812002678','0.785569685882743','10.8401255272293','6','72']
  H_HDV['58_CO_-6_100'] = [' 58','EURO3','RT >20-26t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','6.81900961841183','-0.068953861272632','0.196251829199002','0','0','6','72']
  H_HDV['58_THC_-6_100'] = [' 58','EURO3','RT >20-26t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3196180836464','0.066430958766247','10.4863582336624','0.600800395225086','0.069334421629236','6','72']
  H_HDV['58_NOx_-6_100'] = [' 58','EURO3','RT >20-26t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.79665010094927','0.062301636005549','445.693702228857','0.703749777725779','0.301020373185144','6','72']
  H_HDV['58_PM_-6_100'] = [' 58','EURO3','RT >20-26t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.019300377983587','0.64229244001025','5.13552689557835','1.9762199191721','-0.0075708304873','6','72']
  H_HDV['58_FC_-6_100'] = [' 58','EURO3','RT >20-26t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','311.64526183098','0.0569575890658','15289.466369797','0.792987900400234','9.85623609219067','6','72']
  H_HDV['58_CO_2_0'] = [' 58','EURO3','RT >20-26t Euro-3','2','0','CO','y=(c+(a*exp(b*x)))','10.9457181035539','-0.082445556372131','1.4404590878034','0','0','6','86']
  H_HDV['58_THC_2_0'] = [' 58','EURO3','RT >20-26t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.132613289533206','2.64459208726734','4.59618700383426','1.87053444772655','-0.007911135991406','6','86']
  H_HDV['58_NOx_2_0'] = [' 58','EURO3','RT >20-26t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.8516896195915','-0.014430286415083','-3.90880285709107','0.116285159605954','0','6','86']
  H_HDV['58_PM_2_0'] = [' 58','EURO3','RT >20-26t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.111511692775142','0.671714402486541','7.14645488939521','2.72153592892183','-0.017919310782066','6','86']
  H_HDV['58_FC_2_0'] = [' 58','EURO3','RT >20-26t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1063.21061180647','0.621886809002817','-66.1830154255279','0.081633481249004','0','6','86']
  H_HDV['58_CO_2_50'] = [' 58','EURO3','RT >20-26t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.293360822068','0.005568526072217','-0.783931314090311','0.072143884551892','0','6','86']
  H_HDV['58_THC_2_50'] = [' 58','EURO3','RT >20-26t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.194300580923787','2.34588366064754','5.64825727030232','2.24865555271348','-0.01344775772817','6','86']
  H_HDV['58_NOx_2_50'] = [' 58','EURO3','RT >20-26t Euro-3','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','46.8083713016629','-0.041369298344917','-4.28356031740058','0.126810623615174','0','6','86']
  H_HDV['58_PM_2_50'] = [' 58','EURO3','RT >20-26t Euro-3','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.109320159963081','0.666730070250755','6.65842912734812','2.56757421591619','-0.025027431936317','6','86']
  H_HDV['58_FC_2_50'] = [' 58','EURO3','RT >20-26t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1098.82120361612','0.787704729454681','-53.8240337429092','0.071422137348004','0','6','86']
  H_HDV['58_CO_2_100'] = [' 58','EURO3','RT >20-26t Euro-3','2','100','CO','y=(c+(a*exp(b*x)))','10.8643039013107','-0.069078620765388','1.82887002868622','0','0','6','86']
  H_HDV['58_THC_2_100'] = [' 58','EURO3','RT >20-26t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.259275909533023','2.21061947356332','6.06410602810706','2.41844224220395','-0.014704026301852','6','86']
  H_HDV['58_NOx_2_100'] = [' 58','EURO3','RT >20-26t Euro-3','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','9.79858443070372','95.0039756782128','0.459282850899599','0.927579589996708','0.000808963811954','6','86']
  H_HDV['58_PM_2_100'] = [' 58','EURO3','RT >20-26t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.999610844935543','0.00030716498139','-0.05689048749894','0.06827524473934','0','6','86']
  H_HDV['58_FC_2_100'] = [' 58','EURO3','RT >20-26t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1149.34415592541','0.535272235698728','-46.0267132097548','0.066673075351424','0','6','86']
  H_HDV['58_CO_4_0'] = [' 58','EURO3','RT >20-26t Euro-3','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.36546836343746','7.30825888901411','6.4636438257359','2.52057121986921','-0.017618070063097','6','72']
  H_HDV['58_THC_4_0'] = [' 58','EURO3','RT >20-26t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.187405940441337','2.453717577564','5.47549986760884','2.17911082302992','-0.012468695849365','6','72']
  H_HDV['58_NOx_4_0'] = [' 58','EURO3','RT >20-26t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.029274328224','0.033592138084572','36.8660679083812','0.15681727517058','10.7039022219867','6','72']
  H_HDV['58_PM_4_0'] = [' 58','EURO3','RT >20-26t Euro-3','4','0','PM','y=(c+(a*exp(b*x)))','0.805363718683584','-0.069965408188454','0.160121157898606','0','0','6','72']
  H_HDV['58_FC_4_0'] = [' 58','EURO3','RT >20-26t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','2298.39112155178','1.00991557174617','-0.567131232390422','0','0','6','72']
  H_HDV['58_CO_4_50'] = [' 58','EURO3','RT >20-26t Euro-3','4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.060450539315462','0.009373033571693','-0.000033475340323','0','0','6','71']
  H_HDV['58_THC_4_50'] = [' 58','EURO3','RT >20-26t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.282346193988496','2.2754041718143','6.01413518118828','2.40974275057529','-0.015295658950334','6','71']
  H_HDV['58_NOx_4_50'] = [' 58','EURO3','RT >20-26t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4635128880553','0.050250635186786','31.6454664433091','0.181508794073515','14.6508464767121','6','71']
  H_HDV['58_PM_4_50'] = [' 58','EURO3','RT >20-26t Euro-3','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.170300050518175','0.581245468699762','6.46952336339774','2.43207777194536','-0.01515416732714','6','71']
  H_HDV['58_FC_4_50'] = [' 58','EURO3','RT >20-26t Euro-3','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','649.248297589869','0.084828611238929','11814.3327318623','0.879389032615626','579.289784760356','6','71']
  H_HDV['58_CO_4_100'] = [' 58','EURO3','RT >20-26t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.0357307188643','-0.039251888836047','-0.992797084637455','0.097719213609386','0','6','67']
  H_HDV['58_THC_4_100'] = [' 58','EURO3','RT >20-26t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.364413972735764','2.12610484566542','6.36582792641372','2.56336549911519','-0.018393847612641','6','67']
  H_HDV['58_NOx_4_100'] = [' 58','EURO3','RT >20-26t Euro-3','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','56.1304822871833','-0.038124480810639','-3.90464474001372','0.111560376418486','0','6','67']
  H_HDV['58_PM_4_100'] = [' 58','EURO3','RT >20-26t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004544403835','0.000699820675356','-0.036661966214202','0.929509794179332','0','6','67']
  H_HDV['58_FC_4_100'] = [' 58','EURO3','RT >20-26t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','510.418823207006','0.063056228815602','9501.17355463278','0.751940310143675','738.391847117685','6','67']
  H_HDV['58_CO_6_0'] = [' 58','EURO3','RT >20-26t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.6043271095195','-0.033656358766409','-1.61383193567205','0.148503531374302','0','6','71']
  H_HDV['58_THC_6_0'] = [' 58','EURO3','RT >20-26t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.242916467417919','2.22946644982965','7.01851672683994','2.78110467576422','-0.028258188865074','6','71']
  H_HDV['58_NOx_6_0'] = [' 58','EURO3','RT >20-26t Euro-3','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','60.8550229094588','-0.062221258421129','-7.11140509873286','0.164992141806855','0','6','71']
  H_HDV['58_PM_6_0'] = [' 58','EURO3','RT >20-26t Euro-3','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.02026509366473','-0.001329708478968','-0.07024882717171','0.093142151940354','0','6','71']
  H_HDV['58_FC_6_0'] = [' 58','EURO3','RT >20-26t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','2243.44440153295','1.00957017905466','-0.490386611402654','0','0','6','71']
  H_HDV['58_CO_6_50'] = [' 58','EURO3','RT >20-26t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.73014961242088','-4.75196732030186','-0.981356716846223','0','0','6','65']
  H_HDV['58_THC_6_50'] = [' 58','EURO3','RT >20-26t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.367048040843922','2.17953871202767','6.220428117724','2.51438966204834','-0.01817872317812','6','65']
  H_HDV['58_NOx_6_50'] = [' 58','EURO3','RT >20-26t Euro-3','6','50','NOx','y=((a*(b**x))*(x**c))','95.7600713474291','1.00869968824479','-0.510003917221783','0','0','6','65']
  H_HDV['58_PM_6_50'] = [' 58','EURO3','RT >20-26t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.26864179743792','3.44985402272658','-1.8587815192657','-0.376630253202696','0.11441798807481','6','65']
  H_HDV['58_FC_6_50'] = [' 58','EURO3','RT >20-26t Euro-3','6','50','FC','y=((a*(b**x))*(x**c))','2019.81939429236','1.00536930255638','-0.312107257382325','0','0','6','65']
  H_HDV['58_CO_6_100'] = [' 58','EURO3','RT >20-26t Euro-3','6','100','CO','y=((a*(b**x))*(x**c))','22.5223235356168','0.984863577799334','-0.378748257764068','0','0','6','51']
  H_HDV['58_THC_6_100'] = [' 58','EURO3','RT >20-26t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.609348725139984','8.87616576364302','-1.24619096218646','-0.318829022061273','0.134838747414077','6','51']
  H_HDV['58_NOx_6_100'] = [' 58','EURO3','RT >20-26t Euro-3','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000520345675738','0.059865891006893','-2.35760242278486','58.4910725515641','0','6','51']
  H_HDV['58_PM_6_100'] = [' 58','EURO3','RT >20-26t Euro-3','6','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.768143532018384','0.081047961732842','-0.000881943200457','0','0','6','51']
  H_HDV['58_FC_6_100'] = [' 58','EURO3','RT >20-26t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','485.501498836634','0.067516602079484','12535.6569945653','0.791711484901619','1010.4903039164','6','51']
  H_HDV['59_CO_0_0'] = [' 59','EURO4','RT >20-26t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.035162372991848','0.792065795390107','3.97206639261177','1.74835768779952','-0.011066964147455','6','86']
  H_HDV['59_THC_0_0'] = [' 59','EURO4','RT >20-26t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00411019798097','0.171330299015662','3.49309323934332','1.60664548809586','-0.007057376186469','6','86']
  H_HDV['59_NOx_0_0'] = [' 59','EURO4','RT >20-26t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.75701621823348','0.061638831807963','35.3056968845558','0.394616576772824','2.9209084139772','6','86']
  H_HDV['59_PM_0_0'] = [' 59','EURO4','RT >20-26t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.093517094656483','0.046959921613633','0.24085096581815','0.170554367267388','0.016689205452055','6','86']
  H_HDV['59_FC_0_0'] = [' 59','EURO4','RT >20-26t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','517.624902833896','0.06375300404258','2785.12643645991','0.524776362695295','155.314952810305','6','86']
  H_HDV['59_CO_0_50'] = [' 59','EURO4','RT >20-26t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.051787253018837','0.795593759922335','3.60567893972214','1.62826493136327','-0.006676798249867','6','86']
  H_HDV['59_THC_0_50'] = [' 59','EURO4','RT >20-26t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004531559517366','0.161823593403322','3.71451844651423','1.68824209290668','-0.008010256846751','6','86']
  H_HDV['59_NOx_0_50'] = [' 59','EURO4','RT >20-26t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.5811576132649','0.048741943044301','44.2455720184548','0.442226511592656','3.39252553460743','6','86']
  H_HDV['59_PM_0_50'] = [' 59','EURO4','RT >20-26t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094179108343756','0.046676308556221','0.233790360694818','0.173445545443001','0.017556724523703','6','86']
  H_HDV['59_FC_0_50'] = [' 59','EURO4','RT >20-26t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','492.958084416285','0.051326475553613','5083.97393541747','0.628570815853726','178.211669157967','6','86']
  H_HDV['59_CO_0_100'] = [' 59','EURO4','RT >20-26t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.369819949526538','0.047367290711713','0.573675237489164','0.162843845309384','0.087166034215195','6','86']
  H_HDV['59_THC_0_100'] = [' 59','EURO4','RT >20-26t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004209672784126','0.157635550341115','3.74078731168532','1.70417564621965','-0.009208704334793','6','86']
  H_HDV['59_NOx_0_100'] = [' 59','EURO4','RT >20-26t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.05405775718219','0.039666066363099','80.2599751838029','0.557972834205966','3.64917329830103','6','86']
  H_HDV['59_PM_0_100'] = [' 59','EURO4','RT >20-26t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.098749080774276','0.047034079649512','0.223301355764164','0.1779091647082','0.017897633651605','6','86']
  H_HDV['59_FC_0_100'] = [' 59','EURO4','RT >20-26t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','489.691833054167','0.040712344165014','11992.6306774966','0.775103903785528','192.686344291081','6','86']
  H_HDV['59_CO_-2_0'] = [' 59','EURO4','RT >20-26t Euro-4','-2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-5.34384640869381','0.763823198887716','0.834706873104921','0.07945115889064','0.061149733757082','6','86']
  H_HDV['59_THC_-2_0'] = [' 59','EURO4','RT >20-26t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009427196073432','0.11078379385755','5.80330425077625','2.18386031167453','0.00385273315815','6','86']
  H_HDV['59_NOx_-2_0'] = [' 59','EURO4','RT >20-26t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.59348160573909','0.062282535458113','96.1306587246316','0.589184704921369','1.07513628469074','6','86']
  H_HDV['59_PM_-2_0'] = [' 59','EURO4','RT >20-26t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010402849225738','0.208275459267185','4.23245937352687','1.77714069441258','0.007395045059734','6','86']
  H_HDV['59_FC_-2_0'] = [' 59','EURO4','RT >20-26t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','546.476510750558','0.064174507222183','9599.65248683837','0.765510658935265','61.9601472712788','6','86']
  H_HDV['59_CO_-2_50'] = [' 59','EURO4','RT >20-26t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018972903836592','0.524382300943275','6.71834338216548','2.52453168941357','-0.016726873621559','6','86']
  H_HDV['59_THC_-2_50'] = [' 59','EURO4','RT >20-26t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001442088957','0.112635969046554','5.95172906782478','2.33498291694581','-0.019172825361914','6','86']
  H_HDV['59_NOx_-2_50'] = [' 59','EURO4','RT >20-26t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.17333540029459','0.041976324950249','103.551458564569','0.5639847237923','0.623009344311501','6','86']
  H_HDV['59_PM_-2_50'] = [' 59','EURO4','RT >20-26t Euro-4','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.052947580508033','0.034314502889095','0.212140286383137','0.115414613788863','0.006886720996218','6','86']
  H_HDV['59_FC_-2_50'] = [' 59','EURO4','RT >20-26t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','465.021006039007','0.046763578732145','8179.63024111321','0.689300858835309','39.4388373168614','6','86']
  H_HDV['59_CO_-2_100'] = [' 59','EURO4','RT >20-26t Euro-4','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.38286904026945','-8.36879502617797','-1.52862357008486','0','0','6','86']
  H_HDV['59_THC_-2_100'] = [' 59','EURO4','RT >20-26t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000251932781148','0.116976524454454','5.0410995190991','2.01992541756233','-0.012727117107577','6','86']
  H_HDV['59_NOx_-2_100'] = [' 59','EURO4','RT >20-26t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.10759393543798','0.031939643539784','165.635719421318','0.634268418831893','0.217447010106706','6','86']
  H_HDV['59_PM_-2_100'] = [' 59','EURO4','RT >20-26t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003694451116828','0.279779228742092','2.4560043947258','1.25922779924816','0.012103240670517','6','86']
  H_HDV['59_FC_-2_100'] = [' 59','EURO4','RT >20-26t Euro-4','-2','100','FC','y=((a*(b**x))*(x**c))','1096.00343892424','0.981429769129699','-0.392709021463652','0','0','6','86']
  H_HDV['59_CO_-4_0'] = [' 59','EURO4','RT >20-26t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001025443465979','0.509610142751821','6.42874244840294','2.41803707356715','-0.011220986478632','6','86']
  H_HDV['59_THC_-4_0'] = [' 59','EURO4','RT >20-26t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.151102188681697','-0.000091459301799','-0.01193014338047','0.084982038918312','0','6','86']
  H_HDV['59_NOx_-4_0'] = [' 59','EURO4','RT >20-26t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.58562314874812','0.055695677007713','97.7706812595796','0.558413991435114','0.169714146139836','6','86']
  H_HDV['59_PM_-4_0'] = [' 59','EURO4','RT >20-26t Euro-4','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.2558574582302','-0.000167717979146','-0.023497425876402','0.098386557851442','0','6','86']
  H_HDV['59_FC_-4_0'] = [' 59','EURO4','RT >20-26t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','430.570421079803','0.058184043384954','5712.30832033952','0.626309916481391','11.8754784944737','6','86']
  H_HDV['59_CO_-4_50'] = [' 59','EURO4','RT >20-26t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.641779796596575','-0.000408785193665','-0.048705121891024','0.081025443473567','0','6','86']
  H_HDV['59_THC_-4_50'] = [' 59','EURO4','RT >20-26t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.138308604464367','-0.000097457854209','-0.011244383514555','0.087908183836826','0','6','86']
  H_HDV['59_NOx_-4_50'] = [' 59','EURO4','RT >20-26t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.65487303215796','0.048047730713011','155.167983942322','0.62829919218769','0.073325221875471','6','86']
  H_HDV['59_PM_-4_50'] = [' 59','EURO4','RT >20-26t Euro-4','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002861385299475','1.72314483807081','-0.957537632934524','0.716857962732582','0.032027289718522','6','86']
  H_HDV['59_FC_-4_50'] = [' 59','EURO4','RT >20-26t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','360.584082278457','0.05029231501699','10532.4206955355','0.711261172571326','6.15602304258372','6','86']
  H_HDV['59_CO_-4_100'] = [' 59','EURO4','RT >20-26t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.576584984012255','-0.000409971950302','-0.043042686281764','0.080145037346096','0','6','86']
  H_HDV['59_THC_-4_100'] = [' 59','EURO4','RT >20-26t Euro-4','-4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002654978723632','1.01247010404679','-1.32089292295738','0.513935483280581','0.039468004585601','6','86']
  H_HDV['59_NOx_-4_100'] = [' 59','EURO4','RT >20-26t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.26677405131376','0.042973797264889','229.843212519297','0.693900692122888','0.011616529773334','6','86']
  H_HDV['59_PM_-4_100'] = [' 59','EURO4','RT >20-26t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.121801095157715','0.060517979923853','0.520276908366193','0.397058754539544','0.004263163044266','6','86']
  H_HDV['59_FC_-4_100'] = [' 59','EURO4','RT >20-26t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','327.690098096217','0.044847563858856','18487.3398772273','0.802377045739758','2.59447247017564','6','86']
  H_HDV['59_CO_-6_0'] = [' 59','EURO4','RT >20-26t Euro-4','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.593708441693031','0.00037205517889','-0.041763877241751','0.069214660292312','0','6','72']
  H_HDV['59_THC_-6_0'] = [' 59','EURO4','RT >20-26t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.113175000527686','0.073871540064488','0.113134645810568','0.44082291774','0.003589740723761','6','72']
  H_HDV['59_NOx_-6_0'] = [' 59','EURO4','RT >20-26t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.3219014247349','0.067823223033815','163.709952937465','0.666815103338231','0.054508588832504','6','72']
  H_HDV['59_PM_-6_0'] = [' 59','EURO4','RT >20-26t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.157236895597957','0.073318182776571','0.357778196233975','0.394168414606278','0.004359551358942','6','72']
  H_HDV['59_FC_-6_0'] = [' 59','EURO4','RT >20-26t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','397.201203138609','0.067862408465834','12588.8395907723','0.778332905472057','4.46372174978182','6','72']
  H_HDV['59_CO_-6_50'] = [' 59','EURO4','RT >20-26t Euro-4','-6','50','CO','y=(c+(a*exp(b*x)))','0.488262092411787','-0.073897969193878','0.009710750551331','0','0','6','72']
  H_HDV['59_THC_-6_50'] = [' 59','EURO4','RT >20-26t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089368446296633','0.068987218742187','0.539156472151823','0.611049918776492','0.003239054316754','6','72']
  H_HDV['59_NOx_-6_50'] = [' 59','EURO4','RT >20-26t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.15282755878416','0.062499471530652','221.186526320277','0.707453493945961','0.036889287048496','6','72']
  H_HDV['59_PM_-6_50'] = [' 59','EURO4','RT >20-26t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.130405793208069','0.069452292442082','0.833461073472285','0.507769280740459','0.003984911919323','6','72']
  H_HDV['59_FC_-6_50'] = [' 59','EURO4','RT >20-26t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','311.368116032803','0.061556182077035','18284.2527848566','0.819455908442117','3.09467157667657','6','72']
  H_HDV['59_CO_-6_100'] = [' 59','EURO4','RT >20-26t Euro-4','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005383988465636','1.79917667513716','-0.706221104737278','0.376110133401964','0.050140803219174','6','72']
  H_HDV['59_THC_-6_100'] = [' 59','EURO4','RT >20-26t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.077379924449065','0.066016243779556','0.846179597121593','0.668407923372333','0.003070073706641','6','72']
  H_HDV['59_NOx_-6_100'] = [' 59','EURO4','RT >20-26t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.65254516017762','0.059299627101931','278.097219978397','0.746733577802536','0.028887086733606','6','72']
  H_HDV['59_PM_-6_100'] = [' 59','EURO4','RT >20-26t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.117263082327319','0.067193096960414','1.16202722420167','0.559759880852134','0.003807600185084','6','72']
  H_HDV['59_FC_-6_100'] = [' 59','EURO4','RT >20-26t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','272.579715631175','0.057611901533405','24847.3474281435','0.867636608086833','2.39149376233136','6','72']
  H_HDV['59_CO_2_0'] = [' 59','EURO4','RT >20-26t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108084401594855','0.554486201104929','7.30103660874092','2.85518293082651','-0.018384261042669','6','86']
  H_HDV['59_THC_2_0'] = [' 59','EURO4','RT >20-26t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005014840949204','0.147925708621559','4.60025418153732','1.88344149331917','-0.008207150932475','6','86']
  H_HDV['59_NOx_2_0'] = [' 59','EURO4','RT >20-26t Euro-4','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.6842105440119','0.004838441907284','-1.63002367684936','0.104019172917265','0','6','86']
  H_HDV['59_PM_2_0'] = [' 59','EURO4','RT >20-26t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050743484232414','0.026443101240816','0.247099809696645','0.126820718133141','0.017392344828109','6','86']
  H_HDV['59_FC_2_0'] = [' 59','EURO4','RT >20-26t Euro-4','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1021.8511716475','0.439456719877306','-65.4599448835421','0.084147575049825','0','6','86']
  H_HDV['59_CO_2_50'] = [' 59','EURO4','RT >20-26t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.06861864074779','-0.00104032381434','-0.099902226888503','0.116528786569369','0','6','86']
  H_HDV['59_THC_2_50'] = [' 59','EURO4','RT >20-26t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007058945477038','0.133081897126036','5.55111287782891','2.22948978986021','-0.013859141809704','6','86']
  H_HDV['59_NOx_2_50'] = [' 59','EURO4','RT >20-26t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.9889150716419','-0.016255878520382','-1.84158501542542','0.121735791257409','0','6','86']
  H_HDV['59_PM_2_50'] = [' 59','EURO4','RT >20-26t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.054541242194375','0.021195639983106','0.245432164139552','0.127710461932315','0.015253597996652','6','86']
  H_HDV['59_FC_2_50'] = [' 59','EURO4','RT >20-26t Euro-4','2','50','FC','y=((a*(b**x))*(x**c))','2011.49559552159','1.00794279676001','-0.530859198318602','0','0','6','86']
  H_HDV['59_CO_2_100'] = [' 59','EURO4','RT >20-26t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.10654472246286','-0.002055051659955','-0.10793206636465','0.128726415118466','0','6','86']
  H_HDV['59_THC_2_100'] = [' 59','EURO4','RT >20-26t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009854600373688','0.125347377277718','6.04683529543868','2.42967759755857','-0.015803566330742','6','86']
  H_HDV['59_NOx_2_100'] = [' 59','EURO4','RT >20-26t Euro-4','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','2.87155280644088','1.80866838285815','-0.189112522562288','0','0','6','86']
  H_HDV['59_PM_2_100'] = [' 59','EURO4','RT >20-26t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072444576522899','0.025212505494148','0.247176016428379','0.145429096077728','0.015156270112805','6','86']
  H_HDV['59_FC_2_100'] = [' 59','EURO4','RT >20-26t Euro-4','2','100','FC','y=((a*(b**x))*(x**c))','1816.22159128015','1.00566262128825','-0.415085391794616','0','0','6','86']
  H_HDV['59_CO_4_0'] = [' 59','EURO4','RT >20-26t Euro-4','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.179292943945473','1.67419350800397','-1.06950480542583','-0.55382677155199','0.158339932398094','6','72']
  H_HDV['59_THC_4_0'] = [' 59','EURO4','RT >20-26t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005962308261795','0.140552306993142','5.19617837170816','2.08751151057194','-0.011320407567517','6','72']
  H_HDV['59_NOx_4_0'] = [' 59','EURO4','RT >20-26t Euro-4','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.219274298576','0.10642166601255','11.5265261932296','0.296168538786527','7.48140392572668','6','72']
  H_HDV['59_PM_4_0'] = [' 59','EURO4','RT >20-26t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08771918562919','0.045953937031345','0.210158558705391','0.153713110012267','0.027354751897057','6','72']
  H_HDV['59_FC_4_0'] = [' 59','EURO4','RT >20-26t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','747.083916109721','0.097858900166876','12910.9318952494','0.939939675393158','380.581906660002','6','72']
  H_HDV['59_CO_4_50'] = [' 59','EURO4','RT >20-26t Euro-4','4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.76353790541734','0.130025271621696','-0.000974158021048','0','0','6','71']
  H_HDV['59_THC_4_50'] = [' 59','EURO4','RT >20-26t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009718603227795','0.13017361214728','5.79245801122916','2.33758071480998','-0.014432896971342','6','71']
  H_HDV['59_NOx_4_50'] = [' 59','EURO4','RT >20-26t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.54730245368284','0.048940736194796','19.578281427941','0.259226436682932','9.64077403638153','6','71']
  H_HDV['59_PM_4_50'] = [' 59','EURO4','RT >20-26t Euro-4','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.80522352322818','0.612584722580569','-0.002721577578249','0','0','6','71']
  H_HDV['59_FC_4_50'] = [' 59','EURO4','RT >20-26t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','639.473148642378','0.088429165511044','14134.1596910352','0.909440799346903','546.252858296949','6','71']
  H_HDV['59_CO_4_100'] = [' 59','EURO4','RT >20-26t Euro-4','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.501018515068968','0.023263538535907','0.622158868036226','0.16177818995144','0.037321974345957','6','68']
  H_HDV['59_THC_4_100'] = [' 59','EURO4','RT >20-26t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013480722690772','0.121503572777432','6.20962712075798','2.52649777109467','-0.017665954634933','6','68']
  H_HDV['59_NOx_4_100'] = [' 59','EURO4','RT >20-26t Euro-4','4','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.11676003086192','1.40901070847398','-0.155638502648901','0','0','6','68']
  H_HDV['59_PM_4_100'] = [' 59','EURO4','RT >20-26t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.10234203647033','0.032129170316659','0.229204450943393','0.155761464146266','0.01767333240377','6','68']
  H_HDV['59_FC_4_100'] = [' 59','EURO4','RT >20-26t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','514.930754437447','0.070733390723836','9895.87562768993','0.776683619905596','702.50887712505','6','68']
  H_HDV['59_CO_6_0'] = [' 59','EURO4','RT >20-26t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.45870236188607','-0.002764003067035','-0.231840259274048','0.210543867532528','0','6','71']
  H_HDV['59_THC_6_0'] = [' 59','EURO4','RT >20-26t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00812280361142','0.127109040559733','6.55925809569264','2.6025658946468','-0.023137915121135','6','71']
  H_HDV['59_NOx_6_0'] = [' 59','EURO4','RT >20-26t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.3278702910345','-0.031606183217086','-5.01863279255073','0.208029455144381','0','6','71']
  H_HDV['59_PM_6_0'] = [' 59','EURO4','RT >20-26t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.370500285268707','-0.000677422287487','-0.048262306451575','0.161908487637308','0','6','71']
  H_HDV['59_FC_6_0'] = [' 59','EURO4','RT >20-26t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','2209.30109641499','1.01006541014917','-0.509318698917674','0','0','6','71']
  H_HDV['59_CO_6_50'] = [' 59','EURO4','RT >20-26t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.24144814524517','-0.004631973327722','-0.133882909553196','0.161923335420869','0','6','66']
  H_HDV['59_THC_6_50'] = [' 59','EURO4','RT >20-26t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013302128213419','0.120285683092283','6.48728928773619','2.62510335587782','-0.020835462184377','6','66']
  H_HDV['59_NOx_6_50'] = [' 59','EURO4','RT >20-26t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.30919111203797','0.050031211350521','20.477951719271','0.294393822770789','12.3716471179878','6','66']
  H_HDV['59_PM_6_50'] = [' 59','EURO4','RT >20-26t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.334442842747156','-0.000760845703751','-0.033039932728784','0.125740344130551','0','6','66']
  H_HDV['59_FC_6_50'] = [' 59','EURO4','RT >20-26t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','599.471650866141','0.08810577806389','11286.7573867049','0.854199785047203','728.911519378729','6','66']
  H_HDV['59_CO_6_100'] = [' 59','EURO4','RT >20-26t Euro-4','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.19840453392926','-0.00412250159083','-0.085302473571789','0.100398356984701','0','6','51']
  H_HDV['59_THC_6_100'] = [' 59','EURO4','RT >20-26t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018115108655544','0.112146431276474','6.84130426139055','2.7997779404376','-0.024425485563813','6','51']
  H_HDV['59_NOx_6_100'] = [' 59','EURO4','RT >20-26t Euro-4','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000221303085384','0.025137672733099','-1.00132300981141','31.1433493617344','0','6','51']
  H_HDV['59_PM_6_100'] = [' 59','EURO4','RT >20-26t Euro-4','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.195876546218944','0.066699247059756','0.136075800100293','0.192913208780859','0.033819460576681','6','51']
  H_HDV['59_FC_6_100'] = [' 59','EURO4','RT >20-26t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','477.704206949846','0.069214679697415','9745.32694348858','0.758096554888922','950.673060352487','6','51']
  H_HDV['60_CO_0_0'] = [' 60','EURO5','RT >20-26t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.036257526710185','0.806902249596819','3.94801660611051','1.73719144611241','-0.010355847585028','6','86']
  H_HDV['60_THC_0_0'] = [' 60','EURO5','RT >20-26t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004194213454812','0.174390120960734','3.49088462216552','1.60445970264976','-0.006977174687959','6','86']
  H_HDV['60_NOx_0_0'] = [' 60','EURO5','RT >20-26t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.74967210335932','0.061019478670917','23.7085411248342','0.377088047649353','1.72448387020735','6','86']
  H_HDV['60_PM_0_0'] = [' 60','EURO5','RT >20-26t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095209347659942','0.046810259956394','0.245433604257954','0.170571985825627','0.016823936630951','6','86']
  H_HDV['60_FC_0_0'] = [' 60','EURO5','RT >20-26t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','530.706538954452','0.063386703194507','2704.52844752398','0.511855003431854','157.588210716201','6','86']
  H_HDV['60_CO_0_50'] = [' 60','EURO5','RT >20-26t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.05152802888524','0.809765188469381','3.61686053790662','1.63170851839035','-0.006837591929516','6','86']
  H_HDV['60_THC_0_50'] = [' 60','EURO5','RT >20-26t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004796750182999','0.165044994232262','3.69837199771675','1.68154440358292','-0.00750858321808','6','86']
  H_HDV['60_NOx_0_50'] = [' 60','EURO5','RT >20-26t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.64547354663074','0.048988896597396','27.4546876821401','0.40922167884379','2.00405135978564','6','86']
  H_HDV['60_PM_0_50'] = [' 60','EURO5','RT >20-26t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094761328218793','0.046249567600827','0.238333046076525','0.172042775831038','0.017683827588084','6','86']
  H_HDV['60_FC_0_50'] = [' 60','EURO5','RT >20-26t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','505.76980693203','0.051130608324289','4762.79636324593','0.608807311798855','180.436041616308','6','86']
  H_HDV['60_CO_0_100'] = [' 60','EURO5','RT >20-26t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.067760754792934','0.800229441203193','3.26641349498309','1.46744283073526','0.002194613608779','6','86']
  H_HDV['60_THC_0_100'] = [' 60','EURO5','RT >20-26t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004232182888798','0.159551401708859','3.78250194930984','1.71605420646607','-0.009516284708112','6','86']
  H_HDV['60_NOx_0_100'] = [' 60','EURO5','RT >20-26t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.01874224048381','0.041252063718601','42.4382993998519','0.498709579809226','2.15063870911047','6','86']
  H_HDV['60_PM_0_100'] = [' 60','EURO5','RT >20-26t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.099827867630172','0.046832745943509','0.227635971077132','0.17698411359941','0.018085384600169','6','86']
  H_HDV['60_FC_0_100'] = [' 60','EURO5','RT >20-26t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','502.94057758531','0.040770314176185','9343.09017066658','0.72921397083514','195.202490222373','6','86']
  H_HDV['60_CO_-2_0'] = [' 60','EURO5','RT >20-26t Euro-5','-2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-5.64073337328455','0.766063072989614','0.851107698330198','0.079359183088031','0.062159661205901','6','86']
  H_HDV['60_THC_-2_0'] = [' 60','EURO5','RT >20-26t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009481963042119','0.112734334464143','5.86126068347453','2.20854971419164','0.002541078011703','6','86']
  H_HDV['60_NOx_-2_0'] = [' 60','EURO5','RT >20-26t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.22730980010608','0.06173349832162','51.7524615453985','0.528920382926861','0.631881350847048','6','86']
  H_HDV['60_PM_-2_0'] = [' 60','EURO5','RT >20-26t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010341490420751','0.210587620488341','4.3116755614426','1.80810347437292','0.005839638386187','6','86']
  H_HDV['60_FC_-2_0'] = [' 60','EURO5','RT >20-26t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','561.296437874116','0.063945199328435','8741.12477576509','0.742546502407501','62.9758802147165','6','86']
  H_HDV['60_CO_-2_50'] = [' 60','EURO5','RT >20-26t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.019012764640139','0.536210032993603','6.74144905008552','2.5342827078366','-0.017001624126391','6','86']
  H_HDV['60_THC_-2_50'] = [' 60','EURO5','RT >20-26t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001558178790992','0.115090233304484','5.94900323879651','2.3332249600083','-0.018872357960748','6','86']
  H_HDV['60_NOx_-2_50'] = [' 60','EURO5','RT >20-26t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.39373440304491','0.042418128067171','59.7717650102261','0.523701751893981','0.368452821289526','6','86']
  H_HDV['60_PM_-2_50'] = [' 60','EURO5','RT >20-26t Euro-5','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053101759447644','0.033903843633345','0.217726448765753','0.114950738253953','0.006915132621325','6','86']
  H_HDV['60_FC_-2_50'] = [' 60','EURO5','RT >20-26t Euro-5','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','479.619997392701','0.046977680303309','7858.07093385438','0.676757572587377','40.2460316776853','6','86']
  H_HDV['60_CO_-2_100'] = [' 60','EURO5','RT >20-26t Euro-5','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.42654822175115','-8.42997958398804','-1.53577632714732','0','0','6','86']
  H_HDV['60_THC_-2_100'] = [' 60','EURO5','RT >20-26t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000387255232739','0.119782749955087','5.03788676207987','2.02018588573578','-0.012956818296107','6','86']
  H_HDV['60_NOx_-2_100'] = [' 60','EURO5','RT >20-26t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.35091170013596','0.032882238820683','90.399028935169','0.586809648070516','0.141397682427675','6','86']
  H_HDV['60_PM_-2_100'] = [' 60','EURO5','RT >20-26t Euro-5','-2','100','PM','y=(1/(a+(b*(x**c))))','3.8491761047149','0.189394583394914','1.45898692281237','0','0','6','86']
  H_HDV['60_FC_-2_100'] = [' 60','EURO5','RT >20-26t Euro-5','-2','100','FC','y=((a*(b**x))*(x**c))','1144.82433851774','0.981479256732212','-0.399688105801383','0','0','6','86']
  H_HDV['60_CO_-4_0'] = [' 60','EURO5','RT >20-26t Euro-5','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000895352858323','0.521269759126083','6.44341272000355','2.42436162988268','-0.011439844603916','6','86']
  H_HDV['60_THC_-4_0'] = [' 60','EURO5','RT >20-26t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.154502828690175','-0.000093700887771','-0.01220389055867','0.085013449308864','0','6','86']
  H_HDV['60_NOx_-4_0'] = [' 60','EURO5','RT >20-26t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.10769522133692','0.056329536029904','61.4666633873581','0.533948557065404','0.101742985647936','6','86']
  H_HDV['60_PM_-4_0'] = [' 60','EURO5','RT >20-26t Euro-5','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.261438505713123','-0.000170961119273','-0.024018675505218','0.09840200899085','0','6','86']
  H_HDV['60_FC_-4_0'] = [' 60','EURO5','RT >20-26t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','445.189423391431','0.058356560359956','5649.98690842149','0.618670408503419','12.2087944093872','6','86']
  H_HDV['60_CO_-4_50'] = [' 60','EURO5','RT >20-26t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.655032701443761','-0.000414110479365','-0.049678836568452','0.080938773764955','0','6','86']
  H_HDV['60_THC_-4_50'] = [' 60','EURO5','RT >20-26t Euro-5','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.141093700872031','-0.000099157735214','-0.011454783946514','0.087766186244812','0','6','86']
  H_HDV['60_NOx_-4_50'] = [' 60','EURO5','RT >20-26t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.55202870028639','0.049222985468181','93.0446465030544','0.598602984326741','0.047772086411286','6','86']
  H_HDV['60_PM_-4_50'] = [' 60','EURO5','RT >20-26t Euro-5','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002958280798854','1.55655143085875','-0.819227124710128','0.719544613178913','0.032314398070531','6','86']
  H_HDV['60_FC_-4_50'] = [' 60','EURO5','RT >20-26t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','373.730340686995','0.050670159483639','10510.7963389887','0.70655765483117','6.46997620033449','6','86']
  H_HDV['60_CO_-4_100'] = [' 60','EURO5','RT >20-26t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.589221421262524','-0.000413936073812','-0.044011680807795','0.080132680181123','0','6','86']
  H_HDV['60_THC_-4_100'] = [' 60','EURO5','RT >20-26t Euro-5','-4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002720556467754','0.980261372830622','-1.26090705307031','0.514140097579372','0.0396888874646','6','86']
  H_HDV['60_NOx_-4_100'] = [' 60','EURO5','RT >20-26t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.30876728677529','0.044354777708213','131.036522323371','0.656133388788797','0.013507774415287','6','86']
  H_HDV['60_PM_-4_100'] = [' 60','EURO5','RT >20-26t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.124320675349665','0.06059540405122','0.518152441544862','0.393157435331367','0.004339192822498','6','86']
  H_HDV['60_FC_-4_100'] = [' 60','EURO5','RT >20-26t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','338.821496514015','0.045212063073184','18515.7674852106','0.798025837918611','2.86327760972426','6','86']
  H_HDV['60_CO_-6_0'] = [' 60','EURO5','RT >20-26t Euro-5','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.607729468602667','0.000383835387498','-0.042769653629829','0.06922261932368','0','6','72']
  H_HDV['60_THC_-6_0'] = [' 60','EURO5','RT >20-26t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.116076209748677','0.07403481321736','0.107215812954235','0.432965248202062','0.003662474532934','6','72']
  H_HDV['60_NOx_-6_0'] = [' 60','EURO5','RT >20-26t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.99069063695924','0.068265765685865','96.496816976317','0.629955492991771','0.032568908625983','6','72']
  H_HDV['60_PM_-6_0'] = [' 60','EURO5','RT >20-26t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.160943393138414','0.073430921110752','0.352474548991331','0.389339137657649','0.004443845305564','6','72']
  H_HDV['60_FC_-6_0'] = [' 60','EURO5','RT >20-26t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','412.875905580738','0.068163000825935','12738.8585875846','0.77630208638401','4.65310452986853','6','72']
  H_HDV['60_CO_-6_50'] = [' 60','EURO5','RT >20-26t Euro-5','-6','50','CO','y=(c+(a*exp(b*x)))','0.500487459388252','-0.074098849233626','0.009928058375489','0','0','6','72']
  H_HDV['60_THC_-6_50'] = [' 60','EURO5','RT >20-26t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.09157440504122','0.069168218491981','0.527844635799106','0.605259974227137','0.003308088025398','6','72']
  H_HDV['60_NOx_-6_50'] = [' 60','EURO5','RT >20-26t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.28777903320993','0.063362550960228','128.826186811324','0.671246860446544','0.022634504544433','6','72']
  H_HDV['60_PM_-6_50'] = [' 60','EURO5','RT >20-26t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.133673236062999','0.069690106021152','0.838022522992138','0.506440766392839','0.004076002520591','6','72']
  H_HDV['60_FC_-6_50'] = [' 60','EURO5','RT >20-26t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','322.551136678345','0.061828526564869','18183.5849704185','0.813246098708721','3.2519732068357','6','72']
  H_HDV['60_CO_-6_100'] = [' 60','EURO5','RT >20-26t Euro-5','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005511032919973','1.83451122108902','-0.696476020351452','0.379225975108307','0.050163639147938','6','72']
  H_HDV['60_THC_-6_100'] = [' 60','EURO5','RT >20-26t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.0791297934816','0.066117516000893','0.844019665968491','0.664295955581495','0.003129992393669','6','72']
  H_HDV['60_NOx_-6_100'] = [' 60','EURO5','RT >20-26t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.9969877303567','0.060534170325811','159.855349159163','0.709107681718038','0.01851743585687','6','72']
  H_HDV['60_PM_-6_100'] = [' 60','EURO5','RT >20-26t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.119655057202769','0.067265018866818','1.16228262610167','0.556160595832508','0.003881922663299','6','72']
  H_HDV['60_FC_-6_100'] = [' 60','EURO5','RT >20-26t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','281.805293867068','0.057840161356362','24914.9852257608','0.862600075083547','2.51640755139877','6','72']
  H_HDV['60_CO_2_0'] = [' 60','EURO5','RT >20-26t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.106847240032478','0.570387934863013','7.15829987869257','2.80053293206645','-0.017786882832406','6','86']
  H_HDV['60_THC_2_0'] = [' 60','EURO5','RT >20-26t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005095248356688','0.150534925787698','4.60107745447498','1.88247414618713','-0.008198772920461','6','86']
  H_HDV['60_NOx_2_0'] = [' 60','EURO5','RT >20-26t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1476019578661','0.11225024779723','765.791006815336','1.23897622991208','3.1399626782672','6','86']
  H_HDV['60_PM_2_0'] = [' 60','EURO5','RT >20-26t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.050139978296098','0.024864784757327','0.252680370820537','0.125276356725524','0.016817598591537','6','86']
  H_HDV['60_FC_2_0'] = [' 60','EURO5','RT >20-26t Euro-5','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1051.55216995962','0.424487456797391','-67.6676797430014','0.084270297086601','0','6','86']
  H_HDV['60_CO_2_50'] = [' 60','EURO5','RT >20-26t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.0625884701474','-0.000846285859988','-0.094681033531664','0.110049376301512','0','6','86']
  H_HDV['60_THC_2_50'] = [' 60','EURO5','RT >20-26t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007072372295767','0.137481544922232','5.383526201849','2.16792277731449','-0.012272430242474','6','86']
  H_HDV['60_NOx_2_50'] = [' 60','EURO5','RT >20-26t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.4405963821922','-0.011922967584137','-1.40648285893186','0.131655841703735','0','6','86']
  H_HDV['60_PM_2_50'] = [' 60','EURO5','RT >20-26t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055467294775523','0.022258124209894','0.249196117346033','0.128052272137764','0.016363376641769','6','86']
  H_HDV['60_FC_2_50'] = [' 60','EURO5','RT >20-26t Euro-5','2','50','FC','y=((a*(b**x))*(x**c))','2074.87351473521','1.00784445298587','-0.533713836599163','0','0','6','86']
  H_HDV['60_CO_2_100'] = [' 60','EURO5','RT >20-26t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.13109515771326','-0.002044007306345','-0.111743806162305','0.129943530774214','0','6','86']
  H_HDV['60_THC_2_100'] = [' 60','EURO5','RT >20-26t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009818040287631','0.127861911339338','6.04427332227749','2.42637085407702','-0.015900320221289','6','86']
  H_HDV['60_NOx_2_100'] = [' 60','EURO5','RT >20-26t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.19190560376795','0.016740682760392','13.5369957361616','0.218435184315743','3.25563590159801','6','86']
  H_HDV['60_PM_2_100'] = [' 60','EURO5','RT >20-26t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072486533284499','0.024731680074514','0.252036003104216','0.144567919841661','0.015254475788061','6','86']
  H_HDV['60_FC_2_100'] = [' 60','EURO5','RT >20-26t Euro-5','2','100','FC','y=((a*(b**x))*(x**c))','1882.81335928768','1.005731624212','-0.421575963230256','0','0','6','86']
  H_HDV['60_CO_4_0'] = [' 60','EURO5','RT >20-26t Euro-5','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.180350587124765','1.70524357268831','-1.08101704654243','-0.563925735361638','0.159006885687668','6','72']
  H_HDV['60_THC_4_0'] = [' 60','EURO5','RT >20-26t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005986094366854','0.143314112644751','5.1841305593928','2.08195659672146','-0.011259618327003','6','72']
  H_HDV['60_NOx_4_0'] = [' 60','EURO5','RT >20-26t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.72545086764903','0.094509907892989','10.8945017567249','0.253075803065329','4.44373473133286','6','72']
  H_HDV['60_PM_4_0'] = [' 60','EURO5','RT >20-26t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087102447362564','0.045452225380151','0.214351400549937','0.151371327124756','0.02765537674518','6','72']
  H_HDV['60_FC_4_0'] = [' 60','EURO5','RT >20-26t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','762.466217047054','0.096339973954683','9771.88962951817','0.879513477657399','385.30905654224','6','72']
  H_HDV['60_CO_4_50'] = [' 60','EURO5','RT >20-26t Euro-5','4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.749119206742577','0.128853328581727','-0.000966703019463','0','0','6','71']
  H_HDV['60_THC_4_50'] = [' 60','EURO5','RT >20-26t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009823660756443','0.132344887385539','5.82352785723855','2.34655086033554','-0.014541537124847','6','71']
  H_HDV['60_NOx_4_50'] = [' 60','EURO5','RT >20-26t Euro-5','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','2.28710798681828','2.65793668771706','-0.144443372127791','0','0','6','71']
  H_HDV['60_PM_4_50'] = [' 60','EURO5','RT >20-26t Euro-5','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.7466583483628','0.608841013993688','-0.00279506150887','0','0','6','71']
  H_HDV['60_FC_4_50'] = [' 60','EURO5','RT >20-26t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','658.597054084889','0.088077078416857','11916.9543930947','0.870645274586939','553.818328491998','6','71']
  H_HDV['60_CO_4_100'] = [' 60','EURO5','RT >20-26t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.511158558600405','0.020232662917393','0.672502460247028','0.163590925790608','0.013083296374064','6','68']
  H_HDV['60_THC_4_100'] = [' 60','EURO5','RT >20-26t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01378702901259','0.122925819461784','6.32768693072993','2.56834774533579','-0.018257762215272','6','68']
  H_HDV['60_NOx_4_100'] = [' 60','EURO5','RT >20-26t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.92985404624683','0.025066998955927','15.8356883603902','0.267939618307749','5.25766729970533','6','68']
  H_HDV['60_PM_4_100'] = [' 60','EURO5','RT >20-26t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.101280839873572','0.031790829079845','0.234658586430963','0.154574521736576','0.018404242862996','6','68']
  H_HDV['60_FC_4_100'] = [' 60','EURO5','RT >20-26t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','546.235710561552','0.07345377661343','10609.4498189943','0.79581834464584','714.96297416651','6','68']
  H_HDV['60_CO_6_0'] = [' 60','EURO5','RT >20-26t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.44920660331851','-0.002623541200229','-0.222568469407892','0.203012061655423','0','6','71']
  H_HDV['60_THC_6_0'] = [' 60','EURO5','RT >20-26t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00818341488572','0.130400496824486','6.42632315025978','2.54934054577254','-0.021535466511705','6','71']
  H_HDV['60_NOx_6_0'] = [' 60','EURO5','RT >20-26t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.2830868477091','-0.027435596758936','-4.68532192433385','0.24373422498551','0','6','71']
  H_HDV['60_PM_6_0'] = [' 60','EURO5','RT >20-26t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.372805542888818','-0.0006733885662','-0.047871692829555','0.159520238970983','0','6','71']
  H_HDV['60_FC_6_0'] = [' 60','EURO5','RT >20-26t Euro-5','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1244.91870060955','1.61459153294823','-62.8122823187203','0.076482365442184','0','6','71']
  H_HDV['60_CO_6_50'] = [' 60','EURO5','RT >20-26t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.30693653185798','-0.004651935785251','-0.152191855180704','0.172377908163735','0','6','66']
  H_HDV['60_THC_6_50'] = [' 60','EURO5','RT >20-26t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013460073880378','0.121569678461975','6.65355202607824','2.68687710640992','-0.022437643129604','6','66']
  H_HDV['60_NOx_6_50'] = [' 60','EURO5','RT >20-26t Euro-5','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.6878192266196','-0.044875469424542','-2.40493570423277','0.179989239967139','0','6','66']
  H_HDV['60_PM_6_50'] = [' 60','EURO5','RT >20-26t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.339050778994462','-0.000734289396394','-0.033422958621858','0.124987579239522','0','6','66']
  H_HDV['60_FC_6_50'] = [' 60','EURO5','RT >20-26t Euro-5','6','50','FC','y=((a*(b**x))*(x**c))','2058.57453689237','1.00632511169454','-0.340052182593783','0','0','6','66']
  H_HDV['60_CO_6_100'] = [' 60','EURO5','RT >20-26t Euro-5','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.22665864203378','-0.0043039658039','-0.091277697089513','0.106097080865558','0','6','52']
  H_HDV['60_THC_6_100'] = [' 60','EURO5','RT >20-26t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016689862854442','0.117633631835149','6.55081694016971','2.68073900024221','-0.02345746981106','6','52']
  H_HDV['60_NOx_6_100'] = [' 60','EURO5','RT >20-26t Euro-5','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.3305164821076','-0.056457684042487','-1.99771892301739','0.152034192052076','0','6','52']
  H_HDV['60_PM_6_100'] = [' 60','EURO5','RT >20-26t Euro-5','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.166584612283783','0.053251979478798','0.18515515851735','0.181321412208424','0.027461714639441','6','52']
  H_HDV['60_FC_6_100'] = [' 60','EURO5','RT >20-26t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','465.473139846314','0.059760284300967','6117.88201156229','0.651873812460908','948.385540834898','6','52']
  H_HDV['61_CO_0_0'] = [' 61','EURO0','RT >26-28t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.64914277390132','0.052539698118341','11.3585322203579','0.221634732796563','1.4469411849371','6','86']
  H_HDV['61_THC_0_0'] = [' 61','EURO0','RT >26-28t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.65924357829311','0.024969815581759','5.1088472983362','0.123803632412059','0.278238018338311','6','86']
  H_HDV['61_NOx_0_0'] = [' 61','EURO0','RT >26-28t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.4266247307414','0.064052066761559','60.3414347264393','0.462372330805873','8.55374062720163','6','86']
  H_HDV['61_PM_0_0'] = [' 61','EURO0','RT >26-28t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.30583481684597','0.048797609710004','2.08731946534942','0.19138358897557','0.309201689256166','6','86']
  H_HDV['61_FC_0_0'] = [' 61','EURO0','RT >26-28t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','701.120310681103','0.061014614432468','4934.4477359962','0.555701760635359','205.271613869249','6','86']
  H_HDV['61_CO_0_50'] = [' 61','EURO0','RT >26-28t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.13411883364323','0.054211080306011','15.8643543525583','0.275320506271409','1.64310174818499','6','86']
  H_HDV['61_THC_0_50'] = [' 61','EURO0','RT >26-28t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5316799416869','0.024523097520117','5.36991758512458','0.124006848764087','0.246337810165474','6','86']
  H_HDV['61_NOx_0_50'] = [' 61','EURO0','RT >26-28t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.1184520190778','0.046169038632911','94.2835206151738','0.537881650750213','9.89341403519485','6','86']
  H_HDV['61_PM_0_50'] = [' 61','EURO0','RT >26-28t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.43156187475234','0.049009128667786','2.5973197096804','0.247362040708152','0.355389813811357','6','86']
  H_HDV['61_FC_0_50'] = [' 61','EURO0','RT >26-28t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','666.989178610959','0.051050952659863','5141.67058484724','0.556974731846176','230.898545662676','6','86']
  H_HDV['61_CO_0_100'] = [' 61','EURO0','RT >26-28t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.53980932090852','0.053245110955273','22.5512405773693','0.31858699717415','1.83095871169877','6','86']
  H_HDV['61_THC_0_100'] = [' 61','EURO0','RT >26-28t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.45767195670061','0.026268965547095','5.53419931596532','0.127977604891732','0.277407586809625','6','86']
  H_HDV['61_NOx_0_100'] = [' 61','EURO0','RT >26-28t 80ties','0','100','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.027811640505262','0.001044068110477','-0.000004190168148','0','0','6','86']
  H_HDV['61_PM_0_100'] = [' 61','EURO0','RT >26-28t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.4938074498785','0.046309224824966','3.4726733165552','0.293808415128897','0.395777363082899','6','86']
  H_HDV['61_FC_0_100'] = [' 61','EURO0','RT >26-28t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','655.594636993979','0.042228315741691','7208.34797032394','0.611671081695247','252.020226802023','6','86']
  H_HDV['61_CO_-2_0'] = [' 61','EURO0','RT >26-28t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.3244335042857','0.010310205183399','-0.765984409661694','0.064787715567574','0','6','86']
  H_HDV['61_THC_-2_0'] = [' 61','EURO0','RT >26-28t 80ties','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.39595701107127','14.033640653684','0.014524414882242','0.513355516637493','0.045298640910337','6','86']
  H_HDV['61_NOx_-2_0'] = [' 61','EURO0','RT >26-28t 80ties','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.1809669918964','0.056912819814055','331.959756049179','0.718681395860232','3.03920174718897','6','86']
  H_HDV['61_PM_-2_0'] = [' 61','EURO0','RT >26-28t 80ties','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.20615338871504','2.25850366063339','3.19942827697039','1.23705821807703','0.029213575319867','6','86']
  H_HDV['61_FC_-2_0'] = [' 61','EURO0','RT >26-28t 80ties','-2','0','FC','y=((a*(x**b))+(c*(x**d)))','169352072.088528','-8.3378351206417','2705.25937228478','-0.78687386178677','0','6','86']
  H_HDV['61_CO_-2_50'] = [' 61','EURO0','RT >26-28t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.2586556865815','0.007890420902032','-0.755961061752134','0.063924860595969','0','6','86']
  H_HDV['61_THC_-2_50'] = [' 61','EURO0','RT >26-28t 80ties','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.24874316636622','16.4750503830533','-0.12045741325549','0.580260979475774','0.035041784461719','6','86']
  H_HDV['61_NOx_-2_50'] = [' 61','EURO0','RT >26-28t 80ties','-2','50','NOx','y=((a*(b**x))*(x**c))','48.6726555663217','0.985379377042274','-0.425309461854543','0','0','6','86']
  H_HDV['61_PM_-2_50'] = [' 61','EURO0','RT >26-28t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.69402651916927','0.055154535041311','1.46176972130615','0.220608815106234','0.189606591207031','6','86']
  H_HDV['61_FC_-2_50'] = [' 61','EURO0','RT >26-28t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','574.304558179789','0.043380862273786','22572.6120425829','0.772918874399204','47.5469365064133','6','86']
  H_HDV['61_CO_-2_100'] = [' 61','EURO0','RT >26-28t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.61587628758074','0.058750324398824','117.640300231274','0.703307179817614','0.755653335421584','6','86']
  H_HDV['61_THC_-2_100'] = [' 61','EURO0','RT >26-28t 80ties','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.172225646124919','29.174543353347','-0.860444715521229','0.564395638409775','0.030749599211161','6','86']
  H_HDV['61_NOx_-2_100'] = [' 61','EURO0','RT >26-28t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.4480481057564','-0.088517272810043','-1.52743365824324','0.07788497165302','0','6','86']
  H_HDV['61_PM_-2_100'] = [' 61','EURO0','RT >26-28t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70409579609413','0.050808514802976','3.8686887490192','0.366196076086421','0.174999611398004','6','86']
  H_HDV['61_FC_-2_100'] = [' 61','EURO0','RT >26-28t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','550.987715835746','0.033885880751898','32156.0657317438','0.817754336674595','19.0820075824691','6','86']
  H_HDV['61_CO_-4_0'] = [' 61','EURO0','RT >26-28t 80ties','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.07497715672414','0.007779804260437','0.000334606845498','0','0','6','86']
  H_HDV['61_THC_-4_0'] = [' 61','EURO0','RT >26-28t 80ties','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.13654425324514','0.025258351375938','0.00044537471435','0','0','6','86']
  H_HDV['61_NOx_-4_0'] = [' 61','EURO0','RT >26-28t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.7022447012906','0.05004218839555','246.008540746545','0.612942852977129','0.426389901134009','6','86']
  H_HDV['61_PM_-4_0'] = [' 61','EURO0','RT >26-28t 80ties','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.546366242251649','0.022853680504208','2.10738887109388','0.103383701317542','0.043128543465593','6','86']
  H_HDV['61_FC_-4_0'] = [' 61','EURO0','RT >26-28t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','482.356194131675','0.051616739986716','15727.5178004209','0.706723856479211','12.0103671715192','6','86']
  H_HDV['61_CO_-4_50'] = [' 61','EURO0','RT >26-28t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.3982303593736','0.056918602533117','51.0937311178335','0.549150041865467','0.258068113865926','6','86']
  H_HDV['61_THC_-4_50'] = [' 61','EURO0','RT >26-28t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.90283843651665','0.053797045417413','9.85323837391959','0.397177443388402','0.153557770289316','6','86']
  H_HDV['61_NOx_-4_50'] = [' 61','EURO0','RT >26-28t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4785536621535','0.043152622491957','566.388048378849','0.750072224138258','0.168688173641815','6','86']
  H_HDV['61_PM_-4_50'] = [' 61','EURO0','RT >26-28t 80ties','-4','50','PM','y=((a+(b*x))**((-1)/c))','0.409549051772384','0.049349720253329','0.716130972036924','0','0','6','86']
  H_HDV['61_FC_-4_50'] = [' 61','EURO0','RT >26-28t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','416.812314821744','0.045483687538106','26712.2866034469','0.79228965350411','4.13155573602659','6','86']
  H_HDV['61_CO_-4_100'] = [' 61','EURO0','RT >26-28t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.5807970731851','0.050521530286552','115.816738969992','0.65301633852903','0.18955469065683','6','86']
  H_HDV['61_THC_-4_100'] = [' 61','EURO0','RT >26-28t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.7662004016041','0.053773475553761','13.5256953822161','0.449200520574478','0.148141211824825','6','86']
  H_HDV['61_NOx_-4_100'] = [' 61','EURO0','RT >26-28t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4340345162849','0.037036520362226','1096.49725739939','0.859175177939705','-0.086745465849787','6','86']
  H_HDV['61_PM_-4_100'] = [' 61','EURO0','RT >26-28t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.38814924433273','0.048372278172362','5.11898146315737','0.389276626964946','0.105179439688167','6','86']
  H_HDV['61_FC_-4_100'] = [' 61','EURO0','RT >26-28t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','384.935376699684','0.04092311771011','40819.9858246903','0.862868040669892','-0.933405206439313','6','86']
  H_HDV['61_CO_-6_0'] = [' 61','EURO0','RT >26-28t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.08764213416273','0.069548930357934','51.8560989615508','0.638003125594287','0.218525076204874','6','72']
  H_HDV['61_THC_-6_0'] = [' 61','EURO0','RT >26-28t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.32881311375014','0.064078315313785','15.0246063048622','0.531641016168591','0.152516824696687','6','72']
  H_HDV['61_NOx_-6_0'] = [' 61','EURO0','RT >26-28t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.3766548932315','0.063447317546786','574.485424624863','0.783382134882398','0.237505701646783','6','72']
  H_HDV['61_PM_-6_0'] = [' 61','EURO0','RT >26-28t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.63555307613955','0.062686672362169','1.66329520893018','0.275360765134173','0.115517187980011','6','72']
  H_HDV['61_FC_-6_0'] = [' 61','EURO0','RT >26-28t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','423.243993171322','0.061453799716102','24956.9032892359','0.799574688811852','3.73236880369562','6','72']
  H_HDV['61_CO_-6_50'] = [' 61','EURO0','RT >26-28t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.51821852692581','0.063304318400448','81.6194854482312','0.65503117760927','0.186658743238506','6','72']
  H_HDV['61_THC_-6_50'] = [' 61','EURO0','RT >26-28t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.92437387974093','0.061747843879235','19.9127446436509','0.554560289312462','0.145237796997474','6','72']
  H_HDV['61_NOx_-6_50'] = [' 61','EURO0','RT >26-28t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3022722728023','0.057616854617298','770.257471914313','0.81899026543596','0.186634287249146','6','72']
  H_HDV['61_PM_-6_50'] = [' 61','EURO0','RT >26-28t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51527819037272','0.060033939351761','3.09049906772683','0.367657216805696','0.111770135752631','6','72']
  H_HDV['61_FC_-6_50'] = [' 61','EURO0','RT >26-28t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','343.489089133995','0.057027179023857','30430.2160213324','0.827280998254472','2.37666048287714','6','72']
  H_HDV['61_CO_-6_100'] = [' 61','EURO0','RT >26-28t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.82610880594219','0.059512348987969','106.24573380154','0.677685183055357','0.170520978723683','6','72']
  H_HDV['61_THC_-6_100'] = [' 61','EURO0','RT >26-28t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.70592734711926','0.060487454904392','19.0595804187062','0.542094550021636','0.142230124390616','6','72']
  H_HDV['61_NOx_-6_100'] = [' 61','EURO0','RT >26-28t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.97488500018797','0.054131206626671','869.822138523991','0.83795837161001','0.170228751844572','6','72']
  H_HDV['61_PM_-6_100'] = [' 61','EURO0','RT >26-28t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.46084052044532','0.058512951945936','4.05766424667256','0.406708611607968','0.11027659718924','6','72']
  H_HDV['61_FC_-6_100'] = [' 61','EURO0','RT >26-28t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','305.012231717159','0.05429569659948','33563.8938061784','0.843391739543433','1.97393020959739','6','72']
  H_HDV['61_CO_2_0'] = [' 61','EURO0','RT >26-28t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.89423598974578','11.5886460718499','4.39538993535146','1.61759094180221','0.026718343225708','6','86']
  H_HDV['61_THC_2_0'] = [' 61','EURO0','RT >26-28t 80ties','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','4.6326253325357','-5.46469308955419','-1.23433283727546','0','0','6','86']
  H_HDV['61_NOx_2_0'] = [' 61','EURO0','RT >26-28t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-21.5939986637123','0.034134284899673','49.8036514259648','0.06371620354616','17.0396407811204','6','86']
  H_HDV['61_PM_2_0'] = [' 61','EURO0','RT >26-28t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.465378390351758','1.93829683037123','5.0261265787823','1.88572621261296','0.0177879877923','6','86']
  H_HDV['61_FC_2_0'] = [' 61','EURO0','RT >26-28t 80ties','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1067.21609733526','0.08496126494166','49078.7169010885','1.11940673773002','336.317048715476','6','86']
  H_HDV['61_CO_2_50'] = [' 61','EURO0','RT >26-28t 80ties','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.47626813742067','0.059165504567045','8.37165623873794','0.149596893078761','2.13649959107799','6','86']
  H_HDV['61_THC_2_50'] = [' 61','EURO0','RT >26-28t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.331849763137078','5.06227133708411','5.64771262788014','2.25874963160128','-0.013079376113798','6','86']
  H_HDV['61_NOx_2_50'] = [' 61','EURO0','RT >26-28t 80ties','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-27.5209987237975','0.037025900509512','52.4030232872885','0.05821547308147','22.9905132170011','6','86']
  H_HDV['61_PM_2_50'] = [' 61','EURO0','RT >26-28t 80ties','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.217781649023588','0.039335536571092','-0.000266249634594','0','0','6','86']
  H_HDV['61_FC_2_50'] = [' 61','EURO0','RT >26-28t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1563.89648056527','0.886165200402712','-93.1255818019937','0.081371389998758','0','6','86']
  H_HDV['61_CO_2_100'] = [' 61','EURO0','RT >26-28t 80ties','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.15052964162885','0.046798078001707','12.6990952628654','0.156577941339502','2.19593709102051','6','84']
  H_HDV['61_THC_2_100'] = [' 61','EURO0','RT >26-28t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.425453849740727','4.89604767639386','6.02579712953398','2.43538078088692','-0.015155410443891','6','84']
  H_HDV['61_NOx_2_100'] = [' 61','EURO0','RT >26-28t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00009111932596','0.016833457001373','-1.01467328710342','47.5100705968449','0','6','84']
  H_HDV['61_PM_2_100'] = [' 61','EURO0','RT >26-28t 80ties','2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.232010272993112','0.03178381694244','-0.000217469421968','0','0','6','84']
  H_HDV['61_FC_2_100'] = [' 61','EURO0','RT >26-28t 80ties','2','100','FC','y=((a*(b**x))*(x**c))','2941.74189034258','1.00799530222564','-0.506060280057795','0','0','6','84']
  H_HDV['61_CO_4_0'] = [' 61','EURO0','RT >26-28t 80ties','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.9961667521026','-0.025991971148937','-1.74023916517313','0.106439280592676','0','6','71']
  H_HDV['61_THC_4_0'] = [' 61','EURO0','RT >26-28t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.259730433569298','5.37254384506451','5.72349518424198','2.2771156456521','-0.015686700975438','6','71']
  H_HDV['61_NOx_4_0'] = [' 61','EURO0','RT >26-28t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','54.8788394214628','0.055082468818165','-3.31264334872426','0.093894456907138','0','6','71']
  H_HDV['61_PM_4_0'] = [' 61','EURO0','RT >26-28t 80ties','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.47399388266423','-0.003465953755785','-0.275855491105006','0.102990764273709','0','6','71']
  H_HDV['61_FC_4_0'] = [' 61','EURO0','RT >26-28t 80ties','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1736.73166486076','0.620390480574576','-122.789007907895','0.095049996451583','0','6','71']
  H_HDV['61_CO_4_50'] = [' 61','EURO0','RT >26-28t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.3439878016906','-0.03316973697074','-2.01644760533548','0.112568705858484','0','6','69']
  H_HDV['61_THC_4_50'] = [' 61','EURO0','RT >26-28t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.455398584075588','5.15522620877413','6.22025963001003','2.53157571908092','-0.017015409328221','6','69']
  H_HDV['61_NOx_4_50'] = [' 61','EURO0','RT >26-28t 80ties','4','50','NOx','y=(c+(a*exp(((-1)*b)*x)))','29.5006917919942','0.113579197803612','32.6694121432876','0','0','6','69']
  H_HDV['61_PM_4_50'] = [' 61','EURO0','RT >26-28t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.80060600174315','-0.004519177812283','-0.302421134323413','0.109873318225083','0','6','69']
  H_HDV['61_FC_4_50'] = [' 61','EURO0','RT >26-28t 80ties','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.008598932654351','1.27039454357586','-60.9671789309917','1653.48080954227','0','6','69']
  H_HDV['61_CO_4_100'] = [' 61','EURO0','RT >26-28t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.0207413591321','-0.028581512404271','-2.04787406799885','0.106173922007393','0','6','61']
  H_HDV['61_THC_4_100'] = [' 61','EURO0','RT >26-28t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.594514417370834','4.79412613613032','6.91822945028928','2.83689080847551','-0.024344295047086','6','61']
  H_HDV['61_NOx_4_100'] = [' 61','EURO0','RT >26-28t 80ties','4','100','NOx','y=exp((a+(b/x))+(c*log(x)))','4.00503525128914','0.873666858624635','-0.071270279268268','0','0','6','61']
  H_HDV['61_PM_4_100'] = [' 61','EURO0','RT >26-28t 80ties','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.00329123256262','-0.003145666142387','-0.282864107311293','0.101086034633777','0','6','61']
  H_HDV['61_FC_4_100'] = [' 61','EURO0','RT >26-28t 80ties','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','509.226068228337','0.05964977179042','1800.05003791607','0.301712731987451','887.078506330984','6','61']
  H_HDV['61_CO_6_0'] = [' 61','EURO0','RT >26-28t 80ties','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.46516094305285','13.9421775386025','5.07357682726869','2.06829986287787','-0.015956098729976','6','71']
  H_HDV['61_THC_6_0'] = [' 61','EURO0','RT >26-28t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.441579710197484','5.10421384467995','6.71599468851405','2.67581598926458','-0.018984062920904','6','71']
  H_HDV['61_NOx_6_0'] = [' 61','EURO0','RT >26-28t 80ties','6','0','NOx','y=((a*(b**x))*(x**c))','90.543344253779','1.00936229535939','-0.409596499428227','0','0','6','71']
  H_HDV['61_PM_6_0'] = [' 61','EURO0','RT >26-28t 80ties','6','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.217656896827059','0.035229541696575','-0.000278079798287','0','0','6','71']
  H_HDV['61_FC_6_0'] = [' 61','EURO0','RT >26-28t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1826.14270146827','2.43087821652391','-111.498593694555','0.084430476173889','0','6','71']
  H_HDV['61_CO_6_50'] = [' 61','EURO0','RT >26-28t 80ties','6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.37809376185184','14.8670644157003','5.15757518220827','2.13746998403802','-0.015133354006647','6','57']
  H_HDV['61_THC_6_50'] = [' 61','EURO0','RT >26-28t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.588271026719424','4.84565753123086','7.13791623599244','2.91621410326579','-0.027088399634555','6','57']
  H_HDV['61_NOx_6_50'] = [' 61','EURO0','RT >26-28t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.2837281339039','0.075144194118891','62.3086229979524','0.435026903711811','43.2028823063233','6','57']
  H_HDV['61_PM_6_50'] = [' 61','EURO0','RT >26-28t 80ties','6','50','PM','y=(c+(a*exp(((-1)*b)*x)))','2.77976263166793','0.088671822927601','1.05890533807989','0','0','6','57']
  H_HDV['61_FC_6_50'] = [' 61','EURO0','RT >26-28t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','576.147564533028','0.076119285058371','1539.30787037541','0.285356969940593','918.09169249786','6','57']
  H_HDV['61_CO_6_100'] = [' 61','EURO0','RT >26-28t 80ties','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.5812446595225','-0.053289903816144','-2.88883481369246','0.1299827950927','0','6','45']
  H_HDV['61_THC_6_100'] = [' 61','EURO0','RT >26-28t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.02836927587915','28.9329762027931','-2.05048962859588','-0.634848462836842','0.167199814632177','6','45']
  H_HDV['61_NOx_6_100'] = [' 61','EURO0','RT >26-28t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.1004069747931','0.01702329395152','195.34600542996','0.567553110313164','49.0622940253541','6','45']
  H_HDV['61_PM_6_100'] = [' 61','EURO0','RT >26-28t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.87961326410088','-0.006547643010623','-0.422328150839439','0.131464377189454','0','6','45']
  H_HDV['61_FC_6_100'] = [' 61','EURO0','RT >26-28t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','451.437546794419','0.046555842164901','2690.29116837078','0.366473355329072','1175.42186173362','6','45']
  H_HDV['62_CO_0_0'] = [' 62','EURO1','RT >26-28t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.66316921056225','0.051964217547405','9.03523153506964','0.223251723903451','1.09749268878833','6','86']
  H_HDV['62_THC_0_0'] = [' 62','EURO1','RT >26-28t Euro-1','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.45827597934404','0.031305502053175','4.10390676807895','0.134750223411385','0.334353265045532','6','86']
  H_HDV['62_NOx_0_0'] = [' 62','EURO1','RT >26-28t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.0982645379533','0.06396863999449','61.8049988273877','0.477229002473768','6.09781404791952','6','86']
  H_HDV['62_PM_0_0'] = [' 62','EURO1','RT >26-28t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.956940722594142','0.042597461837158','2.68929485110739','0.164534412421332','0.197377530668736','6','86']
  H_HDV['62_FC_0_0'] = [' 62','EURO1','RT >26-28t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','536.31440596181','0.060857087226929','13293.3091291597','0.767269783982915','174.178206367743','6','86']
  H_HDV['62_CO_0_50'] = [' 62','EURO1','RT >26-28t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.73535271116126','0.051478862500359','13.7292344776583','0.277004301618467','1.3437024869028','6','86']
  H_HDV['62_THC_0_50'] = [' 62','EURO1','RT >26-28t Euro-1','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.36913341907543','0.030076076632926','4.16528444929436','0.130814820111219','0.321334357738875','6','86']
  H_HDV['62_NOx_0_50'] = [' 62','EURO1','RT >26-28t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.7649430218224','0.047139670176391','93.369277972383','0.550739160678484','6.93777830085879','6','86']
  H_HDV['62_PM_0_50'] = [' 62','EURO1','RT >26-28t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.19643219195567','0.049662375479922','2.79763916888652','0.211100124197997','0.246246967566125','6','86']
  H_HDV['62_FC_0_50'] = [' 62','EURO1','RT >26-28t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','523.535820661854','0.049423050788441','17421.4796042017','0.809907162438699','201.80527029554','6','86']
  H_HDV['62_CO_0_100'] = [' 62','EURO1','RT >26-28t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.00256113574229','0.048451951700549','20.214355908474','0.333155478857967','1.60323710673759','6','86']
  H_HDV['62_THC_0_100'] = [' 62','EURO1','RT >26-28t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.29362663709456','0.028604830021661','4.17862821021588','0.129381735726179','0.323373757933844','6','86']
  H_HDV['62_NOx_0_100'] = [' 62','EURO1','RT >26-28t Euro-1','0','100','NOx','y=((a+(b*x))**((-1)/c))','0.001598439356494','0.000226420948078','1.8928515399124','0','0','6','86']
  H_HDV['62_PM_0_100'] = [' 62','EURO1','RT >26-28t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.24915571448588','0.048791005428889','3.30675686661638','0.251588092069605','0.293907209753858','6','86']
  H_HDV['62_FC_0_100'] = [' 62','EURO1','RT >26-28t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','532.873616913478','0.039814977188771','44924.0033034579','0.975060343804592','220.470736258684','6','86']
  H_HDV['62_CO_-2_0'] = [' 62','EURO1','RT >26-28t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.58291163462741','0.008149362116296','-0.612765370907593','0.066457401567695','0','6','86']
  H_HDV['62_THC_-2_0'] = [' 62','EURO1','RT >26-28t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.306726230517462','6.70845225031529','1.22129632658997','0.763585045869575','0.03872657058515','6','86']
  H_HDV['62_NOx_-2_0'] = [' 62','EURO1','RT >26-28t Euro-1','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.5167139393963','0.055853306270949','107.25126940659','0.544640119530015','2.11209042932975','6','86']
  H_HDV['62_PM_-2_0'] = [' 62','EURO1','RT >26-28t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.114482655202014','1.89431500170468','7.44599790021946','2.9225714811628','-0.02915537094703','6','86']
  H_HDV['62_FC_-2_0'] = [' 62','EURO1','RT >26-28t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','517.159057138176','0.05918394665467','26979.2684353458','0.877507916481568','67.7685821444641','6','86']
  H_HDV['62_CO_-2_50'] = [' 62','EURO1','RT >26-28t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.47337264017056','0.006724041590197','-0.589948979373175','0.0643107843537','0','6','86']
  H_HDV['62_THC_-2_50'] = [' 62','EURO1','RT >26-28t Euro-1','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.174012798741401','8.91132685534826','0.684469332214688','0.736108412586021','0.028835584251067','6','86']
  H_HDV['62_NOx_-2_50'] = [' 62','EURO1','RT >26-28t Euro-1','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.041855164528878','0.852967193820805','49.4207556019533','-0.618518908859582','0','6','86']
  H_HDV['62_PM_-2_50'] = [' 62','EURO1','RT >26-28t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.005921975677628','2.0731006436167','6.08807104456999','2.44764788271225','-0.027849550672394','6','86']
  H_HDV['62_FC_-2_50'] = [' 62','EURO1','RT >26-28t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','466.59931547343','0.04255670550686','24490.9265563718','0.83033233327678','40.6355281540601','6','86']
  H_HDV['62_CO_-2_100'] = [' 62','EURO1','RT >26-28t Euro-1','-2','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.545211148023894','51.9143931671961','-0.947413725044413','0.438639977855604','0.040312191697998','6','86']
  H_HDV['62_THC_-2_100'] = [' 62','EURO1','RT >26-28t Euro-1','-2','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.168057443165634','0.02854296913521','0.000437845434145','0','0','6','86']
  H_HDV['62_NOx_-2_100'] = [' 62','EURO1','RT >26-28t Euro-1','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.012916849948962','1.19733371725038','42.593717999094','-0.552766526799477','0','6','86']
  H_HDV['62_PM_-2_100'] = [' 62','EURO1','RT >26-28t Euro-1','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.104765909505264','2.57003440464749','3.21099081534502','1.3712427254277','0.018036376593803','6','86']
  H_HDV['62_FC_-2_100'] = [' 62','EURO1','RT >26-28t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','461.193709708027','0.032920369198712','35415.95636808','0.879138207202779','16.5451414464901','6','86']
  H_HDV['62_CO_-4_0'] = [' 62','EURO1','RT >26-28t Euro-1','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.098278270178805','0.009875408465658','0.000507463524382','0','0','6','86']
  H_HDV['62_THC_-4_0'] = [' 62','EURO1','RT >26-28t Euro-1','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.186825911395139','0.026576844613914','0.000816493708627','0','0','6','86']
  H_HDV['62_NOx_-4_0'] = [' 62','EURO1','RT >26-28t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4305875482627','0.052721864520563','119.268255378702','0.527743249411005','0.358469331470874','6','86']
  H_HDV['62_PM_-4_0'] = [' 62','EURO1','RT >26-28t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.18310708855515','-8.92267793031271','-1.75618349009467','0','0','6','86']
  H_HDV['62_FC_-4_0'] = [' 62','EURO1','RT >26-28t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','392.780572990916','0.051544892108233','12046.9361814621','0.704120562594982','10.14709294692','6','86']
  H_HDV['62_CO_-4_50'] = [' 62','EURO1','RT >26-28t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.43685840344498','0.056111403215301','36.4224383485617','0.518296069180373','0.158888139674228','6','86']
  H_HDV['62_THC_-4_50'] = [' 62','EURO1','RT >26-28t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.34273155684046','0.05450606980505','8.00303618718983','0.389872536919843','0.090571658758563','6','86']
  H_HDV['62_NOx_-4_50'] = [' 62','EURO1','RT >26-28t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6690972243644','0.046656178775418','253.398893885075','0.649308529066488','0.210597426834172','6','86']
  H_HDV['62_PM_-4_50'] = [' 62','EURO1','RT >26-28t Euro-1','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.810543195326','-0.002015574234213','-0.259423408636945','0.099567769054252','0','6','86']
  H_HDV['62_FC_-4_50'] = [' 62','EURO1','RT >26-28t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.384059048395','0.045067330334574','19396.2224708448','0.778155804744071','3.52679024590931','6','86']
  H_HDV['62_CO_-4_100'] = [' 62','EURO1','RT >26-28t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.90964959751342','0.048906497000572','90.6302762576915','0.646284433986002','0.098208491399732','6','86']
  H_HDV['62_THC_-4_100'] = [' 62','EURO1','RT >26-28t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.20022233963313','0.053793193784183','11.6972219412454','0.452213145403629','0.084455451286798','6','86']
  H_HDV['62_NOx_-4_100'] = [' 62','EURO1','RT >26-28t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.7618738639296','0.041024299499691','459.288040461592','0.747980526451202','0.071907391022917','6','86']
  H_HDV['62_PM_-4_100'] = [' 62','EURO1','RT >26-28t Euro-1','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.240823407551821','0.056557044792524','0.001577822554222','0','0','6','86']
  H_HDV['62_FC_-4_100'] = [' 62','EURO1','RT >26-28t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','329.46890055931','0.040469518301144','29869.9680349426','0.849966265795817','-0.953073426789996','6','86']
  H_HDV['62_CO_-6_0'] = [' 62','EURO1','RT >26-28t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.12078504347629','0.071432789637988','32.1715469360296','0.577449371421166','0.134887805225895','6','72']
  H_HDV['62_THC_-6_0'] = [' 62','EURO1','RT >26-28t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.7258768392262','0.066530264625171','11.0846393388307','0.503898313486769','0.093348880915195','6','72']
  H_HDV['62_NOx_-6_0'] = [' 62','EURO1','RT >26-28t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1413453681005','0.067868676796068','272.073317769412','0.697152344372049','0.233533136338635','6','72']
  H_HDV['62_PM_-6_0'] = [' 62','EURO1','RT >26-28t Euro-1','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.056266403397689','2.439092750808','3.07639423922795','1.2785098201097','0.037153027851755','6','72']
  H_HDV['62_FC_-6_0'] = [' 62','EURO1','RT >26-28t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','355.13817740797','0.062061322722973','17220.5698250403','0.780097854430359','3.81349589360028','6','72']
  H_HDV['62_CO_-6_50'] = [' 62','EURO1','RT >26-28t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.81616377124182','0.06405026061009','54.6652161630139','0.605359905538247','0.109978380376859','6','72']
  H_HDV['62_THC_-6_50'] = [' 62','EURO1','RT >26-28t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.31468292424231','0.063541788216029','17.2071398942941','0.551950961432255','0.087012355951759','6','72']
  H_HDV['62_NOx_-6_50'] = [' 62','EURO1','RT >26-28t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.36558690324476','0.061631619213724','388.362765845388','0.736239517522533','0.194682668491626','6','72']
  H_HDV['62_PM_-6_50'] = [' 62','EURO1','RT >26-28t Euro-1','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.55825487859416','0.070648157903065','3.4453746107537','0.357687048139466','0.060254876101198','6','72']
  H_HDV['62_FC_-6_50'] = [' 62','EURO1','RT >26-28t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','293.981225218444','0.057599208002581','21450.6815318933','0.807763975740307','2.75598278577463','6','72']
  H_HDV['62_CO_-6_100'] = [' 62','EURO1','RT >26-28t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.32697552154552','0.059657708190277','82.4762562434729','0.664326241074106','0.096485505263734','6','72']
  H_HDV['62_THC_-6_100'] = [' 62','EURO1','RT >26-28t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.10569196886256','0.061872134927253','17.37267893681','0.547976666586522','0.084134230571161','6','72']
  H_HDV['62_NOx_-6_100'] = [' 62','EURO1','RT >26-28t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.26074087808075','0.058222380826378','443.306264465017','0.757221993129849','0.182115134970145','6','72']
  H_HDV['62_PM_-6_100'] = [' 62','EURO1','RT >26-28t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.40855271928981','0.067696525095938','5.30759684922574','0.43008662878455','0.057858542618055','6','72']
  H_HDV['62_FC_-6_100'] = [' 62','EURO1','RT >26-28t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','264.203903663933','0.054508854696226','24228.6779952598','0.827712059706252','2.25186103796444','6','72']
  H_HDV['62_CO_2_0'] = [' 62','EURO1','RT >26-28t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.246958485797','0.009686526474874','-0.775310325334045','0.06902577679282','0','6','86']
  H_HDV['62_THC_2_0'] = [' 62','EURO1','RT >26-28t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.18645739498749','0.019790095789562','4.50907942477836','0.105255709240906','0.239442622922726','6','86']
  H_HDV['62_NOx_2_0'] = [' 62','EURO1','RT >26-28t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.2577549573322','0.012055727249661','-2.18995701996161','0.090888815424126','0','6','86']
  H_HDV['62_PM_2_0'] = [' 62','EURO1','RT >26-28t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.178777064025102','1.89222677861463','6.58767526980896','2.67922147964818','-0.033513212474238','6','86']
  H_HDV['62_FC_2_0'] = [' 62','EURO1','RT >26-28t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.141662974470316','1.53047841987195','2111.25924359645','-0.546233660200345','0','6','86']
  H_HDV['62_CO_2_50'] = [' 62','EURO1','RT >26-28t Euro-1','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.032475597125361','0.012666017246048','-0.000098673703067','0','0','6','86']
  H_HDV['62_THC_2_50'] = [' 62','EURO1','RT >26-28t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.432906468291128','3.74772643053791','6.39547994332818','2.56952742490774','-0.02566042346771','6','86']
  H_HDV['62_NOx_2_50'] = [' 62','EURO1','RT >26-28t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.070175928071','-0.020512393512586','-1.9157552791592','0.092927648503736','0','6','86']
  H_HDV['62_PM_2_50'] = [' 62','EURO1','RT >26-28t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.78782084015364','0.002782657895296','-0.190995294791737','0.077092249567751','0','6','86']
  H_HDV['62_FC_2_50'] = [' 62','EURO1','RT >26-28t Euro-1','2','50','FC','y=((a*(b**x))*(x**c))','2241.02694183458','1.00852571691185','-0.533927465679209','0','0','6','86']
  H_HDV['62_CO_2_100'] = [' 62','EURO1','RT >26-28t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.6263590497881','-0.014033248778442','-1.19624186807006','0.098431580108145','0','6','85']
  H_HDV['62_THC_2_100'] = [' 62','EURO1','RT >26-28t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.535560775632334','3.66110564558458','6.68861156396999','2.72730976527812','-0.031453994293872','6','85']
  H_HDV['62_NOx_2_100'] = [' 62','EURO1','RT >26-28t Euro-1','2','100','NOx','y=((a*(b**x))*(x**c))','50.2360852541586','1.00251916002984','-0.286956952759897','0','0','6','85']
  H_HDV['62_PM_2_100'] = [' 62','EURO1','RT >26-28t Euro-1','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.90012867872277','0.001649232466616','-0.203217341053429','0.085275213094607','0','6','85']
  H_HDV['62_FC_2_100'] = [' 62','EURO1','RT >26-28t Euro-1','2','100','FC','y=((a*(b**x))*(x**c))','2040.09908011095','1.00606054054526','-0.416890907945992','0','0','6','85']
  H_HDV['62_CO_4_0'] = [' 62','EURO1','RT >26-28t Euro-1','4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-42.1113897628358','0.564073251761011','15.4216993393118','0.102775439952222','2.09949290056553','6','71']
  H_HDV['62_THC_4_0'] = [' 62','EURO1','RT >26-28t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.452528561760217','3.94076821509954','6.22168795995724','2.49700776709269','-0.022115832590964','6','71']
  H_HDV['62_NOx_4_0'] = [' 62','EURO1','RT >26-28t Euro-1','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','41.1643605784495','-0.011106816055617','-2.92748110859084','0.115530051006218','0','6','71']
  H_HDV['62_PM_4_0'] = [' 62','EURO1','RT >26-28t Euro-1','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.79452008780045','0.000747497422759','-0.210500681360649','0.087848101872451','0','6','71']
  H_HDV['62_FC_4_0'] = [' 62','EURO1','RT >26-28t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','2474.26070512354','1.01027542735341','-0.579629345727155','0','0','6','71']
  H_HDV['62_CO_4_50'] = [' 62','EURO1','RT >26-28t Euro-1','4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.043375194680622','0.009436770680883','-0.0000813666458','0','0','6','70']
  H_HDV['62_THC_4_50'] = [' 62','EURO1','RT >26-28t Euro-1','4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.94458296966944','-0.00636014570361','-0.66631942884062','0.11532259816057','0','6','70']
  H_HDV['62_NOx_4_50'] = [' 62','EURO1','RT >26-28t Euro-1','4','50','NOx','y=((a*(b**x))*(x**c))','58.4347509393252','1.00485770491934','-0.315237054317435','0','0','6','70']
  H_HDV['62_PM_4_50'] = [' 62','EURO1','RT >26-28t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000016109103397','0.00236798768756','-0.112927369482198','2.4442946895342','0','6','70']
  H_HDV['62_FC_4_50'] = [' 62','EURO1','RT >26-28t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','636.095949614927','0.084625353176972','5495.59821400898','0.648809355967554','614.720610915863','6','70']
  H_HDV['62_CO_4_100'] = [' 62','EURO1','RT >26-28t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.6144079167491','-0.055902036812954','-2.06779915852027','0.154415444174779','0','6','62']
  H_HDV['62_THC_4_100'] = [' 62','EURO1','RT >26-28t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.913612029488121','3.49157212130433','7.10176658754365','2.92850620138038','-0.028843912312016','6','62']
  H_HDV['62_NOx_4_100'] = [' 62','EURO1','RT >26-28t Euro-1','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000134630257034','0.019159937318515','-0.95851053676221','44.5691294507271','0','6','62']
  H_HDV['62_PM_4_100'] = [' 62','EURO1','RT >26-28t Euro-1','4','100','PM','y=((a*(b**x))*(x**c))','5.38287633692115','1.01027901944902','-0.578133654920664','0','0','6','62']
  H_HDV['62_FC_4_100'] = [' 62','EURO1','RT >26-28t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','478.663405393808','0.060450203699488','4848.44352015887','0.565822061609938','786.778184598519','6','62']
  H_HDV['62_CO_6_0'] = [' 62','EURO1','RT >26-28t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.72383089015042','45.3673166748567','-1.86418241901033','-0.607648516272327','0.158642338263755','6','71']
  H_HDV['62_THC_6_0'] = [' 62','EURO1','RT >26-28t Euro-1','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.658116590560167','3.77708618532207','6.69959936114421','2.6844793940793','-0.019255987741153','6','71']
  H_HDV['62_NOx_6_0'] = [' 62','EURO1','RT >26-28t Euro-1','6','0','NOx','y=((a*(b**x))*(x**c))','61.8267342254312','1.00662543108681','-0.376637190050599','0','0','6','71']
  H_HDV['62_PM_6_0'] = [' 62','EURO1','RT >26-28t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.63103387284173','0.005148495909975','-0.172964188447461','0.075175179644031','0','6','71']
  H_HDV['62_FC_6_0'] = [' 62','EURO1','RT >26-28t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2498.93460914606','1.01068939621949','-0.521460241623324','0','0','6','71']
  H_HDV['62_CO_6_50'] = [' 62','EURO1','RT >26-28t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.1029543999864','-0.041994344436469','-2.05900984547501','0.154235144974465','0','6','59']
  H_HDV['62_THC_6_50'] = [' 62','EURO1','RT >26-28t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.860365561429482','3.7680737663859','6.28620471001132','2.60083372657912','-0.022451147119911','6','59']
  H_HDV['62_NOx_6_50'] = [' 62','EURO1','RT >26-28t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.3520036233218','0.078560541022361','10566.1512083369','1.49970874069666','28.1386875919743','6','59']
  H_HDV['62_PM_6_50'] = [' 62','EURO1','RT >26-28t Euro-1','6','50','PM','y=((a*(b**x))*(x**c))','5.55669691772455','1.01284936129796','-0.620933006053153','0','0','6','59']
  H_HDV['62_FC_6_50'] = [' 62','EURO1','RT >26-28t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','515.657724629579','0.075255212643117','3504.56008015891','0.522314376333925','813.582476993689','6','59']
  H_HDV['62_CO_6_100'] = [' 62','EURO1','RT >26-28t Euro-1','6','100','CO','y=(c+(a*exp(b*x)))','12.3964770842166','-0.049676924704795','3.00052482818014','0','0','6','46']
  H_HDV['62_THC_6_100'] = [' 62','EURO1','RT >26-28t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.2628911350893','3.06412237756201','8.37056980810702','3.43984832269643','-0.038745593265272','6','46']
  H_HDV['62_NOx_6_100'] = [' 62','EURO1','RT >26-28t Euro-1','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4146678530393','0.028006312194412','39.7933237390136','0.313360656638071','33.206116664574','6','46']
  H_HDV['62_PM_6_100'] = [' 62','EURO1','RT >26-28t Euro-1','6','100','PM','y=exp((a+(b/x))+(c*log(x)))','0.411114849371553','2.95542949573324','-0.065049346667962','0','0','6','46']
  H_HDV['62_FC_6_100'] = [' 62','EURO1','RT >26-28t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','410.892122686378','0.047211199318925','4811.48316380002','0.529555304411079','1042.10561943824','6','46']
  H_HDV['63_CO_0_0'] = [' 63','EURO2','RT >26-28t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.18645935999071','0.058552504956403','5.69613725395282','0.261769072378648','0.966484123662079','6','86']
  H_HDV['63_THC_0_0'] = [' 63','EURO2','RT >26-28t Euro-2','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.959428045470749','0.030194145199053','2.55941630195831','0.135441407752692','0.2012204002202','6','86']
  H_HDV['63_NOx_0_0'] = [' 63','EURO2','RT >26-28t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.1405355042366','0.062110528930277','43.8877234392276','0.390718868235076','6.45552636126985','6','86']
  H_HDV['63_PM_0_0'] = [' 63','EURO2','RT >26-28t Euro-2','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.29095940607336','0.190610563094563','-0.001239768461321','0','0','6','86']
  H_HDV['63_FC_0_0'] = [' 63','EURO2','RT >26-28t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','470.237658473003','0.059771399841983','11125.3991771058','0.765239497533142','168.283948164722','6','86']
  H_HDV['63_CO_0_50'] = [' 63','EURO2','RT >26-28t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.077921721471843','0.017406483953447','-0.000109605851016','0','0','6','86']
  H_HDV['63_THC_0_50'] = [' 63','EURO2','RT >26-28t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.828009789857126','0.024811963749179','2.67001448123625','0.124882855805357','0.162905538155383','6','86']
  H_HDV['63_NOx_0_50'] = [' 63','EURO2','RT >26-28t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.4001356804762','0.047819706078286','55.7002667265637','0.444673457893458','7.20536564798271','6','86']
  H_HDV['63_PM_0_50'] = [' 63','EURO2','RT >26-28t Euro-2','0','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.522473149719108','0.004490650780922','-0.016281406775047','0.024923114880099','0','6','86']
  H_HDV['63_FC_0_50'] = [' 63','EURO2','RT >26-28t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','464.243926657849','0.047173861238314','22777.7239789701','0.88418501143649','195.476155665251','6','86']
  H_HDV['63_CO_0_100'] = [' 63','EURO2','RT >26-28t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.41247444140799','0.025253048359992','-0.310635529703583','0.041817607154642','0','6','86']
  H_HDV['63_THC_0_100'] = [' 63','EURO2','RT >26-28t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.786694382711746','0.021180886190047','2.72687850022071','0.122882022500411','0.130970550847622','6','86']
  H_HDV['63_NOx_0_100'] = [' 63','EURO2','RT >26-28t Euro-2','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','29.9035074421407','-0.05132551307521','-1.27613602790239','0.069083074662743','0','6','86']
  H_HDV['63_PM_0_100'] = [' 63','EURO2','RT >26-28t Euro-2','0','100','PM','y=(((a*(x**2))+(b*x))+c)','0.00010194837066','-0.012542262637476','0.556381708945716','0','0','6','86']
  H_HDV['63_FC_0_100'] = [' 63','EURO2','RT >26-28t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','480.197747726278','0.036815568596207','63618.117116502','1.06291056197294','211.537740040522','6','86']
  H_HDV['63_CO_-2_0'] = [' 63','EURO2','RT >26-28t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.09595204765753','0.008412123929575','-0.360261733852988','0.060107233062134','0','6','86']
  H_HDV['63_THC_-2_0'] = [' 63','EURO2','RT >26-28t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.188479374546648','6.06523269282825','0.510437359967714','0.674273993951878','0.034529279630154','6','86']
  H_HDV['63_NOx_-2_0'] = [' 63','EURO2','RT >26-28t Euro-2','-2','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.01993386008108','0.005792798279353','-0.000008579846033','0','0','6','86']
  H_HDV['63_PM_-2_0'] = [' 63','EURO2','RT >26-28t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.50879427342166','0.000510094942037','-0.027233924274003','0.05638105729584','0','6','86']
  H_HDV['63_FC_-2_0'] = [' 63','EURO2','RT >26-28t Euro-2','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','54.9593365735976','5994.63994038827','-1.38402403308356','0.564527802244178','0.03026010521186','6','86']
  H_HDV['63_CO_-2_50'] = [' 63','EURO2','RT >26-28t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.31339211459938','0.008509561824965','-0.358530954547358','0.057282091503973','0','6','86']
  H_HDV['63_THC_-2_50'] = [' 63','EURO2','RT >26-28t Euro-2','-2','50','THC','y=((a+(b*x))**((-1)/c))','0.369019988212763','0.043304236224225','0.741639971896325','0','0','6','86']
  H_HDV['63_NOx_-2_50'] = [' 63','EURO2','RT >26-28t Euro-2','-2','50','NOx','y=((a*(b**x))*(x**c))','44.8630810189635','0.985928242969719','-0.480155170508708','0','0','6','86']
  H_HDV['63_PM_-2_50'] = [' 63','EURO2','RT >26-28t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.534678587550682','0.000798275723728','-0.02606977905157','0.04993524940648','0','6','86']
  H_HDV['63_FC_-2_50'] = [' 63','EURO2','RT >26-28t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','423.02002676418','0.040813352634667','23264.8266697829','0.850543299936558','37.7374236497869','6','86']
  H_HDV['63_CO_-2_100'] = [' 63','EURO2','RT >26-28t Euro-2','-2','100','CO','y=(c+(a*exp(((-1)*b)*x)))','6.21728050946139','0.063398436018846','0.552845038545788','0','0','6','86']
  H_HDV['63_THC_-2_100'] = [' 63','EURO2','RT >26-28t Euro-2','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.058158119938235','19.198798531569','-1.06185662132209','0.609013016641697','0.02171585434618','6','86']
  H_HDV['63_NOx_-2_100'] = [' 63','EURO2','RT >26-28t Euro-2','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.036012725292115','0.993349290317501','45.8311342983774','-0.544869750485597','0','6','86']
  H_HDV['63_PM_-2_100'] = [' 63','EURO2','RT >26-28t Euro-2','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.456688918542891','0.048920840711715','22.1802229615223','0.943991245624141','0.060313059717126','6','86']
  H_HDV['63_FC_-2_100'] = [' 63','EURO2','RT >26-28t Euro-2','-2','100','FC','y=((a*(b**x))*(x**c))','930.146135060376','0.980957409831592','-0.318729660751246','0','0','6','86']
  H_HDV['63_CO_-4_0'] = [' 63','EURO2','RT >26-28t Euro-2','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.144160312372689','0.016028619394727','0.000631161884025','0','0','6','86']
  H_HDV['63_THC_-4_0'] = [' 63','EURO2','RT >26-28t Euro-2','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.271369825124516','0.046671237654069','0.000821523483075','0','0','6','86']
  H_HDV['63_NOx_-4_0'] = [' 63','EURO2','RT >26-28t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4266068905587','0.052318091698415','95.0147112542368','0.477075810170221','0.39936682304153','6','86']
  H_HDV['63_PM_-4_0'] = [' 63','EURO2','RT >26-28t Euro-2','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.555304053377029','-0.000675442404811','-0.041585454902888','0.087005114532536','0','6','86']
  H_HDV['63_FC_-4_0'] = [' 63','EURO2','RT >26-28t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','352.909331124535','0.050704091816925','9373.89709968516','0.686583502049026','10.2610310937573','6','86']
  H_HDV['63_CO_-4_50'] = [' 63','EURO2','RT >26-28t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.53519837293699','0.051417396837537','29.2816917048181','0.5235246386167','0.119323322832971','6','86']
  H_HDV['63_THC_-4_50'] = [' 63','EURO2','RT >26-28t Euro-2','-4','50','THC','y=((a*(b**x))*(x**c))','5.78516495430945','0.983256059943869','-0.642012344059044','0','0','6','86']
  H_HDV['63_NOx_-4_50'] = [' 63','EURO2','RT >26-28t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6950341759444','0.047235429237666','222.254116506523','0.620697199611467','0.255814327153077','6','86']
  H_HDV['63_PM_-4_50'] = [' 63','EURO2','RT >26-28t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','1.27646823539565','0.989210681516973','-0.628810349736766','0','0','6','86']
  H_HDV['63_FC_-4_50'] = [' 63','EURO2','RT >26-28t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','315.996187032991','0.043938935583587','16207.615340239','0.774745409108055','3.73615146586513','6','86']
  H_HDV['63_CO_-4_100'] = [' 63','EURO2','RT >26-28t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.4457594761704','0.044838249704884','76.3001305611351','0.672727106231696','0.068856726384384','6','86']
  H_HDV['63_THC_-4_100'] = [' 63','EURO2','RT >26-28t Euro-2','-4','100','THC','y=((a*(b**x))*(x**c))','5.69723721218007','0.983766034914093','-0.652869827361671','0','0','6','86']
  H_HDV['63_NOx_-4_100'] = [' 63','EURO2','RT >26-28t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5295458018933','0.041317984354646','351.433941194415','0.694682347922421','0.09842114909479','6','86']
  H_HDV['63_PM_-4_100'] = [' 63','EURO2','RT >26-28t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.3351969394597','0.038626115873563','1.83604663544152','0.460353809156643','0.020497512731937','6','86']
  H_HDV['63_FC_-4_100'] = [' 63','EURO2','RT >26-28t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','301.939796751214','0.039282237217495','26345.6959001648','0.854516341870684','-0.768607437308584','6','86']
  H_HDV['63_CO_-6_0'] = [' 63','EURO2','RT >26-28t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.14416567990966','0.070826936827583','16.3688431236186','0.547356616885102','0.1188371369647','6','72']
  H_HDV['63_THC_-6_0'] = [' 63','EURO2','RT >26-28t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.7036310331323','0.061479335289699','5.54275998598854','0.457312722182962','0.082110187535318','6','72']
  H_HDV['63_NOx_-6_0'] = [' 63','EURO2','RT >26-28t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.3946395306721','0.068669300715753','232.693158481476','0.665990085013339','0.286315599914281','6','72']
  H_HDV['63_PM_-6_0'] = [' 63','EURO2','RT >26-28t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014043609799849','0.419150266863497','4.61123270468242','1.86766924192881','-0.006890723622061','6','72']
  H_HDV['63_FC_-6_0'] = [' 63','EURO2','RT >26-28t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','323.250734743799','0.062126989439978','13900.4248006876','0.772341558911128','4.83459184162329','6','72']
  H_HDV['63_CO_-6_50'] = [' 63','EURO2','RT >26-28t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.23779117108617','0.061026803676788','32.6558177396429','0.571408785030274','0.094986224990915','6','72']
  H_HDV['63_THC_-6_50'] = [' 63','EURO2','RT >26-28t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54071073373441','0.059709641179665','7.02295772186753','0.483459089260691','0.079268435538563','6','72']
  H_HDV['63_NOx_-6_50'] = [' 63','EURO2','RT >26-28t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2046961665514','0.062208761198773','354.281383028966','0.712522361302942','0.242159226815475','6','72']
  H_HDV['63_PM_-6_50'] = [' 63','EURO2','RT >26-28t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.303801496915981','0.051506208006693','0.481599523348039','0.250151446656321','0.029557828348486','6','72']
  H_HDV['63_FC_-6_50'] = [' 63','EURO2','RT >26-28t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','268.073739343091','0.057435037630343','18070.9466221088','0.805999145236507','3.80586882535502','6','72']
  H_HDV['63_CO_-6_100'] = [' 63','EURO2','RT >26-28t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.0846823921237','0.056856407858778','58.6661674414516','0.675537416779155','0.084446381799626','6','72']
  H_HDV['63_THC_-6_100'] = [' 63','EURO2','RT >26-28t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.44920492725538','0.058649795721482','6.8013583944079','0.472116051402031','0.077872936583729','6','72']
  H_HDV['63_NOx_-6_100'] = [' 63','EURO2','RT >26-28t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.8984203402206','0.058336514271383','422.396718671642','0.737618750400203','0.221735468287416','6','72']
  H_HDV['63_PM_-6_100'] = [' 63','EURO2','RT >26-28t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.332745275428063','0.051001079732671','0.497429207249158','0.295966487385382','0.029175912410019','6','72']
  H_HDV['63_FC_-6_100'] = [' 63','EURO2','RT >26-28t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','241.562728145637','0.053980426569212','21395.2376226899','0.832046565267747','3.17789899419568','6','72']
  H_HDV['63_CO_2_0'] = [' 63','EURO2','RT >26-28t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.12264285401494','0.008624499562524','-0.494557265785759','0.069914888350201','0','6','86']
  H_HDV['63_THC_2_0'] = [' 63','EURO2','RT >26-28t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.805662030368209','0.017419285812646','2.8716153721653','0.105199507784763','0.095399331445036','6','86']
  H_HDV['63_NOx_2_0'] = [' 63','EURO2','RT >26-28t Euro-2','2','0','NOx','y=(c+(a*exp(b*x)))','26.6111142461283','-0.09146958750466','11.1301563554373','0','0','6','86']
  H_HDV['63_PM_2_0'] = [' 63','EURO2','RT >26-28t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.39066903399898','0.134073607667553','-0.001089001508395','0','0','6','86']
  H_HDV['63_FC_2_0'] = [' 63','EURO2','RT >26-28t Euro-2','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.11859664626615','1.56145840183623','1788.97984674508','-0.508562089856383','0','6','86']
  H_HDV['63_CO_2_50'] = [' 63','EURO2','RT >26-28t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.075126014063713','0.012897886253947','-0.000093085797649','0','0','6','86']
  H_HDV['63_THC_2_50'] = [' 63','EURO2','RT >26-28t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.219968472174571','2.55621108658354','5.62334895285628','2.2647552447369','-0.019366595357642','6','86']
  H_HDV['63_NOx_2_50'] = [' 63','EURO2','RT >26-28t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','41.0869420834632','-0.04350722504777','-2.49037723988539','0.102433294287402','0','6','86']
  H_HDV['63_PM_2_50'] = [' 63','EURO2','RT >26-28t Euro-2','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.624050177139282','0.014670979233842','-0.015391959353362','0.01223403986955','0','6','86']
  H_HDV['63_FC_2_50'] = [' 63','EURO2','RT >26-28t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1031.3398479839','1.23939935971718','-44.8294239674282','0.064978669665348','0','6','86']
  H_HDV['63_CO_2_100'] = [' 63','EURO2','RT >26-28t Euro-2','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.076210219605026','0.009094982258739','-0.000040457630607','0','0','6','86']
  H_HDV['63_THC_2_100'] = [' 63','EURO2','RT >26-28t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.356558910358367','2.2872609021747','6.77310479295688','2.71097712701583','-0.024081657688597','6','86']
  H_HDV['63_NOx_2_100'] = [' 63','EURO2','RT >26-28t Euro-2','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.0861038678155','-0.060790328589307','-1.94477557071184','0.090319352180105','0','6','86']
  H_HDV['63_PM_2_100'] = [' 63','EURO2','RT >26-28t Euro-2','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.766484144915144','0.00597774408739','-0.020592608971179','0.023606058422046','0','6','86']
  H_HDV['63_FC_2_100'] = [' 63','EURO2','RT >26-28t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1103.15264902808','1.2755111863915','-38.5127248865055','0.059332394477899','0','6','86']
  H_HDV['63_CO_4_0'] = [' 63','EURO2','RT >26-28t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.4040582968202','-0.014433104020864','-0.886591018506375','0.113771497692764','0','6','72']
  H_HDV['63_THC_4_0'] = [' 63','EURO2','RT >26-28t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.261198880971667','2.66627771046642','5.49307248508449','2.20618194371928','-0.013596229512898','6','72']
  H_HDV['63_NOx_4_0'] = [' 63','EURO2','RT >26-28t Euro-2','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','44.8827542911681','-0.022922227769156','-3.09404146583063','0.109422078607131','0','6','72']
  H_HDV['63_PM_4_0'] = [' 63','EURO2','RT >26-28t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.38749220360285','0.087872480693056','-0.00066327949872','0','0','6','72']
  H_HDV['63_FC_4_0'] = [' 63','EURO2','RT >26-28t Euro-2','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','687.056479378108','0.098333992684375','10247.5067566187','0.860076413692282','417.99618445068','6','72']
  H_HDV['63_CO_4_50'] = [' 63','EURO2','RT >26-28t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.36794465919234','0.015677014689612','8.27622507925871','0.171504765049371','-0.025511793878988','6','71']
  H_HDV['63_THC_4_50'] = [' 63','EURO2','RT >26-28t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.416559413760596','2.44212622969773','6.42818105238749','2.60023424210613','-0.01772703751955','6','71']
  H_HDV['63_NOx_4_50'] = [' 63','EURO2','RT >26-28t Euro-2','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.9115568396635','0.065673448275015','20.9713488697326','0.352790881923991','20.5700462048278','6','71']
  H_HDV['63_PM_4_50'] = [' 63','EURO2','RT >26-28t Euro-2','4','50','PM','y=(1/(a+(b*(x**c))))','0.734091946538374','0.274708100380787','0.560129461919766','0','0','6','71']
  H_HDV['63_FC_4_50'] = [' 63','EURO2','RT >26-28t Euro-2','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.004700437134735','0.706987723975806','-34.6694020735507','1177.46832328581','0','6','71']
  H_HDV['63_CO_4_100'] = [' 63','EURO2','RT >26-28t Euro-2','4','100','CO','y=((a*(b**x))*(x**c))','22.5898739817508','0.993716022139626','-0.469053287383871','0','0','6','66']
  H_HDV['63_THC_4_100'] = [' 63','EURO2','RT >26-28t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.645993608111074','2.0290840800561','8.3869331870313','3.31025008908717','-0.015990747131145','6','66']
  H_HDV['63_NOx_4_100'] = [' 63','EURO2','RT >26-28t Euro-2','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','62.7197055605751','-0.176696004305645','-5.1907412759031','0.178612567775235','0','6','66']
  H_HDV['63_PM_4_100'] = [' 63','EURO2','RT >26-28t Euro-2','4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.862315423067013','0.058970077344121','-0.000535578089487','0','0','6','66']
  H_HDV['63_FC_4_100'] = [' 63','EURO2','RT >26-28t Euro-2','4','100','FC','y=((a*(b**x))*(x**c))','1915.00507177942','1.00588066843602','-0.289426972939354','0','0','6','66']
  H_HDV['63_CO_6_0'] = [' 63','EURO2','RT >26-28t Euro-2','6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.23848402964146','0.016048433088745','7.15055518365354','0.147837787995505','0.332144842767549','6','71']
  H_HDV['63_THC_6_0'] = [' 63','EURO2','RT >26-28t Euro-2','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.395087556851133','2.39274535642286','7.19894503133859','2.86349717327245','-0.02376944418474','6','71']
  H_HDV['63_NOx_6_0'] = [' 63','EURO2','RT >26-28t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','49.6340341417275','-0.044146847885749','-3.23315424018599','0.115567715818517','0','6','71']
  H_HDV['63_PM_6_0'] = [' 63','EURO2','RT >26-28t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.839017144431718','-0.002099775819633','-0.053241226550246','0.116760429086722','0','6','71']
  H_HDV['63_FC_6_0'] = [' 63','EURO2','RT >26-28t Euro-2','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1186.83012331804','2.99333088954483','-49.7781262319791','0.06372082778481','0','6','71']
  H_HDV['63_CO_6_50'] = [' 63','EURO2','RT >26-28t Euro-2','6','50','CO','y=(1/(a+(b*(x**c))))','0.064956457355257','0.009549098589785','0.901641751276757','0','0','6','62']
  H_HDV['63_THC_6_50'] = [' 63','EURO2','RT >26-28t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.671825463309864','9.0913688153421','-1.04447777057919','-0.32400738731415','0.140538015796103','6','62']
  H_HDV['63_NOx_6_50'] = [' 63','EURO2','RT >26-28t Euro-2','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000261100271305','0.033565953193303','-1.51615222723334','51.9951342970044','0','6','62']
  H_HDV['63_PM_6_50'] = [' 63','EURO2','RT >26-28t Euro-2','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.936540593252321','0.02002918307954','-0.026182999908253','0.01706067829967','0','6','62']
  H_HDV['63_FC_6_50'] = [' 63','EURO2','RT >26-28t Euro-2','6','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000726517143477','0.000021970632729','-0.000000245324743','0','0','6','62']
  H_HDV['63_CO_6_100'] = [' 63','EURO2','RT >26-28t Euro-2','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.54600454431126','0.055132795442617','23.5319414482563','0.404772894171273','2.31346067621193','6','49']
  H_HDV['63_THC_6_100'] = [' 63','EURO2','RT >26-28t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.909648098285841','1.80035957891822','8.49542221463295','3.18289318123666','0.024837029504246','6','49']
  H_HDV['63_NOx_6_100'] = [' 63','EURO2','RT >26-28t Euro-2','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','80.2753904860031','-0.325401226239385','-7.92944865648748','0.229339725695146','0','6','49']
  H_HDV['63_PM_6_100'] = [' 63','EURO2','RT >26-28t Euro-2','6','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.651023005066994','0.058735262059043','-0.000756381702495','0','0','6','49']
  H_HDV['63_FC_6_100'] = [' 63','EURO2','RT >26-28t Euro-2','6','100','FC','y=exp((a+(b/x))+(c*log(x)))','6.805378795158','2.37949964423867','0.048102603752278','0','0','6','49']
  H_HDV['64_CO_0_0'] = [' 64','EURO3','RT >26-28t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.37491717432911','0.04924231395331','9.97344110535071','0.186115103841486','1.01956377991523','6','86']
  H_HDV['64_THC_0_0'] = [' 64','EURO3','RT >26-28t Euro-3','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.818399398353357','0.025438758066608','2.81898736735769','0.129060147590719','0.148165343362157','6','86']
  H_HDV['64_NOx_0_0'] = [' 64','EURO3','RT >26-28t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.438656475408','0.062692257127163','48.6130233830022','0.317905476509848','4.91261089347542','6','86']
  H_HDV['64_PM_0_0'] = [' 64','EURO3','RT >26-28t Euro-3','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.26706653853291','0.024641491228318','1.06112158735198','0.123017241118033','0.068374661743366','6','86']
  H_HDV['64_FC_0_0'] = [' 64','EURO3','RT >26-28t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','504.971965269902','0.0582223920938','1754.76265422619','0.431750342730529','173.077779661656','6','86']
  H_HDV['64_CO_0_50'] = [' 64','EURO3','RT >26-28t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.24588358438859','103.700537481749','-1.3906312471446','0.543451750078654','0.039006642599819','6','86']
  H_HDV['64_THC_0_50'] = [' 64','EURO3','RT >26-28t Euro-3','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.71588074810547','0.023466651359018','2.79878282504916','0.123459782380517','0.135938586321894','6','86']
  H_HDV['64_NOx_0_50'] = [' 64','EURO3','RT >26-28t Euro-3','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.5724996214701','0.051040351505129','45.651882800859','0.309240087785118','5.58300975720938','6','86']
  H_HDV['64_PM_0_50'] = [' 64','EURO3','RT >26-28t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.424449762706025','0.041643678521595','0.864328026775096','0.159945936589218','0.100820480611018','6','86']
  H_HDV['64_FC_0_50'] = [' 64','EURO3','RT >26-28t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','496.037924788222','0.04661832661858','3798.31076366067','0.573715458508514','199.101296810716','6','86']
  H_HDV['64_CO_0_100'] = [' 64','EURO3','RT >26-28t Euro-3','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.03891681564065','0.012595524061722','-0.000063533599478','0','0','6','86']
  H_HDV['64_THC_0_100'] = [' 64','EURO3','RT >26-28t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.666107706484892','0.020476439446406','2.77535949914315','0.121575776137964','0.110633079596146','6','86']
  H_HDV['64_NOx_0_100'] = [' 64','EURO3','RT >26-28t Euro-3','0','100','NOx','y=((a*(b**x))*(x**c))','65.350273438175','1.0018594616947','-0.563967185662642','0','0','6','86']
  H_HDV['64_PM_0_100'] = [' 64','EURO3','RT >26-28t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.479913987859408','0.043612071309007','0.882202038674325','0.195564666437015','0.112983334491709','6','86']
  H_HDV['64_FC_0_100'] = [' 64','EURO3','RT >26-28t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','515.035420793383','0.037333178675055','12509.6654322814','0.78996958540747','214.903786018981','6','86']
  H_HDV['64_CO_-2_0'] = [' 64','EURO3','RT >26-28t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.0272857646451','0.009970553164402','-0.689448309978543','0.06501676309988','0','6','86']
  H_HDV['64_THC_-2_0'] = [' 64','EURO3','RT >26-28t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.109487875456891','2.10971791725802','5.74542780494443','2.25549604701478','-0.015856950983437','6','86']
  H_HDV['64_NOx_-2_0'] = [' 64','EURO3','RT >26-28t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.972258801637','0.063840140121415','87.4976027395203','0.458491785514911','1.97940865511077','6','86']
  H_HDV['64_PM_-2_0'] = [' 64','EURO3','RT >26-28t Euro-3','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.19665199937388','-0.000735804518045','-0.085019502917102','0.080738220325698','0','6','86']
  H_HDV['64_FC_-2_0'] = [' 64','EURO3','RT >26-28t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','543.230239495174','0.058297859792946','4162.76264510177','0.614602237705323','69.2288696657139','6','86']
  H_HDV['64_CO_-2_50'] = [' 64','EURO3','RT >26-28t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.7193130704311','0.01216735429231','-0.644595996872187','0.060921047691746','0','6','86']
  H_HDV['64_THC_-2_50'] = [' 64','EURO3','RT >26-28t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04775465252566','2.12740818836078','4.99344253042514','1.97043481672177','-0.008939146632087','6','86']
  H_HDV['64_NOx_-2_50'] = [' 64','EURO3','RT >26-28t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0187372127805','0.046116751322457','116.52977553952','0.476410850016043','1.16979004189566','6','86']
  H_HDV['64_PM_-2_50'] = [' 64','EURO3','RT >26-28t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053025490668187','0.731060047650581','7.17564316468567','2.68241370062359','-0.019492116395627','6','86']
  H_HDV['64_FC_-2_50'] = [' 64','EURO3','RT >26-28t Euro-3','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','478.343438085324','0.042295156569325','6427.730976414','0.642408264767066','42.8329091866214','6','86']
  H_HDV['64_CO_-2_100'] = [' 64','EURO3','RT >26-28t Euro-3','-2','100','CO','y=(c+(a*exp(b*x)))','10.1745633345441','-0.067943413855281','0.768397224864552','0','0','6','86']
  H_HDV['64_THC_-2_100'] = [' 64','EURO3','RT >26-28t Euro-3','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','4.20804827821948','-6.79857423599872','-1.42321079558161','0','0','6','86']
  H_HDV['64_NOx_-2_100'] = [' 64','EURO3','RT >26-28t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.9238806940301','0.035780495967224','183.938309876049','0.540336231251364','0.626988652720612','6','86']
  H_HDV['64_PM_-2_100'] = [' 64','EURO3','RT >26-28t Euro-3','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037198710848401','0.751043006808751','6.13384033958977','2.29698256949561','-0.013718152957678','6','86']
  H_HDV['64_FC_-2_100'] = [' 64','EURO3','RT >26-28t Euro-3','-2','100','FC','y=((a*(b**x))*(x**c))','1060.7006492834','0.98140992931479','-0.339429354486503','0','0','6','86']
  H_HDV['64_CO_-4_0'] = [' 64','EURO3','RT >26-28t Euro-3','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.03729075711667','-11.2828166225096','-1.82941882027449','0','0','6','86']
  H_HDV['64_THC_-4_0'] = [' 64','EURO3','RT >26-28t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.7084086860293','-0.002107443351325','-0.215774718313969','0.087175980963304','0','6','86']
  H_HDV['64_NOx_-4_0'] = [' 64','EURO3','RT >26-28t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2008893063905','0.061607292108679','115.304539302829','0.482804369177327','0.492461760253351','6','86']
  H_HDV['64_PM_-4_0'] = [' 64','EURO3','RT >26-28t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.95522246107149','-8.97991707214817','-1.55301137352197','0','0','6','86']
  H_HDV['64_FC_-4_0'] = [' 64','EURO3','RT >26-28t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','423.560334584328','0.052439043911868','2962.93778361501','0.501735630252956','15.7006618620701','6','86']
  H_HDV['64_CO_-4_50'] = [' 64','EURO3','RT >26-28t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.88824628300298','-0.008061186772799','-0.7525837586357','0.082821045641621','0','6','86']
  H_HDV['64_THC_-4_50'] = [' 64','EURO3','RT >26-28t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.41299654268364','-0.001859084015848','-0.195610102530979','0.088640538714637','0','6','86']
  H_HDV['64_NOx_-4_50'] = [' 64','EURO3','RT >26-28t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.830433719002','0.0558475440616','252.218280154935','0.613647575023148','0.43121652605396','6','86']
  H_HDV['64_PM_-4_50'] = [' 64','EURO3','RT >26-28t Euro-3','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.026385666864305','0.731263883170211','6.03040052342549','2.30103727552225','-0.012994937739533','6','86']
  H_HDV['64_FC_-4_50'] = [' 64','EURO3','RT >26-28t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','372.739549078954','0.046936331045091','7356.35808070658','0.648091147356522','10.6975902325288','6','86']
  H_HDV['64_CO_-4_100'] = [' 64','EURO3','RT >26-28t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.08594096587053','-0.009010314567944','-0.661153963465823','0.080171764583794','0','6','86']
  H_HDV['64_THC_-4_100'] = [' 64','EURO3','RT >26-28t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.47930056801201','0.061425064146994','3.73517608785814','0.403886759202663','0.075234555132989','6','86']
  H_HDV['64_NOx_-4_100'] = [' 64','EURO3','RT >26-28t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.10278616532563','0.048056432519896','365.123630774433','0.670270824264725','0.321338970088965','6','86']
  H_HDV['64_PM_-4_100'] = [' 64','EURO3','RT >26-28t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.97017377146817','-6.42349108998893','-1.32692666750021','0','0','6','86']
  H_HDV['64_FC_-4_100'] = [' 64','EURO3','RT >26-28t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','346.420125304746','0.041732407836554','14525.9166446712','0.759121401609964','6.16421471822077','6','86']
  H_HDV['64_CO_-6_0'] = [' 64','EURO3','RT >26-28t Euro-3','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.452671270564019','0.073309990815043','8.45846448763797','0.076037661368072','0.233406085091203','6','72']
  H_HDV['64_THC_-6_0'] = [' 64','EURO3','RT >26-28t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.81469637012116','0.071879646909103','1.20771705555959','0.274454027909566','0.079782189941753','6','72']
  H_HDV['64_NOx_-6_0'] = [' 64','EURO3','RT >26-28t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7606626227302','0.075973167943845','209.201536852663','0.599766733561792','0.389834003471658','6','72']
  H_HDV['64_PM_-6_0'] = [' 64','EURO3','RT >26-28t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.79607708896922','-8.56311167040008','-1.54954803346571','0','0','6','72']
  H_HDV['64_FC_-6_0'] = [' 64','EURO3','RT >26-28t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','420.528512028924','0.0667111578594','7497.97875084646','0.699019795026078','12.7019191150959','6','72']
  H_HDV['64_CO_-6_50'] = [' 64','EURO3','RT >26-28t Euro-3','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.60486627311245','0.0679554871727','11.5711308553055','0.483371907622482','0.198950546051503','6','72']
  H_HDV['64_THC_-6_50'] = [' 64','EURO3','RT >26-28t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.44204120499561','0.068555352625614','6.0867252549496','0.503749891912347','0.074819983838065','6','72']
  H_HDV['64_NOx_-6_50'] = [' 64','EURO3','RT >26-28t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.9082785396891','0.066627924870804','346.26205294078','0.66616148053643','0.341873400775243','6','72']
  H_HDV['64_PM_-6_50'] = [' 64','EURO3','RT >26-28t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.021707420741029','0.658170753816558','5.79359401676579','2.23887350876218','-0.014406052012179','6','72']
  H_HDV['64_FC_-6_50'] = [' 64','EURO3','RT >26-28t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','328.121557111956','0.060086172199055','11364.0625406173','0.738803818808702','11.0712951311187','6','72']
  H_HDV['64_CO_-6_100'] = [' 64','EURO3','RT >26-28t Euro-3','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.57400384413408','0.061675466774705','41.8032302998375','0.678582031628576','0.171884018628592','6','72']
  H_HDV['64_THC_-6_100'] = [' 64','EURO3','RT >26-28t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.21618837820798','0.065260535996768','9.1027637861521','0.552645862338708','0.071494835377054','6','72']
  H_HDV['64_NOx_-6_100'] = [' 64','EURO3','RT >26-28t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.3289801174455','0.060526755867009','435.272610532179','0.702111751690368','0.318292995637618','6','72']
  H_HDV['64_PM_-6_100'] = [' 64','EURO3','RT >26-28t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024982020053172','0.62354722909216','4.96170090767872','1.90247281835549','-0.004910420542113','6','72']
  H_HDV['64_FC_-6_100'] = [' 64','EURO3','RT >26-28t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','291.458865304461','0.056133956955467','15205.1592696559','0.785561256942386','10.2036541494228','6','72']
  H_HDV['64_CO_2_0'] = [' 64','EURO3','RT >26-28t Euro-3','2','0','CO','y=(c+(a*exp(b*x)))','10.9789325948583','-0.079704526184433','1.44694308428035','0','0','6','86']
  H_HDV['64_THC_2_0'] = [' 64','EURO3','RT >26-28t Euro-3','2','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.60536333314043','-0.003851403925845','-0.290816839953047','0.094385721252782','0','6','86']
  H_HDV['64_NOx_2_0'] = [' 64','EURO3','RT >26-28t Euro-3','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.7178517985995','0.082310430253114','26.0915190646148','0.261672943099933','8.31685740856897','6','86']
  H_HDV['64_PM_2_0'] = [' 64','EURO3','RT >26-28t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.107559510352486','0.723533227588905','6.27722404003672','2.40192714450422','-0.01417047703349','6','86']
  H_HDV['64_FC_2_0'] = [' 64','EURO3','RT >26-28t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1063.54871810411','0.483510607432464','-64.0741285773428','0.081445422466856','0','6','86']
  H_HDV['64_CO_2_50'] = [' 64','EURO3','RT >26-28t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.2074140210931','0.013053474552203','-0.709549821718693','0.063363851672596','0','6','86']
  H_HDV['64_THC_2_50'] = [' 64','EURO3','RT >26-28t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.216428157481429','2.38354293824792','5.69702453640401','2.27648539783873','-0.014090963326879','6','86']
  H_HDV['64_NOx_2_50'] = [' 64','EURO3','RT >26-28t Euro-3','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','9.22286734610541','50.4006295443182','1.92854390248082','1.30962982209833','-0.002605366576239','6','86']
  H_HDV['64_PM_2_50'] = [' 64','EURO3','RT >26-28t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.03306071424925','0.000596116887861','-0.05936071696426','0.065791002075199','0','6','86']
  H_HDV['64_FC_2_50'] = [' 64','EURO3','RT >26-28t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1098.66445540603','0.867632175259512','-50.2747988052802','0.068824327309525','0','6','86']
  H_HDV['64_CO_2_100'] = [' 64','EURO3','RT >26-28t Euro-3','2','100','CO','y=(c+(a*exp(b*x)))','11.3655294746681','-0.068190968576387','1.86173282213736','0','0','6','86']
  H_HDV['64_THC_2_100'] = [' 64','EURO3','RT >26-28t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.294391043898668','2.15481770450556','6.76321261250849','2.69187812835332','-0.020693111668417','6','86']
  H_HDV['64_NOx_2_100'] = [' 64','EURO3','RT >26-28t Euro-3','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','10.5797164823822','74.6104034579107','0.885947632932143','0.994662321861201','0.000339839610075','6','86']
  H_HDV['64_PM_2_100'] = [' 64','EURO3','RT >26-28t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.03573269412246','0.001060253337145','-0.057374130113653','0.06448441559764','0','6','86']
  H_HDV['64_FC_2_100'] = [' 64','EURO3','RT >26-28t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1153.72943918544','0.987756192259843','-41.3694049592028','0.06062498409724','0','6','86']
  H_HDV['64_CO_4_0'] = [' 64','EURO3','RT >26-28t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.9209646171861','-0.018805211086053','-1.02635425745344','0.099224652264851','0','6','72']
  H_HDV['64_THC_4_0'] = [' 64','EURO3','RT >26-28t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.220255256351095','2.44806645704612','5.77728606810224','2.29396145866263','-0.014558594515824','6','72']
  H_HDV['64_NOx_4_0'] = [' 64','EURO3','RT >26-28t Euro-3','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','53.0675481350605','-0.048830935707422','-5.70378204500479','0.145732461288265','0','6','72']
  H_HDV['64_PM_4_0'] = [' 64','EURO3','RT >26-28t Euro-3','4','0','PM','y=(c+(a*exp(b*x)))','0.841166046005247','-0.069277946189997','0.165760883268518','0','0','6','72']
  H_HDV['64_FC_4_0'] = [' 64','EURO3','RT >26-28t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','2202.00686477827','1.00928767796595','-0.533272605087901','0','0','6','72']
  H_HDV['64_CO_4_50'] = [' 64','EURO3','RT >26-28t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.2818847006292','-0.035322853945115','-1.18204282114181','0.111691466110891','0','6','71']
  H_HDV['64_THC_4_50'] = [' 64','EURO3','RT >26-28t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.326437787189623','2.23683832879948','6.6296648195247','2.65314908811942','-0.01951111446064','6','71']
  H_HDV['64_NOx_4_50'] = [' 64','EURO3','RT >26-28t Euro-3','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','54.3749734963338','-0.049269978138244','-4.60511273389718','0.127877887000366','0','6','71']
  H_HDV['64_PM_4_50'] = [' 64','EURO3','RT >26-28t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004681182798','0.000744193471013','-0.03964354119516','0.946796938325776','0','6','71']
  H_HDV['64_FC_4_50'] = [' 64','EURO3','RT >26-28t Euro-3','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','645.89168474314','0.08687057305447','24385.8110110119','1.00790931432312','620.651048477645','6','71']
  H_HDV['64_CO_4_100'] = [' 64','EURO3','RT >26-28t Euro-3','4','100','CO','y=((a*(b**x))*(x**c))','25.2687500527294','0.989989288549856','-0.467666973815695','0','0','6','66']
  H_HDV['64_THC_4_100'] = [' 64','EURO3','RT >26-28t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.468116805875752','1.927597496525','8.57484609933117','3.46611784609822','-0.037443541883527','6','66']
  H_HDV['64_NOx_4_100'] = [' 64','EURO3','RT >26-28t Euro-3','4','100','NOx','y=((a*(b**x))*(x**c))','89.3288418839837','1.00758064397435','-0.47062943258477','0','0','6','66']
  H_HDV['64_PM_4_100'] = [' 64','EURO3','RT >26-28t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000005303168402','0.000830118843196','-0.041779942868647','1.00396638066723','0','6','66']
  H_HDV['64_FC_4_100'] = [' 64','EURO3','RT >26-28t Euro-3','4','100','FC','y=((a*(b**x))*(x**c))','1954.9421772211','1.00472629985408','-0.280842279677449','0','0','6','66']
  H_HDV['64_CO_6_0'] = [' 64','EURO3','RT >26-28t Euro-3','6','0','CO','y=(c+(a*exp(((-1)*b)*x)))','9.91659226822227','0.064543149967677','1.52028096191962','0','0','6','71']
  H_HDV['64_THC_6_0'] = [' 64','EURO3','RT >26-28t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.316610091936582','2.26831856337076','6.72938632102244','2.67072424664564','-0.01959463837265','6','71']
  H_HDV['64_NOx_6_0'] = [' 64','EURO3','RT >26-28t Euro-3','6','0','NOx','y=((a*(b**x))*(x**c))','92.6662677111074','1.00896493330813','-0.589658413364431','0','0','6','71']
  H_HDV['64_PM_6_0'] = [' 64','EURO3','RT >26-28t Euro-3','6','0','PM','y=(c+(a*exp(b*x)))','0.810433482747854','-0.075005311901197','0.207425298559591','0','0','6','71']
  H_HDV['64_FC_6_0'] = [' 64','EURO3','RT >26-28t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','2162.78587707462','1.00912517457436','-0.459607210813462','0','0','6','71']
  H_HDV['64_CO_6_50'] = [' 64','EURO3','RT >26-28t Euro-3','6','50','CO','y=(1/(a+(b*x)))','0.058886088419601','0.007468647672306','0','0','0','6','64']
  H_HDV['64_THC_6_50'] = [' 64','EURO3','RT >26-28t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.402633644653534','2.03961909019793','8.55570485821982','3.52881247908991','-0.054795696387922','6','64']
  H_HDV['64_NOx_6_50'] = [' 64','EURO3','RT >26-28t Euro-3','6','50','NOx','y=((a*(b**x))*(x**c))','94.3051992031788','1.00842024473625','-0.491645576359742','0','0','6','64']
  H_HDV['64_PM_6_50'] = [' 64','EURO3','RT >26-28t Euro-3','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.03041527610944','0.009075515456288','-0.047690706222021','0.042933114370475','0','6','64']
  H_HDV['64_FC_6_50'] = [' 64','EURO3','RT >26-28t Euro-3','6','50','FC','y=((a*(b**x))*(x**c))','2136.26292919977','1.00693843239253','-0.327049484668621','0','0','6','64']
  H_HDV['64_CO_6_100'] = [' 64','EURO3','RT >26-28t Euro-3','6','100','CO','y=((a*(b**x))*(x**c))','26.9283574614153','0.986225844801114','-0.436389099074002','0','0','6','50']
  H_HDV['64_THC_6_100'] = [' 64','EURO3','RT >26-28t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.672800142013468','1.70701116934002','9.6710657887079','3.99178819038214','-0.048129998536033','6','50']
  H_HDV['64_NOx_6_100'] = [' 64','EURO3','RT >26-28t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','67.3505172297785','0.032169906600598','-4.36135005642696','0.107562527537202','0','6','50']
  H_HDV['64_PM_6_100'] = [' 64','EURO3','RT >26-28t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000011801556716','0.001426048854231','-0.054971481619382','1.14296593283981','0','6','50']
  H_HDV['64_FC_6_100'] = [' 64','EURO3','RT >26-28t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','404.060403339563','0.076060570309819','6227.48317224649','0.637445624351035','1116.08348425207','6','50']
  H_HDV['65_CO_0_0'] = [' 65','EURO4','RT >26-28t Euro-4','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.263817220981472','0.03378236980833','0.776721761251604','0.140149441326421','0.064971432208028','6','86']
  H_HDV['65_THC_0_0'] = [' 65','EURO4','RT >26-28t Euro-4','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048349092395524','0.02915192280347','0.161465575671587','0.137635728032112','0.008056643295259','6','86']
  H_HDV['65_NOx_0_0'] = [' 65','EURO4','RT >26-28t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.26754285044383','0.062597540847398','41.5791503458069','0.44086741636081','2.9770759808185','6','86']
  H_HDV['65_PM_0_0'] = [' 65','EURO4','RT >26-28t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.078750776305501','0.039981199516559','0.266143658269657','0.165478199135779','0.016173272813313','6','86']
  H_HDV['65_FC_0_0'] = [' 65','EURO4','RT >26-28t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','482.886654315725','0.058932506762419','2406.68918700088','0.483190450983658','162.264856113179','6','86']
  H_HDV['65_CO_0_50'] = [' 65','EURO4','RT >26-28t Euro-4','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.265667111853625','0.03435515035584','0.752705539154099','0.145904389808048','0.074528149504989','6','86']
  H_HDV['65_THC_0_50'] = [' 65','EURO4','RT >26-28t Euro-4','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047332517602252','0.032827172194108','0.158205345469862','0.139438004382483','0.008846073798134','6','86']
  H_HDV['65_NOx_0_50'] = [' 65','EURO4','RT >26-28t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.88033924891821','0.04837251511052','53.3788421289147','0.48545604568473','3.4707217299484','6','86']
  H_HDV['65_PM_0_50'] = [' 65','EURO4','RT >26-28t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08035258985785','0.040388193069','0.256780337192614','0.167029598473817','0.016993491076983','6','86']
  H_HDV['65_FC_0_50'] = [' 65','EURO4','RT >26-28t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','468.859933107248','0.046755682026573','4328.60809105834','0.58700550137036','185.204846602321','6','86']
  H_HDV['65_CO_0_100'] = [' 65','EURO4','RT >26-28t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.34863256702377','0.041584791633982','0.687716897385564','0.173009046235233','0.085011518455498','6','86']
  H_HDV['65_THC_0_100'] = [' 65','EURO4','RT >26-28t Euro-4','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047255531441327','0.034832931809705','0.155994909115279','0.143351229684235','0.009124896174052','6','86']
  H_HDV['65_NOx_0_100'] = [' 65','EURO4','RT >26-28t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.42182776097219','0.038992136876017','94.2446050383818','0.604838954826826','3.74111247687783','6','86']
  H_HDV['65_PM_0_100'] = [' 65','EURO4','RT >26-28t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089166870786164','0.042813112778286','0.248335198323528','0.175820896435634','0.017577826213576','6','86']
  H_HDV['65_FC_0_100'] = [' 65','EURO4','RT >26-28t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','482.462807707839','0.03719280089895','10045.5147857013','0.744779582694078','198.679717908436','6','86']
  H_HDV['65_CO_-2_0'] = [' 65','EURO4','RT >26-28t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.038551501990299','0.549377589781708','7.03212210506735','2.67916934159799','-0.021571616401789','6','86']
  H_HDV['65_THC_-2_0'] = [' 65','EURO4','RT >26-28t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004480469044838','0.118766322555695','5.94754384763334','2.35254718207218','-0.0209397970261','6','86']
  H_HDV['65_NOx_-2_0'] = [' 65','EURO4','RT >26-28t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.39620856949215','0.058636935674529','82.5650394535528','0.562598013716133','1.06128950197265','6','86']
  H_HDV['65_PM_-2_0'] = [' 65','EURO4','RT >26-28t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006372570530465','0.209806713767499','4.6844083552332','2.01899085448181','-0.01144561480141','6','86']
  H_HDV['65_FC_-2_0'] = [' 65','EURO4','RT >26-28t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','512.400026393434','0.058961894399134','5564.44184088106','0.651530686481035','63.4519191843533','6','86']
  H_HDV['65_CO_-2_50'] = [' 65','EURO4','RT >26-28t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018227805828372','0.545097350186905','6.10105332802918','2.30627385364035','-0.013258778708586','6','86']
  H_HDV['65_THC_-2_50'] = [' 65','EURO4','RT >26-28t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002224011511454','0.117170814522479','5.37851931890305','2.12937896967575','-0.013557671829051','6','86']
  H_HDV['65_NOx_-2_50'] = [' 65','EURO4','RT >26-28t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.28913672373622','0.040202325143559','105.991807254497','0.574528585048331','0.579429950523932','6','86']
  H_HDV['65_PM_-2_50'] = [' 65','EURO4','RT >26-28t Euro-4','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.046945665012044','0.028706024220444','0.226506355541444','0.115500189929574','0.005684043218938','6','86']
  H_HDV['65_FC_-2_50'] = [' 65','EURO4','RT >26-28t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','449.478061661031','0.042592226185181','7042.2466268404','0.655598468093292','36.8668159025852','6','86']
  H_HDV['65_CO_-2_100'] = [' 65','EURO4','RT >26-28t Euro-4','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.31520810283824','-8.22923158604738','-1.50364075564314','0','0','6','86']
  H_HDV['65_THC_-2_100'] = [' 65','EURO4','RT >26-28t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000204054010017','0.124831150365931','4.37900355269946','1.79955857965557','-0.007346000168426','6','86']
  H_HDV['65_NOx_-2_100'] = [' 65','EURO4','RT >26-28t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.30911468490339','0.030543860738745','190.596195236955','0.664911153968768','0.131778763030781','6','86']
  H_HDV['65_PM_-2_100'] = [' 65','EURO4','RT >26-28t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000312184343189','0.277468836516959','2.73705634831708','1.39919889669813','0.001180867577316','6','86']
  H_HDV['65_FC_-2_100'] = [' 65','EURO4','RT >26-28t Euro-4','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-3.56241342527965','0.726070254056232','1301.9805700412','-0.532605935086874','0','6','86']
  H_HDV['65_CO_-4_0'] = [' 65','EURO4','RT >26-28t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.002784040535726','0.532393041035382','5.92413068238836','2.25483566568312','-0.010434202816372','6','86']
  H_HDV['65_THC_-4_0'] = [' 65','EURO4','RT >26-28t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.152986477275207','-0.000113370026313','-0.012243793474128','0.086992903676467','0','6','86']
  H_HDV['65_NOx_-4_0'] = [' 65','EURO4','RT >26-28t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.53183054143703','0.053915270269968','88.8328484053609','0.545195348473346','0.15481708104258','6','86']
  H_HDV['65_PM_-4_0'] = [' 65','EURO4','RT >26-28t Euro-4','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.036190245332773','0.02081320660918','0.238406641370741','0.112792415907834','-0.001366480546659','6','86']
  H_HDV['65_FC_-4_0'] = [' 65','EURO4','RT >26-28t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','407.345258508555','0.054495314651711','4421.83352352081','0.57045268688596','10.767101734011','6','86']
  H_HDV['65_CO_-4_50'] = [' 65','EURO4','RT >26-28t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.639216104399686','-0.000453721521305','-0.049209982129526','0.082614725875371','0','6','86']
  H_HDV['65_THC_-4_50'] = [' 65','EURO4','RT >26-28t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.138304136382305','-0.000108211863813','-0.011428754243731','0.089887688636887','0','6','86']
  H_HDV['65_NOx_-4_50'] = [' 65','EURO4','RT >26-28t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.70740313091601','0.04769599569448','155.301201403461','0.636290334722436','0.068816411118808','6','86']
  H_HDV['65_PM_-4_50'] = [' 65','EURO4','RT >26-28t Euro-4','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.5073983932588','0.614358227037813','0.023465907487137','0','0','6','86']
  H_HDV['65_FC_-4_50'] = [' 65','EURO4','RT >26-28t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.874056768734','0.048289727101937','9838.0319648303','0.698474499926742','5.33032561448734','6','86']
  H_HDV['65_CO_-4_100'] = [' 65','EURO4','RT >26-28t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.574042588657998','-0.000451238878831','-0.043941249713792','0.08261778013777','0','6','86']
  H_HDV['65_THC_-4_100'] = [' 65','EURO4','RT >26-28t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083362351793912','0.060988122066763','0.221959549378026','0.400043019372314','0.003505723316615','6','86']
  H_HDV['65_NOx_-4_100'] = [' 65','EURO4','RT >26-28t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.3192210328047','0.0425953330068','241.89718012494','0.709938057643501','0.003651828282758','6','86']
  H_HDV['65_PM_-4_100'] = [' 65','EURO4','RT >26-28t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.1186360770361','0.061071698704103','0.506585934783172','0.380934705773619','0.004305680635937','6','86']
  H_HDV['65_FC_-4_100'] = [' 65','EURO4','RT >26-28t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','316.874695994159','0.042809783780495','18291.352134028','0.797463379499536','1.25301186529249','6','86']
  H_HDV['65_CO_-6_0'] = [' 65','EURO4','RT >26-28t Euro-4','-6','0','CO','y=(c+(a*exp(b*x)))','0.605101810070255','-0.077945474656194','0.01203480104218','0','0','6','72']
  H_HDV['65_THC_-6_0'] = [' 65','EURO4','RT >26-28t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.107581869021076','0.07276043735209','0.085845217215505','0.337389868921181','0.003723542441684','6','72']
  H_HDV['65_NOx_-6_0'] = [' 65','EURO4','RT >26-28t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.30375331795368','0.068073515093474','153.852274311027','0.661075867732332','0.064117792552626','6','72']
  H_HDV['65_PM_-6_0'] = [' 65','EURO4','RT >26-28t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.150836198341318','0.074049042063014','0.292059335070242','0.333559899805704','0.004720374347962','6','72']
  H_HDV['65_FC_-6_0'] = [' 65','EURO4','RT >26-28t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','385.579132019221','0.067589256760512','10351.0868246629','0.740105186303959','5.13697960915919','6','72']
  H_HDV['65_CO_-6_50'] = [' 65','EURO4','RT >26-28t Euro-4','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.41157047868997','0.069270729837066','4.19426290550589','0.720017084727005','0.009344563585884','6','72']
  H_HDV['65_THC_-6_50'] = [' 65','EURO4','RT >26-28t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08408259567152','0.068441946738683','0.436122444165065','0.544855038369079','0.003397810460549','6','72']
  H_HDV['65_NOx_-6_50'] = [' 65','EURO4','RT >26-28t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.02589990470448','0.062800791749222','209.654570130285','0.701005417927337','0.047492781111833','6','72']
  H_HDV['65_PM_-6_50'] = [' 65','EURO4','RT >26-28t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.125347456206575','0.07094781752103','0.695649610143395','0.459326331401054','0.00439628848336','6','72']
  H_HDV['65_FC_-6_50'] = [' 65','EURO4','RT >26-28t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','292.504233452597','0.060996953548484','15289.925093869','0.781741438198336','3.72026170939326','6','72']
  H_HDV['65_CO_-6_100'] = [' 65','EURO4','RT >26-28t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.32649276560592','0.063689447907854','6.69818020695952','0.761285030519984','0.007939775787524','6','72']
  H_HDV['65_THC_-6_100'] = [' 65','EURO4','RT >26-28t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.071403287760216','0.065056686695949','0.644201965986357','0.59251774152177','0.003201495486249','6','72']
  H_HDV['65_NOx_-6_100'] = [' 65','EURO4','RT >26-28t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.43258832622958','0.058924491275989','253.639670276508','0.730376130462275','0.038130574543896','6','72']
  H_HDV['65_PM_-6_100'] = [' 65','EURO4','RT >26-28t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.109879270819476','0.067960575278932','0.944825701511142','0.503447197265729','0.004171977087763','6','72']
  H_HDV['65_FC_-6_100'] = [' 65','EURO4','RT >26-28t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','251.624431168851','0.05664440168773','20329.9318903737','0.825664930168834','2.94139247760743','6','72']
  H_HDV['65_CO_2_0'] = [' 65','EURO4','RT >26-28t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.103777129433042','0.602975400760496','6.50135056091811','2.56626104183956','-0.015373853206002','6','86']
  H_HDV['65_THC_2_0'] = [' 65','EURO4','RT >26-28t Euro-4','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.039777666671075','0.015000262730517','0.167086100339484','0.106266629043026','-0.000189127877988','6','86']
  H_HDV['65_NOx_2_0'] = [' 65','EURO4','RT >26-28t Euro-4','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.0490432546624','0.006808749961163','-1.45504019253237','0.096608235255009','0','6','86']
  H_HDV['65_PM_2_0'] = [' 65','EURO4','RT >26-28t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053298221949417','0.025661952939104','0.257408537391636','0.130342019876039','0.016916211724109','6','86']
  H_HDV['65_FC_2_0'] = [' 65','EURO4','RT >26-28t Euro-4','2','0','FC','y=((a*(b**x))*(x**c))','2220.8003668131','1.00964551731827','-0.641584551343567','0','0','6','86']
  H_HDV['65_CO_2_50'] = [' 65','EURO4','RT >26-28t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.07273949349085','-0.001015274577772','-0.092098528020322','0.106582917969822','0','6','86']
  H_HDV['65_THC_2_50'] = [' 65','EURO4','RT >26-28t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007692124797862','0.137601822122678','5.44831690039764','2.205288615388','-0.013772485691859','6','86']
  H_HDV['65_NOx_2_50'] = [' 65','EURO4','RT >26-28t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.2969666191307','-0.016735146944234','-1.60747239223716','0.11300190036731','0','6','86']
  H_HDV['65_PM_2_50'] = [' 65','EURO4','RT >26-28t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.074047929192841','0.035959589305725','0.246020826298974','0.147745424020388','0.022226572739762','6','86']
  H_HDV['65_FC_2_50'] = [' 65','EURO4','RT >26-28t Euro-4','2','50','FC','y=((a*(b**x))*(x**c))','1950.70113761673','1.00751076411093','-0.500712895489572','0','0','6','86']
  H_HDV['65_CO_2_100'] = [' 65','EURO4','RT >26-28t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.13682266247611','-0.002240557208068','-0.103719731855865','0.120503712911372','0','6','86']
  H_HDV['65_THC_2_100'] = [' 65','EURO4','RT >26-28t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01097670411974','0.124421971975867','6.55729319053234','2.63879946008315','-0.021535230115004','6','86']
  H_HDV['65_NOx_2_100'] = [' 65','EURO4','RT >26-28t Euro-4','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.04479796437484','1.18151618841516','-0.220193855570916','0','0','6','86']
  H_HDV['65_PM_2_100'] = [' 65','EURO4','RT >26-28t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.0897817813296','0.039487808213512','0.242179694996107','0.160488364286506','0.023607532378141','6','86']
  H_HDV['65_FC_2_100'] = [' 65','EURO4','RT >26-28t Euro-4','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1114.10059576205','0.74471834631158','-43.1064546563235','0.065224852275869','0','6','86']
  H_HDV['65_CO_4_0'] = [' 65','EURO4','RT >26-28t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.09597195673253','-0.00116182417237','-0.097520149968815','0.111235344133158','0','6','72']
  H_HDV['65_THC_4_0'] = [' 65','EURO4','RT >26-28t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007096974304874','0.142737057684911','5.30795847855787','2.13465883137962','-0.012295128047479','6','72']
  H_HDV['65_NOx_4_0'] = [' 65','EURO4','RT >26-28t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.6165939910598','-0.016855920285216','-2.68880260461784','0.147305478715804','0','6','72']
  H_HDV['65_PM_4_0'] = [' 65','EURO4','RT >26-28t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.060138959533254','0.019190007892536','0.254080588811836','0.129566257193884','0.012903026654714','6','72']
  H_HDV['65_FC_4_0'] = [' 65','EURO4','RT >26-28t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','712.183532579465','0.095369570772147','7077.84113306693','0.8143465655864','402.116811309936','6','72']
  H_HDV['65_CO_4_50'] = [' 65','EURO4','RT >26-28t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.22694067538866','-0.003221067282092','-0.126303954235095','0.142797421950992','0','6','71']
  H_HDV['65_THC_4_50'] = [' 65','EURO4','RT >26-28t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011473191359738','0.129807152053598','6.1903342347909','2.49775930178909','-0.017091656859398','6','71']
  H_HDV['65_NOx_4_50'] = [' 65','EURO4','RT >26-28t Euro-4','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','2.82712734789924','2.14331859116887','-0.130680460203559','0','0','6','71']
  H_HDV['65_PM_4_50'] = [' 65','EURO4','RT >26-28t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082841732198943','0.020824066474817','0.26046017968424','0.145519574997151','0.009566090556398','6','71']
  H_HDV['65_FC_4_50'] = [' 65','EURO4','RT >26-28t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','639.807379204234','0.091687228162364','21657.5053391111','0.984640099366924','586.027232079875','6','71']
  H_HDV['65_CO_4_100'] = [' 65','EURO4','RT >26-28t Euro-4','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.535088212300936','0.029193275990313','0.710020765837018','0.168253392971995','0.071338359841442','6','67']
  H_HDV['65_THC_4_100'] = [' 65','EURO4','RT >26-28t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016978462944249','0.116303823379776','7.26496283033528','2.95959495590403','-0.02587124836262','6','67']
  H_HDV['65_NOx_4_100'] = [' 65','EURO4','RT >26-28t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.55062999517578','0.025108298531769','24.10341443148','0.289015699730828','10.6108656664322','6','67']
  H_HDV['65_PM_4_100'] = [' 65','EURO4','RT >26-28t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.122488612141776','0.052951311022312','0.208966923904969','0.16243515577709','0.03225822104605','6','67']
  H_HDV['65_FC_4_100'] = [' 65','EURO4','RT >26-28t Euro-4','4','100','FC','y=((a*(b**x))*(x**c))','1914.41153348966','1.00538661267678','-0.299289630308631','0','0','6','67']
  H_HDV['65_CO_6_0'] = [' 65','EURO4','RT >26-28t Euro-4','6','0','CO','y=exp((a+(b/x))+(c*log(x)))','1.12051951905796','-1.36423974194499','-0.704342687776712','0','0','6','71']
  H_HDV['65_THC_6_0'] = [' 65','EURO4','RT >26-28t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011186980510552','0.130304896307439','6.33447769311612','2.52848696199578','-0.017579724167637','6','71']
  H_HDV['65_NOx_6_0'] = [' 65','EURO4','RT >26-28t Euro-4','6','0','NOx','y=((a*(b**x))*(x**c))','36.885164743469','1.00590563662174','-0.422522625973093','0','0','6','71']
  H_HDV['65_PM_6_0'] = [' 65','EURO4','RT >26-28t Euro-4','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094926074094963','0.035996421556191','0.244344432251328','0.166510830984165','0.022935580476045','6','71']
  H_HDV['65_FC_6_0'] = [' 65','EURO4','RT >26-28t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','2156.36581946394','1.00987040148618','-0.485508973279826','0','0','6','71']
  H_HDV['65_CO_6_50'] = [' 65','EURO4','RT >26-28t Euro-4','6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.572944891274405','0.020958699066352','0.777068894076638','0.174764371152736','0.003597251009159','6','65']
  H_HDV['65_THC_6_50'] = [' 65','EURO4','RT >26-28t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012418267007275','0.119586118851446','7.8719574121173','3.26879148282081','-0.05090442196959','6','65']
  H_HDV['65_NOx_6_50'] = [' 65','EURO4','RT >26-28t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.23733557975494','0.022116943112986','21.547634802515','0.251597071442647','10.618311410617','6','65']
  H_HDV['65_PM_6_50'] = [' 65','EURO4','RT >26-28t Euro-4','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.172513848286197','0.065554279177133','0.164061480196463','0.197131765393456','0.035602477239864','6','65']
  H_HDV['65_FC_6_50'] = [' 65','EURO4','RT >26-28t Euro-4','6','50','FC','y=((a*(b**x))*(x**c))','2119.88405206876','1.0080121153886','-0.352870389739813','0','0','6','65']
  H_HDV['65_CO_6_100'] = [' 65','EURO4','RT >26-28t Euro-4','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.13161654572679','0.956004325977653','4.68561528819607','1.83843410564464','-0.002033446696299','6','50']
  H_HDV['65_THC_6_100'] = [' 65','EURO4','RT >26-28t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024200597353712','0.105810196407961','7.9958912981597','3.34184217177963','-0.03688380592065','6','50']
  H_HDV['65_NOx_6_100'] = [' 65','EURO4','RT >26-28t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2091246326383','0.033126342572912','30.1152695095677','0.342324610432643','14.4168117456286','6','50']
  H_HDV['65_PM_6_100'] = [' 65','EURO4','RT >26-28t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045959882804534','0.251929428187509','3.74422788236239','1.55245769090529','0.034881616582836','6','50']
  H_HDV['65_FC_6_100'] = [' 65','EURO4','RT >26-28t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','450.916995270466','0.095688674995939','6255.29838855389','0.671513542335526','1063.83496211633','6','50']
  H_HDV['66_CO_0_0'] = [' 66','EURO5','RT >26-28t Euro-5','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.268994174307424','0.033524961420806','0.793874881002855','0.140356172248695','0.065066622700879','6','86']
  H_HDV['66_THC_0_0'] = [' 66','EURO5','RT >26-28t Euro-5','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.049549159923306','0.029272818613756','0.164460835287461','0.138060460627318','0.008248187297678','6','86']
  H_HDV['66_NOx_0_0'] = [' 66','EURO5','RT >26-28t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.20452243813506','0.063357368387525','27.9062680972666','0.422188551187321','1.74866855221534','6','86']
  H_HDV['66_PM_0_0'] = [' 66','EURO5','RT >26-28t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080424238421328','0.039963051312814','0.271679254191697','0.165882709850747','0.016333378494304','6','86']
  H_HDV['66_FC_0_0'] = [' 66','EURO5','RT >26-28t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','494.901500106571','0.058511211094632','2363.0095460998','0.471142585034918','164.543290416328','6','86']
  H_HDV['66_CO_0_50'] = [' 66','EURO5','RT >26-28t Euro-5','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.270049298840546','0.034188393561406','0.76971869027405','0.145795809587426','0.074687760091489','6','86']
  H_HDV['66_THC_0_50'] = [' 66','EURO5','RT >26-28t Euro-5','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048568394690218','0.032982946846601','0.161261047537074','0.139931711915904','0.009045247883962','6','86']
  H_HDV['66_NOx_0_50'] = [' 66','EURO5','RT >26-28t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.94652715700889','0.049843401699463','32.5112840652473','0.449205934763894','2.04096264054747','6','86']
  H_HDV['66_PM_0_50'] = [' 66','EURO5','RT >26-28t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.081798870372668','0.040293507816861','0.262567389873131','0.167383032311784','0.017149339556518','6','86']
  H_HDV['66_FC_0_50'] = [' 66','EURO5','RT >26-28t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','481.368566352197','0.046562270427411','4106.15939816216','0.569360637686097','187.435893371481','6','86']
  H_HDV['66_CO_0_100'] = [' 66','EURO5','RT >26-28t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.355183427609285','0.041726657720251','0.70421678505097','0.174124635120038','0.086150370430973','6','86']
  H_HDV['66_THC_0_100'] = [' 66','EURO5','RT >26-28t Euro-5','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.047583237131357','0.034351119317508','0.159192418984053','0.142430641700512','0.009191147261466','6','86']
  H_HDV['66_NOx_0_100'] = [' 66','EURO5','RT >26-28t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.38260379587578','0.042126574195637','57.013641452962','0.56394183887861','2.2100092002078','6','86']
  H_HDV['66_PM_0_100'] = [' 66','EURO5','RT >26-28t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089408325988946','0.042284535159382','0.253700955377934','0.174571790998516','0.017701907673129','6','86']
  H_HDV['66_FC_0_100'] = [' 66','EURO5','RT >26-28t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','495.731782796796','0.037263764964161','9982.96710806147','0.732495494873868','201.186586298699','6','86']
  H_HDV['66_CO_-2_0'] = [' 66','EURO5','RT >26-28t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.03899060482148','0.560679113165901','7.02533308116798','2.67526701092729','-0.021498736073989','6','86']
  H_HDV['66_THC_-2_0'] = [' 66','EURO5','RT >26-28t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004654055425725','0.120894085427732','5.95837672065157','2.35363022650217','-0.020697577101484','6','86']
  H_HDV['66_NOx_-2_0'] = [' 66','EURO5','RT >26-28t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.19561581367225','0.058972738018538','47.6206581535122','0.516291903297256','0.620685608846634','6','86']
  H_HDV['66_PM_-2_0'] = [' 66','EURO5','RT >26-28t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006616433981369','0.213323835663738','4.69213280130481','2.01863587306633','-0.011091915988107','6','86']
  H_HDV['66_FC_-2_0'] = [' 66','EURO5','RT >26-28t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','527.533091990452','0.058766015238569','5316.66603384482','0.638507666242622','64.4471940150882','6','86']
  H_HDV['66_CO_-2_50'] = [' 66','EURO5','RT >26-28t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01827140512359','0.558578886558234','6.0841901868287','2.30290545756052','-0.01327778802544','6','86']
  H_HDV['66_THC_-2_50'] = [' 66','EURO5','RT >26-28t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002259715233423','0.120024417682342','5.35428038370398','2.12169699344389','-0.013378394663542','6','86']
  H_HDV['66_NOx_-2_50'] = [' 66','EURO5','RT >26-28t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.51060906858157','0.041363972619522','60.7743012034644','0.532599946337914','0.347957170236722','6','86']
  H_HDV['66_PM_-2_50'] = [' 66','EURO5','RT >26-28t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002430464855292','0.2272615873981','3.97512580429388','1.76619129588458','-0.006159836331344','6','86']
  H_HDV['66_FC_-2_50'] = [' 66','EURO5','RT >26-28t Euro-5','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','462.518436029906','0.042722141425634','6630.68300855974','0.638491172049987','37.5804577947579','6','86']
  H_HDV['66_CO_-2_100'] = [' 66','EURO5','RT >26-28t Euro-5','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.36295398445774','-8.31999585253909','-1.51160612471785','0','0','6','86']
  H_HDV['66_THC_-2_100'] = [' 66','EURO5','RT >26-28t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000154477354685','0.126626948223224','4.42791155568656','1.81364622447508','-0.007681337523903','6','86']
  H_HDV['66_NOx_-2_100'] = [' 66','EURO5','RT >26-28t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.50331458318778','0.032295093405009','100.22000716214','0.611868237650594','0.112082413840195','6','86']
  H_HDV['66_PM_-2_100'] = [' 66','EURO5','RT >26-28t Euro-5','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000302349309757','0.273832661356292','2.8800514539585','1.44023206309945','-0.000585536754146','6','86']
  H_HDV['66_FC_-2_100'] = [' 66','EURO5','RT >26-28t Euro-5','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-3.86811076892121','0.712742536705785','1349.29778993628','-0.535757918670622','0','6','86']
  H_HDV['66_CO_-4_0'] = [' 66','EURO5','RT >26-28t Euro-5','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.003086968414038','0.544563933023525','5.92309363376611','2.254916510162','-0.010539289390254','6','86']
  H_HDV['66_THC_-4_0'] = [' 66','EURO5','RT >26-28t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.156439814971572','-0.000116605912486','-0.012529351302246','0.087070266316496','0','6','86']
  H_HDV['66_NOx_-4_0'] = [' 66','EURO5','RT >26-28t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.12266148454292','0.055170006525887','57.3656134977356','0.526330220350248','0.094279033028198','6','86']
  H_HDV['66_PM_-4_0'] = [' 66','EURO5','RT >26-28t Euro-5','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.03704345057467','0.020790643680459','0.243630529644685','0.112903588669974','-0.001452287640595','6','86']
  H_HDV['66_FC_-4_0'] = [' 66','EURO5','RT >26-28t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','420.902233292377','0.054612966061469','4413.58376105676','0.564517067193873','11.0519127586213','6','86']
  H_HDV['66_CO_-4_50'] = [' 66','EURO5','RT >26-28t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.652385448176748','-0.000459870323601','-0.050181407475447','0.082509723861033','0','6','86']
  H_HDV['66_THC_-4_50'] = [' 66','EURO5','RT >26-28t Euro-5','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.141281274600571','-0.000110217593346','-0.011667872866364','0.089810219222921','0','6','86']
  H_HDV['66_NOx_-4_50'] = [' 66','EURO5','RT >26-28t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.62147326940265','0.049405778282529','93.0350653188554','0.606418589548528','0.047242471769939','6','86']
  H_HDV['66_PM_-4_50'] = [' 66','EURO5','RT >26-28t Euro-5','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.4944539715767','0.595141225105763','0.023217044495682','0','0','6','86']
  H_HDV['66_FC_-4_50'] = [' 66','EURO5','RT >26-28t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','361.401064115431','0.048649893397189','9791.07233628892','0.693178995471248','5.63600949266406','6','86']
  H_HDV['66_CO_-4_100'] = [' 66','EURO5','RT >26-28t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.586303616918641','-0.000457647469761','-0.044906232394805','0.082624538506114','0','6','86']
  H_HDV['66_THC_-4_100'] = [' 66','EURO5','RT >26-28t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.085260879096208','0.061039563431966','0.218268788586937','0.394869391603001','0.003567244850159','6','86']
  H_HDV['66_NOx_-4_100'] = [' 66','EURO5','RT >26-28t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.36966488418985','0.044473564437521','135.448777033685','0.668528864614749','0.01196194895294','6','86']
  H_HDV['66_PM_-4_100'] = [' 66','EURO5','RT >26-28t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.121025393485506','0.061126710465848','0.503520685077283','0.37693434526551','0.004382354566736','6','86']
  H_HDV['66_FC_-4_100'] = [' 66','EURO5','RT >26-28t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','327.254475167405','0.043152415391319','18217.5579875512','0.791776281160582','1.52340416576766','6','86']
  H_HDV['66_CO_-6_0'] = [' 66','EURO5','RT >26-28t Euro-5','-6','0','CO','y=(c+(a*exp(b*x)))','0.618847169911255','-0.07793032909885','0.012246488764604','0','0','6','72']
  H_HDV['66_THC_-6_0'] = [' 66','EURO5','RT >26-28t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.109895453008783','0.072771154870705','0.078976104758924','0.321188248626931','0.00379666416367','6','72']
  H_HDV['66_NOx_-6_0'] = [' 66','EURO5','RT >26-28t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.03242896546593','0.069152992943747','92.7006161056445','0.62952260533115','0.039837076976418','6','72']
  H_HDV['66_PM_-6_0'] = [' 66','EURO5','RT >26-28t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.15376379649611','0.074009118988345','0.284972747776436','0.326273476703775','0.004804511822535','6','72']
  H_HDV['66_FC_-6_0'] = [' 66','EURO5','RT >26-28t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','402.729789124793','0.068094802763626','10898.8249186315','0.746840817793472','5.40811726989167','6','72']
  H_HDV['66_CO_-6_50'] = [' 66','EURO5','RT >26-28t Euro-5','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.422172483533194','0.069469405752781','3.64016475076749','0.697207192135161','0.009551863478065','6','72']
  H_HDV['66_THC_-6_50'] = [' 66','EURO5','RT >26-28t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086131239236881','0.068577375688444','0.424055051766006','0.537842690156275','0.003467884716092','6','72']
  H_HDV['66_NOx_-6_50'] = [' 66','EURO5','RT >26-28t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.23158113773285','0.063924654089524','121.287210385726','0.664217760767918','0.029359045323809','6','72']
  H_HDV['66_PM_-6_50'] = [' 66','EURO5','RT >26-28t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.128215942939198','0.071070484235988','0.688169880366748','0.454732217440495','0.004479566290746','6','72']
  H_HDV['66_FC_-6_50'] = [' 66','EURO5','RT >26-28t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','303.562736914021','0.061295852118375','15434.9295308657','0.779073742363721','3.88915498441656','6','72']
  H_HDV['66_CO_-6_100'] = [' 66','EURO5','RT >26-28t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.335636978543117','0.064037398797712','7.19663963163795','0.770188140622494','0.008157234057272','6','72']
  H_HDV['66_THC_-6_100'] = [' 66','EURO5','RT >26-28t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.073008273612439','0.065153958371008','0.645130316499113','0.589823398099035','0.003265351830535','6','72']
  H_HDV['66_NOx_-6_100'] = [' 66','EURO5','RT >26-28t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.85290252590498','0.06006658529171','143.419402671076','0.689446024821995','0.023468476558677','6','72']
  H_HDV['66_PM_-6_100'] = [' 66','EURO5','RT >26-28t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.112015608681151','0.067964958190576','0.942121875390303','0.499613675945186','0.004240945953508','6','72']
  H_HDV['66_FC_-6_100'] = [' 66','EURO5','RT >26-28t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','260.151834044878','0.056865452687018','20637.107578448','0.822818628742352','3.07570849946428','6','72']
  H_HDV['66_CO_2_0'] = [' 66','EURO5','RT >26-28t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.102163026841203','0.622165394788827','6.33614524692167','2.5044408574621','-0.014633881874335','6','86']
  H_HDV['66_THC_2_0'] = [' 66','EURO5','RT >26-28t Euro-5','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.040793231328649','0.014977588794782','0.169942965991011','0.106394642972659','-0.000287135659945','6','86']
  H_HDV['66_NOx_2_0'] = [' 66','EURO5','RT >26-28t Euro-5','2','0','NOx','y=((a*(x**b))+(c*(x**d)))','0.060689345627606','0.78295832901397','30.8185448178251','-0.715247016006065','0','6','86']
  H_HDV['66_PM_2_0'] = [' 66','EURO5','RT >26-28t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.054232888505978','0.025239997448678','0.262401822074974','0.130206031297893','0.016745189456403','6','86']
  H_HDV['66_FC_2_0'] = [' 66','EURO5','RT >26-28t Euro-5','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.215337056301175','1.42094141326169','2017.35568134803','-0.545664570865507','0','6','86']
  H_HDV['66_CO_2_50'] = [' 66','EURO5','RT >26-28t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.09303633971869','-0.001054793438312','-0.094530660163872','0.107295775549574','0','6','86']
  H_HDV['66_THC_2_50'] = [' 66','EURO5','RT >26-28t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007755871793731','0.140338023302556','5.43497249137722','2.19908253695604','-0.013670523960858','6','86']
  H_HDV['66_NOx_2_50'] = [' 66','EURO5','RT >26-28t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.5097590569482','-0.015135349587761','-1.39227362116685','0.132261021753152','0','6','86']
  H_HDV['66_PM_2_50'] = [' 66','EURO5','RT >26-28t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072161697618857','0.034749445620876','0.251000311909264','0.144605314808774','0.022287208334588','6','86']
  H_HDV['66_FC_2_50'] = [' 66','EURO5','RT >26-28t Euro-5','2','50','FC','y=((a*(b**x))*(x**c))','2021.61055820296','1.0075528363634','-0.506502477770264','0','0','6','86']
  H_HDV['66_CO_2_100'] = [' 66','EURO5','RT >26-28t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.17015258447227','-0.002286544436505','-0.109975136401457','0.123797120410105','0','6','86']
  H_HDV['66_THC_2_100'] = [' 66','EURO5','RT >26-28t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010877878077471','0.127189862749143','6.53472349675803','2.62937799585999','-0.02176709128636','6','86']
  H_HDV['66_NOx_2_100'] = [' 66','EURO5','RT >26-28t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.19665910849517','0.012067734897678','12.6674392069797','0.206894766174029','2.38613486981881','6','86']
  H_HDV['66_PM_2_100'] = [' 66','EURO5','RT >26-28t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089721210574318','0.038847255935864','0.248059485164982','0.159738510389388','0.02376935781359','6','86']
  H_HDV['66_FC_2_100'] = [' 66','EURO5','RT >26-28t Euro-5','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1143.88966559219','0.749751643804381','-44.739577965735','0.065366111340104','0','6','86']
  H_HDV['66_CO_4_0'] = [' 66','EURO5','RT >26-28t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.1132013325697','-0.001178824013834','-0.098820455852443','0.11069842731098','0','6','72']
  H_HDV['66_THC_4_0'] = [' 66','EURO5','RT >26-28t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007028138284153','0.146117054939219','5.25118185902067','2.11297274264715','-0.011981944096706','6','72']
  H_HDV['66_NOx_4_0'] = [' 66','EURO5','RT >26-28t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.3869101727894','-0.013751722865748','-2.1636464792188','0.162764598598536','0','6','72']
  H_HDV['66_PM_4_0'] = [' 66','EURO5','RT >26-28t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.06084938168108','0.018465032745419','0.258876107963919','0.128647852001975','0.012196691883454','6','72']
  H_HDV['66_FC_4_0'] = [' 66','EURO5','RT >26-28t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','744.739449785836','0.095406226410236','13024.048193559','0.923810214102413','407.99539808581','6','72']
  H_HDV['66_CO_4_50'] = [' 66','EURO5','RT >26-28t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.2538636446328','-0.003223257208529','-0.130809498337276','0.14407822808478','0','6','71']
  H_HDV['66_THC_4_50'] = [' 66','EURO5','RT >26-28t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011497553735453','0.132399479653197','6.18794566824677','2.49441598431173','-0.017075525064052','6','71']
  H_HDV['66_NOx_4_50'] = [' 66','EURO5','RT >26-28t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.30309953476078','0.024484587207508','15.4401681716756','0.245541320476958','4.74957490132143','6','71']
  H_HDV['66_PM_4_50'] = [' 66','EURO5','RT >26-28t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083019339064434','0.020785415334839','0.264635312331226','0.14471015894789','0.010039723984984','6','71']
  H_HDV['66_FC_4_50'] = [' 66','EURO5','RT >26-28t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','666.790588464312','0.092102600396876','22352.5789703555','0.987255425527048','594.613298926827','6','71']
  H_HDV['66_CO_4_100'] = [' 66','EURO5','RT >26-28t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.54150309812095','0.026425488122809','0.751085138190868','0.168524999156957','0.052911273236327','6','67']
  H_HDV['66_THC_4_100'] = [' 66','EURO5','RT >26-28t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016006283368165','0.122301225063359','6.82654393596303','2.78522636345058','-0.0228592767662','6','67']
  H_HDV['66_NOx_4_100'] = [' 66','EURO5','RT >26-28t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.33364056216671','0.023199930017081','15.5221991984227','0.261790209523523','5.16988877908613','6','67']
  H_HDV['66_PM_4_100'] = [' 66','EURO5','RT >26-28t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.117606136616768','0.04558523675641','0.231026415810284','0.165640945726267','0.028456798682475','6','67']
  H_HDV['66_FC_4_100'] = [' 66','EURO5','RT >26-28t Euro-5','4','100','FC','y=((a*(b**x))*(x**c))','1952.75757130472','1.00499012771154','-0.296968481226729','0','0','6','67']
  H_HDV['66_CO_6_0'] = [' 66','EURO5','RT >26-28t Euro-5','6','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.761054878534708','0.113994025702432','-0.00049542573965','0','0','6','71']
  H_HDV['66_THC_6_0'] = [' 66','EURO5','RT >26-28t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011004960905846','0.133868976272596','6.22952414660959','2.48757116108099','-0.017013681160518','6','71']
  H_HDV['66_NOx_6_0'] = [' 66','EURO5','RT >26-28t Euro-5','6','0','NOx','y=exp((a+(b/x))+(c*log(x)))','2.30825071706644','2.59317460383309','-0.167238927480764','0','0','6','71']
  H_HDV['66_PM_6_0'] = [' 66','EURO5','RT >26-28t Euro-5','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.092815999981341','0.034606292751138','0.250146772736731','0.162984283395868','0.022824227247239','6','71']
  H_HDV['66_FC_6_0'] = [' 66','EURO5','RT >26-28t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','2222.68095668466','1.00982299139485','-0.488903466031654','0','0','6','71']
  H_HDV['66_CO_6_50'] = [' 66','EURO5','RT >26-28t Euro-5','6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.576437791344124','0.021162525144448','0.777656139044918','0.173581021977845','0.006445274376168','6','65']
  H_HDV['66_THC_6_50'] = [' 66','EURO5','RT >26-28t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014842851891014','0.119543969906116','7.82241455738524','3.22132200222975','-0.042935598206528','6','65']
  H_HDV['66_NOx_6_50'] = [' 66','EURO5','RT >26-28t Euro-5','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.920223470301','-0.053568323672052','-2.37591885063432','0.180682993258998','0','6','65']
  H_HDV['66_PM_6_50'] = [' 66','EURO5','RT >26-28t Euro-5','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.149097153083985','0.057783928273707','0.194828512320347','0.181206694831012','0.034154787397072','6','65']
  H_HDV['66_FC_6_50'] = [' 66','EURO5','RT >26-28t Euro-5','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1431.70083321666','3.24297940719234','-49.4525071843713','0.063866054007317','0','6','65']
  H_HDV['66_CO_6_100'] = [' 66','EURO5','RT >26-28t Euro-5','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.101923118025384','1.00169219116084','4.56301184772931','1.78692176509879','-0.005834565731943','6','50']
  H_HDV['66_THC_6_100'] = [' 66','EURO5','RT >26-28t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.022125061002606','0.11232633186933','7.51840015321781','3.13738550729659','-0.034593344436871','6','50']
  H_HDV['66_NOx_6_100'] = [' 66','EURO5','RT >26-28t Euro-5','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000148653949084','0.017479012806976','-0.739507195499685','20.6243917635221','0','6','50']
  H_HDV['66_PM_6_100'] = [' 66','EURO5','RT >26-28t Euro-5','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.210828541702452','0.079445984263212','0.129256430081416','0.1700646956202','0.043459513701304','6','50']
  H_HDV['66_FC_6_100'] = [' 66','EURO5','RT >26-28t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','455.480986424823','0.080409324866942','7533.01452806374','0.695412771145131','1060.02206501163','6','50']
  H_HDV['67_CO_0_0'] = [' 67','EURO0','RT >28-32t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.51604925986408','0.054953929879576','8.77158663771456','0.264012222957298','1.57799618272661','6','86']
  H_HDV['67_THC_0_0'] = [' 67','EURO0','RT >28-32t 80ties','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.400584628787666','8.83577671744813','1.22852308172335','0.834801242447951','0.0212315312484','6','86']
  H_HDV['67_NOx_0_0'] = [' 67','EURO0','RT >28-32t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.9008571068662','0.064647365791464','2003.55879938568','1.10463708145346','9.72859368204577','6','86']
  H_HDV['67_PM_0_0'] = [' 67','EURO0','RT >28-32t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.49008762617728','0.052076437421048','1.68268415997685','0.22510766183602','0.345559080090301','6','86']
  H_HDV['67_FC_0_0'] = [' 67','EURO0','RT >28-32t 80ties','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000778634086655','0.000091526663907','-0.000000611466141','0','0','6','86']
  H_HDV['67_CO_0_50'] = [' 67','EURO0','RT >28-32t 80ties','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.75112716389986','72.0217188921708','-0.97728417846191','0.435846329408036','0.040356494967347','6','86']
  H_HDV['67_THC_0_50'] = [' 67','EURO0','RT >28-32t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.20659533727297','0.038675593468313','4.09861820500472','0.137647976111833','0.393893645785533','6','86']
  H_HDV['67_NOx_0_50'] = [' 67','EURO0','RT >28-32t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.3089117312805','0.040436462564287','282.954267054394','0.803869833375712','11.2495032902766','6','86']
  H_HDV['67_PM_0_50'] = [' 67','EURO0','RT >28-32t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5228149612554','0.0477788784549','2.72544585732826','0.295692940187053','0.392383343689173','6','86']
  H_HDV['67_FC_0_50'] = [' 67','EURO0','RT >28-32t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','628.890924792657','0.048133728339588','68219.943151613','1.05229892554779','262.117369030087','6','86']
  H_HDV['67_CO_0_100'] = [' 67','EURO0','RT >28-32t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.97437108779204','0.054035799968024','26.2097965248159','0.405799977856599','2.03638440646213','6','86']
  H_HDV['67_THC_0_100'] = [' 67','EURO0','RT >28-32t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.81772090608779','0.050482749561611','4.27043693891694','0.185483889378444','0.463813990175935','6','86']
  H_HDV['67_NOx_0_100'] = [' 67','EURO0','RT >28-32t 80ties','0','100','NOx','y=(c+(a*exp(b*x)))','20.886305875799','-0.027109923680174','11.4459271334112','0','0','6','86']
  H_HDV['67_PM_0_100'] = [' 67','EURO0','RT >28-32t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.63257600028121','0.045582022994076','4.04845945123574','0.362514929448319','0.447388929626765','6','86']
  H_HDV['67_FC_0_100'] = [' 67','EURO0','RT >28-32t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','634.013917324954','0.038630288817359','351655.880219549','1.33178985621247','290.231535435107','6','86']
  H_HDV['67_CO_-2_0'] = [' 67','EURO0','RT >28-32t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.9563961690692','0.023459930019512','-0.573863335269301','0.051551036559987','0','6','86']
  H_HDV['67_THC_-2_0'] = [' 67','EURO0','RT >28-32t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.0985412285406','0.063569865106943','40.2600044764089','0.804786238836122','0.459374449394802','6','86']
  H_HDV['67_NOx_-2_0'] = [' 67','EURO0','RT >28-32t 80ties','-2','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.02508955468117','0.004254472484918','-0.000013711006081','0','0','6','86']
  H_HDV['67_PM_-2_0'] = [' 67','EURO0','RT >28-32t 80ties','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.40226724669856','0.000572026608048','-0.140578927035498','0.064209459928904','0','6','86']
  H_HDV['67_FC_-2_0'] = [' 67','EURO0','RT >28-32t 80ties','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','93.5168377334446','5519.45355468566','-1.27514618395086','0.437294985635552','0.03903432935474','6','86']
  H_HDV['67_CO_-2_50'] = [' 67','EURO0','RT >28-32t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.5003924110838','0.018319993748409','-0.616664661576676','0.053024528359653','0','6','86']
  H_HDV['67_THC_-2_50'] = [' 67','EURO0','RT >28-32t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.70758159519721','0.056919943161498','9.74364740316381','0.478204550132885','0.308535906493439','6','86']
  H_HDV['67_NOx_-2_50'] = [' 67','EURO0','RT >28-32t 80ties','-2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.0398327760002','-0.072388774361827','-1.40775319189108','0.074926581244455','0','6','86']
  H_HDV['67_PM_-2_50'] = [' 67','EURO0','RT >28-32t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.98104859334373','0.058526068057735','7.22607136263994','0.604151890305285','0.215136708008537','6','86']
  H_HDV['67_FC_-2_50'] = [' 67','EURO0','RT >28-32t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','534.122752896208','0.038104337586585','69008.8614382651','0.993506444774639','46.9539288370754','6','86']
  H_HDV['67_CO_-2_100'] = [' 67','EURO0','RT >28-32t 80ties','-2','100','CO','y=(c+(a*exp(b*x)))','11.3562312739162','-0.062076910386728','0.887991957118012','0','0','6','86']
  H_HDV['67_THC_-2_100'] = [' 67','EURO0','RT >28-32t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.46708755418349','0.054398262213973','8.93338955559276','0.426019741931502','0.241391443444584','6','86']
  H_HDV['67_NOx_-2_100'] = [' 67','EURO0','RT >28-32t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.7298517247315','-0.101635992196512','-1.10519205664759','0.069409412074111','0','6','86']
  H_HDV['67_PM_-2_100'] = [' 67','EURO0','RT >28-32t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.87819490375888','0.051283444247097','11.7363530199617','0.608711007095631','0.197076011445638','6','86']
  H_HDV['67_FC_-2_100'] = [' 67','EURO0','RT >28-32t 80ties','-2','100','FC','y=((a*(b**x))*(x**c))','1057.95769388575','0.979499304457122','-0.262426855572935','0','0','6','86']
  H_HDV['67_CO_-4_0'] = [' 67','EURO0','RT >28-32t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.7458875600926','-0.00758039781065','-0.764189165519783','0.077529662005507','0','6','86']
  H_HDV['67_THC_-4_0'] = [' 67','EURO0','RT >28-32t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.35982082953903','0.056076009426339','10.3664410821534','0.460716245298503','0.202341755162351','6','86']
  H_HDV['67_NOx_-4_0'] = [' 67','EURO0','RT >28-32t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.3273670864453','0.045648592696961','327.231369342911','0.678890316724772','0.405086241225353','6','86']
  H_HDV['67_PM_-4_0'] = [' 67','EURO0','RT >28-32t 80ties','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.115334310974291','3.01805068738407','1.76645890995086','0.881607165272674','0.029214879801482','6','86']
  H_HDV['67_FC_-4_0'] = [' 67','EURO0','RT >28-32t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','438.097389320482','0.047592352767506','23957.9605769483','0.810719821845644','12.030613332703','6','86']
  H_HDV['67_CO_-4_50'] = [' 67','EURO0','RT >28-32t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.41774256736584','0.054713237968478','47.4470904148289','0.562658613520511','0.2688526216693','6','86']
  H_HDV['67_THC_-4_50'] = [' 67','EURO0','RT >28-32t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.07730160032867','0.054967719187542','11.9482558171622','0.458605497556669','0.171560402512457','6','86']
  H_HDV['67_NOx_-4_50'] = [' 67','EURO0','RT >28-32t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.5884744133889','0.038399924216559','597.620228224533','0.769510425419694','0.051630724082483','6','86']
  H_HDV['67_PM_-4_50'] = [' 67','EURO0','RT >28-32t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5603124565696','0.052870123039217','2.84142785205254','0.337594448963807','0.126664117645334','6','86']
  H_HDV['67_FC_-4_50'] = [' 67','EURO0','RT >28-32t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','386.211275370789','0.041255659352282','33576.621525492','0.857817633898591','1.88440917419557','6','86']
  H_HDV['67_CO_-4_100'] = [' 67','EURO0','RT >28-32t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.03567276054592','0.049788249545014','114.418819646846','0.67509953703617','0.203047445407448','6','86']
  H_HDV['67_THC_-4_100'] = [' 67','EURO0','RT >28-32t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.91760882619382','0.05492888885353','13.3292650853726','0.467993039843994','0.165263385201996','6','86']
  H_HDV['67_NOx_-4_100'] = [' 67','EURO0','RT >28-32t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.2545996700371','0.033113846828317','1060.47775849667','0.865964589888426','-0.264729436928186','6','86']
  H_HDV['67_PM_-4_100'] = [' 67','EURO0','RT >28-32t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56045579877412','0.050564500409228','6.58583975470117','0.468893905315798','0.120737115104116','6','86']
  H_HDV['67_FC_-4_100'] = [' 67','EURO0','RT >28-32t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','366.024683641346','0.036672978655046','41243.4930919556','0.892060323432409','-4.8856366998604','6','86']
  H_HDV['67_CO_-6_0'] = [' 67','EURO0','RT >28-32t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.7711855270025','0.067281502886734','15.727771118695','0.44456222942488','0.232979845834283','6','72']
  H_HDV['67_THC_-6_0'] = [' 67','EURO0','RT >28-32t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.32898947252495','0.062400104132507','13.7860803411984','0.524462269699627','0.157830527715698','6','72']
  H_HDV['67_NOx_-6_0'] = [' 67','EURO0','RT >28-32t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.3818072504133','0.062050287646139','488.774529377975','0.771836150960412','0.285588365219091','6','72']
  H_HDV['67_PM_-6_0'] = [' 67','EURO0','RT >28-32t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66638140382292','0.061057370216004','1.50618848866097','0.273770904828304','0.117811677107248','6','72']
  H_HDV['67_FC_-6_0'] = [' 67','EURO0','RT >28-32t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','387.01216151993','0.059662915950831','25913.9451693932','0.83433166923741','4.83071204619937','6','72']
  H_HDV['67_CO_-6_50'] = [' 67','EURO0','RT >28-32t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.46916405632725','0.061800342463937','43.8668421817362','0.545594168282208','0.202043653569675','6','72']
  H_HDV['67_THC_-6_50'] = [' 67','EURO0','RT >28-32t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.00055720073984','0.061156807711572','15.0166705428356','0.513161756078191','0.152397340365982','6','72']
  H_HDV['67_NOx_-6_50'] = [' 67','EURO0','RT >28-32t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1488372872859','0.058244678106088','533.660745263927','0.771509821723754','0.257484809393325','6','72']
  H_HDV['67_PM_-6_50'] = [' 67','EURO0','RT >28-32t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.55446291200284','0.058583997932292','3.48977723257893','0.388028642829172','0.114179120100716','6','72']
  H_HDV['67_FC_-6_50'] = [' 67','EURO0','RT >28-32t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','326.393855955129','0.056552675974055','26815.181796274','0.830665757507428','3.8293078266913','6','72']
  H_HDV['67_CO_-6_100'] = [' 67','EURO0','RT >28-32t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.30198217117643','0.060371000943071','56.9820278148728','0.582384761969707','0.195424962946365','6','72']
  H_HDV['67_THC_-6_100'] = [' 67','EURO0','RT >28-32t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.8250715249216','0.060451419835645','12.4680017498546','0.471704368030071','0.150177816276897','6','72']
  H_HDV['67_NOx_-6_100'] = [' 67','EURO0','RT >28-32t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4251268235967','0.056444604038174','479.615960963003','0.748754293627192','0.264900519498652','6','72']
  H_HDV['67_PM_-6_100'] = [' 67','EURO0','RT >28-32t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56175835750101','0.058353744840688','4.25969459063665','0.422537270608683','0.114451274521842','6','72']
  H_HDV['67_FC_-6_100'] = [' 67','EURO0','RT >28-32t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','301.983018279038','0.054923025710113','25038.8936744323','0.816414688658024','3.82784123843053','6','72']
  H_HDV['67_CO_2_0'] = [' 67','EURO0','RT >28-32t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.8688688135322','10.3853712051401','5.23941535502864','1.95366112957296','0.005355635261365','6','86']
  H_HDV['67_THC_2_0'] = [' 67','EURO0','RT >28-32t 80ties','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.24920481492491','0.040106517276626','3.31523611811305','0.13855428197777','0.325701850848974','6','86']
  H_HDV['67_NOx_2_0'] = [' 67','EURO0','RT >28-32t 80ties','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.4708971575848','0.055787342597499','-2.1076334318359','0.074779653428401','0','6','86']
  H_HDV['67_PM_2_0'] = [' 67','EURO0','RT >28-32t 80ties','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.88243196633125','-0.000950427547683','-0.182514874914037','0.079450151238531','0','6','86']
  H_HDV['67_FC_2_0'] = [' 67','EURO0','RT >28-32t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1264.88400149067','0.892226902485641','-65.2780359787346','0.069315548015445','0','6','86']
  H_HDV['67_CO_2_50'] = [' 67','EURO0','RT >28-32t 80ties','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.3006448606967','0.056134718508013','7.59952777569249','0.169672007219789','2.33213043347774','6','86']
  H_HDV['67_THC_2_50'] = [' 67','EURO0','RT >28-32t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.386433550303534','4.67083675045373','5.50830407233197','2.15184047638961','-0.008551214134312','6','86']
  H_HDV['67_NOx_2_50'] = [' 67','EURO0','RT >28-32t 80ties','2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000084778143018','0.015892182274725','-0.946081071156529','42.9137151308545','0','6','86']
  H_HDV['67_PM_2_50'] = [' 67','EURO0','RT >28-32t 80ties','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.253336477459938','0.032903704191271','-0.000223612442716','0','0','6','86']
  H_HDV['67_FC_2_50'] = [' 67','EURO0','RT >28-32t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1380.86861534733','1.75767793136216','-60.7316411358263','0.064725259338872','0','6','86']
  H_HDV['67_CO_2_100'] = [' 67','EURO0','RT >28-32t 80ties','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.030464376711181','0.006423564978339','-0.000022402568234','0','0','6','80']
  H_HDV['67_THC_2_100'] = [' 67','EURO0','RT >28-32t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.396741275742062','5.03611842362683','4.92685964849781','2.03165485869732','-0.010194088479208','6','80']
  H_HDV['67_NOx_2_100'] = [' 67','EURO0','RT >28-32t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000058319453955','0.010975726789318','-0.701470527376978','47.4341151411174','0','6','80']
  H_HDV['67_PM_2_100'] = [' 67','EURO0','RT >28-32t 80ties','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.734466366008794','9.6506628168652','-0.227557933652797','0.584300579896671','0.029694708001834','6','80']
  H_HDV['67_FC_2_100'] = [' 67','EURO0','RT >28-32t 80ties','2','100','FC','y=((a*(b**x))*(x**c))','2268.48072670221','1.00474500267073','-0.358173224050616','0','0','6','80']
  H_HDV['67_CO_4_0'] = [' 67','EURO0','RT >28-32t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.95890123423792','11.1775020121805','6.65309562901786','2.62517678490591','-0.025686608649716','6','71']
  H_HDV['67_THC_4_0'] = [' 67','EURO0','RT >28-32t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.265901916201877','5.5907826408287','4.18627085687906','1.66255941239857','0.00133013616038','6','71']
  H_HDV['67_NOx_4_0'] = [' 67','EURO0','RT >28-32t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','51.5688166460618','0.051023098687137','-2.71346517509544','0.093342406645173','0','6','71']
  H_HDV['67_PM_4_0'] = [' 67','EURO0','RT >28-32t 80ties','4','0','PM','y=(c+(a*exp(b*x)))','2.42598470952835','-0.080361417260027','0.695554989697995','0','0','6','71']
  H_HDV['67_FC_4_0'] = [' 67','EURO0','RT >28-32t 80ties','4','0','FC','y=((a*(b**x))*(x**c))','2867.44770637099','1.00908278055622','-0.544994282432151','0','0','6','71']
  H_HDV['67_CO_4_50'] = [' 67','EURO0','RT >28-32t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.1611681437568','-0.026363860966782','-1.9036185947176','0.102229883622337','0','6','65']
  H_HDV['67_THC_4_50'] = [' 67','EURO0','RT >28-32t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.432701450163514','5.18481842746667','5.24838729005668','2.16252251775771','-0.012399563980177','6','65']
  H_HDV['67_NOx_4_50'] = [' 67','EURO0','RT >28-32t 80ties','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.65880838111498','1.69108315983583','-0.015828213430539','0','0','6','65']
  H_HDV['67_PM_4_50'] = [' 67','EURO0','RT >28-32t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.81254307583969','-0.003293465082153','-0.269896731848726','0.100441135447625','0','6','65']
  H_HDV['67_FC_4_50'] = [' 67','EURO0','RT >28-32t 80ties','4','50','FC','y=exp((a+(b/x))+(c*log(x)))','6.77901682582831','3.10014569766112','-0.03755991422598','0','0','6','65']
  H_HDV['67_CO_4_100'] = [' 67','EURO0','RT >28-32t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.5354024557089','-0.051298432254099','-2.41897102388299','0.106636937058784','0','6','55']
  H_HDV['67_THC_4_100'] = [' 67','EURO0','RT >28-32t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.59040089734457','4.73032364345273','5.95029053646116','2.4679192687531','-0.019839705604529','6','55']
  H_HDV['67_NOx_4_100'] = [' 67','EURO0','RT >28-32t 80ties','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','41.424932596188','0.00455375682257','468.831134595824','0.75283502083822','16.0297071820372','6','55']
  H_HDV['67_PM_4_100'] = [' 67','EURO0','RT >28-32t 80ties','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68395013820937','0.039444282760215','2.62395085473531','0.198251056003418','0.967782233621991','6','55']
  H_HDV['67_FC_4_100'] = [' 67','EURO0','RT >28-32t 80ties','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','501.549878167057','0.025997870278238','3271.0770299943','0.420862670153685','902.387372027645','6','55']
  H_HDV['67_CO_6_0'] = [' 67','EURO0','RT >28-32t 80ties','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.15731561942628','14.1901663151761','5.4796633791887','2.19290959315326','-0.024871576514216','6','70']
  H_HDV['67_THC_6_0'] = [' 67','EURO0','RT >28-32t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.437336082638633','4.8690293164041','6.00644127693962','2.36940909271568','-0.013744098415784','6','70']
  H_HDV['67_NOx_6_0'] = [' 67','EURO0','RT >28-32t 80ties','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','57.9508541301431','0.056211333464566','-2.71634438123538','0.098340043115683','0','6','70']
  H_HDV['67_PM_6_0'] = [' 67','EURO0','RT >28-32t 80ties','6','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.236204880261791','0.031242232416044','-0.000253739550297','0','0','6','70']
  H_HDV['67_FC_6_0'] = [' 67','EURO0','RT >28-32t 80ties','6','0','FC','y=((a*(b**x))*(x**c))','2980.11841579296','1.01044295601394','-0.505083137281882','0','0','6','70']
  H_HDV['67_CO_6_50'] = [' 67','EURO0','RT >28-32t 80ties','6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000267929184517','0.031713820420685','-1.31886112575464','23.6263239451528','0','6','50']
  H_HDV['67_THC_6_50'] = [' 67','EURO0','RT >28-32t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.512627676609135','5.09154053412451','5.4767864616415','2.28333513185629','-0.017549394937025','6','50']
  H_HDV['67_NOx_6_50'] = [' 67','EURO0','RT >28-32t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.4713817227034','0.03719908383667','120.578453890477','0.504202631151066','48.7070275688732','6','50']
  H_HDV['67_PM_6_50'] = [' 67','EURO0','RT >28-32t 80ties','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.75689922118757','-0.012449221579662','-0.420185910777826','0.137139028010354','0','6','50']
  H_HDV['67_FC_6_50'] = [' 67','EURO0','RT >28-32t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','499.023341376105','0.013539542622177','2061.15115656974','0.294853292947591','793.022681750781','6','50']
  H_HDV['67_CO_6_100'] = [' 67','EURO0','RT >28-32t 80ties','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.90066723879157','23.7754011545649','5.08192120764547','2.12897673995186','-0.02117346806131','6','40']
  H_HDV['67_THC_6_100'] = [' 67','EURO0','RT >28-32t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.15782452425078','13.7080566714114','-1.2600327088396','-0.70487420806738','0.190488054751487','6','40']
  H_HDV['67_NOx_6_100'] = [' 67','EURO0','RT >28-32t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','49.4357062434274','0.004877306708064','2891.02714713381','1.07398909420119','26.5671258381125','6','40']
  H_HDV['67_PM_6_100'] = [' 67','EURO0','RT >28-32t 80ties','6','100','PM','y=(c+(a*exp(b*x)))','3.68360028679788','-0.086654787924585','1.49598233896691','0','0','6','40']
  H_HDV['67_FC_6_100'] = [' 67','EURO0','RT >28-32t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','600.205632597472','0.065353715402084','17492.995083899','0.798622822766579','1379.94736546171','6','40']
  H_HDV['68_CO_0_0'] = [' 68','EURO1','RT >28-32t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.2510396923683','0.053777947042615','6.41147803629658','0.245642484721808','1.20868812837732','6','86']
  H_HDV['68_THC_0_0'] = [' 68','EURO1','RT >28-32t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.390062848830747','6.18444370399808','1.57636249058072','0.913503706801982','0.020658826549772','6','86']
  H_HDV['68_NOx_0_0'] = [' 68','EURO1','RT >28-32t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.4695118832212','0.065775691305118','89.1268139070441','0.54964094350534','7.0473581542013','6','86']
  H_HDV['68_PM_0_0'] = [' 68','EURO1','RT >28-32t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.24869052382164','0.051176319970783','2.00145721714146','0.185036724565389','0.235349319452624','6','86']
  H_HDV['68_FC_0_0'] = [' 68','EURO1','RT >28-32t Euro-1','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000950001346621','0.000108631342842','-0.0000007558758','0','0','6','86']
  H_HDV['68_CO_0_50'] = [' 68','EURO1','RT >28-32t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.13895076002856','0.050844668436597','15.2921904543979','0.336072898410878','1.54699525101186','6','86']
  H_HDV['68_THC_0_50'] = [' 68','EURO1','RT >28-32t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.34242802168005','4.82996690857443','2.55421545750378','1.19506018139637','0.010385148957065','6','86']
  H_HDV['68_NOx_0_50'] = [' 68','EURO1','RT >28-32t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.737725061334','0.043987824004288','219.818937604503','0.693145723459978','8.03342294786038','6','86']
  H_HDV['68_PM_0_50'] = [' 68','EURO1','RT >28-32t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.33066364996436','0.051467811429908','2.56251598320082','0.249545327179196','0.283990756303267','6','86']
  H_HDV['68_FC_0_50'] = [' 68','EURO1','RT >28-32t Euro-1','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001056225316864','0.000070013615045','-0.000000406332512','','','6','86']
  H_HDV['68_CO_0_100'] = [' 68','EURO1','RT >28-32t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.85089491955423','0.047596542669103','20.2180484761207','0.394768928608515','1.84590545246302','6','86']
  H_HDV['68_THC_0_100'] = [' 68','EURO1','RT >28-32t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.22123163386831','0.047428107060355','2.81121610514585','0.17042780467833','0.46940162093755','6','86']
  H_HDV['68_NOx_0_100'] = [' 68','EURO1','RT >28-32t Euro-1','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000046868800627','0.008683372502834','-0.606593251928427','26.0942777029362','0','6','86']
  H_HDV['68_PM_0_100'] = [' 68','EURO1','RT >28-32t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3695030057171','0.049874066112219','2.88000792757926','0.288298634137675','0.349347661716324','6','86']
  H_HDV['68_FC_0_100'] = [' 68','EURO1','RT >28-32t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','538.144172172179','0.03591599556185','1841399.05106985','1.63555582902167','252.85074788111','6','86']
  H_HDV['68_CO_-2_0'] = [' 68','EURO1','RT >28-32t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.53598951782472','0.016443437591464','-0.460702877328552','0.053757772738358','0','6','86']
  H_HDV['68_THC_-2_0'] = [' 68','EURO1','RT >28-32t Euro-1','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.19476652120211','0.062046067266524','2.96358245998269','0.366105079382584','0.366682739255811','6','86']
  H_HDV['68_NOx_-2_0'] = [' 68','EURO1','RT >28-32t Euro-1','-2','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.024066906772227','0.005887898457026','-0.000017648431226','0','0','6','86']
  H_HDV['68_PM_-2_0'] = [' 68','EURO1','RT >28-32t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.09045468389504','1.81677809447367','7.73371641564218','2.98761513204432','-0.035428593594898','6','86']
  H_HDV['68_FC_-2_0'] = [' 68','EURO1','RT >28-32t Euro-1','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','76.235986053264','4931.43221756554','-1.29268561970441','0.451481471650535','0.038366582745186','6','86']
  H_HDV['68_CO_-2_50'] = [' 68','EURO1','RT >28-32t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.98869794140724','0.014704231087233','-0.486943428627804','0.053554849567652','0','6','86']
  H_HDV['68_THC_-2_50'] = [' 68','EURO1','RT >28-32t Euro-1','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.199629794446706','12.0673873791666','-0.128502857159964','0.529973276662551','0.034536478388708','6','86']
  H_HDV['68_NOx_-2_50'] = [' 68','EURO1','RT >28-32t Euro-1','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.025649626161211','0.993486558369414','46.502525510986','-0.573122671617323','0','6','86']
  H_HDV['68_PM_-2_50'] = [' 68','EURO1','RT >28-32t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.04875430827993','2.10165705380376','6.03063239841849','2.38594143047093','-0.029392974262829','6','86']
  H_HDV['68_FC_-2_50'] = [' 68','EURO1','RT >28-32t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','462.441160822346','0.038676274349978','44400.5415216742','0.94842649645869','43.3306138027629','6','86']
  H_HDV['68_CO_-2_100'] = [' 68','EURO1','RT >28-32t Euro-1','-2','100','CO','y=(c+(a*exp(((-1)*b)*x)))','8.78998875962222','0.061193904591846','0.738478449871406','0','0','6','86']
  H_HDV['68_THC_-2_100'] = [' 68','EURO1','RT >28-32t Euro-1','-2','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.193907897228824','0.025704025453564','0.000419143676942','0','0','6','86']
  H_HDV['68_NOx_-2_100'] = [' 68','EURO1','RT >28-32t Euro-1','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.9639940608048','-0.074331637776216','-1.25044218355152','0.08115608079815','0','6','86']
  H_HDV['68_PM_-2_100'] = [' 68','EURO1','RT >28-32t Euro-1','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.052654148502914','2.37099778823035','4.6202581810872','1.88797191783034','-0.016564716869564','6','86']
  H_HDV['68_FC_-2_100'] = [' 68','EURO1','RT >28-32t Euro-1','-2','100','FC','y=((a*(b**x))*(x**c))','931.307587927571','0.979949435137311','-0.263605474096892','0','0','6','86']
  H_HDV['68_CO_-4_0'] = [' 68','EURO1','RT >28-32t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.6397533671181','-0.007092421450733','-0.655138501787213','0.082863109790489','0','6','86']
  H_HDV['68_THC_-4_0'] = [' 68','EURO1','RT >28-32t Euro-1','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.207657712022522','0.024459475281829','0.000738786835067','0','0','6','86']
  H_HDV['68_NOx_-4_0'] = [' 68','EURO1','RT >28-32t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4892105073223','0.050177449770785','98.867865301104','0.501759039868638','0.386665100140268','6','86']
  H_HDV['68_PM_-4_0'] = [' 68','EURO1','RT >28-32t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.29574279675412','-9.40621232037944','-1.76185702521341','0','0','6','86']
  H_HDV['68_FC_-4_0'] = [' 68','EURO1','RT >28-32t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','389.949332061385','0.049437417267106','14635.73554802','0.755266823151282','11.5639131714924','6','86']
  H_HDV['68_CO_-4_50'] = [' 68','EURO1','RT >28-32t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.43484863987521','0.052770437284959','20.0907106588606','0.434669015297437','0.156670690098661','6','86']
  H_HDV['68_THC_-4_50'] = [' 68','EURO1','RT >28-32t Euro-1','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.028497214896879','16.0347007490856','-0.299089070626783','0.696974647203023','0.023055839386732','6','86']
  H_HDV['68_NOx_-4_50'] = [' 68','EURO1','RT >28-32t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7364397170035','0.043644082605464','187.615255457873','0.596733739524288','0.182011266282673','6','86']
  H_HDV['68_PM_-4_50'] = [' 68','EURO1','RT >28-32t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.97738635481552','-8.28767536554095','-1.7112864337037','0','0','6','86']
  H_HDV['68_FC_-4_50'] = [' 68','EURO1','RT >28-32t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','345.657452904134','0.042125694252722','20331.7589284284','0.797894681842208','2.38008467956381','6','86']
  H_HDV['68_CO_-4_100'] = [' 68','EURO1','RT >28-32t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.2821506681828','0.047484770285188','59.1566038551242','0.589263386444054','0.09803847652996','6','86']
  H_HDV['68_THC_-4_100'] = [' 68','EURO1','RT >28-32t Euro-1','-4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009412663613369','41.0237099039191','-1.41227192607707','0.684276074896669','0.020075442382257','6','86']
  H_HDV['68_NOx_-4_100'] = [' 68','EURO1','RT >28-32t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1472865370668','0.038486545374474','298.368379787526','0.672598640966919','0.015132746358643','6','86']
  H_HDV['68_PM_-4_100'] = [' 68','EURO1','RT >28-32t Euro-1','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.87107567975911','-0.002542852934538','-0.264122002053413','0.100463067213306','0','6','86']
  H_HDV['68_FC_-4_100'] = [' 68','EURO1','RT >28-32t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','329.4848480957','0.036917565135959','26300.7577528','0.838877085506265','-4.17087207091126','6','86']
  H_HDV['68_CO_-6_0'] = [' 68','EURO1','RT >28-32t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.89648890072393','0.068836720367744','4.79355540841663','0.26918368299075','0.148294525930944','6','72']
  H_HDV['68_THC_-6_0'] = [' 68','EURO1','RT >28-32t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.69688891722876','0.064547342488406','4.82049427974782','0.365653895398607','0.100507894391411','6','72']
  H_HDV['68_NOx_-6_0'] = [' 68','EURO1','RT >28-32t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3094635492093','0.067589138695374','157.699643990867','0.608943853771211','0.285968862122177','6','72']
  H_HDV['68_PM_-6_0'] = [' 68','EURO1','RT >28-32t Euro-1','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004911839664315','1.91555341980728','6.53206302716974','2.64302677674411','-0.021595001743856','6','72']
  H_HDV['68_FC_-6_0'] = [' 68','EURO1','RT >28-32t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','353.484816592006','0.061819318914662','15979.8682623309','0.780335989465318','5.1892798352353','6','72']
  H_HDV['68_CO_-6_50'] = [' 68','EURO1','RT >28-32t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.94093506530345','0.063346321869351','21.2439585733976','0.458201913204024','0.127158007014278','6','72']
  H_HDV['68_THC_-6_50'] = [' 68','EURO1','RT >28-32t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.34713497248878','0.06282973465218','6.65436619548017','0.391739492314142','0.095575802271389','6','72']
  H_HDV['68_NOx_-6_50'] = [' 68','EURO1','RT >28-32t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.99435722653664','0.063233575207317','203.335135858959','0.630984788975511','0.257643571488917','6','72']
  H_HDV['68_PM_-6_50'] = [' 68','EURO1','RT >28-32t Euro-1','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.76490305649101','-7.38801537473512','-1.72091940532213','0','0','6','72']
  H_HDV['68_FC_-6_50'] = [' 68','EURO1','RT >28-32t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','296.78010331275','0.057725412806603','18035.3814746336','0.788891527353372','4.0477472438302','6','72']
  H_HDV['68_CO_-6_100'] = [' 68','EURO1','RT >28-32t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.68625257792249','0.060441153925641','29.5230951468382','0.498969318343275','0.11908449327942','6','72']
  H_HDV['68_THC_-6_100'] = [' 68','EURO1','RT >28-32t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.14726899406621','0.061367266121044','6.18524948030457','0.364419584353578','0.092715582031101','6','72']
  H_HDV['68_NOx_-6_100'] = [' 68','EURO1','RT >28-32t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.16402493411198','0.060948086521573','181.296127082207','0.602562948555093','0.256138201481117','6','72']
  H_HDV['68_PM_-6_100'] = [' 68','EURO1','RT >28-32t Euro-1','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.5488718681236','-6.72435716749512','-1.66965718620261','0','0','6','72']
  H_HDV['68_FC_-6_100'] = [' 68','EURO1','RT >28-32t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','274.868609688505','0.055536424489755','18826.1313751494','0.792931402283831','3.87467745858295','6','72']
  H_HDV['68_CO_2_0'] = [' 68','EURO1','RT >28-32t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.2139434123487','0.017778663589859','-0.621131984022029','0.059467284294161','0','6','86']
  H_HDV['68_THC_2_0'] = [' 68','EURO1','RT >28-32t Euro-1','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.11617215352445','0.039052164618309','2.78445241687462','0.123093518043636','0.438034082276175','6','86']
  H_HDV['68_NOx_2_0'] = [' 68','EURO1','RT >28-32t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.8825977473758','0.007943768999094','-2.31554479180313','0.096372557410722','0','6','86']
  H_HDV['68_PM_2_0'] = [' 68','EURO1','RT >28-32t Euro-1','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.43973996309803','0.000627809252764','-0.15548657259365','0.073371147863955','0','6','86']
  H_HDV['68_FC_2_0'] = [' 68','EURO1','RT >28-32t Euro-1','2','0','FC','y=((a*(b**x))*(x**c))','2108.21324434888','1.00887923417753','-0.574749368356503','0','0','6','86']
  H_HDV['68_CO_2_50'] = [' 68','EURO1','RT >28-32t Euro-1','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.049823181779155','0.009649892207703','-0.000069024085894','0','0','6','86']
  H_HDV['68_THC_2_50'] = [' 68','EURO1','RT >28-32t Euro-1','2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.08871471431418','0.001258356811204','-0.325291864981276','0.073289922437128','0','6','86']
  H_HDV['68_NOx_2_50'] = [' 68','EURO1','RT >28-32t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','38.091873804368','-0.028894364504448','-1.89413038251985','0.095277219754903','0','6','86']
  H_HDV['68_PM_2_50'] = [' 68','EURO1','RT >28-32t Euro-1','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.25489982861067','0.053469283299132','-0.000448119227954','0','0','6','86']
  H_HDV['68_FC_2_50'] = [' 68','EURO1','RT >28-32t Euro-1','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1123.23435679436','1.35199933174255','-44.1887004127053','0.061680660392685','0','6','86']
  H_HDV['68_CO_2_100'] = [' 68','EURO1','RT >28-32t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.303021025327','-0.031027716661582','-1.45242257067951','0.116974238730934','0','6','81']
  H_HDV['68_THC_2_100'] = [' 68','EURO1','RT >28-32t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.567233282481023','3.70249732190967','5.1116926841115','2.09411258715458','-0.016877108676613','6','81']
  H_HDV['68_NOx_2_100'] = [' 68','EURO1','RT >28-32t Euro-1','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000063265705878','0.011610745924845','-0.732004208343313','37.1291869064528','0','6','81']
  H_HDV['68_PM_2_100'] = [' 68','EURO1','RT >28-32t Euro-1','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.75248779912796','0.078903258739648','10.6863620951473','0.70354628812969','0.701464279308237','6','81']
  H_HDV['68_FC_2_100'] = [' 68','EURO1','RT >28-32t Euro-1','2','100','FC','y=((a*(b**x))*(x**c))','1798.29280114975','1.00354571529075','-0.312120155306475','0','0','6','81']
  H_HDV['68_CO_4_0'] = [' 68','EURO1','RT >28-32t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.56252815615662','33.1596196619383','-1.84752523095314','-0.785779705010772','0.167900195793019','6','71']
  H_HDV['68_THC_4_0'] = [' 68','EURO1','RT >28-32t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.472018572883051','3.80278206169542','5.01768666742261','1.96972138009892','-0.00782893682549','6','71']
  H_HDV['68_NOx_4_0'] = [' 68','EURO1','RT >28-32t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.113016199185','0.103226788127459','22647.5468073684','1.70925600614255','17.2399020702406','6','71']
  H_HDV['68_PM_4_0'] = [' 68','EURO1','RT >28-32t Euro-1','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-2.17298175554156','0.171170120863512','3.6165567171793','0.106286241146397','0.498174120485333','6','71']
  H_HDV['68_FC_4_0'] = [' 68','EURO1','RT >28-32t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','694.393186663467','0.092433624895305','48190.1438056039','1.09385934068935','475.926700665074','6','71']
  H_HDV['68_CO_4_50'] = [' 68','EURO1','RT >28-32t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.6002490795086','0.014676884392774','-1.01137432291772','0.081447479666636','0','6','67']
  H_HDV['68_THC_4_50'] = [' 68','EURO1','RT >28-32t Euro-1','4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.31175828159056','-0.009461040463853','-0.578276818638299','0.117360794336933','0','6','67']
  H_HDV['68_NOx_4_50'] = [' 68','EURO1','RT >28-32t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.8565767342436','0.07447620998223','725.628832232098','0.987146327602962','24.7663635246556','6','67']
  H_HDV['68_PM_4_50'] = [' 68','EURO1','RT >28-32t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000013314930412','0.001950087182584','-0.093468923995454','2.28689118958177','0','6','67']
  H_HDV['68_FC_4_50'] = [' 68','EURO1','RT >28-32t Euro-1','4','50','FC','y=exp((a+(b/x))+(c*log(x)))','6.78997818961973','2.2892958474802','-0.065263996699785','0','0','6','67']
  H_HDV['68_CO_4_100'] = [' 68','EURO1','RT >28-32t Euro-1','4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000165539152027','0.019540654220949','-0.812549152528108','17.2660806546069','0','6','56']
  H_HDV['68_THC_4_100'] = [' 68','EURO1','RT >28-32t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.850747080186465','3.82331489130438','4.80975929112726','2.02595777263617','-0.014422680784146','6','56']
  H_HDV['68_NOx_4_100'] = [' 68','EURO1','RT >28-32t Euro-1','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000223380869842','0.027002037214366','-1.1624236761834','51.2431755184878','0','6','56']
  H_HDV['68_PM_4_100'] = [' 68','EURO1','RT >28-32t Euro-1','4','100','PM','y=((a+(b*x))**((-1)/c))','-0.059698964967865','0.01494249095692','4.73178720809363','0','0','6','56']
  H_HDV['68_FC_4_100'] = [' 68','EURO1','RT >28-32t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','448.625028902102','0.030982939978371','7682.24772507468','0.617027679209358','835.275082066039','6','56']
  H_HDV['68_CO_6_0'] = [' 68','EURO1','RT >28-32t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.1618569482868','75.6148460947486','-2.68886498507937','-0.716947419728669','0.159376055426988','6','71']
  H_HDV['68_THC_6_0'] = [' 68','EURO1','RT >28-32t Euro-1','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.660933147383839','3.70467178095582','5.33474705669052','2.14217093122673','-0.010893194733351','6','71']
  H_HDV['68_NOx_6_0'] = [' 68','EURO1','RT >28-32t Euro-1','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.0508454151855','0.096371065176204','9681.25991904813','1.5283683715053','22.0369457521905','6','71']
  H_HDV['68_PM_6_0'] = [' 68','EURO1','RT >28-32t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.30518760473764','0.005897383027014','-0.128448287782457','0.066605012352258','0','6','71']
  H_HDV['68_FC_6_0'] = [' 68','EURO1','RT >28-32t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2168.6985868449','1.00856428497301','-0.428524553357187','0','0','6','71']
  H_HDV['68_CO_6_50'] = [' 68','EURO1','RT >28-32t Euro-1','6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000165978944219','0.020048580725324','-0.816827441880109','16.5558207941304','0','6','53']
  H_HDV['68_THC_6_50'] = [' 68','EURO1','RT >28-32t Euro-1','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.43881994269218','-0.015324942563695','-0.586549522082086','0.124970948158642','0','6','53']
  H_HDV['68_NOx_6_50'] = [' 68','EURO1','RT >28-32t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4925571673026','0.092113148359529','3575.49948809874','1.32400337685859','33.3502817395511','6','53']
  H_HDV['68_PM_6_50'] = [' 68','EURO1','RT >28-32t Euro-1','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.31007233111676','-0.004709398770999','-0.359906341189146','0.177034304842055','0','6','53']
  H_HDV['68_FC_6_50'] = [' 68','EURO1','RT >28-32t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','373.748400891699','0.028907512434149','3224.71381736115','0.437383289439283','848.86325570637','6','53']
  H_HDV['68_CO_6_100'] = [' 68','EURO1','RT >28-32t Euro-1','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.5975010935089','-0.191037550885978','-5.97498930886155','0.285650964746816','0','6','41']
  H_HDV['68_THC_6_100'] = [' 68','EURO1','RT >28-32t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.58108865067164','3.42913739017673','1.65862544613607','-0.156416657004841','0.194798662831867','6','41']
  H_HDV['68_NOx_6_100'] = [' 68','EURO1','RT >28-32t Euro-1','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000665041721067','0.059199850343808','-1.89361906058473','67.7604425282304','0','6','41']
  H_HDV['68_PM_6_100'] = [' 68','EURO1','RT >28-32t Euro-1','6','100','PM','y=((a+(b*x))**((-1)/c))','-0.010341347769771','0.002297321015629','6.60449703621997','0','0','6','41']
  H_HDV['68_FC_6_100'] = [' 68','EURO1','RT >28-32t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','470.85957078754','0.045935918343069','11681.6155559569','0.710250444177414','1199.73863488602','6','41']
  H_HDV['69_CO_0_0'] = [' 69','EURO2','RT >28-32t Euro-2','0','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.091684258669861','0.018272735850808','-0.000102891610844','0','0','6','86']
  H_HDV['69_THC_0_0'] = [' 69','EURO2','RT >28-32t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.232887039003357','3.65288960677154','1.80792788693577','0.968053967907039','0.017368070765506','6','86']
  H_HDV['69_NOx_0_0'] = [' 69','EURO2','RT >28-32t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.3640562210763','0.062583475066052','66.8659474346006','0.47937658220425','7.41207234500851','6','86']
  H_HDV['69_PM_0_0'] = [' 69','EURO2','RT >28-32t Euro-2','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.50427417160352','0.165094286498302','-0.001139282682339','0','0','6','86']
  H_HDV['69_FC_0_0'] = [' 69','EURO2','RT >28-32t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001125449873254','0.000109160655553','-0.000000769987896','0','0','6','86']
  H_HDV['69_CO_0_50'] = [' 69','EURO2','RT >28-32t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.084180871216148','0.014332701129293','-0.000085857329393','0','0','6','86']
  H_HDV['69_THC_0_50'] = [' 69','EURO2','RT >28-32t Euro-2','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.123256090190894','2.99558336442699','2.92821388770549','1.32786662973591','-0.001512125430723','6','86']
  H_HDV['69_NOx_0_50'] = [' 69','EURO2','RT >28-32t Euro-2','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.6889655483696','-0.038064385732597','-1.4670174971826','0.074700560263314','0','6','86']
  H_HDV['69_PM_0_50'] = [' 69','EURO2','RT >28-32t Euro-2','0','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000090371688344','-0.011089586550664','0.503445637941912','0','0','6','86']
  H_HDV['69_FC_0_50'] = [' 69','EURO2','RT >28-32t Euro-2','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00122255230437','0.00006690143848','-0.0000003794844','','','6','86']
  H_HDV['69_CO_0_100'] = [' 69','EURO2','RT >28-32t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.11007978322279','0.04558820976368','-0.297086984080529','0.032285607902721','0','6','86']
  H_HDV['69_THC_0_100'] = [' 69','EURO2','RT >28-32t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.780022578207298','0.02405718172483','2.23442124579993','0.111487309931438','0.190321368687189','6','86']
  H_HDV['69_NOx_0_100'] = [' 69','EURO2','RT >28-32t Euro-2','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.1452999950518','-0.058890612967388','-1.0508895471273','0.06154035926103','0','6','86']
  H_HDV['69_PM_0_100'] = [' 69','EURO2','RT >28-32t Euro-2','0','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000000539203784','0.000035526017895','-0.010609577348886','0.589404681637163','0','6','86']
  H_HDV['69_FC_0_100'] = [' 69','EURO2','RT >28-32t Euro-2','0','100','FC','y=(c+(a*exp(b*x)))','514.919884116794','-0.035872838429503','250.501755481092','0','0','6','86']
  H_HDV['69_CO_-2_0'] = [' 69','EURO2','RT >28-32t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.63470060698199','0.015134362284766','-0.286550634812347','0.049012975139158','0','6','86']
  H_HDV['69_THC_-2_0'] = [' 69','EURO2','RT >28-32t Euro-2','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92073907333137','0.057890637535285','3.47621260206305','0.435497252325584','0.232956702222805','6','86']
  H_HDV['69_NOx_-2_0'] = [' 69','EURO2','RT >28-32t Euro-2','-2','0','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.023039518261452','0.005285431442444','-0.000014034068732','0','0','6','86']
  H_HDV['69_PM_-2_0'] = [' 69','EURO2','RT >28-32t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.469210625245064','0.001346461531621','-0.020975955875677','0.043118171571228','0','6','86']
  H_HDV['69_FC_-2_0'] = [' 69','EURO2','RT >28-32t Euro-2','-2','0','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','70.6652651398812','3898.13751123602','-1.19413505054687','0.438356755808566','0.03686792647948','6','86']
  H_HDV['69_CO_-2_50'] = [' 69','EURO2','RT >28-32t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.08898761978132','0.018358231953524','-0.292339652063461','0.044578406698042','0','6','86']
  H_HDV['69_THC_-2_50'] = [' 69','EURO2','RT >28-32t Euro-2','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.124636863607071','14.5371482548013','-1.01364043856681','0.474425431629735','0.030573438795025','6','86']
  H_HDV['69_NOx_-2_50'] = [' 69','EURO2','RT >28-32t Euro-2','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.062306510794127','0.829602789749833','49.0358093661587','-0.557321125155633','0','6','86']
  H_HDV['69_PM_-2_50'] = [' 69','EURO2','RT >28-32t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.508734273539573','0.002506237859669','-0.019730919487585','0.033110711091088','0','6','86']
  H_HDV['69_FC_-2_50'] = [' 69','EURO2','RT >28-32t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','418.780670929121','0.036488611304564','50337.1062298381','0.998634921730852','39.2824432678543','6','86']
  H_HDV['69_CO_-2_100'] = [' 69','EURO2','RT >28-32t Euro-2','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.23041729267487','0.05032026338555','424.048160284083','0.99536697056429','0.567883567980928','6','86']
  H_HDV['69_THC_-2_100'] = [' 69','EURO2','RT >28-32t Euro-2','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.064785862710201','16.084053542226','-1.04235641823742','0.531077689426319','0.023774761640432','6','86']
  H_HDV['69_NOx_-2_100'] = [' 69','EURO2','RT >28-32t Euro-2','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.6271122925554','-0.076075757698348','-1.3391988267278','0.079204164136489','0','6','86']
  H_HDV['69_PM_-2_100'] = [' 69','EURO2','RT >28-32t Euro-2','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.566737299849839','0.001649569360804','-0.021081070311905','0.034890260206414','0','6','86']
  H_HDV['69_FC_-2_100'] = [' 69','EURO2','RT >28-32t Euro-2','-2','100','FC','y=((a*(b**x))*(x**c))','756.824104394146','0.979115293868838','-0.205166926914499','0','0','6','86']
  H_HDV['69_CO_-4_0'] = [' 69','EURO2','RT >28-32t Euro-2','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.15748641779733','0.015357518069083','0.00054502057311','0','0','6','86']
  H_HDV['69_THC_-4_0'] = [' 69','EURO2','RT >28-32t Euro-2','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.063846464966321','20.4477963754208','-1.39594143703638','0.499382199220173','0.029133502982268','6','86']
  H_HDV['69_NOx_-4_0'] = [' 69','EURO2','RT >28-32t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2597623277932','0.049086868603233','91.7516813771427','0.471562332063058','0.414660479263131','6','86']
  H_HDV['69_PM_-4_0'] = [' 69','EURO2','RT >28-32t Euro-2','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.07321392509446','-5.54141209364858','-1.21751850868085','0','0','6','86']
  H_HDV['69_FC_-4_0'] = [' 69','EURO2','RT >28-32t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','344.343839517282','0.047754554167438','11982.078345789','0.742909639645516','11.149275942901','6','86']
  H_HDV['69_CO_-4_50'] = [' 69','EURO2','RT >28-32t Euro-2','-4','50','CO','y=((a*(b**x))*(x**c))','10.861559408755','0.977029179992967','-0.515308799654603','0','0','6','86']
  H_HDV['69_THC_-4_50'] = [' 69','EURO2','RT >28-32t Euro-2','-4','50','THC','y=((a*(b**x))*(x**c))','5.25774167985475','0.982816993327468','-0.597255626208432','0','0','6','86']
  H_HDV['69_NOx_-4_50'] = [' 69','EURO2','RT >28-32t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.5508841154011','0.043511577751224','196.616734725107','0.591230542203614','0.207185574011224','6','86']
  H_HDV['69_PM_-4_50'] = [' 69','EURO2','RT >28-32t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','1.05825011250307','0.98730467581887','-0.533548221154243','0','0','6','86']
  H_HDV['69_FC_-4_50'] = [' 69','EURO2','RT >28-32t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','308.832996390497','0.040025710484906','19144.2479997082','0.809188883653431','1.65966821917105','6','86']
  H_HDV['69_CO_-4_100'] = [' 69','EURO2','RT >28-32t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.68230696309377','0.04183062863214','100.557058624946','0.735222163589941','0.047120570824377','6','86']
  H_HDV['69_THC_-4_100'] = [' 69','EURO2','RT >28-32t Euro-2','-4','100','THC','y=((a*(b**x))*(x**c))','5.35200004609258','0.983796389372767','-0.623928934620488','0','0','6','86']
  H_HDV['69_NOx_-4_100'] = [' 69','EURO2','RT >28-32t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8673121803187','0.03870846509589','323.393803855283','0.673225339328822','0.036896456980459','6','86']
  H_HDV['69_PM_-4_100'] = [' 69','EURO2','RT >28-32t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.353895660919499','0.035777294446358','2.48943967955805','0.530511805253955','0.018004673189294','6','86']
  H_HDV['69_FC_-4_100'] = [' 69','EURO2','RT >28-32t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','300.68062217051','0.035211003068223','32277.1597875822','0.896786673091793','-4.74520484047417','6','86']
  H_HDV['69_CO_-6_0'] = [' 69','EURO2','RT >28-32t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.07504381027127','0.068157661028712','5.3069537644596','0.375586636077006','0.124539989614215','6','72']
  H_HDV['69_THC_-6_0'] = [' 69','EURO2','RT >28-32t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68669365067034','0.059730424685391','4.70783878089769','0.445850983488344','0.086292608944419','6','72']
  H_HDV['69_NOx_-6_0'] = [' 69','EURO2','RT >28-32t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2612125417328','0.067180702874363','160.425062863789','0.598554290001706','0.33286178040452','6','72']
  H_HDV['69_PM_-6_0'] = [' 69','EURO2','RT >28-32t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014333722167631','0.441047490442204','4.30174480913517','1.75816110977274','-0.004581152785966','6','72']
  H_HDV['69_FC_-6_0'] = [' 69','EURO2','RT >28-32t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','308.270526229496','0.059725197022154','11411.3527299909','0.746500291265451','5.40808479952741','6','72']
  H_HDV['69_CO_-6_50'] = [' 69','EURO2','RT >28-32t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.20922215437132','0.058399368390322','22.7100264564887','0.507221770575432','0.099233741456532','6','72']
  H_HDV['69_THC_-6_50'] = [' 69','EURO2','RT >28-32t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51716594420089','0.058239221437409','4.73372892158952','0.418165833977162','0.083682529352194','6','72']
  H_HDV['69_NOx_-6_50'] = [' 69','EURO2','RT >28-32t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4449726609968','0.061965409312849','217.874818507086','0.624878707209413','0.293217224982262','6','72']
  H_HDV['69_PM_-6_50'] = [' 69','EURO2','RT >28-32t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.287533984519593','0.047199180040853','0.620824362905892','0.264125565309783','0.028846342268688','6','72']
  H_HDV['69_FC_-6_50'] = [' 69','EURO2','RT >28-32t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','262.629331943163','0.0560529827983','17220.2212521783','0.803868042832387','4.51931519057438','6','72']
  H_HDV['69_CO_-6_100'] = [' 69','EURO2','RT >28-32t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.2347445126774','0.055978630778961','47.6982687805702','0.64519160708834','0.093379174350662','6','72']
  H_HDV['69_THC_-6_100'] = [' 69','EURO2','RT >28-32t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.42788596270126','0.057290456600851','4.52009376184462','0.397422888625387','0.082385381833976','6','72']
  H_HDV['69_NOx_-6_100'] = [' 69','EURO2','RT >28-32t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.61121730635027','0.059968574601761','203.929063252572','0.607422559184579','0.290309427113298','6','72']
  H_HDV['69_PM_-6_100'] = [' 69','EURO2','RT >28-32t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.336259146165741','0.049212170939589','0.741509722175109','0.336734198730193','0.02987286856716','6','72']
  H_HDV['69_FC_-6_100'] = [' 69','EURO2','RT >28-32t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','242.64516015478','0.053390315021134','18429.995883452','0.811862580938949','4.1646704312105','6','72']
  H_HDV['69_CO_2_0'] = [' 69','EURO2','RT >28-32t Euro-2','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.74904545114652','35.745373835596','-2.07709118845897','-0.43090855976798','0.122039483878759','6','86']
  H_HDV['69_THC_2_0'] = [' 69','EURO2','RT >28-32t Euro-2','2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.17052036584501','0.032776960394914','2.01343963342795','0.11379477751606','0.237333684911267','6','86']
  H_HDV['69_NOx_2_0'] = [' 69','EURO2','RT >28-32t Euro-2','2','0','NOx','y=(c+(a*exp(b*x)))','25.306675893703','-0.089850562033771','12.5465249206802','0','0','6','86']
  H_HDV['69_PM_2_0'] = [' 69','EURO2','RT >28-32t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.51437383276761','0.107394196604213','-0.000843992188042','0','0','6','86']
  H_HDV['69_FC_2_0'] = [' 69','EURO2','RT >28-32t Euro-2','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.039703618005032','1.7980978764236','1514.18300626518','-0.421381223457118','0','6','86']
  H_HDV['69_CO_2_50'] = [' 69','EURO2','RT >28-32t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.067862288677817','0.01061397225208','-0.000065725112982','0','0','6','86']
  H_HDV['69_THC_2_50'] = [' 69','EURO2','RT >28-32t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.343952415434807','2.22467392961561','5.58885246425776','2.15575743276766','-0.006084637110516','6','86']
  H_HDV['69_NOx_2_50'] = [' 69','EURO2','RT >28-32t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','41.7883419413006','-0.047245466305625','-2.18555421003474','0.096146560594806','0','6','86']
  H_HDV['69_PM_2_50'] = [' 69','EURO2','RT >28-32t Euro-2','2','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000107623085037','-0.013242067316353','0.661105325753812','0','0','6','86']
  H_HDV['69_FC_2_50'] = [' 69','EURO2','RT >28-32t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1002.53660559337','1.55300173537729','-33.9988017503144','0.055686308063145','0','6','86']
  H_HDV['69_CO_2_100'] = [' 69','EURO2','RT >28-32t Euro-2','2','100','CO','y=(1/(a+(b*(x**c))))','0.05177861156575','0.011282809249441','0.863852611330066','0','0','6','84']
  H_HDV['69_THC_2_100'] = [' 69','EURO2','RT >28-32t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.385020502216424','2.19277881419075','6.11318102948838','2.44264197834641','-0.021060438410198','6','84']
  H_HDV['69_NOx_2_100'] = [' 69','EURO2','RT >28-32t Euro-2','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000076107095531','0.013955579627441','-0.893264256561797','40.4037646931387','0','6','84']
  H_HDV['69_PM_2_100'] = [' 69','EURO2','RT >28-32t Euro-2','2','100','PM','y=(c+(a*exp(b*x)))','0.628057968293982','-0.047088538013014','0.276877585204142','0','0','6','84']
  H_HDV['69_FC_2_100'] = [' 69','EURO2','RT >28-32t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1107.29360759906','1.08233690821151','-28.6588430656378','0.051618571415212','0','6','84']
  H_HDV['69_CO_4_0'] = [' 69','EURO2','RT >28-32t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.7699703758424','-0.014739713065913','-1.04721582212629','0.117867038810928','0','6','71']
  H_HDV['69_THC_4_0'] = [' 69','EURO2','RT >28-32t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.24795488861776','2.51868256899262','5.11095199349963','2.01254397860631','-0.011848937347847','6','71']
  H_HDV['69_NOx_4_0'] = [' 69','EURO2','RT >28-32t Euro-2','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','16.7788191375003','36.829177402579','1.83255525990244','1.08122272613297','0.027664841825738','6','71']
  H_HDV['69_PM_4_0'] = [' 69','EURO2','RT >28-32t Euro-2','4','0','PM','y=(c+(a*exp(b*x)))','0.44442832028799','-0.059328323454242','0.24649370140093','0','0','6','71']
  H_HDV['69_FC_4_0'] = [' 69','EURO2','RT >28-32t Euro-2','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','583.552415390495','0.09347124193691','65784.074577907','1.17116576502868','471.713908748035','6','71']
  H_HDV['69_CO_4_50'] = [' 69','EURO2','RT >28-32t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.69674116055515','0.018272482349222','9.73300816530302','0.139652747983368','0.337289431476026','6','69']
  H_HDV['69_THC_4_50'] = [' 69','EURO2','RT >28-32t Euro-2','4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.96543362026341','-0.004020497285046','-0.344689425293027','0.108060782246292','0','6','69']
  H_HDV['69_NOx_4_50'] = [' 69','EURO2','RT >28-32t Euro-2','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.065861844018','0.065948112490047','296.525392165277','0.851504494667468','23.5775713414621','6','69']
  H_HDV['69_PM_4_50'] = [' 69','EURO2','RT >28-32t Euro-2','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003474946338','0.000513757774407','-0.027742764712336','0.898891558125545','0','6','69']
  H_HDV['69_FC_4_50'] = [' 69','EURO2','RT >28-32t Euro-2','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.004329194265369','0.628108663742673','-30.0993518410477','1205.27314624983','0','6','69']
  H_HDV['69_CO_4_100'] = [' 69','EURO2','RT >28-32t Euro-2','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.9021378517042','-0.04310576850803','-1.56165805713978','0.10130414954592','0','6','60']
  H_HDV['69_THC_4_100'] = [' 69','EURO2','RT >28-32t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.629346026269132','2.17305094237646','5.96507822565096','2.47060601210757','-0.019227797765298','6','60']
  H_HDV['69_NOx_4_100'] = [' 69','EURO2','RT >28-32t Euro-2','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000227019936605','0.030452754819919','-1.44305070283196','55.5796456772855','0','6','60']
  H_HDV['69_PM_4_100'] = [' 69','EURO2','RT >28-32t Euro-2','4','100','PM','y=(1/(a+(b*(x**c))))','0.39452936419339','0.229465153969161','0.534841953275216','0','0','6','60']
  H_HDV['69_FC_4_100'] = [' 69','EURO2','RT >28-32t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','327.331214510261','0.034867239060007','11039.2346557944','0.717081673048269','900.920007608488','6','60']
  H_HDV['69_CO_6_0'] = [' 69','EURO2','RT >28-32t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.3841782121081','-0.029055654771152','-1.13150974936839','0.116193972260591','0','6','71']
  H_HDV['69_THC_6_0'] = [' 69','EURO2','RT >28-32t Euro-2','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.425407818875052','2.23009494204206','6.46241072229418','2.53537012068698','-0.016760449299685','6','71']
  H_HDV['69_NOx_6_0'] = [' 69','EURO2','RT >28-32t Euro-2','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.22602899185702','0.030605124390711','25.3912028244687','0.143005511294441','20.3302823422542','6','71']
  H_HDV['69_PM_6_0'] = [' 69','EURO2','RT >28-32t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.906985614757863','-0.002148464528957','-0.053007306900544','0.106725065545464','0','6','71']
  H_HDV['69_FC_6_0'] = [' 69','EURO2','RT >28-32t Euro-2','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1163.51115283539','2.98873302490757','-40.6446192880805','0.058980850869389','0','6','71']
  H_HDV['69_CO_6_50'] = [' 69','EURO2','RT >28-32t Euro-2','6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.42911480044184','12.6154667432275','4.99892159325133','1.94288167258734','-0.008926537301325','6','57']
  H_HDV['69_THC_6_50'] = [' 69','EURO2','RT >28-32t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.608361235367426','2.32320106177344','5.57443848703885','2.32722680222008','-0.017390665854216','6','57']
  H_HDV['69_NOx_6_50'] = [' 69','EURO2','RT >28-32t Euro-2','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000264021804371','0.035267054145364','-1.62229825166094','57.7781814086852','0','6','57']
  H_HDV['69_PM_6_50'] = [' 69','EURO2','RT >28-32t Euro-2','6','50','PM','y=(c+(a*exp(b*x)))','0.787275970414954','-0.060635905219139','0.389604178280374','0','0','6','57']
  H_HDV['69_FC_6_50'] = [' 69','EURO2','RT >28-32t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','286.426201615331','0.05339736986775','4481.16471212254','0.554095717328814','945.355865443075','6','57']
  H_HDV['69_CO_6_100'] = [' 69','EURO2','RT >28-32t Euro-2','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.51352182914536','-4.19915094991774','-1.12402275221682','0','0','6','43']
  H_HDV['69_THC_6_100'] = [' 69','EURO2','RT >28-32t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.943900948994169','1.72321738594817','8.62700643719288','3.54647938284185','-0.035606376982273','6','43']
  H_HDV['69_NOx_6_100'] = [' 69','EURO2','RT >28-32t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000608002682228','0.061738739246704','-2.24684178662141','72.3231994576332','0','6','43']
  H_HDV['69_PM_6_100'] = [' 69','EURO2','RT >28-32t Euro-2','6','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.13812277072286','-0.45152650526362','-0.455564153314339','0','0','6','43']
  H_HDV['69_FC_6_100'] = [' 69','EURO2','RT >28-32t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','340.488528740356','0.024480185140502','10657.2298231557','0.689913636560948','1180.76309108056','6','43']
  H_HDV['70_CO_0_0'] = [' 70','EURO3','RT >28-32t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.24893111513026','0.053495300597715','5.8225633239929','0.20571021437487','1.14522644553727','6','86']
  H_HDV['70_THC_0_0'] = [' 70','EURO3','RT >28-32t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053267797801767','2.68124931563981','3.50254121077038','1.49183405742901','-0.008101028639285','6','86']
  H_HDV['70_NOx_0_0'] = [' 70','EURO3','RT >28-32t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.0258970868903','0.053909085406438','41.9952481071822','0.24573859823065','5.48505111132417','6','86']
  H_HDV['70_PM_0_0'] = [' 70','EURO3','RT >28-32t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.057785541989086','0.777521678923099','4.54648303851064','1.79844158892877','-0.011037194069907','6','86']
  H_HDV['70_FC_0_0'] = [' 70','EURO3','RT >28-32t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000997862429128','0.000105989008936','-0.00000073128768','0','0','6','86']
  H_HDV['70_CO_0_50'] = [' 70','EURO3','RT >28-32t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.41276539917252','69.1647866782165','-1.36968803318596','0.347118003933657','0.046005475762594','6','86']
  H_HDV['70_THC_0_50'] = [' 70','EURO3','RT >28-32t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.116548594605999','2.51651982427843','3.60442747135737','1.53815792157423','-0.004837952997433','6','86']
  H_HDV['70_NOx_0_50'] = [' 70','EURO3','RT >28-32t Euro-3','0','50','NOx','y=((a*(b**x))*(x**c))','82.2705800256744','1.00501137318506','-0.665072034169177','0','0','6','86']
  H_HDV['70_PM_0_50'] = [' 70','EURO3','RT >28-32t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.535717772793327','0.048942647174588','0.403999062133277','0.177104870273493','0.119176174706946','6','86']
  H_HDV['70_FC_0_50'] = [' 70','EURO3','RT >28-32t Euro-3','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001132983303311','0.000066157019205','-0.000000369246084','','','6','86']
  H_HDV['70_CO_0_100'] = [' 70','EURO3','RT >28-32t Euro-3','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.9142689187023','0.024397948868275','-0.501717109225265','0.047215112645752','0','6','86']
  H_HDV['70_THC_0_100'] = [' 70','EURO3','RT >28-32t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.601190607727542','0.022025060924477','2.14547046468636','0.103523723364088','0.160224387371687','6','86']
  H_HDV['70_NOx_0_100'] = [' 70','EURO3','RT >28-32t Euro-3','0','100','NOx','y=((a+(b*x))**((-1)/c))','-0.000204588431892','0.000168940760597','2.15576261297773','0','0','6','86']
  H_HDV['70_PM_0_100'] = [' 70','EURO3','RT >28-32t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.554605883102708','0.047706611332535','0.466142028412486','0.235744592601617','0.134169422457351','6','86']
  H_HDV['70_FC_0_100'] = [' 70','EURO3','RT >28-32t Euro-3','0','100','FC','y=(c+(a*exp(((-1)*b)*x)))','550.429379733369','0.037025757729205','255.781636197352','0','0','6','86']
  H_HDV['70_CO_-2_0'] = [' 70','EURO3','RT >28-32t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.13331864201185','0.022253012203614','-0.454071586549674','0.048424719116225','0','6','86']
  H_HDV['70_THC_-2_0'] = [' 70','EURO3','RT >28-32t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.216614205014377','1.85125858335068','4.59879325977063','1.64156989923707','0.017888594788231','6','86']
  H_HDV['70_NOx_-2_0'] = [' 70','EURO3','RT >28-32t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.8121835106045','0.064411191999459','100.441601896436','0.478373708427166','2.35910433637017','6','86']
  H_HDV['70_PM_-2_0'] = [' 70','EURO3','RT >28-32t Euro-3','-2','0','PM','y=(c+(a*exp(b*x)))','0.84778479273613','-0.060577026838398','0.100568392684686','0','0','6','86']
  H_HDV['70_FC_-2_0'] = [' 70','EURO3','RT >28-32t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','525.956079712331','0.056570544434998','28365.0965150306','0.97858232717531','81.5138165402484','6','86']
  H_HDV['70_CO_-2_50'] = [' 70','EURO3','RT >28-32t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.07915476890409','0.028410072350342','-0.432434457856567','0.043850242083306','0','6','86']
  H_HDV['70_THC_-2_50'] = [' 70','EURO3','RT >28-32t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.109251728381556','1.9373906607075','4.12164713110971','1.5455239759703','0.010169898997415','6','86']
  H_HDV['70_NOx_-2_50'] = [' 70','EURO3','RT >28-32t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.3748260843369','0.043394182784956','127.926562705431','0.481714832280616','1.30275853854595','6','86']
  H_HDV['70_PM_-2_50'] = [' 70','EURO3','RT >28-32t Euro-3','-2','50','PM','y=(c+(a*exp(b*x)))','0.794436257273116','-0.059045115182515','0.086415062724586','0','0','6','86']
  H_HDV['70_FC_-2_50'] = [' 70','EURO3','RT >28-32t Euro-3','-2','50','FC','y=((a*(b**x))*(x**c))','1100.97387353115','0.985832410175581','-0.389407236074304','0','0','6','86']
  H_HDV['70_CO_-2_100'] = [' 70','EURO3','RT >28-32t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.63809031733301','0.0140678610502','-0.476248404642515','0.049119760961868','0','6','86']
  H_HDV['70_THC_-2_100'] = [' 70','EURO3','RT >28-32t Euro-3','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.31873656624794','-0.002438348156829','-0.157306706900112','0.077344184922181','0','6','86']
  H_HDV['70_NOx_-2_100'] = [' 70','EURO3','RT >28-32t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.7607416302961','0.032725554331257','233.36993579979','0.572725711498283','0.558032605318881','6','86']
  H_HDV['70_PM_-2_100'] = [' 70','EURO3','RT >28-32t Euro-3','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.8635341399216','-6.87254276142715','-1.21796519762746','0','0','6','86']
  H_HDV['70_FC_-2_100'] = [' 70','EURO3','RT >28-32t Euro-3','-2','100','FC','y=((a*(b**x))*(x**c))','833.58142248104','0.97944430636549','-0.217379309219621','0','0','6','86']
  H_HDV['70_CO_-4_0'] = [' 70','EURO3','RT >28-32t Euro-3','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-0.50045486598072','0.063650368720019','9.00158680409087','0.063305731117526','0.381641626511696','6','86']
  H_HDV['70_THC_-4_0'] = [' 70','EURO3','RT >28-32t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.35197865612266','-0.002002323949492','-0.165816943497871','0.078154285063805','0','6','86']
  H_HDV['70_NOx_-4_0'] = [' 70','EURO3','RT >28-32t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.3672503952648','0.059101545715506','134.211011835699','0.508704080228079','0.533031947883568','6','86']
  H_HDV['70_PM_-4_0'] = [' 70','EURO3','RT >28-32t Euro-3','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.167648677878668','0.011556281133882','0.83100431082484','0.075572425985215','-0.005243752198976','6','86']
  H_HDV['70_FC_-4_0'] = [' 70','EURO3','RT >28-32t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','400.062156808771','0.048223433390728','3715.55695122609','0.550458114592433','15.7632172313347','6','86']
  H_HDV['70_CO_-4_50'] = [' 70','EURO3','RT >28-32t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.6290507974096','-0.00848515667332','-0.583250430996891','0.074771816587348','0','6','86']
  H_HDV['70_THC_-4_50'] = [' 70','EURO3','RT >28-32t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.15292890273336','-0.001887252408828','-0.159939641414755','0.082324448489721','0','6','86']
  H_HDV['70_NOx_-4_50'] = [' 70','EURO3','RT >28-32t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8095238807742','0.052180138007356','368.324078013835','0.672420198818211','0.429961034911982','6','86']
  H_HDV['70_PM_-4_50'] = [' 70','EURO3','RT >28-32t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.13990593017301','-7.39984629166855','-1.34821189818309','0','0','6','86']
  H_HDV['70_FC_-4_50'] = [' 70','EURO3','RT >28-32t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','351.444670021548','0.041766037318161','8914.22213926981','0.683885479885646','7.9833444147897','6','86']
  H_HDV['70_CO_-4_100'] = [' 70','EURO3','RT >28-32t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.08491745054698','-0.009452186904875','-0.5119950173844','0.070830926050056','0','6','86']
  H_HDV['70_THC_-4_100'] = [' 70','EURO3','RT >28-32t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.39258333878906','0.057805788039476','5.15010717486864','0.482586201491664','0.075728039803545','6','86']
  H_HDV['70_NOx_-4_100'] = [' 70','EURO3','RT >28-32t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.51205471489629','0.045250725951221','653.889811620339','0.765985104552641','0.300482446016281','6','86']
  H_HDV['70_PM_-4_100'] = [' 70','EURO3','RT >28-32t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.93109010267951','-7.04346058986122','-1.29524693046598','0','0','6','86']
  H_HDV['70_FC_-4_100'] = [' 70','EURO3','RT >28-32t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','334.865843771971','0.036548045906404','17395.269600567','0.793465663641745','1.33253731918884','6','86']
  H_HDV['70_CO_-6_0'] = [' 70','EURO3','RT >28-32t Euro-3','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-34.826342983184','62.0930907134503','7.84094941126236','0.070026238283763','0.245970321645982','6','72']
  H_HDV['70_THC_-6_0'] = [' 70','EURO3','RT >28-32t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70210010311177','0.067962126523665','1.79936870839774','0.386398676749825','0.08325807452267','6','72']
  H_HDV['70_NOx_-6_0'] = [' 70','EURO3','RT >28-32t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3153534362392','0.072128082805286','249.6148491523','0.626464858105095','0.408633502548447','6','72']
  H_HDV['70_PM_-6_0'] = [' 70','EURO3','RT >28-32t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.50438206304989','-8.31013686573904','-1.45758345618306','0','0','6','72']
  H_HDV['70_FC_-6_0'] = [' 70','EURO3','RT >28-32t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','388.626105281741','0.063196547815357','7458.21411300263','0.694472869051013','13.5136652946038','6','72']
  H_HDV['70_CO_-6_50'] = [' 70','EURO3','RT >28-32t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','6.64012411888957','-0.067855984919935','0.235679472373789','0','0','6','72']
  H_HDV['70_THC_-6_50'] = [' 70','EURO3','RT >28-32t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.32842841894071','0.064274992720829','6.5691592107747','0.533111906464607','0.077266766081468','6','72']
  H_HDV['70_NOx_-6_50'] = [' 70','EURO3','RT >28-32t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.6746722253398','0.062035939488372','445.850245429243','0.708456623966413','0.342201823772105','6','72']
  H_HDV['70_PM_-6_50'] = [' 70','EURO3','RT >28-32t Euro-3','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.78361638441618','-6.44906837873838','-1.2869642615623','0','0','6','72']
  H_HDV['70_FC_-6_50'] = [' 70','EURO3','RT >28-32t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','307.468601554291','0.057073925277958','10144.8121314524','0.71937582097538','11.8061943744138','6','72']
  H_HDV['70_CO_-6_100'] = [' 70','EURO3','RT >28-32t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','5.98818368978837','-0.064006092525196','0.221593602452036','0','0','6','72']
  H_HDV['70_THC_-6_100'] = [' 70','EURO3','RT >28-32t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.15241155381147','0.062141621611885','5.58005525732715','0.475571122510047','0.074856274864158','6','72']
  H_HDV['70_NOx_-6_100'] = [' 70','EURO3','RT >28-32t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.79777794646316','0.059390856730517','463.125771004472','0.714791512868038','0.338086501133346','6','72']
  H_HDV['70_PM_-6_100'] = [' 70','EURO3','RT >28-32t Euro-3','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.67704708063881','-6.31549066793546','-1.2606179003783','0','0','6','72']
  H_HDV['70_FC_-6_100'] = [' 70','EURO3','RT >28-32t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','284.626371169933','0.054537084876851','11214.3418501421','0.732659390922608','11.3876621091606','6','72']
  H_HDV['70_CO_2_0'] = [' 70','EURO3','RT >28-32t Euro-3','2','0','CO','y=(c+(a*exp(b*x)))','9.09753048944194','-0.070385380644442','1.54927588761471','0','0','6','86']
  H_HDV['70_THC_2_0'] = [' 70','EURO3','RT >28-32t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.08804897210049','2.55095785994423','3.93082669439525','1.55704167140765','-0.005150492009958','6','86']
  H_HDV['70_NOx_2_0'] = [' 70','EURO3','RT >28-32t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','49.2823948499636','-0.016349743649274','-4.78375833527605','0.12317019761503','0','6','86']
  H_HDV['70_PM_2_0'] = [' 70','EURO3','RT >28-32t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.094540613031923','0.610978772161622','5.60165358432971','2.02035898294246','-0.010549564983248','6','86']
  H_HDV['70_FC_2_0'] = [' 70','EURO3','RT >28-32t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','976.069113829651','0.716249329588425','-48.0748988596117','0.071288291643737','0','6','86']
  H_HDV['70_CO_2_50'] = [' 70','EURO3','RT >28-32t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.7329951769305','0.012571306427332','-0.602390354446061','0.056870181536289','0','6','86']
  H_HDV['70_THC_2_50'] = [' 70','EURO3','RT >28-32t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.232833528333959','2.13053020563243','5.35421425315265','2.09156545946767','-0.01128332681562','6','86']
  H_HDV['70_NOx_2_50'] = [' 70','EURO3','RT >28-32t Euro-3','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','53.574286572174','-0.039748164249515','-4.71070938092483','0.121575107833112','0','6','86']
  H_HDV['70_PM_2_50'] = [' 70','EURO3','RT >28-32t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.832610960922661','0.001496891665382','-0.035044166405569','0.047383952558665','0','6','86']
  H_HDV['70_FC_2_50'] = [' 70','EURO3','RT >28-32t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1047.96013266473','1.07914093662632','-36.8315694163105','0.058739162012597','0','6','86']
  H_HDV['70_CO_2_100'] = [' 70','EURO3','RT >28-32t Euro-3','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.051624440915751','0.007541709605749','-0.000015827191218','0','0','6','85']
  H_HDV['70_THC_2_100'] = [' 70','EURO3','RT >28-32t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.314780505839234','2.03636345333741','5.70781917319313','2.27253376693691','-0.014174685609088','6','85']
  H_HDV['70_NOx_2_100'] = [' 70','EURO3','RT >28-32t Euro-3','2','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','13.5340912782595','45.1425060765325','2.56192456539851','1.36599566357322','-0.001724412213686','6','85']
  H_HDV['70_PM_2_100'] = [' 70','EURO3','RT >28-32t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.918330620251484','0.000742948599984','-0.042651752259132','0.058775880017995','0','6','85']
  H_HDV['70_FC_2_100'] = [' 70','EURO3','RT >28-32t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1138.41602445683','0.884183845852227','-29.4815344231844','0.051102675101428','0','6','85']
  H_HDV['70_CO_4_0'] = [' 70','EURO3','RT >28-32t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.8877729590001','-0.019393581975635','-0.856247673638169','0.094200530067451','0','6','71']
  H_HDV['70_THC_4_0'] = [' 70','EURO3','RT >28-32t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.214758511770758','2.211787259548','5.20996536732694','2.00226310360583','-0.009111860434256','6','71']
  H_HDV['70_NOx_4_0'] = [' 70','EURO3','RT >28-32t Euro-3','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','60.0807643427655','-0.041459044266885','-6.47549476205242','0.143784176819016','0','6','71']
  H_HDV['70_PM_4_0'] = [' 70','EURO3','RT >28-32t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.178739896995152','5.72622978720085','-2.41356440725251','-0.208831021786651','0.073546470996379','6','71']
  H_HDV['70_FC_4_0'] = [' 70','EURO3','RT >28-32t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','1818.05044291981','1.00708954247039','-0.426858407719381','0','0','6','71']
  H_HDV['70_CO_4_50'] = [' 70','EURO3','RT >28-32t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.7035277125952','-0.035142150221163','-1.0631679096448','0.097738326670876','0','6','70']
  H_HDV['70_THC_4_50'] = [' 70','EURO3','RT >28-32t Euro-3','4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.35272817038482','-0.003441047860953','-0.270107288015858','0.099318092829162','0','6','70']
  H_HDV['70_NOx_4_50'] = [' 70','EURO3','RT >28-32t Euro-3','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','17.3246072112174','33.5169695291155','4.6452715483269','2.11985175452212','-0.011886843597761','6','70']
  H_HDV['70_PM_4_50'] = [' 70','EURO3','RT >28-32t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.0000042544626','0.00064441133891','-0.033249772279686','0.865897593185022','0','6','70']
  H_HDV['70_FC_4_50'] = [' 70','EURO3','RT >28-32t Euro-3','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003903967867805','0.586833272336157','-29.5490473301211','1230.67174393329','0','6','70']
  H_HDV['70_CO_4_100'] = [' 70','EURO3','RT >28-32t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.8793333170632','-0.042208856187064','-1.39970838882146','0.095707273915452','0','6','61']
  H_HDV['70_THC_4_100'] = [' 70','EURO3','RT >28-32t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.477971034322511','1.9013632629039','6.28996205966243','2.5365011203079','-0.020096834508296','6','61']
  H_HDV['70_NOx_4_100'] = [' 70','EURO3','RT >28-32t Euro-3','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','75.6147511402662','-0.09071478820798','-6.49220106066815','0.135438844238222','0','6','61']
  H_HDV['70_PM_4_100'] = [' 70','EURO3','RT >28-32t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004873446186','0.000729712830974','-0.036174169301015','0.977869753397853','0','6','61']
  H_HDV['70_FC_4_100'] = [' 70','EURO3','RT >28-32t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','387.531267123928','0.046336848232465','20713.158755913','0.856999854507975','926.027606743832','6','61']
  H_HDV['70_CO_6_0'] = [' 70','EURO3','RT >28-32t Euro-3','6','0','CO','y=(c+(a*exp(b*x)))','10.046458845467','-0.062444953607773','1.66301035586516','0','0','6','71']
  H_HDV['70_THC_6_0'] = [' 70','EURO3','RT >28-32t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.325786996024966','2.04607715885482','5.94210909935064','2.30906212879272','-0.013462579740854','6','71']
  H_HDV['70_NOx_6_0'] = [' 70','EURO3','RT >28-32t Euro-3','6','0','NOx','y=((a*(b**x))*(x**c))','116.024626895221','1.01072577931493','-0.641179433285575','0','0','6','71']
  H_HDV['70_PM_6_0'] = [' 70','EURO3','RT >28-32t Euro-3','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.236962082173372','8.84737833980194','-3.28719808413405','-0.520535081432311','0.108598171588417','6','71']
  H_HDV['70_FC_6_0'] = [' 70','EURO3','RT >28-32t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','1882.40573679169','1.0071815992775','-0.370315636079459','0','0','6','71']
  H_HDV['70_CO_6_50'] = [' 70','EURO3','RT >28-32t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.98881216886786','-4.4124164227414','-1.0157521047849','0','0','6','58']
  H_HDV['70_THC_6_50'] = [' 70','EURO3','RT >28-32t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.467796152655102','1.97771328483388','6.1126649404685','2.47593380468651','-0.020208700631918','6','58']
  H_HDV['70_NOx_6_50'] = [' 70','EURO3','RT >28-32t Euro-3','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','76.7302232979732','-0.068911765489439','-6.52378249728199','0.131092983209271','0','6','58']
  H_HDV['70_PM_6_50'] = [' 70','EURO3','RT >28-32t Euro-3','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-3.16966608563283','0.249340933973101','2.6977298258694','0.150092479179712','0.369971511444379','6','58']
  H_HDV['70_FC_6_50'] = [' 70','EURO3','RT >28-32t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','439.303704057368','0.073104368998834','12961.585288428','0.811856825540045','960.181448254445','6','58']
  H_HDV['70_CO_6_100'] = [' 70','EURO3','RT >28-32t Euro-3','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.7223940484852','-5.32271309052266','-1.18968974951245','0','0','6','45']
  H_HDV['70_THC_6_100'] = [' 70','EURO3','RT >28-32t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.794062162900085','7.11939759660761','-1.4327093077777','-0.582569159369138','0.16798445880955','6','45']
  H_HDV['70_NOx_6_100'] = [' 70','EURO3','RT >28-32t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','90.5037375885544','-0.125895350750063','-7.19029783832635','0.13192173489837','0','6','45']
  H_HDV['70_PM_6_100'] = [' 70','EURO3','RT >28-32t Euro-3','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.525054957317716','1.46673512967088','-1.75367607285881','-1.19980318481879','0.244892599404401','6','45']
  H_HDV['70_FC_6_100'] = [' 70','EURO3','RT >28-32t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','326.124200393494','0.037250212934621','7739.64916884009','0.640597168744783','1246.80675513413','6','45']
  H_HDV['71_CO_0_0'] = [' 71','EURO4','RT >28-32t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040448941918987','0.669931697289872','4.01669553473155','1.64275155739702','-0.008612662442219','6','86']
  H_HDV['71_THC_0_0'] = [' 71','EURO4','RT >28-32t Euro-4','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.062690058625672','0.040001815621027','0.10826054315108','0.13514705363187','0.011308691615152','6','86']
  H_HDV['71_NOx_0_0'] = [' 71','EURO4','RT >28-32t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.89800224394019','0.059805445356608','29.2828363478289','0.349677938204764','3.41849652038702','6','86']
  H_HDV['71_PM_0_0'] = [' 71','EURO4','RT >28-32t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086154151099128','0.041780868921624','0.274018177843239','0.169821523615538','0.01823839210759','6','86']
  H_HDV['71_FC_0_0'] = [' 71','EURO4','RT >28-32t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','489.317634485999','0.060563545777451','10612.1380124503','0.780047363741303','186.746713770025','6','86']
  H_HDV['71_CO_0_50'] = [' 71','EURO4','RT >28-32t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.055754738383483','0.742503630787644','3.34929431944334','1.43848949355936','-0.003032430036375','6','86']
  H_HDV['71_THC_0_50'] = [' 71','EURO4','RT >28-32t Euro-4','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.063201466417944','0.043431959472508','0.107903293030472','0.142027933931926','0.011251610419701','6','86']
  H_HDV['71_NOx_0_50'] = [' 71','EURO4','RT >28-32t Euro-4','0','50','NOx','y=((a*(b**x))*(x**c))','32.9950565844058','1.00275825577639','-0.522130306052489','0','0','6','86']
  H_HDV['71_PM_0_50'] = [' 71','EURO4','RT >28-32t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086037326813791','0.040249476970745','0.272325889021259','0.170196890939448','0.018491589847437','6','86']
  H_HDV['71_FC_0_50'] = [' 71','EURO4','RT >28-32t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','468.052929290949','0.044919710308723','48602.6109120307','1.03849806919552','215.077922389263','6','86']
  H_HDV['71_CO_0_100'] = [' 71','EURO4','RT >28-32t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.077928620169201','0.728053419949553','3.21583744241137','1.29372965510708','0.009507847561323','6','86']
  H_HDV['71_THC_0_100'] = [' 71','EURO4','RT >28-32t Euro-4','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065496464810783','0.046440316325883','0.106917505850403','0.153560160119806','0.011563248483596','6','86']
  H_HDV['71_NOx_0_100'] = [' 71','EURO4','RT >28-32t Euro-4','0','100','NOx','y=((a*(b**x))*(x**c))','27.6539341434319','0.998200179203919','-0.372783225632077','0','0','6','86']
  H_HDV['71_PM_0_100'] = [' 71','EURO4','RT >28-32t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.096808664946225','0.042842684991031','0.263619225938255','0.17683804209648','0.019122562908081','6','86']
  H_HDV['71_FC_0_100'] = [' 71','EURO4','RT >28-32t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','491.032044174032','0.033406371727041','861521.644622851','1.53657673248492','227.245717804909','6','86']
  H_HDV['71_CO_-2_0'] = [' 71','EURO4','RT >28-32t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.072320055637028','2.77578554045636','-1.4711776450201','-0.095181004974597','0.076316451967159','6','86']
  H_HDV['71_THC_-2_0'] = [' 71','EURO4','RT >28-32t Euro-4','-2','0','THC','y=(c+(a*exp(b*x)))','0.128502520427082','-0.067911183479143','0.013083929593111','0','0','6','86']
  H_HDV['71_NOx_-2_0'] = [' 71','EURO4','RT >28-32t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.86007862957847','0.059724037837553','80.1421290294341','0.554120973997208','1.29661436635034','6','86']
  H_HDV['71_PM_-2_0'] = [' 71','EURO4','RT >28-32t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007725193773094','0.20479506204303','5.15158723899728','2.15180550749183','-0.013801643193291','6','86']
  H_HDV['71_FC_-2_0'] = [' 71','EURO4','RT >28-32t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','495.744363708499','0.057123020051975','23542.3705196758','0.929122122591613','75.0715293387756','6','86']
  H_HDV['71_CO_-2_50'] = [' 71','EURO4','RT >28-32t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.020127337106051','0.473811452542937','6.22897108602504','2.24420784939932','-0.011247542196616','6','86']
  H_HDV['71_THC_-2_50'] = [' 71','EURO4','RT >28-32t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006738571635132','0.106328155750944','4.39517412377245','1.6340630817871','0.012169462841538','6','86']
  H_HDV['71_NOx_-2_50'] = [' 71','EURO4','RT >28-32t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.5909714177932','0.037623784383713','95.6890430367554','0.54536150502533','0.64201223710259','6','86']
  H_HDV['71_PM_-2_50'] = [' 71','EURO4','RT >28-32t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003074387849023','0.222192242832702','4.2778119991815','1.84540781198895','-0.007453809329131','6','86']
  H_HDV['71_FC_-2_50'] = [' 71','EURO4','RT >28-32t Euro-4','-2','50','FC','y=((a*(b**x))*(x**c))','1083.68345105205','0.985489325421938','-0.408047241569069','0','0','6','86']
  H_HDV['71_CO_-2_100'] = [' 71','EURO4','RT >28-32t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.643146575014755','-0.00065811850829','-0.038613511029456','0.068255907213668','0','6','86']
  H_HDV['71_THC_-2_100'] = [' 71','EURO4','RT >28-32t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003120443612364','0.116032490733726','3.73938533711455','1.48181372438681','0.007895589956837','6','86']
  H_HDV['71_NOx_-2_100'] = [' 71','EURO4','RT >28-32t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.96127006900464','0.027900816440599','184.466058569229','0.649357826280077','0.034532895197544','6','86']
  H_HDV['71_PM_-2_100'] = [' 71','EURO4','RT >28-32t Euro-4','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.75212998309959','-5.75634310173094','-1.45099549164314','0','0','6','86']
  H_HDV['71_FC_-2_100'] = [' 71','EURO4','RT >28-32t Euro-4','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','619.988214447444','-1.82180483637727','-31.026212791652','0.068077837341531','0','6','86']
  H_HDV['71_CO_-4_0'] = [' 71','EURO4','RT >28-32t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.001757753165784','0.459020224136229','5.95881063787619','2.15857980377873','-0.00716523241493','6','86']
  H_HDV['71_THC_-4_0'] = [' 71','EURO4','RT >28-32t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.133746778614463','-0.000105650883128','-0.009495832387889','0.077973185294657','0','6','86']
  H_HDV['71_NOx_-4_0'] = [' 71','EURO4','RT >28-32t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.76767391242292','0.052488308113534','75.5026418290808','0.514017031287392','0.182385319466859','6','86']
  H_HDV['71_PM_-4_0'] = [' 71','EURO4','RT >28-32t Euro-4','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.50245204797257','-7.63860059633147','-1.70634984130081','0','0','6','86']
  H_HDV['71_FC_-4_0'] = [' 71','EURO4','RT >28-32t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','390.230559011228','0.050711961457522','5995.32755636229','0.636987027411018','11.2352816133066','6','86']
  H_HDV['71_CO_-4_50'] = [' 71','EURO4','RT >28-32t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.548666548907239','-0.000483511267445','-0.037600253034956','0.074639336423389','0','6','86']
  H_HDV['71_THC_-4_50'] = [' 71','EURO4','RT >28-32t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.125071598594025','-0.00011283206957','-0.009583209885768','0.084415779014429','0','6','86']
  H_HDV['71_NOx_-4_50'] = [' 71','EURO4','RT >28-32t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.89752309102367','0.045394441599428','129.949781307536','0.596562971946126','0.060585761508736','6','86']
  H_HDV['71_PM_-4_50'] = [' 71','EURO4','RT >28-32t Euro-4','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.59013791425404','0.510378406504993','0.023444401169174','0','0','6','86']
  H_HDV['71_FC_-4_50'] = [' 71','EURO4','RT >28-32t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','332.671334822247','0.043562502926423','11905.9329104633','0.736298166180796','3.22598480144323','6','86']
  H_HDV['71_CO_-4_100'] = [' 71','EURO4','RT >28-32t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.506221869225207','-0.000484875727945','-0.034373629409505','0.074307487248491','0','6','86']
  H_HDV['71_THC_-4_100'] = [' 71','EURO4','RT >28-32t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.077806004863687','0.056909260328574','0.227638273656555','0.416433404964385','0.003450133004896','6','86']
  H_HDV['71_NOx_-4_100'] = [' 71','EURO4','RT >28-32t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.6137550822589','0.04031268215449','196.686222353028','0.665036003481665','-0.022657845958836','6','86']
  H_HDV['71_PM_-4_100'] = [' 71','EURO4','RT >28-32t Euro-4','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.06276154858641','0.647962419593426','0.022117294875255','0','0','6','86']
  H_HDV['71_FC_-4_100'] = [' 71','EURO4','RT >28-32t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','309.006222785167','0.038112755909654','20966.0459224289','0.826583020213041','-2.38361299784292','6','86']
  H_HDV['71_CO_-6_0'] = [' 71','EURO4','RT >28-32t Euro-4','-6','0','CO','y=(c+(a*exp(b*x)))','0.51316372925186','-0.070392302923005','0.012502630290197','0','0','6','72']
  H_HDV['71_THC_-6_0'] = [' 71','EURO4','RT >28-32t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.099552573233403','0.068401906152486','0.100094302898422','0.39267385957998','0.003897815502466','6','72']
  H_HDV['71_NOx_-6_0'] = [' 71','EURO4','RT >28-32t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.57821248314249','0.068185793729666','111.04677889616','0.60172055322028','0.088072011104113','6','72']
  H_HDV['71_PM_-6_0'] = [' 71','EURO4','RT >28-32t Euro-4','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.54498944354947','-7.35025164294137','-1.79529317236839','0','0','6','72']
  H_HDV['71_FC_-6_0'] = [' 71','EURO4','RT >28-32t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','366.001532366207','0.065136473426437','10972.1425388903','0.757165684259923','6.11163911703652','6','72']
  H_HDV['71_CO_-6_50'] = [' 71','EURO4','RT >28-32t Euro-4','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.373582237866483','0.064744823437156','5.21861349774004','0.838798519594121','0.010294994477866','6','72']
  H_HDV['71_THC_-6_50'] = [' 71','EURO4','RT >28-32t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.078446105019671','0.064787127427458','0.353847884152591','0.526996536744883','0.003567598180242','6','72']
  H_HDV['71_NOx_-6_50'] = [' 71','EURO4','RT >28-32t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.37817337168399','0.063643489028475','148.14369132818','0.638688710417033','0.071275934692526','6','72']
  H_HDV['71_PM_-6_50'] = [' 71','EURO4','RT >28-32t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115641738698171','0.06644423753653','0.312242051203002','0.280141404236155','0.004711870588784','6','72']
  H_HDV['71_FC_-6_50'] = [' 71','EURO4','RT >28-32t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','282.654321019536','0.05924053384331','14113.2265780559','0.775278592144554','4.65311590979031','6','72']
  H_HDV['71_CO_-6_100'] = [' 71','EURO4','RT >28-32t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.324499253269193','0.062570868657405','4.52924837575197','0.76157825699703','0.00976956595482','6','72']
  H_HDV['71_THC_-6_100'] = [' 71','EURO4','RT >28-32t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068767555749068','0.062724035904926','0.328125649530836','0.486533815931119','0.003432796969433','6','72']
  H_HDV['71_NOx_-6_100'] = [' 71','EURO4','RT >28-32t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.90000293925622','0.060716121510113','152.267482432399','0.638688883963578','0.066818319332443','6','72']
  H_HDV['71_PM_-6_100'] = [' 71','EURO4','RT >28-32t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.106342380339813','0.065122396038859','0.353509664536289','0.297768122559031','0.004585478027156','6','72']
  H_HDV['71_FC_-6_100'] = [' 71','EURO4','RT >28-32t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','254.355867497602','0.056487469392778','14554.9609805205','0.774476716103937','4.33150090967135','6','72']
  H_HDV['71_CO_2_0'] = [' 71','EURO4','RT >28-32t Euro-4','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.904747202398915','-0.000698513731197','-0.066443520636327','0.091374589132468','0','6','86']
  H_HDV['71_THC_2_0'] = [' 71','EURO4','RT >28-32t Euro-4','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','0.711590166487054','-5.04639943358151','-1.15148327163187','0','0','6','86']
  H_HDV['71_NOx_2_0'] = [' 71','EURO4','RT >28-32t Euro-4','2','0','NOx','y=(c+(a*exp(b*x)))','17.320032259639','-0.115587605394932','6.11506704367076','0','0','6','86']
  H_HDV['71_PM_2_0'] = [' 71','EURO4','RT >28-32t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.053931154837205','0.020027257716751','0.261332725020111','0.123374546485231','0.014559794739331','6','86']
  H_HDV['71_FC_2_0'] = [' 71','EURO4','RT >28-32t Euro-4','2','0','FC','y=((a*(b**x))*(x**c))','1850.434661708','1.00821565986047','-0.546520470542421','0','0','6','86']
  H_HDV['71_CO_2_50'] = [' 71','EURO4','RT >28-32t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.12295363226101','-0.001701140048836','-0.098904528730093','0.114925018683092','0','6','86']
  H_HDV['71_THC_2_50'] = [' 71','EURO4','RT >28-32t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007249062775185','0.126311970381355','5.12415158757418','2.03548442536849','-0.012213322882335','6','86']
  H_HDV['71_NOx_2_50'] = [' 71','EURO4','RT >28-32t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.5120030765768','-0.021487868474607','-1.91170573394327','0.118980323687095','0','6','86']
  H_HDV['71_PM_2_50'] = [' 71','EURO4','RT >28-32t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.066658131149526','0.023846206220697','0.272912994562165','0.134579923996913','0.017482782659493','6','86']
  H_HDV['71_FC_2_50'] = [' 71','EURO4','RT >28-32t Euro-4','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2190478.91197135','1.74508339777477','551.280973230187','0.074521953991092','451.125825410098','6','86']
  H_HDV['71_CO_2_100'] = [' 71','EURO4','RT >28-32t Euro-4','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.457351462602454','0.016961534352651','0.92801250899551','0.148178065977323','0.004443569677238','6','85']
  H_HDV['71_THC_2_100'] = [' 71','EURO4','RT >28-32t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01160721293457','0.120018914105723','5.61829902632228','2.26356429935723','-0.014265659425145','6','85']
  H_HDV['71_NOx_2_100'] = [' 71','EURO4','RT >28-32t Euro-4','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.14255055663345','1.29592719525693','-0.205899264629251','0','0','6','85']
  H_HDV['71_PM_2_100'] = [' 71','EURO4','RT >28-32t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.096309290157866','0.036907271205929','0.273714609628498','0.15592366714326','0.023579493731368','6','85']
  H_HDV['71_FC_2_100'] = [' 71','EURO4','RT >28-32t Euro-4','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1103.54120758596','0.845508224261432','-31.4276380643784','0.054731052948998','0','6','85']
  H_HDV['71_CO_4_0'] = [' 71','EURO4','RT >28-32t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.09570642617775','-0.001257251541847','-0.097241414640175','0.115059197394096','0','6','72']
  H_HDV['71_THC_4_0'] = [' 71','EURO4','RT >28-32t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005464742479786','0.13012653026093','4.99753431971135','1.94788797774528','-0.010698205296347','6','72']
  H_HDV['71_NOx_4_0'] = [' 71','EURO4','RT >28-32t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.0681721288768','-0.015650688411815','-3.18437488872353','0.153375475253779','0','6','72']
  H_HDV['71_PM_4_0'] = [' 71','EURO4','RT >28-32t Euro-4','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.330377914565301','-0.000400048564192','-0.031223507203918','0.113616985923052','0','6','72']
  H_HDV['71_FC_4_0'] = [' 71','EURO4','RT >28-32t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','1803.81176916484','1.00737196299907','-0.445577719312023','0','0','6','72']
  H_HDV['71_CO_4_50'] = [' 71','EURO4','RT >28-32t Euro-4','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.506201771390697','0.0185391058517','0.971850787851766','0.15851974685184','0.017733179419301','6','70']
  H_HDV['71_THC_4_50'] = [' 71','EURO4','RT >28-32t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012056220963034','0.120793684043974','5.52812328929265','2.21155989255742','-0.012431334637233','6','70']
  H_HDV['71_NOx_4_50'] = [' 71','EURO4','RT >28-32t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.29838536991662','0.041157114740057','25.3809977754061','0.267557450616045','11.2111269815793','6','70']
  H_HDV['71_PM_4_50'] = [' 71','EURO4','RT >28-32t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.117323738626988','0.00965372774862','0.303570259219737','0.136669750349599','-0.03054382875084','6','70']
  H_HDV['71_FC_4_50'] = [' 71','EURO4','RT >28-32t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','478.847091870998','0.074408036747793','18231.4335846094','0.891479269473928','673.395973437438','6','70']
  H_HDV['71_CO_4_100'] = [' 71','EURO4','RT >28-32t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.69775130096383','-0.002664457638261','-0.137288615673198','0.098260735136627','0','6','62']
  H_HDV['71_THC_4_100'] = [' 71','EURO4','RT >28-32t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017394964232882','0.113012178545871','6.1328366810746','2.4973151003582','-0.019272017906079','6','62']
  H_HDV['71_NOx_4_100'] = [' 71','EURO4','RT >28-32t Euro-4','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.9704769303364','-0.067490332947461','-2.63246890528912','0.139626784804741','0','6','62']
  H_HDV['71_PM_4_100'] = [' 71','EURO4','RT >28-32t Euro-4','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.025704206230515','0.261188517361739','5.1497677641146','2.20415695130445','-0.018720514481192','6','62']
  H_HDV['71_FC_4_100'] = [' 71','EURO4','RT >28-32t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','402.409847909704','0.05934841065064','23021.197044948','0.893765012727717','891.004868864635','6','62']
  H_HDV['71_CO_6_0'] = [' 71','EURO4','RT >28-32t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.21340248136338','-0.002770079733799','-0.103280504037481','0.11636626936565','0','6','71']
  H_HDV['71_THC_6_0'] = [' 71','EURO4','RT >28-32t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011408325384399','0.117533882862482','5.95324173784325','2.32141289783931','-0.014038752594609','6','71']
  H_HDV['71_NOx_6_0'] = [' 71','EURO4','RT >28-32t Euro-4','6','0','NOx','y=((a*(b**x))*(x**c))','42.5061328106517','1.00659737169566','-0.439311956496105','0','0','6','71']
  H_HDV['71_PM_6_0'] = [' 71','EURO4','RT >28-32t Euro-4','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082723452941823','0.020838303516183','0.272275123978661','0.135888454356688','0.012392511361774','6','71']
  H_HDV['71_FC_6_0'] = [' 71','EURO4','RT >28-32t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','1910.18695041462','1.00811647256191','-0.403066028459906','0','0','6','71']
  H_HDV['71_CO_6_50'] = [' 71','EURO4','RT >28-32t Euro-4','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.74727012404428','-4.664375352374','-1.07920989375087','0','0','6','59']
  H_HDV['71_THC_6_50'] = [' 71','EURO4','RT >28-32t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016745959224858','0.114653851620423','6.14376831708381','2.49386339345624','-0.019597670121097','6','59']
  H_HDV['71_NOx_6_50'] = [' 71','EURO4','RT >28-32t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.07557107740072','0.034830461481943','20.8494385716103','0.209360015703733','14.0536206346225','6','59']
  H_HDV['71_PM_6_50'] = [' 71','EURO4','RT >28-32t Euro-4','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024288767579849','0.275212813734274','4.58959957200555','1.95467436003784','-0.010305087366761','6','59']
  H_HDV['71_FC_6_50'] = [' 71','EURO4','RT >28-32t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','473.366013562156','0.082898121222427','17196.3621331552','0.870492914939629','909.020818992708','6','59']
  H_HDV['71_CO_6_100'] = [' 71','EURO4','RT >28-32t Euro-4','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.13776287687974','1.46277120000179','5.83852813669899','2.33195999227443','-0.009272810863492','6','45']
  H_HDV['71_THC_6_100'] = [' 71','EURO4','RT >28-32t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.027871310969925','0.094921351929327','8.33350882895744','3.40867347050964','-0.033647073911704','6','45']
  H_HDV['71_NOx_6_100'] = [' 71','EURO4','RT >28-32t Euro-4','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','47.8794248082468','-0.121019391037656','-3.72430214830627','0.159422658788342','0','6','45']
  H_HDV['71_PM_6_100'] = [' 71','EURO4','RT >28-32t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.050058880274996','0.234269851326527','7.35275080066795','3.00660413478331','-0.014404513323326','6','45']
  H_HDV['71_FC_6_100'] = [' 71','EURO4','RT >28-32t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','346.940106450088','0.067610138186484','8752.84037717753','0.697276167391747','1229.44934170713','6','45']
  H_HDV['72_CO_0_0'] = [' 72','EURO5','RT >28-32t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.039422583567929','0.68091988488531','4.04984229805451','1.65043303040678','-0.008948391907203','6','86']
  H_HDV['72_THC_0_0'] = [' 72','EURO5','RT >28-32t Euro-5','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.064403046793041','0.040183254044444','0.109688223676016','0.135088074414863','0.011615595820505','6','86']
  H_HDV['72_NOx_0_0'] = [' 72','EURO5','RT >28-32t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.66550048066944','0.057292629909113','17.3232620637383','0.290590822430305','1.99385783884558','6','86']
  H_HDV['72_PM_0_0'] = [' 72','EURO5','RT >28-32t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.088161144265882','0.041842060825405','0.278719956558185','0.169583056355165','0.018494276138921','6','86']
  H_HDV['72_FC_0_0'] = [' 72','EURO5','RT >28-32t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','502.355077856952','0.060046888061322','8992.97505532646','0.743233139189756','189.488268226318','6','86']
  H_HDV['72_CO_0_50'] = [' 72','EURO5','RT >28-32t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.056675726999607','0.748362967630823','3.38749744984322','1.44878600888888','-0.003134783075912','6','86']
  H_HDV['72_THC_0_50'] = [' 72','EURO5','RT >28-32t Euro-5','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065194188994437','0.043662643394667','0.109423625149607','0.142526156973524','0.01152327654298','6','86']
  H_HDV['72_NOx_0_50'] = [' 72','EURO5','RT >28-32t Euro-5','0','50','NOx','y=(1/(a+(b*(x**c))))','-0.221447546215896','0.213998453540489','0.248533624859015','0','0','6','86']
  H_HDV['72_PM_0_50'] = [' 72','EURO5','RT >28-32t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08773663375121','0.040291187779084','0.277373920082368','0.170045867253996','0.018784531371019','6','86']
  H_HDV['72_FC_0_50'] = [' 72','EURO5','RT >28-32t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','482.568253242022','0.044891841617856','38612.1183626586','0.992133664222914','217.947335855537','6','86']
  H_HDV['72_CO_0_100'] = [' 72','EURO5','RT >28-32t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.081848640467149','0.777123545925168','2.90291011319591','1.20000823061644','0.012685895348606','6','86']
  H_HDV['72_THC_0_100'] = [' 72','EURO5','RT >28-32t Euro-5','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.066875155668734','0.046240245478197','0.109060480959808','0.153149661945865','0.011739791437048','6','86']
  H_HDV['72_NOx_0_100'] = [' 72','EURO5','RT >28-32t Euro-5','0','100','NOx','y=((a+(b*x))**((-1)/c))','-0.000229890400946','0.001502345113814','2.12149537360657','0','0','6','86']
  H_HDV['72_PM_0_100'] = [' 72','EURO5','RT >28-32t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.099206785939911','0.042713392724557','0.271697232059123','0.178941677924372','0.019285332970775','6','86']
  H_HDV['72_FC_0_100'] = [' 72','EURO5','RT >28-32t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','500.144411840346','0.032799319837411','18756.0075060039','0.925878378438107','228.02329701999','6','86']
  H_HDV['72_CO_-2_0'] = [' 72','EURO5','RT >28-32t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.073703727521528','2.89009995429124','-1.4942843800192','-0.094573641774116','0.075982002987548','6','86']
  H_HDV['72_THC_-2_0'] = [' 72','EURO5','RT >28-32t Euro-5','-2','0','THC','y=(c+(a*exp(b*x)))','0.131262926341367','-0.06775214557484','0.013360293776227','0','0','6','86']
  H_HDV['72_NOx_-2_0'] = [' 72','EURO5','RT >28-32t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.60761531448885','0.060634231601866','38.4963572842599','0.467549206833407','0.76335850155008','6','86']
  H_HDV['72_PM_-2_0'] = [' 72','EURO5','RT >28-32t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007844179817911','0.209589199412598','5.13391747453671','2.14554968919828','-0.013677101164214','6','86']
  H_HDV['72_FC_-2_0'] = [' 72','EURO5','RT >28-32t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','512.463075466789','0.057085982120727','21452.3828647976','0.907428943416307','76.5857482608269','6','86']
  H_HDV['72_CO_-2_50'] = [' 72','EURO5','RT >28-32t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018907223112774','0.484572575765789','6.27810738406789','2.26528866040578','-0.012457033667757','6','86']
  H_HDV['72_THC_-2_50'] = [' 72','EURO5','RT >28-32t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00669017891245','0.107519413005439','4.55455279619871','1.69323178303935','0.009847044915618','6','86']
  H_HDV['72_NOx_-2_50'] = [' 72','EURO5','RT >28-32t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.77435524437134','0.039410412764645','48.7761815566695','0.476621347936705','0.39624101472166','6','86']
  H_HDV['72_PM_-2_50'] = [' 72','EURO5','RT >28-32t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003157202477022','0.225363585888515','4.32738408781993','1.86018996943878','-0.007702913861517','6','86']
  H_HDV['72_FC_-2_50'] = [' 72','EURO5','RT >28-32t Euro-5','-2','50','FC','y=((a*(b**x))*(x**c))','1134.29558686772','0.985551685550123','-0.414911816231798','0','0','6','86']
  H_HDV['72_CO_-2_100'] = [' 72','EURO5','RT >28-32t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.654011167838217','-0.000662989648641','-0.039115405029609','0.067902592711786','0','6','86']
  H_HDV['72_THC_-2_100'] = [' 72','EURO5','RT >28-32t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.040762536115547','0.033307578897351','0.096576417005285','0.098576579357188','0.004199700542528','6','86']
  H_HDV['72_NOx_-2_100'] = [' 72','EURO5','RT >28-32t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.96392815348054','0.030385930642745','83.8380777327612','0.565190969389042','0.088428352296079','6','86']
  H_HDV['72_PM_-2_100'] = [' 72','EURO5','RT >28-32t Euro-5','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.79264325059356','-5.82465339151748','-1.45633580478357','0','0','6','86']
  H_HDV['72_FC_-2_100'] = [' 72','EURO5','RT >28-32t Euro-5','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','641.480816733432','-1.86496924565015','-32.4984863220853','0.068659753052844','0','6','86']
  H_HDV['72_CO_-4_0'] = [' 72','EURO5','RT >28-32t Euro-5','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.003400125285678','0.469173728216517','6.09580912612038','2.2102622507977','-0.009201695353527','6','86']
  H_HDV['72_THC_-4_0'] = [' 72','EURO5','RT >28-32t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.136812799570039','-0.000108253500972','-0.009694519721441','0.077823873083677','0','6','86']
  H_HDV['72_NOx_-4_0'] = [' 72','EURO5','RT >28-32t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.3902987313371','0.054521591265598','44.3284544898179','0.474456700470062','0.11470350245879','6','86']
  H_HDV['72_PM_-4_0'] = [' 72','EURO5','RT >28-32t Euro-5','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.54078885160239','-7.69505751011018','-1.71011497210873','0','0','6','86']
  H_HDV['72_FC_-4_0'] = [' 72','EURO5','RT >28-32t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','404.753458252015','0.050893747080503','5854.92394223691','0.628040689616294','11.6060452991457','6','86']
  H_HDV['72_CO_-4_50'] = [' 72','EURO5','RT >28-32t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.562390545913792','-0.000494193602387','-0.038589853923317','0.074713806667376','0','6','86']
  H_HDV['72_THC_-4_50'] = [' 72','EURO5','RT >28-32t Euro-5','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.128014880442935','-0.000115303355358','-0.009797831873461','0.084304073303846','0','6','86']
  H_HDV['72_NOx_-4_50'] = [' 72','EURO5','RT >28-32t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.83686340471361','0.047866614029936','74.0846500354959','0.553952113457131','0.048144137601696','6','86']
  H_HDV['72_PM_-4_50'] = [' 72','EURO5','RT >28-32t Euro-5','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.55918395916684','0.493689842458592','0.023093949630888','0','0','6','86']
  H_HDV['72_FC_-4_50'] = [' 72','EURO5','RT >28-32t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','344.683603905026','0.043889014450309','11881.5703888776','0.730120528891188','3.515413064858','6','86']
  H_HDV['72_CO_-4_100'] = [' 72','EURO5','RT >28-32t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.517634766240256','-0.000493031538746','-0.035219483333995','0.074425857350934','0','6','86']
  H_HDV['72_THC_-4_100'] = [' 72','EURO5','RT >28-32t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079699394916554','0.056944059269074','0.22253649641422','0.410052236810794','0.003523191309866','6','86']
  H_HDV['72_NOx_-4_100'] = [' 72','EURO5','RT >28-32t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.64766418031936','0.043125627924652','104.943555338129','0.612117993781846','0.006274119118092','6','86']
  H_HDV['72_PM_-4_100'] = [' 72','EURO5','RT >28-32t Euro-5','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.0488993075689','0.628210181145538','0.021836030400374','0','0','6','86']
  H_HDV['72_FC_-4_100'] = [' 72','EURO5','RT >28-32t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','319.309382294711','0.038513667477731','20592.0604541077','0.817984961909426','-2.02554895058015','6','86']
  H_HDV['72_CO_-6_0'] = [' 72','EURO5','RT >28-32t Euro-5','-6','0','CO','y=(c+(a*exp(((-1)*b)*x)))','0.525270873467311','0.070251835930305','0.012698553980485','0','0','6','72']
  H_HDV['72_THC_-6_0'] = [' 72','EURO5','RT >28-32t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.10253444400117','0.068574723887785','0.095614528926568','0.388170414198256','0.003986494802654','6','72']
  H_HDV['72_NOx_-6_0'] = [' 72','EURO5','RT >28-32t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.34193085496501','0.069995627362379','64.1833753425503','0.55965733535119','0.056181872496348','6','72']
  H_HDV['72_PM_-6_0'] = [' 72','EURO5','RT >28-32t Euro-5','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.59897967816918','-7.46214537463672','-1.80306773433597','0','0','6','72']
  H_HDV['72_FC_-6_0'] = [' 72','EURO5','RT >28-32t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','381.669108121314','0.06540991308746','11253.0460559683','0.757348240367081','6.34829797689126','6','72']
  H_HDV['72_CO_-6_50'] = [' 72','EURO5','RT >28-32t Euro-5','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.384796518530633','0.065026626960469','5.24583495087895','0.841800759699105','0.010578222879253','6','72']
  H_HDV['72_THC_-6_50'] = [' 72','EURO5','RT >28-32t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080624385821188','0.064964838108417','0.351973100838703','0.524961315541942','0.003651104595107','6','72']
  H_HDV['72_NOx_-6_50'] = [' 72','EURO5','RT >28-32t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.56192867081956','0.065491297754275','83.7668194320506','0.594464452806131','0.045827574699104','6','72']
  H_HDV['72_PM_-6_50'] = [' 72','EURO5','RT >28-32t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.118071971645711','0.066412991040156','0.313113687740596','0.276778835886181','0.004802782113029','6','72']
  H_HDV['72_FC_-6_50'] = [' 72','EURO5','RT >28-32t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','293.55479651204','0.059547040749578','14232.6406437581','0.771508534835878','4.87807316786674','6','72']
  H_HDV['72_CO_-6_100'] = [' 72','EURO5','RT >28-32t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.329444259574538','0.062375122335543','3.64339378798772','0.72138713884559','0.009909172780663','6','72']
  H_HDV['72_THC_-6_100'] = [' 72','EURO5','RT >28-32t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.070329857717901','0.062717291225072','0.334086764861609','0.486218608463724','0.003501882227464','6','72']
  H_HDV['72_NOx_-6_100'] = [' 72','EURO5','RT >28-32t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.29511382444504','0.063178234954788','90.6877031247769','0.605797435593496','0.044605637472665','6','72']
  H_HDV['72_PM_-6_100'] = [' 72','EURO5','RT >28-32t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108435152770835','0.065067805535618','0.359144713213839','0.296620183288982','0.004673433485277','6','72']
  H_HDV['72_FC_-6_100'] = [' 72','EURO5','RT >28-32t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','256.809701656743','0.055704471679845','11845.6676755026','0.733667070191373','4.1995048654964','6','72']
  H_HDV['72_CO_2_0'] = [' 72','EURO5','RT >28-32t Euro-5','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.916754460959012','-0.000744273801648','-0.066824476155695','0.090712293331135','0','6','86']
  H_HDV['72_THC_2_0'] = [' 72','EURO5','RT >28-32t Euro-5','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','0.738066379509333','-5.07956706721865','-1.15220855971284','0','0','6','86']
  H_HDV['72_NOx_2_0'] = [' 72','EURO5','RT >28-32t Euro-5','2','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','12.6610274518036','0.120063596698193','3.59949046222838','0','0','6','86']
  H_HDV['72_PM_2_0'] = [' 72','EURO5','RT >28-32t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055864405951739','0.020775710470206','0.265745471075484','0.124195318161224','0.015115358367833','6','86']
  H_HDV['72_FC_2_0'] = [' 72','EURO5','RT >28-32t Euro-5','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.077284795611398','1.64089112832593','1682.99949621404','-0.454780805743345','0','6','86']
  H_HDV['72_CO_2_50'] = [' 72','EURO5','RT >28-32t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.1191504342196','-0.001601530292352','-0.095108914736987','0.110473236392705','0','6','86']
  H_HDV['72_THC_2_50'] = [' 72','EURO5','RT >28-32t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00754438224791','0.130625436620318','4.93822628411754','1.96581274738811','-0.009927757807969','6','86']
  H_HDV['72_NOx_2_50'] = [' 72','EURO5','RT >28-32t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.3231564001729','-0.017474726957854','-1.67190499877572','0.132610938117382','0','6','86']
  H_HDV['72_PM_2_50'] = [' 72','EURO5','RT >28-32t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.064561769832634','0.020391449777764','0.279044595005591','0.12977719998131','0.015000930687968','6','86']
  H_HDV['72_FC_2_50'] = [' 72','EURO5','RT >28-32t Euro-5','2','50','FC','y=((a*(b**x))*(x**c))','1725.72312414879','1.00565089619977','-0.40390791313862','0','0','6','86']
  H_HDV['72_CO_2_100'] = [' 72','EURO5','RT >28-32t Euro-5','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.458618812616062','0.015802729144143','0.937694101429131','0.145254805319401','-0.004332317388319','6','85']
  H_HDV['72_THC_2_100'] = [' 72','EURO5','RT >28-32t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011682935549351','0.124805418108413','5.39321601997865','2.17910632893575','-0.011951469481792','6','85']
  H_HDV['72_NOx_2_100'] = [' 72','EURO5','RT >28-32t Euro-5','2','100','NOx','y=((a+(b*x))**((-1)/c))','-0.000124845806926','0.000042603372564','3.48114468942931','0','0','6','85']
  H_HDV['72_PM_2_100'] = [' 72','EURO5','RT >28-32t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.084487942198139','0.029629258170857','0.28840513547948','0.145803360832074','0.020496762707135','6','85']
  H_HDV['72_FC_2_100'] = [' 72','EURO5','RT >28-32t Euro-5','2','100','FC','y=(c+(a*exp(b*x)))','573.706995624739','-0.067798422145531','587.273616001026','0','0','6','85']
  H_HDV['72_CO_4_0'] = [' 72','EURO5','RT >28-32t Euro-5','4','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000005650816176','0.000837715512218','-0.041337715242026','0.885143611211695','0','6','72']
  H_HDV['72_THC_4_0'] = [' 72','EURO5','RT >28-32t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005195421159864','0.133498217396947','4.95703879547398','1.93143470623807','-0.010690093797367','6','72']
  H_HDV['72_NOx_4_0'] = [' 72','EURO5','RT >28-32t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.6294091614767','-0.013168098521437','-2.64366698135757','0.165403815050662','0','6','72']
  H_HDV['72_PM_4_0'] = [' 72','EURO5','RT >28-32t Euro-5','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.336124479553409','-0.000412878188565','-0.03180037455966','0.113774583970034','0','6','72']
  H_HDV['72_FC_4_0'] = [' 72','EURO5','RT >28-32t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','1865.08855880473','1.00734641901661','-0.449966221115202','0','0','6','72']
  H_HDV['72_CO_4_50'] = [' 72','EURO5','RT >28-32t Euro-5','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.508944445857541','0.017466338482878','0.992748883833035','0.157925599030578','0.010058849035925','6','70']
  H_HDV['72_THC_4_50'] = [' 72','EURO5','RT >28-32t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01218806522673','0.122878042099335','5.56702571677451','2.22121247254299','-0.012509777813308','6','70']
  H_HDV['72_NOx_4_50'] = [' 72','EURO5','RT >28-32t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.1529143366342','0.028095493078876','17.9122954467234','0.232233996310997','5.539339358421','6','70']
  H_HDV['72_PM_4_50'] = [' 72','EURO5','RT >28-32t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115814982922885','0.009799670287817','0.307418285016127','0.135804215770969','-0.028183355574721','6','70']
  H_HDV['72_FC_4_50'] = [' 72','EURO5','RT >28-32t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','507.824474490411','0.075811299922058','22465.8617541426','0.928949290793927','683.795015158871','6','70']
  H_HDV['72_CO_4_100'] = [' 72','EURO5','RT >28-32t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.448859270467426','0.02261388761591','1.24037066404694','0.115416404170932','0.062024324588732','6','63']
  H_HDV['72_THC_4_100'] = [' 72','EURO5','RT >28-32t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018424942307525','0.111268425894914','6.63750517571671','2.68782210618983','-0.023140436685084','6','63']
  H_HDV['72_NOx_4_100'] = [' 72','EURO5','RT >28-32t Euro-5','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.412892424627','-0.050255485538544','-2.04516468746507','0.1356686508962','0','6','63']
  H_HDV['72_PM_4_100'] = [' 72','EURO5','RT >28-32t Euro-5','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.025775296791464','0.26473726999863','5.1811135383981','2.22535892684906','-0.020787467654177','6','63']
  H_HDV['72_FC_4_100'] = [' 72','EURO5','RT >28-32t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','447.264702029891','0.068618248744746','52723.3973074931','1.04860831011444','914.929770338355','6','63']
  H_HDV['72_CO_6_0'] = [' 72','EURO5','RT >28-32t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.19346732089629','-0.002489760443819','-0.095319374635589','0.108089024916562','0','6','71']
  H_HDV['72_THC_6_0'] = [' 72','EURO5','RT >28-32t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011251916796398','0.121232174880474','5.82848418236173','2.27461632896004','-0.013421716742174','6','71']
  H_HDV['72_NOx_6_0'] = [' 72','EURO5','RT >28-32t Euro-5','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.22669779030369','0.033296367623626','16.4167214976268','0.216526298411695','5.29345160987424','6','71']
  H_HDV['72_PM_6_0'] = [' 72','EURO5','RT >28-32t Euro-5','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083089132206384','0.021812047227577','0.274848881421068','0.135392629699675','0.01422440897417','6','71']
  H_HDV['72_FC_6_0'] = [' 72','EURO5','RT >28-32t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','1969.82652737852','1.00809131474025','-0.406781322433625','0','0','6','71']
  H_HDV['72_CO_6_50'] = [' 72','EURO5','RT >28-32t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.72944319340658','-0.004680743154671','-0.154895530456776','0.117241649835419','0','6','60']
  H_HDV['72_THC_6_50'] = [' 72','EURO5','RT >28-32t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018684775821906','0.111512816218856','6.66091983688827','2.66436404267397','-0.018707330972456','6','60']
  H_HDV['72_NOx_6_50'] = [' 72','EURO5','RT >28-32t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.67686215072632','0.011706027902256','16.0483890077116','0.166613551931519','4.10652650204626','6','60']
  H_HDV['72_PM_6_50'] = [' 72','EURO5','RT >28-32t Euro-5','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.027026675277412','0.271902086826479','4.75114816049289','2.01303728410273','-0.010070752455211','6','60']
  H_HDV['72_FC_6_50'] = [' 72','EURO5','RT >28-32t Euro-5','6','50','FC','y=((a*(b**x))*(x**c))','2020.40987868926','1.00539988740693','-0.266883483001173','0','0','6','60']
  H_HDV['72_CO_6_100'] = [' 72','EURO5','RT >28-32t Euro-5','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.14359150097641','1.43047715008577','6.2721488057274','2.53668058652103','-0.018892000903613','6','46']
  H_HDV['72_THC_6_100'] = [' 72','EURO5','RT >28-32t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029736757597923','0.09342155681603','8.99971712359632','3.66897698026154','-0.037237611967043','6','46']
  H_HDV['72_NOx_6_100'] = [' 72','EURO5','RT >28-32t Euro-5','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.6544272223002','-0.076941976712876','-2.46629782501124','0.134302011704323','0','6','46']
  H_HDV['72_PM_6_100'] = [' 72','EURO5','RT >28-32t Euro-5','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.054824098796432','0.23553834641565','6.91220465359759','2.74345116940748','0.007640028439581','6','46']
  H_HDV['72_FC_6_100'] = [' 72','EURO5','RT >28-32t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','409.137559344785','0.08855642114559','14238.8878942394','0.802159750637498','1268.97716229817','6','46']
  H_HDV['73_CO_0_0'] = [' 73','EURO0','RT >32t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.38794462812168','0.058741820467993','12.8880697438142','0.255765760598921','1.60286273329407','6','86']
  H_HDV['73_THC_0_0'] = [' 73','EURO0','RT >32t 80ties','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.23391835752485','7.06358603803319','2.93540609430163','1.39362367971644','-0.000557934791847','6','86']
  H_HDV['73_NOx_0_0'] = [' 73','EURO0','RT >32t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.276825961187','0.063108179946608','148.03955395839','0.597092815544079','9.22383182861553','6','86']
  H_HDV['73_PM_0_0'] = [' 73','EURO0','RT >32t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68920898005813','0.056462231896858','2.40718974344941','0.226140897534435','0.345616561938889','6','86']
  H_HDV['73_FC_0_0'] = [' 73','EURO0','RT >32t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','753.530306198167','0.062356755096064','10477.1506270346','0.694421538133744','218.344342733649','6','86']
  H_HDV['73_CO_0_50'] = [' 73','EURO0','RT >32t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.64893701931639','0.0575453279943','19.6155996592779','0.309668659365187','1.86986099527629','6','86']
  H_HDV['73_THC_0_50'] = [' 73','EURO0','RT >32t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.166741217537796','7.07254624090971','3.11503019578364','1.48395782424677','-0.004246450601149','6','86']
  H_HDV['73_NOx_0_50'] = [' 73','EURO0','RT >32t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.2169495480762','0.039817031122571','346.819889809833','0.741077362052963','10.8628517426004','6','86']
  H_HDV['73_PM_0_50'] = [' 73','EURO0','RT >32t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.70275869107249','0.052062438694051','3.30344955629857','0.281104897857064','0.405969754330346','6','86']
  H_HDV['73_FC_0_50'] = [' 73','EURO0','RT >32t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','704.488847262157','0.047567593927597','7979.54981095269','0.633574156882316','254.064429830698','6','86']
  H_HDV['73_CO_0_100'] = [' 73','EURO0','RT >32t 80ties','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.09298862110613','124.3183444004','-1.28111714726012','0.481388807162562','0.041154192398601','6','86']
  H_HDV['73_THC_0_100'] = [' 73','EURO0','RT >32t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.42073394069746','0.043771230558068','5.64056198364329','0.155783658374906','0.421158930947695','6','86']
  H_HDV['73_NOx_0_100'] = [' 73','EURO0','RT >32t 80ties','0','100','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.026687430954781','0.000669687189092','-0.000001046597866','0','0','6','86']
  H_HDV['73_PM_0_100'] = [' 73','EURO0','RT >32t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.75399885751716','0.047259181799243','4.556820487086','0.329089788326419','0.458629157229468','6','86']
  H_HDV['73_FC_0_100'] = [' 73','EURO0','RT >32t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','721.667858670566','0.036759078871853','20235.4720769471','0.804496278113942','276.596759859184','6','86']
  H_HDV['73_CO_-2_0'] = [' 73','EURO0','RT >32t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.4108715902516','0.013619628818688','-0.843396389887455','0.064203528046376','0','6','86']
  H_HDV['73_THC_-2_0'] = [' 73','EURO0','RT >32t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.69034574688805','0.067350918301691','10.2331550305446','0.499512415307249','0.409734835134712','6','86']
  H_HDV['73_NOx_-2_0'] = [' 73','EURO0','RT >32t 80ties','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.3503619658211','0.055015928167157','541.494169427053','0.776802509903539','2.96580532404798','6','86']
  H_HDV['73_PM_-2_0'] = [' 73','EURO0','RT >32t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.2395820814768','0.067509268534951','0.72982912102699','0.178953685157607','0.221971501877728','6','86']
  H_HDV['73_FC_-2_0'] = [' 73','EURO0','RT >32t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','671.614894357095','0.059562685050738','42205.8169725328','0.914311498762993','82.6069006006693','6','86']
  H_HDV['73_CO_-2_50'] = [' 73','EURO0','RT >32t 80ties','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.820638263094987','47.918301112657','-0.530014054012385','0.363612644395596','0.053255908283552','6','86']
  H_HDV['73_THC_-2_50'] = [' 73','EURO0','RT >32t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.12313054272132','0.059543593712132','9.88152712952883','0.427108767720153','0.265514318713069','6','86']
  H_HDV['73_NOx_-2_50'] = [' 73','EURO0','RT >32t 80ties','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.090218344688892','0.842742833977917','55.8184868246817','-0.517799259795413','0','6','86']
  H_HDV['73_PM_-2_50'] = [' 73','EURO0','RT >32t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.08328402950754','0.058771342842185','4.24581281876273','0.399618137364434','0.190693532312235','6','86']
  H_HDV['73_FC_-2_50'] = [' 73','EURO0','RT >32t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','591.770591181365','0.039646389718705','40341.9875778437','0.868948826301113','38.1577188507216','6','86']
  H_HDV['73_CO_-2_100'] = [' 73','EURO0','RT >32t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1069290389167','0.054642404484062','205.668994075768','0.748501893370274','0.720425340208786','6','86']
  H_HDV['73_THC_-2_100'] = [' 73','EURO0','RT >32t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.79322907277191','0.056611204384869','11.1237581193541','0.419649886243827','0.209847262558223','6','86']
  H_HDV['73_NOx_-2_100'] = [' 73','EURO0','RT >32t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.5493928240431','-0.107110159410918','-1.32150170838231','0.071469569657804','0','6','86']
  H_HDV['73_PM_-2_100'] = [' 73','EURO0','RT >32t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.94577948804562','0.050840812299206','9.31291617253445','0.489459646069425','0.173677792340469','6','86']
  H_HDV['73_FC_-2_100'] = [' 73','EURO0','RT >32t 80ties','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-2.46926419212159','0.885336907435208','1730.6477650512','-0.529624264197304','0','6','86']
  H_HDV['73_CO_-4_0'] = [' 73','EURO0','RT >32t 80ties','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.242010660395808','99.0490916572205','-1.42098468521642','0.434677200649841','0.043073681926787','6','86']
  H_HDV['73_THC_-4_0'] = [' 73','EURO0','RT >32t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.78384098072358','0.059898618088639','14.0453227551464','0.478150685526316','0.178524640781863','6','86']
  H_HDV['73_NOx_-4_0'] = [' 73','EURO0','RT >32t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9328112147132','0.04966085523521','405.709656841015','0.68807894514308','0.42934263215056','6','86']
  H_HDV['73_PM_-4_0'] = [' 73','EURO0','RT >32t 80ties','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.098299639213441','3.91931300157599','1.54280096702898','0.888841617440778','0.030372306637023','6','86']
  H_HDV['73_FC_-4_0'] = [' 73','EURO0','RT >32t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','490.247497952172','0.051434749468608','22984.5401441209','0.776653375974513','11.2192896907109','6','86']
  H_HDV['73_CO_-4_50'] = [' 73','EURO0','RT >32t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.007917530074','0.055857480004599','127.994074814355','0.686867648508731','0.230813319843386','6','86']
  H_HDV['73_THC_-4_50'] = [' 73','EURO0','RT >32t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.40176741922107','0.057905611643946','22.0729179655039','0.536941466558032','0.152300011640012','6','86']
  H_HDV['73_NOx_-4_50'] = [' 73','EURO0','RT >32t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.6990047622341','0.040341479165582','948.067278351715','0.825681254416364','0.072683675217088','6','86']
  H_HDV['73_PM_-4_50'] = [' 73','EURO0','RT >32t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.73521081378594','0.055861332677992','5.75441111691569','0.417862032889366','0.111892958174272','6','86']
  H_HDV['73_FC_-4_50'] = [' 73','EURO0','RT >32t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','427.159250007325','0.043862187201405','42301.1591226592','0.875141698264649','2.23315061296067','6','86']
  H_HDV['73_CO_-4_100'] = [' 73','EURO0','RT >32t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.35802854480685','0.049724613362412','231.048155663216','0.763075276063859','0.161237142174019','6','86']
  H_HDV['73_THC_-4_100'] = [' 73','EURO0','RT >32t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.21411399278194','0.057509259378821','27.6675338106402','0.574779358526028','0.145630653263875','6','86']
  H_HDV['73_NOx_-4_100'] = [' 73','EURO0','RT >32t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.1368258749541','0.033810480322378','1944.60054004329','0.948813295717074','-0.288631196886856','6','86']
  H_HDV['73_PM_-4_100'] = [' 73','EURO0','RT >32t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.694471347086','0.052573046673705','11.2725557969292','0.526107442535624','0.105467856871766','6','86']
  H_HDV['73_FC_-4_100'] = [' 73','EURO0','RT >32t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','404.297426969315','0.03877442339001','67364.4028398126','0.954213612953558','-4.22531573049507','6','86']
  H_HDV['73_CO_-6_0'] = [' 73','EURO0','RT >32t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.75894537316206','0.06912362383217','102.138049269019','0.730000201387247','0.195859855615632','6','72']
  H_HDV['73_THC_-6_0'] = [' 73','EURO0','RT >32t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.69808492144401','0.065455883451518','28.9830458382094','0.624729838170881','0.141660437693749','6','72']
  H_HDV['73_NOx_-6_0'] = [' 73','EURO0','RT >32t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6105041495242','0.062408560389366','736.452197687657','0.813921482034381','0.226396817437271','6','72']
  H_HDV['73_PM_-6_0'] = [' 73','EURO0','RT >32t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92889956184161','0.066430425862391','3.15845186451641','0.377961748120345','0.11180407351571','6','72']
  H_HDV['73_FC_-6_0'] = [' 73','EURO0','RT >32t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','433.894244141218','0.061111899864495','32374.4725886199','0.850930292824945','3.09723537460059','6','72']
  H_HDV['73_CO_-6_50'] = [' 73','EURO0','RT >32t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.99891516907228','0.061824839225237','162.163927830088','0.746162420880583','0.156543514074719','6','72']
  H_HDV['73_THC_-6_50'] = [' 73','EURO0','RT >32t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.23896433122242','0.063053342912316','35.2717091379013','0.637102692148089','0.13453256710818','6','72']
  H_HDV['73_NOx_-6_50'] = [' 73','EURO0','RT >32t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7157688885458','0.056274960477683','1016.77664833261','0.859950050737561','0.177864845808873','6','72']
  H_HDV['73_PM_-6_50'] = [' 73','EURO0','RT >32t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.73399706652251','0.062645002074545','7.3348159570635','0.490328378015706','0.107090000283142','6','72']
  H_HDV['73_FC_-6_50'] = [' 73','EURO0','RT >32t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','356.545287898863','0.056314338536648','40213.4108599863','0.882145661344139','1.82028459046304','6','72']
  H_HDV['73_CO_-6_100'] = [' 73','EURO0','RT >32t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.37022422288746','0.057747972902918','232.810025549756','0.785550046014273','0.135257243299739','6','72']
  H_HDV['73_THC_-6_100'] = [' 73','EURO0','RT >32t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.02952749064529','0.062184773382738','32.6021064117056','0.616236349035005','0.133090365812625','6','72']
  H_HDV['73_NOx_-6_100'] = [' 73','EURO0','RT >32t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7017172662943','0.05315659759328','1270.84127187192','0.898045950741787','0.169571061980278','6','72']
  H_HDV['73_PM_-6_100'] = [' 73','EURO0','RT >32t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.67445021826383','0.060700913975069','11.0938378930231','0.549979528217128','0.1053637567826','6','72']
  H_HDV['73_FC_-6_100'] = [' 73','EURO0','RT >32t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','320.064287896031','0.053238355240036','39304.2775633599','0.877807442131736','1.40675720116863','6','72']
  H_HDV['73_CO_2_0'] = [' 73','EURO0','RT >32t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.07662458551046','11.8416487623124','5.42613511821553','2.04587039352311','0.011182558036942','6','86']
  H_HDV['73_THC_2_0'] = [' 73','EURO0','RT >32t 80ties','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.074026817957712','6.66306814886319','4.1428463783772','1.70235334042059','-0.011332564200345','6','86']
  H_HDV['73_NOx_2_0'] = [' 73','EURO0','RT >32t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-27.3363931049876','0.033124800797112','56.4174932394396','0.060335411828425','19.1879511235235','6','86']
  H_HDV['73_PM_2_0'] = [' 73','EURO0','RT >32t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.526683148744162','2.23578393478596','4.56439583206365','1.73450077930137','0.024557418073888','6','86']
  H_HDV['73_FC_2_0'] = [' 73','EURO0','RT >32t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1522.1880599418','0.803297708309906','-94.1474523949488','0.078534886817098','0','6','86']
  H_HDV['73_CO_2_50'] = [' 73','EURO0','RT >32t 80ties','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.65391805708843','0.046036434739686','13.5079370207098','0.151437502991993','2.28170071624293','6','86']
  H_HDV['73_THC_2_50'] = [' 73','EURO0','RT >32t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.385452173872893','5.51991422158736','5.83523674908922','2.34209556286247','-0.013434753117936','6','86']
  H_HDV['73_NOx_2_50'] = [' 73','EURO0','RT >32t 80ties','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','53.0615137011887','0.029836242627914','-2.17721194059817','0.074946653869261','0','6','86']
  H_HDV['73_PM_2_50'] = [' 73','EURO0','RT >32t 80ties','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.37579691373688','0.048608938469383','2.16829471467757','0.160251438843085','0.649737161656118','6','86']
  H_HDV['73_FC_2_50'] = [' 73','EURO0','RT >32t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1681.03543954759','0.783528133908968','-94.0951140123973','0.080248712919816','0','6','86']
  H_HDV['73_CO_2_100'] = [' 73','EURO0','RT >32t 80ties','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.029902305890719','0.006258153463242','-0.00002231945355','0','0','6','78']
  H_HDV['73_THC_2_100'] = [' 73','EURO0','RT >32t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.516293176285532','5.1990672685501','6.50078861526341','2.62916651479959','-0.018375928635762','6','78']
  H_HDV['73_NOx_2_100'] = [' 73','EURO0','RT >32t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000085056498544','0.015365508966944','-0.949089545710548','53.8891175578618','0','6','78']
  H_HDV['73_PM_2_100'] = [' 73','EURO0','RT >32t 80ties','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.08767339249896','0.055584275542166','3.65609075730116','0.343921722010218','0.80902286373561','6','78']
  H_HDV['73_FC_2_100'] = [' 73','EURO0','RT >32t 80ties','2','100','FC','y=(c+(a*exp(b*x)))','1056.39442307623','-0.080959940458118','725.035805019071','0','0','6','78']
  H_HDV['73_CO_4_0'] = [' 73','EURO0','RT >32t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.05565137024675','13.0404785259883','5.73256201474725','2.3153600905117','-0.015466181230287','6','71']
  H_HDV['73_THC_4_0'] = [' 73','EURO0','RT >32t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.206416480917915','6.36628755637843','4.97240331603205','2.01487499682829','-0.009802105421376','6','71']
  H_HDV['73_NOx_4_0'] = [' 73','EURO0','RT >32t 80ties','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-27.4438132577797','0.041024240617207','59.2997744632427','0.073057310034982','26.821555541532','6','71']
  H_HDV['73_PM_4_0'] = [' 73','EURO0','RT >32t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.570124009880475','2.23864373486235','5.59134971506618','2.31823908202327','-0.02167086536452','6','71']
  H_HDV['73_FC_4_0'] = [' 73','EURO0','RT >32t 80ties','4','0','FC','y=(c+(a*exp(b*x)))','1388.72893413479','-0.102959219622108','524.478962358172','0','0','6','71']
  H_HDV['73_CO_4_50'] = [' 73','EURO0','RT >32t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.9863682522677','-0.042836158994496','-2.10526499075608','0.112424420328099','0','6','67']
  H_HDV['73_THC_4_50'] = [' 73','EURO0','RT >32t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.550478144419252','5.46106489028331','6.59210843988596','2.68476619463671','-0.019784971297633','6','67']
  H_HDV['73_NOx_4_50'] = [' 73','EURO0','RT >32t 80ties','4','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','39.2128625697508','-0.454256174879481','0.078905566465619','0','0','6','67']
  H_HDV['73_PM_4_50'] = [' 73','EURO0','RT >32t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.14281023604225','-0.005772093256488','-0.313725849615304','0.108374354617037','0','6','67']
  H_HDV['73_FC_4_50'] = [' 73','EURO0','RT >32t 80ties','4','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2152.28492803399','-0.609478341959207','-148.157074375324','0.115355755182402','0','6','67']
  H_HDV['73_CO_4_100'] = [' 73','EURO0','RT >32t 80ties','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.23322565276883','-0.741943418153435','-0.716200361745287','0','0','6','54']
  H_HDV['73_THC_4_100'] = [' 73','EURO0','RT >32t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.769345261312452','4.89124195808969','7.80532744471496','3.19473262704227','-0.031585361217366','6','54']
  H_HDV['73_NOx_4_100'] = [' 73','EURO0','RT >32t 80ties','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000233815692162','0.025813007106286','-1.09002702390047','71.5143565755196','0','6','54']
  H_HDV['73_PM_4_100'] = [' 73','EURO0','RT >32t 80ties','4','100','PM','y=(1/(a+(b*(x**c))))','-0.402938241457864','0.474618716406371','0.230527654729868','0','0','6','54']
  H_HDV['73_FC_4_100'] = [' 73','EURO0','RT >32t 80ties','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.012580942084469','1.50056074690754','-61.9363708287688','2049.52188625636','0','6','54']
  H_HDV['73_CO_6_0'] = [' 73','EURO0','RT >32t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.7281290599402','-0.043186640314223','-2.8696096785704','0.141450348155286','0','6','71']
  H_HDV['73_THC_6_0'] = [' 73','EURO0','RT >32t 80ties','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.66546818211596','0.005252313690255','-0.663081824708777','0.079957258837544','0','6','71']
  H_HDV['73_NOx_6_0'] = [' 73','EURO0','RT >32t 80ties','6','0','NOx','y=((a*(b**x))*(x**c))','95.738010697428','1.0088075961235','-0.385791287882756','0','0','6','71']
  H_HDV['73_PM_6_0'] = [' 73','EURO0','RT >32t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.32410923320202','-0.005042747400287','-0.408729922792717','0.129038485006111','0','6','71']
  H_HDV['73_FC_6_0'] = [' 73','EURO0','RT >32t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1849.12974387598','5.77014100071122','-90.123644029889','0.063007199879277','0','6','71']
  H_HDV['73_CO_6_50'] = [' 73','EURO0','RT >32t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.1768836589721','-0.046588833392392','-2.3779696564111','0.118319246173682','0','6','52']
  H_HDV['73_THC_6_50'] = [' 73','EURO0','RT >32t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.775773397898722','4.95860681171454','8.02434819473061','3.28342480187401','-0.034225753141911','6','52']
  H_HDV['73_NOx_6_50'] = [' 73','EURO0','RT >32t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.6845087704774','0.055495727131711','143.40093426875','0.54520513562934','52.3468403345576','6','52']
  H_HDV['73_PM_6_50'] = [' 73','EURO0','RT >32t 80ties','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000035291548338','0.004303566106062','-0.179282658404458','3.94535143753248','0','6','52']
  H_HDV['73_FC_6_50'] = [' 73','EURO0','RT >32t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','758.13267748741','0.09099550477284','2039.86084087647','0.391866324503128','1138.56370553583','6','52']
  H_HDV['73_CO_6_100'] = [' 73','EURO0','RT >32t 80ties','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','36.7099207968638','-0.124107564855009','-4.84151373253145','0.179976468045839','0','6','38']
  H_HDV['73_THC_6_100'] = [' 73','EURO0','RT >32t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.32330578527693','23.2740790189489','-2.01576949510444','-0.905953596587179','0.206038169829018','6','38']
  H_HDV['73_NOx_6_100'] = [' 73','EURO0','RT >32t 80ties','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000701189308755','0.054119255749535','-1.58019045823957','92.810050230607','0','6','38']
  H_HDV['73_PM_6_100'] = [' 73','EURO0','RT >32t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.85172028901208','-0.01860125394303','-0.925269944899128','0.211122992587629','0','6','38']
  H_HDV['73_FC_6_100'] = [' 73','EURO0','RT >32t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1238.10791651389','0.006368386358758','3136.43672387657','0.371426229023645','613.673757830966','6','38']
  H_HDV['74_CO_0_0'] = [' 74','EURO1','RT >32t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.58060155526867','0.054223623578057','9.83421753400329','0.224855170114052','1.21881344986908','6','86']
  H_HDV['74_THC_0_0'] = [' 74','EURO1','RT >32t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.172690343070473','5.77763177635388','3.03406434403002','1.44228082403733','-0.005388339887291','6','86']
  H_HDV['74_NOx_0_0'] = [' 74','EURO1','RT >32t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.2049828549658','0.063804027072937','63.6240617572286','0.44798886408067','6.71571485510215','6','86']
  H_HDV['74_PM_0_0'] = [' 74','EURO1','RT >32t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.37293770937019','0.052097546674168','2.6522120925275','0.178829269215678','0.227669956685618','6','86']
  H_HDV['74_FC_0_0'] = [' 74','EURO1','RT >32t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','606.816643042601','0.062672140521622','15645.715437319','0.786594281092017','187.437386221472','6','86']
  H_HDV['74_CO_0_50'] = [' 74','EURO1','RT >32t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.64487547278315','0.051921500543547','16.4253468637669','0.290903722873326','1.56107736637638','6','86']
  H_HDV['74_THC_0_50'] = [' 74','EURO1','RT >32t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.205228622774136','5.70159118234756','3.0437811594252','1.44080282777227','-0.003744737628492','6','86']
  H_HDV['74_NOx_0_50'] = [' 74','EURO1','RT >32t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.7360982493242','0.041589653455644','112.895238040161','0.555691580279569','7.75727674176652','6','86']
  H_HDV['74_PM_0_50'] = [' 74','EURO1','RT >32t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.49111339957183','0.052345358376048','3.05472656896318','0.23479267910004','0.284083201546409','6','86']
  H_HDV['74_FC_0_50'] = [' 74','EURO1','RT >32t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','580.516960872054','0.046482014287769','25294.455329721','0.858498113014284','224.846481296256','6','86']
  H_HDV['74_CO_0_100'] = [' 74','EURO1','RT >32t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.41484289685124','0.049521959878023','22.1399788892027','0.34477549771679','1.91278612157433','6','86']
  H_HDV['74_THC_0_100'] = [' 74','EURO1','RT >32t Euro-1','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.213067494334159','5.59325680637711','3.06237548044295','1.44359394110782','-0.004638793782974','6','86']
  H_HDV['74_NOx_0_100'] = [' 74','EURO1','RT >32t Euro-1','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000050169764641','0.009307688398886','-0.656572051459096','27.7555560642115','0','6','86']
  H_HDV['74_PM_0_100'] = [' 74','EURO1','RT >32t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5240853623587','0.048655540283676','3.7358327138785','0.280211774698725','0.346956739790439','6','86']
  H_HDV['74_FC_0_100'] = [' 74','EURO1','RT >32t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','616.928979958424','0.035512635185845','154560.520613412','1.18067051612367','244.387014944105','6','86']
  H_HDV['74_CO_-2_0'] = [' 74','EURO1','RT >32t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.3859533923797','0.011274068985955','-0.650643842511786','0.063909056599807','0','6','86']
  H_HDV['74_THC_-2_0'] = [' 74','EURO1','RT >32t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.322375362383573','9.16325820199725','0.586110399584542','0.569348688513118','0.048003686826218','6','86']
  H_HDV['74_NOx_-2_0'] = [' 74','EURO1','RT >32t Euro-1','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.7520527639133','0.055565731559099','116.848490845277','0.534757027118577','2.12999442809472','6','86']
  H_HDV['74_PM_-2_0'] = [' 74','EURO1','RT >32t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.072316809388219','2.09604730797496','7.88347715561097','3.08498912755883','-0.036990335029754','6','86']
  H_HDV['74_FC_-2_0'] = [' 74','EURO1','RT >32t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','569.790246549062','0.059721402109298','28548.6494639586','0.87681550835157','67.7666733742437','6','86']
  H_HDV['74_CO_-2_50'] = [' 74','EURO1','RT >32t Euro-1','-2','50','CO','y=(c+(a*exp(b*x)))','10.1854513877921','-0.071286836983808','0.727501099599558','0','0','6','86']
  H_HDV['74_THC_-2_50'] = [' 74','EURO1','RT >32t Euro-1','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.167410811124589','16.7386720949399','-0.312461684852646','0.558088529855513','0.034339936682974','6','86']
  H_HDV['74_NOx_-2_50'] = [' 74','EURO1','RT >32t Euro-1','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.007323671403657','1.27124707275168','54.465884726983','-0.630131645570908','0','6','86']
  H_HDV['74_PM_-2_50'] = [' 74','EURO1','RT >32t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.121089916412111','2.27868933516808','5.00010957152221','1.9341646600016','0.007667779656665','6','86']
  H_HDV['74_FC_-2_50'] = [' 74','EURO1','RT >32t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','509.235320209355','0.039949234867117','32215.4669285112','0.861199383083834','35.5375776276856','6','86']
  H_HDV['74_CO_-2_100'] = [' 74','EURO1','RT >32t Euro-1','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.75330466828144','0.051919812985163','149.7076965929','0.724943508112694','0.59265371771148','6','86']
  H_HDV['74_THC_-2_100'] = [' 74','EURO1','RT >32t Euro-1','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.086595705957161','32.6774018148364','-1.13124252716797','0.559419696212929','0.027191768825131','6','86']
  H_HDV['74_NOx_-2_100'] = [' 74','EURO1','RT >32t Euro-1','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.008329931784791','1.34574803822474','44.1580790118132','-0.524367591061652','0','6','86']
  H_HDV['74_PM_-2_100'] = [' 74','EURO1','RT >32t Euro-1','-2','100','PM','y=(1/(a+(b*(x**c))))','0.304388543743673','0.016779348610482','1.37516971673504','0','0','6','86']
  H_HDV['74_FC_-2_100'] = [' 74','EURO1','RT >32t Euro-1','-2','100','FC','y=((a*(b**x))*(x**c))','1080.33996584949','0.978813385665754','-0.289675023232512','0','0','6','86']
  H_HDV['74_CO_-4_0'] = [' 74','EURO1','RT >32t Euro-1','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.159523264459972','58.9292111794127','-1.03155825918824','0.498079408513245','0.039877789252412','6','86']
  H_HDV['74_THC_-4_0'] = [' 74','EURO1','RT >32t Euro-1','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.184895046007939','0.021775282662664','0.000832043177207','0','0','6','86']
  H_HDV['74_NOx_-4_0'] = [' 74','EURO1','RT >32t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2730968452907','0.053104799368354','144.42690612201','0.53966756402323','0.387588271862211','6','86']
  H_HDV['74_PM_-4_0'] = [' 74','EURO1','RT >32t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.37119608003604','-9.24413587427323','-1.78383759934973','0','0','6','86']
  H_HDV['74_FC_-4_0'] = [' 74','EURO1','RT >32t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','431.580811880859','0.052799016417918','14759.1501436131','0.73001717475351','10.6118353849966','6','86']
  H_HDV['74_CO_-4_50'] = [' 74','EURO1','RT >32t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.97213613554299','0.054198773788148','66.6624185342426','0.605486542612299','0.153222188575919','6','86']
  H_HDV['74_THC_-4_50'] = [' 74','EURO1','RT >32t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.69006245731849','0.056031925111244','12.1309395454648','0.463703663448578','0.098546576307374','6','86']
  H_HDV['74_NOx_-4_50'] = [' 74','EURO1','RT >32t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1237852841997','0.04423755632476','338.071759535768','0.676090841797507','0.177078488697943','6','86']
  H_HDV['74_PM_-4_50'] = [' 74','EURO1','RT >32t Euro-1','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.262535755101604','0.040306140479561','0.001613431746009','0','0','6','86']
  H_HDV['74_FC_-4_50'] = [' 74','EURO1','RT >32t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','380.220705193778','0.044605398391102','27362.2647341906','0.827097519223895','2.98359835607377','6','86']
  H_HDV['74_CO_-4_100'] = [' 74','EURO1','RT >32t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.54608979310681','0.046954329235016','157.088552809466','0.725821432152734','0.082022091817333','6','86']
  H_HDV['74_THC_-4_100'] = [' 74','EURO1','RT >32t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.48653368279332','0.054517718394145','16.7388982468653','0.515894970170211','0.0909441593586','6','86']
  H_HDV['74_NOx_-4_100'] = [' 74','EURO1','RT >32t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.3628306978149','0.037823124129643','639.971059884781','0.784179907198802','-0.023886541474526','6','86']
  H_HDV['74_PM_-4_100'] = [' 74','EURO1','RT >32t Euro-1','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.57991528767702','0.056162828886323','5.75372563438047','0.39561043680102','0.061785569718835','6','86']
  H_HDV['74_FC_-4_100'] = [' 74','EURO1','RT >32t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','359.732598424822','0.038617829840617','39001.4337761646','0.886741946008124','-3.59457247081821','6','86']
  H_HDV['74_CO_-6_0'] = [' 74','EURO1','RT >32t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.82872683684463','0.070977730233169','41.2101987539232','0.627319157282292','0.136666548479689','6','72']
  H_HDV['74_THC_-6_0'] = [' 74','EURO1','RT >32t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.09022516009634','0.066859251881499','17.1688781095574','0.585965418886936','0.096908883605956','6','72']
  H_HDV['74_NOx_-6_0'] = [' 74','EURO1','RT >32t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8813493512362','0.066883850179593','310.528199368202','0.697819357691426','0.232322601165778','6','72']
  H_HDV['74_PM_-6_0'] = [' 74','EURO1','RT >32t Euro-1','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.08599902179513','0.074097760827687','1.05335810513803','0.191711929754881','0.067455527041329','6','72']
  H_HDV['74_FC_-6_0'] = [' 74','EURO1','RT >32t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','392.136069306053','0.062676377127066','21850.8750139416','0.812668859167441','3.74279130032884','6','72']
  H_HDV['74_CO_-6_50'] = [' 74','EURO1','RT >32t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.33204045404061','0.062478980839656','94.1000470751939','0.693596535136586','0.105051402853528','6','72']
  H_HDV['74_THC_-6_50'] = [' 74','EURO1','RT >32t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.60201823519215','0.063431261394044','25.3863933697262','0.623040567032789','0.089428794637742','6','72']
  H_HDV['74_NOx_-6_50'] = [' 74','EURO1','RT >32t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.82508838961415','0.059298093074968','402.934942191377','0.728523048523862','0.181161942405477','6','72']
  H_HDV['74_PM_-6_50'] = [' 74','EURO1','RT >32t Euro-1','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72188206958477','0.068522416383627','5.52306166670541','0.447141462719044','0.060581260164248','6','72']
  H_HDV['74_FC_-6_50'] = [' 74','EURO1','RT >32t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','323.476944195339','0.057264081756053','28937.1569153192','0.852443019303153','2.45621044452935','6','72']
  H_HDV['74_CO_-6_100'] = [' 74','EURO1','RT >32t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.83697718752597','0.057532976803389','158.531148415581','0.755465083202789','0.086002090732563','6','72']
  H_HDV['74_THC_-6_100'] = [' 74','EURO1','RT >32t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.39439489075761','0.062127458025357','25.032546016271','0.614432502437474','0.087564521150847','6','72']
  H_HDV['74_NOx_-6_100'] = [' 74','EURO1','RT >32t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.04530171901967','0.056850689279819','602.708965855467','0.797506903548634','0.17855936166563','6','72']
  H_HDV['74_PM_-6_100'] = [' 74','EURO1','RT >32t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.56786460012317','0.065244404536915','9.82320980647177','0.534379759238417','0.057686980161756','6','72']
  H_HDV['74_FC_-6_100'] = [' 74','EURO1','RT >32t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','294.504165047935','0.054072103661122','34446.3290964179','0.879731891011576','1.98595593718475','6','72']
  H_HDV['74_CO_2_0'] = [' 74','EURO1','RT >32t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.978761694777','0.011752614509902','-0.900957590999299','0.069857763539443','0','6','86']
  H_HDV['74_THC_2_0'] = [' 74','EURO1','RT >32t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.254777160582043','4.74117717871057','4.85241822899332','1.954500002621','-0.012532544766176','6','86']
  H_HDV['74_NOx_2_0'] = [' 74','EURO1','RT >32t Euro-1','2','0','NOx','y=(c+(a*exp(b*x)))','27.8806448077176','-0.104992459328522','12.1946846172181','0','0','6','86']
  H_HDV['74_PM_2_0'] = [' 74','EURO1','RT >32t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.35178258537475','1.98008453690672','5.43398339065829','2.00685195434598','0.024546695111701','6','86']
  H_HDV['74_FC_2_0'] = [' 74','EURO1','RT >32t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.086197076415687','1.67134090055416','2338.15523108232','-0.547364060504927','0','6','86']
  H_HDV['74_CO_2_50'] = [' 74','EURO1','RT >32t Euro-1','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.03168394977688','0.010462521674288','-0.000081169443692','0','0','6','86']
  H_HDV['74_THC_2_50'] = [' 74','EURO1','RT >32t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.576711566482175','4.10627494976152','6.31999725104812','2.50502406369402','-0.015857996377364','6','86']
  H_HDV['74_NOx_2_50'] = [' 74','EURO1','RT >32t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.1658111929269','-0.023446429158878','-1.90628867148125','0.087886920686961','0','6','86']
  H_HDV['74_PM_2_50'] = [' 74','EURO1','RT >32t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.07052603572517','0.002046421374141','-0.211659117439843','0.080404264867282','0','6','86']
  H_HDV['74_FC_2_50'] = [' 74','EURO1','RT >32t Euro-1','2','50','FC','y=((a*(b**x))*(x**c))','2364.91626677216','1.00738364696965','-0.489567797330619','0','0','6','86']
  H_HDV['74_CO_2_100'] = [' 74','EURO1','RT >32t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.4701157152482','-0.028268896578557','-1.46530520251403','0.106531125550204','0','6','79']
  H_HDV['74_THC_2_100'] = [' 74','EURO1','RT >32t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.650348603603523','3.94376373418935','7.16734640759899','2.90992998986471','-0.036036991474101','6','79']
  H_HDV['74_NOx_2_100'] = [' 74','EURO1','RT >32t Euro-1','2','100','NOx','y=((a*(b**x))*(x**c))','53.913461365001','1.00092559621975','-0.229733738359674','0','0','6','79']
  H_HDV['74_PM_2_100'] = [' 74','EURO1','RT >32t Euro-1','2','100','PM','y=(c+(a*exp(b*x)))','2.45822507013921','-0.088472844473679','0.741661300582793','0','0','6','79']
  H_HDV['74_FC_2_100'] = [' 74','EURO1','RT >32t Euro-1','2','100','FC','y=((a*(b**x))*(x**c))','2133.41229343762','1.00420681329823','-0.352008026402716','0','0','6','79']
  H_HDV['74_CO_4_0'] = [' 74','EURO1','RT >32t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.53675273189278','65.9040316414643','-2.64346375143021','-0.947990987730252','0.18132384072671','6','71']
  H_HDV['74_THC_4_0'] = [' 74','EURO1','RT >32t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.467706996628608','4.61308108003509','5.54400551202882','2.22268193015957','-0.012879250864491','6','71']
  H_HDV['74_NOx_4_0'] = [' 74','EURO1','RT >32t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','26.6620429506186','0.107535191003552','6917.70728653716','1.48957319170185','17.4793341106988','6','71']
  H_HDV['74_PM_4_0'] = [' 74','EURO1','RT >32t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.481600154686944','1.70415426747423','5.55215950470755','2.00244795773077','0.032892778877128','6','71']
  H_HDV['74_FC_4_0'] = [' 74','EURO1','RT >32t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','862.26900752806','0.096590154326546','8420.14444685845','0.768215906736492','471.437254050854','6','71']
  H_HDV['74_CO_4_50'] = [' 74','EURO1','RT >32t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.5217232155597','-0.015753821404341','-1.55142513026729','0.110557862105024','0','6','68']
  H_HDV['74_THC_4_50'] = [' 74','EURO1','RT >32t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.783051419838885','4.26781595974173','6.03217738540555','2.45508684324339','-0.016042639938622','6','68']
  H_HDV['74_NOx_4_50'] = [' 74','EURO1','RT >32t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.1086365927497','0.06373413905725','90.1788520333629','0.558258895682586','25.6666670900995','6','68']
  H_HDV['74_PM_4_50'] = [' 74','EURO1','RT >32t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000015340472787','0.002290367160549','-0.111990230242217','2.63776037287522','0','6','68']
  H_HDV['74_FC_4_50'] = [' 74','EURO1','RT >32t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','610.399042961923','0.071726286779252','5648.57961229993','0.605089532568146','732.219627622349','6','68']
  H_HDV['74_CO_4_100'] = [' 74','EURO1','RT >32t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.1781499967499','-0.092481757450573','-2.9281390963169','0.185371264135158','0','6','54']
  H_HDV['74_THC_4_100'] = [' 74','EURO1','RT >32t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.13284067348504','3.65583826228263','7.48109361819511','3.03455783241734','-0.027588487570437','6','54']
  H_HDV['74_NOx_4_100'] = [' 74','EURO1','RT >32t Euro-1','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000226625692197','0.026558417098562','-1.15760616122956','54.1089627573465','0','6','54']
  H_HDV['74_PM_4_100'] = [' 74','EURO1','RT >32t Euro-1','4','100','PM','y=((a+(b*x))**((-1)/c))','-0.044545887236227','0.01307314018511','3.99431415272863','0','0','6','54']
  H_HDV['74_FC_4_100'] = [' 74','EURO1','RT >32t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','496.455706697108','0.024311207337629','4097.18247497877','0.466392104639871','845.715638957624','6','54']
  H_HDV['74_CO_6_0'] = [' 74','EURO1','RT >32t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.89558173240839','8.48144131715073','8.2902732549433','3.34204177733153','-0.028741788727762','6','71']
  H_HDV['74_THC_6_0'] = [' 74','EURO1','RT >32t Euro-1','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.80706585035924','0.006283834736424','-0.483582917602367','0.077196690874802','0','6','71']
  H_HDV['74_NOx_6_0'] = [' 74','EURO1','RT >32t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','52.6122942623173','-0.031822092312417','-3.82464352709638','0.134218074054833','0','6','71']
  H_HDV['74_PM_6_0'] = [' 74','EURO1','RT >32t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.77476205663802','0.008093115858674','-0.16711875071758','0.066783622698437','0','6','71']
  H_HDV['74_FC_6_0'] = [' 74','EURO1','RT >32t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2779.66240348279','1.01103244011207','-0.523758050153542','0','0','6','71']
  H_HDV['74_CO_6_50'] = [' 74','EURO1','RT >32t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.0327516959144','-0.07539798430968','-2.21367711730265','0.164308863929762','0','6','53']
  H_HDV['74_THC_6_50'] = [' 74','EURO1','RT >32t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.16908749981863','3.6483513564768','7.89456862187272','3.22824876700273','-0.032859060604716','6','53']
  H_HDV['74_NOx_6_50'] = [' 74','EURO1','RT >32t Euro-1','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.85725290026957','1.19460389515067','-0.085211597990977','0','0','6','53']
  H_HDV['74_PM_6_50'] = [' 74','EURO1','RT >32t Euro-1','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.5276585244302','-0.002315676007814','-0.33457799034719','0.147189315860475','0','6','53']
  H_HDV['74_FC_6_50'] = [' 74','EURO1','RT >32t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','504.886592189661','0.068079390630379','4489.63390422842','0.529539598036924','991.542474377554','6','53']
  H_HDV['74_CO_6_100'] = [' 74','EURO1','RT >32t Euro-1','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.0227782326464','-0.171299343804039','-10.3242760135316','0.341769544373265','0','6','39']
  H_HDV['74_THC_6_100'] = [' 74','EURO1','RT >32t Euro-1','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000059537369176','0.007830682506349','-0.330344526199297','6.33928933651616','0','6','39']
  H_HDV['74_NOx_6_100'] = [' 74','EURO1','RT >32t Euro-1','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','104.690162581949','-0.262039641135054','-15.1768659837334','0.314808370339116','0','6','39']
  H_HDV['74_PM_6_100'] = [' 74','EURO1','RT >32t Euro-1','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.27065321035275','0.008784471871249','10.2039734654386','0.495606891880558','-0.06368191856116','6','39']
  H_HDV['74_FC_6_100'] = [' 74','EURO1','RT >32t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','565.947599228344','0.034731681588567','11471.8302977524','0.677189284285015','1227.58533016878','6','39']
  H_HDV['75_CO_0_0'] = [' 75','EURO2','RT >32t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.61510583267686','0.060212400231182','6.69627664372037','0.264579918750758','1.10220369559103','6','86']
  H_HDV['75_THC_0_0'] = [' 75','EURO2','RT >32t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.091794107585806','3.77586309355267','2.8423863079002','1.37855484481905','-0.00488375315817','6','86']
  H_HDV['75_NOx_0_0'] = [' 75','EURO2','RT >32t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.9507399896988','0.060347778021156','44.5834244354981','0.357539504855644','7.1235418158353','6','86']
  H_HDV['75_PM_0_0'] = [' 75','EURO2','RT >32t Euro-2','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.600172172824448','0.000710334948006','-0.029706543881305','0.05726820651557','0','6','86']
  H_HDV['75_FC_0_0'] = [' 75','EURO2','RT >32t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000875320544079','0.000120649728487','-0.000000823474458','0','0','6','86']
  H_HDV['75_CO_0_50'] = [' 75','EURO2','RT >32t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.069949581920032','0.01477570970754','-0.000095795565728','0','0','6','86']
  H_HDV['75_THC_0_50'] = [' 75','EURO2','RT >32t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.892970185686123','0.027054467612935','2.90372620535714','0.121937510133619','0.194233734828913','6','86']
  H_HDV['75_NOx_0_50'] = [' 75','EURO2','RT >32t Euro-2','0','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.35023849347409','214.628094826108','-1.40987660144673','0.457996002009187','0.022236261451604','6','86']
  H_HDV['75_PM_0_50'] = [' 75','EURO2','RT >32t Euro-2','0','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.588272089205581','0.013307212324105','-0.015953278047385','0.0129499323116','0','6','86']
  H_HDV['75_FC_0_50'] = [' 75','EURO2','RT >32t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','520.39587419163','0.044293261534827','37089.4764098699','0.955284364242103','218.914549617005','6','86']
  H_HDV['75_CO_0_100'] = [' 75','EURO2','RT >32t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.55580136582512','0.044412370640086','-0.326162881475761','0.034533628017874','0','6','86']
  H_HDV['75_THC_0_100'] = [' 75','EURO2','RT >32t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.875353829696275','0.023760494461988','2.91442637155541','0.12190507795668','0.1699869072425','6','86']
  H_HDV['75_NOx_0_100'] = [' 75','EURO2','RT >32t Euro-2','0','100','NOx','y=(1/(a+(b*(x**c))))','0.024364934317647','0.003556339804472','0.713193777166088','0','0','6','86']
  H_HDV['75_PM_0_100'] = [' 75','EURO2','RT >32t Euro-2','0','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000000451205959','0.000063580540584','-0.013079169143244','0.659819468295644','0','6','86']
  H_HDV['75_FC_0_100'] = [' 75','EURO2','RT >32t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','564.139186234154','0.032205176004364','441221.919132188','1.38880342136289','232.171779471105','6','86']
  H_HDV['75_CO_-2_0'] = [' 75','EURO2','RT >32t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.80728499884928','0.009583507370491','-0.412337488713902','0.061192073774713','0','6','86']
  H_HDV['75_THC_-2_0'] = [' 75','EURO2','RT >32t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.200341033458032','10.0776549897552','-0.305858377230137','0.499783971800954','0.04289196152542','6','86']
  H_HDV['75_NOx_-2_0'] = [' 75','EURO2','RT >32t Euro-2','-2','0','NOx','y=(1/(a+(b*x)))','0.019453947380151','0.005109562128518','0','0','0','6','86']
  H_HDV['75_PM_-2_0'] = [' 75','EURO2','RT >32t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.581385270246751','0.000618954693729','-0.03210453446859','0.05749472264473','0','6','86']
  H_HDV['75_FC_-2_0'] = [' 75','EURO2','RT >32t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','507.863285867057','0.057931948525007','26163.5581104567','0.889139459312771','64.0604725578522','6','86']
  H_HDV['75_CO_-2_50'] = [' 75','EURO2','RT >32t Euro-2','-2','50','CO','y=(c+(a*exp(b*x)))','6.75643016319662','-0.069007931081936','0.608696012599691','0','0','6','86']
  H_HDV['75_THC_-2_50'] = [' 75','EURO2','RT >32t Euro-2','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.109380296634537','10.9825807143887','-0.321384930581019','0.585986171651844','0.02992645484737','6','86']
  H_HDV['75_NOx_-2_50'] = [' 75','EURO2','RT >32t Euro-2','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.028911100103571','0.997585684139859','57.6776377648945','-0.612452715685485','0','6','86']
  H_HDV['75_PM_-2_50'] = [' 75','EURO2','RT >32t Euro-2','-2','50','PM','y=(c+(a*exp(b*x)))','0.572100199992426','-0.061178735155129','0.068688919540817','0','0','6','86']
  H_HDV['75_FC_-2_50'] = [' 75','EURO2','RT >32t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','464.030872284016','0.038125163519356','32934.6927367275','0.893508064162492','32.7908255205689','6','86']
  H_HDV['75_CO_-2_100'] = [' 75','EURO2','RT >32t Euro-2','-2','100','CO','y=((a*(b**x))*(x**c))','16.3344328055968','0.988391746546408','-0.55488706281743','0','0','6','86']
  H_HDV['75_THC_-2_100'] = [' 75','EURO2','RT >32t Euro-2','-2','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.258846651956069','0.03815144117499','0.000615790952208','0','0','6','86']
  H_HDV['75_NOx_-2_100'] = [' 75','EURO2','RT >32t Euro-2','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.023840742731038','1.13096613117168','47.8091513286463','-0.520901460032047','0','6','86']
  H_HDV['75_PM_-2_100'] = [' 75','EURO2','RT >32t Euro-2','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.522525250733236','0.044289910745941','26.4028232339916','0.934349532852619','0.062813930907041','6','86']
  H_HDV['75_FC_-2_100'] = [' 75','EURO2','RT >32t Euro-2','-2','100','FC','y=((a*(b**x))*(x**c))','892.358331719073','0.978167817717628','-0.237469321004386','0','0','6','86']
  H_HDV['75_CO_-4_0'] = [' 75','EURO2','RT >32t Euro-2','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.125119434372366','0.014628505037821','0.000617941735743','0','0','6','86']
  H_HDV['75_THC_-4_0'] = [' 75','EURO2','RT >32t Euro-2','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.060197968233899','23.0713799158002','-1.27897866442168','0.556971268078562','0.029824016248311','6','86']
  H_HDV['75_NOx_-4_0'] = [' 75','EURO2','RT >32t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.5402108424532','0.053098251734523','132.97528450278','0.514860496367008','0.443240900561208','6','86']
  H_HDV['75_PM_-4_0'] = [' 75','EURO2','RT >32t Euro-2','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.650491264643332','-0.000692349734506','-0.050155179031884','0.08813583838904','0','6','86']
  H_HDV['75_FC_-4_0'] = [' 75','EURO2','RT >32t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','390.202041479955','0.051996300796165','12659.8529140633','0.729495837855926','10.7432815898949','6','86']
  H_HDV['75_CO_-4_50'] = [' 75','EURO2','RT >32t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.91112341052268','0.050196331583347','37.5683554616829','0.560374125784938','0.117764438686256','6','86']
  H_HDV['75_THC_-4_50'] = [' 75','EURO2','RT >32t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66231027349871','0.051737810214593','5.51588147238961','0.392499379544381','0.079464889237731','6','86']
  H_HDV['75_NOx_-4_50'] = [' 75','EURO2','RT >32t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1998333920003','0.044752350690518','331.326131971094','0.663627535641529','0.218821313428757','6','86']
  H_HDV['75_PM_-4_50'] = [' 75','EURO2','RT >32t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','1.2695908227552','0.986631959091037','-0.564062912718756','0','0','6','86']
  H_HDV['75_FC_-4_50'] = [' 75','EURO2','RT >32t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','341.44877928142','0.042775576867918','20965.1326704666','0.807156336904433','2.52098145070917','6','86']
  H_HDV['75_CO_-4_100'] = [' 75','EURO2','RT >32t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.93346215110271','0.042630679646112','125.309925836526','0.740141074289708','0.044638748230862','6','86']
  H_HDV['75_THC_-4_100'] = [' 75','EURO2','RT >32t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.58481856269758','0.051218883266663','6.04912898918066','0.411456025495179','0.077085980035728','6','86']
  H_HDV['75_NOx_-4_100'] = [' 75','EURO2','RT >32t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3349413207764','0.03878034693755','666.58862754851','0.78315231197929','0.018322317628937','6','86']
  H_HDV['75_PM_-4_100'] = [' 75','EURO2','RT >32t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.422569585008128','0.040062167568432','3.06386404041232','0.549940604265353','0.022310526779763','6','86']
  H_HDV['75_FC_-4_100'] = [' 75','EURO2','RT >32t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','330.852540298088','0.037273351812269','39260.8162356221','0.913368847141464','-3.77669940706643','6','86']
  H_HDV['75_CO_-6_0'] = [' 75','EURO2','RT >32t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.63158633322503','0.071223315658341','15.222487240822','0.520874253523808','0.123185142460805','6','72']
  H_HDV['75_THC_-6_0'] = [' 75','EURO2','RT >32t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.93728800134027','0.062741723610124','7.64346734173255','0.516021430790087','0.087158360576438','6','72']
  H_HDV['75_NOx_-6_0'] = [' 75','EURO2','RT >32t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.2449821266261','0.067471839190254','297.087847141826','0.684848478410348','0.280236149389314','6','72']
  H_HDV['75_PM_-6_0'] = [' 75','EURO2','RT >32t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012315766854417','0.454281766188929','5.33309687763553','2.11968005652386','-0.013999588385833','6','72']
  H_HDV['75_FC_-6_0'] = [' 75','EURO2','RT >32t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','356.099846339416','0.062479058511932','18352.5713561287','0.809740674985539','4.63056375207496','6','72']
  H_HDV['75_CO_-6_50'] = [' 75','EURO2','RT >32t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.6829691001814','0.060878449165295','54.7165751909635','0.664655243674333','0.097335217173749','6','72']
  H_HDV['75_THC_-6_50'] = [' 75','EURO2','RT >32t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.74269056758566','0.060754974859516','9.47666409511937','0.533525948613541','0.084012857247122','6','72']
  H_HDV['75_NOx_-6_50'] = [' 75','EURO2','RT >32t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8528586431329','0.060129314355403','497.361013582648','0.753755108653497','0.227156871238116','6','72']
  H_HDV['75_PM_-6_50'] = [' 75','EURO2','RT >32t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.365240290791692','0.053546275431927','0.437119753502063','0.242952776317638','0.033392543501325','6','72']
  H_HDV['75_FC_-6_50'] = [' 75','EURO2','RT >32t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','292.538990207842','0.056577831398138','25701.7139076488','0.855690261112757','3.3240044332714','6','72']
  H_HDV['75_CO_-6_100'] = [' 75','EURO2','RT >32t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.50067511177851','0.056195177295585','108.820708855979','0.757901552771875','0.084611219625079','6','72']
  H_HDV['75_THC_-6_100'] = [' 75','EURO2','RT >32t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.65533106416036','0.06016545853859','8.55717769119283','0.51291627685106','0.083690546086012','6','72']
  H_HDV['75_NOx_-6_100'] = [' 75','EURO2','RT >32t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.76230755365619','0.056923228578695','621.632177517461','0.79359712048982','0.212583476801527','6','72']
  H_HDV['75_PM_-6_100'] = [' 75','EURO2','RT >32t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.41358612428636','0.054217789172569','1.04586423457411','0.403867100337702','0.033999197492919','6','72']
  H_HDV['75_FC_-6_100'] = [' 75','EURO2','RT >32t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','268.252142690879','0.053379118691368','32031.6210410733','0.891968846036567','2.85999641396472','6','72']
  H_HDV['75_CO_2_0'] = [' 75','EURO2','RT >32t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.38435089839643','0.008502002771314','-0.61007855987835','0.075964602986232','0','6','86']
  H_HDV['75_THC_2_0'] = [' 75','EURO2','RT >32t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.150237558329662','3.24327490749024','4.09540021138857','1.69269912611','-0.005954869538571','6','86']
  H_HDV['75_NOx_2_0'] = [' 75','EURO2','RT >32t Euro-2','2','0','NOx','y=(c+(a*exp(b*x)))','29.7525265661833','-0.092170452810503','12.5220740565904','0','0','6','86']
  H_HDV['75_PM_2_0'] = [' 75','EURO2','RT >32t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.20616380805876','0.124244593370376','-0.001036884219821','0','0','6','86']
  H_HDV['75_FC_2_0'] = [' 75','EURO2','RT >32t Euro-2','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.137821121616722','1.55821588148561','2006.34258116512','-0.51555213718618','0','6','86']
  H_HDV['75_CO_2_50'] = [' 75','EURO2','RT >32t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.073122587130295','0.009717940336825','-0.000059619105061','0','0','6','86']
  H_HDV['75_THC_2_50'] = [' 75','EURO2','RT >32t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.319422914090841','2.69979673546837','5.88219845968929','2.34044391838198','-0.013887825625093','6','86']
  H_HDV['75_NOx_2_50'] = [' 75','EURO2','RT >32t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.1653518555065','-0.037333678951773','-2.29961954852008','0.088306390571331','0','6','86']
  H_HDV['75_PM_2_50'] = [' 75','EURO2','RT >32t Euro-2','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.743016741252748','0.008331044563707','-0.019180012570494','0.019350222379806','0','6','86']
  H_HDV['75_FC_2_50'] = [' 75','EURO2','RT >32t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1178.41366826985','0.885318597434521','-50.7133201004449','0.068415630573619','0','6','86']
  H_HDV['75_CO_2_100'] = [' 75','EURO2','RT >32t Euro-2','2','100','CO','y=((a*(b**x))*(x**c))','20.1168400516468','0.992434249379327','-0.413334052534117','0','0','6','82']
  H_HDV['75_THC_2_100'] = [' 75','EURO2','RT >32t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.426889144076921','2.60578169791888','5.96630499613492','2.3989936065832','-0.013717043616887','6','82']
  H_HDV['75_NOx_2_100'] = [' 75','EURO2','RT >32t Euro-2','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','22.0097713031023','0.047601223851748','255.081052324672','0.756699128928095','20.3825349741537','6','82']
  H_HDV['75_PM_2_100'] = [' 75','EURO2','RT >32t Euro-2','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.594233807668886','0.032244522727395','21.8831286861602','0.863712957280931','0.23171054062963','6','82']
  H_HDV['75_FC_2_100'] = [' 75','EURO2','RT >32t Euro-2','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002537037234835','0.459421412650981','-27.8739283339295','1217.00334968048','0','6','82']
  H_HDV['75_CO_4_0'] = [' 75','EURO2','RT >32t Euro-2','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.12827405359887','5.59871587424189','6.99119065187925','2.80309040138633','-0.023294131368748','6','71']
  H_HDV['75_THC_4_0'] = [' 75','EURO2','RT >32t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.246847974460611','3.0944546843445','5.05039531972975','2.04636443102449','-0.010580297834444','6','71']
  H_HDV['75_NOx_4_0'] = [' 75','EURO2','RT >32t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9232633896533','0.06654528670154','22.981153337401','0.188335602646045','17.2848844330133','6','71']
  H_HDV['75_PM_4_0'] = [' 75','EURO2','RT >32t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.23775566454085','0.079413935474468','-0.000627837607852','0','0','6','71']
  H_HDV['75_FC_4_0'] = [' 75','EURO2','RT >32t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','2299.38113446567','1.00948322123425','-0.527155044597997','0','0','6','71']
  H_HDV['75_CO_4_50'] = [' 75','EURO2','RT >32t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.16778296368777','0.014814945012766','8.37237917342982','0.172347062171638','-0.67785844914906','6','69']
  H_HDV['75_THC_4_50'] = [' 75','EURO2','RT >32t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.522167014221093','2.545722364113','7.02625734098147','2.83943660747202','-0.02171220526321','6','69']
  H_HDV['75_NOx_4_50'] = [' 75','EURO2','RT >32t Euro-2','4','50','NOx','y=((a*(b**x))*(x**c))','76.2615173211684','1.00319920370542','-0.318351501808696','0','0','6','69']
  H_HDV['75_PM_4_50'] = [' 75','EURO2','RT >32t Euro-2','4','50','PM','y=(1/(a+(b*(x**c))))','0.854988477047414','0.12412391422107','0.69554382194848','0','0','6','69']
  H_HDV['75_FC_4_50'] = [' 75','EURO2','RT >32t Euro-2','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','606.471737517508','0.085738933475644','20634.8721556149','0.907341021276594','744.863002706081','6','69']
  H_HDV['75_CO_4_100'] = [' 75','EURO2','RT >32t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.55793022649244','0.030973031076798','21.2907145849258','0.363277360024858','0.755458903941205','6','57']
  H_HDV['75_THC_4_100'] = [' 75','EURO2','RT >32t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.62527200975242','2.55257762534622','6.30906353618791','2.59497787033335','-0.020795418621708','6','57']
  H_HDV['75_NOx_4_100'] = [' 75','EURO2','RT >32t Euro-2','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000268445143059','0.03464827554616','-1.59063184003349','60.0981818763885','0','6','57']
  H_HDV['75_PM_4_100'] = [' 75','EURO2','RT >32t Euro-2','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.810771793424362','0.025074296088792','8.82400362628631','0.668895455083936','0.194770940976141','6','57']
  H_HDV['75_FC_4_100'] = [' 75','EURO2','RT >32t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','447.186031456004','0.026050030193476','6311.85832638321','0.587988595579015','875.72421340054','6','57']
  H_HDV['75_CO_6_0'] = [' 75','EURO2','RT >32t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.1558904040052','-0.043878151200976','-2.39819905110261','0.213275896644832','0','6','71']
  H_HDV['75_THC_6_0'] = [' 75','EURO2','RT >32t Euro-2','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.39579668696156','0.001817092697053','-0.324020373458135','0.081301332066375','0','6','71']
  H_HDV['75_NOx_6_0'] = [' 75','EURO2','RT >32t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','57.054230557399','-0.051103195978466','-3.86146450054875','0.119673497015386','0','6','71']
  H_HDV['75_PM_6_0'] = [' 75','EURO2','RT >32t Euro-2','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003936108296','0.000533631440729','-0.025794953238343','0.799618467984864','0','6','71']
  H_HDV['75_FC_6_0'] = [' 75','EURO2','RT >32t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','2346.46864639534','1.00997934664599','-0.468873524544497','0','0','6','71']
  H_HDV['75_CO_6_50'] = [' 75','EURO2','RT >32t Euro-2','6','50','CO','y=((a*(b**x))*(x**c))','20.7162485629401','0.987899752253687','-0.353251950347102','0','0','6','56']
  H_HDV['75_THC_6_50'] = [' 75','EURO2','RT >32t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.636059036287439','2.52078907625951','6.8349374451591','2.79808104388906','-0.024985083751512','6','56']
  H_HDV['75_NOx_6_50'] = [' 75','EURO2','RT >32t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','67.7730532782641','-0.059004519508463','-3.26534750708022','0.102897439360023','0','6','56']
  H_HDV['75_PM_6_50'] = [' 75','EURO2','RT >32t Euro-2','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.826446624416919','0.040343465803695','-0.000213015356764','0','0','6','56']
  H_HDV['75_FC_6_50'] = [' 75','EURO2','RT >32t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','419.072482388191','0.058843734274556','5031.29687708905','0.578217170661198','978.533720856176','6','56']
  H_HDV['75_CO_6_100'] = [' 75','EURO2','RT >32t Euro-2','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1691230180593','0.03805837563036','27.6912473550035','0.35360015413903','1.27047743381203','6','42']
  H_HDV['75_THC_6_100'] = [' 75','EURO2','RT >32t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.950840824593449','2.0766556963699','8.61311182337467','3.56091602274045','-0.043316204856114','6','42']
  H_HDV['75_NOx_6_100'] = [' 75','EURO2','RT >32t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000689059659117','0.067852889382586','-2.42494098500521','77.9187042406433','0','6','42']
  H_HDV['75_PM_6_100'] = [' 75','EURO2','RT >32t Euro-2','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.809017734384668','0.034088651414236','3.76859515065754','0.455903393290336','0.400403687659576','6','42']
  H_HDV['75_FC_6_100'] = [' 75','EURO2','RT >32t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','584.283272830643','0.015384091426733','9030.0093498666','0.633453383891612','1070.15222834247','6','42']
  H_HDV['76_CO_0_0'] = [' 76','EURO3','RT >32t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.88170500234846','0.05602888796342','9.81591134504472','0.201575821294511','1.16592716180769','6','86']
  H_HDV['76_THC_0_0'] = [' 76','EURO3','RT >32t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.11441976692114','3.36935948276456','3.48361683286339','1.58736512295087','-0.005899602937937','6','86']
  H_HDV['76_NOx_0_0'] = [' 76','EURO3','RT >32t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.8143312674791','0.057253705720838','48.2183010525698','0.281092900986335','5.56659564438318','6','86']
  H_HDV['76_PM_0_0'] = [' 76','EURO3','RT >32t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.061771158154741','1.02990096647556','4.65034427777758','1.93806266570778','-0.011530888045542','6','86']
  H_HDV['76_FC_0_0'] = [' 76','EURO3','RT >32t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000745651967175','0.00011736183817','-0.000000777497065','0','0','6','86']
  H_HDV['76_CO_0_50'] = [' 76','EURO3','RT >32t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.46566699835177','64.6249336181347','-0.78726282904029','0.517810676435971','0.042769758214732','6','86']
  H_HDV['76_THC_0_50'] = [' 76','EURO3','RT >32t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.117502782244348','3.23587537087023','3.53366861463299','1.60979209239876','-0.006137728833632','6','86']
  H_HDV['76_NOx_0_50'] = [' 76','EURO3','RT >32t Euro-3','0','50','NOx','y=((a*(b**x))*(x**c))','78.7881566657988','1.00395633065411','-0.63276371232106','0','0','6','86']
  H_HDV['76_PM_0_50'] = [' 76','EURO3','RT >32t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.589406790483251','0.050175180669207','0.825322160549505','0.182354516372347','0.118009749238376','6','86']
  H_HDV['76_FC_0_50'] = [' 76','EURO3','RT >32t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','558.802476693429','0.04448827807211','6143.31270582826','0.648773484026201','221.729043512311','6','86']
  H_HDV['76_CO_0_100'] = [' 76','EURO3','RT >32t Euro-3','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.74667274678146','64.5889706169961','-0.89896905572744','0.426246525407498','0.045502873262578','6','86']
  H_HDV['76_THC_0_100'] = [' 76','EURO3','RT >32t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.716814119065638','0.024993983235012','2.88340102669423','0.11872353095591','0.163936041267931','6','86']
  H_HDV['76_NOx_0_100'] = [' 76','EURO3','RT >32t Euro-3','0','100','NOx','y=((a*(b**x))*(x**c))','63.9046385343626','0.99962933034992','-0.479771504072663','0','0','6','86']
  H_HDV['76_PM_0_100'] = [' 76','EURO3','RT >32t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.614929500291963','0.047991199944931','0.923503567409793','0.234163279379938','0.134204047544285','6','86']
  H_HDV['76_FC_0_100'] = [' 76','EURO3','RT >32t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','601.632307109061','0.03355112987054','89962.4665008627','1.12109430540612','237.525134224615','6','86']
  H_HDV['76_CO_-2_0'] = [' 76','EURO3','RT >32t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.1955305579596','0.014601447196356','-0.762521492830865','0.063426182114309','0','6','86']
  H_HDV['76_THC_-2_0'] = [' 76','EURO3','RT >32t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.116673972900075','2.23539925065246','6.21567900794569','2.4125763259308','-0.016600855863596','6','86']
  H_HDV['76_NOx_-2_0'] = [' 76','EURO3','RT >32t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.3906304994631','0.063514042974214','112.949967748746','0.480860213330404','2.09922154906502','6','86']
  H_HDV['76_PM_-2_0'] = [' 76','EURO3','RT >32t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.106248568583125','21.7069091964511','-3.36221490380969','-0.354546505451591','0.099947680203606','6','86']
  H_HDV['76_FC_-2_0'] = [' 76','EURO3','RT >32t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','611.014115994955','0.059715406058277','6944.14515258011','0.695139213862268','70.3248548183748','6','86']
  H_HDV['76_CO_-2_50'] = [' 76','EURO3','RT >32t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.9129231684729','0.010985835856721','-0.72303011929377','0.061748763451407','0','6','86']
  H_HDV['76_THC_-2_50'] = [' 76','EURO3','RT >32t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033984484953234','2.29511037721672','5.07912171810864','1.99588635270333','-0.008824665119452','6','86']
  H_HDV['76_NOx_-2_50'] = [' 76','EURO3','RT >32t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.3758011763431','0.040506918744841','150.691255490689','0.491511834691834','1.03091231695412','6','86']
  H_HDV['76_PM_-2_50'] = [' 76','EURO3','RT >32t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04579789625706','0.821207610990479','7.03953551617317','2.63704529817458','-0.019297470030061','6','86']
  H_HDV['76_FC_-2_50'] = [' 76','EURO3','RT >32t Euro-3','-2','50','FC','y=((a*(b**x))*(x**c))','1394.76444516634','0.984701342148703','-0.448145283845379','0','0','6','86']
  H_HDV['76_CO_-2_100'] = [' 76','EURO3','RT >32t Euro-3','-2','100','CO','y=(c+(a*exp(b*x)))','11.1878003585183','-0.064795114318731','0.793095115795673','0','0','6','86']
  H_HDV['76_THC_-2_100'] = [' 76','EURO3','RT >32t Euro-3','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.87167839233497','-0.002647689705525','-0.221723171089825','0.086059559548062','0','6','86']
  H_HDV['76_NOx_-2_100'] = [' 76','EURO3','RT >32t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.9702717555028','0.030711149758748','280.23728358256','0.589853960271699','0.311772969144502','6','86']
  H_HDV['76_PM_-2_100'] = [' 76','EURO3','RT >32t Euro-3','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031783614109084','0.850168215169172','5.74181312468532','2.1567359034851','-0.01171080405667','6','86']
  H_HDV['76_FC_-2_100'] = [' 76','EURO3','RT >32t Euro-3','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-5.38477533932255','0.717175902536097','1353.46958771732','-0.465761262280531','0','6','86']
  H_HDV['76_CO_-4_0'] = [' 76','EURO3','RT >32t Euro-3','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.142751948431','-11.0883368646686','-1.84488444241292','0','0','6','86']
  H_HDV['76_THC_-4_0'] = [' 76','EURO3','RT >32t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.94240449918665','-0.002013944168783','-0.234218294724297','0.086438887186816','0','6','86']
  H_HDV['76_NOx_-4_0'] = [' 76','EURO3','RT >32t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.1241963371986','0.062166206001711','151.60009066441','0.508952307180135','0.560995366194535','6','86']
  H_HDV['76_PM_-4_0'] = [' 76','EURO3','RT >32t Euro-3','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.034072288098932','0.84868593070856','7.25085984362111','2.72138708744474','-0.018537885168198','6','86']
  H_HDV['76_FC_-4_0'] = [' 76','EURO3','RT >32t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','475.160747121622','0.05447915765141','4196.01282177196','0.554130110622886','17.6073386262125','6','86']
  H_HDV['76_CO_-4_50'] = [' 76','EURO3','RT >32t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.5449344426496','-0.00885190155044','-0.785742941488905','0.081241440749137','0','6','86']
  H_HDV['76_THC_-4_50'] = [' 76','EURO3','RT >32t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.5854481646755','-0.001827283319897','-0.20780251014167','0.087453702585007','0','6','86']
  H_HDV['76_NOx_-4_50'] = [' 76','EURO3','RT >32t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.0599063759272','0.05316957461135','372.770021742517','0.654454523074494','0.430803641102292','6','86']
  H_HDV['76_PM_-4_50'] = [' 76','EURO3','RT >32t Euro-3','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024433593219435','0.806711203224032','5.77483948004807','2.21174226020658','-0.011801167109212','6','86']
  H_HDV['76_FC_-4_50'] = [' 76','EURO3','RT >32t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','403.808424293921','0.045992098306669','11557.3308119034','0.712581403153719','10.7991387531949','6','86']
  H_HDV['76_CO_-4_100'] = [' 76','EURO3','RT >32t Euro-3','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.081959083739877','68.3830973456125','-1.28346208461537','0.438606117033205','0.032618785528295','6','86']
  H_HDV['76_THC_-4_100'] = [' 76','EURO3','RT >32t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.624674294278','0.062174566822778','7.26279328125879','0.514824744013665','0.080847587947879','6','86']
  H_HDV['76_NOx_-4_100'] = [' 76','EURO3','RT >32t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.58446098837965','0.045240277269895','695.657417744109','0.757285800513537','0.294766092168509','6','86']
  H_HDV['76_PM_-4_100'] = [' 76','EURO3','RT >32t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.12324052680332','-6.70489161825413','-1.34243491924574','0','0','6','86']
  H_HDV['76_FC_-4_100'] = [' 76','EURO3','RT >32t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','376.12681303799','0.039415806206845','21845.4660142364','0.818595870154713','4.01569080807504','6','86']
  H_HDV['76_CO_-6_0'] = [' 76','EURO3','RT >32t Euro-3','-6','0','CO','y=(c+(a*exp(b*x)))','9.68486460772323','-0.0752276550111','0.241896103986167','0','0','6','72']
  H_HDV['76_THC_-6_0'] = [' 76','EURO3','RT >32t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.05093226786411','0.072794980657448','2.07567654924644','0.377430002192343','0.084905023934732','6','72']
  H_HDV['76_NOx_-6_0'] = [' 76','EURO3','RT >32t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1565877737059','0.074683793230859','254.789391595666','0.605996381159536','0.392610083721405','6','72']
  H_HDV['76_PM_-6_0'] = [' 76','EURO3','RT >32t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.94983107967633','-8.69635539721386','-1.57118818845873','0','0','6','72']
  H_HDV['76_FC_-6_0'] = [' 76','EURO3','RT >32t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','457.797908852646','0.066331302130273','10326.5349699287','0.739900143332699','13.3064508809222','6','72']
  H_HDV['76_CO_-6_50'] = [' 76','EURO3','RT >32t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','7.79204064974205','-0.069951749743849','0.221135376170057','0','0','6','72']
  H_HDV['76_THC_-6_50'] = [' 76','EURO3','RT >32t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53043541858231','0.066859534605885','10.1730486337967','0.575556261444801','0.077306780750058','6','72']
  H_HDV['76_NOx_-6_50'] = [' 76','EURO3','RT >32t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.86491088906796','0.063563621720198','447.669047145081','0.686975457222683','0.337172448627872','6','72']
  H_HDV['76_PM_-6_50'] = [' 76','EURO3','RT >32t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024700969254705','0.713610313093923','5.43240964282081','2.07644045444005','-0.008321649166424','6','72']
  H_HDV['76_FC_-6_50'] = [' 76','EURO3','RT >32t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','356.011455540247','0.058732088345659','17285.9894524615','0.801322567051603','11.3460649764943','6','72']
  H_HDV['76_CO_-6_100'] = [' 76','EURO3','RT >32t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','6.92681646759056','-0.065452453467185','0.204587364143894','0','0','6','72']
  H_HDV['76_THC_-6_100'] = [' 76','EURO3','RT >32t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.32136050256162','0.064408393391136','12.9628003236046','0.611863740306685','0.074839526288828','6','72']
  H_HDV['76_NOx_-6_100'] = [' 76','EURO3','RT >32t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.56018249692219','0.058745206412355','568.602326315292','0.732641140059524','0.32316145169746','6','72']
  H_HDV['76_PM_-6_100'] = [' 76','EURO3','RT >32t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045736434243174','0.739005387474332','3.68229958225974','1.38137502229521','0.020086510501006','6','72']
  H_HDV['76_FC_-6_100'] = [' 76','EURO3','RT >32t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','322.092498908245','0.055069044021798','23876.5429155171','0.852698849226547','10.5958343341163','6','72']
  H_HDV['76_CO_2_0'] = [' 76','EURO3','RT >32t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.319015401815','0.005308587578939','-0.905584388707976','0.075763993436742','0','6','86']
  H_HDV['76_THC_2_0'] = [' 76','EURO3','RT >32t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.151627068015502','2.9260774826919','4.63748727052558','1.88575080846483','-0.008020738551777','6','86']
  H_HDV['76_NOx_2_0'] = [' 76','EURO3','RT >32t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','48.0726314062119','-0.01856514586146','-4.37496820762103','0.117547931650071','0','6','86']
  H_HDV['76_PM_2_0'] = [' 76','EURO3','RT >32t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.121696949780205','0.749100947969082','6.88817974696091','2.63150509316126','-0.018207423828518','6','86']
  H_HDV['76_FC_2_0'] = [' 76','EURO3','RT >32t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1173.43164929289','0.632267287604387','-71.2033398231872','0.0813113316549','0','6','86']
  H_HDV['76_CO_2_50'] = [' 76','EURO3','RT >32t Euro-3','2','50','CO','y=(c+(a*exp(b*x)))','11.8267143004817','-0.073576022469549','2.04278555365719','0','0','6','86']
  H_HDV['76_THC_2_50'] = [' 76','EURO3','RT >32t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.257940952758242','2.52222914831113','5.90601290322801','2.35348247719129','-0.014246964589059','6','86']
  H_HDV['76_NOx_2_50'] = [' 76','EURO3','RT >32t Euro-3','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','52.1363601749258','-0.053634123809065','-4.37233198282277','0.121583670967794','0','6','86']
  H_HDV['76_PM_2_50'] = [' 76','EURO3','RT >32t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.10015470718834','0.000594206584216','-0.06168567789588','0.065266764766968','0','6','86']
  H_HDV['76_FC_2_50'] = [' 76','EURO3','RT >32t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1233.19345068745','0.783729209982437','-53.7559687874104','0.068294136941669','0','6','86']
  H_HDV['76_CO_2_100'] = [' 76','EURO3','RT >32t Euro-3','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.051010449970965','0.006965104910071','-0.000009701938311','0','0','6','83']
  H_HDV['76_THC_2_100'] = [' 76','EURO3','RT >32t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.328842004742476','2.4200771485805','5.95311064840092','2.39492188453924','-0.014256405407644','6','83']
  H_HDV['76_NOx_2_100'] = [' 76','EURO3','RT >32t Euro-3','2','100','NOx','y=((a*(b**x))*(x**c))','83.3701154925905','1.00439535796304','-0.456945558894729','0','0','6','83']
  H_HDV['76_PM_2_100'] = [' 76','EURO3','RT >32t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.15675800219528','-0.000914377475202','-0.070040068443288','0.080266898821815','0','6','83']
  H_HDV['76_FC_2_100'] = [' 76','EURO3','RT >32t Euro-3','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00253299671254','0.467403905516586','-29.0243959185329','1255.9053746292','0','6','83']
  H_HDV['76_CO_4_0'] = [' 76','EURO3','RT >32t Euro-3','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.44972086486933','8.55422771228264','5.6359440143801','2.210691726998','-0.012915346902757','6','72']
  H_HDV['76_THC_4_0'] = [' 76','EURO3','RT >32t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.217492794888713','2.7280074324647','5.45837495439317','2.17834040419967','-0.012380990521204','6','72']
  H_HDV['76_NOx_4_0'] = [' 76','EURO3','RT >32t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.55320075384349','0.027321889240909','41.6685518231256','0.15586390019075','12.1785499047585','6','72']
  H_HDV['76_PM_4_0'] = [' 76','EURO3','RT >32t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.141276176032496','0.674815789498932','6.56196657052648','2.45119865472393','-0.017230549235848','6','72']
  H_HDV['76_FC_4_0'] = [' 76','EURO3','RT >32t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','2446.84492855133','1.00939367267921','-0.535567308882742','0','0','6','72']
  H_HDV['76_CO_4_50'] = [' 76','EURO3','RT >32t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.0471467735288','-0.042618631642586','-1.13469920744673','0.10535342829579','0','6','70']
  H_HDV['76_THC_4_50'] = [' 76','EURO3','RT >32t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.402128376095181','2.31570945219995','6.95124706857584','2.79012458540661','-0.022581242323997','6','70']
  H_HDV['76_NOx_4_50'] = [' 76','EURO3','RT >32t Euro-3','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','62.329386919235','-0.069217033342643','-5.03098126413791','0.1267695841612','0','6','70']
  H_HDV['76_PM_4_50'] = [' 76','EURO3','RT >32t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004626807705','0.000744801929224','-0.040077417748037','1.00557599627268','0','6','70']
  H_HDV['76_FC_4_50'] = [' 76','EURO3','RT >32t Euro-3','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','665.092020045194','0.081839212191175','32531.0602969575','1.01497433435312','746.099854108681','6','70']
  H_HDV['76_CO_4_100'] = [' 76','EURO3','RT >32t Euro-3','4','100','CO','y=((a*(b**x))*(x**c))','24.285154887184','0.98380248784617','-0.356161474679879','0','0','6','58']
  H_HDV['76_THC_4_100'] = [' 76','EURO3','RT >32t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.492977986781702','2.20960751119936','6.92461510469717','2.80040736700962','-0.024011013330911','6','58']
  H_HDV['76_NOx_4_100'] = [' 76','EURO3','RT >32t Euro-3','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','74.2793710698015','-0.115144019989859','-6.54547200104729','0.150540125760882','0','6','58']
  H_HDV['76_PM_4_100'] = [' 76','EURO3','RT >32t Euro-3','4','100','PM','y=(c+(a*exp(((-1)*b)*x)))','0.815436341542108','0.070282793780098','0.338815372956201','0','0','6','58']
  H_HDV['76_FC_4_100'] = [' 76','EURO3','RT >32t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','490.8122864789','0.037589253026298','8999.45675382097','0.673196274669244','925.554590073465','6','58']
  H_HDV['76_CO_6_0'] = [' 76','EURO3','RT >32t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.8955003583589','-0.038797454955462','-1.68367354208216','0.144368493996825','0','6','71']
  H_HDV['76_THC_6_0'] = [' 76','EURO3','RT >32t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.226123778661095','2.53786969893058','6.95352701169507','2.78560221816329','-0.034723798278793','6','71']
  H_HDV['76_NOx_6_0'] = [' 76','EURO3','RT >32t Euro-3','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','66.638010872692','-0.064204353920647','-7.23435579137274','0.156330683796163','0','6','71']
  H_HDV['76_PM_6_0'] = [' 76','EURO3','RT >32t Euro-3','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.194978184258698','0.623990317503546','6.8396628763884','2.56972610022142','-0.017358023459803','6','71']
  H_HDV['76_FC_6_0'] = [' 76','EURO3','RT >32t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','2391.68186343776','1.00891088141192','-0.455670296058212','0','0','6','71']
  H_HDV['76_CO_6_50'] = [' 76','EURO3','RT >32t Euro-3','6','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.059539860737538','0.005621559160588','0.0000149307113','0','0','6','57']
  H_HDV['76_THC_6_50'] = [' 76','EURO3','RT >32t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.472767787680571','2.30182144262619','6.62875635176958','2.68702732643449','-0.023169094108795','6','57']
  H_HDV['76_NOx_6_50'] = [' 76','EURO3','RT >32t Euro-3','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','72.3579009640412','-0.058145516376591','-5.6535141511455','0.12871408926849','0','6','57']
  H_HDV['76_PM_6_50'] = [' 76','EURO3','RT >32t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.357247708450875','2.87950462820972','-1.57579487222294','-0.419223493575159','0.124855264467075','6','57']
  H_HDV['76_FC_6_50'] = [' 76','EURO3','RT >32t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','528.692154104124','0.06458207221285','6474.66927584885','0.65986752283156','981.519090353664','6','57']
  H_HDV['76_CO_6_100'] = [' 76','EURO3','RT >32t Euro-3','6','100','CO','y=((a*(b**x))*(x**c))','28.9864583505424','0.982114794566803','-0.381591471456334','0','0','6','43']
  H_HDV['76_THC_6_100'] = [' 76','EURO3','RT >32t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.833167640718642','7.57486185475875','-1.37863754795154','-0.660678396448072','0.178586665092274','6','43']
  H_HDV['76_NOx_6_100'] = [' 76','EURO3','RT >32t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','94.935831036431','-0.22739497007005','-9.430176180606','0.179552749196124','0','6','43']
  H_HDV['76_PM_6_100'] = [' 76','EURO3','RT >32t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000015144615233','0.001632332564812','-0.059721003682942','1.29182333327622','0','6','43']
  H_HDV['76_FC_6_100'] = [' 76','EURO3','RT >32t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','452.060811725642','0.030085813859208','7057.02872124942','0.592882771601926','1259.80677103884','6','43']
  H_HDV['77_CO_0_0'] = [' 77','EURO4','RT >32t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.042568185018843','0.87357158483695','3.90889173313098','1.72434902998814','-0.010054230227099','6','86']
  H_HDV['77_THC_0_0'] = [' 77','EURO4','RT >32t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006057307198461','0.188464299290894','3.44542641623969','1.58057229235862','-0.003639687166936','6','86']
  H_HDV['77_NOx_0_0'] = [' 77','EURO4','RT >32t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.0949010377408','0.058252215856327','36.1582098168022','0.376061880271287','3.33879548612414','6','86']
  H_HDV['77_PM_0_0'] = [' 77','EURO4','RT >32t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.101811471134537','0.046230875481957','0.267788381153824','0.169975709290569','0.018610359474968','6','86']
  H_HDV['77_FC_0_0'] = [' 77','EURO4','RT >32t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','544.051003821164','0.060719671178934','3022.9022986952','0.516813353785748','174.447058269148','6','86']
  H_HDV['77_CO_0_50'] = [' 77','EURO4','RT >32t Euro-4','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.39226548055259','0.046214009863203','0.695373894632433','0.165687403536055','0.09410246491891','6','86']
  H_HDV['77_THC_0_50'] = [' 77','EURO4','RT >32t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004949972947205','0.179553316559401','3.65567768707241','1.67205596236876','-0.007631200580914','6','86']
  H_HDV['77_NOx_0_50'] = [' 77','EURO4','RT >32t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.34487523995241','0.042365812445461','55.4711901357312','0.465412829523263','3.9586646211259','6','86']
  H_HDV['77_PM_0_50'] = [' 77','EURO4','RT >32t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.107230534151585','0.046563552270138','0.2598184729771','0.179458112537614','0.019666954053233','6','86']
  H_HDV['77_FC_0_50'] = [' 77','EURO4','RT >32t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','525.170277174723','0.044652163645024','7791.27099199799','0.679732100948377','206.137274524351','6','86']
  H_HDV['77_CO_0_100'] = [' 77','EURO4','RT >32t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.451876802371412','0.048864902036775','0.573820143501615','0.173109635362277','0.10062841169992','6','86']
  H_HDV['77_THC_0_100'] = [' 77','EURO4','RT >32t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003784760966668','0.177568831122943','3.59607015976208','1.66496302554871','-0.009869103735994','6','86']
  H_HDV['77_NOx_0_100'] = [' 77','EURO4','RT >32t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.52081639469928','0.032562333417048','175.759638482337','0.685293760077933','4.14276963846571','6','86']
  H_HDV['77_PM_0_100'] = [' 77','EURO4','RT >32t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.113680322833437','0.047302594496216','0.244830535442778','0.183661579757922','0.020037746409882','6','86']
  H_HDV['77_FC_0_100'] = [' 77','EURO4','RT >32t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','557.714610099456','0.032804857611411','48115.4480196649','1.00743906900974','216.551216820554','6','86']
  H_HDV['77_CO_-2_0'] = [' 77','EURO4','RT >32t Euro-4','-2','0','CO','y=(c+(a*exp(((-1)*b)*x)))','0.826546817620033','0.074140451114087','0.063039515932397','0','0','6','86']
  H_HDV['77_THC_-2_0'] = [' 77','EURO4','RT >32t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010003541162247','0.123676034651863','5.56989305716469','2.08482851110953','0.007382625411934','6','86']
  H_HDV['77_NOx_-2_0'] = [' 77','EURO4','RT >32t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.91315111793942','0.057528100749408','100.992090422553','0.56987436855449','1.11032369188996','6','86']
  H_HDV['77_PM_-2_0'] = [' 77','EURO4','RT >32t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010627237201744','0.229148462453286','4.27956324721033','1.79538570766255','0.005831527169345','6','86']
  H_HDV['77_FC_-2_0'] = [' 77','EURO4','RT >32t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','571.424653980351','0.060431388559677','9815.15294521529','0.741988825368976','63.7912525208077','6','86']
  H_HDV['77_CO_-2_50'] = [' 77','EURO4','RT >32t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011487672788176','0.596236624870026','5.87605055659947','2.21186695495531','-0.011217153828335','6','86']
  H_HDV['77_THC_-2_50'] = [' 77','EURO4','RT >32t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000324447945968','0.128100057577818','5.38802426455162','2.13487059059772','-0.014996212460431','6','86']
  H_HDV['77_NOx_-2_50'] = [' 77','EURO4','RT >32t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.68299770360972','0.034822744445325','140.877766970146','0.593158326062925','0.425108339733715','6','86']
  H_HDV['77_PM_-2_50'] = [' 77','EURO4','RT >32t Euro-4','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.066072737027258','0.034079682247504','0.229703219088558','0.121423382700204','0.005814042754112','6','86']
  H_HDV['77_FC_-2_50'] = [' 77','EURO4','RT >32t Euro-4','-2','50','FC','y=((a*(b**x))*(x**c))','1323.74549645247','0.983858776923888','-0.454565210848704','0','0','6','86']
  H_HDV['77_CO_-2_100'] = [' 77','EURO4','RT >32t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.773589279911238','-0.000804062086252','-0.054095483200656','0.078585842845718','0','6','86']
  H_HDV['77_THC_-2_100'] = [' 77','EURO4','RT >32t Euro-4','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.027496417072063','0.022271903625937','0.137248571740036','0.095653012163391','0.001588524099687','6','86']
  H_HDV['77_NOx_-2_100'] = [' 77','EURO4','RT >32t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.19906359884858','0.025949582138763','298.850349840046','0.718703564922875','-0.204622457135978','6','86']
  H_HDV['77_PM_-2_100'] = [' 77','EURO4','RT >32t Euro-4','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.39094323745476','0.539428908108056','0.011091314159766','0','0','6','86']
  H_HDV['77_FC_-2_100'] = [' 77','EURO4','RT >32t Euro-4','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-4.60204637001482','0.731430462656556','1286.89508728578','-0.482323859273642','0','6','86']
  H_HDV['77_CO_-4_0'] = [' 77','EURO4','RT >32t Euro-4','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000952030231181','0.55951500986246','6.16907635320886','2.32708702582362','-0.009931379624458','6','86']
  H_HDV['77_THC_-4_0'] = [' 77','EURO4','RT >32t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.164758427679864','-0.000107395191725','-0.013057095122039','0.085489713627017','0','6','86']
  H_HDV['77_NOx_-4_0'] = [' 77','EURO4','RT >32t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.89044305190328','0.05350447689863','111.350988586166','0.561422974762152','0.162971496683791','6','86']
  H_HDV['77_PM_-4_0'] = [' 77','EURO4','RT >32t Euro-4','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.88189922347743','0.402468367769599','0.021202028574396','0','0','6','86']
  H_HDV['77_FC_-4_0'] = [' 77','EURO4','RT >32t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','447.803344086622','0.056185138571916','6515.58415170381','0.629143818229104','11.5758393305202','6','86']
  H_HDV['77_CO_-4_50'] = [' 77','EURO4','RT >32t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.660335858395658','-0.000443355135964','-0.049115638347805','0.079627478021899','0','6','86']
  H_HDV['77_THC_-4_50'] = [' 77','EURO4','RT >32t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.146185374829963','-0.000106153708394','-0.011830374782124','0.087668898433395','0','6','86']
  H_HDV['77_NOx_-4_50'] = [' 77','EURO4','RT >32t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.8826818259957','0.044522142390603','212.937972220869','0.665557996524487','0.040875973983277','6','86']
  H_HDV['77_PM_-4_50'] = [' 77','EURO4','RT >32t Euro-4','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.137894923234399','0.060549503060143','0.435123370979872','0.350413868261486','0.0048084656694','6','86']
  H_HDV['77_FC_-4_50'] = [' 77','EURO4','RT >32t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','368.365919848305','0.046748549460422','16248.4460894163','0.768718869587989','4.51257679908403','6','86']
  H_HDV['77_CO_-4_100'] = [' 77','EURO4','RT >32t Euro-4','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006214102924683','5.0612192543433','-1.54983793751947','0.409107519168498','0.038902261572413','6','86']
  H_HDV['77_THC_-4_100'] = [' 77','EURO4','RT >32t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.091943409244444','0.061546151917058','0.437577507017831','0.519016100075378','0.003753457991008','6','86']
  H_HDV['77_NOx_-4_100'] = [' 77','EURO4','RT >32t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.59317910259745','0.038585809553339','355.32638781586','0.753868458283874','-0.061993662939906','6','86']
  H_HDV['77_PM_-4_100'] = [' 77','EURO4','RT >32t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.127900739582059','0.058974309666232','0.772840212628182','0.448813711139498','0.004477077351657','6','86']
  H_HDV['77_FC_-4_100'] = [' 77','EURO4','RT >32t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','337.324855025366','0.040165318124894','32595.6244486714','0.883622987808991','-1.2054343772674','6','86']
  H_HDV['77_CO_-6_0'] = [' 77','EURO4','RT >32t Euro-4','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.631008700885692','0.000389562110886','-0.044171398573716','0.068930928310024','0','6','72']
  H_HDV['77_THC_-6_0'] = [' 77','EURO4','RT >32t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.120311042579919','0.073007646086287','0.179700719418355','0.48227375887981','0.003895639028666','6','72']
  H_HDV['77_NOx_-6_0'] = [' 77','EURO4','RT >32t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.61991552605815','0.066801918573187','181.29049268257','0.666496058048712','0.057749316147027','6','72']
  H_HDV['77_PM_-6_0'] = [' 77','EURO4','RT >32t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.167609915358921','0.072502193035016','0.415922747225454','0.397503182110869','0.004739739005944','6','72']
  H_HDV['77_FC_-6_0'] = [' 77','EURO4','RT >32t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','412.110985181767','0.066891771418294','14463.2978838863','0.784010368217901','4.70211661630671','6','72']
  H_HDV['77_CO_-6_50'] = [' 77','EURO4','RT >32t Euro-4','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006451864455635','2.66267810796162','-1.04922712809469','0.338207238187917','0.052249575332701','6','72']
  H_HDV['77_THC_-6_50'] = [' 77','EURO4','RT >32t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089649766556559','0.066544806732894','0.811794071650255','0.644409262393467','0.00342868133803','6','72']
  H_HDV['77_NOx_-6_50'] = [' 77','EURO4','RT >32t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.25437445850242','0.060326986680452','276.268877268034','0.729546896195036','0.035964690160427','6','72']
  H_HDV['77_PM_-6_50'] = [' 77','EURO4','RT >32t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.133607756807427','0.067520109399998','1.07248286823405','0.52828121680439','0.004251665704798','6','72']
  H_HDV['77_FC_-6_50'] = [' 77','EURO4','RT >32t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','308.766426205581','0.058886475607136','23738.0284737605','0.846295699932004','2.88852547336052','6','72']
  H_HDV['77_CO_-6_100'] = [' 77','EURO4','RT >32t Euro-4','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005178939106873','1.91628860449544','-0.785234972985386','0.378749956381442','0.04727354982446','6','72']
  H_HDV['77_THC_-6_100'] = [' 77','EURO4','RT >32t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.076735320882182','0.063380629002758','0.818940556869107','0.636893704324861','0.003238033864683','6','72']
  H_HDV['77_NOx_-6_100'] = [' 77','EURO4','RT >32t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.77541024526055','0.057002919872395','345.189989525449','0.769896440821866','0.029046963446853','6','72']
  H_HDV['77_PM_-6_100'] = [' 77','EURO4','RT >32t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.120572958653726','0.065497900442459','1.35881695886598','0.569247186278261','0.004086747478577','6','72']
  H_HDV['77_FC_-6_100'] = [' 77','EURO4','RT >32t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','269.618351270684','0.054574339039003','27998.4317248361','0.875252039953312','2.11719497759963','6','72']
  H_HDV['77_CO_2_0'] = [' 77','EURO4','RT >32t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.130215946803595','0.598309505289341','7.47027310736179','2.92189228625153','-0.018585687806069','6','86']
  H_HDV['77_THC_2_0'] = [' 77','EURO4','RT >32t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005614215525103','0.163749110685995','4.62930712701975','1.89600946365373','-0.008345892138243','6','86']
  H_HDV['77_NOx_2_0'] = [' 77','EURO4','RT >32t Euro-4','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.1879171699177','0.00328919463827','-1.82077174096901','0.106030553164091','0','6','86']
  H_HDV['77_PM_2_0'] = [' 77','EURO4','RT >32t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061035005879562','0.029131730412681','0.271379370238703','0.13109154754201','0.02090650778608','6','86']
  H_HDV['77_FC_2_0'] = [' 77','EURO4','RT >32t Euro-4','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.280849377612837','1.38916683093906','2157.6755891158','-0.548512775753992','0','6','86']
  H_HDV['77_CO_2_50'] = [' 77','EURO4','RT >32t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.22979656882204','-0.001926734253098','-0.123869944873995','0.130641434785186','0','6','86']
  H_HDV['77_THC_2_50'] = [' 77','EURO4','RT >32t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009440756883888','0.142678344833308','5.88750519479149','2.36526062891929','-0.01593830519868','6','86']
  H_HDV['77_NOx_2_50'] = [' 77','EURO4','RT >32t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.1223883578662','-0.029243727973879','-1.97613748644519','0.124263867667433','0','6','86']
  H_HDV['77_PM_2_50'] = [' 77','EURO4','RT >32t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.073714823502814','0.023809200197031','0.274626703591259','0.140697463818584','0.016867521740128','6','86']
  H_HDV['77_FC_2_50'] = [' 77','EURO4','RT >32t Euro-4','2','50','FC','y=((a*(b**x))*(x**c))','2057.160496744','1.00664102692359','-0.460726080866267','0','0','6','86']
  H_HDV['77_CO_2_100'] = [' 77','EURO4','RT >32t Euro-4','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.432588649656778','0.015333123446486','0.811036737517398','0.141990953121737','0.001276638339378','6','84']
  H_HDV['77_THC_2_100'] = [' 77','EURO4','RT >32t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012299405293383','0.139524184539485','5.80164760545772','2.36028643117749','-0.014110531230745','6','84']
  H_HDV['77_NOx_2_100'] = [' 77','EURO4','RT >32t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.77467732525346','0.038935279803548','54.8190430489087','0.475906963174968','9.79724816567335','6','84']
  H_HDV['77_PM_2_100'] = [' 77','EURO4','RT >32t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.090316250211296','0.026925039087886','0.269724835573679','0.148214159413957','0.016659434391637','6','84']
  H_HDV['77_FC_2_100'] = [' 77','EURO4','RT >32t Euro-4','2','100','FC','y=(c+(a*exp(b*x)))','665.711221925643','-0.065968064074548','595.12842478956','0','0','6','84']
  H_HDV['77_CO_4_0'] = [' 77','EURO4','RT >32t Euro-4','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.207478484201648','2.15044630058296','-1.10358473472039','-0.40496589144587','0.143209949787789','6','72']
  H_HDV['77_THC_4_0'] = [' 77','EURO4','RT >32t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006584459753356','0.157194373424354','5.12949645323829','2.06966727870122','-0.011026443461085','6','72']
  H_HDV['77_NOx_4_0'] = [' 77','EURO4','RT >32t Euro-4','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.79656451685442','0.087657759688643','16.221966767423','0.229470090212684','8.78937217850704','6','72']
  H_HDV['77_PM_4_0'] = [' 77','EURO4','RT >32t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.07635859195228','0.032989324331083','0.254899856986195','0.141971611073022','0.026356875028159','6','72']
  H_HDV['77_FC_4_0'] = [' 77','EURO4','RT >32t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','2357.30630740523','1.00943275930173','-0.543606182233854','0','0','6','72']
  H_HDV['77_CO_4_50'] = [' 77','EURO4','RT >32t Euro-4','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.616710033292736','0.011597922546863','0.801345998762715','0.159292946872144','-0.125969404478369','6','70']
  H_HDV['77_THC_4_50'] = [' 77','EURO4','RT >32t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014489694921672','0.135274150921086','6.37596859265957','2.58571941622595','-0.018632795212406','6','70']
  H_HDV['77_NOx_4_50'] = [' 77','EURO4','RT >32t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.6044132528662','0.020641763981264','21.5208932927671','0.222326447740726','10.0370048416051','6','70']
  H_HDV['77_PM_4_50'] = [' 77','EURO4','RT >32t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.104692253823833','0.030773829622634','0.265405672124833','0.159702790453896','0.020630777222702','6','70']
  H_HDV['77_FC_4_50'] = [' 77','EURO4','RT >32t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','651.326439353692','0.086330839990561','25410.8011830942','0.973409872880343','703.982024203502','6','70']
  H_HDV['77_CO_4_100'] = [' 77','EURO4','RT >32t Euro-4','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.25591032450634','-4.42093408004259','-0.951819939932105','0','0','6','59']
  H_HDV['77_THC_4_100'] = [' 77','EURO4','RT >32t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017890586069272','0.130027141417362','6.36073416751594','2.61249935562343','-0.020319193981025','6','59']
  H_HDV['77_NOx_4_100'] = [' 77','EURO4','RT >32t Euro-4','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','38.976869198325','-0.086303815123818','-3.20449169371416','0.170998080056483','0','6','59']
  H_HDV['77_PM_4_100'] = [' 77','EURO4','RT >32t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.159449081947412','0.055129840826042','0.200442205114643','0.167421027278119','0.033652809706836','6','59']
  H_HDV['77_FC_4_100'] = [' 77','EURO4','RT >32t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','470.052813682906','0.047843445103632','8981.29450981318','0.696728001457068','904.561920646522','6','59']
  H_HDV['77_CO_6_0'] = [' 77','EURO4','RT >32t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.64699753581131','-0.00361232925564','-0.273705352860835','0.224228172585429','0','6','71']
  H_HDV['77_THC_6_0'] = [' 77','EURO4','RT >32t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005243452947482','0.144863117722284','6.62592090117449','2.66332278339268','-0.032577928286155','6','71']
  H_HDV['77_NOx_6_0'] = [' 77','EURO4','RT >32t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.3078743042359','-0.039529118965741','-5.80071113000545','0.214710025742314','0','6','71']
  H_HDV['77_PM_6_0'] = [' 77','EURO4','RT >32t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.410557036814996','-0.000781647606334','-0.052810070652831','0.160576950664315','0','6','71']
  H_HDV['77_FC_6_0'] = [' 77','EURO4','RT >32t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','2343.84661124329','1.00941641656107','-0.473579424511459','0','0','6','71']
  H_HDV['77_CO_6_50'] = [' 77','EURO4','RT >32t Euro-4','6','50','CO','y=(1/(a+(b*x)))','0.751537294774408','0.073082405883472','0','0','0','6','58']
  H_HDV['77_THC_6_50'] = [' 77','EURO4','RT >32t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017361165120429','0.129986229967116','6.50638009628962','2.66189862564883','-0.022481537252171','6','58']
  H_HDV['77_NOx_6_50'] = [' 77','EURO4','RT >32t Euro-4','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.2986688559923','1.44862102045903','-0.133482040217439','0','0','6','58']
  H_HDV['77_PM_6_50'] = [' 77','EURO4','RT >32t Euro-4','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.11462285559833','0.036476059990154','0.249333854059945','0.142400070924565','0.025447371188635','6','58']
  H_HDV['77_FC_6_50'] = [' 77','EURO4','RT >32t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','547.081115483128','0.075222900716206','7556.10236381331','0.710122795523976','935.398815584142','6','58']
  H_HDV['77_CO_6_100'] = [' 77','EURO4','RT >32t Euro-4','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.51342255608881','-4.1032000362841','-1.01988787069979','0','0','6','44']
  H_HDV['77_THC_6_100'] = [' 77','EURO4','RT >32t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.02568672676988','0.115454618700837','7.48975369028332','3.11460244007635','-0.034809970550837','6','44']
  H_HDV['77_NOx_6_100'] = [' 77','EURO4','RT >32t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9237586941948','0.016648030054231','29.2439431538976','0.269134423654481','13.8521746619453','6','44']
  H_HDV['77_PM_6_100'] = [' 77','EURO4','RT >32t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.048569028532022','0.247828794438918','5.08605262998633','2.09457935693973','0.009444008890366','6','44']
  H_HDV['77_FC_6_100'] = [' 77','EURO4','RT >32t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','385.52821194736','0.035347046572209','4942.52921965298','0.535983929906189','1226.53724627298','6','44']
  H_HDV['78_CO_0_0'] = [' 78','EURO5','RT >32t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045872025064912','0.890642012850063','3.88159374323664','1.71161307827859','-0.00871217115834','6','86']
  H_HDV['78_THC_0_0'] = [' 78','EURO5','RT >32t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006187080375082','0.191584082860284','3.45923613320844','1.58311799051363','-0.003671782289655','6','86']
  H_HDV['78_NOx_0_0'] = [' 78','EURO5','RT >32t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.91641967607817','0.057531068164657','23.8518296906587','0.354860829267291','1.9729605235183','6','86']
  H_HDV['78_PM_0_0'] = [' 78','EURO5','RT >32t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.103384772695268','0.045996789970115','0.273078152684603','0.169306578984312','0.018769464464009','6','86']
  H_HDV['78_FC_0_0'] = [' 78','EURO5','RT >32t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','558.076173655174','0.060358039048685','2946.18581665996','0.503652474933839','176.943297890326','6','86']
  H_HDV['78_CO_0_50'] = [' 78','EURO5','RT >32t Euro-5','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.398609082421774','0.045956391068925','0.714487770473593','0.165590591334624','0.094507750159165','6','86']
  H_HDV['78_THC_0_50'] = [' 78','EURO5','RT >32t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005359925133499','0.18333614604928','3.64830648939257','1.6689269464628','-0.007042535205607','6','86']
  H_HDV['78_NOx_0_50'] = [' 78','EURO5','RT >32t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.15201121957925','0.043144863002622','32.2625797149164','0.422801005273978','2.3320359611849','6','86']
  H_HDV['78_PM_0_50'] = [' 78','EURO5','RT >32t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108946798598211','0.046434500882608','0.26639448823254','0.179471880705797','0.01989300853329','6','86']
  H_HDV['78_FC_0_50'] = [' 78','EURO5','RT >32t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','539.708179118979','0.044648473330761','7150.30541215766','0.65756668822631','208.815334251996','6','86']
  H_HDV['78_CO_0_100'] = [' 78','EURO5','RT >32t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.449406198262125','0.048100146688883','0.590774739215275','0.169289050149496','0.101324585857961','6','86']
  H_HDV['78_THC_0_100'] = [' 78','EURO5','RT >32t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004046906765391','0.180990569088985','3.59590091034112','1.66364100539607','-0.009466400964546','6','86']
  H_HDV['78_NOx_0_100'] = [' 78','EURO5','RT >32t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.97150378092326','0.035786359135027','62.4668683592929','0.565442940441476','2.4766691165481','6','86']
  H_HDV['78_PM_0_100'] = [' 78','EURO5','RT >32t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.114785697785955','0.046948683827098','0.249974607346518','0.182342906784529','0.020229195169523','6','86']
  H_HDV['78_FC_0_100'] = [' 78','EURO5','RT >32t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','572.030070954285','0.032930646569444','37047.239831318','0.954269932262035','219.368863130809','6','86']
  H_HDV['78_CO_-2_0'] = [' 78','EURO5','RT >32t Euro-5','-2','0','CO','y=(c+(a*exp(b*x)))','0.843608238431612','-0.07394915804271','0.064116416311189','0','0','6','86']
  H_HDV['78_THC_-2_0'] = [' 78','EURO5','RT >32t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010092917497356','0.125826630451649','5.6520682104069','2.11680810594766','0.005985439293065','6','86']
  H_HDV['78_NOx_-2_0'] = [' 78','EURO5','RT >32t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.44715122102758','0.057273620927639','55.5144706029484','0.515707922247736','0.653731628870582','6','86']
  H_HDV['78_PM_-2_0'] = [' 78','EURO5','RT >32t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010545715022212','0.231630741443071','4.37556519301411','1.83096119899854','0.004207368300186','6','86']
  H_HDV['78_FC_-2_0'] = [' 78','EURO5','RT >32t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','589.666296810105','0.06039225146031','9158.89037156296','0.724458845164668','64.9601136214214','6','86']
  H_HDV['78_CO_-2_50'] = [' 78','EURO5','RT >32t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011672202892092','0.606749180497727','5.9273224874379','2.22887446833373','-0.011490039528711','6','86']
  H_HDV['78_THC_-2_50'] = [' 78','EURO5','RT >32t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000329728587309','0.130455819232958','5.41940152065764','2.14334169845356','-0.015094449563915','6','86']
  H_HDV['78_NOx_-2_50'] = [' 78','EURO5','RT >32t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.70357132937718','0.035495416707364','78.7001678300807','0.548721105792032','0.257076832254946','6','86']
  H_HDV['78_PM_-2_50'] = [' 78','EURO5','RT >32t Euro-5','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065815912415837','0.033477613733502','0.235109808918583','0.120155147147895','0.005779698934715','6','86']
  H_HDV['78_FC_-2_50'] = [' 78','EURO5','RT >32t Euro-5','-2','50','FC','y=((a*(b**x))*(x**c))','1380.54616303846','0.983871264394553','-0.460377388677695','0','0','6','86']
  H_HDV['78_CO_-2_100'] = [' 78','EURO5','RT >32t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.789939938706988','-0.000816150920531','-0.05530118216539','0.078610945235011','0','6','86']
  H_HDV['78_THC_-2_100'] = [' 78','EURO5','RT >32t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.027740451698389','0.021809699778243','0.14066255076806','0.095212117112304','0.001483835024157','6','86']
  H_HDV['78_NOx_-2_100'] = [' 78','EURO5','RT >32t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.98768698803739','0.027431204133221','150.006890058241','0.657550845866663','-0.067687556923465','6','86']
  H_HDV['78_PM_-2_100'] = [' 78','EURO5','RT >32t Euro-5','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.34491366421938','0.52705931234581','0.010961529333805','0','0','6','86']
  H_HDV['78_FC_-2_100'] = [' 78','EURO5','RT >32t Euro-5','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-4.75227869633413','0.726764072838748','1342.81970845847','-0.489025398981822','0','6','86']
  H_HDV['78_CO_-4_0'] = [' 78','EURO5','RT >32t Euro-5','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.001079337562139','0.572669919898929','6.17899214743343','2.33104488021082','-0.010053013644692','6','86']
  H_HDV['78_THC_-4_0'] = [' 78','EURO5','RT >32t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.168515483278971','-0.000110065711569','-0.013349552652027','0.085454222656734','0','6','86']
  H_HDV['78_NOx_-4_0'] = [' 78','EURO5','RT >32t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.31191556663072','0.054260846220721','70.2020015126443','0.538316532640132','0.098552877835346','6','86']
  H_HDV['78_PM_-4_0'] = [' 78','EURO5','RT >32t Euro-5','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.83325766547016','0.390790329166064','0.020845834447398','0','0','6','86']
  H_HDV['78_FC_-4_0'] = [' 78','EURO5','RT >32t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','463.998957823224','0.056409846253997','6580.50320065361','0.625607992730066','11.9401806077788','6','86']
  H_HDV['78_CO_-4_50'] = [' 78','EURO5','RT >32t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.67672806693862','-0.000449779459966','-0.050390506420093','0.079666799165849','0','6','86']
  H_HDV['78_THC_-4_50'] = [' 78','EURO5','RT >32t Euro-5','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.149654626355602','-0.000108405002754','-0.012111579446196','0.087650161150829','0','6','86']
  H_HDV['78_NOx_-4_50'] = [' 78','EURO5','RT >32t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.69843101863031','0.045808305279509','123.46833779418','0.629833612174298','0.030385044904648','6','86']
  H_HDV['78_PM_-4_50'] = [' 78','EURO5','RT >32t Euro-5','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.140933470101776','0.060606789340802','0.433690108756492','0.346207090835734','0.004902849238429','6','86']
  H_HDV['78_FC_-4_50'] = [' 78','EURO5','RT >32t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','381.638099807407','0.047126962383696','16069.7596448279','0.761434316646406','4.84041838215833','6','86']
  H_HDV['78_CO_-4_100'] = [' 78','EURO5','RT >32t Euro-5','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006693636098','2.08225497336408','-0.386326119533715','0.481968490760179','0.038507111387424','6','86']
  H_HDV['78_THC_-4_100'] = [' 78','EURO5','RT >32t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094389874908778','0.061695254458286','0.429144311639475','0.514052552071568','0.003833427300288','6','86']
  H_HDV['78_NOx_-4_100'] = [' 78','EURO5','RT >32t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.51134182966364','0.040178104839196','188.671997209913','0.703456404740725','-0.025059550116049','6','86']
  H_HDV['78_PM_-4_100'] = [' 78','EURO5','RT >32t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.130803192119413','0.059097012935287','0.762117120342953','0.443356094413467','0.004571265703157','6','86']
  H_HDV['78_FC_-4_100'] = [' 78','EURO5','RT >32t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.671146832037','0.040594169049798','32063.121191078','0.875746637262981','-0.833569147738967','6','86']
  H_HDV['78_CO_-6_0'] = [' 78','EURO5','RT >32t Euro-5','-6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.646844303227755','0.000400400085638','-0.045319765298331','0.068981250677243','0','6','72']
  H_HDV['78_THC_-6_0'] = [' 78','EURO5','RT >32t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.123423344406888','0.073123322571391','0.1694279306517','0.472071163865228','0.003976133495355','6','72']
  H_HDV['78_NOx_-6_0'] = [' 78','EURO5','RT >32t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.21607485317431','0.067592353987872','109.244295720234','0.634787740259218','0.035579777907181','6','72']
  H_HDV['78_PM_-6_0'] = [' 78','EURO5','RT >32t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.171625137881126','0.072588786992809','0.408036625701448','0.391768409754378','0.004837622826705','6','72']
  H_HDV['78_FC_-6_0'] = [' 78','EURO5','RT >32t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','428.115265160309','0.06716369508852','14434.4443899099','0.778865300883074','4.90702953395377','6','72']
  H_HDV['78_CO_-6_50'] = [' 78','EURO5','RT >32t Euro-5','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006763996687401','2.57853231367977','-0.985778576783093','0.335857900204922','0.052907349392415','6','72']
  H_HDV['78_THC_-6_50'] = [' 78','EURO5','RT >32t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.092133477594622','0.066767515441029','0.809501603236989','0.642484490086501','0.003506953956635','6','72']
  H_HDV['78_NOx_-6_50'] = [' 78','EURO5','RT >32t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.36130502482563','0.061185274267558','155.250444137','0.687445283893044','0.021696657935292','6','72']
  H_HDV['78_PM_-6_50'] = [' 78','EURO5','RT >32t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.137072363775871','0.067724322992444','1.06621657281556','0.524917228155262','0.004349162583189','6','72']
  H_HDV['78_FC_-6_50'] = [' 78','EURO5','RT >32t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','320.598403980819','0.059222424138437','23916.2066632999','0.842559940231188','3.06277581869544','6','72']
  H_HDV['78_CO_-6_100'] = [' 78','EURO5','RT >32t Euro-5','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005266505828024','1.99433071992673','-0.80151334595988','0.380376336365009','0.047225675553492','6','72']
  H_HDV['78_THC_-6_100'] = [' 78','EURO5','RT >32t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079709064200129','0.064070327392433','1.0379586687997','0.674621993250204','0.00334746317827','6','72']
  H_HDV['78_NOx_-6_100'] = [' 78','EURO5','RT >32t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.09238853970087','0.058375003956328','194.424830599209','0.728835028345676','0.018705992032102','6','72']
  H_HDV['78_PM_-6_100'] = [' 78','EURO5','RT >32t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.123118036699821','0.065530495502609','1.36435440554597','0.566072879256414','0.00416850770452','6','72']
  H_HDV['78_FC_-6_100'] = [' 78','EURO5','RT >32t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','281.80515270986','0.055183246983393','32484.2817335008','0.893809400601617','2.35408424246582','6','72']
  H_HDV['78_CO_2_0'] = [' 78','EURO5','RT >32t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.128432721577306','0.618069447472298','7.29945087503315','2.85853863094229','-0.018153599484778','6','86']
  H_HDV['78_THC_2_0'] = [' 78','EURO5','RT >32t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005662588361173','0.167411474841741','4.60663427670873','1.88770367549462','-0.0082469535241','6','86']
  H_HDV['78_NOx_2_0'] = [' 78','EURO5','RT >32t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2424417924493','0.111595931719646','17355.7172516826','1.76016527678422','3.68274759957088','6','86']
  H_HDV['78_PM_2_0'] = [' 78','EURO5','RT >32t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061804600903118','0.028598087989814','0.277349138311105','0.130731268169437','0.020791777199568','6','86']
  H_HDV['78_FC_2_0'] = [' 78','EURO5','RT >32t Euro-5','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.286973056900968','1.38655908276079','2239.04997630084','-0.553609412772996','0','6','86']
  H_HDV['78_CO_2_50'] = [' 78','EURO5','RT >32t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.24712731321524','-0.001875817522933','-0.124790559991265','0.129235055868049','0','6','86']
  H_HDV['78_THC_2_50'] = [' 78','EURO5','RT >32t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009323652779014','0.146136847353458','5.84928618488771','2.34962460169207','-0.015862534455935','6','86']
  H_HDV['78_NOx_2_50'] = [' 78','EURO5','RT >32t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.5698123078645','-0.024114088674478','-1.84140883710756','0.150434222794169','0','6','86']
  H_HDV['78_PM_2_50'] = [' 78','EURO5','RT >32t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.074122763184735','0.023845966145938','0.279366075990875','0.13978865731208','0.017356064198986','6','86']
  H_HDV['78_FC_2_50'] = [' 78','EURO5','RT >32t Euro-5','2','50','FC','y=((a*(b**x))*(x**c))','2132.74812684757','1.00666901698377','-0.46665320243237','0','0','6','86']
  H_HDV['78_CO_2_100'] = [' 78','EURO5','RT >32t Euro-5','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.435000089327916','0.015279659260331','0.827192126755215','0.141855497784506','0.002894625429904','6','84']
  H_HDV['78_THC_2_100'] = [' 78','EURO5','RT >32t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01249416619733','0.142095764935915','5.8290612968184','2.36875824948918','-0.014169431022023','6','84']
  H_HDV['78_NOx_2_100'] = [' 78','EURO5','RT >32t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.23944709147414','0.029960341385698','23.240986697817','0.349370135043341','4.91160704694395','6','84']
  H_HDV['78_PM_2_100'] = [' 78','EURO5','RT >32t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.091636182472443','0.027272011397946','0.275457134972696','0.149082832634045','0.017466096988946','6','84']
  H_HDV['78_FC_2_100'] = [' 78','EURO5','RT >32t Euro-5','2','100','FC','y=(c+(a*exp(((-1)*b)*x)))','692.465364889165','0.066870759941931','604.47586166841','0','0','6','84']
  H_HDV['78_CO_4_0'] = [' 78','EURO5','RT >32t Euro-5','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.210299287458391','2.0270284675641','-1.07670741079947','-0.469105519679946','0.150298840212621','6','72']
  H_HDV['78_THC_4_0'] = [' 78','EURO5','RT >32t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006684478230215','0.160278843090683','5.13680024418073','2.07116590841713','-0.011055242340714','6','72']
  H_HDV['78_NOx_4_0'] = [' 78','EURO5','RT >32t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.16936226044048','0.064426894709445','14.2472974252005','0.205490621032229','5.16580076406002','6','72']
  H_HDV['78_PM_4_0'] = [' 78','EURO5','RT >32t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.085825560602974','0.038510967731907','0.251811751197628','0.147193546166174','0.029120215531262','6','72']
  H_HDV['78_FC_4_0'] = [' 78','EURO5','RT >32t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','826.448388736943','0.097016740004867','13210.555273095','0.909572391838963','452.960363870713','6','72']
  H_HDV['78_CO_4_50'] = [' 78','EURO5','RT >32t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.26307331813642','-0.003983868447915','-0.119957971828944','0.138645841221221','0','6','70']
  H_HDV['78_THC_4_50'] = [' 78','EURO5','RT >32t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.0144789934081','0.138050686685904','6.40189152034373','2.59173630048541','-0.018822252913349','6','70']
  H_HDV['78_NOx_4_50'] = [' 78','EURO5','RT >32t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.50511185215404','0.015304102959486','16.6289230774311','0.235904159615398','4.14635036558581','6','70']
  H_HDV['78_PM_4_50'] = [' 78','EURO5','RT >32t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.105833771449138','0.029557392700419','0.272626189712928','0.159955899324758','0.019771424036897','6','70']
  H_HDV['78_FC_4_50'] = [' 78','EURO5','RT >32t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','680.181219060188','0.086425288691604','27161.6352298239','0.982477520331854','713.257492057751','6','70']
  H_HDV['78_CO_4_100'] = [' 78','EURO5','RT >32t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.580116392118671','0.025206759253546','0.684165544475113','0.136395248852765','0.050226669776714','6','61']
  H_HDV['78_THC_4_100'] = [' 78','EURO5','RT >32t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018297170457375','0.132427808657208','6.39445483101102','2.62591448406877','-0.020248238593874','6','61']
  H_HDV['78_NOx_4_100'] = [' 78','EURO5','RT >32t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.03834084341489','0.019353097951277','15.2285554430583','0.228911918909672','5.74800487715772','6','61']
  H_HDV['78_PM_4_100'] = [' 78','EURO5','RT >32t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.147100129144221','0.052923219478845','0.211544746937673','0.155913641006016','0.034670966519673','6','61']
  H_HDV['78_FC_4_100'] = [' 78','EURO5','RT >32t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','497.375504978157','0.053597648389306','10771.6528609869','0.732080847163233','931.144345012804','6','61']
  H_HDV['78_CO_6_0'] = [' 78','EURO5','RT >32t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.60265025805054','-0.003571810705182','-0.251357110928872','0.214307209278588','0','6','71']
  H_HDV['78_THC_6_0'] = [' 78','EURO5','RT >32t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005549596089432','0.147538016239278','6.61645814738464','2.65508226949403','-0.031776884771139','6','71']
  H_HDV['78_NOx_6_0'] = [' 78','EURO5','RT >32t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.0874927792228','-0.035961656381066','-5.48068298059685','0.252998625286668','0','6','71']
  H_HDV['78_PM_6_0'] = [' 78','EURO5','RT >32t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.423695390217337','-0.000795068197189','-0.055715687041205','0.163771145132006','0','6','71']
  H_HDV['78_FC_6_0'] = [' 78','EURO5','RT >32t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','2430.79740776536','1.00949654047049','-0.480312621830028','0','0','6','71']
  H_HDV['78_CO_6_50'] = [' 78','EURO5','RT >32t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.35238225293633','-0.004735859723126','-0.116873269432309','0.126933279935193','0','6','59']
  H_HDV['78_THC_6_50'] = [' 78','EURO5','RT >32t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017783022287079','0.131804281623042','6.62088971323464','2.70597829131251','-0.023313439115558','6','59']
  H_HDV['78_NOx_6_50'] = [' 78','EURO5','RT >32t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.34297749061426','0.017784035679162','14.7467965508528','0.201787400566143','5.89451064465902','6','59']
  H_HDV['78_PM_6_50'] = [' 78','EURO5','RT >32t Euro-5','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.127708854412913','0.042739922626371','0.240140321108038','0.150053719739326','0.030535033533414','6','59']
  H_HDV['78_FC_6_50'] = [' 78','EURO5','RT >32t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','595.36415231579','0.080988535906976','9132.59308743536','0.756686465795928','954.662550083401','6','59']
  H_HDV['78_CO_6_100'] = [' 78','EURO5','RT >32t Euro-5','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.68040638563681','-0.005589370238822','-0.154365891956305','0.124871666000652','0','6','44']
  H_HDV['78_THC_6_100'] = [' 78','EURO5','RT >32t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031868740391071','0.426096402089852','-0.944199630520054','-0.376687021459997','0.151764033233829','6','44']
  H_HDV['78_NOx_6_100'] = [' 78','EURO5','RT >32t Euro-5','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.9002284785895','-0.106068340575797','-3.87333646667035','0.200305117397403','0','6','44']
  H_HDV['78_PM_6_100'] = [' 78','EURO5','RT >32t Euro-5','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.450459137739753','-0.00098175797958','-0.048463816663453','0.135052741058558','0','6','44']
  H_HDV['78_FC_6_100'] = [' 78','EURO5','RT >32t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','482.833964365478','0.058343091203081','14811.0209878083','0.774538795957583','1288.08552937928','6','44']
  H_HDV['79_CO_0_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.66765646882595','0.058991206029865','7.92864032104435','0.217425045397352','1.40488571526275','6','86']
  H_HDV['79_THC_0_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.207382552629722','4.7153419180553','3.98817410216027','1.70009253647175','-0.008251245576381','6','86']
  H_HDV['79_NOx_0_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.2810586192795','0.057313657802416','66.2623077304884','0.476830533093219','7.81073152534769','6','86']
  H_HDV['79_PM_0_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.2703127522075','0.053321961669394','1.53223713505363','0.183019610112927','0.295928935047878','6','86']
  H_HDV['79_FC_0_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','661.174590335514','0.061759765061989','5671.49989920661','0.622670443951973','189.039979158755','6','86']
  H_HDV['79_CO_0_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.87766573002803','0.056766055734216','10.923145135297','0.259323945074437','1.57188433824817','6','86']
  H_HDV['79_THC_0_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.20533321473696','4.81182302874402','3.93800326382425','1.70928301153013','-0.007386211199199','6','86']
  H_HDV['79_NOx_0_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.3754430595003','0.040080467485294','160.776084853154','0.641039942643675','8.95195875320861','6','86']
  H_HDV['79_PM_0_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.36449021118998','0.050645394579466','1.84738495122988','0.231932340833618','0.337037413169068','6','86']
  H_HDV['79_FC_0_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','609.243661791339','0.048819906891246','3149.64209139178','0.514154698625075','213.91011652594','6','86']
  H_HDV['79_CO_0_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.7264029586084','73.7655422442033','-0.962391557829979','0.464898281322835','0.041804370252499','6','86']
  H_HDV['79_THC_0_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.192156684288567','4.66722767880022','4.17889891975463','1.80949572303727','-0.010891137188476','6','86']
  H_HDV['79_NOx_0_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','100','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.031096037818617','0.000918512589454','-0.00000221339895','0','0','6','86']
  H_HDV['79_PM_0_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.43666271515739','0.047162854267712','2.3182048202397','0.271088334543896','0.375569353010566','6','86']
  H_HDV['79_FC_0_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','614.215323684908','0.039606528848716','8513.44163950104','0.683103423866249','232.786767002497','6','86']
  H_HDV['79_CO_-2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.6919117749294','0.014598788948118','-0.63669699462887','0.05984622042911','0','6','86']
  H_HDV['79_THC_-2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.78729794955372','-0.0009829083966','-0.328697601293582','0.075259662729837','0','6','86']
  H_HDV['79_NOx_-2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','0','NOx','y=(1/(a+(b*x)))','0.023830461987924','0.004704834798503','0','0','0','6','86']
  H_HDV['79_PM_-2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','0','PM','y=(c+(a*exp(b*x)))','2.09386317453921','-0.07203328601382','0.197377887567227','0','0','6','86']
  H_HDV['79_FC_-2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','595.862345976695','0.057719122931209','36505.7116589794','0.915490131241069','66.4139819917953','6','86']
  H_HDV['79_CO_-2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.9160367847048','0.007520667718204','-0.660241963567168','0.062275846413769','0','6','86']
  H_HDV['79_THC_-2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3253645740031','0.040400931203885','3.28699021602044','0.095866878358648','0.217068955233353','6','86']
  H_HDV['79_NOx_-2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.221928383540396','0.636066022513975','46.7625694395573','-0.490943832888946','0','6','86']
  H_HDV['79_PM_-2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','50','PM','y=(c+(a*exp(b*x)))','2.09067296247032','-0.067946314046588','0.183552025928144','0','0','6','86']
  H_HDV['79_FC_-2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','528.104962075383','0.041756621288609','29647.7389861484','0.846973537510734','36.1761488328347','6','86']
  H_HDV['79_CO_-2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.11810882518377','0.058815787176036','138.379594552543','0.802316337053717','0.667917738958268','6','86']
  H_HDV['79_THC_-2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.181075844240415','5.49245712973676','1.88293620316893','0.892011284084254','0.033059926456638','6','86']
  H_HDV['79_NOx_-2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000061887601114','0.011270257655032','-0.760937045356019','22.1248181879504','0','6','86']
  H_HDV['79_PM_-2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.75631488037262','0.056331551020879','2.20236922393482','0.382371289891228','0.168727640226197','6','86']
  H_HDV['79_FC_-2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-3.58876959436083','0.755667534615579','1577.34517776951','-0.548680702235443','0','6','86']
  H_HDV['79_CO_-4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.98391563686249','-0.006045282763369','-0.743616160040361','0.080057392008993','0','6','86']
  H_HDV['79_THC_-4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.4217768641803','-0.002732357264416','-0.333221577090364','0.082002035266048','0','6','86']
  H_HDV['79_NOx_-4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.3428004387267','0.050315557195325','271.966063065215','0.662195252590361','0.367762333434999','6','86']
  H_HDV['79_PM_-4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.21697318674227','-0.001023816332595','-0.162473713461248','0.080113855841304','0','6','86']
  H_HDV['79_FC_-4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','443.216585197546','0.052569144912696','17884.2797934638','0.759996861041428','9.87240622867412','6','86']
  H_HDV['79_CO_-4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.16694713243664','0.059459333410577','64.2612046415927','0.655351167064975','0.242614596160939','6','86']
  H_HDV['79_THC_-4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.11560039067352','0.063175808035483','6.6199212945631','0.433456737816407','0.158869178095233','6','86']
  H_HDV['79_NOx_-4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.2544345497689','0.041993312786976','645.170711284284','0.794134469754154','0.117395650105779','6','86']
  H_HDV['79_PM_-4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.16856689623747','-0.001157985999952','-0.15717160619762','0.079686885205562','0','6','86']
  H_HDV['79_FC_-4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','382.405586803658','0.045670456706524','30942.6384250204','0.842830743200258','3.60106290991962','6','86']
  H_HDV['79_CO_-4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.4216445443366','0.052909387075221','108.351905829349','0.702247741729258','0.191439566671785','6','86']
  H_HDV['79_THC_-4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.94966880684339','0.062881356646858','9.52728940210033','0.482839928986371','0.153362014075784','6','86']
  H_HDV['79_NOx_-4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.4166898873411','0.035462291701932','1083.5837185115','0.879083240441405','-0.15355128614894','6','86']
  H_HDV['79_PM_-4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54201650627448','0.059011552915385','2.43150991170296','0.385868658413254','0.112558793919403','6','86']
  H_HDV['79_FC_-4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','357.017010202663','0.040958459694714','47496.4952693589','0.914033612396272','-0.896388891333822','6','86']
  H_HDV['79_CO_-6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.27949444670366','0.071071721765197','49.5925799158624','0.704449139744989','0.218280229388','6','72']
  H_HDV['79_THC_-6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.12809737602481','0.068453340355016','6.13220008451425','0.44333663709514','0.151434433731956','6','72']
  H_HDV['79_NOx_-6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3763339824635','0.062318697986448','518.790600772612','0.782723721363817','0.240206106207095','6','72']
  H_HDV['79_PM_-6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.10923653555097','-0.001055487401642','-0.163915860770387','0.084891797619897','0','6','72']
  H_HDV['79_FC_-6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','371.013562926569','0.061596128282837','24311.2688407697','0.817159432633674','4.15820013068344','6','72']
  H_HDV['79_CO_-6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.89245832845629','0.064068164404168','71.1607184358446','0.668734101367702','0.189604726566713','6','72']
  H_HDV['79_THC_-6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.82093970077616','0.06710136181288','8.99656596142104','0.480527150144586','0.147674771262503','6','72']
  H_HDV['79_NOx_-6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.99503151269232','0.056605821560116','713.752049145626','0.824136630744879','0.199731530903468','6','72']
  H_HDV['79_PM_-6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.41511083383729','0.065167536783914','1.0241953789035','0.233967255387891','0.109938817770879','6','72']
  H_HDV['79_FC_-6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','307.016415175255','0.057094944017772','29100.6519686208','0.842462776464565','3.07762272476332','6','72']
  H_HDV['79_CO_-6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.44202888021642','0.061025356581215','94.3516772979993','0.696374743802182','0.180420227832722','6','72']
  H_HDV['79_THC_-6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.63504320878666','0.066256348325713','8.55252554921426','0.461604675393383','0.146105056670729','6','72']
  H_HDV['79_NOx_-6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.08860969395479','0.053877756687401','804.348215637092','0.843200821392744','0.196638129805492','6','72']
  H_HDV['79_PM_-6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.42026319751869','0.064747539158522','1.4306918030791','0.297015207240502','0.110096028199789','6','72']
  H_HDV['79_FC_-6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','277.184901228954','0.05461781902735','31322.3790855512','0.853320021509073','2.91090597070964','6','72']
  H_HDV['79_CO_2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.47852588029269','8.98285730441598','7.8664379138091','3.00832252906421','-0.028782882338043','6','86']
  H_HDV['79_THC_2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.148595505303487','4.53331408595642','5.50749770687845','2.12075333965333','-0.01330903612917','6','86']
  H_HDV['79_NOx_2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.8730114549767','0.03627652852675','-2.41356491870341','0.080546662770372','0','6','86']
  H_HDV['79_PM_2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.406582283575715','1.54116842994302','7.65597856071909','2.93344901390619','-0.024162395071366','6','86']
  H_HDV['79_FC_2_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1316.5779664403','0.6291257199152','-80.857016315557','0.078602666308335','0','6','86']
  H_HDV['79_CO_2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.680786543775959','12.5205562904223','5.08783683583386','2.03323317023663','-0.022243555727785','6','86']
  H_HDV['79_THC_2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.361865208278779','4.01373246723518','7.49728176634094','2.88723712126162','-0.022778014573309','6','86']
  H_HDV['79_NOx_2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.2080597349279','0.023105844142979','-1.92316305884625','0.075681569832222','0','6','86']
  H_HDV['79_PM_2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.311373521398626','2.06720111352925','4.98377370038296','1.98407581107797','-0.021959613836932','6','86']
  H_HDV['79_FC_2_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1405.07277988313','0.981538042146862','-75.9107360412395','0.075456605194299','0','6','86']
  H_HDV['79_CO_2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.2333004030268','-0.024386428129826','-1.44566385935635','0.092999508166217','0','6','81']
  H_HDV['79_THC_2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.421328502094876','4.06107481766439','7.21952362079054','2.83914777251518','-0.021474641777841','6','81']
  H_HDV['79_NOx_2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00007760789056','0.014159028016925','-0.866490026159517','44.8570122531424','0','6','81']
  H_HDV['79_PM_2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.18284380113195','-0.003201977496666','-0.197951056242362','0.084915278438112','0','6','81']
  H_HDV['79_FC_2_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','2','100','FC','y=(c+(a*exp(b*x)))','921.017924774111','-0.080611364598207','573.165888010277','0','0','6','81']
  H_HDV['79_CO_4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.5518335298508','11.0856288447578','5.6053477011824','2.20115677693491','-0.012982652646939','6','71']
  H_HDV['79_THC_4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.301606259946123','4.36157420515604','7.08979355113324','2.72900470560656','-0.020662778097856','6','71']
  H_HDV['79_NOx_4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','51.4271155100246','0.040603926379793','-3.0108863101317','0.096180929587723','0','6','71']
  H_HDV['79_PM_4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.492888393235406','1.79134713091859','5.56336562202672','2.19303563319093','-0.012880902583775','6','71']
  H_HDV['79_FC_4_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1588.94533781057','0.396963614231933','-109.312752205939','0.095525101897735','0','6','71']
  H_HDV['79_CO_4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.0408960404139','-0.031675665723562','-1.65090396854878','0.104308737936742','0','6','68']
  H_HDV['79_THC_4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.522201849618456','3.95827197688276','8.78715821169646','3.44260600320924','-0.032170079099897','6','68']
  H_HDV['79_NOx_4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','50','NOx','y=(c+(a*exp(b*x)))','23.7553703405267','-0.098579701652059','32.3521761589038','0','0','6','68']
  H_HDV['79_PM_4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.32992278049363','-0.003382988047036','-0.225481768938206','0.095476294642998','0','6','68']
  H_HDV['79_FC_4_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','50','FC','y=(c+(a*exp(b*x)))','1091.83592952921','-0.109267495940076','693.144195821896','0','0','6','68']
  H_HDV['79_CO_4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.1034573449646','-0.051105007544273','-1.94753456943399','0.109142866280354','0','6','56']
  H_HDV['79_THC_4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.603288726796458','3.90660518105784','8.63294706622476','3.45024505065678','-0.036310195214246','6','56']
  H_HDV['79_NOx_4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000188262582936','0.022819094884355','-1.01236473634681','59.0118314119408','0','6','56']
  H_HDV['79_PM_4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.85474680124153','-0.008520884840398','-0.278621792739837','0.111433499471001','0','6','56']
  H_HDV['79_FC_4_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','4','100','FC','y=exp((a+(b/x))+(c*log(x)))','7.24482853509065','1.55276432163133','-0.12481883798031','0','0','6','56']
  H_HDV['79_CO_6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.8658522285587','-0.033943316908448','-1.79853813847912','0.115421496051057','0','6','70']
  H_HDV['79_THC_6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.588427609307173','33.6105236531877','-1.99407376827879','-0.371139627237443','0.120641121061369','6','70']
  H_HDV['79_NOx_6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','56.1238086504088','0.046705375567057','-2.77234182103008','0.095763805311627','0','6','70']
  H_HDV['79_PM_6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.33007050142799','-0.004049963136268','-0.253667033736237','0.106545775714505','0','6','70']
  H_HDV['79_FC_6_0'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1633.77294528719','2.67040398999213','-87.9794949767036','0.076550827846277','0','6','70']
  H_HDV['79_CO_6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.5927508277624','-0.037585054338231','-1.91964686867662','0.10807796226291','0','6','52']
  H_HDV['79_THC_6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.828594101848836','48.1937552964346','-3.02195537509455','-0.84008706553312','0.17218621855045','6','52']
  H_HDV['79_NOx_6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.857873073882','1.32643864044777','-0.028495932688767','0','0','6','52']
  H_HDV['79_PM_6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.892932196060809','2.14671364725334','4.91196470154673','2.05616616667215','-0.015312759864003','6','52']
  H_HDV['79_FC_6_50'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','50','FC','y=((a*(b**x))*(x**c))','2769.3740496851','1.0081464147103','-0.379633825868164','0','0','6','52']
  H_HDV['79_CO_6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','29.7165020171623','-0.064969555290178','-3.233712961218','0.14044208575368','0','6','41']
  H_HDV['79_THC_6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.04829451510272','53.2822963714622','-3.8411447902051','-1.35478464447995','0.226486486987167','6','41']
  H_HDV['79_NOx_6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','39.765292630974','0.005984490669001','515.436041705105','0.767411878613184','26.146281544511','6','41']
  H_HDV['79_PM_6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.03987293245841','-0.009405150786283','-0.498275941821193','0.152272250347141','0','6','41']
  H_HDV['79_FC_6_100'] = [' 79','EURO0','TT/AT >20-28t 80ties','6','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2865.74971434063','-4.61092789949197','-360.643869850891','0.247430413200947','0','6','41']
  H_HDV['80_CO_0_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.58938587951796','0.055288304442054','7.38071680794558','0.226776959567473','1.07084246721316','6','86']
  H_HDV['80_THC_0_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.218328059596478','3.82239904821794','3.82959851463194','1.6420783764044','-0.006536789571172','6','86']
  H_HDV['80_NOx_0_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.4432656471159','0.057717931344799','23.9028588173314','0.314140565491912','5.67966825213065','6','86']
  H_HDV['80_PM_0_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.069196141458852','1.96021808470168','4.41993738141545','1.88905503219899','-0.017293858856442','6','86']
  H_HDV['80_FC_0_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','537.377911251241','0.061206833049239','7875.52237713119','0.699329166758945','164.608978585411','6','86']
  H_HDV['80_CO_0_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.53270718109317','0.052191357503048','10.5336569886342','0.271959217727286','1.30956621545452','6','86']
  H_HDV['80_THC_0_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.222061867153812','3.93099444452623','3.63799806710636','1.58619514312833','-0.004951232305285','6','86']
  H_HDV['80_NOx_0_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.3146400035573','-0.03797024927863','-1.30336010847932','0.076307031517933','0','6','86']
  H_HDV['80_PM_0_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.1077944910935','0.051350424399434','1.64133766516461','0.1955805772073','0.241874957281563','6','86']
  H_HDV['80_FC_0_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','511.494029815909','0.047682276828393','8442.76500537244','0.707168369749234','191.378501052663','6','86']
  H_HDV['80_CO_0_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.9936260006495','0.049602592053402','14.2698805480462','0.320057915124675','1.5587572811817','6','86']
  H_HDV['80_THC_0_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.215697291770332','3.84959384228549','3.66795341824109','1.58911088754787','-0.006276701817262','6','86']
  H_HDV['80_NOx_0_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','100','NOx','y=(1/(a+(b*(x**c))))','0.033387447942285','0.003765751902386','0.744407979998856','0','0','6','86']
  H_HDV['80_PM_0_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.279932117686612','0.059237967251767','-0.000278580716782','0','0','6','86']
  H_HDV['80_FC_0_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','530.285935517819','0.037937301128307','38590.1665976892','0.9722935053952','207.670367089535','6','86']
  H_HDV['80_CO_-2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.21705654047474','0.010968450218065','-0.484908853169049','0.059539170058039','0','6','86']
  H_HDV['80_THC_-2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.222900649911972','2.79142491999824','5.22265958978923','1.95851958672316','0.003930023012037','6','86']
  H_HDV['80_NOx_-2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','0','NOx','y=(1/(a+(b*x)))','0.024893370005002','0.006462936018764','0','0','0','6','86']
  H_HDV['80_PM_-2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.190639120140475','41.8193558241302','-3.80991256582516','-0.616211799678642','0.123652445997352','6','86']
  H_HDV['80_FC_-2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','513.176427713266','0.058266877985419','27954.7688094636','0.899943512002365','56.6787000421183','6','86']
  H_HDV['80_CO_-2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.4513415701326','0.005790844166418','-0.504819689953024','0.061647632236088','0','6','86']
  H_HDV['80_THC_-2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.79559384029536','-0.00311014556763','-0.267134015928985','0.078846767654114','0','6','86']
  H_HDV['80_NOx_-2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.100143055010627','0.70525447870089','43.2395872840501','-0.571110683441675','0','6','86']
  H_HDV['80_PM_-2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','50','PM','y=(c+(a*exp(b*x)))','2.11298075537053','-0.073212291451235','0.141000422266944','0','0','6','86']
  H_HDV['80_FC_-2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','461.260067170235','0.0420499810944','24953.145486052','0.847509148470227','33.7713675492244','6','86']
  H_HDV['80_CO_-2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.86767424984806','0.055782774740676','67.164593111248','0.70271105058493','0.538179565031','6','86']
  H_HDV['80_THC_-2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.7371589225749','-0.003936061757179','-0.269548546128053','0.08141517392098','0','6','86']
  H_HDV['80_NOx_-2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.5364325899491','-0.063221343773862','-1.22646847043366','0.081520313285139','0','6','86']
  H_HDV['80_PM_-2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.044656664949661','1.65170555849678','5.9242572503936','2.25634383141898','-0.01338888547126','6','86']
  H_HDV['80_FC_-2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-2','100','FC','y=((a*(b**x))*(x**c))','1045.02386108507','0.980132730529292','-0.341479421689173','0','0','6','86']
  H_HDV['80_CO_-4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.83262022276266','-0.006131263517674','-0.603236495471756','0.083493289187719','0','6','86']
  H_HDV['80_THC_-4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.6067477096787','-0.003101357970613','-0.275698286158657','0.083846838196759','0','6','86']
  H_HDV['80_NOx_-4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.2026448855047','0.054203162754039','87.1435338301145','0.50551842475419','0.33827495856563','6','86']
  H_HDV['80_PM_-4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.64876700789998','-11.5338185504559','-1.89273011873781','0','0','6','86']
  H_HDV['80_FC_-4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','394.74174722583','0.053565304076357','12060.2146150466','0.721284562027519','9.17297627307392','6','86']
  H_HDV['80_CO_-4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.038719672764677','95.6012097095114','-1.7990964441584','0.52773350379993','0.029918509809868','6','86']
  H_HDV['80_THC_-4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.47073186563061','-0.003347469359698','-0.276109108266672','0.087727544636444','0','6','86']
  H_HDV['80_NOx_-4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.301287267827','0.046532165446031','220.966345463534','0.644706955215749','0.193206049462202','6','86']
  H_HDV['80_PM_-4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.04096560929541','-9.52553083113879','-1.75642304777733','0','0','6','86']
  H_HDV['80_FC_-4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','343.884788807322','0.046120965212971','19792.2733267063','0.79161887063793','3.65698342249172','6','86']
  H_HDV['80_CO_-4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.78576527977478','0.049320467962128','68.4415970792924','0.647686830151989','0.097131178596539','6','86']
  H_HDV['80_THC_-4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037108066019641','31.231095526127','-1.40250198644069','0.594692734356998','0.027650082462076','6','86']
  H_HDV['80_NOx_-4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.43729250999801','0.040399901700699','439.984232309105','0.754141055631766','0.047981110818661','6','86']
  H_HDV['80_PM_-4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.16131258309814','-0.00188177786338','-0.179179698863417','0.090836598086341','0','6','86']
  H_HDV['80_FC_-4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','323.612239439673','0.040934480837652','31670.8517627962','0.867777103908017','-0.742730226274974','6','86']
  H_HDV['80_CO_-6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.60871815897914','0.071441925510586','14.7407721841262','0.542727880738544','0.139337609957449','6','72']
  H_HDV['80_THC_-6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.49595606878098','0.067020041614741','3.56490172592191','0.380676851176345','0.093751755666726','6','72']
  H_HDV['80_NOx_-6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2020065696612','0.067565712054094','211.483805744216','0.669820038361868','0.248971800798976','6','72']
  H_HDV['80_PM_-6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016881922111722','1.40991847884093','7.28266593393078','2.83585721833107','-0.022915874943431','6','72']
  H_HDV['80_FC_-6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','340.964415570912','0.06319857558227','16388.8108375395','0.778002608860078','4.45625684191081','6','72']
  H_HDV['80_CO_-6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.40933601856488','0.063049276091852','43.5386947100799','0.61948061678245','0.114052876922389','6','72']
  H_HDV['80_THC_-6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.16537499302787','0.06455421317898','8.3903262023064','0.485326578164425','0.089262071798214','6','72']
  H_HDV['80_NOx_-6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.70899193497061','0.060736942490445','325.77637376286','0.719274567711531','0.210238318212873','6','72']
  H_HDV['80_PM_-6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.03992085010609','-0.001234786068253','-0.177669202917694','0.093464168351681','0','6','72']
  H_HDV['80_FC_-6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','282.862191273141','0.05806740714078','21018.1453837362','0.811494492198242','3.36092557177682','6','72']
  H_HDV['80_CO_-6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.02812396503992','0.059101459179544','65.5694035878808','0.660467828192306','0.104321069527892','6','72']
  H_HDV['80_THC_-6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.98570982252736','0.063173275048926','8.64611728580719','0.479618746500011','0.087567481226659','6','72']
  H_HDV['80_NOx_-6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.80100572758024','0.057613808560402','367.267451891113','0.736612972555689','0.202615562516444','6','72']
  H_HDV['80_PM_-6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.25209203227911','0.066432056002491','1.73368523325736','0.294127792241','0.06065414267957','6','72']
  H_HDV['80_FC_-6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','256.824668092903','0.055131775360622','24989.2737905114','0.838839643216407','3.02454643967563','6','72']
  H_HDV['80_CO_2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.2351307167909','0.012598919851284','-0.692129422397302','0.066516133876603','0','6','86']
  H_HDV['80_THC_2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.304541449549249','3.37865207981714','5.80358091849087','2.23155803559775','-0.013750229397121','6','86']
  H_HDV['80_NOx_2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','23.0093781572442','0.094708724554963','10.6177129079914','0','0','6','86']
  H_HDV['80_PM_2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.322971163142431','23.9289023213238','-3.17800538472577','-0.528980003662779','0.12234805775497','6','86']
  H_HDV['80_FC_2_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.06454507985184','1.69582431076782','2015.85012435157','-0.533075211502694','0','6','86']
  H_HDV['80_CO_2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.6238468416853','0.005829585358183','-0.831137159510293','0.077272814951901','0','6','86']
  H_HDV['80_THC_2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.623961818291767','21.3694056959223','-1.61689258310137','-0.145872938209783','0.096360754569306','6','86']
  H_HDV['80_NOx_2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.5790711695659','-0.031877422298622','-1.89884145065271','0.096640688275742','0','6','86']
  H_HDV['80_PM_2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.15327982474891','0.003753546839296','-0.120073315783849','0.062742248784864','0','6','86']
  H_HDV['80_FC_2_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1128.87974256141','0.8001630677297','-54.001302955375','0.070767901592262','0','6','86']
  H_HDV['80_CO_2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.9561700489817','-0.017176694183167','-1.16916451430698','0.103178522844825','0','6','82']
  H_HDV['80_THC_2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.593523036549447','2.94760137389745','7.56036549117097','2.91935488873348','-0.023118366644412','6','82']
  H_HDV['80_NOx_2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','100','NOx','y=((a*(b**x))*(x**c))','48.3198616038091','1.00181197142716','-0.26835683790167','0','0','6','82']
  H_HDV['80_PM_2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','100','PM','y=(c+(a*exp(b*x)))','1.77005660657468','-0.080544211520233','0.57262151046617','0','0','6','82']
  H_HDV['80_FC_2_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','2','100','FC','y=(c+(a*exp(((-1)*b)*x)))','688.724270260053','0.070718899438193','514.114346469607','0','0','6','82']
  H_HDV['80_CO_4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.21058760201749','80.1372022666055','-3.12876107202342','-0.943428595510724','0.174766373954093','6','71']
  H_HDV['80_THC_4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.473776769488469','3.22796333613333','7.21898288674917','2.78032826967038','-0.021705413473155','6','71']
  H_HDV['80_NOx_4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','39.3503716718537','-0.009724858401316','-2.54809078654775','0.108249278098954','0','6','71']
  H_HDV['80_PM_4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.438271200695556','23.3165559415743','-3.70962543698634','-0.861332516001545','0.152160187838361','6','71']
  H_HDV['80_FC_4_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','810.220200665232','0.098007793129004','382312.403156402','1.53193293029782','423.942887131718','6','71']
  H_HDV['80_CO_4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.4076607430842','-0.028419016715868','-1.61352175542286','0.13419385556827','0','6','68']
  H_HDV['80_THC_4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.839507662198303','30.9441269365852','-2.45956113466742','-0.493185394147256','0.131537342596424','6','68']
  H_HDV['80_NOx_4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','19.9787841429023','139.89203565472','-0.807097326600325','0.715189000697338','0.014980719585209','6','68']
  H_HDV['80_PM_4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000010759032441','0.001640150798583','-0.081631005965183','2.01798528253536','0','6','68']
  H_HDV['80_FC_4_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','681.175271134946','0.087187575592844','41967.4471735695','1.08256771063067','619.520407803557','6','68']
  H_HDV['80_CO_4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.564862264015','-0.050686249359027','-1.85598095902602','0.148722731328243','0','6','58']
  H_HDV['80_THC_4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.845400490831727','2.98141435145176','7.66088746562214','3.03580432743294','-0.028678526461524','6','58']
  H_HDV['80_NOx_4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000175559313632','0.022581326252995','-1.03600463026027','45.1271733871755','0','6','58']
  H_HDV['80_PM_4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000012666580385','0.001687025466378','-0.077827017025408','2.16712533730179','0','6','58']
  H_HDV['80_FC_4_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','420.680672770261','0.036234066379681','2689.14163713831','0.432310629438131','732.937203389842','6','58']
  H_HDV['80_CO_6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-59.0275517678005','0.538229872299922','17.2957396319912','0.124111615776343','2.8511922904188','6','71']
  H_HDV['80_THC_6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-31.9843193003219','0.676898068272329','5.72373994358908','0.092917194538977','0.732203178384833','6','71']
  H_HDV['80_NOx_6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','44.080617894347','-0.020742492266478','-2.6628170949739','0.114973758796126','0','6','71']
  H_HDV['80_PM_6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.05662250107281','0.007003909980017','-0.107362500209313','0.059012538169772','0','6','71']
  H_HDV['80_FC_6_0'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2313.64678235225','1.00948605150687','-0.485094169441629','0','0','6','71']
  H_HDV['80_CO_6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','50','CO','y=(c+(a*exp(b*x)))','10.5574984792046','-0.084050781147895','4.03141060500626','0','0','6','53']
  H_HDV['80_THC_6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.02321169254226','36.8318191266935','-2.78516692665994','-0.578202110150088','0.139947956142766','6','53']
  H_HDV['80_NOx_6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000279884608064','0.033046965515369','-1.3412565481926','48.2141276171523','0','6','53']
  H_HDV['80_PM_6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000013752450556','0.001789554169152','-0.078183623278489','2.0899084372012','0','6','53']
  H_HDV['80_FC_6_50'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','465.784844998049','0.06238491112898','2507.06353010703','0.464408077286089','806.350953953666','6','53']
  H_HDV['80_CO_6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','100','CO','y=((a*(b**x))*(x**c))','25.1021514642518','0.990538416791529','-0.353864960000557','0','0','6','42']
  H_HDV['80_THC_6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.39348860379295','2.31187998407573','12.8877899969928','5.25495318119922','-0.080781615671587','6','42']
  H_HDV['80_NOx_6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.9868944991362','0.015461389678666','43.0992829819497','0.316448514824854','27.2646810759491','6','42']
  H_HDV['80_PM_6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','100','PM','y=exp((a+(b/x))+(c*log(x)))','0.5362098686487','1.84639858797887','-0.085973906286169','0','0','6','42']
  H_HDV['80_FC_6_100'] = [' 80','EURO1','TT/AT >20-28t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','404.713314528629','0.028422653824164','3324.37215726093','0.453250632258274','986.523763001958','6','42']
  H_HDV['81_CO_0_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.7271318620307','0.055870744123403','5.82494048500021','0.273056798719893','0.912455317871864','6','86']
  H_HDV['81_THC_0_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.064539925941639','2.68170808727205','3.50743064888948','1.55600170377746','-0.010349976712663','6','86']
  H_HDV['81_NOx_0_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.9724110396203','0.055452261572298','39.2253425532251','0.378168292443034','5.88280611244828','6','86']
  H_HDV['81_PM_0_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.475224518389963','0.000735401682299','-0.021756046600634','0.052689300036011','0','6','86']
  H_HDV['81_FC_0_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001121380743425','0.000132758466258','-0.000000875669991','0','0','6','86']
  H_HDV['81_CO_0_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.089024837781731','0.017269197177236','-0.000103893249428','0','0','6','86']
  H_HDV['81_THC_0_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.127946985575339','2.68140426840034','3.38254169214303','1.51761206222195','-0.004300950250154','6','86']
  H_HDV['81_NOx_0_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.8862588824527','-0.041318120005008','-1.40778603212804','0.076909636781569','0','6','86']
  H_HDV['81_PM_0_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.471641223059863','0.009943454428492','-0.01275406461117','0.013543879762497','0','6','86']
  H_HDV['81_FC_0_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','433.066874816258','0.044154981267021','39881.6684149386','1.01138053985463','181.435666375304','6','86']
  H_HDV['81_CO_0_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.92063908784362','0.033506516277097','-0.262049728715909','0.034746382268642','0','6','86']
  H_HDV['81_THC_0_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.637857098371473','0.020887738131873','2.36680302859599','0.112966798898484','0.140730004580481','6','86']
  H_HDV['81_NOx_0_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','100','NOx','y=((a+(b*x))**((-1)/c))','0.003675334708561','0.000387559453788','1.65035692812348','0','0','6','86']
  H_HDV['81_PM_0_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000000403616821','0.000041498252836','-0.009793460703069','0.512730302509779','0','6','86']
  H_HDV['81_FC_0_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','458.914608435044','0.033504260122825','248871.228201014','1.33101881247179','193.120881997025','6','86']
  H_HDV['81_CO_-2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.53148864717284','0.009005424241554','-0.320814195304874','0.057904952676146','0','6','86']
  H_HDV['81_THC_-2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.167094760084009','3.18962945018594','1.60939982299952','0.820437687893097','0.036144720296692','6','86']
  H_HDV['81_NOx_-2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','0','NOx','y=((a*(b**x))*(x**c))','56.2106443421487','0.993417091474029','-0.657985869328628','0','0','6','86']
  H_HDV['81_PM_-2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.457346319592578','0.000636210674149','-0.023843190854033','0.05363629015816','0','6','86']
  H_HDV['81_FC_-2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','417.19177768485','0.054317843491818','25506.5656526963','0.917265357028958','50.5802325047658','6','86']
  H_HDV['81_CO_-2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','50','CO','y=(c+(a*exp(b*x)))','5.58954003813842','-0.068867159303707','0.509629503510312','0','0','6','86']
  H_HDV['81_THC_-2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.525848268710425','0.020953898854864','1.99991729537268','0.094090082794609','0.043923629464168','6','86']
  H_HDV['81_NOx_-2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.073958423144623','0.778895337692228','45.6803904602864','-0.577788574110674','0','6','86']
  H_HDV['81_PM_-2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','50','PM','y=(c+(a*exp(b*x)))','0.454254989388615','-0.061054567280358','0.05744466476267','0','0','6','86']
  H_HDV['81_FC_-2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','388.104709819553','0.038074744021813','23639.1521302535','0.877072076837039','27.8838047043376','6','86']
  H_HDV['81_CO_-2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','100','CO','y=(c+(a*exp(b*x)))','5.64295440061176','-0.059336394976961','0.4922234033744','0','0','6','86']
  H_HDV['81_THC_-2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.301112529325414','0.047877721884309','0.000641336546261','0','0','6','86']
  H_HDV['81_NOx_-2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.5049586003479','-0.065327897990766','-1.28173436669126','0.080883635158883','0','6','86']
  H_HDV['81_PM_-2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','100','PM','y=(c+(a*exp(b*x)))','0.480791679222406','-0.051876858496987','0.058551041373987','0','0','6','86']
  H_HDV['81_FC_-2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-2','100','FC','y=((a*(b**x))*(x**c))','763.939914118689','0.978862663646289','-0.255729182374853','0','0','6','86']
  H_HDV['81_CO_-4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.157200747220025','0.017836970480226','0.000713907995715','0','0','6','86']
  H_HDV['81_THC_-4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.330406442748571','0.047520471333198','0.001041456037723','0','0','6','86']
  H_HDV['81_NOx_-4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.2807810925444','0.052519074448846','82.7333304212791','0.478663551662249','0.355654569529217','6','86']
  H_HDV['81_PM_-4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.88587889722286','-4.93769528391258','-1.2110787773119','0','0','6','86']
  H_HDV['81_FC_-4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','325.620154878182','0.051053938732466','9228.80095318665','0.71330150419758','8.50577220625413','6','86']
  H_HDV['81_CO_-4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.28184500036207','0.050327527538651','30.5102792183106','0.560197723107427','0.103287356755903','6','86']
  H_HDV['81_THC_-4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.324554816411822','0.052133869262468','0.001064075015387','0','0','6','86']
  H_HDV['81_NOx_-4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6597634369928','0.045840424882095','230.410095276043','0.640687531277438','0.210653978552703','6','86']
  H_HDV['81_PM_-4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','1.01536226696532','0.987493126695583','-0.563609638569146','0','0','6','86']
  H_HDV['81_FC_-4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','291.59207434568','0.043575621560251','17658.5310122792','0.810776756974407','3.07936046877815','6','86']
  H_HDV['81_CO_-4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.22519021195695','0.043190040652847','77.1650611509054','0.696519599050953','0.049132666600152','6','86']
  H_HDV['81_THC_-4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','100','THC','y=((a*(b**x))*(x**c))','5.23338281338122','0.984175369366292','-0.659904220710525','0','0','6','86']
  H_HDV['81_NOx_-4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.78961529312048','0.040056880618575','419.111398262646','0.736370904984449','0.058807035060082','6','86']
  H_HDV['81_PM_-4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','100','PM','y=((a*(b**x))*(x**c))','0.826822426337718','0.982317137241632','-0.416425830792499','0','0','6','86']
  H_HDV['81_FC_-4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','276.043062053469','0.037804329757338','24291.1912975244','0.862077919489741','-2.09415665451073','6','86']
  H_HDV['81_CO_-6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.55446236278456','0.068957796488768','9.51042350764282','0.458586365461485','0.112717447623244','6','72']
  H_HDV['81_THC_-6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.50737937252991','0.061348646945295','3.01819843725633','0.372244593787191','0.079422895692197','6','72']
  H_HDV['81_NOx_-6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.5439755988487','0.067096849361568','187.236491398668','0.638771087397927','0.290078918615642','6','72']
  H_HDV['81_PM_-6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011515190555666','0.369015603393199','4.86176698774592','1.94859911187013','-0.010356884647199','6','72']
  H_HDV['81_FC_-6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','286.601793687009','0.062080598076206','12453.0715862545','0.775055215021057','5.11129887314174','6','72']
  H_HDV['81_CO_-6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.87206891773064','0.058971716856673','33.8650411455448','0.611995103140386','0.090614107523536','6','72']
  H_HDV['81_THC_-6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.3856027330984','0.059998078125801','4.11146659390155','0.407499224454819','0.077779388639014','6','72']
  H_HDV['81_NOx_-6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.99542169349332','0.060237877535642','307.031478228413','0.696473648953788','0.247575605816557','6','72']
  H_HDV['81_PM_-6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.275830131578974','0.050764338302339','0.33746581468986','0.238435861034345','0.027560704435549','6','72']
  H_HDV['81_FC_-6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','240.468853687666','0.056824120025358','16853.7769288782','0.81265553044851','4.13108378638706','6','72']
  H_HDV['81_CO_-6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.70797898208724','0.054250973408415','71.5719013210007','0.705996737754382','0.077448827054544','6','72']
  H_HDV['81_THC_-6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.30237951542591','0.058971698579356','3.8103406195736','0.387761981483746','0.076956444858189','6','72']
  H_HDV['81_NOx_-6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.03026801303682','0.056928615984744','345.005439512504','0.712585803945942','0.235002651306033','6','72']
  H_HDV['81_PM_-6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.299036143450937','0.049971433649674','0.890351077457978','0.380493109942054','0.027060458072081','6','72']
  H_HDV['81_FC_-6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','218.30335495114','0.053402668812208','19439.8132761723','0.832502216256638','3.66221706029138','6','72']
  H_HDV['81_CO_2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.99868326133855','0.003402672332893','-0.519057504950527','0.07780162999126','0','6','86']
  H_HDV['81_THC_2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.162946443970672','2.39454922540241','4.80534803869436','1.90599176691731','-0.008389836334647','6','86']
  H_HDV['81_NOx_2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.2859410980724','-0.008743277373236','-2.23036534936275','0.0928201925352','0','6','86']
  H_HDV['81_PM_2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.47162025036309','0.139976095142595','-0.001150866579666','0','0','6','86']
  H_HDV['81_FC_2_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','891.870621640379','0.636851130196087','-48.5761880518203','0.075642979658665','0','6','86']
  H_HDV['81_CO_2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.086246810919457','0.011823734991628','-0.000073326293737','0','0','6','86']
  H_HDV['81_THC_2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.269784477275371','2.04968501910475','7.03781158357049','2.74009595680156','-0.023818956126281','6','86']
  H_HDV['81_NOx_2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','37.8805683138068','-0.04219267045613','-2.05027690527634','0.095211285990932','0','6','86']
  H_HDV['81_PM_2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','50','PM','y=(((a*(x**2))+(b*x))+c)','0.00009728248204','-0.011858337824694','0.576945447456835','0','0','6','86']
  H_HDV['81_FC_2_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','950.854202482912','1.09247172026153','-37.8557426477245','0.062112173332072','0','6','86']
  H_HDV['81_CO_2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','100','CO','y=((a*(b**x))*(x**c))','18.3257959898306','0.995119399145229','-0.468356526266731','0','0','6','85']
  H_HDV['81_THC_2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.357222163474389','1.94940359392806','7.44924988301765','2.90259670480859','-0.023441812862104','6','85']
  H_HDV['81_NOx_2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','100','NOx','y=((a*(b**x))*(x**c))','53.3200391373527','1.00116126859985','-0.290127668294236','0','0','6','85']
  H_HDV['81_PM_2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','100','PM','y=(c+(a*exp(b*x)))','0.532488553870516','-0.04634732413641','0.229712392597536','0','0','6','85']
  H_HDV['81_FC_2_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1023.71875313448','0.745454349564134','-31.79889069165','0.057300035662906','0','6','85']
  H_HDV['81_CO_4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.275240578182','-0.016039101187561','-0.928034855742558','0.123625594577759','0','6','71']
  H_HDV['81_THC_4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.268126188321896','2.20994031041828','6.55127090133089','2.5588919796148','-0.018559486365147','6','71']
  H_HDV['81_NOx_4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','14.6630985069876','37.3158298057798','1.70131989860013','1.06661068783155','0.024610136442968','6','71']
  H_HDV['81_PM_4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.49173986708337','0.085116186007647','-0.000615658850305','0','0','6','71']
  H_HDV['81_FC_4_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1801.76229203171','1.00852114383726','-0.483097421576385','0','0','6','71']
  H_HDV['81_CO_4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.46655668552979','0.02007504841712','7.48559109637788','0.181200312621413','0.321338267175489','6','70']
  H_HDV['81_THC_4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.413833804792937','2.0341646015874','7.47473730311635','2.95140404374119','-0.023695958175695','6','70']
  H_HDV['81_NOx_4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','50','NOx','y=((a*(b**x))*(x**c))','62.8004949511581','1.00347151429341','-0.321267511085018','0','0','6','70']
  H_HDV['81_PM_4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','50','PM','y=(1/(a+(b*(x**c))))','1.07703995492526','0.125985213902889','0.73749600426241','0','0','6','70']
  H_HDV['81_FC_4_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003923908671588','0.591271782367872','-29.389852784265','1102.81313873134','0','6','70']
  H_HDV['81_CO_4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','100','CO','y=((a*(x**b))+(c*(x**d)))','-0.023880387722459','0.994632419670029','22.836944796026','-0.477213769251902','0','6','60']
  H_HDV['81_THC_4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.524788549973379','2.02267369134828','6.94224031551326','2.81249437421807','-0.023026826185128','6','60']
  H_HDV['81_NOx_4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','100','NOx','y=((a*(b**x))*(x**c))','65.585114549021','1.00171904636026','-0.253590774126247','0','0','6','60']
  H_HDV['81_PM_4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004435223269','0.000598188101206','-0.031450945167006','0.985915309642857','0','6','60']
  H_HDV['81_FC_4_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.005078044093411','0.65439577969158','-29.3862198640998','1280.88416178159','0','6','60']
  H_HDV['81_CO_6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.9873598400554','-0.039188993530846','-1.96113764655021','0.201137235157321','0','6','71']
  H_HDV['81_THC_6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.453436682118531','12.9925346008597','-1.66107412493882','-0.32817309988841','0.119214230904421','6','71']
  H_HDV['81_NOx_6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','47.5528032675958','-0.047130872002645','-3.01134817622753','0.116763657541587','0','6','71']
  H_HDV['81_PM_6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.861085212619666','-0.002637889440391','-0.068694707431633','0.15225342559006','0','6','71']
  H_HDV['81_FC_6_0'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1117.37129715131','2.29929938426426','-44.7705894205248','0.066015120520667','0','6','71']
  H_HDV['81_CO_6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','50','CO','y=(1/(a+(b*x)))','0.069592005438127','0.006870550770527','0','0','0','6','58']
  H_HDV['81_THC_6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.645604497561676','12.4761737298505','-1.82917123065715','-0.465345456198621','0.142232578751932','6','58']
  H_HDV['81_NOx_6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','56.1682200968451','-0.071532848358316','-2.83047372376291','0.109363107752591','0','6','58']
  H_HDV['81_PM_6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','50','PM','y=(c+(a*exp(b*x)))','0.638525264070507','-0.056201693619304','0.328905750920743','0','0','6','58']
  H_HDV['81_FC_6_50'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','50','FC','y=((a*(b**x))*(x**c))','1769.25574294052','1.00484476036319','-0.254183645515589','0','0','6','58']
  H_HDV['81_CO_6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0758022546576','0.041537755053502','26.7544841541457','0.375855466348396','1.15014453018171','6','44']
  H_HDV['81_THC_6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.654520186964825','1.9317217390033','7.41218227792409','3.0070392270715','-0.031470275068028','6','44']
  H_HDV['81_NOx_6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000439337585665','0.048081700456274','-1.84851459540844','61.8640702053648','0','6','44']
  H_HDV['81_PM_6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.77971804453082','0.022733182004024','1.96192489850029','0.36859508485131','0.155580243876324','6','44']
  H_HDV['81_FC_6_100'] = [' 81','EURO2','TT/AT >20-28t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','757.510804121987','0.008420302352809','3889.05504317672','0.522648471407387','552.081309161903','6','44']
  H_HDV['82_CO_0_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.73930136188622','0.056527246360606','6.69564301458134','0.19131711652336','1.024277687239','6','86']
  H_HDV['82_THC_0_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.12757743264214','2.26991112491736','4.41736104049485','1.85456464242284','-0.010133426857149','6','86']
  H_HDV['82_NOx_0_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','0','NOx','y=((a*(x**b))+(c*(x**d)))','1.57427297366854','0.09994565873903','91.1962611053319','-0.857102569999918','0','6','86']
  H_HDV['82_PM_0_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.063217514823697','0.733785891673792','5.36683534313888','2.15865019047118','-0.015222478368268','6','86']
  H_HDV['82_FC_0_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000965852896322','0.000128459458664','-0.000000828284087','0','0','6','86']
  H_HDV['82_CO_0_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.23028835321646','85.1169609640944','-1.48764004802412','0.424571053028595','0.045963075353257','6','86']
  H_HDV['82_THC_0_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.136012342499858','2.22447358399502','4.34391060217015','1.83344940544156','-0.008242945548697','6','86']
  H_HDV['82_NOx_0_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','50','NOx','y=((a+(b*x))**((-1)/c))','-0.000895745578547','0.000391874141085','2.09497298993','0','0','6','86']
  H_HDV['82_PM_0_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.017126932266778','0.89643131048341','3.86742404560597','1.62365630162886','-0.018454982462022','6','86']
  H_HDV['82_FC_0_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','472.515457796465','0.045140805970328','6899.96915826875','0.719796689224046','186.717181028301','6','86']
  H_HDV['82_CO_0_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.42875674891096','42.2023357563693','-0.679753154308472','0.413829385565763','0.04642471847205','6','86']
  H_HDV['82_THC_0_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.134319610316364','2.21932980587043','4.13459989071976','1.76485465729931','-0.007132968093426','6','86']
  H_HDV['82_NOx_0_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0105779089584','0.03357017203305','60.1974260290859','0.354600845205778','5.09873840174605','6','86']
  H_HDV['82_PM_0_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.49841043773155','0.049889709049358','0.534451749674295','0.196133668542539','0.114447543905661','6','86']
  H_HDV['82_FC_0_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','495.074239138546','0.035077171623021','49234.6686314099','1.06817461665504','199.796807137283','6','86']
  H_HDV['82_CO_-2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.31687047360865','0.015715863551332','-0.542640316488641','0.058066690789488','0','6','86']
  H_HDV['82_THC_-2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.105754736666857','1.65784137668169','7.21247983146031','2.74357556080312','-0.023909677956776','6','86']
  H_HDV['82_NOx_-2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6414072658835','0.057271467430104','90.9791826333471','0.459807512342969','1.56437492579706','6','86']
  H_HDV['82_PM_-2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.097711383790834','34.8304012994586','-4.71335359804224','-0.745542374643234','0.12679067376492','6','86']
  H_HDV['82_FC_-2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','499.111623256969','0.056308529360517','6399.39656256014','0.723148944197438','57.1473095334263','6','86']
  H_HDV['82_CO_-2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.2813409542085','0.010963558744232','-0.534276035603454','0.05809769003586','0','6','86']
  H_HDV['82_THC_-2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.049714724324809','1.70466308620526','5.80202000186832','2.20833082543703','-0.011481450849964','6','86']
  H_HDV['82_NOx_-2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3982368756267','0.040516348521031','115.215871518982','0.475730051163559','0.821181231180242','6','86']
  H_HDV['82_PM_-2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','50','PM','y=(c+(a*exp(b*x)))','0.837238785571424','-0.067455941812267','0.073870502515958','0','0','6','86']
  H_HDV['82_FC_-2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','50','FC','y=((a*(b**x))*(x**c))','1119.62683127518','0.984464468650594','-0.429793022055169','0','0','6','86']
  H_HDV['82_CO_-2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','100','CO','y=(c+(a*exp(b*x)))','8.99525542268621','-0.064551490500874','0.678044075550705','0','0','6','86']
  H_HDV['82_THC_-2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','4.23067767871522','-7.4346207297213','-1.45730831584434','0','0','6','86']
  H_HDV['82_NOx_-2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0195276887831','0.032111674590452','190.928922744088','0.553599841924273','0.321495892435091','6','86']
  H_HDV['82_PM_-2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037002103963442','0.639176624004967','6.51892863686634','2.40662656294668','-0.015064404142269','6','86']
  H_HDV['82_FC_-2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-2','100','FC','y=((a*(b**x))*(x**c))','873.811318080369','0.97951011292646','-0.278912829672307','0','0','6','86']
  H_HDV['82_CO_-4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.14843076601161','-12.2004400696743','-1.88751077311761','0','0','6','86']
  H_HDV['82_THC_-4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.019481853664293','1.63301944026463','5.73497117582663','2.20218905069365','-0.009877391745557','6','86']
  H_HDV['82_NOx_-4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1301718164417','0.062081561554123','116.30328775298','0.503212901307932','0.473058537456219','6','86']
  H_HDV['82_PM_-4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.80940102405666','-9.08644060632098','-1.54489948977097','0','0','6','86']
  H_HDV['82_FC_-4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','384.881911868141','0.052107785309405','2504.92875628062','0.501705671004754','13.6966025409241','6','86']
  H_HDV['82_CO_-4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.51497429441832','-0.0076999322234','-0.620959953969501','0.079977337593259','0','6','86']
  H_HDV['82_THC_-4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.08183657283982','-0.001502278230795','-0.163708249028857','0.085892389260222','0','6','86']
  H_HDV['82_NOx_-4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.89168178868054','0.054670089376001','307.3455577743','0.658758318003415','0.386634355145353','6','86']
  H_HDV['82_PM_-4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.028647800078428','0.611842564674519','6.41512656620858','2.40838297079331','-0.013931181980816','6','86']
  H_HDV['82_FC_-4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','338.753245710144','0.045460240927158','7892.55153757412','0.680077436064582','9.03374729443036','6','86']
  H_HDV['82_CO_-4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.84296421021083','-0.008624151879685','-0.534895598438471','0.075818780386055','0','6','86']
  H_HDV['82_THC_-4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.055638614053526','15.3629608668259','-1.34185229085688','0.511093712096256','0.038125024289755','6','86']
  H_HDV['82_NOx_-4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.53808953822766','0.047353879569551','533.74226582758','0.743947597579242','0.291520553950882','6','86']
  H_HDV['82_PM_-4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.87885594252812','-6.77346096110831','-1.32024750743377','0','0','6','86']
  H_HDV['82_FC_-4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','316.717842853507','0.039694210929981','16162.7046005257','0.795268170150653','3.98938304491159','6','86']
  H_HDV['82_CO_-6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.246991410495239','0.002648512151788','7.50750342815839','0.074485537954722','0.027868700756029','6','72']
  H_HDV['82_THC_-6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.07299987622858','-0.001334163916829','-0.173108319016072','0.090347298327812','0','6','72']
  H_HDV['82_NOx_-6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0323119813686','0.074420545931555','221.714319974992','0.619470357003884','0.35583082943201','6','72']
  H_HDV['82_PM_-6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.37319797194141','-7.86061162260363','-1.45907086437999','0','0','6','72']
  H_HDV['82_FC_-6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','362.159122858629','0.064697245589458','6525.81862258233','0.688062148545279','12.2888891591032','6','72']
  H_HDV['82_CO_-6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.43392966633093','0.062884386380198','15.4138075035534','0.54311492488342','0.191513085230659','6','72']
  H_HDV['82_THC_-6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.215567335122','0.066113427069026','5.58398736164946','0.500217686984727','0.072140690598317','6','72']
  H_HDV['82_NOx_-6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.79234725128899','0.064117113830228','375.629279264877','0.689036041240156','0.310814669365351','6','72']
  H_HDV['82_PM_-6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.023901586783253','0.569886679598627','5.55356151345','2.12971975290706','-0.011659072860133','6','72']
  H_HDV['82_FC_-6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','287.15997283479','0.057538948220495','11137.9422827698','0.748914970618908','10.6112269485046','6','72']
  H_HDV['82_CO_-6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.78561629247547','0.058051204863184','53.0105419991529','0.708123925231428','0.170967692236345','6','72']
  H_HDV['82_THC_-6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.04137532828445','0.063302847692896','7.20114468944676','0.52453948199174','0.070023245329527','6','72']
  H_HDV['82_NOx_-6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.63639365547205','0.059102068604739','459.682596962943','0.723837672884531','0.296622146557295','6','72']
  H_HDV['82_PM_-6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017407731129714','0.5859697419595','4.8051354905193','1.88018502787271','-0.009044333646285','6','72']
  H_HDV['82_FC_-6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','258.953218360765','0.05400695548079','13682.8263549264','0.776055845667359','9.9963053273475','6','72']
  H_HDV['82_CO_2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','0','CO','y=(c+(a*exp(b*x)))','9.55265195162318','-0.077945728732994','1.40369340403409','0','0','6','86']
  H_HDV['82_THC_2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.162784863675732','2.10232542671279','5.67388551344838','2.20371642467417','-0.011960740606759','6','86']
  H_HDV['82_NOx_2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.05696052434948','49.7403914509194','2.00240383313753','1.39832541108384','-0.002571439334525','6','86']
  H_HDV['82_PM_2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.955400093742894','-0.000435626360163','-0.063410117607537','0.079294946465446','0','6','86']
  H_HDV['82_FC_2_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','966.116710412016','0.591033924138925','-54.220152990054','0.076970051407573','0','6','86']
  H_HDV['82_CO_2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.9279772071669','0.011622805405301','-0.595636839819073','0.059965632661851','0','6','86']
  H_HDV['82_THC_2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.234692673211828','1.89119266904147','7.0324704331088','2.71232932839336','-0.019358434739902','6','86']
  H_HDV['82_NOx_2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.24718950626568','3.27301046499106','-0.223155487856814','0','0','6','86']
  H_HDV['82_PM_2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.872860786475606','0.000806075939854','-0.045840390633183','0.060121395608795','0','6','86']
  H_HDV['82_FC_2_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1010.00752971858','0.767256285097568','-42.3416236880225','0.065669553997514','0','6','86']
  H_HDV['82_CO_2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.056734248091523','0.009275796137715','-0.000029533114748','0','0','6','85']
  H_HDV['82_THC_2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.289163950616395','1.78327081159857','7.66913412985504','2.96831783422746','-0.023706121275829','6','85']
  H_HDV['82_NOx_2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.45761304523661','2.55543668659562','-0.22179391291446','0','0','6','85']
  H_HDV['82_PM_2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.884738287051069','0.000665728727949','-0.044512822890472','0.060656397666639','0','6','85']
  H_HDV['82_FC_2_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1064.46200969424','0.674933057622157','-33.3668792222996','0.056609091246957','0','6','85']
  H_HDV['82_CO_4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.3992469736584','-0.017257819897813','-0.85725302170757','0.096570474698039','0','6','71']
  H_HDV['82_THC_4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.231261082232262','1.95686109977873','7.11695602478836','2.73837498159313','-0.020996727714844','6','71']
  H_HDV['82_NOx_4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','54.5846495131721','-0.056578794435533','-6.27608335926797','0.156785643301853','0','6','71']
  H_HDV['82_PM_4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.161606623835765','9.01639508104918','-2.91288123188492','-0.285159750696153','0.08909521678097','6','71']
  H_HDV['82_FC_4_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1127.88266173631','0.290474251758268','-65.2140717430674','0.09101379734575','0','6','71']
  H_HDV['82_CO_4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.40790619430787','-3.92520590371655','-0.910876709617547','0','0','6','70']
  H_HDV['82_THC_4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.309052332001854','1.88142788973019','7.15657081172188','2.79301681527949','-0.021400772110482','6','70']
  H_HDV['82_NOx_4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.1751045659098','0.05334557820745','39.4886496948578','0.230697722461892','15.3027658127111','6','70']
  H_HDV['82_PM_4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003917551675','0.000612835004225','-0.0327525982954','0.827185431662928','0','6','70']
  H_HDV['82_FC_4_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003893416424108','0.592502877918786','-30.1379048452521','1140.01641143335','0','6','70']
  H_HDV['82_CO_4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.5669395862687','-0.045214688322048','-1.01441401949198','0.095805997991722','0','6','62']
  H_HDV['82_THC_4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.357194263257036','1.9289749628569','6.39079273169116','2.55013442517587','-0.018818902433278','6','62']
  H_HDV['82_NOx_4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','19.7626200647267','82.4927015037126','0.762181381723701','0.927499077919061','0.030189801232239','6','62']
  H_HDV['82_PM_4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.0000050699779','0.000717464699534','-0.035638062590935','0.908000423202908','0','6','62']
  H_HDV['82_FC_4_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','4','100','FC','y=exp((a+(b/x))+(c*log(x)))','7.20494561425633','0.618820770777384','-0.13465699349741','0','0','6','62']
  H_HDV['82_CO_6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.0203366321997','-0.029978900817237','-0.973575485447208','0.111882545947194','0','6','71']
  H_HDV['82_THC_6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.379451609547041','20.3746361251394','-2.33834747623772','-0.333300549945593','0.112007193569903','6','71']
  H_HDV['82_NOx_6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.34855682983848','0.024633515551534','41.7158020908092','0.180183702664753','12.7154946500117','6','71']
  H_HDV['82_PM_6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.203555721368365','10.691196057654','-3.51665063733492','-0.563089365733962','0.114467630674372','6','71']
  H_HDV['82_FC_6_0'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','1905.13131056262','1.0079825253764','-0.41299745540833','0','0','6','71']
  H_HDV['82_CO_6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.4287053358109','-0.040250032766218','-0.899637153743154','0.09224311212938','0','6','58']
  H_HDV['82_THC_6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.427143170693072','1.70433963911406','8.61974775252025','3.39050079083712','-0.035189501141891','6','58']
  H_HDV['82_NOx_6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.1596050383006','0.050103333779435','43.1638533566529','0.210306754214658','20.0173436820583','6','58']
  H_HDV['82_PM_6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.293093155064018','7.59203725017609','-3.52545367887427','-0.863409640542483','0.150448472637366','6','58']
  H_HDV['82_FC_6_50'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','489.505505951901','0.074335308982362','7864.45495411631','0.794491184715908','823.233982692869','6','58']
  H_HDV['82_CO_6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','100','CO','y=((a*(b**x))*(x**c))','25.1544324472764','0.981120413428413','-0.38135926123849','0','0','6','45']
  H_HDV['82_THC_6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.619747662841695','11.5033380213025','-2.00958487363748','-0.548235740596356','0.149711050791074','6','45']
  H_HDV['82_NOx_6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','81.9314421702821','-0.111526106746998','-8.57198473978817','0.171990793610306','0','6','45']
  H_HDV['82_PM_6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000009819022787','0.001127538205396','-0.045519034260981','1.04642177251549','0','6','45']
  H_HDV['82_FC_6_100'] = [' 82','EURO3','TT/AT >20-28t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','405.074670927458','0.029371366965159','3143.29095406759','0.50310920755273','981.812491613713','6','45']
  H_HDV['83_CO_0_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045041674997051','0.639443134373626','4.35916153537052','1.83343374226047','-0.009871575508714','6','86']
  H_HDV['83_THC_0_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004793917127437','0.129763694454977','4.40346131867177','1.87555708518413','-0.012785658350139','6','86']
  H_HDV['83_NOx_0_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.76022414093488','0.051405812856922','37.6956009988334','0.402953195843356','2.71753829437382','6','86']
  H_HDV['83_PM_0_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.077802240924831','0.043935284823941','0.240861844250857','0.166288123547728','0.015855364948759','6','86']
  H_HDV['83_FC_0_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','464.927623521047','0.059569299226114','2733.40479440969','0.557651531303112','152.10309464483','6','86']
  H_HDV['83_CO_0_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.051569629194807','0.689020832251687','3.63468904061032','1.5924228797153','-0.005151012396027','6','86']
  H_HDV['83_THC_0_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005383294390247','0.125493713464328','4.49893170649057','1.92086927710603','-0.012418483109376','6','86']
  H_HDV['83_NOx_0_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.16280919626016','0.03985713359813','60.6056047616298','0.496555671329879','3.10382273893424','6','86']
  H_HDV['83_PM_0_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.078572400984706','0.042807826730053','0.235049264521572','0.167233776495474','0.016302634432543','6','86']
  H_HDV['83_FC_0_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','442.9844225492','0.045036588760945','5407.38061776555','0.67200250549402','173.996004742209','6','86']
  H_HDV['83_CO_0_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.042098654940891','0.695160898217906','3.53492366032969','1.5186033916147','-0.006260772836849','6','86']
  H_HDV['83_THC_0_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005031405972156','0.125627681932948','4.34613453204518','1.88130665904289','-0.012537507380109','6','86']
  H_HDV['83_NOx_0_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.11225299660978','0.032495885050752','144.337133288812','0.662695141297992','3.24392008210271','6','86']
  H_HDV['83_PM_0_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.085587582432909','0.043784344319775','0.228805772434555','0.174300966127811','0.016497717315518','6','86']
  H_HDV['83_FC_0_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','461.334745815375','0.034594697278759','26836.6126762586','0.953010268016571','184.072014968083','6','86']
  H_HDV['83_CO_-2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.057870274981277','5.28544299819971','-2.46052010502679','-0.305716733491659','0.098047606934402','6','86']
  H_HDV['83_THC_-2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004409136437439','0.093440344891208','7.47076254042039','2.87343297957042','-0.030763531790228','6','86']
  H_HDV['83_NOx_-2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.29302828088791','0.052330975758618','80.981483285278','0.550867047945133','0.830702832006971','6','86']
  H_HDV['83_PM_-2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006151167465178','0.180343151729252','5.2993659759524','2.21193018909861','-0.01337769982707','6','86']
  H_HDV['83_FC_-2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','465.366389726575','0.057066143236129','6379.28441557793','0.710365635146364','52.0876220861943','6','86']
  H_HDV['83_CO_-2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014935613455232','0.459538908702722','6.44208421616916','2.38469656353651','-0.013597313446264','6','86']
  H_HDV['83_THC_-2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002734021946907','0.0939624184498','6.22775841720876','2.38321024897483','-0.015443195190988','6','86']
  H_HDV['83_NOx_-2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.66549183595063','0.035596858007633','103.394985906553','0.571417039541983','0.362311200295551','6','86']
  H_HDV['83_PM_-2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001545523536543','0.199962464946755','4.2033247867976','1.82765345112886','-0.007198775889551','6','86']
  H_HDV['83_FC_-2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','50','FC','y=((a*(b**x))*(x**c))','1080.71572239614','0.983853948415744','-0.442060255722813','0','0','6','86']
  H_HDV['83_CO_-2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.37196414713962','-9.00340626111386','-1.54078081815437','0','0','6','86']
  H_HDV['83_THC_-2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001014533545109','0.098879528049802','5.15586261320686','2.02257896625771','-0.009341479196492','6','86']
  H_HDV['83_NOx_-2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.96696748278477','0.027605299369138','185.810860276687','0.666239181778046','-0.078700500784271','6','86']
  H_HDV['83_PM_-2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.66792101015829','-5.81472731606013','-1.47166103292705','0','0','6','86']
  H_HDV['83_FC_-2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-3.94031407147395','0.720779895802128','1093.09691845055','-0.488028366928195','0','6','86']
  H_HDV['83_CO_-4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.262774741284904','0.001540319436821','0.56410477506192','0.077679961828075','-0.217749992235771','6','86']
  H_HDV['83_THC_-4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','0','THC','y=exp((a+(b/x))+(c*log(x)))','2.17971043243571','-9.69122446537226','-1.72823595487568','0','0','6','86']
  H_HDV['83_NOx_-4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.99333629916899','0.053311004614559','77.5661628225325','0.535609547656669','0.126662086547257','6','86']
  H_HDV['83_PM_-4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.245864649566871','-0.000165909091882','-0.023496174627222','0.102359392933473','0','6','86']
  H_HDV['83_FC_-4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','374.216204335788','0.05507267880193','3896.44068064799','0.586287641968774','9.26882860757899','6','86']
  H_HDV['83_CO_-4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.545936255060641','-0.000410544547249','-0.040542927159477','0.080016679891983','0','6','86']
  H_HDV['83_THC_-4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.119207407384999','-0.000090284274152','-0.009576629761856','0.087395864576644','0','6','86']
  H_HDV['83_NOx_-4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.25535185469426','0.046039516881504','149.789996701918','0.639258161486021','0.047140586802502','6','86']
  H_HDV['83_PM_-4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.75457337565676','0.63024199699463','0.025684718790792','0','0','6','86']
  H_HDV['83_FC_-4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','317.271260960207','0.047216971544324','10723.1177540207','0.735807592872163','4.36890664659804','6','86']
  H_HDV['83_CO_-4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.495763668327444','-0.000446111545617','-0.036584373366489','0.080397515211623','0','6','86']
  H_HDV['83_THC_-4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','6.91812255703406','1.03847326115702','0.031708472836193','0','0','6','86']
  H_HDV['83_NOx_-4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.93104643235828','0.040480766383389','231.409880439612','0.709580873188391','-0.025045681447752','6','86']
  H_HDV['83_PM_-4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.106901952001011','0.059074115980171','0.373912062159219','0.332634147894033','0.003932746215394','6','86']
  H_HDV['83_FC_-4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','287.998148968731','0.040881462144751','19494.6667540373','0.826421480108571','-0.161294381881753','6','86']
  H_HDV['83_CO_-6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','0','CO','y=(c+(a*exp(b*x)))','0.497816200004501','-0.075270102740117','0.012331563252453','0','0','6','72']
  H_HDV['83_THC_-6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.116040262783561','-0.000062630297428','-0.009497178634317','0.087445441997607','0','6','72']
  H_HDV['83_NOx_-6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.55476414200376','0.066754593916806','129.329635438602','0.640069601474412','0.066578316854183','6','72']
  H_HDV['83_PM_-6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.125338561173306','0.070110612376147','0.216594711448633','0.265004738055697','0.004393958946225','6','72']
  H_HDV['83_FC_-6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','334.456159703737','0.066601124260943','9591.12514299505','0.747332675268259','5.58823493344704','6','72']
  H_HDV['83_CO_-6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.338871551253268','0.0651917006356','4.00200869670248','0.720616677010568','0.009778410212959','6','72']
  H_HDV['83_THC_-6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.071067439528559','0.06609748068815','0.35728974453915','0.525901414830363','0.003347162845714','6','72']
  H_HDV['83_NOx_-6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.53065420867739','0.06111362881655','187.961755435926','0.69125444503359','0.050824488898799','6','72']
  H_HDV['83_PM_-6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.112109073200319','0.068636213298541','0.507368810621017','0.406292196588233','0.004226145839271','6','72']
  H_HDV['83_FC_-6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','255.038411549008','0.058941050719037','14294.8623582744','0.784402159100197','4.11478231432842','6','72']
  H_HDV['83_CO_-6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.27882754645329','0.06056292190944','6.11393510733853','0.75783885562316','0.008742603150224','6','72']
  H_HDV['83_THC_-6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061569551297772','0.063403535866364','0.476141141489878','0.554331319642609','0.003229536779055','6','72']
  H_HDV['83_NOx_-6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.07297226507141','0.057670283493341','222.757172865797','0.717902160305086','0.044371964968853','6','72']
  H_HDV['83_PM_-6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.100774763838401','0.066411025005139','0.668134195794447','0.449043758339143','0.004089555010975','6','72']
  H_HDV['83_FC_-6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','222.620523983836','0.054859331264816','17461.3264952728','0.812168409806432','3.53918366787689','6','72']
  H_HDV['83_CO_2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.101651734292375','0.521001973106264','6.70342837650979','2.62134161347837','-0.015796300851504','6','86']
  H_HDV['83_THC_2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00646848505847','0.119572353927768','5.55837325167187','2.18406756914196','-0.011978685883356','6','86']
  H_HDV['83_NOx_2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.276279614499','0.090257864644329','38.7501227569131','0.536391674854915','5.26661608653619','6','86']
  H_HDV['83_PM_2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.042542394584045','0.016187744368711','0.244390642237225','0.118483114778316','0.010627744278042','6','86']
  H_HDV['83_FC_2_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','932.753462940698','0.403907971620222','-54.5970758562801','0.080316909111008','0','6','86']
  H_HDV['83_CO_2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.996624319852638','-0.001427810248691','-0.090648203089071','0.11713361092345','0','6','86']
  H_HDV['83_THC_2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008944915311912','0.109241688969886','6.7416254293239','2.6309463939516','-0.019042119221701','6','86']
  H_HDV['83_NOx_2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','50','NOx','y=exp((a+(b/x))+(c*log(x)))','2.59208297090262','2.61828568396561','-0.1653261683366','0','0','6','86']
  H_HDV['83_PM_2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055770152108135','0.023300769682511','0.241628174308295','0.13135807890146','0.015331461176585','6','86']
  H_HDV['83_FC_2_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','979.422201962096','0.45210226795618','-44.2844295351858','0.071182012938581','0','6','86']
  H_HDV['83_CO_2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.0450162670603','-0.00220674654436','-0.091855766335781','0.117591019771363','0','6','86']
  H_HDV['83_THC_2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011177349932805','0.103360310786114','7.37749855817103','2.88272542060253','-0.022730699739126','6','86']
  H_HDV['83_NOx_2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.08813396190519','0.027314648005699','39.0277402938643','0.375699547804956','7.23371165825076','6','86']
  H_HDV['83_PM_2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079025662167996','0.034037669671918','0.233717337760467','0.149630360538597','0.019335490925574','6','86']
  H_HDV['83_FC_2_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','2','100','FC','y=(c+(a*exp(b*x)))','560.639079230468','-0.070207248412889','486.120044114963','0','0','6','86']
  H_HDV['83_CO_4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.980903073024493','-0.001011381510821','-0.086127019122453','0.11286561397174','0','6','72']
  H_HDV['83_THC_4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008095377012812','0.113169820408523','6.62128391219795','2.5690085639912','-0.018482839402568','6','72']
  H_HDV['83_NOx_4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.3143759174496','-0.021451084657853','-3.04279213975809','0.163534307956829','0','6','72']
  H_HDV['83_PM_4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057548836470365','0.0120407382787','0.241719812471229','0.121242362279383','0.003225287985362','6','72']
  H_HDV['83_FC_4_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','0','FC','y=(c+(a*exp(b*x)))','709.864789326686','-0.099861249969892','399.662525502304','0','0','6','72']
  H_HDV['83_CO_4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','1.17626456474038','-1.8292562851511','-0.713116618761023','0','0','6','71']
  H_HDV['83_THC_4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011426805179432','0.107022993157458','7.06717530567043','2.77280586861858','-0.020986522662768','6','71']
  H_HDV['83_NOx_4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.56701286805234','0.038236362206165','28.6425374356045','0.321481199540536','9.59191704794274','6','71']
  H_HDV['83_PM_4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.081837052219077','0.021888833579952','0.245530367201479','0.144133692824013','0.008901881733093','6','71']
  H_HDV['83_FC_4_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003823659266645','0.586576802480212','-29.8157678760513','1093.34261301468','0','6','71']
  H_HDV['83_CO_4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.495373164853287','0.019570998442874','0.79000313234103','0.140701038414237','-0.003837104483816','6','63']
  H_HDV['83_THC_4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013495265948651','0.10737395369359','6.74656715501045','2.70030883640528','-0.021101148602773','6','63']
  H_HDV['83_NOx_4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.09067694214922','0.04127002122375','36.6337594966143','0.383584879923095','12.262750053491','6','63']
  H_HDV['83_PM_4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.093762267886225','0.022211210141669','0.257743314654201','0.141553604857041','0.004120264609023','6','63']
  H_HDV['83_FC_4_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','4','100','FC','y=exp((a+(b/x))+(c*log(x)))','7.04174791212416','1.0494303842681','-0.109118630459024','0','0','6','63']
  H_HDV['83_CO_6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.31461229980959','-0.003449829451236','-0.187172519606581','0.197546257794','0','6','71']
  H_HDV['83_THC_6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01533694049484','0.84576140269975','-1.84710624867207','-0.228934905498106','0.105414074709957','6','71']
  H_HDV['83_NOx_6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.6766120039676','-0.039214922322085','-5.27226496034416','0.220408791179216','0','6','71']
  H_HDV['83_PM_6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.325722240787224','-0.000586333155743','-0.03466390498433','0.132035253413134','0','6','71']
  H_HDV['83_FC_6_0'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1140.98767574666','1.49983852116713','-51.18699790452','0.074615445365517','0','6','71']
  H_HDV['83_CO_6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.20230578447209','-0.004066989710797','-0.099157159463604','0.117510842409212','0','6','58']
  H_HDV['83_THC_6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.181109343063391','-0.000116529540149','-0.015664031970189','0.099766427875047','0','6','58']
  H_HDV['83_NOx_6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.47948022872275','0.050000777597386','34.4279437106807','0.374152305694153','12.9217039794304','6','58']
  H_HDV['83_PM_6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.342133901712195','-0.000746075170717','-0.033748393373019','0.122728638316146','0','6','58']
  H_HDV['83_FC_6_50'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','409.458611886915','0.067309815454527','2164.43580353457','0.504972303446327','771.346510951292','6','58']
  H_HDV['83_CO_6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.41486546836165','-0.001837516603142','-0.102868594881093','0.087813524355816','0','6','46']
  H_HDV['83_THC_6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024817500293303','0.520699031925245','-1.55773499523766','-0.43835195256874','0.146061397766729','6','46']
  H_HDV['83_NOx_6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.47014051225234','0.027069730979546','33.6647813140559','0.317341004984236','14.6897202704424','6','46']
  H_HDV['83_PM_6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.023548030795333','0.245377561331954','4.91424980948797','2.09478470762719','-0.012887184011406','6','46']
  H_HDV['83_FC_6_100'] = [' 83','EURO4','TT/AT >20-28t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','356.745780319733','0.040555600719351','2824.78059800245','0.495482951256499','978.261680255229','6','46']
  H_HDV['84_CO_0_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.044328534961437','0.651053468496076','4.38714263433584','1.8426628210969','-0.010421690622366','6','86']
  H_HDV['84_THC_0_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005008886146774','0.13191977243605','4.41460280734732','1.87739688813605','-0.012584560926484','6','86']
  H_HDV['84_NOx_0_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.09248769183584','0.049669533418999','24.3855043555887','0.371148190760501','1.58472853312178','6','86']
  H_HDV['84_PM_0_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.07946526757218','0.043980218124618','0.245322713139147','0.166256035905233','0.016055577564434','6','86']
  H_HDV['84_FC_0_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','477.038399252032','0.059289150919678','2458.70644229819','0.53105123912326','154.418483918352','6','86']
  H_HDV['84_CO_0_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.05114921324245','0.709869361937947','3.5757044911956','1.57469575934809','-0.00496446829554','6','86']
  H_HDV['84_THC_0_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005150374209599','0.128175716741367','4.51521076185632','1.927681380691','-0.013271837420787','6','86']
  H_HDV['84_NOx_0_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.41409843668729','0.039570172414347','33.9502706273923','0.43904224791178','1.80254628889188','6','86']
  H_HDV['84_PM_0_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.080596162036917','0.042932023505919','0.240480728897115','0.168392028569458','0.016517497671758','6','86']
  H_HDV['84_FC_0_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','456.844697488778','0.045204067991559','5785.91171774855','0.67196035321838','176.55638012647','6','86']
  H_HDV['84_CO_0_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.054760790877066','0.718850376200071','3.31664770086152','1.44903273036963','-0.000988864694325','6','86']
  H_HDV['84_THC_0_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005501318845143','0.127205169848866','4.37510430362439','1.88730390832052','-0.011791917993822','6','86']
  H_HDV['84_NOx_0_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.09701028304562','0.03451954159562','61.6700283556551','0.55805605141334','1.90858091308357','6','86']
  H_HDV['84_PM_0_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086256834804506','0.043425643705312','0.23415675876178','0.173567927302999','0.01666382040397','6','86']
  H_HDV['84_FC_0_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','474.289482119724','0.034807406542422','20693.0065986571','0.900795715010381','186.826206576775','6','86']
  H_HDV['84_CO_-2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.058943079037464','5.4287410698599','-2.46499865646747','-0.305697874219231','0.098056593510955','6','86']
  H_HDV['84_THC_-2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00451357851857','0.095521312292624','7.45991093422666','2.86860052250496','-0.030556201181811','6','86']
  H_HDV['84_NOx_-2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.51359032398367','0.05240573249482','47.4862869092912','0.504029073270419','0.486503837438512','6','86']
  H_HDV['84_PM_-2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006241815844414','0.184314812774576','5.29827251741985','2.21184631651244','-0.013371575738974','6','86']
  H_HDV['84_FC_-2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','488.838789619466','0.057850061701607','8714.75181721184','0.759747941754773','53.4852655575755','6','86']
  H_HDV['84_CO_-2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015152530365009','0.468640486644036','6.47639071162801','2.39724951693361','-0.01383444010771','6','86']
  H_HDV['84_THC_-2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002778052100412','0.095998562287991','6.22747440926534','2.38258482987231','-0.015432604708029','6','86']
  H_HDV['84_NOx_-2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.12820929201688','0.036511298979885','59.3157621840929','0.526590985208505','0.219576565719491','6','86']
  H_HDV['84_PM_-2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001823596432772','0.20325394990228','4.21749505154194','1.82994642107184','-0.006737015569504','6','86']
  H_HDV['84_FC_-2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','50','FC','y=((a*(b**x))*(x**c))','1129.61107778731','0.983923044222147','-0.448945273943506','0','0','6','86']
  H_HDV['84_CO_-2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','100','CO','y=exp((a+(b/x))+(c*log(x)))','3.4068503904563','-9.04701284293357','-1.54532219611668','0','0','6','86']
  H_HDV['84_THC_-2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001049771315585','0.10060520646644','5.19177404331218','2.03333052936783','-0.009483339455338','6','86']
  H_HDV['84_NOx_-2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.28617222196557','0.029118797953771','94.8645330939611','0.603714309191633','-0.010761798871201','6','86']
  H_HDV['84_PM_-2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.6985290935212','-5.84804886201304','-1.47487149122875','0','0','6','86']
  H_HDV['84_FC_-2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-4.23736010987013','0.708700310206938','1136.48954455652','-0.492483237217107','0','6','86']
  H_HDV['84_CO_-4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.26847902860039','0.001547243222102','0.577038800743022','0.077731680143358','-0.222386653140554','6','86']
  H_HDV['84_THC_-4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','0','THC','y=exp((a+(b/x))+(c*log(x)))','2.21050317370581','-9.72091006177694','-1.73050878017377','0','0','6','86']
  H_HDV['84_NOx_-4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.78734164512696','0.054262558269851','49.2631020028254','0.509250791954882','0.076862993216254','6','86']
  H_HDV['84_PM_-4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.251128366476059','-0.000169265923765','-0.023997291733368','0.10234040088328','0','6','86']
  H_HDV['84_FC_-4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','386.625666277249','0.055202807037533','3811.15769030626','0.577350792057401','9.52533965752626','6','86']
  H_HDV['84_CO_-4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.557470372584232','-0.000414086817411','-0.041354654254838','0.079872410268014','0','6','86']
  H_HDV['84_THC_-4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.121648013331787','-0.000091581018208','-0.009751108821915','0.087169894667383','0','6','86']
  H_HDV['84_NOx_-4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.35500200768549','0.047643239749404','87.5855589259074','0.60221518941032','0.033747625305518','6','86']
  H_HDV['84_PM_-4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.75302038407932','0.607578782562009','0.025433556313282','0','0','6','86']
  H_HDV['84_FC_-4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','328.571207274688','0.047575555144951','10698.3869262097','0.73083482383881','4.65037169676251','6','86']
  H_HDV['84_CO_-4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.506871411103968','-0.000450155794929','-0.037412978852955','0.080338057294824','0','6','86']
  H_HDV['84_THC_-4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','6.82471572633399','1.0053920968291','0.031327103509857','0','0','6','86']
  H_HDV['84_NOx_-4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.14757582018485','0.042420227604887','125.4239770049','0.660616221175736','-0.003862535350567','6','86']
  H_HDV['84_PM_-4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108924179282355','0.059072537361812','0.371092305658431','0.327605820926851','0.004004258562316','6','86']
  H_HDV['84_FC_-4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','297.850233238374','0.041301622620069','19336.3356210213','0.820796210266874','0.163212624490209','6','86']
  H_HDV['84_CO_-6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','0','CO','y=(c+(a*exp(((-1)*b)*x)))','0.509857371987052','0.075297928063703','0.01255298649045','0','0','6','72']
  H_HDV['84_THC_-6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.118600935322742','-0.000062813007938','-0.009691347530399','0.087243490107198','0','6','72']
  H_HDV['84_NOx_-6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.60592686469611','0.067960504425139','78.4504197149333','0.607963846710154','0.041079658281346','6','72']
  H_HDV['84_PM_-6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.12759674227724','0.070051914737262','0.213599170274447','0.258601092862658','0.004477341165143','6','72']
  H_HDV['84_FC_-6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.233154216556','0.066953721956907','9597.4728442672','0.743871256806986','5.82639085384421','6','72']
  H_HDV['84_CO_-6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.347933606640605','0.065409347820868','3.46311018259364','0.70162977115244','0.009993755422833','6','72']
  H_HDV['84_THC_-6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.073041821460274','0.066343197320871','0.3518960056339','0.522528174200825','0.003424429292636','6','72']
  H_HDV['84_NOx_-6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.95449086456351','0.062348709943806','107.832104997123','0.651485837713808','0.030829379675279','6','72']
  H_HDV['84_PM_-6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.114615111872142','0.0687144957616','0.500404078916622','0.401107297465478','0.004311358782637','6','72']
  H_HDV['84_FC_-6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','265.667290413414','0.059411106575183','14713.7479958722','0.785023622844163','4.35081002573326','6','72']
  H_HDV['84_CO_-6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.285682390412435','0.060739787745461','6.12723103147324','0.756091264651578','0.008944264278937','6','72']
  H_HDV['84_THC_-6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.063033105590223','0.063500968381','0.482214206833973','0.553773932087908','0.003295566738733','6','72']
  H_HDV['84_NOx_-6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.66368235493063','0.058952195155382','126.427609657722','0.677071731042238','0.026516042801874','6','72']
  H_HDV['84_PM_-6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.102967251623069','0.06648874687773','0.673234218727339','0.447028327385202','0.004172791235533','6','72']
  H_HDV['84_FC_-6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','231.288124282045','0.055280924560943','18171.9525571881','0.813970292667333','3.76288119366733','6','72']
  H_HDV['84_CO_2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.101035596594631','0.533378517529763','6.64992982034124','2.60053557452581','-0.015744800241731','6','86']
  H_HDV['84_THC_2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00657684918446','0.121917290433721','5.55117845264881','2.18056062850453','-0.011931227829807','6','86']
  H_HDV['84_NOx_2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.04858039458972','165.03298643514','-1.56820333074146','0.860101468085028','0.038745778723534','6','86']
  H_HDV['84_PM_2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.043207349533635','0.016637578276086','0.24864422243704','0.118603690627065','0.011144428230134','6','86']
  H_HDV['84_FC_2_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','961.48854569714','0.390763841331304','-56.7225551043929','0.080595554484617','0','6','86']
  H_HDV['84_CO_2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.00856117949806','-0.001383993848413','-0.09089235397955','0.115605096491135','0','6','86']
  H_HDV['84_THC_2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008930431980015','0.111867622038733','6.67401974471178','2.60437529764387','-0.018606723189094','6','86']
  H_HDV['84_NOx_2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','50','NOx','y=((a*(x**b))+(c*(x**d)))','11.6537876754685','-0.256054647481224','133.439099894385','-2.19102682407835','0','6','86']
  H_HDV['84_PM_2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.056601200906328','0.023630505253789','0.245564141557625','0.131245999294576','0.01583352563136','6','86']
  H_HDV['84_FC_2_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1008.83020172456','0.424957315242812','-46.3359595620797','0.071857088382078','0','6','86']
  H_HDV['84_CO_2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.0686653799123','-0.002255741057429','-0.096398913601629','0.120810268067697','0','6','86']
  H_HDV['84_THC_2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01115995749674','0.105685586264051','7.36969912193992','2.88076575456536','-0.023107805677169','6','86']
  H_HDV['84_NOx_2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.9610180536011','0.019213250183383','23.0421030084298','0.317378473521104','3.27307125831813','6','86']
  H_HDV['84_PM_2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.075817496133567','0.031836788754644','0.240153602212105','0.145673921739966','0.018950901837941','6','86']
  H_HDV['84_FC_2_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','2','100','FC','y=(c+(a*exp(b*x)))','583.275592896259','-0.070640842297071','493.039478909011','0','0','6','86']
  H_HDV['84_CO_4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.997018933826499','-0.001052876242186','-0.087789191369366','0.113030455848662','0','6','72']
  H_HDV['84_THC_4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008221337023304','0.115325758982398','6.62559599167129','2.56970834092597','-0.018516154026852','6','72']
  H_HDV['84_NOx_4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.4073251262869','-0.017796867374604','-2.8637236775124','0.190828631008612','0','6','72']
  H_HDV['84_PM_4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.063332319969763','0.009559661688139','0.246729021849474','0.119176477228823','-0.004120884329836','6','72']
  H_HDV['84_FC_4_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','1941.80060365795','1.00862816897148','-0.508485034434916','0','0','6','72']
  H_HDV['84_CO_4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','50','CO','y=exp((a+(b/x))+(c*log(x)))','1.1822566797551','-1.82191475796559','-0.710700438187932','0','0','6','71']
  H_HDV['84_THC_4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011548625245915','0.109113385619569','7.07853200801216','2.77293761348013','-0.020739338635353','6','71']
  H_HDV['84_NOx_4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.68640266453911','0.027252303261729','20.4426924135779','0.29915984486149','4.78132127082672','6','71']
  H_HDV['84_PM_4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082107164751009','0.022547082414615','0.24803133061219','0.143592795380918','0.010346727335268','6','71']
  H_HDV['84_FC_4_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003902994825456','0.602419772895129','-30.7802327210558','1120.01831602001','0','6','71']
  H_HDV['84_CO_4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.486081662879552','0.020295361975507','0.785304472158811','0.138671228942463','0.00977969525403','6','64']
  H_HDV['84_THC_4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013925872887203','0.108357914660382','6.9118004514166','2.76021188498578','-0.022026851679507','6','64']
  H_HDV['84_NOx_4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.06209686212036','0.031267301312767','21.07733781526','0.314980485218257','5.91405650837543','6','64']
  H_HDV['84_PM_4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094140480070613','0.027881772349374','0.254114270619543','0.144132961365832','0.012603966314004','6','64']
  H_HDV['84_FC_4_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','4','100','FC','y=exp((a+(b/x))+(c*log(x)))','7.00588853248998','1.26968242541841','-0.09644214297778','0','0','6','64']
  H_HDV['84_CO_6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.33705698948457','-0.003382874247635','-0.190615336132951','0.196625753205591','0','6','71']
  H_HDV['84_THC_6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015504605814298','0.786511111117271','-1.70099049037181','-0.200503914821015','0.103400016237463','6','71']
  H_HDV['84_NOx_6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.0132597416741','-0.03478807129116','-5.36339743571218','0.260290151626205','0','6','71']
  H_HDV['84_PM_6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.334445756964884','-0.000598084243184','-0.036253154443264','0.134383595589907','0','6','71']
  H_HDV['84_FC_6_0'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1172.30523273586','1.49710784238313','-53.3385676302789','0.075151845985174','0','6','71']
  H_HDV['84_CO_6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.21925473402402','-0.004388949429198','-0.102441131820026','0.121415191943392','0','6','60']
  H_HDV['84_THC_6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01447594251058','0.103949652369877','7.54602993002068','2.98405635345149','-0.026976518105161','6','60']
  H_HDV['84_NOx_6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.22120605451458','0.039382181607417','21.5272546146172','0.325198540615907','6.56357107634592','6','60']
  H_HDV['84_PM_6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.35008965519499','-0.000820464240532','-0.035199007994628','0.1259635429474','0','6','60']
  H_HDV['84_FC_6_50'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','402.929614912789','0.063287496127769','1797.52381537545','0.452278312730816','778.126953666167','6','60']
  H_HDV['84_CO_6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.38192404625917','-3.7230172670522','-1.01900427949434','0','0','6','47']
  H_HDV['84_THC_6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.025303784570718','0.551801863685586','-1.62889186975612','-0.453214231718693','0.146598833457193','6','47']
  H_HDV['84_NOx_6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','100','NOx','y=((a+(b*x))**((-1)/c))','-0.000008938340821','0.000003343636953','3.98617551877403','0','0','6','47']
  H_HDV['84_PM_6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.032453213131564','0.239134434315425','4.8601774052645','2.03924642605813','0.000279659153399','6','47']
  H_HDV['84_FC_6_100'] = [' 84','EURO5','TT/AT >20-28t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','373.80188065349','0.051849853498366','3385.54126199593','0.538891542852358','1023.10950899413','6','47']
  H_HDV['85_CO_0_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.64847408958','0.05873598116659','8.14120258369364','0.217640751849954','1.42290329846186','6','86']
  H_HDV['85_THC_0_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.206028289369385','4.71190006061783','4.00831945148303','1.70856204325106','-0.008257144229342','6','86']
  H_HDV['85_NOx_0_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.9542970762677','0.054980908641651','67.1596370382773','0.475865802037743','7.94808363691646','6','86']
  H_HDV['85_PM_0_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.2949388354464','0.053442862908767','1.56806293631183','0.191021901226062','0.300966991099085','6','86']
  H_HDV['85_FC_0_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','654.732028855196','0.060313521401859','5348.78045713606','0.60995005084745','191.627790749227','6','86']
  H_HDV['85_CO_0_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.6145195393323','133.149153366902','-1.76579946012644','0.401633912253843','0.044928180245792','6','86']
  H_HDV['85_THC_0_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.202348335421533','4.7990500896432','3.96924636158356','1.72889229866793','-0.008131066985315','6','86']
  H_HDV['85_NOx_0_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','50','NOx','y=((a+(b*x))**((-1)/c))','0.000827997726192','0.000126741083645','1.96403667009045','0','0','6','86']
  H_HDV['85_PM_0_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.39131037360261','0.049529287633921','1.89863882612769','0.241689521690913','0.349360615298311','6','86']
  H_HDV['85_FC_0_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','603.411805737419','0.045433185137309','4254.50223838647','0.558542149399589','220.523757132855','6','86']
  H_HDV['85_CO_0_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.83329827040297','76.8876781409083','-0.997021421451205','0.437577520987607','0.043898318508696','6','86']
  H_HDV['85_THC_0_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.187899135944245','4.68015203986403','4.11892514644328','1.79635881168355','-0.011046672237746','6','86']
  H_HDV['85_NOx_0_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','100','NOx','y=(1/(a+(b*x)))','0.03131253194339','0.000680902818218','0','0','0','6','86']
  H_HDV['85_PM_0_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.51594118154655','0.046830279341571','2.6313612061392','0.2963262712392','0.396992736832451','6','86']
  H_HDV['85_FC_0_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','630.015717781783','0.035165647061179','15463.4732949601','0.794231113681235','235.204247854902','6','86']
  H_HDV['85_CO_-2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.6358756229699','0.014459004576331','-0.629948591867593','0.059413212845206','0','6','86']
  H_HDV['85_THC_-2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.77763507319829','-0.001210294390409','-0.329051907109751','0.075518353569332','0','6','86']
  H_HDV['85_NOx_-2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','0','NOx','y=((a*(b**x))*(x**c))','57.0042662427214','0.991463671319084','-0.576662583429935','0','0','6','86']
  H_HDV['85_PM_-2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','0','PM','y=(c+(a*exp(b*x)))','2.08767575482937','-0.07149456742317','0.194582631091314','0','0','6','86']
  H_HDV['85_FC_-2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','584.130316890604','0.055361957573999','33068.2923042901','0.893703687489736','62.2506632576','6','86']
  H_HDV['85_CO_-2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.71151104102545','0.065129459689228','128.841628596689','0.886210251250654','0.73000595947718','6','86']
  H_HDV['85_THC_-2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.20261300227841','0.037022100564945','3.41983561330841','0.095338144126314','0.188993359769557','6','86']
  H_HDV['85_NOx_-2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000068979423018','0.012603886046232','-0.826175934210785','22.5040163727675','0','6','86']
  H_HDV['85_PM_-2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','50','PM','y=(c+(a*exp(b*x)))','2.09350370959272','-0.066639120029261','0.182606160150892','0','0','6','86']
  H_HDV['85_FC_-2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','518.966556855921','0.038292030535064','30948.9418119818','0.848189604854667','27.8814780343867','6','86']
  H_HDV['85_CO_-2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.97009618567247','0.055543801971614','154.879800822958','0.786729157696828','0.634849889628428','6','86']
  H_HDV['85_THC_-2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.948652904914245','0.028850692883581','3.67352525730855','0.09472344038946','0.11920303130554','6','86']
  H_HDV['85_NOx_-2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.4216952209334','-0.096110638996291','-1.06618537430835','0.068971351247511','0','6','86']
  H_HDV['85_PM_-2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.74438479526835','0.053861569343859','3.29452107283594','0.43235569555503','0.165987057401592','6','86']
  H_HDV['85_FC_-2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-2.54220496932905','0.858571902641405','1486.71738678677','-0.520093328657864','0','6','86']
  H_HDV['85_CO_-4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.87319758683699','-0.005953101409037','-0.734728752852653','0.079975429291422','0','6','86']
  H_HDV['85_THC_-4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.39431438459085','-0.002771826132879','-0.331874684620002','0.082213272521863','0','6','86']
  H_HDV['85_NOx_-4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0801123481484','0.049470351712455','305.410006251894','0.681558225394567','0.339820151119868','6','86']
  H_HDV['85_PM_-4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.20357594630941','-0.000976167433295','-0.161118707927565','0.079861280087233','0','6','86']
  H_HDV['85_FC_-4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','435.257582477476','0.051766091110995','19174.2521059878','0.77141908767945','8.9809656028743','6','86']
  H_HDV['85_CO_-4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.86628437652624','0.057150978594422','86.3101803085844','0.679086515070668','0.226121635483756','6','86']
  H_HDV['85_THC_-4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.05274114324523','0.063018734580719','7.89217341202953','0.455751231629949','0.156717102500635','6','86']
  H_HDV['85_NOx_-4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.9158263760318','0.039815359052385','817.994754185258','0.832254918022204','0.039786713184527','6','86']
  H_HDV['85_PM_-4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.57301555513348','0.061221108725864','1.63514443062546','0.320767474743278','0.114849038988209','6','86']
  H_HDV['85_FC_-4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','370.872231648784','0.04392144832293','35275.0156534254','0.862994296016782','2.07986156598487','6','86']
  H_HDV['85_CO_-4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.34790153337841','0.051186432677996','154.911935476815','0.760991337345475','0.174877921544635','6','86']
  H_HDV['85_THC_-4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.87177687852212','0.06256197301973','10.1509929913832','0.488992888065422','0.151237064504612','6','86']
  H_HDV['85_NOx_-4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3342829618363','0.033093214181866','1667.85072188663','0.950228354257083','-0.289839239302063','6','86']
  H_HDV['85_PM_-4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53589455298497','0.057726918399421','2.94131719210284','0.420019618142634','0.111169313179073','6','86']
  H_HDV['85_FC_-4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.413131501905','0.038670332477543','56247.1708436333','0.941771115968168','-3.49494640279559','6','86']
  H_HDV['85_CO_-6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.99028366135277','0.069698148421511','44.4267785643704','0.661707238778588','0.212181267498557','6','72']
  H_HDV['85_THC_-6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.1003250072184','0.068444228279189','6.6648467456848','0.456685326410258','0.151123992908963','6','72']
  H_HDV['85_NOx_-6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0360412130434','0.061642742903611','558.24528860639','0.794237849686361','0.234199683644817','6','72']
  H_HDV['85_PM_-6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.10439594000049','-0.001133846857987','-0.164361923895148','0.08552829378136','0','6','72']
  H_HDV['85_FC_-6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','362.280810252726','0.06108596436206','25681.4406491116','0.826614352504605','3.99930497996615','6','72']
  H_HDV['85_CO_-6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.67374589545923','0.062745341481448','70.2582934282725','0.657302914714069','0.185204116928685','6','72']
  H_HDV['85_THC_-6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.75959020294308','0.066871749365443','9.31057217887388','0.481706805972144','0.147173700642921','6','72']
  H_HDV['85_NOx_-6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.62310080544777','0.055547203066407','771.69767237712','0.83515399771454','0.196860447069563','6','72']
  H_HDV['85_PM_-6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.42516401357192','0.065290197858546','1.16337174267338','0.261208351318887','0.110205081113937','6','72']
  H_HDV['85_FC_-6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','295.428001783574','0.05616048242795','30408.6612491267','0.848003434016858','2.97449631895501','6','72']
  H_HDV['85_CO_-6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.23394695529735','0.059406802748773','82.4038541284433','0.666882933396936','0.175255299602337','6','72']
  H_HDV['85_THC_-6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.56671337116398','0.065929197309716','8.31005021736303','0.450175108184044','0.145559568600071','6','72']
  H_HDV['85_NOx_-6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.83947838198745','0.052828720421489','786.513312459756','0.837532857853875','0.196831515208173','6','72']
  H_HDV['85_PM_-6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.40558709233841','0.064091513475056','1.57520448915142','0.309066954780663','0.109804555353754','6','72']
  H_HDV['85_FC_-6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','268.205988265319','0.053707158754895','31442.5101351855','0.852220394610906','2.91258189810229','6','72']
  H_HDV['85_CO_2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.42626561378683','9.22884864013454','7.64961079482366','2.94511593321324','-0.030160578164168','6','86']
  H_HDV['85_THC_2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.173989213117103','4.45754327863895','5.7770778219917','2.22198122329656','-0.014724439582264','6','86']
  H_HDV['85_NOx_2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','43.3194039256257','0.035619429406377','-2.39052759879035','0.080456944005262','0','6','86']
  H_HDV['85_PM_2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.410771271221905','1.56551823440854','7.45015071372036','2.86662772640993','-0.024549435967859','6','86']
  H_HDV['85_FC_2_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1329.45442659564','0.713190290569463','-80.4474532831686','0.077870006442402','0','6','86']
  H_HDV['85_CO_2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.1968625251511','12.7729496516603','4.75206764285444','1.91342254524475','-0.013561014585661','6','86']
  H_HDV['85_THC_2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.389591531518201','3.97864402340352','7.5856977504948','2.93047344319505','-0.022748207311188','6','86']
  H_HDV['85_NOx_2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.5161428021065','0.019000442547989','-1.6923140857981','0.071474312329426','0','6','86']
  H_HDV['85_PM_2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.355524814845842','2.13081164891355','4.70443495904815','1.88173371539699','-0.018560423357092','6','86']
  H_HDV['85_FC_2_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1427.55386331517','0.829610593475966','-74.3390404181761','0.075816703844461','0','6','86']
  H_HDV['85_CO_2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.8999450423432','-0.037480947080985','-1.68204717177671','0.102109813134959','0','6','74']
  H_HDV['85_THC_2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.446813161326252','4.02523120122368','7.15134906485385','2.82260159668722','-0.021173977807892','6','74']
  H_HDV['85_NOx_2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000077199778696','0.013045584737179','-0.788414189507967','46.8574798459942','0','6','74']
  H_HDV['85_PM_2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.42417852542667','-0.006020491393396','-0.230749405022468','0.097055708735556','0','6','74']
  H_HDV['85_FC_2_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.004724862991113','0.75493806980898','-40.8696254505572','1400.17752119241','0','6','74']
  H_HDV['85_CO_4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.8922064647135','-0.023764531082075','-1.46082398927065','0.101324881269623','0','6','71']
  H_HDV['85_THC_4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.334605037235798','4.2927016738417','7.39076289467957','2.84957031367343','-0.022366496292887','6','71']
  H_HDV['85_NOx_4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','51.5990618785519','0.040831077466171','-2.87529286054196','0.094724511555282','0','6','71']
  H_HDV['85_PM_4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.08643464082055','-0.003466371302054','-0.225323607438102','0.098243550551308','0','6','71']
  H_HDV['85_FC_4_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1601.23360596252','0.489915469582571','-108.197173307804','0.095119108724752','0','6','71']
  H_HDV['85_CO_4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.7502866713981','-0.035790397407558','-1.68869392413206','0.103667951111651','0','6','65']
  H_HDV['85_THC_4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.524936831165387','3.99459045096219','8.33258924832248','3.28826467652988','-0.030838059800418','6','65']
  H_HDV['85_NOx_4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.68379849545891','1.60387148232034','-0.036434128752955','0','0','6','65']
  H_HDV['85_PM_4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.45123153530995','-0.004686446042668','-0.233003151814348','0.0981431129673','0','6','65']
  H_HDV['85_FC_4_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1816.62131436405','-0.766395273052012','-115.738847243969','0.112481421147444','0','6','65']
  H_HDV['85_CO_4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.942092209169','-0.055203846524457','-2.40435515871113','0.119239456472347','0','6','51']
  H_HDV['85_THC_4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.724757708230511','3.65352213347776','9.72796182903514','3.91639087831435','-0.047437791394348','6','51']
  H_HDV['85_NOx_4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000238057529275','0.024348273878097','-0.972064496989095','63.2252450964081','0','6','51']
  H_HDV['85_PM_4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000031396211229','0.003793415230624','-0.160641730785485','3.60812222599464','0','6','51']
  H_HDV['85_FC_4_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.012265558902397','1.38293911796619','-54.5542495669516','1792.18799996876','0','6','51']
  H_HDV['85_CO_6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.92708420251058','11.9352202579681','5.5290013668443','2.19108381108956','-0.012920176051621','6','70']
  H_HDV['85_THC_6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.625749940256109','40.4363075004123','-2.37119390002873','-0.496267031501433','0.132363761929572','6','70']
  H_HDV['85_NOx_6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','0','NOx','y=((a*(b**x))*(x**c))','77.6349894380913','1.00712088727266','-0.327528524280433','0','0','6','70']
  H_HDV['85_PM_6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.701567182368721','1.63875201131463','7.18022117656415','2.80725450557427','-0.024917469580144','6','70']
  H_HDV['85_FC_6_0'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1633.54252988863','3.65113021711956','-82.0986483870534','0.071089932780568','0','6','70']
  H_HDV['85_CO_6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.7797784758955','-0.033977755875589','-2.05786957842509','0.109657523805117','0','6','49']
  H_HDV['85_THC_6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.887993541765686','47.2723949699272','-3.08960999641991','-0.907182882649325','0.181244975615277','6','49']
  H_HDV['85_NOx_6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.154316885697','0.03714409128977','105.660190508369','0.487649952598746','45.9604922728747','6','49']
  H_HDV['85_PM_6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.02383219773018','-0.003213826611142','-0.292617181020772','0.107899796693461','0','6','49']
  H_HDV['85_FC_6_50'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','509.36522382222','0.075367220454213','1324.3033525352','0.302700746612755','1019.29784377825','6','49']
  H_HDV['85_CO_6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','5.13977149318829','-3.19987048495249','-0.979378985753978','0','0','6','36']
  H_HDV['85_THC_6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.03420351305343','3.1528258901112','15.2398750142227','6.40244993778765','-0.130781528530375','6','36']
  H_HDV['85_NOx_6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000944284932984','0.065058472115155','-1.65095579271156','83.0093368767388','0','6','36']
  H_HDV['85_PM_6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.203282671383434','0.018848514016514','-0.000183394053714','0','0','6','36']
  H_HDV['85_FC_6_100'] = [' 85','EURO0','TT/AT >28-34t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','591.63591505814','0.05079531544225','8708.14222510101','0.673259562571176','1291.25805738966','6','36']
  H_HDV['86_CO_0_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.61998038408863','0.055517319695187','7.61908584790785','0.231958661358232','1.09277203703333','6','86']
  H_HDV['86_THC_0_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.220515258879378','3.82903298768432','3.81891527613152','1.63919407954433','-0.006256719435782','6','86']
  H_HDV['86_NOx_0_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0928655901418','0.055076644247008','22.5578788456182','0.300726079377039','5.76025921016181','6','86']
  H_HDV['86_PM_0_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.039317589694831','2.03338918078803','4.22533380552642','1.82260207470076','-0.018374246988932','6','86']
  H_HDV['86_FC_0_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','530.936564175227','0.059417022527117','7398.77751090865','0.686179950585359','167.464044577646','6','86']
  H_HDV['86_CO_0_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.055853757957022','0.013166290170608','-0.000069287662736','0','0','6','86']
  H_HDV['86_THC_0_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.226247918281611','3.97469742174628','3.51934504058286','1.54716984620417','-0.003937336850124','6','86']
  H_HDV['86_NOx_0_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','50','NOx','y=exp((a+(b/x))+(c*log(x)))','4.08901492020422','-1.21725969680561','-0.485692826080843','0','0','6','86']
  H_HDV['86_PM_0_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.10589393156544','0.049662797806859','1.68198615866178','0.204574452205423','0.253108109263193','6','86']
  H_HDV['86_FC_0_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','512.124798789899','0.044010679845468','12940.5391055815','0.776698340022355','197.174373906039','6','86']
  H_HDV['86_CO_0_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.58306329825562','0.0499596661486','17.3458635492899','0.361818687148263','1.67774974789614','6','86']
  H_HDV['86_THC_0_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.213326372175621','3.79786948593247','3.70802956805107','1.60057084588854','-0.007435900342378','6','86']
  H_HDV['86_NOx_0_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000042058704982','0.007772337978793','-0.560145029455378','24.3388482483341','0','6','86']
  H_HDV['86_PM_0_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.22215174496974','0.048413234133255','1.85281883676226','0.250977924167904','0.311064766158195','6','86']
  H_HDV['86_FC_0_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','552.924479948851','0.034186039925252','128054.342898276','1.18142212329399','211.605957307876','6','86']
  H_HDV['86_CO_-2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.24237577332671','0.010923969331776','-0.486521389701848','0.059422046572795','0','6','86']
  H_HDV['86_THC_-2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.194091952084066','2.80505281247226','5.30553400864843','2.00629958058176','-0.000388555451105','6','86']
  H_HDV['86_NOx_-2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','0','NOx','y=((a*(b**x))*(x**c))','51.9362906410864','0.992334798393587','-0.640250278229758','0','0','6','86']
  H_HDV['86_PM_-2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.185349419488562','39.482050805722','-3.6961620715419','-0.579205567123061','0.120848410445527','6','86']
  H_HDV['86_FC_-2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','504.142215520591','0.056059235188857','25500.0670532436','0.878612646586665','53.9160194550001','6','86']
  H_HDV['86_CO_-2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','50','CO','y=(c+(a*exp(b*x)))','8.08632038730843','-0.067265510811967','0.604331669117885','0','0','6','86']
  H_HDV['86_THC_-2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.77568453428194','-0.003432455674693','-0.267828061841486','0.07966089627923','0','6','86']
  H_HDV['86_NOx_-2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.041923571914233','0.91211970740623','42.1001561276186','-0.563875077543836','0','6','86']
  H_HDV['86_PM_-2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.063012110892537','1.53539812278959','7.21096961684294','2.72587986281823','-0.022320656182906','6','86']
  H_HDV['86_FC_-2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','452.955524122736','0.038224402633145','22316.763828866','0.823180374131548','26.4752702870426','6','86']
  H_HDV['86_CO_-2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.73305110768587','0.051449750211179','84.6824939360502','0.709047156721008','0.507249912590102','6','86']
  H_HDV['86_THC_-2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.71992793828597','-0.004237559103023','-0.272079991039466','0.082866995389762','0','6','86']
  H_HDV['86_NOx_-2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.9504044279377','-0.068981638526658','-1.10535028543267','0.078295737766015','0','6','86']
  H_HDV['86_PM_-2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.15634450802565','-7.26427051302616','-1.4156507808741','0','0','6','86']
  H_HDV['86_FC_-2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-3.48483147343843','0.777432850116716','1267.00435437836','-0.493920506526772','0','6','86']
  H_HDV['86_CO_-4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.70059252785636','-0.006078757258954','-0.588720355256494','0.082896215172515','0','6','86']
  H_HDV['86_THC_-4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.59843190205084','-0.003187872543741','-0.277503138774622','0.084685127432135','0','6','86']
  H_HDV['86_NOx_-4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.9636088778578','0.053470859749977','96.0251892688844','0.520831056242289','0.321863197883934','6','86']
  H_HDV['86_PM_-4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.61634409206335','-11.4288705080458','-1.88839945531235','0','0','6','86']
  H_HDV['86_FC_-4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','387.878769748995','0.052700650231186','12667.6940989987','0.728731461437462','8.42169474237116','6','86']
  H_HDV['86_CO_-4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.04174532289258','0.053681843200139','37.9478971815033','0.564138807971351','0.13095745987267','6','86']
  H_HDV['86_THC_-4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.254225730500398','0.028063716105084','0.001092229459598','0','0','6','86']
  H_HDV['86_NOx_-4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.91510872026712','0.044272371359746','283.445630059597','0.682671979861922','0.145596732732952','6','86']
  H_HDV['86_PM_-4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.84941149526192','-8.93128773762212','-1.70898118820566','0','0','6','86']
  H_HDV['86_FC_-4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','334.234045895673','0.044192696523459','22987.2886980671','0.814296137844184','2.17869776740268','6','86']
  H_HDV['86_CO_-4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.64086641965133','0.046223717049027','79.3438689860288','0.671878390416748','0.069877145312675','6','86']
  H_HDV['86_THC_-4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.99915418707406','0.054062820071034','6.6582967611266','0.40062097692657','0.080069278963907','6','86']
  H_HDV['86_NOx_-4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.16787731793064','0.037669891615976','560.809896678897','0.792667572130614','-0.035918344930498','6','86']
  H_HDV['86_PM_-4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.08574307978682','-0.001984081714436','-0.169746255178781','0.089769682202226','0','6','86']
  H_HDV['86_FC_-4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','317.440742917607','0.038569674780589','38796.6948562167','0.901023730984623','-3.2245225512259','6','86']
  H_HDV['86_CO_-6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.29694726243094','0.069478172890492','17.2179529225732','0.528055755512626','0.133729556491995','6','72']
  H_HDV['86_THC_-6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.45891840620349','0.066870672917509','3.93051844805413','0.393612467253724','0.093358675588655','6','72']
  H_HDV['86_NOx_-6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8673800337902','0.066881280142832','229.64677404687','0.682662825063198','0.24483381442264','6','72']
  H_HDV['86_PM_-6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012992878894655','1.43198718089261','6.80643608892969','2.67522743883345','-0.020447307111537','6','72']
  H_HDV['86_FC_-6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','333.202824898006','0.062673083790971','17253.7390654237','0.786899610634443','4.33075138127963','6','72']
  H_HDV['86_CO_-6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.24216408945512','0.061634424478403','52.4153014895493','0.63189829765575','0.110926715520667','6','72']
  H_HDV['86_THC_-6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.10101190288861','0.064062476899541','8.87106509500076','0.488727300929444','0.088571784934248','6','72']
  H_HDV['86_NOx_-6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.29913126741998','0.05932100794491','347.56016538172','0.725853032464614','0.203956261834777','6','72']
  H_HDV['86_PM_-6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.2727416804951','0.066992731978931','1.31275353318765','0.238401053874894','0.060913918732788','6','72']
  H_HDV['86_FC_-6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','272.042508344007','0.056864811928704','22239.1960808527','0.818473465968149','3.16822424030436','6','72']
  H_HDV['86_CO_-6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.92292549222283','0.057606553206947','75.7454598206489','0.683450391557542','0.100370542497587','6','72']
  H_HDV['86_THC_-6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.90791527419275','0.062416061408393','8.20711554697809','0.46196368661352','0.086811288768257','6','72']
  H_HDV['86_NOx_-6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.54834702634692','0.056566797652829','357.89986364744','0.73040561742038','0.202121555991456','6','72']
  H_HDV['86_PM_-6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.22253264359381','0.065671971120722','1.74356451764423','0.296654206451704','0.060456486880812','6','72']
  H_HDV['86_FC_-6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','247.504199075794','0.053864614005121','25434.3501913023','0.839410687596056','2.91118344842014','6','72']
  H_HDV['86_CO_2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.3943036081413','0.013031996672362','-0.711222077198704','0.0676845146186','0','6','86']
  H_HDV['86_THC_2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.285676113640089','3.32869912298978','6.30004205409146','2.42686066103718','-0.020934190646979','6','86']
  H_HDV['86_NOx_2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.4996476701582','-0.008529412152043','-2.31328213903566','0.100505144741772','0','6','86']
  H_HDV['86_PM_2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.337118275250827','23.6189731216437','-3.27050139990972','-0.597219011602633','0.128225739846558','6','86']
  H_HDV['86_FC_2_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.044129635973277','1.78531756100545','1986.47134741163','-0.516035592973496','0','6','86']
  H_HDV['86_CO_2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','50','CO','y=(c+(a*exp(b*x)))','10.7605370209623','-0.082352322556039','2.34959377950591','0','0','6','86']
  H_HDV['86_THC_2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.654561479591814','21.3077910776196','-1.65422909126474','-0.173835155885856','0.099557935192187','6','86']
  H_HDV['86_NOx_2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.4191695936979','-0.031744305036008','-1.62255474345369','0.088036471313335','0','6','86']
  H_HDV['86_PM_2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.19360975092143','0.002914207754788','-0.124177111524821','0.06660692409194','0','6','86']
  H_HDV['86_FC_2_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1151.36047338643','0.644982610724654','-52.0832656645391','0.070159977304078','0','6','86']
  H_HDV['86_CO_2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.052193722617804','0.007622196998863','-0.000047488091081','0','0','6','77']
  H_HDV['86_THC_2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.607026027528461','3.15309086393953','6.30873793621319','2.4732192757384','-0.015088214820939','6','77']
  H_HDV['86_NOx_2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000057523050242','0.010485913979228','-0.677166712756681','35.0414810814562','0','6','77']
  H_HDV['86_PM_2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000008940834376','0.001469075928946','-0.08098295231748','2.16031485122221','0','6','77']
  H_HDV['86_FC_2_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','2','100','FC','y=((a*(b**x))*(x**c))','1777.11689232477','1.0023792308748','-0.310526516848218','0','0','6','77']
  H_HDV['86_CO_4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.34343704609881','78.8213680778395','-3.13168012862938','-0.960259624918084','0.178094815706192','6','71']
  H_HDV['86_THC_4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.499461414688402','3.18902394436661','7.43246971002712','2.8640545715011','-0.022966420170837','6','71']
  H_HDV['86_NOx_4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','39.4494930166052','-0.011012411095622','-2.42929366812199','0.106049782219976','0','6','71']
  H_HDV['86_PM_4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.460752129222032','18.709089169254','-3.52037516678027','-0.883043461994099','0.155178915736812','6','71']
  H_HDV['86_FC_4_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','0','FC','y=(c+(a*exp(b*x)))','852.336831632636','-0.101071371352689','445.337509192582','0','0','6','71']
  H_HDV['86_CO_4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.7396087648427','-0.029878970985523','-1.59484054229964','0.133256898984458','0','6','66']
  H_HDV['86_THC_4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.743283492790993','2.96697250398168','8.00150315190429','3.13030964392559','-0.028069796923003','6','66']
  H_HDV['86_NOx_4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.0877325233617','0.051763165290314','38.3833236271024','0.378415961287987','22.7042029274941','6','66']
  H_HDV['86_PM_4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000010466915109','0.001569642717976','-0.077840727298793','2.03949268994472','0','6','66']
  H_HDV['86_FC_4_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','547.486056503224','0.069543673515402','4391.837443864','0.609227242734289','666.870545242338','6','66']
  H_HDV['86_CO_4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0781454528671','0.019192493570091','13.9447011661231','0.238470394503595','-0.238316567461972','6','53']
  H_HDV['86_THC_4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.08402618734417','2.62163735860468','9.52764656014899','3.82452172956969','-0.043593747559625','6','53']
  H_HDV['86_NOx_4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000204427972974','0.023629124180947','-1.03505578780465','48.2859311603079','0','6','53']
  H_HDV['86_PM_4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000016738721143','0.00200218481127','-0.083994528408789','2.30553042053772','0','6','53']
  H_HDV['86_FC_4_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','542.220396150134','0.014060246489989','2518.93005622599','0.388745640513578','623.669779013516','6','53']
  H_HDV['86_CO_6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.72178243314549','6.89898101363677','8.16971162011736','3.26992172765285','-0.028701565873332','6','70']
  H_HDV['86_THC_6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.797322529410582','31.0738651377542','-2.40226232783025','-0.467465194016808','0.127888503187971','6','70']
  H_HDV['86_NOx_6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','44.8397706504544','-0.02738838741769','-2.63106402299536','0.115597012315523','0','6','70']
  H_HDV['86_PM_6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.0577563316878','0.00856704562902','-0.103471980460112','0.055967974360809','0','6','70']
  H_HDV['86_FC_6_0'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2283.29413381673','1.00920826491659','-0.463898847805465','0','0','6','70']
  H_HDV['86_CO_6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.4584952075061','-0.079990967785037','-1.8814268225734','0.165919367025132','0','6','50']
  H_HDV['86_THC_6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.19449349478545','28.7229293880527','-2.75060099787747','-0.785717765919368','0.170887518890026','6','50']
  H_HDV['86_NOx_6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.80786154134963','1.0183409160122','-0.099981004920485','0','0','6','50']
  H_HDV['86_PM_6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000017592679907','0.002085511986691','-0.083419401129982','2.20197670973435','0','6','50']
  H_HDV['86_FC_6_50'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','423.164147396773','0.061276761317366','2863.76855150518','0.465199869934664','901.585236245924','6','50']
  H_HDV['86_CO_6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','32.3712458133051','-0.078052267272891','-5.93663608822754','0.254211005561657','0','6','37']
  H_HDV['86_THC_6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.31953858933503','18.9309642277836','-2.29146846776211','-0.751877100463249','0.161547639408619','6','37']
  H_HDV['86_NOx_6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','90.4586135645545','-0.199968141105943','-12.3171825475524','0.301556726779373','0','6','37']
  H_HDV['86_PM_6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.52970166133698','-0.014544715117215','-0.460963126848317','0.273069716089367','0','6','37']
  H_HDV['86_FC_6_100'] = [' 86','EURO1','TT/AT >28-34t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1283.45804581135','0.009244000810307','7561.55123012708','0.619298960978328','289.328430994829','6','37']
  H_HDV['87_CO_0_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.7207833035365','0.055170154897768','6.47893009432396','0.284936764780029','0.935645649605029','6','86']
  H_HDV['87_THC_0_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.087979815737497','2.62704889004987','3.57542790668691','1.57625930818414','-0.009156874593394','6','86']
  H_HDV['87_NOx_0_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.9023338689313','0.053584605019378','39.1359648159598','0.380036681698223','5.95463978406103','6','86']
  H_HDV['87_PM_0_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.46220554170613','0.197928253509651','-0.001320490385387','0','0','6','86']
  H_HDV['87_FC_0_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001144113368009','0.000126225082853','-0.000000815115645','0','0','6','86']
  H_HDV['87_CO_0_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.088976120558548','0.015934254558572','-0.000093995459996','0','0','6','86']
  H_HDV['87_THC_0_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.703973143893307','0.027395996416828','2.29085254830178','0.117583548484425','0.182062132543384','6','86']
  H_HDV['87_NOx_0_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.6878815539482','-0.046759773167201','-1.28587747112892','0.073343134115237','0','6','86']
  H_HDV['87_PM_0_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000089342870846','-0.010944324415979','0.478061518719274','0','0','6','86']
  H_HDV['87_FC_0_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','438.345363992542','0.040391570707817','55150.0942076813','1.0699679217288','186.63575094139','6','86']
  H_HDV['87_CO_0_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.36063498523283','0.047426556096717','-0.265878062442636','0.030419286074165','0','6','86']
  H_HDV['87_THC_0_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.644028731743877','0.02083447743471','2.32073322644587','0.112377719967489','0.143361343110821','6','86']
  H_HDV['87_NOx_0_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000048974007406','0.009111807643991','-0.641114725225833','25.8600493608013','0','6','86']
  H_HDV['87_PM_0_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000000490917553','0.000041183532023','-0.010751286759693','0.565467970247094','0','6','86']
  H_HDV['87_FC_0_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','487.166155570038','0.0293356104103','2507986.71538513','1.73166493078092','192.147802726501','6','86']
  H_HDV['87_CO_-2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.52364027921928','0.009378430840503','-0.316630791826786','0.056939112468547','0','6','86']
  H_HDV['87_THC_-2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.157077269173118','3.04494397061926','1.79729673977783','0.882832375873847','0.032721253655783','6','86']
  H_HDV['87_NOx_-2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','0','NOx','y=((a*(b**x))*(x**c))','53.0902601935345','0.991917668127806','-0.625577896459417','0','0','6','86']
  H_HDV['87_PM_-2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.454199418140467','0.000739744294428','-0.022988598235829','0.051374015552498','0','6','86']
  H_HDV['87_FC_-2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','411.180156510908','0.052091870069593','24372.9589727975','0.904985073515594','48.027571556242','6','86']
  H_HDV['87_CO_-2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','50','CO','y=(c+(a*exp(b*x)))','5.61770132236753','-0.065963004471741','0.507530900907632','0','0','6','86']
  H_HDV['87_THC_-2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','50','THC','y=(1/(a+(b*(x**c))))','0.39558781127167','0.018938997947009','1.35101155173205','0','0','6','86']
  H_HDV['87_NOx_-2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.050367867951725','0.886912249224813','43.6821255033531','-0.558520112421787','0','6','86']
  H_HDV['87_PM_-2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','50','PM','y=(c+(a*exp(b*x)))','0.465875529779496','-0.058284780607421','0.058436263239571','0','0','6','86']
  H_HDV['87_FC_-2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','50','FC','y=((a*(b**x))*(x**c))','900.019082345615','0.982234565603608','-0.35990097967323','0','0','6','86']
  H_HDV['87_CO_-2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.65238588271221','0.046331265180948','240.874931622278','0.893742215224112','0.408332899950659','6','86']
  H_HDV['87_THC_-2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.317787497352397','0.046086466356425','0.000713335929199','0','0','6','86']
  H_HDV['87_NOx_-2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.8647692722992','-0.069921804248282','-1.15294040568142','0.077082050965035','0','6','86']
  H_HDV['87_PM_-2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.431716348856236','0.041458942603009','36.3204934002636','1.01298049219701','0.050864654262572','6','86']
  H_HDV['87_FC_-2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','556.810026195311','-1.7225933833922','-26.5230057369556','0.065116093280204','0','6','86']
  H_HDV['87_CO_-4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.159790526706475','0.017762879583673','0.000713884456679','0','0','6','86']
  H_HDV['87_THC_-4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','0','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.329143227099159','0.047958582725365','0.001054709936262','0','0','6','86']
  H_HDV['87_NOx_-4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1172212628679','0.051915081815352','97.7989097544076','0.505561613624838','0.337592550651662','6','86']
  H_HDV['87_PM_-4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','1.90688904355946','-5.13646174979969','-1.21242055627684','0','0','6','86']
  H_HDV['87_FC_-4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','320.656593208011','0.0500508331541','9957.6426561873','0.723491731101532','7.67988955422431','6','86']
  H_HDV['87_CO_-4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.30047384634612','0.048501855442453','43.1407233536777','0.618804124347585','0.091139331860568','6','86']
  H_HDV['87_THC_-4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.035965093152094','15.4243775208838','-0.99549569898628','0.629108635302422','0.024511169787626','6','86']
  H_HDV['87_NOx_-4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.3215442198672','0.043838783902429','288.3576919814','0.676010669089534','0.161927388868261','6','86']
  H_HDV['87_PM_-4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','0.926325746654851','0.985530709109869','-0.505142161146786','0','0','6','86']
  H_HDV['87_FC_-4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','286.287236387241','0.041745994400344','20687.4951148845','0.835561112719702','1.57983413212568','6','86']
  H_HDV['87_CO_-4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.26285111953239','0.04081893580942','114.199719941808','0.752835542297017','0.025246633499972','6','86']
  H_HDV['87_THC_-4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','100','THC','y=((a*(b**x))*(x**c))','5.19748589443557','0.98446323838248','-0.664352848501664','0','0','6','86']
  H_HDV['87_NOx_-4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.57510259317323','0.037760536830265','559.567749958085','0.783215130478242','-0.018682330153934','6','86']
  H_HDV['87_PM_-4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.34494869188593','0.038075241937811','3.11623705780006','0.59561276251496','0.017391966638828','6','86']
  H_HDV['87_FC_-4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','276.403775305272','0.036011059666893','37668.3138874327','0.93516544945627','-4.25199788547068','6','86']
  H_HDV['87_CO_-6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.46623208230691','0.068076328770915','13.9559981151868','0.50223402991095','0.111310300012476','6','72']
  H_HDV['87_THC_-6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.49179299526158','0.061196638913219','3.21826271581158','0.380253081871847','0.079198565004462','6','72']
  H_HDV['87_NOx_-6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.170944346048','0.066342232227162','203.245269186882','0.647968793655666','0.28560114586572','6','72']
  H_HDV['87_PM_-6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012665791981322','0.38732600713808','4.4204580363109','1.8053541143975','-0.006403125792286','6','72']
  H_HDV['87_FC_-6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','273.705218978079','0.060585347366467','10166.9412762975','0.73702676704599','4.7693142595303','6','72']
  H_HDV['87_CO_-6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.82135183006816','0.057839301843982','39.5057900570727','0.623718469409821','0.088666982730269','6','72']
  H_HDV['87_THC_-6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.36286377380345','0.059801335604253','4.10422580122929','0.407175575042322','0.077595064934498','6','72']
  H_HDV['87_NOx_-6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.57455047550878','0.058799910743971','337.52327088851','0.709654116929035','0.239704657229254','6','72']
  H_HDV['87_PM_-6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.280768125569791','0.050456966029561','0.422628834918712','0.264581528028371','0.027508656115756','6','72']
  H_HDV['87_FC_-6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','232.326247359535','0.055648705331499','18285.3981265042','0.82470230924826','3.94879897189576','6','72']
  H_HDV['87_CO_-6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.72973895508574','0.053392831703892','76.9812314588971','0.72582351722347','0.075784484385983','6','72']
  H_HDV['87_THC_-6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.2703644958012','0.05849513656462','3.80768564596697','0.382738883269175','0.076514355720455','6','72']
  H_HDV['87_NOx_-6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.83283830853652','0.056161123714583','346.578600873851','0.713538508511415','0.23543584878779','6','72']
  H_HDV['87_PM_-6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.321824804715385','0.051354043848636','0.816718432795927','0.395386364128726','0.027850626260796','6','72']
  H_HDV['87_FC_-6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','212.979247820538','0.052394407480457','21316.3923891295','0.847641317392431','3.60813356863113','6','72']
  H_HDV['87_CO_2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.078791242559631','0.016887058215326','-0.000128014525033','0','0','6','86']
  H_HDV['87_THC_2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.173328966951176','2.35441000581723','4.99696471223109','1.97314986593351','-0.009274632223724','6','86']
  H_HDV['87_NOx_2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.6481093589842','-0.01287594284556','-2.23262376188661','0.093819738343645','0','6','86']
  H_HDV['87_PM_2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.46373870483074','0.134216664894071','-0.001105826907023','0','0','6','86']
  H_HDV['87_FC_2_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.076075234097296','1.64453731809708','1567.80094817974','-0.465257444554698','0','6','86']
  H_HDV['87_CO_2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.086708979293879','0.010512125740197','-0.000055977163617','0','0','6','86']
  H_HDV['87_THC_2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.308134767729007','2.00144362636314','7.25685403855558','2.81877707340622','-0.022634323250064','6','86']
  H_HDV['87_NOx_2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','38.1891959985747','-0.046128255339429','-1.87370973663059','0.090611471181991','0','6','86']
  H_HDV['87_PM_2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000098215854407','-0.012260520945395','0.610653158980546','0','0','6','86']
  H_HDV['87_FC_2_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','972.82579195476','1.0337891047578','-35.7462753094176','0.0602803389719','0','6','86']
  H_HDV['87_CO_2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','100','CO','y=((a*(b**x))*(x**c))','19.4282815385981','0.991719045606289','-0.433991114630683','0','0','6','81']
  H_HDV['87_THC_2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.384517785555172','1.99624264250821','6.89657137863761','2.7177620664232','-0.019648807844194','6','81']
  H_HDV['87_NOx_2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','39.2517081829828','-0.040773158856143','-1.22304452428637','0.065174184533003','0','6','81']
  H_HDV['87_PM_2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.560156969924553','0.034927763610582','18.0922958926938','0.881902592911437','0.203643169067073','6','81']
  H_HDV['87_FC_2_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001813072797083','0.337821328517912','-21.3285218333196','1031.37227395494','0','6','81']
  H_HDV['87_CO_4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.5022973138358','-0.019097502196168','-0.955525422201129','0.126528426824397','0','6','71']
  H_HDV['87_THC_4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.28706031384438','2.17208709423107','6.82145090105692','2.66311598007776','-0.020111264687038','6','71']
  H_HDV['87_NOx_4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','15.1949033554062','41.2306947068537','1.43075140804302','1.00175486552267','0.024134672360331','6','71']
  H_HDV['87_PM_4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.4698456605043','0.079201768564351','-0.000537389548259','0','0','6','71']
  H_HDV['87_FC_4_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1780.51552320708','1.00820742567178','-0.463388921217547','0','0','6','71']
  H_HDV['87_CO_4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.69061228269379','0.013397822485379','7.57363328319571','0.150640381808296','-1.05092750063613','6','68']
  H_HDV['87_THC_4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.543305591182985','13.3139719671369','-1.77254772729426','-0.363450511229535','0.128280750775844','6','68']
  H_HDV['87_NOx_4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','17.0495846766957','114.975890322304','-0.351177107372065','0.633523099047201','0.001961942218597','6','68']
  H_HDV['87_PM_4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.15225363815432','0.049656829744135','-0.000205208433333','0','0','6','68']
  H_HDV['87_FC_4_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','69712.1249385533','1.17946436202272','497.339121729212','0.084245786049362','676.438224787253','6','68']
  H_HDV['87_CO_4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.14456042315501','0.032368050264202','21.6072695228166','0.347158834713118','0.529215291783774','6','55']
  H_HDV['87_THC_4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.54376341694552','2.03650511573303','6.61727146783564','2.67838703005051','-0.022423975418933','6','55']
  H_HDV['87_NOx_4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000240749232716','0.030250743382832','-1.35923573558527','51.9611649502188','0','6','55']
  H_HDV['87_PM_4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.80903078909242','0.019928854962009','2.95709823257531','0.476961226964839','0.06372602665393','6','55']
  H_HDV['87_FC_4_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','568.03461746703','0.0114000388098','3668.82597763556','0.518280202749064','563.33068263536','6','55']
  H_HDV['87_CO_6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.7222928901394','-0.040789977723818','-1.80715694666953','0.192449653069803','0','6','71']
  H_HDV['87_THC_6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.468635349817979','16.3180330241705','-1.96299539233541','-0.351512289401412','0.120523358626593','6','71']
  H_HDV['87_NOx_6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.54053501124188','0.021719909213003','23.9049870941511','0.133302219229843','18.0901713915577','6','71']
  H_HDV['87_PM_6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.844247333768471','-0.002671248410477','-0.058914498517549','0.137874878553482','0','6','71']
  H_HDV['87_FC_6_0'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1128.03132750221','2.63067720504513','-41.5540227492737','0.061390336793529','0','6','71']
  H_HDV['87_CO_6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','50','CO','y=((a*(b**x))*(x**c))','21.5394957258404','0.986855632926364','-0.392130585350014','0','0','6','51']
  H_HDV['87_THC_6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.576044780829257','1.94323582607295','8.00336076089222','3.23995252448635','-0.036370470462367','6','51']
  H_HDV['87_NOx_6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000358126690369','0.041782313087076','-1.6946888321377','55.0412147378012','0','6','51']
  H_HDV['87_PM_6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.88510517933223','0.04800730242105','-0.000268657500387','0','0','6','51']
  H_HDV['87_FC_6_50'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','336.384042805494','0.053667702129179','3537.67988148481','0.548196821847261','884.640111359926','6','51']
  H_HDV['87_CO_6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.0968077888558','0.043557264871288','34.0713799321714','0.376404508643769','1.19392152791158','6','40']
  H_HDV['87_THC_6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.895533227268841','1.55914429950929','10.6633343786831','4.4121812725181','-0.06400567828468','6','40']
  H_HDV['87_NOx_6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000591359642698','0.056410304908015','-2.00359212302721','67.0160805745636','0','6','40']
  H_HDV['87_PM_6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.714784382074367','0.029017166226765','2.27730269921035','0.355890369331339','0.314282475179698','6','40']
  H_HDV['87_FC_6_100'] = [' 87','EURO2','TT/AT >28-34t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','904.686701992604','0.006416629214876','6208.60128661866','0.608987793536565','544.25590479933','6','40']
  H_HDV['88_CO_0_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.74409139974623','0.056396209242199','6.98457813430972','0.198360272243834','1.04554104048586','6','86']
  H_HDV['88_THC_0_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.128705366705232','2.26040144989239','4.43466193663825','1.86172642094156','-0.010111102866342','6','86']
  H_HDV['88_NOx_0_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','0','NOx','y=((a*(x**b))+(c*(x**d)))','2.60154759001499','0.005478768050321','90.8449528834912','-0.873197145532297','0','6','86']
  H_HDV['88_PM_0_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.061024085976463','0.738621088219226','5.24052354421531','2.11861204978313','-0.015463314200783','6','86']
  H_HDV['88_FC_0_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000991035594325','0.00012266443946','-0.000000776528998','0','0','6','86']
  H_HDV['88_CO_0_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.28860115992556','42.7656441970754','-0.612685528524739','0.482174396860353','0.043858567836429','6','86']
  H_HDV['88_THC_0_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.13440428180731','2.22470955734239','4.25793242910298','1.80476194991232','-0.007760037805482','6','86']
  H_HDV['88_NOx_0_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','50','NOx','y=((a*(x**b))+(c*(x**d)))','51.9488494598802','-0.511512894389555','978.453733889286','-3.41098291902978','0','6','86']
  H_HDV['88_PM_0_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.46787238257802','0.050299000716222','0.546465938454197','0.167101319653247','0.106108717521267','6','86']
  H_HDV['88_FC_0_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','475.342914518653','0.041402449513413','9339.54786830961','0.777020198460383','191.751876465647','6','86']
  H_HDV['88_CO_0_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.21160993086089','0.055348488037981','9.68496640498004','0.295038850249969','1.50290839990113','6','86']
  H_HDV['88_THC_0_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.133279647652855','2.20132573360468','4.05044898764486','1.73048770710169','-0.006546428588255','6','86']
  H_HDV['88_NOx_0_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.6263196632548','0.029723765251977','71.2449320878014','0.385157589356065','4.96643346337811','6','86']
  H_HDV['88_PM_0_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.116746922139786','3.02966612378922','-0.471575154019973','0.47882411936812','0.035782616035468','6','86']
  H_HDV['88_FC_0_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','520.142465400782','0.030866525214523','649191.951341739','1.51567010503146','199.353360343167','6','86']
  H_HDV['88_CO_-2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.27035602342093','0.016234041492593','-0.536553314308114','0.05739487139729','0','6','86']
  H_HDV['88_THC_-2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.102215329891228','1.65641478995162','7.03508125183806','2.67049000769147','-0.021330205603857','6','86']
  H_HDV['88_NOx_-2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.2537733730582','0.054861949004903','91.7142447924496','0.457172003033715','1.4627602895444','6','86']
  H_HDV['88_PM_-2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','0','PM','y=(c+(a*exp(b*x)))','0.880434579764535','-0.07086061512671','0.086361304606026','0','0','6','86']
  H_HDV['88_FC_-2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','485.877243125677','0.053870398080286','5706.38717846655','0.692661253643968','54.1975465134042','6','86']
  H_HDV['88_CO_-2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.3597512218361','0.008055920615934','-0.542467545131065','0.059415321697672','0','6','86']
  H_HDV['88_THC_-2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.036071589103588','1.73235575508645','5.44124741810415','2.08698095655883','-0.009787994541162','6','86']
  H_HDV['88_NOx_-2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1146275640926','0.037144752175334','132.5674199222','0.494986432963756','0.64923120352419','6','86']
  H_HDV['88_PM_-2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.95245349696971','-6.76747036067636','-1.28835652432149','0','0','6','86']
  H_HDV['88_FC_-2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','50','FC','y=((a*(b**x))*(x**c))','1021.69761678864','0.982546738023081','-0.374991710005654','0','0','6','86']
  H_HDV['88_CO_-2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','100','CO','y=(c+(a*exp(b*x)))','8.95785954579051','-0.061414190681944','0.647056204430194','0','0','6','86']
  H_HDV['88_THC_-2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','4.24502691100979','-7.54608118585373','-1.46843564720015','0','0','6','86']
  H_HDV['88_NOx_-2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1874111975139','0.029433025658734','254.89974702412','0.603269695944819','0.081030121782724','6','86']
  H_HDV['88_PM_-2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.97184716988113','-6.95342819749543','-1.28762053278029','0','0','6','86']
  H_HDV['88_FC_-2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','617.779223052182','-1.86232546253059','-30.9004091678944','0.06808354719245','0','6','86']
  H_HDV['88_CO_-4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.95311938174483','0.003748743300706','8.21749113575036','0.078200526260883','-1.17185450680153','6','86']
  H_HDV['88_THC_-4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.25580590901811','-0.001553608342182','-0.173180204304101','0.083616888733861','0','6','86']
  H_HDV['88_NOx_-4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.921740910213','0.061615352032245','134.339790583523','0.527680484306125','0.466896623294979','6','86']
  H_HDV['88_PM_-4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.68958458896265','-8.72792999341859','-1.51725653646392','0','0','6','86']
  H_HDV['88_FC_-4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','379.505941793409','0.051470573320642','2993.42177658432','0.529970275066918','13.1519201707244','6','86']
  H_HDV['88_CO_-4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.24203081015192','-0.007873033002239','-0.586710785598541','0.0783469376628','0','6','86']
  H_HDV['88_THC_-4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.02749742231287','-0.001499993116627','-0.160114134008574','0.086411473907243','0','6','86']
  H_HDV['88_NOx_-4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.35528726688097','0.052143979906697','376.964028825999','0.69039437492259','0.355917829543649','6','86']
  H_HDV['88_PM_-4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.025999919124386','0.600552855337014','6.20427577867463','2.32273885962814','-0.013045561680661','6','86']
  H_HDV['88_FC_-4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','330.567608172139','0.043583915158129','10093.3412860726','0.721409657999906','7.54315255745324','6','86']
  H_HDV['88_CO_-4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.61374883626699','-0.009060970797035','-0.504017943459426','0.074058111168241','0','6','86']
  H_HDV['88_THC_-4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.26441391983946','0.060749385286425','5.57120441375312','0.499631678236506','0.068959577158355','6','86']
  H_HDV['88_NOx_-4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.11962678681276','0.044184946230574','695.415984635081','0.78693071313463','0.239129550168601','6','86']
  H_HDV['88_PM_-4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.79938843991358','-6.64401399821398','-1.29903888847745','0','0','6','86']
  H_HDV['88_FC_-4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','312.063174678104','0.037405470575848','22462.6580317938','0.850317490738474','1.36496667282607','6','86']
  H_HDV['88_CO_-6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.03137564314048','-11.8756890203149','-1.9429240721697','0','0','6','72']
  H_HDV['88_THC_-6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.04708884013031','-0.001385864241478','-0.172443901394234','0.091368322819097','0','6','72']
  H_HDV['88_NOx_-6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.53541924365294','0.073248680834414','234.510053109636','0.625658123476173','0.34996945271635','6','72']
  H_HDV['88_PM_-6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.28089378137381','-7.62973641684224','-1.43688599933868','0','0','6','72']
  H_HDV['88_FC_-6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','352.50603943751','0.0640236832724','6878.66338783365','0.693431550527237','12.1556581545','6','72']
  H_HDV['88_CO_-6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.24430105304246','0.061617752743084','24.9621421963404','0.616755100799701','0.186140757159187','6','72']
  H_HDV['88_THC_-6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.14463003889479','0.064854800885827','7.10984287604224','0.530017474832795','0.071013190394738','6','72']
  H_HDV['88_NOx_-6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.26216508978472','0.061811074581145','418.457153627859','0.705241158263301','0.30177703499055','6','72']
  H_HDV['88_PM_-6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.022454759955598','0.570985650211497','5.31406182972017','2.04651104335289','-0.010471994744686','6','72']
  H_HDV['88_FC_-6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','275.622199196333','0.05616811710047','12280.9145659146','0.761466098323403','10.3262111546101','6','72']
  H_HDV['88_CO_-6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.57095057491941','0.055973487465837','70.9457309209646','0.74385459215187','0.161258376530649','6','72']
  H_HDV['88_THC_-6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.00009103962038','0.062937304631433','6.77161421013217','0.514136051594117','0.069844601781172','6','72']
  H_HDV['88_NOx_-6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.46538095414131','0.05841745273368','485.94068720103','0.737279007201236','0.29887843112743','6','72']
  H_HDV['88_PM_-6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','100','PM','y=exp((a+(b/x))+(c*log(x)))','2.37711784813366','-5.45010912274162','-1.21648008202203','0','0','6','72']
  H_HDV['88_FC_-6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','253.136576393968','0.053010837786898','14802.3693605494','0.78939799479048','9.88253820330549','6','72']
  H_HDV['88_CO_2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.8039644801529','0.004511219313353','-0.690651910358403','0.071771004073809','0','6','86']
  H_HDV['88_THC_2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.167231069103212','2.08695791336616','5.75393603748573','2.23280401540285','-0.012277664983149','6','86']
  H_HDV['88_NOx_2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.23624506538879','56.3028351283896','1.71345640940089','1.33233652542257','-0.002080681085989','6','86']
  H_HDV['88_PM_2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.950514612505309','-0.000405242956621','-0.062405951559788','0.078535732346534','0','6','86']
  H_HDV['88_FC_2_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','976.227896580012','0.541302714833678','-53.9695251690134','0.077085868878074','0','6','86']
  H_HDV['88_CO_2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.0967386954473','0.00921634317864','-0.588331358538944','0.059153794092286','0','6','86']
  H_HDV['88_THC_2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.2581524734591','1.83286842228736','7.50329455137866','2.89341860281734','-0.022359752659262','6','86']
  H_HDV['88_NOx_2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.37986910357905','2.80968093371648','-0.237769935571108','0','0','6','86']
  H_HDV['88_PM_2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.856557987406414','0.001233144207295','-0.042272053300488','0.055628936035252','0','6','86']
  H_HDV['88_FC_2_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1020.35261119128','0.918345373352781','-38.1743297910684','0.060679654598913','0','6','86']
  H_HDV['88_CO_2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','100','CO','y=(1/(a+(b*x)))','0.059175849898517','0.007499113927973','0','0','0','6','82']
  H_HDV['88_THC_2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.306525929901177','1.81067711470665','7.32438439270848','2.86221301516482','-0.021814012973321','6','82']
  H_HDV['88_NOx_2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.57357979077779','2.12787399799829','-0.225831276587024','0','0','6','82']
  H_HDV['88_PM_2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.167913360210608','0.579649962562516','6.21661206076865','2.31709751901194','-0.014247807101678','6','82']
  H_HDV['88_FC_2_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','2','100','FC','y=(c+(a*exp(b*x)))','551.191206982453','-0.057972241973567','561.25891338003','0','0','6','82']
  H_HDV['88_CO_4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.543551012161','-0.019006825603408','-0.870430987034716','0.097794530569871','0','6','71']
  H_HDV['88_THC_4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.245372391362651','1.92799189618784','7.34987033747267','2.83151735956183','-0.022614367841882','6','71']
  H_HDV['88_NOx_4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8290746034132','0.009007137012626','41.0924700503538','0.169544692127422','5.615431254699','6','71']
  H_HDV['88_PM_4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-7.24391090497421','0.702973954662626','0.8438978249943','0.075700106861854','0.165206778299272','6','71']
  H_HDV['88_FC_4_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','1899.51314954707','1.00813112087983','-0.471230400853469','0','0','6','71']
  H_HDV['88_CO_4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.1636076398488','-0.037932180661126','-0.946527631694984','0.100140133083268','0','6','68']
  H_HDV['88_THC_4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.361766712697545','1.74948970962587','8.46587885914569','3.31486166970489','-0.032861314487285','6','68']
  H_HDV['88_NOx_4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','59.2615421571592','-0.067084934521796','-5.48403492036469','0.14182503658623','0','6','68']
  H_HDV['88_PM_4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.240848435537263','5.99568655359105','-2.91974485663807','-0.580775755986756','0.119199862501828','6','68']
  H_HDV['88_FC_4_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','1717.51976524158','1.00431246042314','-0.289252175873571','0','0','6','68']
  H_HDV['88_CO_4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','100','CO','y=((a*(b**x))*(x**c))','23.2978191435753','0.981995425533412','-0.366695552570355','0','0','6','56']
  H_HDV['88_THC_4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.436807408165615','1.7354269549255','7.75570396492286','3.0829864455329','-0.029367236939555','6','56']
  H_HDV['88_NOx_4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','70.5483300149612','-0.103946865932946','-7.04505483497299','0.16297749460349','0','6','56']
  H_HDV['88_PM_4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.03684976596498','-0.001676906599537','-0.058014150087968','0.087014670665773','0','6','56']
  H_HDV['88_FC_4_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','438.337686333983','0.023723475259937','3091.08228418832','0.513040164806434','759.856206582725','6','56']
  H_HDV['88_CO_6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.7646938063636','-0.028762836696049','-0.87763178873429','0.103094902227029','0','6','71']
  H_HDV['88_THC_6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.387101878202037','17.7488589836839','-2.13035902493637','-0.293469167491025','0.110032946648483','6','71']
  H_HDV['88_NOx_6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6159462389664','0.04290783944296','39.9810529041745','0.198079078772324','14.4944261635748','6','71']
  H_HDV['88_PM_6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.21201008452844','11.9315935746823','-3.88557513599262','-0.738280596750785','0.128430920322584','6','71']
  H_HDV['88_FC_6_0'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1202.83831929627','1.33683040078004','-50.0817590011827','0.073730097890986','0','6','71']
  H_HDV['88_CO_6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.9847769499398','-0.05246749324552','-1.1218202870577','0.10480084593452','0','6','54']
  H_HDV['88_THC_6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.427512499509237','1.79058061998062','7.57803194045728','3.01245242722153','-0.029339815158523','6','54']
  H_HDV['88_NOx_6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','68.8602275094279','-0.048087155472213','-6.08019317551146','0.14023449406098','0','6','54']
  H_HDV['88_PM_6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000007191498973','0.000921276150078','-0.040958223474663','0.963231026566586','0','6','54']
  H_HDV['88_FC_6_50'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','374.41575882598','0.050574118980824','2255.97424800983','0.470996250961907','883.138955035945','6','54']
  H_HDV['88_CO_6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','100','CO','y=((a*(b**x))*(x**c))','31.1698589645006','0.982659874271128','-0.450553072721087','0','0','6','41']
  H_HDV['88_THC_6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.75357114404822','13.0717544724216','-2.94169704455219','-1.13432719140938','0.213669063751479','6','41']
  H_HDV['88_NOx_6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','91.6267945838386','-0.215717881979698','-10.5497454542098','0.194960918479098','0','6','41']
  H_HDV['88_PM_6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000017514263169','0.001727055440403','-0.057565644357083','1.1551831116088','0','6','41']
  H_HDV['88_FC_6_100'] = [' 88','EURO3','TT/AT >28-34t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','730.410627636144','0.008030932335422','3044.72943484142','0.466510970039542','729.149792761129','6','41']
  H_HDV['89_CO_0_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045265684318838','0.649044447332522','4.23114475924232','1.79443651320056','-0.009359287885813','6','86']
  H_HDV['89_THC_0_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00486605942305','0.128889352188897','4.44649784913145','1.89204872709809','-0.012976667653873','6','86']
  H_HDV['89_NOx_0_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.71627172616714','0.049228760581535','38.8101828098928','0.407450150306272','2.74614857495521','6','86']
  H_HDV['89_PM_0_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.077165527655502','0.043487223477471','0.24146326949236','0.166245172547382','0.015837292543263','6','86']
  H_HDV['89_FC_0_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','459.937924458944','0.05750575176955','2992.8809230651','0.570797547354141','153.957426837073','6','86']
  H_HDV['89_CO_0_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.046941831760706','0.710456467541277','3.45708606591148','1.5316028333515','-0.00567645480855','6','86']
  H_HDV['89_THC_0_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005289570506919','0.125643392797202','4.43267177610075','1.90237564690549','-0.012207245171438','6','86']
  H_HDV['89_NOx_0_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.41227178230493','0.036832854833115','63.7633954640457','0.515828995579197','3.14770729773922','6','86']
  H_HDV['89_PM_0_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079150162380222','0.042207823667573','0.234166666922933','0.168382279630823','0.016184465401688','6','86']
  H_HDV['89_FC_0_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','447.111828539599','0.041320982215484','10387.6008447907','0.781149696261847','177.838607120057','6','86']
  H_HDV['89_CO_0_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.330727334645375','0.04077440358522','0.478578308676272','0.146617178753556','0.077046663389833','6','86']
  H_HDV['89_THC_0_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00517539584572','0.123660640096958','4.34447964174108','1.87768012641032','-0.012229989954511','6','86']
  H_HDV['89_NOx_0_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.62889647608699','0.028336383492566','217.989700371278','0.74121810253073','3.12395446123297','6','86']
  H_HDV['89_PM_0_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087904371245723','0.04379076856138','0.226033762843157','0.176358008016938','0.016414209260933','6','86']
  H_HDV['89_FC_0_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','481.864112895276','0.029611338101435','7154.70845221299','0.789963856076171','179.256451887704','6','86']
  H_HDV['89_CO_-2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.055088209232717','4.8188970832714','-2.30418962036215','-0.262017194875842','0.09440990495454','6','86']
  H_HDV['89_THC_-2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004148227678432','0.093386790336714','7.34399890099439','2.82188918267006','-0.029033681500447','6','86']
  H_HDV['89_NOx_-2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.10775210500201','0.049509661338914','80.6794578440985','0.544701355219439','0.765351483790064','6','86']
  H_HDV['89_PM_-2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005674335700855','0.182191367782279','5.1594524000129','2.16011743004984','-0.012120905431844','6','86']
  H_HDV['89_FC_-2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','459.679686793632','0.054973909324288','7920.03729415483','0.739334265879113','49.0121963855876','6','86']
  H_HDV['89_CO_-2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.26959192793081','-8.59060406173927','-1.50684823168616','0','0','6','86']
  H_HDV['89_THC_-2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002050207586693','0.095758116479477','5.81159868085041','2.23982315564745','-0.012823381587813','6','86']
  H_HDV['89_NOx_-2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.70446511567977','0.032338150381684','123.46370100238','0.598165510328415','0.211436839501834','6','86']
  H_HDV['89_PM_-2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.037627691617289','0.018640908230906','0.216985033268022','0.109069865507036','-0.000296327127889','6','86']
  H_HDV['89_FC_-2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','50','FC','y=((a*(b**x))*(x**c))','979.178987319042','0.981682855520229','-0.383577653660441','0','0','6','86']
  H_HDV['89_CO_-2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.620783271960366','-0.000728976318915','-0.042625472568568','0.078031512501407','0','6','86']
  H_HDV['89_THC_-2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','1.49379705980491','-7.6911692422441','-1.52039300132304','0','0','6','86']
  H_HDV['89_NOx_-2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.27256943739041','0.02523190355406','259.827244851235','0.725103736763428','-0.293173967072582','6','86']
  H_HDV['89_PM_-2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.042591632686558','0.016991656742219','0.212651949157165','0.11276763273741','-0.004082655114828','6','86']
  H_HDV['89_FC_-2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','580.913903706766','-1.77004844560153','-30.4392655550408','0.070837581154529','0','6','86']
  H_HDV['89_CO_-4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.245677276466823','0.001716422752086','0.55694677673225','0.078166552138675','-0.200358795462149','6','86']
  H_HDV['89_THC_-4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.127619890497237','-0.000087950734788','-0.009918688428589','0.084195478345992','0','6','86']
  H_HDV['89_NOx_-4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.8955488503695','0.052453542814832','86.2133576860291','0.552342238088865','0.114413767148289','6','86']
  H_HDV['89_PM_-4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.244840586186909','-0.000167069593857','-0.023530245190031','0.102952005084528','0','6','86']
  H_HDV['89_FC_-4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','367.262883179189','0.054209739710844','4589.11874379507','0.611227738580096','8.52132026148308','6','86']
  H_HDV['89_CO_-4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.529226785223136','-0.00042006551454','-0.039290460516658','0.080228019414619','0','6','86']
  H_HDV['89_THC_-4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.11650464727318','-0.000090851574406','-0.009418064246668','0.088098209413245','0','6','86']
  H_HDV['89_NOx_-4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.11042534527087','0.044044534350303','174.650322326715','0.663402453849572','0.023150445643655','6','86']
  H_HDV['89_PM_-4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.54639123154495','0.688789531099425','0.024926353447322','0','0','6','86']
  H_HDV['89_FC_-4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','304.826324378091','0.044929472091447','13078.2528106391','0.764268409824679','2.85936422311849','6','86']
  H_HDV['89_CO_-4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002782153132705','1.95317815568364','-0.482038474859206','0.553760808015328','0.030816334575164','6','86']
  H_HDV['89_THC_-4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.07108589236776','0.059906726337198','0.2711930782065','0.460923567311192','0.003178041208776','6','86']
  H_HDV['89_NOx_-4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.87002870091549','0.038305395745207','289.022483269275','0.747326718263722','-0.061459065163412','6','86']
  H_HDV['89_PM_-4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.10443507626346','0.058555343136595','0.464707124054621','0.369758134924411','0.003843555375917','6','86']
  H_HDV['89_FC_-4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','280.315773501546','0.038289529502569','26641.1836772997','0.877376181985672','-2.59242235170043','6','86']
  H_HDV['89_CO_-6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','0','CO','y=(c+(a*exp(((-1)*b)*x)))','0.486663745214725','0.075310761115963','0.012301435067951','0','0','6','72']
  H_HDV['89_THC_-6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.084959062303293','0.069368886291827','0.065422001219179','0.28398260964697','0.003545324020382','6','72']
  H_HDV['89_NOx_-6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.41649097808306','0.066207227576466','135.453867139745','0.646199677188153','0.06488800592351','6','72']
  H_HDV['89_PM_-6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.124956481285974','0.070378242369906','0.24195436753123','0.285501077834605','0.004395757553506','6','72']
  H_HDV['89_FC_-6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','322.529513184077','0.065681623704969','9841.41054166029','0.746803250299977','5.39761458755911','6','72']
  H_HDV['89_CO_-6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.31250831933736','0.063072349602362','5.18662251640766','0.744254560738597','0.009255474197105','6','72']
  H_HDV['89_THC_-6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067195025108345','0.064889522857879','0.440903092904149','0.550646594868774','0.003283572456036','6','72']
  H_HDV['89_NOx_-6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.33483330134355','0.059697165839313','203.719108823037','0.703441455802174','0.047217636948579','6','72']
  H_HDV['89_PM_-6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.107931202392595','0.067818360522599','0.592762677847196','0.430487843559225','0.004167023457641','6','72']
  H_HDV['89_FC_-6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','242.049506080563','0.057378817099949','16274.4844927885','0.803134816372959','3.84002707815528','6','72']
  H_HDV['89_CO_-6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.264415830913233','0.059471849296984','5.83408698808543','0.750113734386575','0.008540126286662','6','72']
  H_HDV['89_THC_-6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.059226362918933','0.062937102123911','0.468555890225163','0.550749203007631','0.003213431411898','6','72']
  H_HDV['89_NOx_-6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.97862298746617','0.056725153503167','237.525095852285','0.729921618834464','0.044118987708059','6','72']
  H_HDV['89_PM_-6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.097831006706846','0.06589443133815','0.697931301801014','0.457448333125319','0.004061411947839','6','72']
  H_HDV['89_FC_-6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','212.712849781116','0.053266899681411','16185.6040618712','0.80088842908899','3.32219443510145','6','72']
  H_HDV['89_CO_2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.106179733273075','0.517399705835442','6.73919171771293','2.63753513095721','-0.016052112165674','6','86']
  H_HDV['89_THC_2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006583858041941','0.11881835696914','5.63367875514877','2.21186066925756','-0.012335267095914','6','86']
  H_HDV['89_NOx_2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1557175608165','0.083985397331778','28.0241847241474','0.44040016891328','5.45000625793794','6','86']
  H_HDV['89_PM_2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.043240669551188','0.015468107884645','0.24439705428581','0.118313835755838','0.010018581791128','6','86']
  H_HDV['89_FC_2_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','940.081370912167','0.356448872762782','-54.0592986603973','0.08020801568436','0','6','86']
  H_HDV['89_CO_2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.05182139320254','-0.001946822460644','-0.103345767001693','0.129179386207518','0','6','86']
  H_HDV['89_THC_2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00985877321512','0.105829982395856','7.24112509703296','2.82452969264373','-0.022891456832171','6','86']
  H_HDV['89_NOx_2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','50','NOx','y=((a+(b*x))**((-1)/c))','-0.000009785912209','0.000002468672895','4.39583125089878','0','0','6','86']
  H_HDV['89_PM_2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065567140500343','0.028315600096267','0.24076859722599','0.141964856629609','0.017369749858345','6','86']
  H_HDV['89_FC_2_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','992.094740822673','0.545687234233438','-40.6133276033107','0.066864653244203','0','6','86']
  H_HDV['89_CO_2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.398183870693713','0.012932758870272','0.765878269924393','0.133192541902768','-0.034742694493353','6','83']
  H_HDV['89_THC_2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011624991730347','0.103698033844866','7.13728229568002','2.80482866573199','-0.021280949430962','6','83']
  H_HDV['89_NOx_2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.21862410948535','0.031261353399577','65.1604750264611','0.492015235433064','8.13381965465016','6','83']
  H_HDV['89_PM_2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.083965283542871','0.034684175977169','0.231857718926915','0.149128400461325','0.019072879221512','6','83']
  H_HDV['89_FC_2_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','2','100','FC','y=(c+(a*exp(((-1)*b)*x)))','529.930859168914','0.061048010426047','531.170759066675','0','0','6','83']
  H_HDV['89_CO_4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.995824952884908','-0.001189917361914','-0.089212308370892','0.116683858634907','0','6','71']
  H_HDV['89_THC_4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008503015020528','0.11210104786553','6.75192696478139','2.6218397587975','-0.019311849551249','6','71']
  H_HDV['89_NOx_4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.7850140954616','-0.024323469386114','-3.09847972789965','0.166411709442085','0','6','71']
  H_HDV['89_PM_4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057792934497396','0.014511283076603','0.241921450856802','0.124980267139135','0.00733006000451','6','71']
  H_HDV['89_FC_4_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','1843.38341825313','1.00822652552037','-0.481821566552741','0','0','6','71']
  H_HDV['89_CO_4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.10868586856956','-0.003354605306258','-0.095084356192166','0.121952172495781','0','6','68']
  H_HDV['89_THC_4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013184038193377','0.101945755307271','7.82439426175439','3.07583951276015','-0.027514806816409','6','68']
  H_HDV['89_NOx_4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.50360270166191','0.013757416491336','23.4848443924067','0.244473919233122','7.6670535328865','6','68']
  H_HDV['89_PM_4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.097918051897313','0.035902258077741','0.225358881951849','0.154454097360329','0.019576700192943','6','68']
  H_HDV['89_FC_4_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','516.73711426162','0.077635083301789','54086.7112961259','1.14426998035148','636.407128275361','6','68']
  H_HDV['89_CO_4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.15357560025489','-3.59818623702555','-0.970047125034695','0','0','6','57']
  H_HDV['89_THC_4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016204217963636','0.100921270096297','7.47329772959','2.99617488450007','-0.027460945550114','6','57']
  H_HDV['89_NOx_4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.78764272597187','0.017288083650197','34.6440851070162','0.326598809057606','10.3130620185102','6','57']
  H_HDV['89_PM_4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014144265332286','0.27171562065767','3.77237116941119','1.6911547037614','-0.009118469216379','6','57']
  H_HDV['89_FC_4_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','372.483639179119','0.029224841571787','2827.97928239087','0.492606928357254','762.100456412231','6','57']
  H_HDV['89_CO_6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.30775061664035','-0.003631322096581','-0.180558495038646','0.19376763770428','0','6','71']
  H_HDV['89_THC_6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015582026243152','0.847756656430005','-1.8610856038951','-0.236538018381619','0.106719446161522','6','71']
  H_HDV['89_NOx_6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.4605296832461','-0.039838828300952','-4.64895114563157','0.208743749659099','0','6','71']
  H_HDV['89_PM_6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.094782952393753','0.012165885644738','0.253547007756008','0.141580649706301','-0.01377185955113','6','71']
  H_HDV['89_FC_6_0'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','1870.91343780839','1.00820305498094','-0.415139444731953','0','0','6','71']
  H_HDV['89_CO_6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.2226310549448','-4.08910684475747','-0.975789074673242','0','0','6','54']
  H_HDV['89_THC_6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015725245333788','0.100126842471903','7.5715013177785','3.01658247584741','-0.028828739473952','6','54']
  H_HDV['89_NOx_6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.80148394306048','0.059519694041175','40.4747448972729','0.423872688689395','14.4208874498122','6','54']
  H_HDV['89_PM_6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.346346332264611','-0.000714715246189','-0.032978633659421','0.117129523475397','0','6','54']
  H_HDV['89_FC_6_50'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','50','FC','y=exp((a+(b/x))+(c*log(x)))','6.99127903493026','1.41447116600728','-0.066393852761254','0','0','6','54']
  H_HDV['89_CO_6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.68769917098468','-0.002552476171432','-0.146070912317601','0.103735163215418','0','6','42']
  H_HDV['89_THC_6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029227777066969','0.590794242111473','-2.19947802427378','-0.796852911378706','0.184443247669872','6','42']
  H_HDV['89_NOx_6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.8531307755454','0.021441645852377','43.0932661140922','0.357423990608423','13.3161281420833','6','42']
  H_HDV['89_PM_6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000005959676501','0.000631340048013','-0.02284172447747','0.338564366637523','0','6','42']
  H_HDV['89_FC_6_100'] = [' 89','EURO4','TT/AT >28-34t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','389.670435314107','0.017120643428802','2581.87800233948','0.444164371130157','995.825262859389','6','42']
  H_HDV['90_CO_0_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.045320157521829','0.659412535277282','4.26654071779326','1.80517464769269','-0.009616887270874','6','86']
  H_HDV['90_THC_0_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005171913825769','0.131001042664927','4.45742349486097','1.89404567804077','-0.012663378406304','6','86']
  H_HDV['90_NOx_0_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.05581174280425','0.047462808734945','24.6992990975606','0.372290013080747','1.59756032871668','6','86']
  H_HDV['90_PM_0_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.07850688189792','0.043397764380568','0.2460457822719','0.165832219529178','0.016013728606511','6','86']
  H_HDV['90_FC_0_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','471.555260301666','0.057205698316393','2684.10165146599','0.54294242161741','156.154859831633','6','86']
  H_HDV['90_CO_0_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.046780363064054','0.718708008048621','3.49517301175705','1.54171107691209','-0.005933548713942','6','86']
  H_HDV['90_THC_0_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005104050289603','0.127921714167695','4.46790763802045','1.91471328719407','-0.013071059725867','6','86']
  H_HDV['90_NOx_0_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.62670378515516','0.037542599016995','40.9921190902815','0.477017455055627','1.83472538868851','6','86']
  H_HDV['90_PM_0_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.081885209502251','0.042739442201806','0.2382926868991','0.169685846083873','0.016484108812823','6','86']
  H_HDV['90_FC_0_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','458.725172063481','0.041280611204842','7847.88785122146','0.729485173079659','180.02400512929','6','86']
  H_HDV['90_CO_0_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.349080154908106','0.042375443733489','0.470336111519288','0.148538484420995','0.07947924151497','6','86']
  H_HDV['90_THC_0_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005518761390939','0.126303392636212','4.32760401808436','1.87136131094887','-0.011491263285449','6','86']
  H_HDV['90_NOx_0_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.45845082481153','0.031834917980696','96.6644627930241','0.641363137665789','1.88896140446953','6','86']
  H_HDV['90_PM_0_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089506813609773','0.043948620642134','0.229563581982245','0.175650869856117','0.016716677542162','6','86']
  H_HDV['90_FC_0_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','497.87717436225','0.030401865471207','110852.753227416','1.19569688482094','184.152992890838','6','86']
  H_HDV['90_CO_-2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.056130587202636','5.14583163265597','-2.36552974481914','-0.270179360840396','0.094788067977487','6','86']
  H_HDV['90_THC_-2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004379755072579','0.095119226725046','7.3620325487033','2.82542741545431','-0.028660698881876','6','86']
  H_HDV['90_NOx_-2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.39461673467962','0.04959183752625','46.8627406911195','0.498154337510237','0.446581424755418','6','86']
  H_HDV['90_PM_-2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005798232554123','0.185253540354633','5.19187159414119','2.16971329341276','-0.012218086935983','6','86']
  H_HDV['90_FC_-2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','474.587162928894','0.05503728089868','7291.5552567185','0.720352297145456','49.9073811670153','6','86']
  H_HDV['90_CO_-2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.30533254181536','-8.63832555321836','-1.5117322576715','0','0','6','86']
  H_HDV['90_THC_-2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002066625896823','0.097660261972198','5.82552282368073','2.24353768771689','-0.0129126750295','6','86']
  H_HDV['90_NOx_-2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.14195006977173','0.033382263742153','69.0330708693904','0.549677420311662','0.135537817814077','6','86']
  H_HDV['90_PM_-2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.038210286517693','0.018490337151484','0.221312134035183','0.108830236793438','-0.000402781096539','6','86']
  H_HDV['90_FC_-2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','50','FC','y=((a*(b**x))*(x**c))','1021.98576278262','0.981736721515883','-0.390232194415413','0','0','6','86']
  H_HDV['90_CO_-2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.632898813628528','-0.000738936709913','-0.043493096818002','0.078032323555344','0','6','86']
  H_HDV['90_THC_-2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','100','THC','y=exp((a+(b/x))+(c*log(x)))','1.53060480706657','-7.74659074916408','-1.52446482528801','0','0','6','86']
  H_HDV['90_NOx_-2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.45695944381711','0.027131781562005','124.834152049395','0.65258791836151','-0.111375185554756','6','86']
  H_HDV['90_PM_-2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.247085468501827','-0.000231642201929','-0.022647560475351','0.101062750320347','0','6','86']
  H_HDV['90_FC_-2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','600.657907647783','-1.80599972142884','-31.8682654144789','0.071413604841955','0','6','86']
  H_HDV['90_CO_-4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.250107700994077','0.001712674265963','0.569244902921659','0.078079736890216','-0.204029994366674','6','86']
  H_HDV['90_THC_-4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000433026200675','0.094273777253306','5.64489967953788','2.17822519116867','-0.009430299367214','6','86']
  H_HDV['90_NOx_-4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.74060945490522','0.053606354762797','53.3940617831482','0.523082509235129','0.070568283285381','6','86']
  H_HDV['90_PM_-4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.249838645871447','-0.000169928244139','-0.023982349145627','0.102809900858685','0','6','86']
  H_HDV['90_FC_-4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','380.724958891081','0.054499902069204','4446.46928895371','0.602247980435845','8.82870630066188','6','86']
  H_HDV['90_CO_-4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.53994558378956','-0.000424260628401','-0.04004747129824','0.080100112456542','0','6','86']
  H_HDV['90_THC_-4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.118969758906312','-0.000092274895669','-0.009603988339372','0.087944881138537','0','6','86']
  H_HDV['90_NOx_-4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.26483247316859','0.045801027045237','99.5036768802056','0.622631656330302','0.0212923316887','6','86']
  H_HDV['90_PM_-4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.5400187223683','0.666525690164606','0.024669997178922','0','0','6','86']
  H_HDV['90_FC_-4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','310.846166706918','0.044674429477038','10569.380018381','0.723691180057345','2.71295339890741','6','86']
  H_HDV['90_CO_-4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003201817982225','6.00294891233013','-1.84010692920108','0.470764824081379','0.032745615625179','6','86']
  H_HDV['90_THC_-4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.072782343446033','0.059982483409663','0.264299792089655','0.454466463190282','0.003241559397313','6','86']
  H_HDV['90_NOx_-4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.1083074150522','0.040391330634283','150.967741791017','0.692325425336459','-0.02258894588866','6','86']
  H_HDV['90_PM_-4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.106405049624341','0.058552526012486','0.460749542321386','0.364654198557716','0.0039115479475','6','86']
  H_HDV['90_FC_-4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','289.236635924045','0.038662303008638','26032.8269742487','0.868096881159805','-2.31911670816583','6','86']
  H_HDV['90_CO_-6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','0','CO','y=(c+(a*exp(b*x)))','0.498685371231699','-0.075364870515493','0.012530413439236','0','0','6','72']
  H_HDV['90_THC_-6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00338529657589','0.357638176331172','-0.318880213793099','0.42881839149581','0.055690993080453','6','72']
  H_HDV['90_NOx_-6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.51063585528928','0.067305736807214','81.4749506660249','0.612205995783491','0.039569416210569','6','72']
  H_HDV['90_PM_-6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.127671568514636','0.0704344171937','0.237808181615616','0.279460760736583','0.004484364221859','6','72']
  H_HDV['90_FC_-6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','336.591146654449','0.06613603709342','10138.8262528368','0.748268714034725','5.64644263146836','6','72']
  H_HDV['90_CO_-6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.321760494999701','0.063439011287297','5.07549891209151','0.741341910438926','0.009513100424566','6','72']
  H_HDV['90_THC_-6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068976124278436','0.065090518169655','0.437966044894363','0.548080764188184','0.003356167350536','6','72']
  H_HDV['90_NOx_-6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.83052652844473','0.060954313171958','115.420075262766','0.662432354266337','0.028614813234893','6','72']
  H_HDV['90_PM_-6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.110476601266435','0.067961610762247','0.587344441398802','0.426154901935784','0.004256089572053','6','72']
  H_HDV['90_FC_-6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','251.172294277155','0.057702520716957','16396.7130990419','0.799884086873914','4.03092594850845','6','72']
  H_HDV['90_CO_-6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.271745564222386','0.059781557250537','6.59944979942765','0.767599247803815','0.008764880352962','6','72']
  H_HDV['90_THC_-6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.060532243821395','0.062961823596421','0.477556470316944','0.550930986463247','0.003273626034084','6','72']
  H_HDV['90_NOx_-6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.60239718993198','0.058021720050801','134.832708598249','0.688927542421122','0.026100131963356','6','72']
  H_HDV['90_PM_-6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.099868902272795','0.065935975705028','0.70742064115581','0.456403296071097','0.004139327022731','6','72']
  H_HDV['90_FC_-6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','219.424440526197','0.053395269130396','16402.1993739243','0.797303792536509','3.43853838156685','6','72']
  H_HDV['90_CO_2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.105567436911912','0.530851158046291','6.65929727454015','2.60578764266208','-0.015495384433688','6','86']
  H_HDV['90_THC_2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006692471358217','0.121155434913118','5.62432474728029','2.20768673461011','-0.012283720568735','6','86']
  H_HDV['90_NOx_2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','0','NOx','y=((a*(x**b))+(c*(x**d)))','1.30847133964458','0.152057154648711','38.1948181986786','-0.936895062956514','0','6','86']
  H_HDV['90_PM_2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.043955394037321','0.016087573510839','0.24871425953024','0.118636562618888','0.010686292043057','6','86']
  H_HDV['90_FC_2_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','966.964590210229','0.347999812554651','-55.8961353493545','0.080266340044587','0','6','86']
  H_HDV['90_CO_2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.06499088416964','-0.001902239428597','-0.104006418001545','0.127925544961876','0','6','86']
  H_HDV['90_THC_2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009817014558645','0.108378838805766','7.19379872750396','2.80797010753547','-0.022947695980933','6','86']
  H_HDV['90_NOx_2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.19145625479284','0.009030017372055','16.5474969362768','0.23629589881378','1.33632674710267','6','86']
  H_HDV['90_PM_2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065818830046004','0.028502260615695','0.24442969751836','0.140940073016803','0.017962548378026','6','86']
  H_HDV['90_FC_2_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1020.4770535127','0.542570489549562','-42.3674824763052','0.067248261217036','0','6','86']
  H_HDV['90_CO_2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.426649899829409','0.010785544433754','0.780400659304927','0.131054606752582','-0.071558551957898','6','84']
  H_HDV['90_THC_2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011738227479574','0.105417969401515','7.22035545638755','2.83353960829611','-0.021974718436599','6','84']
  H_HDV['90_NOx_2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.58020318240034','0.025442444631977','30.0557637810539','0.385444040679419','3.97549798765377','6','84']
  H_HDV['90_PM_2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.085789713578379','0.035286602021488','0.234635491887692','0.149559604357957','0.019763580563037','6','84']
  H_HDV['90_FC_2_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','2','100','FC','y=(c+(a*exp(b*x)))','552.302130302597','-0.062096865756236','539.700611401494','0','0','6','84']
  H_HDV['90_CO_4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.00456637788716','-0.001067326869139','-0.088442689830264','0.113769113828559','0','6','71']
  H_HDV['90_THC_4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008424175675566','0.114952464753649','6.6597838159476','2.58562051344748','-0.018683147034713','6','71']
  H_HDV['90_NOx_4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.8693206990177','-0.019117626610095','-2.95663669974279','0.194255761648319','0','6','71']
  H_HDV['90_PM_4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.057534397034553','0.014350437693363','0.246178692613389','0.124018939820598','0.007729613438362','6','71']
  H_HDV['90_FC_4_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','677.450370970622','0.09474442105421','139322.287212054','1.37917796260751','423.996618511894','6','71']
  H_HDV['90_CO_4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.526900945991446','0.01201547845252','0.745826691974119','0.144982151660513','-0.096269854042589','6','69']
  H_HDV['90_THC_4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013014121475974','0.105272662033045','7.57603506577815','2.9764121004021','-0.025240612457256','6','69']
  H_HDV['90_NOx_4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.61331657649279','0.0173177478518','19.5467791112689','0.275017355573717','4.00101501609421','6','69']
  H_HDV['90_PM_4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.090149378894168','0.028711127880071','0.24101955854671','0.147472101812532','0.015140896595775','6','69']
  H_HDV['90_FC_4_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','528.599373147576','0.076020078554431','34684.5522253254','1.05901588854445','643.917500137308','6','69']
  H_HDV['90_CO_4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.1997740455093','-3.75510787243985','-0.976354661319964','0','0','6','58']
  H_HDV['90_THC_4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016644127349798','0.102514141836569','7.52083708248706','3.01580206465751','-0.02750152353849','6','58']
  H_HDV['90_NOx_4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.96832314387204','0.024636485170774','20.2694605162265','0.292503813708292','5.67477135680117','6','58']
  H_HDV['90_PM_4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016566179364326','0.267309532014887','3.93764611957849','1.75167901172312','-0.009882954607545','6','58']
  H_HDV['90_FC_4_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','373.28779276954','0.036160687366463','3289.78982506788','0.525183581921561','804.581173750295','6','58']
  H_HDV['90_CO_6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.34169273760429','-0.003656377213474','-0.188938563734091','0.196647674646692','0','6','71']
  H_HDV['90_THC_6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015774197582404','0.87994259386825','-1.88571841884053','-0.239113473689052','0.106500737338809','6','71']
  H_HDV['90_NOx_6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.6628310630759','-0.037043082543187','-5.09456927897836','0.255226787227038','0','6','71']
  H_HDV['90_PM_6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.11602145306857','0.008058912433716','0.259703899027368','0.137405527402424','-0.038891202465169','6','71']
  H_HDV['90_FC_6_0'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','1935.71656052142','1.00829756641225','-0.42119282464037','0','0','6','71']
  H_HDV['90_CO_6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.18329638988783','-3.98972111136315','-0.9598438836171','0','0','6','55']
  H_HDV['90_THC_6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.021099291057563','0.731490392585395','-1.75222441786678','-0.27690734081246','0.119943370339885','6','55']
  H_HDV['90_NOx_6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.77082235005455','0.039608986313773','21.6607774308008','0.3194861668118','6.99924273297691','6','55']
  H_HDV['90_PM_6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.347522478503366','-0.000630635789056','-0.032395010330289','0.11372749454722','0','6','55']
  H_HDV['90_FC_6_50'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','50','FC','y=exp((a+(b/x))+(c*log(x)))','6.95427610435332','1.62859993941632','-0.053234112267955','0','0','6','55']
  H_HDV['90_CO_6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.67769417994038','-0.00251597449438','-0.14241644871195','0.101860011257326','0','6','42']
  H_HDV['90_THC_6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029059292020248','0.504221896917934','-1.91250558271668','-0.746045003131634','0.180410482184812','6','42']
  H_HDV['90_NOx_6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.05507208598678','0.025613090734529','21.333024054509','0.262924351493926','7.01802025597071','6','42']
  H_HDV['90_PM_6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.044001573511993','0.239002745505757','4.78115866336769','1.94615443614124','0.023635621453046','6','42']
  H_HDV['90_FC_6_100'] = [' 90','EURO5','TT/AT >28-34t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','371.84264003015','0.026236462958318','3109.05229359591','0.489199066571377','1073.53591568011','6','42']
  H_HDV['91_CO_0_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.40759999378364','0.058909202984417','12.5308754216037','0.256184439769866','1.5879701530523','6','86']
  H_HDV['91_THC_0_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.20095284402335','0.038529899421996','5.06091341156186','0.131957739169608','0.441935776735062','6','86']
  H_HDV['91_NOx_0_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.1185358044811','0.053745693301196','105.838909722185','0.50425362291127','8.49809936809615','6','86']
  H_HDV['91_PM_0_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.62666445671465','0.053908583278669','2.36701638060085','0.216724421508446','0.330739343035213','6','86']
  H_HDV['91_FC_0_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','720.567178459811','0.057924135796715','9583.50335824908','0.668629242648328','206.092342656816','6','86']
  H_HDV['91_CO_0_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.72841900547824','0.056786649735827','20.3581561398851','0.316857664080223','1.88366741905009','6','86']
  H_HDV['91_THC_0_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.187662315860534','6.65101446180206','3.3513731012113','1.55349666340177','-0.005209024760084','6','86']
  H_HDV['91_NOx_0_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','50','NOx','y=((a+(b*x))**((-1)/c))','0.001424041282828','0.000155435330122','1.77058736168716','0','0','6','86']
  H_HDV['91_PM_0_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.69866336239689','0.049725626502355','3.46830735628359','0.287772257912257','0.399152886457762','6','86']
  H_HDV['91_FC_0_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','692.367216912261','0.0422675179723','8971.72007663108','0.65057019563422','241.182021352936','6','86']
  H_HDV['91_CO_0_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.13975240941913','100.927679918868','-1.12267008838229','0.431991466860916','0.043833550495673','6','86']
  H_HDV['91_THC_0_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.30720551119818','0.043064168634866','5.51810998278488','0.150903341778694','0.425220702811027','6','86']
  H_HDV['91_NOx_0_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','100','NOx','y=(1/(a+(b*(x**c))))','0.028040721679364','0.000320439054485','1.13558807418857','0','0','6','86']
  H_HDV['91_PM_0_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.83329882196526','0.046188348896664','5.12577329187219','0.362510172567137','0.457747489970994','6','86']
  H_HDV['91_FC_0_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','744.853826987892','0.031503453578612','40909.3521276807','0.928392875700612','253.432665646926','6','86']
  H_HDV['91_CO_-2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.2995478583075','0.009563999211095','-0.846086868597424','0.065531338841582','0','6','86']
  H_HDV['91_THC_-2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.43968990356761','0.063692951787318','7.98427509943802','0.429383569890191','0.358565698030623','6','86']
  H_HDV['91_NOx_-2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','0','NOx','y=((a*(b**x))*(x**c))','67.5739863867094','0.991220686659018','-0.598683668790543','0','0','6','86']
  H_HDV['91_PM_-2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.98381296775542','0.062022194946314','1.06856897700085','0.169934496160653','0.200341192780417','6','86']
  H_HDV['91_FC_-2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','631.946211752871','0.053076329037645','31216.4132764717','0.850157909826465','63.3122629011963','6','86']
  H_HDV['91_CO_-2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6101690139985','0.060327711991489','160.840209877157','0.749770190344478','0.735528900172428','6','86']
  H_HDV['91_THC_-2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.97463700912621','0.057749462454449','10.1428756349103','0.420684640724602','0.234613381603293','6','86']
  H_HDV['91_NOx_-2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.044247127296116','1.02417359581224','53.1421660373432','-0.50592448950469','0','6','86']
  H_HDV['91_PM_-2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.99733026485325','0.055514499657177','5.20902631089637','0.41393542110179','0.176461218610327','6','86']
  H_HDV['91_FC_-2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','575.131504096717','0.035447502747633','40036.1670587954','0.861866180209187','20.6953495220712','6','86']
  H_HDV['91_CO_-2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.73003040289305','0.050563801622014','253.20791598953','0.77256031415142','0.622120139256347','6','86']
  H_HDV['91_THC_-2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.68759266822935','0.055932989323144','11.7368541895153','0.423862825709229','0.19584005650451','6','86']
  H_HDV['91_NOx_-2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.4485571386761','-0.114726192098432','-1.14956539093614','0.067124034413428','0','6','86']
  H_HDV['91_PM_-2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92699491583645','0.048557224361473','12.5620355445618','0.538481166645867','0.163523983384266','6','86']
  H_HDV['91_FC_-2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-2.96333307790153','0.878209021315899','1584.23754530977','-0.489412373811272','0','6','86']
  H_HDV['91_CO_-4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.32364940573845','0.063033812369109','52.6076968868459','0.598705337940305','0.298768836537025','6','86']
  H_HDV['91_THC_-4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.69660598154306','0.059348222661746','14.378988961718','0.476865941757213','0.170657966524293','6','86']
  H_HDV['91_NOx_-4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.3709172956514','0.048727986285921','429.399804952738','0.696526309896474','0.347376973069583','6','86']
  H_HDV['91_PM_-4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.104813935995196','6.09634073560428','0.471220007760297','0.658594082503824','0.037583880143895','6','86']
  H_HDV['91_FC_-4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','475.239909032619','0.050533059067035','24157.0141366198','0.785740925790438','8.66431033006982','6','86']
  H_HDV['91_CO_-4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.60166384855725','0.053685028926963','140.520270182952','0.693888833136876','0.206619217069371','6','86']
  H_HDV['91_THC_-4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.33902702863671','0.057974933203468','24.641084671569','0.556140311684818','0.152093998028132','6','86']
  H_HDV['91_NOx_-4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.1772360016634','0.038524065678488','1104.44049460185','0.850504951622847','-0.02191975467903','6','86']
  H_HDV['91_PM_-4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.71329780634622','0.055400798714372','7.14315252305141','0.453040909169237','0.112426419828491','6','86']
  H_HDV['91_FC_-4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','411.2098019851','0.042371203550687','45491.5141242195','0.887245492927352','0.141974895348563','6','86']
  H_HDV['91_CO_-4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.1214562419907','0.047698176914147','319.555187379366','0.810393381794856','0.13298193222702','6','86']
  H_HDV['91_THC_-4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.1601397506266','0.05764128699401','30.5243008013615','0.594244634082677','0.146869214130281','6','86']
  H_HDV['91_NOx_-4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8584463369242','0.031598464233796','2190.58733074983','0.970677008456546','-0.455810018040577','6','86']
  H_HDV['91_PM_-4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68143467533438','0.051830551209422','14.3567829014398','0.566861864251596','0.105425882864725','6','86']
  H_HDV['91_FC_-4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','394.437890709991','0.037110078701201','81417.3281672418','0.987349416799254','-6.86157980799606','6','86']
  H_HDV['91_CO_-6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.135567960469923','53.6561144686555','-0.810967276972261','0.480087760801151','0.046896707343913','6','72']
  H_HDV['91_THC_-6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.58147842149853','0.064779047880972','29.5820701611885','0.62141443203427','0.14157262033654','6','72']
  H_HDV['91_NOx_-6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.7826746603674','0.061324806194868','732.927286804162','0.809931163701525','0.215731235576379','6','72']
  H_HDV['91_PM_-6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.85402079130608','0.06534230331142','3.19494400224552','0.37164324331471','0.111665016499153','6','72']
  H_HDV['91_FC_-6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','413.350672536023','0.060333657571848','32731.6671172353','0.851923383773882','2.85279301387631','6','72']
  H_HDV['91_CO_-6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.70314349940998','0.060647621079597','155.66139675596','0.732345664065924','0.15453984367569','6','72']
  H_HDV['91_THC_-6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.12933189195783','0.062398840096794','33.6855770869731','0.622289939582823','0.135078371658239','6','72']
  H_HDV['91_NOx_-6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.0940603465648','0.055105987477133','1008.98601304578','0.855323364346791','0.174640752709345','6','72']
  H_HDV['91_PM_-6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68232498603634','0.06161351879155','6.60315366426057','0.466260201048784','0.107051886425329','6','72']
  H_HDV['91_FC_-6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','338.315439720846','0.055256887259675','39426.8437822882','0.876005383671545','1.70752718308894','6','72']
  H_HDV['91_CO_-6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.04611396178525','0.056355937665085','192.689625660424','0.740874584297236','0.135442904334224','6','72']
  H_HDV['91_THC_-6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.93542137661863','0.061766255222628','29.2851392645856','0.592839669554291','0.134600464375496','6','72']
  H_HDV['91_NOx_-6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2086271130487','0.052077614557556','1218.09530899532','0.886788710380326','0.171492055442186','6','72']
  H_HDV['91_PM_-6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.61675706824536','0.059572232296349','10.4459840425584','0.524790919044225','0.105731644169235','6','72']
  H_HDV['91_FC_-6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','308.837586111533','0.052785615715038','43151.7158687783','0.88994359239114','1.66126295305338','6','72']
  H_HDV['91_CO_2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.07154789184792','11.8804312290926','5.24752494836001','1.95639049311326','0.01530124528233','6','86']
  H_HDV['91_THC_2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.068715154666443','6.51418470095318','4.26864484712639','1.74254935172835','-0.012355061058748','6','86']
  H_HDV['91_NOx_2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-25.0876515572215','0.037956963942666','54.8215706701679','0.062381294934429','18.0689083683986','6','86']
  H_HDV['91_PM_2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.512997212465057','2.21117091732792','4.73078631584339','1.80418088107907','0.019394502426196','6','86']
  H_HDV['91_FC_2_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1503.08765781041','0.510956638200198','-91.9795188606921','0.078796127334018','0','6','86']
  H_HDV['91_CO_2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.24656453122535','0.041102052249364','13.9989193960014','0.144756262770805','2.16316446659898','6','86']
  H_HDV['91_THC_2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.416868333205518','5.39544808805018','6.00853370906694','2.40711781134715','-0.0141446149207','6','86']
  H_HDV['91_NOx_2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','50','NOx','y=(c+(a*exp(b*x)))','26.7036113404697','-0.07907850797736','26.6277283955657','0','0','6','86']
  H_HDV['91_PM_2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.25084222532086','0.042524064482281','2.21777845090915','0.146639716585976','0.643620943114204','6','86']
  H_HDV['91_FC_2_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','50','FC','y=(c+(a*exp(b*x)))','1133.48469998931','-0.086722407277328','575.0578833159','0','0','6','86']
  H_HDV['91_CO_2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.5159997143243','-1.8215409731389','-0.824457371732696','0','0','6','74']
  H_HDV['91_THC_2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.530777793919711','5.08102033618182','6.58383336104062','2.6522143858169','-0.019151549751544','6','74']
  H_HDV['91_NOx_2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000069694190739','0.012380795786007','-0.807310409037457','54.6728276454865','0','6','74']
  H_HDV['91_PM_2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','100','PM','y=((a*(b**x))*(x**c))','7.22157861776581','1.00115951943274','-0.524485072088007','0','0','6','74']
  H_HDV['91_FC_2_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','2','100','FC','y=((a*(b**x))*(x**c))','2604.02321568607','1.003581641334','-0.350368062962641','0','0','6','74']
  H_HDV['91_CO_4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.98631974479577','13.0683210148186','5.82088984531493','2.34568244953115','-0.017192090812363','6','71']
  H_HDV['91_THC_4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.229935844111055','6.24233858280241','5.12985340102844','2.06920650606756','-0.010491131534179','6','71']
  H_HDV['91_NOx_4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','60.3121776713816','0.04325928614533','-3.66657968989263','0.098549371486084','0','6','71']
  H_HDV['91_PM_4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.524625479810287','2.30467832912383','5.50206242865777','2.28258316225406','-0.024721369947229','6','71']
  H_HDV['91_FC_4_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','0','FC','y=(c+(a*exp(b*x)))','1356.39865661899','-0.100127419813637','529.022291662654','0','0','6','71']
  H_HDV['91_CO_4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.63959221852859','0.031701331753634','16.3918552668396','0.13627259184598','2.2999042400136','6','65']
  H_HDV['91_THC_4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.639356229836119','5.12392704012796','7.42955138953174','3.0063027116192','-0.025349289629396','6','65']
  H_HDV['91_NOx_4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.879980284589','1.45969948615412','-0.044827426649171','0','0','6','65']
  H_HDV['91_PM_4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.7701792315471','0.058364212515618','1.9987851593984','0.204568147002406','0.98333106061605','6','65']
  H_HDV['91_FC_4_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','50','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','891.88859749891','-0.691949900485538','0.080673427124092','0','0','6','65']
  H_HDV['91_CO_4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0379230895653','0.026477122247035','22.9343581104171','0.202366012072705','0.544141988413545','6','49']
  H_HDV['91_THC_4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.755592580521239','4.78776581652968','7.95534525400482','3.23767416583176','-0.034622543427165','6','49']
  H_HDV['91_NOx_4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000302900898773','0.027347043470602','-1.0276981179695','74.6972438065517','0','6','49']
  H_HDV['91_PM_4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.19589834768169','0.020066463533963','4.25181203158101','0.254169416414562','0.449109831744776','6','49']
  H_HDV['91_FC_4_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1935.07529552048','0.005203422424922','2897.99085181398','0.372567913389213','-343.361783436057','6','49']
  H_HDV['91_CO_6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.7839721009562','-0.043174573081016','-2.86324321346242','0.140763695631656','0','6','71']
  H_HDV['91_THC_6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.69881180453086','0.004465399270066','-0.673183648871213','0.081349979173552','0','6','71']
  H_HDV['91_NOx_6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','0','NOx','y=((a*(b**x))*(x**c))','93.2229372929494','1.0080790412001','-0.365078036472324','0','0','6','71']
  H_HDV['91_PM_6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.32339454086974','-0.005089803263541','-0.405501154298535','0.128404295164972','0','6','71']
  H_HDV['91_FC_6_0'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1855.39830685927','5.23774768158751','-90.1898440058861','0.064436980207301','0','6','71']
  H_HDV['91_CO_6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.0339180484146','-0.058853010904015','-2.22977382144052','0.113261785426461','0','6','48']
  H_HDV['91_THC_6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.768094473452969','4.86977575008149','8.21022694585506','3.34704730915275','-0.037236022987927','6','48']
  H_HDV['91_NOx_6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4798173940918','0.026137090296212','140.764890754077','0.507366163931643','52.5037224384071','6','48']
  H_HDV['91_PM_6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.41943966100537','0.037167742249852','2.55082047819652','0.171809378888985','1.09949593067636','6','48']
  H_HDV['91_FC_6_50'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','50','FC','y=exp((a+(b/x))+(c*log(x)))','7.35177368301735','1.79950106336783','-0.079369631596191','0','0','6','48']
  H_HDV['91_CO_6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','4.67052775311521','-1.7184278663449','-0.794085324761348','0','0','6','35']
  H_HDV['91_THC_6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.1711252897493','4.18374421268118','9.80913148726432','4.15461963803138','-0.06342692537908','6','35']
  H_HDV['91_NOx_6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001258653550969','0.083806571356103','-2.01984976711714','100.354524641831','0','6','35']
  H_HDV['91_PM_6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.34113007364707','-0.02274990625357','-0.681214864022351','0.182124314039613','0','6','35']
  H_HDV['91_FC_6_100'] = [' 91','EURO0','TT/AT >34-40t 80ties','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1708.90056569976','0.007068863834865','4592.66719961242','0.473616709101701','351.780694827504','6','35']
  H_HDV['92_CO_0_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.57606922496226','0.0547091976346','10.0402776353221','0.228751858907595','1.22421452597575','6','86']
  H_HDV['92_THC_0_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.167580301205255','5.50019288827624','3.23453143362562','1.49734594945031','-0.006689553064209','6','86']
  H_HDV['92_NOx_0_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.6598079456504','0.054543103026868','53.8514067126198','0.39500508056784','6.19011830207354','6','86']
  H_HDV['92_PM_0_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.33045586379659','0.050745745201548','2.71124052446045','0.177315975107174','0.22268237279591','6','86']
  H_HDV['92_FC_0_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','580.71019392959','0.057483634003831','13301.1890384165','0.748319959462196','176.135838917739','6','86']
  H_HDV['92_CO_0_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.72518927526776','0.05108379489595','18.1551906614336','0.305820971076524','1.60426961410819','6','86']
  H_HDV['92_THC_0_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.199485585757468','5.53962363879173','3.11112206256974','1.45221792059079','-0.004061627422907','6','86']
  H_HDV['92_NOx_0_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','50','NOx','y=(1/(a+(b*(x**c))))','0.020067721302217','0.007850844382245','0.594929474075154','0','0','6','86']
  H_HDV['92_PM_0_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.46918985952587','0.050368573606972','3.14198768151553','0.236673228529966','0.285754925946281','6','86']
  H_HDV['92_FC_0_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','575.961498445889','0.040815045742712','27753.1515191331','0.874295579326358','212.477989774916','6','86']
  H_HDV['92_CO_0_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.1064291630099','0.050812648427411','25.6824153238626','0.389700983541322','1.98475893744052','6','86']
  H_HDV['92_THC_0_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.81515553475928','0.036360360158333','4.16109705006172','0.140145226824358','0.410777536439417','6','86']
  H_HDV['92_NOx_0_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000042672536866','0.00793485806466','-0.597395345206292','27.7475305610173','0','6','86']
  H_HDV['92_PM_0_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.57292784342065','0.047509853706634','3.7687103595533','0.29145110280336','0.356550502461557','6','86']
  H_HDV['92_FC_0_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','644.336727253301','0.030428532105304','448919.511667341','1.36645802859276','223.054984123406','6','86']
  H_HDV['92_CO_-2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.3045087372954','0.008300993214072','-0.649799748533631','0.06478323595162','0','6','86']
  H_HDV['92_THC_-2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.269550012035684','8.48538073507069','0.807555387081103','0.648907900662068','0.041301451771605','6','86']
  H_HDV['92_NOx_-2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','0','NOx','y=((a*(b**x))*(x**c))','62.7645550321767','0.992222313212778','-0.670640248353555','0','0','6','86']
  H_HDV['92_PM_-2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.048181667126128','2.12650289139254','7.51172792614437','2.9459077098076','-0.034689256325636','6','86']
  H_HDV['92_FC_-2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','538.155875232432','0.053951907258819','21953.1073577857','0.819641523042582','53.8955591976159','6','86']
  H_HDV['92_CO_-2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.23873924288133','0.059564852141879','96.3046872859382','0.70477738586038','0.601855612247338','6','86']
  H_HDV['92_THC_-2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.162090563825109','0.023790577095631','0.000458028710301','0','0','6','86']
  H_HDV['92_NOx_-2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.005813137733583','1.35589860557297','51.6060455886837','-0.611676169335664','0','6','86']
  H_HDV['92_PM_-2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.081689116706636','2.44536072365007','4.51194647459325','1.81987644907792','0.002439374897754','6','86']
  H_HDV['92_FC_-2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','494.368160750153','0.035435501149355','27265.7385904003','0.827117770368828','20.4623008973231','6','86']
  H_HDV['92_CO_-2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.42647855624203','0.046726073429335','164.892741495714','0.725674520389937','0.492999214529805','6','86']
  H_HDV['92_THC_-2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','100','THC','y=((a*(b**x))*(x**c))','9.76907309246513','0.981466871409058','-0.588460908886116','0','0','6','86']
  H_HDV['92_NOx_-2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.5394215830365','-0.083143613313479','-1.24751648940448','0.079964722060773','0','6','86']
  H_HDV['92_PM_-2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','100','PM','y=((a+(b*x))**((-1)/c))','0.416110977381179','0.038972706372685','0.630622814180861','0','0','6','86']
  H_HDV['92_FC_-2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-3.93688359755886','0.799498323643277','1343.547409844','-0.466096547419172','0','6','86']
  H_HDV['92_CO_-4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.12660585646146','73.1209086185694','-1.24466789595841','0.521639919967485','0.037416205302687','6','86']
  H_HDV['92_THC_-4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.066085932773055','33.9206363955272','-1.19911080477775','0.561532353655295','0.0326095557479','6','86']
  H_HDV['92_NOx_-4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8157726889067','0.052335798226287','159.014506428037','0.554901873918267','0.334825118700442','6','86']
  H_HDV['92_PM_-4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.30656951757565','-8.98333458315699','-1.77587534551582','0','0','6','86']
  H_HDV['92_FC_-4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','417.872428114418','0.051693381676185','14926.9117753232','0.731994045001998','8.33567945440222','6','86']
  H_HDV['92_CO_-4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.73011419691157','0.052036071420435','89.1970781426854','0.644542036657878','0.129487525039372','6','86']
  H_HDV['92_THC_-4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.60340387457441','0.055429615689892','13.9241444683401','0.484265480362143','0.094718322281281','6','86']
  H_HDV['92_NOx_-4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6234508166921','0.042411984014582','405.591790730129','0.705778933606573','0.118153485656779','6','86']
  H_HDV['92_PM_-4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.249605555064703','0.045044969121158','0.001530651148057','0','0','6','86']
  H_HDV['92_FC_-4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','367.703686212473','0.043108478619124','30454.9242501984','0.845350340181891','1.17473876341573','6','86']
  H_HDV['92_CO_-4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.39878609996549','0.044665786408199','225.305748065285','0.781217059719517','0.050962047738698','6','86']
  H_HDV['92_THC_-4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.41232740151989','0.05390969045132','19.2582467120132','0.540203538078616','0.088301158558362','6','86']
  H_HDV['92_NOx_-4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1133874634542','0.035978795962956','841.663073096525','0.832479209034047','-0.103364075284275','6','86']
  H_HDV['92_PM_-4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.52665854061631','0.054496489899355','8.47387234689257','0.460297926262458','0.058680259379221','6','86']
  H_HDV['92_FC_-4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','352.106064638252','0.036821109802079','49070.8622716263','0.927132836279627','-6.25043422121809','6','86']
  H_HDV['92_CO_-6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.40463466508189','0.069641273722135','38.5846307397458','0.5926666439882','0.133389959616297','6','72']
  H_HDV['92_THC_-6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.97716567649098','0.06609040438394','18.9945743445069','0.594540318876141','0.095734446625423','6','72']
  H_HDV['92_NOx_-6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.1603588980446','0.065862931077201','338.218227253385','0.708757843158312','0.226247538082617','6','72']
  H_HDV['92_PM_-6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.00063352415403','0.072935242745942','1.39741976988489','0.23297125061','0.066412433533912','6','72']
  H_HDV['92_FC_-6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','374.87344572372','0.061977349059738','22184.4347798639','0.814427355280734','3.55139411075535','6','72']
  H_HDV['92_CO_-6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.16609814807074','0.061624158413894','92.6038759452084','0.686581798667008','0.104379845938719','6','72']
  H_HDV['92_THC_-6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.50143066465953','0.062700351861413','27.1297529878362','0.627805534343669','0.088974289393468','6','72']
  H_HDV['92_NOx_-6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.47498748746474','0.05893158466594','515.564524972084','0.76806327257474','0.18688690767374','6','72']
  H_HDV['92_PM_-6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.65850150736422','0.067633930763163','6.05065428156319','0.462174943749599','0.060426842816297','6','72']
  H_HDV['92_FC_-6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','307.333901325134','0.056093963564977','29423.0086871028','0.852422502291367','2.29333774914601','6','72']
  H_HDV['92_CO_-6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.56608311267088','0.055725277528131','161.783799634413','0.747003636525362','0.083318827885297','6','72']
  H_HDV['92_THC_-6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.31194252858276','0.061657032305442','24.0845374818475','0.601417029779268','0.088006813937333','6','72']
  H_HDV['92_NOx_-6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.62301062672384','0.055860802483231','611.767518521699','0.796405691181484','0.182100620220008','6','72']
  H_HDV['92_PM_-6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.48998575850018','0.063941565817611','11.3479213829738','0.544322892087757','0.057858758582873','6','72']
  H_HDV['92_FC_-6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','282.431903684027','0.053164093657005','34243.0923872682','0.876897646041298','2.01477038483696','6','72']
  H_HDV['92_CO_2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.9022343369418','0.011677994193444','-0.887848389781273','0.069246590362137','0','6','86']
  H_HDV['92_THC_2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.268574536379858','4.67188263380803','4.90078251861951','1.96531811853493','-0.012070835194225','6','86']
  H_HDV['92_NOx_2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','0','NOx','y=(c+(a*exp(b*x)))','26.4246956247877','-0.096326065827841','12.0170010193827','0','0','6','86']
  H_HDV['92_PM_2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.348811844553415','1.9977074023194','5.40089731800945','2.00361805917333','0.023064109540979','6','86']
  H_HDV['92_FC_2_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.077154420943099','1.67302900485793','2266.85911395762','-0.53358190553361','0','6','86']
  H_HDV['92_CO_2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.9854726932989','0.006510056180186','-1.08198631099861','0.079781811862703','0','6','86']
  H_HDV['92_THC_2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.598759608520059','4.09006699077935','6.21671662422437','2.46674476662526','-0.014927139211024','6','86']
  H_HDV['92_NOx_2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','50','NOx','y=((a*(b**x))*(x**c))','55.3919713545333','1.00280279020027','-0.315610796921468','0','0','6','86']
  H_HDV['92_PM_2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.11112874431897','0.001287818648376','-0.216939832026261','0.083302956648802','0','6','86']
  H_HDV['92_FC_2_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','50','FC','y=((a*(b**x))*(x**c))','2227.97327132364','1.00601797685669','-0.444893546870786','0','0','6','86']
  H_HDV['92_CO_2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.844768251216','-0.043330863435213','-1.74239880208927','0.122400684741588','0','6','76']
  H_HDV['92_THC_2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.740695540599871','3.800720374011','7.18710550253816','2.87929784986556','-0.030116652490998','6','76']
  H_HDV['92_NOx_2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000064973862701','0.011367332626222','-0.731786084420956','40.5336077624395','0','6','76']
  H_HDV['92_PM_2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','100','PM','y=((a*(b**x))*(x**c))','6.54422406980997','1.00602082361869','-0.606908106583751','0','0','6','76']
  H_HDV['92_FC_2_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','2','100','FC','y=((a*(b**x))*(x**c))','1996.17819092367','1.00222565348999','-0.294300364112088','0','0','6','76']
  H_HDV['92_CO_4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.55204719394724','75.3948720908548','-2.71331293224424','-0.872608561978769','0.172672845107826','6','71']
  H_HDV['92_THC_4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.478231003018383','4.56438435122575','5.62348652989602','2.24811059643274','-0.013144980878925','6','71']
  H_HDV['92_NOx_4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.6766677814256','0.091555050246636','31.2034450613114','0.39644931648699','17.5217736562325','6','71']
  H_HDV['92_PM_4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.480629865897238','1.64993890315517','6.258134598899','2.3213100529486','0.016999725724167','6','71']
  H_HDV['92_FC_4_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','815.299272800745','0.091021810084252','7207.11114884122','0.723998851346887','474.633632707287','6','71']
  H_HDV['92_CO_4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.8340312618557','-0.027260769919908','-1.86612090575004','0.128015444500959','0','6','65']
  H_HDV['92_THC_4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.851763836950803','4.02094552593676','6.69103891263204','2.69506767072174','-0.02069932147382','6','65']
  H_HDV['92_NOx_4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.68273139608657','1.34832071458116','-0.098069998595532','0','0','6','65']
  H_HDV['92_PM_4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.17156793245909','-0.001130564665027','-0.240691538869796','0.10753943252598','0','6','65']
  H_HDV['92_FC_4_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','544.048364300678','0.060732361588098','4621.20446152415','0.548297579960463','764.931341876193','6','65']
  H_HDV['92_CO_4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000210026376471','0.023155696314176','-0.90668694843122','18.4332992881107','0','6','50']
  H_HDV['92_THC_4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.05652906221067','3.82405882011074','6.77293024954967','2.72904553778468','-0.025528193084612','6','50']
  H_HDV['92_NOx_4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000309654704282','0.032340474755118','-1.2650187456868','57.2040017625348','0','6','50']
  H_HDV['92_PM_4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.33160138173783','0.01960229382369','5.81161292760874','0.379535993520921','0.666455337961288','6','50']
  H_HDV['92_FC_4_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','971.634487788788','0.010166725450154','5690.9767873333','0.530759688793956','449.21957727509','6','50']
  H_HDV['92_CO_6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.87497267853756','8.74346999938521','7.65991320716404','3.09541286908967','-0.024880920281382','6','71']
  H_HDV['92_THC_6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.80525229552599','0.006196955229521','-0.483622277478576','0.077479001536104','0','6','71']
  H_HDV['92_NOx_6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','53.2973101987075','-0.040287367259671','-3.96423938287652','0.139310701821685','0','6','71']
  H_HDV['92_PM_6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.7679045839589','0.008207215668774','-0.164571570727921','0.06618277448496','0','6','71']
  H_HDV['92_FC_6_0'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2703.60617302915','1.01036601174912','-0.503318997321487','0','0','6','71']
  H_HDV['92_CO_6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000187001595166','0.020534309810337','-0.795985971661188','16.5979293014933','0','6','49']
  H_HDV['92_THC_6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.12997102940449','3.74134558610586','7.40419984447351','3.00059734870247','-0.029603807671658','6','49']
  H_HDV['92_NOx_6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.90155374792511','1.12176740916517','-0.079699664408105','0','0','6','49']
  H_HDV['92_PM_6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','50','PM','y=exp((a+(b/x))+(c*log(x)))','0.501297936530133','2.92658056284368','-0.094011591386011','0','0','6','49']
  H_HDV['92_FC_6_50'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','474.233316408589','0.049076647869396','4636.4596540562','0.520756715077263','1015.00743892123','6','49']
  H_HDV['92_CO_6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','24.549609164798','0.011282216143633','58.3993520572642','0.484586543987516','-10.2407953047453','6','36']
  H_HDV['92_THC_6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000143634373022','0.013405113859629','-0.434664594601943','6.88440254683212','0','6','36']
  H_HDV['92_NOx_6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000968233664908','0.074843622284875','-2.14591734626319','76.5130834952905','0','6','36']
  H_HDV['92_PM_6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.99785024814977','0.006545094271156','11.2221849008879','0.518723896897515','-0.640682207457407','6','36']
  H_HDV['92_FC_6_100'] = [' 92','EURO1','TT/AT >34-40t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1536.51532414272','0.006890271463909','6986.57107066437','0.566599352484713','289.134690970148','6','36']
  H_HDV['93_CO_0_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.3996511222782','0.056481212688218','7.09471908416949','0.264866140616052','1.05601214269122','6','86']
  H_HDV['93_THC_0_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.070911720605285','3.55430260961025','3.09858525846116','1.44598240571932','-0.007390152684028','6','86']
  H_HDV['93_NOx_0_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.5049256911816','0.052189122099116','39.3648890239658','0.317612341775338','6.56818549842632','6','86']
  H_HDV['93_PM_0_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.585818569192405','0.000609720064584','-0.028039519455421','0.0556181999053','0','6','86']
  H_HDV['93_FC_0_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','510.884794195087','0.055940037265021','12685.3137461685','0.767747502676142','170.855319447306','6','86']
  H_HDV['93_CO_0_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.072854852891141','0.013846694694106','-0.000083918080117','0','0','6','86']
  H_HDV['93_THC_0_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.869049304567137','0.025697548262684','2.87234757101772','0.119991315850181','0.183141449809032','6','86']
  H_HDV['93_NOx_0_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','50','NOx','y=((a+(b*x))**((-1)/c))','0.000696620063889','0.000212259707856','1.9205343521472','0','0','6','86']
  H_HDV['93_PM_0_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000108248166887','-0.013362716128929','0.578115016245742','0','0','6','86']
  H_HDV['93_FC_0_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','519.392020120098','0.038318129928787','42872.9848130647','0.976407955865168','205.310106365282','6','86']
  H_HDV['93_CO_0_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.73026746947664','0.0687234155737','-0.309770498544066','0.027971194608642','0','6','86']
  H_HDV['93_THC_0_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.860536600185764','0.022558938656724','2.85659251422722','0.119499399423952','0.161723098503208','6','86']
  H_HDV['93_NOx_0_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000058911550682','0.010827542822591','-0.760360439195459','30.7462022059748','0','6','86']
  H_HDV['93_PM_0_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000000662792016','0.00004417033149','-0.01315160466226','0.693681028955147','0','6','86']
  H_HDV['93_FC_0_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','600.063966969014','0.027283269357675','2034810.93427635','1.65515777389757','206.987858257961','6','86']
  H_HDV['93_CO_-2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.6219077407893','0.007876208218711','-0.396099542150296','0.060709587645962','0','6','86']
  H_HDV['93_THC_-2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.169577199490857','7.62040979776881','0.186042072917629','0.594618265179926','0.036739099103128','6','86']
  H_HDV['93_NOx_-2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','0','NOx','y=((a*(b**x))*(x**c))','67.4524896722865','0.991933095476633','-0.660348842665916','0','0','6','86']
  H_HDV['93_PM_-2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.56287987629975','0.000450341725886','-0.030530947788996','0.057185632430611','0','6','86']
  H_HDV['93_FC_-2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','479.270266304521','0.05207762698553','18230.357122068','0.814216639895753','50.8217550226817','6','86']
  H_HDV['93_CO_-2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','50','CO','y=(c+(a*exp(b*x)))','6.73347907897099','-0.065058907210187','0.559126452972253','0','0','6','86']
  H_HDV['93_THC_-2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.24606556884586','0.039418767607992','0.000544025915445','0','0','6','86']
  H_HDV['93_NOx_-2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.018870217196037','1.12089310716565','54.85454178945','-0.597837911372892','0','6','86']
  H_HDV['93_PM_-2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.501713360501155','0.05163862646273','6.88592871986265','0.759911568724142','0.06188344780093','6','86']
  H_HDV['93_FC_-2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','50','FC','y=((a*(b**x))*(x**c))','1066.10659263043','0.981225337826842','-0.357996821228449','0','0','6','86']
  H_HDV['93_CO_-2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.42472180128448','0.043665686951854','289.27618290882','0.87815138957728','0.420084035112731','6','86']
  H_HDV['93_THC_-2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','100','THC','y=((a*(b**x))*(x**c))','6.27239318166485','0.983528089146997','-0.593683155881706','0','0','6','86']
  H_HDV['93_NOx_-2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.5597934646843','-0.084385501752414','-1.37637603303268','0.078647676452039','0','6','86']
  H_HDV['93_PM_-2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.527974636931744','0.039594533401012','42.4778674963799','0.996287911009121','0.054125068906507','6','86']
  H_HDV['93_FC_-2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','662.173504014251','-2.13542440284579','-31.9338274745785','0.066348645509711','0','6','86']
  H_HDV['93_CO_-4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.2850195921463','0.058236527139526','10.7748254211006','0.379090722133503','0.154739245570667','6','86']
  H_HDV['93_THC_-4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.049319785475878','20.3307119013424','-1.11176661431038','0.585131026748624','0.027555954412768','6','86']
  H_HDV['93_NOx_-4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.9788311430834','0.052172717682564','145.857354256598','0.527720120621052','0.380900487127528','6','86']
  H_HDV['93_PM_-4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.31860046012518','-5.5502493675586','-1.27682887280768','0','0','6','86']
  H_HDV['93_FC_-4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','373.526999067688','0.050492450259147','11829.791490681','0.71531013752331','8.39881647327158','6','86']
  H_HDV['93_CO_-4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.83627639303384','0.047773653267474','50.7424921475806','0.603035745551191','0.095743985080913','6','86']
  H_HDV['93_THC_-4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.60644161748769','0.051032200045631','5.25111961950092','0.382996898368843','0.077432349108959','6','86']
  H_HDV['93_NOx_-4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6467575442426','0.04296901443212','398.501383392162','0.694420111665732','0.155900568938345','6','86']
  H_HDV['93_PM_-4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.33255208513429','0.161840347126747','0.001800897320384','0','0','6','86']
  H_HDV['93_FC_-4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','333.306672652063','0.041636869847085','27113.5944146766','0.851278014186683','1.09897708947181','6','86']
  H_HDV['93_CO_-4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.93449281596217','0.040779628089213','186.501037953561','0.805270050343304','0.019670872331451','6','86']
  H_HDV['93_THC_-4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53238770560718','0.050429076435698','5.65057183054294','0.400376744554857','0.075418479780152','6','86']
  H_HDV['93_NOx_-4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.04557106266','0.037039970339117','878.394343767539','0.832096927520381','-0.059937074878323','6','86']
  H_HDV['93_PM_-4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.437022861832792','0.039149586936065','4.51224728443873','0.62838404130129','0.020502741069795','6','86']
  H_HDV['93_FC_-4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','325.946906323488','0.035640261193089','55334.6301630551','0.972348981961561','-6.23437273288175','6','86']
  H_HDV['93_CO_-6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.28771441385412','0.068908398408711','20.7235838504809','0.524847117398194','0.118889365607254','6','72']
  H_HDV['93_THC_-6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.86700113787061','0.061871538863117','6.89558827613222','0.490199398783546','0.086447846770857','6','72']
  H_HDV['93_NOx_-6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.4071631463005','0.066392211070018','306.890733819996','0.68523851740212','0.275012745651656','6','72']
  H_HDV['93_PM_-6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006093549247479','0.511871405367746','4.43527721240163','1.8513386688658','-0.011529987030994','6','72']
  H_HDV['93_FC_-6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','337.896220824953','0.061636473528008','16982.9255367871','0.79350706323839','4.49230593261185','6','72']
  H_HDV['93_CO_-6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.60881017764261','0.060262741145762','56.9317991917885','0.664054075119389','0.098302111943164','6','72']
  H_HDV['93_THC_-6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68065472138851','0.060014196393904','8.56862793198891','0.511622609502022','0.08385010940175','6','72']
  H_HDV['93_NOx_-6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2339146099752','0.058991661660552','507.150375920404','0.755939439003393','0.227666576160981','6','72']
  H_HDV['93_PM_-6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.368754331261135','0.053440816100853','0.499430564516457','0.253796124127931','0.033463831994579','6','72']
  H_HDV['93_FC_-6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','277.080362726386','0.055436842179552','24668.254954035','0.846541549694322','3.28375939220455','6','72']
  H_HDV['93_CO_-6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.25565779991306','0.052785080599443','108.312997236181','0.729897763692179','0.072668837471993','6','72']
  H_HDV['93_THC_-6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.58839707756832','0.059264826769352','7.11596034469996','0.472760109405122','0.083460287668102','6','72']
  H_HDV['93_NOx_-6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.22457907534801','0.055622161507163','603.228834187828','0.784374076677674','0.213391776621486','6','72']
  H_HDV['93_PM_-6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.392581698930492','0.05096389341427','1.84180271899278','0.448381712713164','0.032030611887891','6','72']
  H_HDV['93_FC_-6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','255.579019834886','0.052229672035517','29767.898111965','0.87710725488629','2.90555808677522','6','72']
  H_HDV['93_CO_2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.22339214658076','0.008161236620083','-0.581827797729164','0.073829133079105','0','6','86']
  H_HDV['93_THC_2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.151778811905144','3.2074875998346','4.12142378141107','1.6994514917061','-0.006065179693468','6','86']
  H_HDV['93_NOx_2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','42.3685079124749','-0.013220298669864','-2.81819645910205','0.096367237815348','0','6','86']
  H_HDV['93_PM_2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.23060275158996','0.122969211284828','-0.001020075767045','0','0','6','86']
  H_HDV['93_FC_2_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','0','FC','y=((a*(b**x))*(x**c))','2234.56233502293','1.00919332715179','-0.605087217927299','0','0','6','86']
  H_HDV['93_CO_2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.073259497417017','0.009122670843953','-0.000050099675965','0','0','6','86']
  H_HDV['93_THC_2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.343122217174699','2.6299906831814','6.06291660060984','2.40507747334471','-0.014532964141042','6','86']
  H_HDV['93_NOx_2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','44.908416295982','-0.047379141477295','-2.12198180978739','0.085590991208246','0','6','86']
  H_HDV['93_PM_2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.758936210485072','0.0088453983976','-0.018829501355633','0.017993130965428','0','6','86']
  H_HDV['93_FC_2_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','50','FC','y=((a*(b**x))*(x**c))','1923.82460186179','1.00564190124161','-0.404941487171038','0','0','6','86']
  H_HDV['93_CO_2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.59620417970619','0.03085265349567','25.5560890218669','0.387193080207436','1.11161306280579','6','78']
  H_HDV['93_THC_2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.461741127837884','2.49838107214762','6.27860931045946','2.51025300461045','-0.01563372201733','6','78']
  H_HDV['93_NOx_2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','4.17995546117414','-0.148253706683648','-0.25757927690369','0','0','6','78']
  H_HDV['93_PM_2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.650797415334009','0.030295588044109','25.0934266953824','0.85675589504355','0.220810601905607','6','78']
  H_HDV['93_FC_2_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002550369516155','0.443830886077937','-26.7099431256004','1247.22885682582','0','6','78']
  H_HDV['93_CO_4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.02472433316732','6.09356322976097','5.99108240554613','2.42590763859863','-0.017341420963196','6','71']
  H_HDV['93_THC_4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.259118933750465','3.03153545775269','5.19263396707518','2.09643513098056','-0.011305595503433','6','71']
  H_HDV['93_NOx_4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0551815245876','0.057700295254251','25.4186637376926','0.186614691188328','17.2228182906829','6','71']
  H_HDV['93_PM_4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.23664109207354','0.077101255904981','-0.000591083032625','0','0','6','71']
  H_HDV['93_FC_4_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','733.935720194644','0.095106729819957','10741.2446737965','0.846599272357755','472.282784044756','6','71']
  H_HDV['93_CO_4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.11804191486405','0.020089368859581','9.01002373727557','0.190565099390602','0.1149450221618','6','68']
  H_HDV['93_THC_4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.537300396679982','2.52591805726743','6.92288439687754','2.79306382615094','-0.021363124455316','6','68']
  H_HDV['93_NOx_4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','24.4040064907571','84.3236147064737','0.19747638988461','0.663406092700081','0.020809832920076','6','68']
  H_HDV['93_PM_4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','50','PM','y=((a*(b**x))*(x**c))','1.34462927617363','0.994525158383146','-0.255028563055559','0','0','6','68']
  H_HDV['93_FC_4_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','525.641817336335','0.071209150461981','12214.4729763761','0.784049807261657','780.853351329546','6','68']
  H_HDV['93_CO_4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7695046020485','0.023612253033653','25.4960970092492','0.341660237842542','-0.623352386759757','6','53']
  H_HDV['93_THC_4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.747686806106681','2.26440495693003','7.64673542602388','3.1191856595164','-0.030263410471016','6','53']
  H_HDV['93_NOx_4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000333435560327','0.038533430432405','-1.65503359271973','62.7448406578746','0','6','53']
  H_HDV['93_PM_4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.1378106679972','0.012878977486167','4.5947485644156','0.489547252192648','-0.154056760273947','6','53']
  H_HDV['93_FC_4_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','639.746973516422','0.012220016840644','6266.71164698433','0.579113077445662','727.126380079932','6','53']
  H_HDV['93_CO_6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.3604536062988','-0.044621838263052','-2.38927662326163','0.208910819002344','0','6','71']
  H_HDV['93_THC_6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.37841569722416','0.001849767700921','-0.322430594423386','0.081384028513863','0','6','71']
  H_HDV['93_NOx_6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','57.6304191808266','-0.060630580177563','-3.95598780381371','0.123295566531165','0','6','71']
  H_HDV['93_PM_6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','0','PM','y=(c+(a*exp(b*x)))','0.503561168323951','-0.039877950787706','0.269326098820557','0','0','6','71']
  H_HDV['93_FC_6_0'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','2281.32328431713','1.00933317442477','-0.448410312457859','0','0','6','71']
  H_HDV['93_CO_6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.30630980100216','0.029938452428794','15.1771461327156','0.297047344256539','0.642721050315378','6','51']
  H_HDV['93_THC_6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.851716096719724','10.8706121157092','-1.33660651386914','-0.417956875806554','0.149561339977033','6','51']
  H_HDV['93_NOx_6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','71.7997425359361','-0.132753628184529','-3.84749173721685','0.122529922940124','0','6','51']
  H_HDV['93_PM_6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.719560183565723','0.0407773098792','4.09601803975907','0.617123482473238','0.36151857103586','6','51']
  H_HDV['93_FC_6_50'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','426.68313894858','0.069209846022537','7367.45958005444','0.662438454503974','1068.85370124547','6','51']
  H_HDV['93_CO_6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','33.3303055477051','-0.181635534767698','-5.92972296362183','0.248304922437089','0','6','38']
  H_HDV['93_THC_6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.11646695963854','1.84920235558736','10.346434197999','4.28896558440433','-0.058817516303449','6','38']
  H_HDV['93_NOx_6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00106397527457','0.090077775676587','-2.84869974874658','83.6644976430964','0','6','38']
  H_HDV['93_PM_6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.20212439169956','0.009370704017778','4.0713477915399','0.452977718293796','-0.948815927792049','6','38']
  H_HDV['93_FC_6_100'] = [' 93','EURO2','TT/AT >34-40t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1207.02064510759','0.006605834940045','13236.9093039407','0.714091673275959','575.224065332548','6','38']
  H_HDV['94_CO_0_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.90211276781142','0.055942427044551','9.84226731725224','0.203009933762561','1.16263736979275','6','86']
  H_HDV['94_THC_0_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.125071998524408','3.16795190102862','3.75461691467342','1.66584058217813','-0.006764163565442','6','86']
  H_HDV['94_NOx_0_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0232503888476','0.050601481642983','50.5251934425144','0.281564006291335','5.12866578876252','6','86']
  H_HDV['94_PM_0_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.059370253173871','1.02733824591747','4.64817798192811','1.93353213396071','-0.011511807564231','6','86']
  H_HDV['94_FC_0_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','547.359233361391','0.054768272174831','1836.31526014435','0.425481828372746','174.438203925765','6','86']
  H_HDV['94_CO_0_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.50245968667796','161.6537920631','-1.90908880410166','0.441187799875664','0.044334306550876','6','86']
  H_HDV['94_THC_0_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.121181403712786','3.10369603527271','3.64831751388675','1.63863261266658','-0.006456304670498','6','86']
  H_HDV['94_NOx_0_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','50','NOx','y=((a+(b*x))**((-1)/c))','-0.000443110105344','0.000261309078361','2.08239071925025','0','0','6','86']
  H_HDV['94_PM_0_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.577053096018188','0.048518150812531','0.827531690937969','0.183627369708756','0.117901080881573','6','86']
  H_HDV['94_FC_0_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','553.391547768183','0.038662676132618','6987.31532379889','0.666585066192232','208.242511143756','6','86']
  H_HDV['94_CO_0_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.77425917853794','0.056076816260305','15.8974030367428','0.324058568613864','1.79216398362515','6','86']
  H_HDV['94_THC_0_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.686489647395772','0.023450383385593','2.79744817506349','0.115500185198822','0.155215405545771','6','86']
  H_HDV['94_NOx_0_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.9233394672287','0.027960374894966','61.5783636554824','0.346397469745551','5.67290142201858','6','86']
  H_HDV['94_PM_0_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.624116222756805','0.046762797326335','0.934978445955305','0.241517550988395','0.135864102561781','6','86']
  H_HDV['94_FC_0_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','634.791854429937','0.028886658070947','476141.205433099','1.4122443942678','215.044403404755','6','86']
  H_HDV['94_CO_-2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.854372330043','0.014690937379407','-0.726740006827156','0.061767297344941','0','6','86']
  H_HDV['94_THC_-2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.066805604742647','2.26322457570952','6.06493883502584','2.36091864997906','-0.018800967129752','6','86']
  H_HDV['94_NOx_-2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.5845276699526','0.056024852085599','105.359032202247','0.45043087727247','1.63496794694878','6','86']
  H_HDV['94_PM_-2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.102799097675371','23.9043916831445','-3.54953631076793','-0.396065959995971','0.101813087112445','6','86']
  H_HDV['94_FC_-2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','560.115737266492','0.053403819509518','3978.79657012363','0.575700299262794','56.8432581305197','6','86']
  H_HDV['94_CO_-2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','50','CO','y=(c+(a*exp(b*x)))','11.3095665268372','-0.069472032513694','0.793319569854163','0','0','6','86']
  H_HDV['94_THC_-2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.96005446076448','-0.002439899595228','-0.22454676806937','0.084122255918505','0','6','86']
  H_HDV['94_NOx_-2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.8338789472738','0.036666300732447','179.863008370706','0.516648360563723','0.694468189858203','6','86']
  H_HDV['94_PM_-2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040526672523088','0.825922757470728','6.59178162467111','2.47077891317483','-0.016301212510009','6','86']
  H_HDV['94_FC_-2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','50','FC','y=((a*(b**x))*(x**c))','1238.01434489585','0.981902947752536','-0.384822575695058','0','0','6','86']
  H_HDV['94_CO_-2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','100','CO','y=(c+(a*exp(b*x)))','11.0511601407595','-0.061766628261294','0.72890179757252','0','0','6','86']
  H_HDV['94_THC_-2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.80914906931743','-0.002821506940281','-0.219977069372014','0.087687287007431','0','6','86']
  H_HDV['94_NOx_-2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0819439274176','0.028563365448149','400.812274182429','0.651409645734158','-0.005990465526925','6','86']
  H_HDV['94_PM_-2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.14742352402576','-6.58651119212232','-1.29403419296284','0','0','6','86']
  H_HDV['94_FC_-2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','734.561942780271','-2.2757336971142','-37.5508012815636','0.069815783618655','0','6','86']
  H_HDV['94_CO_-4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.15537871339896','0.002098026682308','10.772754145887','0.082945056492506','-3.20197167034893','6','86']
  H_HDV['94_THC_-4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.88524034479014','-0.00204583368641','-0.231382488531856','0.08716774605144','0','6','86']
  H_HDV['94_NOx_-4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.6105700609971','0.061745234737133','175.790447364296','0.533415796924756','0.523249539496477','6','86']
  H_HDV['94_PM_-4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033255061270133','0.843806542049296','7.03015596027179','2.65272353722119','-0.017645043064942','6','86']
  H_HDV['94_FC_-4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','451.797153063761','0.052996241972204','4280.40385228506','0.551110989350089','15.4552282041549','6','86']
  H_HDV['94_CO_-4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.1131966557589','-0.009506835813819','-0.743042836911302','0.080663227454555','0','6','86']
  H_HDV['94_THC_-4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.067234568230949','14.0497956887873','-0.953381252521272','0.511963039131196','0.040479223057721','6','86']
  H_HDV['94_NOx_-4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5006069274204','0.051445514437591','474.06243708713','0.694997457630437','0.387197224883404','6','86']
  H_HDV['94_PM_-4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.24187670470895','-6.82129994111177','-1.376163419147','0','0','6','86']
  H_HDV['94_FC_-4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','385.208369945343','0.043957444140733','13162.368704164','0.733887082878224','8.70934267371759','6','86']
  H_HDV['94_CO_-4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.034637235737049','78.7112371459857','-1.51270565771718','0.421539318861135','0.030406712171336','6','86']
  H_HDV['94_THC_-4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5263515740797','0.060821254164934','9.31319463918072','0.555894408792227','0.078371690497837','6','86']
  H_HDV['94_NOx_-4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.24250577508851','0.043380944473831','938.193601522586','0.809946147509595','0.246091146887881','6','86']
  H_HDV['94_PM_-4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.07673128322567','-6.80955945008976','-1.32825077509375','0','0','6','86']
  H_HDV['94_FC_-4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','366.975499098104','0.037455360466644','29388.8860199123','0.871990249928136','1.2192486394216','6','86']
  H_HDV['94_CO_-6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','0','CO','y=(c+(a*exp(b*x)))','9.4212462753265','-0.075633178542954','0.248940081311718','0','0','6','72']
  H_HDV['94_THC_-6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92873278326963','0.071530080430541','2.31575204149176','0.380401347797311','0.083863578207647','6','72']
  H_HDV['94_NOx_-6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2988216693442','0.073425731957819','284.21686302808','0.622695603464486','0.381416450968475','6','72']
  H_HDV['94_PM_-6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.72797388181143','-8.05695968583567','-1.51695135256562','0','0','6','72']
  H_HDV['94_FC_-6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','428.559301001779','0.064963855979538','9344.38844695308','0.714174685813819','13.0381035462345','6','72']
  H_HDV['94_CO_-6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','7.41812478076023','-0.069039312094172','0.22350423501292','0','0','6','72']
  H_HDV['94_THC_-6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.43966581953813','0.065809791516219','10.6456209708717','0.583742366587448','0.077015816023576','6','72']
  H_HDV['94_NOx_-6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.33761291612254','0.062260960607756','489.563569644106','0.703777132161202','0.330975579050306','6','72']
  H_HDV['94_PM_-6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031021468179828','0.716157959363118','4.81497940798591','1.85108459130309','-0.000972448701421','6','72']
  H_HDV['94_FC_-6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','329.523098946476','0.056684643060652','14234.9432339545','0.763935468316543','10.954664332616','6','72']
  H_HDV['94_CO_-6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','6.75960985228223','-0.065263843070168','0.213229856055471','0','0','6','72']
  H_HDV['94_THC_-6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.24016516016428','0.06331710063558','11.7753832025121','0.592758328969221','0.074796394170706','6','72']
  H_HDV['94_NOx_-6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.15256553183945','0.057155598044116','615.742076037779','0.747566378526029','0.31670584009976','6','72']
  H_HDV['94_PM_-6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.590302498469635','0.055170954757978','0.358507264036291','0.184221047504529','0.054062416815282','6','72']
  H_HDV['94_FC_-6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','304.230736337985','0.053504464952199','22594.1244835637','0.836875183968771','10.4384225364733','6','72']
  H_HDV['94_CO_2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-168.181254828233','0.851820925621122','13.5805888226084','0.087641469190495','1.69230894319357','6','86']
  H_HDV['94_THC_2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.155378673542928','2.86927957566987','4.7592528287699','1.92375760443686','-0.008558677954998','6','86']
  H_HDV['94_NOx_2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','47.9202141344632','-0.025498469449203','-4.37439720692493','0.118864428369084','0','6','86']
  H_HDV['94_PM_2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.121005292362572','0.752916219817358','6.9664660801773','2.66117996492411','-0.018932241366782','6','86']
  H_HDV['94_FC_2_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1162.09128805421','0.393008212114736','-69.7009039160543','0.081601369006208','0','6','86']
  H_HDV['94_CO_2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','50','CO','y=(c+(a*exp(b*x)))','11.9679997500017','-0.071196447693379','2.03531618626946','0','0','6','86']
  H_HDV['94_THC_2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.276195885197761','2.44926418523787','6.14406115779633','2.4380505032306','-0.015288129481546','6','86']
  H_HDV['94_NOx_2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','51.7696216527589','-0.060340273252025','-4.13941354347993','0.118632277726916','0','6','86']
  H_HDV['94_PM_2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.10262320808288','0.000513485504899','-0.06169980219936','0.06595759154793','0','6','86']
  H_HDV['94_FC_2_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','50','FC','y=(c+(a*exp(b*x)))','744.8927059286','-0.07514318698492','510.611421064966','0','0','6','86']
  H_HDV['94_CO_2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','100','CO','y=(1/(a+(b*x)))','0.050222949202324','0.00640839776151','0','0','0','6','79']
  H_HDV['94_THC_2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.341403299301752','2.39219886470477','5.91222531766223','2.38137066954863','-0.014472343059861','6','79']
  H_HDV['94_NOx_2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.5429958537091','0.046557032461849','48.6052227757947','0.321228363773826','16.2804840224067','6','79']
  H_HDV['94_PM_2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.21515558521752','-0.001782418293113','-0.080271203962809','0.09197217946544','0','6','79']
  H_HDV['94_FC_2_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00260287164453','0.457499341463535','-28.0271729307095','1286.64533285803','0','6','79']
  H_HDV['94_CO_4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.37783343907909','8.939559885792','5.2945845526622','2.08913972953629','-0.011264922528108','6','71']
  H_HDV['94_THC_4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.226996847626847','2.67103365212963','5.65009156262008','2.24595990601993','-0.013478083685682','6','71']
  H_HDV['94_NOx_4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.3587321207496','0.025455500909618','41.8761800251029','0.160560595971922','11.872760439257','6','71']
  H_HDV['94_PM_4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.142785671857335','0.675944474302894','6.60667586648567','2.4710121119487','-0.017538210026834','6','71']
  H_HDV['94_FC_4_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','2366.1454769125','1.00872773362989','-0.515233774877039','0','0','6','71']
  H_HDV['94_CO_4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.2289960963406','-0.042879608980238','-1.08710196640574','0.099105007084129','0','6','68']
  H_HDV['94_THC_4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.412414506873708','2.29420950443818','6.91086519135142','2.76698046808039','-0.021953655194996','6','68']
  H_HDV['94_NOx_4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','62.9109236293824','-0.064221237008427','-4.84726302161406','0.12283868226914','0','6','68']
  H_HDV['94_PM_4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004722773328','0.000748788397991','-0.039989852590486','1.01654113393366','0','6','68']
  H_HDV['94_FC_4_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','584.084038894811','0.068980421195048','10379.147510467','0.780863038626068','780.021845324768','6','68']
  H_HDV['94_CO_4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','100','CO','y=((a*(x**b))+(c*(x**d)))','-0.145139600857031','0.712063418373961','29.4525898225632','-0.459420738755199','0','6','53']
  H_HDV['94_THC_4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.500881870919233','2.17680438143884','6.9131195411448','2.77709256375316','-0.024872927910124','6','53']
  H_HDV['94_NOx_4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00051666244596','0.060290267453279','-2.42168561433908','62.3608667687871','0','6','53']
  H_HDV['94_PM_4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','100','PM','y=exp((a+(b/x))+(c*log(x)))','1.17159212922926','-1.71364171857064','-0.549207150870736','0','0','6','53']
  H_HDV['94_FC_4_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','990.534882718391','0.008595979085249','5146.74951209102','0.545780460694949','407.987838227679','6','53']
  H_HDV['94_CO_6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.4303813685887','-0.037796918703215','-1.51470987269788','0.134150980786384','0','6','71']
  H_HDV['94_THC_6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.277709773272159','2.4357351514875','7.32585149875321','2.91403706287149','-0.033751675214632','6','71']
  H_HDV['94_NOx_6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','65.4781233398574','-0.065226351163792','-6.77182937303385','0.150711073941998','0','6','71']
  H_HDV['94_PM_6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','0','PM','y=(c+(a*exp(b*x)))','0.841879473032189','-0.072280601582208','0.228327329213441','0','0','6','71']
  H_HDV['94_FC_6_0'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','2318.81348245651','1.00829372281728','-0.434707206523763','0','0','6','71']
  H_HDV['94_CO_6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.5529017957704','-0.071573753570778','-1.4571295731098','0.120984117575558','0','6','51']
  H_HDV['94_THC_6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.49494381796464','2.18826492258521','7.50273407544177','3.03303456585853','-0.034290242101694','6','51']
  H_HDV['94_NOx_6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000623937115375','0.07106474865349','-2.72610684029815','63.9716086495821','0','6','51']
  H_HDV['94_PM_6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.13337938457328','0.004768617311663','-0.049359124561812','0.050375768132961','0','6','51']
  H_HDV['94_FC_6_50'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','519.274336853712','0.059777241317049','9193.9826158855','0.721354937054602','1038.83245586601','6','51']
  H_HDV['94_CO_6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','100','CO','y=((a*(x**b))+(c*(x**d)))','-0.14872787683236','0.812457186172281','33.8461250125072','-0.468694484503137','0','6','38']
  H_HDV['94_THC_6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.791464377291997','8.41218165081544','-1.30976228694936','-0.468669580559178','0.147955561268953','6','38']
  H_HDV['94_NOx_6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00148995607447','0.130920208517234','-3.96248499666656','81.8447422107489','0','6','38']
  H_HDV['94_PM_6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000019940663067','0.001747380880604','-0.059518338165027','1.33104512732062','0','6','38']
  H_HDV['94_FC_6_100'] = [' 94','EURO3','TT/AT >34-40t Euro-3','6','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2923.46737389332','-9.5430175368484','-369.663300368028','0.327446185392601','0','6','38']
  H_HDV['95_CO_0_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.044918210333573','0.860543505648535','3.89907152408942','1.70995498132762','-0.008349914852062','6','86']
  H_HDV['95_THC_0_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005723063880224','0.177500192127486','3.76140948683052','1.68017784420099','-0.006728054066256','6','86']
  H_HDV['95_NOx_0_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.64106681502176','0.05075036610584','39.8862416848182','0.381845917212933','3.07927777383366','6','86']
  H_HDV['95_PM_0_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.103929764982904','0.046373293041471','0.261778839048749','0.169995984726797','0.01805859955206','6','86']
  H_HDV['95_FC_0_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','521.555265139103','0.055737212075363','2817.54290995052','0.495572560125775','163.801319776275','6','86']
  H_HDV['95_CO_0_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.049953454661063','0.863689438030414','3.48708305380594','1.5534279169931','-0.006828029076397','6','86']
  H_HDV['95_THC_0_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003814348548776','0.171877194477446','3.87453692505498','1.74307754617313','-0.011291875533437','6','86']
  H_HDV['95_NOx_0_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.46247948172816','0.03625441800805','71.5410048638702','0.506122530793371','3.63917585302082','6','86']
  H_HDV['95_PM_0_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108684147845586','0.046515055875118','0.249804783991497','0.177851594041774','0.01922799581452','6','86']
  H_HDV['95_FC_0_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','519.214399654931','0.038833217147498','8501.28818331763','0.69268766651828','193.372975288987','6','86']
  H_HDV['95_CO_0_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.018841231847922','0.896030253657486','3.34484303152027','1.45853580572477','-0.010055876601133','6','86']
  H_HDV['95_THC_0_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003533079257704','0.169774915359871','3.76898014017345','1.71694581134724','-0.011743515995895','6','86']
  H_HDV['95_NOx_0_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1901130912516','0.0268993897861','317.826085923051','0.782378672653341','3.55026190578404','6','86']
  H_HDV['95_PM_0_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.117409371653034','0.047605089851251','0.239375985034456','0.186675207266357','0.019643761788431','6','86']
  H_HDV['95_FC_0_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','587.861331794382','0.028079485191362','162548.845282643','1.22303279168538','193.985644961168','6','86']
  H_HDV['95_CO_-2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','0','CO','y=(c+(a*exp(b*x)))','0.807509386845877','-0.072435947845276','0.056261830421811','0','0','6','86']
  H_HDV['95_THC_-2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000137932752373','0.129119238252232','6.21004833163238','2.4370354693724','-0.0259705466754','6','86']
  H_HDV['95_NOx_-2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.1121286767486','0.050019803080714','87.448455286774','0.527508426665812','0.841569932430006','6','86']
  H_HDV['95_PM_-2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008985478403733','0.236747248858632','3.99047635965246','1.7011942555196','0.007011835133756','6','86']
  H_HDV['95_FC_-2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','526.385885006879','0.054338715149626','5656.36048432455','0.626899579617911','51.0500725417311','6','86']
  H_HDV['95_CO_-2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.64503936840316','-8.97755492756577','-1.57126070281892','0','0','6','86']
  H_HDV['95_THC_-2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00065748779326','0.127821929052601','5.03004228381002','1.99648742514504','-0.009676852913371','6','86']
  H_HDV['95_NOx_-2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.64385999772899','0.031265410558512','170.910909640354','0.623082200876965','0.160606893567072','6','86']
  H_HDV['95_PM_-2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00273328497063','0.298981334891479','2.62784974312758','1.31821274871811','0.008360577527242','6','86']
  H_HDV['95_FC_-2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','50','FC','y=((a*(b**x))*(x**c))','1165.15884648477','0.980783681474154','-0.387213653064473','0','0','6','86']
  H_HDV['95_CO_-2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.757297933010198','-0.000912181158833','-0.053753643212461','0.080480854672981','0','6','86']
  H_HDV['95_THC_-2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.026777461310214','0.015643284599687','0.137513672900398','0.09459444885899','-0.002019352400729','6','86']
  H_HDV['95_NOx_-2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.45759929501586','0.023763271194294','422.787645620824','0.778491988820064','-0.503529430392322','6','86']
  H_HDV['95_PM_-2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.46466170795978','0.542093150478434','0.011710126964169','0','0','6','86']
  H_HDV['95_FC_-2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','684.700356763972','-2.13999368195704','-36.3940645859745','0.07204198416992','0','6','86']
  H_HDV['95_CO_-4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.71004716060648','0.0074537940557','0.008626734073934','0','0','6','86']
  H_HDV['95_THC_-4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.162029962352093','-0.00011352827988','-0.012989819090409','0.086671371022849','0','6','86']
  H_HDV['95_NOx_-4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.61166888837282','0.052212180160663','121.811782483766','0.573667289923793','0.125692409333382','6','86']
  H_HDV['95_PM_-4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.77632261682892','0.432707401185269','0.021517633027171','0','0','6','86']
  H_HDV['95_FC_-4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','423.867383283956','0.054529185342299','6518.25902667957','0.622851136653192','9.33608966972541','6','86']
  H_HDV['95_CO_-4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.62642395971222','-0.000461656027657','-0.046386798355926','0.079645772738261','0','6','86']
  H_HDV['95_THC_-4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002906864924995','0.966367719927595','-1.1631413681987','0.514820072602426','0.039237978002011','6','86']
  H_HDV['95_NOx_-4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.67349912665301','0.042948843986243','244.174940870168','0.68847615341384','0.011036001995933','6','86']
  H_HDV['95_PM_-4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.133150616369489','0.060021199794734','0.570089447116566','0.393491881287842','0.004660313346911','6','86']
  H_HDV['95_FC_-4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','349.33608355646','0.044736423516311','18371.3249150387','0.788929318503168','2.69742499765123','6','86']
  H_HDV['95_CO_-4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00439441926074','2.3444120667161','-0.642017073949457','0.4822724189156','0.034734241700112','6','86']
  H_HDV['95_THC_-4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086610200726941','0.060227698179646','0.551499144102887','0.557409266226575','0.003624103572386','6','86']
  H_HDV['95_NOx_-4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.49437551097486','0.037034575901967','445.532201301585','0.794080735609496','-0.09746151474856','6','86']
  H_HDV['95_PM_-4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.122864467361928','0.058075110393159','0.997961298975735','0.491299772106708','0.004343257465916','6','86']
  H_HDV['95_FC_-4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','324.342108000668','0.037804941985726','42366.2822875506','0.929627668214884','-3.96443379570646','6','86']
  H_HDV['95_CO_-6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','0','CO','y=(c+(a*exp(b*x)))','0.605121804086995','-0.075231277892362','0.01166953468737','0','0','6','72']
  H_HDV['95_THC_-6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.113038660925872','0.071719995760521','0.191343389461761','0.467441797156404','0.003851289119899','6','72']
  H_HDV['95_NOx_-6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.27915476481105','0.065963269320331','189.796109114736','0.672302040867869','0.055568420496747','6','72']
  H_HDV['95_PM_-6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.159924333582384','0.071647252761034','0.531504953734552','0.429210287950052','0.004699681080078','6','72']
  H_HDV['95_FC_-6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','385.211002077062','0.06562974250662','13697.5680153891','0.770112778227249','4.52274839222318','6','72']
  H_HDV['95_CO_-6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00626454250517','1.93392606270489','-0.690091861589404','0.381137985721857','0.049526196813316','6','72']
  H_HDV['95_THC_-6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.084372548215744','0.065503198335448','0.845638990597848','0.649892247275179','0.003427362052541','6','72']
  H_HDV['95_NOx_-6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.98524300360069','0.059296454090612','289.83870991699','0.736767338610233','0.035707256077875','6','72']
  H_HDV['95_PM_-6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.127045195693641','0.06658268397956','1.23395701101566','0.548382125224939','0.004238511445704','6','72']
  H_HDV['95_FC_-6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','288.990527945039','0.05758963217802','24428.8112126022','0.850097194949851','2.89755536773248','6','72']
  H_HDV['95_CO_-6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004447122723679','2.4109729903711','-1.06684936372404','0.411100530697611','0.04279857715695','6','72']
  H_HDV['95_THC_-6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.073392979677762','0.063026819249715','0.941224344254929','0.656727867735169','0.003299312588292','6','72']
  H_HDV['95_NOx_-6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.55591456532317','0.055853726883291','350.529161866059','0.770416406797772','0.029125586863019','6','72']
  H_HDV['95_PM_-6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.114982240602025','0.064658790403084','1.47571346691277','0.577484389172471','0.004102581914658','6','72']
  H_HDV['95_FC_-6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','256.646355444837','0.053622470114941','31320.7219363494','0.891079922079346','2.30470672998508','6','72']
  H_HDV['95_CO_2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.123046907524444','0.615445882520927','7.01152776599729','2.74884116698086','-0.017286616732486','6','86']
  H_HDV['95_THC_2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00598152749282','0.159818095937805','4.79982398916743','1.95113540951266','-0.009119396190454','6','86']
  H_HDV['95_NOx_2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.1525786016231','0.099033868313745','310.155287422888','0.974770769412808','6.10433632280696','6','86']
  H_HDV['95_PM_2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.063187854316681','0.029059732790747','0.271048591461148','0.132514926480817','0.020191774667751','6','86']
  H_HDV['95_FC_2_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.326928202101967','1.33536904658347','2106.93114810026','-0.538305121550849','0','6','86']
  H_HDV['95_CO_2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.20740612030855','-0.002065313241703','-0.11586336775401','0.125567174170321','0','6','86']
  H_HDV['95_THC_2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010454653159065','0.138620521760306','6.12564020992944','2.44919419878692','-0.016503248073678','6','86']
  H_HDV['95_NOx_2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','50','NOx','y=exp((a+(b/x))+(c*log(x)))','2.95866094135022','1.80767727890112','-0.196288496817182','0','0','6','86']
  H_HDV['95_PM_2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079340903538934','0.026599456285536','0.272734340696075','0.145982049356855','0.01826106909818','6','86']
  H_HDV['95_FC_2_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','50','FC','y=((a*(b**x))*(x**c))','1949.575800707','1.0054629017038','-0.419819979761686','0','0','6','86']
  H_HDV['95_CO_2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.43494659670637','0.023596636190536','0.764396161832295','0.150186038990878','0.065927495293713','6','81']
  H_HDV['95_THC_2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012839458310908','0.137560695924559','5.82308445711019','2.36670401499984','-0.014090715010635','6','81']
  H_HDV['95_NOx_2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.8492916710492','0.038336510627146','144.012506449271','0.667491211213038','10.1734844254035','6','81']
  H_HDV['95_PM_2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.088502198396185','0.025030706038144','0.270007691748639','0.143203632274508','0.014975772739329','6','81']
  H_HDV['95_FC_2_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002450975238062','0.437869869668464','-26.9008793624985','1215.69467678018','0','6','81']
  H_HDV['95_CO_4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.190322147190316','0.526162444518659','8.69991943636565','3.43343894764162','-0.029445397946066','6','72']
  H_HDV['95_THC_4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00704260904446','0.15398100228322','5.29566324794398','2.12646406243831','-0.011859591760762','6','72']
  H_HDV['95_NOx_4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.09709891749643','0.072374921645618','19.3063280897395','0.232201462268394','8.76145481728656','6','72']
  H_HDV['95_PM_4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.077051136192665','0.031840429455684','0.256372891181707','0.142921939704415','0.025234206710774','6','72']
  H_HDV['95_FC_4_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','751.040860703812','0.09243544920523','6748.09929144473','0.777515536593536','450.629599914302','6','72']
  H_HDV['95_CO_4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.553859702856285','0.016344305492089','0.769502839219492','0.154559828342588','-0.028673671376203','6','68']
  H_HDV['95_THC_4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01500717108981','0.132508269280084','6.5122938274504','2.63031964138051','-0.019447008686307','6','68']
  H_HDV['95_NOx_4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.05128717654865','0.025349505212549','25.430605428702','0.262533784397038','11.0958378315043','6','68']
  H_HDV['95_PM_4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.110960748638558','0.033426096302168','0.256254456662065','0.158902002128674','0.021890350692829','6','68']
  H_HDV['95_FC_4_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','580.099657814859','0.074654733360893','12728.8165116738','0.821715650206985','738.356664873031','6','68']
  H_HDV['95_CO_4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.34924811107481','-0.003335986752378','-0.10145457507952','0.10065183778368','0','6','54']
  H_HDV['95_THC_4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016041793729031','0.133725267871024','5.90658610560795','2.4184562667449','-0.018111901748897','6','54']
  H_HDV['95_NOx_4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1056717157655','0.024015927894319','35.046509383116','0.318429532085697','14.6177885704814','6','54']
  H_HDV['95_PM_4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.117776910210556','0.02439481410364','0.278706277440237','0.140586381239679','0.005207353758999','6','54']
  H_HDV['95_FC_4_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','742.338595422551','0.011230796323597','3706.03036471407','0.49382124964432','571.180087251321','6','54']
  H_HDV['95_CO_6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.62128657876695','-0.003793142330046','-0.260427219987991','0.218528946870073','0','6','71']
  H_HDV['95_THC_6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006864305411434','0.140078632855626','6.99825279145208','2.79955363422911','-0.034188877400877','6','71']
  H_HDV['95_NOx_6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.8147963646084','-0.043909904659764','-5.95313739237505','0.218920469121743','0','6','71']
  H_HDV['95_PM_6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.393083523045147','-0.000757211047281','-0.047101625347124','0.150193808207461','0','6','71']
  H_HDV['95_FC_6_0'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','2276.72305686755','1.00883067534978','-0.453491601906109','0','0','6','71']
  H_HDV['95_CO_6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.27546980257695','-4.23603853691487','-0.951908441898627','0','0','6','53']
  H_HDV['95_THC_6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016726785885009','0.131712185638228','6.26508858011195','2.5595877045913','-0.021078049850877','6','53']
  H_HDV['95_NOx_6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.20870747716635','1.85323601016136','-0.094846056817849','0','0','6','53']
  H_HDV['95_PM_6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.132332799808256','0.03077439815662','0.266126701841429','0.152685503675924','0.012189543421967','6','53']
  H_HDV['95_FC_6_50'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','413.577302389193','0.046331954553737','2752.08297488365','0.46168347974943','957.578206027514','6','53']
  H_HDV['95_CO_6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000020883894838','0.002263493431141','-0.08850770535886','1.51567976949011','0','6','39']
  H_HDV['95_THC_6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003301446188','0.000343606806812','-0.012512916266287','0.193829602860119','0','6','39']
  H_HDV['95_NOx_6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000587555825802','0.05312349105568','-1.68016070144997','44.1812981425988','0','6','39']
  H_HDV['95_PM_6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.013493994354742','0.303045558633215','4.62614788946181','1.93281787443403','-0.01294452992078','6','39']
  H_HDV['95_FC_6_100'] = [' 95','EURO4','TT/AT >34-40t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1266.65951686277','0.008233928060376','8836.40418509198','0.65128102256769','460.42903296204','6','39']
  H_HDV['96_CO_0_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.044382880968493','0.878600293712649','3.90237251916366','1.71054113756569','-0.008516869620824','6','86']
  H_HDV['96_THC_0_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005887834561822','0.180861228073149','3.76261351974658','1.67911514611957','-0.006621251646279','6','86']
  H_HDV['96_NOx_0_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.66434335560303','0.050271047841119','26.6578574580293','0.364881328434846','1.81470203077485','6','86']
  H_HDV['96_PM_0_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.105983630817433','0.046292130044289','0.267332452513819','0.170008210370283','0.01825527715452','6','86']
  H_HDV['96_FC_0_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','534.418257801533','0.055487911330633','2679.33497797126','0.478384235374484','166.243985699228','6','86']
  H_HDV['96_CO_0_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.055132689096939','0.888117553432212','3.40789592652611','1.52895596015048','-0.004948692311641','6','86']
  H_HDV['96_THC_0_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003965386531105','0.174796102546505','3.89803519556914','1.75040062893389','-0.01136296148909','6','86']
  H_HDV['96_NOx_0_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.24782362126118','0.037633312688179','41.8164005534798','0.465058566307053','2.15159734101286','6','86']
  H_HDV['96_PM_0_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.110008821679714','0.04624186153754','0.255842465818702','0.177278513048125','0.019425329887064','6','86']
  H_HDV['96_FC_0_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','533.611794433117','0.038940001207368','7647.02651698507','0.667247824732031','196.024563714581','6','86']
  H_HDV['96_CO_0_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.410610429520358','0.043596712411041','0.573528018660334','0.146840041801701','0.094356610788735','6','86']
  H_HDV['96_THC_0_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003898884761088','0.172054480618053','3.80147932583318','1.72573298721119','-0.011452646485396','6','86']
  H_HDV['96_NOx_0_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.29890835471728','0.030601364002311','106.761633742362','0.650826897941431','2.19285408650338','6','86']
  H_HDV['96_PM_0_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.119647888763397','0.047629853010925','0.24235471379785','0.185452285831281','0.019920323168234','6','86']
  H_HDV['96_FC_0_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','597.791758268613','0.027502277245926','5207.31875902177','0.682011139358128','192.866410337925','6','86']
  H_HDV['96_CO_-2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','0','CO','y=(c+(a*exp(b*x)))','0.823989768751586','-0.072323257116889','0.05725241973685','0','0','6','86']
  H_HDV['96_THC_-2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000205259928226','0.131773944048555','6.20895822535311','2.43513144078676','-0.025779623959822','6','86']
  H_HDV['96_NOx_-2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.98130830985885','0.05017594024023','53.904041374022','0.496266380605828','0.495695132216428','6','86']
  H_HDV['96_PM_-2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008977056960717','0.240375254040367','4.03996713420971','1.71997516917791','0.006098135609882','6','86']
  H_HDV['96_FC_-2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','542.940199745695','0.054407006499198','5551.20856473553','0.617694947264656','52.0847079961541','6','86']
  H_HDV['96_CO_-2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.69127550478485','-9.05372681344686','-1.57864674309696','0','0','6','86']
  H_HDV['96_THC_-2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000671346220128','0.13035024389699','5.04929399786824','2.00180035413881','-0.009741864895336','6','86']
  H_HDV['96_NOx_-2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.67971750617213','0.032205124964626','96.0908625515834','0.579941498635352','0.109917660640312','6','86']
  H_HDV['96_PM_-2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002483063596124','0.299483727708435','2.7173367461332','1.34613486109302','0.007203344192315','6','86']
  H_HDV['96_FC_-2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','50','FC','y=((a*(b**x))*(x**c))','1218.38251833342','0.980857654960695','-0.394493531633842','0','0','6','86']
  H_HDV['96_CO_-2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.771933533679341','-0.000920288642576','-0.054789296342741','0.080379948518646','0','6','86']
  H_HDV['96_THC_-2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.027142425733983','0.015349677826905','0.140540896082585','0.094097560894662','-0.002197869743225','6','86']
  H_HDV['96_NOx_-2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.12104328184394','0.025631331403728','204.889802116811','0.711827405816936','-0.209281295705933','6','86']
  H_HDV['96_PM_-2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.42948559428794','0.528384885872993','0.011604697991696','0','0','6','86']
  H_HDV['96_FC_-2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','708.060339765638','-2.1821300401829','-38.1430212289801','0.072684443402146','0','6','86']
  H_HDV['96_CO_-4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.68830549783628','0.004181225781727','0.008515979281296','0','0','6','86']
  H_HDV['96_THC_-4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.165505291687341','-0.000115522947539','-0.013240942883046','0.086469039396114','0','6','86']
  H_HDV['96_NOx_-4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.16256382856295','0.053219628462448','77.1067090473073','0.552172326163439','0.077708768738853','6','86']
  H_HDV['96_PM_-4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.76677827587202','0.41483945167354','0.021274333702688','0','0','6','86']
  H_HDV['96_FC_-4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','440.328987805503','0.054878286856294','6452.61845891597','0.61692429355633','9.72193212347197','6','86']
  H_HDV['96_CO_-4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.641897226018827','-0.00046872112449','-0.047630540167354','0.07976420805115','0','6','86']
  H_HDV['96_THC_-4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002985996110929','0.878958905017114','-1.02299884934355','0.520138405591429','0.039384521769856','6','86']
  H_HDV['96_NOx_-4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.57406097157482','0.044388332749944','141.218758968615','0.652104360215557','0.014090950946023','6','86']
  H_HDV['96_PM_-4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.135949016729324','0.060074190791237','0.563530405370615','0.387923047267261','0.004755720886873','6','86']
  H_HDV['96_FC_-4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','361.094557087531','0.045054430964974','18010.6974885613','0.779234053249422','2.96323624691488','6','86']
  H_HDV['96_CO_-4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004639272962346','2.33626009164921','-0.614014969936382','0.480714926777212','0.035116364600152','6','86']
  H_HDV['96_THC_-4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.088771720010129','0.060354798358394','0.549168555835914','0.554047598863053','0.003702221723156','6','86']
  H_HDV['96_NOx_-4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.46158696765033','0.038863042446962','234.45225164029','0.741821099594834','-0.04242483696285','6','86']
  H_HDV['96_PM_-4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.125482594019795','0.058148248781919','0.995754860120729','0.487460373862427','0.004429898365081','6','86']
  H_HDV['96_FC_-4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','331.885438121609','0.037810555964312','35591.5925969263','0.895590460519825','-4.07069934659468','6','86']
  H_HDV['96_CO_-6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','0','CO','y=(c+(a*exp(b*x)))','0.623005885969419','-0.075486896316466','0.01196021356562','0','0','6','72']
  H_HDV['96_THC_-6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115652809859448','0.071732690235195','0.181886599440581','0.45264056032653','0.003927778368057','6','72']
  H_HDV['96_NOx_-6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.95075709073479','0.066135463556219','111.328268432095','0.633325854476353','0.032174628562233','6','72']
  H_HDV['96_PM_-6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.163566001777584','0.071680641224816','0.527059422940224','0.423579958747006','0.0047921586446','6','72']
  H_HDV['96_FC_-6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','398.815489105098','0.065769725882489','13309.6687021812','0.758071870175595','4.68674302112541','6','72']
  H_HDV['96_CO_-6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006292020130537','3.03765680419813','-1.24240190997148','0.345279869591624','0.049854320201856','6','72']
  H_HDV['96_THC_-6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086450762405785','0.065603640028854','0.844335214232451','0.646641438908395','0.003498896380426','6','72']
  H_HDV['96_NOx_-6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.21575561454233','0.060456967557773','168.112128860543','0.700583008230679','0.022372486966326','6','72']
  H_HDV['96_PM_-6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.129993539921714','0.066679609542455','1.23941157673963','0.545893226321264','0.004326557302383','6','72']
  H_HDV['96_FC_-6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','299.56464076814','0.057879043426489','24721.8387879471','0.847234663670887','3.06572653172298','6','72']
  H_HDV['96_CO_-6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004701265828557','2.33683722976606','-1.01288343378047','0.407958232337416','0.043319749969715','6','72']
  H_HDV['96_THC_-6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.075069937705504','0.063088627231889','0.952955958502618','0.655628111281791','0.003368221800729','6','72']
  H_HDV['96_NOx_-6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.96037389857438','0.05735516350541','200.240001577345','0.73206658667423','0.018849130538715','6','72']
  H_HDV['96_PM_-6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.116739239575648','0.064458522872124','1.44740602329025','0.5695885957931','0.004162091902493','6','72']
  H_HDV['96_FC_-6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','265.142284868598','0.053838296352447','31481.162545104','0.886541631437286','2.43797091704035','6','72']
  H_HDV['96_CO_2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.122094993014848','0.633564431935299','6.90412005034715','2.70939469020997','-0.016939781879239','6','86']
  H_HDV['96_THC_2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006058106325809','0.163123555660921','4.79029803257971','1.94682654000181','-0.009067362044107','6','86']
  H_HDV['96_NOx_2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.49025025483494','0.09363182309924','22.9745874820711','0.470021770119666','3.6148172829603','6','86']
  H_HDV['96_PM_2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.063725895952489','0.028569688229807','0.27700845633107','0.131938530213857','0.020158872958532','6','86']
  H_HDV['96_FC_2_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','0','FC','y=((a*(x**b))+(c*(x**d)))','0.322663988429031','1.34072947093511','2186.62878086149','-0.543262372115443','0','6','86']
  H_HDV['96_CO_2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.23521606631093','-0.002060035120876','-0.119935026848121','0.126580220511878','0','6','86']
  H_HDV['96_THC_2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01045573327728','0.14174187822112','6.10436398376648','2.43928000381002','-0.016408577836219','6','86']
  H_HDV['96_NOx_2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.2921592674115','-0.028149091548301','-1.70633278756086','0.146493804065856','0','6','86']
  H_HDV['96_PM_2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.079655998976655','0.026256963273518','0.278632346099009','0.145350763271325','0.018489722015669','6','86']
  H_HDV['96_FC_2_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','50','FC','y=((a*(b**x))*(x**c))','2021.18014759249','1.00552155128778','-0.426123616009406','0','0','6','86']
  H_HDV['96_CO_2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.425293025648152','0.020583178811972','0.791804243825647','0.142900305536004','0.051682875823056','6','81']
  H_HDV['96_THC_2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012850889805681','0.14049607895268','5.82394512197492','2.36423470001249','-0.014088949964838','6','81']
  H_HDV['96_NOx_2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.97764324513764','0.032326076689494','34.9977756035871','0.442039957322336','5.18660443126375','6','81']
  H_HDV['96_PM_2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.090362261523263','0.025472325130994','0.274536249565594','0.143989245446501','0.015648688881336','6','81']
  H_HDV['96_FC_2_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002477935635433','0.447982565484634','-27.743988609818','1244.74393711401','0','6','81']
  H_HDV['96_CO_4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.20618318822078','1.01221735045728','0.905104676417479','0.110435211793628','0.121362842986635','6','72']
  H_HDV['96_THC_4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00710557437336','0.157159918176289','5.29208826876375','2.12415922112565','-0.011853184659129','6','72']
  H_HDV['96_NOx_4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.20190183042389','0.055671159982965','15.4471828502366','0.22082592566628','5.11517143376953','6','72']
  H_HDV['96_PM_4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.084943794696589','0.036562940906861','0.254913732621492','0.147568528662663','0.027871297981089','6','72']
  H_HDV['96_FC_4_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','775.113176991182','0.091843471379458','5948.57313357973','0.750776671273828','456.704106649645','6','72']
  H_HDV['96_CO_4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.586067550263402','0.014066023016119','0.806680773411709','0.154786014240136','-0.069197452352717','6','69']
  H_HDV['96_THC_4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015287956861484','0.134394729155587','6.61836086333487','2.6671597786371','-0.020015710513383','6','69']
  H_HDV['96_NOx_4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.54451426853865','0.019536349136139','18.7002777699089','0.26325788275648','4.96208072588882','6','69']
  H_HDV['96_PM_4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.111883460483647','0.033738102014832','0.260706152980513','0.15892178138547','0.022928507243397','6','69']
  H_HDV['96_FC_4_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','609.684745805811','0.076458802004946','12004.9692970541','0.81377833404403','750.494376522662','6','69']
  H_HDV['96_CO_4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','100','CO','y=exp((a+(b/x))+(c*log(x)))','2.10272595943597','-3.50584152995907','-0.912144457836814','0','0','6','56']
  H_HDV['96_THC_4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016844460661068','0.134639345607815','6.06632121377462','2.48127768172393','-0.018968895892974','6','56']
  H_HDV['96_NOx_4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','100','NOx','y=((a*(x**b))+(c*(x**d)))','26.8347819263942','-0.268065103116154','265.804670200593','-3.02332460814376','0','6','56']
  H_HDV['96_PM_4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.133063245901621','0.037062709601955','0.260121187722268','0.154500785114805','0.020744137651198','6','56']
  H_HDV['96_FC_4_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','578.851220918851','0.017293545422162','3822.21056435231','0.49835745843629','772.289741892961','6','56']
  H_HDV['96_CO_6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.66099579107757','-0.003743641105176','-0.269259073885669','0.219310567166538','0','6','71']
  H_HDV['96_THC_6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007174481552647','0.142906081431663','6.95699943589881','2.77820807386573','-0.032922754861527','6','71']
  H_HDV['96_NOx_6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.9056672231693','-0.038630645024867','-5.77896808849363','0.25908643348904','0','6','71']
  H_HDV['96_PM_6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.407812753299667','-0.000778089263093','-0.050465910352679','0.154785823930937','0','6','71']
  H_HDV['96_FC_6_0'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','2358.51370150543','1.00888989885137','-0.459672579734786','0','0','6','71']
  H_HDV['96_CO_6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.21744774377084','-4.05813213023464','-0.929869435357335','0','0','6','54']
  H_HDV['96_THC_6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017271938658968','0.133215406517707','6.36669693397685','2.59773406539708','-0.021541233843162','6','54']
  H_HDV['96_NOx_6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.0436980883742','1.26216060502106','-0.211010501198022','0','0','6','54']
  H_HDV['96_PM_6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.128648248888181','0.029023330718838','0.273727707527401','0.149427791060098','0.011998031547779','6','54']
  H_HDV['96_FC_6_50'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','438.097917006218','0.051014220311285','2904.00509468465','0.474923561810203','981.650399357551','6','54']
  H_HDV['96_CO_6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000021484282965','0.002335825905964','-0.09069085299444','1.53575707594084','0','6','40']
  H_HDV['96_THC_6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003342397743','0.000350254150616','-0.012776033999231','0.197655891508264','0','6','40']
  H_HDV['96_NOx_6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.9743756510738','-0.084369125674398','-3.28418174221926','0.170492678162979','0','6','40']
  H_HDV['96_PM_6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024040725660395','0.289342577759026','4.95823294770036','2.08350770229541','-0.013964551860956','6','40']
  H_HDV['96_FC_6_100'] = [' 96','EURO5','TT/AT >34-40t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1089.29134926456','0.009093698811976','6612.31869978983','0.585198454016727','647.921600432222','6','40']
  H_HDV['97_CO_0_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.76288248275142','0.059257787885916','12.1064786248766','0.248144367820161','1.66777158709461','6','86']
  H_HDV['97_THC_0_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.2344339455265','0.038260819376234','5.19104059958776','0.128390612666841','0.456618672941986','6','86']
  H_HDV['97_NOx_0_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.4520569301348','0.052931565925838','95.9529668713496','0.481645129606051','8.97408057020618','6','86']
  H_HDV['97_PM_0_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.67382061429536','0.053949566435847','2.35115943229011','0.210822781264428','0.348212134164048','6','86']
  H_HDV['97_FC_0_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','744.518218072886','0.057794975080997','9575.34704412791','0.667429055449905','217.177019430187','6','86']
  H_HDV['97_CO_0_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1233023537354','0.05619965622802','22.5836214434235','0.326391075271511','2.05597454103748','6','86']
  H_HDV['97_THC_0_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.180063270010372','6.69055258918042','3.53885614262262','1.61540786065601','-0.007326785521305','6','86']
  H_HDV['97_NOx_0_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','50','NOx','y=((a+(b*x))**((-1)/c))','0.002562060636174','0.000175687597971','1.61849289635205','0','0','6','86']
  H_HDV['97_PM_0_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.75761482044205','0.048576953188854','3.77937175362112','0.298429244608436','0.43845937818453','6','86']
  H_HDV['97_FC_0_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','722.584711600693','0.040013103510525','11484.1232638069','0.693315024794515','265.036373650005','6','86']
  H_HDV['97_CO_0_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.34907540483098','91.5842165984199','-0.982488455002761','0.40406824163739','0.044916836456516','6','86']
  H_HDV['97_THC_0_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.63059076646592','0.046793259805411','5.53991457266461','0.159543788300827','0.468766012127117','6','86']
  H_HDV['97_NOx_0_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','100','NOx','y=(1/(a+(b*(x**c))))','0.026650901968768','0.000127328677221','1.30150354324967','0','0','6','86']
  H_HDV['97_PM_0_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.97904528574768','0.044659720603862','6.24975337018502','0.397665832548666','0.504872680907193','6','86']
  H_HDV['97_FC_0_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','819.899193907274','0.028964816690191','308511.715573958','1.2852503706757','278.077748441611','6','86']
  H_HDV['97_CO_-2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.5570193100239','0.011105354568847','-0.848265308372936','0.064172631710982','0','6','86']
  H_HDV['97_THC_-2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.53656607666338','0.063098146488137','6.92024485535457','0.398250299845822','0.370602847079004','6','86']
  H_HDV['97_NOx_-2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','0','NOx','y=((a*(b**x))*(x**c))','68.3105236787404','0.99122610846236','-0.58885625977123','0','0','6','86']
  H_HDV['97_PM_-2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.93433396400227','0.06096556347102','1.07161001911106','0.143361462655825','0.20924200001667','6','86']
  H_HDV['97_FC_-2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','643.530569246283','0.052245974311379','30944.3988457644','0.842851924007471','66.1109050481283','6','86']
  H_HDV['97_CO_-2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6867619926402','0.058012730681993','174.609862041731','0.753873378255523','0.769166248794244','6','86']
  H_HDV['97_THC_-2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.04182284988459','0.057112506932379','9.86559751309803','0.408605001429939','0.234869391755061','6','86']
  H_HDV['97_NOx_-2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000075498082289','0.013736232158127','-0.917024365130208','26.1072296559562','0','6','86']
  H_HDV['97_PM_-2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.05445470063807','0.054144834227351','5.86101469278482','0.434406741674209','0.185911824489055','6','86']
  H_HDV['97_FC_-2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','593.727745075305','0.032453449497882','48934.6917986159','0.890010475919151','13.1761907897936','6','86']
  H_HDV['97_CO_-2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0037553075988','0.047993229474671','295.150109901606','0.784474681705056','0.637305714905795','6','86']
  H_HDV['97_THC_-2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.68820349815246','0.054939608271383','11.3445303921923','0.405802491978143','0.198280274732559','6','86']
  H_HDV['97_NOx_-2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.655846420274','-0.133922294582531','-0.98227722826151','0.061126208582722','0','6','86']
  H_HDV['97_PM_-2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.00312705908747','0.046467057350128','16.5676210640936','0.580063100538754','0.169294476327947','6','86']
  H_HDV['97_FC_-2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-2','100','FC','y=((a*(x**b))+(c*(x**d)))','-3.28510132320282','0.901271291018488','1498.12455745009','-0.435667870146375','0','6','86']
  H_HDV['97_CO_-4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.4424475395596','0.062458370862947','43.8946340203863','0.564188973016795','0.311760894068038','6','86']
  H_HDV['97_THC_-4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.76844137036693','0.058810585716682','13.8034011075007','0.464076330025435','0.176610431030761','6','86']
  H_HDV['97_NOx_-4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.3882900334357','0.047441301741886','420.293652101345','0.682870680615794','0.343431178921124','6','86']
  H_HDV['97_PM_-4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108449800446628','4.55369057428163','1.13783731418377','0.778194582056392','0.034725188456275','6','86']
  H_HDV['97_FC_-4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','477.581342687654','0.049431055663489','24032.5818115176','0.7773186722662','8.66623299034543','6','86']
  H_HDV['97_CO_-4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.75275816969105','0.052154531741307','180.473674968209','0.735622121487415','0.202624227242251','6','86']
  H_HDV['97_THC_-4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.40156758272271','0.057781773355187','25.2018964587318','0.562239511500602','0.157617406290375','6','86']
  H_HDV['97_NOx_-4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.352597506316','0.035938731227301','1398.75155035899','0.888618180717502','-0.140884023967834','6','86']
  H_HDV['97_PM_-4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.76550867389593','0.054528815319396','7.36909034450557','0.463867790607455','0.11615594175682','6','86']
  H_HDV['97_FC_-4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','416.003876954115','0.040398407210654','56410.9255305413','0.922034570704187','-1.71694886605108','6','86']
  H_HDV['97_CO_-4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','100','CO','y=((a*(x**b))+(c*(x**d)))','-2.53356080855095','-0.032334706315651','26.4262476501735','-0.543874041642604','0','6','86']
  H_HDV['97_THC_-4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.20616225544067','0.057415318113803','30.987605995754','0.596692452934969','0.152753086928161','6','86']
  H_HDV['97_NOx_-4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.5109677364428','0.028514717747905','3664.08485649951','1.05420550447645','-0.767311879892592','6','86']
  H_HDV['97_PM_-4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.74700132974564','0.050668894130873','15.6755594237825','0.583512867359411','0.108164791941984','6','86']
  H_HDV['97_FC_-4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','372.469383097309','0.030312841465363','5525.672117662','0.562865466496243','-19.3811463233797','6','86']
  H_HDV['97_CO_-6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.49064644552297','0.067867463070522','91.8951180886809','0.69722076783008','0.203331266693106','6','72']
  H_HDV['97_THC_-6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.63573504813296','0.064168288454945','26.5941928419714','0.595787818734926','0.146365585619962','6','72']
  H_HDV['97_NOx_-6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.8969439958351','0.060942086338522','708.310820675227','0.797678776721084','0.230373660269942','6','72']
  H_HDV['97_PM_-6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.88535025175242','0.064782034535389','2.90552863857351','0.348113979954951','0.115553595141966','6','72']
  H_HDV['97_FC_-6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','416.180255339865','0.059857700963694','31837.430474877','0.841180605168238','3.10791802698369','6','72']
  H_HDV['97_CO_-6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.71533235685896','0.059497669955441','179.620167277306','0.750996123037458','0.159612516728219','6','72']
  H_HDV['97_THC_-6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.14692281803293','0.06170724973007','31.2438451111887','0.602792503522822','0.139706096648456','6','72']
  H_HDV['97_NOx_-6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.0180814842573','0.05384931816648','1076.23686466334','0.861276437999471','0.183188939402192','6','72']
  H_HDV['97_PM_-6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.71347023307186','0.060959235273394','6.82450044192997','0.469476352639381','0.111342181890546','6','72']
  H_HDV['97_FC_-6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','334.780614126023','0.053978516140819','41129.749140953','0.877389533587249','1.71460689759398','6','72']
  H_HDV['97_CO_-6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.15182893606229','0.055112745828507','227.416452987674','0.765251082307837','0.136108659166762','6','72']
  H_HDV['97_THC_-6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.93995764207023','0.060990005877916','25.8435341672433','0.562411141862478','0.138843842867385','6','72']
  H_HDV['97_NOx_-6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2141766210492','0.050449123991352','1281.35465721767','0.888758635673567','0.18058546780373','6','72']
  H_HDV['97_PM_-6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.65545658291852','0.058702546671298','10.2235380762164','0.521507901011073','0.109216042528763','6','72']
  H_HDV['97_FC_-6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','307.423119544057','0.051342900364799','43589.3086698885','0.884827433940542','1.78073925319966','6','72']
  H_HDV['97_CO_2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.15193514798508','11.7993993847361','5.79204889348179','2.1584406757477','0.00871809804042','6','86']
  H_HDV['97_THC_2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.096580163160004','6.47015943161974','4.38673816143789','1.77364269686366','-0.008423007480903','6','86']
  H_HDV['97_NOx_2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-26.2113424347514','0.038392102014604','56.7909549233519','0.062620359049584','19.0997500330407','6','86']
  H_HDV['97_PM_2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.534207778629737','2.1830752650951','5.24594489988702','2.00133719102065','0.011837810952706','6','86']
  H_HDV['97_FC_2_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1558.34209143082','0.540128869209451','-94.8025776392197','0.078793076411516','0','6','86']
  H_HDV['97_CO_2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.02977623121052','0.006566654898689','-0.000023968157416','0','0','6','83']
  H_HDV['97_THC_2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.468849641625715','5.47346489126747','6.24040197246556','2.49668228245495','-0.015465530764613','6','83']
  H_HDV['97_NOx_2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000099249629804','0.018174154186042','-1.09862523302535','52.5109497223341','0','6','83']
  H_HDV['97_PM_2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.546928874824475','3.64153729188839','2.61203161020243','1.26622078075235','-0.000891157234187','6','83']
  H_HDV['97_FC_2_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','50','FC','y=(c+(a*exp(b*x)))','1133.12565650551','-0.083542989856253','651.229656253226','0','0','6','83']
  H_HDV['97_CO_2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','100','CO','y=((a*(b**x))*(x**c))','47.7051393347611','0.992247413442738','-0.55170132578514','0','0','6','66']
  H_HDV['97_THC_2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.570003294077293','5.22410059620012','6.53184169711913','2.63463213244688','-0.019524156289034','6','66']
  H_HDV['97_NOx_2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','100','NOx','y=(c+(a*exp(b*x)))','23.8906741855913','-0.023663518903912','35.0562464655057','0','0','6','66']
  H_HDV['97_PM_2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','100','PM','y=((a*(x**b))+(c*(x**d)))','-0.000079480376502','1.67942802216462','7.07990388198987','-0.464928240452736','0','6','66']
  H_HDV['97_FC_2_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.005761293666676','0.847363800950675','-44.9398193495912','1764.26002249496','0','6','66']
  H_HDV['97_CO_4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.11195347928938','13.2325927135472','6.00885710344648','2.40516823142924','-0.018070430150658','6','71']
  H_HDV['97_THC_4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.263695122193681','6.28208475435366','5.39259587198214','2.15639651997547','-0.011689392082276','6','71']
  H_HDV['97_NOx_4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','62.806331459975','0.040472491718306','-3.82434104654048','0.100468651173801','0','6','71']
  H_HDV['97_PM_4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.60673359796053','2.26833848895565','5.68723937467727','2.32753578616067','-0.02037760961775','6','71']
  H_HDV['97_FC_4_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','0','FC','y=(c+(a*exp(b*x)))','1392.33155199409','-0.100357236802333','560.511334856913','0','0','6','71']
  H_HDV['97_CO_4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.6481423924763','-0.044241723868313','-2.15690391850014','0.107960284621801','0','6','60']
  H_HDV['97_THC_4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.693059149972194','5.19143422991589','7.64745451780756','3.09010934250945','-0.028254806235467','6','60']
  H_HDV['97_NOx_4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000194657211257','0.025800435002617','-1.20441186008856','67.6985713888966','0','6','60']
  H_HDV['97_PM_4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.867644908985702','3.99862798083538','2.3522265303138','1.22487649557067','-0.00041209006615','6','60']
  H_HDV['97_FC_4_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','50','FC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','964.090710913969','3540.30827354142','0.463633604285891','0.967432568903746','0.017035460237024','6','60']
  H_HDV['97_CO_4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','100','CO','y=((a*(b**x))*(x**c))','58.0560155661355','0.994688379960079','-0.59168418656414','0','0','6','44']
  H_HDV['97_THC_4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.988271097884407','4.54783607384973','9.29098259907836','3.80981498323358','-0.047979839108686','6','44']
  H_HDV['97_NOx_4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000513070810606','0.037896147203637','-1.14951867067512','84.8884102570687','0','6','44']
  H_HDV['97_PM_4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.41971042022183','-0.021344096166745','-0.752952960201838','0.187305603607039','0','6','44']
  H_HDV['97_FC_4_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.022002950968947','2.0123196734861','-68.1159978582765','2349.69243121152','0','6','44']
  H_HDV['97_CO_6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.5513481341024','-0.041562787941093','-2.59420190384268','0.128723008761624','0','6','70']
  H_HDV['97_THC_6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.625485074450627','5.46239554252268','7.05034634717421','2.68889339380216','-0.00181310289997','6','70']
  H_HDV['97_NOx_6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','0','NOx','y=((a*(b**x))*(x**c))','95.4284391395138','1.00776941844678','-0.351955801092085','0','0','6','70']
  H_HDV['97_PM_6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.27026721927251','-0.004374489620525','-0.35645711787686','0.114528816527692','0','6','70']
  H_HDV['97_FC_6_0'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1955.76502836496','4.48010959210445','-99.4081560296924','0.070108427620171','0','6','70']
  H_HDV['97_CO_6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','30.409400277849','-0.064543066608814','-3.02942456952529','0.131634299380326','0','6','44']
  H_HDV['97_THC_6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.17995983491855','33.8057860777806','-2.27033839196407','-0.782574253090015','0.180346970246021','6','44']
  H_HDV['97_NOx_6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','27.3233089285744','0.009227583543155','255.75932888551','0.599769062514713','46.1895215454686','6','44']
  H_HDV['97_PM_6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.453233527731','-0.008834111661113','-0.49270193727928','0.140507015559215','0','6','44']
  H_HDV['97_FC_6_50'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','50','FC','y=exp((a+(b/x))+(c*log(x)))','7.37099794868224','1.85718876985453','-0.045485826577936','0','0','6','44']
  H_HDV['97_CO_6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000906449135825','0.072520603474062','-2.15561939437897','31.3769674863734','0','6','31']
  H_HDV['97_THC_6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.34166753126459','4.39843774152487','8.66954198432169','3.75987736945774','-0.057264258988936','6','31']
  H_HDV['97_NOx_6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002831052485128','0.163349278235346','-3.21124021804501','119.269147417055','0','6','31']
  H_HDV['97_PM_6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00017231966957','0.012544984115293','-0.33433234088935','5.76513983282654','0','6','31']
  H_HDV['97_FC_6_100'] = [' 97','EURO0','TT/AT >40-50t 80ties','6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.088377503882293','5.64808300721457','-125.595747704746','3093.27837290846','0','6','31']
  H_HDV['98_CO_0_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.93346643963263','0.055637441170166','10.7673422022453','0.24082089000224','1.28827300746502','6','86']
  H_HDV['98_THC_0_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.184892583067772','5.5406276228','3.33939427933071','1.52389576043873','-0.006745097940533','6','86']
  H_HDV['98_NOx_0_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.9545772351071','0.053786678632985','52.5632073631685','0.385985179700448','6.52716622569684','6','86']
  H_HDV['98_PM_0_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.36869769752032','0.051132338201818','2.73125766199743','0.174362826632758','0.234694350426437','6','86']
  H_HDV['98_FC_0_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','595.61916020644','0.057269521583214','13907.6271019463','0.755820187171539','185.295350553458','6','86']
  H_HDV['98_CO_0_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.040713084152708','0.010473227728109','-0.000057868849224','0','0','6','86']
  H_HDV['98_THC_0_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54862065125876','0.030907011010201','4.54974636020157','0.124561167668295','0.371447751583643','6','86']
  H_HDV['98_NOx_0_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','50','NOx','y=(1/(a+(b*(x**c))))','0.024696905338617','0.004906471372312','0.661706333179166','0','0','6','86']
  H_HDV['98_PM_0_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.52557895754064','0.049850402893963','3.33783402620298','0.248327968538804','0.321479207585318','6','86']
  H_HDV['98_FC_0_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','602.167519829731','0.038301933734145','47376.4618974517','0.968028086679982','232.387887616347','6','86']
  H_HDV['98_CO_0_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.5952942023187','0.026580591779979','-0.610446835605209','0.04772140870424','0','6','86']
  H_HDV['98_THC_0_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.92290146591772','0.037501980804697','4.11598715302257','0.139171220596727','0.463345979400275','6','86']
  H_HDV['98_NOx_0_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000033495558819','0.006404679175952','-0.541348063145901','29.5701937944487','0','6','86']
  H_HDV['98_PM_0_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72064783950311','0.047035792669418','4.03656179921628','0.319283637129896','0.409028899010297','6','86']
  H_HDV['98_FC_0_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','0','100','FC','y=(c+(a*exp(b*x)))','739.371742472135','-0.031289444954889','262.556345997013','0','0','6','86']
  H_HDV['98_CO_-2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.5994592358873','0.008783342674924','-0.667293162570606','0.064671598372716','0','6','86']
  H_HDV['98_THC_-2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.274533479627379','6.92456138825277','1.40303864734064','0.778831156260346','0.037609254288147','6','86']
  H_HDV['98_NOx_-2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','0','NOx','y=((a*(b**x))*(x**c))','63.620740890314','0.992165158246692','-0.66102340627644','0','0','6','86']
  H_HDV['98_PM_-2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.058597082216485','2.17333947692842','7.65379538413485','2.99822866840825','-0.035211003054051','6','86']
  H_HDV['98_FC_-2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','0','FC','y=((a*(b**x))*(x**c))','2126.77794852247','0.993360648606655','-0.698155761532564','0','0','6','86']
  H_HDV['98_CO_-2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.560132373529173','113.238767540704','-1.75812659374032','0.388572122142765','0.039955736114495','6','86']
  H_HDV['98_THC_-2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.16250982521193','0.022566564186352','0.000457103709929','0','0','6','86']
  H_HDV['98_NOx_-2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.0811156586041','0.025799500941942','319.866395131514','0.652787868353923','-0.337605238224093','6','86']
  H_HDV['98_PM_-2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.088001478739275','2.6862525809803','3.92543097153599','1.62272187266147','0.006941044931188','6','86']
  H_HDV['98_FC_-2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','514.002364939564','0.032700124882476','39551.8638461302','0.88417531178938','14.9415779142897','6','86']
  H_HDV['98_CO_-2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.7410673798499','0.043646070783296','231.262208793673','0.778283935752525','0.505341133057398','6','86']
  H_HDV['98_THC_-2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','100','THC','y=((a*(b**x))*(x**c))','9.85279691215802','0.981406433695775','-0.579819260516496','0','0','6','86']
  H_HDV['98_NOx_-2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.6608368002377','-0.095406825947619','-1.10885987345903','0.074786157352489','0','6','86']
  H_HDV['98_PM_-2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.230534565533316','0.040014585824873','0.00052835936841','0','0','6','86']
  H_HDV['98_FC_-2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','759.247150329325','-2.56675476321795','-36.4065753776919','0.066756573066111','0','6','86']
  H_HDV['98_CO_-4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.125193383485035','83.8713211893184','-1.35225136773478','0.530284048952853','0.03618448524971','6','86']
  H_HDV['98_THC_-4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.065553173673557','32.5007741416192','-1.10377850829839','0.574468108280528','0.031631426329913','6','86']
  H_HDV['98_NOx_-4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.950945080671','0.05134606384269','162.93751701607','0.550018536911797','0.339627522089266','6','86']
  H_HDV['98_PM_-4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.31222571177686','-8.888317587488','-1.76874721951162','0','0','6','86']
  H_HDV['98_FC_-4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','420.144745041971','0.050708542274071','14679.5714224834','0.722599879545454','8.37896370032981','6','86']
  H_HDV['98_CO_-4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.84981980844759','0.050159073006684','107.247948239749','0.680319231677308','0.120728033930876','6','86']
  H_HDV['98_THC_-4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.61462851651259','0.054627211166691','14.5282420309162','0.488946822407271','0.096084287613416','6','86']
  H_HDV['98_NOx_-4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7373913907485','0.039999698213309','509.34929074097','0.741528517067934','0.059311467272159','6','86']
  H_HDV['98_PM_-4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.256732627170569','0.043719430617273','0.001434895733407','0','0','6','86']
  H_HDV['98_FC_-4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','370.326848325625','0.040802300755643','36306.950495905','0.872899998970231','-0.764151203343428','6','86']
  H_HDV['98_CO_-4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.57215898867692','0.042021231426407','287.504410162861','0.81965079003806','0.018492200263435','6','86']
  H_HDV['98_THC_-4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.40267063284745','0.052740612373863','20.0015002613399','0.539907706626933','0.088974466889482','6','86']
  H_HDV['98_NOx_-4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4175804075218','0.03262379449447','1028.29306844117','0.865126706167016','-0.275350014480993','6','86']
  H_HDV['98_PM_-4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5347969445527','0.052059112322167','10.0830872879859','0.484609918572139','0.057148838999298','6','86']
  H_HDV['98_FC_-4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','362.753875605918','0.033914257070255','66256.9051189377','0.975284668283453','-11.0872630748869','6','86']
  H_HDV['98_CO_-6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.56991651373173','0.0694722614455','42.6043460549628','0.599770871893896','0.139384780264737','6','72']
  H_HDV['98_THC_-6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.01481978043367','0.065505111208564','17.7009121524522','0.572134663114813','0.098767147318595','6','72']
  H_HDV['98_NOx_-6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3547504287523','0.065522776780115','349.370278533061','0.708677517322467','0.238104650326106','6','72']
  H_HDV['98_PM_-6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.08620413254239','0.073237901848394','1.57078814446246','0.249913886783579','0.069413352914619','6','72']
  H_HDV['98_FC_-6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','376.60394019481','0.061442779907455','21790.5106962617','0.806045813280829','3.74440304583071','6','72']
  H_HDV['98_CO_-6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.15393785592421','0.06012518534406','118.992147687904','0.722264693611211','0.106265428244227','6','72']
  H_HDV['98_THC_-6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.49665967923942','0.061898853821126','25.823533230833','0.610394030768647','0.091476283032543','6','72']
  H_HDV['98_NOx_-6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.41091241433591','0.057579252593472','554.368310527666','0.775553833728606','0.19259422614393','6','72']
  H_HDV['98_PM_-6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66647499325557','0.066743009247132','7.63922511824754','0.492982885970757','0.06266960660316','6','72']
  H_HDV['98_FC_-6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','303.617254122456','0.054683393748844','31224.9047116795','0.857355698023476','2.24475212657314','6','72']
  H_HDV['98_CO_-6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.6424616176206','0.053932089974649','190.241408591374','0.773387194603131','0.078832726213623','6','72']
  H_HDV['98_THC_-6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.29218164759172','0.060622211256516','21.5291802293691','0.570192517817006','0.090273299953577','6','72']
  H_HDV['98_NOx_-6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.63560008244193','0.054327416182902','660.401747283616','0.802979033676294','0.190799411417424','6','72']
  H_HDV['98_PM_-6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5034826796517','0.06271236200369','12.0849978355938','0.551786823542935','0.05961911466797','6','72']
  H_HDV['98_FC_-6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','281.762840967335','0.051651501117059','35883.8097566421','0.878984603683046','2.05024682883995','6','72']
  H_HDV['98_CO_2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.3507604658458','0.012520929727851','-0.920167528755421','0.069699915549571','0','6','86']
  H_HDV['98_THC_2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.228728401911347','4.78879610019051','5.13125442417184','2.05124729440213','-0.016556255512528','6','86']
  H_HDV['98_NOx_2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.6185100959955','-0.00838903163577','-2.83176401651717','0.103171638927722','0','6','86']
  H_HDV['98_PM_2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.371892576754185','2.02593354511549','5.42848340383593','1.99698045953305','0.025441248567693','6','86']
  H_HDV['98_FC_2_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','0','FC','y=((a*(b**x))*(x**c))','2700.40930854912','1.00967830850887','-0.641079116121478','0','0','6','86']
  H_HDV['98_CO_2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.408329876224','-0.005881183142147','-1.26216121466639','0.090128166006194','0','6','84']
  H_HDV['98_THC_2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.677808729133537','4.12225778232379','6.40707916534828','2.53242590514852','-0.015337089839079','6','84']
  H_HDV['98_NOx_2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.1879871278293','0.050150639011421','380.680145548488','0.804325717021862','19.3999944744759','6','84']
  H_HDV['98_PM_2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','50','PM','y=(c+(a*exp(b*x)))','2.60628843589325','-0.089546910154015','0.656667892450059','0','0','6','84']
  H_HDV['98_FC_2_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','50','FC','y=((a*(b**x))*(x**c))','2199.34383937977','1.00495164843402','-0.396653303291051','0','0','6','84']
  H_HDV['98_CO_2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','100','CO','y=((a+(b*x))**((-1)/c))','0.001194336683241','0.001546567133102','1.74238954243585','0','0','6','68']
  H_HDV['98_THC_2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.782054840403874','4.26283795341992','5.5419338690155','2.23198269966053','-0.013171320276497','6','68']
  H_HDV['98_NOx_2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000060269250275','0.010254510876623','-0.687534595739536','44.3817919791373','0','6','68']
  H_HDV['98_PM_2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','100','PM','y=((a+(b*x))**((-1)/c))','-0.030542083979537','0.024120631816187','2.43724185356612','0','0','6','68']
  H_HDV['98_FC_2_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','2','100','FC','y=((a*(b**x))*(x**c))','1899.16240582738','0.999145395923694','-0.207329026289923','0','0','6','68']
  H_HDV['98_CO_4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.70660361938603','90.1111414069315','-2.97167322055422','-0.939519152057957','0.178039727011909','6','71']
  H_HDV['98_THC_4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.519627955034245','4.60217876510362','5.8514865956834','2.32765379115209','-0.014321472342468','6','71']
  H_HDV['98_NOx_4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.6158979359463','0.086743352930544','28.1324336158394','0.342614944623849','18.4622335805929','6','71']
  H_HDV['98_PM_4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.510125497313035','1.64348012729654','6.72478333938255','2.50553950482224','0.011476180555728','6','71']
  H_HDV['98_FC_4_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','807.403492979178','0.089847960028068','3479.06941041215','0.594687861218032','500.63161414398','6','71']
  H_HDV['98_CO_4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.1726162729634','-0.062392201489931','-2.69247231529115','0.166914709104472','0','6','61']
  H_HDV['98_THC_4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.01491658415174','3.8797553093338','7.46010363666785','3.00331406496645','-0.027791275192685','6','61']
  H_HDV['98_NOx_4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.91983869972097','0.750880052124035','-0.125976596764327','0','0','6','61']
  H_HDV['98_PM_4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','50','PM','y=((a*(b**x))*(x**c))','5.59568525334446','1.0079494182446','-0.534181147434411','0','0','6','61']
  H_HDV['98_FC_4_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','523.983225347243','0.053818590683962','5001.91019454329','0.550602452878164','861.874060546836','6','61']
  H_HDV['98_CO_4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','37.9494604667073','-0.15208842944032','-7.32922688515058','0.273300350728984','0','6','45']
  H_HDV['98_THC_4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000091527573029','0.010037955243258','-0.378337731887899','6.61358871043877','0','6','45']
  H_HDV['98_NOx_4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000459387420156','0.04068929553427','-1.41387858203971','64.7589938830364','0','6','45']
  H_HDV['98_PM_4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.28981383334531','-0.016302234222488','-1.31669720822568','0.312414713314993','0','6','45']
  H_HDV['98_FC_4_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2233.20761690488','0.004665889619096','12475.9958666515','0.67752695464196','-590.109348752472','6','45']
  H_HDV['98_CO_6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.02079746403929','9.23312417831047','7.15149394257131','2.90703930904753','-0.022133408742533','6','71']
  H_HDV['98_THC_6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.0137212870899','0.005938713051819','-0.500424396558879','0.07825883358079','0','6','71']
  H_HDV['98_NOx_6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','54.8087768048966','-0.040208281406831','-3.85969539218178','0.134467944762204','0','6','71']
  H_HDV['98_PM_6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.84258849869736','0.008763063860086','-0.166811880386865','0.065572868301726','0','6','71']
  H_HDV['98_FC_6_0'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2743.81832562859','1.01009866218129','-0.490143041672322','0','0','6','71']
  H_HDV['98_CO_6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','25.9656807529099','-0.115528164789016','-3.44063387134104','0.209938324951955','0','6','45']
  H_HDV['98_THC_6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.63720396127834','14.0657749118985','-1.47122396319291','-0.707587820518919','0.181501866644302','6','45']
  H_HDV['98_NOx_6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.4324425687455','0.025325417529371','54.2490624842051','0.339260406152114','36.5819036371918','6','45']
  H_HDV['98_PM_6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','50','PM','y=exp((a+(b/x))+(c*log(x)))','0.58012064597418','2.65197583561753','-0.072323409381644','0','0','6','45']
  H_HDV['98_FC_6_50'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','446.054432329763','0.042827481928993','5012.07702685645','0.515251246679745','1156.80159084843','6','45']
  H_HDV['98_CO_6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.0854973321666','0.018804717144674','79.6269305141572','0.538130650271008','-4.58211725344451','6','32']
  H_HDV['98_THC_6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000227018397005','0.01826010650405','-0.51510876266848','7.52867216600649','0','6','32']
  H_HDV['98_NOx_6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00180355926034','0.119173424861409','-2.87563507129529','88.9597656860748','0','6','32']
  H_HDV['98_PM_6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.26394066176862','0.00605649335049','75.9445586555688','0.876101777823359','-1.44387153381191','6','32']
  H_HDV['98_FC_6_100'] = [' 98','EURO1','TT/AT >40-50t Euro-1','6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.067255843535984','4.3841747679837','-100.918169750068','2683.66364792251','0','6','32']
  H_HDV['99_CO_0_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.54638420915027','0.056352599760434','7.4577317660927','0.266196716801895','1.11784485457232','6','86']
  H_HDV['99_THC_0_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.088425680675605','3.66410894827622','3.11070965384556','1.44936413254548','-0.006616339421748','6','86']
  H_HDV['99_NOx_0_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.54710819688966','218.850034149431','-1.11798031556632','0.608021636630482','0.0269727421339','6','86']
  H_HDV['99_PM_0_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.600996336891751','0.000792794074051','-0.027725468547464','0.05299448333197','0','6','86']
  H_HDV['99_FC_0_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','529.108247763608','0.055441365786762','14181.7292559918','0.781858363924799','179.736948372047','6','86']
  H_HDV['99_CO_0_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.070543535739778','0.012294491392942','-0.000074488418112','0','0','6','86']
  H_HDV['99_THC_0_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.85587757579362','0.022285212550804','3.02809179928118','0.116839923317653','0.161883162534435','6','86']
  H_HDV['99_NOx_0_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','50','NOx','y=(1/(a+(b*(x**c))))','0.019202358805903','0.00566236415452','0.635706527641712','0','0','6','86']
  H_HDV['99_PM_0_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000117842856999','-0.014568680745065','0.638434997353188','0','0','6','86']
  H_HDV['99_FC_0_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','551.690853538542','0.035518736158286','78911.845281247','1.08173487401191','223.253910475875','6','86']
  H_HDV['99_CO_0_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.66784275954568','0.101679642630798','-0.328101817627832','0.023795591105329','0','6','86']
  H_HDV['99_THC_0_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.883011859347718','0.019967900852298','2.97070494385037','0.117973209877204','0.146944540569648','6','86']
  H_HDV['99_NOx_0_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000051031220007','0.009576981080415','-0.721579553306872','32.7901353956801','0','6','86']
  H_HDV['99_PM_0_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000000806170417','0.000046054719462','-0.014983285561732','0.794495625176299','0','6','86']
  H_HDV['99_FC_0_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','0','100','FC','y=(c+(a*exp(b*x)))','686.962616076064','-0.026755695606221','236.445070509191','0','0','6','86']
  H_HDV['99_CO_-2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.8929319888422','0.00878410004473','-0.409041640214637','0.059979834499763','0','6','86']
  H_HDV['99_THC_-2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.173936091156111','7.69468804558282','0.241215874347321','0.608035389150453','0.035898705919503','6','86']
  H_HDV['99_NOx_-2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','0','NOx','y=((a*(b**x))*(x**c))','68.5604345926125','0.991728405723567','-0.64881928164529','0','0','6','86']
  H_HDV['99_PM_-2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.585102716795941','0.000570019765749','-0.031261936582528','0.055791669702806','0','6','86']
  H_HDV['99_FC_-2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','493.452578179915','0.051311172852772','19059.9141352466','0.814762486075519','52.9545863480647','6','86']
  H_HDV['99_CO_-2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','50','CO','y=(c+(a*exp(b*x)))','7.02232651893356','-0.062588761559175','0.602094668936148','0','0','6','86']
  H_HDV['99_THC_-2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.243071251616317','0.037411681766258','0.000547921835759','0','0','6','86']
  H_HDV['99_NOx_-2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.020432257901688','1.13904365231038','53.4570868664601','-0.563243862896192','0','6','86']
  H_HDV['99_PM_-2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.531740386170689','0.049504826222034','12.5804448620647','0.846726638647829','0.066652951515543','6','86']
  H_HDV['99_FC_-2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','50','FC','y=((a*(b**x))*(x**c))','1019.58957351601','0.980028545174082','-0.306305930159464','0','0','6','86']
  H_HDV['99_CO_-2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.79133317881051','0.040113717733065','537.596340217953','0.966875532549865','0.407249577186663','6','86']
  H_HDV['99_THC_-2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','100','THC','y=((a*(b**x))*(x**c))','6.40155548194576','0.983356763615534','-0.587472753439441','0','0','6','86']
  H_HDV['99_NOx_-2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.975228946914','-0.095762908125094','-1.26329263879469','0.073928832048472','0','6','86']
  H_HDV['99_PM_-2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','100','PM','y=(c+(a*exp(b*x)))','0.656124350062253','-0.042738411643403','0.065480177434629','0','0','6','86']
  H_HDV['99_FC_-2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','681.485531988894','-2.4837102034023','-29.4774550917156','0.061836693889792','0','6','86']
  H_HDV['99_CO_-4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.41294075525759','0.057766502619669','10.6265231644619','0.37314711298811','0.160922453210354','6','86']
  H_HDV['99_THC_-4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.050371214144115','21.8296846033571','-1.1539369897045','0.584093365343595','0.027371420400137','6','86']
  H_HDV['99_NOx_-4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.403094404342','0.051552368507378','156.430784589623','0.533429338923546','0.391957736260413','6','86']
  H_HDV['99_PM_-4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.36077737134105','-5.63981831009897','-1.27638802724655','0','0','6','86']
  H_HDV['99_FC_-4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','385.224567868857','0.04992236332978','12659.806749504','0.721091403273638','8.58040929337207','6','86']
  H_HDV['99_CO_-4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.02153671913482','0.045566143858578','78.7693126500095','0.671016833195126','0.081768431329088','6','86']
  H_HDV['99_THC_-4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.64745599102366','0.050856064007115','5.65425058263953','0.390923433656739','0.080031453829722','6','86']
  H_HDV['99_NOx_-4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.9105625333178','0.040867105326994','518.380290530139','0.734374910662103','0.1001882774108','6','86']
  H_HDV['99_PM_-4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','1.12586948151268','0.983133492522119','-0.460095033429186','0','0','6','86']
  H_HDV['99_FC_-4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','339.824142914433','0.039155341287094','28903.8740668983','0.856990661637397','-1.29790393391269','6','86']
  H_HDV['99_CO_-4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.15806339361201','0.037589831313775','316.802284678102','0.876867294238007','-0.031428124498288','6','86']
  H_HDV['99_THC_-4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5650327683067','0.050114079831973','6.1316818312887','0.407169287527996','0.077960556137668','6','86']
  H_HDV['99_NOx_-4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.5224476733469','0.034509506780024','1232.65067148851','0.885388515792765','-0.188318607129841','6','86']
  H_HDV['99_PM_-4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.47604962335518','0.036762371037173','16.282268271202','0.82744739641354','0.015704956232793','6','86']
  H_HDV['99_FC_-4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','342.319112970457','0.03292372082655','78965.8469821854','1.02697694744591','-11.2102285937731','6','86']
  H_HDV['99_CO_-6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.47472714613955','0.068939414121814','23.1197568982383','0.546604681275524','0.124043937614073','6','72']
  H_HDV['99_THC_-6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.93581824817836','0.061828107188975','7.38228248066931','0.496881325007626','0.089516172055625','6','72']
  H_HDV['99_NOx_-6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.8033688646251','0.065942828427327','332.063910448458','0.692134609757175','0.284109485268986','6','72']
  H_HDV['99_PM_-6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005687290279599','0.516526834066216','4.57118704479396','1.88426476485051','-0.01231977411696','6','72']
  H_HDV['99_FC_-6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.448774462056','0.061252461100253','18299.8880782006','0.800696505723401','4.5997942382554','6','72']
  H_HDV['99_CO_-6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.71784549686271','0.058553285249462','75.0792689574822','0.704822674100809','0.096216740771916','6','72']
  H_HDV['99_THC_-6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72495824288998','0.059879194897348','8.78273473244471','0.508524908373135','0.086819138890537','6','72']
  H_HDV['99_NOx_-6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.356466237546','0.057732451067641','577.790575156539','0.772057717431879','0.228949666526642','6','72']
  H_HDV['99_PM_-6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.404560106278874','0.053924394470077','0.524779223644578','0.277274717696911','0.034830706031948','6','72']
  H_HDV['99_FC_-6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','281.695959984668','0.054195754246588','28157.7604328688','0.863214408716945','3.13013711227174','6','72']
  H_HDV['99_CO_-6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.42940966216784','0.051286967765203','174.898715534807','0.811915828934705','0.068877374426734','6','72']
  H_HDV['99_THC_-6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.62254329814157','0.058979502641566','7.15062901805014','0.464737225127887','0.086170455848342','6','72']
  H_HDV['99_NOx_-6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.38297637698681','0.054144308834122','686.953313510945','0.799056459542106','0.215199892193265','6','72']
  H_HDV['99_PM_-6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.42694867857104','0.05080053649305','2.79180134044714','0.533846300087682','0.033276756017469','6','72']
  H_HDV['99_FC_-6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','262.278053261137','0.050844794223529','34770.0692943826','0.89789648893437','2.74911621213696','6','72']
  H_HDV['99_CO_2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.59285563634113','0.007168110238335','-0.607598627973004','0.07502131747813','0','6','86']
  H_HDV['99_THC_2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.165335646736133','3.23179792757373','4.36209202258042','1.78001468895588','-0.007648866312843','6','86']
  H_HDV['99_NOx_2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','44.1331661729215','-0.016388336289426','-2.92528502908251','0.09700299581734','0','6','86']
  H_HDV['99_PM_2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.20936215091025','0.114654494806','-0.000949578956582','0','0','6','86']
  H_HDV['99_FC_2_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','0','FC','y=((a*(b**x))*(x**c))','2298.30407222316','1.00914089579778','-0.597789868802175','0','0','6','86']
  H_HDV['99_CO_2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.072108030258409','0.007521967119797','-0.000029086307457','0','0','6','86']
  H_HDV['99_THC_2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.394684061658353','2.68606209466479','6.1651421298402','2.45274675363257','-0.014784118008799','6','86']
  H_HDV['99_NOx_2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','50','NOx','y=((a*(b**x))*(x**c))','67.2358870824734','1.00190960815498','-0.323835401845597','0','0','6','86']
  H_HDV['99_PM_2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.858766877357141','0.002517358147769','-0.024124773732527','0.032322247011529','0','6','86']
  H_HDV['99_FC_2_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','167604.415067141','1.26069684072359','609.486739296611','0.067577221538708','572.692340780374','6','86']
  H_HDV['99_CO_2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.04403788279842','0.029005755750234','33.2151994668601','0.40709826363594','0.753471821720502','6','72']
  H_HDV['99_THC_2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.532854872074682','2.48375714177803','6.63982404760703','2.64592009917352','-0.019219763375205','6','72']
  H_HDV['99_NOx_2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00009598060473','0.016077444506069','-1.00643314842308','49.4060301374609','0','6','72']
  H_HDV['99_PM_2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.804786391000529','0.027933000741721','39.24820775009','0.912418338038088','0.199859288623291','6','72']
  H_HDV['99_FC_2_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002106173583505','0.374025486833636','-24.1851299501231','1354.55224727818','0','6','72']
  H_HDV['99_CO_4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.6040657561078','-0.00936574961162','-0.887822235521406','0.10194670327703','0','6','71']
  H_HDV['99_THC_4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.280430325500598','3.1103676796138','5.29401783881731','2.13502465713726','-0.011800630623964','6','71']
  H_HDV['99_NOx_4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.4568407897304','0.055966780962552','26.5660761990873','0.187663258277437','18.1182246690039','6','71']
  H_HDV['99_PM_4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.2034703108203','0.071137088518145','-0.000528711809727','0','0','6','71']
  H_HDV['99_FC_4_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','747.562236905512','0.094203027548897','10686.4311375805','0.835191750503683','498.716901200507','6','71']
  H_HDV['99_CO_4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.56414668403669','0.020072282146709','13.4392918246063','0.246579516386241','-0.29679660715456','6','63']
  H_HDV['99_THC_4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.581582703210429','2.68332503839246','6.35026276051513','2.59040474279719','-0.018450803873803','6','63']
  H_HDV['99_NOx_4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','50','NOx','y=((a*(b**x))*(x**c))','80.6007451145853','1.00235910568081','-0.280102234006861','0','0','6','63']
  H_HDV['99_PM_4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.848260433362175','0.012722208678302','2.23627251909974','0.397037679312223','-0.040171703866598','6','63']
  H_HDV['99_FC_4_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.005795736464543','0.789706201348819','-36.8389977391096','1501.004692073','0','6','63']
  H_HDV['99_CO_4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.8519665636638','0.031458361785736','44.972894353545','0.426985324459007','-0.072736476109791','6','46']
  H_HDV['99_THC_4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00005852431117','0.006381739607575','-0.241026055186491','4.17607223128831','0','6','46']
  H_HDV['99_NOx_4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000467133804164','0.048399176032213','-1.89489078053706','70.8417734711113','0','6','46']
  H_HDV['99_PM_4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.04352953224657','0.009025827556826','7.91464131430271','0.551789714032923','-0.876513960509711','6','46']
  H_HDV['99_FC_4_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.012014671669879','1.06348971763676','-37.7265883077942','1854.1793400158','0','6','46']
  H_HDV['99_CO_6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','0','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000074001164144','0.009786656053062','-0.450949270977547','10.5042204674123','0','6','71']
  H_HDV['99_THC_6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.54632377245915','0.002055433205165','-0.335656977456443','0.081633864827173','0','6','71']
  H_HDV['99_NOx_6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','59.6618630717265','-0.062556282848892','-3.94651296823305','0.120245989060543','0','6','71']
  H_HDV['99_PM_6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000003836617703','0.00053049482448','-0.026476620413151','0.84544682740126','0','6','71']
  H_HDV['99_FC_6_0'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','2353.57393406079','1.00919990765664','-0.440676738384123','0','0','6','71']
  H_HDV['99_CO_6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','50','CO','y=((a*(b**x))*(x**c))','25.8435241710114','0.98674464818215','-0.382844678964773','0','0','6','47']
  H_HDV['99_THC_6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.761012392879344','2.49402190758679','7.10856588249458','2.90852157540684','-0.028712458903803','6','47']
  H_HDV['99_NOx_6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000458157920983','0.052720168981674','-2.12651671575177','71.6964534118089','0','6','47']
  H_HDV['99_PM_6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.788480191033077','0.029343475096783','2.24196057805081','0.418990899158172','0.310541119794548','6','47']
  H_HDV['99_FC_6_50'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','435.805824131308','0.024983572251247','5067.2153221103','0.534656690006897','1070.13124974778','6','47']
  H_HDV['99_CO_6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','100','CO','y=((a*(x**b))+(c*(x**d)))','-0.204409288300385','0.729396887955555','42.017626540383','-0.507688209660937','0','6','33']
  H_HDV['99_THC_6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000092594577077','0.008474886400501','-0.275497347569588','4.481784136773','0','6','33']
  H_HDV['99_NOx_6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001970764710076','0.145926439933412','-3.84066440481853','97.3323227490573','0','6','33']
  H_HDV['99_PM_6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.97031736796529','-0.01741636828865','-0.424069601191277','0.243866493693733','0','6','33']
  H_HDV['99_FC_6_100'] = [' 99','EURO2','TT/AT >40-50t Euro-2','6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.036409996104293','2.44435748879773','-62.4208065362961','2435.31918400116','0','6','33']
  H_HDV['100_CO_0_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.13613672138861','0.055828638047362','10.5114380676755','0.206854541236','1.22066935823371','6','86']
  H_HDV['100_THC_0_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.13038870785936','3.27006748817458','3.77005333482952','1.67082756174505','-0.006791184001115','6','86']
  H_HDV['100_NOx_0_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.5804480419599','0.050184354538447','51.6545527348773','0.28004843561629','5.39786957643357','6','86']
  H_HDV['100_PM_0_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.06114268186184','1.06951868143241','4.56332903797046','1.90825974442228','-0.011410082474197','6','86']
  H_HDV['100_FC_0_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','565.942233110232','0.054239329735692','1958.58559755773','0.432517452382969','183.065332335399','6','86']
  H_HDV['100_CO_0_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.65213088446902','68.0745277389686','-0.852171318398502','0.479669354262482','0.043543167408496','6','86']
  H_HDV['100_THC_0_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.727257381153142','0.024751623605376','3.01848063756747','0.116520467624114','0.168223954988244','6','86']
  H_HDV['100_NOx_0_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','50','NOx','y=((a+(b*x))**((-1)/c))','-0.000225121361104','0.000215453768074','2.06604096827249','0','0','6','86']
  H_HDV['100_PM_0_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.615942143088943','0.048470418937089','0.876920529522125','0.200965640474254','0.128119775775479','6','86']
  H_HDV['100_FC_0_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','591.112817402911','0.036452988005448','16259.4312372549','0.814926484448728','226.9059632895','6','86']
  H_HDV['100_CO_0_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.1024548216228','0.033575941657716','-0.692573313106034','0.049624104223641','0','6','86']
  H_HDV['100_THC_0_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.75049879493439','0.025662499640482','2.82121569052396','0.119253221915271','0.183829778280699','6','86']
  H_HDV['100_NOx_0_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.001575454885897','1.61876707061255','60.6091984979601','-0.408924179139667','0','6','86']
  H_HDV['100_PM_0_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.679926327738634','0.04742078349542','0.992925593661301','0.264669361062212','0.154917661343232','6','86']
  H_HDV['100_FC_0_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','0','100','FC','y=(1/(a+(b*x)))','0.000993067874626','0.000027341611113','0','0','0','6','86']
  H_HDV['100_CO_-2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.2898189186985','0.015512963475106','-0.752751177034396','0.061657217322547','0','6','86']
  H_HDV['100_THC_-2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.073663948537989','2.33876260677313','6.03048207826993','2.34611756301329','-0.017898912930996','6','86']
  H_HDV['100_NOx_-2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.0882024466262','0.055180796732006','111.832563943766','0.452636574991963','1.70070673545558','6','86']
  H_HDV['100_PM_-2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.106104303202772','23.3227304081767','-3.4625622203263','-0.374547482452495','0.100342422069371','6','86']
  H_HDV['100_FC_-2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','575.523138112403','0.052636061998713','4399.01258287345','0.582562827450841','58.9507268033136','6','86']
  H_HDV['100_CO_-2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','50','CO','y=(c+(a*exp(b*x)))','11.7250208725895','-0.067899746937844','0.833208325895706','0','0','6','86']
  H_HDV['100_THC_-2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.03453951024656','-0.002663479792747','-0.232027037513376','0.08506675493035','0','6','86']
  H_HDV['100_NOx_-2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.4239039268653','0.034033945587274','218.481591739267','0.542823697532519','0.577701995446932','6','86']
  H_HDV['100_PM_-2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040434659790502','0.856150691662121','6.40914762126001','2.39957474279917','-0.015344834514861','6','86']
  H_HDV['100_FC_-2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','50','FC','y=((a*(b**x))*(x**c))','1185.14032974123','0.980798794383364','-0.336132758576075','0','0','6','86']
  H_HDV['100_CO_-2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.95870676004133','0.052992429928554','26.6903333131423','0.518921294479219','0.695627507436317','6','86']
  H_HDV['100_THC_-2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.86682033487739','-0.003043784575371','-0.22620180271199','0.088757916073141','0','6','86']
  H_HDV['100_NOx_-2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.3580069961866','0.026030291461603','610.270955008717','0.717017348580789','-0.346156068482004','6','86']
  H_HDV['100_PM_-2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.1344482427325','-6.58035040203871','-1.27229114110038','0','0','6','86']
  H_HDV['100_FC_-2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','751.892414360075','-2.62217046060459','-34.6704519107351','0.065325532364575','0','6','86']
  H_HDV['100_CO_-4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.12924922396871','-10.9658977775887','-1.84152366709809','0','0','6','86']
  H_HDV['100_THC_-4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.9825462252824','-0.002120991458634','-0.239084179517189','0.087152318133559','0','6','86']
  H_HDV['100_NOx_-4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.0392530033717','0.061092865594078','187.644487831175','0.537141621419836','0.544629517356416','6','86']
  H_HDV['100_PM_-4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033815970254009','0.86785460351836','6.92306640786803','2.61349676741503','-0.017105924746648','6','86']
  H_HDV['100_FC_-4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','465.582378764786','0.052480787393179','4681.50986981877','0.560851210946169','15.8683639150629','6','86']
  H_HDV['100_CO_-4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.148818607327','-0.009848705355264','-0.716368122833559','0.077692952795204','0','6','86']
  H_HDV['100_THC_-4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.067733430577255','20.0483030481686','-1.36315371784789','0.497772022804877','0.039652816208251','6','86']
  H_HDV['100_NOx_-4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5799387232531','0.048860048936204','588.492603848232','0.725439930205867','0.369293705013466','6','86']
  H_HDV['100_PM_-4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.24324439022359','-6.90967086833381','-1.36462961291965','0','0','6','86']
  H_HDV['100_FC_-4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','390.636466470937','0.041435443363915','15076.6646764878','0.753257566576506','6.61760357933447','6','86']
  H_HDV['100_CO_-4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.75794646661498','0.045980138793748','112.866033583381','0.709810201829319','0.130926816560694','6','86']
  H_HDV['100_THC_-4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.52461679989581','0.060123623663708','11.0466786670643','0.578600912097966','0.080480656625153','6','86']
  H_HDV['100_NOx_-4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.50481009235767','0.040279600482803','1212.21637636142','0.847398606840761','0.184257016674891','6','86']
  H_HDV['100_PM_-4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','100','PM','y=(1/(a+(b*(x**c))))','1.06953594283748','0.035764005118421','1.38171361683797','0','0','6','86']
  H_HDV['100_FC_-4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','381.900515411908','0.034615845380652','45033.2454154012','0.936057280426468','-3.50325144880045','6','86']
  H_HDV['100_CO_-6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','0','CO','y=(c+(a*exp(b*x)))','9.77685883191123','-0.075716445862329','0.259814590827424','0','0','6','72']
  H_HDV['100_THC_-6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.99449087122486','0.07140857238822','2.64657381718045','0.399400314871878','0.086772652374057','6','72']
  H_HDV['100_NOx_-6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6455500667093','0.072825604129148','309.921049344098','0.630851034555029','0.394705319740659','6','72']
  H_HDV['100_PM_-6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029020738279714','0.778569993935486','7.03038452465826','2.68031500916682','-0.021445235048063','6','72']
  H_HDV['100_FC_-6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','441.138068037567','0.064487660158918','10462.5095409515','0.726332609976078','13.3591469379364','6','72']
  H_HDV['100_CO_-6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','50','CO','y=(c+(a*exp(((-1)*b)*x)))','7.50760923089082','0.067472841233805','0.225502272544725','0','0','6','72']
  H_HDV['100_THC_-6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.43636144993006','0.064955955782956','12.4950454598589','0.601609323569471','0.07899323131999','6','72']
  H_HDV['100_NOx_-6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.34655212925251','0.060523524098945','552.679625218442','0.719999637798255','0.337740813054402','6','72']
  H_HDV['100_PM_-6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.026059592571975','0.715905250299716','5.18983906978971','1.98207838753052','-0.006645558361209','6','72']
  H_HDV['100_FC_-6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','341.925189034716','0.056234505154942','19984.8321925642','0.817242038271508','11.3051340070113','6','72']
  H_HDV['100_CO_-6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','100','CO','y=(c+(a*exp(b*x)))','6.84224153733362','-0.063439507659334','0.216143424946564','0','0','6','72']
  H_HDV['100_THC_-6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.23669698461088','0.062544938678944','12.3743041462339','0.590018402451421','0.076887391531374','6','72']
  H_HDV['100_NOx_-6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.95770873788252','0.053518699769826','602.988995329197','0.738033047614918','0.3110668591173','6','72']
  H_HDV['100_PM_-6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.619111093722482','0.055137076166237','0.406097985098022','0.214960902741643','0.056144355179878','6','72']
  H_HDV['100_FC_-6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','310.533902805378','0.052025317936728','26797.6950694817','0.859194400132175','10.429074201749','6','72']
  H_HDV['100_CO_2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-187.314072464806','0.870624259253829','13.9817677060764','0.087593223825277','1.78327143722769','6','86']
  H_HDV['100_THC_2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.16443719688107','2.96084604345879','4.79175490629402','1.93616108213505','-0.008679388442363','6','86']
  H_HDV['100_NOx_2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','49.8414389368196','-0.027311370088102','-4.5196646651499','0.11864131825693','0','6','86']
  H_HDV['100_PM_2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.124926438224515','0.77928977105927','6.79894651769976','2.60258975016248','-0.018467157744634','6','86']
  H_HDV['100_FC_2_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1203.83350062478','0.402283108907011','-71.4446020516097','0.081303956144938','0','6','86']
  H_HDV['100_CO_2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.76897989715801','0.052376242628872','7.15304167266933','0.191941283330394','1.94643865865084','6','86']
  H_HDV['100_THC_2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.317219493935669','2.47543047359388','6.32253430643427','2.51118497200419','-0.016040341907026','6','86']
  H_HDV['100_NOx_2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','10.9212311004621','148.159982615359','-0.072147720607672','0.82049700039616','0.001149094073579','6','86']
  H_HDV['100_PM_2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','50','PM','y=(c+(a*exp(b*x)))','0.938652614776414','-0.071993682971093','0.220420494880392','0','0','6','86']
  H_HDV['100_FC_2_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','50','FC','y=(c+(a*exp(b*x)))','740.915092706846','-0.070892876825982','575.391789039943','0','0','6','86']
  H_HDV['100_CO_2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','100','CO','y=((a*(b**x))*(x**c))','30.7126397484813','0.988504495150081','-0.460043640414049','0','0','6','74']
  H_HDV['100_THC_2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.419252965320888','2.25376721118098','6.98263237060983','2.78103954399138','-0.02243637456403','6','74']
  H_HDV['100_NOx_2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','4.0800819983418','0.931799645204643','-0.268284722793118','0','0','6','74']
  H_HDV['100_PM_2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.697743658270578','0.070483257422447','-0.000421957272156','0','0','6','74']
  H_HDV['100_FC_2_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002134821104922','0.390404327020415','-25.7375868145131','1394.48599049113','0','6','74']
  H_HDV['100_CO_4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.4060586412385','-0.016764897152628','-1.11216698139166','0.096722449817711','0','6','71']
  H_HDV['100_THC_4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.240586654241527','2.75478194414286','5.6854090730591','2.26054016564695','-0.013665444714011','6','71']
  H_HDV['100_NOx_4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.72178990696319','0.023630036173591','43.4082754264467','0.159293484720479','12.3211981048143','6','71']
  H_HDV['100_PM_4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.149464313209909','0.697157245828653','6.60194092425157','2.46529254625687','-0.017439501127852','6','71']
  H_HDV['100_FC_4_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','0','FC','y=((a*(b**x))*(x**c))','2417.04452205268','1.00849223927375','-0.504130001475675','0','0','6','71']
  H_HDV['100_CO_4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','50','CO','y=((a*(b**x))*(x**c))','24.9606984868531','0.987054206051303','-0.40132402246697','0','0','6','65']
  H_HDV['100_THC_4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.41064558309799','2.52399280303375','5.88846514655656','2.38839698059948','-0.015836151449369','6','65']
  H_HDV['100_NOx_4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','20.4398279411733','0.062163642595729','32.7658739277796','0.20742636633767','22.6736512537242','6','65']
  H_HDV['100_PM_4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000005607808823','0.000830801246999','-0.042540828756352','1.08236159785554','0','6','65']
  H_HDV['100_FC_4_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','50','FC','y=exp((a+(b/x))+(c*log(x)))','7.19512304511165','1.29169294224091','-0.106946352184041','0','0','6','65']
  H_HDV['100_CO_4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.3382384728228','0.036428176809221','27.3296136015631','0.359691410762789','0.369775830202988','6','48']
  H_HDV['100_THC_4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.552725027406944','2.2406154812667','6.72428220625175','2.70218856003096','-0.024245700898835','6','48']
  H_HDV['100_NOx_4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000720135811067','0.076584200772629','-2.82668013411391','70.3438751443223','0','6','48']
  H_HDV['100_PM_4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.43759972936429','-0.006246103988869','-0.099776778366358','0.128222740393427','0','6','48']
  H_HDV['100_FC_4_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2228.10799585886','0.004555383426297','25257.2721244051','0.836835724542374','-585.213218529768','6','48']
  H_HDV['100_CO_6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.7523526901987','-0.038845255007721','-1.47445789914053','0.128213197243048','0','6','71']
  H_HDV['100_THC_6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.299396760933143','2.51628213530301','7.26701500618017','2.89275393901798','-0.032582188911618','6','71']
  H_HDV['100_NOx_6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','66.1827814977927','-0.062155561291758','-6.36180204711973','0.141675564186788','0','6','71']
  H_HDV['100_PM_6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-7.92442944438489','0.719117797074971','1.02776291671621','0.080572971196846','0.244058586430319','6','71']
  H_HDV['100_FC_6_0'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','2381.75031610108','1.0080735134869','-0.425252102566825','0','0','6','71']
  H_HDV['100_CO_6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','50','CO','y=((a*(b**x))*(x**c))','26.5998385846883','0.983255345100146','-0.369563821098686','0','0','6','48']
  H_HDV['100_THC_6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.608777894485201','2.14598065305546','7.82736586580324','3.16669313867815','-0.033325956128196','6','48']
  H_HDV['100_NOx_6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','85.6232854454402','-0.108969443396879','-7.45952747342636','0.149069654180123','0','6','48']
  H_HDV['100_PM_6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.400352262373182','0.620988640451077','6.60273502277366','2.65924783424918','-0.025179938412808','6','48']
  H_HDV['100_FC_6_50'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','454.180973254277','0.042766128556273','5212.88354484015','0.563067319066565','1153.25247475988','6','48']
  H_HDV['100_CO_6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','100','CO','y=((a*(b**x))*(x**c))','31.4971822595326','0.969414151870189','-0.287342342836257','0','0','6','34']
  H_HDV['100_THC_6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.0132129763394','7.90678205917351','-1.71400800239297','-0.884576684304963','0.206482306664129','6','34']
  H_HDV['100_NOx_6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00209499687543','0.164071828290353','-4.5672093341749','92.4801854699251','0','6','34']
  H_HDV['100_PM_6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000025128863446','0.002161039362394','-0.069217393611397','1.48578672902371','0','6','34']
  H_HDV['100_FC_6_100'] = [' 100','EURO3','TT/AT >40-50t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1981.88982108486','0.00655984509734','26062.8682986001','0.852857896926552','158.440122644871','6','34']
  H_HDV['101_CO_0_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.052536136090604','0.893069379886958','3.81982630978952','1.68016598535348','-0.006088100717603','6','86']
  H_HDV['101_THC_0_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004149213673911','0.183879148749526','3.83060085959577','1.70941078293581','-0.010065679302271','6','86']
  H_HDV['101_NOx_0_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.98927207173562','0.050268419856948','40.6156580342613','0.380496637105114','3.24935844506513','6','86']
  H_HDV['101_PM_0_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.106894342345525','0.046126435338549','0.269854400036773','0.169087850690515','0.018809602412722','6','86']
  H_HDV['101_FC_0_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','541.20685628854','0.055235258786396','2947.22362599275','0.500523308391245','171.758632668488','6','86']
  H_HDV['101_CO_0_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.395388508088419','0.044140505055978','0.646093444724652','0.150134768783098','0.097146799529415','6','86']
  H_HDV['101_THC_0_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004511735272893','0.177681516627482','3.8232617110622','1.7273742356269','-0.010187686071334','6','86']
  H_HDV['101_NOx_0_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.20040090336792','0.034321223259597','100.5180827441','0.565579481487381','3.96750661460295','6','86']
  H_HDV['101_PM_0_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115785942100746','0.046958315786394','0.254143100211404','0.179943889420719','0.020317248127156','6','86']
  H_HDV['101_FC_0_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','549.54229679785','0.036061596684322','13534.9461170675','0.776049874846255','208.951710691903','6','86']
  H_HDV['101_CO_0_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.441079283747155','0.044065344993159','0.599970576021612','0.158394593255192','0.099948083761015','6','86']
  H_HDV['101_THC_0_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00218267513894','0.177005835143532','3.73518761276105','1.71001412323114','-0.014032145836725','6','86']
  H_HDV['101_NOx_0_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6067117824585','0.023469615779611','694.980109586391','0.919968182673876','3.55151501164442','6','86']
  H_HDV['101_PM_0_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.12839489654921','0.049338676657615','0.252974231369053','0.198682572641912','0.021374665127842','6','86']
  H_HDV['101_FC_0_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','0','100','FC','y=(1/(a+(b*x)))','0.001062565090215','0.000029730564383','0','0','0','6','86']
  H_HDV['101_CO_-2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.026308747281767','0.602853708938433','7.45101013884652','2.8142138138443','-0.025484404189761','6','86']
  H_HDV['101_THC_-2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000187915552505','0.133714132863587','6.17196069041948','2.42380597571098','-0.025598454642385','6','86']
  H_HDV['101_NOx_-2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.4209502129186','0.049171419297991','94.6040958264726','0.532370273148747','0.875845079507189','6','86']
  H_HDV['101_PM_-2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009288596274618','0.246680068674555','3.93187863491406','1.6813044478679','0.007426928524341','6','86']
  H_HDV['101_FC_-2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','542.424806643031','0.05350675628125','6211.67952301712','0.634098550971578','52.8261701605169','6','86']
  H_HDV['101_CO_-2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.61731995372098','-8.80415212427839','-1.55456011495412','0','0','6','86']
  H_HDV['101_THC_-2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00043033256474','0.134983990212641','4.7406408753884','1.90525379073515','-0.008067375734231','6','86']
  H_HDV['101_NOx_-2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.17374470858113','0.02871167584677','219.206739353498','0.658152192044461','0.014847539533238','6','86']
  H_HDV['101_PM_-2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.20046096333415','0.526562941607964','0.010272648574103','0','0','6','86']
  H_HDV['101_FC_-2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-5.4656487023487','0.66729448217916','1435.76745914766','-0.524121152458757','0','6','86']
  H_HDV['101_CO_-2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.783246722173576','-0.001036019662587','-0.05575191535986','0.081455340136401','0','6','86']
  H_HDV['101_THC_-2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.029858965909401','0.017890218271022','0.139233643876961','0.098615575148109','-0.001392484184272','6','86']
  H_HDV['101_NOx_-2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.49328625639662','0.021397301453518','705.185518556229','0.860080785633598','-0.879746467353655','6','86']
  H_HDV['101_PM_-2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001669284176039','1.89163160175543','-0.859418200572346','0.758666572145709','0.018443482965354','6','86']
  H_HDV['101_FC_-2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','699.460454544182','-2.47294438052091','-33.6639732909118','0.067817331795631','0','6','86']
  H_HDV['101_CO_-4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.65248412761146','0.007783394050201','0.008291827440673','0','0','6','86']
  H_HDV['101_THC_-4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.167496881155109','-0.000117767323022','-0.013426310521148','0.086682578911223','0','6','86']
  H_HDV['101_NOx_-4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.90410749070225','0.051814573045072','130.038763806477','0.578453560744503','0.130326027138911','6','86']
  H_HDV['101_PM_-4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.67810046354177','0.419912221331932','0.020660350947425','0','0','6','86']
  H_HDV['101_FC_-4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','439.570463222077','0.054151001938316','7065.48210648125','0.631773926087784','9.63545701811352','6','86']
  H_HDV['101_CO_-4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.626501145553405','-0.000478217847194','-0.045620284838819','0.078508867842009','0','6','86']
  H_HDV['101_THC_-4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.098980661402563','0.061894313691233','0.272004315058285','0.439376486723101','0.003999199023368','6','86']
  H_HDV['101_NOx_-4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.88237377740637','0.040962944340541','297.618906507389','0.715907024274394','-0.017496136552969','6','86']
  H_HDV['101_PM_-4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.136447013919739','0.059688806003339','0.675026219086104','0.419309624834367','0.004797588646869','6','86']
  H_HDV['101_FC_-4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','356.111812465182','0.042533841377377','23668.6808362396','0.825250325426215','1.08347640277683','6','86']
  H_HDV['101_CO_-4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.404557914142262','0.053691734831176','9.56566990820942','0.786289190675675','0.009862410330208','6','86']
  H_HDV['101_THC_-4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086581248690912','0.059405409883787','0.67565314372494','0.58392648979783','0.003693462288073','6','86']
  H_HDV['101_NOx_-4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.81929022756463','0.034620314597763','596.34139546574','0.837723151992527','-0.164726292269744','6','86']
  H_HDV['101_PM_-4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.125120943055261','0.057376666501115','1.18137294422465','0.515372282088556','0.004438003420898','6','86']
  H_HDV['101_FC_-4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','337.137050100667','0.034955966526226','61246.9318450529','0.986240252797474','-8.13356871101922','6','86']
  H_HDV['101_CO_-6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','0','CO','y=(c+(a*exp(b*x)))','0.626121662350094','-0.075067939815858','0.012073736367664','0','0','6','72']
  H_HDV['101_THC_-6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.116638681568157','0.071487041235741','0.22597301236899','0.485651634745272','0.003978433802542','6','72']
  H_HDV['101_NOx_-6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.53572325804472','0.065495295577989','207.789703810323','0.680167515459921','0.056319457191776','6','72']
  H_HDV['101_PM_-6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.165060508547977','0.071422859236359','0.58562736874042','0.437538232817752','0.004856511981251','6','72']
  H_HDV['101_FC_-6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','398.592754798112','0.06520307944832','15628.7541292705','0.785697551227859','4.59030870799354','6','72']
  H_HDV['101_CO_-6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005983264304651','2.05636028733371','-0.751006607897343','0.386615655597853','0.047850447760825','6','72']
  H_HDV['101_THC_-6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08461254648511','0.064749943610713','0.966197341355056','0.663252040104583','0.00351285149091','6','72']
  H_HDV['101_NOx_-6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.10468333534504','0.057910393316262','329.507935129578','0.752532051426721','0.03250755244729','6','72']
  H_HDV['101_PM_-6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.12895136744046','0.066058445639013','1.4028497631566','0.563916999236184','0.004357441893293','6','72']
  H_HDV['101_FC_-6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','291.827659087681','0.056069714529986','28550.9359144219','0.870525695681337','2.63958482471295','6','72']
  H_HDV['101_CO_-6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.316126077587424','0.057382652636991','16.6280702415798','0.895943840301405','0.006568563953211','6','72']
  H_HDV['101_THC_-6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.073587753227121','0.062358029677401','0.979966991906286','0.651134155281005','0.003394826721919','6','72']
  H_HDV['101_NOx_-6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.70241287861244','0.054347333709147','406.906678889157','0.788831755395504','0.026231590093495','6','72']
  H_HDV['101_PM_-6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115284364457424','0.063562443848584','1.55370161853929','0.575452087127063','0.004169920278245','6','72']
  H_HDV['101_FC_-6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','261.403252789135','0.052035634979622','36242.0400246807','0.908475626162051','2.06350762600502','6','72']
  H_HDV['101_CO_2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.131124697452888','0.635536568932489','7.04316885253816','2.76329607287791','-0.017215690095831','6','86']
  H_HDV['101_THC_2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006287199420038','0.165098038811276','4.82505919859961','1.96122913643098','-0.009244552328431','6','86']
  H_HDV['101_NOx_2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.3746243562548','0.09716174918287','164.311571748885','0.848447952574714','6.45339255486767','6','86']
  H_HDV['101_PM_2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.065339416549104','0.028825155485481','0.281073046113511','0.132449971453697','0.021049599062882','6','86']
  H_HDV['101_FC_2_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','786.754441286754','0.08520340037385','89449.0885706716','1.24952977745047','324.279678442793','6','86']
  H_HDV['101_CO_2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.24366370490514','-0.002474369721482','-0.114733425188059','0.122981669878728','0','6','86']
  H_HDV['101_THC_2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011895255443395','0.142034124523611','6.14071864852094','2.46367146330204','-0.016103149160533','6','86']
  H_HDV['101_NOx_2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','50','NOx','y=exp((a+(b/x))+(c*log(x)))','3.16205149991162','1.26752569098936','-0.216967270072776','0','0','6','86']
  H_HDV['101_PM_2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086447737592111','0.026403217204186','0.281374826336791','0.146719867647257','0.018025213024226','6','86']
  H_HDV['101_FC_2_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','50','FC','y=(c+(a*exp(b*x)))','716.980966861311','-0.072069088221927','538.28450371643','0','0','6','86']
  H_HDV['101_CO_2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.466383056013009','0.019358709547999','0.807451714982556','0.132231989866659','0.028813754596415','6','75']
  H_HDV['101_THC_2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015395022823753','0.134704948994007','6.38113761316077','2.58375573849094','-0.018825317033527','6','75']
  H_HDV['101_NOx_2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6748776374189','0.025070281544115','85.9223070549491','0.529720750578126','10.2042809515686','6','75']
  H_HDV['101_PM_2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.147351163449264','0.051488697120746','0.226921515889847','0.178086637894149','0.030811077345146','6','75']
  H_HDV['101_FC_2_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','2','100','FC','y=(c+(a*exp(b*x)))','628.225343250286','-0.049764066819094','715.713742689403','0','0','6','75']
  H_HDV['101_CO_4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.19263261927137','0.571907556338939','7.63492427950313','3.03642376192074','-0.023739595637461','6','71']
  H_HDV['101_THC_4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007590887617463','0.158151957091326','5.36443033671742','2.1521729943609','-0.012217350777444','6','71']
  H_HDV['101_NOx_4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','9.05443433834232','23.2898266577573','2.46609004909856','1.50162637349013','0.012247506761913','6','71']
  H_HDV['101_PM_4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.074290033349708','0.025024567642687','0.273005442073','0.137234191938888','0.021359337953815','6','71']
  H_HDV['101_FC_4_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','778.792957508705','0.09236312746783','12656.1691697649','0.883644489970113','475.186962613315','6','71']
  H_HDV['101_CO_4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.574930049144364','0.022887315315077','0.753451665607445','0.157148861651827','0.038561961126026','6','66']
  H_HDV['101_THC_4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01522123813863','0.141629898378596','5.95254994703648','2.43173471996985','-0.016413765877772','6','66']
  H_HDV['101_NOx_4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.1024979826105','0.043917963562604','37.3744394322183','0.377042656666255','13.9663503778508','6','66']
  H_HDV['101_PM_4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.115511285092418','0.025814212675691','0.279508119441821','0.151370728576963','0.011580043877989','6','66']
  H_HDV['101_FC_4_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','50','FC','y=exp((a+(b/x))+(c*log(x)))','7.03882138710714','1.68367347035874','-0.083873196130637','0','0','6','66']
  H_HDV['101_CO_4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.55817098939869','-0.003027689679043','-0.124052780273682','0.10172498023618','0','6','49']
  H_HDV['101_THC_4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.020584999957902','0.128937459040541','6.59864173576455','2.69818453132606','-0.023820027533504','6','49']
  H_HDV['101_NOx_4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000290224131656','0.030580265280331','-1.16664981035397','37.6589816840464','0','6','49']
  H_HDV['101_PM_4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014912293443132','0.340810463948134','3.50770770140385','1.59240053730886','-0.007695894600733','6','49']
  H_HDV['101_FC_4_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1815.78093306755','0.004820449901014','9869.08682815497','0.672226895630227','-293.470930758366','6','49']
  H_HDV['101_CO_6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.67141042455676','-0.004051022525237','-0.263848136579775','0.215759355349933','0','6','71']
  H_HDV['101_THC_6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007579723722172','0.144530841897453','7.00687675206074','2.80696500816943','-0.034141155469644','6','71']
  H_HDV['101_NOx_6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','41.7420621010038','-0.046205014805479','-5.85798299716035','0.213769541601612','0','6','71']
  H_HDV['101_PM_6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.398497776132084','-0.000775019259967','-0.045737919519','0.144158937680938','0','6','71']
  H_HDV['101_FC_6_0'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','2341.48577243408','1.00865252302331','-0.444920383595841','0','0','6','71']
  H_HDV['101_CO_6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.33033278609495','-3.92721524319127','-0.958566362884685','0','0','6','49']
  H_HDV['101_THC_6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.02247021687446','0.124649222577408','7.21964014828439','2.95010089385138','-0.027975301856475','6','49']
  H_HDV['101_NOx_6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6844194272093','0.02638532564311','32.0042765360226','0.284503122687914','16.6239724965015','6','49']
  H_HDV['101_PM_6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.038290610585569','0.339617330303426','2.85111399255236','1.26790043092042','0.027053492841407','6','49']
  H_HDV['101_FC_6_50'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','458.431229877972','0.057014842214642','5545.55131555258','0.603711032165858','1118.73849734344','6','49']
  H_HDV['101_CO_6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000030798895105','0.003038861541552','-0.1091600780303','1.73787923201365','0','6','35']
  H_HDV['101_THC_6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.039825660173583','0.433114084506902','-1.31715070154573','-0.646950293450001','0.188089831147729','6','35']
  H_HDV['101_NOx_6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000889452787189','0.069550983952981','-1.99791541889131','50.6952951393167','0','6','35']
  H_HDV['101_PM_6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.049680024307512','0.253050967540455','6.62296196312038','2.82638740334624','-0.028561487765974','6','35']
  H_HDV['101_FC_6_100'] = [' 101','EURO4','TT/AT >40-50t Euro-4','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1595.52989570029','0.006836758595825','10448.9925164268','0.686504441324368','391.479492291591','6','35']
  H_HDV['102_CO_0_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.046928392484028','0.911642144718416','3.8802560166844','1.70329443764044','-0.008365557357877','6','86']
  H_HDV['102_THC_0_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004095367811695','0.188031984118434','3.82120182966642','1.70627628299572','-0.010188201379107','6','86']
  H_HDV['102_NOx_0_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.91275107482118','0.049951602095965','27.7450153040587','0.367201658704291','1.91575885728242','6','86']
  H_HDV['102_PM_0_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.109331073644324','0.04612695782202','0.275692452749339','0.169399482308317','0.019030796032465','6','86']
  H_HDV['102_FC_0_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','557.653941509246','0.05505999992286','2927.11421570516','0.491030501648275','174.298465804993','6','86']
  H_HDV['102_CO_0_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.077052679524498','0.922808165179916','3.16511517328859','1.41769631667816','0.003569475238475','6','86']
  H_HDV['102_THC_0_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004358807097298','0.181522681104033','3.8286515400201','1.72970206504708','-0.010600880108699','6','86']
  H_HDV['102_NOx_0_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.77700676620421','0.036643721142995','55.9539061619014','0.517152313044771','2.35866066448137','6','86']
  H_HDV['102_PM_0_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.117836738357748','0.046852176988045','0.260039597187162','0.179988250435282','0.02056528269313','6','86']
  H_HDV['102_FC_0_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','566.156370054271','0.036277756668697','13365.4686585935','0.762790769931312','212.022823602101','6','86']
  H_HDV['102_CO_0_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.431645964954818','0.042775278836261','0.635911548508985','0.156393018529406','0.100228118509634','6','86']
  H_HDV['102_THC_0_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000996298383514','0.185017065407257','3.63489001681497','1.67678235151352','-0.014330565447841','6','86']
  H_HDV['102_NOx_0_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.9941391399255','0.027447096229704','180.182109140714','0.729601876050781','2.28037002445515','6','86']
  H_HDV['102_PM_0_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.130262546510347','0.049090529406561','0.259385746811248','0.198539197685207','0.02160958682519','6','86']
  H_HDV['102_FC_0_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','0','100','FC','y=(1/(a+(b*x)))','0.001032985756261','0.000029548593068','0','0','0','6','86']
  H_HDV['102_CO_-2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','0','CO','y=(c+(a*exp(b*x)))','0.851706131073386','-0.072152646677588','0.059218863488886','0','0','6','86']
  H_HDV['102_THC_-2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000379435650813','0.136192340456547','6.19139287706112','2.42892144631299','-0.025420221397043','6','86']
  H_HDV['102_NOx_-2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.18490393916172','0.049378374063035','57.824347239805','0.499824150522686','0.515889473787683','6','86']
  H_HDV['102_PM_-2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009209143935497','0.249601969339198','4.01069853002302','1.71070116307473','0.006060329669618','6','86']
  H_HDV['102_FC_-2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','560.668389581417','0.053589070097931','6063.71335715214','0.623671752923724','53.8769868826551','6','86']
  H_HDV['102_CO_-2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','50','CO','y=exp((a+(b/x))+(c*log(x)))','3.65858374476647','-8.86403504437225','-1.56046114099107','0','0','6','86']
  H_HDV['102_THC_-2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00045612103193','0.137794356913724','4.7500464859162','1.90732428664834','-0.00802199274625','6','86']
  H_HDV['102_NOx_-2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.01358420796631','0.029996838128613','120.556331311761','0.612063632878579','0.0413858577479','6','86']
  H_HDV['102_PM_-2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.16354066491414','0.513863723610851','0.010150500531071','0','0','6','86']
  H_HDV['102_FC_-2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-5.83476489597223','0.656686141237937','1498.80532430229','-0.529613154260324','0','6','86']
  H_HDV['102_CO_-2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.799550893985388','-0.001055638612883','-0.05693708881857','0.081443309772685','0','6','86']
  H_HDV['102_THC_-2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.030176782722577','0.017498144493185','0.142481920688681','0.098019771365819','-0.001583567502433','6','86']
  H_HDV['102_NOx_-2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.7029939067856','0.023667763099618','312.3889417707','0.77794037791001','-0.367272455681505','6','86']
  H_HDV['102_PM_-2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001698229008978','1.80006591869767','-0.77550564835818','0.762512781151252','0.018448939449683','6','86']
  H_HDV['102_FC_-2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','724.495072337639','-2.51980215295697','-35.4181533357073','0.068478530767136','0','6','86']
  H_HDV['102_CO_-4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.61902105370797','0.006486261570783','0.008136423727836','0','0','6','86']
  H_HDV['102_THC_-4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.171295658770401','-0.000120497533058','-0.013722313969476','0.086624871516185','0','6','86']
  H_HDV['102_NOx_-4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.34521799961493','0.052730063003513','82.7011288568214','0.556850827563292','0.08006391535547','6','86']
  H_HDV['102_PM_-4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.64243195815154','0.406860921801833','0.020321661003151','0','0','6','86']
  H_HDV['102_FC_-4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','456.143728545382','0.054361822538113','7129.82320121493','0.627407923098691','9.9477322187142','6','86']
  H_HDV['102_CO_-4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.644031543506218','-0.000487222945096','-0.047054994849297','0.078721361246646','0','6','86']
  H_HDV['102_THC_-4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.101336167571904','0.061932899979423','0.262380277648795','0.429099162466699','0.004081525325088','6','86']
  H_HDV['102_NOx_-4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.71751255680185','0.042600235372152','167.418121869689','0.674982831184953','0.000560312132206','6','86']
  H_HDV['102_PM_-4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.139480980267954','0.059747918875904','0.66795241544144','0.413510312197973','0.004891011097202','6','86']
  H_HDV['102_FC_-4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','369.656207502814','0.043000271622155','23434.5902822036','0.818428918179505','1.47779591336275','6','86']
  H_HDV['102_CO_-4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.415753951114577','0.05401754983863','9.34712426406963','0.78150089810335','0.010178041554927','6','86']
  H_HDV['102_THC_-4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.088813306943716','0.059542702215363','0.670872618406285','0.580279968275089','0.003777628580835','6','86']
  H_HDV['102_NOx_-4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.67310570425073','0.036813181750711','301.534307653278','0.778802020012705','-0.072586652496558','6','86']
  H_HDV['102_PM_-4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.128010377101485','0.057502564130366','1.17889558793401','0.511818519569803','0.004536701725405','6','86']
  H_HDV['102_FC_-4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','348.493474911127','0.035402321348503','60308.3980090547','0.978132784344286','-7.71562641998741','6','86']
  H_HDV['102_CO_-6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','0','CO','y=(c+(a*exp(b*x)))','0.643290227561664','-0.075140781656251','0.012311007131114','0','0','6','72']
  H_HDV['102_THC_-6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.120246467768909','0.071772046524341','0.210607758082181','0.476360385614156','0.004072165764059','6','72']
  H_HDV['102_NOx_-6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.16817654597069','0.066096576689724','123.544274658516','0.645636361931611','0.03360706736354','6','72']
  H_HDV['102_PM_-6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.169819972864703','0.071650865224923','0.572516510528016','0.432339392504321','0.004967870253882','6','72']
  H_HDV['102_FC_-6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','416.589301922104','0.065565825964535','15933.8651933555','0.78511898383114','4.79878656880347','6','72']
  H_HDV['102_CO_-6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006280082475522','2.04661262639577','-0.718229942670966','0.382910695111255','0.048500095879496','6','72']
  H_HDV['102_THC_-6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08707510815463','0.064970496142852','0.969730925224622','0.662333353088433','0.003592433307535','6','72']
  H_HDV['102_NOx_-6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.32165673929577','0.059336668263629','191.093489332026','0.716420122222968','0.021183550506731','6','72']
  H_HDV['102_PM_-6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.132461189141772','0.066254895583745','1.40944417600885','0.562171275426487','0.004455161703412','6','72']
  H_HDV['102_FC_-6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','303.826634982325','0.056378390269016','28970.2851587232','0.867794388503037','2.77495830854309','6','72']
  H_HDV['102_CO_-6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003759713788188','2.88426072246233','-1.16902116578962','0.44273075492782','0.03968709925264','6','72']
  H_HDV['102_THC_-6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.075548406520704','0.062485245557396','1.01215843225261','0.65447894872363','0.003462021351898','6','72']
  H_HDV['102_NOx_-6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.07080089724963','0.055972062993548','231.694078727851','0.750190678783374','0.017181992619848','6','72']
  H_HDV['102_PM_-6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.118296181700868','0.063749486814399','1.58357419913666','0.576231551274114','0.004263346949323','6','72']
  H_HDV['102_FC_-6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','271.234152754844','0.052271570581215','37171.6251194088','0.90765659644148','2.15889839227588','6','72']
  H_HDV['102_CO_2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.129964939276388','0.65523621412087','6.9259763712206','2.71986854129502','-0.016876211460438','6','86']
  H_HDV['102_THC_2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006357389319763','0.168811648436421','4.79967409776691','1.95212979758293','-0.009123312638791','6','86']
  H_HDV['102_NOx_2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.43058053069793','0.090561852461067','23.423259886631','0.451075609302397','3.81757654979412','6','86']
  H_HDV['102_PM_2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.067416759899196','0.029179411266057','0.286741182130388','0.13308839284993','0.021337702209163','6','86']
  H_HDV['102_FC_2_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','0','FC','y=(c+(a*exp(b*x)))','877.509147588535','-0.088485608718306','329.693120815879','0','0','6','86']
  H_HDV['102_CO_2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.27870524155882','-0.002488157261461','-0.120510154881607','0.125160372741511','0','6','86']
  H_HDV['102_THC_2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011959132209587','0.14495826886204','6.15174451026891','2.46589662026026','-0.01623377338801','6','86']
  H_HDV['102_NOx_2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.36303281769831','0.020021192162257','19.1209991906642','0.268327179007376','4.00014837041802','6','86']
  H_HDV['102_PM_2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.087780048232962','0.026565898948682','0.287983061600727','0.147509374218565','0.018614520735258','6','86']
  H_HDV['102_FC_2_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','50','FC','y=(c+(a*exp(b*x)))','746.560598086515','-0.072373719973384','545.757314476305','0','0','6','86']
  H_HDV['102_CO_2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.49452904177801','0.022364204657788','0.810594091163379','0.144461436426373','0.048450171695191','6','76']
  H_HDV['102_THC_2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015662647059126','0.13733877608694','6.40335448955172','2.59067588775543','-0.018679995439062','6','76']
  H_HDV['102_NOx_2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.01367916586433','0.026085404815337','37.9273052684232','0.435312849465544','5.25721413702014','6','76']
  H_HDV['102_PM_2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.146023609472273','0.049916886366165','0.238056480152825','0.178472326547735','0.030874681339392','6','76']
  H_HDV['102_FC_2_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002280606828865','0.415767116132379','-26.5664439008691','1353.14951158933','0','6','76']
  H_HDV['102_CO_4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.200420082642682','0.56019811086503','8.54996000436626','3.38000562380459','-0.029159851822428','6','72']
  H_HDV['102_THC_4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007495244179563','0.162196185261664','5.31108180016969','2.13183214454059','-0.011934180978723','6','72']
  H_HDV['102_NOx_4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.14431762403294','0.049419538117937','16.4866041716741','0.22081302977468','5.339751493759','6','72']
  H_HDV['102_PM_4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.081155731547883','0.031344297293774','0.271498100657686','0.142595044347821','0.026334532305751','6','72']
  H_HDV['102_FC_4_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','800.335288678807','0.09140600607476','9762.13118183913','0.827360315765313','481.475570218515','6','72']
  H_HDV['102_CO_4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.580686224080515','0.024753058478811','0.738215694172692','0.157842022720497','0.056518106367075','6','67']
  H_HDV['102_THC_4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015678709717489','0.143070261942244','6.08891505975057','2.48062161036622','-0.01717065069244','6','67']
  H_HDV['102_NOx_4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.17174387381733','0.031350226278085','20.954861418638','0.305298222614513','6.74830506354529','6','67']
  H_HDV['102_PM_4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.120713848338507','0.030568503372825','0.276326198293534','0.156291950472305','0.018169565725288','6','67']
  H_HDV['102_FC_4_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','512.588534496944','0.058013898873628','4872.5053278271','0.585478943158827','831.720888412475','6','67']
  H_HDV['102_CO_4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.54430823811441','-0.002866903715695','-0.117909713142617','0.097420149168047','0','6','50']
  H_HDV['102_THC_4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.021815460008152','0.12903982363421','6.87003805917425','2.80739270694949','-0.025490240747992','6','50']
  H_HDV['102_NOx_4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','32.5891393486157','-0.093207857390116','-3.50616614841605','0.195076643843123','0','6','50']
  H_HDV['102_PM_4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031525328123918','0.32118069186956','3.53011463261388','1.55453317775781','0.008026480829367','6','50']
  H_HDV['102_FC_4_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1509.57812115738','0.005655371145576','7065.30939060607','0.598625958446058','24.3541779637609','6','50']
  H_HDV['102_CO_6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.72149018830137','-0.004038109720045','-0.276856348295714','0.21857463015268','0','6','71']
  H_HDV['102_THC_6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007525166309784','0.147681275078312','6.99874256166079','2.80078667577164','-0.033939788807173','6','71']
  H_HDV['102_NOx_6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.8561597669972','-0.041746051429768','-5.84453416179522','0.256548546260206','0','6','71']
  H_HDV['102_PM_6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.415241693144689','-0.000799639906793','-0.049589905099789','0.149645758026669','0','6','71']
  H_HDV['102_FC_6_0'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','2427.55798281985','1.0087285480763','-0.45141873595291','0','0','6','71']
  H_HDV['102_CO_6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.40650587893733','-4.19527474430475','-0.972283000945051','0','0','6','50']
  H_HDV['102_THC_6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.023024915897397','0.126665055873176','7.28787864875545','2.97656744741203','-0.028238760295337','6','50']
  H_HDV['102_NOx_6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.7070474319938','0.031695159787029','20.743837801101','0.275197470512656','8.50038285124609','6','50']
  H_HDV['102_PM_6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.111818654529902','0.051040939035222','0.261693153676192','0.12656950182833','0.041632509355222','6','50']
  H_HDV['102_FC_6_50'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','498.964370353518','0.065723505738831','6907.15519113479','0.652165145276282','1150.92912666919','6','50']
  H_HDV['102_CO_6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00002971264007','0.002964133708213','-0.107842508264133','1.73957583002617','0','6','36']
  H_HDV['102_THC_6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000004754707103','0.000448412180526','-0.014726919043104','0.211631794903587','0','6','36']
  H_HDV['102_NOx_6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000552742364533','0.047458151045552','-1.49444792107755','32.6214659520679','0','6','36']
  H_HDV['102_PM_6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053312100773731','0.255084910163835','6.59938329176593','2.83902583263783','-0.029852950042964','6','36']
  H_HDV['102_FC_6_100'] = [' 102','EURO5','TT/AT >40-50t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1535.80989883641','0.00652472148004','8141.12938778432','0.626652307528916','465.724846370717','6','36']
  H_HDV['103_CO_0_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1460111436716','0.059527120461945','15.4386698636726','0.270051089050029','1.91148988088644','6','86']
  H_HDV['103_THC_0_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.420776585601971','7.62555084331196','3.0738980836888','1.39677801194832','0.007138975546079','6','86']
  H_HDV['103_NOx_0_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.6791168513567','0.049102942649049','113.020108229038','0.485063856418131','10.2312660844189','6','86']
  H_HDV['103_PM_0_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.8947233167964','0.053516337800067','2.72598707430311','0.217701021483172','0.398892527740697','6','86']
  H_HDV['103_FC_0_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','829.407890963883','0.055510597934395','10347.5185611273','0.656941068164639','247.362907610533','6','86']
  H_HDV['103_CO_0_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.3705351088102','127.620833443527','-1.19090728429137','0.471364502081165','0.041329428682729','6','86']
  H_HDV['103_THC_0_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.295221833443763','7.42373880940481','3.51337413779745','1.60393095389768','-0.003544961629916','6','86']
  H_HDV['103_NOx_0_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','50','NOx','y=(1/(a+(b*(x**c))))','0.022626540626521','0.000841603345774','0.902298035051416','0','0','6','86']
  H_HDV['103_PM_0_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.01254351990331','0.046884667756136','4.62368206321054','0.312860318013084','0.513653117352249','6','86']
  H_HDV['103_FC_0_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','829.102388495966','0.036432127968109','16865.3505254774','0.750570835762477','306.396952786444','6','86']
  H_HDV['103_CO_0_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.8170331979955','86.9709525458202','-0.790695999193488','0.371554842708338','0.047861352204517','6','86']
  H_HDV['103_THC_0_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.304887648098076','7.25846034917554','3.57108823028494','1.64947143962947','-0.006190766900641','6','86']
  H_HDV['103_NOx_0_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','100','NOx','y=(1/(a+(b*(x**c))))','0.022910111344406','0.000039173687422','1.52406065794739','0','0','6','86']
  H_HDV['103_PM_0_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.35945201790484','0.04393111062125','9.04324521707145','0.451586226393155','0.598135722289205','6','86']
  H_HDV['103_FC_0_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','0','100','FC','y=(1/(a+(b*x)))','0.000700129579243','0.000020394872834','0','0','0','6','86']
  H_HDV['103_CO_-2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.3814296524552','0.009745067847425','-0.975623084816483','0.065570871281508','0','6','86']
  H_HDV['103_THC_-2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.9962601810762','0.061583317213907','7.62919381623259','0.382994576044106','0.388601875107653','6','86']
  H_HDV['103_NOx_-2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','0','NOx','y=((a*(b**x))*(x**c))','70.0795048775328','0.988807562926466','-0.533099189422459','0','0','6','86']
  H_HDV['103_PM_-2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.20872458488179','0.059830419918778','1.35511441888455','0.168050010769694','0.227821245216831','6','86']
  H_HDV['103_FC_-2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','708.601382895908','0.048938889072321','32380.869636152','0.824339078668403','66.3955100764193','6','86']
  H_HDV['103_CO_-2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.834287728506','0.055129193291426','216.807063360853','0.748686576425197','0.833808553688804','6','86']
  H_HDV['103_THC_-2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.42862471725147','0.056328349496774','11.2741311745292','0.403654980155415','0.249082577131092','6','86']
  H_HDV['103_NOx_-2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000079123640595','0.014368069917182','-0.981021191500034','29.2987054587247','0','6','86']
  H_HDV['103_PM_-2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.28031104448522','0.051685939971001','8.19530382993273','0.457999571202989','0.204683594350175','6','86']
  H_HDV['103_FC_-2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-2.6500146235971','0.900469558447978','1988.60694787927','-0.530415067817923','0','6','86']
  H_HDV['103_CO_-2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2739236219873','0.045226843946438','480.819417446096','0.832415615194353','0.658337438601302','6','86']
  H_HDV['103_THC_-2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.03300764111994','0.054185948372046','12.6403356181312','0.399967770062202','0.213830339306685','6','86']
  H_HDV['103_NOx_-2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','29.2715377024191','-0.164642637047553','-0.878225959201034','0.051813899589144','0','6','86']
  H_HDV['103_PM_-2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.26737272527169','0.043995892465545','28.5645842215593','0.647641338231933','0.181220546659469','6','86']
  H_HDV['103_FC_-2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','957.097830597143','-3.62113150267991','-42.2747500097994','0.063390400809145','0','6','86']
  H_HDV['103_CO_-4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2946600767103','0.06117206976754','55.9786066916096','0.570868876015298','0.335832301607264','6','86']
  H_HDV['103_THC_-4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.18192579554177','0.05850822441105','16.5280838439342','0.474397438132612','0.193948401401415','6','86']
  H_HDV['103_NOx_-4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.9165182526186','0.045914918050848','531.691121251181','0.702759311091426','0.328464486806757','6','86']
  H_HDV['103_PM_-4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.122398953150432','9.19350002457395','-0.005309240719535','0.592491557686909','0.038328004430012','6','86']
  H_HDV['103_FC_-4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','526.258437073315','0.048125944305306','29583.5565885755','0.791883862442958','8.15510826289678','6','86']
  H_HDV['103_CO_-4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.50643513907526','0.05004533760936','246.227350463538','0.75170904266477','0.201792463608343','6','86']
  H_HDV['103_THC_-4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.7662139898214','0.057676713984095','30.2148410240007','0.573487962670972','0.175287068555222','6','86']
  H_HDV['103_NOx_-4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.9218463337666','0.03324606591255','1779.66668078459','0.91055620563051','-0.357912689361128','6','86']
  H_HDV['103_PM_-4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.96673682964365','0.053225841832133','10.8883971293005','0.500188904874418','0.128048709464499','6','86']
  H_HDV['103_FC_-4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','464.069892454806','0.038542538899868','75679.4390332485','0.951784469649207','-4.80514883278002','6','86']
  H_HDV['103_CO_-4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.12271096090022','0.043158929781397','532.911307043194','0.867035124716848','0.076159357044477','6','86']
  H_HDV['103_THC_-4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.55602240139169','0.057567974056335','34.7281154562366','0.595974314179676','0.17192935052679','6','86']
  H_HDV['103_NOx_-4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.6686339938832','0.025851031084151','5472.24186812471','1.10562525067314','-1.28053081096099','6','86']
  H_HDV['103_PM_-4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.94609350707821','0.048736882669466','22.9903093579149','0.622524870391319','0.116240841775716','6','86']
  H_HDV['103_FC_-4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','457.252922224331','0.031893926833212','136036.049377073','1.05394959988866','-19.2795874698974','6','86']
  H_HDV['103_CO_-6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.06139613674186','0.066065655701736','102.793949609875','0.678347621043298','0.219194589246451','6','72']
  H_HDV['103_THC_-6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.01182059156851','0.063718941973433','32.0567023129748','0.602898787659314','0.162965766714279','6','72']
  H_HDV['103_NOx_-6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.1107804411726','0.059785264952445','851.509188693359','0.80622146205578','0.24860802421764','6','72']
  H_HDV['103_PM_-6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.03978085554069','0.063365828357734','3.73688105675621','0.356538117754322','0.127835276020729','6','72']
  H_HDV['103_FC_-6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','455.477915921545','0.058946851125768','37505.8350468183','0.847652192133871','3.1996415393715','6','72']
  H_HDV['103_CO_-6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.26284655329542','0.05751167048128','222.599921558044','0.748138473861605','0.165868474470793','6','72']
  H_HDV['103_THC_-6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.48525745959214','0.061581388486963','34.1177783323427','0.598538802814698','0.156813147996291','6','72']
  H_HDV['103_NOx_-6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0459876266579','0.052411376189011','1172.79179044476','0.858032133215189','0.197440722858402','6','72']
  H_HDV['103_PM_-6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.89073450828453','0.059881437779389','9.64858176024954','0.49458792591066','0.123562884452773','6','72']
  H_HDV['103_FC_-6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','368.914072192818','0.053102144543914','49096.1072127483','0.888057539181215','1.83947706507991','6','72']
  H_HDV['103_CO_-6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.80073553262759','0.053581920282771','275.928417179237','0.771155861197707','0.140935151047454','6','72']
  H_HDV['103_THC_-6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.27002753342208','0.061012446589129','28.4931910407487','0.557827113398389','0.156158044870621','6','72']
  H_HDV['103_NOx_-6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.4441097193715','0.049242084934692','1599.92648293012','0.907237214419288','0.197276647847342','6','72']
  H_HDV['103_PM_-6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.84671702613231','0.057764829491181','12.775230905432','0.5347189070797','0.121348602808854','6','72']
  H_HDV['103_FC_-6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','342.248434822173','0.050371878969315','52160.2711811518','0.895488255953335','1.8769825723641','6','72']
  H_HDV['103_CO_2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.53873278614099','13.8794641358631','5.21299062002894','1.94104795898323','0.016856015983364','6','86']
  H_HDV['103_THC_2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.052869760861011','7.1608568009122','4.70340607949372','1.89303490476922','-0.012937510163694','6','86']
  H_HDV['103_NOx_2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','57.7700043077258','0.035183939158569','-3.1439403349501','0.081402315058111','0','6','86']
  H_HDV['103_PM_2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.622612102355203','2.56172675814145','4.78752446528651','1.84583241176559','0.015301104573794','6','86']
  H_HDV['103_FC_2_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1774.37170603906','0.587675544253945','-106.271225211876','0.078568737339041','0','6','86']
  H_HDV['103_CO_2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.026858686357753','0.005463739418522','-0.000018387780025','0','0','6','79']
  H_HDV['103_THC_2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.581376385812506','6.03759246108674','6.44864562553625','2.59255533863691','-0.017418296889837','6','79']
  H_HDV['103_NOx_2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000105470821113','0.018724750661392','-1.13556092425039','61.2715269371375','0','6','79']
  H_HDV['103_PM_2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.808848842465525','5.47223060703502','1.40005396560838','0.897193353298567','0.018487735409024','6','79']
  H_HDV['103_FC_2_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','50','FC','y=(c+(a*exp(b*x)))','1216.1494331164','-0.078978451976814','803.703055132963','0','0','6','79']
  H_HDV['103_CO_2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','100','CO','y=((a*(b**x))*(x**c))','58.8346528090931','0.992675232430932','-0.570657994059271','0','0','6','62']
  H_HDV['103_THC_2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.754337562233489','5.51483837214333','7.32350913566752','2.94519226449564','-0.025993854418862','6','62']
  H_HDV['103_NOx_2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','100','NOx','y=(1/(a+(b*x)))','0.01416333851631','0.000097255914256','0','0','0','6','62']
  H_HDV['103_PM_2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.54110056245822','0.040332842899157','7.99660632322966','0.398121354683125','0.978915121560465','6','62']
  H_HDV['103_FC_2_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','928.941638214922','0.036094765658761','90718.6600458394','1.06007521259824','976.279979168483','6','62']
  H_HDV['103_CO_4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.20872072226325','15.7745530438347','5.59872746923465','2.27902234517125','-0.019564913053232','6','71']
  H_HDV['103_THC_4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.381742097167645','6.79328308818045','5.83296599147003','2.32829152930316','-0.014007277092561','6','71']
  H_HDV['103_NOx_4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','71.8758808843471','0.032001658405691','-4.29659872489178','0.103578470407589','0','6','71']
  H_HDV['103_PM_4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.623989259939005','2.77738094283673','5.13335253571096','2.14381313082115','-0.023183800610292','6','71']
  H_HDV['103_FC_4_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','0','FC','y=(c+(a*exp(b*x)))','1565.72982029058','-0.102386213458599','672.821148976371','0','0','6','71']
  H_HDV['103_CO_4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','32.6638382128893','-0.090418003614887','-3.29761667816034','0.135137339911516','0','6','54']
  H_HDV['103_THC_4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.867446933118573','5.59614537912762','8.14490462225059','3.29828762383322','-0.033462853066956','6','54']
  H_HDV['103_NOx_4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000286969556825','0.03189554587816','-1.32797134455662','80.7687933351866','0','6','54']
  H_HDV['103_PM_4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.77550159683854','-0.01512373451165','-0.548526086033263','0.147862552198714','0','6','54']
  H_HDV['103_FC_4_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.015100469555738','1.78520677286078','-72.9226555367318','2326.49284512775','0','6','54']
  H_HDV['103_CO_4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','44.7336429614011','-0.177290186920474','-5.84041744633873','0.179657183538058','0','6','40']
  H_HDV['103_THC_4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.28145842697604','4.96782409182615','9.27956856164813','3.88470800802398','-0.052726242963597','6','40']
  H_HDV['103_NOx_4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00117291777898','0.073285065161613','-1.73051473391781','106.30736447165','0','6','40']
  H_HDV['103_PM_4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000108885867506','0.009533264106343','-0.303083956275101','5.80380257603832','0','6','40']
  H_HDV['103_FC_4_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.047659195169544','3.6418396750479','-99.4077480617569','2924.37202497552','0','6','40']
  H_HDV['103_CO_6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.4862456409352','-0.034055913864667','-2.22593841791043','0.104986080071954','0','6','69']
  H_HDV['103_THC_6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.515583670061443','6.23951536041523','7.84168762211649','3.16198222552313','-0.0379229375834','6','69']
  H_HDV['103_NOx_6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','78.8345119073354','0.037937443938442','-3.91264740717751','0.10380932339968','0','6','69']
  H_HDV['103_PM_6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.00497122667822','2.64976890639219','4.57132870367035','1.81502450018536','0.01112402339444','6','69']
  H_HDV['103_FC_6_0'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2285.4932407189','3.47920315168204','-123.241953020265','0.079740498162103','0','6','69']
  H_HDV['103_CO_6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','50','CO','y=(1/(a+(b*(x**c))))','0.020091500481009','0.00520170638357','0.920113624079673','0','0','6','38']
  H_HDV['103_THC_6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.45030043952766','32.9689182253125','-2.57897488597958','-1.1401780441743','0.218858693510867','6','38']
  H_HDV['103_NOx_6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','70.9541400167761','0.005127582866691','2954.33719360123','1.03564330079303','22.1827849779387','6','38']
  H_HDV['103_PM_6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.160704418617605','0.014799257513478','-0.000144314844206','0','0','6','38']
  H_HDV['103_FC_6_50'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','787.448625303592','0.053760287340596','8328.71261612668','0.615330213864229','1622.57714013282','6','38']
  H_HDV['103_CO_6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','100','CO','y=(1/(a+(b*x)))','0.019505527278595','0.003246823522181','0','0','0','6','27']
  H_HDV['103_THC_6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.81678229891064','4.66317762008323','8.58489920444713','3.83395628745821','-0.061269141284852','6','27']
  H_HDV['103_NOx_6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.008173757130774','0.409428299174092','-6.71994168129579','159.185362165899','0','6','27']
  H_HDV['103_PM_6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.61906260950501','-0.053099514714505','-1.46169777372868','0.270771162907821','0','6','27']
  H_HDV['103_FC_6_100'] = [' 103','EURO0','TT/AT >50-60t 80ties','6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.192376092315772','10.5460901199439','-195.47451116417','3941.2874920516','0','6','27']
  H_HDV['104_CO_0_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.8176684877439','0.055498082532602','12.7947765822037','0.245183411988472','1.48480325816182','6','86']
  H_HDV['104_THC_0_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.221853845704394','6.22359762388869','3.32831784159445','1.5198136916242','-0.00599847015525','6','86']
  H_HDV['104_NOx_0_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.5632513614888','0.049940135231516','61.387201478795','0.39018150990179','7.42425134546887','6','86']
  H_HDV['104_PM_0_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.6075690850672','0.05227620642192','3.08107653579915','0.182794096598724','0.271552572691925','6','86']
  H_HDV['104_FC_0_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','657.682848064568','0.054531934481037','16037.7086565336','0.757946168207109','210.581715667315','6','86']
  H_HDV['104_CO_0_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.43607274478526','0.049786087785096','22.9669792155669','0.331360311778812','2.15201536711627','6','86']
  H_HDV['104_THC_0_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.249606797871534','6.2774052659059','3.19505467841813','1.47337698002567','-0.004838610331987','6','86']
  H_HDV['104_NOx_0_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','50','NOx','y=((a*(b**x))*(x**c))','44.447652026052','0.994394688821307','-0.231963594105857','0','0','6','86']
  H_HDV['104_PM_0_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.73974764226677','0.048531622693709','3.93362324038757','0.263337928457349','0.387638360221315','6','86']
  H_HDV['104_FC_0_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','696.367080692599','0.034905304436706','113335.79510535','1.10463622536806','266.819215737469','6','86']
  H_HDV['104_CO_0_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.7898023880386','0.04910027483547','-0.653357550150351','0.041310686168181','0','6','86']
  H_HDV['104_THC_0_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.99393005227506','0.032998194635983','4.71557433948995','0.135146073895572','0.499607447045577','6','86']
  H_HDV['104_NOx_0_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','100','NOx','y=(c+(a*exp(b*x)))','30.8724846659416','-0.014754271980314','2.83551200923139','0','0','6','86']
  H_HDV['104_PM_0_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.06737151146618','0.046720438437971','5.21550134970637','0.364206455051842','0.498503078740617','6','86']
  H_HDV['104_FC_0_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','0','100','FC','y=(c+(a*exp(((-1)*b)*x)))','881.100160337058','0.02632358680225','276.53222978302','0','0','6','86']
  H_HDV['104_CO_-2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.403099687998','0.070928822784905','0.000006130160689','-0.127601368400435','0.786850636681652','6','86']
  H_HDV['104_THC_-2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.278995138263301','7.87909717981682','1.40325317207672','0.795103534595691','0.034968643547565','6','86']
  H_HDV['104_NOx_-2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','0','NOx','y=((a*(b**x))*(x**c))','65.971410386116','0.990085593458853','-0.612896492569913','0','0','6','86']
  H_HDV['104_PM_-2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053057686241734','2.4777819058186','7.31543351055244','2.87532110856641','-0.032706872117083','6','86']
  H_HDV['104_FC_-2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','0','FC','y=((a*(b**x))*(x**c))','2205.75199203135','0.99127060627769','-0.651053592780712','0','0','6','86']
  H_HDV['104_CO_-2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.93871314625462','0.052143339685233','133.322952211415','0.691692349630721','0.675915531080848','6','86']
  H_HDV['104_THC_-2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.149756570575923','0.019597817155514','0.000427991889329','0','0','6','86']
  H_HDV['104_NOx_-2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.007463756971876','1.3974129327981','51.8878162878545','-0.533630612727476','0','6','86']
  H_HDV['104_PM_-2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','50','PM','y=exp((a+(b/x))+(c*log(x)))','4.23365948539455','-5.72123560511725','-1.36776126743056','0','0','6','86']
  H_HDV['104_FC_-2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-3.13840532401498','0.837666554684543','1665.68401117864','-0.512575207947064','0','6','86']
  H_HDV['104_CO_-2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.84477547291508','0.04049996291154','408.08970872891','0.836059374752511','0.510468582733711','6','86']
  H_HDV['104_THC_-2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','100','THC','y=((a*(x**b))+(c*(x**d)))','-9.20888048741874','-0.294800932757414','20.2409680854089','-0.458750910003797','0','6','86']
  H_HDV['104_NOx_-2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.6522210507038','-0.117011077441039','-1.03568682722376','0.067675136508712','0','6','86']
  H_HDV['104_PM_-2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','100','PM','y=((a*(b**x))*(x**c))','7.32097517717664','0.984758287424433','-0.588854778823019','0','0','6','86']
  H_HDV['104_FC_-2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','835.245588256821','-3.14556855751336','-35.710124161282','0.06138608437266','0','6','86']
  H_HDV['104_CO_-4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.45566227197037','0.059035933347535','23.058361832164','0.430920463880827','0.226255008817288','6','86']
  H_HDV['104_THC_-4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.064421684436164','44.9215901780685','-1.33931308471567','0.577627642795109','0.030329215165406','6','86']
  H_HDV['104_NOx_-4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.1406146077932','0.049805238882516','205.376341131092','0.568210923762611','0.346576526253444','6','86']
  H_HDV['104_PM_-4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.32316370901995','-8.58548605287699','-1.74560464544545','0','0','6','86']
  H_HDV['104_FC_-4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','460.024018309153','0.049335269039127','18237.7823317577','0.739209155221203','8.09755320664493','6','86']
  H_HDV['104_CO_-4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.39215447278974','0.047315217636046','149.059130876402','0.693910517595243','0.10418036027456','6','86']
  H_HDV['104_THC_-4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.82197000623498','0.053371418418412','14.8151318366286','0.471461445019082','0.103213891717265','6','86']
  H_HDV['104_NOx_-4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.8210335469018','0.03719055277109','624.556672256208','0.757539243574915','-0.048457983804678','6','86']
  H_HDV['104_PM_-4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.81350086352173','0.056005114103299','6.00312264687628','0.372938929042021','0.072123365491342','6','86']
  H_HDV['104_FC_-4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','411.499096229987','0.038816853425479','49472.2574291346','0.906120100960926','-3.49393964012788','6','86']
  H_HDV['104_CO_-4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.19408323532861','0.038933964280505','432.94882794286','0.857897072975816','-0.041373079908628','6','86']
  H_HDV['104_THC_-4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.65128622450831','0.052333664449137','23.8536847597068','0.550010773455167','0.099251792945981','6','86']
  H_HDV['104_NOx_-4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.0041673904555','0.030677907330449','1853.82543837828','0.946454446623662','-0.467048099468844','6','86']
  H_HDV['104_PM_-4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66819557985136','0.049365820809594','15.28048379555','0.52661054050369','0.058614318464253','6','86']
  H_HDV['104_FC_-4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','411.841940254089','0.031751886700949','100959.894149898','1.02916220257423','-17.716951218484','6','86']
  H_HDV['104_CO_-6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.06152184984914','0.067951029282337','46.4003510109266','0.584523212221606','0.151795451890893','6','72']
  H_HDV['104_THC_-6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.31060907021823','0.064964207081374','22.0454158296382','0.583884341534891','0.109679189851339','6','72']
  H_HDV['104_NOx_-6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.3453088847292','0.06424620714917','422.020518290945','0.717034491345181','0.2592365551465','6','72']
  H_HDV['104_PM_-6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.24992738444045','0.07206859821814','2.19931247302819','0.276930597128773','0.076808940763637','6','72']
  H_HDV['104_FC_-6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','409.688815891009','0.060513124589103','25555.0667403762','0.812843566868634','3.94110567439527','6','72']
  H_HDV['104_CO_-6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.52554097681028','0.057487367183494','159.352788121088','0.732789734722491','0.106148537520978','6','72']
  H_HDV['104_THC_-6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.75445992358523','0.061640144827983','28.4339897167598','0.606979367425946','0.102533952895295','6','72']
  H_HDV['104_NOx_-6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.3689348198575','0.056638871214079','687.791078483189','0.793803457418985','0.215854885048212','6','72']
  H_HDV['104_PM_-6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.80334078105981','0.065280607345509','10.9328402608421','0.523872755424672','0.069173196074704','6','72']
  H_HDV['104_FC_-6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','332.775648960658','0.053660220453285','37494.8638200534','0.870306280156306','2.34855413264646','6','72']
  H_HDV['104_CO_-6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.214196699319','0.052570631370359','237.696695025793','0.789690177905141','0.080210577633588','6','72']
  H_HDV['104_THC_-6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.53237177725402','0.060336226596863','23.6754980465982','0.564358862052016','0.101085394939112','6','72']
  H_HDV['104_NOx_-6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.59072849197767','0.053114989616973','778.789036207716','0.811325141717865','0.212324936713269','6','72']
  H_HDV['104_PM_-6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.66252764914914','0.061598689844272','13.8850852914124','0.553866822393314','0.066032450099694','6','72']
  H_HDV['104_FC_-6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','311.832887205768','0.050527053459285','43333.2434320128','0.891840716301032','2.094116311386','6','72']
  H_HDV['104_CO_2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.5467513830824','0.015808295731349','-1.08126286379405','0.071160714302654','0','6','86']
  H_HDV['104_THC_2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.277080764674322','5.30466513262356','5.33171343932086','2.1266669077555','-0.018224029987632','6','86']
  H_HDV['104_NOx_2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.9807462545688','-0.016751494155617','-3.12725726500041','0.103929605927472','0','6','86']
  H_HDV['104_PM_2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.6261409405787','0.001315228995471','-0.263282939076692','0.080966118259005','0','6','86']
  H_HDV['104_FC_2_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','0','FC','y=((a*(b**x))*(x**c))','2937.22062758412','1.00911449789488','-0.61396084579487','0','0','6','86']
  H_HDV['104_CO_2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.037572378835533','0.006238915186354','-0.000040217650332','0','0','6','79']
  H_HDV['104_THC_2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.824616644344697','4.62732780321393','6.30894894825362','2.51263307037941','-0.015244577248445','6','79']
  H_HDV['104_NOx_2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','50','NOx','y=(a/(1+(b*exp(((-1)*c)*x))))','23.1642362559767','-0.523979191713955','0.034399442399316','0','0','6','79']
  H_HDV['104_PM_2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.75980736378642','-0.001911266603605','-0.275733320610156','0.096813460530016','0','6','79']
  H_HDV['104_FC_2_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003805269240712','0.661974233869558','-38.7620031115003','1482.76823038428','0','6','79']
  H_HDV['104_CO_2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.7644240140974','-0.09500801132802','-2.84219626517979','0.151710907133288','0','6','63']
  H_HDV['104_THC_2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.05754860088535','4.19282730742154','7.38174001473791','2.94335903051528','-0.031820666465949','6','63']
  H_HDV['104_NOx_2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','100','NOx','y=((a*(b**x))*(x**c))','57.3475686907841','0.995490589836591','-0.071708726226598','0','0','6','63']
  H_HDV['104_PM_2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.88123346028627','0.043365195894728','12.5844790873895','0.526134950861593','0.938866660698988','6','63']
  H_HDV['104_FC_2_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','2','100','FC','y=(1/(a+(b*(x**c))))','0.000445377988283','0.000083602551204','0.485845136363595','0','0','6','63']
  H_HDV['104_CO_4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.23671104513338','60.9413700931691','-2.07571101815086','-0.726136491357861','0.166431479289983','6','71']
  H_HDV['104_THC_4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.654085578449046','5.02238089578358','6.17258759298641','2.45418512237655','-0.016005555680198','6','71']
  H_HDV['104_NOx_4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','17.969287286495','0.072661440545228','32.7179512566386','0.294453062710562','21.7146460557508','6','71']
  H_HDV['104_PM_4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.625646347733115','1.92668825070086','5.3889047119556','1.93845564150519','0.036862099469601','6','71']
  H_HDV['104_FC_4_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','906.484386031975','0.08945313393304','7271.73762574421','0.705994303439057','594.406967880415','6','71']
  H_HDV['104_CO_4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.9157247154289','-0.025138891677404','-1.54865743318508','0.100928248668349','0','6','55']
  H_HDV['104_THC_4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.68050283544188','-0.0143404077973','-0.798057041887136','0.118394440910454','0','6','55']
  H_HDV['104_NOx_4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000255549236746','0.031024800040932','-1.36454044415654','61.2508744077934','0','6','55']
  H_HDV['104_PM_4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.66645485006729','-0.0111404104347','-0.569473673232747','0.193246513109497','0','6','55']
  H_HDV['104_FC_4_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1191.43630279717','0.006767967470214','3123.67310669302','0.380514449068972','225.396080004658','6','55']
  H_HDV['104_CO_4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.1528439526982','-0.219607909276345','-8.93731723213274','0.287594918094211','0','6','40']
  H_HDV['104_THC_4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000141938140107','0.013865266808821','-0.46996597812713','7.73006217530823','0','6','40']
  H_HDV['104_NOx_4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.000000000112197','6.65178627683634','87.117373569146','-0.115186775219825','0','6','40']
  H_HDV['104_PM_4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000062461199644','0.005062399767679','-0.158186702825438','3.89661259233742','0','6','40']
  H_HDV['104_FC_4_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.031733413456865','2.42066156996873','-70.1714630800711','2461.59811786076','0','6','40']
  H_HDV['104_CO_6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.5657992267909','-0.026287593299259','-2.57425276499615','0.14083022649498','0','6','69']
  H_HDV['104_THC_6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.623279141076039','4.918126347671','7.60778934027982','3.09448305690739','-0.047658596393915','6','69']
  H_HDV['104_NOx_6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','61.7983851550579','-0.046092790029232','-3.93860246137585','0.127820902345002','0','6','69']
  H_HDV['104_PM_6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.19054777838147','0.009405228388743','-0.184866954637286','0.066710633707843','0','6','69']
  H_HDV['104_FC_6_0'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','2940.10646781794','1.00888715286251','-0.447023226194678','0','0','6','69']
  H_HDV['104_CO_6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.3690598799889','-0.092049593696647','-7.54825430670828','0.25818481203723','0','6','40']
  H_HDV['104_THC_6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.71847609049755','15.2161767177132','-0.961346325731466','-0.326779554299955','0.1344184439206','6','40']
  H_HDV['104_NOx_6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','113.725178831349','-0.209059716566156','-14.4853649925011','0.274291018443652','0','6','40']
  H_HDV['104_PM_6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.55029324486014','0.016539440541446','8.26874309643173','0.43563801243065','0.916566347825258','6','40']
  H_HDV['104_FC_6_50'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','865.762351962862','0.018092055355057','8162.80922902471','0.579455141673882','1053.00463445872','6','40']
  H_HDV['104_CO_6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','100','CO','y=((a+(b*x))**((-1)/c))','0.505172054582395','0.003397616580155','0.212137778399496','0','0','6','27']
  H_HDV['104_THC_6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.47566772173968','2.07092315271775','-0.512688301844705','0.009509968040084','0','6','27']
  H_HDV['104_NOx_6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','100','NOx','y=((a*(x**b))+(c*(x**d)))','21.6874625001371','-0.123536635712482','95.4140771621986','-0.121512024113854','0','6','27']
  H_HDV['104_PM_6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.72774010272258','0.018348588208094','266.274757410509','1.07738269969145','0.912490493701528','6','27']
  H_HDV['104_FC_6_100'] = [' 104','EURO1','TT/AT >50-60t Euro-1','6','100','FC','y=((a*(b**x))*(x**c))','3336.15801567906','0.996966346756249','-0.103679326878922','0','0','6','27']
  H_HDV['105_CO_0_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.057474168674945','0.015726518788533','-0.000092491986192','0','0','6','86']
  H_HDV['105_THC_0_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.13800111887692','4.06758786194341','3.12207641002234','1.45031891220822','-0.004378811897236','6','86']
  H_HDV['105_NOx_0_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','7.25419295476519','191.261305821432','-0.822770159113221','0.622672560998033','0.022777276834351','6','86']
  H_HDV['105_PM_0_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.01342268303463','0.139709442709895','-0.000864464224767','0','0','6','86']
  H_HDV['105_FC_0_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','584.021154490041','0.052603772515445','16362.1956242336','0.785067004383157','203.570109643423','6','86']
  H_HDV['105_CO_0_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.8186599094423','0.045308084246259','-0.382934056917869','0.03632018334683','0','6','86']
  H_HDV['105_THC_0_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.952412216327649','0.020292367699424','3.43355510164729','0.116372428967371','0.15882240419364','6','86']
  H_HDV['105_NOx_0_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','50','NOx','y=(1/(a+(b*(x**c))))','0.021082646224392','0.003032848597033','0.725062099741749','0','0','6','86']
  H_HDV['105_PM_0_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','50','PM','y=(((a*(x**2))+(b*x))+c)','0.000140824191658','-0.017486276457761','0.772949702594771','0','0','6','86']
  H_HDV['105_FC_0_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','639.504538713627','0.031777271294936','227703.721613008','1.2477926430748','252.215072250204','6','86']
  H_HDV['105_CO_0_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.7326888548533','0.13979532010476','-0.392964820634855','0.022140645531195','0','6','86']
  H_HDV['105_THC_0_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.05771242912088','0.021605689272525','3.24585946313761','0.122146726816295','0.190028477067236','6','86']
  H_HDV['105_NOx_0_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000046307377114','0.008928167031907','-0.739687509948022','37.9305068430628','0','6','86']
  H_HDV['105_PM_0_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','0.000000959546648','0.000065001408691','-0.019047818166426','0.989307501971507','0','6','86']
  H_HDV['105_FC_0_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','0','100','FC','y=(((a*(x**2))+(b*x))+c)','0.08162405311623','-14.6802544002938','1038.88598383179','0','0','6','86']
  H_HDV['105_CO_-2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.90131903967356','0.00841320320018','-0.475296567646161','0.061431946365599','0','6','86']
  H_HDV['105_THC_-2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.178480831629206','7.25205580636245','0.591023488963006','0.67468443845914','0.03271631040278','6','86']
  H_HDV['105_NOx_-2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','0','NOx','y=((a*(b**x))*(x**c))','71.2358110055381','0.98968117803095','-0.602510719287388','0','0','6','86']
  H_HDV['105_PM_-2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.669380488493548','0.000507985636029','-0.036143309402685','0.057214636808738','0','6','86']
  H_HDV['105_FC_-2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','541.158648431523','0.048352090249776','20868.4105341713','0.80659930286625','54.2160860289828','6','86']
  H_HDV['105_CO_-2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','50','CO','y=((a*(b**x))*(x**c))','18.6147282824483','0.98822302584015','-0.552126012631564','0','0','6','86']
  H_HDV['105_THC_-2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','50','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.222213993552882','0.0326095739917','0.000521477873342','0','0','6','86']
  H_HDV['105_NOx_-2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','50','NOx','y=((a*(x**b))+(c*(x**d)))','-0.024064717047652','1.15927825334817','56.3928291324505','-0.528073294768134','0','6','86']
  H_HDV['105_PM_-2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.598317407800353','0.044640897834476','27.9897716809894','0.934258442224067','0.071980057236583','6','86']
  H_HDV['105_FC_-2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','50','FC','y=((a*(b**x))*(x**c))','1032.04020012628','0.978231268117822','-0.2438516225972','0','0','6','86']
  H_HDV['105_CO_-2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.8217999436449','0.037849137316439','843.359591845416','1.0046911196714','0.404619265254634','6','86']
  H_HDV['105_THC_-2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','100','THC','y=((a*(b**x))*(x**c))','7.0234057722342','0.983173632746704','-0.57910049881037','0','0','6','86']
  H_HDV['105_NOx_-2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.3087693957357','-0.116240590984032','-1.22356445681566','0.068109971604306','0','6','86']
  H_HDV['105_PM_-2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','100','PM','y=(c+(a*exp(b*x)))','0.784548223341095','-0.04054867263854','0.070714035004224','0','0','6','86']
  H_HDV['105_FC_-2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','751.613382102147','-3.06400829749281','-28.8354005504742','0.056722352353101','0','6','86']
  H_HDV['105_CO_-4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.100222739007371','64.9919797218085','-1.36353483015999','0.547377690058388','0.032772603133464','6','86']
  H_HDV['105_THC_-4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','0','THC','y=((a*(b**x))*(x**c))','7.44406089781983','0.982310140039332','-0.630024655276701','0','0','6','86']
  H_HDV['105_NOx_-4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.7143989726256','0.050050534280726','195.41071793578','0.549995551022136','0.402003527320564','6','86']
  H_HDV['105_PM_-4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.720650619983527','-0.000760495980827','-0.052491143699975','0.0835700010087','0','6','86']
  H_HDV['105_FC_-4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','420.189928453892','0.04844473934018','15156.0920381309','0.7320138710005','8.25788172670603','6','86']
  H_HDV['105_CO_-4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.54489608814224','0.043052888182149','126.43994449321','0.721238759265356','0.061966648352267','6','86']
  H_HDV['105_THC_-4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.82980213778045','0.050718806566658','6.41949064696877','0.394698876209488','0.089409193602202','6','86']
  H_HDV['105_NOx_-4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2348043165635','0.038756903051853','757.491610894428','0.779996360730726','0.022199971091594','6','86']
  H_HDV['105_PM_-4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','50','PM','y=((a*(b**x))*(x**c))','1.22468174013098','0.981619166064743','-0.421275939595552','0','0','6','86']
  H_HDV['105_FC_-4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','380.860371595306','0.037522710523204','47600.9641252272','0.923110689430416','-3.78961129530622','6','86']
  H_HDV['105_CO_-4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.82279175566506','0.035243686601475','457.669018067598','0.912012033071438','-0.097123395695794','6','86']
  H_HDV['105_THC_-4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72934268531488','0.049765433868334','6.86310048433219','0.406322054496672','0.087098889589936','6','86']
  H_HDV['105_NOx_-4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.0602919890423','0.032296143555119','1874.64455076402','0.93832122061358','-0.372409136242022','6','86']
  H_HDV['105_PM_-4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.560790656739231','0.034767169259455','26.4520050219324','0.874119254259413','0.010901404532958','6','86']
  H_HDV['105_FC_-4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','388.107655166486','0.030468829813164','120352.28380976','1.0810205459472','-18.890803799661','6','86']
  H_HDV['105_CO_-6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.944204598409','0.068363282818728','26.9159815509751','0.557594704445526','0.138617462928776','6','72']
  H_HDV['105_THC_-6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.14843572136723','0.061523331223991','8.97640294561036','0.505958151899499','0.099990019644377','6','72']
  H_HDV['105_NOx_-6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.9472720735896','0.064809730340469','410.106399253797','0.704703412416567','0.311603962358152','6','72']
  H_HDV['105_PM_-6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017929335823093','0.549657322362816','4.65184400018412','1.87708596857122','-0.006634670680104','6','72']
  H_HDV['105_FC_-6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','378.431390071991','0.060385916575691','21657.959416732','0.809918625915803','4.92363992968083','6','72']
  H_HDV['105_CO_-6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.05372359145773','0.056076604572446','123.307830121431','0.753689011629059','0.099643916187222','6','72']
  H_HDV['105_THC_-6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.90523797492791','0.05955902517095','9.33358463993514','0.498318406369768','0.097250673411381','6','72']
  H_HDV['105_NOx_-6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3154736847512','0.056505566337755','705.879477751646','0.787093195924175','0.250774125240267','6','72']
  H_HDV['105_PM_-6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.470280673888376','0.053566918933755','1.07302225003361','0.379653250372724','0.039108193809589','6','72']
  H_HDV['105_FC_-6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','303.648037094387','0.052602917728801','30280.8257958513','0.858657690987639','3.09946842203961','6','72']
  H_HDV['105_CO_-6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.91994414895262','0.049824912264903','209.588855329136','0.822628685803706','0.066517102889107','6','72']
  H_HDV['105_THC_-6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.80552375367979','0.05893059790078','7.99661812675811','0.463040995754234','0.097037972315865','6','72']
  H_HDV['105_NOx_-6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4095416669723','0.053006628605313','814.153713374161','0.807738168945591','0.237871189558706','6','72']
  H_HDV['105_PM_-6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.502519518637187','0.050391323332647','3.81622137837493','0.574463150614879','0.036776096356177','6','72']
  H_HDV['105_FC_-6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','290.582517334106','0.049743321141253','43435.9213721849','0.917460139083664','2.88049772871285','6','72']
  H_HDV['105_CO_2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.9426637442841','0.007472486795315','-0.686722196472487','0.075078980003003','0','6','86']
  H_HDV['105_THC_2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.20242728509752','3.51247771565818','4.67714695766378','1.88831841067274','-0.009620939196543','6','86']
  H_HDV['105_NOx_2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','49.9373505997878','-0.026387190033429','-3.22760855669782','0.097299262893825','0','6','86']
  H_HDV['105_PM_2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.06740509754274','0.096318735661481','-0.000797516987666','0','0','6','86']
  H_HDV['105_FC_2_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','0','FC','y=((a*(b**x))*(x**c))','2490.47505272122','1.00857793233802','-0.570241397776807','0','0','6','86']
  H_HDV['105_CO_2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','50','CO','y=((a*(b**x))*(x**c))','23.609867778837','0.993649783109947','-0.435381315570955','0','0','6','84']
  H_HDV['105_THC_2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.483144117258231','2.98727000858681','6.16431186926513','2.46062067759417','-0.01477018103346','6','84']
  H_HDV['105_NOx_2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','23.9055299634286','0.043608442545607','98.2707847036796','0.525091288227974','22.4920322744987','6','84']
  H_HDV['105_PM_2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.667375754493791','0.035011974669857','36.2059517734496','0.944192377316216','0.279783329187559','6','84']
  H_HDV['105_FC_2_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','50','FC','y=(c+(a*exp(b*x)))','742.916934146921','-0.065657468397492','696.62618051738','0','0','6','84']
  H_HDV['105_CO_2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.8045601516014','0.030982407756978','73.1685108059886','0.505586551369121','0.726296046219487','6','66']
  H_HDV['105_THC_2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.04686148517451','-0.005700060401234','-0.429765847337829','0.106958100253075','0','6','66']
  H_HDV['105_NOx_2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000095792922535','0.016277515669483','-1.0695707993365','58.3937634203444','0','6','66']
  H_HDV['105_PM_2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.15346623248349','0.020622366878843','37.8748046708908','0.834804044661082','0.053539041359943','6','66']
  H_HDV['105_FC_2_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','2','100','FC','y=(1/(a+(b*(x**c))))','0.000576844979945','0.000033525897819','0.648065437535062','0','0','6','66']
  H_HDV['105_CO_4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.064728007321139','0.008665058974424','-0.000061522118349','0','0','6','71']
  H_HDV['105_THC_4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.350437966699702','3.41700323492123','5.52189934315651','2.22234821574322','-0.012788085352802','6','71']
  H_HDV['105_NOx_4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.7768564717017','0.04784529503531','30.6254573007324','0.177543094215955','21.0885003862233','6','71']
  H_HDV['105_PM_4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.06383126181276','0.05751489057073','-0.00040145353071','0','0','6','71']
  H_HDV['105_FC_4_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','777.994491952229','0.08903157040708','6999.92896956576','0.722192896880355','587.987422785375','6','71']
  H_HDV['105_CO_4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4380091205223','0.031291219084923','22.4182869445899','0.340287839164187','1.05907557831378','6','59']
  H_HDV['105_THC_4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.758800730864688','2.75502369867884','7.42388880277863','3.01105530726147','-0.027391309811965','6','59']
  H_HDV['105_NOx_4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000274936884814','0.036609732416726','-1.75005403794119','67.6537184051717','0','6','59']
  H_HDV['105_PM_4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.850793815201661','0.034030311630221','18.0920132333813','0.808927216047814','0.339487361818405','6','59']
  H_HDV['105_FC_4_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','481.527624763422','0.040704468112522','8515.27138516616','0.640758594085507','1042.60556351624','6','59']
  H_HDV['105_CO_4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.474881269062','0.037384147447257','58.9308190552668','0.436803389324552','0.662305622700609','6','43']
  H_HDV['105_THC_4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000071203464076','0.007503358545412','-0.274290213441192','4.74208102335469','0','6','43']
  H_HDV['105_NOx_4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000950911560581','0.082468968220215','-2.67670834379725','87.9849025408387','0','6','43']
  H_HDV['105_PM_4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.53545549373608','0.01945560058781','7.57589536856328','0.510437436928073','-0.034459303417391','6','43']
  H_HDV['105_FC_4_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.016002628102167','1.27932881050139','-42.6987040807714','2240.17406032492','0','6','43']
  H_HDV['105_CO_6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','19.3359349922051','-0.057730237134144','-2.74002006231359','0.206724190948483','0','6','71']
  H_HDV['105_THC_6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','5.08507275523943','0.004185894690466','-0.369948996290672','0.079638679122803','0','6','71']
  H_HDV['105_NOx_6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','70.6087171200559','-0.103226239936148','-5.01292300615991','0.133243018901426','0','6','71']
  H_HDV['105_PM_6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.19886213852444','-0.003248546814085','-0.085422400672497','0.134763616153058','0','6','71']
  H_HDV['105_FC_6_0'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1545.69326952952','4.80202703682519','-54.5172904693659','0.055467098231727','0','6','71']
  H_HDV['105_CO_6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.384093802263','0.035363223095482','27.9517551190647','0.334750658138206','1.24732254258453','6','43']
  H_HDV['105_THC_6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.10252135577783','2.3877120399883','9.02243960429137','3.72445510979548','-0.045409060393986','6','43']
  H_HDV['105_NOx_6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000662419170129','0.068774341711601','-2.60971500338423','86.976284076287','0','6','43']
  H_HDV['105_PM_6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.84534045976031','0.032363498089501','3.22588225595802','0.411856561898847','0.46777981246958','6','43']
  H_HDV['105_FC_6_50'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','502.950125948518','0.019965777702871','7826.85044521176','0.582729534439382','1307.21915628715','6','43']
  H_HDV['105_CO_6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','100','CO','y=((a+(b*x))**((-1)/c))','0.225136294247744','0.008145627734132','0.436085784783035','0','0','6','30']
  H_HDV['105_THC_6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000185374753734','0.014209597486291','-0.38307396273169','5.44331384518355','0','6','30']
  H_HDV['105_NOx_6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003214491035035','0.209629910043824','-4.99415155120772','119.951922742311','0','6','30']
  H_HDV['105_PM_6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.69434063844195','-0.023908948166216','-0.563469649646524','0.268518346466856','0','6','30']
  H_HDV['105_FC_6_100'] = [' 105','EURO2','TT/AT >50-60t Euro-2','6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.096876168668614','5.65870360420594','-114.139031095384','3144.78125211273','0','6','30']
  H_HDV['106_CO_0_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.04987805989916','0.0558511630097','12.3086959391973','0.214662410173191','1.41447734955262','6','86']
  H_HDV['106_THC_0_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.154921597771359','3.66337673127582','3.75176925243373','1.66432821731722','-0.005853592501097','6','86']
  H_HDV['106_NOx_0_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.2411010548677','0.047729062407657','56.5033758034208','0.275264394647022','6.13214037484181','6','86']
  H_HDV['106_PM_0_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.064458984838562','1.21251813511903','4.42118920825711','1.86434819339297','-0.011779993210255','6','86']
  H_HDV['106_FC_0_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','624.050591539956','0.051693287156888','2381.04721243867','0.447112409269715','206.899045258296','6','86']
  H_HDV['106_CO_0_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.97515109897912','75.6928672942391','-0.880094183361447','0.445201618578204','0.044729735205376','6','86']
  H_HDV['106_THC_0_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.777811389732639','0.021481203369162','3.40036134967673','0.114695277918577','0.161666493002462','6','86']
  H_HDV['106_NOx_0_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','50','NOx','y=((a*(b**x))*(x**c))','74.9603423206476','0.999488048394013','-0.487596293350722','0','0','6','86']
  H_HDV['106_PM_0_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.700289806927959','0.047627025987168','1.00900661915716','0.216679431592801','0.151461994890879','6','86']
  H_HDV['106_FC_0_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','681.780268996068','0.033032737808505','48646.393889712','0.989620348944655','257.519697200793','6','86']
  H_HDV['106_CO_0_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.2633164254752','0.052166956789715','-0.752812217103055','0.044703983573709','0','6','86']
  H_HDV['106_THC_0_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.849555058761109','0.024174012760293','3.12459248597697','0.120256919619761','0.196782010738827','6','86']
  H_HDV['106_NOx_0_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.025931327608172','1.15213229798146','63.1176890968654','-0.347586964487523','0','6','86']
  H_HDV['106_PM_0_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.80577931207045','0.046721258444978','1.16321460711412','0.296815313697831','0.183564997689996','6','86']
  H_HDV['106_FC_0_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','0','100','FC','y=(c+(a*exp(b*x)))','866.759400184808','-0.023891292845228','252.714177891916','0','0','6','86']
  H_HDV['106_CO_-2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.7186320771547','0.017015974681661','-0.834476767646827','0.061175575103362','0','6','86']
  H_HDV['106_THC_-2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.084415348772368','2.61473427286368','5.878621134289','2.27841150340229','-0.014666757060377','6','86']
  H_HDV['106_NOx_-2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.511154537414','0.051662339193002','126.736422221498','0.448350375493639','1.7286564265195','6','86']
  H_HDV['106_PM_-2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.115631370968434','26.161588993387','-3.47221373580019','-0.369094230532043','0.099021353620104','6','86']
  H_HDV['106_FC_-2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','623.573345382744','0.049609184195615','4858.33062041337','0.574221938238956','60.4166661110792','6','86']
  H_HDV['106_CO_-2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','50','CO','y=(c+(a*exp(b*x)))','13.0088916832409','-0.065219121025133','0.921026390375036','0','0','6','86']
  H_HDV['106_THC_-2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.35694293135194','-0.003145867424981','-0.258490197764329','0.085945389376423','0','6','86']
  H_HDV['106_NOx_-2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.4461035022079','0.031356004105877','319.614607898511','0.5876721389922','0.394414099223951','6','86']
  H_HDV['106_PM_-2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.039195382309821','0.977062476872495','5.86493925105271','2.19967193290607','-0.012269528167632','6','86']
  H_HDV['106_FC_-2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-8.72062592283643','0.648348375081222','1533.59937394297','-0.456593920766367','0','6','86']
  H_HDV['106_CO_-2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1487718617984','0.05000349206558','61.6090021883923','0.620112185499923','0.744124052527036','6','86']
  H_HDV['106_THC_-2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.16633583860957','-0.003596532449933','-0.252384893657422','0.090163104342339','0','6','86']
  H_HDV['106_NOx_-2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','18.1644322130599','0.023529429943557','1045.0841453987','0.790439608355673','-0.965372055995637','6','86']
  H_HDV['106_PM_-2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','100','PM','y=exp((a+(b/x))+(c*log(x)))','3.25161458268145','-6.76545507414752','-1.26314821162716','0','0','6','86']
  H_HDV['106_FC_-2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','821.43039706596','-3.21218980033312','-33.6553465747171','0.060145776335518','0','6','86']
  H_HDV['106_CO_-4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.23203960165789','-10.9738948100986','-1.84039058925612','0','0','6','86']
  H_HDV['106_THC_-4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.30372255287346','-0.002364253492125','-0.265675304826497','0.087455435502573','0','6','86']
  H_HDV['106_NOx_-4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.2817297312486','0.059781061257484','234.154731912757','0.553632609396445','0.595840245806304','6','86']
  H_HDV['106_PM_-4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.037774699092617','0.958183211377204','6.87907488232115','2.59579591697697','-0.016834457500156','6','86']
  H_HDV['106_FC_-4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','507.466657213695','0.051346746978471','5805.51911126434','0.578710540557317','16.7235764714066','6','86']
  H_HDV['106_CO_-4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.0780317965452','-0.011457147931672','-0.75542450028018','0.075388755031414','0','6','86']
  H_HDV['106_THC_-4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.91002703479066','0.062063923324476','7.00327778709179','0.489889751349598','0.094839751637629','6','86']
  H_HDV['106_NOx_-4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.4609122050386','0.045898101746875','825.114714796008','0.761654597498794','0.356159607449215','6','86']
  H_HDV['106_PM_-4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.30489880906565','-6.88806255458003','-1.34862190329972','0','0','6','86']
  H_HDV['106_FC_-4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','434.997753122567','0.039818357077787','27990.5386623585','0.840358837779105','5.22495808142477','6','86']
  H_HDV['106_CO_-4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.51897728468826','0.043483514691997','198.454727549638','0.778667571289603','0.100273470294155','6','86']
  H_HDV['106_THC_-4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.64146731945066','0.059301980839855','14.6935306153054','0.605129036294191','0.089249978011679','6','86']
  H_HDV['106_NOx_-4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.6077726632754','0.037500763394406','1772.26193998423','0.894613641484565','0.111084136322749','6','86']
  H_HDV['106_PM_-4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','100','PM','y=(1/(a+(b*(x**c))))','0.991647743283025','0.029033192633476','1.39910762859146','0','0','6','86']
  H_HDV['106_FC_-4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','431.67952144986','0.032440089967456','83558.3116129921','1.02220194372608','-9.29792696292301','6','86']
  H_HDV['106_CO_-6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','0','CO','y=(c+(a*exp(((-1)*b)*x)))','10.6401014029382','0.074815637465936','0.287356379980048','0','0','6','72']
  H_HDV['106_THC_-6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.16712027104196','0.070781982111827','3.99074796302818','0.438717437240082','0.096564431428176','6','72']
  H_HDV['106_NOx_-6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.3346484224358','0.07111245288777','370.490614036172','0.63892270886945','0.434976522969975','6','72']
  H_HDV['106_PM_-6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.031084973149217','0.858848656257451','6.87347809067024','2.61806992004215','-0.020511754210525','6','72']
  H_HDV['106_FC_-6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','478.118551380686','0.063709392720327','13586.8560153081','0.752151733578491','14.6741751125908','6','72']
  H_HDV['106_CO_-6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','8.01147145639659','-0.065127443624324','0.243041390643564','0','0','6','72']
  H_HDV['106_THC_-6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.54742483076054','0.064189418560364','14.6772929723244','0.606717218583415','0.087872253748518','6','72']
  H_HDV['106_NOx_-6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.8940372807552','0.058857256329634','683.821943472858','0.73802779755876','0.374031255091504','6','72']
  H_HDV['106_PM_-6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.020597914547318','0.787706705468931','5.14089437751723','1.95684870330907','-0.008491475276264','6','72']
  H_HDV['106_FC_-6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','364.856526125775','0.054320249367773','21853.5571692272','0.815499420079469','11.9373756523499','6','72']
  H_HDV['106_CO_-6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','100','CO','y=((a*(b**x))*(x**c))','12.7646813928038','0.967381613720051','-0.352698011997148','0','0','6','72']
  H_HDV['106_THC_-6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.34510377199956','0.062154175805831','13.5416382501463','0.584499225224934','0.086281762988518','6','72']
  H_HDV['106_NOx_-6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.82817473317679','0.053665406074996','863.76042682634','0.781073453221809','0.360967650438234','6','72']
  H_HDV['106_PM_-6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.71752479657367','0.055822481558013','0.393766895347373','0.224852251653049','0.063732987962328','6','72']
  H_HDV['106_FC_-6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','343.079669504224','0.050916846243546','33345.4514704466','0.880049101657799','11.3409253476266','6','72']
  H_HDV['106_CO_2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.4182954863058','0.008793664212221','-1.00343852168994','0.072278949271405','0','6','86']
  H_HDV['106_THC_2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.201109775928984','3.22620004798208','5.05793195374926','2.02968008649727','-0.010141026544535','6','86']
  H_HDV['106_NOx_2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','57.0884225573888','-0.04006860218011','-5.23500136604812','0.121993383460105','0','6','86']
  H_HDV['106_PM_2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.142655988757028','0.853672103605923','7.01141980266025','2.67452106148117','-0.021034811330596','6','86']
  H_HDV['106_FC_2_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1335.94866556242','0.514277870483739','-75.0064143041884','0.078238155090243','0','6','86']
  H_HDV['106_CO_2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.041867245868627','0.006477547443365','-0.000015321120952','0','0','6','84']
  H_HDV['106_THC_2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.379643482994206','2.75534929316087','6.2526698870808','2.49140655658885','-0.015661671808629','6','84']
  H_HDV['106_NOx_2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','50','NOx','y=((a*(b**x))*(x**c))','95.2866577850935','1.00391427708826','-0.456106671595847','0','0','6','84']
  H_HDV['106_PM_2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.3332542422765','-0.000715423841656','-0.079832406270068','0.077547549420908','0','6','84']
  H_HDV['106_FC_2_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','50','FC','y=(c+(a*exp(((-1)*b)*x)))','789.599034715948','0.063153118146461','693.382452075601','0','0','6','84']
  H_HDV['106_CO_2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','100','CO','y=((a*(b**x))*(x**c))','35.119253284925','0.984983078283555','-0.418960296450744','0','0','6','67']
  H_HDV['106_THC_2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.467889650014723','2.63853693068016','6.0881710612407','2.44234308338219','-0.016639129385905','6','67']
  H_HDV['106_NOx_2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','4.30299320971001','0.643913401299972','-0.268774880337685','0','0','6','67']
  H_HDV['106_PM_2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','100','PM','y=((a*(b**x))*(x**c))','2.22936374461312','0.995165857943878','-0.393385921413869','0','0','6','67']
  H_HDV['106_FC_2_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','2','100','FC','y=(1/(a+(b*(x**c))))','0.000534985970666','0.000046902286151','0.586410344770998','0','0','6','67']
  H_HDV['106_CO_4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.0794333497888','-0.022653384428794','-1.21327841513907','0.096339058095901','0','6','71']
  H_HDV['106_THC_4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.297562322142782','3.02430943452546','5.90748248531726','2.34597359519556','-0.014767807225829','6','71']
  H_HDV['106_NOx_4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.84759642570045','0.020608897936026','49.3745675777735','0.162516920964461','13.7647846889466','6','71']
  H_HDV['106_PM_4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','0','PM','y=(c+(a*exp(b*x)))','0.991821430844536','-0.066775713530784','0.219072962882541','0','0','6','71']
  H_HDV['106_FC_4_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','892.431552863167','0.090127536691724','16542.1979683397','0.947477686500459','598.397883461279','6','71']
  H_HDV['106_CO_4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','50','CO','y=((a*(b**x))*(x**c))','29.6414214535158','0.985907906148688','-0.396730338358428','0','0','6','60']
  H_HDV['106_THC_4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.588729290542605','2.46666178335483','7.60846406992264','3.0501543321124','-0.027798295583344','6','60']
  H_HDV['106_NOx_4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','84.634143936115','-0.133954007334308','-7.41504717682259','0.147526714062459','0','6','60']
  H_HDV['106_PM_4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006564785376','0.00097590949924','-0.049122083635424','1.25914259636078','0','6','60']
  H_HDV['106_FC_4_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','551.20554592446','0.045929760815979','8066.18694110423','0.652719111014282','1050.69025669111','6','60']
  H_HDV['106_CO_4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','16.5384447654901','0.040290143919888','39.1334443047381','0.386657522716711','0.559484369952001','6','43']
  H_HDV['106_THC_4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000062371061604','0.006692578389931','-0.24745999454132','4.15882574494017','0','6','43']
  H_HDV['106_NOx_4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001079186330121','0.101533474083292','-3.45490783803535','85.0318356178361','0','6','43']
  H_HDV['106_PM_4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000017675367396','0.001801574287132','-0.067613752726536','1.55220033862153','0','6','43']
  H_HDV['106_FC_4_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','765.94241937412','0.044582018636049','-0.000000377923973','-0.442778072373682','1469.94624074194','6','43']
  H_HDV['106_CO_6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','17.5592253762124','-0.046193207598635','-1.56288249185688','0.123597284718819','0','6','71']
  H_HDV['106_THC_6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.36513465878281','2.78868066061423','7.33830128785071','2.93198604408518','-0.033476054972395','6','71']
  H_HDV['106_NOx_6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.88645468974103','0.018684850386406','47.9457730955183','0.14661962986758','18.002007431559','6','71']
  H_HDV['106_PM_6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.297107241269695','11.7469633092615','-3.16218429156211','-0.525765324192943','0.117970390411578','6','71']
  H_HDV['106_FC_6_0'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','0','FC','y=((a*(b**x))*(x**c))','2624.79014868176','1.00759857720997','-0.401066774848922','0','0','6','71']
  H_HDV['106_CO_6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','50','CO','y=((a*(b**x))*(x**c))','30.9591499190942','0.979772855871219','-0.343092155979665','0','0','6','44']
  H_HDV['106_THC_6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.920541341083151','10.8318723050468','-1.64332756057384','-0.644557927138876','0.168045562155921','6','44']
  H_HDV['106_NOx_6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001083020918823','0.108338062318899','-3.75545673183222','85.3541659547993','0','6','44']
  H_HDV['106_PM_6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','50','PM','y=(c+(a*exp(((-1)*b)*x)))','0.96865630197384','0.088656561630213','0.555473646667848','0','0','6','44']
  H_HDV['106_FC_6_50'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','538.845926714729','0.036918222440073','11113.0518136346','0.674653380955112','1399.31977708777','6','44']
  H_HDV['106_CO_6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','100','CO','y=((a+(b*x))**((-1)/c))','0.387147796519975','0.007936369143149','0.2845218134944','0','0','6','31']
  H_HDV['106_THC_6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','100','THC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000153971536959','0.012176327900166','-0.337998418185682','4.71401465200194','0','6','31']
  H_HDV['106_NOx_6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002841289979243','0.20340252067324','-5.411600820031','110.981070680834','0','6','31']
  H_HDV['106_PM_6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.41579267400373','-0.008416843031503','-0.314823918380301','0.242427129192042','0','6','31']
  H_HDV['106_FC_6_100'] = [' 106','EURO3','TT/AT >50-60t Euro-3','6','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.094382635666938','5.73358142434484','-120.197323412286','3217.65681262947','0','6','31']
  H_HDV['107_CO_0_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.0538811213271','1.0171627833494','3.75796950197618','1.66796431499687','-0.007968210689047','6','86']
  H_HDV['107_THC_0_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004222848758115','0.205888851201089','3.86336640753186','1.72475694006306','-0.010948651196127','6','86']
  H_HDV['107_NOx_0_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.93431930563914','0.047308887463101','46.0515396827008','0.382209972618594','3.70873800897659','6','86']
  H_HDV['107_PM_0_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.118563126266138','0.04549406764194','0.302883065313422','0.168395565993985','0.021155111987256','6','86']
  H_HDV['107_FC_0_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','599.731184014832','0.052625571419121','3604.8739106315','0.515573015858802','194.748940445705','6','86']
  H_HDV['107_CO_0_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.046531916364807','1.00503464481765','3.50632139353062','1.53406763489754','-0.009088221990705','6','86']
  H_HDV['107_THC_0_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005213006289056','0.199622362336694','3.77280861005869','1.7153645963338','-0.010015636942538','6','86']
  H_HDV['107_NOx_0_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.9537486221824','0.031261707937746','166.476461411088','0.639225306378139','4.49881443354827','6','86']
  H_HDV['107_PM_0_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.131905518345786','0.046891023271684','0.283931439996228','0.182181622350841','0.022933669689632','6','86']
  H_HDV['107_FC_0_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','637.239051582394','0.032577554883698','35297.3800811072','0.928226434192903','236.918054601458','6','86']
  H_HDV['107_CO_0_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.450604728257604','0.038703970600054','0.702424997499177','0.143356285159152','0.106280242927705','6','86']
  H_HDV['107_THC_0_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003261883987604','0.197553013842532','3.68204961379945','1.69448175245623','-0.013340739728302','6','86']
  H_HDV['107_NOx_0_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.7473779831618','0.019525545019708','178.436725611917','0.726707881818754','3.21387728701275','6','86']
  H_HDV['107_PM_0_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.142920047516111','0.04845100566718','0.286270879290616','0.198612784327213','0.024243999812294','6','86']
  H_HDV['107_FC_0_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','0','100','FC','y=(c+(a*exp(b*x)))','812.662025035026','-0.023413167353453','227.751278105341','0','0','6','86']
  H_HDV['107_CO_-2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029279785432507','0.671377865652167','7.24384851542565','2.7188774800089','-0.021439368532019','6','86']
  H_HDV['107_THC_-2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001175524909474','0.147246329225122','6.18130531290605','2.416958777604','-0.023291670940372','6','86']
  H_HDV['107_NOx_-2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.22456317658034','0.045687083945428','107.309286642755','0.529387720975705','0.879067797221513','6','86']
  H_HDV['107_PM_-2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008596638309453','0.274908499473859','3.99489297549667','1.71278373740003','0.003981781244728','6','86']
  H_HDV['107_FC_-2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','589.160268275377','0.050319321854483','6777.65049295577','0.62399547420034','53.7708920855467','6','86']
  H_HDV['107_CO_-2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.906993815437255','-0.000952703292306','-0.063717028783824','0.079024865441281','0','6','86']
  H_HDV['107_THC_-2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000026555566811','0.155397455857983','4.39372376802095','1.7949278471698','-0.006166699006304','6','86']
  H_HDV['107_NOx_-2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.55253380621163','0.026247500698277','334.904544465684','0.711273454399132','-0.225153122282938','6','86']
  H_HDV['107_PM_-2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.03264265507283','0.465214010201376','0.009362269612651','0','0','6','86']
  H_HDV['107_FC_-2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-3.14064464313005','0.831263258814989','1536.81243021278','-0.506332354110484','0','6','86']
  H_HDV['107_CO_-2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.86490027347805','-0.001210460884694','-0.060522107547323','0.080508554266905','0','6','86']
  H_HDV['107_THC_-2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.034645758312134','0.017393375160501','0.153764507352519','0.100515798792207','-0.002375996260238','6','86']
  H_HDV['107_NOx_-2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.5508058275832','0.019017633214888','1339.01824246209','0.951665590818305','-1.56449503626609','6','86']
  H_HDV['107_PM_-2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','100','PM','y=((a*(x**b))+(c*(x**d)))','-0.304070549161281','-0.373248783197305','1.06496166254448','-0.62847036656808','0','6','86']
  H_HDV['107_FC_-2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','766.858380606314','-3.05857477947216','-33.1497685948397','0.063421093420313','0','6','86']
  H_HDV['107_CO_-4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.876049468481483','-0.000577440735041','-0.066638367484979','0.081351523193783','0','6','86']
  H_HDV['107_THC_-4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.185901755863481','-0.000132897260083','-0.01497110453465','0.087134738437152','0','6','86']
  H_HDV['107_NOx_-4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.57609877317033','0.050323949194747','156.632826728863','0.588325209018044','0.127530077415333','6','86']
  H_HDV['107_PM_-4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.32316791564194','0.392761786317247','0.018318404588774','0','0','6','86']
  H_HDV['107_FC_-4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','477.26629103136','0.052684228258865','8629.23436241012','0.643600475766289','9.67781570889551','6','86']
  H_HDV['107_CO_-4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008217875344075','4.38786849171266','-1.22319183138869','0.405973912160991','0.040038166852199','6','86']
  H_HDV['107_THC_-4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.108000168179015','0.061496829655771','0.413429437549804','0.490388037119599','0.004421034966165','6','86']
  H_HDV['107_NOx_-4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.60174673300217','0.039010320454672','411.699700122961','0.755396269004158','-0.061099898659959','6','86']
  H_HDV['107_PM_-4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.150301981429263','0.059124187051234','0.899834020125015','0.449545800317972','0.00529131246439','6','86']
  H_HDV['107_FC_-4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','390.651269913593','0.040239475345058','34833.3788498888','0.872818177281009','-1.26562070934218','6','86']
  H_HDV['107_CO_-4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.434551528525152','0.052041020249097','14.6905037370475','0.829113625569061','0.01026883267332','6','86']
  H_HDV['107_THC_-4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.093298644719606','0.058500523433605','0.88362462418297','0.605155353492213','0.004067966571865','6','86']
  H_HDV['107_NOx_-4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.61179683464316','0.032088359852265','770.029101827301','0.86435012151448','-0.280355168753982','6','86']
  H_HDV['107_PM_-4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.137865886084156','0.056780997409457','1.53850203048725','0.541057222737281','0.004907629388604','6','86']
  H_HDV['107_FC_-4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','376.574795470059','0.032056529208626','84971.3391809537','1.02478424036103','-15.079261664974','6','86']
  H_HDV['107_CO_-6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','0','CO','y=(c+(a*exp(b*x)))','0.679402548273401','-0.074522434127346','0.013363198205621','0','0','6','72']
  H_HDV['107_THC_-6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.127258229064312','0.070938760810494','0.349504150478083','0.532970466085675','0.004416980757507','6','72']
  H_HDV['107_NOx_-6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.14660993895315','0.064613097042944','248.633976279771','0.69059408118967','0.060680503470038','6','72']
  H_HDV['107_PM_-6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.181098012207446','0.070975739495815','0.758420658461062','0.459713214060041','0.005406753886972','6','72']
  H_HDV['107_FC_-6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','430.257029298971','0.064204190622897','19377.3533513231','0.802486755028263','4.9158845249812','6','72']
  H_HDV['107_CO_-6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006260480904761','2.45774339845863','-0.874957186642284','0.38577180539547','0.046373323644947','6','72']
  H_HDV['107_THC_-6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.091459924496016','0.064001462631077','1.17090005944476','0.673748663925925','0.003897874645766','6','72']
  H_HDV['107_NOx_-6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.61524778792057','0.05674347982231','404.262548375706','0.768488573849034','0.033958881953916','6','72']
  H_HDV['107_PM_-6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.140823247018337','0.065438734678678','1.68971914688326','0.575660508204414','0.004846690534351','6','72']
  H_HDV['107_FC_-6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','315.808797767587','0.054711608448578','35949.3950517703','0.891517992513644','2.66158984809524','6','72']
  H_HDV['107_CO_-6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.343235336910123','0.056488966529466','19.186241443329','0.89627019363922','0.007174604861184','6','72']
  H_HDV['107_THC_-6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.08044405554836','0.062029457817803','1.09389694095096','0.650519341311092','0.003808423210093','6','72']
  H_HDV['107_NOx_-6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.25551803813217','0.053265670287372','507.328676083448','0.807848973622369','0.028278095615545','6','72']
  H_HDV['107_PM_-6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.126848158919116','0.06310001855443','1.78960651131263','0.580198098876446','0.004661536934847','6','72']
  H_HDV['107_FC_-6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','285.45348481257','0.050583455996436','40135.0287263152','0.909405425373974','2.06944321633894','6','72']
  H_HDV['107_CO_2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.156077670770847','0.702018950147929','7.11460731690986','2.78791807984391','-0.017132818870603','6','86']
  H_HDV['107_THC_2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007388456843383','0.1832629426413','4.91504997116028','1.99440758353129','-0.009624948957431','6','86']
  H_HDV['107_NOx_2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','15.6363573111512','0.089502879858043','58.2743064976785','0.592889754909853','7.58530177282787','6','86']
  H_HDV['107_PM_2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.073829470805897','0.028671516575176','0.312911079337924','0.132028263598929','0.024111516474423','6','86']
  H_HDV['107_FC_2_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','855.156954191065','0.082473897333281','72154.6831162798','1.18144913392934','379.938960062891','6','86']
  H_HDV['107_CO_2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.57162893474272','0.008828209274674','0.979641280805135','0.129157121793549','-0.133533783429226','6','85']
  H_HDV['107_THC_2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014374254775578','0.157895343698268','6.12090221239088','2.46123267301596','-0.015666539443777','6','85']
  H_HDV['107_NOx_2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.5657237919522','0.033525987013807','51.7089350694756','0.411599634672227','10.594408597867','6','85']
  H_HDV['107_PM_2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.103892864527121','0.028358765746213','0.31071332223976','0.148607600847959','0.021431726218258','6','85']
  H_HDV['107_FC_2_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','50','FC','y=(c+(a*exp(b*x)))','768.022405024777','-0.065956524823273','655.589258355736','0','0','6','85']
  H_HDV['107_CO_2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.586619619584558','0.031421344292679','0.942659073195966','0.159830009503163','0.113565237289867','6','68']
  H_HDV['107_THC_2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016972111724083','0.156047215957343','5.83984612134969','2.38538674232826','-0.015858960298911','6','68']
  H_HDV['107_NOx_2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.5166232905929','0.028477379028665','270.974495651685','0.736232129928324','13.2068531470117','6','68']
  H_HDV['107_PM_2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.133953039263752','0.039014650483597','0.29210081325991','0.155814852512033','0.028243962651487','6','68']
  H_HDV['107_FC_2_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002314754375236','0.388347661511477','-25.5911373763087','1545.46425298083','0','6','68']
  H_HDV['107_CO_4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.30855095618543','-0.001050767878869','-0.108263500021104','0.10625064846833','0','6','71']
  H_HDV['107_THC_4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009412703632064','0.175354763594979','5.47595173414318','2.19919300083183','-0.012771061671356','6','71']
  H_HDV['107_NOx_4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.04893478880215','0.047072885288959','24.7314627910205','0.216533609655768','10.6003891359897','6','71']
  H_HDV['107_PM_4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089864859348816','0.020548594663842','0.313360125893009','0.138827439580795','0.016264324275648','6','71']
  H_HDV['107_FC_4_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','798.023515602824','0.087147036007897','4071.63664291253','0.635891841348503','561.565253236951','6','71']
  H_HDV['107_CO_4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.630738829618279','0.021154936063147','0.87504727857419','0.132589929378755','0.024469450992992','6','61']
  H_HDV['107_THC_4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.021039291667041','0.147404562864198','6.70806189428533','2.72774644645492','-0.021733851059886','6','61']
  H_HDV['107_NOx_4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.4799016672338','0.019380272051971','33.9118926505017','0.287450428484499','13.4814632020381','6','61']
  H_HDV['107_PM_4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.137034140297618','0.046154747228584','0.269644665998224','0.143496602106567','0.037958328269898','6','61']
  H_HDV['107_FC_4_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','542.992400516465','0.054300060208508','8951.6815370983','0.683415711199918','1013.46937488347','6','61']
  H_HDV['107_CO_4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.89419577547094','-0.003604957804302','-0.15543107706826','0.10335968474112','0','6','44']
  H_HDV['107_THC_4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029591267939808','0.133529178619328','7.42907461204731','3.06288417349386','-0.031363040675426','6','44']
  H_HDV['107_NOx_4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000399540844542','0.037530934977798','-1.36485587597915','45.6929088202835','0','6','44']
  H_HDV['107_PM_4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000006120937588','0.000679851098595','-0.025965306065962','0.411375598144429','0','6','44']
  H_HDV['107_FC_4_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.01612263327599','1.40037956931195','-47.2699003835904','2183.03963467164','0','6','44']
  H_HDV['107_CO_6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.86327878322708','-0.005167607644911','-0.288836406131112','0.216735907163211','0','6','71']
  H_HDV['107_THC_6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008904712261923','0.161196568229066','7.06708739243386','2.84550820250481','-0.036234976034877','6','71']
  H_HDV['107_NOx_6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.7038417506784','-0.057119985318896','-5.84158815381172','0.203632479094513','0','6','71']
  H_HDV['107_PM_6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.438199861777484','-0.00089297210136','-0.047692340497627','0.13751974152741','0','6','71']
  H_HDV['107_FC_6_0'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','2587.85080110247','1.00814100157559','-0.419733736690518','0','0','6','71']
  H_HDV['107_CO_6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','2.48349324919644','-3.60031685774917','-0.974296489827567','0','0','6','44']
  H_HDV['107_THC_6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.035713318143513','0.592778737107416','-1.24220211179565','-0.391766777412581','0.147415824837762','6','44']
  H_HDV['107_NOx_6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00049460023839','0.047849037606906','-1.6478228621799','45.9597962108818','0','6','44']
  H_HDV['107_PM_6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.441955283889966','0.000416091395127','-0.034630371703962','0.086321756941154','0','6','44']
  H_HDV['107_FC_6_50'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','524.075119874838','0.049203794148659','11904.4560028319','0.712116234465224','1366.31069503946','6','44']
  H_HDV['107_CO_6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','100','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000086926610483','0.006696953275567','-0.187116285644931','2.35627463238335','0','6','31']
  H_HDV['107_THC_6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.048551004439727','0.657068993297169','-1.89209527280378','-0.794411023784275','0.202721816393612','6','31']
  H_HDV['107_NOx_6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002969162906185','0.189608391289404','-4.13827220179267','69.4386151312978','0','6','31']
  H_HDV['107_PM_6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000406619573856','-0.023761347680918','0.436919312327655','0','0','6','31']
  H_HDV['107_FC_6_100'] = [' 107','EURO4','TT/AT >50-60t Euro-4','6','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3329.3219819688','-14.0837418991388','-293.501188659447','0.339232338696','0','6','31']
  H_HDV['108_CO_0_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053502110996327','1.03610116341948','3.77802955146984','1.67326757079225','-0.008105941653315','6','86']
  H_HDV['108_THC_0_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00496288601085','0.209097797821364','3.8646125424833','1.72232955445322','-0.010086660589675','6','86']
  H_HDV['108_NOx_0_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.48335147959663','0.04708023998925','30.6282133989455','0.3644789137545','2.18567961443938','6','86']
  H_HDV['108_PM_0_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.121072570045163','0.045465367935232','0.309120142205499','0.168341966254594','0.021384465950153','6','86']
  H_HDV['108_FC_0_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','620.34385902348','0.05255273402674','3498.94341188317','0.502719952523532','198.173579544068','6','86']
  H_HDV['108_CO_0_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.04385213332707','1.03563994644874','3.47221544098367','1.52636691366724','-0.009396807654823','6','86']
  H_HDV['108_THC_0_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005226415362831','0.203658610903677','3.77945587331479','1.71736312239623','-0.010137671716675','6','86']
  H_HDV['108_NOx_0_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.88480528176573','0.034499515333227','85.2125761647669','0.576473614947534','2.70877464242495','6','86']
  H_HDV['108_PM_0_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.133581373366185','0.046756786802097','0.289913827135708','0.181216913435195','0.023279447097542','6','86']
  H_HDV['108_FC_0_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','656.984833727167','0.032744224374655','29112.1939171492','0.887235553127938','240.952861069194','6','86']
  H_HDV['108_CO_0_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.465288944562581','0.039414500820762','0.708357775918368','0.144456076761804','0.108975911668761','6','86']
  H_HDV['108_THC_0_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003286384541128','0.20067781121142','3.71407658812371','1.70276394051045','-0.013408265125397','6','86']
  H_HDV['108_NOx_0_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.61403850901534','0.024478665910284','378.98818317277','0.841358815361853','2.39194878575234','6','86']
  H_HDV['108_PM_0_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.145156380369177','0.048317982249072','0.290225200039271','0.196941231563212','0.024559610667947','6','86']
  H_HDV['108_FC_0_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','0','100','FC','y=(c+(a*exp(((-1)*b)*x)))','834.546075162826','0.02401592019434','237.138686003659','0','0','6','86']
  H_HDV['108_CO_-2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.02918968737546','0.689442309146908','7.19892754743841','2.70501409441539','-0.021365543212709','6','86']
  H_HDV['108_THC_-2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00097284842597','0.151380704013277','6.12740970436031','2.39889997921172','-0.023150081078879','6','86']
  H_HDV['108_NOx_-2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.69069529968104','0.046080869699273','66.5876439649573','0.500290447797814','0.519187084475044','6','86']
  H_HDV['108_PM_-2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008734764957929','0.282912531674478','3.95680133700133','1.70195287390998','0.004200571460338','6','86']
  H_HDV['108_FC_-2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','611.292373606292','0.050467113785987','6787.99373476299','0.617063263784272','55.0339297035745','6','86']
  H_HDV['108_CO_-2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.927497316537675','-0.000963496760528','-0.065248888312289','0.07903993962452','0','6','86']
  H_HDV['108_THC_-2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000054326395964','0.158168754000977','4.43325995062684','1.80695338416594','-0.006424326881113','6','86']
  H_HDV['108_NOx_-2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.83776804737858','0.027850354861229','173.13108505004','0.654955494014065','-0.0683885105398','6','86']
  H_HDV['108_PM_-2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.99617817838801','0.453827252760502','0.009262909448837','0','0','6','86']
  H_HDV['108_FC_-2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','50','FC','y=((a*(x**b))+(c*(x**d)))','-3.39586198436524','0.818335427396063','1608.72596696973','-0.511848467503888','0','6','86']
  H_HDV['108_CO_-2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.888163000784835','-0.001259142279309','-0.063135197474343','0.081893304460166','0','6','86']
  H_HDV['108_THC_-2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.035088343590303','0.017214192480546','0.157154789128989','0.099965640876173','-0.002492192083644','6','86']
  H_HDV['108_NOx_-2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.82878943979283','0.021837276009834','528.945696220858','0.849959636031289','-0.625873634186455','6','86']
  H_HDV['108_PM_-2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','100','PM','y=((a*(x**b))+(c*(x**d)))','-0.623955960638','-0.42818725710496','1.3952740943922','-0.591890271618437','0','6','86']
  H_HDV['108_FC_-2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','795.204607055982','-3.11448681966421','-34.8547242734939','0.063818756200797','0','6','86']
  H_HDV['108_CO_-4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','1.45638530240004','0.008079988424687','0.007339681087873','0','0','6','86']
  H_HDV['108_THC_-4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.189990741628557','-0.000135717265454','-0.015282382842563','0.087023405874733','0','6','86']
  H_HDV['108_NOx_-4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.77315498660261','0.05135742547434','98.6477041813948','0.56570665919696','0.079811084579682','6','86']
  H_HDV['108_PM_-4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','0','PM','y=(1/(((c*(x**2))+(b*x))+a))','2.29892079855641','0.379838068557333','0.01804034581115','0','0','6','86']
  H_HDV['108_FC_-4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','497.405836889493','0.052992561739792','8744.94150951842','0.640455688185562','10.0968312145448','6','86']
  H_HDV['108_CO_-4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.692639990503083','-0.000542322738393','-0.049623663669778','0.077372634345954','0','6','86']
  H_HDV['108_THC_-4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.11058246098717','0.061533057344376','0.402551749938501','0.482015137925318','0.004509897753583','6','86']
  H_HDV['108_NOx_-4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.15603744117013','0.040595033522049','222.326336372278','0.704361561115482','-0.02322469909295','6','86']
  H_HDV['108_PM_-4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.153685574598716','0.059216194360926','0.896607086797961','0.445176305124377','0.005403094484777','6','86']
  H_HDV['108_FC_-4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','405.641858334239','0.040639173815933','34145.0092208316','0.862861831708696','-0.887780077937315','6','86']
  H_HDV['108_CO_-4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.443403699859761','0.052053108557278','12.6214450515714','0.800566124613697','0.010390138089976','6','86']
  H_HDV['108_THC_-4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.095648023659491','0.058606776586877','0.881049720317993','0.601755084647685','0.004151583052235','6','86']
  H_HDV['108_NOx_-4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.1955210207393','0.034983353902845','424.565671592128','0.818738425704821','-0.11745498540044','6','86']
  H_HDV['108_PM_-4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.141019718349991','0.056894155842699','1.53839940443539','0.53777136174143','0.005011326139019','6','86']
  H_HDV['108_FC_-4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','389.485150449511','0.032478702937429','83645.4216669033','1.01605379202936','-14.6401807616881','6','86']
  H_HDV['108_CO_-6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','0','CO','y=(c+(a*exp(b*x)))','0.698599906714416','-0.074622145235182','0.01364135838615','0','0','6','72']
  H_HDV['108_THC_-6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.130765629255341','0.07108457730521','0.333209952159205','0.523878214345636','0.004516165496715','6','72']
  H_HDV['108_NOx_-6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.52990072338258','0.064945876588241','143.51113345747','0.650060828783425','0.035177271777797','6','72']
  H_HDV['108_PM_-6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.185590480914553','0.071028628662471','0.74757154907357','0.454239788808505','0.005513962290272','6','72']
  H_HDV['108_FC_-6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','449.158443727585','0.064397894687396','19418.6196618036','0.796437420401439','5.1068794534281','6','72']
  H_HDV['108_CO_-6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006312632572699','2.58824936497475','-0.898416144743691','0.38908979664523','0.046206058520016','6','72']
  H_HDV['108_THC_-6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.093742188708362','0.064086261824086','1.16863525186029','0.669508790379728','0.003977370214092','6','72']
  H_HDV['108_NOx_-6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.64815365868213','0.058144285223461','230.649166575389','0.729050537026444','0.021922872426229','6','72']
  H_HDV['108_PM_-6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.143983600819653','0.065476114462422','1.69578631949982','0.572175908581123','0.004943008441246','6','72']
  H_HDV['108_FC_-6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','324.956138704749','0.054506669455238','31280.9986001776','0.862226764084773','2.63355940615507','6','72']
  H_HDV['108_CO_-6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.351762310205803','0.056650882164006','19.4958256308668','0.895728217223619','0.007385477676401','6','72']
  H_HDV['108_THC_-6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.082296036062558','0.062045237829548','1.1102095124315','0.648741745925692','0.003877724517935','6','72']
  H_HDV['108_NOx_-6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.41817524600125','0.054795042266782','280.246984644205','0.763064303470401','0.017903375657262','6','72']
  H_HDV['108_PM_-6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.129722498005323','0.063164921005706','1.817006041411','0.579105752834838','0.004755135957524','6','72']
  H_HDV['108_FC_-6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','296.089986148138','0.050733871577897','40653.7025433726','0.904952076145721','2.12982656912696','6','72']
  H_HDV['108_CO_2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.155269562881792','0.722312048331936','7.01942052203734','2.7529789116971','-0.016877060746311','6','86']
  H_HDV['108_THC_2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007448182704022','0.187255516581816','4.8965406592765','1.98700640287684','-0.009541257729976','6','86']
  H_HDV['108_NOx_2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.0847828301417','0.080629085772226','20.7878291972701','0.366745285835778','4.47291627621534','6','86']
  H_HDV['108_PM_2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.075000134886468','0.028454395748393','0.319900622195','0.131967758011498','0.024219111192448','6','86']
  H_HDV['108_FC_2_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','892.661692971157','0.082361035565242','109575.140446229','1.24963705808303','386.406011682008','6','86']
  H_HDV['108_CO_2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.544202869781156','0.009895067815194','1.00178067793628','0.131219501749293','-0.094043658135972','6','85']
  H_HDV['108_THC_2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.014456232039527','0.161473551864718','6.10337732590456','2.45270878772843','-0.015434661603682','6','85']
  H_HDV['108_NOx_2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.95979928616537','0.026796740524702','28.7793419524162','0.344341739100003','5.30401450733061','6','85']
  H_HDV['108_PM_2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.105582623336336','0.028039027608637','0.318613792085082','0.149314157347047','0.021438778433879','6','85']
  H_HDV['108_FC_2_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','50','FC','y=(c+(a*exp(((-1)*b)*x)))','801.71324851986','0.066408379628048','666.867364399667','0','0','6','85']
  H_HDV['108_CO_2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.569969626786942','0.028112988675681','0.979547853328211','0.154003082621234','0.100126311475394','6','69']
  H_HDV['108_THC_2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017496178615597','0.157115259091902','6.00985261348189','2.44563978263766','-0.016741876952854','6','69']
  H_HDV['108_NOx_2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.83860950005757','0.029709724804271','57.6780734532469','0.503788462188554','6.83057354595992','6','69']
  H_HDV['108_PM_2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.14681073042977','0.043037428623703','0.28715728914657','0.162458845982947','0.031153917557599','6','69']
  H_HDV['108_FC_2_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','2','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.002396525068449','0.409523193042508','-26.9401307063377','1588.5996729129','0','6','69']
  H_HDV['108_CO_4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.206200630373574','0.733868143052762','5.94219523457157','2.40923322628445','-0.015492428830851','6','71']
  H_HDV['108_THC_4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009363661326586','0.179304088617723','5.4544022265933','2.1893576295034','-0.012647497434756','6','71']
  H_HDV['108_NOx_4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.30749268752579','0.029111983825585','19.1444665577395','0.214253565870477','5.82507064004328','6','71']
  H_HDV['108_PM_4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.089422337643264','0.020267917544762','0.319076134996962','0.137211053654127','0.016780210413245','6','71']
  H_HDV['108_FC_4_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','839.846766698553','0.087615190436875','4770.26071772007','0.661699827220325','571.454362990848','6','71']
  H_HDV['108_CO_4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.653635977457425','0.02560594421127','0.843355347528947','0.14196632288943','0.066198028694081','6','62']
  H_HDV['108_THC_4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.020866666017288','0.152696919958403','6.52117656581959','2.65695183482985','-0.020292023191101','6','62']
  H_HDV['108_NOx_4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.95279304688814','0.026660962800411','22.4479002595499','0.286572448743525','7.49127753348959','6','62']
  H_HDV['108_PM_4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.093875783639608','0.024784450584217','0.323322895127365','0.123714974811542','0.024719257770315','6','62']
  H_HDV['108_FC_4_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','50','FC','y=exp((a+(b/x))+(c*log(x)))','7.29186410649815','1.36568576205916','-0.08792126355754','0','0','6','62']
  H_HDV['108_CO_4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.95981753897846','-0.00420328112943','-0.167321469760396','0.108519030378073','0','6','45']
  H_HDV['108_THC_4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.028889171730181','0.138675141339564','7.17275307276294','2.95713111521242','-0.02971390505192','6','45']
  H_HDV['108_NOx_4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000295722714679','0.030027841366676','-1.13017571610258','29.8002050846667','0','6','45']
  H_HDV['108_PM_4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040767366063328','0.329128348551205','4.64465862461982','2.04319462762264','-0.012513765899378','6','45']
  H_HDV['108_FC_4_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','707.427777622432','0.052856628874625','-0.000013658120698','-0.346231435757672','1468.26844857507','6','45']
  H_HDV['108_CO_6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.80504369095346','-0.00509049395541','-0.260524991078897','0.2045425976145','0','6','71']
  H_HDV['108_THC_6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009096851618298','0.164602284728769','7.04301580005189','2.83099902653506','-0.035300255461078','6','71']
  H_HDV['108_NOx_6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','34.6978604651634','-0.051892507970935','-5.89826990245988','0.245869041929188','0','6','71']
  H_HDV['108_PM_6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.444829639638629','-0.000897658221383','-0.048265001052561','0.13694630007748','0','6','71']
  H_HDV['108_FC_6_0'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','2683.94084760543','1.0081767909971','-0.425068540868791','0','0','6','71']
  H_HDV['108_CO_6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','50','CO','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000021430621413','0.002338037173252','-0.091804969606766','1.62263937927322','0','6','44']
  H_HDV['108_THC_6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.034150892846795','0.573325108167388','-1.05116163814529','-0.300985760821247','0.136487531901628','6','44']
  H_HDV['108_NOx_6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','50','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000357109684192','0.035811182304874','-1.26768968854707','29.8190210256696','0','6','44']
  H_HDV['108_PM_6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.432289947020169','0.000608865674716','-0.031099439417176','0.076502354251073','0','6','44']
  H_HDV['108_FC_6_50'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','598.785022265806','0.045808266059543','17033.3509305105','0.779976488920878','1352.92244265795','6','44']
  H_HDV['108_CO_6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.868444994745963','0.090216351344631','1.13696328873339','0.090184455961658','0.273043245666792','6','31']
  H_HDV['108_THC_6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.051703232103157','0.149598107252466','1.6452653781502','-0.333226730300273','0.243410733727863','6','31']
  H_HDV['108_NOx_6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.001564576994539','0.106015611839198','-2.56272220575878','42.6832970876474','0','6','31']
  H_HDV['108_PM_6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.492219859156999','0.00589485121534','-0.036345345852756','0.061850598163415','0','6','31']
  H_HDV['108_FC_6_100'] = [' 108','EURO5','TT/AT >50-60t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1332.8182839512','0.012928287849673','29314.628168256','0.85967375344416','1228.39038079593','6','31']
  H_HDV['109_CO_0_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.79241496366684','0.052313641404859','13.4319267733393','0.177542638539671','1.80956010382189','6','86']
  H_HDV['109_THC_0_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.3785084185121','0.029739521509275','8.46523250229506','0.12502106344125','0.598353993889889','6','86']
  H_HDV['109_NOx_0_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','21.6301232749682','0.068794075083673','1123.46901556044','1.03968988686123','7.92176816598278','6','86']
  H_HDV['109_PM_0_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.29810489063572','0.050857444198514','2.28278689657273','0.176988169727819','0.283181557100725','6','86']
  H_HDV['109_FC_0_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','635.946071604428','0.067344422069075','10095.3436098671','0.757167157292546','163.92773597741','6','86']
  H_HDV['109_CO_0_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.49963072063242','0.053972960824355','15.7354359472153','0.215318530225914','1.99147772458819','6','86']
  H_HDV['109_THC_0_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.28893756651057','0.030129485693658','8.60556250228666','0.124008775459854','0.560150729959803','6','86']
  H_HDV['109_NOx_0_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','19.9325398444607','0.054695288785187','870.468482195864','0.978517740963375','8.86726711179886','6','86']
  H_HDV['109_PM_0_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.45537334206829','0.052839398365698','2.64659193495616','0.218261763503143','0.30875832818502','6','86']
  H_HDV['109_FC_0_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','598.910359498237','0.058899462303278','4905.18929231562','0.620786033420756','179.286826202685','6','86']
  H_HDV['109_CO_0_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.33168403399515','0.050202146072762','19.8571966367242','0.248756997278318','2.19837242842567','6','86']
  H_HDV['109_THC_0_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.356259407217064','8.91640437251279','3.81873885376849','1.7180595579747','-0.007963382479974','6','86']
  H_HDV['109_NOx_0_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','100','NOx','y=(1/(((c*(x**2))+(b*x))+a))','0.030063801593943','0.001489551845956','-0.000007714107534','0','0','6','86']
  H_HDV['109_PM_0_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.42100996416329','0.048774035021363','3.18565978781957','0.241674491095217','0.334366243445646','6','86']
  H_HDV['109_FC_0_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','561.185037130281','0.050176479243869','3355.4897309448','0.548208076292399','192.889506618923','6','86']
  H_HDV['109_CO_-2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.704603896317','0.007825726013362','-1.15674445101335','0.073384281745277','0','6','86']
  H_HDV['109_THC_-2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.655403776348696','6.67249689577617','4.29046108565619','1.59693296640314','0.028494997108066','6','86']
  H_HDV['109_NOx_-2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.77754995296846','114.097477960278','-0.797848000306855','0.428352455618271','0.043214326857512','6','86']
  H_HDV['109_PM_-2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.46389296424877','0.079825015750796','530.560617776398','1.5338858517655','0.211040980279699','6','86']
  H_HDV['109_FC_-2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','581.922757383685','0.066860712496753','58442.3364725135','1.02959607968986','72.8344094029467','6','86']
  H_HDV['109_CO_-2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.3012889287586','0.004667293610257','-1.11312954204619','0.072048682725301','0','6','86']
  H_HDV['109_THC_-2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.471697875756736','7.07110128263718','3.81156523964357','1.46733290328433','0.025811498303398','6','86']
  H_HDV['109_NOx_-2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','50','NOx','y=((a*(b**x))*(x**c))','45.3913347151679','0.987817727345302','-0.467672430001533','0','0','6','86']
  H_HDV['109_PM_-2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.15838156648262','0.071052516028879','3.66648780005556','0.493370566764771','0.171679997775064','6','86']
  H_HDV['109_FC_-2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','512.172465666587','0.051621291658421','31856.9346784777','0.895272945569687','49.1840968660246','6','86']
  H_HDV['109_CO_-2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.2183187840338','0.065970663054513','27.5361878397718','0.498516774569422','0.938719500320952','6','86']
  H_HDV['109_THC_-2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.350229155558772','7.63575430197931','3.33502586627664','1.34536164050767','0.024400117564516','6','86']
  H_HDV['109_NOx_-2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.0575923797364','-0.061761426341849','-1.29476317020657','0.071239646409523','0','6','86']
  H_HDV['109_PM_-2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.96218635076736','0.064003468729597','4.67005277189149','0.451464124004677','0.149578380483884','6','86']
  H_HDV['109_FC_-2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','479.846878587227','0.042291931371254','29404.6912399408','0.863195388692601','32.286282412141','6','86']
  H_HDV['109_CO_-4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','15.8038321346112','-0.00972706802665','-1.30144986142166','0.088178079683744','0','6','86']
  H_HDV['109_THC_-4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.60524637183056','-0.003838814816202','-0.706214345702821','0.087638704494408','0','6','86']
  H_HDV['109_NOx_-4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.4421469709924','0.051483706659637','400.879471668749','0.735571060714103','0.385213962689885','6','86']
  H_HDV['109_PM_-4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.82576471456861','0.068339286647674','2.44560516542083','0.344380658466707','0.084355791562989','6','86']
  H_HDV['109_FC_-4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','424.275410147469','0.055090971154391','15363.5820307794','0.752479761628932','11.2938676597467','6','86']
  H_HDV['109_CO_-4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2932897463538','0.063139277638281','31.2616311946147','0.43724220311004','0.303618536689238','6','86']
  H_HDV['109_THC_-4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.27948678445636','-0.004267673858855','-0.69871798608735','0.090227376520679','0','6','86']
  H_HDV['109_NOx_-4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.6511444830129','0.044987411463157','744.545456327677','0.831596531812106','0.166595613262192','6','86']
  H_HDV['109_PM_-4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.71432238514713','0.06681377754234','6.06227931507676','0.477854677237566','0.076404175432109','6','86']
  H_HDV['109_FC_-4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','373.455759432917','0.04961744945143','22529.3776107337','0.809467763763532','5.85247490892334','6','86']
  H_HDV['109_CO_-4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.44040400608895','0.059616451464556','69.7099385895833','0.548852840571977','0.260203000162655','6','86']
  H_HDV['109_THC_-4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.60983878779033','0.068723578757164','10.8350450502483','0.391119808869621','0.235385384378113','6','86']
  H_HDV['109_NOx_-4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6896217406748','0.039717740779745','1285.5082503725','0.918869466174355','-0.019752233182548','6','86']
  H_HDV['109_PM_-4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.60006386611292','0.063789115268301','11.7931895936674','0.571420185466252','0.071829047061161','6','86']
  H_HDV['109_FC_-4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','344.149490167765','0.045607161275437','32310.6550670602','0.867210204570611','2.63389571304453','6','86']
  H_HDV['109_CO_-6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.0995871458468','0.074948987968853','37.8852198004128','0.548013539848816','0.238119958505344','6','72']
  H_HDV['109_THC_-6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.16458837146889','0.07595488835951','6.18078141119718','0.344109403243696','0.213609439606963','6','72']
  H_HDV['109_NOx_-6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.5224033727461','0.062308615843866','736.941564393283','0.852454155319515','0.159996133611332','6','72']
  H_HDV['109_PM_-6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.78724418798522','0.075687962103815','5.83318019563238','0.52645074740925','0.065752377230118','6','72']
  H_HDV['109_FC_-6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','366.41056429351','0.063258958813075','22401.8293108215','0.8234650065759','3.71391527678281','6','72']
  H_HDV['109_CO_-6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.07584445873779','0.069737723719625','83.2755348424299','0.609887793010223','0.20660099598093','6','72']
  H_HDV['109_THC_-6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.46422817525544','0.074160697988309','16.5458397837023','0.471254315180109','0.203818428727676','6','72']
  H_HDV['109_NOx_-6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2749845344786','0.057340863634525','887.097487066054','0.869752778628761','0.118479992888395','6','72']
  H_HDV['109_PM_-6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.5475944308808','0.071845464627125','10.8611755076668','0.584383802555297','0.062216371424779','6','72']
  H_HDV['109_FC_-6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','304.875156946893','0.059083576637683','24806.2732969205','0.831775664204657','2.62810740600648','6','72']
  H_HDV['109_CO_-6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.91417973484683','0.065704261556742','114.225606128438','0.636699145129461','0.186866985464704','6','72']
  H_HDV['109_THC_-6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.04714407389657','0.073004724459677','22.4911487993715','0.512678532448197','0.198909256547926','6','72']
  H_HDV['109_NOx_-6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.22942909184103','0.054400370774736','1039.25742028915','0.893934169766815','0.101355844702214','6','72']
  H_HDV['109_PM_-6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.41572091895044','0.069177622022155','14.2417207542469','0.610166244346077','0.060309873241406','6','72']
  H_HDV['109_FC_-6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','275.08504296868','0.056728796553343','27749.1700743262','0.84894789098738','2.2304502138402','6','72']
  H_HDV['109_CO_2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.26227370643763','14.4444131397869','6.54207789182937','2.60717273449693','-0.031914995682556','6','86']
  H_HDV['109_THC_2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','0','THC','y=exp((a+(b/x))+(c*log(x)))','4.8595020992656','-4.68439875168231','-1.17950938492277','0','0','6','86']
  H_HDV['109_NOx_2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','40.9384779574027','0.025619922130164','-2.34263602356821','0.081258438972641','0','6','86']
  H_HDV['109_PM_2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.24590338884794','2.37798786077289','5.62459613661692','2.30502467679082','-0.02302779488442','6','86']
  H_HDV['109_FC_2_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1162.82992396212','0.35931719213495','-74.618845559706','0.080473576791555','0','6','86']
  H_HDV['109_CO_2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.0217948744045','0.006333558458562','-1.70347677216668','0.084924924090762','0','6','86']
  H_HDV['109_THC_2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.357900550307952','8.24833229312133','5.30072512824062','2.1175162450953','-0.012529607378329','6','86']
  H_HDV['109_NOx_2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','43.2702078751696','0.012025548573624','-2.07074074286797','0.077202145465143','0','6','86']
  H_HDV['109_PM_2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.51990092637981','2.37755460750506','4.33794242938457','1.59651389941389','0.039944724734152','6','86']
  H_HDV['109_FC_2_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1222.85947215191','0.831382278421513','-71.5839022484453','0.076074660448101','0','6','86']
  H_HDV['109_CO_2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.1668173342376','0.009553295539227','-1.80395860640255','0.088392603922437','0','6','86']
  H_HDV['109_THC_2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.457330428915611','7.9752965414808','5.75535668560938','2.29386528574626','-0.013552111093906','6','86']
  H_HDV['109_NOx_2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','100','NOx','y=(c+(a*exp(b*x)))','24.4943357832973','-0.070980223033568','20.4781282754747','0','0','6','86']
  H_HDV['109_PM_2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.8487519219011','0.001106991975778','-0.289232958140858','0.089132781687981','0','6','86']
  H_HDV['109_FC_2_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1279.36834344021','0.645648441003855','-71.2069386955299','0.077960804158334','0','6','86']
  H_HDV['109_CO_4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','3.0756287234591','12.9264169428016','8.34595437300206','3.28144676295608','-0.026402601209253','6','71']
  H_HDV['109_THC_4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.290594195329153','8.79565925062653','5.10129487258306','2.03638826973749','-0.010994113604879','6','71']
  H_HDV['109_NOx_4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','49.2322790976914','0.013185261718617','-2.92928324971308','0.094831762295816','0','6','71']
  H_HDV['109_PM_4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.472561622810802','2.24222451669029','6.64356833598355','2.68551605897233','-0.01864662682267','6','71']
  H_HDV['109_FC_4_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','0','FC','y=(c+(a*exp(b*x)))','1055.99183603261','-0.098185942808882','354.063244090515','0','0','6','71']
  H_HDV['109_CO_4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','5.19245856029204','45.9498521495603','-1.61090091554688','-0.950805031166912','0.209813221326938','6','71']
  H_HDV['109_THC_4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.407117068295942','8.53851640870446','5.62351426609114','2.25573567403805','-0.013480316504848','6','71']
  H_HDV['109_NOx_4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','53.1261617428777','-0.018114501331088','-2.58231467090138','0.094536849167428','0','6','71']
  H_HDV['109_PM_4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.793268615236984','34.6312218118588','-2.88719825247338','-0.518341452603262','0.15471057666792','6','71']
  H_HDV['109_FC_4_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','50','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.006674280216951','1.00789868446259','-49.6870612465601','1287.40088763661','0','6','71']
  H_HDV['109_CO_4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.9302264001704','45.8095960156839','-1.2792797442326','-0.702735140004623','0.200168740224957','6','66']
  H_HDV['109_THC_4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.517602613089174','8.11287276045796','6.32203654969662','2.5339795985724','-0.018307873173228','6','66']
  H_HDV['109_NOx_4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','100','NOx','y=((a*(b**x))*(x**c))','73.4756687900749','1.0028679865933','-0.256357132769582','0','0','6','66']
  H_HDV['109_PM_4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.05322597903795','16.1326060037358','-1.9395844365715','-0.480828062737643','0.168326633726969','6','66']
  H_HDV['109_FC_4_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','4','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1612.61037086213','-0.631496008785529','-108.48054347061','0.110675169996099','0','6','66']
  H_HDV['109_CO_6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','24.3358493508761','0.062864561493596','-1.6703555196436','0.073387975057613','0','6','71']
  H_HDV['109_THC_6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.375631007585594','8.2442619840611','6.48264193937586','2.55397478083121','-0.021397952841118','6','71']
  H_HDV['109_NOx_6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','0','NOx','y=(c+(a*exp(((-1)*b)*x)))','30.3173914145644','0.100576946588159','24.4956107866215','0','0','6','71']
  H_HDV['109_PM_6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.00209266290565','0.006590357604348','-0.28890742807517','0.080414467176954','0','6','71']
  H_HDV['109_FC_6_0'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1411.9405388457','2.62393091068724','-78.6444491285696','0.07261712105231','0','6','71']
  H_HDV['109_CO_6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.024351536596036','0.00563364985911','-0.000064655557699','0','0','6','61']
  H_HDV['109_THC_6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.607539348041397','7.84106917461672','7.16776429625378','2.84951012414262','-0.024901415654161','6','61']
  H_HDV['109_NOx_6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','50','NOx','y=((a*(b**x))*(x**c))','79.9795137893352','1.00430143411562','-0.28160239097454','0','0','6','61']
  H_HDV['109_PM_6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.133474654101111','0.036701249241829','-0.000412327597847','0','0','6','61']
  H_HDV['109_FC_6_50'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','50','FC','y=((a*(b**x))*(x**c))','2739.95129120015','1.01003326116292','-0.498337686879818','0','0','6','61']
  H_HDV['109_CO_6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','9.46962887294047','11.2313564251888','8.535420998092','3.9273322881388','-0.05782411191603','6','50']
  H_HDV['109_THC_6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.15865532133693','51.2300497609634','-1.80240223858764','-0.390226152278069','0.136256784248748','6','50']
  H_HDV['109_NOx_6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','100','NOx','y=((a*(b**x))*(x**c))','84.0312887581288','1.00262485209262','-0.216465193634966','0','0','6','50']
  H_HDV['109_PM_6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.48038011573437','2.06269459156096','6.74368935162818','2.81814145663753','0.0272257533778','6','50']
  H_HDV['109_FC_6_100'] = [' 109','EURO0','TT/AT >14-20t 80ties','6','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1894.19263649827','-1.14762156581335','-139.673893675856','0.136308321430322','0','6','50']
  H_HDV['110_CO_0_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.50414643401477','0.054873465068893','5.07784323549647','0.2090304961874','0.838032720951917','6','86']
  H_HDV['110_THC_0_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.181937165554404','3.08071533936726','3.72862591495817','1.60513243323237','-0.005431382520026','6','86']
  H_HDV['110_NOx_0_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.5593419465267','0.069507737541379','32.4414107486126','0.461708508393616','4.69811672060083','6','86']
  H_HDV['110_PM_0_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.077631506130309','1.47511652468397','4.74446402338576','1.98282201793749','-0.016131737812693','6','86']
  H_HDV['110_FC_0_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','472.654483954601','0.068135732038462','16363.2588868782','0.88068718583685','136.289145819937','6','86']
  H_HDV['110_CO_0_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.56595869289723','0.054853647374813','6.97395972608027','0.256691653927806','0.967986420057194','6','86']
  H_HDV['110_THC_0_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.172532081786293','3.09946708303487','3.73352777979288','1.61298932164071','-0.006080891695336','6','86']
  H_HDV['110_NOx_0_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.09450990287','0.054299109966648','37.4444239631321','0.469208003438172','5.26279471848434','6','86']
  H_HDV['110_PM_0_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.868987630064895','0.052807034706726','1.14859185373148','0.169870837400754','0.182829545048528','6','86']
  H_HDV['110_FC_0_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','442.127980534531','0.057008370346343','11456.2111847056','0.804653050585427','154.496540341448','6','86']
  H_HDV['110_CO_0_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.068681735780592','0.016503143237782','-0.000087406877136','0','0','6','86']
  H_HDV['110_THC_0_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.17100212727212','3.11412466696927','3.65748942819465','1.58975334230146','-0.006016799072795','6','86']
  H_HDV['110_NOx_0_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','20.7777107695845','-0.027809802165157','-0.909063910309873','0.069131806262186','0','6','86']
  H_HDV['110_PM_0_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.910887239431719','0.051227464716909','1.25211013778857','0.203936236941152','0.206670828916865','6','86']
  H_HDV['110_FC_0_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001230352551387','0.00009845492636','-0.00000055043199','0','0','6','86']
  H_HDV['110_CO_-2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.7108767419863','0.009059718941204','-0.401935547486314','0.060929201978928','0','6','86']
  H_HDV['110_THC_-2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.23407584595369','2.19274073685856','5.18607093390841','1.90648466981717','0.012741445034288','6','86']
  H_HDV['110_NOx_-2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.60946306568204','67.4757879687792','-0.403374751785307','0.546470617085524','0.03889854493771','6','86']
  H_HDV['110_PM_-2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.161546947036528','36.0425483903311','-3.96970181653185','-0.667311775241849','0.127064620883296','6','86']
  H_HDV['110_FC_-2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','450.118927021953','0.066015115941318','34794.3373473607','0.985147800075526','55.8517699891597','6','86']
  H_HDV['110_CO_-2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.69338962184998','0.009517247364212','-0.391700638445877','0.058419217650086','0','6','86']
  H_HDV['110_THC_-2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.108154404969916','2.31250917251733','5.07846962890261','1.95479206148339','-0.004149159963022','6','86']
  H_HDV['110_NOx_-2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','50','NOx','y=((a*(b**x))*(x**c))','35.3778397175486','0.988881274261227','-0.545067381359122','0','0','6','86']
  H_HDV['110_PM_-2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.139279797645534','25.8095509753311','-3.34920131443582','-0.465071954789245','0.111365826738314','6','86']
  H_HDV['110_FC_-2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','399.067131513853','0.050509445005236','22305.7499007463','0.875580488671132','40.3637608811251','6','86']
  H_HDV['110_CO_-2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.98765149282451','0.005085308730449','-0.416891788922317','0.061482529864995','0','6','86']
  H_HDV['110_THC_-2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','3.04549444229784','-0.002557783852981','-0.216365592333785','0.079635941397007','0','6','86']
  H_HDV['110_NOx_-2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','100','NOx','y=((a*(b**x))*(x**c))','27.3082456008199','0.982861184880712','-0.392276122990422','0','0','6','86']
  H_HDV['110_PM_-2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','100','PM','y=(c+(a*exp(b*x)))','1.69424030298265','-0.07298557175936','0.114771987445186','0','0','6','86']
  H_HDV['110_FC_-2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','381.077932339502','0.040700957477626','23032.0952893947','0.862945820604232','27.2204374941367','6','86']
  H_HDV['110_CO_-4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.43700113311937','-0.004017486043369','-0.487929546366175','0.081524451569302','0','6','86']
  H_HDV['110_THC_-4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.94884398106054','-0.002253969115436','-0.223914388922283','0.082987367611224','0','6','86']
  H_HDV['110_NOx_-4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.4477392099807','0.056230143975725','63.6776721763969','0.493371271555764','0.323013398143223','6','86']
  H_HDV['110_PM_-4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.36030013562598','-11.3730346436653','-1.86437434313432','0','0','6','86']
  H_HDV['110_FC_-4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','342.68665960973','0.056108009611832','9741.72246576963','0.720088979043739','9.76367886004723','6','86']
  H_HDV['110_CO_-4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.31922982489905','-0.005450932851284','-0.493703441061109','0.085045547044178','0','6','86']
  H_HDV['110_THC_-4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.8623547385828','-0.00265028466982','-0.226341857018074','0.087008228879675','0','6','86']
  H_HDV['110_NOx_-4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.94117660082836','0.049767762301918','106.622654062277','0.566736825531803','0.210121570637432','6','86']
  H_HDV['110_PM_-4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','5.0846561520872','-10.3655481016236','-1.8176149090855','0','0','6','86']
  H_HDV['110_FC_-4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','303.586473569672','0.049754478482204','13451.6444983733','0.763459636256199','5.27152594542038','6','86']
  H_HDV['110_CO_-4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.2273814075646','0.055210664970307','34.6291044991059','0.589485466860117','0.114839669058388','6','86']
  H_HDV['110_THC_-4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.78236282729245','-0.002705260730339','-0.224023730834396','0.088843922571303','0','6','86']
  H_HDV['110_NOx_-4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.29398521149043','0.04546890242481','234.051794810408','0.691153743061541','0.139925008129106','6','86']
  H_HDV['110_PM_-4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','100','PM','y=exp((a+(b/x))+(c*log(x)))','4.65766459169316','-9.01950098215771','-1.7145700180158','0','0','6','86']
  H_HDV['110_FC_-4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','282.130941260172','0.045138695048756','19123.328692659','0.817268292990114','2.42362050346718','6','86']
  H_HDV['110_CO_-6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.91801493683472','0.073266734590844','12.8997109384424','0.618474679947198','0.114370170637993','6','72']
  H_HDV['110_THC_-6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.10730813912053','0.067973024926447','2.0031250277254','0.335435811824315','0.075686041899296','6','72']
  H_HDV['110_NOx_-6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.99269930751179','0.069565788619597','160.715228716938','0.674661037459993','0.205448232439684','6','72']
  H_HDV['110_PM_-6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','5.5341011753696','-11.6675179300714','-1.97475136109033','0','0','6','72']
  H_HDV['110_FC_-6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','304.562508319415','0.064827470119374','13486.0046385958','0.783079531008665','3.81881023832482','6','72']
  H_HDV['110_CO_-6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.04847367336275','0.067105201327347','30.0609573925347','0.625070601121198','0.098264730710975','6','72']
  H_HDV['110_THC_-6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.86478316686508','0.065852286802711','5.17256532161626','0.451656695668961','0.072054271448079','6','72']
  H_HDV['110_NOx_-6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.97667179838525','0.063718707403167','242.94656248858','0.714584503855499','0.175399543256497','6','72']
  H_HDV['110_PM_-6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.029389874210382','1.2077118680852','5.10571033625077','2.01245928351263','0.004909116941051','6','72']
  H_HDV['110_FC_-6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','255.536116600568','0.060225017029914','15804.3073828356','0.797455027423802','2.89481850351601','6','72']
  H_HDV['110_CO_-6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.5329960187213','0.062077300384343','42.7262117607675','0.639923535781937','0.086680673782362','6','72']
  H_HDV['110_THC_-6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.72721506661363','0.064631074441405','6.772064936209','0.48704089064317','0.070403012728276','6','72']
  H_HDV['110_NOx_-6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.0136164442544','0.06013641298266','298.233718130668','0.741313686839889','0.161657557304717','6','72']
  H_HDV['110_PM_-6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.07732659182494','0.068640436700786','0.866437468310973','0.229692143849136','0.048997958607058','6','72']
  H_HDV['110_FC_-6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','231.995549069722','0.057456332803203','19028.1985912723','0.826767926861879','2.49173653665349','6','72']
  H_HDV['110_CO_2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.85742508166949','0.008709183181867','-0.539259242083913','0.065745225917307','0','6','86']
  H_HDV['110_THC_2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.219708302841985','2.82253096279755','5.15018691293164','1.99819977558471','-0.009625588210625','6','86']
  H_HDV['110_NOx_2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','26.4591609331376','0.011010588806313','-1.70845179870402','0.090519704269212','0','6','86']
  H_HDV['110_PM_2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.245977974391315','17.4609366004015','-2.95414688581185','-0.447922752536297','0.115794757985798','6','86']
  H_HDV['110_FC_2_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','860.105012540317','0.57055211870145','-53.3400789949258','0.0804896239339','0','6','86']
  H_HDV['110_CO_2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.66526892727733','0.010894043710645','-0.620889297314416','0.071474914137193','0','6','86']
  H_HDV['110_THC_2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.256282137314761','2.58589774593059','7.0213244847856','2.73054337880602','-0.030655349142935','6','86']
  H_HDV['110_NOx_2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.148628710245','-0.012604566312442','-1.68084057943895','0.098050007906184','0','6','86']
  H_HDV['110_PM_2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.71951751907379','0.002734733657325','-0.099423533448098','0.063706102800165','0','6','86']
  H_HDV['110_FC_2_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','50','FC','y=((a*(x**b))+(c*(x**d)))','0.041738878551461','1.78385811944741','1566.56363998951','-0.462972317223816','0','6','86']
  H_HDV['110_CO_2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','100','CO','y=(c+(a*exp(b*x)))','8.63959073466328','-0.084198625048298','1.90177711128453','0','0','6','86']
  H_HDV['110_THC_2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.412833112856315','2.37204011387739','7.94818738233444','3.07119504718766','-0.029811921437816','6','86']
  H_HDV['110_NOx_2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','28.2198842597644','-0.017321917548095','-1.25754652226134','0.084048425741592','0','6','86']
  H_HDV['110_PM_2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.74297143733752','0.00225794084065','-0.098508916271276','0.066751888582303','0','6','86']
  H_HDV['110_FC_2_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','940.121700521408','0.635811195929338','-42.9033513163046','0.070238449798791','0','6','86']
  H_HDV['110_CO_4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.64574877165676','51.7240445252097','-2.86187292885829','-0.924006801386533','0.172626480070273','6','71']
  H_HDV['110_THC_4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.340185432643681','2.66599889284928','6.65306669026799','2.56706466949051','-0.018471337022028','6','71']
  H_HDV['110_NOx_4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','0','NOx','y=(c+(a*exp(b*x)))','19.5624503117864','-0.107312846867226','11.5725362993737','0','0','6','71']
  H_HDV['110_PM_4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.326267656401934','18.2556247448572','-3.51123674811455','-0.757105875233293','0.143804223888472','6','71']
  H_HDV['110_FC_4_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','0','FC','y=((a*(b**x))*(x**c))','1947.49689698044','1.01030305671997','-0.58940571056478','0','0','6','71']
  H_HDV['110_CO_4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.8216821691775','0.016714864907655','-0.725549194027339','0.077468106299389','0','6','71']
  H_HDV['110_THC_4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.458824837471928','2.58683652868595','6.87258524741237','2.68532057751059','-0.019190237414898','6','71']
  H_HDV['110_NOx_4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','50','NOx','y=((a*(b**x))*(x**c))','45.7294679795967','1.00543134991599','-0.348544191341974','0','0','6','71']
  H_HDV['110_PM_4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','50','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000009053757313','0.001385309558786','-0.069519381455277','1.61068586252969','0','6','71']
  H_HDV['110_FC_4_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','589.202735827274','0.091927382081105','27358.6209049049','1.05743641199315','436.702744116959','6','71']
  H_HDV['110_CO_4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.3791144765523','-0.019776777008357','-1.25302783054432','0.128722829038621','0','6','67']
  H_HDV['110_THC_4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.582351205906511','2.41560668359001','7.65127254906225','2.99713603019827','-0.025571311258498','6','67']
  H_HDV['110_NOx_4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.3737625681453','0.058383180278701','38.5088896101654','0.446796742703325','18.3120731015389','6','67']
  H_HDV['110_PM_4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000008518758371','0.001282637223739','-0.063657208687681','1.64687921087468','0','6','67']
  H_HDV['110_FC_4_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','463.343423231332','0.07251439688432','5336.84322048407','0.686352376218185','542.088675013005','6','67']
  H_HDV['110_CO_6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','2.12613486755826','56.6758993280368','-3.36227024821072','-1.25391797332011','0.211130081938045','6','71']
  H_HDV['110_THC_6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.568076300786272','20.1880006722359','-1.99032587942168','-0.34031704520734','0.114828236668153','6','71']
  H_HDV['110_NOx_6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','35.2684325981656','-0.014516169780184','-2.41360853061621','0.123097168006442','0','6','71']
  H_HDV['110_PM_6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.67295864825237','0.004492084979577','-0.093227094404566','0.06331719395297','0','6','71']
  H_HDV['110_FC_6_0'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','0','FC','y=((a*(b**x))*(x**c))','1960.8765380771','1.01074325524546','-0.530839126128517','0','0','6','71']
  H_HDV['110_CO_6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.9962791225472','-0.020325643662204','-1.48372638942391','0.145320399397303','0','6','61']
  H_HDV['110_THC_6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.720425861348605','4.53060660539004','0.882334908009015','0.181674682405433','0.105630691592143','6','61']
  H_HDV['110_NOx_6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','19.4247327033763','102.661860566297','-0.656054842232729','0.719856465139718','0.03132612072516','6','61']
  H_HDV['110_PM_6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','0.490052852268443','0.04921253351841','-0.000525268537022','0','0','6','61']
  H_HDV['110_FC_6_50'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','50','FC','y=((a*(b**x))*(x**c))','1762.36053349864','1.00688165525578','-0.37085534233391','0','0','6','61']
  H_HDV['110_CO_6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.4947092645807','-0.06023271530306','-1.40831442160211','0.157081660338591','0','6','51']
  H_HDV['110_THC_6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.949545577696752','22.8619577863616','-2.71578501365229','-0.76054672072579','0.168345724089864','6','51']
  H_HDV['110_NOx_6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.56842469119303','1.07390244570193','-0.097438341622379','0','0','6','51']
  H_HDV['110_PM_6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000014178076283','0.001689462604044','-0.067655590830868','1.76346182319869','0','6','51']
  H_HDV['110_FC_6_100'] = [' 110','EURO1','TT/AT >14-20t Euro-1','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','353.611735118115','0.0638434922091','2440.57725616857','0.479312252485342','730.767690001662','6','51']
  H_HDV['111_CO_0_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.28609840241863','0.05976129566883','4.1430653198461','0.260198336330686','0.739885227945392','6','86']
  H_HDV['111_THC_0_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.104894550333399','2.07620670140688','3.5559288221832','1.55709957580452','-0.005456036255216','6','86']
  H_HDV['111_NOx_0_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','13.699783137855','0.064197671514989','24.6642841872775','0.350666824346525','4.97190020627976','6','86']
  H_HDV['111_PM_0_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.390254205177133','0.000544633246436','-0.018804510156721','0.055785145603854','0','6','86']
  H_HDV['111_FC_0_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.001141917343342','0.000177268214854','-0.00000125247155','0','0','6','86']
  H_HDV['111_CO_0_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.097033159533766','0.023796294833562','-0.00014612279464','0','0','6','86']
  H_HDV['111_THC_0_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.55167926658453','0.027407202427569','1.82436960709253','0.113656855936047','0.151161374139323','6','86']
  H_HDV['111_NOx_0_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','12.9376693878689','0.051982666674158','25.2939134014078','0.358932398118935','5.44958273316293','6','86']
  H_HDV['111_PM_0_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.39286132958951','0.001991752330169','-0.014024534841376','0.034236378616804','0','6','86']
  H_HDV['111_FC_0_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.0013082785751','0.000129965565409','-0.000000827754977','','','6','86']
  H_HDV['111_CO_0_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.103284742051211','0.019770695480996','-0.0001182071034','0','0','6','86']
  H_HDV['111_THC_0_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.502280093652749','0.020965598409173','1.89881798163074','0.110448263250888','0.115789694270593','6','86']
  H_HDV['111_NOx_0_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','23.5444998754754','-0.031602450407182','-1.13548922626516','0.073449114369184','0','6','86']
  H_HDV['111_PM_0_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','100','PM','y=(((a*(x**2))+(b*x))+c)','0.000072848534591','-0.00889359231196','0.388293721736991','0','0','6','86']
  H_HDV['111_FC_0_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','0','100','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00141190243338','0.000095989278536','-0.00000052277987','0','0','6','86']
  H_HDV['111_CO_-2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.65206511977317','0.007748695920992','-0.273734024096486','0.05895782560285','0','6','86']
  H_HDV['111_THC_-2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.06108961051653','-0.000746917206479','-0.142778969167131','0.077521887117238','0','6','86']
  H_HDV['111_NOx_-2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.3135075793696','-0.01658314187966','-1.57401986856089','0.086044694772294','0','6','86']
  H_HDV['111_PM_-2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.367064333511762','0.000636777293754','-0.018800218145712','0.051837683253644','0','6','86']
  H_HDV['111_FC_-2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','399.218036692364','0.063768108949638','26175.3512687284','0.961961773030662','52.1299690806487','6','86']
  H_HDV['111_CO_-2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.82298676835475','0.008107826554503','-0.276107224738585','0.056785534280282','0','6','86']
  H_HDV['111_THC_-2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.02358848783139','-0.001473113469405','-0.142555712901591','0.079593740471592','0','6','86']
  H_HDV['111_NOx_-2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','50','NOx','y=((a*(b**x))*(x**c))','38.93994306597','0.988664243185899','-0.54459732424906','0','0','6','86']
  H_HDV['111_PM_-2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.380800468445262','0.000901191246192','-0.017947902152253','0.045985020544687','0','6','86']
  H_HDV['111_FC_-2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','358.311228850442','0.048436854631106','20505.4866503796','0.885941412532165','37.7211664157338','6','86']
  H_HDV['111_CO_-2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.98839562565945','0.005393231601028','-0.279096842543933','0.057161888224652','0','6','86']
  H_HDV['111_THC_-2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.99523306843858','-0.001857510099977','-0.141887078005113','0.080683633400727','0','6','86']
  H_HDV['111_NOx_-2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','100','NOx','y=((a*(x**b))+(c*(x**d)))','-0.2287781326485','0.536030923703305','36.0458121048043','-0.526414718857122','0','6','86']
  H_HDV['111_PM_-2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.40320013087751','0.000796145892311','-0.018072342432853','0.044609892012303','0','6','86']
  H_HDV['111_FC_-2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-2','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','345.147844073628','0.038567440332233','20513.1011583039','0.871060048636873','24.5455929728442','6','86']
  H_HDV['111_CO_-4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','4.50943526399938','-0.002747898437064','-0.341414921823582','0.081619786264745','0','6','86']
  H_HDV['111_THC_-4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','0','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.91768079332351','-0.001627544272115','-0.142313504475229','0.082358784832074','0','6','86']
  H_HDV['111_NOx_-4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2624859456431','0.05536216677968','52.3481519770661','0.442039487310386','0.358025940768842','6','86']
  H_HDV['111_PM_-4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.02880738303842','-6.40389835257484','-1.29440832304199','0','0','6','86']
  H_HDV['111_FC_-4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','309.688281743441','0.055399667738539','7838.24363316255','0.709407367509996','9.74661212755083','6','86']
  H_HDV['111_CO_-4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.07626233804411','0.057149893735248','16.4543004976325','0.532916344644133','0.109948858601373','6','86']
  H_HDV['111_THC_-4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.88185909601363','-0.00193803026318','-0.145117341286932','0.086278917943433','0','6','86']
  H_HDV['111_NOx_-4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.0139865398295','0.05078161717367','113.699907955957','0.565854800408246','0.2565323494354','6','86']
  H_HDV['111_PM_-4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','50','PM','y=(1/(a+(b*(x**c))))','2.50848717265315','0.120982526825837','1.30041525300926','0','0','6','86']
  H_HDV['111_FC_-4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','275.605854776855','0.048782509045597','10959.7987992293','0.753940462198153','5.35250335582881','6','86']
  H_HDV['111_CO_-4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.88981375774172','0.050821887399137','38.6047033633156','0.634793252770484','0.084027113253285','6','86']
  H_HDV['111_THC_-4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','100','THC','y=(1/(((c*(x**2))+(b*x))+a))','0.426728885985291','0.06144957100777','0.001386461304209','0','0','6','86']
  H_HDV['111_NOx_-4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.06352384730578','0.045862483534181','208.681446018347','0.658687003726193','0.170607708916555','6','86']
  H_HDV['111_PM_-4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','100','PM','y=((a*(b**x))*(x**c))','0.756298927844','0.985081335353966','-0.50142576275211','0','0','6','86']
  H_HDV['111_FC_-4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','256.688721050524','0.043980758002354','16495.191707444','0.816167519650494','2.52444335371885','6','86']
  H_HDV['111_CO_-6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.41043125591407','0.072794448694851','11.2987807287771','0.611129312558012','0.094511559609026','6','72']
  H_HDV['111_THC_-6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','0','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.32755396865366','0.063526367385234','1.67408742354059','0.355271470089325','0.065186662564654','6','72']
  H_HDV['111_NOx_-6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1413997007029','0.070443936326031','143.537982729254','0.647489753529966','0.247966139131837','6','72']
  H_HDV['111_PM_-6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.015678404173453','0.276637849827999','5.14508698158041','1.97166169126506','-0.002371830559702','6','72']
  H_HDV['111_FC_-6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','278.854492640418','0.065074115191873','11185.5181372807','0.780219312743246','4.48217051739285','6','72']
  H_HDV['111_CO_-6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.69769016869196','0.063752728593485','28.9621419638125','0.62830375510598','0.078645901004602','6','72']
  H_HDV['111_THC_-6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.22415796895691','0.062163340076284','2.3966750909281','0.392587263363283','0.063632250032863','6','72']
  H_HDV['111_NOx_-6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.83529104540972','0.064528919095652','230.973590771235','0.697246648551826','0.213743057513703','6','72']
  H_HDV['111_PM_-6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.244312937384256','0.055298770177834','0.271748101384598','0.263118695248387','0.023479750653953','6','72']
  H_HDV['111_FC_-6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','234.508659793724','0.060353397791201','13695.9122805156','0.800674331925815','3.60701467463357','6','72']
  H_HDV['111_CO_-6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.52774057065138','0.059974218441076','35.8005767431042','0.653918631270623','0.072730601376473','6','72']
  H_HDV['111_THC_-6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.15913812255803','0.061261036490899','2.99931612411771','0.415872132397072','0.062792920340051','6','72']
  H_HDV['111_NOx_-6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.64598738711267','0.060369836933016','295.002785787309','0.72555743064147','0.193676096818219','6','72']
  H_HDV['111_PM_-6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.250532474237108','0.053385448188812','0.276534610850262','0.270066610823029','0.022989569227575','6','72']
  H_HDV['111_FC_-6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','212.107118908589','0.057198130024142','16979.0271469852','0.832551975032103','3.14951865384433','6','72']
  H_HDV['111_CO_2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.57631222949396','0.002444389953779','-0.445038283142613','0.080544666848295','0','6','86']
  H_HDV['111_THC_2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.126900662093374','1.90167852522957','4.88847344948369','1.91854599096399','-0.008589081810997','6','86']
  H_HDV['111_NOx_2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','0','NOx','y=(c+(a*exp(b*x)))','20.979029227188','-0.091960637679084','8.4535850740405','0','0','6','86']
  H_HDV['111_PM_2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.444562192464452','0.000881608066257','-0.020318024299416','0.05688302250689','0','6','86']
  H_HDV['111_FC_2_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','769.371416766523','0.632359439179649','-45.1579153937853','0.077804422202961','0','6','86']
  H_HDV['111_CO_2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.096213889491549','0.017333958029556','-0.000122969458936','0','0','6','86']
  H_HDV['111_THC_2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.182665721413089','1.71744598209559','6.3461329827474','2.457431232526','-0.018014759411214','6','86']
  H_HDV['111_NOx_2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','31.1871228143854','-0.020184635070607','-1.87216599150971','0.095271099011776','0','6','86']
  H_HDV['111_PM_2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','50','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.70781105055297','0.137172657029074','-0.001108335582787','0','0','6','86']
  H_HDV['111_FC_2_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','805.711140076549','0.839871334922036','-38.8501450325688','0.069391043015076','0','6','86']
  H_HDV['111_CO_2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','100','CO','y=(1/(((c*(x**2))+(b*x))+a))','0.100826815858982','0.013090957413036','-0.000068374726824','0','0','6','86']
  H_HDV['111_THC_2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.249702974540107','1.59428574770256','7.43051961456054','2.87501041678731','-0.023658463599635','6','86']
  H_HDV['111_NOx_2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','32.013026192286','-0.030933458109834','-1.60252934164986','0.088473945832802','0','6','86']
  H_HDV['111_PM_2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.512070497780837','0.0104297474066','-0.01192723114316','0.012198375628208','0','6','86']
  H_HDV['111_FC_2_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','842.434583171418','0.908280447975923','-33.671820995445','0.063222854404281','0','6','86']
  H_HDV['111_CO_4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.15571678383242','-0.008678450652607','-0.698721254206529','0.112983974436899','0','6','72']
  H_HDV['111_THC_4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.197777428012837','1.78997909477098','6.40313181995948','2.48287746948085','-0.017172195698696','6','72']
  H_HDV['111_NOx_4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','11.410546681947','24.4214690621172','2.43937551099527','1.205247511446','0.031770914113784','6','72']
  H_HDV['111_PM_4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','0','PM','y=(c+(a*exp(b*x)))','0.33665688077552','-0.069492145952387','0.174876318778368','0','0','6','72']
  H_HDV['111_FC_4_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','0','FC','y=((a*(b**x))*(x**c))','1650.0077724313','1.009699870947','-0.544713917385507','0','0','6','72']
  H_HDV['111_CO_4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.89823322271046','0.012972374787153','6.34225986064374','0.15868408193136','-0.19444351799663','6','71']
  H_HDV['111_THC_4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','50','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','2.94780664350456','-0.001329519994642','-0.238045220204922','0.094119526421698','0','6','71']
  H_HDV['111_NOx_4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','50','NOx','y=((a*(b**x))*(x**c))','53.7130163891272','1.00495740735918','-0.384094576298021','0','0','6','71']
  H_HDV['111_PM_4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','50','PM','y=exp((a+(b/x))+(c*log(x)))','0.34629705419834','-1.61543118254554','-0.463994908068165','0','0','6','71']
  H_HDV['111_FC_4_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','50','FC','y=((a*(b**x))*(x**c))','1494.30508393826','1.00710696904708','-0.4058559276296','0','0','6','71']
  H_HDV['111_CO_4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.27349023081795','0.014686625761673','6.7205811798403','0.157504551316587','-0.609200347583589','6','68']
  H_HDV['111_THC_4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.436259650132024','11.6972860984012','-1.98720750436463','-0.435449518043187','0.132067988664933','6','68']
  H_HDV['111_NOx_4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','100','NOx','y=((a*(b**x))*(x**c))','53.5344284227876','1.00277990611926','-0.308392903454109','0','0','6','68']
  H_HDV['111_PM_4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.39743609166619','0.063112402421501','-0.000268420090541','0','0','6','68']
  H_HDV['111_FC_4_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','4','100','FC','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.003204350601062','0.495149670926537','-25.1144456456031','974.028639301999','0','6','68']
  H_HDV['111_CO_6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.7280871658221','-0.028942767442896','-1.70162188160876','0.200840313899795','0','6','71']
  H_HDV['111_THC_6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.347619285240984','12.3985139514119','-1.8659185398115','-0.318901940390988','0.11449872176959','6','71']
  H_HDV['111_NOx_6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','38.8161265376119','-0.031637635956131','-2.61934505706252','0.116631221664341','0','6','71']
  H_HDV['111_PM_6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','0','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000002688899638','0.000370385548664','-0.018128793628709','0.549962680449258','0','6','71']
  H_HDV['111_FC_6_0'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','0','FC','y=((a*(b**x))*(x**c))','1680.28448749058','1.01024149564918','-0.488489464151693','0','0','6','71']
  H_HDV['111_CO_6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.74126806989499','0.023290420593228','6.18957001880142','0.169214412120323','0.42118710224459','6','65']
  H_HDV['111_THC_6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.450582992375843','12.8264540542526','-2.16125777321574','-0.503486196874043','0.137816155768521','6','65']
  H_HDV['111_NOx_6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','45.082958357848','-0.054522939194351','-2.87982641920664','0.124693084548349','0','6','65']
  H_HDV['111_PM_6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','50','PM','y=(c+(a*exp(b*x)))','0.463347791541852','-0.051938946493791','0.227576905641767','0','0','6','65']
  H_HDV['111_FC_6_50'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','50','FC','y=((a*(b**x))*(x**c))','1579.95740822358','1.00715992710485','-0.34783143772465','0','0','6','65']
  H_HDV['111_CO_6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.0872764469254','-0.047447677754003','-1.15055121713162','0.125069626546604','0','6','52']
  H_HDV['111_THC_6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.544715142291699','11.4624876194232','-2.06883999044548','-0.501487177780281','0.141674657864424','6','52']
  H_HDV['111_NOx_6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','100','NOx','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.00032014459584','0.037047135858511','-1.48391276919469','45.5992779538212','0','6','52']
  H_HDV['111_PM_6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','1.06750321660812','0.062441975213554','-0.000370337376835','0','0','6','52']
  H_HDV['111_FC_6_100'] = [' 111','EURO2','TT/AT >14-20t Euro-2','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','336.224863403572','0.064260974205573','4681.07188142202','0.648887097406325','723.081556805223','6','52']
  H_HDV['112_CO_0_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.40229863120705','0.055099485514346','5.31722510776575','0.172633293190607','0.799159987636525','6','86']
  H_HDV['112_THC_0_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.109444826294952','1.81279535464785','4.32080796491876','1.80630271598891','-0.007967302570731','6','86']
  H_HDV['112_NOx_0_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.0302219947515','0.056174468004308','41.4409363863534','0.296051373810495','3.74721797933917','6','86']
  H_HDV['112_PM_0_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.055065227894227','0.562467000195883','5.50583960243394','2.19080992475608','-0.014164620098475','6','86']
  H_HDV['112_FC_0_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000978391511819','0.000169531376154','-0.000001167046589','0','0','6','86']
  H_HDV['112_CO_0_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.91052494999031','0.057035377078642','6.48373691193773','0.226310429728417','0.928875351610845','6','86']
  H_HDV['112_THC_0_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.110507053420712','1.76265244017845','4.40427155423658','1.84015010129852','-0.008225569976897','6','86']
  H_HDV['112_NOx_0_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.1200232374945','0.049116150074523','43.1526760453362','0.30785590343182','4.13060819163098','6','86']
  H_HDV['112_PM_0_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.033316662757519','0.601645511427657','4.78536091872308','1.95046175589948','-0.018068089728717','6','86']
  H_HDV['112_FC_0_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','50','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00116333410464','0.000126522572969','-0.000000792140945','','','6','86']
  H_HDV['112_CO_0_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','1.03818765995044','33.9452820071762','-0.588743604319895','0.47728014349776','0.044192997460444','6','86']
  H_HDV['112_THC_0_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.10910488850549','1.7624839133754','4.25650083374897','1.7953383959905','-0.007455196947848','6','86']
  H_HDV['112_NOx_0_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.6810420386709','0.043829757437106','46.8069354724156','0.33144114601948','4.41605053197662','6','86']
  H_HDV['112_PM_0_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.369245948723427','0.049751608916093','0.395683882994019','0.161017290344716','0.085834867176987','6','86']
  H_HDV['112_FC_0_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','418.840993270356','0.045712933851657','7717.19681475632','0.759427711789247','167.894673671349','6','86']
  H_HDV['112_CO_-2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.824458602377538','138.404631273262','-3.98423734433141','-0.765979588391558','0.13309020226778','6','86']
  H_HDV['112_THC_-2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.17786771233266','10.4436492324929','-1.83987450552602','-0.165286489276571','0.090059575113811','6','86']
  H_HDV['112_NOx_-2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','14.4035932376923','0.067457844464662','82.533742398308','0.48530884215263','1.62065005089232','6','86']
  H_HDV['112_PM_-2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.080831334621114','28.1334855812377','-4.76397922437323','-0.755349980460583','0.124981594015769','6','86']
  H_HDV['112_FC_-2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','483.018046249627','0.065268477739181','12860.093640914','0.893963859651432','57.4998283590194','6','86']
  H_HDV['112_CO_-2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.55514180579075','0.014641531185377','-0.427955520684594','0.055345522514305','0','6','86']
  H_HDV['112_THC_-2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.075381404930446','1.27763512666747','7.08052758647655','2.64822259747235','-0.018531220776494','6','86']
  H_HDV['112_NOx_-2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.7586469338035','0.050833277369575','87.1853609349518','0.461459709810312','1.09391409387076','6','86']
  H_HDV['112_PM_-2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.072180512764192','22.0839770094224','-4.31590315712682','-0.581108769488275','0.111269931974009','6','86']
  H_HDV['112_FC_-2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','408.852700372181','0.04956404403561','6324.88351282941','0.702651795563613','42.0211838514285','6','86']
  H_HDV['112_CO_-2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','7.6401353702139','0.010324274231368','-0.432176258097437','0.056521562833041','0','6','86']
  H_HDV['112_THC_-2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.042891171575881','1.30203846984654','6.21797656835684','2.33202313391774','-0.013103652413863','6','86']
  H_HDV['112_NOx_-2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.7736267144922','0.041034591461988','105.532050769532','0.481092578749698','0.717281072301686','6','86']
  H_HDV['112_PM_-2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','100','PM','y=(c+(a*exp(b*x)))','0.648869787466181','-0.065476468674874','0.058854009680485','0','0','6','86']
  H_HDV['112_FC_-2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-2','100','FC','y=((a*(b**x))*(x**c))','1003.05719668729','0.984403081218931','-0.43628042271917','0','0','6','86']
  H_HDV['112_CO_-4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.570451812444648','0.003886182883641','6.98478206153345','0.072073418238214','-0.139733948643615','6','86']
  H_HDV['112_THC_-4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.024046965800136','1.27710178151792','6.50474781274419','2.44311053294927','-0.013423324632786','6','86']
  H_HDV['112_NOx_-4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.2720193676108','0.063843388911146','82.897645674903','0.46509574187302','0.399938183703689','6','86']
  H_HDV['112_PM_-4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.81518793682239','-10.0318415046316','-1.58984718476494','0','0','6','86']
  H_HDV['112_FC_-4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','364.204526074348','0.055920156342342','2326.91156687212','0.514117052337805','13.9370009657786','6','86']
  H_HDV['112_CO_-4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.68825795167818','0.003777422583296','6.62865727578787','0.078908948901182','-1.03451007285008','6','86']
  H_HDV['112_THC_-4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.017860733393119','1.23194451569576','5.80681875049609','2.22105785608898','-0.009503527875757','6','86']
  H_HDV['112_NOx_-4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','9.78434860729122','0.060148284785696','159.989345510796','0.574782585344812','0.367884196464854','6','86']
  H_HDV['112_PM_-4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','50','PM','y=exp((a+(b/x))+(c*log(x)))','3.31592197331256','-8.47182558927622','-1.48120606461039','0','0','6','86']
  H_HDV['112_FC_-4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','323.848627558451','0.050681447509687','4744.26987786431','0.618278067836122','10.3053881635007','6','86']
  H_HDV['112_CO_-4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','6.98049169021918','-0.005954482200625','-0.500590833708448','0.078342972623583','0','6','86']
  H_HDV['112_THC_-4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.62195526770998','-0.001099834548325','-0.124585376957918','0.083737605994572','0','6','86']
  H_HDV['112_NOx_-4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.48950993232971','0.054448882167935','274.663760236602','0.659366472980653','0.31228994100798','6','86']
  H_HDV['112_PM_-4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.022316552722502','0.47378651230486','6.67375583045925','2.47434654111156','-0.015216554737371','6','86']
  H_HDV['112_FC_-4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','297.083642366692','0.045805612830331','8581.118174792','0.708858403024552','7.55062454819133','6','86']
  H_HDV['112_CO_-6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','0','CO','y=exp((a+(b/x))+(c*log(x)))','7.50568113156265','-13.8341446040466','-2.10578895111265','0','0','6','72']
  H_HDV['112_THC_-6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.038995245054054','1.18860370408892','5.43142122378914','2.06855656226395','0.004387855479631','6','72']
  H_HDV['112_NOx_-6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','10.2165843038246','0.078032785981003','154.151266534314','0.58610601152006','0.300781311452791','6','72']
  H_HDV['112_PM_-6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','0','PM','y=exp((a+(b/x))+(c*log(x)))','3.61758177514339','-9.34890404504084','-1.57091622864892','0','0','6','72']
  H_HDV['112_FC_-6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','354.43148712651','0.068149556822545','5986.27227261895','0.700497512752374','10.6177777888097','6','72']
  H_HDV['112_CO_-6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','50','CO','y=(c+(a*exp(b*x)))','5.77957147564304','-0.072334367127258','0.181815753001761','0','0','6','72']
  H_HDV['112_THC_-6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.17060322941105','0.070442722518624','1.37803850455947','0.383897419583724','0.060489517554594','6','72']
  H_HDV['112_NOx_-6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.29518992867056','0.069527318769318','257.126352183932','0.652825022287728','0.262503197102531','6','72']
  H_HDV['112_PM_-6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','50','PM','y=exp((a+(b/x))+(c*log(x)))','2.86876176750437','-7.32670180117868','-1.39080639013978','0','0','6','72']
  H_HDV['112_FC_-6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','286.107383897732','0.062091049935452','9512.06086124135','0.749288379272358','9.37579636028735','6','72']
  H_HDV['112_CO_-6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.64790995823944','0.064135281517993','18.706131114817','0.634300912609357','0.155332505853043','6','72']
  H_HDV['112_THC_-6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','1.00812000341315','0.067570905296257','4.46811471083543','0.53483537867108','0.058360634286544','6','72']
  H_HDV['112_NOx_-6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.89704589219623','0.063822645914952','351.613611625735','0.699841718206219','0.243468461265459','6','72']
  H_HDV['112_PM_-6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.019974611476173','0.438031868788294','6.013720689341','2.26830729001507','-0.013644874836258','6','72']
  H_HDV['112_FC_-6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','253.61321419084','0.058251382043505','12662.3614884294','0.78669686996024','8.67497686908049','6','72']
  H_HDV['112_CO_2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','0','CO','y=(c+(a*exp(b*x)))','7.73903087000176','-0.07777739369941','1.08697217964229','0','0','6','86']
  H_HDV['112_THC_2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.128891499020763','1.64324585864673','5.82734833861361','2.23588396028517','-0.012364706738666','6','86']
  H_HDV['112_NOx_2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','0','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','5.85326438898061','33.7790080063348','2.76761236240057','1.61341113231264','-0.000803228885749','6','86']
  H_HDV['112_PM_2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.74676760066351','-0.000343040410202','-0.048671937987835','0.078055123439714','0','6','86']
  H_HDV['112_FC_2_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','837.080042642792','0.595414422639485','-50.5012488986741','0.079220696031606','0','6','86']
  H_HDV['112_CO_2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','8.8792542829266','0.006504706490472','-0.529253905634561','0.066410853265511','0','6','86']
  H_HDV['112_THC_2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.164141837585115','1.53850992322643','6.73945436318089','2.58066270787162','-0.017725167799431','6','86']
  H_HDV['112_NOx_2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','50','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','6.40876480688927','118.345988015462','0.205358232876352','1.09368921845218','-0.000559648310539','6','86']
  H_HDV['112_PM_2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','50','PM','y=(c+(a*exp(b*x)))','0.592407292728042','-0.069047882762368','0.115821327291378','0','0','6','86']
  H_HDV['112_FC_2_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','864.481991388877','0.661177484470319','-43.4754565072853','0.072332229325931','0','6','86']
  H_HDV['112_CO_2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.19972503152758','0.006383799244279','-0.502867478944463','0.061268400852079','0','6','86']
  H_HDV['112_THC_2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.20742434032772','1.44531825072161','7.59149932860286','2.91042649707289','-0.022413475723241','6','86']
  H_HDV['112_NOx_2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','3.0862567595401','3.39342389551848','-0.218511124365127','0','0','6','86']
  H_HDV['112_PM_2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.675410899765057','0.001065954157566','-0.032318965257356','0.053571627137188','0','6','86']
  H_HDV['112_FC_2_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','886.794710037603','0.813526127134674','-35.7960164997534','0.06326208326688','0','6','86']
  H_HDV['112_CO_4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','9.27615379511663','-0.012579099858747','-0.7065655139755','0.095950932255456','0','6','72']
  H_HDV['112_THC_4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.179213156058443','1.5432361577555','7.27133867716561','2.7733241704354','-0.021535171836346','6','72']
  H_HDV['112_NOx_4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','46.5728426344203','-0.044128526982672','-5.60724019805864','0.15845794664797','0','6','72']
  H_HDV['112_PM_4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.127631762382285','8.67304574225459','-3.20879499407912','-0.35187020510365','0.09363490002248','6','72']
  H_HDV['112_FC_4_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','961.853549482312','0.363162011928326','-60.201625531942','0.092363166278589','0','6','72']
  H_HDV['112_CO_4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.0402293148523','-0.022269332396775','-0.767271350206821','0.10228311533007','0','6','71']
  H_HDV['112_THC_4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.235209385995574','1.46182041877734','7.90560801170381','3.042624058474','-0.025562743104396','6','71']
  H_HDV['112_NOx_4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.73735695844238','0.029610843775269','36.4154172104681','0.191493748320585','9.93817280457013','6','71']
  H_HDV['112_PM_4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.15239834735056','6.23951507849492','-2.91955102922286','-0.37764028947815','0.096927090497606','6','71']
  H_HDV['112_FC_4_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','50','FC','y=((a*(b**x))*(x**c))','1571.20014087256','1.00678218046175','-0.40785454410613','0','0','6','71']
  H_HDV['112_CO_4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','11.0618526105594','-0.029981467096429','-0.827916589511339','0.102349381202048','0','6','68']
  H_HDV['112_THC_4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.347783241964831','16.5562904458374','-2.50709557934017','-0.411839601656416','0.121767846148654','6','68']
  H_HDV['112_NOx_4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','51.1183429036682','-0.054220376106435','-4.99715121022843','0.144592173806026','0','6','68']
  H_HDV['112_PM_4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.196049579616957','5.595881481596','-3.31377171035502','-0.748341081670896','0.132977278486035','6','68']
  H_HDV['112_FC_4_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','4','100','FC','y=(a/(1+(b*exp(((-1)*c)*x))))','553.222764304367','-0.557869136956247','0.066831144394795','0','0','6','68']
  H_HDV['112_CO_6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','10.1025426103969','-0.023960259599514','-0.892562503813705','0.119891901245641','0','6','71']
  H_HDV['112_THC_6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.295414373788273','18.8068104440144','-2.54382152417862','-0.35701794831756','0.111316739509922','6','71']
  H_HDV['112_NOx_6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.7680545947257','0.011365571904029','36.2383507932704','0.168129480392093','7.14108244835126','6','71']
  H_HDV['112_PM_6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.701076900400532','-0.000470438751036','-0.040206743439279','0.076289333452008','0','6','71']
  H_HDV['112_FC_6_0'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','998.624510897068','1.14479351354369','-48.9407507418686','0.077971250959164','0','6','71']
  H_HDV['112_CO_6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','50','CO','y=exp((a+(b/x))+(c*log(x)))','4.39921094063338','-4.33305765221548','-0.967194965569948','0','0','6','66']
  H_HDV['112_THC_6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','-16.4073197635178','0.671736394486487','2.71664196663707','0.097530339055572','0.358938373731876','6','66']
  H_HDV['112_NOx_6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','50','NOx','y=((a*(b**x))*(x**c))','82.6942596836323','1.00881782366617','-0.553344096082579','0','0','6','66']
  H_HDV['112_PM_6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.208573326747008','5.39669502620271','-4.05152835145338','-1.33688397803658','0.1872989000261','6','66']
  H_HDV['112_FC_6_50'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1108.69650712175','1.27173785464859','-41.5795973103098','0.071568029439428','0','6','66']
  H_HDV['112_CO_6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','12.159487392003','-0.039068095635536','-0.888463177694275','0.100256143776462','0','6','54']
  H_HDV['112_THC_6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.344308721879655','1.37030323829688','8.06898526640156','3.16569254363368','-0.031798685106475','6','54']
  H_HDV['112_NOx_6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','100','NOx','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','17.5097811659196','36.1203539694031','3.37987882127061','1.82959967747047','-0.009608277508395','6','54']
  H_HDV['112_PM_6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','100','PM','y=((((a*(x**3))+(b*(x**2)))+(c*x))+d)','-0.000005209263893','0.000680372705202','-0.030973730578686','0.753912841526932','0','6','54']
  H_HDV['112_FC_6_100'] = [' 112','EURO3','TT/AT >14-20t Euro-3','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','340.437076389175','0.053808347116761','2023.55635380503','0.48215364553299','720.512611812026','6','54']
  H_HDV['113_CO_0_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.03672500346471','0.522416777923915','4.47992706081853','1.88003999142955','-0.010535700925701','6','86']
  H_HDV['113_THC_0_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004536083453489','0.103997656328505','4.26509455109848','1.81305036093396','-0.009505436559563','6','86']
  H_HDV['113_NOx_0_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.2592549213702','0.059992420788439','27.1025431837648','0.383572493069806','2.28289526806489','6','86']
  H_HDV['113_PM_0_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.058395408120297','0.042283433180723','0.186599052942216','0.158589352455509','0.012738546473516','6','86']
  H_HDV['113_FC_0_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.000997557830982','0.000182088875576','-0.000001255354586','0','0','6','86']
  H_HDV['113_CO_0_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.041209275202484','0.555454065405693','3.89074711271043','1.69613701120423','-0.007470535739057','6','86']
  H_HDV['113_THC_0_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004471999901274','0.100675883395676','4.42947349581235','1.88074444925949','-0.010900227042966','6','86']
  H_HDV['113_NOx_0_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.09938114054791','0.049352243294799','32.8650302907364','0.418186009610005','2.57893513267952','6','86']
  H_HDV['113_PM_0_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.058638748740804','0.041301176100472','0.184785422299835','0.160585896081414','0.013038162409481','6','86']
  H_HDV['113_FC_0_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','401.343135102226','0.054651144846576','3192.62500358205','0.597835610235886','143.754114802714','6','86']
  H_HDV['113_CO_0_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.039899067067832','0.570357124714041','3.64258375756261','1.59454497631256','-0.006592625679973','6','86']
  H_HDV['113_THC_0_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004556363636046','0.100061598188242','4.36884308644364','1.8705940938745','-0.010498316161166','6','86']
  H_HDV['113_NOx_0_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.48050978283153','0.042519938506265','48.302475622684','0.498693386713071','2.78184073557556','6','86']
  H_HDV['113_PM_0_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061292933125052','0.041281325627654','0.181018024973982','0.16278943185209','0.013157971796711','6','86']
  H_HDV['113_FC_0_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','394.269895167775','0.045501355803482','5847.89632488226','0.702188323549878','156.031171189446','6','86']
  H_HDV['113_CO_-2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.05218499858259','6.53866347309401','-3.0540590990789','-0.426199864652905','0.107029333857685','6','86']
  H_HDV['113_THC_-2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.009569515591658','0.620837729053143','-1.9445264684515','-0.202022679146285','0.09460970949455','6','86']
  H_HDV['113_NOx_-2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','7.17413867003596','0.063172386800152','73.1201083894701','0.588784596701458','0.872634296900886','6','86']
  H_HDV['113_PM_-2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006635855073499','0.132367680394294','6.10147181371726','2.46226871990708','-0.01673681610915','6','86']
  H_HDV['113_FC_-2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','457.32432012692','0.066259992284678','13448.7719871902','0.888479736839811','52.9213515874484','6','86']
  H_HDV['113_CO_-2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040180278594765','3.46153145358688','-2.07761934993118','-0.189739549281415','0.087483703588346','6','86']
  H_HDV['113_THC_-2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','50','THC','y=(c+(a*exp(b*x)))','0.099816810303493','-0.073183528640951','0.007145897574655','0','0','6','86']
  H_HDV['113_NOx_-2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.01708937933475','0.045565573753099','71.2704272426302','0.547636017257921','0.576857853847129','6','86']
  H_HDV['113_PM_-2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004083973964238','0.137401647466053','5.38707181646943','2.1971656250273','-0.011745411749971','6','86']
  H_HDV['113_FC_-2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','390.48894446004','0.05060057872075','7914.65397652248','0.742396720871266','37.6676269823901','6','86']
  H_HDV['113_CO_-2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011859201357773','0.377504469192461','6.42352870467212','2.37682511669106','-0.013278972327648','6','86']
  H_HDV['113_THC_-2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002353389357229','0.072562139901454','6.5803293979087','2.4912697047879','-0.016713521039515','6','86']
  H_HDV['113_NOx_-2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.822490837937','0.03590423371952','90.8332207973792','0.577717587441788','0.310653183556501','6','86']
  H_HDV['113_PM_-2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001666166277035','0.147735560565908','4.61040049264572','1.93019405822208','-0.00802574804956','6','86']
  H_HDV['113_FC_-2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-2','100','FC','y=((a*(b**x))*(x**c))','961.657494340465','0.983605484674228','-0.44413964394492','0','0','6','86']
  H_HDV['113_CO_-4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','0','CO','y=exp((a+(b/x))+(c*log(x)))','4.59322027703999','-13.0277901476747','-1.98119683485629','0','0','6','86']
  H_HDV['113_THC_-4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001027311832531','0.071968888261269','6.58904533843431','2.48082647851945','-0.014077778876073','6','86']
  H_HDV['113_NOx_-4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.56517894876966','0.056853406417763','57.6717678107681','0.515920874292823','0.142442211393691','6','86']
  H_HDV['113_PM_-4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.39075992572509','-8.48739841889836','-1.76451229006717','0','0','6','86']
  H_HDV['113_FC_-4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','356.859367696174','0.058829284521144','3536.40393995579','0.593875318658827','10.1907813823234','6','86']
  H_HDV['113_CO_-4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','50','CO','y=(1/(((c*(x**2))+(b*x))+a))','2.81690387669927','-0.008519536479926','0.01353576430655','0','0','6','86']
  H_HDV['113_THC_-4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000451997813425','0.070947487830767','5.71990285428325','2.20226868641174','-0.0094091670399','6','86']
  H_HDV['113_NOx_-4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.93623671596718','0.051151930988988','88.5772397645737','0.58296731299221','0.081281587743178','6','86']
  H_HDV['113_PM_-4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.188743712799923','-0.00012749661907','-0.017884530392014','0.101491616020192','0','6','86']
  H_HDV['113_FC_-4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','310.927025390732','0.052922746062827','6272.3883450308','0.673334229544275','6.30711806176455','6','86']
  H_HDV['113_CO_-4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.449773024461743','-0.000319405285083','-0.033525364643383','0.08007708689759','0','6','86']
  H_HDV['113_THC_-4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.093585862062367','-0.00006716648331','-0.007397196914397','0.085814416823304','0','6','86']
  H_HDV['113_NOx_-4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.54712316687305','0.046430153158245','131.921310832801','0.648402866754698','0.04038551659723','6','86']
  H_HDV['113_PM_-4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','4.03175276973848','0.710955400892019','0.033313001713582','0','0','6','86']
  H_HDV['113_FC_-4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','278.273143984698','0.047503885103193','10577.9710660918','0.74782421032043','3.68079914539678','6','86']
  H_HDV['113_CO_-6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','3.53077689927318','-0.126256242896278','0.020487818721332','0','0','6','72']
  H_HDV['113_THC_-6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002218080717297','0.069891556602409','4.77523776822153','1.77459085868067','0.019082798462968','6','72']
  H_HDV['113_NOx_-6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.32336054069283','0.069626101292533','96.5792551621002','0.625279087655496','0.059263710290827','6','72']
  H_HDV['113_PM_-6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.086489039265889','0.066244817561099','0.114402073216626','0.159612809999201','0.003479317916963','6','72']
  H_HDV['113_FC_-6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','333.94832903486','0.070196420405085','8372.61219866377','0.755159013507269','5.02031216927728','6','72']
  H_HDV['113_CO_-6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.352952428857558','0.071727832108377','1.64645449126901','0.731768590792845','0.00881036043019','6','72']
  H_HDV['113_THC_-6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068436782375291','0.070745585627784','0.098039066668033','0.429189247825472','0.002836513816884','6','72']
  H_HDV['113_NOx_-6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.40404236891803','0.064912393461364','139.961756387307','0.676263331443565','0.046035858248308','6','72']
  H_HDV['113_PM_-6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.101088127884169','0.071465336744414','0.204849837161537','0.31503156209273','0.003520013456504','6','72']
  H_HDV['113_FC_-6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','262.594552866508','0.063980491136721','11894.1165717537','0.783779129529589','3.91164838622176','6','72']
  H_HDV['113_CO_-6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.281553170454878','0.065767281908417','4.19947421652134','0.761353056338786','0.007655696465219','6','72']
  H_HDV['113_THC_-6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.058917179002485','0.06753540156006','0.300093434853482','0.565405008599841','0.002703502783254','6','72']
  H_HDV['113_NOx_-6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.90042631285622','0.061323984888941','175.771947306057','0.709360931875548','0.038282169613424','6','72']
  H_HDV['113_PM_-6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.093098749979799','0.069998148177367','0.350564402550823','0.401288852925985','0.003423584552101','6','72']
  H_HDV['113_FC_-6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','224.734314138003','0.059381697892203','15192.5832905696','0.812443813788116','3.18460312651077','6','72']
  H_HDV['113_CO_2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.08052110714933','0.413373081670221','7.25880041544721','2.81316828573167','-0.018643350198521','6','86']
  H_HDV['113_THC_2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005121938128262','0.093750272765599','5.66535375473258','2.20071449011521','-0.012099283132744','6','86']
  H_HDV['113_NOx_2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','11.5381767492921','0.103040967911899','78.6665198755337','0.776833360055946','4.1194659018589','6','86']
  H_HDV['113_PM_2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.032771843063119','0.011980125315234','0.191285435852523','0.111470754313424','0.005227798103441','6','86']
  H_HDV['113_FC_2_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','808.739999922824','0.431330483036119','-50.3862548871021','0.081963520395594','0','6','86']
  H_HDV['113_CO_2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.792662801906104','-0.000707672015387','-0.068926821348261','0.107678955432141','0','6','86']
  H_HDV['113_THC_2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006233263911805','0.088754887367833','6.44153687014459','2.49796879462619','-0.017072252643854','6','86']
  H_HDV['113_NOx_2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','18.7171830435785','-0.012644326427833','-1.57140957247622','0.122732078555929','0','6','86']
  H_HDV['113_PM_2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.035931829837676','0.016428637801431','0.190088773413059','0.116607655261571','0.009589326458372','6','86']
  H_HDV['113_FC_2_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','50','FC','y=((a*(b**x))*(x**c))','1632.31331982991','1.00844197972756','-0.550232682093085','0','0','6','86']
  H_HDV['113_CO_2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.866049302664468','-0.001516353418381','-0.084157859047962','0.126537914514711','0','6','86']
  H_HDV['113_THC_2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007925899699097','0.083482111949799','7.33204451919593','2.84421329579969','-0.02297175962582','6','86']
  H_HDV['113_NOx_2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','100','NOx','y=exp((a+(b/x))+(c*log(x)))','2.55293930151698','2.27548556904111','-0.18932977815031','0','0','6','86']
  H_HDV['113_PM_2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048699878332937','0.025404615154756','0.18780238251183','0.132242417792253','0.013081531032617','6','86']
  H_HDV['113_FC_2_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','863.699680599349','0.519512925177053','-37.9436496291872','0.069237700219981','0','6','86']
  H_HDV['113_CO_4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.11010973947923','0.414981640791273','6.8100040882218','2.69067369488331','-0.018629281383806','6','72']
  H_HDV['113_THC_4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006223644919227','0.089683461673426','6.66358661450753','2.56452896028185','-0.018340940144966','6','72']
  H_HDV['113_NOx_4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','22.1837331620889','-0.013466218516396','-2.4014472755751','0.153014021107384','0','6','72']
  H_HDV['113_PM_4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.23246040695674','-0.000253820418671','-0.021521113050252','0.110351513345826','0','6','72']
  H_HDV['113_FC_4_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','0','FC','y=((a*(b**x))*(x**c))','1727.83142922855','1.009728015966','-0.562965100389647','0','0','6','72']
  H_HDV['113_CO_4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.872000477033612','-0.001784473548165','-0.081113694410985','0.126412261227335','0','6','71']
  H_HDV['113_THC_4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008459381867503','0.084561531976093','7.41410935992997','2.87170551334479','-0.022791168613556','6','71']
  H_HDV['113_NOx_4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.21288837325444','0.029119096642902','19.6949098996874','0.246005417656381','6.63259587693158','6','71']
  H_HDV['113_PM_4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055482863352583','0.020734770928677','0.189397637633576','0.132282084453221','0.009292234547797','6','71']
  H_HDV['113_FC_4_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','50','FC','y=((a*(b**x))*(x**c))','1555.81629105674','1.00736273391571','-0.427874721026937','0','0','6','71']
  H_HDV['113_CO_4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.942568035711805','-0.002652021079842','-0.083798566263156','0.123427377889278','0','6','68']
  H_HDV['113_THC_4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010664153202135','0.080328039040765','7.97960288109773','3.11881101036526','-0.027891580876341','6','68']
  H_HDV['113_NOx_4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.52180656074961','0.018516307554512','20.0468204808477','0.243184784415817','6.99980196698179','6','68']
  H_HDV['113_PM_4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.069289000602482','0.0305606056296','0.183673890879671','0.140198179330316','0.013859565593762','6','68']
  H_HDV['113_FC_4_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','469.716392326743','0.08098662787535','49677.9012403966','1.14864961724427','522.918070741439','6','68']
  H_HDV['113_CO_6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.04071492753743','-0.002355666923009','-0.139967185557387','0.182513324887498','0','6','71']
  H_HDV['113_THC_6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008176930631723','0.084011949241137','8.03113593788796','3.09507723809825','-0.028923787826509','6','71']
  H_HDV['113_NOx_6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','27.6578450707947','-0.027977500935854','-3.76213492684268','0.199619149123231','0','6','71']
  H_HDV['113_PM_6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.262598424794579','-0.000462111566381','-0.028686639928121','0.135483584090324','0','6','71']
  H_HDV['113_FC_6_0'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','0','FC','y=((a*(b**x))*(x**c))','1721.54176071861','1.00992693879086','-0.49813723882522','0','0','6','71']
  H_HDV['113_CO_6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.03346913896881','-0.003494707191649','-0.109852812833724','0.15257137373634','0','6','67']
  H_HDV['113_THC_6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011303060655087','0.078626586866273','8.73871571824233','3.41621491263199','-0.035357822869246','6','67']
  H_HDV['113_NOx_6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.32492064494419','0.027134788151807','20.5670020404231','0.256329820388549','8.26611363057691','6','67']
  H_HDV['113_PM_6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.061912734578911','0.016271555843806','0.196752140411333','0.122986741690661','0.002807025177458','6','67']
  H_HDV['113_FC_6_50'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1077.30053454048','1.24662143968646','-43.378085881099','0.074992061709733','0','6','67']
  H_HDV['113_CO_6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','1.93496171504008','-3.65962287989678','-0.957152417104496','0','0','6','55']
  H_HDV['113_THC_6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.012617060744955','0.078652907771276','7.83338857703801','3.09923170761865','-0.030134916250959','6','55']
  H_HDV['113_NOx_6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','8.01070766615388','0.059735085873626','28.5572286132485','0.374890385735599','11.5613204969939','6','55']
  H_HDV['113_PM_6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.275606764098949','-0.000566733816636','-0.025714877582495','0.114711231385255','0','6','55']
  H_HDV['113_FC_6_100'] = [' 113','EURO4','TT/AT >14-20t Euro-4','6','100','FC','y=exp((a+(b/x))+(c*log(x)))','6.78539300927119','1.61798526441277','-0.066860707058173','0','0','6','55']
  H_HDV['114_CO_0_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.036905742536757','0.531338329974846','4.4924452848714','1.88203280770232','-0.010506510469349','6','86']
  H_HDV['114_THC_0_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004627729071915','0.105984015310604','4.26414443099866','1.81166370136725','-0.009470275384261','6','86']
  H_HDV['114_NOx_0_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.80463145167893','0.058240337588003','18.6648555011191','0.360000829309486','1.33822984756719','6','86']
  H_HDV['114_PM_0_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.059651223751123','0.04228170274193','0.189982381591994','0.15856048614978','0.012879998906062','6','86']
  H_HDV['114_FC_0_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','0','FC','y=(1/(((c*(x**2))+(b*x))+a))','0.00095582210583','0.000178619901309','-0.000001222509763','0','0','6','86']
  H_HDV['114_CO_0_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.041819018650182','0.560502705843916','3.94973243143321','1.71351821255965','-0.007628965914665','6','86']
  H_HDV['114_THC_0_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004551640742519','0.10250951277415','4.43854660350436','1.882953015839','-0.010982929020205','6','86']
  H_HDV['114_NOx_0_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.72885667044052','0.048482639741599','21.1551188542912','0.383003468512184','1.50817715693198','6','86']
  H_HDV['114_PM_0_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.059379092051235','0.041160771839882','0.188225565062943','0.159883489745284','0.013189514965054','6','86']
  H_HDV['114_FC_0_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','412.54601034572','0.054467525333475','2968.98327039181','0.578285344305591','145.703650933853','6','86']
  H_HDV['114_CO_0_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.042157947731195','0.584315394272158','3.58731978550817','1.57758760666584','-0.00538296345035','6','86']
  H_HDV['114_THC_0_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004611141023514','0.101943569930408','4.38211882722761','1.87472864931838','-0.010670050725557','6','86']
  H_HDV['114_NOx_0_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.04759825000726','0.043042284192406','27.3918239374312','0.441331407135571','1.62267599046877','6','86']
  H_HDV['114_PM_0_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.062393675965235','0.041244956257441','0.18469334613506','0.162849966002346','0.013319030942868','6','86']
  H_HDV['114_FC_0_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','0','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','404.643285597661','0.045431137075515','4969.78216617346','0.666073575441857','158.050880463184','6','86']
  H_HDV['114_CO_-2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.053110353554586','5.40405530032217','-2.81132688297208','-0.414677638449906','0.106527147244946','6','86']
  H_HDV['114_THC_-2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00974830829103','0.62394406052732','-1.91939767787103','-0.196607050647068','0.094091578869314','6','86']
  H_HDV['114_NOx_-2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.42836082962684','0.062788859588407','39.6277888768293','0.52157493178917','0.511980798423433','6','86']
  H_HDV['114_PM_-2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006693435851626','0.135193112247231','6.09220782602706','2.45799291837189','-0.016718580308805','6','86']
  H_HDV['114_FC_-2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','471.621301451588','0.066137721651499','12489.0489656261','0.868757184830828','53.9188637937674','6','86']
  H_HDV['114_CO_-2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.040908795558346','3.67501529022651','-2.13390848858487','-0.198438111114238','0.087963799777408','6','86']
  H_HDV['114_THC_-2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003760759587736','0.072781294793547','7.53129358124336','2.84786255870895','-0.025171193509973','6','86']
  H_HDV['114_NOx_-2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.73604993187501','0.046112634412918','41.5840317055496','0.502481025104272','0.339621283380079','6','86']
  H_HDV['114_PM_-2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','50','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.004148818408082','0.139921216557297','5.41007789048871','2.20493360431222','-0.011910664378906','6','86']
  H_HDV['114_FC_-2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','402.940461493887','0.05078343063101','7402.2124532267','0.725063633632404','38.4726874888507','6','86']
  H_HDV['114_CO_-2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','100','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.011949894019885','0.385251682188508','6.43899921800861','2.38303113626333','-0.013420913624002','6','86']
  H_HDV['114_THC_-2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.00238168292772','0.074140036803944','6.57825391134512','2.49008479468366','-0.0167166382468','6','86']
  H_HDV['114_NOx_-2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.60604606237419','0.036873115445983','51.375438487897','0.530145070161544','0.189255242143306','6','86']
  H_HDV['114_PM_-2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','100','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.001676843624031','0.150716633689265','4.61722400473076','1.93307300408466','-0.008088428143234','6','86']
  H_HDV['114_FC_-2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-2','100','FC','y=((a*(b**x))*(x**c))','1001.75699934787','0.983636763247797','-0.449765856636647','0','0','6','86']
  H_HDV['114_CO_-4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','0','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.149587803430996','0.001685414146124','0.498126503224553','0.075355850883141','-0.116170969252239','6','86']
  H_HDV['114_THC_-4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.0010342458875','0.073582392329857','6.59750070194822','2.4834303650307','-0.014154226183334','6','86']
  H_HDV['114_NOx_-4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.51893070710043','0.057759756178668','37.1482978428225','0.490651089899824','0.085924975175611','6','86']
  H_HDV['114_PM_-4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','0','PM','y=exp((a+(b/x))+(c*log(x)))','2.42569979636129','-8.53176353753251','-1.76819016014207','0','0','6','86']
  H_HDV['114_FC_-4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','369.565334900305','0.059039285554666','3472.10427837637','0.585847339134993','10.5163817095314','6','86']
  H_HDV['114_CO_-4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','50','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','-0.000857521271753','0.34694739324167','6.50570510095932','2.42650438605234','-0.011719247170304','6','86']
  H_HDV['114_THC_-4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.000474916181365','0.072184194899655','5.78073579960911','2.22142952507927','-0.009696058350431','6','86']
  H_HDV['114_NOx_-4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.14351214991224','0.05253479449187','53.3740779622777','0.549008614230539','0.051701281940061','6','86']
  H_HDV['114_PM_-4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.192561943322393','-0.000129415399564','-0.018222670741762','0.101331207424125','0','6','86']
  H_HDV['114_FC_-4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','322.588938829498','0.053303704175143','6166.34603217783','0.665933517193702','6.59844784014092','6','86']
  H_HDV['114_CO_-4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.460247490109786','-0.000324019971279','-0.034342758536559','0.080121127679608','0','6','86']
  H_HDV['114_THC_-4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','100','THC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.095682423191217','-0.000068265833486','-0.007555108799943','0.085686861535356','0','6','86']
  H_HDV['114_NOx_-4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.88945957514233','0.047870103332368','75.3202064995482','0.603412785452087','0.028222161184542','6','86']
  H_HDV['114_PM_-4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','100','PM','y=(1/(((c*(x**2))+(b*x))+a))','3.99670134844465','0.686569879144635','0.032945983579495','0','0','6','86']
  H_HDV['114_FC_-4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','288.32068991922','0.047900688095208','10430.3002412966','0.740235692901113','3.94566331077294','6','86']
  H_HDV['114_CO_-6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','0','CO','y=(1/(((c*(x**2))+(b*x))+a))','3.48230704164875','-0.128772463976243','0.020194517113526','0','0','6','72']
  H_HDV['114_THC_-6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.002191677999177','0.070485682568925','4.95978151541743','1.84445430672582','0.016549702833968','6','72']
  H_HDV['114_NOx_-6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.42641211238066','0.070549387035385','58.4821631666643','0.590047358752224','0.036217157824218','6','72']
  H_HDV['114_PM_-6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','0','PM','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.003006630921165','0.172434298352327','2.97383852498959','1.31371129261418','0.032271714286688','6','72']
  H_HDV['114_FC_-6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','0','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','347.580390523448','0.070533729122578','8610.19322712649','0.756407352568835','5.23071057027933','6','72']
  H_HDV['114_CO_-6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','50','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.36435357813615','0.072166286035267','1.53669337962532','0.73444856592138','0.009052926261942','6','72']
  H_HDV['114_THC_-6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','50','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.070050095922166','0.070813360262423','0.087150002030697','0.408786547298666','0.002892119818139','6','72']
  H_HDV['114_NOx_-6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.83766903509721','0.065722843752893','79.0649913310258','0.631061513742379','0.027471288016156','6','72']
  H_HDV['114_PM_-6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.10313146504232','0.071475078716047','0.199119475882798','0.307154200723929','0.00358595903364','6','72']
  H_HDV['114_FC_-6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','50','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','272.801553688652','0.064312051552587','11835.5593253561','0.778168475247409','4.09250285893149','6','72']
  H_HDV['114_CO_-6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.289755220042501','0.066106198054134','4.13375775679605','0.760160577482981','0.00786077569576','6','72']
  H_HDV['114_THC_-6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','100','THC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.060578447927352','0.067786804845654','0.300445397041634','0.565738282465814','0.002762368795232','6','72']
  H_HDV['114_NOx_-6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','2.54220518942918','0.062524184618867','98.6052395266814','0.665382313345271','0.023241542643406','6','72']
  H_HDV['114_PM_-6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.09520027140628','0.070097361550084','0.34673856987286','0.396656343082279','0.003491874819505','6','72']
  H_HDV['114_FC_-6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','-6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','233.196873457403','0.059689476597777','15258.3124750688','0.808258945733835','3.33843351978963','6','72']
  H_HDV['114_CO_2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.079908836310841','0.424988095749131','7.12576731874248','2.76174715330965','-0.01774543473915','6','86']
  H_HDV['114_THC_2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.005197359226651','0.095814985324602','5.63568586960515','2.19036590560211','-0.011953278228922','6','86']
  H_HDV['114_NOx_2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','0','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','6.51767129819967','0.096334498407175','16.8634995585727','0.446083581122706','2.43252910519996','6','86']
  H_HDV['114_PM_2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','0','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.033162318763212','0.012441380183333','0.194905138581978','0.111760771363278','0.005728955939776','6','86']
  H_HDV['114_FC_2_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','0','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','834.435214802057','0.420478177463417','-52.3921045848709','0.082303464347792','0','6','86']
  H_HDV['114_CO_2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.802466303429104','-0.000693958018128','-0.069373469517929','0.10676527066919','0','6','86']
  H_HDV['114_THC_2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006297356734617','0.090508904634296','6.4399543708564','2.49621221496101','-0.017105300778242','6','86']
  H_HDV['114_NOx_2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','13.5708254564224','-0.01121631374175','-1.42756722438662','0.144495841975922','0','6','86']
  H_HDV['114_PM_2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.03593616694129','0.016321687698884','0.193337785615721','0.115893537599919','0.009820421037524','6','86']
  H_HDV['114_FC_2_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','50','FC','y=((a*(b**x))*(x**c))','1692.57611138162','1.00846455501252','-0.555817090712404','0','0','6','86']
  H_HDV['114_CO_2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','100','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.879987721527144','-0.001475447498687','-0.085693295923989','0.126261913556826','0','6','86']
  H_HDV['114_THC_2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.007854074783194','0.085486932943973','7.27419150565689','2.82158723308222','-0.022868063064817','6','86']
  H_HDV['114_NOx_2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','100','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','14.8980811654844','-0.020603971339796','-1.65163607648909','0.161579088185248','0','6','86']
  H_HDV['114_PM_2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.048118214198802','0.024694233435463','0.191484085933826','0.130549755699619','0.013156078690267','6','86']
  H_HDV['114_FC_2_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','2','100','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','887.551466056286','0.513995043532326','-39.378072070633','0.069447489418317','0','6','86']
  H_HDV['114_CO_4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','0','CO','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.109126410882418','0.428685354537567','6.59277360474431','2.60290008685235','-0.016436450747043','6','72']
  H_HDV['114_THC_4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.006317059416292','0.091393074801288','6.66290101552595','2.56298116900897','-0.018335733909009','6','72']
  H_HDV['114_NOx_4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','16.3267937548501','-0.011661923514698','-2.16264371320301','0.175882088278599','0','6','72']
  H_HDV['114_PM_4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.23671183833707','-0.000261321286681','-0.021975901216127','0.110645872927855','0','6','72']
  H_HDV['114_FC_4_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','0','FC','y=((a*(b**x))*(x**c))','1797.22439133642','1.00979528668564','-0.569956842704824','0','0','6','72']
  H_HDV['114_CO_4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.876488705599682','-0.001691654358833','-0.080259428975616','0.123915539667506','0','6','71']
  H_HDV['114_THC_4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008418702146878','0.08658512383672','7.35357924847283','2.84724906681032','-0.022445037638843','6','71']
  H_HDV['114_NOx_4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','50','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','3.11659070396697','0.020381467575548','15.2760120257538','0.249535614609953','3.2919149080187','6','71']
  H_HDV['114_PM_4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','50','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.055345513963659','0.02035276492966','0.192883462934816','0.131350160711615','0.009481457806459','6','71']
  H_HDV['114_FC_4_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','50','FC','y=((a*(b**x))*(x**c))','1611.47700330197','1.00743238267345','-0.434034929342323','0','0','6','71']
  H_HDV['114_CO_4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','100','CO','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.36852485165934','0.016089642270149','0.626822241737827','0.144448620287637','-0.008956564882712','6','69']
  H_HDV['114_THC_4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.010466483657497','0.082961331919788','7.73820997774979','3.02535468111312','-0.026084105879134','6','69']
  H_HDV['114_NOx_4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','4.45715856237204','0.01887185080885','16.1102816153276','0.264353817650601','3.39122682347787','6','69']
  H_HDV['114_PM_4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','100','PM','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','0.068594770924915','0.027053384027092','0.191479090009095','0.139302373828163','0.011718448493907','6','69']
  H_HDV['114_FC_4_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','4','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','480.82322182026','0.079789766359244','29630.0316256069','1.05055783855347','529.368894309879','6','69']
  H_HDV['114_CO_6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','0','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.02824687883077','-0.002230270512011','-0.132540323342846','0.174717101195089','0','6','71']
  H_HDV['114_THC_6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','0','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.008221796802879','0.086095307074156','7.87235624141533','3.02965643721624','-0.027115283957354','6','71']
  H_HDV['114_NOx_6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','0','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.4983051779563','-0.02421932119706','-3.7653383014038','0.23619540264516','0','6','71']
  H_HDV['114_PM_6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','0','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.266251789762984','-0.000463933045656','-0.029006458169794','0.135008023863409','0','6','71']
  H_HDV['114_FC_6_0'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','0','FC','y=((a*(b**x))*(x**c))','1781.27191046045','1.00994768129651','-0.503591951530148','0','0','6','71']
  H_HDV['114_CO_6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','50','CO','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1.09241882357951','-0.003557240521007','-0.126727473762877','0.164888034349187','0','6','68']
  H_HDV['114_THC_6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','50','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.01123496058126','0.080482529543526','8.71541442108689','3.40400657651395','-0.035386451139539','6','68']
  H_HDV['114_NOx_6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','50','NOx','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','21.6366976489839','-0.037461238853831','-3.13369527358659','0.216390246208908','0','6','68']
  H_HDV['114_PM_6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','50','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.261452897291744','-0.000440751378531','-0.02433902608445','0.114632127905063','0','6','68']
  H_HDV['114_FC_6_50'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','50','FC','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','1101.7012324177','1.32024563002085','-44.3086298597937','0.073996834235967','0','6','68']
  H_HDV['114_CO_6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','100','CO','y=exp((a+(b/x))+(c*log(x)))','1.90549626225348','-3.58505944730746','-0.944315024493165','0','0','6','56']
  H_HDV['114_THC_6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','100','THC','y=(a+(b/(1+exp((((-1)*c)+(d*log(x)))+(e*x)))))','0.016792453483991','0.600546755986289','-1.83448728384049','-0.302072167683647','0.119971550866916','6','56']
  H_HDV['114_NOx_6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','100','NOx','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','5.54834312378796','0.04128674181438','17.4877456710802','0.302501175781767','5.66842364985188','6','56']
  H_HDV['114_PM_6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','100','PM','y=((a+(b*x))+(((c-b)*(1-exp(((-1)*d)*x)))/d))','0.275205525226769','-0.000488762560345','-0.024895228855308','0.110134211651545','0','6','56']
  H_HDV['114_FC_6_100'] = [' 114','EURO5','TT/AT >14-20t Euro-5','6','100','FC','y=((e+(a*exp(((-1)*b)*x)))+(c*exp(((-1)*d)*x)))','346.041642028117','0.062066139648813','1881.48108598056','0.46720976402725','701.458038022096','6','56']
